// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Graph.RBAC;
using Microsoft.Azure.Graph.RBAC.Models;

namespace Microsoft.Azure.Graph.RBAC
{
    /// <summary>
    /// Operations for working with Groups in Azure Active Directory Graph API.
    /// (see http://msdn.microsoft.com/en-us/library/azure/hh974476.aspx for
    /// more information)
    /// </summary>
    public partial interface IGroupOperations
    {
        /// <summary>
        /// Add a memeber to a group.   (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn151600.aspx for
        /// more information)
        /// </summary>
        /// <param name='groupObjectId'>
        /// Group object id
        /// </param>
        /// <param name='parameters'>
        /// Member Object Url as
        /// https://graph.windows.net/contoso.onmicrosoft.com/directoryObjects/f260bbc4-c254-447b-94cf-293b5ec434dd
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> AddMemberAsync(string groupObjectId, GroupAddMemberParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Create a group in the directory.  (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn151614.aspx for
        /// more information)
        /// </summary>
        /// <param name='parameters'>
        /// Parameters to create a group
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Get group information API call
        /// </returns>
        Task<GroupGetResult> CreateAsync(GroupCreateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Delete a group in the directory.  (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn151608.aspx for
        /// more information)
        /// </summary>
        /// <param name='groupObjectId'>
        /// Object id
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string groupObjectId, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets group information from the directory.
        /// </summary>
        /// <param name='objectId'>
        /// User objectId to get group information.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Get group information API call
        /// </returns>
        Task<GroupGetResult> GetAsync(string objectId, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the members of a group.
        /// </summary>
        /// <param name='objectId'>
        /// Group object Id who's members should be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Active Directory objects inquiry API calls
        /// </returns>
        Task<GetObjectsResult> GetGroupMembersAsync(string objectId, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets next set of members of a group.
        /// </summary>
        /// <param name='nextLink'>
        /// NextLink from the previous successful call to GetGroupMembers or
        /// GetGroupMembersNext.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Active Directory objects inquiry API calls
        /// </returns>
        Task<GetObjectsResult> GetGroupMembersNextAsync(string nextLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets a collection that contains the Object IDs of the groups of
        /// which the group is a member.
        /// </summary>
        /// <param name='parameters'>
        /// Group filtering parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for GetMemberGroups API call
        /// </returns>
        Task<GroupGetMemberGroupsResult> GetMemberGroupsAsync(GroupGetMemberGroupsParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets list of groups for the current tenant.
        /// </summary>
        /// <param name='mail'>
        /// Email to filter results.
        /// </param>
        /// <param name='displayNameStartsWith'>
        /// Display name to filter results.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Get tenant groups API call
        /// </returns>
        Task<GroupListResult> ListAsync(string mail, string displayNameStartsWith, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets list of groups for the current tenant.
        /// </summary>
        /// <param name='nextLink'>
        /// NextLink from the previous successful call to List operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Get tenant groups API call
        /// </returns>
        Task<GroupListResult> ListNextAsync(string nextLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Remove a memeber from a group  (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn151607.aspx for
        /// more information)
        /// </summary>
        /// <param name='groupObjectId'>
        /// Group object id
        /// </param>
        /// <param name='memberObjectId'>
        /// Member Object id
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> RemoveMemberAsync(string groupObjectId, string memberObjectId, CancellationToken cancellationToken);
    }
}
