// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure.Graph.RBAC;
using Microsoft.Azure.Graph.RBAC.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Graph.RBAC
{
    /// <summary>
    /// Operations for working with Objects in Azure AD Graph API.  (see
    /// http://msdn.microsoft.com/en-us/library/azure/hh974476.aspx for more
    /// information)
    /// </summary>
    internal partial class ObjectOperations : IServiceOperations<GraphRbacManagementClient>, IObjectOperations
    {
        /// <summary>
        /// Initializes a new instance of the ObjectOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal ObjectOperations(GraphRbacManagementClient client)
        {
            this._client = client;
        }
        
        private GraphRbacManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.Azure.Graph.RBAC.GraphRbacManagementClient.
        /// </summary>
        public GraphRbacManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Gets the details for current logged in user
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Active Directory objects inquiry API calls
        /// </returns>
        public async Task<GetCurrentUserResult> GetCurrentUserAsync(CancellationToken cancellationToken)
        {
            // Validate
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                TracingAdapter.Enter(invocationId, this, "GetCurrentUserAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.TenantID);
            url = url + "/me";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=1.6-internal");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    GetCurrentUserResult result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new GetCurrentUserResult();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            AADObject aADObjectInstance = new AADObject();
                            result.AADObject = aADObjectInstance;
                            
                            JToken objectIdValue = responseDoc["objectId"];
                            if (objectIdValue != null && objectIdValue.Type != JTokenType.Null)
                            {
                                string objectIdInstance = ((string)objectIdValue);
                                aADObjectInstance.ObjectId = objectIdInstance;
                            }
                            
                            JToken objectTypeValue = responseDoc["objectType"];
                            if (objectTypeValue != null && objectTypeValue.Type != JTokenType.Null)
                            {
                                string objectTypeInstance = ((string)objectTypeValue);
                                aADObjectInstance.ObjectType = objectTypeInstance;
                            }
                            
                            JToken displayNameValue = responseDoc["displayName"];
                            if (displayNameValue != null && displayNameValue.Type != JTokenType.Null)
                            {
                                string displayNameInstance = ((string)displayNameValue);
                                aADObjectInstance.DisplayName = displayNameInstance;
                            }
                            
                            JToken userPrincipalNameValue = responseDoc["userPrincipalName"];
                            if (userPrincipalNameValue != null && userPrincipalNameValue.Type != JTokenType.Null)
                            {
                                string userPrincipalNameInstance = ((string)userPrincipalNameValue);
                                aADObjectInstance.UserPrincipalName = userPrincipalNameInstance;
                            }
                            
                            JToken mailValue = responseDoc["mail"];
                            if (mailValue != null && mailValue.Type != JTokenType.Null)
                            {
                                string mailInstance = ((string)mailValue);
                                aADObjectInstance.Mail = mailInstance;
                            }
                            
                            JToken mailEnabledValue = responseDoc["mailEnabled"];
                            if (mailEnabledValue != null && mailEnabledValue.Type != JTokenType.Null)
                            {
                                bool mailEnabledInstance = ((bool)mailEnabledValue);
                                aADObjectInstance.MailEnabled = mailEnabledInstance;
                            }
                            
                            JToken securityEnabledValue = responseDoc["securityEnabled"];
                            if (securityEnabledValue != null && securityEnabledValue.Type != JTokenType.Null)
                            {
                                bool securityEnabledInstance = ((bool)securityEnabledValue);
                                aADObjectInstance.SecurityEnabled = securityEnabledInstance;
                            }
                            
                            JToken signInNameValue = responseDoc["signInName"];
                            if (signInNameValue != null && signInNameValue.Type != JTokenType.Null)
                            {
                                string signInNameInstance = ((string)signInNameValue);
                                aADObjectInstance.SignInName = signInNameInstance;
                            }
                            
                            JToken servicePrincipalNamesArray = responseDoc["servicePrincipalNames"];
                            if (servicePrincipalNamesArray != null && servicePrincipalNamesArray.Type != JTokenType.Null)
                            {
                                foreach (JToken servicePrincipalNamesValue in ((JArray)servicePrincipalNamesArray))
                                {
                                    aADObjectInstance.ServicePrincipalNames.Add(((string)servicePrincipalNamesValue));
                                }
                            }
                            
                            JToken userTypeValue = responseDoc["userType"];
                            if (userTypeValue != null && userTypeValue.Type != JTokenType.Null)
                            {
                                string userTypeInstance = ((string)userTypeValue);
                                aADObjectInstance.UserType = userTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets AD group membership by provided AD object Ids
        /// </summary>
        /// <param name='parameters'>
        /// Required. Objects filtering parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Server response for Active Directory objects inquiry API calls
        /// </returns>
        public async Task<GetObjectsResult> GetObjectsByObjectIdsAsync(GetObjectsParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "GetObjectsByObjectIdsAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.TenantID);
            url = url + "/getObjectsByObjectIds";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=1.6-internal");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject getObjectsParametersValue = new JObject();
                requestDoc = getObjectsParametersValue;
                
                if (parameters.Ids != null)
                {
                    if (parameters.Ids is ILazyCollection == false || ((ILazyCollection)parameters.Ids).IsInitialized)
                    {
                        JArray objectIdsArray = new JArray();
                        foreach (string objectIdsItem in parameters.Ids)
                        {
                            objectIdsArray.Add(objectIdsItem);
                        }
                        getObjectsParametersValue["objectIds"] = objectIdsArray;
                    }
                }
                
                if (parameters.Types != null)
                {
                    if (parameters.Types is ILazyCollection == false || ((ILazyCollection)parameters.Types).IsInitialized)
                    {
                        JArray typesArray = new JArray();
                        foreach (string typesItem in parameters.Types)
                        {
                            typesArray.Add(typesItem);
                        }
                        getObjectsParametersValue["types"] = typesArray;
                    }
                }
                
                getObjectsParametersValue["includeDirectoryObjectReferences"] = parameters.IncludeDirectoryObjectReferences;
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json; charset=utf-8");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    GetObjectsResult result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new GetObjectsResult();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    AADObject aADObjectInstance = new AADObject();
                                    result.AADObject.Add(aADObjectInstance);
                                    
                                    JToken objectIdValue = valueValue["objectId"];
                                    if (objectIdValue != null && objectIdValue.Type != JTokenType.Null)
                                    {
                                        string objectIdInstance = ((string)objectIdValue);
                                        aADObjectInstance.ObjectId = objectIdInstance;
                                    }
                                    
                                    JToken objectTypeValue = valueValue["objectType"];
                                    if (objectTypeValue != null && objectTypeValue.Type != JTokenType.Null)
                                    {
                                        string objectTypeInstance = ((string)objectTypeValue);
                                        aADObjectInstance.ObjectType = objectTypeInstance;
                                    }
                                    
                                    JToken displayNameValue = valueValue["displayName"];
                                    if (displayNameValue != null && displayNameValue.Type != JTokenType.Null)
                                    {
                                        string displayNameInstance = ((string)displayNameValue);
                                        aADObjectInstance.DisplayName = displayNameInstance;
                                    }
                                    
                                    JToken userPrincipalNameValue = valueValue["userPrincipalName"];
                                    if (userPrincipalNameValue != null && userPrincipalNameValue.Type != JTokenType.Null)
                                    {
                                        string userPrincipalNameInstance = ((string)userPrincipalNameValue);
                                        aADObjectInstance.UserPrincipalName = userPrincipalNameInstance;
                                    }
                                    
                                    JToken mailValue = valueValue["mail"];
                                    if (mailValue != null && mailValue.Type != JTokenType.Null)
                                    {
                                        string mailInstance = ((string)mailValue);
                                        aADObjectInstance.Mail = mailInstance;
                                    }
                                    
                                    JToken mailEnabledValue = valueValue["mailEnabled"];
                                    if (mailEnabledValue != null && mailEnabledValue.Type != JTokenType.Null)
                                    {
                                        bool mailEnabledInstance = ((bool)mailEnabledValue);
                                        aADObjectInstance.MailEnabled = mailEnabledInstance;
                                    }
                                    
                                    JToken securityEnabledValue = valueValue["securityEnabled"];
                                    if (securityEnabledValue != null && securityEnabledValue.Type != JTokenType.Null)
                                    {
                                        bool securityEnabledInstance = ((bool)securityEnabledValue);
                                        aADObjectInstance.SecurityEnabled = securityEnabledInstance;
                                    }
                                    
                                    JToken signInNameValue = valueValue["signInName"];
                                    if (signInNameValue != null && signInNameValue.Type != JTokenType.Null)
                                    {
                                        string signInNameInstance = ((string)signInNameValue);
                                        aADObjectInstance.SignInName = signInNameInstance;
                                    }
                                    
                                    JToken servicePrincipalNamesArray = valueValue["servicePrincipalNames"];
                                    if (servicePrincipalNamesArray != null && servicePrincipalNamesArray.Type != JTokenType.Null)
                                    {
                                        foreach (JToken servicePrincipalNamesValue in ((JArray)servicePrincipalNamesArray))
                                        {
                                            aADObjectInstance.ServicePrincipalNames.Add(((string)servicePrincipalNamesValue));
                                        }
                                    }
                                    
                                    JToken userTypeValue = valueValue["userType"];
                                    if (userTypeValue != null && userTypeValue.Type != JTokenType.Null)
                                    {
                                        string userTypeInstance = ((string)userTypeValue);
                                        aADObjectInstance.UserType = userTypeInstance;
                                    }
                                }
                            }
                            
                            JToken odatanextLinkValue = responseDoc["odata.nextLink"];
                            if (odatanextLinkValue != null && odatanextLinkValue.Type != JTokenType.Null)
                            {
                                string odatanextLinkInstance = ((string)odatanextLinkValue);
                                result.NextLink = odatanextLinkInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
