// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.ApiManagement.Models;

namespace Microsoft.Azure.Management.ApiManagement
{
    /// <summary>
    /// Operations for managing Api Management service provisioning
    /// (create/remove, backup/restore, scale, etc.).
    /// </summary>
    public partial interface IResourceProviderOperations
    {
        /// <summary>
        /// Backs up an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Backup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> BackupAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Begin backup operation of an Api Management service.To determine
        /// whether the operation has finished processing the request, call
        /// GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the BeginBackup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> BeginBackupAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Begins creating new or updating existing Api Management service.To
        /// determine whether the operation has finished processing the
        /// request, call GetApiServiceLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the CreateOrUpdate Api Management service
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> BeginCreatingOrUpdatingAsync(string resourceGroupName, string name, ApiServiceCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Begin to manage (CUD) deployments of an Api Management service.To
        /// determine whether the operation has finished processing the
        /// request, call GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the ManageDeployments operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> BeginManagingDeploymentsAsync(string resourceGroupName, string name, ApiServiceManageDeploymentsParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Begin restore from backup operation of an Api Management service.To
        /// determine whether the operation has finished processing the
        /// request, call GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Restore Api Management service from
        /// backup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> BeginRestoringAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Begin updating hostname of an Api Management service.To determine
        /// whether the operation has finished processing the request, call
        /// GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the UpdateHostname operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> BeginUpdatingHostnameAsync(string resourceGroupName, string name, ApiServiceUpdateHostnameParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Checks whether the custom host name maps to an Api Management
        /// service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the CheckCustomHostname operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CheckCustomHostname operation.
        /// </returns>
        Task<ApiServiceCheckCustomHostnameResponse> CheckCustomHostnameAsync(string resourceGroupName, string name, ApiServiceCheckCustomHostnameParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Checks availability and correctness of a name for an Api Management
        /// service.
        /// </summary>
        /// <param name='parameters'>
        /// Parameters supplied to the CheckNameAvailability operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response of the CheckNameAvailability operation.
        /// </returns>
        Task<ApiServiceNameAvailabilityResponse> CheckNameAvailabilityAsync(ApiServiceCheckNameAvailabilityParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Creates new or updates existing Api Management service
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the CreateOrUpdate Api Management service
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> CreateOrUpdateAsync(string resourceGroupName, string name, ApiServiceCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Deletes existing Api Management service
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string name, CancellationToken cancellationToken);
        
        /// <summary>
        /// Get an Api Management service resource description.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the Get Api Management service operation.
        /// </returns>
        Task<ApiServiceGetResponse> GetAsync(string resourceGroupName, string name, CancellationToken cancellationToken);
        
        /// <summary>
        /// The Get ApiService Operation Status operation returns the status of
        /// the create or update operation. After calling the operation, you
        /// can call Get ApiService Operation Status to determine whether the
        /// operation has succeeded, failed, or is still in progress. This
        /// method differs GetLongRunningOperationStatus in providing Api
        /// Management service resource description.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> GetApiServiceLongRunningOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        Task<LongRunningOperationResponse> GetLongRunningOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets SsoToken for an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the GetSsoToken operation.
        /// </returns>
        Task<ApiServiceGetSsoTokenResponse> GetSsoTokenAsync(string resourceGroupName, string name, CancellationToken cancellationToken);
        
        /// <summary>
        /// List all Api Management services within a resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group. If resourceGroupName value is null
        /// the method lists all Api Management services within subscription
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the List Api Management services operation.
        /// </returns>
        Task<ApiServiceListResponse> ListAsync(string resourceGroupName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Manage (CUD) deployments of an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the ManageDeployments operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> ManageDeploymentsAsync(string resourceGroupName, string name, ApiServiceManageDeploymentsParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Restore an Api Management service from backup.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Restore Api Management service from
        /// backup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> RestoreAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Update hostname of an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the UpdateHostname operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        Task<ApiServiceLongRunningOperationResponse> UpdateHostnameAsync(string resourceGroupName, string name, ApiServiceUpdateHostnameParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Upload SSL certificate for an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the Upload SSL certificate for an Api
        /// Management service operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the Upload SSL certificate for an Api Management
        /// service operation.
        /// </returns>
        Task<ApiServiceUploadCertificateResponse> UploadCertificateAsync(string resourceGroupName, string name, ApiServiceUploadCertificateParameters parameters, CancellationToken cancellationToken);
    }
}
