// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.ApiManagement;
using Microsoft.Azure.Management.ApiManagement.SmapiModels;

namespace Microsoft.Azure.Management.ApiManagement
{
    /// <summary>
    /// .Net client wrapper for the REST API for Azure ApiManagement Service
    /// </summary>
    public static partial class OpenIdConnectProvidersOperationsExtensions
    {
        /// <summary>
        /// Creates new OpenID Connect Provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <param name='parameters'>
        /// Required. Create parameters.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Create(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid, OpenidConnectProviderCreateContract parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IOpenIdConnectProvidersOperations)s).CreateAsync(resourceGroupName, serviceName, opid, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates new OpenID Connect Provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <param name='parameters'>
        /// Required. Create parameters.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> CreateAsync(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid, OpenidConnectProviderCreateContract parameters)
        {
            return operations.CreateAsync(resourceGroupName, serviceName, opid, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes specific OpenID Connect Provider of the Api Management
        /// service instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <param name='etag'>
        /// Required. ETag.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid, string etag)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IOpenIdConnectProvidersOperations)s).DeleteAsync(resourceGroupName, serviceName, opid, etag);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes specific OpenID Connect Provider of the Api Management
        /// service instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <param name='etag'>
        /// Required. ETag.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid, string etag)
        {
            return operations.DeleteAsync(resourceGroupName, serviceName, opid, etag, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets specific OpenID Connect Provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <returns>
        /// Get OpenID Connect Provider operation response details.
        /// </returns>
        public static OpenIdConnectProviderGetResponse Get(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IOpenIdConnectProvidersOperations)s).GetAsync(resourceGroupName, serviceName, opid);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets specific OpenID Connect Provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <returns>
        /// Get OpenID Connect Provider operation response details.
        /// </returns>
        public static Task<OpenIdConnectProviderGetResponse> GetAsync(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid)
        {
            return operations.GetAsync(resourceGroupName, serviceName, opid, CancellationToken.None);
        }
        
        /// <summary>
        /// List all OpenID Connect Providers.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='query'>
        /// Optional.
        /// </param>
        /// <returns>
        /// List OpenIdProviders operation response details.
        /// </returns>
        public static OpenIdConnectProvidersListResponse List(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, QueryParameters query)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IOpenIdConnectProvidersOperations)s).ListAsync(resourceGroupName, serviceName, query);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// List all OpenID Connect Providers.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='query'>
        /// Optional.
        /// </param>
        /// <returns>
        /// List OpenIdProviders operation response details.
        /// </returns>
        public static Task<OpenIdConnectProvidersListResponse> ListAsync(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, QueryParameters query)
        {
            return operations.ListAsync(resourceGroupName, serviceName, query, CancellationToken.None);
        }
        
        /// <summary>
        /// List next OpenID Connect Providers page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. NextLink from the previous successful call to List
        /// operation.
        /// </param>
        /// <returns>
        /// List OpenIdProviders operation response details.
        /// </returns>
        public static OpenIdConnectProvidersListResponse ListNext(this IOpenIdConnectProvidersOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IOpenIdConnectProvidersOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// List next OpenID Connect Providers page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. NextLink from the previous successful call to List
        /// operation.
        /// </param>
        /// <returns>
        /// List OpenIdProviders operation response details.
        /// </returns>
        public static Task<OpenIdConnectProvidersListResponse> ListNextAsync(this IOpenIdConnectProvidersOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
        
        /// <summary>
        /// Patches specific OpenID Connect Provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <param name='parameters'>
        /// Required. Update parameters.
        /// </param>
        /// <param name='etag'>
        /// Required. ETag.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Update(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid, OpenidConnectProviderUpdateContract parameters, string etag)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IOpenIdConnectProvidersOperations)s).UpdateAsync(resourceGroupName, serviceName, opid, parameters, etag);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Patches specific OpenID Connect Provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.IOpenIdConnectProvidersOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='opid'>
        /// Required. Identifier of the OpenID Connect Provider.
        /// </param>
        /// <param name='parameters'>
        /// Required. Update parameters.
        /// </param>
        /// <param name='etag'>
        /// Required. ETag.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UpdateAsync(this IOpenIdConnectProvidersOperations operations, string resourceGroupName, string serviceName, string opid, OpenidConnectProviderUpdateContract parameters, string etag)
        {
            return operations.UpdateAsync(resourceGroupName, serviceName, opid, parameters, etag, CancellationToken.None);
        }
    }
}
