// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.ApiManagement;
using Microsoft.Azure.Management.ApiManagement.SmapiModels;

namespace Microsoft.Azure.Management.ApiManagement
{
    /// <summary>
    /// .Net client wrapper for the REST API for Azure ApiManagement Service
    /// </summary>
    public static partial class TenantConfigurationOperationsExtensions
    {
        /// <summary>
        /// Begin deploy operation of configuration from the specified Git
        /// branch to the configuration database in an Api Management
        /// service.To determine whether the operation has finished processing
        /// the request, call GetTenantConfigurationLongRunningOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Deploy Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static TenantLongRunningOperationResponse BeginDeploy(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).BeginDeployAsync(resourceGroupName, serviceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Begin deploy operation of configuration from the specified Git
        /// branch to the configuration database in an Api Management
        /// service.To determine whether the operation has finished processing
        /// the request, call GetTenantConfigurationLongRunningOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Deploy Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static Task<TenantLongRunningOperationResponse> BeginDeployAsync(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return operations.BeginDeployAsync(resourceGroupName, serviceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Begin save operation creates a commit of the current configuration
        /// snapshot to the specified branch in the repository in an Api
        /// Management service.To determine whether the operation has finished
        /// processing the request, call
        /// GetTenantConfigurationLongRunningOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Save Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static TenantLongRunningOperationResponse BeginSave(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, SaveConfigurationParameter parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).BeginSaveAsync(resourceGroupName, serviceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Begin save operation creates a commit of the current configuration
        /// snapshot to the specified branch in the repository in an Api
        /// Management service.To determine whether the operation has finished
        /// processing the request, call
        /// GetTenantConfigurationLongRunningOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Save Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static Task<TenantLongRunningOperationResponse> BeginSaveAsync(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, SaveConfigurationParameter parameters)
        {
            return operations.BeginSaveAsync(resourceGroupName, serviceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Begin validate operation of configuration changes in the specified
        /// Git branch of an Api Management service.To determine whether the
        /// operation has finished processing the request, call
        /// GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Validate Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static TenantLongRunningOperationResponse BeginValidate(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).BeginValidateAsync(resourceGroupName, serviceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Begin validate operation of configuration changes in the specified
        /// Git branch of an Api Management service.To determine whether the
        /// operation has finished processing the request, call
        /// GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Validate Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static Task<TenantLongRunningOperationResponse> BeginValidateAsync(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return operations.BeginValidateAsync(resourceGroupName, serviceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deploy Git changes to configuration database in Api Management
        /// service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Deploy Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static TenantConfigurationLongRunningOperationResponse Deploy(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).DeployAsync(resourceGroupName, serviceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deploy Git changes to configuration database in Api Management
        /// service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Deploy Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static Task<TenantConfigurationLongRunningOperationResponse> DeployAsync(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return operations.DeployAsync(resourceGroupName, serviceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static TenantLongRunningOperationResponse GetLongRunningOperationStatus(this ITenantConfigurationOperations operations, string operationStatusLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).GetLongRunningOperationStatusAsync(operationStatusLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant operations.
        /// </returns>
        public static Task<TenantLongRunningOperationResponse> GetLongRunningOperationStatusAsync(this ITenantConfigurationOperations operations, string operationStatusLink)
        {
            return operations.GetLongRunningOperationStatusAsync(operationStatusLink, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static TenantConfigurationLongRunningOperationResponse GetTenantConfigurationLongRunningOperationStatus(this ITenantConfigurationOperations operations, string operationStatusLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).GetTenantConfigurationLongRunningOperationStatusAsync(operationStatusLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static Task<TenantConfigurationLongRunningOperationResponse> GetTenantConfigurationLongRunningOperationStatusAsync(this ITenantConfigurationOperations operations, string operationStatusLink)
        {
            return operations.GetTenantConfigurationLongRunningOperationStatusAsync(operationStatusLink, CancellationToken.None);
        }
        
        /// <summary>
        /// Commit configuration snapshot to an Api Management service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Save Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static TenantConfigurationLongRunningOperationResponse Save(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, SaveConfigurationParameter parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).SaveAsync(resourceGroupName, serviceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Commit configuration snapshot to an Api Management service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Save Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static Task<TenantConfigurationLongRunningOperationResponse> SaveAsync(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, SaveConfigurationParameter parameters)
        {
            return operations.SaveAsync(resourceGroupName, serviceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Validate Git changes in an Api Management service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Validate Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static TenantConfigurationLongRunningOperationResponse Validate(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ITenantConfigurationOperations)s).ValidateAsync(resourceGroupName, serviceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Validate Git changes in an Api Management service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.ApiManagement.ITenantConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='serviceName'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Validate Configuration parameters.
        /// </param>
        /// <returns>
        /// A standard service response for long running tenant configuration
        /// operations.
        /// </returns>
        public static Task<TenantConfigurationLongRunningOperationResponse> ValidateAsync(this ITenantConfigurationOperations operations, string resourceGroupName, string serviceName, DeployConfigurationParameters parameters)
        {
            return operations.ValidateAsync(resourceGroupName, serviceName, parameters, CancellationToken.None);
        }
    }
}
