// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.BackupServices;
using Microsoft.Azure.Management.BackupServices.Models;

namespace Microsoft.Azure.Management.BackupServices
{
    /// <summary>
    /// Definition of Vault-related operations for the Azure Backup extension.
    /// </summary>
    public partial interface IVaultOperations
    {
        /// <summary>
        /// Creates a new Azure backup vault.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of resource group to which vault belongs
        /// </param>
        /// <param name='resourceName'>
        /// The name of the vault
        /// </param>
        /// <param name='parameters'>
        /// Parameters to create or update the vault
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Vault information.
        /// </returns>
        Task<AzureBackupVaultGetResponse> CreateOrUpdateAsync(string resourceGroupName, string resourceName, AzureBackupVaultCreateOrUpdateParameters parameters, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Deletes the specified Azure backup vault.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of resource group to which vault belongs
        /// </param>
        /// <param name='resourceName'>
        /// The name of the vault
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Vault information.
        /// </returns>
        Task<AzureBackupVaultGetResponse> DeleteAsync(string resourceGroupName, string resourceName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the specified Azure key vault.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of resource group to which vault belongs
        /// </param>
        /// <param name='resourceName'>
        /// The name of the vault
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Vault information.
        /// </returns>
        Task<AzureBackupVaultGetResponse> GetAsync(string resourceGroupName, string resourceName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Fetches resource storage config.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of resource group to which vault belongs
        /// </param>
        /// <param name='resourceName'>
        /// The name of the vault
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The definition of a get resource storage config response.
        /// </returns>
        Task<GetResourceStorageConfigResponse> GetResourceStorageConfigAsync(string resourceGroupName, string resourceName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets information of the backup vaults associated with subscription.
        /// </summary>
        /// <param name='top'>
        /// Maximum number of results to return.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// List of vaults
        /// </returns>
        Task<AzureBackupVaultListResponse> ListAsync(int top, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets information of the backup vaults associated with resource
        /// group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// An optional argument which specifies the name of the resource group
        /// that constrains the set of vaults that are returned.
        /// </param>
        /// <param name='top'>
        /// Maximum number of results to return.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// List of vaults
        /// </returns>
        Task<AzureBackupVaultListResponse> ListByResourceGroupAsync(string resourceGroupName, int top, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Updates vault storage model type.
        /// </summary>
        /// <param name='updateVaultStorageTypeRequest'>
        /// Update Vault Storage Type Request
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The definition of a Operation Response.
        /// </returns>
        Task<OperationResponse> UpdateStorageTypeAsync(string resourceGroupName, string resourceName, UpdateVaultStorageTypeRequest updateVaultStorageTypeRequest, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Uploads vault credential certificate.
        /// </summary>
        /// <param name='certificateName'>
        /// Name of the certificate.
        /// </param>
        /// <param name='vaultCredUploadCertRequest'>
        /// Certificate parameters.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The definition of a certificate response.
        /// </returns>
        Task<VaultCredUploadCertResponse> UploadCertificateAsync(string resourceGroupName, string resourceName, string certificateName, VaultCredUploadCertRequest vaultCredUploadCertRequest, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
    }
}
