// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.AzureStack.Management;
using Microsoft.AzureStack.Management.Models;

namespace Microsoft.AzureStack.Management
{
    public static partial class GalleryItemOperationsExtensions
    {
        /// <summary>
        /// Creates or updates the Gallery Item
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Gallery item Upload parameters.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse CreateOrUpdate(this IGalleryItemOperations operations, GalleryItemCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IGalleryItemOperations)s).CreateOrUpdateAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates or updates the Gallery Item
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Gallery item Upload parameters.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> CreateOrUpdateAsync(this IGalleryItemOperations operations, GalleryItemCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Gallery item Delete operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='galleryItemId'>
        /// Required. Gallery item identity.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IGalleryItemOperations operations, string galleryItemId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IGalleryItemOperations)s).DeleteAsync(galleryItemId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gallery item Delete operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='galleryItemId'>
        /// Required. Gallery item identity.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IGalleryItemOperations operations, string galleryItemId)
        {
            return operations.DeleteAsync(galleryItemId, CancellationToken.None);
        }
        
        /// <summary>
        /// Gallery item Get operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='galleryItemId'>
        /// Required. Gallery item identity.
        /// </param>
        /// <returns>
        /// Gallery item Get operation result.
        /// </returns>
        public static GalleryItemGetResult Get(this IGalleryItemOperations operations, string galleryItemId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IGalleryItemOperations)s).GetAsync(galleryItemId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gallery item Get operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='galleryItemId'>
        /// Required. Gallery item identity.
        /// </param>
        /// <returns>
        /// Gallery item Get operation result.
        /// </returns>
        public static Task<GalleryItemGetResult> GetAsync(this IGalleryItemOperations operations, string galleryItemId)
        {
            return operations.GetAsync(galleryItemId, CancellationToken.None);
        }
        
        /// <summary>
        /// Gallery items List operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <returns>
        /// Gallery item List operation result.
        /// </returns>
        public static GalleryItemListResult List(this IGalleryItemOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IGalleryItemOperations)s).ListAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gallery items List operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <returns>
        /// Gallery item List operation result.
        /// </returns>
        public static Task<GalleryItemListResult> ListAsync(this IGalleryItemOperations operations)
        {
            return operations.ListAsync(CancellationToken.None);
        }
        
        /// <summary>
        /// Gallery items List operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. Gets or sets the URL to get the next set of results.
        /// </param>
        /// <returns>
        /// Gallery item List operation result.
        /// </returns>
        public static GalleryItemListResult ListNext(this IGalleryItemOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IGalleryItemOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gallery items List operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IGalleryItemOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. Gets or sets the URL to get the next set of results.
        /// </param>
        /// <returns>
        /// Gallery item List operation result.
        /// </returns>
        public static Task<GalleryItemListResult> ListNextAsync(this IGalleryItemOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
