// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.AzureStack.Management;
using Microsoft.AzureStack.Management.Models;

namespace Microsoft.AzureStack.Management
{
    public static partial class ManagedLocationOperationsExtensions
    {
        /// <summary>
        /// Create / Update the location.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Location properties
        /// </param>
        /// <returns>
        /// The location update result.
        /// </returns>
        public static ManagedLocationCreateOrUpdateResult CreateOrUpdate(this IManagedLocationOperations operations, ManagedLocationCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IManagedLocationOperations)s).CreateOrUpdateAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create / Update the location.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Location properties
        /// </param>
        /// <returns>
        /// The location update result.
        /// </returns>
        public static Task<ManagedLocationCreateOrUpdateResult> CreateOrUpdateAsync(this IManagedLocationOperations operations, ManagedLocationCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Delete a location.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='locationName'>
        /// Required. Name of location to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IManagedLocationOperations operations, string locationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IManagedLocationOperations)s).DeleteAsync(locationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Delete a location.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='locationName'>
        /// Required. Name of location to delete.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IManagedLocationOperations operations, string locationName)
        {
            return operations.DeleteAsync(locationName, CancellationToken.None);
        }
        
        /// <summary>
        /// Get the location.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='locationName'>
        /// Required. The location name.
        /// </param>
        /// <returns>
        /// Location get result.
        /// </returns>
        public static ManagedLocationGetResult Get(this IManagedLocationOperations operations, string locationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IManagedLocationOperations)s).GetAsync(locationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Get the location.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='locationName'>
        /// Required. The location name.
        /// </param>
        /// <returns>
        /// Location get result.
        /// </returns>
        public static Task<ManagedLocationGetResult> GetAsync(this IManagedLocationOperations operations, string locationName)
        {
            return operations.GetAsync(locationName, CancellationToken.None);
        }
        
        /// <summary>
        /// Get locations under subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <returns>
        /// The location list result.
        /// </returns>
        public static ManagedLocationListResult List(this IManagedLocationOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IManagedLocationOperations)s).ListAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Get locations under subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <returns>
        /// The location list result.
        /// </returns>
        public static Task<ManagedLocationListResult> ListAsync(this IManagedLocationOperations operations)
        {
            return operations.ListAsync(CancellationToken.None);
        }
        
        /// <summary>
        /// Gets locations with the next link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to get the next set of results.
        /// </param>
        /// <returns>
        /// The location list result.
        /// </returns>
        public static ManagedLocationListResult ListNext(this IManagedLocationOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IManagedLocationOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets locations with the next link.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.IManagedLocationOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to get the next set of results.
        /// </param>
        /// <returns>
        /// The location list result.
        /// </returns>
        public static Task<ManagedLocationListResult> ListNextAsync(this IManagedLocationOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
