// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.AzureStack.Management;
using Microsoft.AzureStack.Management.Models;

namespace Microsoft.AzureStack.Management
{
    public static partial class SubscriptionOperationsExtensions
    {
        /// <summary>
        /// Creates or updates the subscription as a tenant
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters for creating or updating the subscription
        /// </param>
        /// <returns>
        /// Subscription definition object after create or update operation
        /// </returns>
        public static SubscriptionCreateOrUpdateResult CreateOrUpdate(this ISubscriptionOperations operations, SubscriptionCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubscriptionOperations)s).CreateOrUpdateAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates or updates the subscription as a tenant
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters for creating or updating the subscription
        /// </param>
        /// <returns>
        /// Subscription definition object after create or update operation
        /// </returns>
        public static Task<SubscriptionCreateOrUpdateResult> CreateOrUpdateAsync(this ISubscriptionOperations operations, SubscriptionCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Delete operation of the subscription
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='subscriptionId'>
        /// Required. Subscription Id
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this ISubscriptionOperations operations, string subscriptionId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubscriptionOperations)s).DeleteAsync(subscriptionId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Delete operation of the subscription
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='subscriptionId'>
        /// Required. Subscription Id
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this ISubscriptionOperations operations, string subscriptionId)
        {
            return operations.DeleteAsync(subscriptionId, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the subscription given the id
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='subscriptionId'>
        /// Optional.
        /// </param>
        /// <returns>
        /// Result of subscription get operation
        /// </returns>
        public static SubscriptionGetResult Get(this ISubscriptionOperations operations, string subscriptionId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubscriptionOperations)s).GetAsync(subscriptionId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the subscription given the id
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='subscriptionId'>
        /// Optional.
        /// </param>
        /// <returns>
        /// Result of subscription get operation
        /// </returns>
        public static Task<SubscriptionGetResult> GetAsync(this ISubscriptionOperations operations, string subscriptionId)
        {
            return operations.GetAsync(subscriptionId, CancellationToken.None);
        }
        
        /// <summary>
        /// Lists the subscriptions under the user account
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='includeDetails'>
        /// Required.
        /// </param>
        /// <returns>
        /// Result of subscription list operation
        /// </returns>
        public static SubscriptionListResult List(this ISubscriptionOperations operations, bool includeDetails)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubscriptionOperations)s).ListAsync(includeDetails);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists the subscriptions under the user account
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='includeDetails'>
        /// Required.
        /// </param>
        /// <returns>
        /// Result of subscription list operation
        /// </returns>
        public static Task<SubscriptionListResult> ListAsync(this ISubscriptionOperations operations, bool includeDetails)
        {
            return operations.ListAsync(includeDetails, CancellationToken.None);
        }
        
        /// <summary>
        /// Lists the next set of subscriptions
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. URL to get the next set of results
        /// </param>
        /// <returns>
        /// Result of subscription list operation
        /// </returns>
        public static SubscriptionListResult ListNext(this ISubscriptionOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubscriptionOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists the next set of subscriptions
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.AzureStack.Management.ISubscriptionOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. URL to get the next set of results
        /// </param>
        /// <returns>
        /// Result of subscription list operation
        /// </returns>
        public static Task<SubscriptionListResult> ListNextAsync(this ISubscriptionOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
