// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.DataLake.AnalyticsJob.Models;

namespace Microsoft.Azure.Management.DataLake.AnalyticsJob
{
    /// <summary>
    /// Operations for submitting and cancelling Data Lake Analytics jobs and
    /// retrieving their status and history
    /// </summary>
    public partial interface IJobOperations
    {
        /// <summary>
        /// Builds (compiles) the specified job in the specified Data Lake
        /// Analytics account for job correctness and validation.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to build the job for
        /// </param>
        /// <param name='parameters'>
        /// The parameters to build a job, which simulates submission.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Data Lake Analytics job information response from a build or create
        /// request.
        /// </returns>
        Task<JobInfoBuildOrCreateResponse> BuildAsync(string resourceGroupName, string accountName, JobInfoBuildOrCreateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Cancels the running job specified by the job ID.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to cancel the job for
        /// </param>
        /// <param name='jobIdentity'>
        /// JobInfo ID to cancel.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> CancelAsync(string resourceGroupName, string accountName, Guid jobIdentity, CancellationToken cancellationToken);
        
        /// <summary>
        /// Submits the specified job to the specified Data Lake Analytics
        /// account for computation.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to create the job for
        /// </param>
        /// <param name='parameters'>
        /// The parameters to submit a job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Data Lake Analytics job information response from a build or create
        /// request.
        /// </returns>
        Task<JobInfoBuildOrCreateResponse> CreateAsync(string resourceGroupName, string accountName, JobInfoBuildOrCreateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the JobInfo object specified by the job ID.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to get the job from
        /// </param>
        /// <param name='jobIdentity'>
        /// JobInfo ID.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Data Lake Analytics job information.
        /// </returns>
        Task<JobInfoGetResponse> GetAsync(string resourceGroupName, string accountName, Guid jobIdentity, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the U-SQL job debug data information specified by the job ID.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to get the job from
        /// </param>
        /// <param name='jobIdentity'>
        /// JobInfo ID.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Data Lake Analytics job data path information.
        /// </returns>
        Task<JobDataPathGetResponse> GetDebugDataPathAsync(string resourceGroupName, string accountName, Guid jobIdentity, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the job statistics object specified by the job ID.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to get the job from
        /// </param>
        /// <param name='jobIdentity'>
        /// JobInfo ID.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Data Lake Analytics Account information.
        /// </returns>
        Task<JobStatisticsGetResponse> GetStatisticsAsync(string resourceGroupName, string accountName, Guid jobIdentity, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the first page of the Data Lake Analytics JobInformation
        /// objects within the specified resource group with a link to the
        /// next page, if any.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to get the job from
        /// </param>
        /// <param name='parameters'>
        /// Query parameters. If null is passed returns all JobInfo items.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// List of jobInfo items.
        /// </returns>
        Task<JobInfoListResponse> ListAsync(string resourceGroupName, string accountName, JobListParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the next page of the Data Lake Analytics JobInformation
        /// objects within the specified resource group with a link to the
        /// next page, if any.
        /// </summary>
        /// <param name='nextLink'>
        /// The url to the next job information page.
        /// </param>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// List of jobInfo items.
        /// </returns>
        Task<JobInfoListResponse> ListNextAsync(string nextLink, string resourceGroupName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the first page of the Data Lake Analytics JobInformation
        /// objects within the specified resource group with a link to the
        /// next page, if any.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='accountName'>
        /// The name of the Data Lake Analytics account to get the job from
        /// </param>
        /// <param name='queryString'>
        /// OData Query parameter string (e.g.
        /// '$orderBy=SubmitTime,$filter=State eq Succeeded'). If null is
        /// passed returns all JobInfo items.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// List of jobInfo items.
        /// </returns>
        Task<JobInfoListResponse> ListWithQueryStringAsync(string resourceGroupName, string accountName, string queryString, CancellationToken cancellationToken);
    }
}
