// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Net.Http;
using Hyak.Common;
using Microsoft.Azure;
using Microsoft.Azure.Management.DataLake.StoreFileSystem;
using Microsoft.Azure.Management.DataLake.StoreFileSystem.Models;

namespace Microsoft.Azure.Management.DataLake.StoreFileSystem
{
    /// <summary>
    /// Creates a Data Lake Store filesystem management client.
    /// </summary>
    public partial class DataLakeStoreFileSystemManagementClient : ServiceClient<DataLakeStoreFileSystemManagementClient>, IDataLakeStoreFileSystemManagementClient
    {
        private string _apiVersion;
        
        /// <summary>
        /// Gets the API version.
        /// </summary>
        public string ApiVersion
        {
            get { return this._apiVersion; }
        }
        
        private SubscriptionCloudCredentials _credentials;
        
        /// <summary>
        /// Gets subscription credentials which uniquely identify Microsoft
        /// Azure subscription. The subscription ID forms part of the URI for
        /// every service call.
        /// </summary>
        public SubscriptionCloudCredentials Credentials
        {
            get { return this._credentials; }
        }
        
        private string _dataLakeServiceUri;
        
        /// <summary>
        /// Gets the URI used as the base for all cloud service requests.
        /// </summary>
        public string DataLakeServiceUri
        {
            get { return this._dataLakeServiceUri; }
            set { this._dataLakeServiceUri = value; }
        }
        
        private int _longRunningOperationInitialTimeout;
        
        /// <summary>
        /// Gets or sets the initial timeout for Long Running Operations.
        /// </summary>
        public int LongRunningOperationInitialTimeout
        {
            get { return this._longRunningOperationInitialTimeout; }
            set { this._longRunningOperationInitialTimeout = value; }
        }
        
        private int _longRunningOperationRetryTimeout;
        
        /// <summary>
        /// Gets or sets the retry timeout for Long Running Operations.
        /// </summary>
        public int LongRunningOperationRetryTimeout
        {
            get { return this._longRunningOperationRetryTimeout; }
            set { this._longRunningOperationRetryTimeout = value; }
        }
        
        private string _userAgentSuffix;
        
        /// <summary>
        /// Gets or sets the additional UserAgent text to be added to the user
        /// agent header. This is used to further differentiate where requests
        /// are coming from internally.
        /// </summary>
        public string UserAgentSuffix
        {
            get { return this._userAgentSuffix; }
            set { this._userAgentSuffix = value; }
        }
        
        private IFileSystemOperations _fileSystem;
        
        /// <summary>
        /// Operations for managing the Data Lake Store filesystem
        /// </summary>
        public virtual IFileSystemOperations FileSystem
        {
            get { return this._fileSystem; }
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// DataLakeStoreFileSystemManagementClient class.
        /// </summary>
        public DataLakeStoreFileSystemManagementClient()
        {
            var handler = new HttpClientHandler
            {
                AllowAutoRedirect = false,
                ClientCertificateOptions = ClientCertificateOption.Automatic
            };

            InitializeHttpClient(handler);

            this._fileSystem = new FileSystemOperations(this);
            this._userAgentSuffix = "";
            this._apiVersion = "2015-10-01-preview";
            this._longRunningOperationInitialTimeout = -1;
            this._longRunningOperationRetryTimeout = -1;
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// DataLakeStoreFileSystemManagementClient class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        /// <param name='dataLakeServiceUri'>
        /// Optional. Gets the URI used as the base for all cloud service
        /// requests.
        /// </param>
        public DataLakeStoreFileSystemManagementClient(SubscriptionCloudCredentials credentials, string dataLakeServiceUri)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (dataLakeServiceUri == null)
            {
                throw new ArgumentNullException("dataLakeServiceUri");
            }
            this._credentials = credentials;
            this._dataLakeServiceUri = dataLakeServiceUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// DataLakeStoreFileSystemManagementClient class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        public DataLakeStoreFileSystemManagementClient(SubscriptionCloudCredentials credentials)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._credentials = credentials;
            this._dataLakeServiceUri = "azuredatalakestore.net";
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// DataLakeStoreFileSystemManagementClient class.
        /// </summary>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public DataLakeStoreFileSystemManagementClient(HttpClient httpClient)
            : base(httpClient)
        {
            this._fileSystem = new FileSystemOperations(this);
            this._userAgentSuffix = "";
            this._apiVersion = "2015-10-01-preview";
            this._longRunningOperationInitialTimeout = -1;
            this._longRunningOperationRetryTimeout = -1;
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// DataLakeStoreFileSystemManagementClient class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        /// <param name='dataLakeServiceUri'>
        /// Optional. Gets the URI used as the base for all cloud service
        /// requests.
        /// </param>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public DataLakeStoreFileSystemManagementClient(SubscriptionCloudCredentials credentials, string dataLakeServiceUri, HttpClient httpClient)
            : this(httpClient)
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (dataLakeServiceUri == null)
            {
                throw new ArgumentNullException("dataLakeServiceUri");
            }
            this._credentials = credentials;
            this._dataLakeServiceUri = dataLakeServiceUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the
        /// DataLakeStoreFileSystemManagementClient class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public DataLakeStoreFileSystemManagementClient(SubscriptionCloudCredentials credentials, HttpClient httpClient)
            : this(httpClient)
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._credentials = credentials;
            this._dataLakeServiceUri = "azuredatalakestore.net";
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Clones properties from current instance to another
        /// DataLakeStoreFileSystemManagementClient instance
        /// </summary>
        /// <param name='client'>
        /// Instance of DataLakeStoreFileSystemManagementClient to clone to
        /// </param>
        protected override void Clone(ServiceClient<DataLakeStoreFileSystemManagementClient> client)
        {
            base.Clone(client);
            
            if (client is DataLakeStoreFileSystemManagementClient)
            {
                DataLakeStoreFileSystemManagementClient clonedClient = ((DataLakeStoreFileSystemManagementClient)client);
                
                clonedClient._userAgentSuffix = this._userAgentSuffix;
                clonedClient._credentials = this._credentials;
                clonedClient._dataLakeServiceUri = this._dataLakeServiceUri;
                clonedClient._apiVersion = this._apiVersion;
                clonedClient._longRunningOperationInitialTimeout = this._longRunningOperationInitialTimeout;
                clonedClient._longRunningOperationRetryTimeout = this._longRunningOperationRetryTimeout;
                
                clonedClient.Credentials.InitializeServiceClient(clonedClient);
            }
        }
        
        /// <summary>
        /// Parse enum values for type FileType.
        /// </summary>
        /// <param name='value'>
        /// The value to parse.
        /// </param>
        /// <returns>
        /// The enum value.
        /// </returns>
        internal static FileType ParseFileType(string value)
        {
            if ("FILE".Equals(value, StringComparison.OrdinalIgnoreCase))
            {
                return FileType.File;
            }
            if ("DIRECTORY".Equals(value, StringComparison.OrdinalIgnoreCase))
            {
                return FileType.Directory;
            }
            throw new ArgumentOutOfRangeException("value");
        }
        
        /// <summary>
        /// Convert an enum of type FileType to a string.
        /// </summary>
        /// <param name='value'>
        /// The value to convert to a string.
        /// </param>
        /// <returns>
        /// The enum value as a string.
        /// </returns>
        internal static string FileTypeToString(FileType value)
        {
            if (value == FileType.File)
            {
                return "FILE";
            }
            if (value == FileType.Directory)
            {
                return "DIRECTORY";
            }
            throw new ArgumentOutOfRangeException("value");
        }
    }
}
