// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Dns.Models;

namespace Microsoft.Azure.Management.Dns
{
    /// <summary>
    /// Operations for managing the RecordSets in a DNS zone.
    /// </summary>
    public partial interface IRecordSetOperations
    {
        /// <summary>
        /// Creates a RecordSet within a DNS zone.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// The name of the RecordSet, relative to the name of the zone.
        /// </param>
        /// <param name='recordType'>
        /// The type of DNS record.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <param name='ifMatch'>
        /// The etag of RecordSet.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Defines the If-None-Match condition. Set to '*' to force
        /// Create-If-Not-Exist. Other values will be ignored.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a RecordSet CreateOrUpdate operation.
        /// </returns>
        Task<RecordSetCreateOrUpdateResponse> CreateOrUpdateAsync(string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, RecordSetCreateOrUpdateParameters parameters, string ifMatch, string ifNoneMatch, CancellationToken cancellationToken);
        
        /// <summary>
        /// Removes a RecordSet from a DNS zone.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// The name of the RecordSet, relative to the name of the zone.
        /// </param>
        /// <param name='recordType'>
        /// The type of DNS record.
        /// </param>
        /// <param name='ifMatch'>
        /// Defines the If-Match condition. The delete operation will be
        /// performed only if the ETag of the zone on the server matches this
        /// value.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Defines the If-None-Match condition. The delete operation will be
        /// performed only if the ETag of the zone on the server does not
        /// match this value.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, string ifMatch, string ifNoneMatch, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets a RecordSet.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// The name of the RecordSet, relative to the name of the zone.
        /// </param>
        /// <param name='recordType'>
        /// The type of DNS record.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a RecordSet Get operation.
        /// </returns>
        Task<RecordSetGetResponse> GetAsync(string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, CancellationToken cancellationToken);
        
        /// <summary>
        /// Lists the RecordSets of a specified type in a DNS zone.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the zone.
        /// </param>
        /// <param name='zoneName'>
        /// The name of the zone from which to enumerate RecordsSets.
        /// </param>
        /// <param name='recordType'>
        /// The type of record sets to enumerate.
        /// </param>
        /// <param name='parameters'>
        /// Query parameters. If null is passed returns the default number of
        /// zones.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        Task<RecordSetListResponse> ListAsync(string resourceGroupName, string zoneName, RecordType recordType, RecordSetListParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Lists all RecordSets in a DNS zone.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group that contains the zone.
        /// </param>
        /// <param name='zoneName'>
        /// The name of the zone from which to enumerate RecordSets.
        /// </param>
        /// <param name='parameters'>
        /// Query parameters. If null is passed returns the default number of
        /// zones.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        Task<RecordSetListResponse> ListAllAsync(string resourceGroupName, string zoneName, RecordSetListParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Lists RecordSets in a DNS zone. Depending on the previous call, it
        /// will list all types or by type.
        /// </summary>
        /// <param name='nextLink'>
        /// NextLink from the previous successful call to List operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        Task<RecordSetListResponse> ListNextAsync(string nextLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Creates a RecordSet within a DNS zone.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// The name of the RecordSet, relative to the name of the zone.
        /// </param>
        /// <param name='recordType'>
        /// The type of DNS record.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <param name='ifMatch'>
        /// The etag of Zone.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Defines the If-None-Match condition. Set to '*' to force
        /// Create-If-Not-Exist. Other values will be ignored.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a RecordSet Update operation.
        /// </returns>
        Task<RecordSetUpdateResponse> UpdateAsync(string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, RecordSetUpdateParameters parameters, string ifMatch, string ifNoneMatch, CancellationToken cancellationToken);
    }
}
