// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Dns;
using Microsoft.Azure.Management.Dns.Models;

namespace Microsoft.Azure.Management.Dns
{
    /// <summary>
    /// Client for managing DNS zones and record.
    /// </summary>
    public static partial class RecordSetOperationsExtensions
    {
        /// <summary>
        /// Creates a RecordSet within a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <param name='ifMatch'>
        /// Optional. The etag of RecordSet.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Optional. Defines the If-None-Match condition. Set to '*' to force
        /// Create-If-Not-Exist. Other values will be ignored.
        /// </param>
        /// <returns>
        /// The response to a RecordSet CreateOrUpdate operation.
        /// </returns>
        public static RecordSetCreateOrUpdateResponse CreateOrUpdate(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, RecordSetCreateOrUpdateParameters parameters, string ifMatch, string ifNoneMatch)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecordSetOperations)s).CreateOrUpdateAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType, parameters, ifMatch, ifNoneMatch);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates a RecordSet within a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <param name='ifMatch'>
        /// Optional. The etag of RecordSet.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Optional. Defines the If-None-Match condition. Set to '*' to force
        /// Create-If-Not-Exist. Other values will be ignored.
        /// </param>
        /// <returns>
        /// The response to a RecordSet CreateOrUpdate operation.
        /// </returns>
        public static Task<RecordSetCreateOrUpdateResponse> CreateOrUpdateAsync(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, RecordSetCreateOrUpdateParameters parameters, string ifMatch, string ifNoneMatch)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType, parameters, ifMatch, ifNoneMatch, CancellationToken.None);
        }
        
        /// <summary>
        /// Removes a RecordSet from a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <param name='ifMatch'>
        /// Optional. Defines the If-Match condition. The delete operation will
        /// be performed only if the ETag of the zone on the server matches
        /// this value.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Optional. Defines the If-None-Match condition. The delete operation
        /// will be performed only if the ETag of the zone on the server does
        /// not match this value.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, string ifMatch, string ifNoneMatch)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecordSetOperations)s).DeleteAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType, ifMatch, ifNoneMatch);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Removes a RecordSet from a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <param name='ifMatch'>
        /// Optional. Defines the If-Match condition. The delete operation will
        /// be performed only if the ETag of the zone on the server matches
        /// this value.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Optional. Defines the If-None-Match condition. The delete operation
        /// will be performed only if the ETag of the zone on the server does
        /// not match this value.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, string ifMatch, string ifNoneMatch)
        {
            return operations.DeleteAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType, ifMatch, ifNoneMatch, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a RecordSet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <returns>
        /// The response to a RecordSet Get operation.
        /// </returns>
        public static RecordSetGetResponse Get(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecordSetOperations)s).GetAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a RecordSet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <returns>
        /// The response to a RecordSet Get operation.
        /// </returns>
        public static Task<RecordSetGetResponse> GetAsync(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType)
        {
            return operations.GetAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType, CancellationToken.None);
        }
        
        /// <summary>
        /// Lists the RecordSets of a specified type in a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group that contains the zone.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone from which to enumerate RecordsSets.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of record sets to enumerate.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Query parameters. If null is passed returns the default
        /// number of zones.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        public static RecordSetListResponse List(this IRecordSetOperations operations, string resourceGroupName, string zoneName, RecordType recordType, RecordSetListParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecordSetOperations)s).ListAsync(resourceGroupName, zoneName, recordType, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists the RecordSets of a specified type in a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group that contains the zone.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone from which to enumerate RecordsSets.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of record sets to enumerate.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Query parameters. If null is passed returns the default
        /// number of zones.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        public static Task<RecordSetListResponse> ListAsync(this IRecordSetOperations operations, string resourceGroupName, string zoneName, RecordType recordType, RecordSetListParameters parameters)
        {
            return operations.ListAsync(resourceGroupName, zoneName, recordType, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Lists all RecordSets in a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group that contains the zone.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone from which to enumerate RecordSets.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Query parameters. If null is passed returns the default
        /// number of zones.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        public static RecordSetListResponse ListAll(this IRecordSetOperations operations, string resourceGroupName, string zoneName, RecordSetListParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecordSetOperations)s).ListAllAsync(resourceGroupName, zoneName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists all RecordSets in a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group that contains the zone.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone from which to enumerate RecordSets.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Query parameters. If null is passed returns the default
        /// number of zones.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        public static Task<RecordSetListResponse> ListAllAsync(this IRecordSetOperations operations, string resourceGroupName, string zoneName, RecordSetListParameters parameters)
        {
            return operations.ListAllAsync(resourceGroupName, zoneName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Lists RecordSets in a DNS zone. Depending on the previous call, it
        /// will list all types or by type.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. NextLink from the previous successful call to List
        /// operation.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        public static RecordSetListResponse ListNext(this IRecordSetOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecordSetOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Lists RecordSets in a DNS zone. Depending on the previous call, it
        /// will list all types or by type.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. NextLink from the previous successful call to List
        /// operation.
        /// </param>
        /// <returns>
        /// The response to a RecordSet List operation.
        /// </returns>
        public static Task<RecordSetListResponse> ListNextAsync(this IRecordSetOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
        
        /// <summary>
        /// Creates a RecordSet within a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <param name='ifMatch'>
        /// Optional. The etag of Zone.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Optional. Defines the If-None-Match condition. Set to '*' to force
        /// Create-If-Not-Exist. Other values will be ignored.
        /// </param>
        /// <returns>
        /// The response to a RecordSet Update operation.
        /// </returns>
        public static RecordSetUpdateResponse Update(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, RecordSetUpdateParameters parameters, string ifMatch, string ifNoneMatch)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRecordSetOperations)s).UpdateAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType, parameters, ifMatch, ifNoneMatch);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates a RecordSet within a DNS zone.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Dns.IRecordSetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='zoneName'>
        /// Required. The name of the zone without a terminating dot.
        /// </param>
        /// <param name='relativeRecordSetName'>
        /// Required. The name of the RecordSet, relative to the name of the
        /// zone.
        /// </param>
        /// <param name='recordType'>
        /// Required. The type of DNS record.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate operation.
        /// </param>
        /// <param name='ifMatch'>
        /// Optional. The etag of Zone.
        /// </param>
        /// <param name='ifNoneMatch'>
        /// Optional. Defines the If-None-Match condition. Set to '*' to force
        /// Create-If-Not-Exist. Other values will be ignored.
        /// </param>
        /// <returns>
        /// The response to a RecordSet Update operation.
        /// </returns>
        public static Task<RecordSetUpdateResponse> UpdateAsync(this IRecordSetOperations operations, string resourceGroupName, string zoneName, string relativeRecordSetName, RecordType recordType, RecordSetUpdateParameters parameters, string ifMatch, string ifNoneMatch)
        {
            return operations.UpdateAsync(resourceGroupName, zoneName, relativeRecordSetName, recordType, parameters, ifMatch, ifNoneMatch, CancellationToken.None);
        }
    }
}
