// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Windows Azure Network management API provides a RESTful set of web
    /// services that interact with Windows Azure Networks service to manage
    /// your network resrources. The API has entities that capture the
    /// relationship between an end user and the Windows Azure Networks
    /// service.
    /// </summary>
    public static partial class ExpressRouteCircuitAuthorizationOperationsExtensions
    {
        /// <summary>
        /// The Put Authorization operation creates/updates an authorization in
        /// thespecified ExpressRouteCircuits
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <param name='authorizationParameters'>
        /// Required. Parameters supplied to the create/update
        /// ExpressRouteCircuitAuthorization operation
        /// </param>
        /// <returns>
        /// Response for Put ExpressRouteCircuit Authorization Api service call
        /// </returns>
        public static AuthorizationPutResponse BeginCreateOrUpdating(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName, ExpressRouteCircuitAuthorization authorizationParameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitAuthorizationOperations)s).BeginCreateOrUpdatingAsync(resourceGroupName, circuitName, authorizationName, authorizationParameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put Authorization operation creates/updates an authorization in
        /// thespecified ExpressRouteCircuits
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <param name='authorizationParameters'>
        /// Required. Parameters supplied to the create/update
        /// ExpressRouteCircuitAuthorization operation
        /// </param>
        /// <returns>
        /// Response for Put ExpressRouteCircuit Authorization Api service call
        /// </returns>
        public static Task<AuthorizationPutResponse> BeginCreateOrUpdatingAsync(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName, ExpressRouteCircuitAuthorization authorizationParameters)
        {
            return operations.BeginCreateOrUpdatingAsync(resourceGroupName, circuitName, authorizationName, authorizationParameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The delete authorization operation deletes the specified
        /// authorization from the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static UpdateOperationResponse BeginDeleting(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitAuthorizationOperations)s).BeginDeletingAsync(resourceGroupName, circuitName, authorizationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The delete authorization operation deletes the specified
        /// authorization from the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static Task<UpdateOperationResponse> BeginDeletingAsync(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName)
        {
            return operations.BeginDeletingAsync(resourceGroupName, circuitName, authorizationName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Put Authorization operation creates/updates an authorization in
        /// thespecified ExpressRouteCircuits
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <param name='authorizationParameters'>
        /// Required. Parameters supplied to the create/update
        /// ExpressRouteCircuitAuthorization operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse CreateOrUpdate(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName, ExpressRouteCircuitAuthorization authorizationParameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitAuthorizationOperations)s).CreateOrUpdateAsync(resourceGroupName, circuitName, authorizationName, authorizationParameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put Authorization operation creates/updates an authorization in
        /// thespecified ExpressRouteCircuits
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <param name='authorizationParameters'>
        /// Required. Parameters supplied to the create/update
        /// ExpressRouteCircuitAuthorization operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName, ExpressRouteCircuitAuthorization authorizationParameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, circuitName, authorizationName, authorizationParameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The delete authorization operation deletes the specified
        /// authorization from the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitAuthorizationOperations)s).DeleteAsync(resourceGroupName, circuitName, authorizationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The delete authorization operation deletes the specified
        /// authorization from the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName)
        {
            return operations.DeleteAsync(resourceGroupName, circuitName, authorizationName, CancellationToken.None);
        }
        
        /// <summary>
        /// The GET authorization operation retrieves the specified
        /// authorization from the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <returns>
        /// Response for GET ExpressRouteCircuit Authorization Api service call
        /// </returns>
        public static AuthorizationGetResponse Get(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitAuthorizationOperations)s).GetAsync(resourceGroupName, circuitName, authorizationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The GET authorization operation retrieves the specified
        /// authorization from the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <param name='authorizationName'>
        /// Required. The name of the authorization.
        /// </param>
        /// <returns>
        /// Response for GET ExpressRouteCircuit Authorization Api service call
        /// </returns>
        public static Task<AuthorizationGetResponse> GetAsync(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName, string authorizationName)
        {
            return operations.GetAsync(resourceGroupName, circuitName, authorizationName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List authorization operation retrieves all the authorizations
        /// in an ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the curcuit.
        /// </param>
        /// <returns>
        /// Response for ListAuthorizations Api service callRetrieves all
        /// authorizations that belongs to an ExpressRouteCircuit
        /// </returns>
        public static AuthorizationListResponse List(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitAuthorizationOperations)s).ListAsync(resourceGroupName, circuitName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List authorization operation retrieves all the authorizations
        /// in an ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitAuthorizationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the curcuit.
        /// </param>
        /// <returns>
        /// Response for ListAuthorizations Api service callRetrieves all
        /// authorizations that belongs to an ExpressRouteCircuit
        /// </returns>
        public static Task<AuthorizationListResponse> ListAsync(this IExpressRouteCircuitAuthorizationOperations operations, string resourceGroupName, string circuitName)
        {
            return operations.ListAsync(resourceGroupName, circuitName, CancellationToken.None);
        }
    }
}
