// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Windows Azure Network management API provides a RESTful set of web
    /// services that interact with Windows Azure Networks service to manage
    /// your network resrources. The API has entities that capture the
    /// relationship between an end user and the Windows Azure Networks
    /// service.
    /// </summary>
    public static partial class ExpressRouteCircuitOperationsExtensions
    {
        /// <summary>
        /// The Put ExpressRouteCircuit operation creates/updates a
        /// ExpressRouteCircuit
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the create/delete
        /// ExpressRouteCircuit operation
        /// </param>
        /// <returns>
        /// Response of a PUT ExpressRouteCircuit operation
        /// </returns>
        public static ExpressRouteCircuitPutResponse BeginCreateOrUpdating(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName, ExpressRouteCircuit parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).BeginCreateOrUpdatingAsync(resourceGroupName, circuitName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put ExpressRouteCircuit operation creates/updates a
        /// ExpressRouteCircuit
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the create/delete
        /// ExpressRouteCircuit operation
        /// </param>
        /// <returns>
        /// Response of a PUT ExpressRouteCircuit operation
        /// </returns>
        public static Task<ExpressRouteCircuitPutResponse> BeginCreateOrUpdatingAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName, ExpressRouteCircuit parameters)
        {
            return operations.BeginCreateOrUpdatingAsync(resourceGroupName, circuitName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The delete ExpressRouteCircuit operation deletes the specified
        /// ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route Circuit.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static UpdateOperationResponse BeginDeleting(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).BeginDeletingAsync(resourceGroupName, circuitName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The delete ExpressRouteCircuit operation deletes the specified
        /// ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route Circuit.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static Task<UpdateOperationResponse> BeginDeletingAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return operations.BeginDeletingAsync(resourceGroupName, circuitName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Put ExpressRouteCircuit operation creates/updates a
        /// ExpressRouteCircuit
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the ExpressRouteCircuit.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the create/update
        /// ExpressRouteCircuit operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse CreateOrUpdate(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName, ExpressRouteCircuit parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).CreateOrUpdateAsync(resourceGroupName, circuitName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put ExpressRouteCircuit operation creates/updates a
        /// ExpressRouteCircuit
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the ExpressRouteCircuit.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the create/update
        /// ExpressRouteCircuit operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName, ExpressRouteCircuit parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, circuitName, parameters, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).DeleteAsync(resourceGroupName, circuitName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the express route circuit.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return operations.DeleteAsync(resourceGroupName, circuitName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get ExpressRouteCircuit operation retreives information about
        /// the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <returns>
        /// Response of a GET ExpressRouteCircuit operation
        /// </returns>
        public static ExpressRouteCircuitGetResponse Get(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).GetAsync(resourceGroupName, circuitName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get ExpressRouteCircuit operation retreives information about
        /// the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <returns>
        /// Response of a GET ExpressRouteCircuit operation
        /// </returns>
        public static Task<ExpressRouteCircuitGetResponse> GetAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return operations.GetAsync(resourceGroupName, circuitName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List ExpressRouteCircuit opertion retrieves all the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// Response for ListExpressRouteCircuit Api service call
        /// </returns>
        public static ExpressRouteCircuitListResponse List(this IExpressRouteCircuitOperations operations, string resourceGroupName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).ListAsync(resourceGroupName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List ExpressRouteCircuit opertion retrieves all the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// Response for ListExpressRouteCircuit Api service call
        /// </returns>
        public static Task<ExpressRouteCircuitListResponse> ListAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName)
        {
            return operations.ListAsync(resourceGroupName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List ExpressRouteCircuit opertion retrieves all the
        /// ExpressRouteCircuits in a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <returns>
        /// Response for ListExpressRouteCircuit Api service call
        /// </returns>
        public static ExpressRouteCircuitListResponse ListAll(this IExpressRouteCircuitOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).ListAllAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List ExpressRouteCircuit opertion retrieves all the
        /// ExpressRouteCircuits in a subscription.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <returns>
        /// Response for ListExpressRouteCircuit Api service call
        /// </returns>
        public static Task<ExpressRouteCircuitListResponse> ListAllAsync(this IExpressRouteCircuitOperations operations)
        {
            return operations.ListAllAsync(CancellationToken.None);
        }
        
        /// <summary>
        /// The ListArpTable from ExpressRouteCircuit opertion retrieves the
        /// currently advertised arp table associated with the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <returns>
        /// Response for ListArpTable associated with the Express Route
        /// Circuits Api
        /// </returns>
        public static ExpressRouteCircuitsArpTableListResponse ListArpTable(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).ListArpTableAsync(resourceGroupName, circuitName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The ListArpTable from ExpressRouteCircuit opertion retrieves the
        /// currently advertised arp table associated with the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <returns>
        /// Response for ListArpTable associated with the Express Route
        /// Circuits Api
        /// </returns>
        public static Task<ExpressRouteCircuitsArpTableListResponse> ListArpTableAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return operations.ListArpTableAsync(resourceGroupName, circuitName, CancellationToken.None);
        }
        
        /// <summary>
        /// The ListRoutesTable from ExpressRouteCircuit opertion retrieves the
        /// currently advertised routes table associated with the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <returns>
        /// Response for ListRoutesTable associated with the Express Route
        /// Circuits Api
        /// </returns>
        public static ExpressRouteCircuitsRoutesTableListResponse ListRoutesTable(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).ListRoutesTableAsync(resourceGroupName, circuitName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The ListRoutesTable from ExpressRouteCircuit opertion retrieves the
        /// currently advertised routes table associated with the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the circuit.
        /// </param>
        /// <returns>
        /// Response for ListRoutesTable associated with the Express Route
        /// Circuits Api
        /// </returns>
        public static Task<ExpressRouteCircuitsRoutesTableListResponse> ListRoutesTableAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return operations.ListRoutesTableAsync(resourceGroupName, circuitName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Liststats ExpressRouteCircuit opertion retrieves all the stats
        /// from a ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the loadBalancer.
        /// </param>
        /// <returns>
        /// Response for ListStats from Express Route Circuits Api service call
        /// </returns>
        public static ExpressRouteCircuitsStatsListResponse ListStats(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IExpressRouteCircuitOperations)s).ListStatsAsync(resourceGroupName, circuitName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Liststats ExpressRouteCircuit opertion retrieves all the stats
        /// from a ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IExpressRouteCircuitOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// Required. The name of the loadBalancer.
        /// </param>
        /// <returns>
        /// Response for ListStats from Express Route Circuits Api service call
        /// </returns>
        public static Task<ExpressRouteCircuitsStatsListResponse> ListStatsAsync(this IExpressRouteCircuitOperations operations, string resourceGroupName, string circuitName)
        {
            return operations.ListStatsAsync(resourceGroupName, circuitName, CancellationToken.None);
        }
    }
}
