// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Network Resource Provider API includes operations for managing the
    /// ExpressRouteCircuits for your subscription.
    /// </summary>
    public partial interface IExpressRouteCircuitOperations
    {
        /// <summary>
        /// The Put ExpressRouteCircuit operation creates/updates a
        /// ExpressRouteCircuit
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the circuit.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the create/delete ExpressRouteCircuit
        /// operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response of a PUT ExpressRouteCircuit operation
        /// </returns>
        Task<ExpressRouteCircuitPutResponse> BeginCreateOrUpdatingAsync(string resourceGroupName, string circuitName, ExpressRouteCircuit parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// The delete ExpressRouteCircuit operation deletes the specified
        /// ExpressRouteCircuit.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the express route Circuit.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        Task<UpdateOperationResponse> BeginDeletingAsync(string resourceGroupName, string circuitName, CancellationToken cancellationToken);
        
        /// <summary>
        /// The Put ExpressRouteCircuit operation creates/updates a
        /// ExpressRouteCircuit
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the ExpressRouteCircuit.
        /// </param>
        /// <param name='parameters'>
        /// Parameters supplied to the create/update ExpressRouteCircuit
        /// operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(string resourceGroupName, string circuitName, ExpressRouteCircuit parameters, CancellationToken cancellationToken);
        
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the express route circuit.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string circuitName, CancellationToken cancellationToken);
        
        /// <summary>
        /// The Get ExpressRouteCircuit operation retreives information about
        /// the specified ExpressRouteCircuit.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the circuit.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response of a GET ExpressRouteCircuit operation
        /// </returns>
        Task<ExpressRouteCircuitGetResponse> GetAsync(string resourceGroupName, string circuitName, CancellationToken cancellationToken);
        
        /// <summary>
        /// The List ExpressRouteCircuit opertion retrieves all the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for ListExpressRouteCircuit Api service call
        /// </returns>
        Task<ExpressRouteCircuitListResponse> ListAsync(string resourceGroupName, CancellationToken cancellationToken);
        
        /// <summary>
        /// The List ExpressRouteCircuit opertion retrieves all the
        /// ExpressRouteCircuits in a subscription.
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for ListExpressRouteCircuit Api service call
        /// </returns>
        Task<ExpressRouteCircuitListResponse> ListAllAsync(CancellationToken cancellationToken);
        
        /// <summary>
        /// The ListArpTable from ExpressRouteCircuit opertion retrieves the
        /// currently advertised arp table associated with the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the circuit.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for ListArpTable associated with the Express Route
        /// Circuits Api
        /// </returns>
        Task<ExpressRouteCircuitsArpTableListResponse> ListArpTableAsync(string resourceGroupName, string circuitName, CancellationToken cancellationToken);
        
        /// <summary>
        /// The ListRoutesTable from ExpressRouteCircuit opertion retrieves the
        /// currently advertised routes table associated with the
        /// ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the circuit.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for ListRoutesTable associated with the Express Route
        /// Circuits Api
        /// </returns>
        Task<ExpressRouteCircuitsRoutesTableListResponse> ListRoutesTableAsync(string resourceGroupName, string circuitName, CancellationToken cancellationToken);
        
        /// <summary>
        /// The Liststats ExpressRouteCircuit opertion retrieves all the stats
        /// from a ExpressRouteCircuits in a resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group.
        /// </param>
        /// <param name='circuitName'>
        /// The name of the loadBalancer.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for ListStats from Express Route Circuits Api service call
        /// </returns>
        Task<ExpressRouteCircuitsStatsListResponse> ListStatsAsync(string resourceGroupName, string circuitName, CancellationToken cancellationToken);
    }
}
