// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Windows Azure Network management API provides a RESTful set of web
    /// services that interact with Windows Azure Networks service to manage
    /// your network resrources. The API has entities that capture the
    /// relationship between an end user and the Windows Azure Networks
    /// service.
    /// </summary>
    public static partial class LocalNetworkGatewayOperationsExtensions
    {
        /// <summary>
        /// The Put LocalNetworkGateway operation creates/updates a local
        /// network gateway in the specified resource group through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Local
        /// Network Gateway operation through Network resource provider.
        /// </param>
        /// <returns>
        /// Response for PutLocalNetworkGateway Api servive call
        /// </returns>
        public static LocalNetworkGatewayPutResponse BeginCreateOrUpdating(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName, LocalNetworkGateway parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILocalNetworkGatewayOperations)s).BeginCreateOrUpdatingAsync(resourceGroupName, localNetworkGatewayName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put LocalNetworkGateway operation creates/updates a local
        /// network gateway in the specified resource group through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Local
        /// Network Gateway operation through Network resource provider.
        /// </param>
        /// <returns>
        /// Response for PutLocalNetworkGateway Api servive call
        /// </returns>
        public static Task<LocalNetworkGatewayPutResponse> BeginCreateOrUpdatingAsync(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName, LocalNetworkGateway parameters)
        {
            return operations.BeginCreateOrUpdatingAsync(resourceGroupName, localNetworkGatewayName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Delete LocalNetworkGateway operation deletes the specifed local
        /// network Gateway through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static UpdateOperationResponse BeginDeleting(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILocalNetworkGatewayOperations)s).BeginDeletingAsync(resourceGroupName, localNetworkGatewayName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Delete LocalNetworkGateway operation deletes the specifed local
        /// network Gateway through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static Task<UpdateOperationResponse> BeginDeletingAsync(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName)
        {
            return operations.BeginDeletingAsync(resourceGroupName, localNetworkGatewayName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Put LocalNetworkGateway operation creates/updates a local
        /// network gateway in the specified resource group through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Local
        /// Network Gateway operation through Network resource provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse CreateOrUpdate(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName, LocalNetworkGateway parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILocalNetworkGatewayOperations)s).CreateOrUpdateAsync(resourceGroupName, localNetworkGatewayName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put LocalNetworkGateway operation creates/updates a local
        /// network gateway in the specified resource group through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Local
        /// Network Gateway operation through Network resource provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName, LocalNetworkGateway parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, localNetworkGatewayName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Delete LocalNetworkGateway operation deletes the specifed local
        /// network Gateway through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILocalNetworkGatewayOperations)s).DeleteAsync(resourceGroupName, localNetworkGatewayName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Delete LocalNetworkGateway operation deletes the specifed local
        /// network Gateway through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName)
        {
            return operations.DeleteAsync(resourceGroupName, localNetworkGatewayName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get LocalNetworkGateway operation retrieves information about
        /// the specified local network gateway through Network resource
        /// provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <returns>
        /// Response for GetLocalNetworkgateway Api servive call.
        /// </returns>
        public static LocalNetworkGatewayGetResponse Get(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILocalNetworkGatewayOperations)s).GetAsync(resourceGroupName, localNetworkGatewayName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get LocalNetworkGateway operation retrieves information about
        /// the specified local network gateway through Network resource
        /// provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='localNetworkGatewayName'>
        /// Required. The name of the local network gateway.
        /// </param>
        /// <returns>
        /// Response for GetLocalNetworkgateway Api servive call.
        /// </returns>
        public static Task<LocalNetworkGatewayGetResponse> GetAsync(this ILocalNetworkGatewayOperations operations, string resourceGroupName, string localNetworkGatewayName)
        {
            return operations.GetAsync(resourceGroupName, localNetworkGatewayName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List LocalNetworkGateways opertion retrieves all the local
        /// network gateways stored.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// Response for ListLocalNetworkGateways Api service call
        /// </returns>
        public static LocalNetworkGatewayListResponse List(this ILocalNetworkGatewayOperations operations, string resourceGroupName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILocalNetworkGatewayOperations)s).ListAsync(resourceGroupName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List LocalNetworkGateways opertion retrieves all the local
        /// network gateways stored.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ILocalNetworkGatewayOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// Response for ListLocalNetworkGateways Api service call
        /// </returns>
        public static Task<LocalNetworkGatewayListResponse> ListAsync(this ILocalNetworkGatewayOperations operations, string resourceGroupName)
        {
            return operations.ListAsync(resourceGroupName, CancellationToken.None);
        }
    }
}
