// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using Hyak.Common;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network.Models
{
    /// <summary>
    /// Load balancer Probe
    /// </summary>
    public partial class Probe : ChildResource
    {
        private int _intervalInSeconds;
        
        /// <summary>
        /// Optional. Gets or sets the interval, in seconds, for how frequently
        /// to probe the endpoint for health status. Typically, the interval
        /// is slightly less than half the allocated timeout period (in
        /// seconds) which allows two full probes before taking the instance
        /// out of rotation. The default value is 15, the minimum value is 5
        /// </summary>
        public int IntervalInSeconds
        {
            get { return this._intervalInSeconds; }
            set { this._intervalInSeconds = value; }
        }
        
        private IList<ResourceId> _loadBalancingRules;
        
        /// <summary>
        /// Optional. Gets Load balancer rules that use this probe
        /// </summary>
        public IList<ResourceId> LoadBalancingRules
        {
            get { return this._loadBalancingRules; }
            set { this._loadBalancingRules = value; }
        }
        
        private int _numberOfProbes;
        
        /// <summary>
        /// Optional. Gets or sets the number of probes where if no response,
        /// will result in stopping further traffic from being delivered to
        /// the endpoint. This values allows endponints to be taken out of
        /// rotation faster or slower than the typical times used in Azure.
        /// </summary>
        public int NumberOfProbes
        {
            get { return this._numberOfProbes; }
            set { this._numberOfProbes = value; }
        }
        
        private int _port;
        
        /// <summary>
        /// Required. Gets or sets Port for communicating the probe. Possible
        /// values range from 1 to 65535, inclusive.
        /// </summary>
        public int Port
        {
            get { return this._port; }
            set { this._port = value; }
        }
        
        private string _protocol;
        
        /// <summary>
        /// Required. Gets or sets the protocol of the end point. Possible
        /// values are http pr Tcp. If Tcp is specified, a received ACK is
        /// required for the probe to be successful. If http is specified,a
        /// 200 OK response from the specifies URI is required for the probe
        /// to be successful
        /// </summary>
        public string Protocol
        {
            get { return this._protocol; }
            set { this._protocol = value; }
        }
        
        private string _provisioningState;
        
        /// <summary>
        /// Optional. Gets or sets Provisioning state of the PublicIP resource
        /// Updating/Deleting/Failed
        /// </summary>
        public string ProvisioningState
        {
            get { return this._provisioningState; }
            set { this._provisioningState = value; }
        }
        
        private string _requestPath;
        
        /// <summary>
        /// Optional. Gets or sets the URI used for requesting health status
        /// from the VM. Path is required if a protocol is set to http.
        /// Otherwise, it is not allowed. There is no default value
        /// </summary>
        public string RequestPath
        {
            get { return this._requestPath; }
            set { this._requestPath = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the Probe class.
        /// </summary>
        public Probe()
        {
            this.LoadBalancingRules = new LazyList<ResourceId>();
        }
        
        /// <summary>
        /// Initializes a new instance of the Probe class with required
        /// arguments.
        /// </summary>
        public Probe(string protocol, int port)
            : this()
        {
            if (protocol == null)
            {
                throw new ArgumentNullException("protocol");
            }
            this.Protocol = protocol;
            this.Port = port;
        }
    }
}
