// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Windows Azure Network management API provides a RESTful set of web
    /// services that interact with Windows Azure Networks service to manage
    /// your network resrources. The API has entities that capture the
    /// relationship between an end user and the Windows Azure Networks
    /// service.
    /// </summary>
    public partial class NetworkResourceProviderClient : ServiceClient<NetworkResourceProviderClient>, INetworkResourceProviderClient
    {
        private string _apiVersion;
        
        /// <summary>
        /// Gets the API version.
        /// </summary>
        public string ApiVersion
        {
            get { return this._apiVersion; }
        }
        
        private Uri _baseUri;
        
        /// <summary>
        /// Gets the URI used as the base for all cloud service requests.
        /// </summary>
        public Uri BaseUri
        {
            get { return this._baseUri; }
        }
        
        private SubscriptionCloudCredentials _credentials;
        
        /// <summary>
        /// Gets subscription credentials which uniquely identify Microsoft
        /// Azure subscription. The subscription ID forms part of the URI for
        /// every service call.
        /// </summary>
        public SubscriptionCloudCredentials Credentials
        {
            get { return this._credentials; }
        }
        
        private int _longRunningOperationInitialTimeout;
        
        /// <summary>
        /// Gets or sets the initial timeout for Long Running Operations.
        /// </summary>
        public int LongRunningOperationInitialTimeout
        {
            get { return this._longRunningOperationInitialTimeout; }
            set { this._longRunningOperationInitialTimeout = value; }
        }
        
        private int _longRunningOperationRetryTimeout;
        
        /// <summary>
        /// Gets or sets the retry timeout for Long Running Operations.
        /// </summary>
        public int LongRunningOperationRetryTimeout
        {
            get { return this._longRunningOperationRetryTimeout; }
            set { this._longRunningOperationRetryTimeout = value; }
        }
        
        private IApplicationGatewayOperations _applicationGateways;
        
        /// <summary>
        /// The Network Resource Provider API includes operations managing the
        /// application gateways for your subscription.
        /// </summary>
        public virtual IApplicationGatewayOperations ApplicationGateways
        {
            get { return this._applicationGateways; }
        }
        
        private IExpressRouteCircuitAuthorizationOperations _expressRouteCircuitAuthorizations;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the Authorizations for your subscription.
        /// </summary>
        public virtual IExpressRouteCircuitAuthorizationOperations ExpressRouteCircuitAuthorizations
        {
            get { return this._expressRouteCircuitAuthorizations; }
        }
        
        private IExpressRouteCircuitOperations _expressRouteCircuits;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the ExpressRouteCircuits for your subscription.
        /// </summary>
        public virtual IExpressRouteCircuitOperations ExpressRouteCircuits
        {
            get { return this._expressRouteCircuits; }
        }
        
        private IExpressRouteCircuitPeeringOperations _expressRouteCircuitPeerings;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the Peerings for your subscription.
        /// </summary>
        public virtual IExpressRouteCircuitPeeringOperations ExpressRouteCircuitPeerings
        {
            get { return this._expressRouteCircuitPeerings; }
        }
        
        private IExpressRouteServiceProviderOperations _expressRouteServiceProviders;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the ExpressRouteServiceProviders for your subscription.
        /// </summary>
        public virtual IExpressRouteServiceProviderOperations ExpressRouteServiceProviders
        {
            get { return this._expressRouteServiceProviders; }
        }
        
        private ILoadBalancerOperations _loadBalancers;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the load balancers for your subscription.
        /// </summary>
        public virtual ILoadBalancerOperations LoadBalancers
        {
            get { return this._loadBalancers; }
        }
        
        private ILocalNetworkGatewayOperations _localNetworkGateways;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the Virtual network Gateway for your subscription.
        /// </summary>
        public virtual ILocalNetworkGatewayOperations LocalNetworkGateways
        {
            get { return this._localNetworkGateways; }
        }
        
        private INetworkInterfaceOperations _networkInterfaces;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the subnets for your subscription.
        /// </summary>
        public virtual INetworkInterfaceOperations NetworkInterfaces
        {
            get { return this._networkInterfaces; }
        }
        
        private INetworkSecurityGroupOperations _networkSecurityGroups;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the NetworkSecurityGroups for your subscription.
        /// </summary>
        public virtual INetworkSecurityGroupOperations NetworkSecurityGroups
        {
            get { return this._networkSecurityGroups; }
        }
        
        private IPublicIpAddressOperations _publicIpAddresses;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the PublicIPAddress for your subscription.
        /// </summary>
        public virtual IPublicIpAddressOperations PublicIpAddresses
        {
            get { return this._publicIpAddresses; }
        }
        
        private IRouteOperations _routes;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the Routes for your subscription.
        /// </summary>
        public virtual IRouteOperations Routes
        {
            get { return this._routes; }
        }
        
        private IRouteTableOperations _routeTables;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the RouteTables for your subscription.
        /// </summary>
        public virtual IRouteTableOperations RouteTables
        {
            get { return this._routeTables; }
        }
        
        private ISecurityRuleOperations _securityRules;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the SecurityRules for your subscription.
        /// </summary>
        public virtual ISecurityRuleOperations SecurityRules
        {
            get { return this._securityRules; }
        }
        
        private ISubnetOperations _subnets;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the subnets for your subscription.
        /// </summary>
        public virtual ISubnetOperations Subnets
        {
            get { return this._subnets; }
        }
        
        private IUsageOperations _usages;
        
        /// <summary>
        /// Operations for listing usage.
        /// </summary>
        public virtual IUsageOperations Usages
        {
            get { return this._usages; }
        }
        
        private IVirtualNetworkGatewayConnectionOperations _virtualNetworkGatewayConnections;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the Virtual network Gateway for your subscription.
        /// </summary>
        public virtual IVirtualNetworkGatewayConnectionOperations VirtualNetworkGatewayConnections
        {
            get { return this._virtualNetworkGatewayConnections; }
        }
        
        private IVirtualNetworkGatewayOperations _virtualNetworkGateways;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the Virtual network Gateway for your subscription.
        /// </summary>
        public virtual IVirtualNetworkGatewayOperations VirtualNetworkGateways
        {
            get { return this._virtualNetworkGateways; }
        }
        
        private IVirtualNetworkOperations _virtualNetworks;
        
        /// <summary>
        /// The Network Resource Provider API includes operations for managing
        /// the Virtual Networks for your subscription.
        /// </summary>
        public virtual IVirtualNetworkOperations VirtualNetworks
        {
            get { return this._virtualNetworks; }
        }
        
        /// <summary>
        /// Initializes a new instance of the NetworkResourceProviderClient
        /// class.
        /// </summary>
        public NetworkResourceProviderClient()
            : base()
        {
            this._applicationGateways = new ApplicationGatewayOperations(this);
            this._expressRouteCircuitAuthorizations = new ExpressRouteCircuitAuthorizationOperations(this);
            this._expressRouteCircuits = new ExpressRouteCircuitOperations(this);
            this._expressRouteCircuitPeerings = new ExpressRouteCircuitPeeringOperations(this);
            this._expressRouteServiceProviders = new ExpressRouteServiceProviderOperations(this);
            this._loadBalancers = new LoadBalancerOperations(this);
            this._localNetworkGateways = new LocalNetworkGatewayOperations(this);
            this._networkInterfaces = new NetworkInterfaceOperations(this);
            this._networkSecurityGroups = new NetworkSecurityGroupOperations(this);
            this._publicIpAddresses = new PublicIpAddressOperations(this);
            this._routes = new RouteOperations(this);
            this._routeTables = new RouteTableOperations(this);
            this._securityRules = new SecurityRuleOperations(this);
            this._subnets = new SubnetOperations(this);
            this._usages = new UsageOperations(this);
            this._virtualNetworkGatewayConnections = new VirtualNetworkGatewayConnectionOperations(this);
            this._virtualNetworkGateways = new VirtualNetworkGatewayOperations(this);
            this._virtualNetworks = new VirtualNetworkOperations(this);
            this._apiVersion = "2015-05-01-preview";
            this._longRunningOperationInitialTimeout = -1;
            this._longRunningOperationRetryTimeout = -1;
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the NetworkResourceProviderClient
        /// class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        /// <param name='baseUri'>
        /// Optional. Gets the URI used as the base for all cloud service
        /// requests.
        /// </param>
        public NetworkResourceProviderClient(SubscriptionCloudCredentials credentials, Uri baseUri)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._credentials = credentials;
            this._baseUri = baseUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the NetworkResourceProviderClient
        /// class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        public NetworkResourceProviderClient(SubscriptionCloudCredentials credentials)
            : this()
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._credentials = credentials;
            this._baseUri = new Uri("https://management.azure.com/");
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the NetworkResourceProviderClient
        /// class.
        /// </summary>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public NetworkResourceProviderClient(HttpClient httpClient)
            : base(httpClient)
        {
            this._applicationGateways = new ApplicationGatewayOperations(this);
            this._expressRouteCircuitAuthorizations = new ExpressRouteCircuitAuthorizationOperations(this);
            this._expressRouteCircuits = new ExpressRouteCircuitOperations(this);
            this._expressRouteCircuitPeerings = new ExpressRouteCircuitPeeringOperations(this);
            this._expressRouteServiceProviders = new ExpressRouteServiceProviderOperations(this);
            this._loadBalancers = new LoadBalancerOperations(this);
            this._localNetworkGateways = new LocalNetworkGatewayOperations(this);
            this._networkInterfaces = new NetworkInterfaceOperations(this);
            this._networkSecurityGroups = new NetworkSecurityGroupOperations(this);
            this._publicIpAddresses = new PublicIpAddressOperations(this);
            this._routes = new RouteOperations(this);
            this._routeTables = new RouteTableOperations(this);
            this._securityRules = new SecurityRuleOperations(this);
            this._subnets = new SubnetOperations(this);
            this._usages = new UsageOperations(this);
            this._virtualNetworkGatewayConnections = new VirtualNetworkGatewayConnectionOperations(this);
            this._virtualNetworkGateways = new VirtualNetworkGatewayOperations(this);
            this._virtualNetworks = new VirtualNetworkOperations(this);
            this._apiVersion = "2015-05-01-preview";
            this._longRunningOperationInitialTimeout = -1;
            this._longRunningOperationRetryTimeout = -1;
            this.HttpClient.Timeout = TimeSpan.FromSeconds(300);
        }
        
        /// <summary>
        /// Initializes a new instance of the NetworkResourceProviderClient
        /// class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        /// <param name='baseUri'>
        /// Optional. Gets the URI used as the base for all cloud service
        /// requests.
        /// </param>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public NetworkResourceProviderClient(SubscriptionCloudCredentials credentials, Uri baseUri, HttpClient httpClient)
            : this(httpClient)
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            if (baseUri == null)
            {
                throw new ArgumentNullException("baseUri");
            }
            this._credentials = credentials;
            this._baseUri = baseUri;
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Initializes a new instance of the NetworkResourceProviderClient
        /// class.
        /// </summary>
        /// <param name='credentials'>
        /// Required. Gets subscription credentials which uniquely identify
        /// Microsoft Azure subscription. The subscription ID forms part of
        /// the URI for every service call.
        /// </param>
        /// <param name='httpClient'>
        /// The Http client
        /// </param>
        public NetworkResourceProviderClient(SubscriptionCloudCredentials credentials, HttpClient httpClient)
            : this(httpClient)
        {
            if (credentials == null)
            {
                throw new ArgumentNullException("credentials");
            }
            this._credentials = credentials;
            this._baseUri = new Uri("https://management.azure.com/");
            
            this.Credentials.InitializeServiceClient(this);
        }
        
        /// <summary>
        /// Clones properties from current instance to another
        /// NetworkResourceProviderClient instance
        /// </summary>
        /// <param name='client'>
        /// Instance of NetworkResourceProviderClient to clone to
        /// </param>
        protected override void Clone(ServiceClient<NetworkResourceProviderClient> client)
        {
            base.Clone(client);
            
            if (client is NetworkResourceProviderClient)
            {
                NetworkResourceProviderClient clonedClient = ((NetworkResourceProviderClient)client);
                
                clonedClient._credentials = this._credentials;
                clonedClient._baseUri = this._baseUri;
                clonedClient._apiVersion = this._apiVersion;
                clonedClient._longRunningOperationInitialTimeout = this._longRunningOperationInitialTimeout;
                clonedClient._longRunningOperationRetryTimeout = this._longRunningOperationRetryTimeout;
                
                clonedClient.Credentials.InitializeServiceClient(clonedClient);
            }
        }
        
        /// <summary>
        /// Checks whether a domain name in the cloudapp.net zone is available
        /// for use.
        /// </summary>
        /// <param name='location'>
        /// Required. The location of the domain name
        /// </param>
        /// <param name='domainNameLabel'>
        /// Required. The domain name to be verified. It must conform to the
        /// following regular expression: ^[a-z][a-z0-9-]{1,61}[a-z0-9]$.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for CheckDnsNameAvailability Api servive call
        /// </returns>
        public async Task<DnsNameAvailabilityResponse> CheckDnsNameAvailabilityAsync(string location, string domainNameLabel, CancellationToken cancellationToken)
        {
            // Validate
            if (location == null)
            {
                throw new ArgumentNullException("location");
            }
            if (domainNameLabel == null)
            {
                throw new ArgumentNullException("domainNameLabel");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("location", location);
                tracingParameters.Add("domainNameLabel", domainNameLabel);
                TracingAdapter.Enter(invocationId, this, "CheckDnsNameAvailabilityAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Credentials.SubscriptionId);
            }
            url = url + "/providers/";
            url = url + "Microsoft.Network";
            url = url + "/locations/";
            url = url + Uri.EscapeDataString(location);
            url = url + "/CheckDnsNameAvailability";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("domainNameLabel=" + Uri.EscapeDataString(domainNameLabel));
            queryParameters.Add("api-version=2015-05-01-preview");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    DnsNameAvailabilityResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new DnsNameAvailabilityResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken availableValue = responseDoc["available"];
                            if (availableValue != null && availableValue.Type != JTokenType.Null)
                            {
                                bool availableInstance = ((bool)availableValue);
                                result.DnsNameAvailability = availableInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='azureAsyncOperation'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public async Task<AzureAsyncOperationResponse> GetLongRunningOperationStatusAsync(string azureAsyncOperation, CancellationToken cancellationToken)
        {
            // Validate
            if (azureAsyncOperation == null)
            {
                throw new ArgumentNullException("azureAsyncOperation");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("azureAsyncOperation", azureAsyncOperation);
                TracingAdapter.Enter(invocationId, this, "GetLongRunningOperationStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + azureAsyncOperation;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2015-05-01-preview");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AzureAsyncOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new AzureAsyncOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken statusValue = responseDoc["status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                string statusInstance = ((string)statusValue);
                                result.Status = statusInstance;
                            }
                            
                            JToken errorValue = responseDoc["error"];
                            if (errorValue != null && errorValue.Type != JTokenType.Null)
                            {
                                Error errorInstance = new Error();
                                result.Error = errorInstance;
                                
                                JToken codeValue = errorValue["code"];
                                if (codeValue != null && codeValue.Type != JTokenType.Null)
                                {
                                    string codeInstance = ((string)codeValue);
                                    errorInstance.Code = codeInstance;
                                }
                                
                                JToken messageValue = errorValue["message"];
                                if (messageValue != null && messageValue.Type != JTokenType.Null)
                                {
                                    string messageInstance = ((string)messageValue);
                                    errorInstance.Message = messageInstance;
                                }
                                
                                JToken targetValue = errorValue["target"];
                                if (targetValue != null && targetValue.Type != JTokenType.Null)
                                {
                                    string targetInstance = ((string)targetValue);
                                    errorInstance.Target = targetInstance;
                                }
                                
                                JToken detailsArray = errorValue["details"];
                                if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken detailsValue in ((JArray)detailsArray))
                                    {
                                        ErrorDetails errorDetailsInstance = new ErrorDetails();
                                        errorInstance.Details.Add(errorDetailsInstance);
                                        
                                        JToken codeValue2 = detailsValue["code"];
                                        if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                        {
                                            string codeInstance2 = ((string)codeValue2);
                                            errorDetailsInstance.Code = codeInstance2;
                                        }
                                        
                                        JToken targetValue2 = detailsValue["target"];
                                        if (targetValue2 != null && targetValue2.Type != JTokenType.Null)
                                        {
                                            string targetInstance2 = ((string)targetValue2);
                                            errorDetailsInstance.Target = targetInstance2;
                                        }
                                        
                                        JToken messageValue2 = detailsValue["message"];
                                        if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                        {
                                            string messageInstance2 = ((string)messageValue2);
                                            errorDetailsInstance.Message = messageInstance2;
                                        }
                                    }
                                }
                                
                                JToken innerErrorValue = errorValue["innerError"];
                                if (innerErrorValue != null && innerErrorValue.Type != JTokenType.Null)
                                {
                                    string innerErrorInstance = ((string)innerErrorValue);
                                    errorInstance.InnerError = innerErrorInstance;
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
