// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Network Resource Provider API includes operations for managing the
    /// NetworkSecurityGroups for your subscription.
    /// </summary>
    internal partial class NetworkSecurityGroupOperations : IServiceOperations<NetworkResourceProviderClient>, INetworkSecurityGroupOperations
    {
        /// <summary>
        /// Initializes a new instance of the NetworkSecurityGroupOperations
        /// class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal NetworkSecurityGroupOperations(NetworkResourceProviderClient client)
        {
            this._client = client;
        }
        
        private NetworkResourceProviderClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.Azure.Management.Network.NetworkResourceProviderClient.
        /// </summary>
        public NetworkResourceProviderClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// The Put NetworkSecurityGroup operation creates/updates a network
        /// security groupin the specified resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='networkSecurityGroupName'>
        /// Required. The name of the network security group.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the create/update Network Security
        /// Group operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for PUT NetworkSecurityGroups Api servive call
        /// </returns>
        public async Task<NetworkSecurityGroupPutResponse> BeginCreateOrUpdatingAsync(string resourceGroupName, string networkSecurityGroupName, NetworkSecurityGroup parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (networkSecurityGroupName == null)
            {
                throw new ArgumentNullException("networkSecurityGroupName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.DefaultSecurityRules != null)
            {
                foreach (SecurityRule defaultSecurityRulesParameterItem in parameters.DefaultSecurityRules)
                {
                    if (defaultSecurityRulesParameterItem.Access == null)
                    {
                        throw new ArgumentNullException("parameters.DefaultSecurityRules.Access");
                    }
                    if (defaultSecurityRulesParameterItem.DestinationAddressPrefix == null)
                    {
                        throw new ArgumentNullException("parameters.DefaultSecurityRules.DestinationAddressPrefix");
                    }
                    if (defaultSecurityRulesParameterItem.Direction == null)
                    {
                        throw new ArgumentNullException("parameters.DefaultSecurityRules.Direction");
                    }
                    if (defaultSecurityRulesParameterItem.Protocol == null)
                    {
                        throw new ArgumentNullException("parameters.DefaultSecurityRules.Protocol");
                    }
                    if (defaultSecurityRulesParameterItem.SourceAddressPrefix == null)
                    {
                        throw new ArgumentNullException("parameters.DefaultSecurityRules.SourceAddressPrefix");
                    }
                }
            }
            if (parameters.Location == null)
            {
                throw new ArgumentNullException("parameters.Location");
            }
            if (parameters.SecurityRules != null)
            {
                foreach (SecurityRule securityRulesParameterItem in parameters.SecurityRules)
                {
                    if (securityRulesParameterItem.Access == null)
                    {
                        throw new ArgumentNullException("parameters.SecurityRules.Access");
                    }
                    if (securityRulesParameterItem.DestinationAddressPrefix == null)
                    {
                        throw new ArgumentNullException("parameters.SecurityRules.DestinationAddressPrefix");
                    }
                    if (securityRulesParameterItem.Direction == null)
                    {
                        throw new ArgumentNullException("parameters.SecurityRules.Direction");
                    }
                    if (securityRulesParameterItem.Protocol == null)
                    {
                        throw new ArgumentNullException("parameters.SecurityRules.Protocol");
                    }
                    if (securityRulesParameterItem.SourceAddressPrefix == null)
                    {
                        throw new ArgumentNullException("parameters.SecurityRules.SourceAddressPrefix");
                    }
                }
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("networkSecurityGroupName", networkSecurityGroupName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginCreateOrUpdatingAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.Network";
            url = url + "/networkSecurityGroups/";
            url = url + Uri.EscapeDataString(networkSecurityGroupName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-05-01-preview");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject networkSecurityGroupJsonFormatValue = new JObject();
                requestDoc = networkSecurityGroupJsonFormatValue;
                
                JObject propertiesValue = new JObject();
                networkSecurityGroupJsonFormatValue["properties"] = propertiesValue;
                
                if (parameters.SecurityRules != null)
                {
                    if (parameters.SecurityRules is ILazyCollection == false || ((ILazyCollection)parameters.SecurityRules).IsInitialized)
                    {
                        JArray securityRulesArray = new JArray();
                        foreach (SecurityRule securityRulesItem in parameters.SecurityRules)
                        {
                            JObject securityRuleJsonFormatValue = new JObject();
                            securityRulesArray.Add(securityRuleJsonFormatValue);
                            
                            JObject propertiesValue2 = new JObject();
                            securityRuleJsonFormatValue["properties"] = propertiesValue2;
                            
                            if (securityRulesItem.Description != null)
                            {
                                propertiesValue2["description"] = securityRulesItem.Description;
                            }
                            
                            propertiesValue2["protocol"] = securityRulesItem.Protocol;
                            
                            if (securityRulesItem.SourcePortRange != null)
                            {
                                propertiesValue2["sourcePortRange"] = securityRulesItem.SourcePortRange;
                            }
                            
                            if (securityRulesItem.DestinationPortRange != null)
                            {
                                propertiesValue2["destinationPortRange"] = securityRulesItem.DestinationPortRange;
                            }
                            
                            propertiesValue2["sourceAddressPrefix"] = securityRulesItem.SourceAddressPrefix;
                            
                            propertiesValue2["destinationAddressPrefix"] = securityRulesItem.DestinationAddressPrefix;
                            
                            propertiesValue2["access"] = securityRulesItem.Access;
                            
                            propertiesValue2["priority"] = securityRulesItem.Priority;
                            
                            propertiesValue2["direction"] = securityRulesItem.Direction;
                            
                            if (securityRulesItem.ProvisioningState != null)
                            {
                                propertiesValue2["provisioningState"] = securityRulesItem.ProvisioningState;
                            }
                            
                            if (securityRulesItem.Name != null)
                            {
                                securityRuleJsonFormatValue["name"] = securityRulesItem.Name;
                            }
                            
                            if (securityRulesItem.Etag != null)
                            {
                                securityRuleJsonFormatValue["etag"] = securityRulesItem.Etag;
                            }
                            
                            if (securityRulesItem.Id != null)
                            {
                                securityRuleJsonFormatValue["id"] = securityRulesItem.Id;
                            }
                        }
                        propertiesValue["securityRules"] = securityRulesArray;
                    }
                }
                
                if (parameters.DefaultSecurityRules != null)
                {
                    if (parameters.DefaultSecurityRules is ILazyCollection == false || ((ILazyCollection)parameters.DefaultSecurityRules).IsInitialized)
                    {
                        JArray defaultSecurityRulesArray = new JArray();
                        foreach (SecurityRule defaultSecurityRulesItem in parameters.DefaultSecurityRules)
                        {
                            JObject securityRuleJsonFormatValue2 = new JObject();
                            defaultSecurityRulesArray.Add(securityRuleJsonFormatValue2);
                            
                            JObject propertiesValue3 = new JObject();
                            securityRuleJsonFormatValue2["properties"] = propertiesValue3;
                            
                            if (defaultSecurityRulesItem.Description != null)
                            {
                                propertiesValue3["description"] = defaultSecurityRulesItem.Description;
                            }
                            
                            propertiesValue3["protocol"] = defaultSecurityRulesItem.Protocol;
                            
                            if (defaultSecurityRulesItem.SourcePortRange != null)
                            {
                                propertiesValue3["sourcePortRange"] = defaultSecurityRulesItem.SourcePortRange;
                            }
                            
                            if (defaultSecurityRulesItem.DestinationPortRange != null)
                            {
                                propertiesValue3["destinationPortRange"] = defaultSecurityRulesItem.DestinationPortRange;
                            }
                            
                            propertiesValue3["sourceAddressPrefix"] = defaultSecurityRulesItem.SourceAddressPrefix;
                            
                            propertiesValue3["destinationAddressPrefix"] = defaultSecurityRulesItem.DestinationAddressPrefix;
                            
                            propertiesValue3["access"] = defaultSecurityRulesItem.Access;
                            
                            propertiesValue3["priority"] = defaultSecurityRulesItem.Priority;
                            
                            propertiesValue3["direction"] = defaultSecurityRulesItem.Direction;
                            
                            if (defaultSecurityRulesItem.ProvisioningState != null)
                            {
                                propertiesValue3["provisioningState"] = defaultSecurityRulesItem.ProvisioningState;
                            }
                            
                            if (defaultSecurityRulesItem.Name != null)
                            {
                                securityRuleJsonFormatValue2["name"] = defaultSecurityRulesItem.Name;
                            }
                            
                            if (defaultSecurityRulesItem.Etag != null)
                            {
                                securityRuleJsonFormatValue2["etag"] = defaultSecurityRulesItem.Etag;
                            }
                            
                            if (defaultSecurityRulesItem.Id != null)
                            {
                                securityRuleJsonFormatValue2["id"] = defaultSecurityRulesItem.Id;
                            }
                        }
                        propertiesValue["defaultSecurityRules"] = defaultSecurityRulesArray;
                    }
                }
                
                if (parameters.NetworkInterfaces != null)
                {
                    if (parameters.NetworkInterfaces is ILazyCollection == false || ((ILazyCollection)parameters.NetworkInterfaces).IsInitialized)
                    {
                        JArray networkInterfacesArray = new JArray();
                        foreach (ResourceId networkInterfacesItem in parameters.NetworkInterfaces)
                        {
                            JObject resourceIdValue = new JObject();
                            networkInterfacesArray.Add(resourceIdValue);
                            
                            if (networkInterfacesItem.Id != null)
                            {
                                resourceIdValue["id"] = networkInterfacesItem.Id;
                            }
                        }
                        propertiesValue["networkInterfaces"] = networkInterfacesArray;
                    }
                }
                
                if (parameters.Subnets != null)
                {
                    if (parameters.Subnets is ILazyCollection == false || ((ILazyCollection)parameters.Subnets).IsInitialized)
                    {
                        JArray subnetsArray = new JArray();
                        foreach (ResourceId subnetsItem in parameters.Subnets)
                        {
                            JObject resourceIdValue2 = new JObject();
                            subnetsArray.Add(resourceIdValue2);
                            
                            if (subnetsItem.Id != null)
                            {
                                resourceIdValue2["id"] = subnetsItem.Id;
                            }
                        }
                        propertiesValue["subnets"] = subnetsArray;
                    }
                }
                
                if (parameters.ResourceGuid != null)
                {
                    propertiesValue["resourceGuid"] = parameters.ResourceGuid;
                }
                
                if (parameters.ProvisioningState != null)
                {
                    propertiesValue["provisioningState"] = parameters.ProvisioningState;
                }
                
                if (parameters.Etag != null)
                {
                    networkSecurityGroupJsonFormatValue["etag"] = parameters.Etag;
                }
                
                if (parameters.Id != null)
                {
                    networkSecurityGroupJsonFormatValue["id"] = parameters.Id;
                }
                
                if (parameters.Name != null)
                {
                    networkSecurityGroupJsonFormatValue["name"] = parameters.Name;
                }
                
                if (parameters.Type != null)
                {
                    networkSecurityGroupJsonFormatValue["type"] = parameters.Type;
                }
                
                networkSecurityGroupJsonFormatValue["location"] = parameters.Location;
                
                if (parameters.Tags != null)
                {
                    JObject tagsDictionary = new JObject();
                    foreach (KeyValuePair<string, string> pair in parameters.Tags)
                    {
                        string tagsKey = pair.Key;
                        string tagsValue = pair.Value;
                        tagsDictionary[tagsKey] = tagsValue;
                    }
                    networkSecurityGroupJsonFormatValue["tags"] = tagsDictionary;
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    NetworkSecurityGroupPutResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new NetworkSecurityGroupPutResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            NetworkSecurityGroup networkSecurityGroupInstance = new NetworkSecurityGroup();
                            result.NetworkSecurityGroup = networkSecurityGroupInstance;
                            
                            JToken propertiesValue4 = responseDoc["properties"];
                            if (propertiesValue4 != null && propertiesValue4.Type != JTokenType.Null)
                            {
                                JToken securityRulesArray2 = propertiesValue4["securityRules"];
                                if (securityRulesArray2 != null && securityRulesArray2.Type != JTokenType.Null)
                                {
                                    foreach (JToken securityRulesValue in ((JArray)securityRulesArray2))
                                    {
                                        SecurityRule securityRuleJsonFormatInstance = new SecurityRule();
                                        networkSecurityGroupInstance.SecurityRules.Add(securityRuleJsonFormatInstance);
                                        
                                        JToken propertiesValue5 = securityRulesValue["properties"];
                                        if (propertiesValue5 != null && propertiesValue5.Type != JTokenType.Null)
                                        {
                                            JToken descriptionValue = propertiesValue5["description"];
                                            if (descriptionValue != null && descriptionValue.Type != JTokenType.Null)
                                            {
                                                string descriptionInstance = ((string)descriptionValue);
                                                securityRuleJsonFormatInstance.Description = descriptionInstance;
                                            }
                                            
                                            JToken protocolValue = propertiesValue5["protocol"];
                                            if (protocolValue != null && protocolValue.Type != JTokenType.Null)
                                            {
                                                string protocolInstance = ((string)protocolValue);
                                                securityRuleJsonFormatInstance.Protocol = protocolInstance;
                                            }
                                            
                                            JToken sourcePortRangeValue = propertiesValue5["sourcePortRange"];
                                            if (sourcePortRangeValue != null && sourcePortRangeValue.Type != JTokenType.Null)
                                            {
                                                string sourcePortRangeInstance = ((string)sourcePortRangeValue);
                                                securityRuleJsonFormatInstance.SourcePortRange = sourcePortRangeInstance;
                                            }
                                            
                                            JToken destinationPortRangeValue = propertiesValue5["destinationPortRange"];
                                            if (destinationPortRangeValue != null && destinationPortRangeValue.Type != JTokenType.Null)
                                            {
                                                string destinationPortRangeInstance = ((string)destinationPortRangeValue);
                                                securityRuleJsonFormatInstance.DestinationPortRange = destinationPortRangeInstance;
                                            }
                                            
                                            JToken sourceAddressPrefixValue = propertiesValue5["sourceAddressPrefix"];
                                            if (sourceAddressPrefixValue != null && sourceAddressPrefixValue.Type != JTokenType.Null)
                                            {
                                                string sourceAddressPrefixInstance = ((string)sourceAddressPrefixValue);
                                                securityRuleJsonFormatInstance.SourceAddressPrefix = sourceAddressPrefixInstance;
                                            }
                                            
                                            JToken destinationAddressPrefixValue = propertiesValue5["destinationAddressPrefix"];
                                            if (destinationAddressPrefixValue != null && destinationAddressPrefixValue.Type != JTokenType.Null)
                                            {
                                                string destinationAddressPrefixInstance = ((string)destinationAddressPrefixValue);
                                                securityRuleJsonFormatInstance.DestinationAddressPrefix = destinationAddressPrefixInstance;
                                            }
                                            
                                            JToken accessValue = propertiesValue5["access"];
                                            if (accessValue != null && accessValue.Type != JTokenType.Null)
                                            {
                                                string accessInstance = ((string)accessValue);
                                                securityRuleJsonFormatInstance.Access = accessInstance;
                                            }
                                            
                                            JToken priorityValue = propertiesValue5["priority"];
                                            if (priorityValue != null && priorityValue.Type != JTokenType.Null)
                                            {
                                                int priorityInstance = ((int)priorityValue);
                                                securityRuleJsonFormatInstance.Priority = priorityInstance;
                                            }
                                            
                                            JToken directionValue = propertiesValue5["direction"];
                                            if (directionValue != null && directionValue.Type != JTokenType.Null)
                                            {
                                                string directionInstance = ((string)directionValue);
                                                securityRuleJsonFormatInstance.Direction = directionInstance;
                                            }
                                            
                                            JToken provisioningStateValue = propertiesValue5["provisioningState"];
                                            if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                            {
                                                string provisioningStateInstance = ((string)provisioningStateValue);
                                                securityRuleJsonFormatInstance.ProvisioningState = provisioningStateInstance;
                                            }
                                        }
                                        
                                        JToken nameValue = securityRulesValue["name"];
                                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                                        {
                                            string nameInstance = ((string)nameValue);
                                            securityRuleJsonFormatInstance.Name = nameInstance;
                                        }
                                        
                                        JToken etagValue = securityRulesValue["etag"];
                                        if (etagValue != null && etagValue.Type != JTokenType.Null)
                                        {
                                            string etagInstance = ((string)etagValue);
                                            securityRuleJsonFormatInstance.Etag = etagInstance;
                                        }
                                        
                                        JToken idValue = securityRulesValue["id"];
                                        if (idValue != null && idValue.Type != JTokenType.Null)
                                        {
                                            string idInstance = ((string)idValue);
                                            securityRuleJsonFormatInstance.Id = idInstance;
                                        }
                                    }
                                }
                                
                                JToken defaultSecurityRulesArray2 = propertiesValue4["defaultSecurityRules"];
                                if (defaultSecurityRulesArray2 != null && defaultSecurityRulesArray2.Type != JTokenType.Null)
                                {
                                    foreach (JToken defaultSecurityRulesValue in ((JArray)defaultSecurityRulesArray2))
                                    {
                                        SecurityRule securityRuleJsonFormatInstance2 = new SecurityRule();
                                        networkSecurityGroupInstance.DefaultSecurityRules.Add(securityRuleJsonFormatInstance2);
                                        
                                        JToken propertiesValue6 = defaultSecurityRulesValue["properties"];
                                        if (propertiesValue6 != null && propertiesValue6.Type != JTokenType.Null)
                                        {
                                            JToken descriptionValue2 = propertiesValue6["description"];
                                            if (descriptionValue2 != null && descriptionValue2.Type != JTokenType.Null)
                                            {
                                                string descriptionInstance2 = ((string)descriptionValue2);
                                                securityRuleJsonFormatInstance2.Description = descriptionInstance2;
                                            }
                                            
                                            JToken protocolValue2 = propertiesValue6["protocol"];
                                            if (protocolValue2 != null && protocolValue2.Type != JTokenType.Null)
                                            {
                                                string protocolInstance2 = ((string)protocolValue2);
                                                securityRuleJsonFormatInstance2.Protocol = protocolInstance2;
                                            }
                                            
                                            JToken sourcePortRangeValue2 = propertiesValue6["sourcePortRange"];
                                            if (sourcePortRangeValue2 != null && sourcePortRangeValue2.Type != JTokenType.Null)
                                            {
                                                string sourcePortRangeInstance2 = ((string)sourcePortRangeValue2);
                                                securityRuleJsonFormatInstance2.SourcePortRange = sourcePortRangeInstance2;
                                            }
                                            
                                            JToken destinationPortRangeValue2 = propertiesValue6["destinationPortRange"];
                                            if (destinationPortRangeValue2 != null && destinationPortRangeValue2.Type != JTokenType.Null)
                                            {
                                                string destinationPortRangeInstance2 = ((string)destinationPortRangeValue2);
                                                securityRuleJsonFormatInstance2.DestinationPortRange = destinationPortRangeInstance2;
                                            }
                                            
                                            JToken sourceAddressPrefixValue2 = propertiesValue6["sourceAddressPrefix"];
                                            if (sourceAddressPrefixValue2 != null && sourceAddressPrefixValue2.Type != JTokenType.Null)
                                            {
                                                string sourceAddressPrefixInstance2 = ((string)sourceAddressPrefixValue2);
                                                securityRuleJsonFormatInstance2.SourceAddressPrefix = sourceAddressPrefixInstance2;
                                            }
                                            
                                            JToken destinationAddressPrefixValue2 = propertiesValue6["destinationAddressPrefix"];
                                            if (destinationAddressPrefixValue2 != null && destinationAddressPrefixValue2.Type != JTokenType.Null)
                                            {
                                                string destinationAddressPrefixInstance2 = ((string)destinationAddressPrefixValue2);
                                                securityRuleJsonFormatInstance2.DestinationAddressPrefix = destinationAddressPrefixInstance2;
                                            }
                                            
                                            JToken accessValue2 = propertiesValue6["access"];
                                            if (accessValue2 != null && accessValue2.Type != JTokenType.Null)
                                            {
                                                string accessInstance2 = ((string)accessValue2);
                                                securityRuleJsonFormatInstance2.Access = accessInstance2;
                                            }
                                            
                                            JToken priorityValue2 = propertiesValue6["priority"];
                                            if (priorityValue2 != null && priorityValue2.Type != JTokenType.Null)
                                            {
                                                int priorityInstance2 = ((int)priorityValue2);
                                                securityRuleJsonFormatInstance2.Priority = priorityInstance2;
                                            }
                                            
                                            JToken directionValue2 = propertiesValue6["direction"];
                                            if (directionValue2 != null && directionValue2.Type != JTokenType.Null)
                                            {
                                                string directionInstance2 = ((string)directionValue2);
                                                securityRuleJsonFormatInstance2.Direction = directionInstance2;
                                            }
                                            
                                            JToken provisioningStateValue2 = propertiesValue6["provisioningState"];
                                            if (provisioningStateValue2 != null && provisioningStateValue2.Type != JTokenType.Null)
                                            {
                                                string provisioningStateInstance2 = ((string)provisioningStateValue2);
                                                securityRuleJsonFormatInstance2.ProvisioningState = provisioningStateInstance2;
                                            }
                                        }
                                        
                                        JToken nameValue2 = defaultSecurityRulesValue["name"];
                                        if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                        {
                                            string nameInstance2 = ((string)nameValue2);
                                            securityRuleJsonFormatInstance2.Name = nameInstance2;
                                        }
                                        
                                        JToken etagValue2 = defaultSecurityRulesValue["etag"];
                                        if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                        {
                                            string etagInstance2 = ((string)etagValue2);
                                            securityRuleJsonFormatInstance2.Etag = etagInstance2;
                                        }
                                        
                                        JToken idValue2 = defaultSecurityRulesValue["id"];
                                        if (idValue2 != null && idValue2.Type != JTokenType.Null)
                                        {
                                            string idInstance2 = ((string)idValue2);
                                            securityRuleJsonFormatInstance2.Id = idInstance2;
                                        }
                                    }
                                }
                                
                                JToken networkInterfacesArray2 = propertiesValue4["networkInterfaces"];
                                if (networkInterfacesArray2 != null && networkInterfacesArray2.Type != JTokenType.Null)
                                {
                                    foreach (JToken networkInterfacesValue in ((JArray)networkInterfacesArray2))
                                    {
                                        ResourceId resourceIdInstance = new ResourceId();
                                        networkSecurityGroupInstance.NetworkInterfaces.Add(resourceIdInstance);
                                        
                                        JToken idValue3 = networkInterfacesValue["id"];
                                        if (idValue3 != null && idValue3.Type != JTokenType.Null)
                                        {
                                            string idInstance3 = ((string)idValue3);
                                            resourceIdInstance.Id = idInstance3;
                                        }
                                    }
                                }
                                
                                JToken subnetsArray2 = propertiesValue4["subnets"];
                                if (subnetsArray2 != null && subnetsArray2.Type != JTokenType.Null)
                                {
                                    foreach (JToken subnetsValue in ((JArray)subnetsArray2))
                                    {
                                        ResourceId resourceIdInstance2 = new ResourceId();
                                        networkSecurityGroupInstance.Subnets.Add(resourceIdInstance2);
                                        
                                        JToken idValue4 = subnetsValue["id"];
                                        if (idValue4 != null && idValue4.Type != JTokenType.Null)
                                        {
                                            string idInstance4 = ((string)idValue4);
                                            resourceIdInstance2.Id = idInstance4;
                                        }
                                    }
                                }
                                
                                JToken resourceGuidValue = propertiesValue4["resourceGuid"];
                                if (resourceGuidValue != null && resourceGuidValue.Type != JTokenType.Null)
                                {
                                    string resourceGuidInstance = ((string)resourceGuidValue);
                                    networkSecurityGroupInstance.ResourceGuid = resourceGuidInstance;
                                }
                                
                                JToken provisioningStateValue3 = propertiesValue4["provisioningState"];
                                if (provisioningStateValue3 != null && provisioningStateValue3.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance3 = ((string)provisioningStateValue3);
                                    networkSecurityGroupInstance.ProvisioningState = provisioningStateInstance3;
                                }
                            }
                            
                            JToken etagValue3 = responseDoc["etag"];
                            if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                            {
                                string etagInstance3 = ((string)etagValue3);
                                networkSecurityGroupInstance.Etag = etagInstance3;
                            }
                            
                            JToken idValue5 = responseDoc["id"];
                            if (idValue5 != null && idValue5.Type != JTokenType.Null)
                            {
                                string idInstance5 = ((string)idValue5);
                                networkSecurityGroupInstance.Id = idInstance5;
                            }
                            
                            JToken nameValue3 = responseDoc["name"];
                            if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                            {
                                string nameInstance3 = ((string)nameValue3);
                                networkSecurityGroupInstance.Name = nameInstance3;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                networkSecurityGroupInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                networkSecurityGroupInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey2 = ((string)property.Name);
                                    string tagsValue2 = ((string)property.Value);
                                    networkSecurityGroupInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken errorValue = responseDoc["error"];
                            if (errorValue != null && errorValue.Type != JTokenType.Null)
                            {
                                Error errorInstance = new Error();
                                result.Error = errorInstance;
                                
                                JToken codeValue = errorValue["code"];
                                if (codeValue != null && codeValue.Type != JTokenType.Null)
                                {
                                    string codeInstance = ((string)codeValue);
                                    errorInstance.Code = codeInstance;
                                }
                                
                                JToken messageValue = errorValue["message"];
                                if (messageValue != null && messageValue.Type != JTokenType.Null)
                                {
                                    string messageInstance = ((string)messageValue);
                                    errorInstance.Message = messageInstance;
                                }
                                
                                JToken targetValue = errorValue["target"];
                                if (targetValue != null && targetValue.Type != JTokenType.Null)
                                {
                                    string targetInstance = ((string)targetValue);
                                    errorInstance.Target = targetInstance;
                                }
                                
                                JToken detailsArray = errorValue["details"];
                                if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken detailsValue in ((JArray)detailsArray))
                                    {
                                        ErrorDetails errorDetailsInstance = new ErrorDetails();
                                        errorInstance.Details.Add(errorDetailsInstance);
                                        
                                        JToken codeValue2 = detailsValue["code"];
                                        if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                        {
                                            string codeInstance2 = ((string)codeValue2);
                                            errorDetailsInstance.Code = codeInstance2;
                                        }
                                        
                                        JToken targetValue2 = detailsValue["target"];
                                        if (targetValue2 != null && targetValue2.Type != JTokenType.Null)
                                        {
                                            string targetInstance2 = ((string)targetValue2);
                                            errorDetailsInstance.Target = targetInstance2;
                                        }
                                        
                                        JToken messageValue2 = detailsValue["message"];
                                        if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                        {
                                            string messageInstance2 = ((string)messageValue2);
                                            errorDetailsInstance.Message = messageInstance2;
                                        }
                                    }
                                }
                                
                                JToken innerErrorValue = errorValue["innerError"];
                                if (innerErrorValue != null && innerErrorValue.Type != JTokenType.Null)
                                {
                                    string innerErrorInstance = ((string)innerErrorValue);
                                    errorInstance.InnerError = innerErrorInstance;
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AzureAsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Delete NetworkSecurityGroup operation deletes the specifed
        /// network security group
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='networkSecurityGroupName'>
        /// Required. The name of the network security group.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public async Task<UpdateOperationResponse> BeginDeletingAsync(string resourceGroupName, string networkSecurityGroupName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (networkSecurityGroupName == null)
            {
                throw new ArgumentNullException("networkSecurityGroupName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("networkSecurityGroupName", networkSecurityGroupName);
                TracingAdapter.Enter(invocationId, this, "BeginDeletingAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.Network";
            url = url + "/networkSecurityGroups/";
            url = url + Uri.EscapeDataString(networkSecurityGroupName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-05-01-preview");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    UpdateOperationResponse result = null;
                    // Deserialize Response
                    result = new UpdateOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AzureAsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Put NetworkSecurityGroup operation creates/updates a network
        /// security groupin the specified resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='networkSecurityGroupName'>
        /// Required. The name of the network security group.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the create/update Network Security
        /// Group operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public async Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(string resourceGroupName, string networkSecurityGroupName, NetworkSecurityGroup parameters, CancellationToken cancellationToken)
        {
            NetworkResourceProviderClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("networkSecurityGroupName", networkSecurityGroupName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            NetworkSecurityGroupPutResponse response = await client.NetworkSecurityGroups.BeginCreateOrUpdatingAsync(resourceGroupName, networkSecurityGroupName, parameters, cancellationToken).ConfigureAwait(false);
            cancellationToken.ThrowIfCancellationRequested();
            AzureAsyncOperationResponse result = await client.GetLongRunningOperationStatusAsync(response.AzureAsyncOperation, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 30;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == NetworkOperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.GetLongRunningOperationStatusAsync(response.AzureAsyncOperation, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 15;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// The Delete NetworkSecurityGroup operation deletes the specifed
        /// network security group
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='networkSecurityGroupName'>
        /// Required. The name of the network security group.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string networkSecurityGroupName, CancellationToken cancellationToken)
        {
            NetworkResourceProviderClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("networkSecurityGroupName", networkSecurityGroupName);
                TracingAdapter.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            UpdateOperationResponse response = await client.NetworkSecurityGroups.BeginDeletingAsync(resourceGroupName, networkSecurityGroupName, cancellationToken).ConfigureAwait(false);
            cancellationToken.ThrowIfCancellationRequested();
            AzureAsyncOperationResponse result = await client.GetLongRunningOperationStatusAsync(response.AzureAsyncOperation, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 30;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == NetworkOperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.GetLongRunningOperationStatusAsync(response.AzureAsyncOperation, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 15;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// The Get NetworkSecurityGroups operation retrieves information about
        /// the specified network security group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='networkSecurityGroupName'>
        /// Required. The name of the network security group.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for GetNetworkSecurityGroup Api service call
        /// </returns>
        public async Task<NetworkSecurityGroupGetResponse> GetAsync(string resourceGroupName, string networkSecurityGroupName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (networkSecurityGroupName == null)
            {
                throw new ArgumentNullException("networkSecurityGroupName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("networkSecurityGroupName", networkSecurityGroupName);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.Network";
            url = url + "/networkSecurityGroups/";
            url = url + Uri.EscapeDataString(networkSecurityGroupName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-05-01-preview");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    NetworkSecurityGroupGetResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new NetworkSecurityGroupGetResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            NetworkSecurityGroup networkSecurityGroupInstance = new NetworkSecurityGroup();
                            result.NetworkSecurityGroup = networkSecurityGroupInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JToken securityRulesArray = propertiesValue["securityRules"];
                                if (securityRulesArray != null && securityRulesArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken securityRulesValue in ((JArray)securityRulesArray))
                                    {
                                        SecurityRule securityRuleJsonFormatInstance = new SecurityRule();
                                        networkSecurityGroupInstance.SecurityRules.Add(securityRuleJsonFormatInstance);
                                        
                                        JToken propertiesValue2 = securityRulesValue["properties"];
                                        if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                                        {
                                            JToken descriptionValue = propertiesValue2["description"];
                                            if (descriptionValue != null && descriptionValue.Type != JTokenType.Null)
                                            {
                                                string descriptionInstance = ((string)descriptionValue);
                                                securityRuleJsonFormatInstance.Description = descriptionInstance;
                                            }
                                            
                                            JToken protocolValue = propertiesValue2["protocol"];
                                            if (protocolValue != null && protocolValue.Type != JTokenType.Null)
                                            {
                                                string protocolInstance = ((string)protocolValue);
                                                securityRuleJsonFormatInstance.Protocol = protocolInstance;
                                            }
                                            
                                            JToken sourcePortRangeValue = propertiesValue2["sourcePortRange"];
                                            if (sourcePortRangeValue != null && sourcePortRangeValue.Type != JTokenType.Null)
                                            {
                                                string sourcePortRangeInstance = ((string)sourcePortRangeValue);
                                                securityRuleJsonFormatInstance.SourcePortRange = sourcePortRangeInstance;
                                            }
                                            
                                            JToken destinationPortRangeValue = propertiesValue2["destinationPortRange"];
                                            if (destinationPortRangeValue != null && destinationPortRangeValue.Type != JTokenType.Null)
                                            {
                                                string destinationPortRangeInstance = ((string)destinationPortRangeValue);
                                                securityRuleJsonFormatInstance.DestinationPortRange = destinationPortRangeInstance;
                                            }
                                            
                                            JToken sourceAddressPrefixValue = propertiesValue2["sourceAddressPrefix"];
                                            if (sourceAddressPrefixValue != null && sourceAddressPrefixValue.Type != JTokenType.Null)
                                            {
                                                string sourceAddressPrefixInstance = ((string)sourceAddressPrefixValue);
                                                securityRuleJsonFormatInstance.SourceAddressPrefix = sourceAddressPrefixInstance;
                                            }
                                            
                                            JToken destinationAddressPrefixValue = propertiesValue2["destinationAddressPrefix"];
                                            if (destinationAddressPrefixValue != null && destinationAddressPrefixValue.Type != JTokenType.Null)
                                            {
                                                string destinationAddressPrefixInstance = ((string)destinationAddressPrefixValue);
                                                securityRuleJsonFormatInstance.DestinationAddressPrefix = destinationAddressPrefixInstance;
                                            }
                                            
                                            JToken accessValue = propertiesValue2["access"];
                                            if (accessValue != null && accessValue.Type != JTokenType.Null)
                                            {
                                                string accessInstance = ((string)accessValue);
                                                securityRuleJsonFormatInstance.Access = accessInstance;
                                            }
                                            
                                            JToken priorityValue = propertiesValue2["priority"];
                                            if (priorityValue != null && priorityValue.Type != JTokenType.Null)
                                            {
                                                int priorityInstance = ((int)priorityValue);
                                                securityRuleJsonFormatInstance.Priority = priorityInstance;
                                            }
                                            
                                            JToken directionValue = propertiesValue2["direction"];
                                            if (directionValue != null && directionValue.Type != JTokenType.Null)
                                            {
                                                string directionInstance = ((string)directionValue);
                                                securityRuleJsonFormatInstance.Direction = directionInstance;
                                            }
                                            
                                            JToken provisioningStateValue = propertiesValue2["provisioningState"];
                                            if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                            {
                                                string provisioningStateInstance = ((string)provisioningStateValue);
                                                securityRuleJsonFormatInstance.ProvisioningState = provisioningStateInstance;
                                            }
                                        }
                                        
                                        JToken nameValue = securityRulesValue["name"];
                                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                                        {
                                            string nameInstance = ((string)nameValue);
                                            securityRuleJsonFormatInstance.Name = nameInstance;
                                        }
                                        
                                        JToken etagValue = securityRulesValue["etag"];
                                        if (etagValue != null && etagValue.Type != JTokenType.Null)
                                        {
                                            string etagInstance = ((string)etagValue);
                                            securityRuleJsonFormatInstance.Etag = etagInstance;
                                        }
                                        
                                        JToken idValue = securityRulesValue["id"];
                                        if (idValue != null && idValue.Type != JTokenType.Null)
                                        {
                                            string idInstance = ((string)idValue);
                                            securityRuleJsonFormatInstance.Id = idInstance;
                                        }
                                    }
                                }
                                
                                JToken defaultSecurityRulesArray = propertiesValue["defaultSecurityRules"];
                                if (defaultSecurityRulesArray != null && defaultSecurityRulesArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken defaultSecurityRulesValue in ((JArray)defaultSecurityRulesArray))
                                    {
                                        SecurityRule securityRuleJsonFormatInstance2 = new SecurityRule();
                                        networkSecurityGroupInstance.DefaultSecurityRules.Add(securityRuleJsonFormatInstance2);
                                        
                                        JToken propertiesValue3 = defaultSecurityRulesValue["properties"];
                                        if (propertiesValue3 != null && propertiesValue3.Type != JTokenType.Null)
                                        {
                                            JToken descriptionValue2 = propertiesValue3["description"];
                                            if (descriptionValue2 != null && descriptionValue2.Type != JTokenType.Null)
                                            {
                                                string descriptionInstance2 = ((string)descriptionValue2);
                                                securityRuleJsonFormatInstance2.Description = descriptionInstance2;
                                            }
                                            
                                            JToken protocolValue2 = propertiesValue3["protocol"];
                                            if (protocolValue2 != null && protocolValue2.Type != JTokenType.Null)
                                            {
                                                string protocolInstance2 = ((string)protocolValue2);
                                                securityRuleJsonFormatInstance2.Protocol = protocolInstance2;
                                            }
                                            
                                            JToken sourcePortRangeValue2 = propertiesValue3["sourcePortRange"];
                                            if (sourcePortRangeValue2 != null && sourcePortRangeValue2.Type != JTokenType.Null)
                                            {
                                                string sourcePortRangeInstance2 = ((string)sourcePortRangeValue2);
                                                securityRuleJsonFormatInstance2.SourcePortRange = sourcePortRangeInstance2;
                                            }
                                            
                                            JToken destinationPortRangeValue2 = propertiesValue3["destinationPortRange"];
                                            if (destinationPortRangeValue2 != null && destinationPortRangeValue2.Type != JTokenType.Null)
                                            {
                                                string destinationPortRangeInstance2 = ((string)destinationPortRangeValue2);
                                                securityRuleJsonFormatInstance2.DestinationPortRange = destinationPortRangeInstance2;
                                            }
                                            
                                            JToken sourceAddressPrefixValue2 = propertiesValue3["sourceAddressPrefix"];
                                            if (sourceAddressPrefixValue2 != null && sourceAddressPrefixValue2.Type != JTokenType.Null)
                                            {
                                                string sourceAddressPrefixInstance2 = ((string)sourceAddressPrefixValue2);
                                                securityRuleJsonFormatInstance2.SourceAddressPrefix = sourceAddressPrefixInstance2;
                                            }
                                            
                                            JToken destinationAddressPrefixValue2 = propertiesValue3["destinationAddressPrefix"];
                                            if (destinationAddressPrefixValue2 != null && destinationAddressPrefixValue2.Type != JTokenType.Null)
                                            {
                                                string destinationAddressPrefixInstance2 = ((string)destinationAddressPrefixValue2);
                                                securityRuleJsonFormatInstance2.DestinationAddressPrefix = destinationAddressPrefixInstance2;
                                            }
                                            
                                            JToken accessValue2 = propertiesValue3["access"];
                                            if (accessValue2 != null && accessValue2.Type != JTokenType.Null)
                                            {
                                                string accessInstance2 = ((string)accessValue2);
                                                securityRuleJsonFormatInstance2.Access = accessInstance2;
                                            }
                                            
                                            JToken priorityValue2 = propertiesValue3["priority"];
                                            if (priorityValue2 != null && priorityValue2.Type != JTokenType.Null)
                                            {
                                                int priorityInstance2 = ((int)priorityValue2);
                                                securityRuleJsonFormatInstance2.Priority = priorityInstance2;
                                            }
                                            
                                            JToken directionValue2 = propertiesValue3["direction"];
                                            if (directionValue2 != null && directionValue2.Type != JTokenType.Null)
                                            {
                                                string directionInstance2 = ((string)directionValue2);
                                                securityRuleJsonFormatInstance2.Direction = directionInstance2;
                                            }
                                            
                                            JToken provisioningStateValue2 = propertiesValue3["provisioningState"];
                                            if (provisioningStateValue2 != null && provisioningStateValue2.Type != JTokenType.Null)
                                            {
                                                string provisioningStateInstance2 = ((string)provisioningStateValue2);
                                                securityRuleJsonFormatInstance2.ProvisioningState = provisioningStateInstance2;
                                            }
                                        }
                                        
                                        JToken nameValue2 = defaultSecurityRulesValue["name"];
                                        if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                        {
                                            string nameInstance2 = ((string)nameValue2);
                                            securityRuleJsonFormatInstance2.Name = nameInstance2;
                                        }
                                        
                                        JToken etagValue2 = defaultSecurityRulesValue["etag"];
                                        if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                        {
                                            string etagInstance2 = ((string)etagValue2);
                                            securityRuleJsonFormatInstance2.Etag = etagInstance2;
                                        }
                                        
                                        JToken idValue2 = defaultSecurityRulesValue["id"];
                                        if (idValue2 != null && idValue2.Type != JTokenType.Null)
                                        {
                                            string idInstance2 = ((string)idValue2);
                                            securityRuleJsonFormatInstance2.Id = idInstance2;
                                        }
                                    }
                                }
                                
                                JToken networkInterfacesArray = propertiesValue["networkInterfaces"];
                                if (networkInterfacesArray != null && networkInterfacesArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken networkInterfacesValue in ((JArray)networkInterfacesArray))
                                    {
                                        ResourceId resourceIdInstance = new ResourceId();
                                        networkSecurityGroupInstance.NetworkInterfaces.Add(resourceIdInstance);
                                        
                                        JToken idValue3 = networkInterfacesValue["id"];
                                        if (idValue3 != null && idValue3.Type != JTokenType.Null)
                                        {
                                            string idInstance3 = ((string)idValue3);
                                            resourceIdInstance.Id = idInstance3;
                                        }
                                    }
                                }
                                
                                JToken subnetsArray = propertiesValue["subnets"];
                                if (subnetsArray != null && subnetsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken subnetsValue in ((JArray)subnetsArray))
                                    {
                                        ResourceId resourceIdInstance2 = new ResourceId();
                                        networkSecurityGroupInstance.Subnets.Add(resourceIdInstance2);
                                        
                                        JToken idValue4 = subnetsValue["id"];
                                        if (idValue4 != null && idValue4.Type != JTokenType.Null)
                                        {
                                            string idInstance4 = ((string)idValue4);
                                            resourceIdInstance2.Id = idInstance4;
                                        }
                                    }
                                }
                                
                                JToken resourceGuidValue = propertiesValue["resourceGuid"];
                                if (resourceGuidValue != null && resourceGuidValue.Type != JTokenType.Null)
                                {
                                    string resourceGuidInstance = ((string)resourceGuidValue);
                                    networkSecurityGroupInstance.ResourceGuid = resourceGuidInstance;
                                }
                                
                                JToken provisioningStateValue3 = propertiesValue["provisioningState"];
                                if (provisioningStateValue3 != null && provisioningStateValue3.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance3 = ((string)provisioningStateValue3);
                                    networkSecurityGroupInstance.ProvisioningState = provisioningStateInstance3;
                                }
                            }
                            
                            JToken etagValue3 = responseDoc["etag"];
                            if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                            {
                                string etagInstance3 = ((string)etagValue3);
                                networkSecurityGroupInstance.Etag = etagInstance3;
                            }
                            
                            JToken idValue5 = responseDoc["id"];
                            if (idValue5 != null && idValue5.Type != JTokenType.Null)
                            {
                                string idInstance5 = ((string)idValue5);
                                networkSecurityGroupInstance.Id = idInstance5;
                            }
                            
                            JToken nameValue3 = responseDoc["name"];
                            if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                            {
                                string nameInstance3 = ((string)nameValue3);
                                networkSecurityGroupInstance.Name = nameInstance3;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                networkSecurityGroupInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                networkSecurityGroupInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property.Name);
                                    string tagsValue = ((string)property.Value);
                                    networkSecurityGroupInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The list NetworkSecurityGroups returns all network security groups
        /// in a resource group
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for ListNetworkSecurityGroups Api servive call
        /// </returns>
        public async Task<NetworkSecurityGroupListResponse> ListAsync(string resourceGroupName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                TracingAdapter.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.Network";
            url = url + "/networkSecurityGroups";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-05-01-preview");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    NetworkSecurityGroupListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new NetworkSecurityGroupListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    NetworkSecurityGroup networkSecurityGroupJsonFormatInstance = new NetworkSecurityGroup();
                                    result.NetworkSecurityGroups.Add(networkSecurityGroupJsonFormatInstance);
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        JToken securityRulesArray = propertiesValue["securityRules"];
                                        if (securityRulesArray != null && securityRulesArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken securityRulesValue in ((JArray)securityRulesArray))
                                            {
                                                SecurityRule securityRuleJsonFormatInstance = new SecurityRule();
                                                networkSecurityGroupJsonFormatInstance.SecurityRules.Add(securityRuleJsonFormatInstance);
                                                
                                                JToken propertiesValue2 = securityRulesValue["properties"];
                                                if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                                                {
                                                    JToken descriptionValue = propertiesValue2["description"];
                                                    if (descriptionValue != null && descriptionValue.Type != JTokenType.Null)
                                                    {
                                                        string descriptionInstance = ((string)descriptionValue);
                                                        securityRuleJsonFormatInstance.Description = descriptionInstance;
                                                    }
                                                    
                                                    JToken protocolValue = propertiesValue2["protocol"];
                                                    if (protocolValue != null && protocolValue.Type != JTokenType.Null)
                                                    {
                                                        string protocolInstance = ((string)protocolValue);
                                                        securityRuleJsonFormatInstance.Protocol = protocolInstance;
                                                    }
                                                    
                                                    JToken sourcePortRangeValue = propertiesValue2["sourcePortRange"];
                                                    if (sourcePortRangeValue != null && sourcePortRangeValue.Type != JTokenType.Null)
                                                    {
                                                        string sourcePortRangeInstance = ((string)sourcePortRangeValue);
                                                        securityRuleJsonFormatInstance.SourcePortRange = sourcePortRangeInstance;
                                                    }
                                                    
                                                    JToken destinationPortRangeValue = propertiesValue2["destinationPortRange"];
                                                    if (destinationPortRangeValue != null && destinationPortRangeValue.Type != JTokenType.Null)
                                                    {
                                                        string destinationPortRangeInstance = ((string)destinationPortRangeValue);
                                                        securityRuleJsonFormatInstance.DestinationPortRange = destinationPortRangeInstance;
                                                    }
                                                    
                                                    JToken sourceAddressPrefixValue = propertiesValue2["sourceAddressPrefix"];
                                                    if (sourceAddressPrefixValue != null && sourceAddressPrefixValue.Type != JTokenType.Null)
                                                    {
                                                        string sourceAddressPrefixInstance = ((string)sourceAddressPrefixValue);
                                                        securityRuleJsonFormatInstance.SourceAddressPrefix = sourceAddressPrefixInstance;
                                                    }
                                                    
                                                    JToken destinationAddressPrefixValue = propertiesValue2["destinationAddressPrefix"];
                                                    if (destinationAddressPrefixValue != null && destinationAddressPrefixValue.Type != JTokenType.Null)
                                                    {
                                                        string destinationAddressPrefixInstance = ((string)destinationAddressPrefixValue);
                                                        securityRuleJsonFormatInstance.DestinationAddressPrefix = destinationAddressPrefixInstance;
                                                    }
                                                    
                                                    JToken accessValue = propertiesValue2["access"];
                                                    if (accessValue != null && accessValue.Type != JTokenType.Null)
                                                    {
                                                        string accessInstance = ((string)accessValue);
                                                        securityRuleJsonFormatInstance.Access = accessInstance;
                                                    }
                                                    
                                                    JToken priorityValue = propertiesValue2["priority"];
                                                    if (priorityValue != null && priorityValue.Type != JTokenType.Null)
                                                    {
                                                        int priorityInstance = ((int)priorityValue);
                                                        securityRuleJsonFormatInstance.Priority = priorityInstance;
                                                    }
                                                    
                                                    JToken directionValue = propertiesValue2["direction"];
                                                    if (directionValue != null && directionValue.Type != JTokenType.Null)
                                                    {
                                                        string directionInstance = ((string)directionValue);
                                                        securityRuleJsonFormatInstance.Direction = directionInstance;
                                                    }
                                                    
                                                    JToken provisioningStateValue = propertiesValue2["provisioningState"];
                                                    if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                                    {
                                                        string provisioningStateInstance = ((string)provisioningStateValue);
                                                        securityRuleJsonFormatInstance.ProvisioningState = provisioningStateInstance;
                                                    }
                                                }
                                                
                                                JToken nameValue = securityRulesValue["name"];
                                                if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                {
                                                    string nameInstance = ((string)nameValue);
                                                    securityRuleJsonFormatInstance.Name = nameInstance;
                                                }
                                                
                                                JToken etagValue = securityRulesValue["etag"];
                                                if (etagValue != null && etagValue.Type != JTokenType.Null)
                                                {
                                                    string etagInstance = ((string)etagValue);
                                                    securityRuleJsonFormatInstance.Etag = etagInstance;
                                                }
                                                
                                                JToken idValue = securityRulesValue["id"];
                                                if (idValue != null && idValue.Type != JTokenType.Null)
                                                {
                                                    string idInstance = ((string)idValue);
                                                    securityRuleJsonFormatInstance.Id = idInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken defaultSecurityRulesArray = propertiesValue["defaultSecurityRules"];
                                        if (defaultSecurityRulesArray != null && defaultSecurityRulesArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken defaultSecurityRulesValue in ((JArray)defaultSecurityRulesArray))
                                            {
                                                SecurityRule securityRuleJsonFormatInstance2 = new SecurityRule();
                                                networkSecurityGroupJsonFormatInstance.DefaultSecurityRules.Add(securityRuleJsonFormatInstance2);
                                                
                                                JToken propertiesValue3 = defaultSecurityRulesValue["properties"];
                                                if (propertiesValue3 != null && propertiesValue3.Type != JTokenType.Null)
                                                {
                                                    JToken descriptionValue2 = propertiesValue3["description"];
                                                    if (descriptionValue2 != null && descriptionValue2.Type != JTokenType.Null)
                                                    {
                                                        string descriptionInstance2 = ((string)descriptionValue2);
                                                        securityRuleJsonFormatInstance2.Description = descriptionInstance2;
                                                    }
                                                    
                                                    JToken protocolValue2 = propertiesValue3["protocol"];
                                                    if (protocolValue2 != null && protocolValue2.Type != JTokenType.Null)
                                                    {
                                                        string protocolInstance2 = ((string)protocolValue2);
                                                        securityRuleJsonFormatInstance2.Protocol = protocolInstance2;
                                                    }
                                                    
                                                    JToken sourcePortRangeValue2 = propertiesValue3["sourcePortRange"];
                                                    if (sourcePortRangeValue2 != null && sourcePortRangeValue2.Type != JTokenType.Null)
                                                    {
                                                        string sourcePortRangeInstance2 = ((string)sourcePortRangeValue2);
                                                        securityRuleJsonFormatInstance2.SourcePortRange = sourcePortRangeInstance2;
                                                    }
                                                    
                                                    JToken destinationPortRangeValue2 = propertiesValue3["destinationPortRange"];
                                                    if (destinationPortRangeValue2 != null && destinationPortRangeValue2.Type != JTokenType.Null)
                                                    {
                                                        string destinationPortRangeInstance2 = ((string)destinationPortRangeValue2);
                                                        securityRuleJsonFormatInstance2.DestinationPortRange = destinationPortRangeInstance2;
                                                    }
                                                    
                                                    JToken sourceAddressPrefixValue2 = propertiesValue3["sourceAddressPrefix"];
                                                    if (sourceAddressPrefixValue2 != null && sourceAddressPrefixValue2.Type != JTokenType.Null)
                                                    {
                                                        string sourceAddressPrefixInstance2 = ((string)sourceAddressPrefixValue2);
                                                        securityRuleJsonFormatInstance2.SourceAddressPrefix = sourceAddressPrefixInstance2;
                                                    }
                                                    
                                                    JToken destinationAddressPrefixValue2 = propertiesValue3["destinationAddressPrefix"];
                                                    if (destinationAddressPrefixValue2 != null && destinationAddressPrefixValue2.Type != JTokenType.Null)
                                                    {
                                                        string destinationAddressPrefixInstance2 = ((string)destinationAddressPrefixValue2);
                                                        securityRuleJsonFormatInstance2.DestinationAddressPrefix = destinationAddressPrefixInstance2;
                                                    }
                                                    
                                                    JToken accessValue2 = propertiesValue3["access"];
                                                    if (accessValue2 != null && accessValue2.Type != JTokenType.Null)
                                                    {
                                                        string accessInstance2 = ((string)accessValue2);
                                                        securityRuleJsonFormatInstance2.Access = accessInstance2;
                                                    }
                                                    
                                                    JToken priorityValue2 = propertiesValue3["priority"];
                                                    if (priorityValue2 != null && priorityValue2.Type != JTokenType.Null)
                                                    {
                                                        int priorityInstance2 = ((int)priorityValue2);
                                                        securityRuleJsonFormatInstance2.Priority = priorityInstance2;
                                                    }
                                                    
                                                    JToken directionValue2 = propertiesValue3["direction"];
                                                    if (directionValue2 != null && directionValue2.Type != JTokenType.Null)
                                                    {
                                                        string directionInstance2 = ((string)directionValue2);
                                                        securityRuleJsonFormatInstance2.Direction = directionInstance2;
                                                    }
                                                    
                                                    JToken provisioningStateValue2 = propertiesValue3["provisioningState"];
                                                    if (provisioningStateValue2 != null && provisioningStateValue2.Type != JTokenType.Null)
                                                    {
                                                        string provisioningStateInstance2 = ((string)provisioningStateValue2);
                                                        securityRuleJsonFormatInstance2.ProvisioningState = provisioningStateInstance2;
                                                    }
                                                }
                                                
                                                JToken nameValue2 = defaultSecurityRulesValue["name"];
                                                if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                {
                                                    string nameInstance2 = ((string)nameValue2);
                                                    securityRuleJsonFormatInstance2.Name = nameInstance2;
                                                }
                                                
                                                JToken etagValue2 = defaultSecurityRulesValue["etag"];
                                                if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                {
                                                    string etagInstance2 = ((string)etagValue2);
                                                    securityRuleJsonFormatInstance2.Etag = etagInstance2;
                                                }
                                                
                                                JToken idValue2 = defaultSecurityRulesValue["id"];
                                                if (idValue2 != null && idValue2.Type != JTokenType.Null)
                                                {
                                                    string idInstance2 = ((string)idValue2);
                                                    securityRuleJsonFormatInstance2.Id = idInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken networkInterfacesArray = propertiesValue["networkInterfaces"];
                                        if (networkInterfacesArray != null && networkInterfacesArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken networkInterfacesValue in ((JArray)networkInterfacesArray))
                                            {
                                                ResourceId resourceIdInstance = new ResourceId();
                                                networkSecurityGroupJsonFormatInstance.NetworkInterfaces.Add(resourceIdInstance);
                                                
                                                JToken idValue3 = networkInterfacesValue["id"];
                                                if (idValue3 != null && idValue3.Type != JTokenType.Null)
                                                {
                                                    string idInstance3 = ((string)idValue3);
                                                    resourceIdInstance.Id = idInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken subnetsArray = propertiesValue["subnets"];
                                        if (subnetsArray != null && subnetsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken subnetsValue in ((JArray)subnetsArray))
                                            {
                                                ResourceId resourceIdInstance2 = new ResourceId();
                                                networkSecurityGroupJsonFormatInstance.Subnets.Add(resourceIdInstance2);
                                                
                                                JToken idValue4 = subnetsValue["id"];
                                                if (idValue4 != null && idValue4.Type != JTokenType.Null)
                                                {
                                                    string idInstance4 = ((string)idValue4);
                                                    resourceIdInstance2.Id = idInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken resourceGuidValue = propertiesValue["resourceGuid"];
                                        if (resourceGuidValue != null && resourceGuidValue.Type != JTokenType.Null)
                                        {
                                            string resourceGuidInstance = ((string)resourceGuidValue);
                                            networkSecurityGroupJsonFormatInstance.ResourceGuid = resourceGuidInstance;
                                        }
                                        
                                        JToken provisioningStateValue3 = propertiesValue["provisioningState"];
                                        if (provisioningStateValue3 != null && provisioningStateValue3.Type != JTokenType.Null)
                                        {
                                            string provisioningStateInstance3 = ((string)provisioningStateValue3);
                                            networkSecurityGroupJsonFormatInstance.ProvisioningState = provisioningStateInstance3;
                                        }
                                    }
                                    
                                    JToken etagValue3 = valueValue["etag"];
                                    if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                    {
                                        string etagInstance3 = ((string)etagValue3);
                                        networkSecurityGroupJsonFormatInstance.Etag = etagInstance3;
                                    }
                                    
                                    JToken idValue5 = valueValue["id"];
                                    if (idValue5 != null && idValue5.Type != JTokenType.Null)
                                    {
                                        string idInstance5 = ((string)idValue5);
                                        networkSecurityGroupJsonFormatInstance.Id = idInstance5;
                                    }
                                    
                                    JToken nameValue3 = valueValue["name"];
                                    if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                    {
                                        string nameInstance3 = ((string)nameValue3);
                                        networkSecurityGroupJsonFormatInstance.Name = nameInstance3;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        networkSecurityGroupJsonFormatInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        networkSecurityGroupJsonFormatInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property.Name);
                                            string tagsValue = ((string)property.Value);
                                            networkSecurityGroupJsonFormatInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The list NetworkSecurityGroups returns all network security groups
        /// in a subscription
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for ListNetworkSecurityGroups Api servive call
        /// </returns>
        public async Task<NetworkSecurityGroupListResponse> ListAllAsync(CancellationToken cancellationToken)
        {
            // Validate
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                TracingAdapter.Enter(invocationId, this, "ListAllAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/providers/";
            url = url + "Microsoft.Network";
            url = url + "/networkSecurityGroups";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-05-01-preview");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    NetworkSecurityGroupListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new NetworkSecurityGroupListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    NetworkSecurityGroup networkSecurityGroupJsonFormatInstance = new NetworkSecurityGroup();
                                    result.NetworkSecurityGroups.Add(networkSecurityGroupJsonFormatInstance);
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        JToken securityRulesArray = propertiesValue["securityRules"];
                                        if (securityRulesArray != null && securityRulesArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken securityRulesValue in ((JArray)securityRulesArray))
                                            {
                                                SecurityRule securityRuleJsonFormatInstance = new SecurityRule();
                                                networkSecurityGroupJsonFormatInstance.SecurityRules.Add(securityRuleJsonFormatInstance);
                                                
                                                JToken propertiesValue2 = securityRulesValue["properties"];
                                                if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                                                {
                                                    JToken descriptionValue = propertiesValue2["description"];
                                                    if (descriptionValue != null && descriptionValue.Type != JTokenType.Null)
                                                    {
                                                        string descriptionInstance = ((string)descriptionValue);
                                                        securityRuleJsonFormatInstance.Description = descriptionInstance;
                                                    }
                                                    
                                                    JToken protocolValue = propertiesValue2["protocol"];
                                                    if (protocolValue != null && protocolValue.Type != JTokenType.Null)
                                                    {
                                                        string protocolInstance = ((string)protocolValue);
                                                        securityRuleJsonFormatInstance.Protocol = protocolInstance;
                                                    }
                                                    
                                                    JToken sourcePortRangeValue = propertiesValue2["sourcePortRange"];
                                                    if (sourcePortRangeValue != null && sourcePortRangeValue.Type != JTokenType.Null)
                                                    {
                                                        string sourcePortRangeInstance = ((string)sourcePortRangeValue);
                                                        securityRuleJsonFormatInstance.SourcePortRange = sourcePortRangeInstance;
                                                    }
                                                    
                                                    JToken destinationPortRangeValue = propertiesValue2["destinationPortRange"];
                                                    if (destinationPortRangeValue != null && destinationPortRangeValue.Type != JTokenType.Null)
                                                    {
                                                        string destinationPortRangeInstance = ((string)destinationPortRangeValue);
                                                        securityRuleJsonFormatInstance.DestinationPortRange = destinationPortRangeInstance;
                                                    }
                                                    
                                                    JToken sourceAddressPrefixValue = propertiesValue2["sourceAddressPrefix"];
                                                    if (sourceAddressPrefixValue != null && sourceAddressPrefixValue.Type != JTokenType.Null)
                                                    {
                                                        string sourceAddressPrefixInstance = ((string)sourceAddressPrefixValue);
                                                        securityRuleJsonFormatInstance.SourceAddressPrefix = sourceAddressPrefixInstance;
                                                    }
                                                    
                                                    JToken destinationAddressPrefixValue = propertiesValue2["destinationAddressPrefix"];
                                                    if (destinationAddressPrefixValue != null && destinationAddressPrefixValue.Type != JTokenType.Null)
                                                    {
                                                        string destinationAddressPrefixInstance = ((string)destinationAddressPrefixValue);
                                                        securityRuleJsonFormatInstance.DestinationAddressPrefix = destinationAddressPrefixInstance;
                                                    }
                                                    
                                                    JToken accessValue = propertiesValue2["access"];
                                                    if (accessValue != null && accessValue.Type != JTokenType.Null)
                                                    {
                                                        string accessInstance = ((string)accessValue);
                                                        securityRuleJsonFormatInstance.Access = accessInstance;
                                                    }
                                                    
                                                    JToken priorityValue = propertiesValue2["priority"];
                                                    if (priorityValue != null && priorityValue.Type != JTokenType.Null)
                                                    {
                                                        int priorityInstance = ((int)priorityValue);
                                                        securityRuleJsonFormatInstance.Priority = priorityInstance;
                                                    }
                                                    
                                                    JToken directionValue = propertiesValue2["direction"];
                                                    if (directionValue != null && directionValue.Type != JTokenType.Null)
                                                    {
                                                        string directionInstance = ((string)directionValue);
                                                        securityRuleJsonFormatInstance.Direction = directionInstance;
                                                    }
                                                    
                                                    JToken provisioningStateValue = propertiesValue2["provisioningState"];
                                                    if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                                    {
                                                        string provisioningStateInstance = ((string)provisioningStateValue);
                                                        securityRuleJsonFormatInstance.ProvisioningState = provisioningStateInstance;
                                                    }
                                                }
                                                
                                                JToken nameValue = securityRulesValue["name"];
                                                if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                {
                                                    string nameInstance = ((string)nameValue);
                                                    securityRuleJsonFormatInstance.Name = nameInstance;
                                                }
                                                
                                                JToken etagValue = securityRulesValue["etag"];
                                                if (etagValue != null && etagValue.Type != JTokenType.Null)
                                                {
                                                    string etagInstance = ((string)etagValue);
                                                    securityRuleJsonFormatInstance.Etag = etagInstance;
                                                }
                                                
                                                JToken idValue = securityRulesValue["id"];
                                                if (idValue != null && idValue.Type != JTokenType.Null)
                                                {
                                                    string idInstance = ((string)idValue);
                                                    securityRuleJsonFormatInstance.Id = idInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken defaultSecurityRulesArray = propertiesValue["defaultSecurityRules"];
                                        if (defaultSecurityRulesArray != null && defaultSecurityRulesArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken defaultSecurityRulesValue in ((JArray)defaultSecurityRulesArray))
                                            {
                                                SecurityRule securityRuleJsonFormatInstance2 = new SecurityRule();
                                                networkSecurityGroupJsonFormatInstance.DefaultSecurityRules.Add(securityRuleJsonFormatInstance2);
                                                
                                                JToken propertiesValue3 = defaultSecurityRulesValue["properties"];
                                                if (propertiesValue3 != null && propertiesValue3.Type != JTokenType.Null)
                                                {
                                                    JToken descriptionValue2 = propertiesValue3["description"];
                                                    if (descriptionValue2 != null && descriptionValue2.Type != JTokenType.Null)
                                                    {
                                                        string descriptionInstance2 = ((string)descriptionValue2);
                                                        securityRuleJsonFormatInstance2.Description = descriptionInstance2;
                                                    }
                                                    
                                                    JToken protocolValue2 = propertiesValue3["protocol"];
                                                    if (protocolValue2 != null && protocolValue2.Type != JTokenType.Null)
                                                    {
                                                        string protocolInstance2 = ((string)protocolValue2);
                                                        securityRuleJsonFormatInstance2.Protocol = protocolInstance2;
                                                    }
                                                    
                                                    JToken sourcePortRangeValue2 = propertiesValue3["sourcePortRange"];
                                                    if (sourcePortRangeValue2 != null && sourcePortRangeValue2.Type != JTokenType.Null)
                                                    {
                                                        string sourcePortRangeInstance2 = ((string)sourcePortRangeValue2);
                                                        securityRuleJsonFormatInstance2.SourcePortRange = sourcePortRangeInstance2;
                                                    }
                                                    
                                                    JToken destinationPortRangeValue2 = propertiesValue3["destinationPortRange"];
                                                    if (destinationPortRangeValue2 != null && destinationPortRangeValue2.Type != JTokenType.Null)
                                                    {
                                                        string destinationPortRangeInstance2 = ((string)destinationPortRangeValue2);
                                                        securityRuleJsonFormatInstance2.DestinationPortRange = destinationPortRangeInstance2;
                                                    }
                                                    
                                                    JToken sourceAddressPrefixValue2 = propertiesValue3["sourceAddressPrefix"];
                                                    if (sourceAddressPrefixValue2 != null && sourceAddressPrefixValue2.Type != JTokenType.Null)
                                                    {
                                                        string sourceAddressPrefixInstance2 = ((string)sourceAddressPrefixValue2);
                                                        securityRuleJsonFormatInstance2.SourceAddressPrefix = sourceAddressPrefixInstance2;
                                                    }
                                                    
                                                    JToken destinationAddressPrefixValue2 = propertiesValue3["destinationAddressPrefix"];
                                                    if (destinationAddressPrefixValue2 != null && destinationAddressPrefixValue2.Type != JTokenType.Null)
                                                    {
                                                        string destinationAddressPrefixInstance2 = ((string)destinationAddressPrefixValue2);
                                                        securityRuleJsonFormatInstance2.DestinationAddressPrefix = destinationAddressPrefixInstance2;
                                                    }
                                                    
                                                    JToken accessValue2 = propertiesValue3["access"];
                                                    if (accessValue2 != null && accessValue2.Type != JTokenType.Null)
                                                    {
                                                        string accessInstance2 = ((string)accessValue2);
                                                        securityRuleJsonFormatInstance2.Access = accessInstance2;
                                                    }
                                                    
                                                    JToken priorityValue2 = propertiesValue3["priority"];
                                                    if (priorityValue2 != null && priorityValue2.Type != JTokenType.Null)
                                                    {
                                                        int priorityInstance2 = ((int)priorityValue2);
                                                        securityRuleJsonFormatInstance2.Priority = priorityInstance2;
                                                    }
                                                    
                                                    JToken directionValue2 = propertiesValue3["direction"];
                                                    if (directionValue2 != null && directionValue2.Type != JTokenType.Null)
                                                    {
                                                        string directionInstance2 = ((string)directionValue2);
                                                        securityRuleJsonFormatInstance2.Direction = directionInstance2;
                                                    }
                                                    
                                                    JToken provisioningStateValue2 = propertiesValue3["provisioningState"];
                                                    if (provisioningStateValue2 != null && provisioningStateValue2.Type != JTokenType.Null)
                                                    {
                                                        string provisioningStateInstance2 = ((string)provisioningStateValue2);
                                                        securityRuleJsonFormatInstance2.ProvisioningState = provisioningStateInstance2;
                                                    }
                                                }
                                                
                                                JToken nameValue2 = defaultSecurityRulesValue["name"];
                                                if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                {
                                                    string nameInstance2 = ((string)nameValue2);
                                                    securityRuleJsonFormatInstance2.Name = nameInstance2;
                                                }
                                                
                                                JToken etagValue2 = defaultSecurityRulesValue["etag"];
                                                if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                {
                                                    string etagInstance2 = ((string)etagValue2);
                                                    securityRuleJsonFormatInstance2.Etag = etagInstance2;
                                                }
                                                
                                                JToken idValue2 = defaultSecurityRulesValue["id"];
                                                if (idValue2 != null && idValue2.Type != JTokenType.Null)
                                                {
                                                    string idInstance2 = ((string)idValue2);
                                                    securityRuleJsonFormatInstance2.Id = idInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken networkInterfacesArray = propertiesValue["networkInterfaces"];
                                        if (networkInterfacesArray != null && networkInterfacesArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken networkInterfacesValue in ((JArray)networkInterfacesArray))
                                            {
                                                ResourceId resourceIdInstance = new ResourceId();
                                                networkSecurityGroupJsonFormatInstance.NetworkInterfaces.Add(resourceIdInstance);
                                                
                                                JToken idValue3 = networkInterfacesValue["id"];
                                                if (idValue3 != null && idValue3.Type != JTokenType.Null)
                                                {
                                                    string idInstance3 = ((string)idValue3);
                                                    resourceIdInstance.Id = idInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken subnetsArray = propertiesValue["subnets"];
                                        if (subnetsArray != null && subnetsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken subnetsValue in ((JArray)subnetsArray))
                                            {
                                                ResourceId resourceIdInstance2 = new ResourceId();
                                                networkSecurityGroupJsonFormatInstance.Subnets.Add(resourceIdInstance2);
                                                
                                                JToken idValue4 = subnetsValue["id"];
                                                if (idValue4 != null && idValue4.Type != JTokenType.Null)
                                                {
                                                    string idInstance4 = ((string)idValue4);
                                                    resourceIdInstance2.Id = idInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken resourceGuidValue = propertiesValue["resourceGuid"];
                                        if (resourceGuidValue != null && resourceGuidValue.Type != JTokenType.Null)
                                        {
                                            string resourceGuidInstance = ((string)resourceGuidValue);
                                            networkSecurityGroupJsonFormatInstance.ResourceGuid = resourceGuidInstance;
                                        }
                                        
                                        JToken provisioningStateValue3 = propertiesValue["provisioningState"];
                                        if (provisioningStateValue3 != null && provisioningStateValue3.Type != JTokenType.Null)
                                        {
                                            string provisioningStateInstance3 = ((string)provisioningStateValue3);
                                            networkSecurityGroupJsonFormatInstance.ProvisioningState = provisioningStateInstance3;
                                        }
                                    }
                                    
                                    JToken etagValue3 = valueValue["etag"];
                                    if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                    {
                                        string etagInstance3 = ((string)etagValue3);
                                        networkSecurityGroupJsonFormatInstance.Etag = etagInstance3;
                                    }
                                    
                                    JToken idValue5 = valueValue["id"];
                                    if (idValue5 != null && idValue5.Type != JTokenType.Null)
                                    {
                                        string idInstance5 = ((string)idValue5);
                                        networkSecurityGroupJsonFormatInstance.Id = idInstance5;
                                    }
                                    
                                    JToken nameValue3 = valueValue["name"];
                                    if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                    {
                                        string nameInstance3 = ((string)nameValue3);
                                        networkSecurityGroupJsonFormatInstance.Name = nameInstance3;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        networkSecurityGroupJsonFormatInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        networkSecurityGroupJsonFormatInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property.Name);
                                            string tagsValue = ((string)property.Value);
                                            networkSecurityGroupJsonFormatInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
