// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Windows Azure Network management API provides a RESTful set of web
    /// services that interact with Windows Azure Networks service to manage
    /// your network resrources. The API has entities that capture the
    /// relationship between an end user and the Windows Azure Networks
    /// service.
    /// </summary>
    public static partial class RouteOperationsExtensions
    {
        /// <summary>
        /// The Put route operation creates/updates a route in the specified
        /// route table
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <param name='routeParameters'>
        /// Required. Parameters supplied to the create/update routeoperation
        /// </param>
        /// <returns>
        /// Response for PUT Routes Api servive call
        /// </returns>
        public static RoutePutResponse BeginCreateOrUpdating(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName, Route routeParameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRouteOperations)s).BeginCreateOrUpdatingAsync(resourceGroupName, routeTableName, routeName, routeParameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put route operation creates/updates a route in the specified
        /// route table
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <param name='routeParameters'>
        /// Required. Parameters supplied to the create/update routeoperation
        /// </param>
        /// <returns>
        /// Response for PUT Routes Api servive call
        /// </returns>
        public static Task<RoutePutResponse> BeginCreateOrUpdatingAsync(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName, Route routeParameters)
        {
            return operations.BeginCreateOrUpdatingAsync(resourceGroupName, routeTableName, routeName, routeParameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The delete route operation deletes the specified route from a route
        /// table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static UpdateOperationResponse BeginDeleting(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRouteOperations)s).BeginDeletingAsync(resourceGroupName, routeTableName, routeName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The delete route operation deletes the specified route from a route
        /// table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static Task<UpdateOperationResponse> BeginDeletingAsync(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName)
        {
            return operations.BeginDeletingAsync(resourceGroupName, routeTableName, routeName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Put route operation creates/updates a route in the specified
        /// route table
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <param name='routeParameters'>
        /// Required. Parameters supplied to the create/update route operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse CreateOrUpdate(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName, Route routeParameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRouteOperations)s).CreateOrUpdateAsync(resourceGroupName, routeTableName, routeName, routeParameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put route operation creates/updates a route in the specified
        /// route table
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <param name='routeParameters'>
        /// Required. Parameters supplied to the create/update route operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName, Route routeParameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, routeTableName, routeName, routeParameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The delete route operation deletes the specified route from a route
        /// table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRouteOperations)s).DeleteAsync(resourceGroupName, routeTableName, routeName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The delete route operation deletes the specified route from a route
        /// table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName)
        {
            return operations.DeleteAsync(resourceGroupName, routeTableName, routeName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get route operation retreives information about the specified
        /// route from the route table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <returns>
        /// Response for GetRoute Api service call
        /// </returns>
        public static RouteGetResponse Get(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRouteOperations)s).GetAsync(resourceGroupName, routeTableName, routeName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get route operation retreives information about the specified
        /// route from the route table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <param name='routeName'>
        /// Required. The name of the route.
        /// </param>
        /// <returns>
        /// Response for GetRoute Api service call
        /// </returns>
        public static Task<RouteGetResponse> GetAsync(this IRouteOperations operations, string resourceGroupName, string routeTableName, string routeName)
        {
            return operations.GetAsync(resourceGroupName, routeTableName, routeName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List network security rule opertion retrieves all the routes in
        /// a route table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <returns>
        /// Response for ListRoute Api servive call
        /// </returns>
        public static RouteListResponse List(this IRouteOperations operations, string resourceGroupName, string routeTableName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IRouteOperations)s).ListAsync(resourceGroupName, routeTableName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List network security rule opertion retrieves all the routes in
        /// a route table.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IRouteOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='routeTableName'>
        /// Required. The name of the route table.
        /// </param>
        /// <returns>
        /// Response for ListRoute Api servive call
        /// </returns>
        public static Task<RouteListResponse> ListAsync(this IRouteOperations operations, string resourceGroupName, string routeTableName)
        {
            return operations.ListAsync(resourceGroupName, routeTableName, CancellationToken.None);
        }
    }
}
