// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Windows Azure Network management API provides a RESTful set of web
    /// services that interact with Windows Azure Networks service to manage
    /// your network resrources. The API has entities that capture the
    /// relationship between an end user and the Windows Azure Networks
    /// service.
    /// </summary>
    public static partial class SubnetOperationsExtensions
    {
        /// <summary>
        /// The Put Subnet operation creates/updates a subnet in thespecified
        /// virtual network
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <param name='subnetParameters'>
        /// Required. Parameters supplied to the create/update Subnet operation
        /// </param>
        /// <returns>
        /// Response for PutSubnet Api service call
        /// </returns>
        public static SubnetPutResponse BeginCreateOrUpdating(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName, Subnet subnetParameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubnetOperations)s).BeginCreateOrUpdatingAsync(resourceGroupName, virtualNetworkName, subnetName, subnetParameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put Subnet operation creates/updates a subnet in thespecified
        /// virtual network
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <param name='subnetParameters'>
        /// Required. Parameters supplied to the create/update Subnet operation
        /// </param>
        /// <returns>
        /// Response for PutSubnet Api service call
        /// </returns>
        public static Task<SubnetPutResponse> BeginCreateOrUpdatingAsync(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName, Subnet subnetParameters)
        {
            return operations.BeginCreateOrUpdatingAsync(resourceGroupName, virtualNetworkName, subnetName, subnetParameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The delete subnet operation deletes the specified subnet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static UpdateOperationResponse BeginDeleting(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubnetOperations)s).BeginDeletingAsync(resourceGroupName, virtualNetworkName, subnetName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The delete subnet operation deletes the specified subnet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static Task<UpdateOperationResponse> BeginDeletingAsync(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName)
        {
            return operations.BeginDeletingAsync(resourceGroupName, virtualNetworkName, subnetName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Put Subnet operation creates/updates a subnet in thespecified
        /// virtual network
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <param name='subnetParameters'>
        /// Required. Parameters supplied to the create/update Subnet operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse CreateOrUpdate(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName, Subnet subnetParameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubnetOperations)s).CreateOrUpdateAsync(resourceGroupName, virtualNetworkName, subnetName, subnetParameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put Subnet operation creates/updates a subnet in thespecified
        /// virtual network
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <param name='subnetParameters'>
        /// Required. Parameters supplied to the create/update Subnet operation
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName, Subnet subnetParameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, virtualNetworkName, subnetName, subnetParameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The delete subnet operation deletes the specified subnet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubnetOperations)s).DeleteAsync(resourceGroupName, virtualNetworkName, subnetName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The delete subnet operation deletes the specified subnet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName)
        {
            return operations.DeleteAsync(resourceGroupName, virtualNetworkName, subnetName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get subnet operation retreives information about the specified
        /// subnet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <returns>
        /// Response for GetSubnet Api service call
        /// </returns>
        public static SubnetGetResponse Get(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubnetOperations)s).GetAsync(resourceGroupName, virtualNetworkName, subnetName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get subnet operation retreives information about the specified
        /// subnet.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <param name='subnetName'>
        /// Required. The name of the subnet.
        /// </param>
        /// <returns>
        /// Response for GetSubnet Api service call
        /// </returns>
        public static Task<SubnetGetResponse> GetAsync(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName, string subnetName)
        {
            return operations.GetAsync(resourceGroupName, virtualNetworkName, subnetName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List subnets opertion retrieves all the subnets in a virtual
        /// network.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <returns>
        /// Response for ListSubnets Api service callRetrieves all subnet that
        /// belongs to a virtual network
        /// </returns>
        public static SubnetListResponse List(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ISubnetOperations)s).ListAsync(resourceGroupName, virtualNetworkName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List subnets opertion retrieves all the subnets in a virtual
        /// network.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.ISubnetOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkName'>
        /// Required. The name of the virtual network.
        /// </param>
        /// <returns>
        /// Response for ListSubnets Api service callRetrieves all subnet that
        /// belongs to a virtual network
        /// </returns>
        public static Task<SubnetListResponse> ListAsync(this ISubnetOperations operations, string resourceGroupName, string virtualNetworkName)
        {
            return operations.ListAsync(resourceGroupName, virtualNetworkName, CancellationToken.None);
        }
    }
}
