// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Network;
using Microsoft.Azure.Management.Network.Models;

namespace Microsoft.Azure.Management.Network
{
    /// <summary>
    /// The Windows Azure Network management API provides a RESTful set of web
    /// services that interact with Windows Azure Networks service to manage
    /// your network resrources. The API has entities that capture the
    /// relationship between an end user and the Windows Azure Networks
    /// service.
    /// </summary>
    public static partial class VirtualNetworkGatewayConnectionOperationsExtensions
    {
        /// <summary>
        /// The Put VirtualNetworkGatewayConnection operation creates/updates a
        /// virtual network gateway connection in the specified resource group
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway conenction.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Virtual
        /// Network Gateway connection operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// Response for CreateOrUpdateVirtualNetworkGatewayConnection Api
        /// servive call
        /// </returns>
        public static VirtualNetworkGatewayConnectionPutResponse BeginCreateOrUpdating(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).BeginCreateOrUpdatingAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put VirtualNetworkGatewayConnection operation creates/updates a
        /// virtual network gateway connection in the specified resource group
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway conenction.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Virtual
        /// Network Gateway connection operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// Response for CreateOrUpdateVirtualNetworkGatewayConnection Api
        /// servive call
        /// </returns>
        public static Task<VirtualNetworkGatewayConnectionPutResponse> BeginCreateOrUpdatingAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters)
        {
            return operations.BeginCreateOrUpdatingAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Delete VirtualNetworkGatewayConnection operation deletes the
        /// specifed virtual network Gateway connection through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway connection.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static UpdateOperationResponse BeginDeleting(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).BeginDeletingAsync(resourceGroupName, virtualNetworkGatewayConnectionName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Delete VirtualNetworkGatewayConnection operation deletes the
        /// specifed virtual network Gateway connection through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway connection.
        /// </param>
        /// <returns>
        /// If the resource provide needs to return an error to any operation,
        /// it should return the appropriate HTTP error code and a message
        /// body as can be seen below.The message should be localized per the
        /// Accept-Language header specified in the original request such
        /// thatit could be directly be exposed to users
        /// </returns>
        public static Task<UpdateOperationResponse> BeginDeletingAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName)
        {
            return operations.BeginDeletingAsync(resourceGroupName, virtualNetworkGatewayConnectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// The VirtualNetworkGatewayConnectionResetSharedKey operation resets
        /// the virtual network gateway connection shared key for passed
        /// virtual network gateway connection in the specified resource group
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection reset shared key
        /// Name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Reset Virtual Network
        /// Gateway connection shared key operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// Response for PutVirtualNetworkGatewayConnectionResetSharedKey Api
        /// servive call
        /// </returns>
        public static ConnectionResetSharedKeyPutResponse BeginResetSharedKey(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).BeginResetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The VirtualNetworkGatewayConnectionResetSharedKey operation resets
        /// the virtual network gateway connection shared key for passed
        /// virtual network gateway connection in the specified resource group
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection reset shared key
        /// Name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Reset Virtual Network
        /// Gateway connection shared key operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// Response for PutVirtualNetworkGatewayConnectionResetSharedKey Api
        /// servive call
        /// </returns>
        public static Task<ConnectionResetSharedKeyPutResponse> BeginResetSharedKeyAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters)
        {
            return operations.BeginResetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
        /// virtual network gateway connection shared key for passed virtual
        /// network gateway connection in the specified resource group through
        /// Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Set Virtual Network
        /// Gateway conection Shared key operation throughNetwork resource
        /// provider.
        /// </param>
        /// <returns>
        /// Response for PutVirtualNetworkGatewayConnectionSharedKey Api
        /// servive call
        /// </returns>
        public static ConnectionSharedKeyPutResponse BeginSetSharedKey(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).BeginSetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
        /// virtual network gateway connection shared key for passed virtual
        /// network gateway connection in the specified resource group through
        /// Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Set Virtual Network
        /// Gateway conection Shared key operation throughNetwork resource
        /// provider.
        /// </param>
        /// <returns>
        /// Response for PutVirtualNetworkGatewayConnectionSharedKey Api
        /// servive call
        /// </returns>
        public static Task<ConnectionSharedKeyPutResponse> BeginSetSharedKeyAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters)
        {
            return operations.BeginSetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The CreateOrUpdate Virtual network Gateway connection creates a new
        /// or updates an existing virtualnetwork gateway connection through
        /// Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway conenction.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Virtual
        /// Network Gateway connection operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse CreateOrUpdate(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).CreateOrUpdateAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The CreateOrUpdate Virtual network Gateway connection creates a new
        /// or updates an existing virtualnetwork gateway connection through
        /// Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway conenction.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Virtual
        /// Network Gateway connection operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> CreateOrUpdateAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, VirtualNetworkGatewayConnection parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Delete VirtualNetworkGatewayConnection operation deletes the
        /// specifed virtual network Gateway connection through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway connection.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).DeleteAsync(resourceGroupName, virtualNetworkGatewayConnectionName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Delete VirtualNetworkGatewayConnection operation deletes the
        /// specifed virtual network Gateway connection through Network
        /// resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway connection.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName)
        {
            return operations.DeleteAsync(resourceGroupName, virtualNetworkGatewayConnectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get VirtualNetworkGatewayConnection operation retrieves
        /// information about the specified virtual network gateway connection
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway connection.
        /// </param>
        /// <returns>
        /// Response for GetVitualNetworkGatewayConnection Api servive call.
        /// </returns>
        public static VirtualNetworkGatewayConnectionGetResponse Get(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).GetAsync(resourceGroupName, virtualNetworkGatewayConnectionName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get VirtualNetworkGatewayConnection operation retrieves
        /// information about the specified virtual network gateway connection
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The name of the virtual network gateway connection.
        /// </param>
        /// <returns>
        /// Response for GetVitualNetworkGatewayConnection Api servive call.
        /// </returns>
        public static Task<VirtualNetworkGatewayConnectionGetResponse> GetAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName)
        {
            return operations.GetAsync(resourceGroupName, virtualNetworkGatewayConnectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Get VirtualNetworkGatewayConnectionSharedKey operation
        /// retrieves information about the specified virtual network gateway
        /// connection shared key through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='connectionSharedKeyName'>
        /// Required. The virtual network gateway connection shared key name.
        /// </param>
        /// <returns>
        /// Response for CheckConnectionSharedKey Api servive call
        /// </returns>
        public static ConnectionSharedKeyResponse GetSharedKey(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string connectionSharedKeyName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).GetSharedKeyAsync(resourceGroupName, connectionSharedKeyName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Get VirtualNetworkGatewayConnectionSharedKey operation
        /// retrieves information about the specified virtual network gateway
        /// connection shared key through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='connectionSharedKeyName'>
        /// Required. The virtual network gateway connection shared key name.
        /// </param>
        /// <returns>
        /// Response for CheckConnectionSharedKey Api servive call
        /// </returns>
        public static Task<ConnectionSharedKeyResponse> GetSharedKeyAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string connectionSharedKeyName)
        {
            return operations.GetSharedKeyAsync(resourceGroupName, connectionSharedKeyName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List VirtualNetworkGatewayConnections operation retrieves all
        /// the virtual network gateways connections created.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// Response for ListVirtualNetworkGatewayConnections Api service call
        /// </returns>
        public static VirtualNetworkGatewayConnectionListResponse List(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).ListAsync(resourceGroupName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List VirtualNetworkGatewayConnections operation retrieves all
        /// the virtual network gateways connections created.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <returns>
        /// Response for ListVirtualNetworkGatewayConnections Api service call
        /// </returns>
        public static Task<VirtualNetworkGatewayConnectionListResponse> ListAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName)
        {
            return operations.ListAsync(resourceGroupName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Reset VirtualNetworkGatewayConnectionSharedKey operation resets
        /// the virtual network gateway connection shared key for passed
        /// virtual network gateway connection in the specified resource group
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Reset Virtual Network
        /// Gateway connection shared key operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse ResetSharedKey(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).ResetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Reset VirtualNetworkGatewayConnectionSharedKey operation resets
        /// the virtual network gateway connection shared key for passed
        /// virtual network gateway connection in the specified resource group
        /// through Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Reset Virtual Network
        /// Gateway connection shared key operation through Network resource
        /// provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> ResetSharedKeyAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionResetSharedKey parameters)
        {
            return operations.ResetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
        /// virtual network gateway connection shared key for passed virtual
        /// network gateway connection in the specified resource group through
        /// Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Virtual
        /// Network Gateway connection shared key operation through Network
        /// resource provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static AzureAsyncOperationResponse SetSharedKey(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualNetworkGatewayConnectionOperations)s).SetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Put VirtualNetworkGatewayConnectionSharedKey operation sets the
        /// virtual network gateway connection shared key for passed virtual
        /// network gateway connection in the specified resource group through
        /// Network resource provider.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Network.IVirtualNetworkGatewayConnectionOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='virtualNetworkGatewayConnectionName'>
        /// Required. The virtual network gateway connection name.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Begin Create or update Virtual
        /// Network Gateway connection shared key operation through Network
        /// resource provider.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<AzureAsyncOperationResponse> SetSharedKeyAsync(this IVirtualNetworkGatewayConnectionOperations operations, string resourceGroupName, string virtualNetworkGatewayConnectionName, ConnectionSharedKey parameters)
        {
            return operations.SetSharedKeyAsync(resourceGroupName, virtualNetworkGatewayConnectionName, parameters, CancellationToken.None);
        }
    }
}
