// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.OperationalInsights;
using Microsoft.Azure.Management.OperationalInsights.Models;

namespace Microsoft.Azure.Management.OperationalInsights
{
    /// <summary>
    /// .Net client wrapper for the REST API for Azure Operational Insights
    /// </summary>
    public static partial class DataSourceOperationsExtensions
    {
        /// <summary>
        /// Create or update a data source.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data source.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the parent workspace that will contain the
        /// data source
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a data source.
        /// </param>
        /// <returns>
        /// The create or update data source operation response.
        /// </returns>
        public static DataSourceCreateOrUpdateResponse CreateOrUpdate(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, DataSourceCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDataSourceOperations)s).CreateOrUpdateAsync(resourceGroupName, workspaceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or update a data source.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data source.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the parent workspace that will contain the
        /// data source
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a data source.
        /// </param>
        /// <returns>
        /// The create or update data source operation response.
        /// </returns>
        public static Task<DataSourceCreateOrUpdateResponse> CreateOrUpdateAsync(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, DataSourceCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, workspaceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a data source instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data source.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the data source.
        /// </param>
        /// <param name='datasourceName'>
        /// Required. Name of the data source.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, string datasourceName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDataSourceOperations)s).DeleteAsync(resourceGroupName, workspaceName, datasourceName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a data source instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data source.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the data source.
        /// </param>
        /// <param name='datasourceName'>
        /// Required. Name of the data source.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, string datasourceName)
        {
            return operations.DeleteAsync(resourceGroupName, workspaceName, datasourceName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a data source instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data source.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the data source.
        /// </param>
        /// <param name='dataSourceName'>
        /// Required. Name of the data source
        /// </param>
        /// <returns>
        /// The get data source operation response.
        /// </returns>
        public static DataSourceGetResponse Get(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, string dataSourceName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDataSourceOperations)s).GetAsync(resourceGroupName, workspaceName, dataSourceName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a data source instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data source.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the data source.
        /// </param>
        /// <param name='dataSourceName'>
        /// Required. Name of the data source
        /// </param>
        /// <returns>
        /// The get data source operation response.
        /// </returns>
        public static Task<DataSourceGetResponse> GetAsync(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, string dataSourceName)
        {
            return operations.GetAsync(resourceGroupName, workspaceName, dataSourceName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the first page of data source instances in a workspace with
        /// the link to the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data sources.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The workspace that contains the data sources.
        /// </param>
        /// <param name='kind'>
        /// Required. Filter data sources by Kind.
        /// </param>
        /// <param name='skiptoken'>
        /// Optional. Token for paging support.
        /// </param>
        /// <returns>
        /// The list data source operation response.
        /// </returns>
        public static DataSourceListResponse ListInWorkspace(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, string kind, string skiptoken)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDataSourceOperations)s).ListInWorkspaceAsync(resourceGroupName, workspaceName, kind, skiptoken);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the first page of data source instances in a workspace with
        /// the link to the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the data sources.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The workspace that contains the data sources.
        /// </param>
        /// <param name='kind'>
        /// Required. Filter data sources by Kind.
        /// </param>
        /// <param name='skiptoken'>
        /// Optional. Token for paging support.
        /// </param>
        /// <returns>
        /// The list data source operation response.
        /// </returns>
        public static Task<DataSourceListResponse> ListInWorkspaceAsync(this IDataSourceOperations operations, string resourceGroupName, string workspaceName, string kind, string skiptoken)
        {
            return operations.ListInWorkspaceAsync(resourceGroupName, workspaceName, kind, skiptoken, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the next page of data source instances with the link to the
        /// next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to the next data source page.
        /// </param>
        /// <returns>
        /// The list data source operation response.
        /// </returns>
        public static DataSourceListResponse ListNext(this IDataSourceOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDataSourceOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the next page of data source instances with the link to the
        /// next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.IDataSourceOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to the next data source page.
        /// </param>
        /// <returns>
        /// The list data source operation response.
        /// </returns>
        public static Task<DataSourceListResponse> ListNextAsync(this IDataSourceOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
