// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.OperationalInsights.Models;

namespace Microsoft.Azure.Management.OperationalInsights
{
    /// <summary>
    /// Operations for managing Operational Insights linked services.
    /// </summary>
    public partial interface ILinkedServiceOperations
    {
        /// <summary>
        /// Create or update a linked service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// The name of the parent workspace that will contain the linked
        /// service
        /// </param>
        /// <param name='parameters'>
        /// The parameters required to create or update a linked service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The create or update linked service operation response.
        /// </returns>
        Task<LinkedServiceCreateOrUpdateResponse> CreateOrUpdateAsync(string resourceGroupName, string workspaceName, LinkedServiceCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Deletes a linked service instance.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// The name of the workspace that contains the linked service.
        /// </param>
        /// <param name='linkedServiceName'>
        /// Name of the linked service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string workspaceName, string linkedServiceName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets a linked service instance.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// The name of the workspace that contains the linked service.
        /// </param>
        /// <param name='linkedServiceName'>
        /// Name of the linked service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The get linked service operation response.
        /// </returns>
        Task<LinkedServiceGetResponse> GetAsync(string resourceGroupName, string workspaceName, string linkedServiceName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the first page of linked services instances in a workspace
        /// with the link to the next page.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The resource group name of the linked services.
        /// </param>
        /// <param name='workspaceName'>
        /// The workspace that contains the linked services.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The list linked service operation response.
        /// </returns>
        Task<LinkedServiceListResponse> ListAsync(string resourceGroupName, string workspaceName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the next page of linked service instances with the link to the
        /// next page.
        /// </summary>
        /// <param name='nextLink'>
        /// The url to the next linked service page.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The list linked service operation response.
        /// </returns>
        Task<LinkedServiceListResponse> ListNextAsync(string nextLink, CancellationToken cancellationToken);
    }
}
