// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.OperationalInsights;
using Microsoft.Azure.Management.OperationalInsights.Models;

namespace Microsoft.Azure.Management.OperationalInsights
{
    /// <summary>
    /// .Net client wrapper for the REST API for Azure Operational Insights
    /// </summary>
    public static partial class LinkedServiceOperationsExtensions
    {
        /// <summary>
        /// Create or update a linked service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the parent workspace that will contain the
        /// linked service
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a linked
        /// service.
        /// </param>
        /// <returns>
        /// The create or update linked service operation response.
        /// </returns>
        public static LinkedServiceCreateOrUpdateResponse CreateOrUpdate(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName, LinkedServiceCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILinkedServiceOperations)s).CreateOrUpdateAsync(resourceGroupName, workspaceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or update a linked service.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the parent workspace that will contain the
        /// linked service
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a linked
        /// service.
        /// </param>
        /// <returns>
        /// The create or update linked service operation response.
        /// </returns>
        public static Task<LinkedServiceCreateOrUpdateResponse> CreateOrUpdateAsync(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName, LinkedServiceCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, workspaceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a linked service instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the linked
        /// service.
        /// </param>
        /// <param name='linkedServiceName'>
        /// Required. Name of the linked service.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName, string linkedServiceName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILinkedServiceOperations)s).DeleteAsync(resourceGroupName, workspaceName, linkedServiceName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a linked service instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the linked
        /// service.
        /// </param>
        /// <param name='linkedServiceName'>
        /// Required. Name of the linked service.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName, string linkedServiceName)
        {
            return operations.DeleteAsync(resourceGroupName, workspaceName, linkedServiceName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a linked service instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the linked
        /// service.
        /// </param>
        /// <param name='linkedServiceName'>
        /// Required. Name of the linked service.
        /// </param>
        /// <returns>
        /// The get linked service operation response.
        /// </returns>
        public static LinkedServiceGetResponse Get(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName, string linkedServiceName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILinkedServiceOperations)s).GetAsync(resourceGroupName, workspaceName, linkedServiceName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a linked service instance.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked service.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The name of the workspace that contains the linked
        /// service.
        /// </param>
        /// <param name='linkedServiceName'>
        /// Required. Name of the linked service.
        /// </param>
        /// <returns>
        /// The get linked service operation response.
        /// </returns>
        public static Task<LinkedServiceGetResponse> GetAsync(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName, string linkedServiceName)
        {
            return operations.GetAsync(resourceGroupName, workspaceName, linkedServiceName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the first page of linked services instances in a workspace
        /// with the link to the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked services.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The workspace that contains the linked services.
        /// </param>
        /// <returns>
        /// The list linked service operation response.
        /// </returns>
        public static LinkedServiceListResponse List(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILinkedServiceOperations)s).ListAsync(resourceGroupName, workspaceName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the first page of linked services instances in a workspace
        /// with the link to the next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the linked services.
        /// </param>
        /// <param name='workspaceName'>
        /// Required. The workspace that contains the linked services.
        /// </param>
        /// <returns>
        /// The list linked service operation response.
        /// </returns>
        public static Task<LinkedServiceListResponse> ListAsync(this ILinkedServiceOperations operations, string resourceGroupName, string workspaceName)
        {
            return operations.ListAsync(resourceGroupName, workspaceName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the next page of linked service instances with the link to the
        /// next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to the next linked service page.
        /// </param>
        /// <returns>
        /// The list linked service operation response.
        /// </returns>
        public static LinkedServiceListResponse ListNext(this ILinkedServiceOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((ILinkedServiceOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the next page of linked service instances with the link to the
        /// next page.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.OperationalInsights.ILinkedServiceOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The url to the next linked service page.
        /// </param>
        /// <returns>
        /// The list linked service operation response.
        /// </returns>
        public static Task<LinkedServiceListResponse> ListNextAsync(this ILinkedServiceOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
