// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Microsoft.Azure;
using Microsoft.Azure.Management.RecoveryServices.Backup;
using Microsoft.Azure.Management.RecoveryServices.Backup.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Management.RecoveryServices.Backup
{
    /// <summary>
    /// The Resource Manager API includes operations for managing the
    /// containers registered to your Recovery Services Vault.
    /// </summary>
    internal partial class ContainerOperations : IServiceOperations<RecoveryServicesBackupManagementClient>, IContainerOperations
    {
        /// <summary>
        /// Initializes a new instance of the ContainerOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal ContainerOperations(RecoveryServicesBackupManagementClient client)
        {
            this._client = client;
        }
        
        private RecoveryServicesBackupManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.Azure.Management.RecoveryServices.Backup.RecoveryServicesBackupManagementClient.
        /// </summary>
        public RecoveryServicesBackupManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// The Begin Refresh Operation triggers an operation in the service
        /// which would discover all the containers in the subscription that
        /// are ready to be protected by your Recovery Services Vault. This is
        /// an asynchronous operation. To determine whether the backend
        /// service has finished processing the request, call Get Refresh
        /// Operation Result APIs.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. Resource group name of your recovery services vault.
        /// </param>
        /// <param name='resourceName'>
        /// Required. Nme of your recovery services vault.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Required. Request header parameters.
        /// </param>
        /// <param name='fabricName'>
        /// Optional. Fabric name for the protection containers.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Base recovery job response for all the asynchronous operations.
        /// </returns>
        public async Task<BaseRecoveryServicesJobResponse> BeginRefreshAsync(string resourceGroupName, string resourceName, CustomRequestHeaders customRequestHeaders, string fabricName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (resourceName == null)
            {
                throw new ArgumentNullException("resourceName");
            }
            if (customRequestHeaders == null)
            {
                throw new ArgumentNullException("customRequestHeaders");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("resourceName", resourceName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                tracingParameters.Add("fabricName", fabricName);
                TracingAdapter.Enter(invocationId, this, "BeginRefreshAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            if (this.Client.ResourceNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            }
            url = url + "/";
            url = url + "vaults";
            url = url + "/";
            url = url + Uri.EscapeDataString(resourceName);
            url = url + "/backupFabrics/";
            if (fabricName != null)
            {
                url = url + Uri.EscapeDataString(fabricName);
            }
            url = url + "/refreshContainers";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2016-05-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    BaseRecoveryServicesJobResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new BaseRecoveryServicesJobResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                result.Location = locationInstance;
                            }
                            
                            JToken azureAsyncOperationValue = responseDoc["azureAsyncOperation"];
                            if (azureAsyncOperationValue != null && azureAsyncOperationValue.Type != JTokenType.Null)
                            {
                                string azureAsyncOperationInstance = ((string)azureAsyncOperationValue);
                                result.AzureAsyncOperation = azureAsyncOperationInstance;
                            }
                            
                            JToken retryAfterValue = responseDoc["retryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                string retryAfterInstance = ((string)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AzureAsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = httpResponse.Headers.GetValues("Retry-After").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Fetches the result of any operation on the container given the ID
        /// of operation.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. Resource group name of your recovery services vault.
        /// </param>
        /// <param name='resourceName'>
        /// Required. Name of your recovery services vault.
        /// </param>
        /// <param name='fabricName'>
        /// Optional. Fabric name of the protected item.
        /// </param>
        /// <param name='containerName'>
        /// Required. Name of the container where the protected item belongs to.
        /// </param>
        /// <param name='operationId'>
        /// Required. ID of the container operation whose result has to be
        /// fetched.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Protection container response.
        /// </returns>
        public async Task<ProtectionContainerResponse> GetContainerOperationResultAsync(string resourceGroupName, string resourceName, string fabricName, string containerName, string operationId, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (resourceName == null)
            {
                throw new ArgumentNullException("resourceName");
            }
            if (containerName == null)
            {
                throw new ArgumentNullException("containerName");
            }
            if (operationId == null)
            {
                throw new ArgumentNullException("operationId");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("resourceName", resourceName);
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("containerName", containerName);
                tracingParameters.Add("operationId", operationId);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "GetContainerOperationResultAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            if (this.Client.ResourceNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            }
            url = url + "/";
            url = url + "vaults";
            url = url + "/";
            url = url + Uri.EscapeDataString(resourceName);
            url = url + "/backupFabrics/";
            if (fabricName != null)
            {
                url = url + Uri.EscapeDataString(fabricName);
            }
            url = url + "/protectionContainers/";
            url = url + Uri.EscapeDataString(containerName);
            url = url + "/operationResults/";
            url = url + Uri.EscapeDataString(operationId);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2016-05-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ProtectionContainerResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted || statusCode == HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ProtectionContainerResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ProtectionContainerResource itemInstance = new ProtectionContainerResource();
                            result.Item = itemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                string typeName = ((string)propertiesValue["protectableObjectType"]);
                                if (typeName == "ProtectionContainer")
                                {
                                    ProtectionContainer protectionContainerInstance = new ProtectionContainer();
                                    
                                    JToken friendlyNameValue = propertiesValue["friendlyName"];
                                    if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance = ((string)friendlyNameValue);
                                        protectionContainerInstance.FriendlyName = friendlyNameInstance;
                                    }
                                    
                                    JToken registrationStatusValue = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue != null && registrationStatusValue.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance = ((string)registrationStatusValue);
                                        protectionContainerInstance.RegistrationStatus = registrationStatusInstance;
                                    }
                                    
                                    JToken healthStatusValue = propertiesValue["healthStatus"];
                                    if (healthStatusValue != null && healthStatusValue.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance = ((string)healthStatusValue);
                                        protectionContainerInstance.HealthStatus = healthStatusInstance;
                                    }
                                    
                                    JToken containerTypeValue = propertiesValue["containerType"];
                                    if (containerTypeValue != null && containerTypeValue.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance = ((string)containerTypeValue);
                                        protectionContainerInstance.ContainerType = containerTypeInstance;
                                    }
                                    
                                    JToken backupManagementTypeValue = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue != null && backupManagementTypeValue.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance = ((string)backupManagementTypeValue);
                                        protectionContainerInstance.BackupManagementType = backupManagementTypeInstance;
                                    }
                                    
                                    JToken protectableObjectTypeValue = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue != null && protectableObjectTypeValue.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance = ((string)protectableObjectTypeValue);
                                        protectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance;
                                    }
                                    itemInstance.Properties = protectionContainerInstance;
                                }
                                if (typeName == "AzureSqlContainer")
                                {
                                    AzureSqlProtectionContainer azureSqlProtectionContainerInstance = new AzureSqlProtectionContainer();
                                    
                                    JToken friendlyNameValue2 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                        azureSqlProtectionContainerInstance.FriendlyName = friendlyNameInstance2;
                                    }
                                    
                                    JToken registrationStatusValue2 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue2 != null && registrationStatusValue2.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance2 = ((string)registrationStatusValue2);
                                        azureSqlProtectionContainerInstance.RegistrationStatus = registrationStatusInstance2;
                                    }
                                    
                                    JToken healthStatusValue2 = propertiesValue["healthStatus"];
                                    if (healthStatusValue2 != null && healthStatusValue2.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance2 = ((string)healthStatusValue2);
                                        azureSqlProtectionContainerInstance.HealthStatus = healthStatusInstance2;
                                    }
                                    
                                    JToken containerTypeValue2 = propertiesValue["containerType"];
                                    if (containerTypeValue2 != null && containerTypeValue2.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance2 = ((string)containerTypeValue2);
                                        azureSqlProtectionContainerInstance.ContainerType = containerTypeInstance2;
                                    }
                                    
                                    JToken backupManagementTypeValue2 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue2 != null && backupManagementTypeValue2.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance2 = ((string)backupManagementTypeValue2);
                                        azureSqlProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance2;
                                    }
                                    
                                    JToken protectableObjectTypeValue2 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue2 != null && protectableObjectTypeValue2.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance2 = ((string)protectableObjectTypeValue2);
                                        azureSqlProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance2;
                                    }
                                    itemInstance.Properties = azureSqlProtectionContainerInstance;
                                }
                                if (typeName == "IaaSVMContainer")
                                {
                                    AzureIaaSVMProtectionContainer azureIaaSVMProtectionContainerInstance = new AzureIaaSVMProtectionContainer();
                                    
                                    JToken resourceGroupValue = propertiesValue["resourceGroup"];
                                    if (resourceGroupValue != null && resourceGroupValue.Type != JTokenType.Null)
                                    {
                                        string resourceGroupInstance = ((string)resourceGroupValue);
                                        azureIaaSVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance;
                                    }
                                    
                                    JToken virtualMachineIdValue = propertiesValue["virtualMachineId"];
                                    if (virtualMachineIdValue != null && virtualMachineIdValue.Type != JTokenType.Null)
                                    {
                                        string virtualMachineIdInstance = ((string)virtualMachineIdValue);
                                        azureIaaSVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance;
                                    }
                                    
                                    JToken friendlyNameValue3 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                        azureIaaSVMProtectionContainerInstance.FriendlyName = friendlyNameInstance3;
                                    }
                                    
                                    JToken registrationStatusValue3 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue3 != null && registrationStatusValue3.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance3 = ((string)registrationStatusValue3);
                                        azureIaaSVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance3;
                                    }
                                    
                                    JToken healthStatusValue3 = propertiesValue["healthStatus"];
                                    if (healthStatusValue3 != null && healthStatusValue3.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance3 = ((string)healthStatusValue3);
                                        azureIaaSVMProtectionContainerInstance.HealthStatus = healthStatusInstance3;
                                    }
                                    
                                    JToken containerTypeValue3 = propertiesValue["containerType"];
                                    if (containerTypeValue3 != null && containerTypeValue3.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance3 = ((string)containerTypeValue3);
                                        azureIaaSVMProtectionContainerInstance.ContainerType = containerTypeInstance3;
                                    }
                                    
                                    JToken backupManagementTypeValue3 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue3 != null && backupManagementTypeValue3.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance3 = ((string)backupManagementTypeValue3);
                                        azureIaaSVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance3;
                                    }
                                    
                                    JToken protectableObjectTypeValue3 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue3 != null && protectableObjectTypeValue3.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance3 = ((string)protectableObjectTypeValue3);
                                        azureIaaSVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance3;
                                    }
                                    itemInstance.Properties = azureIaaSVMProtectionContainerInstance;
                                }
                                if (typeName == "Microsoft.ClassicCompute/virtualMachines")
                                {
                                    AzureIaaSClassicComputeVMProtectionContainer azureIaaSClassicComputeVMProtectionContainerInstance = new AzureIaaSClassicComputeVMProtectionContainer();
                                    
                                    JToken resourceGroupValue2 = propertiesValue["resourceGroup"];
                                    if (resourceGroupValue2 != null && resourceGroupValue2.Type != JTokenType.Null)
                                    {
                                        string resourceGroupInstance2 = ((string)resourceGroupValue2);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance2;
                                    }
                                    
                                    JToken virtualMachineIdValue2 = propertiesValue["virtualMachineId"];
                                    if (virtualMachineIdValue2 != null && virtualMachineIdValue2.Type != JTokenType.Null)
                                    {
                                        string virtualMachineIdInstance2 = ((string)virtualMachineIdValue2);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance2;
                                    }
                                    
                                    JToken friendlyNameValue4 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.FriendlyName = friendlyNameInstance4;
                                    }
                                    
                                    JToken registrationStatusValue4 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue4 != null && registrationStatusValue4.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance4 = ((string)registrationStatusValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance4;
                                    }
                                    
                                    JToken healthStatusValue4 = propertiesValue["healthStatus"];
                                    if (healthStatusValue4 != null && healthStatusValue4.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance4 = ((string)healthStatusValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.HealthStatus = healthStatusInstance4;
                                    }
                                    
                                    JToken containerTypeValue4 = propertiesValue["containerType"];
                                    if (containerTypeValue4 != null && containerTypeValue4.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance4 = ((string)containerTypeValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.ContainerType = containerTypeInstance4;
                                    }
                                    
                                    JToken backupManagementTypeValue4 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue4 != null && backupManagementTypeValue4.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance4 = ((string)backupManagementTypeValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance4;
                                    }
                                    
                                    JToken protectableObjectTypeValue4 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue4 != null && protectableObjectTypeValue4.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance4 = ((string)protectableObjectTypeValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance4;
                                    }
                                    itemInstance.Properties = azureIaaSClassicComputeVMProtectionContainerInstance;
                                }
                                if (typeName == "Microsoft.Compute/virtualMachines")
                                {
                                    AzureIaaSComputeVMProtectionContainer azureIaaSComputeVMProtectionContainerInstance = new AzureIaaSComputeVMProtectionContainer();
                                    
                                    JToken resourceGroupValue3 = propertiesValue["resourceGroup"];
                                    if (resourceGroupValue3 != null && resourceGroupValue3.Type != JTokenType.Null)
                                    {
                                        string resourceGroupInstance3 = ((string)resourceGroupValue3);
                                        azureIaaSComputeVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance3;
                                    }
                                    
                                    JToken virtualMachineIdValue3 = propertiesValue["virtualMachineId"];
                                    if (virtualMachineIdValue3 != null && virtualMachineIdValue3.Type != JTokenType.Null)
                                    {
                                        string virtualMachineIdInstance3 = ((string)virtualMachineIdValue3);
                                        azureIaaSComputeVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance3;
                                    }
                                    
                                    JToken friendlyNameValue5 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.FriendlyName = friendlyNameInstance5;
                                    }
                                    
                                    JToken registrationStatusValue5 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue5 != null && registrationStatusValue5.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance5 = ((string)registrationStatusValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance5;
                                    }
                                    
                                    JToken healthStatusValue5 = propertiesValue["healthStatus"];
                                    if (healthStatusValue5 != null && healthStatusValue5.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance5 = ((string)healthStatusValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.HealthStatus = healthStatusInstance5;
                                    }
                                    
                                    JToken containerTypeValue5 = propertiesValue["containerType"];
                                    if (containerTypeValue5 != null && containerTypeValue5.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance5 = ((string)containerTypeValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.ContainerType = containerTypeInstance5;
                                    }
                                    
                                    JToken backupManagementTypeValue5 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue5 != null && backupManagementTypeValue5.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance5 = ((string)backupManagementTypeValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance5;
                                    }
                                    
                                    JToken protectableObjectTypeValue5 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue5 != null && protectableObjectTypeValue5.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance5 = ((string)protectableObjectTypeValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance5;
                                    }
                                    itemInstance.Properties = azureIaaSComputeVMProtectionContainerInstance;
                                }
                                if (typeName == "MABWindowsContainer")
                                {
                                    MabProtectionContainer mabProtectionContainerInstance = new MabProtectionContainer();
                                    
                                    JToken canReRegisterValue = propertiesValue["canReRegister"];
                                    if (canReRegisterValue != null && canReRegisterValue.Type != JTokenType.Null)
                                    {
                                        bool canReRegisterInstance = ((bool)canReRegisterValue);
                                        mabProtectionContainerInstance.CanReRegister = canReRegisterInstance;
                                    }
                                    
                                    JToken containerIdValue = propertiesValue["containerId"];
                                    if (containerIdValue != null && containerIdValue.Type != JTokenType.Null)
                                    {
                                        long containerIdInstance = ((long)containerIdValue);
                                        mabProtectionContainerInstance.ContainerId = containerIdInstance;
                                    }
                                    
                                    JToken friendlyNameValue6 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                        mabProtectionContainerInstance.FriendlyName = friendlyNameInstance6;
                                    }
                                    
                                    JToken registrationStatusValue6 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue6 != null && registrationStatusValue6.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance6 = ((string)registrationStatusValue6);
                                        mabProtectionContainerInstance.RegistrationStatus = registrationStatusInstance6;
                                    }
                                    
                                    JToken healthStatusValue6 = propertiesValue["healthStatus"];
                                    if (healthStatusValue6 != null && healthStatusValue6.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance6 = ((string)healthStatusValue6);
                                        mabProtectionContainerInstance.HealthStatus = healthStatusInstance6;
                                    }
                                    
                                    JToken containerTypeValue6 = propertiesValue["containerType"];
                                    if (containerTypeValue6 != null && containerTypeValue6.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance6 = ((string)containerTypeValue6);
                                        mabProtectionContainerInstance.ContainerType = containerTypeInstance6;
                                    }
                                    
                                    JToken backupManagementTypeValue6 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue6 != null && backupManagementTypeValue6.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance6 = ((string)backupManagementTypeValue6);
                                        mabProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance6;
                                    }
                                    
                                    JToken protectableObjectTypeValue6 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue6 != null && protectableObjectTypeValue6.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance6 = ((string)protectableObjectTypeValue6);
                                        mabProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance6;
                                    }
                                    itemInstance.Properties = mabProtectionContainerInstance;
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                itemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                itemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                itemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                itemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property.Name);
                                    string tagsValue = ((string)property.Value);
                                    itemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken eTagValue = responseDoc["eTag"];
                            if (eTagValue != null && eTagValue.Type != JTokenType.Null)
                            {
                                string eTagInstance = ((string)eTagValue);
                                itemInstance.ETag = eTagInstance;
                            }
                            
                            JToken locationValue2 = responseDoc["location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken azureAsyncOperationValue = responseDoc["azureAsyncOperation"];
                            if (azureAsyncOperationValue != null && azureAsyncOperationValue.Type != JTokenType.Null)
                            {
                                string azureAsyncOperationInstance = ((string)azureAsyncOperationValue);
                                result.AzureAsyncOperation = azureAsyncOperationInstance;
                            }
                            
                            JToken retryAfterValue = responseDoc["retryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                string retryAfterInstance = ((string)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AzureAsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = httpResponse.Headers.GetValues("Retry-After").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Fetches the result of any operation on the container given the URL
        /// for tracking the operation as returned by APIs such as Unregister
        /// etc.
        /// </summary>
        /// <param name='operationResultLink'>
        /// Required. Location value returned by operation.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Protection container response.
        /// </returns>
        public async Task<ProtectionContainerResponse> GetContainerOperationResultByURLAsync(string operationResultLink, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (operationResultLink == null)
            {
                throw new ArgumentNullException("operationResultLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationResultLink", operationResultLink);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "GetContainerOperationResultByURLAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationResultLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ProtectionContainerResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted || statusCode == HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ProtectionContainerResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ProtectionContainerResource itemInstance = new ProtectionContainerResource();
                            result.Item = itemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                string typeName = ((string)propertiesValue["protectableObjectType"]);
                                if (typeName == "ProtectionContainer")
                                {
                                    ProtectionContainer protectionContainerInstance = new ProtectionContainer();
                                    
                                    JToken friendlyNameValue = propertiesValue["friendlyName"];
                                    if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance = ((string)friendlyNameValue);
                                        protectionContainerInstance.FriendlyName = friendlyNameInstance;
                                    }
                                    
                                    JToken registrationStatusValue = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue != null && registrationStatusValue.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance = ((string)registrationStatusValue);
                                        protectionContainerInstance.RegistrationStatus = registrationStatusInstance;
                                    }
                                    
                                    JToken healthStatusValue = propertiesValue["healthStatus"];
                                    if (healthStatusValue != null && healthStatusValue.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance = ((string)healthStatusValue);
                                        protectionContainerInstance.HealthStatus = healthStatusInstance;
                                    }
                                    
                                    JToken containerTypeValue = propertiesValue["containerType"];
                                    if (containerTypeValue != null && containerTypeValue.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance = ((string)containerTypeValue);
                                        protectionContainerInstance.ContainerType = containerTypeInstance;
                                    }
                                    
                                    JToken backupManagementTypeValue = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue != null && backupManagementTypeValue.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance = ((string)backupManagementTypeValue);
                                        protectionContainerInstance.BackupManagementType = backupManagementTypeInstance;
                                    }
                                    
                                    JToken protectableObjectTypeValue = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue != null && protectableObjectTypeValue.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance = ((string)protectableObjectTypeValue);
                                        protectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance;
                                    }
                                    itemInstance.Properties = protectionContainerInstance;
                                }
                                if (typeName == "AzureSqlContainer")
                                {
                                    AzureSqlProtectionContainer azureSqlProtectionContainerInstance = new AzureSqlProtectionContainer();
                                    
                                    JToken friendlyNameValue2 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                        azureSqlProtectionContainerInstance.FriendlyName = friendlyNameInstance2;
                                    }
                                    
                                    JToken registrationStatusValue2 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue2 != null && registrationStatusValue2.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance2 = ((string)registrationStatusValue2);
                                        azureSqlProtectionContainerInstance.RegistrationStatus = registrationStatusInstance2;
                                    }
                                    
                                    JToken healthStatusValue2 = propertiesValue["healthStatus"];
                                    if (healthStatusValue2 != null && healthStatusValue2.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance2 = ((string)healthStatusValue2);
                                        azureSqlProtectionContainerInstance.HealthStatus = healthStatusInstance2;
                                    }
                                    
                                    JToken containerTypeValue2 = propertiesValue["containerType"];
                                    if (containerTypeValue2 != null && containerTypeValue2.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance2 = ((string)containerTypeValue2);
                                        azureSqlProtectionContainerInstance.ContainerType = containerTypeInstance2;
                                    }
                                    
                                    JToken backupManagementTypeValue2 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue2 != null && backupManagementTypeValue2.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance2 = ((string)backupManagementTypeValue2);
                                        azureSqlProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance2;
                                    }
                                    
                                    JToken protectableObjectTypeValue2 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue2 != null && protectableObjectTypeValue2.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance2 = ((string)protectableObjectTypeValue2);
                                        azureSqlProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance2;
                                    }
                                    itemInstance.Properties = azureSqlProtectionContainerInstance;
                                }
                                if (typeName == "IaaSVMContainer")
                                {
                                    AzureIaaSVMProtectionContainer azureIaaSVMProtectionContainerInstance = new AzureIaaSVMProtectionContainer();
                                    
                                    JToken resourceGroupValue = propertiesValue["resourceGroup"];
                                    if (resourceGroupValue != null && resourceGroupValue.Type != JTokenType.Null)
                                    {
                                        string resourceGroupInstance = ((string)resourceGroupValue);
                                        azureIaaSVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance;
                                    }
                                    
                                    JToken virtualMachineIdValue = propertiesValue["virtualMachineId"];
                                    if (virtualMachineIdValue != null && virtualMachineIdValue.Type != JTokenType.Null)
                                    {
                                        string virtualMachineIdInstance = ((string)virtualMachineIdValue);
                                        azureIaaSVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance;
                                    }
                                    
                                    JToken friendlyNameValue3 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                        azureIaaSVMProtectionContainerInstance.FriendlyName = friendlyNameInstance3;
                                    }
                                    
                                    JToken registrationStatusValue3 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue3 != null && registrationStatusValue3.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance3 = ((string)registrationStatusValue3);
                                        azureIaaSVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance3;
                                    }
                                    
                                    JToken healthStatusValue3 = propertiesValue["healthStatus"];
                                    if (healthStatusValue3 != null && healthStatusValue3.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance3 = ((string)healthStatusValue3);
                                        azureIaaSVMProtectionContainerInstance.HealthStatus = healthStatusInstance3;
                                    }
                                    
                                    JToken containerTypeValue3 = propertiesValue["containerType"];
                                    if (containerTypeValue3 != null && containerTypeValue3.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance3 = ((string)containerTypeValue3);
                                        azureIaaSVMProtectionContainerInstance.ContainerType = containerTypeInstance3;
                                    }
                                    
                                    JToken backupManagementTypeValue3 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue3 != null && backupManagementTypeValue3.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance3 = ((string)backupManagementTypeValue3);
                                        azureIaaSVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance3;
                                    }
                                    
                                    JToken protectableObjectTypeValue3 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue3 != null && protectableObjectTypeValue3.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance3 = ((string)protectableObjectTypeValue3);
                                        azureIaaSVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance3;
                                    }
                                    itemInstance.Properties = azureIaaSVMProtectionContainerInstance;
                                }
                                if (typeName == "Microsoft.ClassicCompute/virtualMachines")
                                {
                                    AzureIaaSClassicComputeVMProtectionContainer azureIaaSClassicComputeVMProtectionContainerInstance = new AzureIaaSClassicComputeVMProtectionContainer();
                                    
                                    JToken resourceGroupValue2 = propertiesValue["resourceGroup"];
                                    if (resourceGroupValue2 != null && resourceGroupValue2.Type != JTokenType.Null)
                                    {
                                        string resourceGroupInstance2 = ((string)resourceGroupValue2);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance2;
                                    }
                                    
                                    JToken virtualMachineIdValue2 = propertiesValue["virtualMachineId"];
                                    if (virtualMachineIdValue2 != null && virtualMachineIdValue2.Type != JTokenType.Null)
                                    {
                                        string virtualMachineIdInstance2 = ((string)virtualMachineIdValue2);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance2;
                                    }
                                    
                                    JToken friendlyNameValue4 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.FriendlyName = friendlyNameInstance4;
                                    }
                                    
                                    JToken registrationStatusValue4 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue4 != null && registrationStatusValue4.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance4 = ((string)registrationStatusValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance4;
                                    }
                                    
                                    JToken healthStatusValue4 = propertiesValue["healthStatus"];
                                    if (healthStatusValue4 != null && healthStatusValue4.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance4 = ((string)healthStatusValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.HealthStatus = healthStatusInstance4;
                                    }
                                    
                                    JToken containerTypeValue4 = propertiesValue["containerType"];
                                    if (containerTypeValue4 != null && containerTypeValue4.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance4 = ((string)containerTypeValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.ContainerType = containerTypeInstance4;
                                    }
                                    
                                    JToken backupManagementTypeValue4 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue4 != null && backupManagementTypeValue4.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance4 = ((string)backupManagementTypeValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance4;
                                    }
                                    
                                    JToken protectableObjectTypeValue4 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue4 != null && protectableObjectTypeValue4.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance4 = ((string)protectableObjectTypeValue4);
                                        azureIaaSClassicComputeVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance4;
                                    }
                                    itemInstance.Properties = azureIaaSClassicComputeVMProtectionContainerInstance;
                                }
                                if (typeName == "Microsoft.Compute/virtualMachines")
                                {
                                    AzureIaaSComputeVMProtectionContainer azureIaaSComputeVMProtectionContainerInstance = new AzureIaaSComputeVMProtectionContainer();
                                    
                                    JToken resourceGroupValue3 = propertiesValue["resourceGroup"];
                                    if (resourceGroupValue3 != null && resourceGroupValue3.Type != JTokenType.Null)
                                    {
                                        string resourceGroupInstance3 = ((string)resourceGroupValue3);
                                        azureIaaSComputeVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance3;
                                    }
                                    
                                    JToken virtualMachineIdValue3 = propertiesValue["virtualMachineId"];
                                    if (virtualMachineIdValue3 != null && virtualMachineIdValue3.Type != JTokenType.Null)
                                    {
                                        string virtualMachineIdInstance3 = ((string)virtualMachineIdValue3);
                                        azureIaaSComputeVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance3;
                                    }
                                    
                                    JToken friendlyNameValue5 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.FriendlyName = friendlyNameInstance5;
                                    }
                                    
                                    JToken registrationStatusValue5 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue5 != null && registrationStatusValue5.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance5 = ((string)registrationStatusValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance5;
                                    }
                                    
                                    JToken healthStatusValue5 = propertiesValue["healthStatus"];
                                    if (healthStatusValue5 != null && healthStatusValue5.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance5 = ((string)healthStatusValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.HealthStatus = healthStatusInstance5;
                                    }
                                    
                                    JToken containerTypeValue5 = propertiesValue["containerType"];
                                    if (containerTypeValue5 != null && containerTypeValue5.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance5 = ((string)containerTypeValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.ContainerType = containerTypeInstance5;
                                    }
                                    
                                    JToken backupManagementTypeValue5 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue5 != null && backupManagementTypeValue5.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance5 = ((string)backupManagementTypeValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance5;
                                    }
                                    
                                    JToken protectableObjectTypeValue5 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue5 != null && protectableObjectTypeValue5.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance5 = ((string)protectableObjectTypeValue5);
                                        azureIaaSComputeVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance5;
                                    }
                                    itemInstance.Properties = azureIaaSComputeVMProtectionContainerInstance;
                                }
                                if (typeName == "MABWindowsContainer")
                                {
                                    MabProtectionContainer mabProtectionContainerInstance = new MabProtectionContainer();
                                    
                                    JToken canReRegisterValue = propertiesValue["canReRegister"];
                                    if (canReRegisterValue != null && canReRegisterValue.Type != JTokenType.Null)
                                    {
                                        bool canReRegisterInstance = ((bool)canReRegisterValue);
                                        mabProtectionContainerInstance.CanReRegister = canReRegisterInstance;
                                    }
                                    
                                    JToken containerIdValue = propertiesValue["containerId"];
                                    if (containerIdValue != null && containerIdValue.Type != JTokenType.Null)
                                    {
                                        long containerIdInstance = ((long)containerIdValue);
                                        mabProtectionContainerInstance.ContainerId = containerIdInstance;
                                    }
                                    
                                    JToken friendlyNameValue6 = propertiesValue["friendlyName"];
                                    if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                        mabProtectionContainerInstance.FriendlyName = friendlyNameInstance6;
                                    }
                                    
                                    JToken registrationStatusValue6 = propertiesValue["registrationStatus"];
                                    if (registrationStatusValue6 != null && registrationStatusValue6.Type != JTokenType.Null)
                                    {
                                        string registrationStatusInstance6 = ((string)registrationStatusValue6);
                                        mabProtectionContainerInstance.RegistrationStatus = registrationStatusInstance6;
                                    }
                                    
                                    JToken healthStatusValue6 = propertiesValue["healthStatus"];
                                    if (healthStatusValue6 != null && healthStatusValue6.Type != JTokenType.Null)
                                    {
                                        string healthStatusInstance6 = ((string)healthStatusValue6);
                                        mabProtectionContainerInstance.HealthStatus = healthStatusInstance6;
                                    }
                                    
                                    JToken containerTypeValue6 = propertiesValue["containerType"];
                                    if (containerTypeValue6 != null && containerTypeValue6.Type != JTokenType.Null)
                                    {
                                        string containerTypeInstance6 = ((string)containerTypeValue6);
                                        mabProtectionContainerInstance.ContainerType = containerTypeInstance6;
                                    }
                                    
                                    JToken backupManagementTypeValue6 = propertiesValue["backupManagementType"];
                                    if (backupManagementTypeValue6 != null && backupManagementTypeValue6.Type != JTokenType.Null)
                                    {
                                        string backupManagementTypeInstance6 = ((string)backupManagementTypeValue6);
                                        mabProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance6;
                                    }
                                    
                                    JToken protectableObjectTypeValue6 = propertiesValue["protectableObjectType"];
                                    if (protectableObjectTypeValue6 != null && protectableObjectTypeValue6.Type != JTokenType.Null)
                                    {
                                        string protectableObjectTypeInstance6 = ((string)protectableObjectTypeValue6);
                                        mabProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance6;
                                    }
                                    itemInstance.Properties = mabProtectionContainerInstance;
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                itemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                itemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                itemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                itemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property.Name);
                                    string tagsValue = ((string)property.Value);
                                    itemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken eTagValue = responseDoc["eTag"];
                            if (eTagValue != null && eTagValue.Type != JTokenType.Null)
                            {
                                string eTagInstance = ((string)eTagValue);
                                itemInstance.ETag = eTagInstance;
                            }
                            
                            JToken locationValue2 = responseDoc["location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken azureAsyncOperationValue = responseDoc["azureAsyncOperation"];
                            if (azureAsyncOperationValue != null && azureAsyncOperationValue.Type != JTokenType.Null)
                            {
                                string azureAsyncOperationInstance = ((string)azureAsyncOperationValue);
                                result.AzureAsyncOperation = azureAsyncOperationInstance;
                            }
                            
                            JToken retryAfterValue = responseDoc["retryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                string retryAfterInstance = ((string)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AzureAsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = httpResponse.Headers.GetValues("Retry-After").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Fetches the result of the refresh operation triggered by the Begin
        /// Refresh API given the ID of the operation.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. Resource group name of your recovery services vault.
        /// </param>
        /// <param name='resourceName'>
        /// Required. Name of your recovery services vault.
        /// </param>
        /// <param name='fabricName'>
        /// Optional. Fabric name of the protected item.
        /// </param>
        /// <param name='operationId'>
        /// Required. ID of the container operation whose result has to be
        /// fetched.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Base recovery job response for all the asynchronous operations.
        /// </returns>
        public async Task<BaseRecoveryServicesJobResponse> GetRefreshOperationResultAsync(string resourceGroupName, string resourceName, string fabricName, string operationId, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (resourceName == null)
            {
                throw new ArgumentNullException("resourceName");
            }
            if (operationId == null)
            {
                throw new ArgumentNullException("operationId");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("resourceName", resourceName);
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("operationId", operationId);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "GetRefreshOperationResultAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            if (this.Client.ResourceNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            }
            url = url + "/";
            url = url + "vaults";
            url = url + "/";
            url = url + Uri.EscapeDataString(resourceName);
            url = url + "/backupFabrics/";
            if (fabricName != null)
            {
                url = url + Uri.EscapeDataString(fabricName);
            }
            url = url + "/operationResults/";
            url = url + Uri.EscapeDataString(operationId);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2016-05-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    BaseRecoveryServicesJobResponse result = null;
                    // Deserialize Response
                    result = new BaseRecoveryServicesJobResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AzureAsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = httpResponse.Headers.GetValues("Retry-After").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Fetches the result of the refresh operation triggered by the Begin
        /// Refresh operation given the URL for tracking the operation as
        /// returned by the Begin Refresh operation.
        /// </summary>
        /// <param name='operationResultLink'>
        /// Required. Location value returned by operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Base recovery job response for all the asynchronous operations.
        /// </returns>
        public async Task<BaseRecoveryServicesJobResponse> GetRefreshOperationResultByURLAsync(string operationResultLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationResultLink == null)
            {
                throw new ArgumentNullException("operationResultLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationResultLink", operationResultLink);
                TracingAdapter.Enter(invocationId, this, "GetRefreshOperationResultByURLAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationResultLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    BaseRecoveryServicesJobResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.Accepted || statusCode == HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new BaseRecoveryServicesJobResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                result.Location = locationInstance;
                            }
                            
                            JToken azureAsyncOperationValue = responseDoc["azureAsyncOperation"];
                            if (azureAsyncOperationValue != null && azureAsyncOperationValue.Type != JTokenType.Null)
                            {
                                string azureAsyncOperationInstance = ((string)azureAsyncOperationValue);
                                result.AzureAsyncOperation = azureAsyncOperationInstance;
                            }
                            
                            JToken retryAfterValue = responseDoc["retryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                string retryAfterInstance = ((string)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AzureAsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = httpResponse.Headers.GetValues("Retry-After").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.InternalServerError)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Lists all the containers registered to your Recovery Services Vault
        /// according to the query parameters supplied in the arguments.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. Resource group name of your recovery services vault.
        /// </param>
        /// <param name='resourceName'>
        /// Required. Name of your recovery services vault.
        /// </param>
        /// <param name='queryParams'>
        /// Required. Query parameters for listing protection containers.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Required. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// List of protection containers returned as a response by the list
        /// protection containers API.
        /// </returns>
        public async Task<ProtectionContainerListResponse> ListAsync(string resourceGroupName, string resourceName, ProtectionContainerListQueryParams queryParams, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (resourceName == null)
            {
                throw new ArgumentNullException("resourceName");
            }
            if (queryParams == null)
            {
                throw new ArgumentNullException("queryParams");
            }
            if (customRequestHeaders == null)
            {
                throw new ArgumentNullException("customRequestHeaders");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("resourceName", resourceName);
                tracingParameters.Add("queryParams", queryParams);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            if (this.Client.ResourceNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            }
            url = url + "/";
            url = url + "vaults";
            url = url + "/";
            url = url + Uri.EscapeDataString(resourceName);
            url = url + "/backupProtectionContainers";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2016-05-01");
            List<string> odataFilter = new List<string>();
            if (queryParams.BackupManagementType != null)
            {
                odataFilter.Add("backupManagementType eq '" + Uri.EscapeDataString(queryParams.BackupManagementType) + "'");
            }
            if (queryParams.RegistrationStatus != null)
            {
                odataFilter.Add("status eq '" + Uri.EscapeDataString(queryParams.RegistrationStatus) + "'");
            }
            if (queryParams.FriendlyName != null)
            {
                odataFilter.Add("friendlyName eq '" + Uri.EscapeDataString(queryParams.FriendlyName) + "'");
            }
            if (odataFilter.Count > 0)
            {
                queryParameters.Add("$filter=" + string.Join(" and ", odataFilter));
            }
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ProtectionContainerListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ProtectionContainerListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ProtectionContainerResourceList itemListInstance = new ProtectionContainerResourceList();
                            result.ItemList = itemListInstance;
                            
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    ProtectionContainerResource protectionContainerResourceInstance = new ProtectionContainerResource();
                                    itemListInstance.ProtectionContainers.Add(protectionContainerResourceInstance);
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        string typeName = ((string)propertiesValue["protectableObjectType"]);
                                        if (typeName == "ProtectionContainer")
                                        {
                                            ProtectionContainer protectionContainerInstance = new ProtectionContainer();
                                            
                                            JToken friendlyNameValue = propertiesValue["friendlyName"];
                                            if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                            {
                                                string friendlyNameInstance = ((string)friendlyNameValue);
                                                protectionContainerInstance.FriendlyName = friendlyNameInstance;
                                            }
                                            
                                            JToken registrationStatusValue = propertiesValue["registrationStatus"];
                                            if (registrationStatusValue != null && registrationStatusValue.Type != JTokenType.Null)
                                            {
                                                string registrationStatusInstance = ((string)registrationStatusValue);
                                                protectionContainerInstance.RegistrationStatus = registrationStatusInstance;
                                            }
                                            
                                            JToken healthStatusValue = propertiesValue["healthStatus"];
                                            if (healthStatusValue != null && healthStatusValue.Type != JTokenType.Null)
                                            {
                                                string healthStatusInstance = ((string)healthStatusValue);
                                                protectionContainerInstance.HealthStatus = healthStatusInstance;
                                            }
                                            
                                            JToken containerTypeValue = propertiesValue["containerType"];
                                            if (containerTypeValue != null && containerTypeValue.Type != JTokenType.Null)
                                            {
                                                string containerTypeInstance = ((string)containerTypeValue);
                                                protectionContainerInstance.ContainerType = containerTypeInstance;
                                            }
                                            
                                            JToken backupManagementTypeValue = propertiesValue["backupManagementType"];
                                            if (backupManagementTypeValue != null && backupManagementTypeValue.Type != JTokenType.Null)
                                            {
                                                string backupManagementTypeInstance = ((string)backupManagementTypeValue);
                                                protectionContainerInstance.BackupManagementType = backupManagementTypeInstance;
                                            }
                                            
                                            JToken protectableObjectTypeValue = propertiesValue["protectableObjectType"];
                                            if (protectableObjectTypeValue != null && protectableObjectTypeValue.Type != JTokenType.Null)
                                            {
                                                string protectableObjectTypeInstance = ((string)protectableObjectTypeValue);
                                                protectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance;
                                            }
                                            protectionContainerResourceInstance.Properties = protectionContainerInstance;
                                        }
                                        if (typeName == "AzureSqlContainer")
                                        {
                                            AzureSqlProtectionContainer azureSqlProtectionContainerInstance = new AzureSqlProtectionContainer();
                                            
                                            JToken friendlyNameValue2 = propertiesValue["friendlyName"];
                                            if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                            {
                                                string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                                azureSqlProtectionContainerInstance.FriendlyName = friendlyNameInstance2;
                                            }
                                            
                                            JToken registrationStatusValue2 = propertiesValue["registrationStatus"];
                                            if (registrationStatusValue2 != null && registrationStatusValue2.Type != JTokenType.Null)
                                            {
                                                string registrationStatusInstance2 = ((string)registrationStatusValue2);
                                                azureSqlProtectionContainerInstance.RegistrationStatus = registrationStatusInstance2;
                                            }
                                            
                                            JToken healthStatusValue2 = propertiesValue["healthStatus"];
                                            if (healthStatusValue2 != null && healthStatusValue2.Type != JTokenType.Null)
                                            {
                                                string healthStatusInstance2 = ((string)healthStatusValue2);
                                                azureSqlProtectionContainerInstance.HealthStatus = healthStatusInstance2;
                                            }
                                            
                                            JToken containerTypeValue2 = propertiesValue["containerType"];
                                            if (containerTypeValue2 != null && containerTypeValue2.Type != JTokenType.Null)
                                            {
                                                string containerTypeInstance2 = ((string)containerTypeValue2);
                                                azureSqlProtectionContainerInstance.ContainerType = containerTypeInstance2;
                                            }
                                            
                                            JToken backupManagementTypeValue2 = propertiesValue["backupManagementType"];
                                            if (backupManagementTypeValue2 != null && backupManagementTypeValue2.Type != JTokenType.Null)
                                            {
                                                string backupManagementTypeInstance2 = ((string)backupManagementTypeValue2);
                                                azureSqlProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance2;
                                            }
                                            
                                            JToken protectableObjectTypeValue2 = propertiesValue["protectableObjectType"];
                                            if (protectableObjectTypeValue2 != null && protectableObjectTypeValue2.Type != JTokenType.Null)
                                            {
                                                string protectableObjectTypeInstance2 = ((string)protectableObjectTypeValue2);
                                                azureSqlProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance2;
                                            }
                                            protectionContainerResourceInstance.Properties = azureSqlProtectionContainerInstance;
                                        }
                                        if (typeName == "IaaSVMContainer")
                                        {
                                            AzureIaaSVMProtectionContainer azureIaaSVMProtectionContainerInstance = new AzureIaaSVMProtectionContainer();
                                            
                                            JToken resourceGroupValue = propertiesValue["resourceGroup"];
                                            if (resourceGroupValue != null && resourceGroupValue.Type != JTokenType.Null)
                                            {
                                                string resourceGroupInstance = ((string)resourceGroupValue);
                                                azureIaaSVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance;
                                            }
                                            
                                            JToken virtualMachineIdValue = propertiesValue["virtualMachineId"];
                                            if (virtualMachineIdValue != null && virtualMachineIdValue.Type != JTokenType.Null)
                                            {
                                                string virtualMachineIdInstance = ((string)virtualMachineIdValue);
                                                azureIaaSVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance;
                                            }
                                            
                                            JToken friendlyNameValue3 = propertiesValue["friendlyName"];
                                            if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                            {
                                                string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                                azureIaaSVMProtectionContainerInstance.FriendlyName = friendlyNameInstance3;
                                            }
                                            
                                            JToken registrationStatusValue3 = propertiesValue["registrationStatus"];
                                            if (registrationStatusValue3 != null && registrationStatusValue3.Type != JTokenType.Null)
                                            {
                                                string registrationStatusInstance3 = ((string)registrationStatusValue3);
                                                azureIaaSVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance3;
                                            }
                                            
                                            JToken healthStatusValue3 = propertiesValue["healthStatus"];
                                            if (healthStatusValue3 != null && healthStatusValue3.Type != JTokenType.Null)
                                            {
                                                string healthStatusInstance3 = ((string)healthStatusValue3);
                                                azureIaaSVMProtectionContainerInstance.HealthStatus = healthStatusInstance3;
                                            }
                                            
                                            JToken containerTypeValue3 = propertiesValue["containerType"];
                                            if (containerTypeValue3 != null && containerTypeValue3.Type != JTokenType.Null)
                                            {
                                                string containerTypeInstance3 = ((string)containerTypeValue3);
                                                azureIaaSVMProtectionContainerInstance.ContainerType = containerTypeInstance3;
                                            }
                                            
                                            JToken backupManagementTypeValue3 = propertiesValue["backupManagementType"];
                                            if (backupManagementTypeValue3 != null && backupManagementTypeValue3.Type != JTokenType.Null)
                                            {
                                                string backupManagementTypeInstance3 = ((string)backupManagementTypeValue3);
                                                azureIaaSVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance3;
                                            }
                                            
                                            JToken protectableObjectTypeValue3 = propertiesValue["protectableObjectType"];
                                            if (protectableObjectTypeValue3 != null && protectableObjectTypeValue3.Type != JTokenType.Null)
                                            {
                                                string protectableObjectTypeInstance3 = ((string)protectableObjectTypeValue3);
                                                azureIaaSVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance3;
                                            }
                                            protectionContainerResourceInstance.Properties = azureIaaSVMProtectionContainerInstance;
                                        }
                                        if (typeName == "Microsoft.ClassicCompute/virtualMachines")
                                        {
                                            AzureIaaSClassicComputeVMProtectionContainer azureIaaSClassicComputeVMProtectionContainerInstance = new AzureIaaSClassicComputeVMProtectionContainer();
                                            
                                            JToken resourceGroupValue2 = propertiesValue["resourceGroup"];
                                            if (resourceGroupValue2 != null && resourceGroupValue2.Type != JTokenType.Null)
                                            {
                                                string resourceGroupInstance2 = ((string)resourceGroupValue2);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance2;
                                            }
                                            
                                            JToken virtualMachineIdValue2 = propertiesValue["virtualMachineId"];
                                            if (virtualMachineIdValue2 != null && virtualMachineIdValue2.Type != JTokenType.Null)
                                            {
                                                string virtualMachineIdInstance2 = ((string)virtualMachineIdValue2);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance2;
                                            }
                                            
                                            JToken friendlyNameValue4 = propertiesValue["friendlyName"];
                                            if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                            {
                                                string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.FriendlyName = friendlyNameInstance4;
                                            }
                                            
                                            JToken registrationStatusValue4 = propertiesValue["registrationStatus"];
                                            if (registrationStatusValue4 != null && registrationStatusValue4.Type != JTokenType.Null)
                                            {
                                                string registrationStatusInstance4 = ((string)registrationStatusValue4);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance4;
                                            }
                                            
                                            JToken healthStatusValue4 = propertiesValue["healthStatus"];
                                            if (healthStatusValue4 != null && healthStatusValue4.Type != JTokenType.Null)
                                            {
                                                string healthStatusInstance4 = ((string)healthStatusValue4);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.HealthStatus = healthStatusInstance4;
                                            }
                                            
                                            JToken containerTypeValue4 = propertiesValue["containerType"];
                                            if (containerTypeValue4 != null && containerTypeValue4.Type != JTokenType.Null)
                                            {
                                                string containerTypeInstance4 = ((string)containerTypeValue4);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.ContainerType = containerTypeInstance4;
                                            }
                                            
                                            JToken backupManagementTypeValue4 = propertiesValue["backupManagementType"];
                                            if (backupManagementTypeValue4 != null && backupManagementTypeValue4.Type != JTokenType.Null)
                                            {
                                                string backupManagementTypeInstance4 = ((string)backupManagementTypeValue4);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance4;
                                            }
                                            
                                            JToken protectableObjectTypeValue4 = propertiesValue["protectableObjectType"];
                                            if (protectableObjectTypeValue4 != null && protectableObjectTypeValue4.Type != JTokenType.Null)
                                            {
                                                string protectableObjectTypeInstance4 = ((string)protectableObjectTypeValue4);
                                                azureIaaSClassicComputeVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance4;
                                            }
                                            protectionContainerResourceInstance.Properties = azureIaaSClassicComputeVMProtectionContainerInstance;
                                        }
                                        if (typeName == "Microsoft.Compute/virtualMachines")
                                        {
                                            AzureIaaSComputeVMProtectionContainer azureIaaSComputeVMProtectionContainerInstance = new AzureIaaSComputeVMProtectionContainer();
                                            
                                            JToken resourceGroupValue3 = propertiesValue["resourceGroup"];
                                            if (resourceGroupValue3 != null && resourceGroupValue3.Type != JTokenType.Null)
                                            {
                                                string resourceGroupInstance3 = ((string)resourceGroupValue3);
                                                azureIaaSComputeVMProtectionContainerInstance.ResourceGroup = resourceGroupInstance3;
                                            }
                                            
                                            JToken virtualMachineIdValue3 = propertiesValue["virtualMachineId"];
                                            if (virtualMachineIdValue3 != null && virtualMachineIdValue3.Type != JTokenType.Null)
                                            {
                                                string virtualMachineIdInstance3 = ((string)virtualMachineIdValue3);
                                                azureIaaSComputeVMProtectionContainerInstance.VirtualMachineId = virtualMachineIdInstance3;
                                            }
                                            
                                            JToken friendlyNameValue5 = propertiesValue["friendlyName"];
                                            if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                            {
                                                string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                                azureIaaSComputeVMProtectionContainerInstance.FriendlyName = friendlyNameInstance5;
                                            }
                                            
                                            JToken registrationStatusValue5 = propertiesValue["registrationStatus"];
                                            if (registrationStatusValue5 != null && registrationStatusValue5.Type != JTokenType.Null)
                                            {
                                                string registrationStatusInstance5 = ((string)registrationStatusValue5);
                                                azureIaaSComputeVMProtectionContainerInstance.RegistrationStatus = registrationStatusInstance5;
                                            }
                                            
                                            JToken healthStatusValue5 = propertiesValue["healthStatus"];
                                            if (healthStatusValue5 != null && healthStatusValue5.Type != JTokenType.Null)
                                            {
                                                string healthStatusInstance5 = ((string)healthStatusValue5);
                                                azureIaaSComputeVMProtectionContainerInstance.HealthStatus = healthStatusInstance5;
                                            }
                                            
                                            JToken containerTypeValue5 = propertiesValue["containerType"];
                                            if (containerTypeValue5 != null && containerTypeValue5.Type != JTokenType.Null)
                                            {
                                                string containerTypeInstance5 = ((string)containerTypeValue5);
                                                azureIaaSComputeVMProtectionContainerInstance.ContainerType = containerTypeInstance5;
                                            }
                                            
                                            JToken backupManagementTypeValue5 = propertiesValue["backupManagementType"];
                                            if (backupManagementTypeValue5 != null && backupManagementTypeValue5.Type != JTokenType.Null)
                                            {
                                                string backupManagementTypeInstance5 = ((string)backupManagementTypeValue5);
                                                azureIaaSComputeVMProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance5;
                                            }
                                            
                                            JToken protectableObjectTypeValue5 = propertiesValue["protectableObjectType"];
                                            if (protectableObjectTypeValue5 != null && protectableObjectTypeValue5.Type != JTokenType.Null)
                                            {
                                                string protectableObjectTypeInstance5 = ((string)protectableObjectTypeValue5);
                                                azureIaaSComputeVMProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance5;
                                            }
                                            protectionContainerResourceInstance.Properties = azureIaaSComputeVMProtectionContainerInstance;
                                        }
                                        if (typeName == "MABWindowsContainer")
                                        {
                                            MabProtectionContainer mabProtectionContainerInstance = new MabProtectionContainer();
                                            
                                            JToken canReRegisterValue = propertiesValue["canReRegister"];
                                            if (canReRegisterValue != null && canReRegisterValue.Type != JTokenType.Null)
                                            {
                                                bool canReRegisterInstance = ((bool)canReRegisterValue);
                                                mabProtectionContainerInstance.CanReRegister = canReRegisterInstance;
                                            }
                                            
                                            JToken containerIdValue = propertiesValue["containerId"];
                                            if (containerIdValue != null && containerIdValue.Type != JTokenType.Null)
                                            {
                                                long containerIdInstance = ((long)containerIdValue);
                                                mabProtectionContainerInstance.ContainerId = containerIdInstance;
                                            }
                                            
                                            JToken friendlyNameValue6 = propertiesValue["friendlyName"];
                                            if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                            {
                                                string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                                mabProtectionContainerInstance.FriendlyName = friendlyNameInstance6;
                                            }
                                            
                                            JToken registrationStatusValue6 = propertiesValue["registrationStatus"];
                                            if (registrationStatusValue6 != null && registrationStatusValue6.Type != JTokenType.Null)
                                            {
                                                string registrationStatusInstance6 = ((string)registrationStatusValue6);
                                                mabProtectionContainerInstance.RegistrationStatus = registrationStatusInstance6;
                                            }
                                            
                                            JToken healthStatusValue6 = propertiesValue["healthStatus"];
                                            if (healthStatusValue6 != null && healthStatusValue6.Type != JTokenType.Null)
                                            {
                                                string healthStatusInstance6 = ((string)healthStatusValue6);
                                                mabProtectionContainerInstance.HealthStatus = healthStatusInstance6;
                                            }
                                            
                                            JToken containerTypeValue6 = propertiesValue["containerType"];
                                            if (containerTypeValue6 != null && containerTypeValue6.Type != JTokenType.Null)
                                            {
                                                string containerTypeInstance6 = ((string)containerTypeValue6);
                                                mabProtectionContainerInstance.ContainerType = containerTypeInstance6;
                                            }
                                            
                                            JToken backupManagementTypeValue6 = propertiesValue["backupManagementType"];
                                            if (backupManagementTypeValue6 != null && backupManagementTypeValue6.Type != JTokenType.Null)
                                            {
                                                string backupManagementTypeInstance6 = ((string)backupManagementTypeValue6);
                                                mabProtectionContainerInstance.BackupManagementType = backupManagementTypeInstance6;
                                            }
                                            
                                            JToken protectableObjectTypeValue6 = propertiesValue["protectableObjectType"];
                                            if (protectableObjectTypeValue6 != null && protectableObjectTypeValue6.Type != JTokenType.Null)
                                            {
                                                string protectableObjectTypeInstance6 = ((string)protectableObjectTypeValue6);
                                                mabProtectionContainerInstance.ProtectableObjectType = protectableObjectTypeInstance6;
                                            }
                                            protectionContainerResourceInstance.Properties = mabProtectionContainerInstance;
                                        }
                                    }
                                    
                                    JToken idValue = valueValue["id"];
                                    if (idValue != null && idValue.Type != JTokenType.Null)
                                    {
                                        string idInstance = ((string)idValue);
                                        protectionContainerResourceInstance.Id = idInstance;
                                    }
                                    
                                    JToken nameValue = valueValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        protectionContainerResourceInstance.Name = nameInstance;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        protectionContainerResourceInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        protectionContainerResourceInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property.Name);
                                            string tagsValue = ((string)property.Value);
                                            protectionContainerResourceInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                    
                                    JToken eTagValue = valueValue["eTag"];
                                    if (eTagValue != null && eTagValue.Type != JTokenType.Null)
                                    {
                                        string eTagInstance = ((string)eTagValue);
                                        protectionContainerResourceInstance.ETag = eTagInstance;
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                itemListInstance.NextLink = nextLinkInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Refresh Operation triggers an operation in the service which
        /// would discover all the containers in the subscription that are
        /// ready to be protected by your Recovery Services Vault.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. Resource group name of your recovery services vault.
        /// </param>
        /// <param name='resourceName'>
        /// Required. Name of your recovery services vault.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Required. Request header parameters.
        /// </param>
        /// <param name='fabricName'>
        /// Optional. Fabric name for the protection containers.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Base recovery job response for all the asynchronous operations.
        /// </returns>
        public async Task<BaseRecoveryServicesJobResponse> RefreshAsync(string resourceGroupName, string resourceName, CustomRequestHeaders customRequestHeaders, string fabricName, CancellationToken cancellationToken)
        {
            RecoveryServicesBackupManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("resourceName", resourceName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                tracingParameters.Add("fabricName", fabricName);
                TracingAdapter.Enter(invocationId, this, "RefreshAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            BaseRecoveryServicesJobResponse response = await client.Containers.BeginRefreshAsync(resourceGroupName, resourceName, customRequestHeaders, fabricName, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            BaseRecoveryServicesJobResponse result = await client.Containers.GetRefreshOperationResultByURLAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.Containers.GetRefreshOperationResultByURLAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// The Begin Unregister Operation unregisters the given container from
        /// your Recovery Services Vault. This is an asynchronous operation.
        /// To determine whether the backend service has finished processing
        /// the request, call Get Container Operation Result API.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. Resource group name of your recovery services vault.
        /// </param>
        /// <param name='resourceName'>
        /// Required. Name of your recovery services vault.
        /// </param>
        /// <param name='identityName'>
        /// Required. Name of the protection container to unregister.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AzureOperationResponse> UnregisterAsync(string resourceGroupName, string resourceName, string identityName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (resourceName == null)
            {
                throw new ArgumentNullException("resourceName");
            }
            if (identityName == null)
            {
                throw new ArgumentNullException("identityName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("resourceName", resourceName);
                tracingParameters.Add("identityName", identityName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "UnregisterAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            if (this.Client.ResourceNamespace != null)
            {
                url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            }
            url = url + "/";
            url = url + "vaults";
            url = url + "/";
            url = url + Uri.EscapeDataString(resourceName);
            url = url + "/registeredIdentities/";
            url = url + Uri.EscapeDataString(identityName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2016-05-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AzureOperationResponse result = null;
                    // Deserialize Response
                    result = new AzureOperationResponse();
                    result.StatusCode = statusCode;
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
