// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.RecoveryServices.Backup;
using Microsoft.Azure.Management.RecoveryServices.Backup.Models;

namespace Microsoft.Azure.Management.RecoveryServices.Backup
{
    public partial interface IRecoveryServicesBackupManagementClient : IDisposable
    {
        /// <summary>
        /// Gets the API version.
        /// </summary>
        string ApiVersion
        {
            get; 
        }
        
        /// <summary>
        /// Gets the URI used as the base for all cloud service requests.
        /// </summary>
        Uri BaseUri
        {
            get; 
        }
        
        /// <summary>
        /// Gets subscription credentials which uniquely identify Microsoft
        /// Azure subscription. The subscription ID forms part of the URI for
        /// every service call.
        /// </summary>
        SubscriptionCloudCredentials Credentials
        {
            get; 
        }
        
        /// <summary>
        /// Gets or sets the initial timeout for Long Running Operations.
        /// </summary>
        int LongRunningOperationInitialTimeout
        {
            get; set; 
        }
        
        /// <summary>
        /// Gets or sets the retry timeout for Long Running Operations.
        /// </summary>
        int LongRunningOperationRetryTimeout
        {
            get; set; 
        }
        
        /// <summary>
        /// Get or sets the resource namespace. By default it points to Azure
        /// Production. Override this value only if needed
        /// </summary>
        string ResourceNamespace
        {
            get; set; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for managing the
        /// backup engines registered to your Recovery Services Vault.
        /// </summary>
        IBackupEngineOperations BackupEngines
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for triggering and
        /// managing the backups of items protected by your Recovery Services
        /// Vault.
        /// </summary>
        IBackupOperations Backups
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for managing the
        /// containers registered to your Recovery Services Vault.
        /// </summary>
        IContainerOperations Containers
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for managing the jobs
        /// created by various operations on a given item protected by your
        /// Recovery Services Vault.
        /// </summary>
        IJobOperations Jobs
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for managing the
        /// protectable objects registered to your Recovery Services Vault.
        /// </summary>
        IProtectableObjectOperations ProtectableObjects
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for managing the items
        /// protected by your Recovery Services Vault.
        /// </summary>
        IProtectedItemOperations ProtectedItems
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for managing
        /// protection policies defined for your Recovery Services Vault.
        /// </summary>
        IProtectionPolicyOperations ProtectionPolicies
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for managing recovery
        /// points created by backup operations on the items protected by your
        /// Recovery Services Vault.
        /// </summary>
        IRecoveryPointOperations RecoveryPoints
        {
            get; 
        }
        
        /// <summary>
        /// The Resource Manager API includes operations for triggering and
        /// managing restore actions of the items protected by your Recovery
        /// Services Vault.
        /// </summary>
        IRestoreOperations Restores
        {
            get; 
        }
        
        /// <summary>
        /// Once you trigger a delete operation such as deleting a protected
        /// item etc., you may use this method to get the operation's result.
        /// Once the operation has started, the status code in the response
        /// would be Accepted. It will continue to be in this state till it
        /// reaches completetion. On successful completion, the status code
        /// will be OK. This method expects the ID of the operation to be
        /// passed in the arguments. This usually is part of the Location
        /// header of the operation response.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Resource group name of your recovery services vault.
        /// </param>
        /// <param name='resourceName'>
        /// Name of your recovery services vault.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response object returned by the get operation result APIs.
        /// </returns>
        Task<GetOperationResultResponse> GetDeleteOperationResultAsync(string resourceGroupName, string resourceName, string operationId, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// Once you trigger a delete operation such as deleting a protected
        /// item etc., you may use this method to get the operation's result.
        /// Once the operation has started, the status code in the response
        /// would be Accepted. It will continue to be in this state till it
        /// reaches completetion. On successful completion, the status code
        /// will be OK. This method expects the tracking URL of the operation
        /// to be passed in the arguments. This usually is the Location header
        /// of the operation response.
        /// </summary>
        /// <param name='operationResultLink'>
        /// Location value returned by operation.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response object returned by the get operation result APIs.
        /// </returns>
        Task<GetOperationResultResponse> GetDeleteOperationResultByURLAsync(string operationResultLink, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
        
        /// <summary>
        /// The GetOperationStatusByURL method can be used to fetch the status
        /// of an operation such as triggering a backup, restore etc. The
        /// status can be in progress, completed or failed. You can refer to
        /// the Operation Status enum for all the possible states of the
        /// operation. Some operations may create jobs within the backend
        /// service. This method can return the list of jobs when the
        /// operation is complete.
        /// </summary>
        /// <param name='operationResultLink'>
        /// Location value returned by operation.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response of the get operation status API calls.
        /// </returns>
        Task<BackUpOperationStatusResponse> GetOperationStatusByURLAsync(string operationResultLink, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken);
    }
}
