// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using Microsoft.Azure.Management.RecoveryServices.Backup.Models;

namespace Microsoft.Azure.Management.RecoveryServices.Backup.Models
{
    /// <summary>
    /// IaaS VM workload-specific restore request.
    /// </summary>
    public partial class IaasVMRestoreRequest : RestoreRequest
    {
        private string _affinityGroup;
        
        /// <summary>
        /// Optional. Affinity group to which the VM should be restored to.
        /// Used only for Classic Compute Virtual Machines.
        /// </summary>
        public string AffinityGroup
        {
            get { return this._affinityGroup; }
            set { this._affinityGroup = value; }
        }
        
        private bool _createNewCloudService;
        
        /// <summary>
        /// Optional. Specifies whether a new cloud service has to be created
        /// while the VM is being restored. If this is false, VM will be
        /// restored to the same cloud service as that of the source VM.
        /// </summary>
        public bool CreateNewCloudService
        {
            get { return this._createNewCloudService; }
            set { this._createNewCloudService = value; }
        }
        
        private EncryptionDetails _encryptionDetails;
        
        /// <summary>
        /// Optional. Details needed if the source VM was encrypted at the time
        /// of backup.
        /// </summary>
        public EncryptionDetails EncryptionDetails
        {
            get { return this._encryptionDetails; }
            set { this._encryptionDetails = value; }
        }
        
        private string _recoveryPointId;
        
        /// <summary>
        /// Optional. ID of the recovery point to where the VM has to be
        /// restored back to.
        /// </summary>
        public string RecoveryPointId
        {
            get { return this._recoveryPointId; }
            set { this._recoveryPointId = value; }
        }
        
        private string _recoveryType;
        
        /// <summary>
        /// Optional. Type of this recovery. Possible values: OriginalLocation,
        /// AlternateLocation, RestoreDisks.
        /// </summary>
        public string RecoveryType
        {
            get { return this._recoveryType; }
            set { this._recoveryType = value; }
        }
        
        private string _region;
        
        /// <summary>
        /// Optional. Azure region where the VM should be restored to.
        /// </summary>
        public string Region
        {
            get { return this._region; }
            set { this._region = value; }
        }
        
        private string _sourceResourceId;
        
        /// <summary>
        /// Optional. Fully qualified ARM ID of the source VM which is being
        /// recovered.
        /// </summary>
        public string SourceResourceId
        {
            get { return this._sourceResourceId; }
            set { this._sourceResourceId = value; }
        }
        
        private string _storageAccountId;
        
        /// <summary>
        /// Optional. Fully qualified ARM ID of the storage account to which
        /// the VM has to be restored to.
        /// </summary>
        public string StorageAccountId
        {
            get { return this._storageAccountId; }
            set { this._storageAccountId = value; }
        }
        
        private string _subnetId;
        
        /// <summary>
        /// Optional. Fully qualified ARM ID of the subnet to which the
        /// restored VM should belong to.
        /// </summary>
        public string SubnetId
        {
            get { return this._subnetId; }
            set { this._subnetId = value; }
        }
        
        private string _targetDomainNameId;
        
        /// <summary>
        /// Optional. Fully qualified ARM ID of the domain name to be used with
        /// the VM being cretaed as part of the restore operation. This
        /// applies only to Classic Virtual Machines.
        /// </summary>
        public string TargetDomainNameId
        {
            get { return this._targetDomainNameId; }
            set { this._targetDomainNameId = value; }
        }
        
        private string _targetResourceGroupId;
        
        /// <summary>
        /// Optional. Fully qualified ARM ID of the resource group where the VM
        /// has to be cretaed as part of the restore operation.
        /// </summary>
        public string TargetResourceGroupId
        {
            get { return this._targetResourceGroupId; }
            set { this._targetResourceGroupId = value; }
        }
        
        private string _targetVirtualMachineId;
        
        /// <summary>
        /// Optional. Fully qualified ARM ID of the VM to be cretaed as part of
        /// the restore operation.
        /// </summary>
        public string TargetVirtualMachineId
        {
            get { return this._targetVirtualMachineId; }
            set { this._targetVirtualMachineId = value; }
        }
        
        private string _virtualNetworkId;
        
        /// <summary>
        /// Optional. Fully qualified ARM ID of the virtual network to which
        /// the restored VM should belong to.
        /// </summary>
        public string VirtualNetworkId
        {
            get { return this._virtualNetworkId; }
            set { this._virtualNetworkId = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the IaasVMRestoreRequest class.
        /// </summary>
        public IaasVMRestoreRequest()
        {
        }
    }
}
