// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Sql;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// The Windows Azure SQL Database management API provides a RESTful set of
    /// web services that interact with Windows Azure SQL Database services to
    /// manage your databases. The API enables users to create, retrieve,
    /// update, and delete databases and servers.
    /// </summary>
    public static partial class ElasticPoolOperationsExtensions
    {
        /// <summary>
        /// Begins creating a new Azure SQL Database Elastic Pool or updating
        /// an existing Azure SQL Database Elastic Pool. To determine the
        /// status of the operation call GetElasticPoolOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// operated on (Updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for createing or updating an
        /// Elastic Pool.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        public static ElasticPoolCreateOrUpdateResponse BeginCreateOrUpdate(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName, ElasticPoolCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).BeginCreateOrUpdateAsync(resourceGroupName, serverName, elasticPoolName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Begins creating a new Azure SQL Database Elastic Pool or updating
        /// an existing Azure SQL Database Elastic Pool. To determine the
        /// status of the operation call GetElasticPoolOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// operated on (Updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for createing or updating an
        /// Elastic Pool.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        public static Task<ElasticPoolCreateOrUpdateResponse> BeginCreateOrUpdateAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName, ElasticPoolCreateOrUpdateParameters parameters)
        {
            return operations.BeginCreateOrUpdateAsync(resourceGroupName, serverName, elasticPoolName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Creates a new Azure SQL Database Elastic Pool or updates an
        /// existing Azure SQL Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Azure SQL Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// operated on (updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for createing or updating an
        /// Azure Sql Databaser Elastic Pool.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        public static ElasticPoolCreateOrUpdateResponse CreateOrUpdate(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName, ElasticPoolCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).CreateOrUpdateAsync(resourceGroupName, serverName, elasticPoolName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates a new Azure SQL Database Elastic Pool or updates an
        /// existing Azure SQL Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Azure SQL Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// operated on (updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for createing or updating an
        /// Azure Sql Databaser Elastic Pool.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        public static Task<ElasticPoolCreateOrUpdateResponse> CreateOrUpdateAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName, ElasticPoolCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, serverName, elasticPoolName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes the Azure SQL Database Elastic Pool with the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Azure SQL Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).DeleteAsync(resourceGroupName, serverName, elasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes the Azure SQL Database Elastic Pool with the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Azure SQL Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return operations.DeleteAsync(resourceGroupName, serverName, elasticPoolName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Azure SQL Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Elastic Pool request.
        /// </returns>
        public static ElasticPoolGetResponse Get(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).GetAsync(resourceGroupName, serverName, elasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Azure SQL Database Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Elastic Pool request.
        /// </returns>
        public static Task<ElasticPoolGetResponse> GetAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return operations.GetAsync(resourceGroupName, serverName, elasticPoolName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL Database to be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database request.
        /// </returns>
        public static DatabaseGetResponse GetDatabases(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName, string databaseName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).GetDatabasesAsync(resourceGroupName, serverName, elasticPoolName, databaseName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <param name='databaseName'>
        /// Required. The name of the Azure SQL Database to be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database request.
        /// </returns>
        public static Task<DatabaseGetResponse> GetDatabasesAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName, string databaseName)
        {
            return operations.GetDatabasesAsync(resourceGroupName, serverName, elasticPoolName, databaseName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the status of an Azure Sql Database Elastic Pool create or
        /// update operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        public static ElasticPoolCreateOrUpdateResponse GetElasticPoolOperationStatus(this IElasticPoolOperations operations, string operationStatusLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).GetElasticPoolOperationStatusAsync(operationStatusLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the status of an Azure Sql Database Elastic Pool create or
        /// update operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database Elastic Pool operation.
        /// </returns>
        public static Task<ElasticPoolCreateOrUpdateResponse> GetElasticPoolOperationStatusAsync(this IElasticPoolOperations operations, string operationStatusLink)
        {
            return operations.GetElasticPoolOperationStatusAsync(operationStatusLink, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about Azure SQL Database Elastic Pools.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Database Elastic Pools are hosted.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool request.
        /// </returns>
        public static ElasticPoolListResponse List(this IElasticPoolOperations operations, string resourceGroupName, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).ListAsync(resourceGroupName, serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about Azure SQL Database Elastic Pools.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Database Elastic Pools are hosted.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool request.
        /// </returns>
        public static Task<ElasticPoolListResponse> ListAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName)
        {
            return operations.ListAsync(resourceGroupName, serverName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about Azure SQL Database Elastic Pool Activity.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Database Elastic Pools are hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool for which
        /// to get the current activity.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool Activity
        /// request.
        /// </returns>
        public static ElasticPoolActivityListResponse ListActivity(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).ListActivityAsync(resourceGroupName, serverName, elasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about Azure SQL Database Elastic Pool Activity.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server in which Azure
        /// SQL Database Elastic Pools are hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool for which
        /// to get the current activity.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool Activity
        /// request.
        /// </returns>
        public static Task<ElasticPoolActivityListResponse> ListActivityAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return operations.ListActivityAsync(resourceGroupName, serverName, elasticPoolName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about activity on Azure SQL Databases inside of
        /// an Azure Sql Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool Database
        /// Activity request.
        /// </returns>
        public static ElasticPoolDatabaseActivityListResponse ListDatabaseActivity(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).ListDatabaseActivityAsync(resourceGroupName, serverName, elasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about activity on Azure SQL Databases inside of
        /// an Azure Sql Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// Elastic Pool is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Elastic Pool Database
        /// Activity request.
        /// </returns>
        public static Task<ElasticPoolDatabaseActivityListResponse> ListDatabaseActivityAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return operations.ListDatabaseActivityAsync(resourceGroupName, serverName, elasticPoolName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database request.
        /// </returns>
        public static DatabaseListResponse ListDatabases(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IElasticPoolOperations)s).ListDatabasesAsync(resourceGroupName, serverName, elasticPoolName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Database Elastic Pool.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IElasticPoolOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Database Server on which the
        /// database is hosted.
        /// </param>
        /// <param name='elasticPoolName'>
        /// Required. The name of the Azure SQL Database Elastic Pool to be
        /// retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database request.
        /// </returns>
        public static Task<DatabaseListResponse> ListDatabasesAsync(this IElasticPoolOperations operations, string resourceGroupName, string serverName, string elasticPoolName)
        {
            return operations.ListDatabasesAsync(resourceGroupName, serverName, elasticPoolName, CancellationToken.None);
        }
    }
}
