// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// Represents all the operations for operating pertaining to activation on
    /// Azure SQL Data Warehouse databases. Contains operations to: Pause and
    /// Resume databases
    /// </summary>
    public partial interface IDatabaseActivationOperations
    {
        /// <summary>
        /// Start an Azure SQL Data Warehouse database pause operation.To
        /// determine the status of the operation call
        /// GetDatabaseActivationOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server on which the data warehouse
        /// database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Data Warehouse database to pause.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database operations.
        /// </returns>
        Task<DatabaseCreateOrUpdateResponse> BeginPauseAsync(string resourceGroupName, string serverName, string databaseName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Start an Azure SQL Data Warehouse database resume operation. To
        /// determine the status of the operation call
        /// GetDatabaseActivationOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server on which the data warehouse
        /// database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Data Warehouse database to resume.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database operations.
        /// </returns>
        Task<DatabaseCreateOrUpdateResponse> BeginResumeAsync(string resourceGroupName, string serverName, string databaseName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the status of an Azure SQL Data Warehouse Database pause or
        /// resume operation.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Location value returned by the Begin operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database operations.
        /// </returns>
        Task<DatabaseCreateOrUpdateResponse> GetDatabaseActivationOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Start an Azure SQL Data Warehouse database pause operation.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server on which the data warehouse
        /// database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Data Warehouse database to pause.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database operations.
        /// </returns>
        Task<DatabaseCreateOrUpdateResponse> PauseAsync(string resourceGroupName, string serverName, string databaseName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Start an Azure SQL Data Warehouse database resume operation.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server on which the data warehouse
        /// database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Data Warehouse database to resume.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Database operations.
        /// </returns>
        Task<DatabaseCreateOrUpdateResponse> ResumeAsync(string resourceGroupName, string serverName, string databaseName, CancellationToken cancellationToken);
    }
}
