// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// Represents all the operations for import/export on Azure SQL Databases.
    /// Contains operations to: Import, Export, Get Import/Export status for
    /// a database.
    /// </summary>
    public partial interface IImportExportOperations
    {
        /// <summary>
        /// Exports a Azure SQL Database to bacpac. To determine the status of
        /// the operation call GetImportExportOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server on which the database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Database to export.
        /// </param>
        /// <param name='parameters'>
        /// The required parameters for exporting a database.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response Azure Sql Import/Export operations.
        /// </returns>
        Task<ImportExportResponse> ExportAsync(string resourceGroupName, string serverName, string databaseName, ExportRequestParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the status of an Azure Sql Database import/export operation.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Location value returned by the Begin operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for Azure Sql Import/Export Status operation.
        /// </returns>
        Task<ImportExportOperationStatusResponse> GetImportExportOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Imports a bacpac to Azure SQL Database. To determine the status of
        /// the operation call GetImportExportOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server on which the database is hosted.
        /// </param>
        /// <param name='parameters'>
        /// The required parameters for importing to a database.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response Azure Sql Import/Export operations.
        /// </returns>
        Task<ImportExportResponse> ImportAsync(string resourceGroupName, string serverName, ImportRequestParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Imports a bacpac to an empty Azure SQL Database. To determine the
        /// status of the operation call GetImportExportOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Server on which the database is hosted.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Database to import to.
        /// </param>
        /// <param name='parameters'>
        /// The required parameters for importing to a database.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response Azure Sql Import/Export operations.
        /// </returns>
        Task<ImportExportResponse> ImportToExistingDatabaseAsync(string resourceGroupName, string serverName, string databaseName, ImportExtensionRequestParameteres parameters, CancellationToken cancellationToken);
    }
}
