// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// Represents all the operations for operating on Azure SQL Job Accounts.
    /// Contains operations to: Create, Retrieve, Update, and Delete Job
    /// Accounts
    /// </summary>
    public partial interface IJobAccountOperations
    {
        /// <summary>
        /// Begins creating a new Azure SQL Job Account or updating an existing
        /// Azure SQL Job Account. To determine the status of the operation
        /// call GetJobAccountOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server that the Job Account is
        /// hosted in.
        /// </param>
        /// <param name='jobAccountName'>
        /// The name of the Azure SQL Job Account to be created or updated.
        /// </param>
        /// <param name='parameters'>
        /// The required parameters for creating or updating a Job Account.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Job Account operations.
        /// </returns>
        Task<JobAccountOperationResponse> BeginCreateOrUpdateAsync(string resourceGroupName, string serverName, string jobAccountName, JobAccountCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Begins deleting the Azure SQL Job Account with the given name. To
        /// determine the status of the operation call
        /// GetJobAccountOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Database
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server that the Job Account is
        /// hosted in.
        /// </param>
        /// <param name='jobAccountName'>
        /// The name of the Azure SQL Job Account to be deleted.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Job Account operations.
        /// </returns>
        Task<JobAccountOperationResponse> BeginDeleteAsync(string resourceGroupName, string serverName, string jobAccountName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Creates a new Azure SQL Job Account or updates an existing Azure
        /// SQL Job Account.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Job Database Server that the Job Account
        /// is hosted in.
        /// </param>
        /// <param name='jobAccountName'>
        /// The name of the Azure SQL Job Account to be created or updated.
        /// </param>
        /// <param name='parameters'>
        /// The required parameters for creating or updating a Job Account.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Job Account operations.
        /// </returns>
        Task<JobAccountOperationResponse> CreateOrUpdateAsync(string resourceGroupName, string serverName, string jobAccountName, JobAccountCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Creates a new Azure SQL Job Account or updates an existing Azure
        /// SQL Job Account.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Job Database Server that the Job Account
        /// is hosted in.
        /// </param>
        /// <param name='jobAccountName'>
        /// The name of the Azure SQL Job Account to be created or updated.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Job Account operations.
        /// </returns>
        Task<JobAccountOperationResponse> DeleteAsync(string resourceGroupName, string serverName, string jobAccountName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about an Azure SQL Job Account.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server that the Job Account is
        /// hosted in.
        /// </param>
        /// <param name='jobAccountName'>
        /// The name of the Azure SQL Job Account to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Job Account request.
        /// </returns>
        Task<JobAccountGetResponse> GetAsync(string resourceGroupName, string serverName, string jobAccountName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets the status of an Azure Sql Job Account create or update
        /// operation.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Location value returned by the Begin operation
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql Job Account operations.
        /// </returns>
        Task<JobAccountOperationResponse> GetJobAccountOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about Azure SQL Job Accounts.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server that the Job Accounts are
        /// hosted in.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Job Accounts request.
        /// </returns>
        Task<JobAccountListResponse> ListAsync(string resourceGroupName, string serverName, CancellationToken cancellationToken);
    }
}
