// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using Hyak.Common;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql.Models
{
    /// <summary>
    /// Represents the properties of an Azure SQL Advisor Recommended Ac.
    /// </summary>
    public partial class RecommendedActionProperties
    {
        private IDictionary<string, object> _details;
        
        /// <summary>
        /// Optional. Gets additional details specific to this recommended
        /// action
        /// </summary>
        public IDictionary<string, object> Details
        {
            get { return this._details; }
            set { this._details = value; }
        }
        
        private RecommendedActionErrorInfo _errorDetails;
        
        /// <summary>
        /// Optional. Gets the error details if and why this recommended action
        /// is put to Error state.
        /// </summary>
        public RecommendedActionErrorInfo ErrorDetails
        {
            get { return this._errorDetails; }
            set { this._errorDetails = value; }
        }
        
        private IList<RecommendedActionImpactInfo> _estimatedImpact;
        
        /// <summary>
        /// Optional. Gets the estimated impact info for this recommended
        /// action E.g., Estimated CPU gain, Estimated Disk Space change
        /// </summary>
        public IList<RecommendedActionImpactInfo> EstimatedImpact
        {
            get { return this._estimatedImpact; }
            set { this._estimatedImpact = value; }
        }
        
        private string _executeActionDuration;
        
        /// <summary>
        /// Optional. Gets the time taken for applying this recommended action
        /// on user resource. E.g., Time taken for index creation
        /// </summary>
        public string ExecuteActionDuration
        {
            get { return this._executeActionDuration; }
            set { this._executeActionDuration = value; }
        }
        
        private string _executeActionInitiatedBy;
        
        /// <summary>
        /// Optional. Gets if approval for applying this recommended action is
        /// given by User/System.
        /// </summary>
        public string ExecuteActionInitiatedBy
        {
            get { return this._executeActionInitiatedBy; }
            set { this._executeActionInitiatedBy = value; }
        }
        
        private System.DateTime? _executeActionInitiatedTime;
        
        /// <summary>
        /// Optional. Gets the time when this recommended action is approved
        /// for execution
        /// </summary>
        public System.DateTime? ExecuteActionInitiatedTime
        {
            get { return this._executeActionInitiatedTime; }
            set { this._executeActionInitiatedTime = value; }
        }
        
        private System.DateTime? _executeActionStartTime;
        
        /// <summary>
        /// Optional. Gets the time when system started applying this
        /// recommended action on the user resource. E.g., Index creation
        /// start time
        /// </summary>
        public System.DateTime? ExecuteActionStartTime
        {
            get { return this._executeActionStartTime; }
            set { this._executeActionStartTime = value; }
        }
        
        private RecommendedActionImplementationInfo _implementationDetails;
        
        /// <summary>
        /// Optional. Gets the implementation details of this recommended
        /// action for user to apply it manually.
        /// </summary>
        public RecommendedActionImplementationInfo ImplementationDetails
        {
            get { return this._implementationDetails; }
            set { this._implementationDetails = value; }
        }
        
        private bool? _isArchivedAction;
        
        /// <summary>
        /// Optional. Gets if this recommended action was already suggested
        /// some time ago but user chose to ignore this and System recommends
        /// again.
        /// </summary>
        public bool? IsArchivedAction
        {
            get { return this._isArchivedAction; }
            set { this._isArchivedAction = value; }
        }
        
        private bool? _isExecutableAction;
        
        /// <summary>
        /// Optional. Gets if this recommended action is actionable to User
        /// </summary>
        public bool? IsExecutableAction
        {
            get { return this._isExecutableAction; }
            set { this._isExecutableAction = value; }
        }
        
        private bool? _isRevertableAction;
        
        /// <summary>
        /// Optional. Gets if changes applied by this recommended action can be
        /// reverted by User
        /// </summary>
        public bool? IsRevertableAction
        {
            get { return this._isRevertableAction; }
            set { this._isRevertableAction = value; }
        }
        
        private DateTime _lastRefresh;
        
        /// <summary>
        /// Optional. Gets time when this Recommended Action was last refreshed.
        /// </summary>
        public DateTime LastRefresh
        {
            get { return this._lastRefresh; }
            set { this._lastRefresh = value; }
        }
        
        private IList<string> _linkedObjects;
        
        /// <summary>
        /// Optional. Gets the observed/actual impact info for this recommended
        /// action E.g., Actual CPU gain, Actual Disk Space change
        /// </summary>
        public IList<string> LinkedObjects
        {
            get { return this._linkedObjects; }
            set { this._linkedObjects = value; }
        }
        
        private IList<RecommendedActionImpactInfo> _observedImpact;
        
        /// <summary>
        /// Optional. Gets the observed/actual impact info for this recommended
        /// action E.g., Actual CPU gain, Actual Disk Space change
        /// </summary>
        public IList<RecommendedActionImpactInfo> ObservedImpact
        {
            get { return this._observedImpact; }
            set { this._observedImpact = value; }
        }
        
        private string _recommendationReason;
        
        /// <summary>
        /// Optional. Gets the reason for recommending this action. E.g.,
        /// DuplicateIndex
        /// </summary>
        public string RecommendationReason
        {
            get { return this._recommendationReason; }
            set { this._recommendationReason = value; }
        }
        
        private string _revertActionDuration;
        
        /// <summary>
        /// Optional. Gets the time taken for reverting changes of this
        /// recommended action on user resource. E.g., Time taken for dropping
        /// the created index.
        /// </summary>
        public string RevertActionDuration
        {
            get { return this._revertActionDuration; }
            set { this._revertActionDuration = value; }
        }
        
        private string _revertActionInitiatedBy;
        
        /// <summary>
        /// Optional. Gets if approval for reverting this recommended action is
        /// given by User/System.
        /// </summary>
        public string RevertActionInitiatedBy
        {
            get { return this._revertActionInitiatedBy; }
            set { this._revertActionInitiatedBy = value; }
        }
        
        private System.DateTime? _revertActionInitiatedTime;
        
        /// <summary>
        /// Optional. Gets the time when this recommended action is approved
        /// for revert
        /// </summary>
        public System.DateTime? RevertActionInitiatedTime
        {
            get { return this._revertActionInitiatedTime; }
            set { this._revertActionInitiatedTime = value; }
        }
        
        private System.DateTime? _revertActionStartTime;
        
        /// <summary>
        /// Optional. Gets the time when system started reverting changes of
        /// this recommended action on user resource. E.g., Time when Index
        /// drop is executed.
        /// </summary>
        public System.DateTime? RevertActionStartTime
        {
            get { return this._revertActionStartTime; }
            set { this._revertActionStartTime = value; }
        }
        
        private int _score;
        
        /// <summary>
        /// Optional. Gets the impact of this recommended actions. Possible
        /// values are 1 - Low impact, 2 - Medium Impact and 3 - High Impact
        /// </summary>
        public int Score
        {
            get { return this._score; }
            set { this._score = value; }
        }
        
        private RecommendedActionStateInfo _state;
        
        /// <summary>
        /// Optional. Gets the info of the state the Recommended Action is in.
        /// </summary>
        public RecommendedActionStateInfo State
        {
            get { return this._state; }
            set { this._state = value; }
        }
        
        private IList<RecommendedActionMetricInfo> _timeSeries;
        
        /// <summary>
        /// Optional. Gets the time series info of metrics for this recommended
        /// action E.g., CPU time series
        /// </summary>
        public IList<RecommendedActionMetricInfo> TimeSeries
        {
            get { return this._timeSeries; }
            set { this._timeSeries = value; }
        }
        
        private DateTime _validSince;
        
        /// <summary>
        /// Optional. Gets the time since when this Recommended action is valid.
        /// </summary>
        public DateTime ValidSince
        {
            get { return this._validSince; }
            set { this._validSince = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the RecommendedActionProperties class.
        /// </summary>
        public RecommendedActionProperties()
        {
            this.Details = new LazyDictionary<string, object>();
            this.EstimatedImpact = new LazyList<RecommendedActionImpactInfo>();
            this.LinkedObjects = new LazyList<string>();
            this.ObservedImpact = new LazyList<RecommendedActionImpactInfo>();
            this.TimeSeries = new LazyList<RecommendedActionMetricInfo>();
        }
    }
}
