// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.Sql;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// The Windows Azure SQL Database management API provides a RESTful set of
    /// web services that interact with Windows Azure SQL Database services to
    /// manage your databases. The API enables users to create, retrieve,
    /// update, and delete databases and servers.
    /// </summary>
    public static partial class ServerDisasterRecoveryConfigurationOperationsExtensions
    {
        /// <summary>
        /// Begins creating a new or updating an existing Azure SQL Server
        /// disaster recovery configuration. To determine the status of the
        /// operation call
        /// GetServerDisasterRecoveryConfigurationOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL Server disaster recovery
        /// configuration to be operated on (Updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for creating or updating a Server
        /// disaster recovery configuration.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql server disaster recovery
        /// configuration operation.
        /// </returns>
        public static ServerDisasterRecoveryConfigurationCreateOrUpdateResponse BeginCreateOrUpdate(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName, ServerDisasterRecoveryConfigurationCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).BeginCreateOrUpdateAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Begins creating a new or updating an existing Azure SQL Server
        /// disaster recovery configuration. To determine the status of the
        /// operation call
        /// GetServerDisasterRecoveryConfigurationOperationStatus.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL Server disaster recovery
        /// configuration to be operated on (Updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for creating or updating a Server
        /// disaster recovery configuration.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql server disaster recovery
        /// configuration operation.
        /// </returns>
        public static Task<ServerDisasterRecoveryConfigurationCreateOrUpdateResponse> BeginCreateOrUpdateAsync(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName, ServerDisasterRecoveryConfigurationCreateOrUpdateParameters parameters)
        {
            return operations.BeginCreateOrUpdateAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Creates a new or updates an existing Azure SQL Server disaster
        /// recovery configuration.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL Server disaster recovery
        /// configuration to be operated on (Updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for creating or updating a Server
        /// disaster recovery configuration.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql server disaster recovery
        /// configuration operation.
        /// </returns>
        public static ServerDisasterRecoveryConfigurationCreateOrUpdateResponse CreateOrUpdate(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName, ServerDisasterRecoveryConfigurationCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).CreateOrUpdateAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates a new or updates an existing Azure SQL Server disaster
        /// recovery configuration.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Database Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL Server disaster recovery
        /// configuration to be operated on (Updated or created).
        /// </param>
        /// <param name='parameters'>
        /// Required. The required parameters for creating or updating a Server
        /// disaster recovery configuration.
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql server disaster recovery
        /// configuration operation.
        /// </returns>
        public static Task<ServerDisasterRecoveryConfigurationCreateOrUpdateResponse> CreateOrUpdateAsync(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName, ServerDisasterRecoveryConfigurationCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes the Azure SQL server disaster recovery configuration with
        /// the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to be deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).DeleteAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes the Azure SQL server disaster recovery configuration with
        /// the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to be deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return operations.DeleteAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, CancellationToken.None);
        }
        
        /// <summary>
        /// Begins failover for the Azure SQL server disaster recovery
        /// configuration with the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to start failover.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Failover(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).FailoverAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Begins failover for the Azure SQL server disaster recovery
        /// configuration with the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to start failover.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> FailoverAsync(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return operations.FailoverAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, CancellationToken.None);
        }
        
        /// <summary>
        /// Begins failover for the Azure SQL server disaster recovery
        /// configuration with the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to start failover.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse FailoverAllowDataLoss(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).FailoverAllowDataLossAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Begins failover for the Azure SQL server disaster recovery
        /// configuration with the given name.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to start failover.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> FailoverAllowDataLossAsync(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return operations.FailoverAllowDataLossAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Server disaster recovery
        /// configurations.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a get server disaster recovery
        /// configuration request.
        /// </returns>
        public static ServerDisasterRecoveryConfigurationGetResponse Get(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).GetAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about an Azure SQL Server disaster recovery
        /// configurations.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the server
        /// belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <param name='serverDisasterRecoveryConfigurationName'>
        /// Required. The name of the Azure SQL server disaster recovery
        /// configuration to be retrieved.
        /// </param>
        /// <returns>
        /// Represents the response to a get server disaster recovery
        /// configuration request.
        /// </returns>
        public static Task<ServerDisasterRecoveryConfigurationGetResponse> GetAsync(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName, string serverDisasterRecoveryConfigurationName)
        {
            return operations.GetAsync(resourceGroupName, serverName, serverDisasterRecoveryConfigurationName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets the status of an Azure Sql Server disaster recovery
        /// configuration create or update operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql server disaster recovery
        /// configuration operation.
        /// </returns>
        public static ServerDisasterRecoveryConfigurationCreateOrUpdateResponse GetServerDisasterRecoveryConfigurationOperationStatus(this IServerDisasterRecoveryConfigurationOperations operations, string operationStatusLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).GetServerDisasterRecoveryConfigurationOperationStatusAsync(operationStatusLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets the status of an Azure Sql Server disaster recovery
        /// configuration create or update operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation
        /// </param>
        /// <returns>
        /// Response for long running Azure Sql server disaster recovery
        /// configuration operation.
        /// </returns>
        public static Task<ServerDisasterRecoveryConfigurationCreateOrUpdateResponse> GetServerDisasterRecoveryConfigurationOperationStatusAsync(this IServerDisasterRecoveryConfigurationOperations operations, string operationStatusLink)
        {
            return operations.GetServerDisasterRecoveryConfigurationOperationStatusAsync(operationStatusLink, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns information about Azure SQL Server disaster recovery
        /// configurations.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Server disaster
        /// recovery configuration request.
        /// </returns>
        public static ServerDisasterRecoveryConfigurationListResponse List(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IServerDisasterRecoveryConfigurationOperations)s).ListAsync(resourceGroupName, serverName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns information about Azure SQL Server disaster recovery
        /// configurations.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Sql.IServerDisasterRecoveryConfigurationOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the Resource Group to which the Azure SQL
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// Required. The name of the Azure SQL Server.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Server disaster
        /// recovery configuration request.
        /// </returns>
        public static Task<ServerDisasterRecoveryConfigurationListResponse> ListAsync(this IServerDisasterRecoveryConfigurationOperations operations, string resourceGroupName, string serverName)
        {
            return operations.ListAsync(resourceGroupName, serverName, CancellationToken.None);
        }
    }
}
