// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.TrafficManager;
using Microsoft.Azure.Management.TrafficManager.Models;

namespace Microsoft.Azure.Management.TrafficManager
{
    /// <summary>
    /// Client for creating, updating, listing and deleting Azure Traffic
    /// Manager profiles  (see
    /// http://azure.microsoft.com/en-gb/documentation/articles/traffic-manager-overview/
    /// for more information)
    /// </summary>
    public static partial class EndpointOperationsExtensions
    {
        /// <summary>
        /// Create or update a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint to be created or updated.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. The Traffic Manager endpoint parameters supplied to the
        /// CreateOrUpdate operation.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'CreateOrUpdate'
        /// operation.
        /// </returns>
        public static EndpointCreateOrUpdateResponse CreateOrUpdate(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName, EndpointCreateOrUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IEndpointOperations)s).CreateOrUpdateAsync(resourceGroupName, profileName, endpointType, endpointName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or update a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint to be created or updated.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. The Traffic Manager endpoint parameters supplied to the
        /// CreateOrUpdate operation.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'CreateOrUpdate'
        /// operation.
        /// </returns>
        public static Task<EndpointCreateOrUpdateResponse> CreateOrUpdateAsync(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName, EndpointCreateOrUpdateParameters parameters)
        {
            return operations.CreateOrUpdateAsync(resourceGroupName, profileName, endpointType, endpointName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint to be deleted.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IEndpointOperations)s).DeleteAsync(resourceGroupName, profileName, endpointType, endpointName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint to be deleted.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName)
        {
            return operations.DeleteAsync(resourceGroupName, profileName, endpointType, endpointName, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'Get' operation.
        /// </returns>
        public static EndpointGetResponse Get(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IEndpointOperations)s).GetAsync(resourceGroupName, profileName, endpointType, endpointName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'Get' operation.
        /// </returns>
        public static Task<EndpointGetResponse> GetAsync(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName)
        {
            return operations.GetAsync(resourceGroupName, profileName, endpointType, endpointName, CancellationToken.None);
        }
        
        /// <summary>
        /// Create or update a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint to be created or updated.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. The Traffic Manager endpoint parameters supplied to the
        /// Update operation.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'CreateOrUpdate'
        /// operation.
        /// </returns>
        public static EndpointUpdateResponse Update(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName, EndpointUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IEndpointOperations)s).UpdateAsync(resourceGroupName, profileName, endpointType, endpointName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Create or update a Traffic Manager endpoint.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.TrafficManager.IEndpointOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group containing the Traffic
        /// Manager endpoint to be created or updated.
        /// </param>
        /// <param name='profileName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointType'>
        /// Required. The type of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='endpointName'>
        /// Required. The name of the Traffic Manager endpoint to be created or
        /// updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. The Traffic Manager endpoint parameters supplied to the
        /// Update operation.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'CreateOrUpdate'
        /// operation.
        /// </returns>
        public static Task<EndpointUpdateResponse> UpdateAsync(this IEndpointOperations operations, string resourceGroupName, string profileName, string endpointType, string endpointName, EndpointUpdateParameters parameters)
        {
            return operations.UpdateAsync(resourceGroupName, profileName, endpointType, endpointName, parameters, CancellationToken.None);
        }
    }
}
