// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.Azure.Management.TrafficManager.Models;

namespace Microsoft.Azure.Management.TrafficManager
{
    /// <summary>
    /// Operations for managing Traffic Manager endpoints.
    /// </summary>
    public partial interface IEndpointOperations
    {
        /// <summary>
        /// Create or update a Traffic Manager endpoint.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group containing the Traffic Manager
        /// endpoint to be created or updated.
        /// </param>
        /// <param name='profileName'>
        /// The name of the Traffic Manager endpoint to be created or updated.
        /// </param>
        /// <param name='endpointType'>
        /// The type of the Traffic Manager endpoint to be created or updated.
        /// </param>
        /// <param name='endpointName'>
        /// The name of the Traffic Manager endpoint to be created or updated.
        /// </param>
        /// <param name='parameters'>
        /// The Traffic Manager endpoint parameters supplied to the
        /// CreateOrUpdate operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'CreateOrUpdate'
        /// operation.
        /// </returns>
        Task<EndpointCreateOrUpdateResponse> CreateOrUpdateAsync(string resourceGroupName, string profileName, string endpointType, string endpointName, EndpointCreateOrUpdateParameters parameters, CancellationToken cancellationToken);
        
        /// <summary>
        /// Deletes a Traffic Manager endpoint.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group containing the Traffic Manager
        /// endpoint to be deleted.
        /// </param>
        /// <param name='profileName'>
        /// The name of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <param name='endpointType'>
        /// The type of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <param name='endpointName'>
        /// The name of the Traffic Manager endpoint to be deleted.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string profileName, string endpointType, string endpointName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets a Traffic Manager endpoint.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group containing the Traffic Manager
        /// endpoint.
        /// </param>
        /// <param name='profileName'>
        /// The name of the Traffic Manager endpoint.
        /// </param>
        /// <param name='endpointType'>
        /// The type of the Traffic Manager endpoint.
        /// </param>
        /// <param name='endpointName'>
        /// The name of the Traffic Manager endpoint.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'Get' operation.
        /// </returns>
        Task<EndpointGetResponse> GetAsync(string resourceGroupName, string profileName, string endpointType, string endpointName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Create or update a Traffic Manager endpoint.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the resource group containing the Traffic Manager
        /// endpoint to be created or updated.
        /// </param>
        /// <param name='profileName'>
        /// The name of the Traffic Manager endpoint to be created or updated.
        /// </param>
        /// <param name='endpointType'>
        /// The type of the Traffic Manager endpoint to be created or updated.
        /// </param>
        /// <param name='endpointName'>
        /// The name of the Traffic Manager endpoint to be created or updated.
        /// </param>
        /// <param name='parameters'>
        /// The Traffic Manager endpoint parameters supplied to the Update
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response to a Traffic Manager endpoint 'CreateOrUpdate'
        /// operation.
        /// </returns>
        Task<EndpointUpdateResponse> UpdateAsync(string resourceGroupName, string profileName, string endpointType, string endpointName, EndpointUpdateParameters parameters, CancellationToken cancellationToken);
    }
}
