// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;

namespace Microsoft.Azure.Management.TrafficManager.Models
{
    /// <summary>
    /// Class containing DNS settings in a Traffic Manager profile.
    /// </summary>
    public partial class DnsConfig
    {
        private string _fqdn;
        
        /// <summary>
        /// Optional. Gets or sets the fully-qualified domain name (FQDN) of
        /// the Traffic Manager profile.  This is formed from the
        /// concatenation of the RelativeName with the DNS domain used by
        /// Azure Traffic Manager.
        /// </summary>
        public string Fqdn
        {
            get { return this._fqdn; }
            set { this._fqdn = value; }
        }
        
        private string _relativeName;
        
        /// <summary>
        /// Required. Gets or sets the relative DNS name provided by this
        /// Traffic Manager profile.  This value is combined with the DNS
        /// domain name used by Azure Traffic Manager to form the
        /// fully-qualified domain name (FQDN) of the profile.
        /// </summary>
        public string RelativeName
        {
            get { return this._relativeName; }
            set { this._relativeName = value; }
        }
        
        private uint _ttl;
        
        /// <summary>
        /// Required. Gets or sets the DNS Ttime-To-Live (TTL), in seconds.
        /// This informs the local DNS resolvers and DNS clients how long to
        /// cache DNS responses provided by this Traffic Manager profile.
        /// </summary>
        public uint Ttl
        {
            get { return this._ttl; }
            set { this._ttl = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the DnsConfig class.
        /// </summary>
        public DnsConfig()
        {
        }
        
        /// <summary>
        /// Initializes a new instance of the DnsConfig class with required
        /// arguments.
        /// </summary>
        public DnsConfig(string relativeName, uint ttl)
            : this()
        {
            if (relativeName == null)
            {
                throw new ArgumentNullException("relativeName");
            }
            this.RelativeName = relativeName;
            this.Ttl = ttl;
        }
    }
}
