// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using Hyak.Common;
using Microsoft.WindowsAzure.Management.Compute.Models;

namespace Microsoft.WindowsAzure.Management.Compute.Models
{
    /// <summary>
    /// Optional. Declared endpoints to automatically add to each deployment
    /// with this extension added.
    /// </summary>
    public partial class ExtensionEndpointConfiguration
    {
        private IList<ExtensionEndpointConfiguration.InputEndpoint> _inputEndpoints;
        
        /// <summary>
        /// Optional. Optional. A list of input endpoints required by this
        /// extension. Supported on WebRole or WorkerRole, but ignored on
        /// VmRole. Each input endpoint will be automatically added to the
        /// service definition of the deployment utilizing this extension.
        /// </summary>
        public IList<ExtensionEndpointConfiguration.InputEndpoint> InputEndpoints
        {
            get { return this._inputEndpoints; }
            set { this._inputEndpoints = value; }
        }
        
        private IList<ExtensionEndpointConfiguration.InstanceInputEndpoint> _instanceInputEndpoints;
        
        /// <summary>
        /// Optional. Optional. A list of instance input endpoints required by
        /// this extension. Supported on WebRole or WorkerRole, but ignored on
        /// VmRole. Each instance input endpoint will be automatically added
        /// to the service definition of the deployment utilizing this
        /// extension.
        /// </summary>
        public IList<ExtensionEndpointConfiguration.InstanceInputEndpoint> InstanceInputEndpoints
        {
            get { return this._instanceInputEndpoints; }
            set { this._instanceInputEndpoints = value; }
        }
        
        private IList<ExtensionEndpointConfiguration.InternalEndpoint> _internalEndpoints;
        
        /// <summary>
        /// Optional. Optional. A list of internal endpoints required by this
        /// extension. Supported on WebRole or WorkerRole, but ignored on
        /// VmRole. Each internal endpoint will be automatically added to the
        /// service definition of the deployment utilizing this extension.
        /// </summary>
        public IList<ExtensionEndpointConfiguration.InternalEndpoint> InternalEndpoints
        {
            get { return this._internalEndpoints; }
            set { this._internalEndpoints = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the ExtensionEndpointConfiguration
        /// class.
        /// </summary>
        public ExtensionEndpointConfiguration()
        {
            this.InputEndpoints = new LazyList<ExtensionEndpointConfiguration.InputEndpoint>();
            this.InstanceInputEndpoints = new LazyList<ExtensionEndpointConfiguration.InstanceInputEndpoint>();
            this.InternalEndpoints = new LazyList<ExtensionEndpointConfiguration.InternalEndpoint>();
        }
        
        /// <summary>
        /// The extension's input endpoint configuration.
        /// </summary>
        public partial class InputEndpoint
        {
            private string _localPort;
            
            /// <summary>
            /// Required. Required for each input endpoint. Specifies a port
            /// used for internal connections on the endpoint. Its value has
            /// to be between 1-65535 inclusive, or '*' indicating any port
            /// number automatically allocated by Windows Azure.
            /// </summary>
            public string LocalPort
            {
                get { return this._localPort; }
                set { this._localPort = value; }
            }
            
            private string _name;
            
            /// <summary>
            /// Required. Required for each input endpoint. A unique name for
            /// the external endpoint.
            /// </summary>
            public string Name
            {
                get { return this._name; }
                set { this._name = value; }
            }
            
            private int _port;
            
            /// <summary>
            /// Required. Required for each input endpoint. The port for the
            /// external endpoint. You can specify any port number you choose,
            /// but the port number specified must not collide with port
            /// numbers occupied by roles in the service. Possible values
            /// range between 1 and 65535, inclusive
            /// </summary>
            public int Port
            {
                get { return this._port; }
                set { this._port = value; }
            }
            
            private string _protocol;
            
            /// <summary>
            /// Required. Required for each input endpoint. Transport protocol
            /// for the endpoint. Its value has to be one of the following:
            /// 'tcp', 'udp', 'http', 'https'.
            /// </summary>
            public string Protocol
            {
                get { return this._protocol; }
                set { this._protocol = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the InputEndpoint class.
            /// </summary>
            public InputEndpoint()
            {
            }
        }
        
        /// <summary>
        /// The extension's instance input endpoint configuration.
        /// </summary>
        public partial class InstanceInputEndpoint
        {
            private int _fixedPortMax;
            
            /// <summary>
            /// Required. Required for each instance input endpoint. Specifies
            /// a max port valuefor the instance input endpoint port range
            /// used for instance endpointdistribution by. Its value has to be
            /// between 1-65535 inclusiveand should be unique range that can
            /// be injected without conflict.Should also be equal or higher
            /// than FixedPortMin.
            /// </summary>
            public int FixedPortMax
            {
                get { return this._fixedPortMax; }
                set { this._fixedPortMax = value; }
            }
            
            private int _fixedPortMin;
            
            /// <summary>
            /// Required. Required for each instance input endpoint. Specifies
            /// a min port valuefor the instance input endpoint port range
            /// used for instance endpointdistribution by. Its value has to be
            /// between 1-65535 inclusiveand should be unique range that can
            /// be injected without conflict.
            /// </summary>
            public int FixedPortMin
            {
                get { return this._fixedPortMin; }
                set { this._fixedPortMin = value; }
            }
            
            private string _localPort;
            
            /// <summary>
            /// Required. Required for each instance input endpoint. Specifies
            /// a port used for internal connections on the endpoint. Its
            /// value has to be between 1-65535 inclusive, or '*' indicating
            /// any port number automatically allocated by Windows Azure.
            /// </summary>
            public string LocalPort
            {
                get { return this._localPort; }
                set { this._localPort = value; }
            }
            
            private string _name;
            
            /// <summary>
            /// Required. Required for each instance input endpoint. A unique
            /// name for the instance input endpoint.
            /// </summary>
            public string Name
            {
                get { return this._name; }
                set { this._name = value; }
            }
            
            private string _protocol;
            
            /// <summary>
            /// Required. Required for each instance input endpoint. Transport
            /// protocol for the endpoint. Its value has to be one of the
            /// following: 'tcp', 'udp', 'http', 'https'.
            /// </summary>
            public string Protocol
            {
                get { return this._protocol; }
                set { this._protocol = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the InstanceInputEndpoint class.
            /// </summary>
            public InstanceInputEndpoint()
            {
            }
        }
        
        /// <summary>
        /// The extension's internal endpoint configuration.
        /// </summary>
        public partial class InternalEndpoint
        {
            private string _name;
            
            /// <summary>
            /// Required. Required for each internal endpoint. A unique name
            /// for the external endpoint.
            /// </summary>
            public string Name
            {
                get { return this._name; }
                set { this._name = value; }
            }
            
            private int _port;
            
            /// <summary>
            /// Required. Required for each internal endpoint. The port for the
            /// external endpoint. You can specify any port number you choose,
            /// but the port number specified must not collide with port
            /// numbers occupied by roles in the service. Possible values
            /// range between 1 and 65535, inclusive
            /// </summary>
            public int Port
            {
                get { return this._port; }
                set { this._port = value; }
            }
            
            private string _protocol;
            
            /// <summary>
            /// Required. Required for each internal endpoint. Transport
            /// protocol for the endpoint. Its value has to be one of the
            /// following: 'tcp', 'udp', 'http', 'https'.
            /// </summary>
            public string Protocol
            {
                get { return this._protocol; }
                set { this._protocol = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the InternalEndpoint class.
            /// </summary>
            public InternalEndpoint()
            {
            }
            
            /// <summary>
            /// Initializes a new instance of the InternalEndpoint class with
            /// required arguments.
            /// </summary>
            public InternalEndpoint(string name, string protocol, int port)
                : this()
            {
                if (name == null)
                {
                    throw new ArgumentNullException("name");
                }
                if (protocol == null)
                {
                    throw new ArgumentNullException("protocol");
                }
                this.Name = name;
                this.Protocol = protocol;
                this.Port = port;
            }
        }
    }
}
