// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure;
using Microsoft.Azure.Management.SiteRecovery;
using Microsoft.Azure.Management.SiteRecovery.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Management.SiteRecovery
{
    /// <summary>
    /// Definition for Job Operations.
    /// </summary>
    internal partial class JobOperations : IServiceOperations<SiteRecoveryManagementClient>, IJobOperations
    {
        /// <summary>
        /// Initializes a new instance of the JobOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal JobOperations(SiteRecoveryManagementClient client)
        {
            this._client = client;
        }
        
        private SiteRecoveryManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.Azure.Management.SiteRecovery.SiteRecoveryManagementClient.
        /// </summary>
        public SiteRecoveryManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Cancel the job .
        /// </summary>
        /// <param name='jobName'>
        /// Required. Job Name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginCancellingAsync(string jobName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginCancellingAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationJobs/";
            url = url + Uri.EscapeDataString(jobName);
            url = url + "/Cancel";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Export jobs to blob.
        /// </summary>
        /// <param name='parameters'>
        /// Required. Job Query Filters
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginExportingAsync(JobQueryParameter parameters, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginExportingAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationJobs/export";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject jobQueryParameterValue = new JObject();
                requestDoc = jobQueryParameterValue;
                
                if (parameters.StartTime != null)
                {
                    jobQueryParameterValue["startTime"] = parameters.StartTime;
                }
                
                if (parameters.EndTime != null)
                {
                    jobQueryParameterValue["endTime"] = parameters.EndTime;
                }
                
                if (parameters.FabricId != null)
                {
                    jobQueryParameterValue["fabricId"] = parameters.FabricId;
                }
                
                if (parameters.AffectedObjectTypes != null)
                {
                    if (parameters.AffectedObjectTypes is ILazyCollection == false || ((ILazyCollection)parameters.AffectedObjectTypes).IsInitialized)
                    {
                        JArray affectedObjectTypesArray = new JArray();
                        foreach (string affectedObjectTypesItem in parameters.AffectedObjectTypes)
                        {
                            affectedObjectTypesArray.Add(affectedObjectTypesItem);
                        }
                        jobQueryParameterValue["affectedObjectTypes"] = affectedObjectTypesArray;
                    }
                }
                
                if (parameters.JobStatus != null)
                {
                    if (parameters.JobStatus is ILazyCollection == false || ((ILazyCollection)parameters.JobStatus).IsInitialized)
                    {
                        JArray jobStatusArray = new JArray();
                        foreach (string jobStatusItem in parameters.JobStatus)
                        {
                            jobStatusArray.Add(jobStatusItem);
                        }
                        jobQueryParameterValue["jobStatus"] = jobStatusArray;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Restart the job .
        /// </summary>
        /// <param name='jobName'>
        /// Required. Job Name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginRestartingAsync(string jobName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginRestartingAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationJobs/";
            url = url + Uri.EscapeDataString(jobName);
            url = url + "/Restart";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Resume the job .
        /// </summary>
        /// <param name='jobId'>
        /// Required. Job ID.
        /// </param>
        /// <param name='resumeJobParameters'>
        /// Optional. Resume job parameters.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginResumingAsync(string jobId, ResumeJobParams resumeJobParameters, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (jobId == null)
            {
                throw new ArgumentNullException("jobId");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                tracingParameters.Add("resumeJobParameters", resumeJobParameters);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginResumingAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationJobs/";
            url = url + Uri.EscapeDataString(jobId);
            url = url + "/Resume";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                if (resumeJobParameters != null)
                {
                    JObject resumeJobParamsValue = new JObject();
                    requestDoc = resumeJobParamsValue;
                    
                    if (resumeJobParameters.Properties != null)
                    {
                        JObject propertiesValue = new JObject();
                        resumeJobParamsValue["properties"] = propertiesValue;
                        
                        if (resumeJobParameters.Properties.Comments != null)
                        {
                            propertiesValue["comments"] = resumeJobParameters.Properties.Comments;
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Cancel the job .
        /// </summary>
        /// <param name='jobName'>
        /// Required. Job Name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> CancelAsync(string jobName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "CancelAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.Jobs.BeginCancellingAsync(jobName, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            JobOperationResponse result = await client.Jobs.GetCancelStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.Jobs.GetCancelStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Export jobs to blob.
        /// </summary>
        /// <param name='parameters'>
        /// Required. Job Query Filters
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> ExportAsync(JobQueryParameter parameters, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ExportAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.Jobs.BeginExportingAsync(parameters, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            JobOperationResponse result = await client.Jobs.GetExportStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.Jobs.GetExportStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Get the job details.
        /// </summary>
        /// <param name='jobId'>
        /// Required. Job ID.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the Job details object.
        /// </returns>
        public async Task<JobResponse> GetAsync(string jobId, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (jobId == null)
            {
                throw new ArgumentNullException("jobId");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationJobs/";
            url = url + Uri.EscapeDataString(jobId);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken scenarioNameValue = propertiesValue["scenarioName"];
                                if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                {
                                    string scenarioNameInstance = ((string)scenarioNameValue);
                                    propertiesInstance.ScenarioName = scenarioNameInstance;
                                }
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken activityIdValue = propertiesValue["activityId"];
                                if (activityIdValue != null && activityIdValue.Type != JTokenType.Null)
                                {
                                    string activityIdInstance = ((string)activityIdValue);
                                    propertiesInstance.ActivityId = activityIdInstance;
                                }
                                
                                JToken startTimeValue = propertiesValue["startTime"];
                                if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime startTimeInstance = ((DateTime)startTimeValue);
                                    propertiesInstance.StartTime = startTimeInstance;
                                }
                                
                                JToken endTimeValue = propertiesValue["endTime"];
                                if (endTimeValue != null && endTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime endTimeInstance = ((DateTime)endTimeValue);
                                    propertiesInstance.EndTime = endTimeInstance;
                                }
                                
                                JToken targetObjectIdValue = propertiesValue["targetObjectId"];
                                if (targetObjectIdValue != null && targetObjectIdValue.Type != JTokenType.Null)
                                {
                                    string targetObjectIdInstance = ((string)targetObjectIdValue);
                                    propertiesInstance.TargetObjectId = targetObjectIdInstance;
                                }
                                
                                JToken targetInstanceTypeValue = propertiesValue["targetInstanceType"];
                                if (targetInstanceTypeValue != null && targetInstanceTypeValue.Type != JTokenType.Null)
                                {
                                    string targetInstanceTypeInstance = ((string)targetInstanceTypeValue);
                                    propertiesInstance.TargetInstanceType = targetInstanceTypeInstance;
                                }
                                
                                JToken targetObjectNameValue = propertiesValue["targetObjectName"];
                                if (targetObjectNameValue != null && targetObjectNameValue.Type != JTokenType.Null)
                                {
                                    string targetObjectNameInstance = ((string)targetObjectNameValue);
                                    propertiesInstance.TargetObjectName = targetObjectNameInstance;
                                }
                                
                                JToken allowedActionsArray = propertiesValue["allowedActions"];
                                if (allowedActionsArray != null && allowedActionsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedActionsValue in ((JArray)allowedActionsArray))
                                    {
                                        propertiesInstance.AllowedActions.Add(((string)allowedActionsValue));
                                    }
                                }
                                
                                JToken stateValue = propertiesValue["state"];
                                if (stateValue != null && stateValue.Type != JTokenType.Null)
                                {
                                    string stateInstance = ((string)stateValue);
                                    propertiesInstance.State = stateInstance;
                                }
                                
                                JToken stateDescriptionValue = propertiesValue["stateDescription"];
                                if (stateDescriptionValue != null && stateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string stateDescriptionInstance = ((string)stateDescriptionValue);
                                    propertiesInstance.StateDescription = stateDescriptionInstance;
                                }
                                
                                JToken tasksArray = propertiesValue["tasks"];
                                if (tasksArray != null && tasksArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken tasksValue in ((JArray)tasksArray))
                                    {
                                        AsrTask taskInstance = new AsrTask();
                                        propertiesInstance.Tasks.Add(taskInstance);
                                        
                                        JToken groupTaskCustomDetailsValue = tasksValue["groupTaskCustomDetails"];
                                        if (groupTaskCustomDetailsValue != null && groupTaskCustomDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)groupTaskCustomDetailsValue["instanceType"]);
                                            if (typeName == "InlineWorkflowTaskDetails")
                                            {
                                                InlineWorkflowTaskDetails inlineWorkflowTaskDetailsInstance = new InlineWorkflowTaskDetails();
                                                
                                                JToken workflowIdsArray = groupTaskCustomDetailsValue["workflowIds"];
                                                if (workflowIdsArray != null && workflowIdsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken workflowIdsValue in ((JArray)workflowIdsArray))
                                                    {
                                                        inlineWorkflowTaskDetailsInstance.WorkflowIds.Add(((string)workflowIdsValue));
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    inlineWorkflowTaskDetailsInstance.Type = instanceTypeInstance;
                                                }
                                                
                                                JToken childTasksArray = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray != null && childTasksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue in ((JArray)childTasksArray))
                                                    {
                                                        AsrTaskBase taskBaseInstance = new AsrTaskBase();
                                                        inlineWorkflowTaskDetailsInstance.ChildTasks.Add(taskBaseInstance);
                                                        
                                                        JToken taskIdValue = childTasksValue["taskId"];
                                                        if (taskIdValue != null && taskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance = ((string)taskIdValue);
                                                            taskBaseInstance.ID = taskIdInstance;
                                                        }
                                                        
                                                        JToken nameValue = childTasksValue["name"];
                                                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance = ((string)nameValue);
                                                            taskBaseInstance.Name = nameInstance;
                                                        }
                                                        
                                                        JToken startTimeValue2 = childTasksValue["startTime"];
                                                        if (startTimeValue2 != null && startTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance2 = ((DateTime)startTimeValue2);
                                                            taskBaseInstance.StartTime = startTimeInstance2;
                                                        }
                                                        
                                                        JToken endTimeValue2 = childTasksValue["endTime"];
                                                        if (endTimeValue2 != null && endTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance2 = ((DateTime)endTimeValue2);
                                                            taskBaseInstance.EndTime = endTimeInstance2;
                                                        }
                                                        
                                                        JToken allowedActionsArray2 = childTasksValue["allowedActions"];
                                                        if (allowedActionsArray2 != null && allowedActionsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue2 in ((JArray)allowedActionsArray2))
                                                            {
                                                                taskBaseInstance.Actions.Add(((string)allowedActionsValue2));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue2 = childTasksValue["friendlyName"];
                                                        if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                                            taskBaseInstance.TaskFriendlyName = friendlyNameInstance2;
                                                        }
                                                        
                                                        JToken stateValue2 = childTasksValue["state"];
                                                        if (stateValue2 != null && stateValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance2 = ((string)stateValue2);
                                                            taskBaseInstance.State = stateInstance2;
                                                        }
                                                        
                                                        JToken stateDescriptionValue2 = childTasksValue["stateDescription"];
                                                        if (stateDescriptionValue2 != null && stateDescriptionValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance2 = ((string)stateDescriptionValue2);
                                                            taskBaseInstance.StateDescription = stateDescriptionInstance2;
                                                        }
                                                        
                                                        JToken taskTypeValue = childTasksValue["taskType"];
                                                        if (taskTypeValue != null && taskTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance = ((string)taskTypeValue);
                                                            taskBaseInstance.TaskType = taskTypeInstance;
                                                        }
                                                        
                                                        JToken errorsArray = childTasksValue["errors"];
                                                        if (errorsArray != null && errorsArray.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue in ((JArray)errorsArray))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance = new JobErrorDetails();
                                                                taskBaseInstance.Errors.Add(jobErrorDetailsInstance);
                                                                
                                                                JToken serviceErrorDetailsValue = errorsValue["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue != null && serviceErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance = new ServiceError();
                                                                    jobErrorDetailsInstance.ServiceErrorDetails = serviceErrorDetailsInstance;
                                                                    
                                                                    JToken codeValue = serviceErrorDetailsValue["code"];
                                                                    if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance = ((string)codeValue);
                                                                        serviceErrorDetailsInstance.Code = codeInstance;
                                                                    }
                                                                    
                                                                    JToken messageValue = serviceErrorDetailsValue["message"];
                                                                    if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance = ((string)messageValue);
                                                                        serviceErrorDetailsInstance.Message = messageInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue = serviceErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance = ((string)possibleCausesValue);
                                                                        serviceErrorDetailsInstance.PossibleCauses = possibleCausesInstance;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue = serviceErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance = ((string)recommendedActionValue);
                                                                        serviceErrorDetailsInstance.RecommendedAction = recommendedActionInstance;
                                                                    }
                                                                    
                                                                    JToken activityIdValue2 = serviceErrorDetailsValue["activityId"];
                                                                    if (activityIdValue2 != null && activityIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance2 = ((string)activityIdValue2);
                                                                        serviceErrorDetailsInstance.ActivityId = activityIdInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue = errorsValue["providerErrorDetails"];
                                                                if (providerErrorDetailsValue != null && providerErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance = new ProviderError();
                                                                    jobErrorDetailsInstance.ProviderErrorDetails = providerErrorDetailsInstance;
                                                                    
                                                                    JToken errorCodeValue = providerErrorDetailsValue["errorCode"];
                                                                    if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance = ((int)errorCodeValue);
                                                                        providerErrorDetailsInstance.ErrorCode = errorCodeInstance;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue = providerErrorDetailsValue["errorMessage"];
                                                                    if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance = ((string)errorMessageValue);
                                                                        providerErrorDetailsInstance.ErrorMessage = errorMessageInstance;
                                                                    }
                                                                    
                                                                    JToken errorIdValue = providerErrorDetailsValue["errorId"];
                                                                    if (errorIdValue != null && errorIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance = ((string)errorIdValue);
                                                                        providerErrorDetailsInstance.ErrorId = errorIdInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue2 = providerErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue2 != null && possibleCausesValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance2 = ((string)possibleCausesValue2);
                                                                        providerErrorDetailsInstance.PossibleCauses = possibleCausesInstance2;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue2 = providerErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue2 != null && recommendedActionValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance2 = ((string)recommendedActionValue2);
                                                                        providerErrorDetailsInstance.RecommendedAction = recommendedActionInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue = errorsValue["errorLevel"];
                                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                                    jobErrorDetailsInstance.ErrorLevel = errorLevelInstance;
                                                                }
                                                                
                                                                JToken creationTimeValue = errorsValue["creationTime"];
                                                                if (creationTimeValue != null && creationTimeValue.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance = ((DateTime)creationTimeValue);
                                                                    jobErrorDetailsInstance.CreationTime = creationTimeInstance;
                                                                }
                                                                
                                                                JToken taskIdValue2 = errorsValue["taskId"];
                                                                if (taskIdValue2 != null && taskIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance2 = ((string)taskIdValue2);
                                                                    jobErrorDetailsInstance.TaskId = taskIdInstance2;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue = childTasksValue["customDetails"];
                                                        if (customDetailsValue != null && customDetailsValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName2 = ((string)customDetailsValue["instanceType"]);
                                                            if (typeName2 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue = customDetailsValue["jobTask"];
                                                                if (jobTaskValue != null && jobTaskValue.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance = new JobEntity();
                                                                    jobTaskDetailsInstance.JobTask = jobTaskInstance;
                                                                    
                                                                    JToken jobIdValue = jobTaskValue["jobId"];
                                                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance = ((string)jobIdValue);
                                                                        jobTaskInstance.JobId = jobIdInstance;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue = jobTaskValue["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue != null && jobFriendlyNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance = ((string)jobFriendlyNameValue);
                                                                        jobTaskInstance.JobFriendlyName = jobFriendlyNameInstance;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue2 = jobTaskValue["targetObjectId"];
                                                                    if (targetObjectIdValue2 != null && targetObjectIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance2 = ((string)targetObjectIdValue2);
                                                                        jobTaskInstance.TargetObjectId = targetObjectIdInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue2 = jobTaskValue["targetObjectName"];
                                                                    if (targetObjectNameValue2 != null && targetObjectNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance2 = ((string)targetObjectNameValue2);
                                                                        jobTaskInstance.TargetObjectName = targetObjectNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue2 = jobTaskValue["targetInstanceType"];
                                                                    if (targetInstanceTypeValue2 != null && targetInstanceTypeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance2 = ((string)targetInstanceTypeValue2);
                                                                        jobTaskInstance.TargetInstanceType = targetInstanceTypeInstance2;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue = jobTaskValue["jobScenarioName"];
                                                                    if (jobScenarioNameValue != null && jobScenarioNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance = ((string)jobScenarioNameValue);
                                                                        jobTaskInstance.JobScenarioName = jobScenarioNameInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue2 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                                    jobTaskDetailsInstance.Type = instanceTypeInstance2;
                                                                }
                                                                taskBaseInstance.CustomDetails = jobTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue != null && skippedReasonValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance = ((string)skippedReasonValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReason = skippedReasonInstance;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue != null && skippedReasonStringValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance = ((string)skippedReasonStringValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance;
                                                                }
                                                                
                                                                JToken jobTaskValue2 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue2 != null && jobTaskValue2.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance2 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance.JobTask = jobTaskInstance2;
                                                                    
                                                                    JToken jobIdValue2 = jobTaskValue2["jobId"];
                                                                    if (jobIdValue2 != null && jobIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance2 = ((string)jobIdValue2);
                                                                        jobTaskInstance2.JobId = jobIdInstance2;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue2 = jobTaskValue2["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue2 != null && jobFriendlyNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance2 = ((string)jobFriendlyNameValue2);
                                                                        jobTaskInstance2.JobFriendlyName = jobFriendlyNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue3 = jobTaskValue2["targetObjectId"];
                                                                    if (targetObjectIdValue3 != null && targetObjectIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance3 = ((string)targetObjectIdValue3);
                                                                        jobTaskInstance2.TargetObjectId = targetObjectIdInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue3 = jobTaskValue2["targetObjectName"];
                                                                    if (targetObjectNameValue3 != null && targetObjectNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance3 = ((string)targetObjectNameValue3);
                                                                        jobTaskInstance2.TargetObjectName = targetObjectNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue3 = jobTaskValue2["targetInstanceType"];
                                                                    if (targetInstanceTypeValue3 != null && targetInstanceTypeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance3 = ((string)targetInstanceTypeValue3);
                                                                        jobTaskInstance2.TargetInstanceType = targetInstanceTypeInstance3;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue2 = jobTaskValue2["jobScenarioName"];
                                                                    if (jobScenarioNameValue2 != null && jobScenarioNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance2 = ((string)jobScenarioNameValue2);
                                                                        jobTaskInstance2.JobScenarioName = jobScenarioNameInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue3 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                                    virtualMachineTaskDetailsInstance.Type = instanceTypeInstance3;
                                                                }
                                                                taskBaseInstance.CustomDetails = virtualMachineTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue2 = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue2 != null && skippedReasonValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance2 = ((string)skippedReasonValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReason = skippedReasonInstance2;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue2 = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue2 != null && skippedReasonStringValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance2 = ((string)skippedReasonStringValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance2;
                                                                }
                                                                
                                                                JToken jobTaskValue3 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue3 != null && jobTaskValue3.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance3 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance.JobTask = jobTaskInstance3;
                                                                    
                                                                    JToken jobIdValue3 = jobTaskValue3["jobId"];
                                                                    if (jobIdValue3 != null && jobIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance3 = ((string)jobIdValue3);
                                                                        jobTaskInstance3.JobId = jobIdInstance3;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue3 = jobTaskValue3["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue3 != null && jobFriendlyNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance3 = ((string)jobFriendlyNameValue3);
                                                                        jobTaskInstance3.JobFriendlyName = jobFriendlyNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue4 = jobTaskValue3["targetObjectId"];
                                                                    if (targetObjectIdValue4 != null && targetObjectIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance4 = ((string)targetObjectIdValue4);
                                                                        jobTaskInstance3.TargetObjectId = targetObjectIdInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue4 = jobTaskValue3["targetObjectName"];
                                                                    if (targetObjectNameValue4 != null && targetObjectNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance4 = ((string)targetObjectNameValue4);
                                                                        jobTaskInstance3.TargetObjectName = targetObjectNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue4 = jobTaskValue3["targetInstanceType"];
                                                                    if (targetInstanceTypeValue4 != null && targetInstanceTypeValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance4 = ((string)targetInstanceTypeValue4);
                                                                        jobTaskInstance3.TargetInstanceType = targetInstanceTypeInstance4;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue3 = jobTaskValue3["jobScenarioName"];
                                                                    if (jobScenarioNameValue3 != null && jobScenarioNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance3 = ((string)jobScenarioNameValue3);
                                                                        jobTaskInstance3.JobScenarioName = jobScenarioNameInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue4 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance.Type = instanceTypeInstance4;
                                                                }
                                                                taskBaseInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue2 = customDetailsValue["name"];
                                                                if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance2 = ((string)nameValue2);
                                                                    manualActionTaskDetailsInstance.Name = nameInstance2;
                                                                }
                                                                
                                                                JToken instructionsValue = customDetailsValue["instructions"];
                                                                if (instructionsValue != null && instructionsValue.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance = ((string)instructionsValue);
                                                                    manualActionTaskDetailsInstance.Instructions = instructionsInstance;
                                                                }
                                                                
                                                                JToken observationValue = customDetailsValue["observation"];
                                                                if (observationValue != null && observationValue.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance = ((string)observationValue);
                                                                    manualActionTaskDetailsInstance.Observation = observationInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue5 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                                    manualActionTaskDetailsInstance.Type = instanceTypeInstance5;
                                                                }
                                                                taskBaseInstance.CustomDetails = manualActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue3 = customDetailsValue["name"];
                                                                if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance3 = ((string)nameValue3);
                                                                    scriptActionTaskDetailsInstance.Name = nameInstance3;
                                                                }
                                                                
                                                                JToken pathValue = customDetailsValue["path"];
                                                                if (pathValue != null && pathValue.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance = ((string)pathValue);
                                                                    scriptActionTaskDetailsInstance.Path = pathInstance;
                                                                }
                                                                
                                                                JToken outputValue = customDetailsValue["output"];
                                                                if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance = ((string)outputValue);
                                                                    scriptActionTaskDetailsInstance.Output = outputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue != null && isPrimarySideScriptValue.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance = ((bool)isPrimarySideScriptValue);
                                                                    scriptActionTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue6 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                                    scriptActionTaskDetailsInstance.Type = instanceTypeInstance6;
                                                                }
                                                                taskBaseInstance.CustomDetails = scriptActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue = customDetailsValue["vmId"];
                                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance = ((string)vmIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.VmId = vmIdInstance;
                                                                }
                                                                
                                                                JToken nicIdValue = customDetailsValue["nicId"];
                                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance = ((string)nicIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.NicId = nicIdInstance;
                                                                }
                                                                
                                                                JToken nameValue4 = customDetailsValue["name"];
                                                                if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance4 = ((string)nameValue4);
                                                                    vmNicUpdatesTaskDetailsInstance.Name = nameInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue7 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue7 != null && instanceTypeValue7.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance7 = ((string)instanceTypeValue7);
                                                                    vmNicUpdatesTaskDetailsInstance.Type = instanceTypeInstance7;
                                                                }
                                                                taskBaseInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray = customDetailsValue["vmDetails"];
                                                                if (vmDetailsArray != null && vmDetailsArray.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue in ((JArray)vmDetailsArray))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance.VmDetails.Add(inconsistentVmDetailsInstance);
                                                                        
                                                                        JToken vmNameValue = vmDetailsValue["vmName"];
                                                                        if (vmNameValue != null && vmNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance = ((string)vmNameValue);
                                                                            inconsistentVmDetailsInstance.VmName = vmNameInstance;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue = vmDetailsValue["cloudName"];
                                                                        if (cloudNameValue != null && cloudNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance = ((string)cloudNameValue);
                                                                            inconsistentVmDetailsInstance.CloudName = cloudNameInstance;
                                                                        }
                                                                        
                                                                        JToken detailsArray = vmDetailsValue["details"];
                                                                        if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue in ((JArray)detailsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.Details.Add(((string)detailsValue));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray = vmDetailsValue["errorIds"];
                                                                        if (errorIdsArray != null && errorIdsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue in ((JArray)errorIdsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.ErrorIds.Add(((string)errorIdsValue));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue8 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue8 != null && instanceTypeValue8.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance8 = ((string)instanceTypeValue8);
                                                                    consistencyCheckTaskDetailsInstance.Type = instanceTypeInstance8;
                                                                }
                                                                taskBaseInstance.CustomDetails = consistencyCheckTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue5 = customDetailsValue["name"];
                                                                if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance5 = ((string)nameValue5);
                                                                    automationRunbookTaskDetailsInstance.Name = nameInstance5;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue = customDetailsValue["cloudServiceName"];
                                                                if (cloudServiceNameValue != null && cloudServiceNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance = ((string)cloudServiceNameValue);
                                                                    automationRunbookTaskDetailsInstance.CloudServiceName = cloudServiceNameInstance;
                                                                }
                                                                
                                                                JToken subscriptionIdValue = customDetailsValue["subscriptionId"];
                                                                if (subscriptionIdValue != null && subscriptionIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance = ((string)subscriptionIdValue);
                                                                    automationRunbookTaskDetailsInstance.SubscriptionId = subscriptionIdInstance;
                                                                }
                                                                
                                                                JToken accountNameValue = customDetailsValue["accountName"];
                                                                if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance = ((string)accountNameValue);
                                                                    automationRunbookTaskDetailsInstance.AccountName = accountNameInstance;
                                                                }
                                                                
                                                                JToken runbookIdValue = customDetailsValue["runbookId"];
                                                                if (runbookIdValue != null && runbookIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance = ((string)runbookIdValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookId = runbookIdInstance;
                                                                }
                                                                
                                                                JToken runbookNameValue = customDetailsValue["runbookName"];
                                                                if (runbookNameValue != null && runbookNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance = ((string)runbookNameValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookName = runbookNameInstance;
                                                                }
                                                                
                                                                JToken jobIdValue4 = customDetailsValue["jobId"];
                                                                if (jobIdValue4 != null && jobIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance4 = ((string)jobIdValue4);
                                                                    automationRunbookTaskDetailsInstance.JobId = jobIdInstance4;
                                                                }
                                                                
                                                                JToken jobOutputValue = customDetailsValue["jobOutput"];
                                                                if (jobOutputValue != null && jobOutputValue.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance = ((string)jobOutputValue);
                                                                    automationRunbookTaskDetailsInstance.JobOutput = jobOutputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue2 = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue2 != null && isPrimarySideScriptValue2.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance2 = ((bool)isPrimarySideScriptValue2);
                                                                    automationRunbookTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue9 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue9 != null && instanceTypeValue9.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance9 = ((string)instanceTypeValue9);
                                                                    automationRunbookTaskDetailsInstance.Type = instanceTypeInstance9;
                                                                }
                                                                taskBaseInstance.CustomDetails = automationRunbookTaskDetailsInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = inlineWorkflowTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanGroupTaskDetails")
                                            {
                                                RecoveryPlanGroupTaskDetails recoveryPlanGroupTaskDetailsInstance = new RecoveryPlanGroupTaskDetails();
                                                
                                                JToken nameValue6 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                {
                                                    string nameInstance6 = ((string)nameValue6);
                                                    recoveryPlanGroupTaskDetailsInstance.Name = nameInstance6;
                                                }
                                                
                                                JToken groupIdValue = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue != null && groupIdValue.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance = ((string)groupIdValue);
                                                    recoveryPlanGroupTaskDetailsInstance.GroupId = groupIdInstance;
                                                }
                                                
                                                JToken rpGroupTypeValue = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue != null && rpGroupTypeValue.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance = ((string)rpGroupTypeValue);
                                                    recoveryPlanGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance;
                                                }
                                                
                                                JToken instanceTypeValue10 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue10 != null && instanceTypeValue10.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance10 = ((string)instanceTypeValue10);
                                                    recoveryPlanGroupTaskDetailsInstance.Type = instanceTypeInstance10;
                                                }
                                                
                                                JToken childTasksArray2 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray2 != null && childTasksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue2 in ((JArray)childTasksArray2))
                                                    {
                                                        AsrTaskBase taskBaseInstance2 = new AsrTaskBase();
                                                        recoveryPlanGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance2);
                                                        
                                                        JToken taskIdValue3 = childTasksValue2["taskId"];
                                                        if (taskIdValue3 != null && taskIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance3 = ((string)taskIdValue3);
                                                            taskBaseInstance2.ID = taskIdInstance3;
                                                        }
                                                        
                                                        JToken nameValue7 = childTasksValue2["name"];
                                                        if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance7 = ((string)nameValue7);
                                                            taskBaseInstance2.Name = nameInstance7;
                                                        }
                                                        
                                                        JToken startTimeValue3 = childTasksValue2["startTime"];
                                                        if (startTimeValue3 != null && startTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance3 = ((DateTime)startTimeValue3);
                                                            taskBaseInstance2.StartTime = startTimeInstance3;
                                                        }
                                                        
                                                        JToken endTimeValue3 = childTasksValue2["endTime"];
                                                        if (endTimeValue3 != null && endTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance3 = ((DateTime)endTimeValue3);
                                                            taskBaseInstance2.EndTime = endTimeInstance3;
                                                        }
                                                        
                                                        JToken allowedActionsArray3 = childTasksValue2["allowedActions"];
                                                        if (allowedActionsArray3 != null && allowedActionsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue3 in ((JArray)allowedActionsArray3))
                                                            {
                                                                taskBaseInstance2.Actions.Add(((string)allowedActionsValue3));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue3 = childTasksValue2["friendlyName"];
                                                        if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                                            taskBaseInstance2.TaskFriendlyName = friendlyNameInstance3;
                                                        }
                                                        
                                                        JToken stateValue3 = childTasksValue2["state"];
                                                        if (stateValue3 != null && stateValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance3 = ((string)stateValue3);
                                                            taskBaseInstance2.State = stateInstance3;
                                                        }
                                                        
                                                        JToken stateDescriptionValue3 = childTasksValue2["stateDescription"];
                                                        if (stateDescriptionValue3 != null && stateDescriptionValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance3 = ((string)stateDescriptionValue3);
                                                            taskBaseInstance2.StateDescription = stateDescriptionInstance3;
                                                        }
                                                        
                                                        JToken taskTypeValue2 = childTasksValue2["taskType"];
                                                        if (taskTypeValue2 != null && taskTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance2 = ((string)taskTypeValue2);
                                                            taskBaseInstance2.TaskType = taskTypeInstance2;
                                                        }
                                                        
                                                        JToken errorsArray2 = childTasksValue2["errors"];
                                                        if (errorsArray2 != null && errorsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue2 in ((JArray)errorsArray2))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance2 = new JobErrorDetails();
                                                                taskBaseInstance2.Errors.Add(jobErrorDetailsInstance2);
                                                                
                                                                JToken serviceErrorDetailsValue2 = errorsValue2["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue2 != null && serviceErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance2 = new ServiceError();
                                                                    jobErrorDetailsInstance2.ServiceErrorDetails = serviceErrorDetailsInstance2;
                                                                    
                                                                    JToken codeValue2 = serviceErrorDetailsValue2["code"];
                                                                    if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance2 = ((string)codeValue2);
                                                                        serviceErrorDetailsInstance2.Code = codeInstance2;
                                                                    }
                                                                    
                                                                    JToken messageValue2 = serviceErrorDetailsValue2["message"];
                                                                    if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance2 = ((string)messageValue2);
                                                                        serviceErrorDetailsInstance2.Message = messageInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue3 = serviceErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue3 != null && possibleCausesValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance3 = ((string)possibleCausesValue3);
                                                                        serviceErrorDetailsInstance2.PossibleCauses = possibleCausesInstance3;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue3 = serviceErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue3 != null && recommendedActionValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance3 = ((string)recommendedActionValue3);
                                                                        serviceErrorDetailsInstance2.RecommendedAction = recommendedActionInstance3;
                                                                    }
                                                                    
                                                                    JToken activityIdValue3 = serviceErrorDetailsValue2["activityId"];
                                                                    if (activityIdValue3 != null && activityIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance3 = ((string)activityIdValue3);
                                                                        serviceErrorDetailsInstance2.ActivityId = activityIdInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue2 = errorsValue2["providerErrorDetails"];
                                                                if (providerErrorDetailsValue2 != null && providerErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance2 = new ProviderError();
                                                                    jobErrorDetailsInstance2.ProviderErrorDetails = providerErrorDetailsInstance2;
                                                                    
                                                                    JToken errorCodeValue2 = providerErrorDetailsValue2["errorCode"];
                                                                    if (errorCodeValue2 != null && errorCodeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance2 = ((int)errorCodeValue2);
                                                                        providerErrorDetailsInstance2.ErrorCode = errorCodeInstance2;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue2 = providerErrorDetailsValue2["errorMessage"];
                                                                    if (errorMessageValue2 != null && errorMessageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance2 = ((string)errorMessageValue2);
                                                                        providerErrorDetailsInstance2.ErrorMessage = errorMessageInstance2;
                                                                    }
                                                                    
                                                                    JToken errorIdValue2 = providerErrorDetailsValue2["errorId"];
                                                                    if (errorIdValue2 != null && errorIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance2 = ((string)errorIdValue2);
                                                                        providerErrorDetailsInstance2.ErrorId = errorIdInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue4 = providerErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue4 != null && possibleCausesValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance4 = ((string)possibleCausesValue4);
                                                                        providerErrorDetailsInstance2.PossibleCauses = possibleCausesInstance4;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue4 = providerErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue4 != null && recommendedActionValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance4 = ((string)recommendedActionValue4);
                                                                        providerErrorDetailsInstance2.RecommendedAction = recommendedActionInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue2 = errorsValue2["errorLevel"];
                                                                if (errorLevelValue2 != null && errorLevelValue2.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance2 = ((string)errorLevelValue2);
                                                                    jobErrorDetailsInstance2.ErrorLevel = errorLevelInstance2;
                                                                }
                                                                
                                                                JToken creationTimeValue2 = errorsValue2["creationTime"];
                                                                if (creationTimeValue2 != null && creationTimeValue2.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance2 = ((DateTime)creationTimeValue2);
                                                                    jobErrorDetailsInstance2.CreationTime = creationTimeInstance2;
                                                                }
                                                                
                                                                JToken taskIdValue4 = errorsValue2["taskId"];
                                                                if (taskIdValue4 != null && taskIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance4 = ((string)taskIdValue4);
                                                                    jobErrorDetailsInstance2.TaskId = taskIdInstance4;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue2 = childTasksValue2["customDetails"];
                                                        if (customDetailsValue2 != null && customDetailsValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName3 = ((string)customDetailsValue2["instanceType"]);
                                                            if (typeName3 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance2 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue4 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue4 != null && jobTaskValue4.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance4 = new JobEntity();
                                                                    jobTaskDetailsInstance2.JobTask = jobTaskInstance4;
                                                                    
                                                                    JToken jobIdValue5 = jobTaskValue4["jobId"];
                                                                    if (jobIdValue5 != null && jobIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance5 = ((string)jobIdValue5);
                                                                        jobTaskInstance4.JobId = jobIdInstance5;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue4 = jobTaskValue4["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue4 != null && jobFriendlyNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance4 = ((string)jobFriendlyNameValue4);
                                                                        jobTaskInstance4.JobFriendlyName = jobFriendlyNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue5 = jobTaskValue4["targetObjectId"];
                                                                    if (targetObjectIdValue5 != null && targetObjectIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance5 = ((string)targetObjectIdValue5);
                                                                        jobTaskInstance4.TargetObjectId = targetObjectIdInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue5 = jobTaskValue4["targetObjectName"];
                                                                    if (targetObjectNameValue5 != null && targetObjectNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance5 = ((string)targetObjectNameValue5);
                                                                        jobTaskInstance4.TargetObjectName = targetObjectNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue5 = jobTaskValue4["targetInstanceType"];
                                                                    if (targetInstanceTypeValue5 != null && targetInstanceTypeValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance5 = ((string)targetInstanceTypeValue5);
                                                                        jobTaskInstance4.TargetInstanceType = targetInstanceTypeInstance5;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue4 = jobTaskValue4["jobScenarioName"];
                                                                    if (jobScenarioNameValue4 != null && jobScenarioNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance4 = ((string)jobScenarioNameValue4);
                                                                        jobTaskInstance4.JobScenarioName = jobScenarioNameInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue11 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue11 != null && instanceTypeValue11.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance11 = ((string)instanceTypeValue11);
                                                                    jobTaskDetailsInstance2.Type = instanceTypeInstance11;
                                                                }
                                                                taskBaseInstance2.CustomDetails = jobTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance2 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue3 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue3 != null && skippedReasonValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance3 = ((string)skippedReasonValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReason = skippedReasonInstance3;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue3 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue3 != null && skippedReasonStringValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance3 = ((string)skippedReasonStringValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance3;
                                                                }
                                                                
                                                                JToken jobTaskValue5 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue5 != null && jobTaskValue5.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance5 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance2.JobTask = jobTaskInstance5;
                                                                    
                                                                    JToken jobIdValue6 = jobTaskValue5["jobId"];
                                                                    if (jobIdValue6 != null && jobIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance6 = ((string)jobIdValue6);
                                                                        jobTaskInstance5.JobId = jobIdInstance6;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue5 = jobTaskValue5["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue5 != null && jobFriendlyNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance5 = ((string)jobFriendlyNameValue5);
                                                                        jobTaskInstance5.JobFriendlyName = jobFriendlyNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue6 = jobTaskValue5["targetObjectId"];
                                                                    if (targetObjectIdValue6 != null && targetObjectIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance6 = ((string)targetObjectIdValue6);
                                                                        jobTaskInstance5.TargetObjectId = targetObjectIdInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue6 = jobTaskValue5["targetObjectName"];
                                                                    if (targetObjectNameValue6 != null && targetObjectNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance6 = ((string)targetObjectNameValue6);
                                                                        jobTaskInstance5.TargetObjectName = targetObjectNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue6 = jobTaskValue5["targetInstanceType"];
                                                                    if (targetInstanceTypeValue6 != null && targetInstanceTypeValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance6 = ((string)targetInstanceTypeValue6);
                                                                        jobTaskInstance5.TargetInstanceType = targetInstanceTypeInstance6;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue5 = jobTaskValue5["jobScenarioName"];
                                                                    if (jobScenarioNameValue5 != null && jobScenarioNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance5 = ((string)jobScenarioNameValue5);
                                                                        jobTaskInstance5.JobScenarioName = jobScenarioNameInstance5;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue12 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue12 != null && instanceTypeValue12.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance12 = ((string)instanceTypeValue12);
                                                                    virtualMachineTaskDetailsInstance2.Type = instanceTypeInstance12;
                                                                }
                                                                taskBaseInstance2.CustomDetails = virtualMachineTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance2 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue4 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue4 != null && skippedReasonValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance4 = ((string)skippedReasonValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReason = skippedReasonInstance4;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue4 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue4 != null && skippedReasonStringValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance4 = ((string)skippedReasonStringValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance4;
                                                                }
                                                                
                                                                JToken jobTaskValue6 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue6 != null && jobTaskValue6.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance6 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance2.JobTask = jobTaskInstance6;
                                                                    
                                                                    JToken jobIdValue7 = jobTaskValue6["jobId"];
                                                                    if (jobIdValue7 != null && jobIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance7 = ((string)jobIdValue7);
                                                                        jobTaskInstance6.JobId = jobIdInstance7;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue6 = jobTaskValue6["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue6 != null && jobFriendlyNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance6 = ((string)jobFriendlyNameValue6);
                                                                        jobTaskInstance6.JobFriendlyName = jobFriendlyNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue7 = jobTaskValue6["targetObjectId"];
                                                                    if (targetObjectIdValue7 != null && targetObjectIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance7 = ((string)targetObjectIdValue7);
                                                                        jobTaskInstance6.TargetObjectId = targetObjectIdInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue7 = jobTaskValue6["targetObjectName"];
                                                                    if (targetObjectNameValue7 != null && targetObjectNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance7 = ((string)targetObjectNameValue7);
                                                                        jobTaskInstance6.TargetObjectName = targetObjectNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue7 = jobTaskValue6["targetInstanceType"];
                                                                    if (targetInstanceTypeValue7 != null && targetInstanceTypeValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance7 = ((string)targetInstanceTypeValue7);
                                                                        jobTaskInstance6.TargetInstanceType = targetInstanceTypeInstance7;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue6 = jobTaskValue6["jobScenarioName"];
                                                                    if (jobScenarioNameValue6 != null && jobScenarioNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance6 = ((string)jobScenarioNameValue6);
                                                                        jobTaskInstance6.JobScenarioName = jobScenarioNameInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue13 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue13 != null && instanceTypeValue13.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance13 = ((string)instanceTypeValue13);
                                                                    fabricReplicationGroupTaskDetailsInstance2.Type = instanceTypeInstance13;
                                                                }
                                                                taskBaseInstance2.CustomDetails = fabricReplicationGroupTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance2 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue8 = customDetailsValue2["name"];
                                                                if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance8 = ((string)nameValue8);
                                                                    manualActionTaskDetailsInstance2.Name = nameInstance8;
                                                                }
                                                                
                                                                JToken instructionsValue2 = customDetailsValue2["instructions"];
                                                                if (instructionsValue2 != null && instructionsValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance2 = ((string)instructionsValue2);
                                                                    manualActionTaskDetailsInstance2.Instructions = instructionsInstance2;
                                                                }
                                                                
                                                                JToken observationValue2 = customDetailsValue2["observation"];
                                                                if (observationValue2 != null && observationValue2.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance2 = ((string)observationValue2);
                                                                    manualActionTaskDetailsInstance2.Observation = observationInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue14 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue14 != null && instanceTypeValue14.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance14 = ((string)instanceTypeValue14);
                                                                    manualActionTaskDetailsInstance2.Type = instanceTypeInstance14;
                                                                }
                                                                taskBaseInstance2.CustomDetails = manualActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance2 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue9 = customDetailsValue2["name"];
                                                                if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance9 = ((string)nameValue9);
                                                                    scriptActionTaskDetailsInstance2.Name = nameInstance9;
                                                                }
                                                                
                                                                JToken pathValue2 = customDetailsValue2["path"];
                                                                if (pathValue2 != null && pathValue2.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance2 = ((string)pathValue2);
                                                                    scriptActionTaskDetailsInstance2.Path = pathInstance2;
                                                                }
                                                                
                                                                JToken outputValue2 = customDetailsValue2["output"];
                                                                if (outputValue2 != null && outputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance2 = ((string)outputValue2);
                                                                    scriptActionTaskDetailsInstance2.Output = outputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue3 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue3 != null && isPrimarySideScriptValue3.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance3 = ((bool)isPrimarySideScriptValue3);
                                                                    scriptActionTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue15 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue15 != null && instanceTypeValue15.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance15 = ((string)instanceTypeValue15);
                                                                    scriptActionTaskDetailsInstance2.Type = instanceTypeInstance15;
                                                                }
                                                                taskBaseInstance2.CustomDetails = scriptActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance2 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue2 = customDetailsValue2["vmId"];
                                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.VmId = vmIdInstance2;
                                                                }
                                                                
                                                                JToken nicIdValue2 = customDetailsValue2["nicId"];
                                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.NicId = nicIdInstance2;
                                                                }
                                                                
                                                                JToken nameValue10 = customDetailsValue2["name"];
                                                                if (nameValue10 != null && nameValue10.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance10 = ((string)nameValue10);
                                                                    vmNicUpdatesTaskDetailsInstance2.Name = nameInstance10;
                                                                }
                                                                
                                                                JToken instanceTypeValue16 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue16 != null && instanceTypeValue16.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance16 = ((string)instanceTypeValue16);
                                                                    vmNicUpdatesTaskDetailsInstance2.Type = instanceTypeInstance16;
                                                                }
                                                                taskBaseInstance2.CustomDetails = vmNicUpdatesTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance2 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray2 = customDetailsValue2["vmDetails"];
                                                                if (vmDetailsArray2 != null && vmDetailsArray2.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue2 in ((JArray)vmDetailsArray2))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance2 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance2.VmDetails.Add(inconsistentVmDetailsInstance2);
                                                                        
                                                                        JToken vmNameValue2 = vmDetailsValue2["vmName"];
                                                                        if (vmNameValue2 != null && vmNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance2 = ((string)vmNameValue2);
                                                                            inconsistentVmDetailsInstance2.VmName = vmNameInstance2;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue2 = vmDetailsValue2["cloudName"];
                                                                        if (cloudNameValue2 != null && cloudNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance2 = ((string)cloudNameValue2);
                                                                            inconsistentVmDetailsInstance2.CloudName = cloudNameInstance2;
                                                                        }
                                                                        
                                                                        JToken detailsArray2 = vmDetailsValue2["details"];
                                                                        if (detailsArray2 != null && detailsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue2 in ((JArray)detailsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.Details.Add(((string)detailsValue2));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray2 = vmDetailsValue2["errorIds"];
                                                                        if (errorIdsArray2 != null && errorIdsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue2 in ((JArray)errorIdsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.ErrorIds.Add(((string)errorIdsValue2));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue17 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue17 != null && instanceTypeValue17.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance17 = ((string)instanceTypeValue17);
                                                                    consistencyCheckTaskDetailsInstance2.Type = instanceTypeInstance17;
                                                                }
                                                                taskBaseInstance2.CustomDetails = consistencyCheckTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance2 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue11 = customDetailsValue2["name"];
                                                                if (nameValue11 != null && nameValue11.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance11 = ((string)nameValue11);
                                                                    automationRunbookTaskDetailsInstance2.Name = nameInstance11;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue2 = customDetailsValue2["cloudServiceName"];
                                                                if (cloudServiceNameValue2 != null && cloudServiceNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance2 = ((string)cloudServiceNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.CloudServiceName = cloudServiceNameInstance2;
                                                                }
                                                                
                                                                JToken subscriptionIdValue2 = customDetailsValue2["subscriptionId"];
                                                                if (subscriptionIdValue2 != null && subscriptionIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance2 = ((string)subscriptionIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.SubscriptionId = subscriptionIdInstance2;
                                                                }
                                                                
                                                                JToken accountNameValue2 = customDetailsValue2["accountName"];
                                                                if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance2 = ((string)accountNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.AccountName = accountNameInstance2;
                                                                }
                                                                
                                                                JToken runbookIdValue2 = customDetailsValue2["runbookId"];
                                                                if (runbookIdValue2 != null && runbookIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance2 = ((string)runbookIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookId = runbookIdInstance2;
                                                                }
                                                                
                                                                JToken runbookNameValue2 = customDetailsValue2["runbookName"];
                                                                if (runbookNameValue2 != null && runbookNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance2 = ((string)runbookNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookName = runbookNameInstance2;
                                                                }
                                                                
                                                                JToken jobIdValue8 = customDetailsValue2["jobId"];
                                                                if (jobIdValue8 != null && jobIdValue8.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance8 = ((string)jobIdValue8);
                                                                    automationRunbookTaskDetailsInstance2.JobId = jobIdInstance8;
                                                                }
                                                                
                                                                JToken jobOutputValue2 = customDetailsValue2["jobOutput"];
                                                                if (jobOutputValue2 != null && jobOutputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance2 = ((string)jobOutputValue2);
                                                                    automationRunbookTaskDetailsInstance2.JobOutput = jobOutputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue4 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue4 != null && isPrimarySideScriptValue4.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance4 = ((bool)isPrimarySideScriptValue4);
                                                                    automationRunbookTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue18 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue18 != null && instanceTypeValue18.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance18 = ((string)instanceTypeValue18);
                                                                    automationRunbookTaskDetailsInstance2.Type = instanceTypeInstance18;
                                                                }
                                                                taskBaseInstance2.CustomDetails = automationRunbookTaskDetailsInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanGroupTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanShutdownGroupTaskDetails")
                                            {
                                                RecoveryPlanShutdownGroupTaskDetails recoveryPlanShutdownGroupTaskDetailsInstance = new RecoveryPlanShutdownGroupTaskDetails();
                                                
                                                JToken nameValue12 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue12 != null && nameValue12.Type != JTokenType.Null)
                                                {
                                                    string nameInstance12 = ((string)nameValue12);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Name = nameInstance12;
                                                }
                                                
                                                JToken groupIdValue2 = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue2 != null && groupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance2 = ((string)groupIdValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.GroupId = groupIdInstance2;
                                                }
                                                
                                                JToken rpGroupTypeValue2 = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue2 != null && rpGroupTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance2 = ((string)rpGroupTypeValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance2;
                                                }
                                                
                                                JToken instanceTypeValue19 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue19 != null && instanceTypeValue19.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance19 = ((string)instanceTypeValue19);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Type = instanceTypeInstance19;
                                                }
                                                
                                                JToken childTasksArray3 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray3 != null && childTasksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue3 in ((JArray)childTasksArray3))
                                                    {
                                                        AsrTaskBase taskBaseInstance3 = new AsrTaskBase();
                                                        recoveryPlanShutdownGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance3);
                                                        
                                                        JToken taskIdValue5 = childTasksValue3["taskId"];
                                                        if (taskIdValue5 != null && taskIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance5 = ((string)taskIdValue5);
                                                            taskBaseInstance3.ID = taskIdInstance5;
                                                        }
                                                        
                                                        JToken nameValue13 = childTasksValue3["name"];
                                                        if (nameValue13 != null && nameValue13.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance13 = ((string)nameValue13);
                                                            taskBaseInstance3.Name = nameInstance13;
                                                        }
                                                        
                                                        JToken startTimeValue4 = childTasksValue3["startTime"];
                                                        if (startTimeValue4 != null && startTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance4 = ((DateTime)startTimeValue4);
                                                            taskBaseInstance3.StartTime = startTimeInstance4;
                                                        }
                                                        
                                                        JToken endTimeValue4 = childTasksValue3["endTime"];
                                                        if (endTimeValue4 != null && endTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance4 = ((DateTime)endTimeValue4);
                                                            taskBaseInstance3.EndTime = endTimeInstance4;
                                                        }
                                                        
                                                        JToken allowedActionsArray4 = childTasksValue3["allowedActions"];
                                                        if (allowedActionsArray4 != null && allowedActionsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue4 in ((JArray)allowedActionsArray4))
                                                            {
                                                                taskBaseInstance3.Actions.Add(((string)allowedActionsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue4 = childTasksValue3["friendlyName"];
                                                        if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                                            taskBaseInstance3.TaskFriendlyName = friendlyNameInstance4;
                                                        }
                                                        
                                                        JToken stateValue4 = childTasksValue3["state"];
                                                        if (stateValue4 != null && stateValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance4 = ((string)stateValue4);
                                                            taskBaseInstance3.State = stateInstance4;
                                                        }
                                                        
                                                        JToken stateDescriptionValue4 = childTasksValue3["stateDescription"];
                                                        if (stateDescriptionValue4 != null && stateDescriptionValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance4 = ((string)stateDescriptionValue4);
                                                            taskBaseInstance3.StateDescription = stateDescriptionInstance4;
                                                        }
                                                        
                                                        JToken taskTypeValue3 = childTasksValue3["taskType"];
                                                        if (taskTypeValue3 != null && taskTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance3 = ((string)taskTypeValue3);
                                                            taskBaseInstance3.TaskType = taskTypeInstance3;
                                                        }
                                                        
                                                        JToken errorsArray3 = childTasksValue3["errors"];
                                                        if (errorsArray3 != null && errorsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue3 in ((JArray)errorsArray3))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance3 = new JobErrorDetails();
                                                                taskBaseInstance3.Errors.Add(jobErrorDetailsInstance3);
                                                                
                                                                JToken serviceErrorDetailsValue3 = errorsValue3["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue3 != null && serviceErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance3 = new ServiceError();
                                                                    jobErrorDetailsInstance3.ServiceErrorDetails = serviceErrorDetailsInstance3;
                                                                    
                                                                    JToken codeValue3 = serviceErrorDetailsValue3["code"];
                                                                    if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance3 = ((string)codeValue3);
                                                                        serviceErrorDetailsInstance3.Code = codeInstance3;
                                                                    }
                                                                    
                                                                    JToken messageValue3 = serviceErrorDetailsValue3["message"];
                                                                    if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance3 = ((string)messageValue3);
                                                                        serviceErrorDetailsInstance3.Message = messageInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue5 = serviceErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue5 != null && possibleCausesValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance5 = ((string)possibleCausesValue5);
                                                                        serviceErrorDetailsInstance3.PossibleCauses = possibleCausesInstance5;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue5 = serviceErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue5 != null && recommendedActionValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance5 = ((string)recommendedActionValue5);
                                                                        serviceErrorDetailsInstance3.RecommendedAction = recommendedActionInstance5;
                                                                    }
                                                                    
                                                                    JToken activityIdValue4 = serviceErrorDetailsValue3["activityId"];
                                                                    if (activityIdValue4 != null && activityIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance4 = ((string)activityIdValue4);
                                                                        serviceErrorDetailsInstance3.ActivityId = activityIdInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue3 = errorsValue3["providerErrorDetails"];
                                                                if (providerErrorDetailsValue3 != null && providerErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance3 = new ProviderError();
                                                                    jobErrorDetailsInstance3.ProviderErrorDetails = providerErrorDetailsInstance3;
                                                                    
                                                                    JToken errorCodeValue3 = providerErrorDetailsValue3["errorCode"];
                                                                    if (errorCodeValue3 != null && errorCodeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance3 = ((int)errorCodeValue3);
                                                                        providerErrorDetailsInstance3.ErrorCode = errorCodeInstance3;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue3 = providerErrorDetailsValue3["errorMessage"];
                                                                    if (errorMessageValue3 != null && errorMessageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance3 = ((string)errorMessageValue3);
                                                                        providerErrorDetailsInstance3.ErrorMessage = errorMessageInstance3;
                                                                    }
                                                                    
                                                                    JToken errorIdValue3 = providerErrorDetailsValue3["errorId"];
                                                                    if (errorIdValue3 != null && errorIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance3 = ((string)errorIdValue3);
                                                                        providerErrorDetailsInstance3.ErrorId = errorIdInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue6 = providerErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue6 != null && possibleCausesValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance6 = ((string)possibleCausesValue6);
                                                                        providerErrorDetailsInstance3.PossibleCauses = possibleCausesInstance6;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue6 = providerErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue6 != null && recommendedActionValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance6 = ((string)recommendedActionValue6);
                                                                        providerErrorDetailsInstance3.RecommendedAction = recommendedActionInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue3 = errorsValue3["errorLevel"];
                                                                if (errorLevelValue3 != null && errorLevelValue3.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance3 = ((string)errorLevelValue3);
                                                                    jobErrorDetailsInstance3.ErrorLevel = errorLevelInstance3;
                                                                }
                                                                
                                                                JToken creationTimeValue3 = errorsValue3["creationTime"];
                                                                if (creationTimeValue3 != null && creationTimeValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance3 = ((DateTime)creationTimeValue3);
                                                                    jobErrorDetailsInstance3.CreationTime = creationTimeInstance3;
                                                                }
                                                                
                                                                JToken taskIdValue6 = errorsValue3["taskId"];
                                                                if (taskIdValue6 != null && taskIdValue6.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance6 = ((string)taskIdValue6);
                                                                    jobErrorDetailsInstance3.TaskId = taskIdInstance6;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue3 = childTasksValue3["customDetails"];
                                                        if (customDetailsValue3 != null && customDetailsValue3.Type != JTokenType.Null)
                                                        {
                                                            string typeName4 = ((string)customDetailsValue3["instanceType"]);
                                                            if (typeName4 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance3 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue7 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue7 != null && jobTaskValue7.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance7 = new JobEntity();
                                                                    jobTaskDetailsInstance3.JobTask = jobTaskInstance7;
                                                                    
                                                                    JToken jobIdValue9 = jobTaskValue7["jobId"];
                                                                    if (jobIdValue9 != null && jobIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance9 = ((string)jobIdValue9);
                                                                        jobTaskInstance7.JobId = jobIdInstance9;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue7 = jobTaskValue7["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue7 != null && jobFriendlyNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance7 = ((string)jobFriendlyNameValue7);
                                                                        jobTaskInstance7.JobFriendlyName = jobFriendlyNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue8 = jobTaskValue7["targetObjectId"];
                                                                    if (targetObjectIdValue8 != null && targetObjectIdValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance8 = ((string)targetObjectIdValue8);
                                                                        jobTaskInstance7.TargetObjectId = targetObjectIdInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue8 = jobTaskValue7["targetObjectName"];
                                                                    if (targetObjectNameValue8 != null && targetObjectNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance8 = ((string)targetObjectNameValue8);
                                                                        jobTaskInstance7.TargetObjectName = targetObjectNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue8 = jobTaskValue7["targetInstanceType"];
                                                                    if (targetInstanceTypeValue8 != null && targetInstanceTypeValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance8 = ((string)targetInstanceTypeValue8);
                                                                        jobTaskInstance7.TargetInstanceType = targetInstanceTypeInstance8;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue7 = jobTaskValue7["jobScenarioName"];
                                                                    if (jobScenarioNameValue7 != null && jobScenarioNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance7 = ((string)jobScenarioNameValue7);
                                                                        jobTaskInstance7.JobScenarioName = jobScenarioNameInstance7;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue20 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue20 != null && instanceTypeValue20.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance20 = ((string)instanceTypeValue20);
                                                                    jobTaskDetailsInstance3.Type = instanceTypeInstance20;
                                                                }
                                                                taskBaseInstance3.CustomDetails = jobTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance3 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue5 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue5 != null && skippedReasonValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance5 = ((string)skippedReasonValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReason = skippedReasonInstance5;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue5 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue5 != null && skippedReasonStringValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance5 = ((string)skippedReasonStringValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance5;
                                                                }
                                                                
                                                                JToken jobTaskValue8 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue8 != null && jobTaskValue8.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance8 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance3.JobTask = jobTaskInstance8;
                                                                    
                                                                    JToken jobIdValue10 = jobTaskValue8["jobId"];
                                                                    if (jobIdValue10 != null && jobIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance10 = ((string)jobIdValue10);
                                                                        jobTaskInstance8.JobId = jobIdInstance10;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue8 = jobTaskValue8["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue8 != null && jobFriendlyNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance8 = ((string)jobFriendlyNameValue8);
                                                                        jobTaskInstance8.JobFriendlyName = jobFriendlyNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue9 = jobTaskValue8["targetObjectId"];
                                                                    if (targetObjectIdValue9 != null && targetObjectIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance9 = ((string)targetObjectIdValue9);
                                                                        jobTaskInstance8.TargetObjectId = targetObjectIdInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue9 = jobTaskValue8["targetObjectName"];
                                                                    if (targetObjectNameValue9 != null && targetObjectNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance9 = ((string)targetObjectNameValue9);
                                                                        jobTaskInstance8.TargetObjectName = targetObjectNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue9 = jobTaskValue8["targetInstanceType"];
                                                                    if (targetInstanceTypeValue9 != null && targetInstanceTypeValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance9 = ((string)targetInstanceTypeValue9);
                                                                        jobTaskInstance8.TargetInstanceType = targetInstanceTypeInstance9;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue8 = jobTaskValue8["jobScenarioName"];
                                                                    if (jobScenarioNameValue8 != null && jobScenarioNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance8 = ((string)jobScenarioNameValue8);
                                                                        jobTaskInstance8.JobScenarioName = jobScenarioNameInstance8;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue21 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue21 != null && instanceTypeValue21.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance21 = ((string)instanceTypeValue21);
                                                                    virtualMachineTaskDetailsInstance3.Type = instanceTypeInstance21;
                                                                }
                                                                taskBaseInstance3.CustomDetails = virtualMachineTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance3 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue6 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue6 != null && skippedReasonValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance6 = ((string)skippedReasonValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReason = skippedReasonInstance6;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue6 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue6 != null && skippedReasonStringValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance6 = ((string)skippedReasonStringValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance6;
                                                                }
                                                                
                                                                JToken jobTaskValue9 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue9 != null && jobTaskValue9.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance9 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance3.JobTask = jobTaskInstance9;
                                                                    
                                                                    JToken jobIdValue11 = jobTaskValue9["jobId"];
                                                                    if (jobIdValue11 != null && jobIdValue11.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance11 = ((string)jobIdValue11);
                                                                        jobTaskInstance9.JobId = jobIdInstance11;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue9 = jobTaskValue9["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue9 != null && jobFriendlyNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance9 = ((string)jobFriendlyNameValue9);
                                                                        jobTaskInstance9.JobFriendlyName = jobFriendlyNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue10 = jobTaskValue9["targetObjectId"];
                                                                    if (targetObjectIdValue10 != null && targetObjectIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance10 = ((string)targetObjectIdValue10);
                                                                        jobTaskInstance9.TargetObjectId = targetObjectIdInstance10;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue10 = jobTaskValue9["targetObjectName"];
                                                                    if (targetObjectNameValue10 != null && targetObjectNameValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance10 = ((string)targetObjectNameValue10);
                                                                        jobTaskInstance9.TargetObjectName = targetObjectNameInstance10;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue10 = jobTaskValue9["targetInstanceType"];
                                                                    if (targetInstanceTypeValue10 != null && targetInstanceTypeValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance10 = ((string)targetInstanceTypeValue10);
                                                                        jobTaskInstance9.TargetInstanceType = targetInstanceTypeInstance10;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue9 = jobTaskValue9["jobScenarioName"];
                                                                    if (jobScenarioNameValue9 != null && jobScenarioNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance9 = ((string)jobScenarioNameValue9);
                                                                        jobTaskInstance9.JobScenarioName = jobScenarioNameInstance9;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue22 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue22 != null && instanceTypeValue22.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance22 = ((string)instanceTypeValue22);
                                                                    fabricReplicationGroupTaskDetailsInstance3.Type = instanceTypeInstance22;
                                                                }
                                                                taskBaseInstance3.CustomDetails = fabricReplicationGroupTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance3 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue14 = customDetailsValue3["name"];
                                                                if (nameValue14 != null && nameValue14.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance14 = ((string)nameValue14);
                                                                    manualActionTaskDetailsInstance3.Name = nameInstance14;
                                                                }
                                                                
                                                                JToken instructionsValue3 = customDetailsValue3["instructions"];
                                                                if (instructionsValue3 != null && instructionsValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance3 = ((string)instructionsValue3);
                                                                    manualActionTaskDetailsInstance3.Instructions = instructionsInstance3;
                                                                }
                                                                
                                                                JToken observationValue3 = customDetailsValue3["observation"];
                                                                if (observationValue3 != null && observationValue3.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance3 = ((string)observationValue3);
                                                                    manualActionTaskDetailsInstance3.Observation = observationInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue23 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue23 != null && instanceTypeValue23.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance23 = ((string)instanceTypeValue23);
                                                                    manualActionTaskDetailsInstance3.Type = instanceTypeInstance23;
                                                                }
                                                                taskBaseInstance3.CustomDetails = manualActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance3 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue15 = customDetailsValue3["name"];
                                                                if (nameValue15 != null && nameValue15.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance15 = ((string)nameValue15);
                                                                    scriptActionTaskDetailsInstance3.Name = nameInstance15;
                                                                }
                                                                
                                                                JToken pathValue3 = customDetailsValue3["path"];
                                                                if (pathValue3 != null && pathValue3.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance3 = ((string)pathValue3);
                                                                    scriptActionTaskDetailsInstance3.Path = pathInstance3;
                                                                }
                                                                
                                                                JToken outputValue3 = customDetailsValue3["output"];
                                                                if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance3 = ((string)outputValue3);
                                                                    scriptActionTaskDetailsInstance3.Output = outputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue5 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue5 != null && isPrimarySideScriptValue5.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance5 = ((bool)isPrimarySideScriptValue5);
                                                                    scriptActionTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance5;
                                                                }
                                                                
                                                                JToken instanceTypeValue24 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue24 != null && instanceTypeValue24.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance24 = ((string)instanceTypeValue24);
                                                                    scriptActionTaskDetailsInstance3.Type = instanceTypeInstance24;
                                                                }
                                                                taskBaseInstance3.CustomDetails = scriptActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance3 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue3 = customDetailsValue3["vmId"];
                                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.VmId = vmIdInstance3;
                                                                }
                                                                
                                                                JToken nicIdValue3 = customDetailsValue3["nicId"];
                                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.NicId = nicIdInstance3;
                                                                }
                                                                
                                                                JToken nameValue16 = customDetailsValue3["name"];
                                                                if (nameValue16 != null && nameValue16.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance16 = ((string)nameValue16);
                                                                    vmNicUpdatesTaskDetailsInstance3.Name = nameInstance16;
                                                                }
                                                                
                                                                JToken instanceTypeValue25 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue25 != null && instanceTypeValue25.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance25 = ((string)instanceTypeValue25);
                                                                    vmNicUpdatesTaskDetailsInstance3.Type = instanceTypeInstance25;
                                                                }
                                                                taskBaseInstance3.CustomDetails = vmNicUpdatesTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance3 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray3 = customDetailsValue3["vmDetails"];
                                                                if (vmDetailsArray3 != null && vmDetailsArray3.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue3 in ((JArray)vmDetailsArray3))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance3 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance3.VmDetails.Add(inconsistentVmDetailsInstance3);
                                                                        
                                                                        JToken vmNameValue3 = vmDetailsValue3["vmName"];
                                                                        if (vmNameValue3 != null && vmNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance3 = ((string)vmNameValue3);
                                                                            inconsistentVmDetailsInstance3.VmName = vmNameInstance3;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue3 = vmDetailsValue3["cloudName"];
                                                                        if (cloudNameValue3 != null && cloudNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance3 = ((string)cloudNameValue3);
                                                                            inconsistentVmDetailsInstance3.CloudName = cloudNameInstance3;
                                                                        }
                                                                        
                                                                        JToken detailsArray3 = vmDetailsValue3["details"];
                                                                        if (detailsArray3 != null && detailsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue3 in ((JArray)detailsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.Details.Add(((string)detailsValue3));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray3 = vmDetailsValue3["errorIds"];
                                                                        if (errorIdsArray3 != null && errorIdsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue3 in ((JArray)errorIdsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.ErrorIds.Add(((string)errorIdsValue3));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue26 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue26 != null && instanceTypeValue26.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance26 = ((string)instanceTypeValue26);
                                                                    consistencyCheckTaskDetailsInstance3.Type = instanceTypeInstance26;
                                                                }
                                                                taskBaseInstance3.CustomDetails = consistencyCheckTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance3 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue17 = customDetailsValue3["name"];
                                                                if (nameValue17 != null && nameValue17.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance17 = ((string)nameValue17);
                                                                    automationRunbookTaskDetailsInstance3.Name = nameInstance17;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue3 = customDetailsValue3["cloudServiceName"];
                                                                if (cloudServiceNameValue3 != null && cloudServiceNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance3 = ((string)cloudServiceNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.CloudServiceName = cloudServiceNameInstance3;
                                                                }
                                                                
                                                                JToken subscriptionIdValue3 = customDetailsValue3["subscriptionId"];
                                                                if (subscriptionIdValue3 != null && subscriptionIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance3 = ((string)subscriptionIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.SubscriptionId = subscriptionIdInstance3;
                                                                }
                                                                
                                                                JToken accountNameValue3 = customDetailsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken runbookIdValue3 = customDetailsValue3["runbookId"];
                                                                if (runbookIdValue3 != null && runbookIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance3 = ((string)runbookIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookId = runbookIdInstance3;
                                                                }
                                                                
                                                                JToken runbookNameValue3 = customDetailsValue3["runbookName"];
                                                                if (runbookNameValue3 != null && runbookNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance3 = ((string)runbookNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookName = runbookNameInstance3;
                                                                }
                                                                
                                                                JToken jobIdValue12 = customDetailsValue3["jobId"];
                                                                if (jobIdValue12 != null && jobIdValue12.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance12 = ((string)jobIdValue12);
                                                                    automationRunbookTaskDetailsInstance3.JobId = jobIdInstance12;
                                                                }
                                                                
                                                                JToken jobOutputValue3 = customDetailsValue3["jobOutput"];
                                                                if (jobOutputValue3 != null && jobOutputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance3 = ((string)jobOutputValue3);
                                                                    automationRunbookTaskDetailsInstance3.JobOutput = jobOutputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue6 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue6 != null && isPrimarySideScriptValue6.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance6 = ((bool)isPrimarySideScriptValue6);
                                                                    automationRunbookTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance6;
                                                                }
                                                                
                                                                JToken instanceTypeValue27 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue27 != null && instanceTypeValue27.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance27 = ((string)instanceTypeValue27);
                                                                    automationRunbookTaskDetailsInstance3.Type = instanceTypeInstance27;
                                                                }
                                                                taskBaseInstance3.CustomDetails = automationRunbookTaskDetailsInstance3;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanShutdownGroupTaskDetailsInstance;
                                            }
                                        }
                                        
                                        JToken taskIdValue7 = tasksValue["taskId"];
                                        if (taskIdValue7 != null && taskIdValue7.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance7 = ((string)taskIdValue7);
                                            taskInstance.ID = taskIdInstance7;
                                        }
                                        
                                        JToken nameValue18 = tasksValue["name"];
                                        if (nameValue18 != null && nameValue18.Type != JTokenType.Null)
                                        {
                                            string nameInstance18 = ((string)nameValue18);
                                            taskInstance.Name = nameInstance18;
                                        }
                                        
                                        JToken startTimeValue5 = tasksValue["startTime"];
                                        if (startTimeValue5 != null && startTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime startTimeInstance5 = ((DateTime)startTimeValue5);
                                            taskInstance.StartTime = startTimeInstance5;
                                        }
                                        
                                        JToken endTimeValue5 = tasksValue["endTime"];
                                        if (endTimeValue5 != null && endTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime endTimeInstance5 = ((DateTime)endTimeValue5);
                                            taskInstance.EndTime = endTimeInstance5;
                                        }
                                        
                                        JToken allowedActionsArray5 = tasksValue["allowedActions"];
                                        if (allowedActionsArray5 != null && allowedActionsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedActionsValue5 in ((JArray)allowedActionsArray5))
                                            {
                                                taskInstance.Actions.Add(((string)allowedActionsValue5));
                                            }
                                        }
                                        
                                        JToken friendlyNameValue5 = tasksValue["friendlyName"];
                                        if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                            taskInstance.TaskFriendlyName = friendlyNameInstance5;
                                        }
                                        
                                        JToken stateValue5 = tasksValue["state"];
                                        if (stateValue5 != null && stateValue5.Type != JTokenType.Null)
                                        {
                                            string stateInstance5 = ((string)stateValue5);
                                            taskInstance.State = stateInstance5;
                                        }
                                        
                                        JToken stateDescriptionValue5 = tasksValue["stateDescription"];
                                        if (stateDescriptionValue5 != null && stateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string stateDescriptionInstance5 = ((string)stateDescriptionValue5);
                                            taskInstance.StateDescription = stateDescriptionInstance5;
                                        }
                                        
                                        JToken taskTypeValue4 = tasksValue["taskType"];
                                        if (taskTypeValue4 != null && taskTypeValue4.Type != JTokenType.Null)
                                        {
                                            string taskTypeInstance4 = ((string)taskTypeValue4);
                                            taskInstance.TaskType = taskTypeInstance4;
                                        }
                                        
                                        JToken errorsArray4 = tasksValue["errors"];
                                        if (errorsArray4 != null && errorsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken errorsValue4 in ((JArray)errorsArray4))
                                            {
                                                JobErrorDetails jobErrorDetailsInstance4 = new JobErrorDetails();
                                                taskInstance.Errors.Add(jobErrorDetailsInstance4);
                                                
                                                JToken serviceErrorDetailsValue4 = errorsValue4["serviceErrorDetails"];
                                                if (serviceErrorDetailsValue4 != null && serviceErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ServiceError serviceErrorDetailsInstance4 = new ServiceError();
                                                    jobErrorDetailsInstance4.ServiceErrorDetails = serviceErrorDetailsInstance4;
                                                    
                                                    JToken codeValue4 = serviceErrorDetailsValue4["code"];
                                                    if (codeValue4 != null && codeValue4.Type != JTokenType.Null)
                                                    {
                                                        string codeInstance4 = ((string)codeValue4);
                                                        serviceErrorDetailsInstance4.Code = codeInstance4;
                                                    }
                                                    
                                                    JToken messageValue4 = serviceErrorDetailsValue4["message"];
                                                    if (messageValue4 != null && messageValue4.Type != JTokenType.Null)
                                                    {
                                                        string messageInstance4 = ((string)messageValue4);
                                                        serviceErrorDetailsInstance4.Message = messageInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue7 = serviceErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue7 != null && possibleCausesValue7.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance7 = ((string)possibleCausesValue7);
                                                        serviceErrorDetailsInstance4.PossibleCauses = possibleCausesInstance7;
                                                    }
                                                    
                                                    JToken recommendedActionValue7 = serviceErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue7 != null && recommendedActionValue7.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance7 = ((string)recommendedActionValue7);
                                                        serviceErrorDetailsInstance4.RecommendedAction = recommendedActionInstance7;
                                                    }
                                                    
                                                    JToken activityIdValue5 = serviceErrorDetailsValue4["activityId"];
                                                    if (activityIdValue5 != null && activityIdValue5.Type != JTokenType.Null)
                                                    {
                                                        string activityIdInstance5 = ((string)activityIdValue5);
                                                        serviceErrorDetailsInstance4.ActivityId = activityIdInstance5;
                                                    }
                                                }
                                                
                                                JToken providerErrorDetailsValue4 = errorsValue4["providerErrorDetails"];
                                                if (providerErrorDetailsValue4 != null && providerErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ProviderError providerErrorDetailsInstance4 = new ProviderError();
                                                    jobErrorDetailsInstance4.ProviderErrorDetails = providerErrorDetailsInstance4;
                                                    
                                                    JToken errorCodeValue4 = providerErrorDetailsValue4["errorCode"];
                                                    if (errorCodeValue4 != null && errorCodeValue4.Type != JTokenType.Null)
                                                    {
                                                        int errorCodeInstance4 = ((int)errorCodeValue4);
                                                        providerErrorDetailsInstance4.ErrorCode = errorCodeInstance4;
                                                    }
                                                    
                                                    JToken errorMessageValue4 = providerErrorDetailsValue4["errorMessage"];
                                                    if (errorMessageValue4 != null && errorMessageValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorMessageInstance4 = ((string)errorMessageValue4);
                                                        providerErrorDetailsInstance4.ErrorMessage = errorMessageInstance4;
                                                    }
                                                    
                                                    JToken errorIdValue4 = providerErrorDetailsValue4["errorId"];
                                                    if (errorIdValue4 != null && errorIdValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorIdInstance4 = ((string)errorIdValue4);
                                                        providerErrorDetailsInstance4.ErrorId = errorIdInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue8 = providerErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue8 != null && possibleCausesValue8.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance8 = ((string)possibleCausesValue8);
                                                        providerErrorDetailsInstance4.PossibleCauses = possibleCausesInstance8;
                                                    }
                                                    
                                                    JToken recommendedActionValue8 = providerErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue8 != null && recommendedActionValue8.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance8 = ((string)recommendedActionValue8);
                                                        providerErrorDetailsInstance4.RecommendedAction = recommendedActionInstance8;
                                                    }
                                                }
                                                
                                                JToken errorLevelValue4 = errorsValue4["errorLevel"];
                                                if (errorLevelValue4 != null && errorLevelValue4.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance4 = ((string)errorLevelValue4);
                                                    jobErrorDetailsInstance4.ErrorLevel = errorLevelInstance4;
                                                }
                                                
                                                JToken creationTimeValue4 = errorsValue4["creationTime"];
                                                if (creationTimeValue4 != null && creationTimeValue4.Type != JTokenType.Null)
                                                {
                                                    DateTime creationTimeInstance4 = ((DateTime)creationTimeValue4);
                                                    jobErrorDetailsInstance4.CreationTime = creationTimeInstance4;
                                                }
                                                
                                                JToken taskIdValue8 = errorsValue4["taskId"];
                                                if (taskIdValue8 != null && taskIdValue8.Type != JTokenType.Null)
                                                {
                                                    string taskIdInstance8 = ((string)taskIdValue8);
                                                    jobErrorDetailsInstance4.TaskId = taskIdInstance8;
                                                }
                                            }
                                        }
                                        
                                        JToken customDetailsValue4 = tasksValue["customDetails"];
                                        if (customDetailsValue4 != null && customDetailsValue4.Type != JTokenType.Null)
                                        {
                                            string typeName5 = ((string)customDetailsValue4["instanceType"]);
                                            if (typeName5 == "JobTaskDetails")
                                            {
                                                JobTaskDetails jobTaskDetailsInstance4 = new JobTaskDetails();
                                                
                                                JToken jobTaskValue10 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue10 != null && jobTaskValue10.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance10 = new JobEntity();
                                                    jobTaskDetailsInstance4.JobTask = jobTaskInstance10;
                                                    
                                                    JToken jobIdValue13 = jobTaskValue10["jobId"];
                                                    if (jobIdValue13 != null && jobIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance13 = ((string)jobIdValue13);
                                                        jobTaskInstance10.JobId = jobIdInstance13;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue10 = jobTaskValue10["jobFriendlyName"];
                                                    if (jobFriendlyNameValue10 != null && jobFriendlyNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance10 = ((string)jobFriendlyNameValue10);
                                                        jobTaskInstance10.JobFriendlyName = jobFriendlyNameInstance10;
                                                    }
                                                    
                                                    JToken targetObjectIdValue11 = jobTaskValue10["targetObjectId"];
                                                    if (targetObjectIdValue11 != null && targetObjectIdValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance11 = ((string)targetObjectIdValue11);
                                                        jobTaskInstance10.TargetObjectId = targetObjectIdInstance11;
                                                    }
                                                    
                                                    JToken targetObjectNameValue11 = jobTaskValue10["targetObjectName"];
                                                    if (targetObjectNameValue11 != null && targetObjectNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance11 = ((string)targetObjectNameValue11);
                                                        jobTaskInstance10.TargetObjectName = targetObjectNameInstance11;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue11 = jobTaskValue10["targetInstanceType"];
                                                    if (targetInstanceTypeValue11 != null && targetInstanceTypeValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance11 = ((string)targetInstanceTypeValue11);
                                                        jobTaskInstance10.TargetInstanceType = targetInstanceTypeInstance11;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue10 = jobTaskValue10["jobScenarioName"];
                                                    if (jobScenarioNameValue10 != null && jobScenarioNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance10 = ((string)jobScenarioNameValue10);
                                                        jobTaskInstance10.JobScenarioName = jobScenarioNameInstance10;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue28 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue28 != null && instanceTypeValue28.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance28 = ((string)instanceTypeValue28);
                                                    jobTaskDetailsInstance4.Type = instanceTypeInstance28;
                                                }
                                                taskInstance.CustomDetails = jobTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VirtualMachineTaskDetails")
                                            {
                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance4 = new VirtualMachineTaskDetails();
                                                
                                                JToken skippedReasonValue7 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue7 != null && skippedReasonValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance7 = ((string)skippedReasonValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReason = skippedReasonInstance7;
                                                }
                                                
                                                JToken skippedReasonStringValue7 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue7 != null && skippedReasonStringValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance7 = ((string)skippedReasonStringValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance7;
                                                }
                                                
                                                JToken jobTaskValue11 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue11 != null && jobTaskValue11.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance11 = new JobEntity();
                                                    virtualMachineTaskDetailsInstance4.JobTask = jobTaskInstance11;
                                                    
                                                    JToken jobIdValue14 = jobTaskValue11["jobId"];
                                                    if (jobIdValue14 != null && jobIdValue14.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance14 = ((string)jobIdValue14);
                                                        jobTaskInstance11.JobId = jobIdInstance14;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue11 = jobTaskValue11["jobFriendlyName"];
                                                    if (jobFriendlyNameValue11 != null && jobFriendlyNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance11 = ((string)jobFriendlyNameValue11);
                                                        jobTaskInstance11.JobFriendlyName = jobFriendlyNameInstance11;
                                                    }
                                                    
                                                    JToken targetObjectIdValue12 = jobTaskValue11["targetObjectId"];
                                                    if (targetObjectIdValue12 != null && targetObjectIdValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance12 = ((string)targetObjectIdValue12);
                                                        jobTaskInstance11.TargetObjectId = targetObjectIdInstance12;
                                                    }
                                                    
                                                    JToken targetObjectNameValue12 = jobTaskValue11["targetObjectName"];
                                                    if (targetObjectNameValue12 != null && targetObjectNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance12 = ((string)targetObjectNameValue12);
                                                        jobTaskInstance11.TargetObjectName = targetObjectNameInstance12;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue12 = jobTaskValue11["targetInstanceType"];
                                                    if (targetInstanceTypeValue12 != null && targetInstanceTypeValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance12 = ((string)targetInstanceTypeValue12);
                                                        jobTaskInstance11.TargetInstanceType = targetInstanceTypeInstance12;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue11 = jobTaskValue11["jobScenarioName"];
                                                    if (jobScenarioNameValue11 != null && jobScenarioNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance11 = ((string)jobScenarioNameValue11);
                                                        jobTaskInstance11.JobScenarioName = jobScenarioNameInstance11;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue29 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue29 != null && instanceTypeValue29.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance29 = ((string)instanceTypeValue29);
                                                    virtualMachineTaskDetailsInstance4.Type = instanceTypeInstance29;
                                                }
                                                taskInstance.CustomDetails = virtualMachineTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "FabricReplicationGroupTaskDetails")
                                            {
                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance4 = new FabricReplicationGroupTaskDetails();
                                                
                                                JToken skippedReasonValue8 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue8 != null && skippedReasonValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance8 = ((string)skippedReasonValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReason = skippedReasonInstance8;
                                                }
                                                
                                                JToken skippedReasonStringValue8 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue8 != null && skippedReasonStringValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance8 = ((string)skippedReasonStringValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance8;
                                                }
                                                
                                                JToken jobTaskValue12 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue12 != null && jobTaskValue12.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance12 = new JobEntity();
                                                    fabricReplicationGroupTaskDetailsInstance4.JobTask = jobTaskInstance12;
                                                    
                                                    JToken jobIdValue15 = jobTaskValue12["jobId"];
                                                    if (jobIdValue15 != null && jobIdValue15.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance15 = ((string)jobIdValue15);
                                                        jobTaskInstance12.JobId = jobIdInstance15;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue12 = jobTaskValue12["jobFriendlyName"];
                                                    if (jobFriendlyNameValue12 != null && jobFriendlyNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance12 = ((string)jobFriendlyNameValue12);
                                                        jobTaskInstance12.JobFriendlyName = jobFriendlyNameInstance12;
                                                    }
                                                    
                                                    JToken targetObjectIdValue13 = jobTaskValue12["targetObjectId"];
                                                    if (targetObjectIdValue13 != null && targetObjectIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance13 = ((string)targetObjectIdValue13);
                                                        jobTaskInstance12.TargetObjectId = targetObjectIdInstance13;
                                                    }
                                                    
                                                    JToken targetObjectNameValue13 = jobTaskValue12["targetObjectName"];
                                                    if (targetObjectNameValue13 != null && targetObjectNameValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance13 = ((string)targetObjectNameValue13);
                                                        jobTaskInstance12.TargetObjectName = targetObjectNameInstance13;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue13 = jobTaskValue12["targetInstanceType"];
                                                    if (targetInstanceTypeValue13 != null && targetInstanceTypeValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance13 = ((string)targetInstanceTypeValue13);
                                                        jobTaskInstance12.TargetInstanceType = targetInstanceTypeInstance13;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue12 = jobTaskValue12["jobScenarioName"];
                                                    if (jobScenarioNameValue12 != null && jobScenarioNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance12 = ((string)jobScenarioNameValue12);
                                                        jobTaskInstance12.JobScenarioName = jobScenarioNameInstance12;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue30 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue30 != null && instanceTypeValue30.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance30 = ((string)instanceTypeValue30);
                                                    fabricReplicationGroupTaskDetailsInstance4.Type = instanceTypeInstance30;
                                                }
                                                taskInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ManualActionTaskDetails")
                                            {
                                                ManualActionTaskDetails manualActionTaskDetailsInstance4 = new ManualActionTaskDetails();
                                                
                                                JToken nameValue19 = customDetailsValue4["name"];
                                                if (nameValue19 != null && nameValue19.Type != JTokenType.Null)
                                                {
                                                    string nameInstance19 = ((string)nameValue19);
                                                    manualActionTaskDetailsInstance4.Name = nameInstance19;
                                                }
                                                
                                                JToken instructionsValue4 = customDetailsValue4["instructions"];
                                                if (instructionsValue4 != null && instructionsValue4.Type != JTokenType.Null)
                                                {
                                                    string instructionsInstance4 = ((string)instructionsValue4);
                                                    manualActionTaskDetailsInstance4.Instructions = instructionsInstance4;
                                                }
                                                
                                                JToken observationValue4 = customDetailsValue4["observation"];
                                                if (observationValue4 != null && observationValue4.Type != JTokenType.Null)
                                                {
                                                    string observationInstance4 = ((string)observationValue4);
                                                    manualActionTaskDetailsInstance4.Observation = observationInstance4;
                                                }
                                                
                                                JToken instanceTypeValue31 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue31 != null && instanceTypeValue31.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance31 = ((string)instanceTypeValue31);
                                                    manualActionTaskDetailsInstance4.Type = instanceTypeInstance31;
                                                }
                                                taskInstance.CustomDetails = manualActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ScriptActionTaskDetails")
                                            {
                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance4 = new ScriptActionTaskDetails();
                                                
                                                JToken nameValue20 = customDetailsValue4["name"];
                                                if (nameValue20 != null && nameValue20.Type != JTokenType.Null)
                                                {
                                                    string nameInstance20 = ((string)nameValue20);
                                                    scriptActionTaskDetailsInstance4.Name = nameInstance20;
                                                }
                                                
                                                JToken pathValue4 = customDetailsValue4["path"];
                                                if (pathValue4 != null && pathValue4.Type != JTokenType.Null)
                                                {
                                                    string pathInstance4 = ((string)pathValue4);
                                                    scriptActionTaskDetailsInstance4.Path = pathInstance4;
                                                }
                                                
                                                JToken outputValue4 = customDetailsValue4["output"];
                                                if (outputValue4 != null && outputValue4.Type != JTokenType.Null)
                                                {
                                                    string outputInstance4 = ((string)outputValue4);
                                                    scriptActionTaskDetailsInstance4.Output = outputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue7 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue7 != null && isPrimarySideScriptValue7.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance7 = ((bool)isPrimarySideScriptValue7);
                                                    scriptActionTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance7;
                                                }
                                                
                                                JToken instanceTypeValue32 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue32 != null && instanceTypeValue32.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance32 = ((string)instanceTypeValue32);
                                                    scriptActionTaskDetailsInstance4.Type = instanceTypeInstance32;
                                                }
                                                taskInstance.CustomDetails = scriptActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VmNicUpdatesTaskDetails")
                                            {
                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance4 = new VmNicUpdatesTaskDetails();
                                                
                                                JToken vmIdValue4 = customDetailsValue4["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken nicIdValue4 = customDetailsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken nameValue21 = customDetailsValue4["name"];
                                                if (nameValue21 != null && nameValue21.Type != JTokenType.Null)
                                                {
                                                    string nameInstance21 = ((string)nameValue21);
                                                    vmNicUpdatesTaskDetailsInstance4.Name = nameInstance21;
                                                }
                                                
                                                JToken instanceTypeValue33 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue33 != null && instanceTypeValue33.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance33 = ((string)instanceTypeValue33);
                                                    vmNicUpdatesTaskDetailsInstance4.Type = instanceTypeInstance33;
                                                }
                                                taskInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ConsistencyCheckTaskDetails")
                                            {
                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance4 = new ConsistencyCheckTaskDetails();
                                                
                                                JToken vmDetailsArray4 = customDetailsValue4["vmDetails"];
                                                if (vmDetailsArray4 != null && vmDetailsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmDetailsValue4 in ((JArray)vmDetailsArray4))
                                                    {
                                                        InconsistentVmDetails inconsistentVmDetailsInstance4 = new InconsistentVmDetails();
                                                        consistencyCheckTaskDetailsInstance4.VmDetails.Add(inconsistentVmDetailsInstance4);
                                                        
                                                        JToken vmNameValue4 = vmDetailsValue4["vmName"];
                                                        if (vmNameValue4 != null && vmNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vmNameInstance4 = ((string)vmNameValue4);
                                                            inconsistentVmDetailsInstance4.VmName = vmNameInstance4;
                                                        }
                                                        
                                                        JToken cloudNameValue4 = vmDetailsValue4["cloudName"];
                                                        if (cloudNameValue4 != null && cloudNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string cloudNameInstance4 = ((string)cloudNameValue4);
                                                            inconsistentVmDetailsInstance4.CloudName = cloudNameInstance4;
                                                        }
                                                        
                                                        JToken detailsArray4 = vmDetailsValue4["details"];
                                                        if (detailsArray4 != null && detailsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken detailsValue4 in ((JArray)detailsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.Details.Add(((string)detailsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken errorIdsArray4 = vmDetailsValue4["errorIds"];
                                                        if (errorIdsArray4 != null && errorIdsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorIdsValue4 in ((JArray)errorIdsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.ErrorIds.Add(((string)errorIdsValue4));
                                                            }
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue34 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue34 != null && instanceTypeValue34.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance34 = ((string)instanceTypeValue34);
                                                    consistencyCheckTaskDetailsInstance4.Type = instanceTypeInstance34;
                                                }
                                                taskInstance.CustomDetails = consistencyCheckTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "AutomationRunbookTaskDetails")
                                            {
                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance4 = new AutomationRunbookTaskDetails();
                                                
                                                JToken nameValue22 = customDetailsValue4["name"];
                                                if (nameValue22 != null && nameValue22.Type != JTokenType.Null)
                                                {
                                                    string nameInstance22 = ((string)nameValue22);
                                                    automationRunbookTaskDetailsInstance4.Name = nameInstance22;
                                                }
                                                
                                                JToken cloudServiceNameValue4 = customDetailsValue4["cloudServiceName"];
                                                if (cloudServiceNameValue4 != null && cloudServiceNameValue4.Type != JTokenType.Null)
                                                {
                                                    string cloudServiceNameInstance4 = ((string)cloudServiceNameValue4);
                                                    automationRunbookTaskDetailsInstance4.CloudServiceName = cloudServiceNameInstance4;
                                                }
                                                
                                                JToken subscriptionIdValue4 = customDetailsValue4["subscriptionId"];
                                                if (subscriptionIdValue4 != null && subscriptionIdValue4.Type != JTokenType.Null)
                                                {
                                                    string subscriptionIdInstance4 = ((string)subscriptionIdValue4);
                                                    automationRunbookTaskDetailsInstance4.SubscriptionId = subscriptionIdInstance4;
                                                }
                                                
                                                JToken accountNameValue4 = customDetailsValue4["accountName"];
                                                if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                {
                                                    string accountNameInstance4 = ((string)accountNameValue4);
                                                    automationRunbookTaskDetailsInstance4.AccountName = accountNameInstance4;
                                                }
                                                
                                                JToken runbookIdValue4 = customDetailsValue4["runbookId"];
                                                if (runbookIdValue4 != null && runbookIdValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookIdInstance4 = ((string)runbookIdValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookId = runbookIdInstance4;
                                                }
                                                
                                                JToken runbookNameValue4 = customDetailsValue4["runbookName"];
                                                if (runbookNameValue4 != null && runbookNameValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookNameInstance4 = ((string)runbookNameValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookName = runbookNameInstance4;
                                                }
                                                
                                                JToken jobIdValue16 = customDetailsValue4["jobId"];
                                                if (jobIdValue16 != null && jobIdValue16.Type != JTokenType.Null)
                                                {
                                                    string jobIdInstance16 = ((string)jobIdValue16);
                                                    automationRunbookTaskDetailsInstance4.JobId = jobIdInstance16;
                                                }
                                                
                                                JToken jobOutputValue4 = customDetailsValue4["jobOutput"];
                                                if (jobOutputValue4 != null && jobOutputValue4.Type != JTokenType.Null)
                                                {
                                                    string jobOutputInstance4 = ((string)jobOutputValue4);
                                                    automationRunbookTaskDetailsInstance4.JobOutput = jobOutputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue8 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue8 != null && isPrimarySideScriptValue8.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance8 = ((bool)isPrimarySideScriptValue8);
                                                    automationRunbookTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance8;
                                                }
                                                
                                                JToken instanceTypeValue35 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue35 != null && instanceTypeValue35.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance35 = ((string)instanceTypeValue35);
                                                    automationRunbookTaskDetailsInstance4.Type = instanceTypeInstance35;
                                                }
                                                taskInstance.CustomDetails = automationRunbookTaskDetailsInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken errorsArray5 = propertiesValue["errors"];
                                if (errorsArray5 != null && errorsArray5.Type != JTokenType.Null)
                                {
                                    foreach (JToken errorsValue5 in ((JArray)errorsArray5))
                                    {
                                        JobErrorDetails jobErrorDetailsInstance5 = new JobErrorDetails();
                                        propertiesInstance.Errors.Add(jobErrorDetailsInstance5);
                                        
                                        JToken serviceErrorDetailsValue5 = errorsValue5["serviceErrorDetails"];
                                        if (serviceErrorDetailsValue5 != null && serviceErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ServiceError serviceErrorDetailsInstance5 = new ServiceError();
                                            jobErrorDetailsInstance5.ServiceErrorDetails = serviceErrorDetailsInstance5;
                                            
                                            JToken codeValue5 = serviceErrorDetailsValue5["code"];
                                            if (codeValue5 != null && codeValue5.Type != JTokenType.Null)
                                            {
                                                string codeInstance5 = ((string)codeValue5);
                                                serviceErrorDetailsInstance5.Code = codeInstance5;
                                            }
                                            
                                            JToken messageValue5 = serviceErrorDetailsValue5["message"];
                                            if (messageValue5 != null && messageValue5.Type != JTokenType.Null)
                                            {
                                                string messageInstance5 = ((string)messageValue5);
                                                serviceErrorDetailsInstance5.Message = messageInstance5;
                                            }
                                            
                                            JToken possibleCausesValue9 = serviceErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue9 != null && possibleCausesValue9.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance9 = ((string)possibleCausesValue9);
                                                serviceErrorDetailsInstance5.PossibleCauses = possibleCausesInstance9;
                                            }
                                            
                                            JToken recommendedActionValue9 = serviceErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue9 != null && recommendedActionValue9.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance9 = ((string)recommendedActionValue9);
                                                serviceErrorDetailsInstance5.RecommendedAction = recommendedActionInstance9;
                                            }
                                            
                                            JToken activityIdValue6 = serviceErrorDetailsValue5["activityId"];
                                            if (activityIdValue6 != null && activityIdValue6.Type != JTokenType.Null)
                                            {
                                                string activityIdInstance6 = ((string)activityIdValue6);
                                                serviceErrorDetailsInstance5.ActivityId = activityIdInstance6;
                                            }
                                        }
                                        
                                        JToken providerErrorDetailsValue5 = errorsValue5["providerErrorDetails"];
                                        if (providerErrorDetailsValue5 != null && providerErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ProviderError providerErrorDetailsInstance5 = new ProviderError();
                                            jobErrorDetailsInstance5.ProviderErrorDetails = providerErrorDetailsInstance5;
                                            
                                            JToken errorCodeValue5 = providerErrorDetailsValue5["errorCode"];
                                            if (errorCodeValue5 != null && errorCodeValue5.Type != JTokenType.Null)
                                            {
                                                int errorCodeInstance5 = ((int)errorCodeValue5);
                                                providerErrorDetailsInstance5.ErrorCode = errorCodeInstance5;
                                            }
                                            
                                            JToken errorMessageValue5 = providerErrorDetailsValue5["errorMessage"];
                                            if (errorMessageValue5 != null && errorMessageValue5.Type != JTokenType.Null)
                                            {
                                                string errorMessageInstance5 = ((string)errorMessageValue5);
                                                providerErrorDetailsInstance5.ErrorMessage = errorMessageInstance5;
                                            }
                                            
                                            JToken errorIdValue5 = providerErrorDetailsValue5["errorId"];
                                            if (errorIdValue5 != null && errorIdValue5.Type != JTokenType.Null)
                                            {
                                                string errorIdInstance5 = ((string)errorIdValue5);
                                                providerErrorDetailsInstance5.ErrorId = errorIdInstance5;
                                            }
                                            
                                            JToken possibleCausesValue10 = providerErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue10 != null && possibleCausesValue10.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance10 = ((string)possibleCausesValue10);
                                                providerErrorDetailsInstance5.PossibleCauses = possibleCausesInstance10;
                                            }
                                            
                                            JToken recommendedActionValue10 = providerErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue10 != null && recommendedActionValue10.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance10 = ((string)recommendedActionValue10);
                                                providerErrorDetailsInstance5.RecommendedAction = recommendedActionInstance10;
                                            }
                                        }
                                        
                                        JToken errorLevelValue5 = errorsValue5["errorLevel"];
                                        if (errorLevelValue5 != null && errorLevelValue5.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance5 = ((string)errorLevelValue5);
                                            jobErrorDetailsInstance5.ErrorLevel = errorLevelInstance5;
                                        }
                                        
                                        JToken creationTimeValue5 = errorsValue5["creationTime"];
                                        if (creationTimeValue5 != null && creationTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime creationTimeInstance5 = ((DateTime)creationTimeValue5);
                                            jobErrorDetailsInstance5.CreationTime = creationTimeInstance5;
                                        }
                                        
                                        JToken taskIdValue9 = errorsValue5["taskId"];
                                        if (taskIdValue9 != null && taskIdValue9.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance9 = ((string)taskIdValue9);
                                            jobErrorDetailsInstance5.TaskId = taskIdInstance9;
                                        }
                                    }
                                }
                                
                                JToken customDetailsValue5 = propertiesValue["customDetails"];
                                if (customDetailsValue5 != null && customDetailsValue5.Type != JTokenType.Null)
                                {
                                    string typeName6 = ((string)customDetailsValue5["instanceType"]);
                                    if (typeName6 == "ExportJobDetails")
                                    {
                                        ExportJobDetails exportJobDetailsInstance = new ExportJobDetails();
                                        
                                        JToken blobUriValue = customDetailsValue5["blobUri"];
                                        if (blobUriValue != null && blobUriValue.Type != JTokenType.Null)
                                        {
                                            string blobUriInstance = ((string)blobUriValue);
                                            exportJobDetailsInstance.BlobUri = blobUriInstance;
                                        }
                                        
                                        JToken sasTokenValue = customDetailsValue5["sasToken"];
                                        if (sasTokenValue != null && sasTokenValue.Type != JTokenType.Null)
                                        {
                                            string sasTokenInstance = ((string)sasTokenValue);
                                            exportJobDetailsInstance.SasToken = sasTokenInstance;
                                        }
                                        
                                        JToken instanceTypeValue36 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue36 != null && instanceTypeValue36.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance36 = ((string)instanceTypeValue36);
                                            exportJobDetailsInstance.InstanceType = instanceTypeInstance36;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement != null && affectedObjectDetailsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in affectedObjectDetailsSequenceElement)
                                            {
                                                string affectedObjectDetailsKey = ((string)property.Name);
                                                string affectedObjectDetailsValue = ((string)property.Value);
                                                exportJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey, affectedObjectDetailsValue);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = exportJobDetailsInstance;
                                    }
                                    if (typeName6 == "AsrJobDetails")
                                    {
                                        AsrJobDetails asrJobDetailsInstance = new AsrJobDetails();
                                        
                                        JToken instanceTypeValue37 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue37 != null && instanceTypeValue37.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance37 = ((string)instanceTypeValue37);
                                            asrJobDetailsInstance.InstanceType = instanceTypeInstance37;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement2 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement2 != null && affectedObjectDetailsSequenceElement2.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property2 in affectedObjectDetailsSequenceElement2)
                                            {
                                                string affectedObjectDetailsKey2 = ((string)property2.Name);
                                                string affectedObjectDetailsValue2 = ((string)property2.Value);
                                                asrJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey2, affectedObjectDetailsValue2);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = asrJobDetailsInstance;
                                    }
                                    if (typeName6 == "TestFailoverJobDetails")
                                    {
                                        TestFailoverJobDetails testFailoverJobDetailsInstance = new TestFailoverJobDetails();
                                        
                                        JToken testFailoverStatusValue = customDetailsValue5["testFailoverStatus"];
                                        if (testFailoverStatusValue != null && testFailoverStatusValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStatusInstance = ((string)testFailoverStatusValue);
                                            testFailoverJobDetailsInstance.TestFailoverStatus = testFailoverStatusInstance;
                                        }
                                        
                                        JToken commentsValue = customDetailsValue5["comments"];
                                        if (commentsValue != null && commentsValue.Type != JTokenType.Null)
                                        {
                                            string commentsInstance = ((string)commentsValue);
                                            testFailoverJobDetailsInstance.Comments = commentsInstance;
                                        }
                                        
                                        JToken networkNameValue = customDetailsValue5["networkName"];
                                        if (networkNameValue != null && networkNameValue.Type != JTokenType.Null)
                                        {
                                            string networkNameInstance = ((string)networkNameValue);
                                            testFailoverJobDetailsInstance.NetworkName = networkNameInstance;
                                        }
                                        
                                        JToken networkFriendlyNameValue = customDetailsValue5["networkFriendlyName"];
                                        if (networkFriendlyNameValue != null && networkFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string networkFriendlyNameInstance = ((string)networkFriendlyNameValue);
                                            testFailoverJobDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance;
                                        }
                                        
                                        JToken networkTypeValue = customDetailsValue5["networkType"];
                                        if (networkTypeValue != null && networkTypeValue.Type != JTokenType.Null)
                                        {
                                            string networkTypeInstance = ((string)networkTypeValue);
                                            testFailoverJobDetailsInstance.NetworkType = networkTypeInstance;
                                        }
                                        
                                        JToken protectedItemDetailsArray = customDetailsValue5["protectedItemDetails"];
                                        if (protectedItemDetailsArray != null && protectedItemDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedItemDetailsValue in ((JArray)protectedItemDetailsArray))
                                            {
                                                TestFailoverReplicationProtectedItemDetails testFailoverReplicationProtectedItemDetailsInstance = new TestFailoverReplicationProtectedItemDetails();
                                                testFailoverJobDetailsInstance.ProtectedItemDetails.Add(testFailoverReplicationProtectedItemDetailsInstance);
                                                
                                                JToken nameValue23 = protectedItemDetailsValue["name"];
                                                if (nameValue23 != null && nameValue23.Type != JTokenType.Null)
                                                {
                                                    string nameInstance23 = ((string)nameValue23);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Name = nameInstance23;
                                                }
                                                
                                                JToken friendlyNameValue6 = protectedItemDetailsValue["friendlyName"];
                                                if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                                {
                                                    string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                                    testFailoverReplicationProtectedItemDetailsInstance.FriendlyName = friendlyNameInstance6;
                                                }
                                                
                                                JToken testVmNameValue = protectedItemDetailsValue["testVmName"];
                                                if (testVmNameValue != null && testVmNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmNameInstance = ((string)testVmNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmName = testVmNameInstance;
                                                }
                                                
                                                JToken testVmFriendlyNameValue = protectedItemDetailsValue["testVmFriendlyName"];
                                                if (testVmFriendlyNameValue != null && testVmFriendlyNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmFriendlyNameInstance = ((string)testVmFriendlyNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmFriendlyName = testVmFriendlyNameInstance;
                                                }
                                                
                                                JToken networkConnectionStatusValue = protectedItemDetailsValue["networkConnectionStatus"];
                                                if (networkConnectionStatusValue != null && networkConnectionStatusValue.Type != JTokenType.Null)
                                                {
                                                    string networkConnectionStatusInstance = ((string)networkConnectionStatusValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkConnectionStatus = networkConnectionStatusInstance;
                                                }
                                                
                                                JToken networkFriendlyNameValue2 = protectedItemDetailsValue["networkFriendlyName"];
                                                if (networkFriendlyNameValue2 != null && networkFriendlyNameValue2.Type != JTokenType.Null)
                                                {
                                                    string networkFriendlyNameInstance2 = ((string)networkFriendlyNameValue2);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance2;
                                                }
                                                
                                                JToken subnetValue = protectedItemDetailsValue["subnet"];
                                                if (subnetValue != null && subnetValue.Type != JTokenType.Null)
                                                {
                                                    string subnetInstance = ((string)subnetValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Subnet = subnetInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue38 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue38 != null && instanceTypeValue38.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance38 = ((string)instanceTypeValue38);
                                            testFailoverJobDetailsInstance.InstanceType = instanceTypeInstance38;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement3 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement3 != null && affectedObjectDetailsSequenceElement3.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property3 in affectedObjectDetailsSequenceElement3)
                                            {
                                                string affectedObjectDetailsKey3 = ((string)property3.Name);
                                                string affectedObjectDetailsValue3 = ((string)property3.Value);
                                                testFailoverJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey3, affectedObjectDetailsValue3);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = testFailoverJobDetailsInstance;
                                    }
                                }
                                
                                JToken idValue = propertiesValue["id"];
                                if (idValue != null && idValue.Type != JTokenType.Null)
                                {
                                    string idInstance = ((string)idValue);
                                    propertiesInstance.Id = idInstance;
                                }
                                
                                JToken nameValue24 = propertiesValue["name"];
                                if (nameValue24 != null && nameValue24.Type != JTokenType.Null)
                                {
                                    string nameInstance24 = ((string)nameValue24);
                                    propertiesInstance.Name = nameInstance24;
                                }
                                
                                JToken typeValue = propertiesValue["type"];
                                if (typeValue != null && typeValue.Type != JTokenType.Null)
                                {
                                    string typeInstance = ((string)typeValue);
                                    propertiesInstance.Type = typeInstance;
                                }
                                
                                JToken locationValue = propertiesValue["location"];
                                if (locationValue != null && locationValue.Type != JTokenType.Null)
                                {
                                    string locationInstance = ((string)locationValue);
                                    propertiesInstance.Location = locationInstance;
                                }
                                
                                JToken tagsSequenceElement = ((JToken)propertiesValue["tags"]);
                                if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property4 in tagsSequenceElement)
                                    {
                                        string tagsKey = ((string)property4.Name);
                                        string tagsValue = ((string)property4.Value);
                                        propertiesInstance.Tags.Add(tagsKey, tagsValue);
                                    }
                                }
                            }
                            
                            JToken idValue2 = responseDoc["id"];
                            if (idValue2 != null && idValue2.Type != JTokenType.Null)
                            {
                                string idInstance2 = ((string)idValue2);
                                jobInstance.Id = idInstance2;
                            }
                            
                            JToken nameValue25 = responseDoc["name"];
                            if (nameValue25 != null && nameValue25.Type != JTokenType.Null)
                            {
                                string nameInstance25 = ((string)nameValue25);
                                jobInstance.Name = nameInstance25;
                            }
                            
                            JToken typeValue2 = responseDoc["type"];
                            if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                            {
                                string typeInstance2 = ((string)typeValue2);
                                jobInstance.Type = typeInstance2;
                            }
                            
                            JToken locationValue2 = responseDoc["location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                jobInstance.Location = locationInstance2;
                            }
                            
                            JToken tagsSequenceElement2 = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement2 != null && tagsSequenceElement2.Type != JTokenType.Null)
                            {
                                foreach (JProperty property5 in tagsSequenceElement2)
                                {
                                    string tagsKey2 = ((string)property5.Name);
                                    string tagsValue2 = ((string)property5.Value);
                                    jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<JobOperationResponse> GetCancelStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetCancelStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken scenarioNameValue = propertiesValue["scenarioName"];
                                if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                {
                                    string scenarioNameInstance = ((string)scenarioNameValue);
                                    propertiesInstance.ScenarioName = scenarioNameInstance;
                                }
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken activityIdValue = propertiesValue["activityId"];
                                if (activityIdValue != null && activityIdValue.Type != JTokenType.Null)
                                {
                                    string activityIdInstance = ((string)activityIdValue);
                                    propertiesInstance.ActivityId = activityIdInstance;
                                }
                                
                                JToken startTimeValue = propertiesValue["startTime"];
                                if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime startTimeInstance = ((DateTime)startTimeValue);
                                    propertiesInstance.StartTime = startTimeInstance;
                                }
                                
                                JToken endTimeValue = propertiesValue["endTime"];
                                if (endTimeValue != null && endTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime endTimeInstance = ((DateTime)endTimeValue);
                                    propertiesInstance.EndTime = endTimeInstance;
                                }
                                
                                JToken targetObjectIdValue = propertiesValue["targetObjectId"];
                                if (targetObjectIdValue != null && targetObjectIdValue.Type != JTokenType.Null)
                                {
                                    string targetObjectIdInstance = ((string)targetObjectIdValue);
                                    propertiesInstance.TargetObjectId = targetObjectIdInstance;
                                }
                                
                                JToken targetInstanceTypeValue = propertiesValue["targetInstanceType"];
                                if (targetInstanceTypeValue != null && targetInstanceTypeValue.Type != JTokenType.Null)
                                {
                                    string targetInstanceTypeInstance = ((string)targetInstanceTypeValue);
                                    propertiesInstance.TargetInstanceType = targetInstanceTypeInstance;
                                }
                                
                                JToken targetObjectNameValue = propertiesValue["targetObjectName"];
                                if (targetObjectNameValue != null && targetObjectNameValue.Type != JTokenType.Null)
                                {
                                    string targetObjectNameInstance = ((string)targetObjectNameValue);
                                    propertiesInstance.TargetObjectName = targetObjectNameInstance;
                                }
                                
                                JToken allowedActionsArray = propertiesValue["allowedActions"];
                                if (allowedActionsArray != null && allowedActionsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedActionsValue in ((JArray)allowedActionsArray))
                                    {
                                        propertiesInstance.AllowedActions.Add(((string)allowedActionsValue));
                                    }
                                }
                                
                                JToken stateValue = propertiesValue["state"];
                                if (stateValue != null && stateValue.Type != JTokenType.Null)
                                {
                                    string stateInstance = ((string)stateValue);
                                    propertiesInstance.State = stateInstance;
                                }
                                
                                JToken stateDescriptionValue = propertiesValue["stateDescription"];
                                if (stateDescriptionValue != null && stateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string stateDescriptionInstance = ((string)stateDescriptionValue);
                                    propertiesInstance.StateDescription = stateDescriptionInstance;
                                }
                                
                                JToken tasksArray = propertiesValue["tasks"];
                                if (tasksArray != null && tasksArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken tasksValue in ((JArray)tasksArray))
                                    {
                                        AsrTask taskInstance = new AsrTask();
                                        propertiesInstance.Tasks.Add(taskInstance);
                                        
                                        JToken groupTaskCustomDetailsValue = tasksValue["groupTaskCustomDetails"];
                                        if (groupTaskCustomDetailsValue != null && groupTaskCustomDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)groupTaskCustomDetailsValue["instanceType"]);
                                            if (typeName == "InlineWorkflowTaskDetails")
                                            {
                                                InlineWorkflowTaskDetails inlineWorkflowTaskDetailsInstance = new InlineWorkflowTaskDetails();
                                                
                                                JToken workflowIdsArray = groupTaskCustomDetailsValue["workflowIds"];
                                                if (workflowIdsArray != null && workflowIdsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken workflowIdsValue in ((JArray)workflowIdsArray))
                                                    {
                                                        inlineWorkflowTaskDetailsInstance.WorkflowIds.Add(((string)workflowIdsValue));
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    inlineWorkflowTaskDetailsInstance.Type = instanceTypeInstance;
                                                }
                                                
                                                JToken childTasksArray = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray != null && childTasksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue in ((JArray)childTasksArray))
                                                    {
                                                        AsrTaskBase taskBaseInstance = new AsrTaskBase();
                                                        inlineWorkflowTaskDetailsInstance.ChildTasks.Add(taskBaseInstance);
                                                        
                                                        JToken taskIdValue = childTasksValue["taskId"];
                                                        if (taskIdValue != null && taskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance = ((string)taskIdValue);
                                                            taskBaseInstance.ID = taskIdInstance;
                                                        }
                                                        
                                                        JToken nameValue = childTasksValue["name"];
                                                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance = ((string)nameValue);
                                                            taskBaseInstance.Name = nameInstance;
                                                        }
                                                        
                                                        JToken startTimeValue2 = childTasksValue["startTime"];
                                                        if (startTimeValue2 != null && startTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance2 = ((DateTime)startTimeValue2);
                                                            taskBaseInstance.StartTime = startTimeInstance2;
                                                        }
                                                        
                                                        JToken endTimeValue2 = childTasksValue["endTime"];
                                                        if (endTimeValue2 != null && endTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance2 = ((DateTime)endTimeValue2);
                                                            taskBaseInstance.EndTime = endTimeInstance2;
                                                        }
                                                        
                                                        JToken allowedActionsArray2 = childTasksValue["allowedActions"];
                                                        if (allowedActionsArray2 != null && allowedActionsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue2 in ((JArray)allowedActionsArray2))
                                                            {
                                                                taskBaseInstance.Actions.Add(((string)allowedActionsValue2));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue2 = childTasksValue["friendlyName"];
                                                        if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                                            taskBaseInstance.TaskFriendlyName = friendlyNameInstance2;
                                                        }
                                                        
                                                        JToken stateValue2 = childTasksValue["state"];
                                                        if (stateValue2 != null && stateValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance2 = ((string)stateValue2);
                                                            taskBaseInstance.State = stateInstance2;
                                                        }
                                                        
                                                        JToken stateDescriptionValue2 = childTasksValue["stateDescription"];
                                                        if (stateDescriptionValue2 != null && stateDescriptionValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance2 = ((string)stateDescriptionValue2);
                                                            taskBaseInstance.StateDescription = stateDescriptionInstance2;
                                                        }
                                                        
                                                        JToken taskTypeValue = childTasksValue["taskType"];
                                                        if (taskTypeValue != null && taskTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance = ((string)taskTypeValue);
                                                            taskBaseInstance.TaskType = taskTypeInstance;
                                                        }
                                                        
                                                        JToken errorsArray = childTasksValue["errors"];
                                                        if (errorsArray != null && errorsArray.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue in ((JArray)errorsArray))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance = new JobErrorDetails();
                                                                taskBaseInstance.Errors.Add(jobErrorDetailsInstance);
                                                                
                                                                JToken serviceErrorDetailsValue = errorsValue["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue != null && serviceErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance = new ServiceError();
                                                                    jobErrorDetailsInstance.ServiceErrorDetails = serviceErrorDetailsInstance;
                                                                    
                                                                    JToken codeValue = serviceErrorDetailsValue["code"];
                                                                    if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance = ((string)codeValue);
                                                                        serviceErrorDetailsInstance.Code = codeInstance;
                                                                    }
                                                                    
                                                                    JToken messageValue = serviceErrorDetailsValue["message"];
                                                                    if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance = ((string)messageValue);
                                                                        serviceErrorDetailsInstance.Message = messageInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue = serviceErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance = ((string)possibleCausesValue);
                                                                        serviceErrorDetailsInstance.PossibleCauses = possibleCausesInstance;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue = serviceErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance = ((string)recommendedActionValue);
                                                                        serviceErrorDetailsInstance.RecommendedAction = recommendedActionInstance;
                                                                    }
                                                                    
                                                                    JToken activityIdValue2 = serviceErrorDetailsValue["activityId"];
                                                                    if (activityIdValue2 != null && activityIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance2 = ((string)activityIdValue2);
                                                                        serviceErrorDetailsInstance.ActivityId = activityIdInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue = errorsValue["providerErrorDetails"];
                                                                if (providerErrorDetailsValue != null && providerErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance = new ProviderError();
                                                                    jobErrorDetailsInstance.ProviderErrorDetails = providerErrorDetailsInstance;
                                                                    
                                                                    JToken errorCodeValue = providerErrorDetailsValue["errorCode"];
                                                                    if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance = ((int)errorCodeValue);
                                                                        providerErrorDetailsInstance.ErrorCode = errorCodeInstance;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue = providerErrorDetailsValue["errorMessage"];
                                                                    if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance = ((string)errorMessageValue);
                                                                        providerErrorDetailsInstance.ErrorMessage = errorMessageInstance;
                                                                    }
                                                                    
                                                                    JToken errorIdValue = providerErrorDetailsValue["errorId"];
                                                                    if (errorIdValue != null && errorIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance = ((string)errorIdValue);
                                                                        providerErrorDetailsInstance.ErrorId = errorIdInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue2 = providerErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue2 != null && possibleCausesValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance2 = ((string)possibleCausesValue2);
                                                                        providerErrorDetailsInstance.PossibleCauses = possibleCausesInstance2;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue2 = providerErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue2 != null && recommendedActionValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance2 = ((string)recommendedActionValue2);
                                                                        providerErrorDetailsInstance.RecommendedAction = recommendedActionInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue = errorsValue["errorLevel"];
                                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                                    jobErrorDetailsInstance.ErrorLevel = errorLevelInstance;
                                                                }
                                                                
                                                                JToken creationTimeValue = errorsValue["creationTime"];
                                                                if (creationTimeValue != null && creationTimeValue.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance = ((DateTime)creationTimeValue);
                                                                    jobErrorDetailsInstance.CreationTime = creationTimeInstance;
                                                                }
                                                                
                                                                JToken taskIdValue2 = errorsValue["taskId"];
                                                                if (taskIdValue2 != null && taskIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance2 = ((string)taskIdValue2);
                                                                    jobErrorDetailsInstance.TaskId = taskIdInstance2;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue = childTasksValue["customDetails"];
                                                        if (customDetailsValue != null && customDetailsValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName2 = ((string)customDetailsValue["instanceType"]);
                                                            if (typeName2 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue = customDetailsValue["jobTask"];
                                                                if (jobTaskValue != null && jobTaskValue.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance = new JobEntity();
                                                                    jobTaskDetailsInstance.JobTask = jobTaskInstance;
                                                                    
                                                                    JToken jobIdValue = jobTaskValue["jobId"];
                                                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance = ((string)jobIdValue);
                                                                        jobTaskInstance.JobId = jobIdInstance;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue = jobTaskValue["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue != null && jobFriendlyNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance = ((string)jobFriendlyNameValue);
                                                                        jobTaskInstance.JobFriendlyName = jobFriendlyNameInstance;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue2 = jobTaskValue["targetObjectId"];
                                                                    if (targetObjectIdValue2 != null && targetObjectIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance2 = ((string)targetObjectIdValue2);
                                                                        jobTaskInstance.TargetObjectId = targetObjectIdInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue2 = jobTaskValue["targetObjectName"];
                                                                    if (targetObjectNameValue2 != null && targetObjectNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance2 = ((string)targetObjectNameValue2);
                                                                        jobTaskInstance.TargetObjectName = targetObjectNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue2 = jobTaskValue["targetInstanceType"];
                                                                    if (targetInstanceTypeValue2 != null && targetInstanceTypeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance2 = ((string)targetInstanceTypeValue2);
                                                                        jobTaskInstance.TargetInstanceType = targetInstanceTypeInstance2;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue = jobTaskValue["jobScenarioName"];
                                                                    if (jobScenarioNameValue != null && jobScenarioNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance = ((string)jobScenarioNameValue);
                                                                        jobTaskInstance.JobScenarioName = jobScenarioNameInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue2 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                                    jobTaskDetailsInstance.Type = instanceTypeInstance2;
                                                                }
                                                                taskBaseInstance.CustomDetails = jobTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue != null && skippedReasonValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance = ((string)skippedReasonValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReason = skippedReasonInstance;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue != null && skippedReasonStringValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance = ((string)skippedReasonStringValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance;
                                                                }
                                                                
                                                                JToken jobTaskValue2 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue2 != null && jobTaskValue2.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance2 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance.JobTask = jobTaskInstance2;
                                                                    
                                                                    JToken jobIdValue2 = jobTaskValue2["jobId"];
                                                                    if (jobIdValue2 != null && jobIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance2 = ((string)jobIdValue2);
                                                                        jobTaskInstance2.JobId = jobIdInstance2;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue2 = jobTaskValue2["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue2 != null && jobFriendlyNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance2 = ((string)jobFriendlyNameValue2);
                                                                        jobTaskInstance2.JobFriendlyName = jobFriendlyNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue3 = jobTaskValue2["targetObjectId"];
                                                                    if (targetObjectIdValue3 != null && targetObjectIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance3 = ((string)targetObjectIdValue3);
                                                                        jobTaskInstance2.TargetObjectId = targetObjectIdInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue3 = jobTaskValue2["targetObjectName"];
                                                                    if (targetObjectNameValue3 != null && targetObjectNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance3 = ((string)targetObjectNameValue3);
                                                                        jobTaskInstance2.TargetObjectName = targetObjectNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue3 = jobTaskValue2["targetInstanceType"];
                                                                    if (targetInstanceTypeValue3 != null && targetInstanceTypeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance3 = ((string)targetInstanceTypeValue3);
                                                                        jobTaskInstance2.TargetInstanceType = targetInstanceTypeInstance3;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue2 = jobTaskValue2["jobScenarioName"];
                                                                    if (jobScenarioNameValue2 != null && jobScenarioNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance2 = ((string)jobScenarioNameValue2);
                                                                        jobTaskInstance2.JobScenarioName = jobScenarioNameInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue3 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                                    virtualMachineTaskDetailsInstance.Type = instanceTypeInstance3;
                                                                }
                                                                taskBaseInstance.CustomDetails = virtualMachineTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue2 = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue2 != null && skippedReasonValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance2 = ((string)skippedReasonValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReason = skippedReasonInstance2;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue2 = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue2 != null && skippedReasonStringValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance2 = ((string)skippedReasonStringValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance2;
                                                                }
                                                                
                                                                JToken jobTaskValue3 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue3 != null && jobTaskValue3.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance3 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance.JobTask = jobTaskInstance3;
                                                                    
                                                                    JToken jobIdValue3 = jobTaskValue3["jobId"];
                                                                    if (jobIdValue3 != null && jobIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance3 = ((string)jobIdValue3);
                                                                        jobTaskInstance3.JobId = jobIdInstance3;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue3 = jobTaskValue3["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue3 != null && jobFriendlyNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance3 = ((string)jobFriendlyNameValue3);
                                                                        jobTaskInstance3.JobFriendlyName = jobFriendlyNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue4 = jobTaskValue3["targetObjectId"];
                                                                    if (targetObjectIdValue4 != null && targetObjectIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance4 = ((string)targetObjectIdValue4);
                                                                        jobTaskInstance3.TargetObjectId = targetObjectIdInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue4 = jobTaskValue3["targetObjectName"];
                                                                    if (targetObjectNameValue4 != null && targetObjectNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance4 = ((string)targetObjectNameValue4);
                                                                        jobTaskInstance3.TargetObjectName = targetObjectNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue4 = jobTaskValue3["targetInstanceType"];
                                                                    if (targetInstanceTypeValue4 != null && targetInstanceTypeValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance4 = ((string)targetInstanceTypeValue4);
                                                                        jobTaskInstance3.TargetInstanceType = targetInstanceTypeInstance4;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue3 = jobTaskValue3["jobScenarioName"];
                                                                    if (jobScenarioNameValue3 != null && jobScenarioNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance3 = ((string)jobScenarioNameValue3);
                                                                        jobTaskInstance3.JobScenarioName = jobScenarioNameInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue4 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance.Type = instanceTypeInstance4;
                                                                }
                                                                taskBaseInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue2 = customDetailsValue["name"];
                                                                if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance2 = ((string)nameValue2);
                                                                    manualActionTaskDetailsInstance.Name = nameInstance2;
                                                                }
                                                                
                                                                JToken instructionsValue = customDetailsValue["instructions"];
                                                                if (instructionsValue != null && instructionsValue.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance = ((string)instructionsValue);
                                                                    manualActionTaskDetailsInstance.Instructions = instructionsInstance;
                                                                }
                                                                
                                                                JToken observationValue = customDetailsValue["observation"];
                                                                if (observationValue != null && observationValue.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance = ((string)observationValue);
                                                                    manualActionTaskDetailsInstance.Observation = observationInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue5 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                                    manualActionTaskDetailsInstance.Type = instanceTypeInstance5;
                                                                }
                                                                taskBaseInstance.CustomDetails = manualActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue3 = customDetailsValue["name"];
                                                                if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance3 = ((string)nameValue3);
                                                                    scriptActionTaskDetailsInstance.Name = nameInstance3;
                                                                }
                                                                
                                                                JToken pathValue = customDetailsValue["path"];
                                                                if (pathValue != null && pathValue.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance = ((string)pathValue);
                                                                    scriptActionTaskDetailsInstance.Path = pathInstance;
                                                                }
                                                                
                                                                JToken outputValue = customDetailsValue["output"];
                                                                if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance = ((string)outputValue);
                                                                    scriptActionTaskDetailsInstance.Output = outputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue != null && isPrimarySideScriptValue.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance = ((bool)isPrimarySideScriptValue);
                                                                    scriptActionTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue6 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                                    scriptActionTaskDetailsInstance.Type = instanceTypeInstance6;
                                                                }
                                                                taskBaseInstance.CustomDetails = scriptActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue = customDetailsValue["vmId"];
                                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance = ((string)vmIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.VmId = vmIdInstance;
                                                                }
                                                                
                                                                JToken nicIdValue = customDetailsValue["nicId"];
                                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance = ((string)nicIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.NicId = nicIdInstance;
                                                                }
                                                                
                                                                JToken nameValue4 = customDetailsValue["name"];
                                                                if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance4 = ((string)nameValue4);
                                                                    vmNicUpdatesTaskDetailsInstance.Name = nameInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue7 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue7 != null && instanceTypeValue7.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance7 = ((string)instanceTypeValue7);
                                                                    vmNicUpdatesTaskDetailsInstance.Type = instanceTypeInstance7;
                                                                }
                                                                taskBaseInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray = customDetailsValue["vmDetails"];
                                                                if (vmDetailsArray != null && vmDetailsArray.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue in ((JArray)vmDetailsArray))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance.VmDetails.Add(inconsistentVmDetailsInstance);
                                                                        
                                                                        JToken vmNameValue = vmDetailsValue["vmName"];
                                                                        if (vmNameValue != null && vmNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance = ((string)vmNameValue);
                                                                            inconsistentVmDetailsInstance.VmName = vmNameInstance;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue = vmDetailsValue["cloudName"];
                                                                        if (cloudNameValue != null && cloudNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance = ((string)cloudNameValue);
                                                                            inconsistentVmDetailsInstance.CloudName = cloudNameInstance;
                                                                        }
                                                                        
                                                                        JToken detailsArray = vmDetailsValue["details"];
                                                                        if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue in ((JArray)detailsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.Details.Add(((string)detailsValue));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray = vmDetailsValue["errorIds"];
                                                                        if (errorIdsArray != null && errorIdsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue in ((JArray)errorIdsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.ErrorIds.Add(((string)errorIdsValue));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue8 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue8 != null && instanceTypeValue8.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance8 = ((string)instanceTypeValue8);
                                                                    consistencyCheckTaskDetailsInstance.Type = instanceTypeInstance8;
                                                                }
                                                                taskBaseInstance.CustomDetails = consistencyCheckTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue5 = customDetailsValue["name"];
                                                                if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance5 = ((string)nameValue5);
                                                                    automationRunbookTaskDetailsInstance.Name = nameInstance5;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue = customDetailsValue["cloudServiceName"];
                                                                if (cloudServiceNameValue != null && cloudServiceNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance = ((string)cloudServiceNameValue);
                                                                    automationRunbookTaskDetailsInstance.CloudServiceName = cloudServiceNameInstance;
                                                                }
                                                                
                                                                JToken subscriptionIdValue = customDetailsValue["subscriptionId"];
                                                                if (subscriptionIdValue != null && subscriptionIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance = ((string)subscriptionIdValue);
                                                                    automationRunbookTaskDetailsInstance.SubscriptionId = subscriptionIdInstance;
                                                                }
                                                                
                                                                JToken accountNameValue = customDetailsValue["accountName"];
                                                                if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance = ((string)accountNameValue);
                                                                    automationRunbookTaskDetailsInstance.AccountName = accountNameInstance;
                                                                }
                                                                
                                                                JToken runbookIdValue = customDetailsValue["runbookId"];
                                                                if (runbookIdValue != null && runbookIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance = ((string)runbookIdValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookId = runbookIdInstance;
                                                                }
                                                                
                                                                JToken runbookNameValue = customDetailsValue["runbookName"];
                                                                if (runbookNameValue != null && runbookNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance = ((string)runbookNameValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookName = runbookNameInstance;
                                                                }
                                                                
                                                                JToken jobIdValue4 = customDetailsValue["jobId"];
                                                                if (jobIdValue4 != null && jobIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance4 = ((string)jobIdValue4);
                                                                    automationRunbookTaskDetailsInstance.JobId = jobIdInstance4;
                                                                }
                                                                
                                                                JToken jobOutputValue = customDetailsValue["jobOutput"];
                                                                if (jobOutputValue != null && jobOutputValue.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance = ((string)jobOutputValue);
                                                                    automationRunbookTaskDetailsInstance.JobOutput = jobOutputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue2 = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue2 != null && isPrimarySideScriptValue2.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance2 = ((bool)isPrimarySideScriptValue2);
                                                                    automationRunbookTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue9 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue9 != null && instanceTypeValue9.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance9 = ((string)instanceTypeValue9);
                                                                    automationRunbookTaskDetailsInstance.Type = instanceTypeInstance9;
                                                                }
                                                                taskBaseInstance.CustomDetails = automationRunbookTaskDetailsInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = inlineWorkflowTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanGroupTaskDetails")
                                            {
                                                RecoveryPlanGroupTaskDetails recoveryPlanGroupTaskDetailsInstance = new RecoveryPlanGroupTaskDetails();
                                                
                                                JToken nameValue6 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                {
                                                    string nameInstance6 = ((string)nameValue6);
                                                    recoveryPlanGroupTaskDetailsInstance.Name = nameInstance6;
                                                }
                                                
                                                JToken groupIdValue = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue != null && groupIdValue.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance = ((string)groupIdValue);
                                                    recoveryPlanGroupTaskDetailsInstance.GroupId = groupIdInstance;
                                                }
                                                
                                                JToken rpGroupTypeValue = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue != null && rpGroupTypeValue.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance = ((string)rpGroupTypeValue);
                                                    recoveryPlanGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance;
                                                }
                                                
                                                JToken instanceTypeValue10 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue10 != null && instanceTypeValue10.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance10 = ((string)instanceTypeValue10);
                                                    recoveryPlanGroupTaskDetailsInstance.Type = instanceTypeInstance10;
                                                }
                                                
                                                JToken childTasksArray2 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray2 != null && childTasksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue2 in ((JArray)childTasksArray2))
                                                    {
                                                        AsrTaskBase taskBaseInstance2 = new AsrTaskBase();
                                                        recoveryPlanGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance2);
                                                        
                                                        JToken taskIdValue3 = childTasksValue2["taskId"];
                                                        if (taskIdValue3 != null && taskIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance3 = ((string)taskIdValue3);
                                                            taskBaseInstance2.ID = taskIdInstance3;
                                                        }
                                                        
                                                        JToken nameValue7 = childTasksValue2["name"];
                                                        if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance7 = ((string)nameValue7);
                                                            taskBaseInstance2.Name = nameInstance7;
                                                        }
                                                        
                                                        JToken startTimeValue3 = childTasksValue2["startTime"];
                                                        if (startTimeValue3 != null && startTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance3 = ((DateTime)startTimeValue3);
                                                            taskBaseInstance2.StartTime = startTimeInstance3;
                                                        }
                                                        
                                                        JToken endTimeValue3 = childTasksValue2["endTime"];
                                                        if (endTimeValue3 != null && endTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance3 = ((DateTime)endTimeValue3);
                                                            taskBaseInstance2.EndTime = endTimeInstance3;
                                                        }
                                                        
                                                        JToken allowedActionsArray3 = childTasksValue2["allowedActions"];
                                                        if (allowedActionsArray3 != null && allowedActionsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue3 in ((JArray)allowedActionsArray3))
                                                            {
                                                                taskBaseInstance2.Actions.Add(((string)allowedActionsValue3));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue3 = childTasksValue2["friendlyName"];
                                                        if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                                            taskBaseInstance2.TaskFriendlyName = friendlyNameInstance3;
                                                        }
                                                        
                                                        JToken stateValue3 = childTasksValue2["state"];
                                                        if (stateValue3 != null && stateValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance3 = ((string)stateValue3);
                                                            taskBaseInstance2.State = stateInstance3;
                                                        }
                                                        
                                                        JToken stateDescriptionValue3 = childTasksValue2["stateDescription"];
                                                        if (stateDescriptionValue3 != null && stateDescriptionValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance3 = ((string)stateDescriptionValue3);
                                                            taskBaseInstance2.StateDescription = stateDescriptionInstance3;
                                                        }
                                                        
                                                        JToken taskTypeValue2 = childTasksValue2["taskType"];
                                                        if (taskTypeValue2 != null && taskTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance2 = ((string)taskTypeValue2);
                                                            taskBaseInstance2.TaskType = taskTypeInstance2;
                                                        }
                                                        
                                                        JToken errorsArray2 = childTasksValue2["errors"];
                                                        if (errorsArray2 != null && errorsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue2 in ((JArray)errorsArray2))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance2 = new JobErrorDetails();
                                                                taskBaseInstance2.Errors.Add(jobErrorDetailsInstance2);
                                                                
                                                                JToken serviceErrorDetailsValue2 = errorsValue2["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue2 != null && serviceErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance2 = new ServiceError();
                                                                    jobErrorDetailsInstance2.ServiceErrorDetails = serviceErrorDetailsInstance2;
                                                                    
                                                                    JToken codeValue2 = serviceErrorDetailsValue2["code"];
                                                                    if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance2 = ((string)codeValue2);
                                                                        serviceErrorDetailsInstance2.Code = codeInstance2;
                                                                    }
                                                                    
                                                                    JToken messageValue2 = serviceErrorDetailsValue2["message"];
                                                                    if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance2 = ((string)messageValue2);
                                                                        serviceErrorDetailsInstance2.Message = messageInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue3 = serviceErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue3 != null && possibleCausesValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance3 = ((string)possibleCausesValue3);
                                                                        serviceErrorDetailsInstance2.PossibleCauses = possibleCausesInstance3;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue3 = serviceErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue3 != null && recommendedActionValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance3 = ((string)recommendedActionValue3);
                                                                        serviceErrorDetailsInstance2.RecommendedAction = recommendedActionInstance3;
                                                                    }
                                                                    
                                                                    JToken activityIdValue3 = serviceErrorDetailsValue2["activityId"];
                                                                    if (activityIdValue3 != null && activityIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance3 = ((string)activityIdValue3);
                                                                        serviceErrorDetailsInstance2.ActivityId = activityIdInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue2 = errorsValue2["providerErrorDetails"];
                                                                if (providerErrorDetailsValue2 != null && providerErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance2 = new ProviderError();
                                                                    jobErrorDetailsInstance2.ProviderErrorDetails = providerErrorDetailsInstance2;
                                                                    
                                                                    JToken errorCodeValue2 = providerErrorDetailsValue2["errorCode"];
                                                                    if (errorCodeValue2 != null && errorCodeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance2 = ((int)errorCodeValue2);
                                                                        providerErrorDetailsInstance2.ErrorCode = errorCodeInstance2;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue2 = providerErrorDetailsValue2["errorMessage"];
                                                                    if (errorMessageValue2 != null && errorMessageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance2 = ((string)errorMessageValue2);
                                                                        providerErrorDetailsInstance2.ErrorMessage = errorMessageInstance2;
                                                                    }
                                                                    
                                                                    JToken errorIdValue2 = providerErrorDetailsValue2["errorId"];
                                                                    if (errorIdValue2 != null && errorIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance2 = ((string)errorIdValue2);
                                                                        providerErrorDetailsInstance2.ErrorId = errorIdInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue4 = providerErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue4 != null && possibleCausesValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance4 = ((string)possibleCausesValue4);
                                                                        providerErrorDetailsInstance2.PossibleCauses = possibleCausesInstance4;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue4 = providerErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue4 != null && recommendedActionValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance4 = ((string)recommendedActionValue4);
                                                                        providerErrorDetailsInstance2.RecommendedAction = recommendedActionInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue2 = errorsValue2["errorLevel"];
                                                                if (errorLevelValue2 != null && errorLevelValue2.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance2 = ((string)errorLevelValue2);
                                                                    jobErrorDetailsInstance2.ErrorLevel = errorLevelInstance2;
                                                                }
                                                                
                                                                JToken creationTimeValue2 = errorsValue2["creationTime"];
                                                                if (creationTimeValue2 != null && creationTimeValue2.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance2 = ((DateTime)creationTimeValue2);
                                                                    jobErrorDetailsInstance2.CreationTime = creationTimeInstance2;
                                                                }
                                                                
                                                                JToken taskIdValue4 = errorsValue2["taskId"];
                                                                if (taskIdValue4 != null && taskIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance4 = ((string)taskIdValue4);
                                                                    jobErrorDetailsInstance2.TaskId = taskIdInstance4;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue2 = childTasksValue2["customDetails"];
                                                        if (customDetailsValue2 != null && customDetailsValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName3 = ((string)customDetailsValue2["instanceType"]);
                                                            if (typeName3 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance2 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue4 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue4 != null && jobTaskValue4.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance4 = new JobEntity();
                                                                    jobTaskDetailsInstance2.JobTask = jobTaskInstance4;
                                                                    
                                                                    JToken jobIdValue5 = jobTaskValue4["jobId"];
                                                                    if (jobIdValue5 != null && jobIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance5 = ((string)jobIdValue5);
                                                                        jobTaskInstance4.JobId = jobIdInstance5;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue4 = jobTaskValue4["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue4 != null && jobFriendlyNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance4 = ((string)jobFriendlyNameValue4);
                                                                        jobTaskInstance4.JobFriendlyName = jobFriendlyNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue5 = jobTaskValue4["targetObjectId"];
                                                                    if (targetObjectIdValue5 != null && targetObjectIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance5 = ((string)targetObjectIdValue5);
                                                                        jobTaskInstance4.TargetObjectId = targetObjectIdInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue5 = jobTaskValue4["targetObjectName"];
                                                                    if (targetObjectNameValue5 != null && targetObjectNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance5 = ((string)targetObjectNameValue5);
                                                                        jobTaskInstance4.TargetObjectName = targetObjectNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue5 = jobTaskValue4["targetInstanceType"];
                                                                    if (targetInstanceTypeValue5 != null && targetInstanceTypeValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance5 = ((string)targetInstanceTypeValue5);
                                                                        jobTaskInstance4.TargetInstanceType = targetInstanceTypeInstance5;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue4 = jobTaskValue4["jobScenarioName"];
                                                                    if (jobScenarioNameValue4 != null && jobScenarioNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance4 = ((string)jobScenarioNameValue4);
                                                                        jobTaskInstance4.JobScenarioName = jobScenarioNameInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue11 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue11 != null && instanceTypeValue11.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance11 = ((string)instanceTypeValue11);
                                                                    jobTaskDetailsInstance2.Type = instanceTypeInstance11;
                                                                }
                                                                taskBaseInstance2.CustomDetails = jobTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance2 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue3 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue3 != null && skippedReasonValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance3 = ((string)skippedReasonValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReason = skippedReasonInstance3;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue3 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue3 != null && skippedReasonStringValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance3 = ((string)skippedReasonStringValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance3;
                                                                }
                                                                
                                                                JToken jobTaskValue5 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue5 != null && jobTaskValue5.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance5 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance2.JobTask = jobTaskInstance5;
                                                                    
                                                                    JToken jobIdValue6 = jobTaskValue5["jobId"];
                                                                    if (jobIdValue6 != null && jobIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance6 = ((string)jobIdValue6);
                                                                        jobTaskInstance5.JobId = jobIdInstance6;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue5 = jobTaskValue5["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue5 != null && jobFriendlyNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance5 = ((string)jobFriendlyNameValue5);
                                                                        jobTaskInstance5.JobFriendlyName = jobFriendlyNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue6 = jobTaskValue5["targetObjectId"];
                                                                    if (targetObjectIdValue6 != null && targetObjectIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance6 = ((string)targetObjectIdValue6);
                                                                        jobTaskInstance5.TargetObjectId = targetObjectIdInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue6 = jobTaskValue5["targetObjectName"];
                                                                    if (targetObjectNameValue6 != null && targetObjectNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance6 = ((string)targetObjectNameValue6);
                                                                        jobTaskInstance5.TargetObjectName = targetObjectNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue6 = jobTaskValue5["targetInstanceType"];
                                                                    if (targetInstanceTypeValue6 != null && targetInstanceTypeValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance6 = ((string)targetInstanceTypeValue6);
                                                                        jobTaskInstance5.TargetInstanceType = targetInstanceTypeInstance6;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue5 = jobTaskValue5["jobScenarioName"];
                                                                    if (jobScenarioNameValue5 != null && jobScenarioNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance5 = ((string)jobScenarioNameValue5);
                                                                        jobTaskInstance5.JobScenarioName = jobScenarioNameInstance5;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue12 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue12 != null && instanceTypeValue12.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance12 = ((string)instanceTypeValue12);
                                                                    virtualMachineTaskDetailsInstance2.Type = instanceTypeInstance12;
                                                                }
                                                                taskBaseInstance2.CustomDetails = virtualMachineTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance2 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue4 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue4 != null && skippedReasonValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance4 = ((string)skippedReasonValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReason = skippedReasonInstance4;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue4 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue4 != null && skippedReasonStringValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance4 = ((string)skippedReasonStringValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance4;
                                                                }
                                                                
                                                                JToken jobTaskValue6 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue6 != null && jobTaskValue6.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance6 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance2.JobTask = jobTaskInstance6;
                                                                    
                                                                    JToken jobIdValue7 = jobTaskValue6["jobId"];
                                                                    if (jobIdValue7 != null && jobIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance7 = ((string)jobIdValue7);
                                                                        jobTaskInstance6.JobId = jobIdInstance7;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue6 = jobTaskValue6["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue6 != null && jobFriendlyNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance6 = ((string)jobFriendlyNameValue6);
                                                                        jobTaskInstance6.JobFriendlyName = jobFriendlyNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue7 = jobTaskValue6["targetObjectId"];
                                                                    if (targetObjectIdValue7 != null && targetObjectIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance7 = ((string)targetObjectIdValue7);
                                                                        jobTaskInstance6.TargetObjectId = targetObjectIdInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue7 = jobTaskValue6["targetObjectName"];
                                                                    if (targetObjectNameValue7 != null && targetObjectNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance7 = ((string)targetObjectNameValue7);
                                                                        jobTaskInstance6.TargetObjectName = targetObjectNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue7 = jobTaskValue6["targetInstanceType"];
                                                                    if (targetInstanceTypeValue7 != null && targetInstanceTypeValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance7 = ((string)targetInstanceTypeValue7);
                                                                        jobTaskInstance6.TargetInstanceType = targetInstanceTypeInstance7;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue6 = jobTaskValue6["jobScenarioName"];
                                                                    if (jobScenarioNameValue6 != null && jobScenarioNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance6 = ((string)jobScenarioNameValue6);
                                                                        jobTaskInstance6.JobScenarioName = jobScenarioNameInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue13 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue13 != null && instanceTypeValue13.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance13 = ((string)instanceTypeValue13);
                                                                    fabricReplicationGroupTaskDetailsInstance2.Type = instanceTypeInstance13;
                                                                }
                                                                taskBaseInstance2.CustomDetails = fabricReplicationGroupTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance2 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue8 = customDetailsValue2["name"];
                                                                if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance8 = ((string)nameValue8);
                                                                    manualActionTaskDetailsInstance2.Name = nameInstance8;
                                                                }
                                                                
                                                                JToken instructionsValue2 = customDetailsValue2["instructions"];
                                                                if (instructionsValue2 != null && instructionsValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance2 = ((string)instructionsValue2);
                                                                    manualActionTaskDetailsInstance2.Instructions = instructionsInstance2;
                                                                }
                                                                
                                                                JToken observationValue2 = customDetailsValue2["observation"];
                                                                if (observationValue2 != null && observationValue2.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance2 = ((string)observationValue2);
                                                                    manualActionTaskDetailsInstance2.Observation = observationInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue14 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue14 != null && instanceTypeValue14.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance14 = ((string)instanceTypeValue14);
                                                                    manualActionTaskDetailsInstance2.Type = instanceTypeInstance14;
                                                                }
                                                                taskBaseInstance2.CustomDetails = manualActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance2 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue9 = customDetailsValue2["name"];
                                                                if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance9 = ((string)nameValue9);
                                                                    scriptActionTaskDetailsInstance2.Name = nameInstance9;
                                                                }
                                                                
                                                                JToken pathValue2 = customDetailsValue2["path"];
                                                                if (pathValue2 != null && pathValue2.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance2 = ((string)pathValue2);
                                                                    scriptActionTaskDetailsInstance2.Path = pathInstance2;
                                                                }
                                                                
                                                                JToken outputValue2 = customDetailsValue2["output"];
                                                                if (outputValue2 != null && outputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance2 = ((string)outputValue2);
                                                                    scriptActionTaskDetailsInstance2.Output = outputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue3 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue3 != null && isPrimarySideScriptValue3.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance3 = ((bool)isPrimarySideScriptValue3);
                                                                    scriptActionTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue15 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue15 != null && instanceTypeValue15.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance15 = ((string)instanceTypeValue15);
                                                                    scriptActionTaskDetailsInstance2.Type = instanceTypeInstance15;
                                                                }
                                                                taskBaseInstance2.CustomDetails = scriptActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance2 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue2 = customDetailsValue2["vmId"];
                                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.VmId = vmIdInstance2;
                                                                }
                                                                
                                                                JToken nicIdValue2 = customDetailsValue2["nicId"];
                                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.NicId = nicIdInstance2;
                                                                }
                                                                
                                                                JToken nameValue10 = customDetailsValue2["name"];
                                                                if (nameValue10 != null && nameValue10.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance10 = ((string)nameValue10);
                                                                    vmNicUpdatesTaskDetailsInstance2.Name = nameInstance10;
                                                                }
                                                                
                                                                JToken instanceTypeValue16 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue16 != null && instanceTypeValue16.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance16 = ((string)instanceTypeValue16);
                                                                    vmNicUpdatesTaskDetailsInstance2.Type = instanceTypeInstance16;
                                                                }
                                                                taskBaseInstance2.CustomDetails = vmNicUpdatesTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance2 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray2 = customDetailsValue2["vmDetails"];
                                                                if (vmDetailsArray2 != null && vmDetailsArray2.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue2 in ((JArray)vmDetailsArray2))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance2 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance2.VmDetails.Add(inconsistentVmDetailsInstance2);
                                                                        
                                                                        JToken vmNameValue2 = vmDetailsValue2["vmName"];
                                                                        if (vmNameValue2 != null && vmNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance2 = ((string)vmNameValue2);
                                                                            inconsistentVmDetailsInstance2.VmName = vmNameInstance2;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue2 = vmDetailsValue2["cloudName"];
                                                                        if (cloudNameValue2 != null && cloudNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance2 = ((string)cloudNameValue2);
                                                                            inconsistentVmDetailsInstance2.CloudName = cloudNameInstance2;
                                                                        }
                                                                        
                                                                        JToken detailsArray2 = vmDetailsValue2["details"];
                                                                        if (detailsArray2 != null && detailsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue2 in ((JArray)detailsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.Details.Add(((string)detailsValue2));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray2 = vmDetailsValue2["errorIds"];
                                                                        if (errorIdsArray2 != null && errorIdsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue2 in ((JArray)errorIdsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.ErrorIds.Add(((string)errorIdsValue2));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue17 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue17 != null && instanceTypeValue17.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance17 = ((string)instanceTypeValue17);
                                                                    consistencyCheckTaskDetailsInstance2.Type = instanceTypeInstance17;
                                                                }
                                                                taskBaseInstance2.CustomDetails = consistencyCheckTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance2 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue11 = customDetailsValue2["name"];
                                                                if (nameValue11 != null && nameValue11.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance11 = ((string)nameValue11);
                                                                    automationRunbookTaskDetailsInstance2.Name = nameInstance11;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue2 = customDetailsValue2["cloudServiceName"];
                                                                if (cloudServiceNameValue2 != null && cloudServiceNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance2 = ((string)cloudServiceNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.CloudServiceName = cloudServiceNameInstance2;
                                                                }
                                                                
                                                                JToken subscriptionIdValue2 = customDetailsValue2["subscriptionId"];
                                                                if (subscriptionIdValue2 != null && subscriptionIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance2 = ((string)subscriptionIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.SubscriptionId = subscriptionIdInstance2;
                                                                }
                                                                
                                                                JToken accountNameValue2 = customDetailsValue2["accountName"];
                                                                if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance2 = ((string)accountNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.AccountName = accountNameInstance2;
                                                                }
                                                                
                                                                JToken runbookIdValue2 = customDetailsValue2["runbookId"];
                                                                if (runbookIdValue2 != null && runbookIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance2 = ((string)runbookIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookId = runbookIdInstance2;
                                                                }
                                                                
                                                                JToken runbookNameValue2 = customDetailsValue2["runbookName"];
                                                                if (runbookNameValue2 != null && runbookNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance2 = ((string)runbookNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookName = runbookNameInstance2;
                                                                }
                                                                
                                                                JToken jobIdValue8 = customDetailsValue2["jobId"];
                                                                if (jobIdValue8 != null && jobIdValue8.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance8 = ((string)jobIdValue8);
                                                                    automationRunbookTaskDetailsInstance2.JobId = jobIdInstance8;
                                                                }
                                                                
                                                                JToken jobOutputValue2 = customDetailsValue2["jobOutput"];
                                                                if (jobOutputValue2 != null && jobOutputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance2 = ((string)jobOutputValue2);
                                                                    automationRunbookTaskDetailsInstance2.JobOutput = jobOutputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue4 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue4 != null && isPrimarySideScriptValue4.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance4 = ((bool)isPrimarySideScriptValue4);
                                                                    automationRunbookTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue18 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue18 != null && instanceTypeValue18.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance18 = ((string)instanceTypeValue18);
                                                                    automationRunbookTaskDetailsInstance2.Type = instanceTypeInstance18;
                                                                }
                                                                taskBaseInstance2.CustomDetails = automationRunbookTaskDetailsInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanGroupTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanShutdownGroupTaskDetails")
                                            {
                                                RecoveryPlanShutdownGroupTaskDetails recoveryPlanShutdownGroupTaskDetailsInstance = new RecoveryPlanShutdownGroupTaskDetails();
                                                
                                                JToken nameValue12 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue12 != null && nameValue12.Type != JTokenType.Null)
                                                {
                                                    string nameInstance12 = ((string)nameValue12);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Name = nameInstance12;
                                                }
                                                
                                                JToken groupIdValue2 = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue2 != null && groupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance2 = ((string)groupIdValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.GroupId = groupIdInstance2;
                                                }
                                                
                                                JToken rpGroupTypeValue2 = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue2 != null && rpGroupTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance2 = ((string)rpGroupTypeValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance2;
                                                }
                                                
                                                JToken instanceTypeValue19 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue19 != null && instanceTypeValue19.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance19 = ((string)instanceTypeValue19);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Type = instanceTypeInstance19;
                                                }
                                                
                                                JToken childTasksArray3 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray3 != null && childTasksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue3 in ((JArray)childTasksArray3))
                                                    {
                                                        AsrTaskBase taskBaseInstance3 = new AsrTaskBase();
                                                        recoveryPlanShutdownGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance3);
                                                        
                                                        JToken taskIdValue5 = childTasksValue3["taskId"];
                                                        if (taskIdValue5 != null && taskIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance5 = ((string)taskIdValue5);
                                                            taskBaseInstance3.ID = taskIdInstance5;
                                                        }
                                                        
                                                        JToken nameValue13 = childTasksValue3["name"];
                                                        if (nameValue13 != null && nameValue13.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance13 = ((string)nameValue13);
                                                            taskBaseInstance3.Name = nameInstance13;
                                                        }
                                                        
                                                        JToken startTimeValue4 = childTasksValue3["startTime"];
                                                        if (startTimeValue4 != null && startTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance4 = ((DateTime)startTimeValue4);
                                                            taskBaseInstance3.StartTime = startTimeInstance4;
                                                        }
                                                        
                                                        JToken endTimeValue4 = childTasksValue3["endTime"];
                                                        if (endTimeValue4 != null && endTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance4 = ((DateTime)endTimeValue4);
                                                            taskBaseInstance3.EndTime = endTimeInstance4;
                                                        }
                                                        
                                                        JToken allowedActionsArray4 = childTasksValue3["allowedActions"];
                                                        if (allowedActionsArray4 != null && allowedActionsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue4 in ((JArray)allowedActionsArray4))
                                                            {
                                                                taskBaseInstance3.Actions.Add(((string)allowedActionsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue4 = childTasksValue3["friendlyName"];
                                                        if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                                            taskBaseInstance3.TaskFriendlyName = friendlyNameInstance4;
                                                        }
                                                        
                                                        JToken stateValue4 = childTasksValue3["state"];
                                                        if (stateValue4 != null && stateValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance4 = ((string)stateValue4);
                                                            taskBaseInstance3.State = stateInstance4;
                                                        }
                                                        
                                                        JToken stateDescriptionValue4 = childTasksValue3["stateDescription"];
                                                        if (stateDescriptionValue4 != null && stateDescriptionValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance4 = ((string)stateDescriptionValue4);
                                                            taskBaseInstance3.StateDescription = stateDescriptionInstance4;
                                                        }
                                                        
                                                        JToken taskTypeValue3 = childTasksValue3["taskType"];
                                                        if (taskTypeValue3 != null && taskTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance3 = ((string)taskTypeValue3);
                                                            taskBaseInstance3.TaskType = taskTypeInstance3;
                                                        }
                                                        
                                                        JToken errorsArray3 = childTasksValue3["errors"];
                                                        if (errorsArray3 != null && errorsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue3 in ((JArray)errorsArray3))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance3 = new JobErrorDetails();
                                                                taskBaseInstance3.Errors.Add(jobErrorDetailsInstance3);
                                                                
                                                                JToken serviceErrorDetailsValue3 = errorsValue3["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue3 != null && serviceErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance3 = new ServiceError();
                                                                    jobErrorDetailsInstance3.ServiceErrorDetails = serviceErrorDetailsInstance3;
                                                                    
                                                                    JToken codeValue3 = serviceErrorDetailsValue3["code"];
                                                                    if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance3 = ((string)codeValue3);
                                                                        serviceErrorDetailsInstance3.Code = codeInstance3;
                                                                    }
                                                                    
                                                                    JToken messageValue3 = serviceErrorDetailsValue3["message"];
                                                                    if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance3 = ((string)messageValue3);
                                                                        serviceErrorDetailsInstance3.Message = messageInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue5 = serviceErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue5 != null && possibleCausesValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance5 = ((string)possibleCausesValue5);
                                                                        serviceErrorDetailsInstance3.PossibleCauses = possibleCausesInstance5;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue5 = serviceErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue5 != null && recommendedActionValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance5 = ((string)recommendedActionValue5);
                                                                        serviceErrorDetailsInstance3.RecommendedAction = recommendedActionInstance5;
                                                                    }
                                                                    
                                                                    JToken activityIdValue4 = serviceErrorDetailsValue3["activityId"];
                                                                    if (activityIdValue4 != null && activityIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance4 = ((string)activityIdValue4);
                                                                        serviceErrorDetailsInstance3.ActivityId = activityIdInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue3 = errorsValue3["providerErrorDetails"];
                                                                if (providerErrorDetailsValue3 != null && providerErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance3 = new ProviderError();
                                                                    jobErrorDetailsInstance3.ProviderErrorDetails = providerErrorDetailsInstance3;
                                                                    
                                                                    JToken errorCodeValue3 = providerErrorDetailsValue3["errorCode"];
                                                                    if (errorCodeValue3 != null && errorCodeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance3 = ((int)errorCodeValue3);
                                                                        providerErrorDetailsInstance3.ErrorCode = errorCodeInstance3;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue3 = providerErrorDetailsValue3["errorMessage"];
                                                                    if (errorMessageValue3 != null && errorMessageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance3 = ((string)errorMessageValue3);
                                                                        providerErrorDetailsInstance3.ErrorMessage = errorMessageInstance3;
                                                                    }
                                                                    
                                                                    JToken errorIdValue3 = providerErrorDetailsValue3["errorId"];
                                                                    if (errorIdValue3 != null && errorIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance3 = ((string)errorIdValue3);
                                                                        providerErrorDetailsInstance3.ErrorId = errorIdInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue6 = providerErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue6 != null && possibleCausesValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance6 = ((string)possibleCausesValue6);
                                                                        providerErrorDetailsInstance3.PossibleCauses = possibleCausesInstance6;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue6 = providerErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue6 != null && recommendedActionValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance6 = ((string)recommendedActionValue6);
                                                                        providerErrorDetailsInstance3.RecommendedAction = recommendedActionInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue3 = errorsValue3["errorLevel"];
                                                                if (errorLevelValue3 != null && errorLevelValue3.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance3 = ((string)errorLevelValue3);
                                                                    jobErrorDetailsInstance3.ErrorLevel = errorLevelInstance3;
                                                                }
                                                                
                                                                JToken creationTimeValue3 = errorsValue3["creationTime"];
                                                                if (creationTimeValue3 != null && creationTimeValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance3 = ((DateTime)creationTimeValue3);
                                                                    jobErrorDetailsInstance3.CreationTime = creationTimeInstance3;
                                                                }
                                                                
                                                                JToken taskIdValue6 = errorsValue3["taskId"];
                                                                if (taskIdValue6 != null && taskIdValue6.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance6 = ((string)taskIdValue6);
                                                                    jobErrorDetailsInstance3.TaskId = taskIdInstance6;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue3 = childTasksValue3["customDetails"];
                                                        if (customDetailsValue3 != null && customDetailsValue3.Type != JTokenType.Null)
                                                        {
                                                            string typeName4 = ((string)customDetailsValue3["instanceType"]);
                                                            if (typeName4 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance3 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue7 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue7 != null && jobTaskValue7.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance7 = new JobEntity();
                                                                    jobTaskDetailsInstance3.JobTask = jobTaskInstance7;
                                                                    
                                                                    JToken jobIdValue9 = jobTaskValue7["jobId"];
                                                                    if (jobIdValue9 != null && jobIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance9 = ((string)jobIdValue9);
                                                                        jobTaskInstance7.JobId = jobIdInstance9;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue7 = jobTaskValue7["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue7 != null && jobFriendlyNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance7 = ((string)jobFriendlyNameValue7);
                                                                        jobTaskInstance7.JobFriendlyName = jobFriendlyNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue8 = jobTaskValue7["targetObjectId"];
                                                                    if (targetObjectIdValue8 != null && targetObjectIdValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance8 = ((string)targetObjectIdValue8);
                                                                        jobTaskInstance7.TargetObjectId = targetObjectIdInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue8 = jobTaskValue7["targetObjectName"];
                                                                    if (targetObjectNameValue8 != null && targetObjectNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance8 = ((string)targetObjectNameValue8);
                                                                        jobTaskInstance7.TargetObjectName = targetObjectNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue8 = jobTaskValue7["targetInstanceType"];
                                                                    if (targetInstanceTypeValue8 != null && targetInstanceTypeValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance8 = ((string)targetInstanceTypeValue8);
                                                                        jobTaskInstance7.TargetInstanceType = targetInstanceTypeInstance8;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue7 = jobTaskValue7["jobScenarioName"];
                                                                    if (jobScenarioNameValue7 != null && jobScenarioNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance7 = ((string)jobScenarioNameValue7);
                                                                        jobTaskInstance7.JobScenarioName = jobScenarioNameInstance7;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue20 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue20 != null && instanceTypeValue20.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance20 = ((string)instanceTypeValue20);
                                                                    jobTaskDetailsInstance3.Type = instanceTypeInstance20;
                                                                }
                                                                taskBaseInstance3.CustomDetails = jobTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance3 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue5 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue5 != null && skippedReasonValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance5 = ((string)skippedReasonValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReason = skippedReasonInstance5;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue5 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue5 != null && skippedReasonStringValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance5 = ((string)skippedReasonStringValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance5;
                                                                }
                                                                
                                                                JToken jobTaskValue8 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue8 != null && jobTaskValue8.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance8 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance3.JobTask = jobTaskInstance8;
                                                                    
                                                                    JToken jobIdValue10 = jobTaskValue8["jobId"];
                                                                    if (jobIdValue10 != null && jobIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance10 = ((string)jobIdValue10);
                                                                        jobTaskInstance8.JobId = jobIdInstance10;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue8 = jobTaskValue8["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue8 != null && jobFriendlyNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance8 = ((string)jobFriendlyNameValue8);
                                                                        jobTaskInstance8.JobFriendlyName = jobFriendlyNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue9 = jobTaskValue8["targetObjectId"];
                                                                    if (targetObjectIdValue9 != null && targetObjectIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance9 = ((string)targetObjectIdValue9);
                                                                        jobTaskInstance8.TargetObjectId = targetObjectIdInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue9 = jobTaskValue8["targetObjectName"];
                                                                    if (targetObjectNameValue9 != null && targetObjectNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance9 = ((string)targetObjectNameValue9);
                                                                        jobTaskInstance8.TargetObjectName = targetObjectNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue9 = jobTaskValue8["targetInstanceType"];
                                                                    if (targetInstanceTypeValue9 != null && targetInstanceTypeValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance9 = ((string)targetInstanceTypeValue9);
                                                                        jobTaskInstance8.TargetInstanceType = targetInstanceTypeInstance9;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue8 = jobTaskValue8["jobScenarioName"];
                                                                    if (jobScenarioNameValue8 != null && jobScenarioNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance8 = ((string)jobScenarioNameValue8);
                                                                        jobTaskInstance8.JobScenarioName = jobScenarioNameInstance8;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue21 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue21 != null && instanceTypeValue21.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance21 = ((string)instanceTypeValue21);
                                                                    virtualMachineTaskDetailsInstance3.Type = instanceTypeInstance21;
                                                                }
                                                                taskBaseInstance3.CustomDetails = virtualMachineTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance3 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue6 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue6 != null && skippedReasonValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance6 = ((string)skippedReasonValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReason = skippedReasonInstance6;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue6 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue6 != null && skippedReasonStringValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance6 = ((string)skippedReasonStringValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance6;
                                                                }
                                                                
                                                                JToken jobTaskValue9 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue9 != null && jobTaskValue9.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance9 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance3.JobTask = jobTaskInstance9;
                                                                    
                                                                    JToken jobIdValue11 = jobTaskValue9["jobId"];
                                                                    if (jobIdValue11 != null && jobIdValue11.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance11 = ((string)jobIdValue11);
                                                                        jobTaskInstance9.JobId = jobIdInstance11;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue9 = jobTaskValue9["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue9 != null && jobFriendlyNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance9 = ((string)jobFriendlyNameValue9);
                                                                        jobTaskInstance9.JobFriendlyName = jobFriendlyNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue10 = jobTaskValue9["targetObjectId"];
                                                                    if (targetObjectIdValue10 != null && targetObjectIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance10 = ((string)targetObjectIdValue10);
                                                                        jobTaskInstance9.TargetObjectId = targetObjectIdInstance10;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue10 = jobTaskValue9["targetObjectName"];
                                                                    if (targetObjectNameValue10 != null && targetObjectNameValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance10 = ((string)targetObjectNameValue10);
                                                                        jobTaskInstance9.TargetObjectName = targetObjectNameInstance10;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue10 = jobTaskValue9["targetInstanceType"];
                                                                    if (targetInstanceTypeValue10 != null && targetInstanceTypeValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance10 = ((string)targetInstanceTypeValue10);
                                                                        jobTaskInstance9.TargetInstanceType = targetInstanceTypeInstance10;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue9 = jobTaskValue9["jobScenarioName"];
                                                                    if (jobScenarioNameValue9 != null && jobScenarioNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance9 = ((string)jobScenarioNameValue9);
                                                                        jobTaskInstance9.JobScenarioName = jobScenarioNameInstance9;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue22 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue22 != null && instanceTypeValue22.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance22 = ((string)instanceTypeValue22);
                                                                    fabricReplicationGroupTaskDetailsInstance3.Type = instanceTypeInstance22;
                                                                }
                                                                taskBaseInstance3.CustomDetails = fabricReplicationGroupTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance3 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue14 = customDetailsValue3["name"];
                                                                if (nameValue14 != null && nameValue14.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance14 = ((string)nameValue14);
                                                                    manualActionTaskDetailsInstance3.Name = nameInstance14;
                                                                }
                                                                
                                                                JToken instructionsValue3 = customDetailsValue3["instructions"];
                                                                if (instructionsValue3 != null && instructionsValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance3 = ((string)instructionsValue3);
                                                                    manualActionTaskDetailsInstance3.Instructions = instructionsInstance3;
                                                                }
                                                                
                                                                JToken observationValue3 = customDetailsValue3["observation"];
                                                                if (observationValue3 != null && observationValue3.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance3 = ((string)observationValue3);
                                                                    manualActionTaskDetailsInstance3.Observation = observationInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue23 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue23 != null && instanceTypeValue23.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance23 = ((string)instanceTypeValue23);
                                                                    manualActionTaskDetailsInstance3.Type = instanceTypeInstance23;
                                                                }
                                                                taskBaseInstance3.CustomDetails = manualActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance3 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue15 = customDetailsValue3["name"];
                                                                if (nameValue15 != null && nameValue15.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance15 = ((string)nameValue15);
                                                                    scriptActionTaskDetailsInstance3.Name = nameInstance15;
                                                                }
                                                                
                                                                JToken pathValue3 = customDetailsValue3["path"];
                                                                if (pathValue3 != null && pathValue3.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance3 = ((string)pathValue3);
                                                                    scriptActionTaskDetailsInstance3.Path = pathInstance3;
                                                                }
                                                                
                                                                JToken outputValue3 = customDetailsValue3["output"];
                                                                if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance3 = ((string)outputValue3);
                                                                    scriptActionTaskDetailsInstance3.Output = outputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue5 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue5 != null && isPrimarySideScriptValue5.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance5 = ((bool)isPrimarySideScriptValue5);
                                                                    scriptActionTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance5;
                                                                }
                                                                
                                                                JToken instanceTypeValue24 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue24 != null && instanceTypeValue24.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance24 = ((string)instanceTypeValue24);
                                                                    scriptActionTaskDetailsInstance3.Type = instanceTypeInstance24;
                                                                }
                                                                taskBaseInstance3.CustomDetails = scriptActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance3 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue3 = customDetailsValue3["vmId"];
                                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.VmId = vmIdInstance3;
                                                                }
                                                                
                                                                JToken nicIdValue3 = customDetailsValue3["nicId"];
                                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.NicId = nicIdInstance3;
                                                                }
                                                                
                                                                JToken nameValue16 = customDetailsValue3["name"];
                                                                if (nameValue16 != null && nameValue16.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance16 = ((string)nameValue16);
                                                                    vmNicUpdatesTaskDetailsInstance3.Name = nameInstance16;
                                                                }
                                                                
                                                                JToken instanceTypeValue25 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue25 != null && instanceTypeValue25.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance25 = ((string)instanceTypeValue25);
                                                                    vmNicUpdatesTaskDetailsInstance3.Type = instanceTypeInstance25;
                                                                }
                                                                taskBaseInstance3.CustomDetails = vmNicUpdatesTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance3 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray3 = customDetailsValue3["vmDetails"];
                                                                if (vmDetailsArray3 != null && vmDetailsArray3.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue3 in ((JArray)vmDetailsArray3))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance3 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance3.VmDetails.Add(inconsistentVmDetailsInstance3);
                                                                        
                                                                        JToken vmNameValue3 = vmDetailsValue3["vmName"];
                                                                        if (vmNameValue3 != null && vmNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance3 = ((string)vmNameValue3);
                                                                            inconsistentVmDetailsInstance3.VmName = vmNameInstance3;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue3 = vmDetailsValue3["cloudName"];
                                                                        if (cloudNameValue3 != null && cloudNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance3 = ((string)cloudNameValue3);
                                                                            inconsistentVmDetailsInstance3.CloudName = cloudNameInstance3;
                                                                        }
                                                                        
                                                                        JToken detailsArray3 = vmDetailsValue3["details"];
                                                                        if (detailsArray3 != null && detailsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue3 in ((JArray)detailsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.Details.Add(((string)detailsValue3));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray3 = vmDetailsValue3["errorIds"];
                                                                        if (errorIdsArray3 != null && errorIdsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue3 in ((JArray)errorIdsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.ErrorIds.Add(((string)errorIdsValue3));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue26 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue26 != null && instanceTypeValue26.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance26 = ((string)instanceTypeValue26);
                                                                    consistencyCheckTaskDetailsInstance3.Type = instanceTypeInstance26;
                                                                }
                                                                taskBaseInstance3.CustomDetails = consistencyCheckTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance3 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue17 = customDetailsValue3["name"];
                                                                if (nameValue17 != null && nameValue17.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance17 = ((string)nameValue17);
                                                                    automationRunbookTaskDetailsInstance3.Name = nameInstance17;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue3 = customDetailsValue3["cloudServiceName"];
                                                                if (cloudServiceNameValue3 != null && cloudServiceNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance3 = ((string)cloudServiceNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.CloudServiceName = cloudServiceNameInstance3;
                                                                }
                                                                
                                                                JToken subscriptionIdValue3 = customDetailsValue3["subscriptionId"];
                                                                if (subscriptionIdValue3 != null && subscriptionIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance3 = ((string)subscriptionIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.SubscriptionId = subscriptionIdInstance3;
                                                                }
                                                                
                                                                JToken accountNameValue3 = customDetailsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken runbookIdValue3 = customDetailsValue3["runbookId"];
                                                                if (runbookIdValue3 != null && runbookIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance3 = ((string)runbookIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookId = runbookIdInstance3;
                                                                }
                                                                
                                                                JToken runbookNameValue3 = customDetailsValue3["runbookName"];
                                                                if (runbookNameValue3 != null && runbookNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance3 = ((string)runbookNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookName = runbookNameInstance3;
                                                                }
                                                                
                                                                JToken jobIdValue12 = customDetailsValue3["jobId"];
                                                                if (jobIdValue12 != null && jobIdValue12.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance12 = ((string)jobIdValue12);
                                                                    automationRunbookTaskDetailsInstance3.JobId = jobIdInstance12;
                                                                }
                                                                
                                                                JToken jobOutputValue3 = customDetailsValue3["jobOutput"];
                                                                if (jobOutputValue3 != null && jobOutputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance3 = ((string)jobOutputValue3);
                                                                    automationRunbookTaskDetailsInstance3.JobOutput = jobOutputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue6 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue6 != null && isPrimarySideScriptValue6.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance6 = ((bool)isPrimarySideScriptValue6);
                                                                    automationRunbookTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance6;
                                                                }
                                                                
                                                                JToken instanceTypeValue27 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue27 != null && instanceTypeValue27.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance27 = ((string)instanceTypeValue27);
                                                                    automationRunbookTaskDetailsInstance3.Type = instanceTypeInstance27;
                                                                }
                                                                taskBaseInstance3.CustomDetails = automationRunbookTaskDetailsInstance3;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanShutdownGroupTaskDetailsInstance;
                                            }
                                        }
                                        
                                        JToken taskIdValue7 = tasksValue["taskId"];
                                        if (taskIdValue7 != null && taskIdValue7.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance7 = ((string)taskIdValue7);
                                            taskInstance.ID = taskIdInstance7;
                                        }
                                        
                                        JToken nameValue18 = tasksValue["name"];
                                        if (nameValue18 != null && nameValue18.Type != JTokenType.Null)
                                        {
                                            string nameInstance18 = ((string)nameValue18);
                                            taskInstance.Name = nameInstance18;
                                        }
                                        
                                        JToken startTimeValue5 = tasksValue["startTime"];
                                        if (startTimeValue5 != null && startTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime startTimeInstance5 = ((DateTime)startTimeValue5);
                                            taskInstance.StartTime = startTimeInstance5;
                                        }
                                        
                                        JToken endTimeValue5 = tasksValue["endTime"];
                                        if (endTimeValue5 != null && endTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime endTimeInstance5 = ((DateTime)endTimeValue5);
                                            taskInstance.EndTime = endTimeInstance5;
                                        }
                                        
                                        JToken allowedActionsArray5 = tasksValue["allowedActions"];
                                        if (allowedActionsArray5 != null && allowedActionsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedActionsValue5 in ((JArray)allowedActionsArray5))
                                            {
                                                taskInstance.Actions.Add(((string)allowedActionsValue5));
                                            }
                                        }
                                        
                                        JToken friendlyNameValue5 = tasksValue["friendlyName"];
                                        if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                            taskInstance.TaskFriendlyName = friendlyNameInstance5;
                                        }
                                        
                                        JToken stateValue5 = tasksValue["state"];
                                        if (stateValue5 != null && stateValue5.Type != JTokenType.Null)
                                        {
                                            string stateInstance5 = ((string)stateValue5);
                                            taskInstance.State = stateInstance5;
                                        }
                                        
                                        JToken stateDescriptionValue5 = tasksValue["stateDescription"];
                                        if (stateDescriptionValue5 != null && stateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string stateDescriptionInstance5 = ((string)stateDescriptionValue5);
                                            taskInstance.StateDescription = stateDescriptionInstance5;
                                        }
                                        
                                        JToken taskTypeValue4 = tasksValue["taskType"];
                                        if (taskTypeValue4 != null && taskTypeValue4.Type != JTokenType.Null)
                                        {
                                            string taskTypeInstance4 = ((string)taskTypeValue4);
                                            taskInstance.TaskType = taskTypeInstance4;
                                        }
                                        
                                        JToken errorsArray4 = tasksValue["errors"];
                                        if (errorsArray4 != null && errorsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken errorsValue4 in ((JArray)errorsArray4))
                                            {
                                                JobErrorDetails jobErrorDetailsInstance4 = new JobErrorDetails();
                                                taskInstance.Errors.Add(jobErrorDetailsInstance4);
                                                
                                                JToken serviceErrorDetailsValue4 = errorsValue4["serviceErrorDetails"];
                                                if (serviceErrorDetailsValue4 != null && serviceErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ServiceError serviceErrorDetailsInstance4 = new ServiceError();
                                                    jobErrorDetailsInstance4.ServiceErrorDetails = serviceErrorDetailsInstance4;
                                                    
                                                    JToken codeValue4 = serviceErrorDetailsValue4["code"];
                                                    if (codeValue4 != null && codeValue4.Type != JTokenType.Null)
                                                    {
                                                        string codeInstance4 = ((string)codeValue4);
                                                        serviceErrorDetailsInstance4.Code = codeInstance4;
                                                    }
                                                    
                                                    JToken messageValue4 = serviceErrorDetailsValue4["message"];
                                                    if (messageValue4 != null && messageValue4.Type != JTokenType.Null)
                                                    {
                                                        string messageInstance4 = ((string)messageValue4);
                                                        serviceErrorDetailsInstance4.Message = messageInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue7 = serviceErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue7 != null && possibleCausesValue7.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance7 = ((string)possibleCausesValue7);
                                                        serviceErrorDetailsInstance4.PossibleCauses = possibleCausesInstance7;
                                                    }
                                                    
                                                    JToken recommendedActionValue7 = serviceErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue7 != null && recommendedActionValue7.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance7 = ((string)recommendedActionValue7);
                                                        serviceErrorDetailsInstance4.RecommendedAction = recommendedActionInstance7;
                                                    }
                                                    
                                                    JToken activityIdValue5 = serviceErrorDetailsValue4["activityId"];
                                                    if (activityIdValue5 != null && activityIdValue5.Type != JTokenType.Null)
                                                    {
                                                        string activityIdInstance5 = ((string)activityIdValue5);
                                                        serviceErrorDetailsInstance4.ActivityId = activityIdInstance5;
                                                    }
                                                }
                                                
                                                JToken providerErrorDetailsValue4 = errorsValue4["providerErrorDetails"];
                                                if (providerErrorDetailsValue4 != null && providerErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ProviderError providerErrorDetailsInstance4 = new ProviderError();
                                                    jobErrorDetailsInstance4.ProviderErrorDetails = providerErrorDetailsInstance4;
                                                    
                                                    JToken errorCodeValue4 = providerErrorDetailsValue4["errorCode"];
                                                    if (errorCodeValue4 != null && errorCodeValue4.Type != JTokenType.Null)
                                                    {
                                                        int errorCodeInstance4 = ((int)errorCodeValue4);
                                                        providerErrorDetailsInstance4.ErrorCode = errorCodeInstance4;
                                                    }
                                                    
                                                    JToken errorMessageValue4 = providerErrorDetailsValue4["errorMessage"];
                                                    if (errorMessageValue4 != null && errorMessageValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorMessageInstance4 = ((string)errorMessageValue4);
                                                        providerErrorDetailsInstance4.ErrorMessage = errorMessageInstance4;
                                                    }
                                                    
                                                    JToken errorIdValue4 = providerErrorDetailsValue4["errorId"];
                                                    if (errorIdValue4 != null && errorIdValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorIdInstance4 = ((string)errorIdValue4);
                                                        providerErrorDetailsInstance4.ErrorId = errorIdInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue8 = providerErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue8 != null && possibleCausesValue8.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance8 = ((string)possibleCausesValue8);
                                                        providerErrorDetailsInstance4.PossibleCauses = possibleCausesInstance8;
                                                    }
                                                    
                                                    JToken recommendedActionValue8 = providerErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue8 != null && recommendedActionValue8.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance8 = ((string)recommendedActionValue8);
                                                        providerErrorDetailsInstance4.RecommendedAction = recommendedActionInstance8;
                                                    }
                                                }
                                                
                                                JToken errorLevelValue4 = errorsValue4["errorLevel"];
                                                if (errorLevelValue4 != null && errorLevelValue4.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance4 = ((string)errorLevelValue4);
                                                    jobErrorDetailsInstance4.ErrorLevel = errorLevelInstance4;
                                                }
                                                
                                                JToken creationTimeValue4 = errorsValue4["creationTime"];
                                                if (creationTimeValue4 != null && creationTimeValue4.Type != JTokenType.Null)
                                                {
                                                    DateTime creationTimeInstance4 = ((DateTime)creationTimeValue4);
                                                    jobErrorDetailsInstance4.CreationTime = creationTimeInstance4;
                                                }
                                                
                                                JToken taskIdValue8 = errorsValue4["taskId"];
                                                if (taskIdValue8 != null && taskIdValue8.Type != JTokenType.Null)
                                                {
                                                    string taskIdInstance8 = ((string)taskIdValue8);
                                                    jobErrorDetailsInstance4.TaskId = taskIdInstance8;
                                                }
                                            }
                                        }
                                        
                                        JToken customDetailsValue4 = tasksValue["customDetails"];
                                        if (customDetailsValue4 != null && customDetailsValue4.Type != JTokenType.Null)
                                        {
                                            string typeName5 = ((string)customDetailsValue4["instanceType"]);
                                            if (typeName5 == "JobTaskDetails")
                                            {
                                                JobTaskDetails jobTaskDetailsInstance4 = new JobTaskDetails();
                                                
                                                JToken jobTaskValue10 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue10 != null && jobTaskValue10.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance10 = new JobEntity();
                                                    jobTaskDetailsInstance4.JobTask = jobTaskInstance10;
                                                    
                                                    JToken jobIdValue13 = jobTaskValue10["jobId"];
                                                    if (jobIdValue13 != null && jobIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance13 = ((string)jobIdValue13);
                                                        jobTaskInstance10.JobId = jobIdInstance13;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue10 = jobTaskValue10["jobFriendlyName"];
                                                    if (jobFriendlyNameValue10 != null && jobFriendlyNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance10 = ((string)jobFriendlyNameValue10);
                                                        jobTaskInstance10.JobFriendlyName = jobFriendlyNameInstance10;
                                                    }
                                                    
                                                    JToken targetObjectIdValue11 = jobTaskValue10["targetObjectId"];
                                                    if (targetObjectIdValue11 != null && targetObjectIdValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance11 = ((string)targetObjectIdValue11);
                                                        jobTaskInstance10.TargetObjectId = targetObjectIdInstance11;
                                                    }
                                                    
                                                    JToken targetObjectNameValue11 = jobTaskValue10["targetObjectName"];
                                                    if (targetObjectNameValue11 != null && targetObjectNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance11 = ((string)targetObjectNameValue11);
                                                        jobTaskInstance10.TargetObjectName = targetObjectNameInstance11;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue11 = jobTaskValue10["targetInstanceType"];
                                                    if (targetInstanceTypeValue11 != null && targetInstanceTypeValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance11 = ((string)targetInstanceTypeValue11);
                                                        jobTaskInstance10.TargetInstanceType = targetInstanceTypeInstance11;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue10 = jobTaskValue10["jobScenarioName"];
                                                    if (jobScenarioNameValue10 != null && jobScenarioNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance10 = ((string)jobScenarioNameValue10);
                                                        jobTaskInstance10.JobScenarioName = jobScenarioNameInstance10;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue28 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue28 != null && instanceTypeValue28.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance28 = ((string)instanceTypeValue28);
                                                    jobTaskDetailsInstance4.Type = instanceTypeInstance28;
                                                }
                                                taskInstance.CustomDetails = jobTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VirtualMachineTaskDetails")
                                            {
                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance4 = new VirtualMachineTaskDetails();
                                                
                                                JToken skippedReasonValue7 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue7 != null && skippedReasonValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance7 = ((string)skippedReasonValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReason = skippedReasonInstance7;
                                                }
                                                
                                                JToken skippedReasonStringValue7 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue7 != null && skippedReasonStringValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance7 = ((string)skippedReasonStringValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance7;
                                                }
                                                
                                                JToken jobTaskValue11 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue11 != null && jobTaskValue11.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance11 = new JobEntity();
                                                    virtualMachineTaskDetailsInstance4.JobTask = jobTaskInstance11;
                                                    
                                                    JToken jobIdValue14 = jobTaskValue11["jobId"];
                                                    if (jobIdValue14 != null && jobIdValue14.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance14 = ((string)jobIdValue14);
                                                        jobTaskInstance11.JobId = jobIdInstance14;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue11 = jobTaskValue11["jobFriendlyName"];
                                                    if (jobFriendlyNameValue11 != null && jobFriendlyNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance11 = ((string)jobFriendlyNameValue11);
                                                        jobTaskInstance11.JobFriendlyName = jobFriendlyNameInstance11;
                                                    }
                                                    
                                                    JToken targetObjectIdValue12 = jobTaskValue11["targetObjectId"];
                                                    if (targetObjectIdValue12 != null && targetObjectIdValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance12 = ((string)targetObjectIdValue12);
                                                        jobTaskInstance11.TargetObjectId = targetObjectIdInstance12;
                                                    }
                                                    
                                                    JToken targetObjectNameValue12 = jobTaskValue11["targetObjectName"];
                                                    if (targetObjectNameValue12 != null && targetObjectNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance12 = ((string)targetObjectNameValue12);
                                                        jobTaskInstance11.TargetObjectName = targetObjectNameInstance12;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue12 = jobTaskValue11["targetInstanceType"];
                                                    if (targetInstanceTypeValue12 != null && targetInstanceTypeValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance12 = ((string)targetInstanceTypeValue12);
                                                        jobTaskInstance11.TargetInstanceType = targetInstanceTypeInstance12;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue11 = jobTaskValue11["jobScenarioName"];
                                                    if (jobScenarioNameValue11 != null && jobScenarioNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance11 = ((string)jobScenarioNameValue11);
                                                        jobTaskInstance11.JobScenarioName = jobScenarioNameInstance11;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue29 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue29 != null && instanceTypeValue29.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance29 = ((string)instanceTypeValue29);
                                                    virtualMachineTaskDetailsInstance4.Type = instanceTypeInstance29;
                                                }
                                                taskInstance.CustomDetails = virtualMachineTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "FabricReplicationGroupTaskDetails")
                                            {
                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance4 = new FabricReplicationGroupTaskDetails();
                                                
                                                JToken skippedReasonValue8 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue8 != null && skippedReasonValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance8 = ((string)skippedReasonValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReason = skippedReasonInstance8;
                                                }
                                                
                                                JToken skippedReasonStringValue8 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue8 != null && skippedReasonStringValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance8 = ((string)skippedReasonStringValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance8;
                                                }
                                                
                                                JToken jobTaskValue12 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue12 != null && jobTaskValue12.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance12 = new JobEntity();
                                                    fabricReplicationGroupTaskDetailsInstance4.JobTask = jobTaskInstance12;
                                                    
                                                    JToken jobIdValue15 = jobTaskValue12["jobId"];
                                                    if (jobIdValue15 != null && jobIdValue15.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance15 = ((string)jobIdValue15);
                                                        jobTaskInstance12.JobId = jobIdInstance15;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue12 = jobTaskValue12["jobFriendlyName"];
                                                    if (jobFriendlyNameValue12 != null && jobFriendlyNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance12 = ((string)jobFriendlyNameValue12);
                                                        jobTaskInstance12.JobFriendlyName = jobFriendlyNameInstance12;
                                                    }
                                                    
                                                    JToken targetObjectIdValue13 = jobTaskValue12["targetObjectId"];
                                                    if (targetObjectIdValue13 != null && targetObjectIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance13 = ((string)targetObjectIdValue13);
                                                        jobTaskInstance12.TargetObjectId = targetObjectIdInstance13;
                                                    }
                                                    
                                                    JToken targetObjectNameValue13 = jobTaskValue12["targetObjectName"];
                                                    if (targetObjectNameValue13 != null && targetObjectNameValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance13 = ((string)targetObjectNameValue13);
                                                        jobTaskInstance12.TargetObjectName = targetObjectNameInstance13;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue13 = jobTaskValue12["targetInstanceType"];
                                                    if (targetInstanceTypeValue13 != null && targetInstanceTypeValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance13 = ((string)targetInstanceTypeValue13);
                                                        jobTaskInstance12.TargetInstanceType = targetInstanceTypeInstance13;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue12 = jobTaskValue12["jobScenarioName"];
                                                    if (jobScenarioNameValue12 != null && jobScenarioNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance12 = ((string)jobScenarioNameValue12);
                                                        jobTaskInstance12.JobScenarioName = jobScenarioNameInstance12;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue30 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue30 != null && instanceTypeValue30.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance30 = ((string)instanceTypeValue30);
                                                    fabricReplicationGroupTaskDetailsInstance4.Type = instanceTypeInstance30;
                                                }
                                                taskInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ManualActionTaskDetails")
                                            {
                                                ManualActionTaskDetails manualActionTaskDetailsInstance4 = new ManualActionTaskDetails();
                                                
                                                JToken nameValue19 = customDetailsValue4["name"];
                                                if (nameValue19 != null && nameValue19.Type != JTokenType.Null)
                                                {
                                                    string nameInstance19 = ((string)nameValue19);
                                                    manualActionTaskDetailsInstance4.Name = nameInstance19;
                                                }
                                                
                                                JToken instructionsValue4 = customDetailsValue4["instructions"];
                                                if (instructionsValue4 != null && instructionsValue4.Type != JTokenType.Null)
                                                {
                                                    string instructionsInstance4 = ((string)instructionsValue4);
                                                    manualActionTaskDetailsInstance4.Instructions = instructionsInstance4;
                                                }
                                                
                                                JToken observationValue4 = customDetailsValue4["observation"];
                                                if (observationValue4 != null && observationValue4.Type != JTokenType.Null)
                                                {
                                                    string observationInstance4 = ((string)observationValue4);
                                                    manualActionTaskDetailsInstance4.Observation = observationInstance4;
                                                }
                                                
                                                JToken instanceTypeValue31 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue31 != null && instanceTypeValue31.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance31 = ((string)instanceTypeValue31);
                                                    manualActionTaskDetailsInstance4.Type = instanceTypeInstance31;
                                                }
                                                taskInstance.CustomDetails = manualActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ScriptActionTaskDetails")
                                            {
                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance4 = new ScriptActionTaskDetails();
                                                
                                                JToken nameValue20 = customDetailsValue4["name"];
                                                if (nameValue20 != null && nameValue20.Type != JTokenType.Null)
                                                {
                                                    string nameInstance20 = ((string)nameValue20);
                                                    scriptActionTaskDetailsInstance4.Name = nameInstance20;
                                                }
                                                
                                                JToken pathValue4 = customDetailsValue4["path"];
                                                if (pathValue4 != null && pathValue4.Type != JTokenType.Null)
                                                {
                                                    string pathInstance4 = ((string)pathValue4);
                                                    scriptActionTaskDetailsInstance4.Path = pathInstance4;
                                                }
                                                
                                                JToken outputValue4 = customDetailsValue4["output"];
                                                if (outputValue4 != null && outputValue4.Type != JTokenType.Null)
                                                {
                                                    string outputInstance4 = ((string)outputValue4);
                                                    scriptActionTaskDetailsInstance4.Output = outputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue7 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue7 != null && isPrimarySideScriptValue7.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance7 = ((bool)isPrimarySideScriptValue7);
                                                    scriptActionTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance7;
                                                }
                                                
                                                JToken instanceTypeValue32 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue32 != null && instanceTypeValue32.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance32 = ((string)instanceTypeValue32);
                                                    scriptActionTaskDetailsInstance4.Type = instanceTypeInstance32;
                                                }
                                                taskInstance.CustomDetails = scriptActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VmNicUpdatesTaskDetails")
                                            {
                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance4 = new VmNicUpdatesTaskDetails();
                                                
                                                JToken vmIdValue4 = customDetailsValue4["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken nicIdValue4 = customDetailsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken nameValue21 = customDetailsValue4["name"];
                                                if (nameValue21 != null && nameValue21.Type != JTokenType.Null)
                                                {
                                                    string nameInstance21 = ((string)nameValue21);
                                                    vmNicUpdatesTaskDetailsInstance4.Name = nameInstance21;
                                                }
                                                
                                                JToken instanceTypeValue33 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue33 != null && instanceTypeValue33.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance33 = ((string)instanceTypeValue33);
                                                    vmNicUpdatesTaskDetailsInstance4.Type = instanceTypeInstance33;
                                                }
                                                taskInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ConsistencyCheckTaskDetails")
                                            {
                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance4 = new ConsistencyCheckTaskDetails();
                                                
                                                JToken vmDetailsArray4 = customDetailsValue4["vmDetails"];
                                                if (vmDetailsArray4 != null && vmDetailsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmDetailsValue4 in ((JArray)vmDetailsArray4))
                                                    {
                                                        InconsistentVmDetails inconsistentVmDetailsInstance4 = new InconsistentVmDetails();
                                                        consistencyCheckTaskDetailsInstance4.VmDetails.Add(inconsistentVmDetailsInstance4);
                                                        
                                                        JToken vmNameValue4 = vmDetailsValue4["vmName"];
                                                        if (vmNameValue4 != null && vmNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vmNameInstance4 = ((string)vmNameValue4);
                                                            inconsistentVmDetailsInstance4.VmName = vmNameInstance4;
                                                        }
                                                        
                                                        JToken cloudNameValue4 = vmDetailsValue4["cloudName"];
                                                        if (cloudNameValue4 != null && cloudNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string cloudNameInstance4 = ((string)cloudNameValue4);
                                                            inconsistentVmDetailsInstance4.CloudName = cloudNameInstance4;
                                                        }
                                                        
                                                        JToken detailsArray4 = vmDetailsValue4["details"];
                                                        if (detailsArray4 != null && detailsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken detailsValue4 in ((JArray)detailsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.Details.Add(((string)detailsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken errorIdsArray4 = vmDetailsValue4["errorIds"];
                                                        if (errorIdsArray4 != null && errorIdsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorIdsValue4 in ((JArray)errorIdsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.ErrorIds.Add(((string)errorIdsValue4));
                                                            }
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue34 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue34 != null && instanceTypeValue34.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance34 = ((string)instanceTypeValue34);
                                                    consistencyCheckTaskDetailsInstance4.Type = instanceTypeInstance34;
                                                }
                                                taskInstance.CustomDetails = consistencyCheckTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "AutomationRunbookTaskDetails")
                                            {
                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance4 = new AutomationRunbookTaskDetails();
                                                
                                                JToken nameValue22 = customDetailsValue4["name"];
                                                if (nameValue22 != null && nameValue22.Type != JTokenType.Null)
                                                {
                                                    string nameInstance22 = ((string)nameValue22);
                                                    automationRunbookTaskDetailsInstance4.Name = nameInstance22;
                                                }
                                                
                                                JToken cloudServiceNameValue4 = customDetailsValue4["cloudServiceName"];
                                                if (cloudServiceNameValue4 != null && cloudServiceNameValue4.Type != JTokenType.Null)
                                                {
                                                    string cloudServiceNameInstance4 = ((string)cloudServiceNameValue4);
                                                    automationRunbookTaskDetailsInstance4.CloudServiceName = cloudServiceNameInstance4;
                                                }
                                                
                                                JToken subscriptionIdValue4 = customDetailsValue4["subscriptionId"];
                                                if (subscriptionIdValue4 != null && subscriptionIdValue4.Type != JTokenType.Null)
                                                {
                                                    string subscriptionIdInstance4 = ((string)subscriptionIdValue4);
                                                    automationRunbookTaskDetailsInstance4.SubscriptionId = subscriptionIdInstance4;
                                                }
                                                
                                                JToken accountNameValue4 = customDetailsValue4["accountName"];
                                                if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                {
                                                    string accountNameInstance4 = ((string)accountNameValue4);
                                                    automationRunbookTaskDetailsInstance4.AccountName = accountNameInstance4;
                                                }
                                                
                                                JToken runbookIdValue4 = customDetailsValue4["runbookId"];
                                                if (runbookIdValue4 != null && runbookIdValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookIdInstance4 = ((string)runbookIdValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookId = runbookIdInstance4;
                                                }
                                                
                                                JToken runbookNameValue4 = customDetailsValue4["runbookName"];
                                                if (runbookNameValue4 != null && runbookNameValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookNameInstance4 = ((string)runbookNameValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookName = runbookNameInstance4;
                                                }
                                                
                                                JToken jobIdValue16 = customDetailsValue4["jobId"];
                                                if (jobIdValue16 != null && jobIdValue16.Type != JTokenType.Null)
                                                {
                                                    string jobIdInstance16 = ((string)jobIdValue16);
                                                    automationRunbookTaskDetailsInstance4.JobId = jobIdInstance16;
                                                }
                                                
                                                JToken jobOutputValue4 = customDetailsValue4["jobOutput"];
                                                if (jobOutputValue4 != null && jobOutputValue4.Type != JTokenType.Null)
                                                {
                                                    string jobOutputInstance4 = ((string)jobOutputValue4);
                                                    automationRunbookTaskDetailsInstance4.JobOutput = jobOutputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue8 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue8 != null && isPrimarySideScriptValue8.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance8 = ((bool)isPrimarySideScriptValue8);
                                                    automationRunbookTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance8;
                                                }
                                                
                                                JToken instanceTypeValue35 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue35 != null && instanceTypeValue35.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance35 = ((string)instanceTypeValue35);
                                                    automationRunbookTaskDetailsInstance4.Type = instanceTypeInstance35;
                                                }
                                                taskInstance.CustomDetails = automationRunbookTaskDetailsInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken errorsArray5 = propertiesValue["errors"];
                                if (errorsArray5 != null && errorsArray5.Type != JTokenType.Null)
                                {
                                    foreach (JToken errorsValue5 in ((JArray)errorsArray5))
                                    {
                                        JobErrorDetails jobErrorDetailsInstance5 = new JobErrorDetails();
                                        propertiesInstance.Errors.Add(jobErrorDetailsInstance5);
                                        
                                        JToken serviceErrorDetailsValue5 = errorsValue5["serviceErrorDetails"];
                                        if (serviceErrorDetailsValue5 != null && serviceErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ServiceError serviceErrorDetailsInstance5 = new ServiceError();
                                            jobErrorDetailsInstance5.ServiceErrorDetails = serviceErrorDetailsInstance5;
                                            
                                            JToken codeValue5 = serviceErrorDetailsValue5["code"];
                                            if (codeValue5 != null && codeValue5.Type != JTokenType.Null)
                                            {
                                                string codeInstance5 = ((string)codeValue5);
                                                serviceErrorDetailsInstance5.Code = codeInstance5;
                                            }
                                            
                                            JToken messageValue5 = serviceErrorDetailsValue5["message"];
                                            if (messageValue5 != null && messageValue5.Type != JTokenType.Null)
                                            {
                                                string messageInstance5 = ((string)messageValue5);
                                                serviceErrorDetailsInstance5.Message = messageInstance5;
                                            }
                                            
                                            JToken possibleCausesValue9 = serviceErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue9 != null && possibleCausesValue9.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance9 = ((string)possibleCausesValue9);
                                                serviceErrorDetailsInstance5.PossibleCauses = possibleCausesInstance9;
                                            }
                                            
                                            JToken recommendedActionValue9 = serviceErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue9 != null && recommendedActionValue9.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance9 = ((string)recommendedActionValue9);
                                                serviceErrorDetailsInstance5.RecommendedAction = recommendedActionInstance9;
                                            }
                                            
                                            JToken activityIdValue6 = serviceErrorDetailsValue5["activityId"];
                                            if (activityIdValue6 != null && activityIdValue6.Type != JTokenType.Null)
                                            {
                                                string activityIdInstance6 = ((string)activityIdValue6);
                                                serviceErrorDetailsInstance5.ActivityId = activityIdInstance6;
                                            }
                                        }
                                        
                                        JToken providerErrorDetailsValue5 = errorsValue5["providerErrorDetails"];
                                        if (providerErrorDetailsValue5 != null && providerErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ProviderError providerErrorDetailsInstance5 = new ProviderError();
                                            jobErrorDetailsInstance5.ProviderErrorDetails = providerErrorDetailsInstance5;
                                            
                                            JToken errorCodeValue5 = providerErrorDetailsValue5["errorCode"];
                                            if (errorCodeValue5 != null && errorCodeValue5.Type != JTokenType.Null)
                                            {
                                                int errorCodeInstance5 = ((int)errorCodeValue5);
                                                providerErrorDetailsInstance5.ErrorCode = errorCodeInstance5;
                                            }
                                            
                                            JToken errorMessageValue5 = providerErrorDetailsValue5["errorMessage"];
                                            if (errorMessageValue5 != null && errorMessageValue5.Type != JTokenType.Null)
                                            {
                                                string errorMessageInstance5 = ((string)errorMessageValue5);
                                                providerErrorDetailsInstance5.ErrorMessage = errorMessageInstance5;
                                            }
                                            
                                            JToken errorIdValue5 = providerErrorDetailsValue5["errorId"];
                                            if (errorIdValue5 != null && errorIdValue5.Type != JTokenType.Null)
                                            {
                                                string errorIdInstance5 = ((string)errorIdValue5);
                                                providerErrorDetailsInstance5.ErrorId = errorIdInstance5;
                                            }
                                            
                                            JToken possibleCausesValue10 = providerErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue10 != null && possibleCausesValue10.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance10 = ((string)possibleCausesValue10);
                                                providerErrorDetailsInstance5.PossibleCauses = possibleCausesInstance10;
                                            }
                                            
                                            JToken recommendedActionValue10 = providerErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue10 != null && recommendedActionValue10.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance10 = ((string)recommendedActionValue10);
                                                providerErrorDetailsInstance5.RecommendedAction = recommendedActionInstance10;
                                            }
                                        }
                                        
                                        JToken errorLevelValue5 = errorsValue5["errorLevel"];
                                        if (errorLevelValue5 != null && errorLevelValue5.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance5 = ((string)errorLevelValue5);
                                            jobErrorDetailsInstance5.ErrorLevel = errorLevelInstance5;
                                        }
                                        
                                        JToken creationTimeValue5 = errorsValue5["creationTime"];
                                        if (creationTimeValue5 != null && creationTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime creationTimeInstance5 = ((DateTime)creationTimeValue5);
                                            jobErrorDetailsInstance5.CreationTime = creationTimeInstance5;
                                        }
                                        
                                        JToken taskIdValue9 = errorsValue5["taskId"];
                                        if (taskIdValue9 != null && taskIdValue9.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance9 = ((string)taskIdValue9);
                                            jobErrorDetailsInstance5.TaskId = taskIdInstance9;
                                        }
                                    }
                                }
                                
                                JToken customDetailsValue5 = propertiesValue["customDetails"];
                                if (customDetailsValue5 != null && customDetailsValue5.Type != JTokenType.Null)
                                {
                                    string typeName6 = ((string)customDetailsValue5["instanceType"]);
                                    if (typeName6 == "ExportJobDetails")
                                    {
                                        ExportJobDetails exportJobDetailsInstance = new ExportJobDetails();
                                        
                                        JToken blobUriValue = customDetailsValue5["blobUri"];
                                        if (blobUriValue != null && blobUriValue.Type != JTokenType.Null)
                                        {
                                            string blobUriInstance = ((string)blobUriValue);
                                            exportJobDetailsInstance.BlobUri = blobUriInstance;
                                        }
                                        
                                        JToken sasTokenValue = customDetailsValue5["sasToken"];
                                        if (sasTokenValue != null && sasTokenValue.Type != JTokenType.Null)
                                        {
                                            string sasTokenInstance = ((string)sasTokenValue);
                                            exportJobDetailsInstance.SasToken = sasTokenInstance;
                                        }
                                        
                                        JToken instanceTypeValue36 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue36 != null && instanceTypeValue36.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance36 = ((string)instanceTypeValue36);
                                            exportJobDetailsInstance.InstanceType = instanceTypeInstance36;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement != null && affectedObjectDetailsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in affectedObjectDetailsSequenceElement)
                                            {
                                                string affectedObjectDetailsKey = ((string)property.Name);
                                                string affectedObjectDetailsValue = ((string)property.Value);
                                                exportJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey, affectedObjectDetailsValue);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = exportJobDetailsInstance;
                                    }
                                    if (typeName6 == "AsrJobDetails")
                                    {
                                        AsrJobDetails asrJobDetailsInstance = new AsrJobDetails();
                                        
                                        JToken instanceTypeValue37 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue37 != null && instanceTypeValue37.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance37 = ((string)instanceTypeValue37);
                                            asrJobDetailsInstance.InstanceType = instanceTypeInstance37;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement2 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement2 != null && affectedObjectDetailsSequenceElement2.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property2 in affectedObjectDetailsSequenceElement2)
                                            {
                                                string affectedObjectDetailsKey2 = ((string)property2.Name);
                                                string affectedObjectDetailsValue2 = ((string)property2.Value);
                                                asrJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey2, affectedObjectDetailsValue2);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = asrJobDetailsInstance;
                                    }
                                    if (typeName6 == "TestFailoverJobDetails")
                                    {
                                        TestFailoverJobDetails testFailoverJobDetailsInstance = new TestFailoverJobDetails();
                                        
                                        JToken testFailoverStatusValue = customDetailsValue5["testFailoverStatus"];
                                        if (testFailoverStatusValue != null && testFailoverStatusValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStatusInstance = ((string)testFailoverStatusValue);
                                            testFailoverJobDetailsInstance.TestFailoverStatus = testFailoverStatusInstance;
                                        }
                                        
                                        JToken commentsValue = customDetailsValue5["comments"];
                                        if (commentsValue != null && commentsValue.Type != JTokenType.Null)
                                        {
                                            string commentsInstance = ((string)commentsValue);
                                            testFailoverJobDetailsInstance.Comments = commentsInstance;
                                        }
                                        
                                        JToken networkNameValue = customDetailsValue5["networkName"];
                                        if (networkNameValue != null && networkNameValue.Type != JTokenType.Null)
                                        {
                                            string networkNameInstance = ((string)networkNameValue);
                                            testFailoverJobDetailsInstance.NetworkName = networkNameInstance;
                                        }
                                        
                                        JToken networkFriendlyNameValue = customDetailsValue5["networkFriendlyName"];
                                        if (networkFriendlyNameValue != null && networkFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string networkFriendlyNameInstance = ((string)networkFriendlyNameValue);
                                            testFailoverJobDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance;
                                        }
                                        
                                        JToken networkTypeValue = customDetailsValue5["networkType"];
                                        if (networkTypeValue != null && networkTypeValue.Type != JTokenType.Null)
                                        {
                                            string networkTypeInstance = ((string)networkTypeValue);
                                            testFailoverJobDetailsInstance.NetworkType = networkTypeInstance;
                                        }
                                        
                                        JToken protectedItemDetailsArray = customDetailsValue5["protectedItemDetails"];
                                        if (protectedItemDetailsArray != null && protectedItemDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedItemDetailsValue in ((JArray)protectedItemDetailsArray))
                                            {
                                                TestFailoverReplicationProtectedItemDetails testFailoverReplicationProtectedItemDetailsInstance = new TestFailoverReplicationProtectedItemDetails();
                                                testFailoverJobDetailsInstance.ProtectedItemDetails.Add(testFailoverReplicationProtectedItemDetailsInstance);
                                                
                                                JToken nameValue23 = protectedItemDetailsValue["name"];
                                                if (nameValue23 != null && nameValue23.Type != JTokenType.Null)
                                                {
                                                    string nameInstance23 = ((string)nameValue23);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Name = nameInstance23;
                                                }
                                                
                                                JToken friendlyNameValue6 = protectedItemDetailsValue["friendlyName"];
                                                if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                                {
                                                    string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                                    testFailoverReplicationProtectedItemDetailsInstance.FriendlyName = friendlyNameInstance6;
                                                }
                                                
                                                JToken testVmNameValue = protectedItemDetailsValue["testVmName"];
                                                if (testVmNameValue != null && testVmNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmNameInstance = ((string)testVmNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmName = testVmNameInstance;
                                                }
                                                
                                                JToken testVmFriendlyNameValue = protectedItemDetailsValue["testVmFriendlyName"];
                                                if (testVmFriendlyNameValue != null && testVmFriendlyNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmFriendlyNameInstance = ((string)testVmFriendlyNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmFriendlyName = testVmFriendlyNameInstance;
                                                }
                                                
                                                JToken networkConnectionStatusValue = protectedItemDetailsValue["networkConnectionStatus"];
                                                if (networkConnectionStatusValue != null && networkConnectionStatusValue.Type != JTokenType.Null)
                                                {
                                                    string networkConnectionStatusInstance = ((string)networkConnectionStatusValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkConnectionStatus = networkConnectionStatusInstance;
                                                }
                                                
                                                JToken networkFriendlyNameValue2 = protectedItemDetailsValue["networkFriendlyName"];
                                                if (networkFriendlyNameValue2 != null && networkFriendlyNameValue2.Type != JTokenType.Null)
                                                {
                                                    string networkFriendlyNameInstance2 = ((string)networkFriendlyNameValue2);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance2;
                                                }
                                                
                                                JToken subnetValue = protectedItemDetailsValue["subnet"];
                                                if (subnetValue != null && subnetValue.Type != JTokenType.Null)
                                                {
                                                    string subnetInstance = ((string)subnetValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Subnet = subnetInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue38 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue38 != null && instanceTypeValue38.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance38 = ((string)instanceTypeValue38);
                                            testFailoverJobDetailsInstance.InstanceType = instanceTypeInstance38;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement3 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement3 != null && affectedObjectDetailsSequenceElement3.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property3 in affectedObjectDetailsSequenceElement3)
                                            {
                                                string affectedObjectDetailsKey3 = ((string)property3.Name);
                                                string affectedObjectDetailsValue3 = ((string)property3.Value);
                                                testFailoverJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey3, affectedObjectDetailsValue3);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = testFailoverJobDetailsInstance;
                                    }
                                }
                                
                                JToken idValue = propertiesValue["id"];
                                if (idValue != null && idValue.Type != JTokenType.Null)
                                {
                                    string idInstance = ((string)idValue);
                                    propertiesInstance.Id = idInstance;
                                }
                                
                                JToken nameValue24 = propertiesValue["name"];
                                if (nameValue24 != null && nameValue24.Type != JTokenType.Null)
                                {
                                    string nameInstance24 = ((string)nameValue24);
                                    propertiesInstance.Name = nameInstance24;
                                }
                                
                                JToken typeValue = propertiesValue["type"];
                                if (typeValue != null && typeValue.Type != JTokenType.Null)
                                {
                                    string typeInstance = ((string)typeValue);
                                    propertiesInstance.Type = typeInstance;
                                }
                                
                                JToken locationValue = propertiesValue["location"];
                                if (locationValue != null && locationValue.Type != JTokenType.Null)
                                {
                                    string locationInstance = ((string)locationValue);
                                    propertiesInstance.Location = locationInstance;
                                }
                                
                                JToken tagsSequenceElement = ((JToken)propertiesValue["tags"]);
                                if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property4 in tagsSequenceElement)
                                    {
                                        string tagsKey = ((string)property4.Name);
                                        string tagsValue = ((string)property4.Value);
                                        propertiesInstance.Tags.Add(tagsKey, tagsValue);
                                    }
                                }
                            }
                            
                            JToken idValue2 = responseDoc["id"];
                            if (idValue2 != null && idValue2.Type != JTokenType.Null)
                            {
                                string idInstance2 = ((string)idValue2);
                                jobInstance.Id = idInstance2;
                            }
                            
                            JToken nameValue25 = responseDoc["name"];
                            if (nameValue25 != null && nameValue25.Type != JTokenType.Null)
                            {
                                string nameInstance25 = ((string)nameValue25);
                                jobInstance.Name = nameInstance25;
                            }
                            
                            JToken typeValue2 = responseDoc["type"];
                            if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                            {
                                string typeInstance2 = ((string)typeValue2);
                                jobInstance.Type = typeInstance2;
                            }
                            
                            JToken locationValue2 = responseDoc["location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                jobInstance.Location = locationInstance2;
                            }
                            
                            JToken tagsSequenceElement2 = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement2 != null && tagsSequenceElement2.Type != JTokenType.Null)
                            {
                                foreach (JProperty property5 in tagsSequenceElement2)
                                {
                                    string tagsKey2 = ((string)property5.Name);
                                    string tagsValue2 = ((string)property5.Value);
                                    jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken locationValue3 = responseDoc["Location"];
                            if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                            {
                                string locationInstance3 = ((string)locationValue3);
                                result.Location = locationInstance3;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<JobOperationResponse> GetExportStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetExportStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted || statusCode == HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken scenarioNameValue = propertiesValue["scenarioName"];
                                if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                {
                                    string scenarioNameInstance = ((string)scenarioNameValue);
                                    propertiesInstance.ScenarioName = scenarioNameInstance;
                                }
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken activityIdValue = propertiesValue["activityId"];
                                if (activityIdValue != null && activityIdValue.Type != JTokenType.Null)
                                {
                                    string activityIdInstance = ((string)activityIdValue);
                                    propertiesInstance.ActivityId = activityIdInstance;
                                }
                                
                                JToken startTimeValue = propertiesValue["startTime"];
                                if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime startTimeInstance = ((DateTime)startTimeValue);
                                    propertiesInstance.StartTime = startTimeInstance;
                                }
                                
                                JToken endTimeValue = propertiesValue["endTime"];
                                if (endTimeValue != null && endTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime endTimeInstance = ((DateTime)endTimeValue);
                                    propertiesInstance.EndTime = endTimeInstance;
                                }
                                
                                JToken targetObjectIdValue = propertiesValue["targetObjectId"];
                                if (targetObjectIdValue != null && targetObjectIdValue.Type != JTokenType.Null)
                                {
                                    string targetObjectIdInstance = ((string)targetObjectIdValue);
                                    propertiesInstance.TargetObjectId = targetObjectIdInstance;
                                }
                                
                                JToken targetInstanceTypeValue = propertiesValue["targetInstanceType"];
                                if (targetInstanceTypeValue != null && targetInstanceTypeValue.Type != JTokenType.Null)
                                {
                                    string targetInstanceTypeInstance = ((string)targetInstanceTypeValue);
                                    propertiesInstance.TargetInstanceType = targetInstanceTypeInstance;
                                }
                                
                                JToken targetObjectNameValue = propertiesValue["targetObjectName"];
                                if (targetObjectNameValue != null && targetObjectNameValue.Type != JTokenType.Null)
                                {
                                    string targetObjectNameInstance = ((string)targetObjectNameValue);
                                    propertiesInstance.TargetObjectName = targetObjectNameInstance;
                                }
                                
                                JToken allowedActionsArray = propertiesValue["allowedActions"];
                                if (allowedActionsArray != null && allowedActionsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedActionsValue in ((JArray)allowedActionsArray))
                                    {
                                        propertiesInstance.AllowedActions.Add(((string)allowedActionsValue));
                                    }
                                }
                                
                                JToken stateValue = propertiesValue["state"];
                                if (stateValue != null && stateValue.Type != JTokenType.Null)
                                {
                                    string stateInstance = ((string)stateValue);
                                    propertiesInstance.State = stateInstance;
                                }
                                
                                JToken stateDescriptionValue = propertiesValue["stateDescription"];
                                if (stateDescriptionValue != null && stateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string stateDescriptionInstance = ((string)stateDescriptionValue);
                                    propertiesInstance.StateDescription = stateDescriptionInstance;
                                }
                                
                                JToken tasksArray = propertiesValue["tasks"];
                                if (tasksArray != null && tasksArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken tasksValue in ((JArray)tasksArray))
                                    {
                                        AsrTask taskInstance = new AsrTask();
                                        propertiesInstance.Tasks.Add(taskInstance);
                                        
                                        JToken groupTaskCustomDetailsValue = tasksValue["groupTaskCustomDetails"];
                                        if (groupTaskCustomDetailsValue != null && groupTaskCustomDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)groupTaskCustomDetailsValue["instanceType"]);
                                            if (typeName == "InlineWorkflowTaskDetails")
                                            {
                                                InlineWorkflowTaskDetails inlineWorkflowTaskDetailsInstance = new InlineWorkflowTaskDetails();
                                                
                                                JToken workflowIdsArray = groupTaskCustomDetailsValue["workflowIds"];
                                                if (workflowIdsArray != null && workflowIdsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken workflowIdsValue in ((JArray)workflowIdsArray))
                                                    {
                                                        inlineWorkflowTaskDetailsInstance.WorkflowIds.Add(((string)workflowIdsValue));
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    inlineWorkflowTaskDetailsInstance.Type = instanceTypeInstance;
                                                }
                                                
                                                JToken childTasksArray = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray != null && childTasksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue in ((JArray)childTasksArray))
                                                    {
                                                        AsrTaskBase taskBaseInstance = new AsrTaskBase();
                                                        inlineWorkflowTaskDetailsInstance.ChildTasks.Add(taskBaseInstance);
                                                        
                                                        JToken taskIdValue = childTasksValue["taskId"];
                                                        if (taskIdValue != null && taskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance = ((string)taskIdValue);
                                                            taskBaseInstance.ID = taskIdInstance;
                                                        }
                                                        
                                                        JToken nameValue = childTasksValue["name"];
                                                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance = ((string)nameValue);
                                                            taskBaseInstance.Name = nameInstance;
                                                        }
                                                        
                                                        JToken startTimeValue2 = childTasksValue["startTime"];
                                                        if (startTimeValue2 != null && startTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance2 = ((DateTime)startTimeValue2);
                                                            taskBaseInstance.StartTime = startTimeInstance2;
                                                        }
                                                        
                                                        JToken endTimeValue2 = childTasksValue["endTime"];
                                                        if (endTimeValue2 != null && endTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance2 = ((DateTime)endTimeValue2);
                                                            taskBaseInstance.EndTime = endTimeInstance2;
                                                        }
                                                        
                                                        JToken allowedActionsArray2 = childTasksValue["allowedActions"];
                                                        if (allowedActionsArray2 != null && allowedActionsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue2 in ((JArray)allowedActionsArray2))
                                                            {
                                                                taskBaseInstance.Actions.Add(((string)allowedActionsValue2));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue2 = childTasksValue["friendlyName"];
                                                        if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                                            taskBaseInstance.TaskFriendlyName = friendlyNameInstance2;
                                                        }
                                                        
                                                        JToken stateValue2 = childTasksValue["state"];
                                                        if (stateValue2 != null && stateValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance2 = ((string)stateValue2);
                                                            taskBaseInstance.State = stateInstance2;
                                                        }
                                                        
                                                        JToken stateDescriptionValue2 = childTasksValue["stateDescription"];
                                                        if (stateDescriptionValue2 != null && stateDescriptionValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance2 = ((string)stateDescriptionValue2);
                                                            taskBaseInstance.StateDescription = stateDescriptionInstance2;
                                                        }
                                                        
                                                        JToken taskTypeValue = childTasksValue["taskType"];
                                                        if (taskTypeValue != null && taskTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance = ((string)taskTypeValue);
                                                            taskBaseInstance.TaskType = taskTypeInstance;
                                                        }
                                                        
                                                        JToken errorsArray = childTasksValue["errors"];
                                                        if (errorsArray != null && errorsArray.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue in ((JArray)errorsArray))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance = new JobErrorDetails();
                                                                taskBaseInstance.Errors.Add(jobErrorDetailsInstance);
                                                                
                                                                JToken serviceErrorDetailsValue = errorsValue["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue != null && serviceErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance = new ServiceError();
                                                                    jobErrorDetailsInstance.ServiceErrorDetails = serviceErrorDetailsInstance;
                                                                    
                                                                    JToken codeValue = serviceErrorDetailsValue["code"];
                                                                    if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance = ((string)codeValue);
                                                                        serviceErrorDetailsInstance.Code = codeInstance;
                                                                    }
                                                                    
                                                                    JToken messageValue = serviceErrorDetailsValue["message"];
                                                                    if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance = ((string)messageValue);
                                                                        serviceErrorDetailsInstance.Message = messageInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue = serviceErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance = ((string)possibleCausesValue);
                                                                        serviceErrorDetailsInstance.PossibleCauses = possibleCausesInstance;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue = serviceErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance = ((string)recommendedActionValue);
                                                                        serviceErrorDetailsInstance.RecommendedAction = recommendedActionInstance;
                                                                    }
                                                                    
                                                                    JToken activityIdValue2 = serviceErrorDetailsValue["activityId"];
                                                                    if (activityIdValue2 != null && activityIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance2 = ((string)activityIdValue2);
                                                                        serviceErrorDetailsInstance.ActivityId = activityIdInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue = errorsValue["providerErrorDetails"];
                                                                if (providerErrorDetailsValue != null && providerErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance = new ProviderError();
                                                                    jobErrorDetailsInstance.ProviderErrorDetails = providerErrorDetailsInstance;
                                                                    
                                                                    JToken errorCodeValue = providerErrorDetailsValue["errorCode"];
                                                                    if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance = ((int)errorCodeValue);
                                                                        providerErrorDetailsInstance.ErrorCode = errorCodeInstance;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue = providerErrorDetailsValue["errorMessage"];
                                                                    if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance = ((string)errorMessageValue);
                                                                        providerErrorDetailsInstance.ErrorMessage = errorMessageInstance;
                                                                    }
                                                                    
                                                                    JToken errorIdValue = providerErrorDetailsValue["errorId"];
                                                                    if (errorIdValue != null && errorIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance = ((string)errorIdValue);
                                                                        providerErrorDetailsInstance.ErrorId = errorIdInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue2 = providerErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue2 != null && possibleCausesValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance2 = ((string)possibleCausesValue2);
                                                                        providerErrorDetailsInstance.PossibleCauses = possibleCausesInstance2;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue2 = providerErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue2 != null && recommendedActionValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance2 = ((string)recommendedActionValue2);
                                                                        providerErrorDetailsInstance.RecommendedAction = recommendedActionInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue = errorsValue["errorLevel"];
                                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                                    jobErrorDetailsInstance.ErrorLevel = errorLevelInstance;
                                                                }
                                                                
                                                                JToken creationTimeValue = errorsValue["creationTime"];
                                                                if (creationTimeValue != null && creationTimeValue.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance = ((DateTime)creationTimeValue);
                                                                    jobErrorDetailsInstance.CreationTime = creationTimeInstance;
                                                                }
                                                                
                                                                JToken taskIdValue2 = errorsValue["taskId"];
                                                                if (taskIdValue2 != null && taskIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance2 = ((string)taskIdValue2);
                                                                    jobErrorDetailsInstance.TaskId = taskIdInstance2;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue = childTasksValue["customDetails"];
                                                        if (customDetailsValue != null && customDetailsValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName2 = ((string)customDetailsValue["instanceType"]);
                                                            if (typeName2 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue = customDetailsValue["jobTask"];
                                                                if (jobTaskValue != null && jobTaskValue.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance = new JobEntity();
                                                                    jobTaskDetailsInstance.JobTask = jobTaskInstance;
                                                                    
                                                                    JToken jobIdValue = jobTaskValue["jobId"];
                                                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance = ((string)jobIdValue);
                                                                        jobTaskInstance.JobId = jobIdInstance;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue = jobTaskValue["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue != null && jobFriendlyNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance = ((string)jobFriendlyNameValue);
                                                                        jobTaskInstance.JobFriendlyName = jobFriendlyNameInstance;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue2 = jobTaskValue["targetObjectId"];
                                                                    if (targetObjectIdValue2 != null && targetObjectIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance2 = ((string)targetObjectIdValue2);
                                                                        jobTaskInstance.TargetObjectId = targetObjectIdInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue2 = jobTaskValue["targetObjectName"];
                                                                    if (targetObjectNameValue2 != null && targetObjectNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance2 = ((string)targetObjectNameValue2);
                                                                        jobTaskInstance.TargetObjectName = targetObjectNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue2 = jobTaskValue["targetInstanceType"];
                                                                    if (targetInstanceTypeValue2 != null && targetInstanceTypeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance2 = ((string)targetInstanceTypeValue2);
                                                                        jobTaskInstance.TargetInstanceType = targetInstanceTypeInstance2;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue = jobTaskValue["jobScenarioName"];
                                                                    if (jobScenarioNameValue != null && jobScenarioNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance = ((string)jobScenarioNameValue);
                                                                        jobTaskInstance.JobScenarioName = jobScenarioNameInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue2 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                                    jobTaskDetailsInstance.Type = instanceTypeInstance2;
                                                                }
                                                                taskBaseInstance.CustomDetails = jobTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue != null && skippedReasonValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance = ((string)skippedReasonValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReason = skippedReasonInstance;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue != null && skippedReasonStringValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance = ((string)skippedReasonStringValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance;
                                                                }
                                                                
                                                                JToken jobTaskValue2 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue2 != null && jobTaskValue2.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance2 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance.JobTask = jobTaskInstance2;
                                                                    
                                                                    JToken jobIdValue2 = jobTaskValue2["jobId"];
                                                                    if (jobIdValue2 != null && jobIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance2 = ((string)jobIdValue2);
                                                                        jobTaskInstance2.JobId = jobIdInstance2;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue2 = jobTaskValue2["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue2 != null && jobFriendlyNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance2 = ((string)jobFriendlyNameValue2);
                                                                        jobTaskInstance2.JobFriendlyName = jobFriendlyNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue3 = jobTaskValue2["targetObjectId"];
                                                                    if (targetObjectIdValue3 != null && targetObjectIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance3 = ((string)targetObjectIdValue3);
                                                                        jobTaskInstance2.TargetObjectId = targetObjectIdInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue3 = jobTaskValue2["targetObjectName"];
                                                                    if (targetObjectNameValue3 != null && targetObjectNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance3 = ((string)targetObjectNameValue3);
                                                                        jobTaskInstance2.TargetObjectName = targetObjectNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue3 = jobTaskValue2["targetInstanceType"];
                                                                    if (targetInstanceTypeValue3 != null && targetInstanceTypeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance3 = ((string)targetInstanceTypeValue3);
                                                                        jobTaskInstance2.TargetInstanceType = targetInstanceTypeInstance3;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue2 = jobTaskValue2["jobScenarioName"];
                                                                    if (jobScenarioNameValue2 != null && jobScenarioNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance2 = ((string)jobScenarioNameValue2);
                                                                        jobTaskInstance2.JobScenarioName = jobScenarioNameInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue3 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                                    virtualMachineTaskDetailsInstance.Type = instanceTypeInstance3;
                                                                }
                                                                taskBaseInstance.CustomDetails = virtualMachineTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue2 = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue2 != null && skippedReasonValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance2 = ((string)skippedReasonValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReason = skippedReasonInstance2;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue2 = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue2 != null && skippedReasonStringValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance2 = ((string)skippedReasonStringValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance2;
                                                                }
                                                                
                                                                JToken jobTaskValue3 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue3 != null && jobTaskValue3.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance3 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance.JobTask = jobTaskInstance3;
                                                                    
                                                                    JToken jobIdValue3 = jobTaskValue3["jobId"];
                                                                    if (jobIdValue3 != null && jobIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance3 = ((string)jobIdValue3);
                                                                        jobTaskInstance3.JobId = jobIdInstance3;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue3 = jobTaskValue3["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue3 != null && jobFriendlyNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance3 = ((string)jobFriendlyNameValue3);
                                                                        jobTaskInstance3.JobFriendlyName = jobFriendlyNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue4 = jobTaskValue3["targetObjectId"];
                                                                    if (targetObjectIdValue4 != null && targetObjectIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance4 = ((string)targetObjectIdValue4);
                                                                        jobTaskInstance3.TargetObjectId = targetObjectIdInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue4 = jobTaskValue3["targetObjectName"];
                                                                    if (targetObjectNameValue4 != null && targetObjectNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance4 = ((string)targetObjectNameValue4);
                                                                        jobTaskInstance3.TargetObjectName = targetObjectNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue4 = jobTaskValue3["targetInstanceType"];
                                                                    if (targetInstanceTypeValue4 != null && targetInstanceTypeValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance4 = ((string)targetInstanceTypeValue4);
                                                                        jobTaskInstance3.TargetInstanceType = targetInstanceTypeInstance4;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue3 = jobTaskValue3["jobScenarioName"];
                                                                    if (jobScenarioNameValue3 != null && jobScenarioNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance3 = ((string)jobScenarioNameValue3);
                                                                        jobTaskInstance3.JobScenarioName = jobScenarioNameInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue4 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance.Type = instanceTypeInstance4;
                                                                }
                                                                taskBaseInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue2 = customDetailsValue["name"];
                                                                if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance2 = ((string)nameValue2);
                                                                    manualActionTaskDetailsInstance.Name = nameInstance2;
                                                                }
                                                                
                                                                JToken instructionsValue = customDetailsValue["instructions"];
                                                                if (instructionsValue != null && instructionsValue.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance = ((string)instructionsValue);
                                                                    manualActionTaskDetailsInstance.Instructions = instructionsInstance;
                                                                }
                                                                
                                                                JToken observationValue = customDetailsValue["observation"];
                                                                if (observationValue != null && observationValue.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance = ((string)observationValue);
                                                                    manualActionTaskDetailsInstance.Observation = observationInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue5 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                                    manualActionTaskDetailsInstance.Type = instanceTypeInstance5;
                                                                }
                                                                taskBaseInstance.CustomDetails = manualActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue3 = customDetailsValue["name"];
                                                                if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance3 = ((string)nameValue3);
                                                                    scriptActionTaskDetailsInstance.Name = nameInstance3;
                                                                }
                                                                
                                                                JToken pathValue = customDetailsValue["path"];
                                                                if (pathValue != null && pathValue.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance = ((string)pathValue);
                                                                    scriptActionTaskDetailsInstance.Path = pathInstance;
                                                                }
                                                                
                                                                JToken outputValue = customDetailsValue["output"];
                                                                if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance = ((string)outputValue);
                                                                    scriptActionTaskDetailsInstance.Output = outputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue != null && isPrimarySideScriptValue.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance = ((bool)isPrimarySideScriptValue);
                                                                    scriptActionTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue6 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                                    scriptActionTaskDetailsInstance.Type = instanceTypeInstance6;
                                                                }
                                                                taskBaseInstance.CustomDetails = scriptActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue = customDetailsValue["vmId"];
                                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance = ((string)vmIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.VmId = vmIdInstance;
                                                                }
                                                                
                                                                JToken nicIdValue = customDetailsValue["nicId"];
                                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance = ((string)nicIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.NicId = nicIdInstance;
                                                                }
                                                                
                                                                JToken nameValue4 = customDetailsValue["name"];
                                                                if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance4 = ((string)nameValue4);
                                                                    vmNicUpdatesTaskDetailsInstance.Name = nameInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue7 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue7 != null && instanceTypeValue7.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance7 = ((string)instanceTypeValue7);
                                                                    vmNicUpdatesTaskDetailsInstance.Type = instanceTypeInstance7;
                                                                }
                                                                taskBaseInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray = customDetailsValue["vmDetails"];
                                                                if (vmDetailsArray != null && vmDetailsArray.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue in ((JArray)vmDetailsArray))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance.VmDetails.Add(inconsistentVmDetailsInstance);
                                                                        
                                                                        JToken vmNameValue = vmDetailsValue["vmName"];
                                                                        if (vmNameValue != null && vmNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance = ((string)vmNameValue);
                                                                            inconsistentVmDetailsInstance.VmName = vmNameInstance;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue = vmDetailsValue["cloudName"];
                                                                        if (cloudNameValue != null && cloudNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance = ((string)cloudNameValue);
                                                                            inconsistentVmDetailsInstance.CloudName = cloudNameInstance;
                                                                        }
                                                                        
                                                                        JToken detailsArray = vmDetailsValue["details"];
                                                                        if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue in ((JArray)detailsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.Details.Add(((string)detailsValue));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray = vmDetailsValue["errorIds"];
                                                                        if (errorIdsArray != null && errorIdsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue in ((JArray)errorIdsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.ErrorIds.Add(((string)errorIdsValue));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue8 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue8 != null && instanceTypeValue8.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance8 = ((string)instanceTypeValue8);
                                                                    consistencyCheckTaskDetailsInstance.Type = instanceTypeInstance8;
                                                                }
                                                                taskBaseInstance.CustomDetails = consistencyCheckTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue5 = customDetailsValue["name"];
                                                                if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance5 = ((string)nameValue5);
                                                                    automationRunbookTaskDetailsInstance.Name = nameInstance5;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue = customDetailsValue["cloudServiceName"];
                                                                if (cloudServiceNameValue != null && cloudServiceNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance = ((string)cloudServiceNameValue);
                                                                    automationRunbookTaskDetailsInstance.CloudServiceName = cloudServiceNameInstance;
                                                                }
                                                                
                                                                JToken subscriptionIdValue = customDetailsValue["subscriptionId"];
                                                                if (subscriptionIdValue != null && subscriptionIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance = ((string)subscriptionIdValue);
                                                                    automationRunbookTaskDetailsInstance.SubscriptionId = subscriptionIdInstance;
                                                                }
                                                                
                                                                JToken accountNameValue = customDetailsValue["accountName"];
                                                                if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance = ((string)accountNameValue);
                                                                    automationRunbookTaskDetailsInstance.AccountName = accountNameInstance;
                                                                }
                                                                
                                                                JToken runbookIdValue = customDetailsValue["runbookId"];
                                                                if (runbookIdValue != null && runbookIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance = ((string)runbookIdValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookId = runbookIdInstance;
                                                                }
                                                                
                                                                JToken runbookNameValue = customDetailsValue["runbookName"];
                                                                if (runbookNameValue != null && runbookNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance = ((string)runbookNameValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookName = runbookNameInstance;
                                                                }
                                                                
                                                                JToken jobIdValue4 = customDetailsValue["jobId"];
                                                                if (jobIdValue4 != null && jobIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance4 = ((string)jobIdValue4);
                                                                    automationRunbookTaskDetailsInstance.JobId = jobIdInstance4;
                                                                }
                                                                
                                                                JToken jobOutputValue = customDetailsValue["jobOutput"];
                                                                if (jobOutputValue != null && jobOutputValue.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance = ((string)jobOutputValue);
                                                                    automationRunbookTaskDetailsInstance.JobOutput = jobOutputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue2 = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue2 != null && isPrimarySideScriptValue2.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance2 = ((bool)isPrimarySideScriptValue2);
                                                                    automationRunbookTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue9 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue9 != null && instanceTypeValue9.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance9 = ((string)instanceTypeValue9);
                                                                    automationRunbookTaskDetailsInstance.Type = instanceTypeInstance9;
                                                                }
                                                                taskBaseInstance.CustomDetails = automationRunbookTaskDetailsInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = inlineWorkflowTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanGroupTaskDetails")
                                            {
                                                RecoveryPlanGroupTaskDetails recoveryPlanGroupTaskDetailsInstance = new RecoveryPlanGroupTaskDetails();
                                                
                                                JToken nameValue6 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                {
                                                    string nameInstance6 = ((string)nameValue6);
                                                    recoveryPlanGroupTaskDetailsInstance.Name = nameInstance6;
                                                }
                                                
                                                JToken groupIdValue = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue != null && groupIdValue.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance = ((string)groupIdValue);
                                                    recoveryPlanGroupTaskDetailsInstance.GroupId = groupIdInstance;
                                                }
                                                
                                                JToken rpGroupTypeValue = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue != null && rpGroupTypeValue.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance = ((string)rpGroupTypeValue);
                                                    recoveryPlanGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance;
                                                }
                                                
                                                JToken instanceTypeValue10 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue10 != null && instanceTypeValue10.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance10 = ((string)instanceTypeValue10);
                                                    recoveryPlanGroupTaskDetailsInstance.Type = instanceTypeInstance10;
                                                }
                                                
                                                JToken childTasksArray2 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray2 != null && childTasksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue2 in ((JArray)childTasksArray2))
                                                    {
                                                        AsrTaskBase taskBaseInstance2 = new AsrTaskBase();
                                                        recoveryPlanGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance2);
                                                        
                                                        JToken taskIdValue3 = childTasksValue2["taskId"];
                                                        if (taskIdValue3 != null && taskIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance3 = ((string)taskIdValue3);
                                                            taskBaseInstance2.ID = taskIdInstance3;
                                                        }
                                                        
                                                        JToken nameValue7 = childTasksValue2["name"];
                                                        if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance7 = ((string)nameValue7);
                                                            taskBaseInstance2.Name = nameInstance7;
                                                        }
                                                        
                                                        JToken startTimeValue3 = childTasksValue2["startTime"];
                                                        if (startTimeValue3 != null && startTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance3 = ((DateTime)startTimeValue3);
                                                            taskBaseInstance2.StartTime = startTimeInstance3;
                                                        }
                                                        
                                                        JToken endTimeValue3 = childTasksValue2["endTime"];
                                                        if (endTimeValue3 != null && endTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance3 = ((DateTime)endTimeValue3);
                                                            taskBaseInstance2.EndTime = endTimeInstance3;
                                                        }
                                                        
                                                        JToken allowedActionsArray3 = childTasksValue2["allowedActions"];
                                                        if (allowedActionsArray3 != null && allowedActionsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue3 in ((JArray)allowedActionsArray3))
                                                            {
                                                                taskBaseInstance2.Actions.Add(((string)allowedActionsValue3));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue3 = childTasksValue2["friendlyName"];
                                                        if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                                            taskBaseInstance2.TaskFriendlyName = friendlyNameInstance3;
                                                        }
                                                        
                                                        JToken stateValue3 = childTasksValue2["state"];
                                                        if (stateValue3 != null && stateValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance3 = ((string)stateValue3);
                                                            taskBaseInstance2.State = stateInstance3;
                                                        }
                                                        
                                                        JToken stateDescriptionValue3 = childTasksValue2["stateDescription"];
                                                        if (stateDescriptionValue3 != null && stateDescriptionValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance3 = ((string)stateDescriptionValue3);
                                                            taskBaseInstance2.StateDescription = stateDescriptionInstance3;
                                                        }
                                                        
                                                        JToken taskTypeValue2 = childTasksValue2["taskType"];
                                                        if (taskTypeValue2 != null && taskTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance2 = ((string)taskTypeValue2);
                                                            taskBaseInstance2.TaskType = taskTypeInstance2;
                                                        }
                                                        
                                                        JToken errorsArray2 = childTasksValue2["errors"];
                                                        if (errorsArray2 != null && errorsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue2 in ((JArray)errorsArray2))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance2 = new JobErrorDetails();
                                                                taskBaseInstance2.Errors.Add(jobErrorDetailsInstance2);
                                                                
                                                                JToken serviceErrorDetailsValue2 = errorsValue2["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue2 != null && serviceErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance2 = new ServiceError();
                                                                    jobErrorDetailsInstance2.ServiceErrorDetails = serviceErrorDetailsInstance2;
                                                                    
                                                                    JToken codeValue2 = serviceErrorDetailsValue2["code"];
                                                                    if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance2 = ((string)codeValue2);
                                                                        serviceErrorDetailsInstance2.Code = codeInstance2;
                                                                    }
                                                                    
                                                                    JToken messageValue2 = serviceErrorDetailsValue2["message"];
                                                                    if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance2 = ((string)messageValue2);
                                                                        serviceErrorDetailsInstance2.Message = messageInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue3 = serviceErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue3 != null && possibleCausesValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance3 = ((string)possibleCausesValue3);
                                                                        serviceErrorDetailsInstance2.PossibleCauses = possibleCausesInstance3;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue3 = serviceErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue3 != null && recommendedActionValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance3 = ((string)recommendedActionValue3);
                                                                        serviceErrorDetailsInstance2.RecommendedAction = recommendedActionInstance3;
                                                                    }
                                                                    
                                                                    JToken activityIdValue3 = serviceErrorDetailsValue2["activityId"];
                                                                    if (activityIdValue3 != null && activityIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance3 = ((string)activityIdValue3);
                                                                        serviceErrorDetailsInstance2.ActivityId = activityIdInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue2 = errorsValue2["providerErrorDetails"];
                                                                if (providerErrorDetailsValue2 != null && providerErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance2 = new ProviderError();
                                                                    jobErrorDetailsInstance2.ProviderErrorDetails = providerErrorDetailsInstance2;
                                                                    
                                                                    JToken errorCodeValue2 = providerErrorDetailsValue2["errorCode"];
                                                                    if (errorCodeValue2 != null && errorCodeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance2 = ((int)errorCodeValue2);
                                                                        providerErrorDetailsInstance2.ErrorCode = errorCodeInstance2;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue2 = providerErrorDetailsValue2["errorMessage"];
                                                                    if (errorMessageValue2 != null && errorMessageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance2 = ((string)errorMessageValue2);
                                                                        providerErrorDetailsInstance2.ErrorMessage = errorMessageInstance2;
                                                                    }
                                                                    
                                                                    JToken errorIdValue2 = providerErrorDetailsValue2["errorId"];
                                                                    if (errorIdValue2 != null && errorIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance2 = ((string)errorIdValue2);
                                                                        providerErrorDetailsInstance2.ErrorId = errorIdInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue4 = providerErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue4 != null && possibleCausesValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance4 = ((string)possibleCausesValue4);
                                                                        providerErrorDetailsInstance2.PossibleCauses = possibleCausesInstance4;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue4 = providerErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue4 != null && recommendedActionValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance4 = ((string)recommendedActionValue4);
                                                                        providerErrorDetailsInstance2.RecommendedAction = recommendedActionInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue2 = errorsValue2["errorLevel"];
                                                                if (errorLevelValue2 != null && errorLevelValue2.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance2 = ((string)errorLevelValue2);
                                                                    jobErrorDetailsInstance2.ErrorLevel = errorLevelInstance2;
                                                                }
                                                                
                                                                JToken creationTimeValue2 = errorsValue2["creationTime"];
                                                                if (creationTimeValue2 != null && creationTimeValue2.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance2 = ((DateTime)creationTimeValue2);
                                                                    jobErrorDetailsInstance2.CreationTime = creationTimeInstance2;
                                                                }
                                                                
                                                                JToken taskIdValue4 = errorsValue2["taskId"];
                                                                if (taskIdValue4 != null && taskIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance4 = ((string)taskIdValue4);
                                                                    jobErrorDetailsInstance2.TaskId = taskIdInstance4;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue2 = childTasksValue2["customDetails"];
                                                        if (customDetailsValue2 != null && customDetailsValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName3 = ((string)customDetailsValue2["instanceType"]);
                                                            if (typeName3 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance2 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue4 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue4 != null && jobTaskValue4.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance4 = new JobEntity();
                                                                    jobTaskDetailsInstance2.JobTask = jobTaskInstance4;
                                                                    
                                                                    JToken jobIdValue5 = jobTaskValue4["jobId"];
                                                                    if (jobIdValue5 != null && jobIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance5 = ((string)jobIdValue5);
                                                                        jobTaskInstance4.JobId = jobIdInstance5;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue4 = jobTaskValue4["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue4 != null && jobFriendlyNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance4 = ((string)jobFriendlyNameValue4);
                                                                        jobTaskInstance4.JobFriendlyName = jobFriendlyNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue5 = jobTaskValue4["targetObjectId"];
                                                                    if (targetObjectIdValue5 != null && targetObjectIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance5 = ((string)targetObjectIdValue5);
                                                                        jobTaskInstance4.TargetObjectId = targetObjectIdInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue5 = jobTaskValue4["targetObjectName"];
                                                                    if (targetObjectNameValue5 != null && targetObjectNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance5 = ((string)targetObjectNameValue5);
                                                                        jobTaskInstance4.TargetObjectName = targetObjectNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue5 = jobTaskValue4["targetInstanceType"];
                                                                    if (targetInstanceTypeValue5 != null && targetInstanceTypeValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance5 = ((string)targetInstanceTypeValue5);
                                                                        jobTaskInstance4.TargetInstanceType = targetInstanceTypeInstance5;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue4 = jobTaskValue4["jobScenarioName"];
                                                                    if (jobScenarioNameValue4 != null && jobScenarioNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance4 = ((string)jobScenarioNameValue4);
                                                                        jobTaskInstance4.JobScenarioName = jobScenarioNameInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue11 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue11 != null && instanceTypeValue11.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance11 = ((string)instanceTypeValue11);
                                                                    jobTaskDetailsInstance2.Type = instanceTypeInstance11;
                                                                }
                                                                taskBaseInstance2.CustomDetails = jobTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance2 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue3 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue3 != null && skippedReasonValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance3 = ((string)skippedReasonValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReason = skippedReasonInstance3;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue3 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue3 != null && skippedReasonStringValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance3 = ((string)skippedReasonStringValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance3;
                                                                }
                                                                
                                                                JToken jobTaskValue5 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue5 != null && jobTaskValue5.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance5 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance2.JobTask = jobTaskInstance5;
                                                                    
                                                                    JToken jobIdValue6 = jobTaskValue5["jobId"];
                                                                    if (jobIdValue6 != null && jobIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance6 = ((string)jobIdValue6);
                                                                        jobTaskInstance5.JobId = jobIdInstance6;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue5 = jobTaskValue5["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue5 != null && jobFriendlyNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance5 = ((string)jobFriendlyNameValue5);
                                                                        jobTaskInstance5.JobFriendlyName = jobFriendlyNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue6 = jobTaskValue5["targetObjectId"];
                                                                    if (targetObjectIdValue6 != null && targetObjectIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance6 = ((string)targetObjectIdValue6);
                                                                        jobTaskInstance5.TargetObjectId = targetObjectIdInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue6 = jobTaskValue5["targetObjectName"];
                                                                    if (targetObjectNameValue6 != null && targetObjectNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance6 = ((string)targetObjectNameValue6);
                                                                        jobTaskInstance5.TargetObjectName = targetObjectNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue6 = jobTaskValue5["targetInstanceType"];
                                                                    if (targetInstanceTypeValue6 != null && targetInstanceTypeValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance6 = ((string)targetInstanceTypeValue6);
                                                                        jobTaskInstance5.TargetInstanceType = targetInstanceTypeInstance6;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue5 = jobTaskValue5["jobScenarioName"];
                                                                    if (jobScenarioNameValue5 != null && jobScenarioNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance5 = ((string)jobScenarioNameValue5);
                                                                        jobTaskInstance5.JobScenarioName = jobScenarioNameInstance5;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue12 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue12 != null && instanceTypeValue12.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance12 = ((string)instanceTypeValue12);
                                                                    virtualMachineTaskDetailsInstance2.Type = instanceTypeInstance12;
                                                                }
                                                                taskBaseInstance2.CustomDetails = virtualMachineTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance2 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue4 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue4 != null && skippedReasonValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance4 = ((string)skippedReasonValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReason = skippedReasonInstance4;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue4 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue4 != null && skippedReasonStringValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance4 = ((string)skippedReasonStringValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance4;
                                                                }
                                                                
                                                                JToken jobTaskValue6 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue6 != null && jobTaskValue6.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance6 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance2.JobTask = jobTaskInstance6;
                                                                    
                                                                    JToken jobIdValue7 = jobTaskValue6["jobId"];
                                                                    if (jobIdValue7 != null && jobIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance7 = ((string)jobIdValue7);
                                                                        jobTaskInstance6.JobId = jobIdInstance7;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue6 = jobTaskValue6["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue6 != null && jobFriendlyNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance6 = ((string)jobFriendlyNameValue6);
                                                                        jobTaskInstance6.JobFriendlyName = jobFriendlyNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue7 = jobTaskValue6["targetObjectId"];
                                                                    if (targetObjectIdValue7 != null && targetObjectIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance7 = ((string)targetObjectIdValue7);
                                                                        jobTaskInstance6.TargetObjectId = targetObjectIdInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue7 = jobTaskValue6["targetObjectName"];
                                                                    if (targetObjectNameValue7 != null && targetObjectNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance7 = ((string)targetObjectNameValue7);
                                                                        jobTaskInstance6.TargetObjectName = targetObjectNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue7 = jobTaskValue6["targetInstanceType"];
                                                                    if (targetInstanceTypeValue7 != null && targetInstanceTypeValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance7 = ((string)targetInstanceTypeValue7);
                                                                        jobTaskInstance6.TargetInstanceType = targetInstanceTypeInstance7;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue6 = jobTaskValue6["jobScenarioName"];
                                                                    if (jobScenarioNameValue6 != null && jobScenarioNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance6 = ((string)jobScenarioNameValue6);
                                                                        jobTaskInstance6.JobScenarioName = jobScenarioNameInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue13 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue13 != null && instanceTypeValue13.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance13 = ((string)instanceTypeValue13);
                                                                    fabricReplicationGroupTaskDetailsInstance2.Type = instanceTypeInstance13;
                                                                }
                                                                taskBaseInstance2.CustomDetails = fabricReplicationGroupTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance2 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue8 = customDetailsValue2["name"];
                                                                if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance8 = ((string)nameValue8);
                                                                    manualActionTaskDetailsInstance2.Name = nameInstance8;
                                                                }
                                                                
                                                                JToken instructionsValue2 = customDetailsValue2["instructions"];
                                                                if (instructionsValue2 != null && instructionsValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance2 = ((string)instructionsValue2);
                                                                    manualActionTaskDetailsInstance2.Instructions = instructionsInstance2;
                                                                }
                                                                
                                                                JToken observationValue2 = customDetailsValue2["observation"];
                                                                if (observationValue2 != null && observationValue2.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance2 = ((string)observationValue2);
                                                                    manualActionTaskDetailsInstance2.Observation = observationInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue14 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue14 != null && instanceTypeValue14.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance14 = ((string)instanceTypeValue14);
                                                                    manualActionTaskDetailsInstance2.Type = instanceTypeInstance14;
                                                                }
                                                                taskBaseInstance2.CustomDetails = manualActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance2 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue9 = customDetailsValue2["name"];
                                                                if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance9 = ((string)nameValue9);
                                                                    scriptActionTaskDetailsInstance2.Name = nameInstance9;
                                                                }
                                                                
                                                                JToken pathValue2 = customDetailsValue2["path"];
                                                                if (pathValue2 != null && pathValue2.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance2 = ((string)pathValue2);
                                                                    scriptActionTaskDetailsInstance2.Path = pathInstance2;
                                                                }
                                                                
                                                                JToken outputValue2 = customDetailsValue2["output"];
                                                                if (outputValue2 != null && outputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance2 = ((string)outputValue2);
                                                                    scriptActionTaskDetailsInstance2.Output = outputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue3 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue3 != null && isPrimarySideScriptValue3.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance3 = ((bool)isPrimarySideScriptValue3);
                                                                    scriptActionTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue15 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue15 != null && instanceTypeValue15.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance15 = ((string)instanceTypeValue15);
                                                                    scriptActionTaskDetailsInstance2.Type = instanceTypeInstance15;
                                                                }
                                                                taskBaseInstance2.CustomDetails = scriptActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance2 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue2 = customDetailsValue2["vmId"];
                                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.VmId = vmIdInstance2;
                                                                }
                                                                
                                                                JToken nicIdValue2 = customDetailsValue2["nicId"];
                                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.NicId = nicIdInstance2;
                                                                }
                                                                
                                                                JToken nameValue10 = customDetailsValue2["name"];
                                                                if (nameValue10 != null && nameValue10.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance10 = ((string)nameValue10);
                                                                    vmNicUpdatesTaskDetailsInstance2.Name = nameInstance10;
                                                                }
                                                                
                                                                JToken instanceTypeValue16 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue16 != null && instanceTypeValue16.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance16 = ((string)instanceTypeValue16);
                                                                    vmNicUpdatesTaskDetailsInstance2.Type = instanceTypeInstance16;
                                                                }
                                                                taskBaseInstance2.CustomDetails = vmNicUpdatesTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance2 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray2 = customDetailsValue2["vmDetails"];
                                                                if (vmDetailsArray2 != null && vmDetailsArray2.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue2 in ((JArray)vmDetailsArray2))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance2 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance2.VmDetails.Add(inconsistentVmDetailsInstance2);
                                                                        
                                                                        JToken vmNameValue2 = vmDetailsValue2["vmName"];
                                                                        if (vmNameValue2 != null && vmNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance2 = ((string)vmNameValue2);
                                                                            inconsistentVmDetailsInstance2.VmName = vmNameInstance2;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue2 = vmDetailsValue2["cloudName"];
                                                                        if (cloudNameValue2 != null && cloudNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance2 = ((string)cloudNameValue2);
                                                                            inconsistentVmDetailsInstance2.CloudName = cloudNameInstance2;
                                                                        }
                                                                        
                                                                        JToken detailsArray2 = vmDetailsValue2["details"];
                                                                        if (detailsArray2 != null && detailsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue2 in ((JArray)detailsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.Details.Add(((string)detailsValue2));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray2 = vmDetailsValue2["errorIds"];
                                                                        if (errorIdsArray2 != null && errorIdsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue2 in ((JArray)errorIdsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.ErrorIds.Add(((string)errorIdsValue2));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue17 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue17 != null && instanceTypeValue17.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance17 = ((string)instanceTypeValue17);
                                                                    consistencyCheckTaskDetailsInstance2.Type = instanceTypeInstance17;
                                                                }
                                                                taskBaseInstance2.CustomDetails = consistencyCheckTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance2 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue11 = customDetailsValue2["name"];
                                                                if (nameValue11 != null && nameValue11.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance11 = ((string)nameValue11);
                                                                    automationRunbookTaskDetailsInstance2.Name = nameInstance11;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue2 = customDetailsValue2["cloudServiceName"];
                                                                if (cloudServiceNameValue2 != null && cloudServiceNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance2 = ((string)cloudServiceNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.CloudServiceName = cloudServiceNameInstance2;
                                                                }
                                                                
                                                                JToken subscriptionIdValue2 = customDetailsValue2["subscriptionId"];
                                                                if (subscriptionIdValue2 != null && subscriptionIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance2 = ((string)subscriptionIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.SubscriptionId = subscriptionIdInstance2;
                                                                }
                                                                
                                                                JToken accountNameValue2 = customDetailsValue2["accountName"];
                                                                if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance2 = ((string)accountNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.AccountName = accountNameInstance2;
                                                                }
                                                                
                                                                JToken runbookIdValue2 = customDetailsValue2["runbookId"];
                                                                if (runbookIdValue2 != null && runbookIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance2 = ((string)runbookIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookId = runbookIdInstance2;
                                                                }
                                                                
                                                                JToken runbookNameValue2 = customDetailsValue2["runbookName"];
                                                                if (runbookNameValue2 != null && runbookNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance2 = ((string)runbookNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookName = runbookNameInstance2;
                                                                }
                                                                
                                                                JToken jobIdValue8 = customDetailsValue2["jobId"];
                                                                if (jobIdValue8 != null && jobIdValue8.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance8 = ((string)jobIdValue8);
                                                                    automationRunbookTaskDetailsInstance2.JobId = jobIdInstance8;
                                                                }
                                                                
                                                                JToken jobOutputValue2 = customDetailsValue2["jobOutput"];
                                                                if (jobOutputValue2 != null && jobOutputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance2 = ((string)jobOutputValue2);
                                                                    automationRunbookTaskDetailsInstance2.JobOutput = jobOutputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue4 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue4 != null && isPrimarySideScriptValue4.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance4 = ((bool)isPrimarySideScriptValue4);
                                                                    automationRunbookTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue18 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue18 != null && instanceTypeValue18.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance18 = ((string)instanceTypeValue18);
                                                                    automationRunbookTaskDetailsInstance2.Type = instanceTypeInstance18;
                                                                }
                                                                taskBaseInstance2.CustomDetails = automationRunbookTaskDetailsInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanGroupTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanShutdownGroupTaskDetails")
                                            {
                                                RecoveryPlanShutdownGroupTaskDetails recoveryPlanShutdownGroupTaskDetailsInstance = new RecoveryPlanShutdownGroupTaskDetails();
                                                
                                                JToken nameValue12 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue12 != null && nameValue12.Type != JTokenType.Null)
                                                {
                                                    string nameInstance12 = ((string)nameValue12);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Name = nameInstance12;
                                                }
                                                
                                                JToken groupIdValue2 = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue2 != null && groupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance2 = ((string)groupIdValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.GroupId = groupIdInstance2;
                                                }
                                                
                                                JToken rpGroupTypeValue2 = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue2 != null && rpGroupTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance2 = ((string)rpGroupTypeValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance2;
                                                }
                                                
                                                JToken instanceTypeValue19 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue19 != null && instanceTypeValue19.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance19 = ((string)instanceTypeValue19);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Type = instanceTypeInstance19;
                                                }
                                                
                                                JToken childTasksArray3 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray3 != null && childTasksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue3 in ((JArray)childTasksArray3))
                                                    {
                                                        AsrTaskBase taskBaseInstance3 = new AsrTaskBase();
                                                        recoveryPlanShutdownGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance3);
                                                        
                                                        JToken taskIdValue5 = childTasksValue3["taskId"];
                                                        if (taskIdValue5 != null && taskIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance5 = ((string)taskIdValue5);
                                                            taskBaseInstance3.ID = taskIdInstance5;
                                                        }
                                                        
                                                        JToken nameValue13 = childTasksValue3["name"];
                                                        if (nameValue13 != null && nameValue13.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance13 = ((string)nameValue13);
                                                            taskBaseInstance3.Name = nameInstance13;
                                                        }
                                                        
                                                        JToken startTimeValue4 = childTasksValue3["startTime"];
                                                        if (startTimeValue4 != null && startTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance4 = ((DateTime)startTimeValue4);
                                                            taskBaseInstance3.StartTime = startTimeInstance4;
                                                        }
                                                        
                                                        JToken endTimeValue4 = childTasksValue3["endTime"];
                                                        if (endTimeValue4 != null && endTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance4 = ((DateTime)endTimeValue4);
                                                            taskBaseInstance3.EndTime = endTimeInstance4;
                                                        }
                                                        
                                                        JToken allowedActionsArray4 = childTasksValue3["allowedActions"];
                                                        if (allowedActionsArray4 != null && allowedActionsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue4 in ((JArray)allowedActionsArray4))
                                                            {
                                                                taskBaseInstance3.Actions.Add(((string)allowedActionsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue4 = childTasksValue3["friendlyName"];
                                                        if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                                            taskBaseInstance3.TaskFriendlyName = friendlyNameInstance4;
                                                        }
                                                        
                                                        JToken stateValue4 = childTasksValue3["state"];
                                                        if (stateValue4 != null && stateValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance4 = ((string)stateValue4);
                                                            taskBaseInstance3.State = stateInstance4;
                                                        }
                                                        
                                                        JToken stateDescriptionValue4 = childTasksValue3["stateDescription"];
                                                        if (stateDescriptionValue4 != null && stateDescriptionValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance4 = ((string)stateDescriptionValue4);
                                                            taskBaseInstance3.StateDescription = stateDescriptionInstance4;
                                                        }
                                                        
                                                        JToken taskTypeValue3 = childTasksValue3["taskType"];
                                                        if (taskTypeValue3 != null && taskTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance3 = ((string)taskTypeValue3);
                                                            taskBaseInstance3.TaskType = taskTypeInstance3;
                                                        }
                                                        
                                                        JToken errorsArray3 = childTasksValue3["errors"];
                                                        if (errorsArray3 != null && errorsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue3 in ((JArray)errorsArray3))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance3 = new JobErrorDetails();
                                                                taskBaseInstance3.Errors.Add(jobErrorDetailsInstance3);
                                                                
                                                                JToken serviceErrorDetailsValue3 = errorsValue3["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue3 != null && serviceErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance3 = new ServiceError();
                                                                    jobErrorDetailsInstance3.ServiceErrorDetails = serviceErrorDetailsInstance3;
                                                                    
                                                                    JToken codeValue3 = serviceErrorDetailsValue3["code"];
                                                                    if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance3 = ((string)codeValue3);
                                                                        serviceErrorDetailsInstance3.Code = codeInstance3;
                                                                    }
                                                                    
                                                                    JToken messageValue3 = serviceErrorDetailsValue3["message"];
                                                                    if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance3 = ((string)messageValue3);
                                                                        serviceErrorDetailsInstance3.Message = messageInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue5 = serviceErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue5 != null && possibleCausesValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance5 = ((string)possibleCausesValue5);
                                                                        serviceErrorDetailsInstance3.PossibleCauses = possibleCausesInstance5;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue5 = serviceErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue5 != null && recommendedActionValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance5 = ((string)recommendedActionValue5);
                                                                        serviceErrorDetailsInstance3.RecommendedAction = recommendedActionInstance5;
                                                                    }
                                                                    
                                                                    JToken activityIdValue4 = serviceErrorDetailsValue3["activityId"];
                                                                    if (activityIdValue4 != null && activityIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance4 = ((string)activityIdValue4);
                                                                        serviceErrorDetailsInstance3.ActivityId = activityIdInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue3 = errorsValue3["providerErrorDetails"];
                                                                if (providerErrorDetailsValue3 != null && providerErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance3 = new ProviderError();
                                                                    jobErrorDetailsInstance3.ProviderErrorDetails = providerErrorDetailsInstance3;
                                                                    
                                                                    JToken errorCodeValue3 = providerErrorDetailsValue3["errorCode"];
                                                                    if (errorCodeValue3 != null && errorCodeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance3 = ((int)errorCodeValue3);
                                                                        providerErrorDetailsInstance3.ErrorCode = errorCodeInstance3;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue3 = providerErrorDetailsValue3["errorMessage"];
                                                                    if (errorMessageValue3 != null && errorMessageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance3 = ((string)errorMessageValue3);
                                                                        providerErrorDetailsInstance3.ErrorMessage = errorMessageInstance3;
                                                                    }
                                                                    
                                                                    JToken errorIdValue3 = providerErrorDetailsValue3["errorId"];
                                                                    if (errorIdValue3 != null && errorIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance3 = ((string)errorIdValue3);
                                                                        providerErrorDetailsInstance3.ErrorId = errorIdInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue6 = providerErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue6 != null && possibleCausesValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance6 = ((string)possibleCausesValue6);
                                                                        providerErrorDetailsInstance3.PossibleCauses = possibleCausesInstance6;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue6 = providerErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue6 != null && recommendedActionValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance6 = ((string)recommendedActionValue6);
                                                                        providerErrorDetailsInstance3.RecommendedAction = recommendedActionInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue3 = errorsValue3["errorLevel"];
                                                                if (errorLevelValue3 != null && errorLevelValue3.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance3 = ((string)errorLevelValue3);
                                                                    jobErrorDetailsInstance3.ErrorLevel = errorLevelInstance3;
                                                                }
                                                                
                                                                JToken creationTimeValue3 = errorsValue3["creationTime"];
                                                                if (creationTimeValue3 != null && creationTimeValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance3 = ((DateTime)creationTimeValue3);
                                                                    jobErrorDetailsInstance3.CreationTime = creationTimeInstance3;
                                                                }
                                                                
                                                                JToken taskIdValue6 = errorsValue3["taskId"];
                                                                if (taskIdValue6 != null && taskIdValue6.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance6 = ((string)taskIdValue6);
                                                                    jobErrorDetailsInstance3.TaskId = taskIdInstance6;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue3 = childTasksValue3["customDetails"];
                                                        if (customDetailsValue3 != null && customDetailsValue3.Type != JTokenType.Null)
                                                        {
                                                            string typeName4 = ((string)customDetailsValue3["instanceType"]);
                                                            if (typeName4 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance3 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue7 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue7 != null && jobTaskValue7.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance7 = new JobEntity();
                                                                    jobTaskDetailsInstance3.JobTask = jobTaskInstance7;
                                                                    
                                                                    JToken jobIdValue9 = jobTaskValue7["jobId"];
                                                                    if (jobIdValue9 != null && jobIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance9 = ((string)jobIdValue9);
                                                                        jobTaskInstance7.JobId = jobIdInstance9;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue7 = jobTaskValue7["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue7 != null && jobFriendlyNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance7 = ((string)jobFriendlyNameValue7);
                                                                        jobTaskInstance7.JobFriendlyName = jobFriendlyNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue8 = jobTaskValue7["targetObjectId"];
                                                                    if (targetObjectIdValue8 != null && targetObjectIdValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance8 = ((string)targetObjectIdValue8);
                                                                        jobTaskInstance7.TargetObjectId = targetObjectIdInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue8 = jobTaskValue7["targetObjectName"];
                                                                    if (targetObjectNameValue8 != null && targetObjectNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance8 = ((string)targetObjectNameValue8);
                                                                        jobTaskInstance7.TargetObjectName = targetObjectNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue8 = jobTaskValue7["targetInstanceType"];
                                                                    if (targetInstanceTypeValue8 != null && targetInstanceTypeValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance8 = ((string)targetInstanceTypeValue8);
                                                                        jobTaskInstance7.TargetInstanceType = targetInstanceTypeInstance8;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue7 = jobTaskValue7["jobScenarioName"];
                                                                    if (jobScenarioNameValue7 != null && jobScenarioNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance7 = ((string)jobScenarioNameValue7);
                                                                        jobTaskInstance7.JobScenarioName = jobScenarioNameInstance7;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue20 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue20 != null && instanceTypeValue20.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance20 = ((string)instanceTypeValue20);
                                                                    jobTaskDetailsInstance3.Type = instanceTypeInstance20;
                                                                }
                                                                taskBaseInstance3.CustomDetails = jobTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance3 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue5 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue5 != null && skippedReasonValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance5 = ((string)skippedReasonValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReason = skippedReasonInstance5;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue5 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue5 != null && skippedReasonStringValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance5 = ((string)skippedReasonStringValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance5;
                                                                }
                                                                
                                                                JToken jobTaskValue8 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue8 != null && jobTaskValue8.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance8 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance3.JobTask = jobTaskInstance8;
                                                                    
                                                                    JToken jobIdValue10 = jobTaskValue8["jobId"];
                                                                    if (jobIdValue10 != null && jobIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance10 = ((string)jobIdValue10);
                                                                        jobTaskInstance8.JobId = jobIdInstance10;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue8 = jobTaskValue8["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue8 != null && jobFriendlyNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance8 = ((string)jobFriendlyNameValue8);
                                                                        jobTaskInstance8.JobFriendlyName = jobFriendlyNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue9 = jobTaskValue8["targetObjectId"];
                                                                    if (targetObjectIdValue9 != null && targetObjectIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance9 = ((string)targetObjectIdValue9);
                                                                        jobTaskInstance8.TargetObjectId = targetObjectIdInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue9 = jobTaskValue8["targetObjectName"];
                                                                    if (targetObjectNameValue9 != null && targetObjectNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance9 = ((string)targetObjectNameValue9);
                                                                        jobTaskInstance8.TargetObjectName = targetObjectNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue9 = jobTaskValue8["targetInstanceType"];
                                                                    if (targetInstanceTypeValue9 != null && targetInstanceTypeValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance9 = ((string)targetInstanceTypeValue9);
                                                                        jobTaskInstance8.TargetInstanceType = targetInstanceTypeInstance9;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue8 = jobTaskValue8["jobScenarioName"];
                                                                    if (jobScenarioNameValue8 != null && jobScenarioNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance8 = ((string)jobScenarioNameValue8);
                                                                        jobTaskInstance8.JobScenarioName = jobScenarioNameInstance8;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue21 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue21 != null && instanceTypeValue21.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance21 = ((string)instanceTypeValue21);
                                                                    virtualMachineTaskDetailsInstance3.Type = instanceTypeInstance21;
                                                                }
                                                                taskBaseInstance3.CustomDetails = virtualMachineTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance3 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue6 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue6 != null && skippedReasonValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance6 = ((string)skippedReasonValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReason = skippedReasonInstance6;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue6 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue6 != null && skippedReasonStringValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance6 = ((string)skippedReasonStringValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance6;
                                                                }
                                                                
                                                                JToken jobTaskValue9 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue9 != null && jobTaskValue9.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance9 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance3.JobTask = jobTaskInstance9;
                                                                    
                                                                    JToken jobIdValue11 = jobTaskValue9["jobId"];
                                                                    if (jobIdValue11 != null && jobIdValue11.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance11 = ((string)jobIdValue11);
                                                                        jobTaskInstance9.JobId = jobIdInstance11;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue9 = jobTaskValue9["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue9 != null && jobFriendlyNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance9 = ((string)jobFriendlyNameValue9);
                                                                        jobTaskInstance9.JobFriendlyName = jobFriendlyNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue10 = jobTaskValue9["targetObjectId"];
                                                                    if (targetObjectIdValue10 != null && targetObjectIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance10 = ((string)targetObjectIdValue10);
                                                                        jobTaskInstance9.TargetObjectId = targetObjectIdInstance10;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue10 = jobTaskValue9["targetObjectName"];
                                                                    if (targetObjectNameValue10 != null && targetObjectNameValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance10 = ((string)targetObjectNameValue10);
                                                                        jobTaskInstance9.TargetObjectName = targetObjectNameInstance10;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue10 = jobTaskValue9["targetInstanceType"];
                                                                    if (targetInstanceTypeValue10 != null && targetInstanceTypeValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance10 = ((string)targetInstanceTypeValue10);
                                                                        jobTaskInstance9.TargetInstanceType = targetInstanceTypeInstance10;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue9 = jobTaskValue9["jobScenarioName"];
                                                                    if (jobScenarioNameValue9 != null && jobScenarioNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance9 = ((string)jobScenarioNameValue9);
                                                                        jobTaskInstance9.JobScenarioName = jobScenarioNameInstance9;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue22 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue22 != null && instanceTypeValue22.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance22 = ((string)instanceTypeValue22);
                                                                    fabricReplicationGroupTaskDetailsInstance3.Type = instanceTypeInstance22;
                                                                }
                                                                taskBaseInstance3.CustomDetails = fabricReplicationGroupTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance3 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue14 = customDetailsValue3["name"];
                                                                if (nameValue14 != null && nameValue14.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance14 = ((string)nameValue14);
                                                                    manualActionTaskDetailsInstance3.Name = nameInstance14;
                                                                }
                                                                
                                                                JToken instructionsValue3 = customDetailsValue3["instructions"];
                                                                if (instructionsValue3 != null && instructionsValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance3 = ((string)instructionsValue3);
                                                                    manualActionTaskDetailsInstance3.Instructions = instructionsInstance3;
                                                                }
                                                                
                                                                JToken observationValue3 = customDetailsValue3["observation"];
                                                                if (observationValue3 != null && observationValue3.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance3 = ((string)observationValue3);
                                                                    manualActionTaskDetailsInstance3.Observation = observationInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue23 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue23 != null && instanceTypeValue23.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance23 = ((string)instanceTypeValue23);
                                                                    manualActionTaskDetailsInstance3.Type = instanceTypeInstance23;
                                                                }
                                                                taskBaseInstance3.CustomDetails = manualActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance3 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue15 = customDetailsValue3["name"];
                                                                if (nameValue15 != null && nameValue15.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance15 = ((string)nameValue15);
                                                                    scriptActionTaskDetailsInstance3.Name = nameInstance15;
                                                                }
                                                                
                                                                JToken pathValue3 = customDetailsValue3["path"];
                                                                if (pathValue3 != null && pathValue3.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance3 = ((string)pathValue3);
                                                                    scriptActionTaskDetailsInstance3.Path = pathInstance3;
                                                                }
                                                                
                                                                JToken outputValue3 = customDetailsValue3["output"];
                                                                if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance3 = ((string)outputValue3);
                                                                    scriptActionTaskDetailsInstance3.Output = outputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue5 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue5 != null && isPrimarySideScriptValue5.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance5 = ((bool)isPrimarySideScriptValue5);
                                                                    scriptActionTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance5;
                                                                }
                                                                
                                                                JToken instanceTypeValue24 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue24 != null && instanceTypeValue24.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance24 = ((string)instanceTypeValue24);
                                                                    scriptActionTaskDetailsInstance3.Type = instanceTypeInstance24;
                                                                }
                                                                taskBaseInstance3.CustomDetails = scriptActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance3 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue3 = customDetailsValue3["vmId"];
                                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.VmId = vmIdInstance3;
                                                                }
                                                                
                                                                JToken nicIdValue3 = customDetailsValue3["nicId"];
                                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.NicId = nicIdInstance3;
                                                                }
                                                                
                                                                JToken nameValue16 = customDetailsValue3["name"];
                                                                if (nameValue16 != null && nameValue16.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance16 = ((string)nameValue16);
                                                                    vmNicUpdatesTaskDetailsInstance3.Name = nameInstance16;
                                                                }
                                                                
                                                                JToken instanceTypeValue25 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue25 != null && instanceTypeValue25.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance25 = ((string)instanceTypeValue25);
                                                                    vmNicUpdatesTaskDetailsInstance3.Type = instanceTypeInstance25;
                                                                }
                                                                taskBaseInstance3.CustomDetails = vmNicUpdatesTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance3 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray3 = customDetailsValue3["vmDetails"];
                                                                if (vmDetailsArray3 != null && vmDetailsArray3.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue3 in ((JArray)vmDetailsArray3))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance3 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance3.VmDetails.Add(inconsistentVmDetailsInstance3);
                                                                        
                                                                        JToken vmNameValue3 = vmDetailsValue3["vmName"];
                                                                        if (vmNameValue3 != null && vmNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance3 = ((string)vmNameValue3);
                                                                            inconsistentVmDetailsInstance3.VmName = vmNameInstance3;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue3 = vmDetailsValue3["cloudName"];
                                                                        if (cloudNameValue3 != null && cloudNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance3 = ((string)cloudNameValue3);
                                                                            inconsistentVmDetailsInstance3.CloudName = cloudNameInstance3;
                                                                        }
                                                                        
                                                                        JToken detailsArray3 = vmDetailsValue3["details"];
                                                                        if (detailsArray3 != null && detailsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue3 in ((JArray)detailsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.Details.Add(((string)detailsValue3));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray3 = vmDetailsValue3["errorIds"];
                                                                        if (errorIdsArray3 != null && errorIdsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue3 in ((JArray)errorIdsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.ErrorIds.Add(((string)errorIdsValue3));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue26 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue26 != null && instanceTypeValue26.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance26 = ((string)instanceTypeValue26);
                                                                    consistencyCheckTaskDetailsInstance3.Type = instanceTypeInstance26;
                                                                }
                                                                taskBaseInstance3.CustomDetails = consistencyCheckTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance3 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue17 = customDetailsValue3["name"];
                                                                if (nameValue17 != null && nameValue17.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance17 = ((string)nameValue17);
                                                                    automationRunbookTaskDetailsInstance3.Name = nameInstance17;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue3 = customDetailsValue3["cloudServiceName"];
                                                                if (cloudServiceNameValue3 != null && cloudServiceNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance3 = ((string)cloudServiceNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.CloudServiceName = cloudServiceNameInstance3;
                                                                }
                                                                
                                                                JToken subscriptionIdValue3 = customDetailsValue3["subscriptionId"];
                                                                if (subscriptionIdValue3 != null && subscriptionIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance3 = ((string)subscriptionIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.SubscriptionId = subscriptionIdInstance3;
                                                                }
                                                                
                                                                JToken accountNameValue3 = customDetailsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken runbookIdValue3 = customDetailsValue3["runbookId"];
                                                                if (runbookIdValue3 != null && runbookIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance3 = ((string)runbookIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookId = runbookIdInstance3;
                                                                }
                                                                
                                                                JToken runbookNameValue3 = customDetailsValue3["runbookName"];
                                                                if (runbookNameValue3 != null && runbookNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance3 = ((string)runbookNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookName = runbookNameInstance3;
                                                                }
                                                                
                                                                JToken jobIdValue12 = customDetailsValue3["jobId"];
                                                                if (jobIdValue12 != null && jobIdValue12.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance12 = ((string)jobIdValue12);
                                                                    automationRunbookTaskDetailsInstance3.JobId = jobIdInstance12;
                                                                }
                                                                
                                                                JToken jobOutputValue3 = customDetailsValue3["jobOutput"];
                                                                if (jobOutputValue3 != null && jobOutputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance3 = ((string)jobOutputValue3);
                                                                    automationRunbookTaskDetailsInstance3.JobOutput = jobOutputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue6 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue6 != null && isPrimarySideScriptValue6.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance6 = ((bool)isPrimarySideScriptValue6);
                                                                    automationRunbookTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance6;
                                                                }
                                                                
                                                                JToken instanceTypeValue27 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue27 != null && instanceTypeValue27.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance27 = ((string)instanceTypeValue27);
                                                                    automationRunbookTaskDetailsInstance3.Type = instanceTypeInstance27;
                                                                }
                                                                taskBaseInstance3.CustomDetails = automationRunbookTaskDetailsInstance3;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanShutdownGroupTaskDetailsInstance;
                                            }
                                        }
                                        
                                        JToken taskIdValue7 = tasksValue["taskId"];
                                        if (taskIdValue7 != null && taskIdValue7.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance7 = ((string)taskIdValue7);
                                            taskInstance.ID = taskIdInstance7;
                                        }
                                        
                                        JToken nameValue18 = tasksValue["name"];
                                        if (nameValue18 != null && nameValue18.Type != JTokenType.Null)
                                        {
                                            string nameInstance18 = ((string)nameValue18);
                                            taskInstance.Name = nameInstance18;
                                        }
                                        
                                        JToken startTimeValue5 = tasksValue["startTime"];
                                        if (startTimeValue5 != null && startTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime startTimeInstance5 = ((DateTime)startTimeValue5);
                                            taskInstance.StartTime = startTimeInstance5;
                                        }
                                        
                                        JToken endTimeValue5 = tasksValue["endTime"];
                                        if (endTimeValue5 != null && endTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime endTimeInstance5 = ((DateTime)endTimeValue5);
                                            taskInstance.EndTime = endTimeInstance5;
                                        }
                                        
                                        JToken allowedActionsArray5 = tasksValue["allowedActions"];
                                        if (allowedActionsArray5 != null && allowedActionsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedActionsValue5 in ((JArray)allowedActionsArray5))
                                            {
                                                taskInstance.Actions.Add(((string)allowedActionsValue5));
                                            }
                                        }
                                        
                                        JToken friendlyNameValue5 = tasksValue["friendlyName"];
                                        if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                            taskInstance.TaskFriendlyName = friendlyNameInstance5;
                                        }
                                        
                                        JToken stateValue5 = tasksValue["state"];
                                        if (stateValue5 != null && stateValue5.Type != JTokenType.Null)
                                        {
                                            string stateInstance5 = ((string)stateValue5);
                                            taskInstance.State = stateInstance5;
                                        }
                                        
                                        JToken stateDescriptionValue5 = tasksValue["stateDescription"];
                                        if (stateDescriptionValue5 != null && stateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string stateDescriptionInstance5 = ((string)stateDescriptionValue5);
                                            taskInstance.StateDescription = stateDescriptionInstance5;
                                        }
                                        
                                        JToken taskTypeValue4 = tasksValue["taskType"];
                                        if (taskTypeValue4 != null && taskTypeValue4.Type != JTokenType.Null)
                                        {
                                            string taskTypeInstance4 = ((string)taskTypeValue4);
                                            taskInstance.TaskType = taskTypeInstance4;
                                        }
                                        
                                        JToken errorsArray4 = tasksValue["errors"];
                                        if (errorsArray4 != null && errorsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken errorsValue4 in ((JArray)errorsArray4))
                                            {
                                                JobErrorDetails jobErrorDetailsInstance4 = new JobErrorDetails();
                                                taskInstance.Errors.Add(jobErrorDetailsInstance4);
                                                
                                                JToken serviceErrorDetailsValue4 = errorsValue4["serviceErrorDetails"];
                                                if (serviceErrorDetailsValue4 != null && serviceErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ServiceError serviceErrorDetailsInstance4 = new ServiceError();
                                                    jobErrorDetailsInstance4.ServiceErrorDetails = serviceErrorDetailsInstance4;
                                                    
                                                    JToken codeValue4 = serviceErrorDetailsValue4["code"];
                                                    if (codeValue4 != null && codeValue4.Type != JTokenType.Null)
                                                    {
                                                        string codeInstance4 = ((string)codeValue4);
                                                        serviceErrorDetailsInstance4.Code = codeInstance4;
                                                    }
                                                    
                                                    JToken messageValue4 = serviceErrorDetailsValue4["message"];
                                                    if (messageValue4 != null && messageValue4.Type != JTokenType.Null)
                                                    {
                                                        string messageInstance4 = ((string)messageValue4);
                                                        serviceErrorDetailsInstance4.Message = messageInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue7 = serviceErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue7 != null && possibleCausesValue7.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance7 = ((string)possibleCausesValue7);
                                                        serviceErrorDetailsInstance4.PossibleCauses = possibleCausesInstance7;
                                                    }
                                                    
                                                    JToken recommendedActionValue7 = serviceErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue7 != null && recommendedActionValue7.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance7 = ((string)recommendedActionValue7);
                                                        serviceErrorDetailsInstance4.RecommendedAction = recommendedActionInstance7;
                                                    }
                                                    
                                                    JToken activityIdValue5 = serviceErrorDetailsValue4["activityId"];
                                                    if (activityIdValue5 != null && activityIdValue5.Type != JTokenType.Null)
                                                    {
                                                        string activityIdInstance5 = ((string)activityIdValue5);
                                                        serviceErrorDetailsInstance4.ActivityId = activityIdInstance5;
                                                    }
                                                }
                                                
                                                JToken providerErrorDetailsValue4 = errorsValue4["providerErrorDetails"];
                                                if (providerErrorDetailsValue4 != null && providerErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ProviderError providerErrorDetailsInstance4 = new ProviderError();
                                                    jobErrorDetailsInstance4.ProviderErrorDetails = providerErrorDetailsInstance4;
                                                    
                                                    JToken errorCodeValue4 = providerErrorDetailsValue4["errorCode"];
                                                    if (errorCodeValue4 != null && errorCodeValue4.Type != JTokenType.Null)
                                                    {
                                                        int errorCodeInstance4 = ((int)errorCodeValue4);
                                                        providerErrorDetailsInstance4.ErrorCode = errorCodeInstance4;
                                                    }
                                                    
                                                    JToken errorMessageValue4 = providerErrorDetailsValue4["errorMessage"];
                                                    if (errorMessageValue4 != null && errorMessageValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorMessageInstance4 = ((string)errorMessageValue4);
                                                        providerErrorDetailsInstance4.ErrorMessage = errorMessageInstance4;
                                                    }
                                                    
                                                    JToken errorIdValue4 = providerErrorDetailsValue4["errorId"];
                                                    if (errorIdValue4 != null && errorIdValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorIdInstance4 = ((string)errorIdValue4);
                                                        providerErrorDetailsInstance4.ErrorId = errorIdInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue8 = providerErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue8 != null && possibleCausesValue8.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance8 = ((string)possibleCausesValue8);
                                                        providerErrorDetailsInstance4.PossibleCauses = possibleCausesInstance8;
                                                    }
                                                    
                                                    JToken recommendedActionValue8 = providerErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue8 != null && recommendedActionValue8.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance8 = ((string)recommendedActionValue8);
                                                        providerErrorDetailsInstance4.RecommendedAction = recommendedActionInstance8;
                                                    }
                                                }
                                                
                                                JToken errorLevelValue4 = errorsValue4["errorLevel"];
                                                if (errorLevelValue4 != null && errorLevelValue4.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance4 = ((string)errorLevelValue4);
                                                    jobErrorDetailsInstance4.ErrorLevel = errorLevelInstance4;
                                                }
                                                
                                                JToken creationTimeValue4 = errorsValue4["creationTime"];
                                                if (creationTimeValue4 != null && creationTimeValue4.Type != JTokenType.Null)
                                                {
                                                    DateTime creationTimeInstance4 = ((DateTime)creationTimeValue4);
                                                    jobErrorDetailsInstance4.CreationTime = creationTimeInstance4;
                                                }
                                                
                                                JToken taskIdValue8 = errorsValue4["taskId"];
                                                if (taskIdValue8 != null && taskIdValue8.Type != JTokenType.Null)
                                                {
                                                    string taskIdInstance8 = ((string)taskIdValue8);
                                                    jobErrorDetailsInstance4.TaskId = taskIdInstance8;
                                                }
                                            }
                                        }
                                        
                                        JToken customDetailsValue4 = tasksValue["customDetails"];
                                        if (customDetailsValue4 != null && customDetailsValue4.Type != JTokenType.Null)
                                        {
                                            string typeName5 = ((string)customDetailsValue4["instanceType"]);
                                            if (typeName5 == "JobTaskDetails")
                                            {
                                                JobTaskDetails jobTaskDetailsInstance4 = new JobTaskDetails();
                                                
                                                JToken jobTaskValue10 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue10 != null && jobTaskValue10.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance10 = new JobEntity();
                                                    jobTaskDetailsInstance4.JobTask = jobTaskInstance10;
                                                    
                                                    JToken jobIdValue13 = jobTaskValue10["jobId"];
                                                    if (jobIdValue13 != null && jobIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance13 = ((string)jobIdValue13);
                                                        jobTaskInstance10.JobId = jobIdInstance13;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue10 = jobTaskValue10["jobFriendlyName"];
                                                    if (jobFriendlyNameValue10 != null && jobFriendlyNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance10 = ((string)jobFriendlyNameValue10);
                                                        jobTaskInstance10.JobFriendlyName = jobFriendlyNameInstance10;
                                                    }
                                                    
                                                    JToken targetObjectIdValue11 = jobTaskValue10["targetObjectId"];
                                                    if (targetObjectIdValue11 != null && targetObjectIdValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance11 = ((string)targetObjectIdValue11);
                                                        jobTaskInstance10.TargetObjectId = targetObjectIdInstance11;
                                                    }
                                                    
                                                    JToken targetObjectNameValue11 = jobTaskValue10["targetObjectName"];
                                                    if (targetObjectNameValue11 != null && targetObjectNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance11 = ((string)targetObjectNameValue11);
                                                        jobTaskInstance10.TargetObjectName = targetObjectNameInstance11;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue11 = jobTaskValue10["targetInstanceType"];
                                                    if (targetInstanceTypeValue11 != null && targetInstanceTypeValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance11 = ((string)targetInstanceTypeValue11);
                                                        jobTaskInstance10.TargetInstanceType = targetInstanceTypeInstance11;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue10 = jobTaskValue10["jobScenarioName"];
                                                    if (jobScenarioNameValue10 != null && jobScenarioNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance10 = ((string)jobScenarioNameValue10);
                                                        jobTaskInstance10.JobScenarioName = jobScenarioNameInstance10;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue28 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue28 != null && instanceTypeValue28.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance28 = ((string)instanceTypeValue28);
                                                    jobTaskDetailsInstance4.Type = instanceTypeInstance28;
                                                }
                                                taskInstance.CustomDetails = jobTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VirtualMachineTaskDetails")
                                            {
                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance4 = new VirtualMachineTaskDetails();
                                                
                                                JToken skippedReasonValue7 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue7 != null && skippedReasonValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance7 = ((string)skippedReasonValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReason = skippedReasonInstance7;
                                                }
                                                
                                                JToken skippedReasonStringValue7 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue7 != null && skippedReasonStringValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance7 = ((string)skippedReasonStringValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance7;
                                                }
                                                
                                                JToken jobTaskValue11 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue11 != null && jobTaskValue11.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance11 = new JobEntity();
                                                    virtualMachineTaskDetailsInstance4.JobTask = jobTaskInstance11;
                                                    
                                                    JToken jobIdValue14 = jobTaskValue11["jobId"];
                                                    if (jobIdValue14 != null && jobIdValue14.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance14 = ((string)jobIdValue14);
                                                        jobTaskInstance11.JobId = jobIdInstance14;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue11 = jobTaskValue11["jobFriendlyName"];
                                                    if (jobFriendlyNameValue11 != null && jobFriendlyNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance11 = ((string)jobFriendlyNameValue11);
                                                        jobTaskInstance11.JobFriendlyName = jobFriendlyNameInstance11;
                                                    }
                                                    
                                                    JToken targetObjectIdValue12 = jobTaskValue11["targetObjectId"];
                                                    if (targetObjectIdValue12 != null && targetObjectIdValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance12 = ((string)targetObjectIdValue12);
                                                        jobTaskInstance11.TargetObjectId = targetObjectIdInstance12;
                                                    }
                                                    
                                                    JToken targetObjectNameValue12 = jobTaskValue11["targetObjectName"];
                                                    if (targetObjectNameValue12 != null && targetObjectNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance12 = ((string)targetObjectNameValue12);
                                                        jobTaskInstance11.TargetObjectName = targetObjectNameInstance12;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue12 = jobTaskValue11["targetInstanceType"];
                                                    if (targetInstanceTypeValue12 != null && targetInstanceTypeValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance12 = ((string)targetInstanceTypeValue12);
                                                        jobTaskInstance11.TargetInstanceType = targetInstanceTypeInstance12;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue11 = jobTaskValue11["jobScenarioName"];
                                                    if (jobScenarioNameValue11 != null && jobScenarioNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance11 = ((string)jobScenarioNameValue11);
                                                        jobTaskInstance11.JobScenarioName = jobScenarioNameInstance11;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue29 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue29 != null && instanceTypeValue29.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance29 = ((string)instanceTypeValue29);
                                                    virtualMachineTaskDetailsInstance4.Type = instanceTypeInstance29;
                                                }
                                                taskInstance.CustomDetails = virtualMachineTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "FabricReplicationGroupTaskDetails")
                                            {
                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance4 = new FabricReplicationGroupTaskDetails();
                                                
                                                JToken skippedReasonValue8 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue8 != null && skippedReasonValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance8 = ((string)skippedReasonValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReason = skippedReasonInstance8;
                                                }
                                                
                                                JToken skippedReasonStringValue8 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue8 != null && skippedReasonStringValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance8 = ((string)skippedReasonStringValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance8;
                                                }
                                                
                                                JToken jobTaskValue12 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue12 != null && jobTaskValue12.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance12 = new JobEntity();
                                                    fabricReplicationGroupTaskDetailsInstance4.JobTask = jobTaskInstance12;
                                                    
                                                    JToken jobIdValue15 = jobTaskValue12["jobId"];
                                                    if (jobIdValue15 != null && jobIdValue15.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance15 = ((string)jobIdValue15);
                                                        jobTaskInstance12.JobId = jobIdInstance15;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue12 = jobTaskValue12["jobFriendlyName"];
                                                    if (jobFriendlyNameValue12 != null && jobFriendlyNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance12 = ((string)jobFriendlyNameValue12);
                                                        jobTaskInstance12.JobFriendlyName = jobFriendlyNameInstance12;
                                                    }
                                                    
                                                    JToken targetObjectIdValue13 = jobTaskValue12["targetObjectId"];
                                                    if (targetObjectIdValue13 != null && targetObjectIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance13 = ((string)targetObjectIdValue13);
                                                        jobTaskInstance12.TargetObjectId = targetObjectIdInstance13;
                                                    }
                                                    
                                                    JToken targetObjectNameValue13 = jobTaskValue12["targetObjectName"];
                                                    if (targetObjectNameValue13 != null && targetObjectNameValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance13 = ((string)targetObjectNameValue13);
                                                        jobTaskInstance12.TargetObjectName = targetObjectNameInstance13;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue13 = jobTaskValue12["targetInstanceType"];
                                                    if (targetInstanceTypeValue13 != null && targetInstanceTypeValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance13 = ((string)targetInstanceTypeValue13);
                                                        jobTaskInstance12.TargetInstanceType = targetInstanceTypeInstance13;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue12 = jobTaskValue12["jobScenarioName"];
                                                    if (jobScenarioNameValue12 != null && jobScenarioNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance12 = ((string)jobScenarioNameValue12);
                                                        jobTaskInstance12.JobScenarioName = jobScenarioNameInstance12;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue30 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue30 != null && instanceTypeValue30.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance30 = ((string)instanceTypeValue30);
                                                    fabricReplicationGroupTaskDetailsInstance4.Type = instanceTypeInstance30;
                                                }
                                                taskInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ManualActionTaskDetails")
                                            {
                                                ManualActionTaskDetails manualActionTaskDetailsInstance4 = new ManualActionTaskDetails();
                                                
                                                JToken nameValue19 = customDetailsValue4["name"];
                                                if (nameValue19 != null && nameValue19.Type != JTokenType.Null)
                                                {
                                                    string nameInstance19 = ((string)nameValue19);
                                                    manualActionTaskDetailsInstance4.Name = nameInstance19;
                                                }
                                                
                                                JToken instructionsValue4 = customDetailsValue4["instructions"];
                                                if (instructionsValue4 != null && instructionsValue4.Type != JTokenType.Null)
                                                {
                                                    string instructionsInstance4 = ((string)instructionsValue4);
                                                    manualActionTaskDetailsInstance4.Instructions = instructionsInstance4;
                                                }
                                                
                                                JToken observationValue4 = customDetailsValue4["observation"];
                                                if (observationValue4 != null && observationValue4.Type != JTokenType.Null)
                                                {
                                                    string observationInstance4 = ((string)observationValue4);
                                                    manualActionTaskDetailsInstance4.Observation = observationInstance4;
                                                }
                                                
                                                JToken instanceTypeValue31 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue31 != null && instanceTypeValue31.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance31 = ((string)instanceTypeValue31);
                                                    manualActionTaskDetailsInstance4.Type = instanceTypeInstance31;
                                                }
                                                taskInstance.CustomDetails = manualActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ScriptActionTaskDetails")
                                            {
                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance4 = new ScriptActionTaskDetails();
                                                
                                                JToken nameValue20 = customDetailsValue4["name"];
                                                if (nameValue20 != null && nameValue20.Type != JTokenType.Null)
                                                {
                                                    string nameInstance20 = ((string)nameValue20);
                                                    scriptActionTaskDetailsInstance4.Name = nameInstance20;
                                                }
                                                
                                                JToken pathValue4 = customDetailsValue4["path"];
                                                if (pathValue4 != null && pathValue4.Type != JTokenType.Null)
                                                {
                                                    string pathInstance4 = ((string)pathValue4);
                                                    scriptActionTaskDetailsInstance4.Path = pathInstance4;
                                                }
                                                
                                                JToken outputValue4 = customDetailsValue4["output"];
                                                if (outputValue4 != null && outputValue4.Type != JTokenType.Null)
                                                {
                                                    string outputInstance4 = ((string)outputValue4);
                                                    scriptActionTaskDetailsInstance4.Output = outputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue7 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue7 != null && isPrimarySideScriptValue7.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance7 = ((bool)isPrimarySideScriptValue7);
                                                    scriptActionTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance7;
                                                }
                                                
                                                JToken instanceTypeValue32 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue32 != null && instanceTypeValue32.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance32 = ((string)instanceTypeValue32);
                                                    scriptActionTaskDetailsInstance4.Type = instanceTypeInstance32;
                                                }
                                                taskInstance.CustomDetails = scriptActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VmNicUpdatesTaskDetails")
                                            {
                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance4 = new VmNicUpdatesTaskDetails();
                                                
                                                JToken vmIdValue4 = customDetailsValue4["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken nicIdValue4 = customDetailsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken nameValue21 = customDetailsValue4["name"];
                                                if (nameValue21 != null && nameValue21.Type != JTokenType.Null)
                                                {
                                                    string nameInstance21 = ((string)nameValue21);
                                                    vmNicUpdatesTaskDetailsInstance4.Name = nameInstance21;
                                                }
                                                
                                                JToken instanceTypeValue33 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue33 != null && instanceTypeValue33.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance33 = ((string)instanceTypeValue33);
                                                    vmNicUpdatesTaskDetailsInstance4.Type = instanceTypeInstance33;
                                                }
                                                taskInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ConsistencyCheckTaskDetails")
                                            {
                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance4 = new ConsistencyCheckTaskDetails();
                                                
                                                JToken vmDetailsArray4 = customDetailsValue4["vmDetails"];
                                                if (vmDetailsArray4 != null && vmDetailsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmDetailsValue4 in ((JArray)vmDetailsArray4))
                                                    {
                                                        InconsistentVmDetails inconsistentVmDetailsInstance4 = new InconsistentVmDetails();
                                                        consistencyCheckTaskDetailsInstance4.VmDetails.Add(inconsistentVmDetailsInstance4);
                                                        
                                                        JToken vmNameValue4 = vmDetailsValue4["vmName"];
                                                        if (vmNameValue4 != null && vmNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vmNameInstance4 = ((string)vmNameValue4);
                                                            inconsistentVmDetailsInstance4.VmName = vmNameInstance4;
                                                        }
                                                        
                                                        JToken cloudNameValue4 = vmDetailsValue4["cloudName"];
                                                        if (cloudNameValue4 != null && cloudNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string cloudNameInstance4 = ((string)cloudNameValue4);
                                                            inconsistentVmDetailsInstance4.CloudName = cloudNameInstance4;
                                                        }
                                                        
                                                        JToken detailsArray4 = vmDetailsValue4["details"];
                                                        if (detailsArray4 != null && detailsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken detailsValue4 in ((JArray)detailsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.Details.Add(((string)detailsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken errorIdsArray4 = vmDetailsValue4["errorIds"];
                                                        if (errorIdsArray4 != null && errorIdsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorIdsValue4 in ((JArray)errorIdsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.ErrorIds.Add(((string)errorIdsValue4));
                                                            }
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue34 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue34 != null && instanceTypeValue34.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance34 = ((string)instanceTypeValue34);
                                                    consistencyCheckTaskDetailsInstance4.Type = instanceTypeInstance34;
                                                }
                                                taskInstance.CustomDetails = consistencyCheckTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "AutomationRunbookTaskDetails")
                                            {
                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance4 = new AutomationRunbookTaskDetails();
                                                
                                                JToken nameValue22 = customDetailsValue4["name"];
                                                if (nameValue22 != null && nameValue22.Type != JTokenType.Null)
                                                {
                                                    string nameInstance22 = ((string)nameValue22);
                                                    automationRunbookTaskDetailsInstance4.Name = nameInstance22;
                                                }
                                                
                                                JToken cloudServiceNameValue4 = customDetailsValue4["cloudServiceName"];
                                                if (cloudServiceNameValue4 != null && cloudServiceNameValue4.Type != JTokenType.Null)
                                                {
                                                    string cloudServiceNameInstance4 = ((string)cloudServiceNameValue4);
                                                    automationRunbookTaskDetailsInstance4.CloudServiceName = cloudServiceNameInstance4;
                                                }
                                                
                                                JToken subscriptionIdValue4 = customDetailsValue4["subscriptionId"];
                                                if (subscriptionIdValue4 != null && subscriptionIdValue4.Type != JTokenType.Null)
                                                {
                                                    string subscriptionIdInstance4 = ((string)subscriptionIdValue4);
                                                    automationRunbookTaskDetailsInstance4.SubscriptionId = subscriptionIdInstance4;
                                                }
                                                
                                                JToken accountNameValue4 = customDetailsValue4["accountName"];
                                                if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                {
                                                    string accountNameInstance4 = ((string)accountNameValue4);
                                                    automationRunbookTaskDetailsInstance4.AccountName = accountNameInstance4;
                                                }
                                                
                                                JToken runbookIdValue4 = customDetailsValue4["runbookId"];
                                                if (runbookIdValue4 != null && runbookIdValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookIdInstance4 = ((string)runbookIdValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookId = runbookIdInstance4;
                                                }
                                                
                                                JToken runbookNameValue4 = customDetailsValue4["runbookName"];
                                                if (runbookNameValue4 != null && runbookNameValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookNameInstance4 = ((string)runbookNameValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookName = runbookNameInstance4;
                                                }
                                                
                                                JToken jobIdValue16 = customDetailsValue4["jobId"];
                                                if (jobIdValue16 != null && jobIdValue16.Type != JTokenType.Null)
                                                {
                                                    string jobIdInstance16 = ((string)jobIdValue16);
                                                    automationRunbookTaskDetailsInstance4.JobId = jobIdInstance16;
                                                }
                                                
                                                JToken jobOutputValue4 = customDetailsValue4["jobOutput"];
                                                if (jobOutputValue4 != null && jobOutputValue4.Type != JTokenType.Null)
                                                {
                                                    string jobOutputInstance4 = ((string)jobOutputValue4);
                                                    automationRunbookTaskDetailsInstance4.JobOutput = jobOutputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue8 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue8 != null && isPrimarySideScriptValue8.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance8 = ((bool)isPrimarySideScriptValue8);
                                                    automationRunbookTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance8;
                                                }
                                                
                                                JToken instanceTypeValue35 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue35 != null && instanceTypeValue35.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance35 = ((string)instanceTypeValue35);
                                                    automationRunbookTaskDetailsInstance4.Type = instanceTypeInstance35;
                                                }
                                                taskInstance.CustomDetails = automationRunbookTaskDetailsInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken errorsArray5 = propertiesValue["errors"];
                                if (errorsArray5 != null && errorsArray5.Type != JTokenType.Null)
                                {
                                    foreach (JToken errorsValue5 in ((JArray)errorsArray5))
                                    {
                                        JobErrorDetails jobErrorDetailsInstance5 = new JobErrorDetails();
                                        propertiesInstance.Errors.Add(jobErrorDetailsInstance5);
                                        
                                        JToken serviceErrorDetailsValue5 = errorsValue5["serviceErrorDetails"];
                                        if (serviceErrorDetailsValue5 != null && serviceErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ServiceError serviceErrorDetailsInstance5 = new ServiceError();
                                            jobErrorDetailsInstance5.ServiceErrorDetails = serviceErrorDetailsInstance5;
                                            
                                            JToken codeValue5 = serviceErrorDetailsValue5["code"];
                                            if (codeValue5 != null && codeValue5.Type != JTokenType.Null)
                                            {
                                                string codeInstance5 = ((string)codeValue5);
                                                serviceErrorDetailsInstance5.Code = codeInstance5;
                                            }
                                            
                                            JToken messageValue5 = serviceErrorDetailsValue5["message"];
                                            if (messageValue5 != null && messageValue5.Type != JTokenType.Null)
                                            {
                                                string messageInstance5 = ((string)messageValue5);
                                                serviceErrorDetailsInstance5.Message = messageInstance5;
                                            }
                                            
                                            JToken possibleCausesValue9 = serviceErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue9 != null && possibleCausesValue9.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance9 = ((string)possibleCausesValue9);
                                                serviceErrorDetailsInstance5.PossibleCauses = possibleCausesInstance9;
                                            }
                                            
                                            JToken recommendedActionValue9 = serviceErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue9 != null && recommendedActionValue9.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance9 = ((string)recommendedActionValue9);
                                                serviceErrorDetailsInstance5.RecommendedAction = recommendedActionInstance9;
                                            }
                                            
                                            JToken activityIdValue6 = serviceErrorDetailsValue5["activityId"];
                                            if (activityIdValue6 != null && activityIdValue6.Type != JTokenType.Null)
                                            {
                                                string activityIdInstance6 = ((string)activityIdValue6);
                                                serviceErrorDetailsInstance5.ActivityId = activityIdInstance6;
                                            }
                                        }
                                        
                                        JToken providerErrorDetailsValue5 = errorsValue5["providerErrorDetails"];
                                        if (providerErrorDetailsValue5 != null && providerErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ProviderError providerErrorDetailsInstance5 = new ProviderError();
                                            jobErrorDetailsInstance5.ProviderErrorDetails = providerErrorDetailsInstance5;
                                            
                                            JToken errorCodeValue5 = providerErrorDetailsValue5["errorCode"];
                                            if (errorCodeValue5 != null && errorCodeValue5.Type != JTokenType.Null)
                                            {
                                                int errorCodeInstance5 = ((int)errorCodeValue5);
                                                providerErrorDetailsInstance5.ErrorCode = errorCodeInstance5;
                                            }
                                            
                                            JToken errorMessageValue5 = providerErrorDetailsValue5["errorMessage"];
                                            if (errorMessageValue5 != null && errorMessageValue5.Type != JTokenType.Null)
                                            {
                                                string errorMessageInstance5 = ((string)errorMessageValue5);
                                                providerErrorDetailsInstance5.ErrorMessage = errorMessageInstance5;
                                            }
                                            
                                            JToken errorIdValue5 = providerErrorDetailsValue5["errorId"];
                                            if (errorIdValue5 != null && errorIdValue5.Type != JTokenType.Null)
                                            {
                                                string errorIdInstance5 = ((string)errorIdValue5);
                                                providerErrorDetailsInstance5.ErrorId = errorIdInstance5;
                                            }
                                            
                                            JToken possibleCausesValue10 = providerErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue10 != null && possibleCausesValue10.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance10 = ((string)possibleCausesValue10);
                                                providerErrorDetailsInstance5.PossibleCauses = possibleCausesInstance10;
                                            }
                                            
                                            JToken recommendedActionValue10 = providerErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue10 != null && recommendedActionValue10.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance10 = ((string)recommendedActionValue10);
                                                providerErrorDetailsInstance5.RecommendedAction = recommendedActionInstance10;
                                            }
                                        }
                                        
                                        JToken errorLevelValue5 = errorsValue5["errorLevel"];
                                        if (errorLevelValue5 != null && errorLevelValue5.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance5 = ((string)errorLevelValue5);
                                            jobErrorDetailsInstance5.ErrorLevel = errorLevelInstance5;
                                        }
                                        
                                        JToken creationTimeValue5 = errorsValue5["creationTime"];
                                        if (creationTimeValue5 != null && creationTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime creationTimeInstance5 = ((DateTime)creationTimeValue5);
                                            jobErrorDetailsInstance5.CreationTime = creationTimeInstance5;
                                        }
                                        
                                        JToken taskIdValue9 = errorsValue5["taskId"];
                                        if (taskIdValue9 != null && taskIdValue9.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance9 = ((string)taskIdValue9);
                                            jobErrorDetailsInstance5.TaskId = taskIdInstance9;
                                        }
                                    }
                                }
                                
                                JToken customDetailsValue5 = propertiesValue["customDetails"];
                                if (customDetailsValue5 != null && customDetailsValue5.Type != JTokenType.Null)
                                {
                                    string typeName6 = ((string)customDetailsValue5["instanceType"]);
                                    if (typeName6 == "ExportJobDetails")
                                    {
                                        ExportJobDetails exportJobDetailsInstance = new ExportJobDetails();
                                        
                                        JToken blobUriValue = customDetailsValue5["blobUri"];
                                        if (blobUriValue != null && blobUriValue.Type != JTokenType.Null)
                                        {
                                            string blobUriInstance = ((string)blobUriValue);
                                            exportJobDetailsInstance.BlobUri = blobUriInstance;
                                        }
                                        
                                        JToken sasTokenValue = customDetailsValue5["sasToken"];
                                        if (sasTokenValue != null && sasTokenValue.Type != JTokenType.Null)
                                        {
                                            string sasTokenInstance = ((string)sasTokenValue);
                                            exportJobDetailsInstance.SasToken = sasTokenInstance;
                                        }
                                        
                                        JToken instanceTypeValue36 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue36 != null && instanceTypeValue36.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance36 = ((string)instanceTypeValue36);
                                            exportJobDetailsInstance.InstanceType = instanceTypeInstance36;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement != null && affectedObjectDetailsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in affectedObjectDetailsSequenceElement)
                                            {
                                                string affectedObjectDetailsKey = ((string)property.Name);
                                                string affectedObjectDetailsValue = ((string)property.Value);
                                                exportJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey, affectedObjectDetailsValue);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = exportJobDetailsInstance;
                                    }
                                    if (typeName6 == "AsrJobDetails")
                                    {
                                        AsrJobDetails asrJobDetailsInstance = new AsrJobDetails();
                                        
                                        JToken instanceTypeValue37 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue37 != null && instanceTypeValue37.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance37 = ((string)instanceTypeValue37);
                                            asrJobDetailsInstance.InstanceType = instanceTypeInstance37;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement2 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement2 != null && affectedObjectDetailsSequenceElement2.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property2 in affectedObjectDetailsSequenceElement2)
                                            {
                                                string affectedObjectDetailsKey2 = ((string)property2.Name);
                                                string affectedObjectDetailsValue2 = ((string)property2.Value);
                                                asrJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey2, affectedObjectDetailsValue2);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = asrJobDetailsInstance;
                                    }
                                    if (typeName6 == "TestFailoverJobDetails")
                                    {
                                        TestFailoverJobDetails testFailoverJobDetailsInstance = new TestFailoverJobDetails();
                                        
                                        JToken testFailoverStatusValue = customDetailsValue5["testFailoverStatus"];
                                        if (testFailoverStatusValue != null && testFailoverStatusValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStatusInstance = ((string)testFailoverStatusValue);
                                            testFailoverJobDetailsInstance.TestFailoverStatus = testFailoverStatusInstance;
                                        }
                                        
                                        JToken commentsValue = customDetailsValue5["comments"];
                                        if (commentsValue != null && commentsValue.Type != JTokenType.Null)
                                        {
                                            string commentsInstance = ((string)commentsValue);
                                            testFailoverJobDetailsInstance.Comments = commentsInstance;
                                        }
                                        
                                        JToken networkNameValue = customDetailsValue5["networkName"];
                                        if (networkNameValue != null && networkNameValue.Type != JTokenType.Null)
                                        {
                                            string networkNameInstance = ((string)networkNameValue);
                                            testFailoverJobDetailsInstance.NetworkName = networkNameInstance;
                                        }
                                        
                                        JToken networkFriendlyNameValue = customDetailsValue5["networkFriendlyName"];
                                        if (networkFriendlyNameValue != null && networkFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string networkFriendlyNameInstance = ((string)networkFriendlyNameValue);
                                            testFailoverJobDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance;
                                        }
                                        
                                        JToken networkTypeValue = customDetailsValue5["networkType"];
                                        if (networkTypeValue != null && networkTypeValue.Type != JTokenType.Null)
                                        {
                                            string networkTypeInstance = ((string)networkTypeValue);
                                            testFailoverJobDetailsInstance.NetworkType = networkTypeInstance;
                                        }
                                        
                                        JToken protectedItemDetailsArray = customDetailsValue5["protectedItemDetails"];
                                        if (protectedItemDetailsArray != null && protectedItemDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedItemDetailsValue in ((JArray)protectedItemDetailsArray))
                                            {
                                                TestFailoverReplicationProtectedItemDetails testFailoverReplicationProtectedItemDetailsInstance = new TestFailoverReplicationProtectedItemDetails();
                                                testFailoverJobDetailsInstance.ProtectedItemDetails.Add(testFailoverReplicationProtectedItemDetailsInstance);
                                                
                                                JToken nameValue23 = protectedItemDetailsValue["name"];
                                                if (nameValue23 != null && nameValue23.Type != JTokenType.Null)
                                                {
                                                    string nameInstance23 = ((string)nameValue23);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Name = nameInstance23;
                                                }
                                                
                                                JToken friendlyNameValue6 = protectedItemDetailsValue["friendlyName"];
                                                if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                                {
                                                    string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                                    testFailoverReplicationProtectedItemDetailsInstance.FriendlyName = friendlyNameInstance6;
                                                }
                                                
                                                JToken testVmNameValue = protectedItemDetailsValue["testVmName"];
                                                if (testVmNameValue != null && testVmNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmNameInstance = ((string)testVmNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmName = testVmNameInstance;
                                                }
                                                
                                                JToken testVmFriendlyNameValue = protectedItemDetailsValue["testVmFriendlyName"];
                                                if (testVmFriendlyNameValue != null && testVmFriendlyNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmFriendlyNameInstance = ((string)testVmFriendlyNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmFriendlyName = testVmFriendlyNameInstance;
                                                }
                                                
                                                JToken networkConnectionStatusValue = protectedItemDetailsValue["networkConnectionStatus"];
                                                if (networkConnectionStatusValue != null && networkConnectionStatusValue.Type != JTokenType.Null)
                                                {
                                                    string networkConnectionStatusInstance = ((string)networkConnectionStatusValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkConnectionStatus = networkConnectionStatusInstance;
                                                }
                                                
                                                JToken networkFriendlyNameValue2 = protectedItemDetailsValue["networkFriendlyName"];
                                                if (networkFriendlyNameValue2 != null && networkFriendlyNameValue2.Type != JTokenType.Null)
                                                {
                                                    string networkFriendlyNameInstance2 = ((string)networkFriendlyNameValue2);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance2;
                                                }
                                                
                                                JToken subnetValue = protectedItemDetailsValue["subnet"];
                                                if (subnetValue != null && subnetValue.Type != JTokenType.Null)
                                                {
                                                    string subnetInstance = ((string)subnetValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Subnet = subnetInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue38 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue38 != null && instanceTypeValue38.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance38 = ((string)instanceTypeValue38);
                                            testFailoverJobDetailsInstance.InstanceType = instanceTypeInstance38;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement3 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement3 != null && affectedObjectDetailsSequenceElement3.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property3 in affectedObjectDetailsSequenceElement3)
                                            {
                                                string affectedObjectDetailsKey3 = ((string)property3.Name);
                                                string affectedObjectDetailsValue3 = ((string)property3.Value);
                                                testFailoverJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey3, affectedObjectDetailsValue3);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = testFailoverJobDetailsInstance;
                                    }
                                }
                                
                                JToken idValue = propertiesValue["id"];
                                if (idValue != null && idValue.Type != JTokenType.Null)
                                {
                                    string idInstance = ((string)idValue);
                                    propertiesInstance.Id = idInstance;
                                }
                                
                                JToken nameValue24 = propertiesValue["name"];
                                if (nameValue24 != null && nameValue24.Type != JTokenType.Null)
                                {
                                    string nameInstance24 = ((string)nameValue24);
                                    propertiesInstance.Name = nameInstance24;
                                }
                                
                                JToken typeValue = propertiesValue["type"];
                                if (typeValue != null && typeValue.Type != JTokenType.Null)
                                {
                                    string typeInstance = ((string)typeValue);
                                    propertiesInstance.Type = typeInstance;
                                }
                                
                                JToken locationValue = propertiesValue["location"];
                                if (locationValue != null && locationValue.Type != JTokenType.Null)
                                {
                                    string locationInstance = ((string)locationValue);
                                    propertiesInstance.Location = locationInstance;
                                }
                                
                                JToken tagsSequenceElement = ((JToken)propertiesValue["tags"]);
                                if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property4 in tagsSequenceElement)
                                    {
                                        string tagsKey = ((string)property4.Name);
                                        string tagsValue = ((string)property4.Value);
                                        propertiesInstance.Tags.Add(tagsKey, tagsValue);
                                    }
                                }
                            }
                            
                            JToken idValue2 = responseDoc["id"];
                            if (idValue2 != null && idValue2.Type != JTokenType.Null)
                            {
                                string idInstance2 = ((string)idValue2);
                                jobInstance.Id = idInstance2;
                            }
                            
                            JToken nameValue25 = responseDoc["name"];
                            if (nameValue25 != null && nameValue25.Type != JTokenType.Null)
                            {
                                string nameInstance25 = ((string)nameValue25);
                                jobInstance.Name = nameInstance25;
                            }
                            
                            JToken typeValue2 = responseDoc["type"];
                            if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                            {
                                string typeInstance2 = ((string)typeValue2);
                                jobInstance.Type = typeInstance2;
                            }
                            
                            JToken locationValue2 = responseDoc["location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                jobInstance.Location = locationInstance2;
                            }
                            
                            JToken tagsSequenceElement2 = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement2 != null && tagsSequenceElement2.Type != JTokenType.Null)
                            {
                                foreach (JProperty property5 in tagsSequenceElement2)
                                {
                                    string tagsKey2 = ((string)property5.Name);
                                    string tagsValue2 = ((string)property5.Value);
                                    jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken locationValue3 = responseDoc["Location"];
                            if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                            {
                                string locationInstance3 = ((string)locationValue3);
                                result.Location = locationInstance3;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<JobOperationResponse> GetRestartStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetRestartStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted || statusCode == HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken scenarioNameValue = propertiesValue["scenarioName"];
                                if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                {
                                    string scenarioNameInstance = ((string)scenarioNameValue);
                                    propertiesInstance.ScenarioName = scenarioNameInstance;
                                }
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken activityIdValue = propertiesValue["activityId"];
                                if (activityIdValue != null && activityIdValue.Type != JTokenType.Null)
                                {
                                    string activityIdInstance = ((string)activityIdValue);
                                    propertiesInstance.ActivityId = activityIdInstance;
                                }
                                
                                JToken startTimeValue = propertiesValue["startTime"];
                                if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime startTimeInstance = ((DateTime)startTimeValue);
                                    propertiesInstance.StartTime = startTimeInstance;
                                }
                                
                                JToken endTimeValue = propertiesValue["endTime"];
                                if (endTimeValue != null && endTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime endTimeInstance = ((DateTime)endTimeValue);
                                    propertiesInstance.EndTime = endTimeInstance;
                                }
                                
                                JToken targetObjectIdValue = propertiesValue["targetObjectId"];
                                if (targetObjectIdValue != null && targetObjectIdValue.Type != JTokenType.Null)
                                {
                                    string targetObjectIdInstance = ((string)targetObjectIdValue);
                                    propertiesInstance.TargetObjectId = targetObjectIdInstance;
                                }
                                
                                JToken targetInstanceTypeValue = propertiesValue["targetInstanceType"];
                                if (targetInstanceTypeValue != null && targetInstanceTypeValue.Type != JTokenType.Null)
                                {
                                    string targetInstanceTypeInstance = ((string)targetInstanceTypeValue);
                                    propertiesInstance.TargetInstanceType = targetInstanceTypeInstance;
                                }
                                
                                JToken targetObjectNameValue = propertiesValue["targetObjectName"];
                                if (targetObjectNameValue != null && targetObjectNameValue.Type != JTokenType.Null)
                                {
                                    string targetObjectNameInstance = ((string)targetObjectNameValue);
                                    propertiesInstance.TargetObjectName = targetObjectNameInstance;
                                }
                                
                                JToken allowedActionsArray = propertiesValue["allowedActions"];
                                if (allowedActionsArray != null && allowedActionsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedActionsValue in ((JArray)allowedActionsArray))
                                    {
                                        propertiesInstance.AllowedActions.Add(((string)allowedActionsValue));
                                    }
                                }
                                
                                JToken stateValue = propertiesValue["state"];
                                if (stateValue != null && stateValue.Type != JTokenType.Null)
                                {
                                    string stateInstance = ((string)stateValue);
                                    propertiesInstance.State = stateInstance;
                                }
                                
                                JToken stateDescriptionValue = propertiesValue["stateDescription"];
                                if (stateDescriptionValue != null && stateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string stateDescriptionInstance = ((string)stateDescriptionValue);
                                    propertiesInstance.StateDescription = stateDescriptionInstance;
                                }
                                
                                JToken tasksArray = propertiesValue["tasks"];
                                if (tasksArray != null && tasksArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken tasksValue in ((JArray)tasksArray))
                                    {
                                        AsrTask taskInstance = new AsrTask();
                                        propertiesInstance.Tasks.Add(taskInstance);
                                        
                                        JToken groupTaskCustomDetailsValue = tasksValue["groupTaskCustomDetails"];
                                        if (groupTaskCustomDetailsValue != null && groupTaskCustomDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)groupTaskCustomDetailsValue["instanceType"]);
                                            if (typeName == "InlineWorkflowTaskDetails")
                                            {
                                                InlineWorkflowTaskDetails inlineWorkflowTaskDetailsInstance = new InlineWorkflowTaskDetails();
                                                
                                                JToken workflowIdsArray = groupTaskCustomDetailsValue["workflowIds"];
                                                if (workflowIdsArray != null && workflowIdsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken workflowIdsValue in ((JArray)workflowIdsArray))
                                                    {
                                                        inlineWorkflowTaskDetailsInstance.WorkflowIds.Add(((string)workflowIdsValue));
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    inlineWorkflowTaskDetailsInstance.Type = instanceTypeInstance;
                                                }
                                                
                                                JToken childTasksArray = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray != null && childTasksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue in ((JArray)childTasksArray))
                                                    {
                                                        AsrTaskBase taskBaseInstance = new AsrTaskBase();
                                                        inlineWorkflowTaskDetailsInstance.ChildTasks.Add(taskBaseInstance);
                                                        
                                                        JToken taskIdValue = childTasksValue["taskId"];
                                                        if (taskIdValue != null && taskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance = ((string)taskIdValue);
                                                            taskBaseInstance.ID = taskIdInstance;
                                                        }
                                                        
                                                        JToken nameValue = childTasksValue["name"];
                                                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance = ((string)nameValue);
                                                            taskBaseInstance.Name = nameInstance;
                                                        }
                                                        
                                                        JToken startTimeValue2 = childTasksValue["startTime"];
                                                        if (startTimeValue2 != null && startTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance2 = ((DateTime)startTimeValue2);
                                                            taskBaseInstance.StartTime = startTimeInstance2;
                                                        }
                                                        
                                                        JToken endTimeValue2 = childTasksValue["endTime"];
                                                        if (endTimeValue2 != null && endTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance2 = ((DateTime)endTimeValue2);
                                                            taskBaseInstance.EndTime = endTimeInstance2;
                                                        }
                                                        
                                                        JToken allowedActionsArray2 = childTasksValue["allowedActions"];
                                                        if (allowedActionsArray2 != null && allowedActionsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue2 in ((JArray)allowedActionsArray2))
                                                            {
                                                                taskBaseInstance.Actions.Add(((string)allowedActionsValue2));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue2 = childTasksValue["friendlyName"];
                                                        if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                                            taskBaseInstance.TaskFriendlyName = friendlyNameInstance2;
                                                        }
                                                        
                                                        JToken stateValue2 = childTasksValue["state"];
                                                        if (stateValue2 != null && stateValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance2 = ((string)stateValue2);
                                                            taskBaseInstance.State = stateInstance2;
                                                        }
                                                        
                                                        JToken stateDescriptionValue2 = childTasksValue["stateDescription"];
                                                        if (stateDescriptionValue2 != null && stateDescriptionValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance2 = ((string)stateDescriptionValue2);
                                                            taskBaseInstance.StateDescription = stateDescriptionInstance2;
                                                        }
                                                        
                                                        JToken taskTypeValue = childTasksValue["taskType"];
                                                        if (taskTypeValue != null && taskTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance = ((string)taskTypeValue);
                                                            taskBaseInstance.TaskType = taskTypeInstance;
                                                        }
                                                        
                                                        JToken errorsArray = childTasksValue["errors"];
                                                        if (errorsArray != null && errorsArray.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue in ((JArray)errorsArray))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance = new JobErrorDetails();
                                                                taskBaseInstance.Errors.Add(jobErrorDetailsInstance);
                                                                
                                                                JToken serviceErrorDetailsValue = errorsValue["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue != null && serviceErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance = new ServiceError();
                                                                    jobErrorDetailsInstance.ServiceErrorDetails = serviceErrorDetailsInstance;
                                                                    
                                                                    JToken codeValue = serviceErrorDetailsValue["code"];
                                                                    if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance = ((string)codeValue);
                                                                        serviceErrorDetailsInstance.Code = codeInstance;
                                                                    }
                                                                    
                                                                    JToken messageValue = serviceErrorDetailsValue["message"];
                                                                    if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance = ((string)messageValue);
                                                                        serviceErrorDetailsInstance.Message = messageInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue = serviceErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance = ((string)possibleCausesValue);
                                                                        serviceErrorDetailsInstance.PossibleCauses = possibleCausesInstance;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue = serviceErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance = ((string)recommendedActionValue);
                                                                        serviceErrorDetailsInstance.RecommendedAction = recommendedActionInstance;
                                                                    }
                                                                    
                                                                    JToken activityIdValue2 = serviceErrorDetailsValue["activityId"];
                                                                    if (activityIdValue2 != null && activityIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance2 = ((string)activityIdValue2);
                                                                        serviceErrorDetailsInstance.ActivityId = activityIdInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue = errorsValue["providerErrorDetails"];
                                                                if (providerErrorDetailsValue != null && providerErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance = new ProviderError();
                                                                    jobErrorDetailsInstance.ProviderErrorDetails = providerErrorDetailsInstance;
                                                                    
                                                                    JToken errorCodeValue = providerErrorDetailsValue["errorCode"];
                                                                    if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance = ((int)errorCodeValue);
                                                                        providerErrorDetailsInstance.ErrorCode = errorCodeInstance;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue = providerErrorDetailsValue["errorMessage"];
                                                                    if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance = ((string)errorMessageValue);
                                                                        providerErrorDetailsInstance.ErrorMessage = errorMessageInstance;
                                                                    }
                                                                    
                                                                    JToken errorIdValue = providerErrorDetailsValue["errorId"];
                                                                    if (errorIdValue != null && errorIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance = ((string)errorIdValue);
                                                                        providerErrorDetailsInstance.ErrorId = errorIdInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue2 = providerErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue2 != null && possibleCausesValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance2 = ((string)possibleCausesValue2);
                                                                        providerErrorDetailsInstance.PossibleCauses = possibleCausesInstance2;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue2 = providerErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue2 != null && recommendedActionValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance2 = ((string)recommendedActionValue2);
                                                                        providerErrorDetailsInstance.RecommendedAction = recommendedActionInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue = errorsValue["errorLevel"];
                                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                                    jobErrorDetailsInstance.ErrorLevel = errorLevelInstance;
                                                                }
                                                                
                                                                JToken creationTimeValue = errorsValue["creationTime"];
                                                                if (creationTimeValue != null && creationTimeValue.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance = ((DateTime)creationTimeValue);
                                                                    jobErrorDetailsInstance.CreationTime = creationTimeInstance;
                                                                }
                                                                
                                                                JToken taskIdValue2 = errorsValue["taskId"];
                                                                if (taskIdValue2 != null && taskIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance2 = ((string)taskIdValue2);
                                                                    jobErrorDetailsInstance.TaskId = taskIdInstance2;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue = childTasksValue["customDetails"];
                                                        if (customDetailsValue != null && customDetailsValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName2 = ((string)customDetailsValue["instanceType"]);
                                                            if (typeName2 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue = customDetailsValue["jobTask"];
                                                                if (jobTaskValue != null && jobTaskValue.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance = new JobEntity();
                                                                    jobTaskDetailsInstance.JobTask = jobTaskInstance;
                                                                    
                                                                    JToken jobIdValue = jobTaskValue["jobId"];
                                                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance = ((string)jobIdValue);
                                                                        jobTaskInstance.JobId = jobIdInstance;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue = jobTaskValue["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue != null && jobFriendlyNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance = ((string)jobFriendlyNameValue);
                                                                        jobTaskInstance.JobFriendlyName = jobFriendlyNameInstance;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue2 = jobTaskValue["targetObjectId"];
                                                                    if (targetObjectIdValue2 != null && targetObjectIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance2 = ((string)targetObjectIdValue2);
                                                                        jobTaskInstance.TargetObjectId = targetObjectIdInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue2 = jobTaskValue["targetObjectName"];
                                                                    if (targetObjectNameValue2 != null && targetObjectNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance2 = ((string)targetObjectNameValue2);
                                                                        jobTaskInstance.TargetObjectName = targetObjectNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue2 = jobTaskValue["targetInstanceType"];
                                                                    if (targetInstanceTypeValue2 != null && targetInstanceTypeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance2 = ((string)targetInstanceTypeValue2);
                                                                        jobTaskInstance.TargetInstanceType = targetInstanceTypeInstance2;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue = jobTaskValue["jobScenarioName"];
                                                                    if (jobScenarioNameValue != null && jobScenarioNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance = ((string)jobScenarioNameValue);
                                                                        jobTaskInstance.JobScenarioName = jobScenarioNameInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue2 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                                    jobTaskDetailsInstance.Type = instanceTypeInstance2;
                                                                }
                                                                taskBaseInstance.CustomDetails = jobTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue != null && skippedReasonValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance = ((string)skippedReasonValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReason = skippedReasonInstance;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue != null && skippedReasonStringValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance = ((string)skippedReasonStringValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance;
                                                                }
                                                                
                                                                JToken jobTaskValue2 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue2 != null && jobTaskValue2.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance2 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance.JobTask = jobTaskInstance2;
                                                                    
                                                                    JToken jobIdValue2 = jobTaskValue2["jobId"];
                                                                    if (jobIdValue2 != null && jobIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance2 = ((string)jobIdValue2);
                                                                        jobTaskInstance2.JobId = jobIdInstance2;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue2 = jobTaskValue2["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue2 != null && jobFriendlyNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance2 = ((string)jobFriendlyNameValue2);
                                                                        jobTaskInstance2.JobFriendlyName = jobFriendlyNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue3 = jobTaskValue2["targetObjectId"];
                                                                    if (targetObjectIdValue3 != null && targetObjectIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance3 = ((string)targetObjectIdValue3);
                                                                        jobTaskInstance2.TargetObjectId = targetObjectIdInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue3 = jobTaskValue2["targetObjectName"];
                                                                    if (targetObjectNameValue3 != null && targetObjectNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance3 = ((string)targetObjectNameValue3);
                                                                        jobTaskInstance2.TargetObjectName = targetObjectNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue3 = jobTaskValue2["targetInstanceType"];
                                                                    if (targetInstanceTypeValue3 != null && targetInstanceTypeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance3 = ((string)targetInstanceTypeValue3);
                                                                        jobTaskInstance2.TargetInstanceType = targetInstanceTypeInstance3;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue2 = jobTaskValue2["jobScenarioName"];
                                                                    if (jobScenarioNameValue2 != null && jobScenarioNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance2 = ((string)jobScenarioNameValue2);
                                                                        jobTaskInstance2.JobScenarioName = jobScenarioNameInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue3 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                                    virtualMachineTaskDetailsInstance.Type = instanceTypeInstance3;
                                                                }
                                                                taskBaseInstance.CustomDetails = virtualMachineTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue2 = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue2 != null && skippedReasonValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance2 = ((string)skippedReasonValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReason = skippedReasonInstance2;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue2 = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue2 != null && skippedReasonStringValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance2 = ((string)skippedReasonStringValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance2;
                                                                }
                                                                
                                                                JToken jobTaskValue3 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue3 != null && jobTaskValue3.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance3 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance.JobTask = jobTaskInstance3;
                                                                    
                                                                    JToken jobIdValue3 = jobTaskValue3["jobId"];
                                                                    if (jobIdValue3 != null && jobIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance3 = ((string)jobIdValue3);
                                                                        jobTaskInstance3.JobId = jobIdInstance3;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue3 = jobTaskValue3["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue3 != null && jobFriendlyNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance3 = ((string)jobFriendlyNameValue3);
                                                                        jobTaskInstance3.JobFriendlyName = jobFriendlyNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue4 = jobTaskValue3["targetObjectId"];
                                                                    if (targetObjectIdValue4 != null && targetObjectIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance4 = ((string)targetObjectIdValue4);
                                                                        jobTaskInstance3.TargetObjectId = targetObjectIdInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue4 = jobTaskValue3["targetObjectName"];
                                                                    if (targetObjectNameValue4 != null && targetObjectNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance4 = ((string)targetObjectNameValue4);
                                                                        jobTaskInstance3.TargetObjectName = targetObjectNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue4 = jobTaskValue3["targetInstanceType"];
                                                                    if (targetInstanceTypeValue4 != null && targetInstanceTypeValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance4 = ((string)targetInstanceTypeValue4);
                                                                        jobTaskInstance3.TargetInstanceType = targetInstanceTypeInstance4;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue3 = jobTaskValue3["jobScenarioName"];
                                                                    if (jobScenarioNameValue3 != null && jobScenarioNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance3 = ((string)jobScenarioNameValue3);
                                                                        jobTaskInstance3.JobScenarioName = jobScenarioNameInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue4 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance.Type = instanceTypeInstance4;
                                                                }
                                                                taskBaseInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue2 = customDetailsValue["name"];
                                                                if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance2 = ((string)nameValue2);
                                                                    manualActionTaskDetailsInstance.Name = nameInstance2;
                                                                }
                                                                
                                                                JToken instructionsValue = customDetailsValue["instructions"];
                                                                if (instructionsValue != null && instructionsValue.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance = ((string)instructionsValue);
                                                                    manualActionTaskDetailsInstance.Instructions = instructionsInstance;
                                                                }
                                                                
                                                                JToken observationValue = customDetailsValue["observation"];
                                                                if (observationValue != null && observationValue.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance = ((string)observationValue);
                                                                    manualActionTaskDetailsInstance.Observation = observationInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue5 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                                    manualActionTaskDetailsInstance.Type = instanceTypeInstance5;
                                                                }
                                                                taskBaseInstance.CustomDetails = manualActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue3 = customDetailsValue["name"];
                                                                if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance3 = ((string)nameValue3);
                                                                    scriptActionTaskDetailsInstance.Name = nameInstance3;
                                                                }
                                                                
                                                                JToken pathValue = customDetailsValue["path"];
                                                                if (pathValue != null && pathValue.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance = ((string)pathValue);
                                                                    scriptActionTaskDetailsInstance.Path = pathInstance;
                                                                }
                                                                
                                                                JToken outputValue = customDetailsValue["output"];
                                                                if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance = ((string)outputValue);
                                                                    scriptActionTaskDetailsInstance.Output = outputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue != null && isPrimarySideScriptValue.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance = ((bool)isPrimarySideScriptValue);
                                                                    scriptActionTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue6 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                                    scriptActionTaskDetailsInstance.Type = instanceTypeInstance6;
                                                                }
                                                                taskBaseInstance.CustomDetails = scriptActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue = customDetailsValue["vmId"];
                                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance = ((string)vmIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.VmId = vmIdInstance;
                                                                }
                                                                
                                                                JToken nicIdValue = customDetailsValue["nicId"];
                                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance = ((string)nicIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.NicId = nicIdInstance;
                                                                }
                                                                
                                                                JToken nameValue4 = customDetailsValue["name"];
                                                                if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance4 = ((string)nameValue4);
                                                                    vmNicUpdatesTaskDetailsInstance.Name = nameInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue7 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue7 != null && instanceTypeValue7.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance7 = ((string)instanceTypeValue7);
                                                                    vmNicUpdatesTaskDetailsInstance.Type = instanceTypeInstance7;
                                                                }
                                                                taskBaseInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray = customDetailsValue["vmDetails"];
                                                                if (vmDetailsArray != null && vmDetailsArray.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue in ((JArray)vmDetailsArray))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance.VmDetails.Add(inconsistentVmDetailsInstance);
                                                                        
                                                                        JToken vmNameValue = vmDetailsValue["vmName"];
                                                                        if (vmNameValue != null && vmNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance = ((string)vmNameValue);
                                                                            inconsistentVmDetailsInstance.VmName = vmNameInstance;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue = vmDetailsValue["cloudName"];
                                                                        if (cloudNameValue != null && cloudNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance = ((string)cloudNameValue);
                                                                            inconsistentVmDetailsInstance.CloudName = cloudNameInstance;
                                                                        }
                                                                        
                                                                        JToken detailsArray = vmDetailsValue["details"];
                                                                        if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue in ((JArray)detailsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.Details.Add(((string)detailsValue));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray = vmDetailsValue["errorIds"];
                                                                        if (errorIdsArray != null && errorIdsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue in ((JArray)errorIdsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.ErrorIds.Add(((string)errorIdsValue));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue8 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue8 != null && instanceTypeValue8.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance8 = ((string)instanceTypeValue8);
                                                                    consistencyCheckTaskDetailsInstance.Type = instanceTypeInstance8;
                                                                }
                                                                taskBaseInstance.CustomDetails = consistencyCheckTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue5 = customDetailsValue["name"];
                                                                if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance5 = ((string)nameValue5);
                                                                    automationRunbookTaskDetailsInstance.Name = nameInstance5;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue = customDetailsValue["cloudServiceName"];
                                                                if (cloudServiceNameValue != null && cloudServiceNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance = ((string)cloudServiceNameValue);
                                                                    automationRunbookTaskDetailsInstance.CloudServiceName = cloudServiceNameInstance;
                                                                }
                                                                
                                                                JToken subscriptionIdValue = customDetailsValue["subscriptionId"];
                                                                if (subscriptionIdValue != null && subscriptionIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance = ((string)subscriptionIdValue);
                                                                    automationRunbookTaskDetailsInstance.SubscriptionId = subscriptionIdInstance;
                                                                }
                                                                
                                                                JToken accountNameValue = customDetailsValue["accountName"];
                                                                if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance = ((string)accountNameValue);
                                                                    automationRunbookTaskDetailsInstance.AccountName = accountNameInstance;
                                                                }
                                                                
                                                                JToken runbookIdValue = customDetailsValue["runbookId"];
                                                                if (runbookIdValue != null && runbookIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance = ((string)runbookIdValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookId = runbookIdInstance;
                                                                }
                                                                
                                                                JToken runbookNameValue = customDetailsValue["runbookName"];
                                                                if (runbookNameValue != null && runbookNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance = ((string)runbookNameValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookName = runbookNameInstance;
                                                                }
                                                                
                                                                JToken jobIdValue4 = customDetailsValue["jobId"];
                                                                if (jobIdValue4 != null && jobIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance4 = ((string)jobIdValue4);
                                                                    automationRunbookTaskDetailsInstance.JobId = jobIdInstance4;
                                                                }
                                                                
                                                                JToken jobOutputValue = customDetailsValue["jobOutput"];
                                                                if (jobOutputValue != null && jobOutputValue.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance = ((string)jobOutputValue);
                                                                    automationRunbookTaskDetailsInstance.JobOutput = jobOutputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue2 = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue2 != null && isPrimarySideScriptValue2.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance2 = ((bool)isPrimarySideScriptValue2);
                                                                    automationRunbookTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue9 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue9 != null && instanceTypeValue9.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance9 = ((string)instanceTypeValue9);
                                                                    automationRunbookTaskDetailsInstance.Type = instanceTypeInstance9;
                                                                }
                                                                taskBaseInstance.CustomDetails = automationRunbookTaskDetailsInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = inlineWorkflowTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanGroupTaskDetails")
                                            {
                                                RecoveryPlanGroupTaskDetails recoveryPlanGroupTaskDetailsInstance = new RecoveryPlanGroupTaskDetails();
                                                
                                                JToken nameValue6 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                {
                                                    string nameInstance6 = ((string)nameValue6);
                                                    recoveryPlanGroupTaskDetailsInstance.Name = nameInstance6;
                                                }
                                                
                                                JToken groupIdValue = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue != null && groupIdValue.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance = ((string)groupIdValue);
                                                    recoveryPlanGroupTaskDetailsInstance.GroupId = groupIdInstance;
                                                }
                                                
                                                JToken rpGroupTypeValue = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue != null && rpGroupTypeValue.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance = ((string)rpGroupTypeValue);
                                                    recoveryPlanGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance;
                                                }
                                                
                                                JToken instanceTypeValue10 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue10 != null && instanceTypeValue10.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance10 = ((string)instanceTypeValue10);
                                                    recoveryPlanGroupTaskDetailsInstance.Type = instanceTypeInstance10;
                                                }
                                                
                                                JToken childTasksArray2 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray2 != null && childTasksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue2 in ((JArray)childTasksArray2))
                                                    {
                                                        AsrTaskBase taskBaseInstance2 = new AsrTaskBase();
                                                        recoveryPlanGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance2);
                                                        
                                                        JToken taskIdValue3 = childTasksValue2["taskId"];
                                                        if (taskIdValue3 != null && taskIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance3 = ((string)taskIdValue3);
                                                            taskBaseInstance2.ID = taskIdInstance3;
                                                        }
                                                        
                                                        JToken nameValue7 = childTasksValue2["name"];
                                                        if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance7 = ((string)nameValue7);
                                                            taskBaseInstance2.Name = nameInstance7;
                                                        }
                                                        
                                                        JToken startTimeValue3 = childTasksValue2["startTime"];
                                                        if (startTimeValue3 != null && startTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance3 = ((DateTime)startTimeValue3);
                                                            taskBaseInstance2.StartTime = startTimeInstance3;
                                                        }
                                                        
                                                        JToken endTimeValue3 = childTasksValue2["endTime"];
                                                        if (endTimeValue3 != null && endTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance3 = ((DateTime)endTimeValue3);
                                                            taskBaseInstance2.EndTime = endTimeInstance3;
                                                        }
                                                        
                                                        JToken allowedActionsArray3 = childTasksValue2["allowedActions"];
                                                        if (allowedActionsArray3 != null && allowedActionsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue3 in ((JArray)allowedActionsArray3))
                                                            {
                                                                taskBaseInstance2.Actions.Add(((string)allowedActionsValue3));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue3 = childTasksValue2["friendlyName"];
                                                        if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                                            taskBaseInstance2.TaskFriendlyName = friendlyNameInstance3;
                                                        }
                                                        
                                                        JToken stateValue3 = childTasksValue2["state"];
                                                        if (stateValue3 != null && stateValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance3 = ((string)stateValue3);
                                                            taskBaseInstance2.State = stateInstance3;
                                                        }
                                                        
                                                        JToken stateDescriptionValue3 = childTasksValue2["stateDescription"];
                                                        if (stateDescriptionValue3 != null && stateDescriptionValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance3 = ((string)stateDescriptionValue3);
                                                            taskBaseInstance2.StateDescription = stateDescriptionInstance3;
                                                        }
                                                        
                                                        JToken taskTypeValue2 = childTasksValue2["taskType"];
                                                        if (taskTypeValue2 != null && taskTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance2 = ((string)taskTypeValue2);
                                                            taskBaseInstance2.TaskType = taskTypeInstance2;
                                                        }
                                                        
                                                        JToken errorsArray2 = childTasksValue2["errors"];
                                                        if (errorsArray2 != null && errorsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue2 in ((JArray)errorsArray2))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance2 = new JobErrorDetails();
                                                                taskBaseInstance2.Errors.Add(jobErrorDetailsInstance2);
                                                                
                                                                JToken serviceErrorDetailsValue2 = errorsValue2["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue2 != null && serviceErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance2 = new ServiceError();
                                                                    jobErrorDetailsInstance2.ServiceErrorDetails = serviceErrorDetailsInstance2;
                                                                    
                                                                    JToken codeValue2 = serviceErrorDetailsValue2["code"];
                                                                    if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance2 = ((string)codeValue2);
                                                                        serviceErrorDetailsInstance2.Code = codeInstance2;
                                                                    }
                                                                    
                                                                    JToken messageValue2 = serviceErrorDetailsValue2["message"];
                                                                    if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance2 = ((string)messageValue2);
                                                                        serviceErrorDetailsInstance2.Message = messageInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue3 = serviceErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue3 != null && possibleCausesValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance3 = ((string)possibleCausesValue3);
                                                                        serviceErrorDetailsInstance2.PossibleCauses = possibleCausesInstance3;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue3 = serviceErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue3 != null && recommendedActionValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance3 = ((string)recommendedActionValue3);
                                                                        serviceErrorDetailsInstance2.RecommendedAction = recommendedActionInstance3;
                                                                    }
                                                                    
                                                                    JToken activityIdValue3 = serviceErrorDetailsValue2["activityId"];
                                                                    if (activityIdValue3 != null && activityIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance3 = ((string)activityIdValue3);
                                                                        serviceErrorDetailsInstance2.ActivityId = activityIdInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue2 = errorsValue2["providerErrorDetails"];
                                                                if (providerErrorDetailsValue2 != null && providerErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance2 = new ProviderError();
                                                                    jobErrorDetailsInstance2.ProviderErrorDetails = providerErrorDetailsInstance2;
                                                                    
                                                                    JToken errorCodeValue2 = providerErrorDetailsValue2["errorCode"];
                                                                    if (errorCodeValue2 != null && errorCodeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance2 = ((int)errorCodeValue2);
                                                                        providerErrorDetailsInstance2.ErrorCode = errorCodeInstance2;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue2 = providerErrorDetailsValue2["errorMessage"];
                                                                    if (errorMessageValue2 != null && errorMessageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance2 = ((string)errorMessageValue2);
                                                                        providerErrorDetailsInstance2.ErrorMessage = errorMessageInstance2;
                                                                    }
                                                                    
                                                                    JToken errorIdValue2 = providerErrorDetailsValue2["errorId"];
                                                                    if (errorIdValue2 != null && errorIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance2 = ((string)errorIdValue2);
                                                                        providerErrorDetailsInstance2.ErrorId = errorIdInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue4 = providerErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue4 != null && possibleCausesValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance4 = ((string)possibleCausesValue4);
                                                                        providerErrorDetailsInstance2.PossibleCauses = possibleCausesInstance4;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue4 = providerErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue4 != null && recommendedActionValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance4 = ((string)recommendedActionValue4);
                                                                        providerErrorDetailsInstance2.RecommendedAction = recommendedActionInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue2 = errorsValue2["errorLevel"];
                                                                if (errorLevelValue2 != null && errorLevelValue2.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance2 = ((string)errorLevelValue2);
                                                                    jobErrorDetailsInstance2.ErrorLevel = errorLevelInstance2;
                                                                }
                                                                
                                                                JToken creationTimeValue2 = errorsValue2["creationTime"];
                                                                if (creationTimeValue2 != null && creationTimeValue2.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance2 = ((DateTime)creationTimeValue2);
                                                                    jobErrorDetailsInstance2.CreationTime = creationTimeInstance2;
                                                                }
                                                                
                                                                JToken taskIdValue4 = errorsValue2["taskId"];
                                                                if (taskIdValue4 != null && taskIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance4 = ((string)taskIdValue4);
                                                                    jobErrorDetailsInstance2.TaskId = taskIdInstance4;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue2 = childTasksValue2["customDetails"];
                                                        if (customDetailsValue2 != null && customDetailsValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName3 = ((string)customDetailsValue2["instanceType"]);
                                                            if (typeName3 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance2 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue4 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue4 != null && jobTaskValue4.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance4 = new JobEntity();
                                                                    jobTaskDetailsInstance2.JobTask = jobTaskInstance4;
                                                                    
                                                                    JToken jobIdValue5 = jobTaskValue4["jobId"];
                                                                    if (jobIdValue5 != null && jobIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance5 = ((string)jobIdValue5);
                                                                        jobTaskInstance4.JobId = jobIdInstance5;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue4 = jobTaskValue4["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue4 != null && jobFriendlyNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance4 = ((string)jobFriendlyNameValue4);
                                                                        jobTaskInstance4.JobFriendlyName = jobFriendlyNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue5 = jobTaskValue4["targetObjectId"];
                                                                    if (targetObjectIdValue5 != null && targetObjectIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance5 = ((string)targetObjectIdValue5);
                                                                        jobTaskInstance4.TargetObjectId = targetObjectIdInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue5 = jobTaskValue4["targetObjectName"];
                                                                    if (targetObjectNameValue5 != null && targetObjectNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance5 = ((string)targetObjectNameValue5);
                                                                        jobTaskInstance4.TargetObjectName = targetObjectNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue5 = jobTaskValue4["targetInstanceType"];
                                                                    if (targetInstanceTypeValue5 != null && targetInstanceTypeValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance5 = ((string)targetInstanceTypeValue5);
                                                                        jobTaskInstance4.TargetInstanceType = targetInstanceTypeInstance5;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue4 = jobTaskValue4["jobScenarioName"];
                                                                    if (jobScenarioNameValue4 != null && jobScenarioNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance4 = ((string)jobScenarioNameValue4);
                                                                        jobTaskInstance4.JobScenarioName = jobScenarioNameInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue11 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue11 != null && instanceTypeValue11.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance11 = ((string)instanceTypeValue11);
                                                                    jobTaskDetailsInstance2.Type = instanceTypeInstance11;
                                                                }
                                                                taskBaseInstance2.CustomDetails = jobTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance2 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue3 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue3 != null && skippedReasonValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance3 = ((string)skippedReasonValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReason = skippedReasonInstance3;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue3 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue3 != null && skippedReasonStringValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance3 = ((string)skippedReasonStringValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance3;
                                                                }
                                                                
                                                                JToken jobTaskValue5 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue5 != null && jobTaskValue5.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance5 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance2.JobTask = jobTaskInstance5;
                                                                    
                                                                    JToken jobIdValue6 = jobTaskValue5["jobId"];
                                                                    if (jobIdValue6 != null && jobIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance6 = ((string)jobIdValue6);
                                                                        jobTaskInstance5.JobId = jobIdInstance6;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue5 = jobTaskValue5["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue5 != null && jobFriendlyNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance5 = ((string)jobFriendlyNameValue5);
                                                                        jobTaskInstance5.JobFriendlyName = jobFriendlyNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue6 = jobTaskValue5["targetObjectId"];
                                                                    if (targetObjectIdValue6 != null && targetObjectIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance6 = ((string)targetObjectIdValue6);
                                                                        jobTaskInstance5.TargetObjectId = targetObjectIdInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue6 = jobTaskValue5["targetObjectName"];
                                                                    if (targetObjectNameValue6 != null && targetObjectNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance6 = ((string)targetObjectNameValue6);
                                                                        jobTaskInstance5.TargetObjectName = targetObjectNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue6 = jobTaskValue5["targetInstanceType"];
                                                                    if (targetInstanceTypeValue6 != null && targetInstanceTypeValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance6 = ((string)targetInstanceTypeValue6);
                                                                        jobTaskInstance5.TargetInstanceType = targetInstanceTypeInstance6;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue5 = jobTaskValue5["jobScenarioName"];
                                                                    if (jobScenarioNameValue5 != null && jobScenarioNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance5 = ((string)jobScenarioNameValue5);
                                                                        jobTaskInstance5.JobScenarioName = jobScenarioNameInstance5;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue12 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue12 != null && instanceTypeValue12.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance12 = ((string)instanceTypeValue12);
                                                                    virtualMachineTaskDetailsInstance2.Type = instanceTypeInstance12;
                                                                }
                                                                taskBaseInstance2.CustomDetails = virtualMachineTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance2 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue4 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue4 != null && skippedReasonValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance4 = ((string)skippedReasonValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReason = skippedReasonInstance4;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue4 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue4 != null && skippedReasonStringValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance4 = ((string)skippedReasonStringValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance4;
                                                                }
                                                                
                                                                JToken jobTaskValue6 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue6 != null && jobTaskValue6.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance6 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance2.JobTask = jobTaskInstance6;
                                                                    
                                                                    JToken jobIdValue7 = jobTaskValue6["jobId"];
                                                                    if (jobIdValue7 != null && jobIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance7 = ((string)jobIdValue7);
                                                                        jobTaskInstance6.JobId = jobIdInstance7;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue6 = jobTaskValue6["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue6 != null && jobFriendlyNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance6 = ((string)jobFriendlyNameValue6);
                                                                        jobTaskInstance6.JobFriendlyName = jobFriendlyNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue7 = jobTaskValue6["targetObjectId"];
                                                                    if (targetObjectIdValue7 != null && targetObjectIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance7 = ((string)targetObjectIdValue7);
                                                                        jobTaskInstance6.TargetObjectId = targetObjectIdInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue7 = jobTaskValue6["targetObjectName"];
                                                                    if (targetObjectNameValue7 != null && targetObjectNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance7 = ((string)targetObjectNameValue7);
                                                                        jobTaskInstance6.TargetObjectName = targetObjectNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue7 = jobTaskValue6["targetInstanceType"];
                                                                    if (targetInstanceTypeValue7 != null && targetInstanceTypeValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance7 = ((string)targetInstanceTypeValue7);
                                                                        jobTaskInstance6.TargetInstanceType = targetInstanceTypeInstance7;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue6 = jobTaskValue6["jobScenarioName"];
                                                                    if (jobScenarioNameValue6 != null && jobScenarioNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance6 = ((string)jobScenarioNameValue6);
                                                                        jobTaskInstance6.JobScenarioName = jobScenarioNameInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue13 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue13 != null && instanceTypeValue13.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance13 = ((string)instanceTypeValue13);
                                                                    fabricReplicationGroupTaskDetailsInstance2.Type = instanceTypeInstance13;
                                                                }
                                                                taskBaseInstance2.CustomDetails = fabricReplicationGroupTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance2 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue8 = customDetailsValue2["name"];
                                                                if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance8 = ((string)nameValue8);
                                                                    manualActionTaskDetailsInstance2.Name = nameInstance8;
                                                                }
                                                                
                                                                JToken instructionsValue2 = customDetailsValue2["instructions"];
                                                                if (instructionsValue2 != null && instructionsValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance2 = ((string)instructionsValue2);
                                                                    manualActionTaskDetailsInstance2.Instructions = instructionsInstance2;
                                                                }
                                                                
                                                                JToken observationValue2 = customDetailsValue2["observation"];
                                                                if (observationValue2 != null && observationValue2.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance2 = ((string)observationValue2);
                                                                    manualActionTaskDetailsInstance2.Observation = observationInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue14 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue14 != null && instanceTypeValue14.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance14 = ((string)instanceTypeValue14);
                                                                    manualActionTaskDetailsInstance2.Type = instanceTypeInstance14;
                                                                }
                                                                taskBaseInstance2.CustomDetails = manualActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance2 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue9 = customDetailsValue2["name"];
                                                                if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance9 = ((string)nameValue9);
                                                                    scriptActionTaskDetailsInstance2.Name = nameInstance9;
                                                                }
                                                                
                                                                JToken pathValue2 = customDetailsValue2["path"];
                                                                if (pathValue2 != null && pathValue2.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance2 = ((string)pathValue2);
                                                                    scriptActionTaskDetailsInstance2.Path = pathInstance2;
                                                                }
                                                                
                                                                JToken outputValue2 = customDetailsValue2["output"];
                                                                if (outputValue2 != null && outputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance2 = ((string)outputValue2);
                                                                    scriptActionTaskDetailsInstance2.Output = outputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue3 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue3 != null && isPrimarySideScriptValue3.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance3 = ((bool)isPrimarySideScriptValue3);
                                                                    scriptActionTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue15 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue15 != null && instanceTypeValue15.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance15 = ((string)instanceTypeValue15);
                                                                    scriptActionTaskDetailsInstance2.Type = instanceTypeInstance15;
                                                                }
                                                                taskBaseInstance2.CustomDetails = scriptActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance2 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue2 = customDetailsValue2["vmId"];
                                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.VmId = vmIdInstance2;
                                                                }
                                                                
                                                                JToken nicIdValue2 = customDetailsValue2["nicId"];
                                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.NicId = nicIdInstance2;
                                                                }
                                                                
                                                                JToken nameValue10 = customDetailsValue2["name"];
                                                                if (nameValue10 != null && nameValue10.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance10 = ((string)nameValue10);
                                                                    vmNicUpdatesTaskDetailsInstance2.Name = nameInstance10;
                                                                }
                                                                
                                                                JToken instanceTypeValue16 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue16 != null && instanceTypeValue16.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance16 = ((string)instanceTypeValue16);
                                                                    vmNicUpdatesTaskDetailsInstance2.Type = instanceTypeInstance16;
                                                                }
                                                                taskBaseInstance2.CustomDetails = vmNicUpdatesTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance2 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray2 = customDetailsValue2["vmDetails"];
                                                                if (vmDetailsArray2 != null && vmDetailsArray2.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue2 in ((JArray)vmDetailsArray2))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance2 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance2.VmDetails.Add(inconsistentVmDetailsInstance2);
                                                                        
                                                                        JToken vmNameValue2 = vmDetailsValue2["vmName"];
                                                                        if (vmNameValue2 != null && vmNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance2 = ((string)vmNameValue2);
                                                                            inconsistentVmDetailsInstance2.VmName = vmNameInstance2;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue2 = vmDetailsValue2["cloudName"];
                                                                        if (cloudNameValue2 != null && cloudNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance2 = ((string)cloudNameValue2);
                                                                            inconsistentVmDetailsInstance2.CloudName = cloudNameInstance2;
                                                                        }
                                                                        
                                                                        JToken detailsArray2 = vmDetailsValue2["details"];
                                                                        if (detailsArray2 != null && detailsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue2 in ((JArray)detailsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.Details.Add(((string)detailsValue2));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray2 = vmDetailsValue2["errorIds"];
                                                                        if (errorIdsArray2 != null && errorIdsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue2 in ((JArray)errorIdsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.ErrorIds.Add(((string)errorIdsValue2));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue17 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue17 != null && instanceTypeValue17.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance17 = ((string)instanceTypeValue17);
                                                                    consistencyCheckTaskDetailsInstance2.Type = instanceTypeInstance17;
                                                                }
                                                                taskBaseInstance2.CustomDetails = consistencyCheckTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance2 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue11 = customDetailsValue2["name"];
                                                                if (nameValue11 != null && nameValue11.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance11 = ((string)nameValue11);
                                                                    automationRunbookTaskDetailsInstance2.Name = nameInstance11;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue2 = customDetailsValue2["cloudServiceName"];
                                                                if (cloudServiceNameValue2 != null && cloudServiceNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance2 = ((string)cloudServiceNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.CloudServiceName = cloudServiceNameInstance2;
                                                                }
                                                                
                                                                JToken subscriptionIdValue2 = customDetailsValue2["subscriptionId"];
                                                                if (subscriptionIdValue2 != null && subscriptionIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance2 = ((string)subscriptionIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.SubscriptionId = subscriptionIdInstance2;
                                                                }
                                                                
                                                                JToken accountNameValue2 = customDetailsValue2["accountName"];
                                                                if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance2 = ((string)accountNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.AccountName = accountNameInstance2;
                                                                }
                                                                
                                                                JToken runbookIdValue2 = customDetailsValue2["runbookId"];
                                                                if (runbookIdValue2 != null && runbookIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance2 = ((string)runbookIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookId = runbookIdInstance2;
                                                                }
                                                                
                                                                JToken runbookNameValue2 = customDetailsValue2["runbookName"];
                                                                if (runbookNameValue2 != null && runbookNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance2 = ((string)runbookNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookName = runbookNameInstance2;
                                                                }
                                                                
                                                                JToken jobIdValue8 = customDetailsValue2["jobId"];
                                                                if (jobIdValue8 != null && jobIdValue8.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance8 = ((string)jobIdValue8);
                                                                    automationRunbookTaskDetailsInstance2.JobId = jobIdInstance8;
                                                                }
                                                                
                                                                JToken jobOutputValue2 = customDetailsValue2["jobOutput"];
                                                                if (jobOutputValue2 != null && jobOutputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance2 = ((string)jobOutputValue2);
                                                                    automationRunbookTaskDetailsInstance2.JobOutput = jobOutputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue4 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue4 != null && isPrimarySideScriptValue4.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance4 = ((bool)isPrimarySideScriptValue4);
                                                                    automationRunbookTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue18 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue18 != null && instanceTypeValue18.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance18 = ((string)instanceTypeValue18);
                                                                    automationRunbookTaskDetailsInstance2.Type = instanceTypeInstance18;
                                                                }
                                                                taskBaseInstance2.CustomDetails = automationRunbookTaskDetailsInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanGroupTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanShutdownGroupTaskDetails")
                                            {
                                                RecoveryPlanShutdownGroupTaskDetails recoveryPlanShutdownGroupTaskDetailsInstance = new RecoveryPlanShutdownGroupTaskDetails();
                                                
                                                JToken nameValue12 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue12 != null && nameValue12.Type != JTokenType.Null)
                                                {
                                                    string nameInstance12 = ((string)nameValue12);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Name = nameInstance12;
                                                }
                                                
                                                JToken groupIdValue2 = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue2 != null && groupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance2 = ((string)groupIdValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.GroupId = groupIdInstance2;
                                                }
                                                
                                                JToken rpGroupTypeValue2 = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue2 != null && rpGroupTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance2 = ((string)rpGroupTypeValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance2;
                                                }
                                                
                                                JToken instanceTypeValue19 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue19 != null && instanceTypeValue19.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance19 = ((string)instanceTypeValue19);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Type = instanceTypeInstance19;
                                                }
                                                
                                                JToken childTasksArray3 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray3 != null && childTasksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue3 in ((JArray)childTasksArray3))
                                                    {
                                                        AsrTaskBase taskBaseInstance3 = new AsrTaskBase();
                                                        recoveryPlanShutdownGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance3);
                                                        
                                                        JToken taskIdValue5 = childTasksValue3["taskId"];
                                                        if (taskIdValue5 != null && taskIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance5 = ((string)taskIdValue5);
                                                            taskBaseInstance3.ID = taskIdInstance5;
                                                        }
                                                        
                                                        JToken nameValue13 = childTasksValue3["name"];
                                                        if (nameValue13 != null && nameValue13.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance13 = ((string)nameValue13);
                                                            taskBaseInstance3.Name = nameInstance13;
                                                        }
                                                        
                                                        JToken startTimeValue4 = childTasksValue3["startTime"];
                                                        if (startTimeValue4 != null && startTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance4 = ((DateTime)startTimeValue4);
                                                            taskBaseInstance3.StartTime = startTimeInstance4;
                                                        }
                                                        
                                                        JToken endTimeValue4 = childTasksValue3["endTime"];
                                                        if (endTimeValue4 != null && endTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance4 = ((DateTime)endTimeValue4);
                                                            taskBaseInstance3.EndTime = endTimeInstance4;
                                                        }
                                                        
                                                        JToken allowedActionsArray4 = childTasksValue3["allowedActions"];
                                                        if (allowedActionsArray4 != null && allowedActionsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue4 in ((JArray)allowedActionsArray4))
                                                            {
                                                                taskBaseInstance3.Actions.Add(((string)allowedActionsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue4 = childTasksValue3["friendlyName"];
                                                        if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                                            taskBaseInstance3.TaskFriendlyName = friendlyNameInstance4;
                                                        }
                                                        
                                                        JToken stateValue4 = childTasksValue3["state"];
                                                        if (stateValue4 != null && stateValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance4 = ((string)stateValue4);
                                                            taskBaseInstance3.State = stateInstance4;
                                                        }
                                                        
                                                        JToken stateDescriptionValue4 = childTasksValue3["stateDescription"];
                                                        if (stateDescriptionValue4 != null && stateDescriptionValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance4 = ((string)stateDescriptionValue4);
                                                            taskBaseInstance3.StateDescription = stateDescriptionInstance4;
                                                        }
                                                        
                                                        JToken taskTypeValue3 = childTasksValue3["taskType"];
                                                        if (taskTypeValue3 != null && taskTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance3 = ((string)taskTypeValue3);
                                                            taskBaseInstance3.TaskType = taskTypeInstance3;
                                                        }
                                                        
                                                        JToken errorsArray3 = childTasksValue3["errors"];
                                                        if (errorsArray3 != null && errorsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue3 in ((JArray)errorsArray3))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance3 = new JobErrorDetails();
                                                                taskBaseInstance3.Errors.Add(jobErrorDetailsInstance3);
                                                                
                                                                JToken serviceErrorDetailsValue3 = errorsValue3["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue3 != null && serviceErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance3 = new ServiceError();
                                                                    jobErrorDetailsInstance3.ServiceErrorDetails = serviceErrorDetailsInstance3;
                                                                    
                                                                    JToken codeValue3 = serviceErrorDetailsValue3["code"];
                                                                    if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance3 = ((string)codeValue3);
                                                                        serviceErrorDetailsInstance3.Code = codeInstance3;
                                                                    }
                                                                    
                                                                    JToken messageValue3 = serviceErrorDetailsValue3["message"];
                                                                    if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance3 = ((string)messageValue3);
                                                                        serviceErrorDetailsInstance3.Message = messageInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue5 = serviceErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue5 != null && possibleCausesValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance5 = ((string)possibleCausesValue5);
                                                                        serviceErrorDetailsInstance3.PossibleCauses = possibleCausesInstance5;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue5 = serviceErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue5 != null && recommendedActionValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance5 = ((string)recommendedActionValue5);
                                                                        serviceErrorDetailsInstance3.RecommendedAction = recommendedActionInstance5;
                                                                    }
                                                                    
                                                                    JToken activityIdValue4 = serviceErrorDetailsValue3["activityId"];
                                                                    if (activityIdValue4 != null && activityIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance4 = ((string)activityIdValue4);
                                                                        serviceErrorDetailsInstance3.ActivityId = activityIdInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue3 = errorsValue3["providerErrorDetails"];
                                                                if (providerErrorDetailsValue3 != null && providerErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance3 = new ProviderError();
                                                                    jobErrorDetailsInstance3.ProviderErrorDetails = providerErrorDetailsInstance3;
                                                                    
                                                                    JToken errorCodeValue3 = providerErrorDetailsValue3["errorCode"];
                                                                    if (errorCodeValue3 != null && errorCodeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance3 = ((int)errorCodeValue3);
                                                                        providerErrorDetailsInstance3.ErrorCode = errorCodeInstance3;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue3 = providerErrorDetailsValue3["errorMessage"];
                                                                    if (errorMessageValue3 != null && errorMessageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance3 = ((string)errorMessageValue3);
                                                                        providerErrorDetailsInstance3.ErrorMessage = errorMessageInstance3;
                                                                    }
                                                                    
                                                                    JToken errorIdValue3 = providerErrorDetailsValue3["errorId"];
                                                                    if (errorIdValue3 != null && errorIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance3 = ((string)errorIdValue3);
                                                                        providerErrorDetailsInstance3.ErrorId = errorIdInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue6 = providerErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue6 != null && possibleCausesValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance6 = ((string)possibleCausesValue6);
                                                                        providerErrorDetailsInstance3.PossibleCauses = possibleCausesInstance6;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue6 = providerErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue6 != null && recommendedActionValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance6 = ((string)recommendedActionValue6);
                                                                        providerErrorDetailsInstance3.RecommendedAction = recommendedActionInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue3 = errorsValue3["errorLevel"];
                                                                if (errorLevelValue3 != null && errorLevelValue3.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance3 = ((string)errorLevelValue3);
                                                                    jobErrorDetailsInstance3.ErrorLevel = errorLevelInstance3;
                                                                }
                                                                
                                                                JToken creationTimeValue3 = errorsValue3["creationTime"];
                                                                if (creationTimeValue3 != null && creationTimeValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance3 = ((DateTime)creationTimeValue3);
                                                                    jobErrorDetailsInstance3.CreationTime = creationTimeInstance3;
                                                                }
                                                                
                                                                JToken taskIdValue6 = errorsValue3["taskId"];
                                                                if (taskIdValue6 != null && taskIdValue6.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance6 = ((string)taskIdValue6);
                                                                    jobErrorDetailsInstance3.TaskId = taskIdInstance6;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue3 = childTasksValue3["customDetails"];
                                                        if (customDetailsValue3 != null && customDetailsValue3.Type != JTokenType.Null)
                                                        {
                                                            string typeName4 = ((string)customDetailsValue3["instanceType"]);
                                                            if (typeName4 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance3 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue7 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue7 != null && jobTaskValue7.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance7 = new JobEntity();
                                                                    jobTaskDetailsInstance3.JobTask = jobTaskInstance7;
                                                                    
                                                                    JToken jobIdValue9 = jobTaskValue7["jobId"];
                                                                    if (jobIdValue9 != null && jobIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance9 = ((string)jobIdValue9);
                                                                        jobTaskInstance7.JobId = jobIdInstance9;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue7 = jobTaskValue7["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue7 != null && jobFriendlyNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance7 = ((string)jobFriendlyNameValue7);
                                                                        jobTaskInstance7.JobFriendlyName = jobFriendlyNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue8 = jobTaskValue7["targetObjectId"];
                                                                    if (targetObjectIdValue8 != null && targetObjectIdValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance8 = ((string)targetObjectIdValue8);
                                                                        jobTaskInstance7.TargetObjectId = targetObjectIdInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue8 = jobTaskValue7["targetObjectName"];
                                                                    if (targetObjectNameValue8 != null && targetObjectNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance8 = ((string)targetObjectNameValue8);
                                                                        jobTaskInstance7.TargetObjectName = targetObjectNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue8 = jobTaskValue7["targetInstanceType"];
                                                                    if (targetInstanceTypeValue8 != null && targetInstanceTypeValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance8 = ((string)targetInstanceTypeValue8);
                                                                        jobTaskInstance7.TargetInstanceType = targetInstanceTypeInstance8;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue7 = jobTaskValue7["jobScenarioName"];
                                                                    if (jobScenarioNameValue7 != null && jobScenarioNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance7 = ((string)jobScenarioNameValue7);
                                                                        jobTaskInstance7.JobScenarioName = jobScenarioNameInstance7;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue20 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue20 != null && instanceTypeValue20.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance20 = ((string)instanceTypeValue20);
                                                                    jobTaskDetailsInstance3.Type = instanceTypeInstance20;
                                                                }
                                                                taskBaseInstance3.CustomDetails = jobTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance3 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue5 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue5 != null && skippedReasonValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance5 = ((string)skippedReasonValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReason = skippedReasonInstance5;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue5 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue5 != null && skippedReasonStringValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance5 = ((string)skippedReasonStringValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance5;
                                                                }
                                                                
                                                                JToken jobTaskValue8 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue8 != null && jobTaskValue8.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance8 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance3.JobTask = jobTaskInstance8;
                                                                    
                                                                    JToken jobIdValue10 = jobTaskValue8["jobId"];
                                                                    if (jobIdValue10 != null && jobIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance10 = ((string)jobIdValue10);
                                                                        jobTaskInstance8.JobId = jobIdInstance10;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue8 = jobTaskValue8["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue8 != null && jobFriendlyNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance8 = ((string)jobFriendlyNameValue8);
                                                                        jobTaskInstance8.JobFriendlyName = jobFriendlyNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue9 = jobTaskValue8["targetObjectId"];
                                                                    if (targetObjectIdValue9 != null && targetObjectIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance9 = ((string)targetObjectIdValue9);
                                                                        jobTaskInstance8.TargetObjectId = targetObjectIdInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue9 = jobTaskValue8["targetObjectName"];
                                                                    if (targetObjectNameValue9 != null && targetObjectNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance9 = ((string)targetObjectNameValue9);
                                                                        jobTaskInstance8.TargetObjectName = targetObjectNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue9 = jobTaskValue8["targetInstanceType"];
                                                                    if (targetInstanceTypeValue9 != null && targetInstanceTypeValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance9 = ((string)targetInstanceTypeValue9);
                                                                        jobTaskInstance8.TargetInstanceType = targetInstanceTypeInstance9;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue8 = jobTaskValue8["jobScenarioName"];
                                                                    if (jobScenarioNameValue8 != null && jobScenarioNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance8 = ((string)jobScenarioNameValue8);
                                                                        jobTaskInstance8.JobScenarioName = jobScenarioNameInstance8;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue21 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue21 != null && instanceTypeValue21.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance21 = ((string)instanceTypeValue21);
                                                                    virtualMachineTaskDetailsInstance3.Type = instanceTypeInstance21;
                                                                }
                                                                taskBaseInstance3.CustomDetails = virtualMachineTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance3 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue6 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue6 != null && skippedReasonValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance6 = ((string)skippedReasonValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReason = skippedReasonInstance6;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue6 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue6 != null && skippedReasonStringValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance6 = ((string)skippedReasonStringValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance6;
                                                                }
                                                                
                                                                JToken jobTaskValue9 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue9 != null && jobTaskValue9.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance9 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance3.JobTask = jobTaskInstance9;
                                                                    
                                                                    JToken jobIdValue11 = jobTaskValue9["jobId"];
                                                                    if (jobIdValue11 != null && jobIdValue11.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance11 = ((string)jobIdValue11);
                                                                        jobTaskInstance9.JobId = jobIdInstance11;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue9 = jobTaskValue9["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue9 != null && jobFriendlyNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance9 = ((string)jobFriendlyNameValue9);
                                                                        jobTaskInstance9.JobFriendlyName = jobFriendlyNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue10 = jobTaskValue9["targetObjectId"];
                                                                    if (targetObjectIdValue10 != null && targetObjectIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance10 = ((string)targetObjectIdValue10);
                                                                        jobTaskInstance9.TargetObjectId = targetObjectIdInstance10;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue10 = jobTaskValue9["targetObjectName"];
                                                                    if (targetObjectNameValue10 != null && targetObjectNameValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance10 = ((string)targetObjectNameValue10);
                                                                        jobTaskInstance9.TargetObjectName = targetObjectNameInstance10;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue10 = jobTaskValue9["targetInstanceType"];
                                                                    if (targetInstanceTypeValue10 != null && targetInstanceTypeValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance10 = ((string)targetInstanceTypeValue10);
                                                                        jobTaskInstance9.TargetInstanceType = targetInstanceTypeInstance10;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue9 = jobTaskValue9["jobScenarioName"];
                                                                    if (jobScenarioNameValue9 != null && jobScenarioNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance9 = ((string)jobScenarioNameValue9);
                                                                        jobTaskInstance9.JobScenarioName = jobScenarioNameInstance9;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue22 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue22 != null && instanceTypeValue22.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance22 = ((string)instanceTypeValue22);
                                                                    fabricReplicationGroupTaskDetailsInstance3.Type = instanceTypeInstance22;
                                                                }
                                                                taskBaseInstance3.CustomDetails = fabricReplicationGroupTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance3 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue14 = customDetailsValue3["name"];
                                                                if (nameValue14 != null && nameValue14.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance14 = ((string)nameValue14);
                                                                    manualActionTaskDetailsInstance3.Name = nameInstance14;
                                                                }
                                                                
                                                                JToken instructionsValue3 = customDetailsValue3["instructions"];
                                                                if (instructionsValue3 != null && instructionsValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance3 = ((string)instructionsValue3);
                                                                    manualActionTaskDetailsInstance3.Instructions = instructionsInstance3;
                                                                }
                                                                
                                                                JToken observationValue3 = customDetailsValue3["observation"];
                                                                if (observationValue3 != null && observationValue3.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance3 = ((string)observationValue3);
                                                                    manualActionTaskDetailsInstance3.Observation = observationInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue23 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue23 != null && instanceTypeValue23.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance23 = ((string)instanceTypeValue23);
                                                                    manualActionTaskDetailsInstance3.Type = instanceTypeInstance23;
                                                                }
                                                                taskBaseInstance3.CustomDetails = manualActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance3 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue15 = customDetailsValue3["name"];
                                                                if (nameValue15 != null && nameValue15.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance15 = ((string)nameValue15);
                                                                    scriptActionTaskDetailsInstance3.Name = nameInstance15;
                                                                }
                                                                
                                                                JToken pathValue3 = customDetailsValue3["path"];
                                                                if (pathValue3 != null && pathValue3.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance3 = ((string)pathValue3);
                                                                    scriptActionTaskDetailsInstance3.Path = pathInstance3;
                                                                }
                                                                
                                                                JToken outputValue3 = customDetailsValue3["output"];
                                                                if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance3 = ((string)outputValue3);
                                                                    scriptActionTaskDetailsInstance3.Output = outputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue5 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue5 != null && isPrimarySideScriptValue5.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance5 = ((bool)isPrimarySideScriptValue5);
                                                                    scriptActionTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance5;
                                                                }
                                                                
                                                                JToken instanceTypeValue24 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue24 != null && instanceTypeValue24.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance24 = ((string)instanceTypeValue24);
                                                                    scriptActionTaskDetailsInstance3.Type = instanceTypeInstance24;
                                                                }
                                                                taskBaseInstance3.CustomDetails = scriptActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance3 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue3 = customDetailsValue3["vmId"];
                                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.VmId = vmIdInstance3;
                                                                }
                                                                
                                                                JToken nicIdValue3 = customDetailsValue3["nicId"];
                                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.NicId = nicIdInstance3;
                                                                }
                                                                
                                                                JToken nameValue16 = customDetailsValue3["name"];
                                                                if (nameValue16 != null && nameValue16.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance16 = ((string)nameValue16);
                                                                    vmNicUpdatesTaskDetailsInstance3.Name = nameInstance16;
                                                                }
                                                                
                                                                JToken instanceTypeValue25 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue25 != null && instanceTypeValue25.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance25 = ((string)instanceTypeValue25);
                                                                    vmNicUpdatesTaskDetailsInstance3.Type = instanceTypeInstance25;
                                                                }
                                                                taskBaseInstance3.CustomDetails = vmNicUpdatesTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance3 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray3 = customDetailsValue3["vmDetails"];
                                                                if (vmDetailsArray3 != null && vmDetailsArray3.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue3 in ((JArray)vmDetailsArray3))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance3 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance3.VmDetails.Add(inconsistentVmDetailsInstance3);
                                                                        
                                                                        JToken vmNameValue3 = vmDetailsValue3["vmName"];
                                                                        if (vmNameValue3 != null && vmNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance3 = ((string)vmNameValue3);
                                                                            inconsistentVmDetailsInstance3.VmName = vmNameInstance3;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue3 = vmDetailsValue3["cloudName"];
                                                                        if (cloudNameValue3 != null && cloudNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance3 = ((string)cloudNameValue3);
                                                                            inconsistentVmDetailsInstance3.CloudName = cloudNameInstance3;
                                                                        }
                                                                        
                                                                        JToken detailsArray3 = vmDetailsValue3["details"];
                                                                        if (detailsArray3 != null && detailsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue3 in ((JArray)detailsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.Details.Add(((string)detailsValue3));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray3 = vmDetailsValue3["errorIds"];
                                                                        if (errorIdsArray3 != null && errorIdsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue3 in ((JArray)errorIdsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.ErrorIds.Add(((string)errorIdsValue3));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue26 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue26 != null && instanceTypeValue26.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance26 = ((string)instanceTypeValue26);
                                                                    consistencyCheckTaskDetailsInstance3.Type = instanceTypeInstance26;
                                                                }
                                                                taskBaseInstance3.CustomDetails = consistencyCheckTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance3 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue17 = customDetailsValue3["name"];
                                                                if (nameValue17 != null && nameValue17.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance17 = ((string)nameValue17);
                                                                    automationRunbookTaskDetailsInstance3.Name = nameInstance17;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue3 = customDetailsValue3["cloudServiceName"];
                                                                if (cloudServiceNameValue3 != null && cloudServiceNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance3 = ((string)cloudServiceNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.CloudServiceName = cloudServiceNameInstance3;
                                                                }
                                                                
                                                                JToken subscriptionIdValue3 = customDetailsValue3["subscriptionId"];
                                                                if (subscriptionIdValue3 != null && subscriptionIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance3 = ((string)subscriptionIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.SubscriptionId = subscriptionIdInstance3;
                                                                }
                                                                
                                                                JToken accountNameValue3 = customDetailsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken runbookIdValue3 = customDetailsValue3["runbookId"];
                                                                if (runbookIdValue3 != null && runbookIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance3 = ((string)runbookIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookId = runbookIdInstance3;
                                                                }
                                                                
                                                                JToken runbookNameValue3 = customDetailsValue3["runbookName"];
                                                                if (runbookNameValue3 != null && runbookNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance3 = ((string)runbookNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookName = runbookNameInstance3;
                                                                }
                                                                
                                                                JToken jobIdValue12 = customDetailsValue3["jobId"];
                                                                if (jobIdValue12 != null && jobIdValue12.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance12 = ((string)jobIdValue12);
                                                                    automationRunbookTaskDetailsInstance3.JobId = jobIdInstance12;
                                                                }
                                                                
                                                                JToken jobOutputValue3 = customDetailsValue3["jobOutput"];
                                                                if (jobOutputValue3 != null && jobOutputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance3 = ((string)jobOutputValue3);
                                                                    automationRunbookTaskDetailsInstance3.JobOutput = jobOutputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue6 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue6 != null && isPrimarySideScriptValue6.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance6 = ((bool)isPrimarySideScriptValue6);
                                                                    automationRunbookTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance6;
                                                                }
                                                                
                                                                JToken instanceTypeValue27 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue27 != null && instanceTypeValue27.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance27 = ((string)instanceTypeValue27);
                                                                    automationRunbookTaskDetailsInstance3.Type = instanceTypeInstance27;
                                                                }
                                                                taskBaseInstance3.CustomDetails = automationRunbookTaskDetailsInstance3;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanShutdownGroupTaskDetailsInstance;
                                            }
                                        }
                                        
                                        JToken taskIdValue7 = tasksValue["taskId"];
                                        if (taskIdValue7 != null && taskIdValue7.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance7 = ((string)taskIdValue7);
                                            taskInstance.ID = taskIdInstance7;
                                        }
                                        
                                        JToken nameValue18 = tasksValue["name"];
                                        if (nameValue18 != null && nameValue18.Type != JTokenType.Null)
                                        {
                                            string nameInstance18 = ((string)nameValue18);
                                            taskInstance.Name = nameInstance18;
                                        }
                                        
                                        JToken startTimeValue5 = tasksValue["startTime"];
                                        if (startTimeValue5 != null && startTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime startTimeInstance5 = ((DateTime)startTimeValue5);
                                            taskInstance.StartTime = startTimeInstance5;
                                        }
                                        
                                        JToken endTimeValue5 = tasksValue["endTime"];
                                        if (endTimeValue5 != null && endTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime endTimeInstance5 = ((DateTime)endTimeValue5);
                                            taskInstance.EndTime = endTimeInstance5;
                                        }
                                        
                                        JToken allowedActionsArray5 = tasksValue["allowedActions"];
                                        if (allowedActionsArray5 != null && allowedActionsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedActionsValue5 in ((JArray)allowedActionsArray5))
                                            {
                                                taskInstance.Actions.Add(((string)allowedActionsValue5));
                                            }
                                        }
                                        
                                        JToken friendlyNameValue5 = tasksValue["friendlyName"];
                                        if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                            taskInstance.TaskFriendlyName = friendlyNameInstance5;
                                        }
                                        
                                        JToken stateValue5 = tasksValue["state"];
                                        if (stateValue5 != null && stateValue5.Type != JTokenType.Null)
                                        {
                                            string stateInstance5 = ((string)stateValue5);
                                            taskInstance.State = stateInstance5;
                                        }
                                        
                                        JToken stateDescriptionValue5 = tasksValue["stateDescription"];
                                        if (stateDescriptionValue5 != null && stateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string stateDescriptionInstance5 = ((string)stateDescriptionValue5);
                                            taskInstance.StateDescription = stateDescriptionInstance5;
                                        }
                                        
                                        JToken taskTypeValue4 = tasksValue["taskType"];
                                        if (taskTypeValue4 != null && taskTypeValue4.Type != JTokenType.Null)
                                        {
                                            string taskTypeInstance4 = ((string)taskTypeValue4);
                                            taskInstance.TaskType = taskTypeInstance4;
                                        }
                                        
                                        JToken errorsArray4 = tasksValue["errors"];
                                        if (errorsArray4 != null && errorsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken errorsValue4 in ((JArray)errorsArray4))
                                            {
                                                JobErrorDetails jobErrorDetailsInstance4 = new JobErrorDetails();
                                                taskInstance.Errors.Add(jobErrorDetailsInstance4);
                                                
                                                JToken serviceErrorDetailsValue4 = errorsValue4["serviceErrorDetails"];
                                                if (serviceErrorDetailsValue4 != null && serviceErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ServiceError serviceErrorDetailsInstance4 = new ServiceError();
                                                    jobErrorDetailsInstance4.ServiceErrorDetails = serviceErrorDetailsInstance4;
                                                    
                                                    JToken codeValue4 = serviceErrorDetailsValue4["code"];
                                                    if (codeValue4 != null && codeValue4.Type != JTokenType.Null)
                                                    {
                                                        string codeInstance4 = ((string)codeValue4);
                                                        serviceErrorDetailsInstance4.Code = codeInstance4;
                                                    }
                                                    
                                                    JToken messageValue4 = serviceErrorDetailsValue4["message"];
                                                    if (messageValue4 != null && messageValue4.Type != JTokenType.Null)
                                                    {
                                                        string messageInstance4 = ((string)messageValue4);
                                                        serviceErrorDetailsInstance4.Message = messageInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue7 = serviceErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue7 != null && possibleCausesValue7.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance7 = ((string)possibleCausesValue7);
                                                        serviceErrorDetailsInstance4.PossibleCauses = possibleCausesInstance7;
                                                    }
                                                    
                                                    JToken recommendedActionValue7 = serviceErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue7 != null && recommendedActionValue7.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance7 = ((string)recommendedActionValue7);
                                                        serviceErrorDetailsInstance4.RecommendedAction = recommendedActionInstance7;
                                                    }
                                                    
                                                    JToken activityIdValue5 = serviceErrorDetailsValue4["activityId"];
                                                    if (activityIdValue5 != null && activityIdValue5.Type != JTokenType.Null)
                                                    {
                                                        string activityIdInstance5 = ((string)activityIdValue5);
                                                        serviceErrorDetailsInstance4.ActivityId = activityIdInstance5;
                                                    }
                                                }
                                                
                                                JToken providerErrorDetailsValue4 = errorsValue4["providerErrorDetails"];
                                                if (providerErrorDetailsValue4 != null && providerErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ProviderError providerErrorDetailsInstance4 = new ProviderError();
                                                    jobErrorDetailsInstance4.ProviderErrorDetails = providerErrorDetailsInstance4;
                                                    
                                                    JToken errorCodeValue4 = providerErrorDetailsValue4["errorCode"];
                                                    if (errorCodeValue4 != null && errorCodeValue4.Type != JTokenType.Null)
                                                    {
                                                        int errorCodeInstance4 = ((int)errorCodeValue4);
                                                        providerErrorDetailsInstance4.ErrorCode = errorCodeInstance4;
                                                    }
                                                    
                                                    JToken errorMessageValue4 = providerErrorDetailsValue4["errorMessage"];
                                                    if (errorMessageValue4 != null && errorMessageValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorMessageInstance4 = ((string)errorMessageValue4);
                                                        providerErrorDetailsInstance4.ErrorMessage = errorMessageInstance4;
                                                    }
                                                    
                                                    JToken errorIdValue4 = providerErrorDetailsValue4["errorId"];
                                                    if (errorIdValue4 != null && errorIdValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorIdInstance4 = ((string)errorIdValue4);
                                                        providerErrorDetailsInstance4.ErrorId = errorIdInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue8 = providerErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue8 != null && possibleCausesValue8.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance8 = ((string)possibleCausesValue8);
                                                        providerErrorDetailsInstance4.PossibleCauses = possibleCausesInstance8;
                                                    }
                                                    
                                                    JToken recommendedActionValue8 = providerErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue8 != null && recommendedActionValue8.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance8 = ((string)recommendedActionValue8);
                                                        providerErrorDetailsInstance4.RecommendedAction = recommendedActionInstance8;
                                                    }
                                                }
                                                
                                                JToken errorLevelValue4 = errorsValue4["errorLevel"];
                                                if (errorLevelValue4 != null && errorLevelValue4.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance4 = ((string)errorLevelValue4);
                                                    jobErrorDetailsInstance4.ErrorLevel = errorLevelInstance4;
                                                }
                                                
                                                JToken creationTimeValue4 = errorsValue4["creationTime"];
                                                if (creationTimeValue4 != null && creationTimeValue4.Type != JTokenType.Null)
                                                {
                                                    DateTime creationTimeInstance4 = ((DateTime)creationTimeValue4);
                                                    jobErrorDetailsInstance4.CreationTime = creationTimeInstance4;
                                                }
                                                
                                                JToken taskIdValue8 = errorsValue4["taskId"];
                                                if (taskIdValue8 != null && taskIdValue8.Type != JTokenType.Null)
                                                {
                                                    string taskIdInstance8 = ((string)taskIdValue8);
                                                    jobErrorDetailsInstance4.TaskId = taskIdInstance8;
                                                }
                                            }
                                        }
                                        
                                        JToken customDetailsValue4 = tasksValue["customDetails"];
                                        if (customDetailsValue4 != null && customDetailsValue4.Type != JTokenType.Null)
                                        {
                                            string typeName5 = ((string)customDetailsValue4["instanceType"]);
                                            if (typeName5 == "JobTaskDetails")
                                            {
                                                JobTaskDetails jobTaskDetailsInstance4 = new JobTaskDetails();
                                                
                                                JToken jobTaskValue10 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue10 != null && jobTaskValue10.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance10 = new JobEntity();
                                                    jobTaskDetailsInstance4.JobTask = jobTaskInstance10;
                                                    
                                                    JToken jobIdValue13 = jobTaskValue10["jobId"];
                                                    if (jobIdValue13 != null && jobIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance13 = ((string)jobIdValue13);
                                                        jobTaskInstance10.JobId = jobIdInstance13;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue10 = jobTaskValue10["jobFriendlyName"];
                                                    if (jobFriendlyNameValue10 != null && jobFriendlyNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance10 = ((string)jobFriendlyNameValue10);
                                                        jobTaskInstance10.JobFriendlyName = jobFriendlyNameInstance10;
                                                    }
                                                    
                                                    JToken targetObjectIdValue11 = jobTaskValue10["targetObjectId"];
                                                    if (targetObjectIdValue11 != null && targetObjectIdValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance11 = ((string)targetObjectIdValue11);
                                                        jobTaskInstance10.TargetObjectId = targetObjectIdInstance11;
                                                    }
                                                    
                                                    JToken targetObjectNameValue11 = jobTaskValue10["targetObjectName"];
                                                    if (targetObjectNameValue11 != null && targetObjectNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance11 = ((string)targetObjectNameValue11);
                                                        jobTaskInstance10.TargetObjectName = targetObjectNameInstance11;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue11 = jobTaskValue10["targetInstanceType"];
                                                    if (targetInstanceTypeValue11 != null && targetInstanceTypeValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance11 = ((string)targetInstanceTypeValue11);
                                                        jobTaskInstance10.TargetInstanceType = targetInstanceTypeInstance11;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue10 = jobTaskValue10["jobScenarioName"];
                                                    if (jobScenarioNameValue10 != null && jobScenarioNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance10 = ((string)jobScenarioNameValue10);
                                                        jobTaskInstance10.JobScenarioName = jobScenarioNameInstance10;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue28 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue28 != null && instanceTypeValue28.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance28 = ((string)instanceTypeValue28);
                                                    jobTaskDetailsInstance4.Type = instanceTypeInstance28;
                                                }
                                                taskInstance.CustomDetails = jobTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VirtualMachineTaskDetails")
                                            {
                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance4 = new VirtualMachineTaskDetails();
                                                
                                                JToken skippedReasonValue7 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue7 != null && skippedReasonValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance7 = ((string)skippedReasonValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReason = skippedReasonInstance7;
                                                }
                                                
                                                JToken skippedReasonStringValue7 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue7 != null && skippedReasonStringValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance7 = ((string)skippedReasonStringValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance7;
                                                }
                                                
                                                JToken jobTaskValue11 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue11 != null && jobTaskValue11.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance11 = new JobEntity();
                                                    virtualMachineTaskDetailsInstance4.JobTask = jobTaskInstance11;
                                                    
                                                    JToken jobIdValue14 = jobTaskValue11["jobId"];
                                                    if (jobIdValue14 != null && jobIdValue14.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance14 = ((string)jobIdValue14);
                                                        jobTaskInstance11.JobId = jobIdInstance14;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue11 = jobTaskValue11["jobFriendlyName"];
                                                    if (jobFriendlyNameValue11 != null && jobFriendlyNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance11 = ((string)jobFriendlyNameValue11);
                                                        jobTaskInstance11.JobFriendlyName = jobFriendlyNameInstance11;
                                                    }
                                                    
                                                    JToken targetObjectIdValue12 = jobTaskValue11["targetObjectId"];
                                                    if (targetObjectIdValue12 != null && targetObjectIdValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance12 = ((string)targetObjectIdValue12);
                                                        jobTaskInstance11.TargetObjectId = targetObjectIdInstance12;
                                                    }
                                                    
                                                    JToken targetObjectNameValue12 = jobTaskValue11["targetObjectName"];
                                                    if (targetObjectNameValue12 != null && targetObjectNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance12 = ((string)targetObjectNameValue12);
                                                        jobTaskInstance11.TargetObjectName = targetObjectNameInstance12;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue12 = jobTaskValue11["targetInstanceType"];
                                                    if (targetInstanceTypeValue12 != null && targetInstanceTypeValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance12 = ((string)targetInstanceTypeValue12);
                                                        jobTaskInstance11.TargetInstanceType = targetInstanceTypeInstance12;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue11 = jobTaskValue11["jobScenarioName"];
                                                    if (jobScenarioNameValue11 != null && jobScenarioNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance11 = ((string)jobScenarioNameValue11);
                                                        jobTaskInstance11.JobScenarioName = jobScenarioNameInstance11;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue29 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue29 != null && instanceTypeValue29.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance29 = ((string)instanceTypeValue29);
                                                    virtualMachineTaskDetailsInstance4.Type = instanceTypeInstance29;
                                                }
                                                taskInstance.CustomDetails = virtualMachineTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "FabricReplicationGroupTaskDetails")
                                            {
                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance4 = new FabricReplicationGroupTaskDetails();
                                                
                                                JToken skippedReasonValue8 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue8 != null && skippedReasonValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance8 = ((string)skippedReasonValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReason = skippedReasonInstance8;
                                                }
                                                
                                                JToken skippedReasonStringValue8 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue8 != null && skippedReasonStringValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance8 = ((string)skippedReasonStringValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance8;
                                                }
                                                
                                                JToken jobTaskValue12 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue12 != null && jobTaskValue12.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance12 = new JobEntity();
                                                    fabricReplicationGroupTaskDetailsInstance4.JobTask = jobTaskInstance12;
                                                    
                                                    JToken jobIdValue15 = jobTaskValue12["jobId"];
                                                    if (jobIdValue15 != null && jobIdValue15.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance15 = ((string)jobIdValue15);
                                                        jobTaskInstance12.JobId = jobIdInstance15;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue12 = jobTaskValue12["jobFriendlyName"];
                                                    if (jobFriendlyNameValue12 != null && jobFriendlyNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance12 = ((string)jobFriendlyNameValue12);
                                                        jobTaskInstance12.JobFriendlyName = jobFriendlyNameInstance12;
                                                    }
                                                    
                                                    JToken targetObjectIdValue13 = jobTaskValue12["targetObjectId"];
                                                    if (targetObjectIdValue13 != null && targetObjectIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance13 = ((string)targetObjectIdValue13);
                                                        jobTaskInstance12.TargetObjectId = targetObjectIdInstance13;
                                                    }
                                                    
                                                    JToken targetObjectNameValue13 = jobTaskValue12["targetObjectName"];
                                                    if (targetObjectNameValue13 != null && targetObjectNameValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance13 = ((string)targetObjectNameValue13);
                                                        jobTaskInstance12.TargetObjectName = targetObjectNameInstance13;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue13 = jobTaskValue12["targetInstanceType"];
                                                    if (targetInstanceTypeValue13 != null && targetInstanceTypeValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance13 = ((string)targetInstanceTypeValue13);
                                                        jobTaskInstance12.TargetInstanceType = targetInstanceTypeInstance13;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue12 = jobTaskValue12["jobScenarioName"];
                                                    if (jobScenarioNameValue12 != null && jobScenarioNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance12 = ((string)jobScenarioNameValue12);
                                                        jobTaskInstance12.JobScenarioName = jobScenarioNameInstance12;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue30 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue30 != null && instanceTypeValue30.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance30 = ((string)instanceTypeValue30);
                                                    fabricReplicationGroupTaskDetailsInstance4.Type = instanceTypeInstance30;
                                                }
                                                taskInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ManualActionTaskDetails")
                                            {
                                                ManualActionTaskDetails manualActionTaskDetailsInstance4 = new ManualActionTaskDetails();
                                                
                                                JToken nameValue19 = customDetailsValue4["name"];
                                                if (nameValue19 != null && nameValue19.Type != JTokenType.Null)
                                                {
                                                    string nameInstance19 = ((string)nameValue19);
                                                    manualActionTaskDetailsInstance4.Name = nameInstance19;
                                                }
                                                
                                                JToken instructionsValue4 = customDetailsValue4["instructions"];
                                                if (instructionsValue4 != null && instructionsValue4.Type != JTokenType.Null)
                                                {
                                                    string instructionsInstance4 = ((string)instructionsValue4);
                                                    manualActionTaskDetailsInstance4.Instructions = instructionsInstance4;
                                                }
                                                
                                                JToken observationValue4 = customDetailsValue4["observation"];
                                                if (observationValue4 != null && observationValue4.Type != JTokenType.Null)
                                                {
                                                    string observationInstance4 = ((string)observationValue4);
                                                    manualActionTaskDetailsInstance4.Observation = observationInstance4;
                                                }
                                                
                                                JToken instanceTypeValue31 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue31 != null && instanceTypeValue31.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance31 = ((string)instanceTypeValue31);
                                                    manualActionTaskDetailsInstance4.Type = instanceTypeInstance31;
                                                }
                                                taskInstance.CustomDetails = manualActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ScriptActionTaskDetails")
                                            {
                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance4 = new ScriptActionTaskDetails();
                                                
                                                JToken nameValue20 = customDetailsValue4["name"];
                                                if (nameValue20 != null && nameValue20.Type != JTokenType.Null)
                                                {
                                                    string nameInstance20 = ((string)nameValue20);
                                                    scriptActionTaskDetailsInstance4.Name = nameInstance20;
                                                }
                                                
                                                JToken pathValue4 = customDetailsValue4["path"];
                                                if (pathValue4 != null && pathValue4.Type != JTokenType.Null)
                                                {
                                                    string pathInstance4 = ((string)pathValue4);
                                                    scriptActionTaskDetailsInstance4.Path = pathInstance4;
                                                }
                                                
                                                JToken outputValue4 = customDetailsValue4["output"];
                                                if (outputValue4 != null && outputValue4.Type != JTokenType.Null)
                                                {
                                                    string outputInstance4 = ((string)outputValue4);
                                                    scriptActionTaskDetailsInstance4.Output = outputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue7 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue7 != null && isPrimarySideScriptValue7.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance7 = ((bool)isPrimarySideScriptValue7);
                                                    scriptActionTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance7;
                                                }
                                                
                                                JToken instanceTypeValue32 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue32 != null && instanceTypeValue32.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance32 = ((string)instanceTypeValue32);
                                                    scriptActionTaskDetailsInstance4.Type = instanceTypeInstance32;
                                                }
                                                taskInstance.CustomDetails = scriptActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VmNicUpdatesTaskDetails")
                                            {
                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance4 = new VmNicUpdatesTaskDetails();
                                                
                                                JToken vmIdValue4 = customDetailsValue4["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken nicIdValue4 = customDetailsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken nameValue21 = customDetailsValue4["name"];
                                                if (nameValue21 != null && nameValue21.Type != JTokenType.Null)
                                                {
                                                    string nameInstance21 = ((string)nameValue21);
                                                    vmNicUpdatesTaskDetailsInstance4.Name = nameInstance21;
                                                }
                                                
                                                JToken instanceTypeValue33 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue33 != null && instanceTypeValue33.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance33 = ((string)instanceTypeValue33);
                                                    vmNicUpdatesTaskDetailsInstance4.Type = instanceTypeInstance33;
                                                }
                                                taskInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ConsistencyCheckTaskDetails")
                                            {
                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance4 = new ConsistencyCheckTaskDetails();
                                                
                                                JToken vmDetailsArray4 = customDetailsValue4["vmDetails"];
                                                if (vmDetailsArray4 != null && vmDetailsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmDetailsValue4 in ((JArray)vmDetailsArray4))
                                                    {
                                                        InconsistentVmDetails inconsistentVmDetailsInstance4 = new InconsistentVmDetails();
                                                        consistencyCheckTaskDetailsInstance4.VmDetails.Add(inconsistentVmDetailsInstance4);
                                                        
                                                        JToken vmNameValue4 = vmDetailsValue4["vmName"];
                                                        if (vmNameValue4 != null && vmNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vmNameInstance4 = ((string)vmNameValue4);
                                                            inconsistentVmDetailsInstance4.VmName = vmNameInstance4;
                                                        }
                                                        
                                                        JToken cloudNameValue4 = vmDetailsValue4["cloudName"];
                                                        if (cloudNameValue4 != null && cloudNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string cloudNameInstance4 = ((string)cloudNameValue4);
                                                            inconsistentVmDetailsInstance4.CloudName = cloudNameInstance4;
                                                        }
                                                        
                                                        JToken detailsArray4 = vmDetailsValue4["details"];
                                                        if (detailsArray4 != null && detailsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken detailsValue4 in ((JArray)detailsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.Details.Add(((string)detailsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken errorIdsArray4 = vmDetailsValue4["errorIds"];
                                                        if (errorIdsArray4 != null && errorIdsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorIdsValue4 in ((JArray)errorIdsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.ErrorIds.Add(((string)errorIdsValue4));
                                                            }
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue34 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue34 != null && instanceTypeValue34.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance34 = ((string)instanceTypeValue34);
                                                    consistencyCheckTaskDetailsInstance4.Type = instanceTypeInstance34;
                                                }
                                                taskInstance.CustomDetails = consistencyCheckTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "AutomationRunbookTaskDetails")
                                            {
                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance4 = new AutomationRunbookTaskDetails();
                                                
                                                JToken nameValue22 = customDetailsValue4["name"];
                                                if (nameValue22 != null && nameValue22.Type != JTokenType.Null)
                                                {
                                                    string nameInstance22 = ((string)nameValue22);
                                                    automationRunbookTaskDetailsInstance4.Name = nameInstance22;
                                                }
                                                
                                                JToken cloudServiceNameValue4 = customDetailsValue4["cloudServiceName"];
                                                if (cloudServiceNameValue4 != null && cloudServiceNameValue4.Type != JTokenType.Null)
                                                {
                                                    string cloudServiceNameInstance4 = ((string)cloudServiceNameValue4);
                                                    automationRunbookTaskDetailsInstance4.CloudServiceName = cloudServiceNameInstance4;
                                                }
                                                
                                                JToken subscriptionIdValue4 = customDetailsValue4["subscriptionId"];
                                                if (subscriptionIdValue4 != null && subscriptionIdValue4.Type != JTokenType.Null)
                                                {
                                                    string subscriptionIdInstance4 = ((string)subscriptionIdValue4);
                                                    automationRunbookTaskDetailsInstance4.SubscriptionId = subscriptionIdInstance4;
                                                }
                                                
                                                JToken accountNameValue4 = customDetailsValue4["accountName"];
                                                if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                {
                                                    string accountNameInstance4 = ((string)accountNameValue4);
                                                    automationRunbookTaskDetailsInstance4.AccountName = accountNameInstance4;
                                                }
                                                
                                                JToken runbookIdValue4 = customDetailsValue4["runbookId"];
                                                if (runbookIdValue4 != null && runbookIdValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookIdInstance4 = ((string)runbookIdValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookId = runbookIdInstance4;
                                                }
                                                
                                                JToken runbookNameValue4 = customDetailsValue4["runbookName"];
                                                if (runbookNameValue4 != null && runbookNameValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookNameInstance4 = ((string)runbookNameValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookName = runbookNameInstance4;
                                                }
                                                
                                                JToken jobIdValue16 = customDetailsValue4["jobId"];
                                                if (jobIdValue16 != null && jobIdValue16.Type != JTokenType.Null)
                                                {
                                                    string jobIdInstance16 = ((string)jobIdValue16);
                                                    automationRunbookTaskDetailsInstance4.JobId = jobIdInstance16;
                                                }
                                                
                                                JToken jobOutputValue4 = customDetailsValue4["jobOutput"];
                                                if (jobOutputValue4 != null && jobOutputValue4.Type != JTokenType.Null)
                                                {
                                                    string jobOutputInstance4 = ((string)jobOutputValue4);
                                                    automationRunbookTaskDetailsInstance4.JobOutput = jobOutputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue8 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue8 != null && isPrimarySideScriptValue8.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance8 = ((bool)isPrimarySideScriptValue8);
                                                    automationRunbookTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance8;
                                                }
                                                
                                                JToken instanceTypeValue35 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue35 != null && instanceTypeValue35.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance35 = ((string)instanceTypeValue35);
                                                    automationRunbookTaskDetailsInstance4.Type = instanceTypeInstance35;
                                                }
                                                taskInstance.CustomDetails = automationRunbookTaskDetailsInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken errorsArray5 = propertiesValue["errors"];
                                if (errorsArray5 != null && errorsArray5.Type != JTokenType.Null)
                                {
                                    foreach (JToken errorsValue5 in ((JArray)errorsArray5))
                                    {
                                        JobErrorDetails jobErrorDetailsInstance5 = new JobErrorDetails();
                                        propertiesInstance.Errors.Add(jobErrorDetailsInstance5);
                                        
                                        JToken serviceErrorDetailsValue5 = errorsValue5["serviceErrorDetails"];
                                        if (serviceErrorDetailsValue5 != null && serviceErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ServiceError serviceErrorDetailsInstance5 = new ServiceError();
                                            jobErrorDetailsInstance5.ServiceErrorDetails = serviceErrorDetailsInstance5;
                                            
                                            JToken codeValue5 = serviceErrorDetailsValue5["code"];
                                            if (codeValue5 != null && codeValue5.Type != JTokenType.Null)
                                            {
                                                string codeInstance5 = ((string)codeValue5);
                                                serviceErrorDetailsInstance5.Code = codeInstance5;
                                            }
                                            
                                            JToken messageValue5 = serviceErrorDetailsValue5["message"];
                                            if (messageValue5 != null && messageValue5.Type != JTokenType.Null)
                                            {
                                                string messageInstance5 = ((string)messageValue5);
                                                serviceErrorDetailsInstance5.Message = messageInstance5;
                                            }
                                            
                                            JToken possibleCausesValue9 = serviceErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue9 != null && possibleCausesValue9.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance9 = ((string)possibleCausesValue9);
                                                serviceErrorDetailsInstance5.PossibleCauses = possibleCausesInstance9;
                                            }
                                            
                                            JToken recommendedActionValue9 = serviceErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue9 != null && recommendedActionValue9.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance9 = ((string)recommendedActionValue9);
                                                serviceErrorDetailsInstance5.RecommendedAction = recommendedActionInstance9;
                                            }
                                            
                                            JToken activityIdValue6 = serviceErrorDetailsValue5["activityId"];
                                            if (activityIdValue6 != null && activityIdValue6.Type != JTokenType.Null)
                                            {
                                                string activityIdInstance6 = ((string)activityIdValue6);
                                                serviceErrorDetailsInstance5.ActivityId = activityIdInstance6;
                                            }
                                        }
                                        
                                        JToken providerErrorDetailsValue5 = errorsValue5["providerErrorDetails"];
                                        if (providerErrorDetailsValue5 != null && providerErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ProviderError providerErrorDetailsInstance5 = new ProviderError();
                                            jobErrorDetailsInstance5.ProviderErrorDetails = providerErrorDetailsInstance5;
                                            
                                            JToken errorCodeValue5 = providerErrorDetailsValue5["errorCode"];
                                            if (errorCodeValue5 != null && errorCodeValue5.Type != JTokenType.Null)
                                            {
                                                int errorCodeInstance5 = ((int)errorCodeValue5);
                                                providerErrorDetailsInstance5.ErrorCode = errorCodeInstance5;
                                            }
                                            
                                            JToken errorMessageValue5 = providerErrorDetailsValue5["errorMessage"];
                                            if (errorMessageValue5 != null && errorMessageValue5.Type != JTokenType.Null)
                                            {
                                                string errorMessageInstance5 = ((string)errorMessageValue5);
                                                providerErrorDetailsInstance5.ErrorMessage = errorMessageInstance5;
                                            }
                                            
                                            JToken errorIdValue5 = providerErrorDetailsValue5["errorId"];
                                            if (errorIdValue5 != null && errorIdValue5.Type != JTokenType.Null)
                                            {
                                                string errorIdInstance5 = ((string)errorIdValue5);
                                                providerErrorDetailsInstance5.ErrorId = errorIdInstance5;
                                            }
                                            
                                            JToken possibleCausesValue10 = providerErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue10 != null && possibleCausesValue10.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance10 = ((string)possibleCausesValue10);
                                                providerErrorDetailsInstance5.PossibleCauses = possibleCausesInstance10;
                                            }
                                            
                                            JToken recommendedActionValue10 = providerErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue10 != null && recommendedActionValue10.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance10 = ((string)recommendedActionValue10);
                                                providerErrorDetailsInstance5.RecommendedAction = recommendedActionInstance10;
                                            }
                                        }
                                        
                                        JToken errorLevelValue5 = errorsValue5["errorLevel"];
                                        if (errorLevelValue5 != null && errorLevelValue5.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance5 = ((string)errorLevelValue5);
                                            jobErrorDetailsInstance5.ErrorLevel = errorLevelInstance5;
                                        }
                                        
                                        JToken creationTimeValue5 = errorsValue5["creationTime"];
                                        if (creationTimeValue5 != null && creationTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime creationTimeInstance5 = ((DateTime)creationTimeValue5);
                                            jobErrorDetailsInstance5.CreationTime = creationTimeInstance5;
                                        }
                                        
                                        JToken taskIdValue9 = errorsValue5["taskId"];
                                        if (taskIdValue9 != null && taskIdValue9.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance9 = ((string)taskIdValue9);
                                            jobErrorDetailsInstance5.TaskId = taskIdInstance9;
                                        }
                                    }
                                }
                                
                                JToken customDetailsValue5 = propertiesValue["customDetails"];
                                if (customDetailsValue5 != null && customDetailsValue5.Type != JTokenType.Null)
                                {
                                    string typeName6 = ((string)customDetailsValue5["instanceType"]);
                                    if (typeName6 == "ExportJobDetails")
                                    {
                                        ExportJobDetails exportJobDetailsInstance = new ExportJobDetails();
                                        
                                        JToken blobUriValue = customDetailsValue5["blobUri"];
                                        if (blobUriValue != null && blobUriValue.Type != JTokenType.Null)
                                        {
                                            string blobUriInstance = ((string)blobUriValue);
                                            exportJobDetailsInstance.BlobUri = blobUriInstance;
                                        }
                                        
                                        JToken sasTokenValue = customDetailsValue5["sasToken"];
                                        if (sasTokenValue != null && sasTokenValue.Type != JTokenType.Null)
                                        {
                                            string sasTokenInstance = ((string)sasTokenValue);
                                            exportJobDetailsInstance.SasToken = sasTokenInstance;
                                        }
                                        
                                        JToken instanceTypeValue36 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue36 != null && instanceTypeValue36.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance36 = ((string)instanceTypeValue36);
                                            exportJobDetailsInstance.InstanceType = instanceTypeInstance36;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement != null && affectedObjectDetailsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in affectedObjectDetailsSequenceElement)
                                            {
                                                string affectedObjectDetailsKey = ((string)property.Name);
                                                string affectedObjectDetailsValue = ((string)property.Value);
                                                exportJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey, affectedObjectDetailsValue);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = exportJobDetailsInstance;
                                    }
                                    if (typeName6 == "AsrJobDetails")
                                    {
                                        AsrJobDetails asrJobDetailsInstance = new AsrJobDetails();
                                        
                                        JToken instanceTypeValue37 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue37 != null && instanceTypeValue37.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance37 = ((string)instanceTypeValue37);
                                            asrJobDetailsInstance.InstanceType = instanceTypeInstance37;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement2 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement2 != null && affectedObjectDetailsSequenceElement2.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property2 in affectedObjectDetailsSequenceElement2)
                                            {
                                                string affectedObjectDetailsKey2 = ((string)property2.Name);
                                                string affectedObjectDetailsValue2 = ((string)property2.Value);
                                                asrJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey2, affectedObjectDetailsValue2);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = asrJobDetailsInstance;
                                    }
                                    if (typeName6 == "TestFailoverJobDetails")
                                    {
                                        TestFailoverJobDetails testFailoverJobDetailsInstance = new TestFailoverJobDetails();
                                        
                                        JToken testFailoverStatusValue = customDetailsValue5["testFailoverStatus"];
                                        if (testFailoverStatusValue != null && testFailoverStatusValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStatusInstance = ((string)testFailoverStatusValue);
                                            testFailoverJobDetailsInstance.TestFailoverStatus = testFailoverStatusInstance;
                                        }
                                        
                                        JToken commentsValue = customDetailsValue5["comments"];
                                        if (commentsValue != null && commentsValue.Type != JTokenType.Null)
                                        {
                                            string commentsInstance = ((string)commentsValue);
                                            testFailoverJobDetailsInstance.Comments = commentsInstance;
                                        }
                                        
                                        JToken networkNameValue = customDetailsValue5["networkName"];
                                        if (networkNameValue != null && networkNameValue.Type != JTokenType.Null)
                                        {
                                            string networkNameInstance = ((string)networkNameValue);
                                            testFailoverJobDetailsInstance.NetworkName = networkNameInstance;
                                        }
                                        
                                        JToken networkFriendlyNameValue = customDetailsValue5["networkFriendlyName"];
                                        if (networkFriendlyNameValue != null && networkFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string networkFriendlyNameInstance = ((string)networkFriendlyNameValue);
                                            testFailoverJobDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance;
                                        }
                                        
                                        JToken networkTypeValue = customDetailsValue5["networkType"];
                                        if (networkTypeValue != null && networkTypeValue.Type != JTokenType.Null)
                                        {
                                            string networkTypeInstance = ((string)networkTypeValue);
                                            testFailoverJobDetailsInstance.NetworkType = networkTypeInstance;
                                        }
                                        
                                        JToken protectedItemDetailsArray = customDetailsValue5["protectedItemDetails"];
                                        if (protectedItemDetailsArray != null && protectedItemDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedItemDetailsValue in ((JArray)protectedItemDetailsArray))
                                            {
                                                TestFailoverReplicationProtectedItemDetails testFailoverReplicationProtectedItemDetailsInstance = new TestFailoverReplicationProtectedItemDetails();
                                                testFailoverJobDetailsInstance.ProtectedItemDetails.Add(testFailoverReplicationProtectedItemDetailsInstance);
                                                
                                                JToken nameValue23 = protectedItemDetailsValue["name"];
                                                if (nameValue23 != null && nameValue23.Type != JTokenType.Null)
                                                {
                                                    string nameInstance23 = ((string)nameValue23);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Name = nameInstance23;
                                                }
                                                
                                                JToken friendlyNameValue6 = protectedItemDetailsValue["friendlyName"];
                                                if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                                {
                                                    string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                                    testFailoverReplicationProtectedItemDetailsInstance.FriendlyName = friendlyNameInstance6;
                                                }
                                                
                                                JToken testVmNameValue = protectedItemDetailsValue["testVmName"];
                                                if (testVmNameValue != null && testVmNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmNameInstance = ((string)testVmNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmName = testVmNameInstance;
                                                }
                                                
                                                JToken testVmFriendlyNameValue = protectedItemDetailsValue["testVmFriendlyName"];
                                                if (testVmFriendlyNameValue != null && testVmFriendlyNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmFriendlyNameInstance = ((string)testVmFriendlyNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmFriendlyName = testVmFriendlyNameInstance;
                                                }
                                                
                                                JToken networkConnectionStatusValue = protectedItemDetailsValue["networkConnectionStatus"];
                                                if (networkConnectionStatusValue != null && networkConnectionStatusValue.Type != JTokenType.Null)
                                                {
                                                    string networkConnectionStatusInstance = ((string)networkConnectionStatusValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkConnectionStatus = networkConnectionStatusInstance;
                                                }
                                                
                                                JToken networkFriendlyNameValue2 = protectedItemDetailsValue["networkFriendlyName"];
                                                if (networkFriendlyNameValue2 != null && networkFriendlyNameValue2.Type != JTokenType.Null)
                                                {
                                                    string networkFriendlyNameInstance2 = ((string)networkFriendlyNameValue2);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance2;
                                                }
                                                
                                                JToken subnetValue = protectedItemDetailsValue["subnet"];
                                                if (subnetValue != null && subnetValue.Type != JTokenType.Null)
                                                {
                                                    string subnetInstance = ((string)subnetValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Subnet = subnetInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue38 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue38 != null && instanceTypeValue38.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance38 = ((string)instanceTypeValue38);
                                            testFailoverJobDetailsInstance.InstanceType = instanceTypeInstance38;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement3 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement3 != null && affectedObjectDetailsSequenceElement3.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property3 in affectedObjectDetailsSequenceElement3)
                                            {
                                                string affectedObjectDetailsKey3 = ((string)property3.Name);
                                                string affectedObjectDetailsValue3 = ((string)property3.Value);
                                                testFailoverJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey3, affectedObjectDetailsValue3);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = testFailoverJobDetailsInstance;
                                    }
                                }
                                
                                JToken idValue = propertiesValue["id"];
                                if (idValue != null && idValue.Type != JTokenType.Null)
                                {
                                    string idInstance = ((string)idValue);
                                    propertiesInstance.Id = idInstance;
                                }
                                
                                JToken nameValue24 = propertiesValue["name"];
                                if (nameValue24 != null && nameValue24.Type != JTokenType.Null)
                                {
                                    string nameInstance24 = ((string)nameValue24);
                                    propertiesInstance.Name = nameInstance24;
                                }
                                
                                JToken typeValue = propertiesValue["type"];
                                if (typeValue != null && typeValue.Type != JTokenType.Null)
                                {
                                    string typeInstance = ((string)typeValue);
                                    propertiesInstance.Type = typeInstance;
                                }
                                
                                JToken locationValue = propertiesValue["location"];
                                if (locationValue != null && locationValue.Type != JTokenType.Null)
                                {
                                    string locationInstance = ((string)locationValue);
                                    propertiesInstance.Location = locationInstance;
                                }
                                
                                JToken tagsSequenceElement = ((JToken)propertiesValue["tags"]);
                                if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property4 in tagsSequenceElement)
                                    {
                                        string tagsKey = ((string)property4.Name);
                                        string tagsValue = ((string)property4.Value);
                                        propertiesInstance.Tags.Add(tagsKey, tagsValue);
                                    }
                                }
                            }
                            
                            JToken idValue2 = responseDoc["id"];
                            if (idValue2 != null && idValue2.Type != JTokenType.Null)
                            {
                                string idInstance2 = ((string)idValue2);
                                jobInstance.Id = idInstance2;
                            }
                            
                            JToken nameValue25 = responseDoc["name"];
                            if (nameValue25 != null && nameValue25.Type != JTokenType.Null)
                            {
                                string nameInstance25 = ((string)nameValue25);
                                jobInstance.Name = nameInstance25;
                            }
                            
                            JToken typeValue2 = responseDoc["type"];
                            if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                            {
                                string typeInstance2 = ((string)typeValue2);
                                jobInstance.Type = typeInstance2;
                            }
                            
                            JToken locationValue2 = responseDoc["location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                jobInstance.Location = locationInstance2;
                            }
                            
                            JToken tagsSequenceElement2 = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement2 != null && tagsSequenceElement2.Type != JTokenType.Null)
                            {
                                foreach (JProperty property5 in tagsSequenceElement2)
                                {
                                    string tagsKey2 = ((string)property5.Name);
                                    string tagsValue2 = ((string)property5.Value);
                                    jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken locationValue3 = responseDoc["Location"];
                            if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                            {
                                string locationInstance3 = ((string)locationValue3);
                                result.Location = locationInstance3;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<JobOperationResponse> GetResumeStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetResumeStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken scenarioNameValue = propertiesValue["scenarioName"];
                                if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                {
                                    string scenarioNameInstance = ((string)scenarioNameValue);
                                    propertiesInstance.ScenarioName = scenarioNameInstance;
                                }
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken activityIdValue = propertiesValue["activityId"];
                                if (activityIdValue != null && activityIdValue.Type != JTokenType.Null)
                                {
                                    string activityIdInstance = ((string)activityIdValue);
                                    propertiesInstance.ActivityId = activityIdInstance;
                                }
                                
                                JToken startTimeValue = propertiesValue["startTime"];
                                if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime startTimeInstance = ((DateTime)startTimeValue);
                                    propertiesInstance.StartTime = startTimeInstance;
                                }
                                
                                JToken endTimeValue = propertiesValue["endTime"];
                                if (endTimeValue != null && endTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime endTimeInstance = ((DateTime)endTimeValue);
                                    propertiesInstance.EndTime = endTimeInstance;
                                }
                                
                                JToken targetObjectIdValue = propertiesValue["targetObjectId"];
                                if (targetObjectIdValue != null && targetObjectIdValue.Type != JTokenType.Null)
                                {
                                    string targetObjectIdInstance = ((string)targetObjectIdValue);
                                    propertiesInstance.TargetObjectId = targetObjectIdInstance;
                                }
                                
                                JToken targetInstanceTypeValue = propertiesValue["targetInstanceType"];
                                if (targetInstanceTypeValue != null && targetInstanceTypeValue.Type != JTokenType.Null)
                                {
                                    string targetInstanceTypeInstance = ((string)targetInstanceTypeValue);
                                    propertiesInstance.TargetInstanceType = targetInstanceTypeInstance;
                                }
                                
                                JToken targetObjectNameValue = propertiesValue["targetObjectName"];
                                if (targetObjectNameValue != null && targetObjectNameValue.Type != JTokenType.Null)
                                {
                                    string targetObjectNameInstance = ((string)targetObjectNameValue);
                                    propertiesInstance.TargetObjectName = targetObjectNameInstance;
                                }
                                
                                JToken allowedActionsArray = propertiesValue["allowedActions"];
                                if (allowedActionsArray != null && allowedActionsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedActionsValue in ((JArray)allowedActionsArray))
                                    {
                                        propertiesInstance.AllowedActions.Add(((string)allowedActionsValue));
                                    }
                                }
                                
                                JToken stateValue = propertiesValue["state"];
                                if (stateValue != null && stateValue.Type != JTokenType.Null)
                                {
                                    string stateInstance = ((string)stateValue);
                                    propertiesInstance.State = stateInstance;
                                }
                                
                                JToken stateDescriptionValue = propertiesValue["stateDescription"];
                                if (stateDescriptionValue != null && stateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string stateDescriptionInstance = ((string)stateDescriptionValue);
                                    propertiesInstance.StateDescription = stateDescriptionInstance;
                                }
                                
                                JToken tasksArray = propertiesValue["tasks"];
                                if (tasksArray != null && tasksArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken tasksValue in ((JArray)tasksArray))
                                    {
                                        AsrTask taskInstance = new AsrTask();
                                        propertiesInstance.Tasks.Add(taskInstance);
                                        
                                        JToken groupTaskCustomDetailsValue = tasksValue["groupTaskCustomDetails"];
                                        if (groupTaskCustomDetailsValue != null && groupTaskCustomDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)groupTaskCustomDetailsValue["instanceType"]);
                                            if (typeName == "InlineWorkflowTaskDetails")
                                            {
                                                InlineWorkflowTaskDetails inlineWorkflowTaskDetailsInstance = new InlineWorkflowTaskDetails();
                                                
                                                JToken workflowIdsArray = groupTaskCustomDetailsValue["workflowIds"];
                                                if (workflowIdsArray != null && workflowIdsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken workflowIdsValue in ((JArray)workflowIdsArray))
                                                    {
                                                        inlineWorkflowTaskDetailsInstance.WorkflowIds.Add(((string)workflowIdsValue));
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    inlineWorkflowTaskDetailsInstance.Type = instanceTypeInstance;
                                                }
                                                
                                                JToken childTasksArray = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray != null && childTasksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue in ((JArray)childTasksArray))
                                                    {
                                                        AsrTaskBase taskBaseInstance = new AsrTaskBase();
                                                        inlineWorkflowTaskDetailsInstance.ChildTasks.Add(taskBaseInstance);
                                                        
                                                        JToken taskIdValue = childTasksValue["taskId"];
                                                        if (taskIdValue != null && taskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance = ((string)taskIdValue);
                                                            taskBaseInstance.ID = taskIdInstance;
                                                        }
                                                        
                                                        JToken nameValue = childTasksValue["name"];
                                                        if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance = ((string)nameValue);
                                                            taskBaseInstance.Name = nameInstance;
                                                        }
                                                        
                                                        JToken startTimeValue2 = childTasksValue["startTime"];
                                                        if (startTimeValue2 != null && startTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance2 = ((DateTime)startTimeValue2);
                                                            taskBaseInstance.StartTime = startTimeInstance2;
                                                        }
                                                        
                                                        JToken endTimeValue2 = childTasksValue["endTime"];
                                                        if (endTimeValue2 != null && endTimeValue2.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance2 = ((DateTime)endTimeValue2);
                                                            taskBaseInstance.EndTime = endTimeInstance2;
                                                        }
                                                        
                                                        JToken allowedActionsArray2 = childTasksValue["allowedActions"];
                                                        if (allowedActionsArray2 != null && allowedActionsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue2 in ((JArray)allowedActionsArray2))
                                                            {
                                                                taskBaseInstance.Actions.Add(((string)allowedActionsValue2));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue2 = childTasksValue["friendlyName"];
                                                        if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                                            taskBaseInstance.TaskFriendlyName = friendlyNameInstance2;
                                                        }
                                                        
                                                        JToken stateValue2 = childTasksValue["state"];
                                                        if (stateValue2 != null && stateValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance2 = ((string)stateValue2);
                                                            taskBaseInstance.State = stateInstance2;
                                                        }
                                                        
                                                        JToken stateDescriptionValue2 = childTasksValue["stateDescription"];
                                                        if (stateDescriptionValue2 != null && stateDescriptionValue2.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance2 = ((string)stateDescriptionValue2);
                                                            taskBaseInstance.StateDescription = stateDescriptionInstance2;
                                                        }
                                                        
                                                        JToken taskTypeValue = childTasksValue["taskType"];
                                                        if (taskTypeValue != null && taskTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance = ((string)taskTypeValue);
                                                            taskBaseInstance.TaskType = taskTypeInstance;
                                                        }
                                                        
                                                        JToken errorsArray = childTasksValue["errors"];
                                                        if (errorsArray != null && errorsArray.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue in ((JArray)errorsArray))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance = new JobErrorDetails();
                                                                taskBaseInstance.Errors.Add(jobErrorDetailsInstance);
                                                                
                                                                JToken serviceErrorDetailsValue = errorsValue["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue != null && serviceErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance = new ServiceError();
                                                                    jobErrorDetailsInstance.ServiceErrorDetails = serviceErrorDetailsInstance;
                                                                    
                                                                    JToken codeValue = serviceErrorDetailsValue["code"];
                                                                    if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance = ((string)codeValue);
                                                                        serviceErrorDetailsInstance.Code = codeInstance;
                                                                    }
                                                                    
                                                                    JToken messageValue = serviceErrorDetailsValue["message"];
                                                                    if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance = ((string)messageValue);
                                                                        serviceErrorDetailsInstance.Message = messageInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue = serviceErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance = ((string)possibleCausesValue);
                                                                        serviceErrorDetailsInstance.PossibleCauses = possibleCausesInstance;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue = serviceErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance = ((string)recommendedActionValue);
                                                                        serviceErrorDetailsInstance.RecommendedAction = recommendedActionInstance;
                                                                    }
                                                                    
                                                                    JToken activityIdValue2 = serviceErrorDetailsValue["activityId"];
                                                                    if (activityIdValue2 != null && activityIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance2 = ((string)activityIdValue2);
                                                                        serviceErrorDetailsInstance.ActivityId = activityIdInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue = errorsValue["providerErrorDetails"];
                                                                if (providerErrorDetailsValue != null && providerErrorDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance = new ProviderError();
                                                                    jobErrorDetailsInstance.ProviderErrorDetails = providerErrorDetailsInstance;
                                                                    
                                                                    JToken errorCodeValue = providerErrorDetailsValue["errorCode"];
                                                                    if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance = ((int)errorCodeValue);
                                                                        providerErrorDetailsInstance.ErrorCode = errorCodeInstance;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue = providerErrorDetailsValue["errorMessage"];
                                                                    if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance = ((string)errorMessageValue);
                                                                        providerErrorDetailsInstance.ErrorMessage = errorMessageInstance;
                                                                    }
                                                                    
                                                                    JToken errorIdValue = providerErrorDetailsValue["errorId"];
                                                                    if (errorIdValue != null && errorIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance = ((string)errorIdValue);
                                                                        providerErrorDetailsInstance.ErrorId = errorIdInstance;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue2 = providerErrorDetailsValue["possibleCauses"];
                                                                    if (possibleCausesValue2 != null && possibleCausesValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance2 = ((string)possibleCausesValue2);
                                                                        providerErrorDetailsInstance.PossibleCauses = possibleCausesInstance2;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue2 = providerErrorDetailsValue["recommendedAction"];
                                                                    if (recommendedActionValue2 != null && recommendedActionValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance2 = ((string)recommendedActionValue2);
                                                                        providerErrorDetailsInstance.RecommendedAction = recommendedActionInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue = errorsValue["errorLevel"];
                                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                                    jobErrorDetailsInstance.ErrorLevel = errorLevelInstance;
                                                                }
                                                                
                                                                JToken creationTimeValue = errorsValue["creationTime"];
                                                                if (creationTimeValue != null && creationTimeValue.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance = ((DateTime)creationTimeValue);
                                                                    jobErrorDetailsInstance.CreationTime = creationTimeInstance;
                                                                }
                                                                
                                                                JToken taskIdValue2 = errorsValue["taskId"];
                                                                if (taskIdValue2 != null && taskIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance2 = ((string)taskIdValue2);
                                                                    jobErrorDetailsInstance.TaskId = taskIdInstance2;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue = childTasksValue["customDetails"];
                                                        if (customDetailsValue != null && customDetailsValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName2 = ((string)customDetailsValue["instanceType"]);
                                                            if (typeName2 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue = customDetailsValue["jobTask"];
                                                                if (jobTaskValue != null && jobTaskValue.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance = new JobEntity();
                                                                    jobTaskDetailsInstance.JobTask = jobTaskInstance;
                                                                    
                                                                    JToken jobIdValue = jobTaskValue["jobId"];
                                                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance = ((string)jobIdValue);
                                                                        jobTaskInstance.JobId = jobIdInstance;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue = jobTaskValue["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue != null && jobFriendlyNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance = ((string)jobFriendlyNameValue);
                                                                        jobTaskInstance.JobFriendlyName = jobFriendlyNameInstance;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue2 = jobTaskValue["targetObjectId"];
                                                                    if (targetObjectIdValue2 != null && targetObjectIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance2 = ((string)targetObjectIdValue2);
                                                                        jobTaskInstance.TargetObjectId = targetObjectIdInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue2 = jobTaskValue["targetObjectName"];
                                                                    if (targetObjectNameValue2 != null && targetObjectNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance2 = ((string)targetObjectNameValue2);
                                                                        jobTaskInstance.TargetObjectName = targetObjectNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue2 = jobTaskValue["targetInstanceType"];
                                                                    if (targetInstanceTypeValue2 != null && targetInstanceTypeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance2 = ((string)targetInstanceTypeValue2);
                                                                        jobTaskInstance.TargetInstanceType = targetInstanceTypeInstance2;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue = jobTaskValue["jobScenarioName"];
                                                                    if (jobScenarioNameValue != null && jobScenarioNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance = ((string)jobScenarioNameValue);
                                                                        jobTaskInstance.JobScenarioName = jobScenarioNameInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue2 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                                    jobTaskDetailsInstance.Type = instanceTypeInstance2;
                                                                }
                                                                taskBaseInstance.CustomDetails = jobTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue != null && skippedReasonValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance = ((string)skippedReasonValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReason = skippedReasonInstance;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue != null && skippedReasonStringValue.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance = ((string)skippedReasonStringValue);
                                                                    virtualMachineTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance;
                                                                }
                                                                
                                                                JToken jobTaskValue2 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue2 != null && jobTaskValue2.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance2 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance.JobTask = jobTaskInstance2;
                                                                    
                                                                    JToken jobIdValue2 = jobTaskValue2["jobId"];
                                                                    if (jobIdValue2 != null && jobIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance2 = ((string)jobIdValue2);
                                                                        jobTaskInstance2.JobId = jobIdInstance2;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue2 = jobTaskValue2["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue2 != null && jobFriendlyNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance2 = ((string)jobFriendlyNameValue2);
                                                                        jobTaskInstance2.JobFriendlyName = jobFriendlyNameInstance2;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue3 = jobTaskValue2["targetObjectId"];
                                                                    if (targetObjectIdValue3 != null && targetObjectIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance3 = ((string)targetObjectIdValue3);
                                                                        jobTaskInstance2.TargetObjectId = targetObjectIdInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue3 = jobTaskValue2["targetObjectName"];
                                                                    if (targetObjectNameValue3 != null && targetObjectNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance3 = ((string)targetObjectNameValue3);
                                                                        jobTaskInstance2.TargetObjectName = targetObjectNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue3 = jobTaskValue2["targetInstanceType"];
                                                                    if (targetInstanceTypeValue3 != null && targetInstanceTypeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance3 = ((string)targetInstanceTypeValue3);
                                                                        jobTaskInstance2.TargetInstanceType = targetInstanceTypeInstance3;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue2 = jobTaskValue2["jobScenarioName"];
                                                                    if (jobScenarioNameValue2 != null && jobScenarioNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance2 = ((string)jobScenarioNameValue2);
                                                                        jobTaskInstance2.JobScenarioName = jobScenarioNameInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue3 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                                    virtualMachineTaskDetailsInstance.Type = instanceTypeInstance3;
                                                                }
                                                                taskBaseInstance.CustomDetails = virtualMachineTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue2 = customDetailsValue["skippedReason"];
                                                                if (skippedReasonValue2 != null && skippedReasonValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance2 = ((string)skippedReasonValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReason = skippedReasonInstance2;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue2 = customDetailsValue["skippedReasonString"];
                                                                if (skippedReasonStringValue2 != null && skippedReasonStringValue2.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance2 = ((string)skippedReasonStringValue2);
                                                                    fabricReplicationGroupTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance2;
                                                                }
                                                                
                                                                JToken jobTaskValue3 = customDetailsValue["jobTask"];
                                                                if (jobTaskValue3 != null && jobTaskValue3.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance3 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance.JobTask = jobTaskInstance3;
                                                                    
                                                                    JToken jobIdValue3 = jobTaskValue3["jobId"];
                                                                    if (jobIdValue3 != null && jobIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance3 = ((string)jobIdValue3);
                                                                        jobTaskInstance3.JobId = jobIdInstance3;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue3 = jobTaskValue3["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue3 != null && jobFriendlyNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance3 = ((string)jobFriendlyNameValue3);
                                                                        jobTaskInstance3.JobFriendlyName = jobFriendlyNameInstance3;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue4 = jobTaskValue3["targetObjectId"];
                                                                    if (targetObjectIdValue4 != null && targetObjectIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance4 = ((string)targetObjectIdValue4);
                                                                        jobTaskInstance3.TargetObjectId = targetObjectIdInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue4 = jobTaskValue3["targetObjectName"];
                                                                    if (targetObjectNameValue4 != null && targetObjectNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance4 = ((string)targetObjectNameValue4);
                                                                        jobTaskInstance3.TargetObjectName = targetObjectNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue4 = jobTaskValue3["targetInstanceType"];
                                                                    if (targetInstanceTypeValue4 != null && targetInstanceTypeValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance4 = ((string)targetInstanceTypeValue4);
                                                                        jobTaskInstance3.TargetInstanceType = targetInstanceTypeInstance4;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue3 = jobTaskValue3["jobScenarioName"];
                                                                    if (jobScenarioNameValue3 != null && jobScenarioNameValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance3 = ((string)jobScenarioNameValue3);
                                                                        jobTaskInstance3.JobScenarioName = jobScenarioNameInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue4 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance.Type = instanceTypeInstance4;
                                                                }
                                                                taskBaseInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue2 = customDetailsValue["name"];
                                                                if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance2 = ((string)nameValue2);
                                                                    manualActionTaskDetailsInstance.Name = nameInstance2;
                                                                }
                                                                
                                                                JToken instructionsValue = customDetailsValue["instructions"];
                                                                if (instructionsValue != null && instructionsValue.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance = ((string)instructionsValue);
                                                                    manualActionTaskDetailsInstance.Instructions = instructionsInstance;
                                                                }
                                                                
                                                                JToken observationValue = customDetailsValue["observation"];
                                                                if (observationValue != null && observationValue.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance = ((string)observationValue);
                                                                    manualActionTaskDetailsInstance.Observation = observationInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue5 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                                    manualActionTaskDetailsInstance.Type = instanceTypeInstance5;
                                                                }
                                                                taskBaseInstance.CustomDetails = manualActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue3 = customDetailsValue["name"];
                                                                if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance3 = ((string)nameValue3);
                                                                    scriptActionTaskDetailsInstance.Name = nameInstance3;
                                                                }
                                                                
                                                                JToken pathValue = customDetailsValue["path"];
                                                                if (pathValue != null && pathValue.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance = ((string)pathValue);
                                                                    scriptActionTaskDetailsInstance.Path = pathInstance;
                                                                }
                                                                
                                                                JToken outputValue = customDetailsValue["output"];
                                                                if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance = ((string)outputValue);
                                                                    scriptActionTaskDetailsInstance.Output = outputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue != null && isPrimarySideScriptValue.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance = ((bool)isPrimarySideScriptValue);
                                                                    scriptActionTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance;
                                                                }
                                                                
                                                                JToken instanceTypeValue6 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                                    scriptActionTaskDetailsInstance.Type = instanceTypeInstance6;
                                                                }
                                                                taskBaseInstance.CustomDetails = scriptActionTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue = customDetailsValue["vmId"];
                                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance = ((string)vmIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.VmId = vmIdInstance;
                                                                }
                                                                
                                                                JToken nicIdValue = customDetailsValue["nicId"];
                                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance = ((string)nicIdValue);
                                                                    vmNicUpdatesTaskDetailsInstance.NicId = nicIdInstance;
                                                                }
                                                                
                                                                JToken nameValue4 = customDetailsValue["name"];
                                                                if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance4 = ((string)nameValue4);
                                                                    vmNicUpdatesTaskDetailsInstance.Name = nameInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue7 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue7 != null && instanceTypeValue7.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance7 = ((string)instanceTypeValue7);
                                                                    vmNicUpdatesTaskDetailsInstance.Type = instanceTypeInstance7;
                                                                }
                                                                taskBaseInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray = customDetailsValue["vmDetails"];
                                                                if (vmDetailsArray != null && vmDetailsArray.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue in ((JArray)vmDetailsArray))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance.VmDetails.Add(inconsistentVmDetailsInstance);
                                                                        
                                                                        JToken vmNameValue = vmDetailsValue["vmName"];
                                                                        if (vmNameValue != null && vmNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance = ((string)vmNameValue);
                                                                            inconsistentVmDetailsInstance.VmName = vmNameInstance;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue = vmDetailsValue["cloudName"];
                                                                        if (cloudNameValue != null && cloudNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance = ((string)cloudNameValue);
                                                                            inconsistentVmDetailsInstance.CloudName = cloudNameInstance;
                                                                        }
                                                                        
                                                                        JToken detailsArray = vmDetailsValue["details"];
                                                                        if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue in ((JArray)detailsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.Details.Add(((string)detailsValue));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray = vmDetailsValue["errorIds"];
                                                                        if (errorIdsArray != null && errorIdsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue in ((JArray)errorIdsArray))
                                                                            {
                                                                                inconsistentVmDetailsInstance.ErrorIds.Add(((string)errorIdsValue));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue8 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue8 != null && instanceTypeValue8.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance8 = ((string)instanceTypeValue8);
                                                                    consistencyCheckTaskDetailsInstance.Type = instanceTypeInstance8;
                                                                }
                                                                taskBaseInstance.CustomDetails = consistencyCheckTaskDetailsInstance;
                                                            }
                                                            if (typeName2 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue5 = customDetailsValue["name"];
                                                                if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance5 = ((string)nameValue5);
                                                                    automationRunbookTaskDetailsInstance.Name = nameInstance5;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue = customDetailsValue["cloudServiceName"];
                                                                if (cloudServiceNameValue != null && cloudServiceNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance = ((string)cloudServiceNameValue);
                                                                    automationRunbookTaskDetailsInstance.CloudServiceName = cloudServiceNameInstance;
                                                                }
                                                                
                                                                JToken subscriptionIdValue = customDetailsValue["subscriptionId"];
                                                                if (subscriptionIdValue != null && subscriptionIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance = ((string)subscriptionIdValue);
                                                                    automationRunbookTaskDetailsInstance.SubscriptionId = subscriptionIdInstance;
                                                                }
                                                                
                                                                JToken accountNameValue = customDetailsValue["accountName"];
                                                                if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance = ((string)accountNameValue);
                                                                    automationRunbookTaskDetailsInstance.AccountName = accountNameInstance;
                                                                }
                                                                
                                                                JToken runbookIdValue = customDetailsValue["runbookId"];
                                                                if (runbookIdValue != null && runbookIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance = ((string)runbookIdValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookId = runbookIdInstance;
                                                                }
                                                                
                                                                JToken runbookNameValue = customDetailsValue["runbookName"];
                                                                if (runbookNameValue != null && runbookNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance = ((string)runbookNameValue);
                                                                    automationRunbookTaskDetailsInstance.RunbookName = runbookNameInstance;
                                                                }
                                                                
                                                                JToken jobIdValue4 = customDetailsValue["jobId"];
                                                                if (jobIdValue4 != null && jobIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance4 = ((string)jobIdValue4);
                                                                    automationRunbookTaskDetailsInstance.JobId = jobIdInstance4;
                                                                }
                                                                
                                                                JToken jobOutputValue = customDetailsValue["jobOutput"];
                                                                if (jobOutputValue != null && jobOutputValue.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance = ((string)jobOutputValue);
                                                                    automationRunbookTaskDetailsInstance.JobOutput = jobOutputInstance;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue2 = customDetailsValue["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue2 != null && isPrimarySideScriptValue2.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance2 = ((bool)isPrimarySideScriptValue2);
                                                                    automationRunbookTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue9 = customDetailsValue["instanceType"];
                                                                if (instanceTypeValue9 != null && instanceTypeValue9.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance9 = ((string)instanceTypeValue9);
                                                                    automationRunbookTaskDetailsInstance.Type = instanceTypeInstance9;
                                                                }
                                                                taskBaseInstance.CustomDetails = automationRunbookTaskDetailsInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = inlineWorkflowTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanGroupTaskDetails")
                                            {
                                                RecoveryPlanGroupTaskDetails recoveryPlanGroupTaskDetailsInstance = new RecoveryPlanGroupTaskDetails();
                                                
                                                JToken nameValue6 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                {
                                                    string nameInstance6 = ((string)nameValue6);
                                                    recoveryPlanGroupTaskDetailsInstance.Name = nameInstance6;
                                                }
                                                
                                                JToken groupIdValue = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue != null && groupIdValue.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance = ((string)groupIdValue);
                                                    recoveryPlanGroupTaskDetailsInstance.GroupId = groupIdInstance;
                                                }
                                                
                                                JToken rpGroupTypeValue = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue != null && rpGroupTypeValue.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance = ((string)rpGroupTypeValue);
                                                    recoveryPlanGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance;
                                                }
                                                
                                                JToken instanceTypeValue10 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue10 != null && instanceTypeValue10.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance10 = ((string)instanceTypeValue10);
                                                    recoveryPlanGroupTaskDetailsInstance.Type = instanceTypeInstance10;
                                                }
                                                
                                                JToken childTasksArray2 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray2 != null && childTasksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue2 in ((JArray)childTasksArray2))
                                                    {
                                                        AsrTaskBase taskBaseInstance2 = new AsrTaskBase();
                                                        recoveryPlanGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance2);
                                                        
                                                        JToken taskIdValue3 = childTasksValue2["taskId"];
                                                        if (taskIdValue3 != null && taskIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance3 = ((string)taskIdValue3);
                                                            taskBaseInstance2.ID = taskIdInstance3;
                                                        }
                                                        
                                                        JToken nameValue7 = childTasksValue2["name"];
                                                        if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance7 = ((string)nameValue7);
                                                            taskBaseInstance2.Name = nameInstance7;
                                                        }
                                                        
                                                        JToken startTimeValue3 = childTasksValue2["startTime"];
                                                        if (startTimeValue3 != null && startTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance3 = ((DateTime)startTimeValue3);
                                                            taskBaseInstance2.StartTime = startTimeInstance3;
                                                        }
                                                        
                                                        JToken endTimeValue3 = childTasksValue2["endTime"];
                                                        if (endTimeValue3 != null && endTimeValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance3 = ((DateTime)endTimeValue3);
                                                            taskBaseInstance2.EndTime = endTimeInstance3;
                                                        }
                                                        
                                                        JToken allowedActionsArray3 = childTasksValue2["allowedActions"];
                                                        if (allowedActionsArray3 != null && allowedActionsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue3 in ((JArray)allowedActionsArray3))
                                                            {
                                                                taskBaseInstance2.Actions.Add(((string)allowedActionsValue3));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue3 = childTasksValue2["friendlyName"];
                                                        if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                                            taskBaseInstance2.TaskFriendlyName = friendlyNameInstance3;
                                                        }
                                                        
                                                        JToken stateValue3 = childTasksValue2["state"];
                                                        if (stateValue3 != null && stateValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance3 = ((string)stateValue3);
                                                            taskBaseInstance2.State = stateInstance3;
                                                        }
                                                        
                                                        JToken stateDescriptionValue3 = childTasksValue2["stateDescription"];
                                                        if (stateDescriptionValue3 != null && stateDescriptionValue3.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance3 = ((string)stateDescriptionValue3);
                                                            taskBaseInstance2.StateDescription = stateDescriptionInstance3;
                                                        }
                                                        
                                                        JToken taskTypeValue2 = childTasksValue2["taskType"];
                                                        if (taskTypeValue2 != null && taskTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance2 = ((string)taskTypeValue2);
                                                            taskBaseInstance2.TaskType = taskTypeInstance2;
                                                        }
                                                        
                                                        JToken errorsArray2 = childTasksValue2["errors"];
                                                        if (errorsArray2 != null && errorsArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue2 in ((JArray)errorsArray2))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance2 = new JobErrorDetails();
                                                                taskBaseInstance2.Errors.Add(jobErrorDetailsInstance2);
                                                                
                                                                JToken serviceErrorDetailsValue2 = errorsValue2["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue2 != null && serviceErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance2 = new ServiceError();
                                                                    jobErrorDetailsInstance2.ServiceErrorDetails = serviceErrorDetailsInstance2;
                                                                    
                                                                    JToken codeValue2 = serviceErrorDetailsValue2["code"];
                                                                    if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance2 = ((string)codeValue2);
                                                                        serviceErrorDetailsInstance2.Code = codeInstance2;
                                                                    }
                                                                    
                                                                    JToken messageValue2 = serviceErrorDetailsValue2["message"];
                                                                    if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance2 = ((string)messageValue2);
                                                                        serviceErrorDetailsInstance2.Message = messageInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue3 = serviceErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue3 != null && possibleCausesValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance3 = ((string)possibleCausesValue3);
                                                                        serviceErrorDetailsInstance2.PossibleCauses = possibleCausesInstance3;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue3 = serviceErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue3 != null && recommendedActionValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance3 = ((string)recommendedActionValue3);
                                                                        serviceErrorDetailsInstance2.RecommendedAction = recommendedActionInstance3;
                                                                    }
                                                                    
                                                                    JToken activityIdValue3 = serviceErrorDetailsValue2["activityId"];
                                                                    if (activityIdValue3 != null && activityIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance3 = ((string)activityIdValue3);
                                                                        serviceErrorDetailsInstance2.ActivityId = activityIdInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue2 = errorsValue2["providerErrorDetails"];
                                                                if (providerErrorDetailsValue2 != null && providerErrorDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance2 = new ProviderError();
                                                                    jobErrorDetailsInstance2.ProviderErrorDetails = providerErrorDetailsInstance2;
                                                                    
                                                                    JToken errorCodeValue2 = providerErrorDetailsValue2["errorCode"];
                                                                    if (errorCodeValue2 != null && errorCodeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance2 = ((int)errorCodeValue2);
                                                                        providerErrorDetailsInstance2.ErrorCode = errorCodeInstance2;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue2 = providerErrorDetailsValue2["errorMessage"];
                                                                    if (errorMessageValue2 != null && errorMessageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance2 = ((string)errorMessageValue2);
                                                                        providerErrorDetailsInstance2.ErrorMessage = errorMessageInstance2;
                                                                    }
                                                                    
                                                                    JToken errorIdValue2 = providerErrorDetailsValue2["errorId"];
                                                                    if (errorIdValue2 != null && errorIdValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance2 = ((string)errorIdValue2);
                                                                        providerErrorDetailsInstance2.ErrorId = errorIdInstance2;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue4 = providerErrorDetailsValue2["possibleCauses"];
                                                                    if (possibleCausesValue4 != null && possibleCausesValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance4 = ((string)possibleCausesValue4);
                                                                        providerErrorDetailsInstance2.PossibleCauses = possibleCausesInstance4;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue4 = providerErrorDetailsValue2["recommendedAction"];
                                                                    if (recommendedActionValue4 != null && recommendedActionValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance4 = ((string)recommendedActionValue4);
                                                                        providerErrorDetailsInstance2.RecommendedAction = recommendedActionInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue2 = errorsValue2["errorLevel"];
                                                                if (errorLevelValue2 != null && errorLevelValue2.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance2 = ((string)errorLevelValue2);
                                                                    jobErrorDetailsInstance2.ErrorLevel = errorLevelInstance2;
                                                                }
                                                                
                                                                JToken creationTimeValue2 = errorsValue2["creationTime"];
                                                                if (creationTimeValue2 != null && creationTimeValue2.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance2 = ((DateTime)creationTimeValue2);
                                                                    jobErrorDetailsInstance2.CreationTime = creationTimeInstance2;
                                                                }
                                                                
                                                                JToken taskIdValue4 = errorsValue2["taskId"];
                                                                if (taskIdValue4 != null && taskIdValue4.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance4 = ((string)taskIdValue4);
                                                                    jobErrorDetailsInstance2.TaskId = taskIdInstance4;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue2 = childTasksValue2["customDetails"];
                                                        if (customDetailsValue2 != null && customDetailsValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName3 = ((string)customDetailsValue2["instanceType"]);
                                                            if (typeName3 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance2 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue4 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue4 != null && jobTaskValue4.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance4 = new JobEntity();
                                                                    jobTaskDetailsInstance2.JobTask = jobTaskInstance4;
                                                                    
                                                                    JToken jobIdValue5 = jobTaskValue4["jobId"];
                                                                    if (jobIdValue5 != null && jobIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance5 = ((string)jobIdValue5);
                                                                        jobTaskInstance4.JobId = jobIdInstance5;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue4 = jobTaskValue4["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue4 != null && jobFriendlyNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance4 = ((string)jobFriendlyNameValue4);
                                                                        jobTaskInstance4.JobFriendlyName = jobFriendlyNameInstance4;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue5 = jobTaskValue4["targetObjectId"];
                                                                    if (targetObjectIdValue5 != null && targetObjectIdValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance5 = ((string)targetObjectIdValue5);
                                                                        jobTaskInstance4.TargetObjectId = targetObjectIdInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue5 = jobTaskValue4["targetObjectName"];
                                                                    if (targetObjectNameValue5 != null && targetObjectNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance5 = ((string)targetObjectNameValue5);
                                                                        jobTaskInstance4.TargetObjectName = targetObjectNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue5 = jobTaskValue4["targetInstanceType"];
                                                                    if (targetInstanceTypeValue5 != null && targetInstanceTypeValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance5 = ((string)targetInstanceTypeValue5);
                                                                        jobTaskInstance4.TargetInstanceType = targetInstanceTypeInstance5;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue4 = jobTaskValue4["jobScenarioName"];
                                                                    if (jobScenarioNameValue4 != null && jobScenarioNameValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance4 = ((string)jobScenarioNameValue4);
                                                                        jobTaskInstance4.JobScenarioName = jobScenarioNameInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue11 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue11 != null && instanceTypeValue11.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance11 = ((string)instanceTypeValue11);
                                                                    jobTaskDetailsInstance2.Type = instanceTypeInstance11;
                                                                }
                                                                taskBaseInstance2.CustomDetails = jobTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance2 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue3 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue3 != null && skippedReasonValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance3 = ((string)skippedReasonValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReason = skippedReasonInstance3;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue3 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue3 != null && skippedReasonStringValue3.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance3 = ((string)skippedReasonStringValue3);
                                                                    virtualMachineTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance3;
                                                                }
                                                                
                                                                JToken jobTaskValue5 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue5 != null && jobTaskValue5.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance5 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance2.JobTask = jobTaskInstance5;
                                                                    
                                                                    JToken jobIdValue6 = jobTaskValue5["jobId"];
                                                                    if (jobIdValue6 != null && jobIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance6 = ((string)jobIdValue6);
                                                                        jobTaskInstance5.JobId = jobIdInstance6;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue5 = jobTaskValue5["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue5 != null && jobFriendlyNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance5 = ((string)jobFriendlyNameValue5);
                                                                        jobTaskInstance5.JobFriendlyName = jobFriendlyNameInstance5;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue6 = jobTaskValue5["targetObjectId"];
                                                                    if (targetObjectIdValue6 != null && targetObjectIdValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance6 = ((string)targetObjectIdValue6);
                                                                        jobTaskInstance5.TargetObjectId = targetObjectIdInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue6 = jobTaskValue5["targetObjectName"];
                                                                    if (targetObjectNameValue6 != null && targetObjectNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance6 = ((string)targetObjectNameValue6);
                                                                        jobTaskInstance5.TargetObjectName = targetObjectNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue6 = jobTaskValue5["targetInstanceType"];
                                                                    if (targetInstanceTypeValue6 != null && targetInstanceTypeValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance6 = ((string)targetInstanceTypeValue6);
                                                                        jobTaskInstance5.TargetInstanceType = targetInstanceTypeInstance6;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue5 = jobTaskValue5["jobScenarioName"];
                                                                    if (jobScenarioNameValue5 != null && jobScenarioNameValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance5 = ((string)jobScenarioNameValue5);
                                                                        jobTaskInstance5.JobScenarioName = jobScenarioNameInstance5;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue12 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue12 != null && instanceTypeValue12.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance12 = ((string)instanceTypeValue12);
                                                                    virtualMachineTaskDetailsInstance2.Type = instanceTypeInstance12;
                                                                }
                                                                taskBaseInstance2.CustomDetails = virtualMachineTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance2 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue4 = customDetailsValue2["skippedReason"];
                                                                if (skippedReasonValue4 != null && skippedReasonValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance4 = ((string)skippedReasonValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReason = skippedReasonInstance4;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue4 = customDetailsValue2["skippedReasonString"];
                                                                if (skippedReasonStringValue4 != null && skippedReasonStringValue4.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance4 = ((string)skippedReasonStringValue4);
                                                                    fabricReplicationGroupTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance4;
                                                                }
                                                                
                                                                JToken jobTaskValue6 = customDetailsValue2["jobTask"];
                                                                if (jobTaskValue6 != null && jobTaskValue6.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance6 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance2.JobTask = jobTaskInstance6;
                                                                    
                                                                    JToken jobIdValue7 = jobTaskValue6["jobId"];
                                                                    if (jobIdValue7 != null && jobIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance7 = ((string)jobIdValue7);
                                                                        jobTaskInstance6.JobId = jobIdInstance7;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue6 = jobTaskValue6["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue6 != null && jobFriendlyNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance6 = ((string)jobFriendlyNameValue6);
                                                                        jobTaskInstance6.JobFriendlyName = jobFriendlyNameInstance6;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue7 = jobTaskValue6["targetObjectId"];
                                                                    if (targetObjectIdValue7 != null && targetObjectIdValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance7 = ((string)targetObjectIdValue7);
                                                                        jobTaskInstance6.TargetObjectId = targetObjectIdInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue7 = jobTaskValue6["targetObjectName"];
                                                                    if (targetObjectNameValue7 != null && targetObjectNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance7 = ((string)targetObjectNameValue7);
                                                                        jobTaskInstance6.TargetObjectName = targetObjectNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue7 = jobTaskValue6["targetInstanceType"];
                                                                    if (targetInstanceTypeValue7 != null && targetInstanceTypeValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance7 = ((string)targetInstanceTypeValue7);
                                                                        jobTaskInstance6.TargetInstanceType = targetInstanceTypeInstance7;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue6 = jobTaskValue6["jobScenarioName"];
                                                                    if (jobScenarioNameValue6 != null && jobScenarioNameValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance6 = ((string)jobScenarioNameValue6);
                                                                        jobTaskInstance6.JobScenarioName = jobScenarioNameInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue13 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue13 != null && instanceTypeValue13.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance13 = ((string)instanceTypeValue13);
                                                                    fabricReplicationGroupTaskDetailsInstance2.Type = instanceTypeInstance13;
                                                                }
                                                                taskBaseInstance2.CustomDetails = fabricReplicationGroupTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance2 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue8 = customDetailsValue2["name"];
                                                                if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance8 = ((string)nameValue8);
                                                                    manualActionTaskDetailsInstance2.Name = nameInstance8;
                                                                }
                                                                
                                                                JToken instructionsValue2 = customDetailsValue2["instructions"];
                                                                if (instructionsValue2 != null && instructionsValue2.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance2 = ((string)instructionsValue2);
                                                                    manualActionTaskDetailsInstance2.Instructions = instructionsInstance2;
                                                                }
                                                                
                                                                JToken observationValue2 = customDetailsValue2["observation"];
                                                                if (observationValue2 != null && observationValue2.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance2 = ((string)observationValue2);
                                                                    manualActionTaskDetailsInstance2.Observation = observationInstance2;
                                                                }
                                                                
                                                                JToken instanceTypeValue14 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue14 != null && instanceTypeValue14.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance14 = ((string)instanceTypeValue14);
                                                                    manualActionTaskDetailsInstance2.Type = instanceTypeInstance14;
                                                                }
                                                                taskBaseInstance2.CustomDetails = manualActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance2 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue9 = customDetailsValue2["name"];
                                                                if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance9 = ((string)nameValue9);
                                                                    scriptActionTaskDetailsInstance2.Name = nameInstance9;
                                                                }
                                                                
                                                                JToken pathValue2 = customDetailsValue2["path"];
                                                                if (pathValue2 != null && pathValue2.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance2 = ((string)pathValue2);
                                                                    scriptActionTaskDetailsInstance2.Path = pathInstance2;
                                                                }
                                                                
                                                                JToken outputValue2 = customDetailsValue2["output"];
                                                                if (outputValue2 != null && outputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance2 = ((string)outputValue2);
                                                                    scriptActionTaskDetailsInstance2.Output = outputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue3 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue3 != null && isPrimarySideScriptValue3.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance3 = ((bool)isPrimarySideScriptValue3);
                                                                    scriptActionTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue15 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue15 != null && instanceTypeValue15.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance15 = ((string)instanceTypeValue15);
                                                                    scriptActionTaskDetailsInstance2.Type = instanceTypeInstance15;
                                                                }
                                                                taskBaseInstance2.CustomDetails = scriptActionTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance2 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue2 = customDetailsValue2["vmId"];
                                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.VmId = vmIdInstance2;
                                                                }
                                                                
                                                                JToken nicIdValue2 = customDetailsValue2["nicId"];
                                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                                    vmNicUpdatesTaskDetailsInstance2.NicId = nicIdInstance2;
                                                                }
                                                                
                                                                JToken nameValue10 = customDetailsValue2["name"];
                                                                if (nameValue10 != null && nameValue10.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance10 = ((string)nameValue10);
                                                                    vmNicUpdatesTaskDetailsInstance2.Name = nameInstance10;
                                                                }
                                                                
                                                                JToken instanceTypeValue16 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue16 != null && instanceTypeValue16.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance16 = ((string)instanceTypeValue16);
                                                                    vmNicUpdatesTaskDetailsInstance2.Type = instanceTypeInstance16;
                                                                }
                                                                taskBaseInstance2.CustomDetails = vmNicUpdatesTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance2 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray2 = customDetailsValue2["vmDetails"];
                                                                if (vmDetailsArray2 != null && vmDetailsArray2.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue2 in ((JArray)vmDetailsArray2))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance2 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance2.VmDetails.Add(inconsistentVmDetailsInstance2);
                                                                        
                                                                        JToken vmNameValue2 = vmDetailsValue2["vmName"];
                                                                        if (vmNameValue2 != null && vmNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance2 = ((string)vmNameValue2);
                                                                            inconsistentVmDetailsInstance2.VmName = vmNameInstance2;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue2 = vmDetailsValue2["cloudName"];
                                                                        if (cloudNameValue2 != null && cloudNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance2 = ((string)cloudNameValue2);
                                                                            inconsistentVmDetailsInstance2.CloudName = cloudNameInstance2;
                                                                        }
                                                                        
                                                                        JToken detailsArray2 = vmDetailsValue2["details"];
                                                                        if (detailsArray2 != null && detailsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue2 in ((JArray)detailsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.Details.Add(((string)detailsValue2));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray2 = vmDetailsValue2["errorIds"];
                                                                        if (errorIdsArray2 != null && errorIdsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue2 in ((JArray)errorIdsArray2))
                                                                            {
                                                                                inconsistentVmDetailsInstance2.ErrorIds.Add(((string)errorIdsValue2));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue17 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue17 != null && instanceTypeValue17.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance17 = ((string)instanceTypeValue17);
                                                                    consistencyCheckTaskDetailsInstance2.Type = instanceTypeInstance17;
                                                                }
                                                                taskBaseInstance2.CustomDetails = consistencyCheckTaskDetailsInstance2;
                                                            }
                                                            if (typeName3 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance2 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue11 = customDetailsValue2["name"];
                                                                if (nameValue11 != null && nameValue11.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance11 = ((string)nameValue11);
                                                                    automationRunbookTaskDetailsInstance2.Name = nameInstance11;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue2 = customDetailsValue2["cloudServiceName"];
                                                                if (cloudServiceNameValue2 != null && cloudServiceNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance2 = ((string)cloudServiceNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.CloudServiceName = cloudServiceNameInstance2;
                                                                }
                                                                
                                                                JToken subscriptionIdValue2 = customDetailsValue2["subscriptionId"];
                                                                if (subscriptionIdValue2 != null && subscriptionIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance2 = ((string)subscriptionIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.SubscriptionId = subscriptionIdInstance2;
                                                                }
                                                                
                                                                JToken accountNameValue2 = customDetailsValue2["accountName"];
                                                                if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance2 = ((string)accountNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.AccountName = accountNameInstance2;
                                                                }
                                                                
                                                                JToken runbookIdValue2 = customDetailsValue2["runbookId"];
                                                                if (runbookIdValue2 != null && runbookIdValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance2 = ((string)runbookIdValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookId = runbookIdInstance2;
                                                                }
                                                                
                                                                JToken runbookNameValue2 = customDetailsValue2["runbookName"];
                                                                if (runbookNameValue2 != null && runbookNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance2 = ((string)runbookNameValue2);
                                                                    automationRunbookTaskDetailsInstance2.RunbookName = runbookNameInstance2;
                                                                }
                                                                
                                                                JToken jobIdValue8 = customDetailsValue2["jobId"];
                                                                if (jobIdValue8 != null && jobIdValue8.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance8 = ((string)jobIdValue8);
                                                                    automationRunbookTaskDetailsInstance2.JobId = jobIdInstance8;
                                                                }
                                                                
                                                                JToken jobOutputValue2 = customDetailsValue2["jobOutput"];
                                                                if (jobOutputValue2 != null && jobOutputValue2.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance2 = ((string)jobOutputValue2);
                                                                    automationRunbookTaskDetailsInstance2.JobOutput = jobOutputInstance2;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue4 = customDetailsValue2["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue4 != null && isPrimarySideScriptValue4.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance4 = ((bool)isPrimarySideScriptValue4);
                                                                    automationRunbookTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance4;
                                                                }
                                                                
                                                                JToken instanceTypeValue18 = customDetailsValue2["instanceType"];
                                                                if (instanceTypeValue18 != null && instanceTypeValue18.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance18 = ((string)instanceTypeValue18);
                                                                    automationRunbookTaskDetailsInstance2.Type = instanceTypeInstance18;
                                                                }
                                                                taskBaseInstance2.CustomDetails = automationRunbookTaskDetailsInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanGroupTaskDetailsInstance;
                                            }
                                            if (typeName == "RecoveryPlanShutdownGroupTaskDetails")
                                            {
                                                RecoveryPlanShutdownGroupTaskDetails recoveryPlanShutdownGroupTaskDetailsInstance = new RecoveryPlanShutdownGroupTaskDetails();
                                                
                                                JToken nameValue12 = groupTaskCustomDetailsValue["name"];
                                                if (nameValue12 != null && nameValue12.Type != JTokenType.Null)
                                                {
                                                    string nameInstance12 = ((string)nameValue12);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Name = nameInstance12;
                                                }
                                                
                                                JToken groupIdValue2 = groupTaskCustomDetailsValue["groupId"];
                                                if (groupIdValue2 != null && groupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string groupIdInstance2 = ((string)groupIdValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.GroupId = groupIdInstance2;
                                                }
                                                
                                                JToken rpGroupTypeValue2 = groupTaskCustomDetailsValue["rpGroupType"];
                                                if (rpGroupTypeValue2 != null && rpGroupTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string rpGroupTypeInstance2 = ((string)rpGroupTypeValue2);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance2;
                                                }
                                                
                                                JToken instanceTypeValue19 = groupTaskCustomDetailsValue["instanceType"];
                                                if (instanceTypeValue19 != null && instanceTypeValue19.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance19 = ((string)instanceTypeValue19);
                                                    recoveryPlanShutdownGroupTaskDetailsInstance.Type = instanceTypeInstance19;
                                                }
                                                
                                                JToken childTasksArray3 = groupTaskCustomDetailsValue["childTasks"];
                                                if (childTasksArray3 != null && childTasksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken childTasksValue3 in ((JArray)childTasksArray3))
                                                    {
                                                        AsrTaskBase taskBaseInstance3 = new AsrTaskBase();
                                                        recoveryPlanShutdownGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance3);
                                                        
                                                        JToken taskIdValue5 = childTasksValue3["taskId"];
                                                        if (taskIdValue5 != null && taskIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance5 = ((string)taskIdValue5);
                                                            taskBaseInstance3.ID = taskIdInstance5;
                                                        }
                                                        
                                                        JToken nameValue13 = childTasksValue3["name"];
                                                        if (nameValue13 != null && nameValue13.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance13 = ((string)nameValue13);
                                                            taskBaseInstance3.Name = nameInstance13;
                                                        }
                                                        
                                                        JToken startTimeValue4 = childTasksValue3["startTime"];
                                                        if (startTimeValue4 != null && startTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime startTimeInstance4 = ((DateTime)startTimeValue4);
                                                            taskBaseInstance3.StartTime = startTimeInstance4;
                                                        }
                                                        
                                                        JToken endTimeValue4 = childTasksValue3["endTime"];
                                                        if (endTimeValue4 != null && endTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime endTimeInstance4 = ((DateTime)endTimeValue4);
                                                            taskBaseInstance3.EndTime = endTimeInstance4;
                                                        }
                                                        
                                                        JToken allowedActionsArray4 = childTasksValue3["allowedActions"];
                                                        if (allowedActionsArray4 != null && allowedActionsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken allowedActionsValue4 in ((JArray)allowedActionsArray4))
                                                            {
                                                                taskBaseInstance3.Actions.Add(((string)allowedActionsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken friendlyNameValue4 = childTasksValue3["friendlyName"];
                                                        if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                                            taskBaseInstance3.TaskFriendlyName = friendlyNameInstance4;
                                                        }
                                                        
                                                        JToken stateValue4 = childTasksValue3["state"];
                                                        if (stateValue4 != null && stateValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateInstance4 = ((string)stateValue4);
                                                            taskBaseInstance3.State = stateInstance4;
                                                        }
                                                        
                                                        JToken stateDescriptionValue4 = childTasksValue3["stateDescription"];
                                                        if (stateDescriptionValue4 != null && stateDescriptionValue4.Type != JTokenType.Null)
                                                        {
                                                            string stateDescriptionInstance4 = ((string)stateDescriptionValue4);
                                                            taskBaseInstance3.StateDescription = stateDescriptionInstance4;
                                                        }
                                                        
                                                        JToken taskTypeValue3 = childTasksValue3["taskType"];
                                                        if (taskTypeValue3 != null && taskTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string taskTypeInstance3 = ((string)taskTypeValue3);
                                                            taskBaseInstance3.TaskType = taskTypeInstance3;
                                                        }
                                                        
                                                        JToken errorsArray3 = childTasksValue3["errors"];
                                                        if (errorsArray3 != null && errorsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorsValue3 in ((JArray)errorsArray3))
                                                            {
                                                                JobErrorDetails jobErrorDetailsInstance3 = new JobErrorDetails();
                                                                taskBaseInstance3.Errors.Add(jobErrorDetailsInstance3);
                                                                
                                                                JToken serviceErrorDetailsValue3 = errorsValue3["serviceErrorDetails"];
                                                                if (serviceErrorDetailsValue3 != null && serviceErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ServiceError serviceErrorDetailsInstance3 = new ServiceError();
                                                                    jobErrorDetailsInstance3.ServiceErrorDetails = serviceErrorDetailsInstance3;
                                                                    
                                                                    JToken codeValue3 = serviceErrorDetailsValue3["code"];
                                                                    if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance3 = ((string)codeValue3);
                                                                        serviceErrorDetailsInstance3.Code = codeInstance3;
                                                                    }
                                                                    
                                                                    JToken messageValue3 = serviceErrorDetailsValue3["message"];
                                                                    if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance3 = ((string)messageValue3);
                                                                        serviceErrorDetailsInstance3.Message = messageInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue5 = serviceErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue5 != null && possibleCausesValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance5 = ((string)possibleCausesValue5);
                                                                        serviceErrorDetailsInstance3.PossibleCauses = possibleCausesInstance5;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue5 = serviceErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue5 != null && recommendedActionValue5.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance5 = ((string)recommendedActionValue5);
                                                                        serviceErrorDetailsInstance3.RecommendedAction = recommendedActionInstance5;
                                                                    }
                                                                    
                                                                    JToken activityIdValue4 = serviceErrorDetailsValue3["activityId"];
                                                                    if (activityIdValue4 != null && activityIdValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string activityIdInstance4 = ((string)activityIdValue4);
                                                                        serviceErrorDetailsInstance3.ActivityId = activityIdInstance4;
                                                                    }
                                                                }
                                                                
                                                                JToken providerErrorDetailsValue3 = errorsValue3["providerErrorDetails"];
                                                                if (providerErrorDetailsValue3 != null && providerErrorDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    ProviderError providerErrorDetailsInstance3 = new ProviderError();
                                                                    jobErrorDetailsInstance3.ProviderErrorDetails = providerErrorDetailsInstance3;
                                                                    
                                                                    JToken errorCodeValue3 = providerErrorDetailsValue3["errorCode"];
                                                                    if (errorCodeValue3 != null && errorCodeValue3.Type != JTokenType.Null)
                                                                    {
                                                                        int errorCodeInstance3 = ((int)errorCodeValue3);
                                                                        providerErrorDetailsInstance3.ErrorCode = errorCodeInstance3;
                                                                    }
                                                                    
                                                                    JToken errorMessageValue3 = providerErrorDetailsValue3["errorMessage"];
                                                                    if (errorMessageValue3 != null && errorMessageValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorMessageInstance3 = ((string)errorMessageValue3);
                                                                        providerErrorDetailsInstance3.ErrorMessage = errorMessageInstance3;
                                                                    }
                                                                    
                                                                    JToken errorIdValue3 = providerErrorDetailsValue3["errorId"];
                                                                    if (errorIdValue3 != null && errorIdValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string errorIdInstance3 = ((string)errorIdValue3);
                                                                        providerErrorDetailsInstance3.ErrorId = errorIdInstance3;
                                                                    }
                                                                    
                                                                    JToken possibleCausesValue6 = providerErrorDetailsValue3["possibleCauses"];
                                                                    if (possibleCausesValue6 != null && possibleCausesValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string possibleCausesInstance6 = ((string)possibleCausesValue6);
                                                                        providerErrorDetailsInstance3.PossibleCauses = possibleCausesInstance6;
                                                                    }
                                                                    
                                                                    JToken recommendedActionValue6 = providerErrorDetailsValue3["recommendedAction"];
                                                                    if (recommendedActionValue6 != null && recommendedActionValue6.Type != JTokenType.Null)
                                                                    {
                                                                        string recommendedActionInstance6 = ((string)recommendedActionValue6);
                                                                        providerErrorDetailsInstance3.RecommendedAction = recommendedActionInstance6;
                                                                    }
                                                                }
                                                                
                                                                JToken errorLevelValue3 = errorsValue3["errorLevel"];
                                                                if (errorLevelValue3 != null && errorLevelValue3.Type != JTokenType.Null)
                                                                {
                                                                    string errorLevelInstance3 = ((string)errorLevelValue3);
                                                                    jobErrorDetailsInstance3.ErrorLevel = errorLevelInstance3;
                                                                }
                                                                
                                                                JToken creationTimeValue3 = errorsValue3["creationTime"];
                                                                if (creationTimeValue3 != null && creationTimeValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime creationTimeInstance3 = ((DateTime)creationTimeValue3);
                                                                    jobErrorDetailsInstance3.CreationTime = creationTimeInstance3;
                                                                }
                                                                
                                                                JToken taskIdValue6 = errorsValue3["taskId"];
                                                                if (taskIdValue6 != null && taskIdValue6.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance6 = ((string)taskIdValue6);
                                                                    jobErrorDetailsInstance3.TaskId = taskIdInstance6;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken customDetailsValue3 = childTasksValue3["customDetails"];
                                                        if (customDetailsValue3 != null && customDetailsValue3.Type != JTokenType.Null)
                                                        {
                                                            string typeName4 = ((string)customDetailsValue3["instanceType"]);
                                                            if (typeName4 == "JobTaskDetails")
                                                            {
                                                                JobTaskDetails jobTaskDetailsInstance3 = new JobTaskDetails();
                                                                
                                                                JToken jobTaskValue7 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue7 != null && jobTaskValue7.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance7 = new JobEntity();
                                                                    jobTaskDetailsInstance3.JobTask = jobTaskInstance7;
                                                                    
                                                                    JToken jobIdValue9 = jobTaskValue7["jobId"];
                                                                    if (jobIdValue9 != null && jobIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance9 = ((string)jobIdValue9);
                                                                        jobTaskInstance7.JobId = jobIdInstance9;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue7 = jobTaskValue7["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue7 != null && jobFriendlyNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance7 = ((string)jobFriendlyNameValue7);
                                                                        jobTaskInstance7.JobFriendlyName = jobFriendlyNameInstance7;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue8 = jobTaskValue7["targetObjectId"];
                                                                    if (targetObjectIdValue8 != null && targetObjectIdValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance8 = ((string)targetObjectIdValue8);
                                                                        jobTaskInstance7.TargetObjectId = targetObjectIdInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue8 = jobTaskValue7["targetObjectName"];
                                                                    if (targetObjectNameValue8 != null && targetObjectNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance8 = ((string)targetObjectNameValue8);
                                                                        jobTaskInstance7.TargetObjectName = targetObjectNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue8 = jobTaskValue7["targetInstanceType"];
                                                                    if (targetInstanceTypeValue8 != null && targetInstanceTypeValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance8 = ((string)targetInstanceTypeValue8);
                                                                        jobTaskInstance7.TargetInstanceType = targetInstanceTypeInstance8;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue7 = jobTaskValue7["jobScenarioName"];
                                                                    if (jobScenarioNameValue7 != null && jobScenarioNameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance7 = ((string)jobScenarioNameValue7);
                                                                        jobTaskInstance7.JobScenarioName = jobScenarioNameInstance7;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue20 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue20 != null && instanceTypeValue20.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance20 = ((string)instanceTypeValue20);
                                                                    jobTaskDetailsInstance3.Type = instanceTypeInstance20;
                                                                }
                                                                taskBaseInstance3.CustomDetails = jobTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VirtualMachineTaskDetails")
                                                            {
                                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance3 = new VirtualMachineTaskDetails();
                                                                
                                                                JToken skippedReasonValue5 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue5 != null && skippedReasonValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance5 = ((string)skippedReasonValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReason = skippedReasonInstance5;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue5 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue5 != null && skippedReasonStringValue5.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance5 = ((string)skippedReasonStringValue5);
                                                                    virtualMachineTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance5;
                                                                }
                                                                
                                                                JToken jobTaskValue8 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue8 != null && jobTaskValue8.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance8 = new JobEntity();
                                                                    virtualMachineTaskDetailsInstance3.JobTask = jobTaskInstance8;
                                                                    
                                                                    JToken jobIdValue10 = jobTaskValue8["jobId"];
                                                                    if (jobIdValue10 != null && jobIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance10 = ((string)jobIdValue10);
                                                                        jobTaskInstance8.JobId = jobIdInstance10;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue8 = jobTaskValue8["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue8 != null && jobFriendlyNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance8 = ((string)jobFriendlyNameValue8);
                                                                        jobTaskInstance8.JobFriendlyName = jobFriendlyNameInstance8;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue9 = jobTaskValue8["targetObjectId"];
                                                                    if (targetObjectIdValue9 != null && targetObjectIdValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance9 = ((string)targetObjectIdValue9);
                                                                        jobTaskInstance8.TargetObjectId = targetObjectIdInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue9 = jobTaskValue8["targetObjectName"];
                                                                    if (targetObjectNameValue9 != null && targetObjectNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance9 = ((string)targetObjectNameValue9);
                                                                        jobTaskInstance8.TargetObjectName = targetObjectNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue9 = jobTaskValue8["targetInstanceType"];
                                                                    if (targetInstanceTypeValue9 != null && targetInstanceTypeValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance9 = ((string)targetInstanceTypeValue9);
                                                                        jobTaskInstance8.TargetInstanceType = targetInstanceTypeInstance9;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue8 = jobTaskValue8["jobScenarioName"];
                                                                    if (jobScenarioNameValue8 != null && jobScenarioNameValue8.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance8 = ((string)jobScenarioNameValue8);
                                                                        jobTaskInstance8.JobScenarioName = jobScenarioNameInstance8;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue21 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue21 != null && instanceTypeValue21.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance21 = ((string)instanceTypeValue21);
                                                                    virtualMachineTaskDetailsInstance3.Type = instanceTypeInstance21;
                                                                }
                                                                taskBaseInstance3.CustomDetails = virtualMachineTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "FabricReplicationGroupTaskDetails")
                                                            {
                                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance3 = new FabricReplicationGroupTaskDetails();
                                                                
                                                                JToken skippedReasonValue6 = customDetailsValue3["skippedReason"];
                                                                if (skippedReasonValue6 != null && skippedReasonValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonInstance6 = ((string)skippedReasonValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReason = skippedReasonInstance6;
                                                                }
                                                                
                                                                JToken skippedReasonStringValue6 = customDetailsValue3["skippedReasonString"];
                                                                if (skippedReasonStringValue6 != null && skippedReasonStringValue6.Type != JTokenType.Null)
                                                                {
                                                                    string skippedReasonStringInstance6 = ((string)skippedReasonStringValue6);
                                                                    fabricReplicationGroupTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance6;
                                                                }
                                                                
                                                                JToken jobTaskValue9 = customDetailsValue3["jobTask"];
                                                                if (jobTaskValue9 != null && jobTaskValue9.Type != JTokenType.Null)
                                                                {
                                                                    JobEntity jobTaskInstance9 = new JobEntity();
                                                                    fabricReplicationGroupTaskDetailsInstance3.JobTask = jobTaskInstance9;
                                                                    
                                                                    JToken jobIdValue11 = jobTaskValue9["jobId"];
                                                                    if (jobIdValue11 != null && jobIdValue11.Type != JTokenType.Null)
                                                                    {
                                                                        string jobIdInstance11 = ((string)jobIdValue11);
                                                                        jobTaskInstance9.JobId = jobIdInstance11;
                                                                    }
                                                                    
                                                                    JToken jobFriendlyNameValue9 = jobTaskValue9["jobFriendlyName"];
                                                                    if (jobFriendlyNameValue9 != null && jobFriendlyNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobFriendlyNameInstance9 = ((string)jobFriendlyNameValue9);
                                                                        jobTaskInstance9.JobFriendlyName = jobFriendlyNameInstance9;
                                                                    }
                                                                    
                                                                    JToken targetObjectIdValue10 = jobTaskValue9["targetObjectId"];
                                                                    if (targetObjectIdValue10 != null && targetObjectIdValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectIdInstance10 = ((string)targetObjectIdValue10);
                                                                        jobTaskInstance9.TargetObjectId = targetObjectIdInstance10;
                                                                    }
                                                                    
                                                                    JToken targetObjectNameValue10 = jobTaskValue9["targetObjectName"];
                                                                    if (targetObjectNameValue10 != null && targetObjectNameValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetObjectNameInstance10 = ((string)targetObjectNameValue10);
                                                                        jobTaskInstance9.TargetObjectName = targetObjectNameInstance10;
                                                                    }
                                                                    
                                                                    JToken targetInstanceTypeValue10 = jobTaskValue9["targetInstanceType"];
                                                                    if (targetInstanceTypeValue10 != null && targetInstanceTypeValue10.Type != JTokenType.Null)
                                                                    {
                                                                        string targetInstanceTypeInstance10 = ((string)targetInstanceTypeValue10);
                                                                        jobTaskInstance9.TargetInstanceType = targetInstanceTypeInstance10;
                                                                    }
                                                                    
                                                                    JToken jobScenarioNameValue9 = jobTaskValue9["jobScenarioName"];
                                                                    if (jobScenarioNameValue9 != null && jobScenarioNameValue9.Type != JTokenType.Null)
                                                                    {
                                                                        string jobScenarioNameInstance9 = ((string)jobScenarioNameValue9);
                                                                        jobTaskInstance9.JobScenarioName = jobScenarioNameInstance9;
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue22 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue22 != null && instanceTypeValue22.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance22 = ((string)instanceTypeValue22);
                                                                    fabricReplicationGroupTaskDetailsInstance3.Type = instanceTypeInstance22;
                                                                }
                                                                taskBaseInstance3.CustomDetails = fabricReplicationGroupTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ManualActionTaskDetails")
                                                            {
                                                                ManualActionTaskDetails manualActionTaskDetailsInstance3 = new ManualActionTaskDetails();
                                                                
                                                                JToken nameValue14 = customDetailsValue3["name"];
                                                                if (nameValue14 != null && nameValue14.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance14 = ((string)nameValue14);
                                                                    manualActionTaskDetailsInstance3.Name = nameInstance14;
                                                                }
                                                                
                                                                JToken instructionsValue3 = customDetailsValue3["instructions"];
                                                                if (instructionsValue3 != null && instructionsValue3.Type != JTokenType.Null)
                                                                {
                                                                    string instructionsInstance3 = ((string)instructionsValue3);
                                                                    manualActionTaskDetailsInstance3.Instructions = instructionsInstance3;
                                                                }
                                                                
                                                                JToken observationValue3 = customDetailsValue3["observation"];
                                                                if (observationValue3 != null && observationValue3.Type != JTokenType.Null)
                                                                {
                                                                    string observationInstance3 = ((string)observationValue3);
                                                                    manualActionTaskDetailsInstance3.Observation = observationInstance3;
                                                                }
                                                                
                                                                JToken instanceTypeValue23 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue23 != null && instanceTypeValue23.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance23 = ((string)instanceTypeValue23);
                                                                    manualActionTaskDetailsInstance3.Type = instanceTypeInstance23;
                                                                }
                                                                taskBaseInstance3.CustomDetails = manualActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ScriptActionTaskDetails")
                                                            {
                                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance3 = new ScriptActionTaskDetails();
                                                                
                                                                JToken nameValue15 = customDetailsValue3["name"];
                                                                if (nameValue15 != null && nameValue15.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance15 = ((string)nameValue15);
                                                                    scriptActionTaskDetailsInstance3.Name = nameInstance15;
                                                                }
                                                                
                                                                JToken pathValue3 = customDetailsValue3["path"];
                                                                if (pathValue3 != null && pathValue3.Type != JTokenType.Null)
                                                                {
                                                                    string pathInstance3 = ((string)pathValue3);
                                                                    scriptActionTaskDetailsInstance3.Path = pathInstance3;
                                                                }
                                                                
                                                                JToken outputValue3 = customDetailsValue3["output"];
                                                                if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string outputInstance3 = ((string)outputValue3);
                                                                    scriptActionTaskDetailsInstance3.Output = outputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue5 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue5 != null && isPrimarySideScriptValue5.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance5 = ((bool)isPrimarySideScriptValue5);
                                                                    scriptActionTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance5;
                                                                }
                                                                
                                                                JToken instanceTypeValue24 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue24 != null && instanceTypeValue24.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance24 = ((string)instanceTypeValue24);
                                                                    scriptActionTaskDetailsInstance3.Type = instanceTypeInstance24;
                                                                }
                                                                taskBaseInstance3.CustomDetails = scriptActionTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "VmNicUpdatesTaskDetails")
                                                            {
                                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance3 = new VmNicUpdatesTaskDetails();
                                                                
                                                                JToken vmIdValue3 = customDetailsValue3["vmId"];
                                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.VmId = vmIdInstance3;
                                                                }
                                                                
                                                                JToken nicIdValue3 = customDetailsValue3["nicId"];
                                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                                    vmNicUpdatesTaskDetailsInstance3.NicId = nicIdInstance3;
                                                                }
                                                                
                                                                JToken nameValue16 = customDetailsValue3["name"];
                                                                if (nameValue16 != null && nameValue16.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance16 = ((string)nameValue16);
                                                                    vmNicUpdatesTaskDetailsInstance3.Name = nameInstance16;
                                                                }
                                                                
                                                                JToken instanceTypeValue25 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue25 != null && instanceTypeValue25.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance25 = ((string)instanceTypeValue25);
                                                                    vmNicUpdatesTaskDetailsInstance3.Type = instanceTypeInstance25;
                                                                }
                                                                taskBaseInstance3.CustomDetails = vmNicUpdatesTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "ConsistencyCheckTaskDetails")
                                                            {
                                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance3 = new ConsistencyCheckTaskDetails();
                                                                
                                                                JToken vmDetailsArray3 = customDetailsValue3["vmDetails"];
                                                                if (vmDetailsArray3 != null && vmDetailsArray3.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken vmDetailsValue3 in ((JArray)vmDetailsArray3))
                                                                    {
                                                                        InconsistentVmDetails inconsistentVmDetailsInstance3 = new InconsistentVmDetails();
                                                                        consistencyCheckTaskDetailsInstance3.VmDetails.Add(inconsistentVmDetailsInstance3);
                                                                        
                                                                        JToken vmNameValue3 = vmDetailsValue3["vmName"];
                                                                        if (vmNameValue3 != null && vmNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string vmNameInstance3 = ((string)vmNameValue3);
                                                                            inconsistentVmDetailsInstance3.VmName = vmNameInstance3;
                                                                        }
                                                                        
                                                                        JToken cloudNameValue3 = vmDetailsValue3["cloudName"];
                                                                        if (cloudNameValue3 != null && cloudNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudNameInstance3 = ((string)cloudNameValue3);
                                                                            inconsistentVmDetailsInstance3.CloudName = cloudNameInstance3;
                                                                        }
                                                                        
                                                                        JToken detailsArray3 = vmDetailsValue3["details"];
                                                                        if (detailsArray3 != null && detailsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken detailsValue3 in ((JArray)detailsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.Details.Add(((string)detailsValue3));
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorIdsArray3 = vmDetailsValue3["errorIds"];
                                                                        if (errorIdsArray3 != null && errorIdsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken errorIdsValue3 in ((JArray)errorIdsArray3))
                                                                            {
                                                                                inconsistentVmDetailsInstance3.ErrorIds.Add(((string)errorIdsValue3));
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken instanceTypeValue26 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue26 != null && instanceTypeValue26.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance26 = ((string)instanceTypeValue26);
                                                                    consistencyCheckTaskDetailsInstance3.Type = instanceTypeInstance26;
                                                                }
                                                                taskBaseInstance3.CustomDetails = consistencyCheckTaskDetailsInstance3;
                                                            }
                                                            if (typeName4 == "AutomationRunbookTaskDetails")
                                                            {
                                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance3 = new AutomationRunbookTaskDetails();
                                                                
                                                                JToken nameValue17 = customDetailsValue3["name"];
                                                                if (nameValue17 != null && nameValue17.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance17 = ((string)nameValue17);
                                                                    automationRunbookTaskDetailsInstance3.Name = nameInstance17;
                                                                }
                                                                
                                                                JToken cloudServiceNameValue3 = customDetailsValue3["cloudServiceName"];
                                                                if (cloudServiceNameValue3 != null && cloudServiceNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string cloudServiceNameInstance3 = ((string)cloudServiceNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.CloudServiceName = cloudServiceNameInstance3;
                                                                }
                                                                
                                                                JToken subscriptionIdValue3 = customDetailsValue3["subscriptionId"];
                                                                if (subscriptionIdValue3 != null && subscriptionIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string subscriptionIdInstance3 = ((string)subscriptionIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.SubscriptionId = subscriptionIdInstance3;
                                                                }
                                                                
                                                                JToken accountNameValue3 = customDetailsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken runbookIdValue3 = customDetailsValue3["runbookId"];
                                                                if (runbookIdValue3 != null && runbookIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookIdInstance3 = ((string)runbookIdValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookId = runbookIdInstance3;
                                                                }
                                                                
                                                                JToken runbookNameValue3 = customDetailsValue3["runbookName"];
                                                                if (runbookNameValue3 != null && runbookNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string runbookNameInstance3 = ((string)runbookNameValue3);
                                                                    automationRunbookTaskDetailsInstance3.RunbookName = runbookNameInstance3;
                                                                }
                                                                
                                                                JToken jobIdValue12 = customDetailsValue3["jobId"];
                                                                if (jobIdValue12 != null && jobIdValue12.Type != JTokenType.Null)
                                                                {
                                                                    string jobIdInstance12 = ((string)jobIdValue12);
                                                                    automationRunbookTaskDetailsInstance3.JobId = jobIdInstance12;
                                                                }
                                                                
                                                                JToken jobOutputValue3 = customDetailsValue3["jobOutput"];
                                                                if (jobOutputValue3 != null && jobOutputValue3.Type != JTokenType.Null)
                                                                {
                                                                    string jobOutputInstance3 = ((string)jobOutputValue3);
                                                                    automationRunbookTaskDetailsInstance3.JobOutput = jobOutputInstance3;
                                                                }
                                                                
                                                                JToken isPrimarySideScriptValue6 = customDetailsValue3["isPrimarySideScript"];
                                                                if (isPrimarySideScriptValue6 != null && isPrimarySideScriptValue6.Type != JTokenType.Null)
                                                                {
                                                                    bool isPrimarySideScriptInstance6 = ((bool)isPrimarySideScriptValue6);
                                                                    automationRunbookTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance6;
                                                                }
                                                                
                                                                JToken instanceTypeValue27 = customDetailsValue3["instanceType"];
                                                                if (instanceTypeValue27 != null && instanceTypeValue27.Type != JTokenType.Null)
                                                                {
                                                                    string instanceTypeInstance27 = ((string)instanceTypeValue27);
                                                                    automationRunbookTaskDetailsInstance3.Type = instanceTypeInstance27;
                                                                }
                                                                taskBaseInstance3.CustomDetails = automationRunbookTaskDetailsInstance3;
                                                            }
                                                        }
                                                    }
                                                }
                                                taskInstance.GroupTaskCustomDetails = recoveryPlanShutdownGroupTaskDetailsInstance;
                                            }
                                        }
                                        
                                        JToken taskIdValue7 = tasksValue["taskId"];
                                        if (taskIdValue7 != null && taskIdValue7.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance7 = ((string)taskIdValue7);
                                            taskInstance.ID = taskIdInstance7;
                                        }
                                        
                                        JToken nameValue18 = tasksValue["name"];
                                        if (nameValue18 != null && nameValue18.Type != JTokenType.Null)
                                        {
                                            string nameInstance18 = ((string)nameValue18);
                                            taskInstance.Name = nameInstance18;
                                        }
                                        
                                        JToken startTimeValue5 = tasksValue["startTime"];
                                        if (startTimeValue5 != null && startTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime startTimeInstance5 = ((DateTime)startTimeValue5);
                                            taskInstance.StartTime = startTimeInstance5;
                                        }
                                        
                                        JToken endTimeValue5 = tasksValue["endTime"];
                                        if (endTimeValue5 != null && endTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime endTimeInstance5 = ((DateTime)endTimeValue5);
                                            taskInstance.EndTime = endTimeInstance5;
                                        }
                                        
                                        JToken allowedActionsArray5 = tasksValue["allowedActions"];
                                        if (allowedActionsArray5 != null && allowedActionsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedActionsValue5 in ((JArray)allowedActionsArray5))
                                            {
                                                taskInstance.Actions.Add(((string)allowedActionsValue5));
                                            }
                                        }
                                        
                                        JToken friendlyNameValue5 = tasksValue["friendlyName"];
                                        if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                            taskInstance.TaskFriendlyName = friendlyNameInstance5;
                                        }
                                        
                                        JToken stateValue5 = tasksValue["state"];
                                        if (stateValue5 != null && stateValue5.Type != JTokenType.Null)
                                        {
                                            string stateInstance5 = ((string)stateValue5);
                                            taskInstance.State = stateInstance5;
                                        }
                                        
                                        JToken stateDescriptionValue5 = tasksValue["stateDescription"];
                                        if (stateDescriptionValue5 != null && stateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string stateDescriptionInstance5 = ((string)stateDescriptionValue5);
                                            taskInstance.StateDescription = stateDescriptionInstance5;
                                        }
                                        
                                        JToken taskTypeValue4 = tasksValue["taskType"];
                                        if (taskTypeValue4 != null && taskTypeValue4.Type != JTokenType.Null)
                                        {
                                            string taskTypeInstance4 = ((string)taskTypeValue4);
                                            taskInstance.TaskType = taskTypeInstance4;
                                        }
                                        
                                        JToken errorsArray4 = tasksValue["errors"];
                                        if (errorsArray4 != null && errorsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken errorsValue4 in ((JArray)errorsArray4))
                                            {
                                                JobErrorDetails jobErrorDetailsInstance4 = new JobErrorDetails();
                                                taskInstance.Errors.Add(jobErrorDetailsInstance4);
                                                
                                                JToken serviceErrorDetailsValue4 = errorsValue4["serviceErrorDetails"];
                                                if (serviceErrorDetailsValue4 != null && serviceErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ServiceError serviceErrorDetailsInstance4 = new ServiceError();
                                                    jobErrorDetailsInstance4.ServiceErrorDetails = serviceErrorDetailsInstance4;
                                                    
                                                    JToken codeValue4 = serviceErrorDetailsValue4["code"];
                                                    if (codeValue4 != null && codeValue4.Type != JTokenType.Null)
                                                    {
                                                        string codeInstance4 = ((string)codeValue4);
                                                        serviceErrorDetailsInstance4.Code = codeInstance4;
                                                    }
                                                    
                                                    JToken messageValue4 = serviceErrorDetailsValue4["message"];
                                                    if (messageValue4 != null && messageValue4.Type != JTokenType.Null)
                                                    {
                                                        string messageInstance4 = ((string)messageValue4);
                                                        serviceErrorDetailsInstance4.Message = messageInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue7 = serviceErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue7 != null && possibleCausesValue7.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance7 = ((string)possibleCausesValue7);
                                                        serviceErrorDetailsInstance4.PossibleCauses = possibleCausesInstance7;
                                                    }
                                                    
                                                    JToken recommendedActionValue7 = serviceErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue7 != null && recommendedActionValue7.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance7 = ((string)recommendedActionValue7);
                                                        serviceErrorDetailsInstance4.RecommendedAction = recommendedActionInstance7;
                                                    }
                                                    
                                                    JToken activityIdValue5 = serviceErrorDetailsValue4["activityId"];
                                                    if (activityIdValue5 != null && activityIdValue5.Type != JTokenType.Null)
                                                    {
                                                        string activityIdInstance5 = ((string)activityIdValue5);
                                                        serviceErrorDetailsInstance4.ActivityId = activityIdInstance5;
                                                    }
                                                }
                                                
                                                JToken providerErrorDetailsValue4 = errorsValue4["providerErrorDetails"];
                                                if (providerErrorDetailsValue4 != null && providerErrorDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    ProviderError providerErrorDetailsInstance4 = new ProviderError();
                                                    jobErrorDetailsInstance4.ProviderErrorDetails = providerErrorDetailsInstance4;
                                                    
                                                    JToken errorCodeValue4 = providerErrorDetailsValue4["errorCode"];
                                                    if (errorCodeValue4 != null && errorCodeValue4.Type != JTokenType.Null)
                                                    {
                                                        int errorCodeInstance4 = ((int)errorCodeValue4);
                                                        providerErrorDetailsInstance4.ErrorCode = errorCodeInstance4;
                                                    }
                                                    
                                                    JToken errorMessageValue4 = providerErrorDetailsValue4["errorMessage"];
                                                    if (errorMessageValue4 != null && errorMessageValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorMessageInstance4 = ((string)errorMessageValue4);
                                                        providerErrorDetailsInstance4.ErrorMessage = errorMessageInstance4;
                                                    }
                                                    
                                                    JToken errorIdValue4 = providerErrorDetailsValue4["errorId"];
                                                    if (errorIdValue4 != null && errorIdValue4.Type != JTokenType.Null)
                                                    {
                                                        string errorIdInstance4 = ((string)errorIdValue4);
                                                        providerErrorDetailsInstance4.ErrorId = errorIdInstance4;
                                                    }
                                                    
                                                    JToken possibleCausesValue8 = providerErrorDetailsValue4["possibleCauses"];
                                                    if (possibleCausesValue8 != null && possibleCausesValue8.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance8 = ((string)possibleCausesValue8);
                                                        providerErrorDetailsInstance4.PossibleCauses = possibleCausesInstance8;
                                                    }
                                                    
                                                    JToken recommendedActionValue8 = providerErrorDetailsValue4["recommendedAction"];
                                                    if (recommendedActionValue8 != null && recommendedActionValue8.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance8 = ((string)recommendedActionValue8);
                                                        providerErrorDetailsInstance4.RecommendedAction = recommendedActionInstance8;
                                                    }
                                                }
                                                
                                                JToken errorLevelValue4 = errorsValue4["errorLevel"];
                                                if (errorLevelValue4 != null && errorLevelValue4.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance4 = ((string)errorLevelValue4);
                                                    jobErrorDetailsInstance4.ErrorLevel = errorLevelInstance4;
                                                }
                                                
                                                JToken creationTimeValue4 = errorsValue4["creationTime"];
                                                if (creationTimeValue4 != null && creationTimeValue4.Type != JTokenType.Null)
                                                {
                                                    DateTime creationTimeInstance4 = ((DateTime)creationTimeValue4);
                                                    jobErrorDetailsInstance4.CreationTime = creationTimeInstance4;
                                                }
                                                
                                                JToken taskIdValue8 = errorsValue4["taskId"];
                                                if (taskIdValue8 != null && taskIdValue8.Type != JTokenType.Null)
                                                {
                                                    string taskIdInstance8 = ((string)taskIdValue8);
                                                    jobErrorDetailsInstance4.TaskId = taskIdInstance8;
                                                }
                                            }
                                        }
                                        
                                        JToken customDetailsValue4 = tasksValue["customDetails"];
                                        if (customDetailsValue4 != null && customDetailsValue4.Type != JTokenType.Null)
                                        {
                                            string typeName5 = ((string)customDetailsValue4["instanceType"]);
                                            if (typeName5 == "JobTaskDetails")
                                            {
                                                JobTaskDetails jobTaskDetailsInstance4 = new JobTaskDetails();
                                                
                                                JToken jobTaskValue10 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue10 != null && jobTaskValue10.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance10 = new JobEntity();
                                                    jobTaskDetailsInstance4.JobTask = jobTaskInstance10;
                                                    
                                                    JToken jobIdValue13 = jobTaskValue10["jobId"];
                                                    if (jobIdValue13 != null && jobIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance13 = ((string)jobIdValue13);
                                                        jobTaskInstance10.JobId = jobIdInstance13;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue10 = jobTaskValue10["jobFriendlyName"];
                                                    if (jobFriendlyNameValue10 != null && jobFriendlyNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance10 = ((string)jobFriendlyNameValue10);
                                                        jobTaskInstance10.JobFriendlyName = jobFriendlyNameInstance10;
                                                    }
                                                    
                                                    JToken targetObjectIdValue11 = jobTaskValue10["targetObjectId"];
                                                    if (targetObjectIdValue11 != null && targetObjectIdValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance11 = ((string)targetObjectIdValue11);
                                                        jobTaskInstance10.TargetObjectId = targetObjectIdInstance11;
                                                    }
                                                    
                                                    JToken targetObjectNameValue11 = jobTaskValue10["targetObjectName"];
                                                    if (targetObjectNameValue11 != null && targetObjectNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance11 = ((string)targetObjectNameValue11);
                                                        jobTaskInstance10.TargetObjectName = targetObjectNameInstance11;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue11 = jobTaskValue10["targetInstanceType"];
                                                    if (targetInstanceTypeValue11 != null && targetInstanceTypeValue11.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance11 = ((string)targetInstanceTypeValue11);
                                                        jobTaskInstance10.TargetInstanceType = targetInstanceTypeInstance11;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue10 = jobTaskValue10["jobScenarioName"];
                                                    if (jobScenarioNameValue10 != null && jobScenarioNameValue10.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance10 = ((string)jobScenarioNameValue10);
                                                        jobTaskInstance10.JobScenarioName = jobScenarioNameInstance10;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue28 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue28 != null && instanceTypeValue28.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance28 = ((string)instanceTypeValue28);
                                                    jobTaskDetailsInstance4.Type = instanceTypeInstance28;
                                                }
                                                taskInstance.CustomDetails = jobTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VirtualMachineTaskDetails")
                                            {
                                                VirtualMachineTaskDetails virtualMachineTaskDetailsInstance4 = new VirtualMachineTaskDetails();
                                                
                                                JToken skippedReasonValue7 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue7 != null && skippedReasonValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance7 = ((string)skippedReasonValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReason = skippedReasonInstance7;
                                                }
                                                
                                                JToken skippedReasonStringValue7 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue7 != null && skippedReasonStringValue7.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance7 = ((string)skippedReasonStringValue7);
                                                    virtualMachineTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance7;
                                                }
                                                
                                                JToken jobTaskValue11 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue11 != null && jobTaskValue11.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance11 = new JobEntity();
                                                    virtualMachineTaskDetailsInstance4.JobTask = jobTaskInstance11;
                                                    
                                                    JToken jobIdValue14 = jobTaskValue11["jobId"];
                                                    if (jobIdValue14 != null && jobIdValue14.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance14 = ((string)jobIdValue14);
                                                        jobTaskInstance11.JobId = jobIdInstance14;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue11 = jobTaskValue11["jobFriendlyName"];
                                                    if (jobFriendlyNameValue11 != null && jobFriendlyNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance11 = ((string)jobFriendlyNameValue11);
                                                        jobTaskInstance11.JobFriendlyName = jobFriendlyNameInstance11;
                                                    }
                                                    
                                                    JToken targetObjectIdValue12 = jobTaskValue11["targetObjectId"];
                                                    if (targetObjectIdValue12 != null && targetObjectIdValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance12 = ((string)targetObjectIdValue12);
                                                        jobTaskInstance11.TargetObjectId = targetObjectIdInstance12;
                                                    }
                                                    
                                                    JToken targetObjectNameValue12 = jobTaskValue11["targetObjectName"];
                                                    if (targetObjectNameValue12 != null && targetObjectNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance12 = ((string)targetObjectNameValue12);
                                                        jobTaskInstance11.TargetObjectName = targetObjectNameInstance12;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue12 = jobTaskValue11["targetInstanceType"];
                                                    if (targetInstanceTypeValue12 != null && targetInstanceTypeValue12.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance12 = ((string)targetInstanceTypeValue12);
                                                        jobTaskInstance11.TargetInstanceType = targetInstanceTypeInstance12;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue11 = jobTaskValue11["jobScenarioName"];
                                                    if (jobScenarioNameValue11 != null && jobScenarioNameValue11.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance11 = ((string)jobScenarioNameValue11);
                                                        jobTaskInstance11.JobScenarioName = jobScenarioNameInstance11;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue29 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue29 != null && instanceTypeValue29.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance29 = ((string)instanceTypeValue29);
                                                    virtualMachineTaskDetailsInstance4.Type = instanceTypeInstance29;
                                                }
                                                taskInstance.CustomDetails = virtualMachineTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "FabricReplicationGroupTaskDetails")
                                            {
                                                FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance4 = new FabricReplicationGroupTaskDetails();
                                                
                                                JToken skippedReasonValue8 = customDetailsValue4["skippedReason"];
                                                if (skippedReasonValue8 != null && skippedReasonValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonInstance8 = ((string)skippedReasonValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReason = skippedReasonInstance8;
                                                }
                                                
                                                JToken skippedReasonStringValue8 = customDetailsValue4["skippedReasonString"];
                                                if (skippedReasonStringValue8 != null && skippedReasonStringValue8.Type != JTokenType.Null)
                                                {
                                                    string skippedReasonStringInstance8 = ((string)skippedReasonStringValue8);
                                                    fabricReplicationGroupTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance8;
                                                }
                                                
                                                JToken jobTaskValue12 = customDetailsValue4["jobTask"];
                                                if (jobTaskValue12 != null && jobTaskValue12.Type != JTokenType.Null)
                                                {
                                                    JobEntity jobTaskInstance12 = new JobEntity();
                                                    fabricReplicationGroupTaskDetailsInstance4.JobTask = jobTaskInstance12;
                                                    
                                                    JToken jobIdValue15 = jobTaskValue12["jobId"];
                                                    if (jobIdValue15 != null && jobIdValue15.Type != JTokenType.Null)
                                                    {
                                                        string jobIdInstance15 = ((string)jobIdValue15);
                                                        jobTaskInstance12.JobId = jobIdInstance15;
                                                    }
                                                    
                                                    JToken jobFriendlyNameValue12 = jobTaskValue12["jobFriendlyName"];
                                                    if (jobFriendlyNameValue12 != null && jobFriendlyNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobFriendlyNameInstance12 = ((string)jobFriendlyNameValue12);
                                                        jobTaskInstance12.JobFriendlyName = jobFriendlyNameInstance12;
                                                    }
                                                    
                                                    JToken targetObjectIdValue13 = jobTaskValue12["targetObjectId"];
                                                    if (targetObjectIdValue13 != null && targetObjectIdValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectIdInstance13 = ((string)targetObjectIdValue13);
                                                        jobTaskInstance12.TargetObjectId = targetObjectIdInstance13;
                                                    }
                                                    
                                                    JToken targetObjectNameValue13 = jobTaskValue12["targetObjectName"];
                                                    if (targetObjectNameValue13 != null && targetObjectNameValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetObjectNameInstance13 = ((string)targetObjectNameValue13);
                                                        jobTaskInstance12.TargetObjectName = targetObjectNameInstance13;
                                                    }
                                                    
                                                    JToken targetInstanceTypeValue13 = jobTaskValue12["targetInstanceType"];
                                                    if (targetInstanceTypeValue13 != null && targetInstanceTypeValue13.Type != JTokenType.Null)
                                                    {
                                                        string targetInstanceTypeInstance13 = ((string)targetInstanceTypeValue13);
                                                        jobTaskInstance12.TargetInstanceType = targetInstanceTypeInstance13;
                                                    }
                                                    
                                                    JToken jobScenarioNameValue12 = jobTaskValue12["jobScenarioName"];
                                                    if (jobScenarioNameValue12 != null && jobScenarioNameValue12.Type != JTokenType.Null)
                                                    {
                                                        string jobScenarioNameInstance12 = ((string)jobScenarioNameValue12);
                                                        jobTaskInstance12.JobScenarioName = jobScenarioNameInstance12;
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue30 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue30 != null && instanceTypeValue30.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance30 = ((string)instanceTypeValue30);
                                                    fabricReplicationGroupTaskDetailsInstance4.Type = instanceTypeInstance30;
                                                }
                                                taskInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ManualActionTaskDetails")
                                            {
                                                ManualActionTaskDetails manualActionTaskDetailsInstance4 = new ManualActionTaskDetails();
                                                
                                                JToken nameValue19 = customDetailsValue4["name"];
                                                if (nameValue19 != null && nameValue19.Type != JTokenType.Null)
                                                {
                                                    string nameInstance19 = ((string)nameValue19);
                                                    manualActionTaskDetailsInstance4.Name = nameInstance19;
                                                }
                                                
                                                JToken instructionsValue4 = customDetailsValue4["instructions"];
                                                if (instructionsValue4 != null && instructionsValue4.Type != JTokenType.Null)
                                                {
                                                    string instructionsInstance4 = ((string)instructionsValue4);
                                                    manualActionTaskDetailsInstance4.Instructions = instructionsInstance4;
                                                }
                                                
                                                JToken observationValue4 = customDetailsValue4["observation"];
                                                if (observationValue4 != null && observationValue4.Type != JTokenType.Null)
                                                {
                                                    string observationInstance4 = ((string)observationValue4);
                                                    manualActionTaskDetailsInstance4.Observation = observationInstance4;
                                                }
                                                
                                                JToken instanceTypeValue31 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue31 != null && instanceTypeValue31.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance31 = ((string)instanceTypeValue31);
                                                    manualActionTaskDetailsInstance4.Type = instanceTypeInstance31;
                                                }
                                                taskInstance.CustomDetails = manualActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ScriptActionTaskDetails")
                                            {
                                                ScriptActionTaskDetails scriptActionTaskDetailsInstance4 = new ScriptActionTaskDetails();
                                                
                                                JToken nameValue20 = customDetailsValue4["name"];
                                                if (nameValue20 != null && nameValue20.Type != JTokenType.Null)
                                                {
                                                    string nameInstance20 = ((string)nameValue20);
                                                    scriptActionTaskDetailsInstance4.Name = nameInstance20;
                                                }
                                                
                                                JToken pathValue4 = customDetailsValue4["path"];
                                                if (pathValue4 != null && pathValue4.Type != JTokenType.Null)
                                                {
                                                    string pathInstance4 = ((string)pathValue4);
                                                    scriptActionTaskDetailsInstance4.Path = pathInstance4;
                                                }
                                                
                                                JToken outputValue4 = customDetailsValue4["output"];
                                                if (outputValue4 != null && outputValue4.Type != JTokenType.Null)
                                                {
                                                    string outputInstance4 = ((string)outputValue4);
                                                    scriptActionTaskDetailsInstance4.Output = outputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue7 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue7 != null && isPrimarySideScriptValue7.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance7 = ((bool)isPrimarySideScriptValue7);
                                                    scriptActionTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance7;
                                                }
                                                
                                                JToken instanceTypeValue32 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue32 != null && instanceTypeValue32.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance32 = ((string)instanceTypeValue32);
                                                    scriptActionTaskDetailsInstance4.Type = instanceTypeInstance32;
                                                }
                                                taskInstance.CustomDetails = scriptActionTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "VmNicUpdatesTaskDetails")
                                            {
                                                VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance4 = new VmNicUpdatesTaskDetails();
                                                
                                                JToken vmIdValue4 = customDetailsValue4["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken nicIdValue4 = customDetailsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vmNicUpdatesTaskDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken nameValue21 = customDetailsValue4["name"];
                                                if (nameValue21 != null && nameValue21.Type != JTokenType.Null)
                                                {
                                                    string nameInstance21 = ((string)nameValue21);
                                                    vmNicUpdatesTaskDetailsInstance4.Name = nameInstance21;
                                                }
                                                
                                                JToken instanceTypeValue33 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue33 != null && instanceTypeValue33.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance33 = ((string)instanceTypeValue33);
                                                    vmNicUpdatesTaskDetailsInstance4.Type = instanceTypeInstance33;
                                                }
                                                taskInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "ConsistencyCheckTaskDetails")
                                            {
                                                ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance4 = new ConsistencyCheckTaskDetails();
                                                
                                                JToken vmDetailsArray4 = customDetailsValue4["vmDetails"];
                                                if (vmDetailsArray4 != null && vmDetailsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmDetailsValue4 in ((JArray)vmDetailsArray4))
                                                    {
                                                        InconsistentVmDetails inconsistentVmDetailsInstance4 = new InconsistentVmDetails();
                                                        consistencyCheckTaskDetailsInstance4.VmDetails.Add(inconsistentVmDetailsInstance4);
                                                        
                                                        JToken vmNameValue4 = vmDetailsValue4["vmName"];
                                                        if (vmNameValue4 != null && vmNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vmNameInstance4 = ((string)vmNameValue4);
                                                            inconsistentVmDetailsInstance4.VmName = vmNameInstance4;
                                                        }
                                                        
                                                        JToken cloudNameValue4 = vmDetailsValue4["cloudName"];
                                                        if (cloudNameValue4 != null && cloudNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string cloudNameInstance4 = ((string)cloudNameValue4);
                                                            inconsistentVmDetailsInstance4.CloudName = cloudNameInstance4;
                                                        }
                                                        
                                                        JToken detailsArray4 = vmDetailsValue4["details"];
                                                        if (detailsArray4 != null && detailsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken detailsValue4 in ((JArray)detailsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.Details.Add(((string)detailsValue4));
                                                            }
                                                        }
                                                        
                                                        JToken errorIdsArray4 = vmDetailsValue4["errorIds"];
                                                        if (errorIdsArray4 != null && errorIdsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken errorIdsValue4 in ((JArray)errorIdsArray4))
                                                            {
                                                                inconsistentVmDetailsInstance4.ErrorIds.Add(((string)errorIdsValue4));
                                                            }
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue34 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue34 != null && instanceTypeValue34.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance34 = ((string)instanceTypeValue34);
                                                    consistencyCheckTaskDetailsInstance4.Type = instanceTypeInstance34;
                                                }
                                                taskInstance.CustomDetails = consistencyCheckTaskDetailsInstance4;
                                            }
                                            if (typeName5 == "AutomationRunbookTaskDetails")
                                            {
                                                AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance4 = new AutomationRunbookTaskDetails();
                                                
                                                JToken nameValue22 = customDetailsValue4["name"];
                                                if (nameValue22 != null && nameValue22.Type != JTokenType.Null)
                                                {
                                                    string nameInstance22 = ((string)nameValue22);
                                                    automationRunbookTaskDetailsInstance4.Name = nameInstance22;
                                                }
                                                
                                                JToken cloudServiceNameValue4 = customDetailsValue4["cloudServiceName"];
                                                if (cloudServiceNameValue4 != null && cloudServiceNameValue4.Type != JTokenType.Null)
                                                {
                                                    string cloudServiceNameInstance4 = ((string)cloudServiceNameValue4);
                                                    automationRunbookTaskDetailsInstance4.CloudServiceName = cloudServiceNameInstance4;
                                                }
                                                
                                                JToken subscriptionIdValue4 = customDetailsValue4["subscriptionId"];
                                                if (subscriptionIdValue4 != null && subscriptionIdValue4.Type != JTokenType.Null)
                                                {
                                                    string subscriptionIdInstance4 = ((string)subscriptionIdValue4);
                                                    automationRunbookTaskDetailsInstance4.SubscriptionId = subscriptionIdInstance4;
                                                }
                                                
                                                JToken accountNameValue4 = customDetailsValue4["accountName"];
                                                if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                {
                                                    string accountNameInstance4 = ((string)accountNameValue4);
                                                    automationRunbookTaskDetailsInstance4.AccountName = accountNameInstance4;
                                                }
                                                
                                                JToken runbookIdValue4 = customDetailsValue4["runbookId"];
                                                if (runbookIdValue4 != null && runbookIdValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookIdInstance4 = ((string)runbookIdValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookId = runbookIdInstance4;
                                                }
                                                
                                                JToken runbookNameValue4 = customDetailsValue4["runbookName"];
                                                if (runbookNameValue4 != null && runbookNameValue4.Type != JTokenType.Null)
                                                {
                                                    string runbookNameInstance4 = ((string)runbookNameValue4);
                                                    automationRunbookTaskDetailsInstance4.RunbookName = runbookNameInstance4;
                                                }
                                                
                                                JToken jobIdValue16 = customDetailsValue4["jobId"];
                                                if (jobIdValue16 != null && jobIdValue16.Type != JTokenType.Null)
                                                {
                                                    string jobIdInstance16 = ((string)jobIdValue16);
                                                    automationRunbookTaskDetailsInstance4.JobId = jobIdInstance16;
                                                }
                                                
                                                JToken jobOutputValue4 = customDetailsValue4["jobOutput"];
                                                if (jobOutputValue4 != null && jobOutputValue4.Type != JTokenType.Null)
                                                {
                                                    string jobOutputInstance4 = ((string)jobOutputValue4);
                                                    automationRunbookTaskDetailsInstance4.JobOutput = jobOutputInstance4;
                                                }
                                                
                                                JToken isPrimarySideScriptValue8 = customDetailsValue4["isPrimarySideScript"];
                                                if (isPrimarySideScriptValue8 != null && isPrimarySideScriptValue8.Type != JTokenType.Null)
                                                {
                                                    bool isPrimarySideScriptInstance8 = ((bool)isPrimarySideScriptValue8);
                                                    automationRunbookTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance8;
                                                }
                                                
                                                JToken instanceTypeValue35 = customDetailsValue4["instanceType"];
                                                if (instanceTypeValue35 != null && instanceTypeValue35.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance35 = ((string)instanceTypeValue35);
                                                    automationRunbookTaskDetailsInstance4.Type = instanceTypeInstance35;
                                                }
                                                taskInstance.CustomDetails = automationRunbookTaskDetailsInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken errorsArray5 = propertiesValue["errors"];
                                if (errorsArray5 != null && errorsArray5.Type != JTokenType.Null)
                                {
                                    foreach (JToken errorsValue5 in ((JArray)errorsArray5))
                                    {
                                        JobErrorDetails jobErrorDetailsInstance5 = new JobErrorDetails();
                                        propertiesInstance.Errors.Add(jobErrorDetailsInstance5);
                                        
                                        JToken serviceErrorDetailsValue5 = errorsValue5["serviceErrorDetails"];
                                        if (serviceErrorDetailsValue5 != null && serviceErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ServiceError serviceErrorDetailsInstance5 = new ServiceError();
                                            jobErrorDetailsInstance5.ServiceErrorDetails = serviceErrorDetailsInstance5;
                                            
                                            JToken codeValue5 = serviceErrorDetailsValue5["code"];
                                            if (codeValue5 != null && codeValue5.Type != JTokenType.Null)
                                            {
                                                string codeInstance5 = ((string)codeValue5);
                                                serviceErrorDetailsInstance5.Code = codeInstance5;
                                            }
                                            
                                            JToken messageValue5 = serviceErrorDetailsValue5["message"];
                                            if (messageValue5 != null && messageValue5.Type != JTokenType.Null)
                                            {
                                                string messageInstance5 = ((string)messageValue5);
                                                serviceErrorDetailsInstance5.Message = messageInstance5;
                                            }
                                            
                                            JToken possibleCausesValue9 = serviceErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue9 != null && possibleCausesValue9.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance9 = ((string)possibleCausesValue9);
                                                serviceErrorDetailsInstance5.PossibleCauses = possibleCausesInstance9;
                                            }
                                            
                                            JToken recommendedActionValue9 = serviceErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue9 != null && recommendedActionValue9.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance9 = ((string)recommendedActionValue9);
                                                serviceErrorDetailsInstance5.RecommendedAction = recommendedActionInstance9;
                                            }
                                            
                                            JToken activityIdValue6 = serviceErrorDetailsValue5["activityId"];
                                            if (activityIdValue6 != null && activityIdValue6.Type != JTokenType.Null)
                                            {
                                                string activityIdInstance6 = ((string)activityIdValue6);
                                                serviceErrorDetailsInstance5.ActivityId = activityIdInstance6;
                                            }
                                        }
                                        
                                        JToken providerErrorDetailsValue5 = errorsValue5["providerErrorDetails"];
                                        if (providerErrorDetailsValue5 != null && providerErrorDetailsValue5.Type != JTokenType.Null)
                                        {
                                            ProviderError providerErrorDetailsInstance5 = new ProviderError();
                                            jobErrorDetailsInstance5.ProviderErrorDetails = providerErrorDetailsInstance5;
                                            
                                            JToken errorCodeValue5 = providerErrorDetailsValue5["errorCode"];
                                            if (errorCodeValue5 != null && errorCodeValue5.Type != JTokenType.Null)
                                            {
                                                int errorCodeInstance5 = ((int)errorCodeValue5);
                                                providerErrorDetailsInstance5.ErrorCode = errorCodeInstance5;
                                            }
                                            
                                            JToken errorMessageValue5 = providerErrorDetailsValue5["errorMessage"];
                                            if (errorMessageValue5 != null && errorMessageValue5.Type != JTokenType.Null)
                                            {
                                                string errorMessageInstance5 = ((string)errorMessageValue5);
                                                providerErrorDetailsInstance5.ErrorMessage = errorMessageInstance5;
                                            }
                                            
                                            JToken errorIdValue5 = providerErrorDetailsValue5["errorId"];
                                            if (errorIdValue5 != null && errorIdValue5.Type != JTokenType.Null)
                                            {
                                                string errorIdInstance5 = ((string)errorIdValue5);
                                                providerErrorDetailsInstance5.ErrorId = errorIdInstance5;
                                            }
                                            
                                            JToken possibleCausesValue10 = providerErrorDetailsValue5["possibleCauses"];
                                            if (possibleCausesValue10 != null && possibleCausesValue10.Type != JTokenType.Null)
                                            {
                                                string possibleCausesInstance10 = ((string)possibleCausesValue10);
                                                providerErrorDetailsInstance5.PossibleCauses = possibleCausesInstance10;
                                            }
                                            
                                            JToken recommendedActionValue10 = providerErrorDetailsValue5["recommendedAction"];
                                            if (recommendedActionValue10 != null && recommendedActionValue10.Type != JTokenType.Null)
                                            {
                                                string recommendedActionInstance10 = ((string)recommendedActionValue10);
                                                providerErrorDetailsInstance5.RecommendedAction = recommendedActionInstance10;
                                            }
                                        }
                                        
                                        JToken errorLevelValue5 = errorsValue5["errorLevel"];
                                        if (errorLevelValue5 != null && errorLevelValue5.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance5 = ((string)errorLevelValue5);
                                            jobErrorDetailsInstance5.ErrorLevel = errorLevelInstance5;
                                        }
                                        
                                        JToken creationTimeValue5 = errorsValue5["creationTime"];
                                        if (creationTimeValue5 != null && creationTimeValue5.Type != JTokenType.Null)
                                        {
                                            DateTime creationTimeInstance5 = ((DateTime)creationTimeValue5);
                                            jobErrorDetailsInstance5.CreationTime = creationTimeInstance5;
                                        }
                                        
                                        JToken taskIdValue9 = errorsValue5["taskId"];
                                        if (taskIdValue9 != null && taskIdValue9.Type != JTokenType.Null)
                                        {
                                            string taskIdInstance9 = ((string)taskIdValue9);
                                            jobErrorDetailsInstance5.TaskId = taskIdInstance9;
                                        }
                                    }
                                }
                                
                                JToken customDetailsValue5 = propertiesValue["customDetails"];
                                if (customDetailsValue5 != null && customDetailsValue5.Type != JTokenType.Null)
                                {
                                    string typeName6 = ((string)customDetailsValue5["instanceType"]);
                                    if (typeName6 == "ExportJobDetails")
                                    {
                                        ExportJobDetails exportJobDetailsInstance = new ExportJobDetails();
                                        
                                        JToken blobUriValue = customDetailsValue5["blobUri"];
                                        if (blobUriValue != null && blobUriValue.Type != JTokenType.Null)
                                        {
                                            string blobUriInstance = ((string)blobUriValue);
                                            exportJobDetailsInstance.BlobUri = blobUriInstance;
                                        }
                                        
                                        JToken sasTokenValue = customDetailsValue5["sasToken"];
                                        if (sasTokenValue != null && sasTokenValue.Type != JTokenType.Null)
                                        {
                                            string sasTokenInstance = ((string)sasTokenValue);
                                            exportJobDetailsInstance.SasToken = sasTokenInstance;
                                        }
                                        
                                        JToken instanceTypeValue36 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue36 != null && instanceTypeValue36.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance36 = ((string)instanceTypeValue36);
                                            exportJobDetailsInstance.InstanceType = instanceTypeInstance36;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement != null && affectedObjectDetailsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in affectedObjectDetailsSequenceElement)
                                            {
                                                string affectedObjectDetailsKey = ((string)property.Name);
                                                string affectedObjectDetailsValue = ((string)property.Value);
                                                exportJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey, affectedObjectDetailsValue);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = exportJobDetailsInstance;
                                    }
                                    if (typeName6 == "AsrJobDetails")
                                    {
                                        AsrJobDetails asrJobDetailsInstance = new AsrJobDetails();
                                        
                                        JToken instanceTypeValue37 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue37 != null && instanceTypeValue37.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance37 = ((string)instanceTypeValue37);
                                            asrJobDetailsInstance.InstanceType = instanceTypeInstance37;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement2 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement2 != null && affectedObjectDetailsSequenceElement2.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property2 in affectedObjectDetailsSequenceElement2)
                                            {
                                                string affectedObjectDetailsKey2 = ((string)property2.Name);
                                                string affectedObjectDetailsValue2 = ((string)property2.Value);
                                                asrJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey2, affectedObjectDetailsValue2);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = asrJobDetailsInstance;
                                    }
                                    if (typeName6 == "TestFailoverJobDetails")
                                    {
                                        TestFailoverJobDetails testFailoverJobDetailsInstance = new TestFailoverJobDetails();
                                        
                                        JToken testFailoverStatusValue = customDetailsValue5["testFailoverStatus"];
                                        if (testFailoverStatusValue != null && testFailoverStatusValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStatusInstance = ((string)testFailoverStatusValue);
                                            testFailoverJobDetailsInstance.TestFailoverStatus = testFailoverStatusInstance;
                                        }
                                        
                                        JToken commentsValue = customDetailsValue5["comments"];
                                        if (commentsValue != null && commentsValue.Type != JTokenType.Null)
                                        {
                                            string commentsInstance = ((string)commentsValue);
                                            testFailoverJobDetailsInstance.Comments = commentsInstance;
                                        }
                                        
                                        JToken networkNameValue = customDetailsValue5["networkName"];
                                        if (networkNameValue != null && networkNameValue.Type != JTokenType.Null)
                                        {
                                            string networkNameInstance = ((string)networkNameValue);
                                            testFailoverJobDetailsInstance.NetworkName = networkNameInstance;
                                        }
                                        
                                        JToken networkFriendlyNameValue = customDetailsValue5["networkFriendlyName"];
                                        if (networkFriendlyNameValue != null && networkFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string networkFriendlyNameInstance = ((string)networkFriendlyNameValue);
                                            testFailoverJobDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance;
                                        }
                                        
                                        JToken networkTypeValue = customDetailsValue5["networkType"];
                                        if (networkTypeValue != null && networkTypeValue.Type != JTokenType.Null)
                                        {
                                            string networkTypeInstance = ((string)networkTypeValue);
                                            testFailoverJobDetailsInstance.NetworkType = networkTypeInstance;
                                        }
                                        
                                        JToken protectedItemDetailsArray = customDetailsValue5["protectedItemDetails"];
                                        if (protectedItemDetailsArray != null && protectedItemDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedItemDetailsValue in ((JArray)protectedItemDetailsArray))
                                            {
                                                TestFailoverReplicationProtectedItemDetails testFailoverReplicationProtectedItemDetailsInstance = new TestFailoverReplicationProtectedItemDetails();
                                                testFailoverJobDetailsInstance.ProtectedItemDetails.Add(testFailoverReplicationProtectedItemDetailsInstance);
                                                
                                                JToken nameValue23 = protectedItemDetailsValue["name"];
                                                if (nameValue23 != null && nameValue23.Type != JTokenType.Null)
                                                {
                                                    string nameInstance23 = ((string)nameValue23);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Name = nameInstance23;
                                                }
                                                
                                                JToken friendlyNameValue6 = protectedItemDetailsValue["friendlyName"];
                                                if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                                {
                                                    string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                                    testFailoverReplicationProtectedItemDetailsInstance.FriendlyName = friendlyNameInstance6;
                                                }
                                                
                                                JToken testVmNameValue = protectedItemDetailsValue["testVmName"];
                                                if (testVmNameValue != null && testVmNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmNameInstance = ((string)testVmNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmName = testVmNameInstance;
                                                }
                                                
                                                JToken testVmFriendlyNameValue = protectedItemDetailsValue["testVmFriendlyName"];
                                                if (testVmFriendlyNameValue != null && testVmFriendlyNameValue.Type != JTokenType.Null)
                                                {
                                                    string testVmFriendlyNameInstance = ((string)testVmFriendlyNameValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.TestVmFriendlyName = testVmFriendlyNameInstance;
                                                }
                                                
                                                JToken networkConnectionStatusValue = protectedItemDetailsValue["networkConnectionStatus"];
                                                if (networkConnectionStatusValue != null && networkConnectionStatusValue.Type != JTokenType.Null)
                                                {
                                                    string networkConnectionStatusInstance = ((string)networkConnectionStatusValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkConnectionStatus = networkConnectionStatusInstance;
                                                }
                                                
                                                JToken networkFriendlyNameValue2 = protectedItemDetailsValue["networkFriendlyName"];
                                                if (networkFriendlyNameValue2 != null && networkFriendlyNameValue2.Type != JTokenType.Null)
                                                {
                                                    string networkFriendlyNameInstance2 = ((string)networkFriendlyNameValue2);
                                                    testFailoverReplicationProtectedItemDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance2;
                                                }
                                                
                                                JToken subnetValue = protectedItemDetailsValue["subnet"];
                                                if (subnetValue != null && subnetValue.Type != JTokenType.Null)
                                                {
                                                    string subnetInstance = ((string)subnetValue);
                                                    testFailoverReplicationProtectedItemDetailsInstance.Subnet = subnetInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue38 = customDetailsValue5["instanceType"];
                                        if (instanceTypeValue38 != null && instanceTypeValue38.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance38 = ((string)instanceTypeValue38);
                                            testFailoverJobDetailsInstance.InstanceType = instanceTypeInstance38;
                                        }
                                        
                                        JToken affectedObjectDetailsSequenceElement3 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                        if (affectedObjectDetailsSequenceElement3 != null && affectedObjectDetailsSequenceElement3.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property3 in affectedObjectDetailsSequenceElement3)
                                            {
                                                string affectedObjectDetailsKey3 = ((string)property3.Name);
                                                string affectedObjectDetailsValue3 = ((string)property3.Value);
                                                testFailoverJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey3, affectedObjectDetailsValue3);
                                            }
                                        }
                                        propertiesInstance.CustomDetails = testFailoverJobDetailsInstance;
                                    }
                                }
                                
                                JToken idValue = propertiesValue["id"];
                                if (idValue != null && idValue.Type != JTokenType.Null)
                                {
                                    string idInstance = ((string)idValue);
                                    propertiesInstance.Id = idInstance;
                                }
                                
                                JToken nameValue24 = propertiesValue["name"];
                                if (nameValue24 != null && nameValue24.Type != JTokenType.Null)
                                {
                                    string nameInstance24 = ((string)nameValue24);
                                    propertiesInstance.Name = nameInstance24;
                                }
                                
                                JToken typeValue = propertiesValue["type"];
                                if (typeValue != null && typeValue.Type != JTokenType.Null)
                                {
                                    string typeInstance = ((string)typeValue);
                                    propertiesInstance.Type = typeInstance;
                                }
                                
                                JToken locationValue = propertiesValue["location"];
                                if (locationValue != null && locationValue.Type != JTokenType.Null)
                                {
                                    string locationInstance = ((string)locationValue);
                                    propertiesInstance.Location = locationInstance;
                                }
                                
                                JToken tagsSequenceElement = ((JToken)propertiesValue["tags"]);
                                if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property4 in tagsSequenceElement)
                                    {
                                        string tagsKey = ((string)property4.Name);
                                        string tagsValue = ((string)property4.Value);
                                        propertiesInstance.Tags.Add(tagsKey, tagsValue);
                                    }
                                }
                            }
                            
                            JToken idValue2 = responseDoc["id"];
                            if (idValue2 != null && idValue2.Type != JTokenType.Null)
                            {
                                string idInstance2 = ((string)idValue2);
                                jobInstance.Id = idInstance2;
                            }
                            
                            JToken nameValue25 = responseDoc["name"];
                            if (nameValue25 != null && nameValue25.Type != JTokenType.Null)
                            {
                                string nameInstance25 = ((string)nameValue25);
                                jobInstance.Name = nameInstance25;
                            }
                            
                            JToken typeValue2 = responseDoc["type"];
                            if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                            {
                                string typeInstance2 = ((string)typeValue2);
                                jobInstance.Type = typeInstance2;
                            }
                            
                            JToken locationValue2 = responseDoc["location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                jobInstance.Location = locationInstance2;
                            }
                            
                            JToken tagsSequenceElement2 = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement2 != null && tagsSequenceElement2.Type != JTokenType.Null)
                            {
                                foreach (JProperty property5 in tagsSequenceElement2)
                                {
                                    string tagsKey2 = ((string)property5.Name);
                                    string tagsValue2 = ((string)property5.Value);
                                    jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken locationValue3 = responseDoc["Location"];
                            if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                            {
                                string locationInstance3 = ((string)locationValue3);
                                result.Location = locationInstance3;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Get the list of all jobs.
        /// </summary>
        /// <param name='parameters'>
        /// Optional. Job query parameter.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list Jobs operation.
        /// </returns>
        public async Task<JobListResponse> ListAsync(JobQueryParameter parameters, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationJobs";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            List<string> odataFilter = new List<string>();
            if (parameters != null && parameters.ToQueryString() != null)
            {
                odataFilter.Add(Uri.EscapeDataString(parameters.ToQueryString().ToString()));
            }
            if (odataFilter.Count > 0)
            {
                queryParameters.Add("$filter=" + string.Join(null, odataFilter));
            }
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    Job jobInstance = new Job();
                                    result.Jobs.Add(jobInstance);
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        JobProperties propertiesInstance = new JobProperties();
                                        jobInstance.Properties = propertiesInstance;
                                        
                                        JToken scenarioNameValue = propertiesValue["scenarioName"];
                                        if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                        {
                                            string scenarioNameInstance = ((string)scenarioNameValue);
                                            propertiesInstance.ScenarioName = scenarioNameInstance;
                                        }
                                        
                                        JToken friendlyNameValue = propertiesValue["friendlyName"];
                                        if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance = ((string)friendlyNameValue);
                                            propertiesInstance.FriendlyName = friendlyNameInstance;
                                        }
                                        
                                        JToken activityIdValue = propertiesValue["activityId"];
                                        if (activityIdValue != null && activityIdValue.Type != JTokenType.Null)
                                        {
                                            string activityIdInstance = ((string)activityIdValue);
                                            propertiesInstance.ActivityId = activityIdInstance;
                                        }
                                        
                                        JToken startTimeValue = propertiesValue["startTime"];
                                        if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime startTimeInstance = ((DateTime)startTimeValue);
                                            propertiesInstance.StartTime = startTimeInstance;
                                        }
                                        
                                        JToken endTimeValue = propertiesValue["endTime"];
                                        if (endTimeValue != null && endTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime endTimeInstance = ((DateTime)endTimeValue);
                                            propertiesInstance.EndTime = endTimeInstance;
                                        }
                                        
                                        JToken targetObjectIdValue = propertiesValue["targetObjectId"];
                                        if (targetObjectIdValue != null && targetObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string targetObjectIdInstance = ((string)targetObjectIdValue);
                                            propertiesInstance.TargetObjectId = targetObjectIdInstance;
                                        }
                                        
                                        JToken targetInstanceTypeValue = propertiesValue["targetInstanceType"];
                                        if (targetInstanceTypeValue != null && targetInstanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string targetInstanceTypeInstance = ((string)targetInstanceTypeValue);
                                            propertiesInstance.TargetInstanceType = targetInstanceTypeInstance;
                                        }
                                        
                                        JToken targetObjectNameValue = propertiesValue["targetObjectName"];
                                        if (targetObjectNameValue != null && targetObjectNameValue.Type != JTokenType.Null)
                                        {
                                            string targetObjectNameInstance = ((string)targetObjectNameValue);
                                            propertiesInstance.TargetObjectName = targetObjectNameInstance;
                                        }
                                        
                                        JToken allowedActionsArray = propertiesValue["allowedActions"];
                                        if (allowedActionsArray != null && allowedActionsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedActionsValue in ((JArray)allowedActionsArray))
                                            {
                                                propertiesInstance.AllowedActions.Add(((string)allowedActionsValue));
                                            }
                                        }
                                        
                                        JToken stateValue = propertiesValue["state"];
                                        if (stateValue != null && stateValue.Type != JTokenType.Null)
                                        {
                                            string stateInstance = ((string)stateValue);
                                            propertiesInstance.State = stateInstance;
                                        }
                                        
                                        JToken stateDescriptionValue = propertiesValue["stateDescription"];
                                        if (stateDescriptionValue != null && stateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string stateDescriptionInstance = ((string)stateDescriptionValue);
                                            propertiesInstance.StateDescription = stateDescriptionInstance;
                                        }
                                        
                                        JToken tasksArray = propertiesValue["tasks"];
                                        if (tasksArray != null && tasksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken tasksValue in ((JArray)tasksArray))
                                            {
                                                AsrTask taskInstance = new AsrTask();
                                                propertiesInstance.Tasks.Add(taskInstance);
                                                
                                                JToken groupTaskCustomDetailsValue = tasksValue["groupTaskCustomDetails"];
                                                if (groupTaskCustomDetailsValue != null && groupTaskCustomDetailsValue.Type != JTokenType.Null)
                                                {
                                                    string typeName = ((string)groupTaskCustomDetailsValue["instanceType"]);
                                                    if (typeName == "InlineWorkflowTaskDetails")
                                                    {
                                                        InlineWorkflowTaskDetails inlineWorkflowTaskDetailsInstance = new InlineWorkflowTaskDetails();
                                                        
                                                        JToken workflowIdsArray = groupTaskCustomDetailsValue["workflowIds"];
                                                        if (workflowIdsArray != null && workflowIdsArray.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken workflowIdsValue in ((JArray)workflowIdsArray))
                                                            {
                                                                inlineWorkflowTaskDetailsInstance.WorkflowIds.Add(((string)workflowIdsValue));
                                                            }
                                                        }
                                                        
                                                        JToken instanceTypeValue = groupTaskCustomDetailsValue["instanceType"];
                                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                                            inlineWorkflowTaskDetailsInstance.Type = instanceTypeInstance;
                                                        }
                                                        
                                                        JToken childTasksArray = groupTaskCustomDetailsValue["childTasks"];
                                                        if (childTasksArray != null && childTasksArray.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken childTasksValue in ((JArray)childTasksArray))
                                                            {
                                                                AsrTaskBase taskBaseInstance = new AsrTaskBase();
                                                                inlineWorkflowTaskDetailsInstance.ChildTasks.Add(taskBaseInstance);
                                                                
                                                                JToken taskIdValue = childTasksValue["taskId"];
                                                                if (taskIdValue != null && taskIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance = ((string)taskIdValue);
                                                                    taskBaseInstance.ID = taskIdInstance;
                                                                }
                                                                
                                                                JToken nameValue = childTasksValue["name"];
                                                                if (nameValue != null && nameValue.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance = ((string)nameValue);
                                                                    taskBaseInstance.Name = nameInstance;
                                                                }
                                                                
                                                                JToken startTimeValue2 = childTasksValue["startTime"];
                                                                if (startTimeValue2 != null && startTimeValue2.Type != JTokenType.Null)
                                                                {
                                                                    DateTime startTimeInstance2 = ((DateTime)startTimeValue2);
                                                                    taskBaseInstance.StartTime = startTimeInstance2;
                                                                }
                                                                
                                                                JToken endTimeValue2 = childTasksValue["endTime"];
                                                                if (endTimeValue2 != null && endTimeValue2.Type != JTokenType.Null)
                                                                {
                                                                    DateTime endTimeInstance2 = ((DateTime)endTimeValue2);
                                                                    taskBaseInstance.EndTime = endTimeInstance2;
                                                                }
                                                                
                                                                JToken allowedActionsArray2 = childTasksValue["allowedActions"];
                                                                if (allowedActionsArray2 != null && allowedActionsArray2.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken allowedActionsValue2 in ((JArray)allowedActionsArray2))
                                                                    {
                                                                        taskBaseInstance.Actions.Add(((string)allowedActionsValue2));
                                                                    }
                                                                }
                                                                
                                                                JToken friendlyNameValue2 = childTasksValue["friendlyName"];
                                                                if (friendlyNameValue2 != null && friendlyNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string friendlyNameInstance2 = ((string)friendlyNameValue2);
                                                                    taskBaseInstance.TaskFriendlyName = friendlyNameInstance2;
                                                                }
                                                                
                                                                JToken stateValue2 = childTasksValue["state"];
                                                                if (stateValue2 != null && stateValue2.Type != JTokenType.Null)
                                                                {
                                                                    string stateInstance2 = ((string)stateValue2);
                                                                    taskBaseInstance.State = stateInstance2;
                                                                }
                                                                
                                                                JToken stateDescriptionValue2 = childTasksValue["stateDescription"];
                                                                if (stateDescriptionValue2 != null && stateDescriptionValue2.Type != JTokenType.Null)
                                                                {
                                                                    string stateDescriptionInstance2 = ((string)stateDescriptionValue2);
                                                                    taskBaseInstance.StateDescription = stateDescriptionInstance2;
                                                                }
                                                                
                                                                JToken taskTypeValue = childTasksValue["taskType"];
                                                                if (taskTypeValue != null && taskTypeValue.Type != JTokenType.Null)
                                                                {
                                                                    string taskTypeInstance = ((string)taskTypeValue);
                                                                    taskBaseInstance.TaskType = taskTypeInstance;
                                                                }
                                                                
                                                                JToken errorsArray = childTasksValue["errors"];
                                                                if (errorsArray != null && errorsArray.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken errorsValue in ((JArray)errorsArray))
                                                                    {
                                                                        JobErrorDetails jobErrorDetailsInstance = new JobErrorDetails();
                                                                        taskBaseInstance.Errors.Add(jobErrorDetailsInstance);
                                                                        
                                                                        JToken serviceErrorDetailsValue = errorsValue["serviceErrorDetails"];
                                                                        if (serviceErrorDetailsValue != null && serviceErrorDetailsValue.Type != JTokenType.Null)
                                                                        {
                                                                            ServiceError serviceErrorDetailsInstance = new ServiceError();
                                                                            jobErrorDetailsInstance.ServiceErrorDetails = serviceErrorDetailsInstance;
                                                                            
                                                                            JToken codeValue = serviceErrorDetailsValue["code"];
                                                                            if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                            {
                                                                                string codeInstance = ((string)codeValue);
                                                                                serviceErrorDetailsInstance.Code = codeInstance;
                                                                            }
                                                                            
                                                                            JToken messageValue = serviceErrorDetailsValue["message"];
                                                                            if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                            {
                                                                                string messageInstance = ((string)messageValue);
                                                                                serviceErrorDetailsInstance.Message = messageInstance;
                                                                            }
                                                                            
                                                                            JToken possibleCausesValue = serviceErrorDetailsValue["possibleCauses"];
                                                                            if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                                            {
                                                                                string possibleCausesInstance = ((string)possibleCausesValue);
                                                                                serviceErrorDetailsInstance.PossibleCauses = possibleCausesInstance;
                                                                            }
                                                                            
                                                                            JToken recommendedActionValue = serviceErrorDetailsValue["recommendedAction"];
                                                                            if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                                            {
                                                                                string recommendedActionInstance = ((string)recommendedActionValue);
                                                                                serviceErrorDetailsInstance.RecommendedAction = recommendedActionInstance;
                                                                            }
                                                                            
                                                                            JToken activityIdValue2 = serviceErrorDetailsValue["activityId"];
                                                                            if (activityIdValue2 != null && activityIdValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string activityIdInstance2 = ((string)activityIdValue2);
                                                                                serviceErrorDetailsInstance.ActivityId = activityIdInstance2;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken providerErrorDetailsValue = errorsValue["providerErrorDetails"];
                                                                        if (providerErrorDetailsValue != null && providerErrorDetailsValue.Type != JTokenType.Null)
                                                                        {
                                                                            ProviderError providerErrorDetailsInstance = new ProviderError();
                                                                            jobErrorDetailsInstance.ProviderErrorDetails = providerErrorDetailsInstance;
                                                                            
                                                                            JToken errorCodeValue = providerErrorDetailsValue["errorCode"];
                                                                            if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                                            {
                                                                                int errorCodeInstance = ((int)errorCodeValue);
                                                                                providerErrorDetailsInstance.ErrorCode = errorCodeInstance;
                                                                            }
                                                                            
                                                                            JToken errorMessageValue = providerErrorDetailsValue["errorMessage"];
                                                                            if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                                            {
                                                                                string errorMessageInstance = ((string)errorMessageValue);
                                                                                providerErrorDetailsInstance.ErrorMessage = errorMessageInstance;
                                                                            }
                                                                            
                                                                            JToken errorIdValue = providerErrorDetailsValue["errorId"];
                                                                            if (errorIdValue != null && errorIdValue.Type != JTokenType.Null)
                                                                            {
                                                                                string errorIdInstance = ((string)errorIdValue);
                                                                                providerErrorDetailsInstance.ErrorId = errorIdInstance;
                                                                            }
                                                                            
                                                                            JToken possibleCausesValue2 = providerErrorDetailsValue["possibleCauses"];
                                                                            if (possibleCausesValue2 != null && possibleCausesValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string possibleCausesInstance2 = ((string)possibleCausesValue2);
                                                                                providerErrorDetailsInstance.PossibleCauses = possibleCausesInstance2;
                                                                            }
                                                                            
                                                                            JToken recommendedActionValue2 = providerErrorDetailsValue["recommendedAction"];
                                                                            if (recommendedActionValue2 != null && recommendedActionValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string recommendedActionInstance2 = ((string)recommendedActionValue2);
                                                                                providerErrorDetailsInstance.RecommendedAction = recommendedActionInstance2;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorLevelValue = errorsValue["errorLevel"];
                                                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                                        {
                                                                            string errorLevelInstance = ((string)errorLevelValue);
                                                                            jobErrorDetailsInstance.ErrorLevel = errorLevelInstance;
                                                                        }
                                                                        
                                                                        JToken creationTimeValue = errorsValue["creationTime"];
                                                                        if (creationTimeValue != null && creationTimeValue.Type != JTokenType.Null)
                                                                        {
                                                                            DateTime creationTimeInstance = ((DateTime)creationTimeValue);
                                                                            jobErrorDetailsInstance.CreationTime = creationTimeInstance;
                                                                        }
                                                                        
                                                                        JToken taskIdValue2 = errorsValue["taskId"];
                                                                        if (taskIdValue2 != null && taskIdValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string taskIdInstance2 = ((string)taskIdValue2);
                                                                            jobErrorDetailsInstance.TaskId = taskIdInstance2;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken customDetailsValue = childTasksValue["customDetails"];
                                                                if (customDetailsValue != null && customDetailsValue.Type != JTokenType.Null)
                                                                {
                                                                    string typeName2 = ((string)customDetailsValue["instanceType"]);
                                                                    if (typeName2 == "JobTaskDetails")
                                                                    {
                                                                        JobTaskDetails jobTaskDetailsInstance = new JobTaskDetails();
                                                                        
                                                                        JToken jobTaskValue = customDetailsValue["jobTask"];
                                                                        if (jobTaskValue != null && jobTaskValue.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance = new JobEntity();
                                                                            jobTaskDetailsInstance.JobTask = jobTaskInstance;
                                                                            
                                                                            JToken jobIdValue = jobTaskValue["jobId"];
                                                                            if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance = ((string)jobIdValue);
                                                                                jobTaskInstance.JobId = jobIdInstance;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue = jobTaskValue["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue != null && jobFriendlyNameValue.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance = ((string)jobFriendlyNameValue);
                                                                                jobTaskInstance.JobFriendlyName = jobFriendlyNameInstance;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue2 = jobTaskValue["targetObjectId"];
                                                                            if (targetObjectIdValue2 != null && targetObjectIdValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance2 = ((string)targetObjectIdValue2);
                                                                                jobTaskInstance.TargetObjectId = targetObjectIdInstance2;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue2 = jobTaskValue["targetObjectName"];
                                                                            if (targetObjectNameValue2 != null && targetObjectNameValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance2 = ((string)targetObjectNameValue2);
                                                                                jobTaskInstance.TargetObjectName = targetObjectNameInstance2;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue2 = jobTaskValue["targetInstanceType"];
                                                                            if (targetInstanceTypeValue2 != null && targetInstanceTypeValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance2 = ((string)targetInstanceTypeValue2);
                                                                                jobTaskInstance.TargetInstanceType = targetInstanceTypeInstance2;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue = jobTaskValue["jobScenarioName"];
                                                                            if (jobScenarioNameValue != null && jobScenarioNameValue.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance = ((string)jobScenarioNameValue);
                                                                                jobTaskInstance.JobScenarioName = jobScenarioNameInstance;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue2 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                                            jobTaskDetailsInstance.Type = instanceTypeInstance2;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = jobTaskDetailsInstance;
                                                                    }
                                                                    if (typeName2 == "VirtualMachineTaskDetails")
                                                                    {
                                                                        VirtualMachineTaskDetails virtualMachineTaskDetailsInstance = new VirtualMachineTaskDetails();
                                                                        
                                                                        JToken skippedReasonValue = customDetailsValue["skippedReason"];
                                                                        if (skippedReasonValue != null && skippedReasonValue.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonInstance = ((string)skippedReasonValue);
                                                                            virtualMachineTaskDetailsInstance.SkippedReason = skippedReasonInstance;
                                                                        }
                                                                        
                                                                        JToken skippedReasonStringValue = customDetailsValue["skippedReasonString"];
                                                                        if (skippedReasonStringValue != null && skippedReasonStringValue.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonStringInstance = ((string)skippedReasonStringValue);
                                                                            virtualMachineTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance;
                                                                        }
                                                                        
                                                                        JToken jobTaskValue2 = customDetailsValue["jobTask"];
                                                                        if (jobTaskValue2 != null && jobTaskValue2.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance2 = new JobEntity();
                                                                            virtualMachineTaskDetailsInstance.JobTask = jobTaskInstance2;
                                                                            
                                                                            JToken jobIdValue2 = jobTaskValue2["jobId"];
                                                                            if (jobIdValue2 != null && jobIdValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance2 = ((string)jobIdValue2);
                                                                                jobTaskInstance2.JobId = jobIdInstance2;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue2 = jobTaskValue2["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue2 != null && jobFriendlyNameValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance2 = ((string)jobFriendlyNameValue2);
                                                                                jobTaskInstance2.JobFriendlyName = jobFriendlyNameInstance2;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue3 = jobTaskValue2["targetObjectId"];
                                                                            if (targetObjectIdValue3 != null && targetObjectIdValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance3 = ((string)targetObjectIdValue3);
                                                                                jobTaskInstance2.TargetObjectId = targetObjectIdInstance3;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue3 = jobTaskValue2["targetObjectName"];
                                                                            if (targetObjectNameValue3 != null && targetObjectNameValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance3 = ((string)targetObjectNameValue3);
                                                                                jobTaskInstance2.TargetObjectName = targetObjectNameInstance3;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue3 = jobTaskValue2["targetInstanceType"];
                                                                            if (targetInstanceTypeValue3 != null && targetInstanceTypeValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance3 = ((string)targetInstanceTypeValue3);
                                                                                jobTaskInstance2.TargetInstanceType = targetInstanceTypeInstance3;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue2 = jobTaskValue2["jobScenarioName"];
                                                                            if (jobScenarioNameValue2 != null && jobScenarioNameValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance2 = ((string)jobScenarioNameValue2);
                                                                                jobTaskInstance2.JobScenarioName = jobScenarioNameInstance2;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue3 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                                            virtualMachineTaskDetailsInstance.Type = instanceTypeInstance3;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = virtualMachineTaskDetailsInstance;
                                                                    }
                                                                    if (typeName2 == "FabricReplicationGroupTaskDetails")
                                                                    {
                                                                        FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance = new FabricReplicationGroupTaskDetails();
                                                                        
                                                                        JToken skippedReasonValue2 = customDetailsValue["skippedReason"];
                                                                        if (skippedReasonValue2 != null && skippedReasonValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonInstance2 = ((string)skippedReasonValue2);
                                                                            fabricReplicationGroupTaskDetailsInstance.SkippedReason = skippedReasonInstance2;
                                                                        }
                                                                        
                                                                        JToken skippedReasonStringValue2 = customDetailsValue["skippedReasonString"];
                                                                        if (skippedReasonStringValue2 != null && skippedReasonStringValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonStringInstance2 = ((string)skippedReasonStringValue2);
                                                                            fabricReplicationGroupTaskDetailsInstance.SkippedReasonString = skippedReasonStringInstance2;
                                                                        }
                                                                        
                                                                        JToken jobTaskValue3 = customDetailsValue["jobTask"];
                                                                        if (jobTaskValue3 != null && jobTaskValue3.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance3 = new JobEntity();
                                                                            fabricReplicationGroupTaskDetailsInstance.JobTask = jobTaskInstance3;
                                                                            
                                                                            JToken jobIdValue3 = jobTaskValue3["jobId"];
                                                                            if (jobIdValue3 != null && jobIdValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance3 = ((string)jobIdValue3);
                                                                                jobTaskInstance3.JobId = jobIdInstance3;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue3 = jobTaskValue3["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue3 != null && jobFriendlyNameValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance3 = ((string)jobFriendlyNameValue3);
                                                                                jobTaskInstance3.JobFriendlyName = jobFriendlyNameInstance3;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue4 = jobTaskValue3["targetObjectId"];
                                                                            if (targetObjectIdValue4 != null && targetObjectIdValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance4 = ((string)targetObjectIdValue4);
                                                                                jobTaskInstance3.TargetObjectId = targetObjectIdInstance4;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue4 = jobTaskValue3["targetObjectName"];
                                                                            if (targetObjectNameValue4 != null && targetObjectNameValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance4 = ((string)targetObjectNameValue4);
                                                                                jobTaskInstance3.TargetObjectName = targetObjectNameInstance4;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue4 = jobTaskValue3["targetInstanceType"];
                                                                            if (targetInstanceTypeValue4 != null && targetInstanceTypeValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance4 = ((string)targetInstanceTypeValue4);
                                                                                jobTaskInstance3.TargetInstanceType = targetInstanceTypeInstance4;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue3 = jobTaskValue3["jobScenarioName"];
                                                                            if (jobScenarioNameValue3 != null && jobScenarioNameValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance3 = ((string)jobScenarioNameValue3);
                                                                                jobTaskInstance3.JobScenarioName = jobScenarioNameInstance3;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue4 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                                            fabricReplicationGroupTaskDetailsInstance.Type = instanceTypeInstance4;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance;
                                                                    }
                                                                    if (typeName2 == "ManualActionTaskDetails")
                                                                    {
                                                                        ManualActionTaskDetails manualActionTaskDetailsInstance = new ManualActionTaskDetails();
                                                                        
                                                                        JToken nameValue2 = customDetailsValue["name"];
                                                                        if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance2 = ((string)nameValue2);
                                                                            manualActionTaskDetailsInstance.Name = nameInstance2;
                                                                        }
                                                                        
                                                                        JToken instructionsValue = customDetailsValue["instructions"];
                                                                        if (instructionsValue != null && instructionsValue.Type != JTokenType.Null)
                                                                        {
                                                                            string instructionsInstance = ((string)instructionsValue);
                                                                            manualActionTaskDetailsInstance.Instructions = instructionsInstance;
                                                                        }
                                                                        
                                                                        JToken observationValue = customDetailsValue["observation"];
                                                                        if (observationValue != null && observationValue.Type != JTokenType.Null)
                                                                        {
                                                                            string observationInstance = ((string)observationValue);
                                                                            manualActionTaskDetailsInstance.Observation = observationInstance;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue5 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                                            manualActionTaskDetailsInstance.Type = instanceTypeInstance5;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = manualActionTaskDetailsInstance;
                                                                    }
                                                                    if (typeName2 == "ScriptActionTaskDetails")
                                                                    {
                                                                        ScriptActionTaskDetails scriptActionTaskDetailsInstance = new ScriptActionTaskDetails();
                                                                        
                                                                        JToken nameValue3 = customDetailsValue["name"];
                                                                        if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance3 = ((string)nameValue3);
                                                                            scriptActionTaskDetailsInstance.Name = nameInstance3;
                                                                        }
                                                                        
                                                                        JToken pathValue = customDetailsValue["path"];
                                                                        if (pathValue != null && pathValue.Type != JTokenType.Null)
                                                                        {
                                                                            string pathInstance = ((string)pathValue);
                                                                            scriptActionTaskDetailsInstance.Path = pathInstance;
                                                                        }
                                                                        
                                                                        JToken outputValue = customDetailsValue["output"];
                                                                        if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                                        {
                                                                            string outputInstance = ((string)outputValue);
                                                                            scriptActionTaskDetailsInstance.Output = outputInstance;
                                                                        }
                                                                        
                                                                        JToken isPrimarySideScriptValue = customDetailsValue["isPrimarySideScript"];
                                                                        if (isPrimarySideScriptValue != null && isPrimarySideScriptValue.Type != JTokenType.Null)
                                                                        {
                                                                            bool isPrimarySideScriptInstance = ((bool)isPrimarySideScriptValue);
                                                                            scriptActionTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue6 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                                            scriptActionTaskDetailsInstance.Type = instanceTypeInstance6;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = scriptActionTaskDetailsInstance;
                                                                    }
                                                                    if (typeName2 == "VmNicUpdatesTaskDetails")
                                                                    {
                                                                        VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance = new VmNicUpdatesTaskDetails();
                                                                        
                                                                        JToken vmIdValue = customDetailsValue["vmId"];
                                                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                                        {
                                                                            string vmIdInstance = ((string)vmIdValue);
                                                                            vmNicUpdatesTaskDetailsInstance.VmId = vmIdInstance;
                                                                        }
                                                                        
                                                                        JToken nicIdValue = customDetailsValue["nicId"];
                                                                        if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                                        {
                                                                            string nicIdInstance = ((string)nicIdValue);
                                                                            vmNicUpdatesTaskDetailsInstance.NicId = nicIdInstance;
                                                                        }
                                                                        
                                                                        JToken nameValue4 = customDetailsValue["name"];
                                                                        if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance4 = ((string)nameValue4);
                                                                            vmNicUpdatesTaskDetailsInstance.Name = nameInstance4;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue7 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue7 != null && instanceTypeValue7.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance7 = ((string)instanceTypeValue7);
                                                                            vmNicUpdatesTaskDetailsInstance.Type = instanceTypeInstance7;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance;
                                                                    }
                                                                    if (typeName2 == "ConsistencyCheckTaskDetails")
                                                                    {
                                                                        ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance = new ConsistencyCheckTaskDetails();
                                                                        
                                                                        JToken vmDetailsArray = customDetailsValue["vmDetails"];
                                                                        if (vmDetailsArray != null && vmDetailsArray.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken vmDetailsValue in ((JArray)vmDetailsArray))
                                                                            {
                                                                                InconsistentVmDetails inconsistentVmDetailsInstance = new InconsistentVmDetails();
                                                                                consistencyCheckTaskDetailsInstance.VmDetails.Add(inconsistentVmDetailsInstance);
                                                                                
                                                                                JToken vmNameValue = vmDetailsValue["vmName"];
                                                                                if (vmNameValue != null && vmNameValue.Type != JTokenType.Null)
                                                                                {
                                                                                    string vmNameInstance = ((string)vmNameValue);
                                                                                    inconsistentVmDetailsInstance.VmName = vmNameInstance;
                                                                                }
                                                                                
                                                                                JToken cloudNameValue = vmDetailsValue["cloudName"];
                                                                                if (cloudNameValue != null && cloudNameValue.Type != JTokenType.Null)
                                                                                {
                                                                                    string cloudNameInstance = ((string)cloudNameValue);
                                                                                    inconsistentVmDetailsInstance.CloudName = cloudNameInstance;
                                                                                }
                                                                                
                                                                                JToken detailsArray = vmDetailsValue["details"];
                                                                                if (detailsArray != null && detailsArray.Type != JTokenType.Null)
                                                                                {
                                                                                    foreach (JToken detailsValue in ((JArray)detailsArray))
                                                                                    {
                                                                                        inconsistentVmDetailsInstance.Details.Add(((string)detailsValue));
                                                                                    }
                                                                                }
                                                                                
                                                                                JToken errorIdsArray = vmDetailsValue["errorIds"];
                                                                                if (errorIdsArray != null && errorIdsArray.Type != JTokenType.Null)
                                                                                {
                                                                                    foreach (JToken errorIdsValue in ((JArray)errorIdsArray))
                                                                                    {
                                                                                        inconsistentVmDetailsInstance.ErrorIds.Add(((string)errorIdsValue));
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue8 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue8 != null && instanceTypeValue8.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance8 = ((string)instanceTypeValue8);
                                                                            consistencyCheckTaskDetailsInstance.Type = instanceTypeInstance8;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = consistencyCheckTaskDetailsInstance;
                                                                    }
                                                                    if (typeName2 == "AutomationRunbookTaskDetails")
                                                                    {
                                                                        AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance = new AutomationRunbookTaskDetails();
                                                                        
                                                                        JToken nameValue5 = customDetailsValue["name"];
                                                                        if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance5 = ((string)nameValue5);
                                                                            automationRunbookTaskDetailsInstance.Name = nameInstance5;
                                                                        }
                                                                        
                                                                        JToken cloudServiceNameValue = customDetailsValue["cloudServiceName"];
                                                                        if (cloudServiceNameValue != null && cloudServiceNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudServiceNameInstance = ((string)cloudServiceNameValue);
                                                                            automationRunbookTaskDetailsInstance.CloudServiceName = cloudServiceNameInstance;
                                                                        }
                                                                        
                                                                        JToken subscriptionIdValue = customDetailsValue["subscriptionId"];
                                                                        if (subscriptionIdValue != null && subscriptionIdValue.Type != JTokenType.Null)
                                                                        {
                                                                            string subscriptionIdInstance = ((string)subscriptionIdValue);
                                                                            automationRunbookTaskDetailsInstance.SubscriptionId = subscriptionIdInstance;
                                                                        }
                                                                        
                                                                        JToken accountNameValue = customDetailsValue["accountName"];
                                                                        if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string accountNameInstance = ((string)accountNameValue);
                                                                            automationRunbookTaskDetailsInstance.AccountName = accountNameInstance;
                                                                        }
                                                                        
                                                                        JToken runbookIdValue = customDetailsValue["runbookId"];
                                                                        if (runbookIdValue != null && runbookIdValue.Type != JTokenType.Null)
                                                                        {
                                                                            string runbookIdInstance = ((string)runbookIdValue);
                                                                            automationRunbookTaskDetailsInstance.RunbookId = runbookIdInstance;
                                                                        }
                                                                        
                                                                        JToken runbookNameValue = customDetailsValue["runbookName"];
                                                                        if (runbookNameValue != null && runbookNameValue.Type != JTokenType.Null)
                                                                        {
                                                                            string runbookNameInstance = ((string)runbookNameValue);
                                                                            automationRunbookTaskDetailsInstance.RunbookName = runbookNameInstance;
                                                                        }
                                                                        
                                                                        JToken jobIdValue4 = customDetailsValue["jobId"];
                                                                        if (jobIdValue4 != null && jobIdValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string jobIdInstance4 = ((string)jobIdValue4);
                                                                            automationRunbookTaskDetailsInstance.JobId = jobIdInstance4;
                                                                        }
                                                                        
                                                                        JToken jobOutputValue = customDetailsValue["jobOutput"];
                                                                        if (jobOutputValue != null && jobOutputValue.Type != JTokenType.Null)
                                                                        {
                                                                            string jobOutputInstance = ((string)jobOutputValue);
                                                                            automationRunbookTaskDetailsInstance.JobOutput = jobOutputInstance;
                                                                        }
                                                                        
                                                                        JToken isPrimarySideScriptValue2 = customDetailsValue["isPrimarySideScript"];
                                                                        if (isPrimarySideScriptValue2 != null && isPrimarySideScriptValue2.Type != JTokenType.Null)
                                                                        {
                                                                            bool isPrimarySideScriptInstance2 = ((bool)isPrimarySideScriptValue2);
                                                                            automationRunbookTaskDetailsInstance.IsPrimarySideScript = isPrimarySideScriptInstance2;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue9 = customDetailsValue["instanceType"];
                                                                        if (instanceTypeValue9 != null && instanceTypeValue9.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance9 = ((string)instanceTypeValue9);
                                                                            automationRunbookTaskDetailsInstance.Type = instanceTypeInstance9;
                                                                        }
                                                                        taskBaseInstance.CustomDetails = automationRunbookTaskDetailsInstance;
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        taskInstance.GroupTaskCustomDetails = inlineWorkflowTaskDetailsInstance;
                                                    }
                                                    if (typeName == "RecoveryPlanGroupTaskDetails")
                                                    {
                                                        RecoveryPlanGroupTaskDetails recoveryPlanGroupTaskDetailsInstance = new RecoveryPlanGroupTaskDetails();
                                                        
                                                        JToken nameValue6 = groupTaskCustomDetailsValue["name"];
                                                        if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance6 = ((string)nameValue6);
                                                            recoveryPlanGroupTaskDetailsInstance.Name = nameInstance6;
                                                        }
                                                        
                                                        JToken groupIdValue = groupTaskCustomDetailsValue["groupId"];
                                                        if (groupIdValue != null && groupIdValue.Type != JTokenType.Null)
                                                        {
                                                            string groupIdInstance = ((string)groupIdValue);
                                                            recoveryPlanGroupTaskDetailsInstance.GroupId = groupIdInstance;
                                                        }
                                                        
                                                        JToken rpGroupTypeValue = groupTaskCustomDetailsValue["rpGroupType"];
                                                        if (rpGroupTypeValue != null && rpGroupTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string rpGroupTypeInstance = ((string)rpGroupTypeValue);
                                                            recoveryPlanGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance;
                                                        }
                                                        
                                                        JToken instanceTypeValue10 = groupTaskCustomDetailsValue["instanceType"];
                                                        if (instanceTypeValue10 != null && instanceTypeValue10.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance10 = ((string)instanceTypeValue10);
                                                            recoveryPlanGroupTaskDetailsInstance.Type = instanceTypeInstance10;
                                                        }
                                                        
                                                        JToken childTasksArray2 = groupTaskCustomDetailsValue["childTasks"];
                                                        if (childTasksArray2 != null && childTasksArray2.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken childTasksValue2 in ((JArray)childTasksArray2))
                                                            {
                                                                AsrTaskBase taskBaseInstance2 = new AsrTaskBase();
                                                                recoveryPlanGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance2);
                                                                
                                                                JToken taskIdValue3 = childTasksValue2["taskId"];
                                                                if (taskIdValue3 != null && taskIdValue3.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance3 = ((string)taskIdValue3);
                                                                    taskBaseInstance2.ID = taskIdInstance3;
                                                                }
                                                                
                                                                JToken nameValue7 = childTasksValue2["name"];
                                                                if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance7 = ((string)nameValue7);
                                                                    taskBaseInstance2.Name = nameInstance7;
                                                                }
                                                                
                                                                JToken startTimeValue3 = childTasksValue2["startTime"];
                                                                if (startTimeValue3 != null && startTimeValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime startTimeInstance3 = ((DateTime)startTimeValue3);
                                                                    taskBaseInstance2.StartTime = startTimeInstance3;
                                                                }
                                                                
                                                                JToken endTimeValue3 = childTasksValue2["endTime"];
                                                                if (endTimeValue3 != null && endTimeValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime endTimeInstance3 = ((DateTime)endTimeValue3);
                                                                    taskBaseInstance2.EndTime = endTimeInstance3;
                                                                }
                                                                
                                                                JToken allowedActionsArray3 = childTasksValue2["allowedActions"];
                                                                if (allowedActionsArray3 != null && allowedActionsArray3.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken allowedActionsValue3 in ((JArray)allowedActionsArray3))
                                                                    {
                                                                        taskBaseInstance2.Actions.Add(((string)allowedActionsValue3));
                                                                    }
                                                                }
                                                                
                                                                JToken friendlyNameValue3 = childTasksValue2["friendlyName"];
                                                                if (friendlyNameValue3 != null && friendlyNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string friendlyNameInstance3 = ((string)friendlyNameValue3);
                                                                    taskBaseInstance2.TaskFriendlyName = friendlyNameInstance3;
                                                                }
                                                                
                                                                JToken stateValue3 = childTasksValue2["state"];
                                                                if (stateValue3 != null && stateValue3.Type != JTokenType.Null)
                                                                {
                                                                    string stateInstance3 = ((string)stateValue3);
                                                                    taskBaseInstance2.State = stateInstance3;
                                                                }
                                                                
                                                                JToken stateDescriptionValue3 = childTasksValue2["stateDescription"];
                                                                if (stateDescriptionValue3 != null && stateDescriptionValue3.Type != JTokenType.Null)
                                                                {
                                                                    string stateDescriptionInstance3 = ((string)stateDescriptionValue3);
                                                                    taskBaseInstance2.StateDescription = stateDescriptionInstance3;
                                                                }
                                                                
                                                                JToken taskTypeValue2 = childTasksValue2["taskType"];
                                                                if (taskTypeValue2 != null && taskTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string taskTypeInstance2 = ((string)taskTypeValue2);
                                                                    taskBaseInstance2.TaskType = taskTypeInstance2;
                                                                }
                                                                
                                                                JToken errorsArray2 = childTasksValue2["errors"];
                                                                if (errorsArray2 != null && errorsArray2.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken errorsValue2 in ((JArray)errorsArray2))
                                                                    {
                                                                        JobErrorDetails jobErrorDetailsInstance2 = new JobErrorDetails();
                                                                        taskBaseInstance2.Errors.Add(jobErrorDetailsInstance2);
                                                                        
                                                                        JToken serviceErrorDetailsValue2 = errorsValue2["serviceErrorDetails"];
                                                                        if (serviceErrorDetailsValue2 != null && serviceErrorDetailsValue2.Type != JTokenType.Null)
                                                                        {
                                                                            ServiceError serviceErrorDetailsInstance2 = new ServiceError();
                                                                            jobErrorDetailsInstance2.ServiceErrorDetails = serviceErrorDetailsInstance2;
                                                                            
                                                                            JToken codeValue2 = serviceErrorDetailsValue2["code"];
                                                                            if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string codeInstance2 = ((string)codeValue2);
                                                                                serviceErrorDetailsInstance2.Code = codeInstance2;
                                                                            }
                                                                            
                                                                            JToken messageValue2 = serviceErrorDetailsValue2["message"];
                                                                            if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string messageInstance2 = ((string)messageValue2);
                                                                                serviceErrorDetailsInstance2.Message = messageInstance2;
                                                                            }
                                                                            
                                                                            JToken possibleCausesValue3 = serviceErrorDetailsValue2["possibleCauses"];
                                                                            if (possibleCausesValue3 != null && possibleCausesValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string possibleCausesInstance3 = ((string)possibleCausesValue3);
                                                                                serviceErrorDetailsInstance2.PossibleCauses = possibleCausesInstance3;
                                                                            }
                                                                            
                                                                            JToken recommendedActionValue3 = serviceErrorDetailsValue2["recommendedAction"];
                                                                            if (recommendedActionValue3 != null && recommendedActionValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string recommendedActionInstance3 = ((string)recommendedActionValue3);
                                                                                serviceErrorDetailsInstance2.RecommendedAction = recommendedActionInstance3;
                                                                            }
                                                                            
                                                                            JToken activityIdValue3 = serviceErrorDetailsValue2["activityId"];
                                                                            if (activityIdValue3 != null && activityIdValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string activityIdInstance3 = ((string)activityIdValue3);
                                                                                serviceErrorDetailsInstance2.ActivityId = activityIdInstance3;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken providerErrorDetailsValue2 = errorsValue2["providerErrorDetails"];
                                                                        if (providerErrorDetailsValue2 != null && providerErrorDetailsValue2.Type != JTokenType.Null)
                                                                        {
                                                                            ProviderError providerErrorDetailsInstance2 = new ProviderError();
                                                                            jobErrorDetailsInstance2.ProviderErrorDetails = providerErrorDetailsInstance2;
                                                                            
                                                                            JToken errorCodeValue2 = providerErrorDetailsValue2["errorCode"];
                                                                            if (errorCodeValue2 != null && errorCodeValue2.Type != JTokenType.Null)
                                                                            {
                                                                                int errorCodeInstance2 = ((int)errorCodeValue2);
                                                                                providerErrorDetailsInstance2.ErrorCode = errorCodeInstance2;
                                                                            }
                                                                            
                                                                            JToken errorMessageValue2 = providerErrorDetailsValue2["errorMessage"];
                                                                            if (errorMessageValue2 != null && errorMessageValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string errorMessageInstance2 = ((string)errorMessageValue2);
                                                                                providerErrorDetailsInstance2.ErrorMessage = errorMessageInstance2;
                                                                            }
                                                                            
                                                                            JToken errorIdValue2 = providerErrorDetailsValue2["errorId"];
                                                                            if (errorIdValue2 != null && errorIdValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string errorIdInstance2 = ((string)errorIdValue2);
                                                                                providerErrorDetailsInstance2.ErrorId = errorIdInstance2;
                                                                            }
                                                                            
                                                                            JToken possibleCausesValue4 = providerErrorDetailsValue2["possibleCauses"];
                                                                            if (possibleCausesValue4 != null && possibleCausesValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string possibleCausesInstance4 = ((string)possibleCausesValue4);
                                                                                providerErrorDetailsInstance2.PossibleCauses = possibleCausesInstance4;
                                                                            }
                                                                            
                                                                            JToken recommendedActionValue4 = providerErrorDetailsValue2["recommendedAction"];
                                                                            if (recommendedActionValue4 != null && recommendedActionValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string recommendedActionInstance4 = ((string)recommendedActionValue4);
                                                                                providerErrorDetailsInstance2.RecommendedAction = recommendedActionInstance4;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorLevelValue2 = errorsValue2["errorLevel"];
                                                                        if (errorLevelValue2 != null && errorLevelValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string errorLevelInstance2 = ((string)errorLevelValue2);
                                                                            jobErrorDetailsInstance2.ErrorLevel = errorLevelInstance2;
                                                                        }
                                                                        
                                                                        JToken creationTimeValue2 = errorsValue2["creationTime"];
                                                                        if (creationTimeValue2 != null && creationTimeValue2.Type != JTokenType.Null)
                                                                        {
                                                                            DateTime creationTimeInstance2 = ((DateTime)creationTimeValue2);
                                                                            jobErrorDetailsInstance2.CreationTime = creationTimeInstance2;
                                                                        }
                                                                        
                                                                        JToken taskIdValue4 = errorsValue2["taskId"];
                                                                        if (taskIdValue4 != null && taskIdValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string taskIdInstance4 = ((string)taskIdValue4);
                                                                            jobErrorDetailsInstance2.TaskId = taskIdInstance4;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken customDetailsValue2 = childTasksValue2["customDetails"];
                                                                if (customDetailsValue2 != null && customDetailsValue2.Type != JTokenType.Null)
                                                                {
                                                                    string typeName3 = ((string)customDetailsValue2["instanceType"]);
                                                                    if (typeName3 == "JobTaskDetails")
                                                                    {
                                                                        JobTaskDetails jobTaskDetailsInstance2 = new JobTaskDetails();
                                                                        
                                                                        JToken jobTaskValue4 = customDetailsValue2["jobTask"];
                                                                        if (jobTaskValue4 != null && jobTaskValue4.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance4 = new JobEntity();
                                                                            jobTaskDetailsInstance2.JobTask = jobTaskInstance4;
                                                                            
                                                                            JToken jobIdValue5 = jobTaskValue4["jobId"];
                                                                            if (jobIdValue5 != null && jobIdValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance5 = ((string)jobIdValue5);
                                                                                jobTaskInstance4.JobId = jobIdInstance5;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue4 = jobTaskValue4["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue4 != null && jobFriendlyNameValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance4 = ((string)jobFriendlyNameValue4);
                                                                                jobTaskInstance4.JobFriendlyName = jobFriendlyNameInstance4;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue5 = jobTaskValue4["targetObjectId"];
                                                                            if (targetObjectIdValue5 != null && targetObjectIdValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance5 = ((string)targetObjectIdValue5);
                                                                                jobTaskInstance4.TargetObjectId = targetObjectIdInstance5;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue5 = jobTaskValue4["targetObjectName"];
                                                                            if (targetObjectNameValue5 != null && targetObjectNameValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance5 = ((string)targetObjectNameValue5);
                                                                                jobTaskInstance4.TargetObjectName = targetObjectNameInstance5;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue5 = jobTaskValue4["targetInstanceType"];
                                                                            if (targetInstanceTypeValue5 != null && targetInstanceTypeValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance5 = ((string)targetInstanceTypeValue5);
                                                                                jobTaskInstance4.TargetInstanceType = targetInstanceTypeInstance5;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue4 = jobTaskValue4["jobScenarioName"];
                                                                            if (jobScenarioNameValue4 != null && jobScenarioNameValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance4 = ((string)jobScenarioNameValue4);
                                                                                jobTaskInstance4.JobScenarioName = jobScenarioNameInstance4;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue11 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue11 != null && instanceTypeValue11.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance11 = ((string)instanceTypeValue11);
                                                                            jobTaskDetailsInstance2.Type = instanceTypeInstance11;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = jobTaskDetailsInstance2;
                                                                    }
                                                                    if (typeName3 == "VirtualMachineTaskDetails")
                                                                    {
                                                                        VirtualMachineTaskDetails virtualMachineTaskDetailsInstance2 = new VirtualMachineTaskDetails();
                                                                        
                                                                        JToken skippedReasonValue3 = customDetailsValue2["skippedReason"];
                                                                        if (skippedReasonValue3 != null && skippedReasonValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonInstance3 = ((string)skippedReasonValue3);
                                                                            virtualMachineTaskDetailsInstance2.SkippedReason = skippedReasonInstance3;
                                                                        }
                                                                        
                                                                        JToken skippedReasonStringValue3 = customDetailsValue2["skippedReasonString"];
                                                                        if (skippedReasonStringValue3 != null && skippedReasonStringValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonStringInstance3 = ((string)skippedReasonStringValue3);
                                                                            virtualMachineTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance3;
                                                                        }
                                                                        
                                                                        JToken jobTaskValue5 = customDetailsValue2["jobTask"];
                                                                        if (jobTaskValue5 != null && jobTaskValue5.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance5 = new JobEntity();
                                                                            virtualMachineTaskDetailsInstance2.JobTask = jobTaskInstance5;
                                                                            
                                                                            JToken jobIdValue6 = jobTaskValue5["jobId"];
                                                                            if (jobIdValue6 != null && jobIdValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance6 = ((string)jobIdValue6);
                                                                                jobTaskInstance5.JobId = jobIdInstance6;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue5 = jobTaskValue5["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue5 != null && jobFriendlyNameValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance5 = ((string)jobFriendlyNameValue5);
                                                                                jobTaskInstance5.JobFriendlyName = jobFriendlyNameInstance5;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue6 = jobTaskValue5["targetObjectId"];
                                                                            if (targetObjectIdValue6 != null && targetObjectIdValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance6 = ((string)targetObjectIdValue6);
                                                                                jobTaskInstance5.TargetObjectId = targetObjectIdInstance6;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue6 = jobTaskValue5["targetObjectName"];
                                                                            if (targetObjectNameValue6 != null && targetObjectNameValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance6 = ((string)targetObjectNameValue6);
                                                                                jobTaskInstance5.TargetObjectName = targetObjectNameInstance6;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue6 = jobTaskValue5["targetInstanceType"];
                                                                            if (targetInstanceTypeValue6 != null && targetInstanceTypeValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance6 = ((string)targetInstanceTypeValue6);
                                                                                jobTaskInstance5.TargetInstanceType = targetInstanceTypeInstance6;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue5 = jobTaskValue5["jobScenarioName"];
                                                                            if (jobScenarioNameValue5 != null && jobScenarioNameValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance5 = ((string)jobScenarioNameValue5);
                                                                                jobTaskInstance5.JobScenarioName = jobScenarioNameInstance5;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue12 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue12 != null && instanceTypeValue12.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance12 = ((string)instanceTypeValue12);
                                                                            virtualMachineTaskDetailsInstance2.Type = instanceTypeInstance12;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = virtualMachineTaskDetailsInstance2;
                                                                    }
                                                                    if (typeName3 == "FabricReplicationGroupTaskDetails")
                                                                    {
                                                                        FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance2 = new FabricReplicationGroupTaskDetails();
                                                                        
                                                                        JToken skippedReasonValue4 = customDetailsValue2["skippedReason"];
                                                                        if (skippedReasonValue4 != null && skippedReasonValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonInstance4 = ((string)skippedReasonValue4);
                                                                            fabricReplicationGroupTaskDetailsInstance2.SkippedReason = skippedReasonInstance4;
                                                                        }
                                                                        
                                                                        JToken skippedReasonStringValue4 = customDetailsValue2["skippedReasonString"];
                                                                        if (skippedReasonStringValue4 != null && skippedReasonStringValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonStringInstance4 = ((string)skippedReasonStringValue4);
                                                                            fabricReplicationGroupTaskDetailsInstance2.SkippedReasonString = skippedReasonStringInstance4;
                                                                        }
                                                                        
                                                                        JToken jobTaskValue6 = customDetailsValue2["jobTask"];
                                                                        if (jobTaskValue6 != null && jobTaskValue6.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance6 = new JobEntity();
                                                                            fabricReplicationGroupTaskDetailsInstance2.JobTask = jobTaskInstance6;
                                                                            
                                                                            JToken jobIdValue7 = jobTaskValue6["jobId"];
                                                                            if (jobIdValue7 != null && jobIdValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance7 = ((string)jobIdValue7);
                                                                                jobTaskInstance6.JobId = jobIdInstance7;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue6 = jobTaskValue6["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue6 != null && jobFriendlyNameValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance6 = ((string)jobFriendlyNameValue6);
                                                                                jobTaskInstance6.JobFriendlyName = jobFriendlyNameInstance6;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue7 = jobTaskValue6["targetObjectId"];
                                                                            if (targetObjectIdValue7 != null && targetObjectIdValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance7 = ((string)targetObjectIdValue7);
                                                                                jobTaskInstance6.TargetObjectId = targetObjectIdInstance7;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue7 = jobTaskValue6["targetObjectName"];
                                                                            if (targetObjectNameValue7 != null && targetObjectNameValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance7 = ((string)targetObjectNameValue7);
                                                                                jobTaskInstance6.TargetObjectName = targetObjectNameInstance7;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue7 = jobTaskValue6["targetInstanceType"];
                                                                            if (targetInstanceTypeValue7 != null && targetInstanceTypeValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance7 = ((string)targetInstanceTypeValue7);
                                                                                jobTaskInstance6.TargetInstanceType = targetInstanceTypeInstance7;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue6 = jobTaskValue6["jobScenarioName"];
                                                                            if (jobScenarioNameValue6 != null && jobScenarioNameValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance6 = ((string)jobScenarioNameValue6);
                                                                                jobTaskInstance6.JobScenarioName = jobScenarioNameInstance6;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue13 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue13 != null && instanceTypeValue13.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance13 = ((string)instanceTypeValue13);
                                                                            fabricReplicationGroupTaskDetailsInstance2.Type = instanceTypeInstance13;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = fabricReplicationGroupTaskDetailsInstance2;
                                                                    }
                                                                    if (typeName3 == "ManualActionTaskDetails")
                                                                    {
                                                                        ManualActionTaskDetails manualActionTaskDetailsInstance2 = new ManualActionTaskDetails();
                                                                        
                                                                        JToken nameValue8 = customDetailsValue2["name"];
                                                                        if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance8 = ((string)nameValue8);
                                                                            manualActionTaskDetailsInstance2.Name = nameInstance8;
                                                                        }
                                                                        
                                                                        JToken instructionsValue2 = customDetailsValue2["instructions"];
                                                                        if (instructionsValue2 != null && instructionsValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string instructionsInstance2 = ((string)instructionsValue2);
                                                                            manualActionTaskDetailsInstance2.Instructions = instructionsInstance2;
                                                                        }
                                                                        
                                                                        JToken observationValue2 = customDetailsValue2["observation"];
                                                                        if (observationValue2 != null && observationValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string observationInstance2 = ((string)observationValue2);
                                                                            manualActionTaskDetailsInstance2.Observation = observationInstance2;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue14 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue14 != null && instanceTypeValue14.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance14 = ((string)instanceTypeValue14);
                                                                            manualActionTaskDetailsInstance2.Type = instanceTypeInstance14;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = manualActionTaskDetailsInstance2;
                                                                    }
                                                                    if (typeName3 == "ScriptActionTaskDetails")
                                                                    {
                                                                        ScriptActionTaskDetails scriptActionTaskDetailsInstance2 = new ScriptActionTaskDetails();
                                                                        
                                                                        JToken nameValue9 = customDetailsValue2["name"];
                                                                        if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance9 = ((string)nameValue9);
                                                                            scriptActionTaskDetailsInstance2.Name = nameInstance9;
                                                                        }
                                                                        
                                                                        JToken pathValue2 = customDetailsValue2["path"];
                                                                        if (pathValue2 != null && pathValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string pathInstance2 = ((string)pathValue2);
                                                                            scriptActionTaskDetailsInstance2.Path = pathInstance2;
                                                                        }
                                                                        
                                                                        JToken outputValue2 = customDetailsValue2["output"];
                                                                        if (outputValue2 != null && outputValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string outputInstance2 = ((string)outputValue2);
                                                                            scriptActionTaskDetailsInstance2.Output = outputInstance2;
                                                                        }
                                                                        
                                                                        JToken isPrimarySideScriptValue3 = customDetailsValue2["isPrimarySideScript"];
                                                                        if (isPrimarySideScriptValue3 != null && isPrimarySideScriptValue3.Type != JTokenType.Null)
                                                                        {
                                                                            bool isPrimarySideScriptInstance3 = ((bool)isPrimarySideScriptValue3);
                                                                            scriptActionTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance3;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue15 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue15 != null && instanceTypeValue15.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance15 = ((string)instanceTypeValue15);
                                                                            scriptActionTaskDetailsInstance2.Type = instanceTypeInstance15;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = scriptActionTaskDetailsInstance2;
                                                                    }
                                                                    if (typeName3 == "VmNicUpdatesTaskDetails")
                                                                    {
                                                                        VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance2 = new VmNicUpdatesTaskDetails();
                                                                        
                                                                        JToken vmIdValue2 = customDetailsValue2["vmId"];
                                                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string vmIdInstance2 = ((string)vmIdValue2);
                                                                            vmNicUpdatesTaskDetailsInstance2.VmId = vmIdInstance2;
                                                                        }
                                                                        
                                                                        JToken nicIdValue2 = customDetailsValue2["nicId"];
                                                                        if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string nicIdInstance2 = ((string)nicIdValue2);
                                                                            vmNicUpdatesTaskDetailsInstance2.NicId = nicIdInstance2;
                                                                        }
                                                                        
                                                                        JToken nameValue10 = customDetailsValue2["name"];
                                                                        if (nameValue10 != null && nameValue10.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance10 = ((string)nameValue10);
                                                                            vmNicUpdatesTaskDetailsInstance2.Name = nameInstance10;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue16 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue16 != null && instanceTypeValue16.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance16 = ((string)instanceTypeValue16);
                                                                            vmNicUpdatesTaskDetailsInstance2.Type = instanceTypeInstance16;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = vmNicUpdatesTaskDetailsInstance2;
                                                                    }
                                                                    if (typeName3 == "ConsistencyCheckTaskDetails")
                                                                    {
                                                                        ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance2 = new ConsistencyCheckTaskDetails();
                                                                        
                                                                        JToken vmDetailsArray2 = customDetailsValue2["vmDetails"];
                                                                        if (vmDetailsArray2 != null && vmDetailsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken vmDetailsValue2 in ((JArray)vmDetailsArray2))
                                                                            {
                                                                                InconsistentVmDetails inconsistentVmDetailsInstance2 = new InconsistentVmDetails();
                                                                                consistencyCheckTaskDetailsInstance2.VmDetails.Add(inconsistentVmDetailsInstance2);
                                                                                
                                                                                JToken vmNameValue2 = vmDetailsValue2["vmName"];
                                                                                if (vmNameValue2 != null && vmNameValue2.Type != JTokenType.Null)
                                                                                {
                                                                                    string vmNameInstance2 = ((string)vmNameValue2);
                                                                                    inconsistentVmDetailsInstance2.VmName = vmNameInstance2;
                                                                                }
                                                                                
                                                                                JToken cloudNameValue2 = vmDetailsValue2["cloudName"];
                                                                                if (cloudNameValue2 != null && cloudNameValue2.Type != JTokenType.Null)
                                                                                {
                                                                                    string cloudNameInstance2 = ((string)cloudNameValue2);
                                                                                    inconsistentVmDetailsInstance2.CloudName = cloudNameInstance2;
                                                                                }
                                                                                
                                                                                JToken detailsArray2 = vmDetailsValue2["details"];
                                                                                if (detailsArray2 != null && detailsArray2.Type != JTokenType.Null)
                                                                                {
                                                                                    foreach (JToken detailsValue2 in ((JArray)detailsArray2))
                                                                                    {
                                                                                        inconsistentVmDetailsInstance2.Details.Add(((string)detailsValue2));
                                                                                    }
                                                                                }
                                                                                
                                                                                JToken errorIdsArray2 = vmDetailsValue2["errorIds"];
                                                                                if (errorIdsArray2 != null && errorIdsArray2.Type != JTokenType.Null)
                                                                                {
                                                                                    foreach (JToken errorIdsValue2 in ((JArray)errorIdsArray2))
                                                                                    {
                                                                                        inconsistentVmDetailsInstance2.ErrorIds.Add(((string)errorIdsValue2));
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue17 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue17 != null && instanceTypeValue17.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance17 = ((string)instanceTypeValue17);
                                                                            consistencyCheckTaskDetailsInstance2.Type = instanceTypeInstance17;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = consistencyCheckTaskDetailsInstance2;
                                                                    }
                                                                    if (typeName3 == "AutomationRunbookTaskDetails")
                                                                    {
                                                                        AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance2 = new AutomationRunbookTaskDetails();
                                                                        
                                                                        JToken nameValue11 = customDetailsValue2["name"];
                                                                        if (nameValue11 != null && nameValue11.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance11 = ((string)nameValue11);
                                                                            automationRunbookTaskDetailsInstance2.Name = nameInstance11;
                                                                        }
                                                                        
                                                                        JToken cloudServiceNameValue2 = customDetailsValue2["cloudServiceName"];
                                                                        if (cloudServiceNameValue2 != null && cloudServiceNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudServiceNameInstance2 = ((string)cloudServiceNameValue2);
                                                                            automationRunbookTaskDetailsInstance2.CloudServiceName = cloudServiceNameInstance2;
                                                                        }
                                                                        
                                                                        JToken subscriptionIdValue2 = customDetailsValue2["subscriptionId"];
                                                                        if (subscriptionIdValue2 != null && subscriptionIdValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string subscriptionIdInstance2 = ((string)subscriptionIdValue2);
                                                                            automationRunbookTaskDetailsInstance2.SubscriptionId = subscriptionIdInstance2;
                                                                        }
                                                                        
                                                                        JToken accountNameValue2 = customDetailsValue2["accountName"];
                                                                        if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string accountNameInstance2 = ((string)accountNameValue2);
                                                                            automationRunbookTaskDetailsInstance2.AccountName = accountNameInstance2;
                                                                        }
                                                                        
                                                                        JToken runbookIdValue2 = customDetailsValue2["runbookId"];
                                                                        if (runbookIdValue2 != null && runbookIdValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string runbookIdInstance2 = ((string)runbookIdValue2);
                                                                            automationRunbookTaskDetailsInstance2.RunbookId = runbookIdInstance2;
                                                                        }
                                                                        
                                                                        JToken runbookNameValue2 = customDetailsValue2["runbookName"];
                                                                        if (runbookNameValue2 != null && runbookNameValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string runbookNameInstance2 = ((string)runbookNameValue2);
                                                                            automationRunbookTaskDetailsInstance2.RunbookName = runbookNameInstance2;
                                                                        }
                                                                        
                                                                        JToken jobIdValue8 = customDetailsValue2["jobId"];
                                                                        if (jobIdValue8 != null && jobIdValue8.Type != JTokenType.Null)
                                                                        {
                                                                            string jobIdInstance8 = ((string)jobIdValue8);
                                                                            automationRunbookTaskDetailsInstance2.JobId = jobIdInstance8;
                                                                        }
                                                                        
                                                                        JToken jobOutputValue2 = customDetailsValue2["jobOutput"];
                                                                        if (jobOutputValue2 != null && jobOutputValue2.Type != JTokenType.Null)
                                                                        {
                                                                            string jobOutputInstance2 = ((string)jobOutputValue2);
                                                                            automationRunbookTaskDetailsInstance2.JobOutput = jobOutputInstance2;
                                                                        }
                                                                        
                                                                        JToken isPrimarySideScriptValue4 = customDetailsValue2["isPrimarySideScript"];
                                                                        if (isPrimarySideScriptValue4 != null && isPrimarySideScriptValue4.Type != JTokenType.Null)
                                                                        {
                                                                            bool isPrimarySideScriptInstance4 = ((bool)isPrimarySideScriptValue4);
                                                                            automationRunbookTaskDetailsInstance2.IsPrimarySideScript = isPrimarySideScriptInstance4;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue18 = customDetailsValue2["instanceType"];
                                                                        if (instanceTypeValue18 != null && instanceTypeValue18.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance18 = ((string)instanceTypeValue18);
                                                                            automationRunbookTaskDetailsInstance2.Type = instanceTypeInstance18;
                                                                        }
                                                                        taskBaseInstance2.CustomDetails = automationRunbookTaskDetailsInstance2;
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        taskInstance.GroupTaskCustomDetails = recoveryPlanGroupTaskDetailsInstance;
                                                    }
                                                    if (typeName == "RecoveryPlanShutdownGroupTaskDetails")
                                                    {
                                                        RecoveryPlanShutdownGroupTaskDetails recoveryPlanShutdownGroupTaskDetailsInstance = new RecoveryPlanShutdownGroupTaskDetails();
                                                        
                                                        JToken nameValue12 = groupTaskCustomDetailsValue["name"];
                                                        if (nameValue12 != null && nameValue12.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance12 = ((string)nameValue12);
                                                            recoveryPlanShutdownGroupTaskDetailsInstance.Name = nameInstance12;
                                                        }
                                                        
                                                        JToken groupIdValue2 = groupTaskCustomDetailsValue["groupId"];
                                                        if (groupIdValue2 != null && groupIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string groupIdInstance2 = ((string)groupIdValue2);
                                                            recoveryPlanShutdownGroupTaskDetailsInstance.GroupId = groupIdInstance2;
                                                        }
                                                        
                                                        JToken rpGroupTypeValue2 = groupTaskCustomDetailsValue["rpGroupType"];
                                                        if (rpGroupTypeValue2 != null && rpGroupTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string rpGroupTypeInstance2 = ((string)rpGroupTypeValue2);
                                                            recoveryPlanShutdownGroupTaskDetailsInstance.RpGroupType = rpGroupTypeInstance2;
                                                        }
                                                        
                                                        JToken instanceTypeValue19 = groupTaskCustomDetailsValue["instanceType"];
                                                        if (instanceTypeValue19 != null && instanceTypeValue19.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance19 = ((string)instanceTypeValue19);
                                                            recoveryPlanShutdownGroupTaskDetailsInstance.Type = instanceTypeInstance19;
                                                        }
                                                        
                                                        JToken childTasksArray3 = groupTaskCustomDetailsValue["childTasks"];
                                                        if (childTasksArray3 != null && childTasksArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken childTasksValue3 in ((JArray)childTasksArray3))
                                                            {
                                                                AsrTaskBase taskBaseInstance3 = new AsrTaskBase();
                                                                recoveryPlanShutdownGroupTaskDetailsInstance.ChildTasks.Add(taskBaseInstance3);
                                                                
                                                                JToken taskIdValue5 = childTasksValue3["taskId"];
                                                                if (taskIdValue5 != null && taskIdValue5.Type != JTokenType.Null)
                                                                {
                                                                    string taskIdInstance5 = ((string)taskIdValue5);
                                                                    taskBaseInstance3.ID = taskIdInstance5;
                                                                }
                                                                
                                                                JToken nameValue13 = childTasksValue3["name"];
                                                                if (nameValue13 != null && nameValue13.Type != JTokenType.Null)
                                                                {
                                                                    string nameInstance13 = ((string)nameValue13);
                                                                    taskBaseInstance3.Name = nameInstance13;
                                                                }
                                                                
                                                                JToken startTimeValue4 = childTasksValue3["startTime"];
                                                                if (startTimeValue4 != null && startTimeValue4.Type != JTokenType.Null)
                                                                {
                                                                    DateTime startTimeInstance4 = ((DateTime)startTimeValue4);
                                                                    taskBaseInstance3.StartTime = startTimeInstance4;
                                                                }
                                                                
                                                                JToken endTimeValue4 = childTasksValue3["endTime"];
                                                                if (endTimeValue4 != null && endTimeValue4.Type != JTokenType.Null)
                                                                {
                                                                    DateTime endTimeInstance4 = ((DateTime)endTimeValue4);
                                                                    taskBaseInstance3.EndTime = endTimeInstance4;
                                                                }
                                                                
                                                                JToken allowedActionsArray4 = childTasksValue3["allowedActions"];
                                                                if (allowedActionsArray4 != null && allowedActionsArray4.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken allowedActionsValue4 in ((JArray)allowedActionsArray4))
                                                                    {
                                                                        taskBaseInstance3.Actions.Add(((string)allowedActionsValue4));
                                                                    }
                                                                }
                                                                
                                                                JToken friendlyNameValue4 = childTasksValue3["friendlyName"];
                                                                if (friendlyNameValue4 != null && friendlyNameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string friendlyNameInstance4 = ((string)friendlyNameValue4);
                                                                    taskBaseInstance3.TaskFriendlyName = friendlyNameInstance4;
                                                                }
                                                                
                                                                JToken stateValue4 = childTasksValue3["state"];
                                                                if (stateValue4 != null && stateValue4.Type != JTokenType.Null)
                                                                {
                                                                    string stateInstance4 = ((string)stateValue4);
                                                                    taskBaseInstance3.State = stateInstance4;
                                                                }
                                                                
                                                                JToken stateDescriptionValue4 = childTasksValue3["stateDescription"];
                                                                if (stateDescriptionValue4 != null && stateDescriptionValue4.Type != JTokenType.Null)
                                                                {
                                                                    string stateDescriptionInstance4 = ((string)stateDescriptionValue4);
                                                                    taskBaseInstance3.StateDescription = stateDescriptionInstance4;
                                                                }
                                                                
                                                                JToken taskTypeValue3 = childTasksValue3["taskType"];
                                                                if (taskTypeValue3 != null && taskTypeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string taskTypeInstance3 = ((string)taskTypeValue3);
                                                                    taskBaseInstance3.TaskType = taskTypeInstance3;
                                                                }
                                                                
                                                                JToken errorsArray3 = childTasksValue3["errors"];
                                                                if (errorsArray3 != null && errorsArray3.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken errorsValue3 in ((JArray)errorsArray3))
                                                                    {
                                                                        JobErrorDetails jobErrorDetailsInstance3 = new JobErrorDetails();
                                                                        taskBaseInstance3.Errors.Add(jobErrorDetailsInstance3);
                                                                        
                                                                        JToken serviceErrorDetailsValue3 = errorsValue3["serviceErrorDetails"];
                                                                        if (serviceErrorDetailsValue3 != null && serviceErrorDetailsValue3.Type != JTokenType.Null)
                                                                        {
                                                                            ServiceError serviceErrorDetailsInstance3 = new ServiceError();
                                                                            jobErrorDetailsInstance3.ServiceErrorDetails = serviceErrorDetailsInstance3;
                                                                            
                                                                            JToken codeValue3 = serviceErrorDetailsValue3["code"];
                                                                            if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string codeInstance3 = ((string)codeValue3);
                                                                                serviceErrorDetailsInstance3.Code = codeInstance3;
                                                                            }
                                                                            
                                                                            JToken messageValue3 = serviceErrorDetailsValue3["message"];
                                                                            if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string messageInstance3 = ((string)messageValue3);
                                                                                serviceErrorDetailsInstance3.Message = messageInstance3;
                                                                            }
                                                                            
                                                                            JToken possibleCausesValue5 = serviceErrorDetailsValue3["possibleCauses"];
                                                                            if (possibleCausesValue5 != null && possibleCausesValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string possibleCausesInstance5 = ((string)possibleCausesValue5);
                                                                                serviceErrorDetailsInstance3.PossibleCauses = possibleCausesInstance5;
                                                                            }
                                                                            
                                                                            JToken recommendedActionValue5 = serviceErrorDetailsValue3["recommendedAction"];
                                                                            if (recommendedActionValue5 != null && recommendedActionValue5.Type != JTokenType.Null)
                                                                            {
                                                                                string recommendedActionInstance5 = ((string)recommendedActionValue5);
                                                                                serviceErrorDetailsInstance3.RecommendedAction = recommendedActionInstance5;
                                                                            }
                                                                            
                                                                            JToken activityIdValue4 = serviceErrorDetailsValue3["activityId"];
                                                                            if (activityIdValue4 != null && activityIdValue4.Type != JTokenType.Null)
                                                                            {
                                                                                string activityIdInstance4 = ((string)activityIdValue4);
                                                                                serviceErrorDetailsInstance3.ActivityId = activityIdInstance4;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken providerErrorDetailsValue3 = errorsValue3["providerErrorDetails"];
                                                                        if (providerErrorDetailsValue3 != null && providerErrorDetailsValue3.Type != JTokenType.Null)
                                                                        {
                                                                            ProviderError providerErrorDetailsInstance3 = new ProviderError();
                                                                            jobErrorDetailsInstance3.ProviderErrorDetails = providerErrorDetailsInstance3;
                                                                            
                                                                            JToken errorCodeValue3 = providerErrorDetailsValue3["errorCode"];
                                                                            if (errorCodeValue3 != null && errorCodeValue3.Type != JTokenType.Null)
                                                                            {
                                                                                int errorCodeInstance3 = ((int)errorCodeValue3);
                                                                                providerErrorDetailsInstance3.ErrorCode = errorCodeInstance3;
                                                                            }
                                                                            
                                                                            JToken errorMessageValue3 = providerErrorDetailsValue3["errorMessage"];
                                                                            if (errorMessageValue3 != null && errorMessageValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string errorMessageInstance3 = ((string)errorMessageValue3);
                                                                                providerErrorDetailsInstance3.ErrorMessage = errorMessageInstance3;
                                                                            }
                                                                            
                                                                            JToken errorIdValue3 = providerErrorDetailsValue3["errorId"];
                                                                            if (errorIdValue3 != null && errorIdValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string errorIdInstance3 = ((string)errorIdValue3);
                                                                                providerErrorDetailsInstance3.ErrorId = errorIdInstance3;
                                                                            }
                                                                            
                                                                            JToken possibleCausesValue6 = providerErrorDetailsValue3["possibleCauses"];
                                                                            if (possibleCausesValue6 != null && possibleCausesValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string possibleCausesInstance6 = ((string)possibleCausesValue6);
                                                                                providerErrorDetailsInstance3.PossibleCauses = possibleCausesInstance6;
                                                                            }
                                                                            
                                                                            JToken recommendedActionValue6 = providerErrorDetailsValue3["recommendedAction"];
                                                                            if (recommendedActionValue6 != null && recommendedActionValue6.Type != JTokenType.Null)
                                                                            {
                                                                                string recommendedActionInstance6 = ((string)recommendedActionValue6);
                                                                                providerErrorDetailsInstance3.RecommendedAction = recommendedActionInstance6;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken errorLevelValue3 = errorsValue3["errorLevel"];
                                                                        if (errorLevelValue3 != null && errorLevelValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string errorLevelInstance3 = ((string)errorLevelValue3);
                                                                            jobErrorDetailsInstance3.ErrorLevel = errorLevelInstance3;
                                                                        }
                                                                        
                                                                        JToken creationTimeValue3 = errorsValue3["creationTime"];
                                                                        if (creationTimeValue3 != null && creationTimeValue3.Type != JTokenType.Null)
                                                                        {
                                                                            DateTime creationTimeInstance3 = ((DateTime)creationTimeValue3);
                                                                            jobErrorDetailsInstance3.CreationTime = creationTimeInstance3;
                                                                        }
                                                                        
                                                                        JToken taskIdValue6 = errorsValue3["taskId"];
                                                                        if (taskIdValue6 != null && taskIdValue6.Type != JTokenType.Null)
                                                                        {
                                                                            string taskIdInstance6 = ((string)taskIdValue6);
                                                                            jobErrorDetailsInstance3.TaskId = taskIdInstance6;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken customDetailsValue3 = childTasksValue3["customDetails"];
                                                                if (customDetailsValue3 != null && customDetailsValue3.Type != JTokenType.Null)
                                                                {
                                                                    string typeName4 = ((string)customDetailsValue3["instanceType"]);
                                                                    if (typeName4 == "JobTaskDetails")
                                                                    {
                                                                        JobTaskDetails jobTaskDetailsInstance3 = new JobTaskDetails();
                                                                        
                                                                        JToken jobTaskValue7 = customDetailsValue3["jobTask"];
                                                                        if (jobTaskValue7 != null && jobTaskValue7.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance7 = new JobEntity();
                                                                            jobTaskDetailsInstance3.JobTask = jobTaskInstance7;
                                                                            
                                                                            JToken jobIdValue9 = jobTaskValue7["jobId"];
                                                                            if (jobIdValue9 != null && jobIdValue9.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance9 = ((string)jobIdValue9);
                                                                                jobTaskInstance7.JobId = jobIdInstance9;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue7 = jobTaskValue7["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue7 != null && jobFriendlyNameValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance7 = ((string)jobFriendlyNameValue7);
                                                                                jobTaskInstance7.JobFriendlyName = jobFriendlyNameInstance7;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue8 = jobTaskValue7["targetObjectId"];
                                                                            if (targetObjectIdValue8 != null && targetObjectIdValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance8 = ((string)targetObjectIdValue8);
                                                                                jobTaskInstance7.TargetObjectId = targetObjectIdInstance8;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue8 = jobTaskValue7["targetObjectName"];
                                                                            if (targetObjectNameValue8 != null && targetObjectNameValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance8 = ((string)targetObjectNameValue8);
                                                                                jobTaskInstance7.TargetObjectName = targetObjectNameInstance8;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue8 = jobTaskValue7["targetInstanceType"];
                                                                            if (targetInstanceTypeValue8 != null && targetInstanceTypeValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance8 = ((string)targetInstanceTypeValue8);
                                                                                jobTaskInstance7.TargetInstanceType = targetInstanceTypeInstance8;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue7 = jobTaskValue7["jobScenarioName"];
                                                                            if (jobScenarioNameValue7 != null && jobScenarioNameValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance7 = ((string)jobScenarioNameValue7);
                                                                                jobTaskInstance7.JobScenarioName = jobScenarioNameInstance7;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue20 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue20 != null && instanceTypeValue20.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance20 = ((string)instanceTypeValue20);
                                                                            jobTaskDetailsInstance3.Type = instanceTypeInstance20;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = jobTaskDetailsInstance3;
                                                                    }
                                                                    if (typeName4 == "VirtualMachineTaskDetails")
                                                                    {
                                                                        VirtualMachineTaskDetails virtualMachineTaskDetailsInstance3 = new VirtualMachineTaskDetails();
                                                                        
                                                                        JToken skippedReasonValue5 = customDetailsValue3["skippedReason"];
                                                                        if (skippedReasonValue5 != null && skippedReasonValue5.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonInstance5 = ((string)skippedReasonValue5);
                                                                            virtualMachineTaskDetailsInstance3.SkippedReason = skippedReasonInstance5;
                                                                        }
                                                                        
                                                                        JToken skippedReasonStringValue5 = customDetailsValue3["skippedReasonString"];
                                                                        if (skippedReasonStringValue5 != null && skippedReasonStringValue5.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonStringInstance5 = ((string)skippedReasonStringValue5);
                                                                            virtualMachineTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance5;
                                                                        }
                                                                        
                                                                        JToken jobTaskValue8 = customDetailsValue3["jobTask"];
                                                                        if (jobTaskValue8 != null && jobTaskValue8.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance8 = new JobEntity();
                                                                            virtualMachineTaskDetailsInstance3.JobTask = jobTaskInstance8;
                                                                            
                                                                            JToken jobIdValue10 = jobTaskValue8["jobId"];
                                                                            if (jobIdValue10 != null && jobIdValue10.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance10 = ((string)jobIdValue10);
                                                                                jobTaskInstance8.JobId = jobIdInstance10;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue8 = jobTaskValue8["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue8 != null && jobFriendlyNameValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance8 = ((string)jobFriendlyNameValue8);
                                                                                jobTaskInstance8.JobFriendlyName = jobFriendlyNameInstance8;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue9 = jobTaskValue8["targetObjectId"];
                                                                            if (targetObjectIdValue9 != null && targetObjectIdValue9.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance9 = ((string)targetObjectIdValue9);
                                                                                jobTaskInstance8.TargetObjectId = targetObjectIdInstance9;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue9 = jobTaskValue8["targetObjectName"];
                                                                            if (targetObjectNameValue9 != null && targetObjectNameValue9.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance9 = ((string)targetObjectNameValue9);
                                                                                jobTaskInstance8.TargetObjectName = targetObjectNameInstance9;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue9 = jobTaskValue8["targetInstanceType"];
                                                                            if (targetInstanceTypeValue9 != null && targetInstanceTypeValue9.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance9 = ((string)targetInstanceTypeValue9);
                                                                                jobTaskInstance8.TargetInstanceType = targetInstanceTypeInstance9;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue8 = jobTaskValue8["jobScenarioName"];
                                                                            if (jobScenarioNameValue8 != null && jobScenarioNameValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance8 = ((string)jobScenarioNameValue8);
                                                                                jobTaskInstance8.JobScenarioName = jobScenarioNameInstance8;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue21 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue21 != null && instanceTypeValue21.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance21 = ((string)instanceTypeValue21);
                                                                            virtualMachineTaskDetailsInstance3.Type = instanceTypeInstance21;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = virtualMachineTaskDetailsInstance3;
                                                                    }
                                                                    if (typeName4 == "FabricReplicationGroupTaskDetails")
                                                                    {
                                                                        FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance3 = new FabricReplicationGroupTaskDetails();
                                                                        
                                                                        JToken skippedReasonValue6 = customDetailsValue3["skippedReason"];
                                                                        if (skippedReasonValue6 != null && skippedReasonValue6.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonInstance6 = ((string)skippedReasonValue6);
                                                                            fabricReplicationGroupTaskDetailsInstance3.SkippedReason = skippedReasonInstance6;
                                                                        }
                                                                        
                                                                        JToken skippedReasonStringValue6 = customDetailsValue3["skippedReasonString"];
                                                                        if (skippedReasonStringValue6 != null && skippedReasonStringValue6.Type != JTokenType.Null)
                                                                        {
                                                                            string skippedReasonStringInstance6 = ((string)skippedReasonStringValue6);
                                                                            fabricReplicationGroupTaskDetailsInstance3.SkippedReasonString = skippedReasonStringInstance6;
                                                                        }
                                                                        
                                                                        JToken jobTaskValue9 = customDetailsValue3["jobTask"];
                                                                        if (jobTaskValue9 != null && jobTaskValue9.Type != JTokenType.Null)
                                                                        {
                                                                            JobEntity jobTaskInstance9 = new JobEntity();
                                                                            fabricReplicationGroupTaskDetailsInstance3.JobTask = jobTaskInstance9;
                                                                            
                                                                            JToken jobIdValue11 = jobTaskValue9["jobId"];
                                                                            if (jobIdValue11 != null && jobIdValue11.Type != JTokenType.Null)
                                                                            {
                                                                                string jobIdInstance11 = ((string)jobIdValue11);
                                                                                jobTaskInstance9.JobId = jobIdInstance11;
                                                                            }
                                                                            
                                                                            JToken jobFriendlyNameValue9 = jobTaskValue9["jobFriendlyName"];
                                                                            if (jobFriendlyNameValue9 != null && jobFriendlyNameValue9.Type != JTokenType.Null)
                                                                            {
                                                                                string jobFriendlyNameInstance9 = ((string)jobFriendlyNameValue9);
                                                                                jobTaskInstance9.JobFriendlyName = jobFriendlyNameInstance9;
                                                                            }
                                                                            
                                                                            JToken targetObjectIdValue10 = jobTaskValue9["targetObjectId"];
                                                                            if (targetObjectIdValue10 != null && targetObjectIdValue10.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectIdInstance10 = ((string)targetObjectIdValue10);
                                                                                jobTaskInstance9.TargetObjectId = targetObjectIdInstance10;
                                                                            }
                                                                            
                                                                            JToken targetObjectNameValue10 = jobTaskValue9["targetObjectName"];
                                                                            if (targetObjectNameValue10 != null && targetObjectNameValue10.Type != JTokenType.Null)
                                                                            {
                                                                                string targetObjectNameInstance10 = ((string)targetObjectNameValue10);
                                                                                jobTaskInstance9.TargetObjectName = targetObjectNameInstance10;
                                                                            }
                                                                            
                                                                            JToken targetInstanceTypeValue10 = jobTaskValue9["targetInstanceType"];
                                                                            if (targetInstanceTypeValue10 != null && targetInstanceTypeValue10.Type != JTokenType.Null)
                                                                            {
                                                                                string targetInstanceTypeInstance10 = ((string)targetInstanceTypeValue10);
                                                                                jobTaskInstance9.TargetInstanceType = targetInstanceTypeInstance10;
                                                                            }
                                                                            
                                                                            JToken jobScenarioNameValue9 = jobTaskValue9["jobScenarioName"];
                                                                            if (jobScenarioNameValue9 != null && jobScenarioNameValue9.Type != JTokenType.Null)
                                                                            {
                                                                                string jobScenarioNameInstance9 = ((string)jobScenarioNameValue9);
                                                                                jobTaskInstance9.JobScenarioName = jobScenarioNameInstance9;
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue22 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue22 != null && instanceTypeValue22.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance22 = ((string)instanceTypeValue22);
                                                                            fabricReplicationGroupTaskDetailsInstance3.Type = instanceTypeInstance22;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = fabricReplicationGroupTaskDetailsInstance3;
                                                                    }
                                                                    if (typeName4 == "ManualActionTaskDetails")
                                                                    {
                                                                        ManualActionTaskDetails manualActionTaskDetailsInstance3 = new ManualActionTaskDetails();
                                                                        
                                                                        JToken nameValue14 = customDetailsValue3["name"];
                                                                        if (nameValue14 != null && nameValue14.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance14 = ((string)nameValue14);
                                                                            manualActionTaskDetailsInstance3.Name = nameInstance14;
                                                                        }
                                                                        
                                                                        JToken instructionsValue3 = customDetailsValue3["instructions"];
                                                                        if (instructionsValue3 != null && instructionsValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string instructionsInstance3 = ((string)instructionsValue3);
                                                                            manualActionTaskDetailsInstance3.Instructions = instructionsInstance3;
                                                                        }
                                                                        
                                                                        JToken observationValue3 = customDetailsValue3["observation"];
                                                                        if (observationValue3 != null && observationValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string observationInstance3 = ((string)observationValue3);
                                                                            manualActionTaskDetailsInstance3.Observation = observationInstance3;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue23 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue23 != null && instanceTypeValue23.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance23 = ((string)instanceTypeValue23);
                                                                            manualActionTaskDetailsInstance3.Type = instanceTypeInstance23;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = manualActionTaskDetailsInstance3;
                                                                    }
                                                                    if (typeName4 == "ScriptActionTaskDetails")
                                                                    {
                                                                        ScriptActionTaskDetails scriptActionTaskDetailsInstance3 = new ScriptActionTaskDetails();
                                                                        
                                                                        JToken nameValue15 = customDetailsValue3["name"];
                                                                        if (nameValue15 != null && nameValue15.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance15 = ((string)nameValue15);
                                                                            scriptActionTaskDetailsInstance3.Name = nameInstance15;
                                                                        }
                                                                        
                                                                        JToken pathValue3 = customDetailsValue3["path"];
                                                                        if (pathValue3 != null && pathValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string pathInstance3 = ((string)pathValue3);
                                                                            scriptActionTaskDetailsInstance3.Path = pathInstance3;
                                                                        }
                                                                        
                                                                        JToken outputValue3 = customDetailsValue3["output"];
                                                                        if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string outputInstance3 = ((string)outputValue3);
                                                                            scriptActionTaskDetailsInstance3.Output = outputInstance3;
                                                                        }
                                                                        
                                                                        JToken isPrimarySideScriptValue5 = customDetailsValue3["isPrimarySideScript"];
                                                                        if (isPrimarySideScriptValue5 != null && isPrimarySideScriptValue5.Type != JTokenType.Null)
                                                                        {
                                                                            bool isPrimarySideScriptInstance5 = ((bool)isPrimarySideScriptValue5);
                                                                            scriptActionTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance5;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue24 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue24 != null && instanceTypeValue24.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance24 = ((string)instanceTypeValue24);
                                                                            scriptActionTaskDetailsInstance3.Type = instanceTypeInstance24;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = scriptActionTaskDetailsInstance3;
                                                                    }
                                                                    if (typeName4 == "VmNicUpdatesTaskDetails")
                                                                    {
                                                                        VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance3 = new VmNicUpdatesTaskDetails();
                                                                        
                                                                        JToken vmIdValue3 = customDetailsValue3["vmId"];
                                                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string vmIdInstance3 = ((string)vmIdValue3);
                                                                            vmNicUpdatesTaskDetailsInstance3.VmId = vmIdInstance3;
                                                                        }
                                                                        
                                                                        JToken nicIdValue3 = customDetailsValue3["nicId"];
                                                                        if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string nicIdInstance3 = ((string)nicIdValue3);
                                                                            vmNicUpdatesTaskDetailsInstance3.NicId = nicIdInstance3;
                                                                        }
                                                                        
                                                                        JToken nameValue16 = customDetailsValue3["name"];
                                                                        if (nameValue16 != null && nameValue16.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance16 = ((string)nameValue16);
                                                                            vmNicUpdatesTaskDetailsInstance3.Name = nameInstance16;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue25 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue25 != null && instanceTypeValue25.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance25 = ((string)instanceTypeValue25);
                                                                            vmNicUpdatesTaskDetailsInstance3.Type = instanceTypeInstance25;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = vmNicUpdatesTaskDetailsInstance3;
                                                                    }
                                                                    if (typeName4 == "ConsistencyCheckTaskDetails")
                                                                    {
                                                                        ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance3 = new ConsistencyCheckTaskDetails();
                                                                        
                                                                        JToken vmDetailsArray3 = customDetailsValue3["vmDetails"];
                                                                        if (vmDetailsArray3 != null && vmDetailsArray3.Type != JTokenType.Null)
                                                                        {
                                                                            foreach (JToken vmDetailsValue3 in ((JArray)vmDetailsArray3))
                                                                            {
                                                                                InconsistentVmDetails inconsistentVmDetailsInstance3 = new InconsistentVmDetails();
                                                                                consistencyCheckTaskDetailsInstance3.VmDetails.Add(inconsistentVmDetailsInstance3);
                                                                                
                                                                                JToken vmNameValue3 = vmDetailsValue3["vmName"];
                                                                                if (vmNameValue3 != null && vmNameValue3.Type != JTokenType.Null)
                                                                                {
                                                                                    string vmNameInstance3 = ((string)vmNameValue3);
                                                                                    inconsistentVmDetailsInstance3.VmName = vmNameInstance3;
                                                                                }
                                                                                
                                                                                JToken cloudNameValue3 = vmDetailsValue3["cloudName"];
                                                                                if (cloudNameValue3 != null && cloudNameValue3.Type != JTokenType.Null)
                                                                                {
                                                                                    string cloudNameInstance3 = ((string)cloudNameValue3);
                                                                                    inconsistentVmDetailsInstance3.CloudName = cloudNameInstance3;
                                                                                }
                                                                                
                                                                                JToken detailsArray3 = vmDetailsValue3["details"];
                                                                                if (detailsArray3 != null && detailsArray3.Type != JTokenType.Null)
                                                                                {
                                                                                    foreach (JToken detailsValue3 in ((JArray)detailsArray3))
                                                                                    {
                                                                                        inconsistentVmDetailsInstance3.Details.Add(((string)detailsValue3));
                                                                                    }
                                                                                }
                                                                                
                                                                                JToken errorIdsArray3 = vmDetailsValue3["errorIds"];
                                                                                if (errorIdsArray3 != null && errorIdsArray3.Type != JTokenType.Null)
                                                                                {
                                                                                    foreach (JToken errorIdsValue3 in ((JArray)errorIdsArray3))
                                                                                    {
                                                                                        inconsistentVmDetailsInstance3.ErrorIds.Add(((string)errorIdsValue3));
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue26 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue26 != null && instanceTypeValue26.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance26 = ((string)instanceTypeValue26);
                                                                            consistencyCheckTaskDetailsInstance3.Type = instanceTypeInstance26;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = consistencyCheckTaskDetailsInstance3;
                                                                    }
                                                                    if (typeName4 == "AutomationRunbookTaskDetails")
                                                                    {
                                                                        AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance3 = new AutomationRunbookTaskDetails();
                                                                        
                                                                        JToken nameValue17 = customDetailsValue3["name"];
                                                                        if (nameValue17 != null && nameValue17.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance17 = ((string)nameValue17);
                                                                            automationRunbookTaskDetailsInstance3.Name = nameInstance17;
                                                                        }
                                                                        
                                                                        JToken cloudServiceNameValue3 = customDetailsValue3["cloudServiceName"];
                                                                        if (cloudServiceNameValue3 != null && cloudServiceNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string cloudServiceNameInstance3 = ((string)cloudServiceNameValue3);
                                                                            automationRunbookTaskDetailsInstance3.CloudServiceName = cloudServiceNameInstance3;
                                                                        }
                                                                        
                                                                        JToken subscriptionIdValue3 = customDetailsValue3["subscriptionId"];
                                                                        if (subscriptionIdValue3 != null && subscriptionIdValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string subscriptionIdInstance3 = ((string)subscriptionIdValue3);
                                                                            automationRunbookTaskDetailsInstance3.SubscriptionId = subscriptionIdInstance3;
                                                                        }
                                                                        
                                                                        JToken accountNameValue3 = customDetailsValue3["accountName"];
                                                                        if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string accountNameInstance3 = ((string)accountNameValue3);
                                                                            automationRunbookTaskDetailsInstance3.AccountName = accountNameInstance3;
                                                                        }
                                                                        
                                                                        JToken runbookIdValue3 = customDetailsValue3["runbookId"];
                                                                        if (runbookIdValue3 != null && runbookIdValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string runbookIdInstance3 = ((string)runbookIdValue3);
                                                                            automationRunbookTaskDetailsInstance3.RunbookId = runbookIdInstance3;
                                                                        }
                                                                        
                                                                        JToken runbookNameValue3 = customDetailsValue3["runbookName"];
                                                                        if (runbookNameValue3 != null && runbookNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string runbookNameInstance3 = ((string)runbookNameValue3);
                                                                            automationRunbookTaskDetailsInstance3.RunbookName = runbookNameInstance3;
                                                                        }
                                                                        
                                                                        JToken jobIdValue12 = customDetailsValue3["jobId"];
                                                                        if (jobIdValue12 != null && jobIdValue12.Type != JTokenType.Null)
                                                                        {
                                                                            string jobIdInstance12 = ((string)jobIdValue12);
                                                                            automationRunbookTaskDetailsInstance3.JobId = jobIdInstance12;
                                                                        }
                                                                        
                                                                        JToken jobOutputValue3 = customDetailsValue3["jobOutput"];
                                                                        if (jobOutputValue3 != null && jobOutputValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string jobOutputInstance3 = ((string)jobOutputValue3);
                                                                            automationRunbookTaskDetailsInstance3.JobOutput = jobOutputInstance3;
                                                                        }
                                                                        
                                                                        JToken isPrimarySideScriptValue6 = customDetailsValue3["isPrimarySideScript"];
                                                                        if (isPrimarySideScriptValue6 != null && isPrimarySideScriptValue6.Type != JTokenType.Null)
                                                                        {
                                                                            bool isPrimarySideScriptInstance6 = ((bool)isPrimarySideScriptValue6);
                                                                            automationRunbookTaskDetailsInstance3.IsPrimarySideScript = isPrimarySideScriptInstance6;
                                                                        }
                                                                        
                                                                        JToken instanceTypeValue27 = customDetailsValue3["instanceType"];
                                                                        if (instanceTypeValue27 != null && instanceTypeValue27.Type != JTokenType.Null)
                                                                        {
                                                                            string instanceTypeInstance27 = ((string)instanceTypeValue27);
                                                                            automationRunbookTaskDetailsInstance3.Type = instanceTypeInstance27;
                                                                        }
                                                                        taskBaseInstance3.CustomDetails = automationRunbookTaskDetailsInstance3;
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        taskInstance.GroupTaskCustomDetails = recoveryPlanShutdownGroupTaskDetailsInstance;
                                                    }
                                                }
                                                
                                                JToken taskIdValue7 = tasksValue["taskId"];
                                                if (taskIdValue7 != null && taskIdValue7.Type != JTokenType.Null)
                                                {
                                                    string taskIdInstance7 = ((string)taskIdValue7);
                                                    taskInstance.ID = taskIdInstance7;
                                                }
                                                
                                                JToken nameValue18 = tasksValue["name"];
                                                if (nameValue18 != null && nameValue18.Type != JTokenType.Null)
                                                {
                                                    string nameInstance18 = ((string)nameValue18);
                                                    taskInstance.Name = nameInstance18;
                                                }
                                                
                                                JToken startTimeValue5 = tasksValue["startTime"];
                                                if (startTimeValue5 != null && startTimeValue5.Type != JTokenType.Null)
                                                {
                                                    DateTime startTimeInstance5 = ((DateTime)startTimeValue5);
                                                    taskInstance.StartTime = startTimeInstance5;
                                                }
                                                
                                                JToken endTimeValue5 = tasksValue["endTime"];
                                                if (endTimeValue5 != null && endTimeValue5.Type != JTokenType.Null)
                                                {
                                                    DateTime endTimeInstance5 = ((DateTime)endTimeValue5);
                                                    taskInstance.EndTime = endTimeInstance5;
                                                }
                                                
                                                JToken allowedActionsArray5 = tasksValue["allowedActions"];
                                                if (allowedActionsArray5 != null && allowedActionsArray5.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken allowedActionsValue5 in ((JArray)allowedActionsArray5))
                                                    {
                                                        taskInstance.Actions.Add(((string)allowedActionsValue5));
                                                    }
                                                }
                                                
                                                JToken friendlyNameValue5 = tasksValue["friendlyName"];
                                                if (friendlyNameValue5 != null && friendlyNameValue5.Type != JTokenType.Null)
                                                {
                                                    string friendlyNameInstance5 = ((string)friendlyNameValue5);
                                                    taskInstance.TaskFriendlyName = friendlyNameInstance5;
                                                }
                                                
                                                JToken stateValue5 = tasksValue["state"];
                                                if (stateValue5 != null && stateValue5.Type != JTokenType.Null)
                                                {
                                                    string stateInstance5 = ((string)stateValue5);
                                                    taskInstance.State = stateInstance5;
                                                }
                                                
                                                JToken stateDescriptionValue5 = tasksValue["stateDescription"];
                                                if (stateDescriptionValue5 != null && stateDescriptionValue5.Type != JTokenType.Null)
                                                {
                                                    string stateDescriptionInstance5 = ((string)stateDescriptionValue5);
                                                    taskInstance.StateDescription = stateDescriptionInstance5;
                                                }
                                                
                                                JToken taskTypeValue4 = tasksValue["taskType"];
                                                if (taskTypeValue4 != null && taskTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string taskTypeInstance4 = ((string)taskTypeValue4);
                                                    taskInstance.TaskType = taskTypeInstance4;
                                                }
                                                
                                                JToken errorsArray4 = tasksValue["errors"];
                                                if (errorsArray4 != null && errorsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken errorsValue4 in ((JArray)errorsArray4))
                                                    {
                                                        JobErrorDetails jobErrorDetailsInstance4 = new JobErrorDetails();
                                                        taskInstance.Errors.Add(jobErrorDetailsInstance4);
                                                        
                                                        JToken serviceErrorDetailsValue4 = errorsValue4["serviceErrorDetails"];
                                                        if (serviceErrorDetailsValue4 != null && serviceErrorDetailsValue4.Type != JTokenType.Null)
                                                        {
                                                            ServiceError serviceErrorDetailsInstance4 = new ServiceError();
                                                            jobErrorDetailsInstance4.ServiceErrorDetails = serviceErrorDetailsInstance4;
                                                            
                                                            JToken codeValue4 = serviceErrorDetailsValue4["code"];
                                                            if (codeValue4 != null && codeValue4.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance4 = ((string)codeValue4);
                                                                serviceErrorDetailsInstance4.Code = codeInstance4;
                                                            }
                                                            
                                                            JToken messageValue4 = serviceErrorDetailsValue4["message"];
                                                            if (messageValue4 != null && messageValue4.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance4 = ((string)messageValue4);
                                                                serviceErrorDetailsInstance4.Message = messageInstance4;
                                                            }
                                                            
                                                            JToken possibleCausesValue7 = serviceErrorDetailsValue4["possibleCauses"];
                                                            if (possibleCausesValue7 != null && possibleCausesValue7.Type != JTokenType.Null)
                                                            {
                                                                string possibleCausesInstance7 = ((string)possibleCausesValue7);
                                                                serviceErrorDetailsInstance4.PossibleCauses = possibleCausesInstance7;
                                                            }
                                                            
                                                            JToken recommendedActionValue7 = serviceErrorDetailsValue4["recommendedAction"];
                                                            if (recommendedActionValue7 != null && recommendedActionValue7.Type != JTokenType.Null)
                                                            {
                                                                string recommendedActionInstance7 = ((string)recommendedActionValue7);
                                                                serviceErrorDetailsInstance4.RecommendedAction = recommendedActionInstance7;
                                                            }
                                                            
                                                            JToken activityIdValue5 = serviceErrorDetailsValue4["activityId"];
                                                            if (activityIdValue5 != null && activityIdValue5.Type != JTokenType.Null)
                                                            {
                                                                string activityIdInstance5 = ((string)activityIdValue5);
                                                                serviceErrorDetailsInstance4.ActivityId = activityIdInstance5;
                                                            }
                                                        }
                                                        
                                                        JToken providerErrorDetailsValue4 = errorsValue4["providerErrorDetails"];
                                                        if (providerErrorDetailsValue4 != null && providerErrorDetailsValue4.Type != JTokenType.Null)
                                                        {
                                                            ProviderError providerErrorDetailsInstance4 = new ProviderError();
                                                            jobErrorDetailsInstance4.ProviderErrorDetails = providerErrorDetailsInstance4;
                                                            
                                                            JToken errorCodeValue4 = providerErrorDetailsValue4["errorCode"];
                                                            if (errorCodeValue4 != null && errorCodeValue4.Type != JTokenType.Null)
                                                            {
                                                                int errorCodeInstance4 = ((int)errorCodeValue4);
                                                                providerErrorDetailsInstance4.ErrorCode = errorCodeInstance4;
                                                            }
                                                            
                                                            JToken errorMessageValue4 = providerErrorDetailsValue4["errorMessage"];
                                                            if (errorMessageValue4 != null && errorMessageValue4.Type != JTokenType.Null)
                                                            {
                                                                string errorMessageInstance4 = ((string)errorMessageValue4);
                                                                providerErrorDetailsInstance4.ErrorMessage = errorMessageInstance4;
                                                            }
                                                            
                                                            JToken errorIdValue4 = providerErrorDetailsValue4["errorId"];
                                                            if (errorIdValue4 != null && errorIdValue4.Type != JTokenType.Null)
                                                            {
                                                                string errorIdInstance4 = ((string)errorIdValue4);
                                                                providerErrorDetailsInstance4.ErrorId = errorIdInstance4;
                                                            }
                                                            
                                                            JToken possibleCausesValue8 = providerErrorDetailsValue4["possibleCauses"];
                                                            if (possibleCausesValue8 != null && possibleCausesValue8.Type != JTokenType.Null)
                                                            {
                                                                string possibleCausesInstance8 = ((string)possibleCausesValue8);
                                                                providerErrorDetailsInstance4.PossibleCauses = possibleCausesInstance8;
                                                            }
                                                            
                                                            JToken recommendedActionValue8 = providerErrorDetailsValue4["recommendedAction"];
                                                            if (recommendedActionValue8 != null && recommendedActionValue8.Type != JTokenType.Null)
                                                            {
                                                                string recommendedActionInstance8 = ((string)recommendedActionValue8);
                                                                providerErrorDetailsInstance4.RecommendedAction = recommendedActionInstance8;
                                                            }
                                                        }
                                                        
                                                        JToken errorLevelValue4 = errorsValue4["errorLevel"];
                                                        if (errorLevelValue4 != null && errorLevelValue4.Type != JTokenType.Null)
                                                        {
                                                            string errorLevelInstance4 = ((string)errorLevelValue4);
                                                            jobErrorDetailsInstance4.ErrorLevel = errorLevelInstance4;
                                                        }
                                                        
                                                        JToken creationTimeValue4 = errorsValue4["creationTime"];
                                                        if (creationTimeValue4 != null && creationTimeValue4.Type != JTokenType.Null)
                                                        {
                                                            DateTime creationTimeInstance4 = ((DateTime)creationTimeValue4);
                                                            jobErrorDetailsInstance4.CreationTime = creationTimeInstance4;
                                                        }
                                                        
                                                        JToken taskIdValue8 = errorsValue4["taskId"];
                                                        if (taskIdValue8 != null && taskIdValue8.Type != JTokenType.Null)
                                                        {
                                                            string taskIdInstance8 = ((string)taskIdValue8);
                                                            jobErrorDetailsInstance4.TaskId = taskIdInstance8;
                                                        }
                                                    }
                                                }
                                                
                                                JToken customDetailsValue4 = tasksValue["customDetails"];
                                                if (customDetailsValue4 != null && customDetailsValue4.Type != JTokenType.Null)
                                                {
                                                    string typeName5 = ((string)customDetailsValue4["instanceType"]);
                                                    if (typeName5 == "JobTaskDetails")
                                                    {
                                                        JobTaskDetails jobTaskDetailsInstance4 = new JobTaskDetails();
                                                        
                                                        JToken jobTaskValue10 = customDetailsValue4["jobTask"];
                                                        if (jobTaskValue10 != null && jobTaskValue10.Type != JTokenType.Null)
                                                        {
                                                            JobEntity jobTaskInstance10 = new JobEntity();
                                                            jobTaskDetailsInstance4.JobTask = jobTaskInstance10;
                                                            
                                                            JToken jobIdValue13 = jobTaskValue10["jobId"];
                                                            if (jobIdValue13 != null && jobIdValue13.Type != JTokenType.Null)
                                                            {
                                                                string jobIdInstance13 = ((string)jobIdValue13);
                                                                jobTaskInstance10.JobId = jobIdInstance13;
                                                            }
                                                            
                                                            JToken jobFriendlyNameValue10 = jobTaskValue10["jobFriendlyName"];
                                                            if (jobFriendlyNameValue10 != null && jobFriendlyNameValue10.Type != JTokenType.Null)
                                                            {
                                                                string jobFriendlyNameInstance10 = ((string)jobFriendlyNameValue10);
                                                                jobTaskInstance10.JobFriendlyName = jobFriendlyNameInstance10;
                                                            }
                                                            
                                                            JToken targetObjectIdValue11 = jobTaskValue10["targetObjectId"];
                                                            if (targetObjectIdValue11 != null && targetObjectIdValue11.Type != JTokenType.Null)
                                                            {
                                                                string targetObjectIdInstance11 = ((string)targetObjectIdValue11);
                                                                jobTaskInstance10.TargetObjectId = targetObjectIdInstance11;
                                                            }
                                                            
                                                            JToken targetObjectNameValue11 = jobTaskValue10["targetObjectName"];
                                                            if (targetObjectNameValue11 != null && targetObjectNameValue11.Type != JTokenType.Null)
                                                            {
                                                                string targetObjectNameInstance11 = ((string)targetObjectNameValue11);
                                                                jobTaskInstance10.TargetObjectName = targetObjectNameInstance11;
                                                            }
                                                            
                                                            JToken targetInstanceTypeValue11 = jobTaskValue10["targetInstanceType"];
                                                            if (targetInstanceTypeValue11 != null && targetInstanceTypeValue11.Type != JTokenType.Null)
                                                            {
                                                                string targetInstanceTypeInstance11 = ((string)targetInstanceTypeValue11);
                                                                jobTaskInstance10.TargetInstanceType = targetInstanceTypeInstance11;
                                                            }
                                                            
                                                            JToken jobScenarioNameValue10 = jobTaskValue10["jobScenarioName"];
                                                            if (jobScenarioNameValue10 != null && jobScenarioNameValue10.Type != JTokenType.Null)
                                                            {
                                                                string jobScenarioNameInstance10 = ((string)jobScenarioNameValue10);
                                                                jobTaskInstance10.JobScenarioName = jobScenarioNameInstance10;
                                                            }
                                                        }
                                                        
                                                        JToken instanceTypeValue28 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue28 != null && instanceTypeValue28.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance28 = ((string)instanceTypeValue28);
                                                            jobTaskDetailsInstance4.Type = instanceTypeInstance28;
                                                        }
                                                        taskInstance.CustomDetails = jobTaskDetailsInstance4;
                                                    }
                                                    if (typeName5 == "VirtualMachineTaskDetails")
                                                    {
                                                        VirtualMachineTaskDetails virtualMachineTaskDetailsInstance4 = new VirtualMachineTaskDetails();
                                                        
                                                        JToken skippedReasonValue7 = customDetailsValue4["skippedReason"];
                                                        if (skippedReasonValue7 != null && skippedReasonValue7.Type != JTokenType.Null)
                                                        {
                                                            string skippedReasonInstance7 = ((string)skippedReasonValue7);
                                                            virtualMachineTaskDetailsInstance4.SkippedReason = skippedReasonInstance7;
                                                        }
                                                        
                                                        JToken skippedReasonStringValue7 = customDetailsValue4["skippedReasonString"];
                                                        if (skippedReasonStringValue7 != null && skippedReasonStringValue7.Type != JTokenType.Null)
                                                        {
                                                            string skippedReasonStringInstance7 = ((string)skippedReasonStringValue7);
                                                            virtualMachineTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance7;
                                                        }
                                                        
                                                        JToken jobTaskValue11 = customDetailsValue4["jobTask"];
                                                        if (jobTaskValue11 != null && jobTaskValue11.Type != JTokenType.Null)
                                                        {
                                                            JobEntity jobTaskInstance11 = new JobEntity();
                                                            virtualMachineTaskDetailsInstance4.JobTask = jobTaskInstance11;
                                                            
                                                            JToken jobIdValue14 = jobTaskValue11["jobId"];
                                                            if (jobIdValue14 != null && jobIdValue14.Type != JTokenType.Null)
                                                            {
                                                                string jobIdInstance14 = ((string)jobIdValue14);
                                                                jobTaskInstance11.JobId = jobIdInstance14;
                                                            }
                                                            
                                                            JToken jobFriendlyNameValue11 = jobTaskValue11["jobFriendlyName"];
                                                            if (jobFriendlyNameValue11 != null && jobFriendlyNameValue11.Type != JTokenType.Null)
                                                            {
                                                                string jobFriendlyNameInstance11 = ((string)jobFriendlyNameValue11);
                                                                jobTaskInstance11.JobFriendlyName = jobFriendlyNameInstance11;
                                                            }
                                                            
                                                            JToken targetObjectIdValue12 = jobTaskValue11["targetObjectId"];
                                                            if (targetObjectIdValue12 != null && targetObjectIdValue12.Type != JTokenType.Null)
                                                            {
                                                                string targetObjectIdInstance12 = ((string)targetObjectIdValue12);
                                                                jobTaskInstance11.TargetObjectId = targetObjectIdInstance12;
                                                            }
                                                            
                                                            JToken targetObjectNameValue12 = jobTaskValue11["targetObjectName"];
                                                            if (targetObjectNameValue12 != null && targetObjectNameValue12.Type != JTokenType.Null)
                                                            {
                                                                string targetObjectNameInstance12 = ((string)targetObjectNameValue12);
                                                                jobTaskInstance11.TargetObjectName = targetObjectNameInstance12;
                                                            }
                                                            
                                                            JToken targetInstanceTypeValue12 = jobTaskValue11["targetInstanceType"];
                                                            if (targetInstanceTypeValue12 != null && targetInstanceTypeValue12.Type != JTokenType.Null)
                                                            {
                                                                string targetInstanceTypeInstance12 = ((string)targetInstanceTypeValue12);
                                                                jobTaskInstance11.TargetInstanceType = targetInstanceTypeInstance12;
                                                            }
                                                            
                                                            JToken jobScenarioNameValue11 = jobTaskValue11["jobScenarioName"];
                                                            if (jobScenarioNameValue11 != null && jobScenarioNameValue11.Type != JTokenType.Null)
                                                            {
                                                                string jobScenarioNameInstance11 = ((string)jobScenarioNameValue11);
                                                                jobTaskInstance11.JobScenarioName = jobScenarioNameInstance11;
                                                            }
                                                        }
                                                        
                                                        JToken instanceTypeValue29 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue29 != null && instanceTypeValue29.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance29 = ((string)instanceTypeValue29);
                                                            virtualMachineTaskDetailsInstance4.Type = instanceTypeInstance29;
                                                        }
                                                        taskInstance.CustomDetails = virtualMachineTaskDetailsInstance4;
                                                    }
                                                    if (typeName5 == "FabricReplicationGroupTaskDetails")
                                                    {
                                                        FabricReplicationGroupTaskDetails fabricReplicationGroupTaskDetailsInstance4 = new FabricReplicationGroupTaskDetails();
                                                        
                                                        JToken skippedReasonValue8 = customDetailsValue4["skippedReason"];
                                                        if (skippedReasonValue8 != null && skippedReasonValue8.Type != JTokenType.Null)
                                                        {
                                                            string skippedReasonInstance8 = ((string)skippedReasonValue8);
                                                            fabricReplicationGroupTaskDetailsInstance4.SkippedReason = skippedReasonInstance8;
                                                        }
                                                        
                                                        JToken skippedReasonStringValue8 = customDetailsValue4["skippedReasonString"];
                                                        if (skippedReasonStringValue8 != null && skippedReasonStringValue8.Type != JTokenType.Null)
                                                        {
                                                            string skippedReasonStringInstance8 = ((string)skippedReasonStringValue8);
                                                            fabricReplicationGroupTaskDetailsInstance4.SkippedReasonString = skippedReasonStringInstance8;
                                                        }
                                                        
                                                        JToken jobTaskValue12 = customDetailsValue4["jobTask"];
                                                        if (jobTaskValue12 != null && jobTaskValue12.Type != JTokenType.Null)
                                                        {
                                                            JobEntity jobTaskInstance12 = new JobEntity();
                                                            fabricReplicationGroupTaskDetailsInstance4.JobTask = jobTaskInstance12;
                                                            
                                                            JToken jobIdValue15 = jobTaskValue12["jobId"];
                                                            if (jobIdValue15 != null && jobIdValue15.Type != JTokenType.Null)
                                                            {
                                                                string jobIdInstance15 = ((string)jobIdValue15);
                                                                jobTaskInstance12.JobId = jobIdInstance15;
                                                            }
                                                            
                                                            JToken jobFriendlyNameValue12 = jobTaskValue12["jobFriendlyName"];
                                                            if (jobFriendlyNameValue12 != null && jobFriendlyNameValue12.Type != JTokenType.Null)
                                                            {
                                                                string jobFriendlyNameInstance12 = ((string)jobFriendlyNameValue12);
                                                                jobTaskInstance12.JobFriendlyName = jobFriendlyNameInstance12;
                                                            }
                                                            
                                                            JToken targetObjectIdValue13 = jobTaskValue12["targetObjectId"];
                                                            if (targetObjectIdValue13 != null && targetObjectIdValue13.Type != JTokenType.Null)
                                                            {
                                                                string targetObjectIdInstance13 = ((string)targetObjectIdValue13);
                                                                jobTaskInstance12.TargetObjectId = targetObjectIdInstance13;
                                                            }
                                                            
                                                            JToken targetObjectNameValue13 = jobTaskValue12["targetObjectName"];
                                                            if (targetObjectNameValue13 != null && targetObjectNameValue13.Type != JTokenType.Null)
                                                            {
                                                                string targetObjectNameInstance13 = ((string)targetObjectNameValue13);
                                                                jobTaskInstance12.TargetObjectName = targetObjectNameInstance13;
                                                            }
                                                            
                                                            JToken targetInstanceTypeValue13 = jobTaskValue12["targetInstanceType"];
                                                            if (targetInstanceTypeValue13 != null && targetInstanceTypeValue13.Type != JTokenType.Null)
                                                            {
                                                                string targetInstanceTypeInstance13 = ((string)targetInstanceTypeValue13);
                                                                jobTaskInstance12.TargetInstanceType = targetInstanceTypeInstance13;
                                                            }
                                                            
                                                            JToken jobScenarioNameValue12 = jobTaskValue12["jobScenarioName"];
                                                            if (jobScenarioNameValue12 != null && jobScenarioNameValue12.Type != JTokenType.Null)
                                                            {
                                                                string jobScenarioNameInstance12 = ((string)jobScenarioNameValue12);
                                                                jobTaskInstance12.JobScenarioName = jobScenarioNameInstance12;
                                                            }
                                                        }
                                                        
                                                        JToken instanceTypeValue30 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue30 != null && instanceTypeValue30.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance30 = ((string)instanceTypeValue30);
                                                            fabricReplicationGroupTaskDetailsInstance4.Type = instanceTypeInstance30;
                                                        }
                                                        taskInstance.CustomDetails = fabricReplicationGroupTaskDetailsInstance4;
                                                    }
                                                    if (typeName5 == "ManualActionTaskDetails")
                                                    {
                                                        ManualActionTaskDetails manualActionTaskDetailsInstance4 = new ManualActionTaskDetails();
                                                        
                                                        JToken nameValue19 = customDetailsValue4["name"];
                                                        if (nameValue19 != null && nameValue19.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance19 = ((string)nameValue19);
                                                            manualActionTaskDetailsInstance4.Name = nameInstance19;
                                                        }
                                                        
                                                        JToken instructionsValue4 = customDetailsValue4["instructions"];
                                                        if (instructionsValue4 != null && instructionsValue4.Type != JTokenType.Null)
                                                        {
                                                            string instructionsInstance4 = ((string)instructionsValue4);
                                                            manualActionTaskDetailsInstance4.Instructions = instructionsInstance4;
                                                        }
                                                        
                                                        JToken observationValue4 = customDetailsValue4["observation"];
                                                        if (observationValue4 != null && observationValue4.Type != JTokenType.Null)
                                                        {
                                                            string observationInstance4 = ((string)observationValue4);
                                                            manualActionTaskDetailsInstance4.Observation = observationInstance4;
                                                        }
                                                        
                                                        JToken instanceTypeValue31 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue31 != null && instanceTypeValue31.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance31 = ((string)instanceTypeValue31);
                                                            manualActionTaskDetailsInstance4.Type = instanceTypeInstance31;
                                                        }
                                                        taskInstance.CustomDetails = manualActionTaskDetailsInstance4;
                                                    }
                                                    if (typeName5 == "ScriptActionTaskDetails")
                                                    {
                                                        ScriptActionTaskDetails scriptActionTaskDetailsInstance4 = new ScriptActionTaskDetails();
                                                        
                                                        JToken nameValue20 = customDetailsValue4["name"];
                                                        if (nameValue20 != null && nameValue20.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance20 = ((string)nameValue20);
                                                            scriptActionTaskDetailsInstance4.Name = nameInstance20;
                                                        }
                                                        
                                                        JToken pathValue4 = customDetailsValue4["path"];
                                                        if (pathValue4 != null && pathValue4.Type != JTokenType.Null)
                                                        {
                                                            string pathInstance4 = ((string)pathValue4);
                                                            scriptActionTaskDetailsInstance4.Path = pathInstance4;
                                                        }
                                                        
                                                        JToken outputValue4 = customDetailsValue4["output"];
                                                        if (outputValue4 != null && outputValue4.Type != JTokenType.Null)
                                                        {
                                                            string outputInstance4 = ((string)outputValue4);
                                                            scriptActionTaskDetailsInstance4.Output = outputInstance4;
                                                        }
                                                        
                                                        JToken isPrimarySideScriptValue7 = customDetailsValue4["isPrimarySideScript"];
                                                        if (isPrimarySideScriptValue7 != null && isPrimarySideScriptValue7.Type != JTokenType.Null)
                                                        {
                                                            bool isPrimarySideScriptInstance7 = ((bool)isPrimarySideScriptValue7);
                                                            scriptActionTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance7;
                                                        }
                                                        
                                                        JToken instanceTypeValue32 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue32 != null && instanceTypeValue32.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance32 = ((string)instanceTypeValue32);
                                                            scriptActionTaskDetailsInstance4.Type = instanceTypeInstance32;
                                                        }
                                                        taskInstance.CustomDetails = scriptActionTaskDetailsInstance4;
                                                    }
                                                    if (typeName5 == "VmNicUpdatesTaskDetails")
                                                    {
                                                        VmNicUpdatesTaskDetails vmNicUpdatesTaskDetailsInstance4 = new VmNicUpdatesTaskDetails();
                                                        
                                                        JToken vmIdValue4 = customDetailsValue4["vmId"];
                                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string vmIdInstance4 = ((string)vmIdValue4);
                                                            vmNicUpdatesTaskDetailsInstance4.VmId = vmIdInstance4;
                                                        }
                                                        
                                                        JToken nicIdValue4 = customDetailsValue4["nicId"];
                                                        if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance4 = ((string)nicIdValue4);
                                                            vmNicUpdatesTaskDetailsInstance4.NicId = nicIdInstance4;
                                                        }
                                                        
                                                        JToken nameValue21 = customDetailsValue4["name"];
                                                        if (nameValue21 != null && nameValue21.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance21 = ((string)nameValue21);
                                                            vmNicUpdatesTaskDetailsInstance4.Name = nameInstance21;
                                                        }
                                                        
                                                        JToken instanceTypeValue33 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue33 != null && instanceTypeValue33.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance33 = ((string)instanceTypeValue33);
                                                            vmNicUpdatesTaskDetailsInstance4.Type = instanceTypeInstance33;
                                                        }
                                                        taskInstance.CustomDetails = vmNicUpdatesTaskDetailsInstance4;
                                                    }
                                                    if (typeName5 == "ConsistencyCheckTaskDetails")
                                                    {
                                                        ConsistencyCheckTaskDetails consistencyCheckTaskDetailsInstance4 = new ConsistencyCheckTaskDetails();
                                                        
                                                        JToken vmDetailsArray4 = customDetailsValue4["vmDetails"];
                                                        if (vmDetailsArray4 != null && vmDetailsArray4.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken vmDetailsValue4 in ((JArray)vmDetailsArray4))
                                                            {
                                                                InconsistentVmDetails inconsistentVmDetailsInstance4 = new InconsistentVmDetails();
                                                                consistencyCheckTaskDetailsInstance4.VmDetails.Add(inconsistentVmDetailsInstance4);
                                                                
                                                                JToken vmNameValue4 = vmDetailsValue4["vmName"];
                                                                if (vmNameValue4 != null && vmNameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string vmNameInstance4 = ((string)vmNameValue4);
                                                                    inconsistentVmDetailsInstance4.VmName = vmNameInstance4;
                                                                }
                                                                
                                                                JToken cloudNameValue4 = vmDetailsValue4["cloudName"];
                                                                if (cloudNameValue4 != null && cloudNameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string cloudNameInstance4 = ((string)cloudNameValue4);
                                                                    inconsistentVmDetailsInstance4.CloudName = cloudNameInstance4;
                                                                }
                                                                
                                                                JToken detailsArray4 = vmDetailsValue4["details"];
                                                                if (detailsArray4 != null && detailsArray4.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken detailsValue4 in ((JArray)detailsArray4))
                                                                    {
                                                                        inconsistentVmDetailsInstance4.Details.Add(((string)detailsValue4));
                                                                    }
                                                                }
                                                                
                                                                JToken errorIdsArray4 = vmDetailsValue4["errorIds"];
                                                                if (errorIdsArray4 != null && errorIdsArray4.Type != JTokenType.Null)
                                                                {
                                                                    foreach (JToken errorIdsValue4 in ((JArray)errorIdsArray4))
                                                                    {
                                                                        inconsistentVmDetailsInstance4.ErrorIds.Add(((string)errorIdsValue4));
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken instanceTypeValue34 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue34 != null && instanceTypeValue34.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance34 = ((string)instanceTypeValue34);
                                                            consistencyCheckTaskDetailsInstance4.Type = instanceTypeInstance34;
                                                        }
                                                        taskInstance.CustomDetails = consistencyCheckTaskDetailsInstance4;
                                                    }
                                                    if (typeName5 == "AutomationRunbookTaskDetails")
                                                    {
                                                        AutomationRunbookTaskDetails automationRunbookTaskDetailsInstance4 = new AutomationRunbookTaskDetails();
                                                        
                                                        JToken nameValue22 = customDetailsValue4["name"];
                                                        if (nameValue22 != null && nameValue22.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance22 = ((string)nameValue22);
                                                            automationRunbookTaskDetailsInstance4.Name = nameInstance22;
                                                        }
                                                        
                                                        JToken cloudServiceNameValue4 = customDetailsValue4["cloudServiceName"];
                                                        if (cloudServiceNameValue4 != null && cloudServiceNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string cloudServiceNameInstance4 = ((string)cloudServiceNameValue4);
                                                            automationRunbookTaskDetailsInstance4.CloudServiceName = cloudServiceNameInstance4;
                                                        }
                                                        
                                                        JToken subscriptionIdValue4 = customDetailsValue4["subscriptionId"];
                                                        if (subscriptionIdValue4 != null && subscriptionIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string subscriptionIdInstance4 = ((string)subscriptionIdValue4);
                                                            automationRunbookTaskDetailsInstance4.SubscriptionId = subscriptionIdInstance4;
                                                        }
                                                        
                                                        JToken accountNameValue4 = customDetailsValue4["accountName"];
                                                        if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountNameInstance4 = ((string)accountNameValue4);
                                                            automationRunbookTaskDetailsInstance4.AccountName = accountNameInstance4;
                                                        }
                                                        
                                                        JToken runbookIdValue4 = customDetailsValue4["runbookId"];
                                                        if (runbookIdValue4 != null && runbookIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string runbookIdInstance4 = ((string)runbookIdValue4);
                                                            automationRunbookTaskDetailsInstance4.RunbookId = runbookIdInstance4;
                                                        }
                                                        
                                                        JToken runbookNameValue4 = customDetailsValue4["runbookName"];
                                                        if (runbookNameValue4 != null && runbookNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string runbookNameInstance4 = ((string)runbookNameValue4);
                                                            automationRunbookTaskDetailsInstance4.RunbookName = runbookNameInstance4;
                                                        }
                                                        
                                                        JToken jobIdValue16 = customDetailsValue4["jobId"];
                                                        if (jobIdValue16 != null && jobIdValue16.Type != JTokenType.Null)
                                                        {
                                                            string jobIdInstance16 = ((string)jobIdValue16);
                                                            automationRunbookTaskDetailsInstance4.JobId = jobIdInstance16;
                                                        }
                                                        
                                                        JToken jobOutputValue4 = customDetailsValue4["jobOutput"];
                                                        if (jobOutputValue4 != null && jobOutputValue4.Type != JTokenType.Null)
                                                        {
                                                            string jobOutputInstance4 = ((string)jobOutputValue4);
                                                            automationRunbookTaskDetailsInstance4.JobOutput = jobOutputInstance4;
                                                        }
                                                        
                                                        JToken isPrimarySideScriptValue8 = customDetailsValue4["isPrimarySideScript"];
                                                        if (isPrimarySideScriptValue8 != null && isPrimarySideScriptValue8.Type != JTokenType.Null)
                                                        {
                                                            bool isPrimarySideScriptInstance8 = ((bool)isPrimarySideScriptValue8);
                                                            automationRunbookTaskDetailsInstance4.IsPrimarySideScript = isPrimarySideScriptInstance8;
                                                        }
                                                        
                                                        JToken instanceTypeValue35 = customDetailsValue4["instanceType"];
                                                        if (instanceTypeValue35 != null && instanceTypeValue35.Type != JTokenType.Null)
                                                        {
                                                            string instanceTypeInstance35 = ((string)instanceTypeValue35);
                                                            automationRunbookTaskDetailsInstance4.Type = instanceTypeInstance35;
                                                        }
                                                        taskInstance.CustomDetails = automationRunbookTaskDetailsInstance4;
                                                    }
                                                }
                                            }
                                        }
                                        
                                        JToken errorsArray5 = propertiesValue["errors"];
                                        if (errorsArray5 != null && errorsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken errorsValue5 in ((JArray)errorsArray5))
                                            {
                                                JobErrorDetails jobErrorDetailsInstance5 = new JobErrorDetails();
                                                propertiesInstance.Errors.Add(jobErrorDetailsInstance5);
                                                
                                                JToken serviceErrorDetailsValue5 = errorsValue5["serviceErrorDetails"];
                                                if (serviceErrorDetailsValue5 != null && serviceErrorDetailsValue5.Type != JTokenType.Null)
                                                {
                                                    ServiceError serviceErrorDetailsInstance5 = new ServiceError();
                                                    jobErrorDetailsInstance5.ServiceErrorDetails = serviceErrorDetailsInstance5;
                                                    
                                                    JToken codeValue5 = serviceErrorDetailsValue5["code"];
                                                    if (codeValue5 != null && codeValue5.Type != JTokenType.Null)
                                                    {
                                                        string codeInstance5 = ((string)codeValue5);
                                                        serviceErrorDetailsInstance5.Code = codeInstance5;
                                                    }
                                                    
                                                    JToken messageValue5 = serviceErrorDetailsValue5["message"];
                                                    if (messageValue5 != null && messageValue5.Type != JTokenType.Null)
                                                    {
                                                        string messageInstance5 = ((string)messageValue5);
                                                        serviceErrorDetailsInstance5.Message = messageInstance5;
                                                    }
                                                    
                                                    JToken possibleCausesValue9 = serviceErrorDetailsValue5["possibleCauses"];
                                                    if (possibleCausesValue9 != null && possibleCausesValue9.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance9 = ((string)possibleCausesValue9);
                                                        serviceErrorDetailsInstance5.PossibleCauses = possibleCausesInstance9;
                                                    }
                                                    
                                                    JToken recommendedActionValue9 = serviceErrorDetailsValue5["recommendedAction"];
                                                    if (recommendedActionValue9 != null && recommendedActionValue9.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance9 = ((string)recommendedActionValue9);
                                                        serviceErrorDetailsInstance5.RecommendedAction = recommendedActionInstance9;
                                                    }
                                                    
                                                    JToken activityIdValue6 = serviceErrorDetailsValue5["activityId"];
                                                    if (activityIdValue6 != null && activityIdValue6.Type != JTokenType.Null)
                                                    {
                                                        string activityIdInstance6 = ((string)activityIdValue6);
                                                        serviceErrorDetailsInstance5.ActivityId = activityIdInstance6;
                                                    }
                                                }
                                                
                                                JToken providerErrorDetailsValue5 = errorsValue5["providerErrorDetails"];
                                                if (providerErrorDetailsValue5 != null && providerErrorDetailsValue5.Type != JTokenType.Null)
                                                {
                                                    ProviderError providerErrorDetailsInstance5 = new ProviderError();
                                                    jobErrorDetailsInstance5.ProviderErrorDetails = providerErrorDetailsInstance5;
                                                    
                                                    JToken errorCodeValue5 = providerErrorDetailsValue5["errorCode"];
                                                    if (errorCodeValue5 != null && errorCodeValue5.Type != JTokenType.Null)
                                                    {
                                                        int errorCodeInstance5 = ((int)errorCodeValue5);
                                                        providerErrorDetailsInstance5.ErrorCode = errorCodeInstance5;
                                                    }
                                                    
                                                    JToken errorMessageValue5 = providerErrorDetailsValue5["errorMessage"];
                                                    if (errorMessageValue5 != null && errorMessageValue5.Type != JTokenType.Null)
                                                    {
                                                        string errorMessageInstance5 = ((string)errorMessageValue5);
                                                        providerErrorDetailsInstance5.ErrorMessage = errorMessageInstance5;
                                                    }
                                                    
                                                    JToken errorIdValue5 = providerErrorDetailsValue5["errorId"];
                                                    if (errorIdValue5 != null && errorIdValue5.Type != JTokenType.Null)
                                                    {
                                                        string errorIdInstance5 = ((string)errorIdValue5);
                                                        providerErrorDetailsInstance5.ErrorId = errorIdInstance5;
                                                    }
                                                    
                                                    JToken possibleCausesValue10 = providerErrorDetailsValue5["possibleCauses"];
                                                    if (possibleCausesValue10 != null && possibleCausesValue10.Type != JTokenType.Null)
                                                    {
                                                        string possibleCausesInstance10 = ((string)possibleCausesValue10);
                                                        providerErrorDetailsInstance5.PossibleCauses = possibleCausesInstance10;
                                                    }
                                                    
                                                    JToken recommendedActionValue10 = providerErrorDetailsValue5["recommendedAction"];
                                                    if (recommendedActionValue10 != null && recommendedActionValue10.Type != JTokenType.Null)
                                                    {
                                                        string recommendedActionInstance10 = ((string)recommendedActionValue10);
                                                        providerErrorDetailsInstance5.RecommendedAction = recommendedActionInstance10;
                                                    }
                                                }
                                                
                                                JToken errorLevelValue5 = errorsValue5["errorLevel"];
                                                if (errorLevelValue5 != null && errorLevelValue5.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance5 = ((string)errorLevelValue5);
                                                    jobErrorDetailsInstance5.ErrorLevel = errorLevelInstance5;
                                                }
                                                
                                                JToken creationTimeValue5 = errorsValue5["creationTime"];
                                                if (creationTimeValue5 != null && creationTimeValue5.Type != JTokenType.Null)
                                                {
                                                    DateTime creationTimeInstance5 = ((DateTime)creationTimeValue5);
                                                    jobErrorDetailsInstance5.CreationTime = creationTimeInstance5;
                                                }
                                                
                                                JToken taskIdValue9 = errorsValue5["taskId"];
                                                if (taskIdValue9 != null && taskIdValue9.Type != JTokenType.Null)
                                                {
                                                    string taskIdInstance9 = ((string)taskIdValue9);
                                                    jobErrorDetailsInstance5.TaskId = taskIdInstance9;
                                                }
                                            }
                                        }
                                        
                                        JToken customDetailsValue5 = propertiesValue["customDetails"];
                                        if (customDetailsValue5 != null && customDetailsValue5.Type != JTokenType.Null)
                                        {
                                            string typeName6 = ((string)customDetailsValue5["instanceType"]);
                                            if (typeName6 == "ExportJobDetails")
                                            {
                                                ExportJobDetails exportJobDetailsInstance = new ExportJobDetails();
                                                
                                                JToken blobUriValue = customDetailsValue5["blobUri"];
                                                if (blobUriValue != null && blobUriValue.Type != JTokenType.Null)
                                                {
                                                    string blobUriInstance = ((string)blobUriValue);
                                                    exportJobDetailsInstance.BlobUri = blobUriInstance;
                                                }
                                                
                                                JToken sasTokenValue = customDetailsValue5["sasToken"];
                                                if (sasTokenValue != null && sasTokenValue.Type != JTokenType.Null)
                                                {
                                                    string sasTokenInstance = ((string)sasTokenValue);
                                                    exportJobDetailsInstance.SasToken = sasTokenInstance;
                                                }
                                                
                                                JToken instanceTypeValue36 = customDetailsValue5["instanceType"];
                                                if (instanceTypeValue36 != null && instanceTypeValue36.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance36 = ((string)instanceTypeValue36);
                                                    exportJobDetailsInstance.InstanceType = instanceTypeInstance36;
                                                }
                                                
                                                JToken affectedObjectDetailsSequenceElement = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                                if (affectedObjectDetailsSequenceElement != null && affectedObjectDetailsSequenceElement.Type != JTokenType.Null)
                                                {
                                                    foreach (JProperty property in affectedObjectDetailsSequenceElement)
                                                    {
                                                        string affectedObjectDetailsKey = ((string)property.Name);
                                                        string affectedObjectDetailsValue = ((string)property.Value);
                                                        exportJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey, affectedObjectDetailsValue);
                                                    }
                                                }
                                                propertiesInstance.CustomDetails = exportJobDetailsInstance;
                                            }
                                            if (typeName6 == "AsrJobDetails")
                                            {
                                                AsrJobDetails asrJobDetailsInstance = new AsrJobDetails();
                                                
                                                JToken instanceTypeValue37 = customDetailsValue5["instanceType"];
                                                if (instanceTypeValue37 != null && instanceTypeValue37.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance37 = ((string)instanceTypeValue37);
                                                    asrJobDetailsInstance.InstanceType = instanceTypeInstance37;
                                                }
                                                
                                                JToken affectedObjectDetailsSequenceElement2 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                                if (affectedObjectDetailsSequenceElement2 != null && affectedObjectDetailsSequenceElement2.Type != JTokenType.Null)
                                                {
                                                    foreach (JProperty property2 in affectedObjectDetailsSequenceElement2)
                                                    {
                                                        string affectedObjectDetailsKey2 = ((string)property2.Name);
                                                        string affectedObjectDetailsValue2 = ((string)property2.Value);
                                                        asrJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey2, affectedObjectDetailsValue2);
                                                    }
                                                }
                                                propertiesInstance.CustomDetails = asrJobDetailsInstance;
                                            }
                                            if (typeName6 == "TestFailoverJobDetails")
                                            {
                                                TestFailoverJobDetails testFailoverJobDetailsInstance = new TestFailoverJobDetails();
                                                
                                                JToken testFailoverStatusValue = customDetailsValue5["testFailoverStatus"];
                                                if (testFailoverStatusValue != null && testFailoverStatusValue.Type != JTokenType.Null)
                                                {
                                                    string testFailoverStatusInstance = ((string)testFailoverStatusValue);
                                                    testFailoverJobDetailsInstance.TestFailoverStatus = testFailoverStatusInstance;
                                                }
                                                
                                                JToken commentsValue = customDetailsValue5["comments"];
                                                if (commentsValue != null && commentsValue.Type != JTokenType.Null)
                                                {
                                                    string commentsInstance = ((string)commentsValue);
                                                    testFailoverJobDetailsInstance.Comments = commentsInstance;
                                                }
                                                
                                                JToken networkNameValue = customDetailsValue5["networkName"];
                                                if (networkNameValue != null && networkNameValue.Type != JTokenType.Null)
                                                {
                                                    string networkNameInstance = ((string)networkNameValue);
                                                    testFailoverJobDetailsInstance.NetworkName = networkNameInstance;
                                                }
                                                
                                                JToken networkFriendlyNameValue = customDetailsValue5["networkFriendlyName"];
                                                if (networkFriendlyNameValue != null && networkFriendlyNameValue.Type != JTokenType.Null)
                                                {
                                                    string networkFriendlyNameInstance = ((string)networkFriendlyNameValue);
                                                    testFailoverJobDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance;
                                                }
                                                
                                                JToken networkTypeValue = customDetailsValue5["networkType"];
                                                if (networkTypeValue != null && networkTypeValue.Type != JTokenType.Null)
                                                {
                                                    string networkTypeInstance = ((string)networkTypeValue);
                                                    testFailoverJobDetailsInstance.NetworkType = networkTypeInstance;
                                                }
                                                
                                                JToken protectedItemDetailsArray = customDetailsValue5["protectedItemDetails"];
                                                if (protectedItemDetailsArray != null && protectedItemDetailsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedItemDetailsValue in ((JArray)protectedItemDetailsArray))
                                                    {
                                                        TestFailoverReplicationProtectedItemDetails testFailoverReplicationProtectedItemDetailsInstance = new TestFailoverReplicationProtectedItemDetails();
                                                        testFailoverJobDetailsInstance.ProtectedItemDetails.Add(testFailoverReplicationProtectedItemDetailsInstance);
                                                        
                                                        JToken nameValue23 = protectedItemDetailsValue["name"];
                                                        if (nameValue23 != null && nameValue23.Type != JTokenType.Null)
                                                        {
                                                            string nameInstance23 = ((string)nameValue23);
                                                            testFailoverReplicationProtectedItemDetailsInstance.Name = nameInstance23;
                                                        }
                                                        
                                                        JToken friendlyNameValue6 = protectedItemDetailsValue["friendlyName"];
                                                        if (friendlyNameValue6 != null && friendlyNameValue6.Type != JTokenType.Null)
                                                        {
                                                            string friendlyNameInstance6 = ((string)friendlyNameValue6);
                                                            testFailoverReplicationProtectedItemDetailsInstance.FriendlyName = friendlyNameInstance6;
                                                        }
                                                        
                                                        JToken testVmNameValue = protectedItemDetailsValue["testVmName"];
                                                        if (testVmNameValue != null && testVmNameValue.Type != JTokenType.Null)
                                                        {
                                                            string testVmNameInstance = ((string)testVmNameValue);
                                                            testFailoverReplicationProtectedItemDetailsInstance.TestVmName = testVmNameInstance;
                                                        }
                                                        
                                                        JToken testVmFriendlyNameValue = protectedItemDetailsValue["testVmFriendlyName"];
                                                        if (testVmFriendlyNameValue != null && testVmFriendlyNameValue.Type != JTokenType.Null)
                                                        {
                                                            string testVmFriendlyNameInstance = ((string)testVmFriendlyNameValue);
                                                            testFailoverReplicationProtectedItemDetailsInstance.TestVmFriendlyName = testVmFriendlyNameInstance;
                                                        }
                                                        
                                                        JToken networkConnectionStatusValue = protectedItemDetailsValue["networkConnectionStatus"];
                                                        if (networkConnectionStatusValue != null && networkConnectionStatusValue.Type != JTokenType.Null)
                                                        {
                                                            string networkConnectionStatusInstance = ((string)networkConnectionStatusValue);
                                                            testFailoverReplicationProtectedItemDetailsInstance.NetworkConnectionStatus = networkConnectionStatusInstance;
                                                        }
                                                        
                                                        JToken networkFriendlyNameValue2 = protectedItemDetailsValue["networkFriendlyName"];
                                                        if (networkFriendlyNameValue2 != null && networkFriendlyNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string networkFriendlyNameInstance2 = ((string)networkFriendlyNameValue2);
                                                            testFailoverReplicationProtectedItemDetailsInstance.NetworkFriendlyName = networkFriendlyNameInstance2;
                                                        }
                                                        
                                                        JToken subnetValue = protectedItemDetailsValue["subnet"];
                                                        if (subnetValue != null && subnetValue.Type != JTokenType.Null)
                                                        {
                                                            string subnetInstance = ((string)subnetValue);
                                                            testFailoverReplicationProtectedItemDetailsInstance.Subnet = subnetInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue38 = customDetailsValue5["instanceType"];
                                                if (instanceTypeValue38 != null && instanceTypeValue38.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance38 = ((string)instanceTypeValue38);
                                                    testFailoverJobDetailsInstance.InstanceType = instanceTypeInstance38;
                                                }
                                                
                                                JToken affectedObjectDetailsSequenceElement3 = ((JToken)customDetailsValue5["affectedObjectDetails"]);
                                                if (affectedObjectDetailsSequenceElement3 != null && affectedObjectDetailsSequenceElement3.Type != JTokenType.Null)
                                                {
                                                    foreach (JProperty property3 in affectedObjectDetailsSequenceElement3)
                                                    {
                                                        string affectedObjectDetailsKey3 = ((string)property3.Name);
                                                        string affectedObjectDetailsValue3 = ((string)property3.Value);
                                                        testFailoverJobDetailsInstance.AffectedObjectDetails.Add(affectedObjectDetailsKey3, affectedObjectDetailsValue3);
                                                    }
                                                }
                                                propertiesInstance.CustomDetails = testFailoverJobDetailsInstance;
                                            }
                                        }
                                        
                                        JToken idValue = propertiesValue["id"];
                                        if (idValue != null && idValue.Type != JTokenType.Null)
                                        {
                                            string idInstance = ((string)idValue);
                                            propertiesInstance.Id = idInstance;
                                        }
                                        
                                        JToken nameValue24 = propertiesValue["name"];
                                        if (nameValue24 != null && nameValue24.Type != JTokenType.Null)
                                        {
                                            string nameInstance24 = ((string)nameValue24);
                                            propertiesInstance.Name = nameInstance24;
                                        }
                                        
                                        JToken typeValue = propertiesValue["type"];
                                        if (typeValue != null && typeValue.Type != JTokenType.Null)
                                        {
                                            string typeInstance = ((string)typeValue);
                                            propertiesInstance.Type = typeInstance;
                                        }
                                        
                                        JToken locationValue = propertiesValue["location"];
                                        if (locationValue != null && locationValue.Type != JTokenType.Null)
                                        {
                                            string locationInstance = ((string)locationValue);
                                            propertiesInstance.Location = locationInstance;
                                        }
                                        
                                        JToken tagsSequenceElement = ((JToken)propertiesValue["tags"]);
                                        if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property4 in tagsSequenceElement)
                                            {
                                                string tagsKey = ((string)property4.Name);
                                                string tagsValue = ((string)property4.Value);
                                                propertiesInstance.Tags.Add(tagsKey, tagsValue);
                                            }
                                        }
                                    }
                                    
                                    JToken idValue2 = valueValue["id"];
                                    if (idValue2 != null && idValue2.Type != JTokenType.Null)
                                    {
                                        string idInstance2 = ((string)idValue2);
                                        jobInstance.Id = idInstance2;
                                    }
                                    
                                    JToken nameValue25 = valueValue["name"];
                                    if (nameValue25 != null && nameValue25.Type != JTokenType.Null)
                                    {
                                        string nameInstance25 = ((string)nameValue25);
                                        jobInstance.Name = nameInstance25;
                                    }
                                    
                                    JToken typeValue2 = valueValue["type"];
                                    if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                    {
                                        string typeInstance2 = ((string)typeValue2);
                                        jobInstance.Type = typeInstance2;
                                    }
                                    
                                    JToken locationValue2 = valueValue["location"];
                                    if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                                    {
                                        string locationInstance2 = ((string)locationValue2);
                                        jobInstance.Location = locationInstance2;
                                    }
                                    
                                    JToken tagsSequenceElement2 = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement2 != null && tagsSequenceElement2.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property5 in tagsSequenceElement2)
                                        {
                                            string tagsKey2 = ((string)property5.Name);
                                            string tagsValue2 = ((string)property5.Value);
                                            jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Restart the job .
        /// </summary>
        /// <param name='jobName'>
        /// Required. Job Name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> RestartAsync(string jobName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "RestartAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.Jobs.BeginRestartingAsync(jobName, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            JobOperationResponse result = await client.Jobs.GetRestartStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.Jobs.GetRestartStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Resume the job .
        /// </summary>
        /// <param name='jobId'>
        /// Required. Job ID.
        /// </param>
        /// <param name='resumeJobParameters'>
        /// Optional. Resume job parameters.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> ResumeAsync(string jobId, ResumeJobParams resumeJobParameters, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("jobId", jobId);
                tracingParameters.Add("resumeJobParameters", resumeJobParameters);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ResumeAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.Jobs.BeginResumingAsync(jobId, resumeJobParameters, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            JobOperationResponse result = await client.Jobs.GetResumeStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.Jobs.GetResumeStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
    }
}
