// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure;
using Microsoft.Azure.Management.SiteRecovery;
using Microsoft.Azure.Management.SiteRecovery.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Management.SiteRecovery
{
    /// <summary>
    /// Definition of Replication protected item operations for the Site
    /// Recovery extension.
    /// </summary>
    internal partial class ReplicationProtectedItemOperations : IServiceOperations<SiteRecoveryManagementClient>, IReplicationProtectedItemOperations
    {
        /// <summary>
        /// Initializes a new instance of the
        /// ReplicationProtectedItemOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal ReplicationProtectedItemOperations(SiteRecoveryManagementClient client)
        {
            this._client = client;
        }
        
        private SiteRecoveryManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.Azure.Management.SiteRecovery.SiteRecoveryManagementClient.
        /// </summary>
        public SiteRecoveryManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Apply recovery point for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Apply recovery point input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> ApplyRecoveryPointAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, ApplyRecoveryPointInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ApplyRecoveryPointAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginApplyRecoveryPointAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetApplyRecoveryPointStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetApplyRecoveryPointStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Apply recovery point for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Apply recovery point input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginApplyRecoveryPointAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, ApplyRecoveryPointInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginApplyRecoveryPointAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/applyRecoveryPoint";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject applyRecoveryPointInputValue = new JObject();
                requestDoc = applyRecoveryPointInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    applyRecoveryPointInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.RecoveryPointId != null)
                    {
                        propertiesValue["recoveryPointId"] = input.Properties.RecoveryPointId;
                    }
                    
                    if (input.Properties.ProviderSpecificDetails != null)
                    {
                        JObject providerSpecificDetailsValue = new JObject();
                        propertiesValue["providerSpecificDetails"] = providerSpecificDetailsValue;
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureApplyRecoveryPointInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzure";
                            HyperVReplicaAzureApplyRecoveryPointInput derived = ((HyperVReplicaAzureApplyRecoveryPointInput)input.Properties.ProviderSpecificDetails);
                            
                            providerSpecificDetailsValue["vaultLocation"] = derived.VaultLocation;
                            
                            if (derived.PrimaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["primaryKekCertificatePfx"] = derived.PrimaryKekCertificatePfx;
                            }
                            
                            if (derived.SecondaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["secondaryKekCertificatePfx"] = derived.SecondaryKekCertificatePfx;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageAzureV2ApplyRecoveryPointInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMageAzureV2";
                            InMageAzureV2ApplyRecoveryPointInput derived2 = ((InMageAzureV2ApplyRecoveryPointInput)input.Properties.ProviderSpecificDetails);
                            
                            providerSpecificDetailsValue["vaultLocation"] = derived2.VaultLocation;
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Execute commit failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginCommitFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginCommitFailoverAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/failoverCommit";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Disable Protection for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='input'>
        /// Required. Disable protection input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginDisableProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, DisableProtectionInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginDisableProtectionAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/remove";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject disableProtectionInputValue = new JObject();
                requestDoc = disableProtectionInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    disableProtectionInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.DisableProtectionReason != null)
                    {
                        propertiesValue["DisableProtectionReason"] = input.Properties.DisableProtectionReason;
                    }
                    
                    if (input.Properties.ProviderSettings != null)
                    {
                        JObject replicationProviderInputValue = new JObject();
                        propertiesValue["replicationProviderInput"] = replicationProviderInputValue;
                        if (input.Properties.ProviderSettings is InMageDisableProtectionInput)
                        {
                            replicationProviderInputValue["instanceType"] = "InMage";
                            InMageDisableProtectionInput derived = ((InMageDisableProtectionInput)input.Properties.ProviderSettings);
                            
                            if (derived.ReplicaVmDeletionStatus != null)
                            {
                                replicationProviderInputValue["replicaVmDeletionStatus"] = derived.ReplicaVmDeletionStatus;
                            }
                            
                            if (derived.InstanceType != null)
                            {
                                replicationProviderInputValue["instanceType"] = derived.InstanceType;
                            }
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Enable Protection for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='input'>
        /// Required. Enable protection input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginEnableProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, EnableProtectionInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginEnableProtectionAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject enableProtectionInputValue = new JObject();
                requestDoc = enableProtectionInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    enableProtectionInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.PolicyId != null)
                    {
                        propertiesValue["policyId"] = input.Properties.PolicyId;
                    }
                    
                    if (input.Properties.ProtectableItemId != null)
                    {
                        propertiesValue["protectableItemId"] = input.Properties.ProtectableItemId;
                    }
                    
                    if (input.Properties.ProviderSpecificDetails != null)
                    {
                        JObject providerSpecificDetailsValue = new JObject();
                        propertiesValue["providerSpecificDetails"] = providerSpecificDetailsValue;
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureEnableProtectionInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzure";
                            HyperVReplicaAzureEnableProtectionInput derived = ((HyperVReplicaAzureEnableProtectionInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived.HvHostVmId != null)
                            {
                                providerSpecificDetailsValue["hvHostVmId"] = derived.HvHostVmId;
                            }
                            
                            if (derived.VmName != null)
                            {
                                providerSpecificDetailsValue["vmName"] = derived.VmName;
                            }
                            
                            if (derived.OSType != null)
                            {
                                providerSpecificDetailsValue["osType"] = derived.OSType;
                            }
                            
                            if (derived.VhdId != null)
                            {
                                providerSpecificDetailsValue["vhdId"] = derived.VhdId;
                            }
                            
                            if (derived.TargetStorageAccountId != null)
                            {
                                providerSpecificDetailsValue["targetStorageAccountId"] = derived.TargetStorageAccountId;
                            }
                            
                            if (derived.TargetAzureVmName != null)
                            {
                                providerSpecificDetailsValue["targetAzureVmName"] = derived.TargetAzureVmName;
                            }
                            
                            if (derived.TargetAzureNetworkId != null)
                            {
                                providerSpecificDetailsValue["targetAzureNetworkId"] = derived.TargetAzureNetworkId;
                            }
                            
                            if (derived.TargetAzureSubnetId != null)
                            {
                                providerSpecificDetailsValue["targetAzureSubnetId"] = derived.TargetAzureSubnetId;
                            }
                            
                            if (derived.EnableRDPOnTargetOption != null)
                            {
                                providerSpecificDetailsValue["enableRDPOnTargetOption"] = derived.EnableRDPOnTargetOption;
                            }
                            
                            if (derived.DisksToInclude != null)
                            {
                                if (derived.DisksToInclude is ILazyCollection == false || ((ILazyCollection)derived.DisksToInclude).IsInitialized)
                                {
                                    JArray disksToIncludeArray = new JArray();
                                    foreach (string disksToIncludeItem in derived.DisksToInclude)
                                    {
                                        disksToIncludeArray.Add(disksToIncludeItem);
                                    }
                                    providerSpecificDetailsValue["disksToInclude"] = disksToIncludeArray;
                                }
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageAzureV2EnableProtectionInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMageAzureV2";
                            InMageAzureV2EnableProtectionInput derived2 = ((InMageAzureV2EnableProtectionInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived2.MasterTargetId != null)
                            {
                                providerSpecificDetailsValue["masterTargetId"] = derived2.MasterTargetId;
                            }
                            
                            if (derived2.ProcessServerId != null)
                            {
                                providerSpecificDetailsValue["processServerId"] = derived2.ProcessServerId;
                            }
                            
                            if (derived2.StorageAccountId != null)
                            {
                                providerSpecificDetailsValue["storageAccountId"] = derived2.StorageAccountId;
                            }
                            
                            if (derived2.StorageSubscriptionId != null)
                            {
                                providerSpecificDetailsValue["storageSubscriptionId"] = derived2.StorageSubscriptionId;
                            }
                            
                            if (derived2.RunAsAccountId != null)
                            {
                                providerSpecificDetailsValue["runAsAccountId"] = derived2.RunAsAccountId;
                            }
                            
                            if (derived2.MultiVmGroupId != null)
                            {
                                providerSpecificDetailsValue["multiVmGroupId"] = derived2.MultiVmGroupId;
                            }
                            
                            if (derived2.MultiVmGroupName != null)
                            {
                                providerSpecificDetailsValue["multiVmGroupName"] = derived2.MultiVmGroupName;
                            }
                            
                            if (derived2.TargetAzureVmName != null)
                            {
                                providerSpecificDetailsValue["targetAzureVmName"] = derived2.TargetAzureVmName;
                            }
                            
                            if (derived2.TargetAzureNetworkId != null)
                            {
                                providerSpecificDetailsValue["targetAzureNetworkId"] = derived2.TargetAzureNetworkId;
                            }
                            
                            if (derived2.TargetAzureSubnetId != null)
                            {
                                providerSpecificDetailsValue["targetAzureSubnetId"] = derived2.TargetAzureSubnetId;
                            }
                            
                            if (derived2.EnableRDPOnTargetOption != null)
                            {
                                providerSpecificDetailsValue["enableRDPOnTargetOption"] = derived2.EnableRDPOnTargetOption;
                            }
                            
                            if (derived2.LogStorageAccountId != null)
                            {
                                providerSpecificDetailsValue["logStorageAccountId"] = derived2.LogStorageAccountId;
                            }
                            
                            if (derived2.DisksToInclude != null)
                            {
                                if (derived2.DisksToInclude is ILazyCollection == false || ((ILazyCollection)derived2.DisksToInclude).IsInitialized)
                                {
                                    JArray disksToIncludeArray2 = new JArray();
                                    foreach (string disksToIncludeItem2 in derived2.DisksToInclude)
                                    {
                                        disksToIncludeArray2.Add(disksToIncludeItem2);
                                    }
                                    providerSpecificDetailsValue["disksToInclude"] = disksToIncludeArray2;
                                }
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageEnableProtectionInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMage";
                            InMageEnableProtectionInput derived3 = ((InMageEnableProtectionInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived3.VmFriendlyName != null)
                            {
                                providerSpecificDetailsValue["vmFriendlyName"] = derived3.VmFriendlyName;
                            }
                            
                            if (derived3.MasterTargetId != null)
                            {
                                providerSpecificDetailsValue["masterTargetId"] = derived3.MasterTargetId;
                            }
                            
                            if (derived3.ProcessServerId != null)
                            {
                                providerSpecificDetailsValue["processServerId"] = derived3.ProcessServerId;
                            }
                            
                            if (derived3.RetentionDrive != null)
                            {
                                providerSpecificDetailsValue["retentionDrive"] = derived3.RetentionDrive;
                            }
                            
                            if (derived3.RunAsAccountId != null)
                            {
                                providerSpecificDetailsValue["runAsAccountId"] = derived3.RunAsAccountId;
                            }
                            
                            if (derived3.MultiVmGroupId != null)
                            {
                                providerSpecificDetailsValue["multiVmGroupId"] = derived3.MultiVmGroupId;
                            }
                            
                            if (derived3.MultiVmGroupName != null)
                            {
                                providerSpecificDetailsValue["multiVmGroupName"] = derived3.MultiVmGroupName;
                            }
                            
                            if (derived3.DatastoreName != null)
                            {
                                providerSpecificDetailsValue["datastoreName"] = derived3.DatastoreName;
                            }
                            
                            if (derived3.DiskExclusionInput != null)
                            {
                                JObject diskExclusionInputValue = new JObject();
                                providerSpecificDetailsValue["diskExclusionInput"] = diskExclusionInputValue;
                                
                                if (derived3.DiskExclusionInput.VolumeOptions != null)
                                {
                                    if (derived3.DiskExclusionInput.VolumeOptions is ILazyCollection == false || ((ILazyCollection)derived3.DiskExclusionInput.VolumeOptions).IsInitialized)
                                    {
                                        JArray volumeOptionsArray = new JArray();
                                        foreach (InMageVolumeExclusionOptions volumeOptionsItem in derived3.DiskExclusionInput.VolumeOptions)
                                        {
                                            JObject inMageVolumeExclusionOptionsValue = new JObject();
                                            volumeOptionsArray.Add(inMageVolumeExclusionOptionsValue);
                                            
                                            if (volumeOptionsItem.VolumeLabel != null)
                                            {
                                                inMageVolumeExclusionOptionsValue["volumeLabel"] = volumeOptionsItem.VolumeLabel;
                                            }
                                            
                                            if (volumeOptionsItem.OnlyExcludeIfSingleVolume != null)
                                            {
                                                inMageVolumeExclusionOptionsValue["OnlyExcludeIfSingleVolume"] = volumeOptionsItem.OnlyExcludeIfSingleVolume;
                                            }
                                        }
                                        diskExclusionInputValue["volumeOptions"] = volumeOptionsArray;
                                    }
                                }
                                
                                if (derived3.DiskExclusionInput.DiskSignatureOptions != null)
                                {
                                    if (derived3.DiskExclusionInput.DiskSignatureOptions is ILazyCollection == false || ((ILazyCollection)derived3.DiskExclusionInput.DiskSignatureOptions).IsInitialized)
                                    {
                                        JArray diskSignatureOptionsArray = new JArray();
                                        foreach (InMageDiskSignatureExclusionOptions diskSignatureOptionsItem in derived3.DiskExclusionInput.DiskSignatureOptions)
                                        {
                                            JObject inMageDiskSignatureExclusionOptionsValue = new JObject();
                                            diskSignatureOptionsArray.Add(inMageDiskSignatureExclusionOptionsValue);
                                            
                                            if (diskSignatureOptionsItem.DiskSignature != null)
                                            {
                                                inMageDiskSignatureExclusionOptionsValue["diskSignature"] = diskSignatureOptionsItem.DiskSignature;
                                            }
                                        }
                                        diskExclusionInputValue["diskSignatureOptions"] = diskSignatureOptionsArray;
                                    }
                                }
                            }
                            
                            if (derived3.DisksToInclude != null)
                            {
                                if (derived3.DisksToInclude is ILazyCollection == false || ((ILazyCollection)derived3.DisksToInclude).IsInitialized)
                                {
                                    JArray disksToIncludeArray3 = new JArray();
                                    foreach (string disksToIncludeItem3 in derived3.DisksToInclude)
                                    {
                                        disksToIncludeArray3.Add(disksToIncludeItem3);
                                    }
                                    providerSpecificDetailsValue["disksToInclude"] = disksToIncludeArray3;
                                }
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is A2AEnableProtectionInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "A2A";
                            A2AEnableProtectionInput derived4 = ((A2AEnableProtectionInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived4.FabricObjectId != null)
                            {
                                providerSpecificDetailsValue["fabricObjectId"] = derived4.FabricObjectId;
                            }
                            
                            if (derived4.RecoveryContainerId != null)
                            {
                                providerSpecificDetailsValue["recoveryContainerId"] = derived4.RecoveryContainerId;
                            }
                            
                            if (derived4.RecoveryResourceGroupId != null)
                            {
                                providerSpecificDetailsValue["recoveryResourceGroupId"] = derived4.RecoveryResourceGroupId;
                            }
                            
                            if (derived4.RecoveryCloudServiceId != null)
                            {
                                providerSpecificDetailsValue["recoveryCloudServiceId"] = derived4.RecoveryCloudServiceId;
                            }
                            
                            if (derived4.RecoveryAvailabilitySetId != null)
                            {
                                providerSpecificDetailsValue["recoveryAvailabilitySetId"] = derived4.RecoveryAvailabilitySetId;
                            }
                            
                            if (derived4.VmDisks != null)
                            {
                                if (derived4.VmDisks is ILazyCollection == false || ((ILazyCollection)derived4.VmDisks).IsInitialized)
                                {
                                    JArray vmDisksArray = new JArray();
                                    foreach (A2AVmDiskInputDetails vmDisksItem in derived4.VmDisks)
                                    {
                                        JObject a2AVmDiskInputDetailsValue = new JObject();
                                        vmDisksArray.Add(a2AVmDiskInputDetailsValue);
                                        
                                        if (vmDisksItem.DiskUri != null)
                                        {
                                            a2AVmDiskInputDetailsValue["diskUri"] = vmDisksItem.DiskUri;
                                        }
                                        
                                        if (vmDisksItem.RecoveryAzureStorageAccountId != null)
                                        {
                                            a2AVmDiskInputDetailsValue["recoveryAzureStorageAccountId"] = vmDisksItem.RecoveryAzureStorageAccountId;
                                        }
                                        
                                        if (vmDisksItem.PrimaryStagingAzureStorageAccountId != null)
                                        {
                                            a2AVmDiskInputDetailsValue["primaryStagingAzureStorageAccountId"] = vmDisksItem.PrimaryStagingAzureStorageAccountId;
                                        }
                                    }
                                    providerSpecificDetailsValue["vmDisks"] = vmDisksArray;
                                }
                            }
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Execute planned failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Planned failover input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginPlannedFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, PlannedFailoverInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginPlannedFailoverAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/plannedFailover";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject plannedFailoverInputValue = new JObject();
                requestDoc = plannedFailoverInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    plannedFailoverInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.FailoverDirection != null)
                    {
                        propertiesValue["failoverDirection"] = input.Properties.FailoverDirection;
                    }
                    
                    if (input.Properties.ProviderSpecificDetails != null)
                    {
                        JObject providerSpecificDetailsValue = new JObject();
                        propertiesValue["providerSpecificDetails"] = providerSpecificDetailsValue;
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzure";
                            HyperVReplicaAzureFailoverProviderInput derived = ((HyperVReplicaAzureFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived.VaultLocation != null)
                            {
                                providerSpecificDetailsValue["vaultLocation"] = derived.VaultLocation;
                            }
                            
                            if (derived.PrimaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["primaryKekCertificatePfx"] = derived.PrimaryKekCertificatePfx;
                            }
                            
                            if (derived.SecondaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["secondaryKekCertificatePfx"] = derived.SecondaryKekCertificatePfx;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureFailbackProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzureFailback";
                            HyperVReplicaAzureFailbackProviderInput derived2 = ((HyperVReplicaAzureFailbackProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived2.DataSyncOption != null)
                            {
                                providerSpecificDetailsValue["dataSyncOption"] = derived2.DataSyncOption;
                            }
                            
                            if (derived2.RecoveryVmCreationOption != null)
                            {
                                providerSpecificDetailsValue["recoveryVmCreationOption"] = derived2.RecoveryVmCreationOption;
                            }
                            
                            if (derived2.ProviderIdForAlternateRecovery != null)
                            {
                                providerSpecificDetailsValue["providerIdForAlternateRecovery"] = derived2.ProviderIdForAlternateRecovery;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageAzureV2FailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMageAzureV2";
                            InMageAzureV2FailoverProviderInput derived3 = ((InMageAzureV2FailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived3.VaultLocation != null)
                            {
                                providerSpecificDetailsValue["vaultLocation"] = derived3.VaultLocation;
                            }
                            
                            if (derived3.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived3.RecoveryPointId;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMage";
                            InMageFailoverProviderInput derived4 = ((InMageFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived4.RecoveryPointType != null)
                            {
                                providerSpecificDetailsValue["recoveryPointType"] = derived4.RecoveryPointType;
                            }
                            
                            if (derived4.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived4.RecoveryPointId;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is A2AFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "A2A";
                            A2AFailoverProviderInput derived5 = ((A2AFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived5.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived5.RecoveryPointId;
                            }
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Purges Protection for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginPurgeProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginPurgeProtectionAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Executes repair replication for the given Replication protected
        /// item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginRepairReplicationAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginRepairReplicationAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/repairReplication";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Execute reprotect for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Reprotect input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginReprotectAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, ReverseReplicationInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginReprotectAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/reProtect";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject reverseReplicationInputValue = new JObject();
                requestDoc = reverseReplicationInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    reverseReplicationInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.FailoverDirection != null)
                    {
                        propertiesValue["failoverDirection"] = input.Properties.FailoverDirection;
                    }
                    
                    if (input.Properties.ProviderSpecificDetails != null)
                    {
                        JObject providerSpecificDetailsValue = new JObject();
                        propertiesValue["providerSpecificDetails"] = providerSpecificDetailsValue;
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureReprotectInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzure";
                            HyperVReplicaAzureReprotectInput derived = ((HyperVReplicaAzureReprotectInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived.HvHostVmId != null)
                            {
                                providerSpecificDetailsValue["hvHostVmId"] = derived.HvHostVmId;
                            }
                            
                            if (derived.VmName != null)
                            {
                                providerSpecificDetailsValue["vmName"] = derived.VmName;
                            }
                            
                            if (derived.OSType != null)
                            {
                                providerSpecificDetailsValue["osType"] = derived.OSType;
                            }
                            
                            if (derived.VHDId != null)
                            {
                                providerSpecificDetailsValue["vHDId"] = derived.VHDId;
                            }
                            
                            if (derived.StorageAccountId != null)
                            {
                                providerSpecificDetailsValue["storageAccountId"] = derived.StorageAccountId;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageAzureV2ReprotectInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMageAzureV2";
                            InMageAzureV2ReprotectInput derived2 = ((InMageAzureV2ReprotectInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived2.ProcessServerId != null)
                            {
                                providerSpecificDetailsValue["processServerId"] = derived2.ProcessServerId;
                            }
                            
                            if (derived2.MasterTargetId != null)
                            {
                                providerSpecificDetailsValue["masterTargetId"] = derived2.MasterTargetId;
                            }
                            
                            if (derived2.PolicyId != null)
                            {
                                providerSpecificDetailsValue["policyId"] = derived2.PolicyId;
                            }
                            
                            if (derived2.RunAsAccountId != null)
                            {
                                providerSpecificDetailsValue["runAsAccountId"] = derived2.RunAsAccountId;
                            }
                            
                            if (derived2.StorageAccountId != null)
                            {
                                providerSpecificDetailsValue["storageAccountId"] = derived2.StorageAccountId;
                            }
                            
                            if (derived2.LogStorageAccountId != null)
                            {
                                providerSpecificDetailsValue["logStorageAccountId"] = derived2.LogStorageAccountId;
                            }
                            
                            if (derived2.DisksToInclude != null)
                            {
                                if (derived2.DisksToInclude is ILazyCollection == false || ((ILazyCollection)derived2.DisksToInclude).IsInitialized)
                                {
                                    JArray disksToIncludeArray = new JArray();
                                    foreach (string disksToIncludeItem in derived2.DisksToInclude)
                                    {
                                        disksToIncludeArray.Add(disksToIncludeItem);
                                    }
                                    providerSpecificDetailsValue["disksToInclude"] = disksToIncludeArray;
                                }
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageReprotectInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMage";
                            InMageReprotectInput derived3 = ((InMageReprotectInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived3.MasterTargetId != null)
                            {
                                providerSpecificDetailsValue["masterTargetId"] = derived3.MasterTargetId;
                            }
                            
                            if (derived3.ProcessServerId != null)
                            {
                                providerSpecificDetailsValue["processServerId"] = derived3.ProcessServerId;
                            }
                            
                            if (derived3.RetentionDrive != null)
                            {
                                providerSpecificDetailsValue["retentionDrive"] = derived3.RetentionDrive;
                            }
                            
                            if (derived3.RunAsAccountId != null)
                            {
                                providerSpecificDetailsValue["runAsAccountId"] = derived3.RunAsAccountId;
                            }
                            
                            if (derived3.DatastoreName != null)
                            {
                                providerSpecificDetailsValue["datastoreName"] = derived3.DatastoreName;
                            }
                            
                            if (derived3.DiskExclusionInput != null)
                            {
                                JObject diskExclusionInputValue = new JObject();
                                providerSpecificDetailsValue["diskExclusionInput"] = diskExclusionInputValue;
                                
                                if (derived3.DiskExclusionInput.VolumeOptions != null)
                                {
                                    if (derived3.DiskExclusionInput.VolumeOptions is ILazyCollection == false || ((ILazyCollection)derived3.DiskExclusionInput.VolumeOptions).IsInitialized)
                                    {
                                        JArray volumeOptionsArray = new JArray();
                                        foreach (InMageVolumeExclusionOptions volumeOptionsItem in derived3.DiskExclusionInput.VolumeOptions)
                                        {
                                            JObject inMageVolumeExclusionOptionsValue = new JObject();
                                            volumeOptionsArray.Add(inMageVolumeExclusionOptionsValue);
                                            
                                            if (volumeOptionsItem.VolumeLabel != null)
                                            {
                                                inMageVolumeExclusionOptionsValue["volumeLabel"] = volumeOptionsItem.VolumeLabel;
                                            }
                                            
                                            if (volumeOptionsItem.OnlyExcludeIfSingleVolume != null)
                                            {
                                                inMageVolumeExclusionOptionsValue["OnlyExcludeIfSingleVolume"] = volumeOptionsItem.OnlyExcludeIfSingleVolume;
                                            }
                                        }
                                        diskExclusionInputValue["volumeOptions"] = volumeOptionsArray;
                                    }
                                }
                                
                                if (derived3.DiskExclusionInput.DiskSignatureOptions != null)
                                {
                                    if (derived3.DiskExclusionInput.DiskSignatureOptions is ILazyCollection == false || ((ILazyCollection)derived3.DiskExclusionInput.DiskSignatureOptions).IsInitialized)
                                    {
                                        JArray diskSignatureOptionsArray = new JArray();
                                        foreach (InMageDiskSignatureExclusionOptions diskSignatureOptionsItem in derived3.DiskExclusionInput.DiskSignatureOptions)
                                        {
                                            JObject inMageDiskSignatureExclusionOptionsValue = new JObject();
                                            diskSignatureOptionsArray.Add(inMageDiskSignatureExclusionOptionsValue);
                                            
                                            if (diskSignatureOptionsItem.DiskSignature != null)
                                            {
                                                inMageDiskSignatureExclusionOptionsValue["diskSignature"] = diskSignatureOptionsItem.DiskSignature;
                                            }
                                        }
                                        diskExclusionInputValue["diskSignatureOptions"] = diskSignatureOptionsArray;
                                    }
                                }
                            }
                            
                            if (derived3.ProfileId != null)
                            {
                                providerSpecificDetailsValue["profileId"] = derived3.ProfileId;
                            }
                            
                            if (derived3.DisksToInclude != null)
                            {
                                if (derived3.DisksToInclude is ILazyCollection == false || ((ILazyCollection)derived3.DisksToInclude).IsInitialized)
                                {
                                    JArray disksToIncludeArray2 = new JArray();
                                    foreach (string disksToIncludeItem2 in derived3.DisksToInclude)
                                    {
                                        disksToIncludeArray2.Add(disksToIncludeItem2);
                                    }
                                    providerSpecificDetailsValue["disksToInclude"] = disksToIncludeArray2;
                                }
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is A2AReprotectInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "A2A";
                            A2AReprotectInput derived4 = ((A2AReprotectInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived4.RecoveryContainerId != null)
                            {
                                providerSpecificDetailsValue["recoveryContainerId"] = derived4.RecoveryContainerId;
                            }
                            
                            if (derived4.VmDisks != null)
                            {
                                if (derived4.VmDisks is ILazyCollection == false || ((ILazyCollection)derived4.VmDisks).IsInitialized)
                                {
                                    JArray vmDisksArray = new JArray();
                                    foreach (A2AVmDiskInputDetails vmDisksItem in derived4.VmDisks)
                                    {
                                        JObject a2AVmDiskInputDetailsValue = new JObject();
                                        vmDisksArray.Add(a2AVmDiskInputDetailsValue);
                                        
                                        if (vmDisksItem.DiskUri != null)
                                        {
                                            a2AVmDiskInputDetailsValue["diskUri"] = vmDisksItem.DiskUri;
                                        }
                                        
                                        if (vmDisksItem.RecoveryAzureStorageAccountId != null)
                                        {
                                            a2AVmDiskInputDetailsValue["recoveryAzureStorageAccountId"] = vmDisksItem.RecoveryAzureStorageAccountId;
                                        }
                                        
                                        if (vmDisksItem.PrimaryStagingAzureStorageAccountId != null)
                                        {
                                            a2AVmDiskInputDetailsValue["primaryStagingAzureStorageAccountId"] = vmDisksItem.PrimaryStagingAzureStorageAccountId;
                                        }
                                    }
                                    providerSpecificDetailsValue["vmDisks"] = vmDisksArray;
                                }
                            }
                            
                            if (derived4.PolicyId != null)
                            {
                                providerSpecificDetailsValue["policyId"] = derived4.PolicyId;
                            }
                            
                            if (derived4.RecoveryResourceGroupId != null)
                            {
                                providerSpecificDetailsValue["recoveryResourceGroupId"] = derived4.RecoveryResourceGroupId;
                            }
                            
                            if (derived4.RecoveryCloudServiceId != null)
                            {
                                providerSpecificDetailsValue["recoveryCloudServiceId"] = derived4.RecoveryCloudServiceId;
                            }
                            
                            if (derived4.RecoveryAvailabilitySetId != null)
                            {
                                providerSpecificDetailsValue["recoveryAvailabilitySetId"] = derived4.RecoveryAvailabilitySetId;
                            }
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Execute Test failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Test failover input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginTestFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, TestFailoverInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginTestFailoverAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/testFailover";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject testFailoverInputValue = new JObject();
                requestDoc = testFailoverInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    testFailoverInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.FailoverDirection != null)
                    {
                        propertiesValue["failoverDirection"] = input.Properties.FailoverDirection;
                    }
                    
                    if (input.Properties.NetworkType != null)
                    {
                        propertiesValue["networkType"] = input.Properties.NetworkType;
                    }
                    
                    if (input.Properties.NetworkId != null)
                    {
                        propertiesValue["networkId"] = input.Properties.NetworkId;
                    }
                    
                    if (input.Properties.ProviderSpecificDetails != null)
                    {
                        JObject providerSpecificDetailsValue = new JObject();
                        propertiesValue["providerSpecificDetails"] = providerSpecificDetailsValue;
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzure";
                            HyperVReplicaAzureFailoverProviderInput derived = ((HyperVReplicaAzureFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived.VaultLocation != null)
                            {
                                providerSpecificDetailsValue["vaultLocation"] = derived.VaultLocation;
                            }
                            
                            if (derived.PrimaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["primaryKekCertificatePfx"] = derived.PrimaryKekCertificatePfx;
                            }
                            
                            if (derived.SecondaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["secondaryKekCertificatePfx"] = derived.SecondaryKekCertificatePfx;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureFailbackProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzureFailback";
                            HyperVReplicaAzureFailbackProviderInput derived2 = ((HyperVReplicaAzureFailbackProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived2.DataSyncOption != null)
                            {
                                providerSpecificDetailsValue["dataSyncOption"] = derived2.DataSyncOption;
                            }
                            
                            if (derived2.RecoveryVmCreationOption != null)
                            {
                                providerSpecificDetailsValue["recoveryVmCreationOption"] = derived2.RecoveryVmCreationOption;
                            }
                            
                            if (derived2.ProviderIdForAlternateRecovery != null)
                            {
                                providerSpecificDetailsValue["providerIdForAlternateRecovery"] = derived2.ProviderIdForAlternateRecovery;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageAzureV2FailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMageAzureV2";
                            InMageAzureV2FailoverProviderInput derived3 = ((InMageAzureV2FailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived3.VaultLocation != null)
                            {
                                providerSpecificDetailsValue["vaultLocation"] = derived3.VaultLocation;
                            }
                            
                            if (derived3.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived3.RecoveryPointId;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMage";
                            InMageFailoverProviderInput derived4 = ((InMageFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived4.RecoveryPointType != null)
                            {
                                providerSpecificDetailsValue["recoveryPointType"] = derived4.RecoveryPointType;
                            }
                            
                            if (derived4.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived4.RecoveryPointId;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is A2AFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "A2A";
                            A2AFailoverProviderInput derived5 = ((A2AFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived5.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived5.RecoveryPointId;
                            }
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Execute unplanned failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Unplanned failover input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginUnplannedFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, UnplannedFailoverInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginUnplannedFailoverAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/unplannedFailover";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject unplannedFailoverInputValue = new JObject();
                requestDoc = unplannedFailoverInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    unplannedFailoverInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.FailoverDirection != null)
                    {
                        propertiesValue["failoverDirection"] = input.Properties.FailoverDirection;
                    }
                    
                    if (input.Properties.SourceSiteOperations != null)
                    {
                        propertiesValue["sourceSiteOperations"] = input.Properties.SourceSiteOperations;
                    }
                    
                    if (input.Properties.ProviderSpecificDetails != null)
                    {
                        JObject providerSpecificDetailsValue = new JObject();
                        propertiesValue["ProviderSpecificDetails"] = providerSpecificDetailsValue;
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzure";
                            HyperVReplicaAzureFailoverProviderInput derived = ((HyperVReplicaAzureFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived.VaultLocation != null)
                            {
                                providerSpecificDetailsValue["vaultLocation"] = derived.VaultLocation;
                            }
                            
                            if (derived.PrimaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["primaryKekCertificatePfx"] = derived.PrimaryKekCertificatePfx;
                            }
                            
                            if (derived.SecondaryKekCertificatePfx != null)
                            {
                                providerSpecificDetailsValue["secondaryKekCertificatePfx"] = derived.SecondaryKekCertificatePfx;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is HyperVReplicaAzureFailbackProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "HyperVReplicaAzureFailback";
                            HyperVReplicaAzureFailbackProviderInput derived2 = ((HyperVReplicaAzureFailbackProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived2.DataSyncOption != null)
                            {
                                providerSpecificDetailsValue["dataSyncOption"] = derived2.DataSyncOption;
                            }
                            
                            if (derived2.RecoveryVmCreationOption != null)
                            {
                                providerSpecificDetailsValue["recoveryVmCreationOption"] = derived2.RecoveryVmCreationOption;
                            }
                            
                            if (derived2.ProviderIdForAlternateRecovery != null)
                            {
                                providerSpecificDetailsValue["providerIdForAlternateRecovery"] = derived2.ProviderIdForAlternateRecovery;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageAzureV2FailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMageAzureV2";
                            InMageAzureV2FailoverProviderInput derived3 = ((InMageAzureV2FailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived3.VaultLocation != null)
                            {
                                providerSpecificDetailsValue["vaultLocation"] = derived3.VaultLocation;
                            }
                            
                            if (derived3.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived3.RecoveryPointId;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is InMageFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "InMage";
                            InMageFailoverProviderInput derived4 = ((InMageFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived4.RecoveryPointType != null)
                            {
                                providerSpecificDetailsValue["recoveryPointType"] = derived4.RecoveryPointType;
                            }
                            
                            if (derived4.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived4.RecoveryPointId;
                            }
                        }
                        if (input.Properties.ProviderSpecificDetails is A2AFailoverProviderInput)
                        {
                            providerSpecificDetailsValue["instanceType"] = "A2A";
                            A2AFailoverProviderInput derived5 = ((A2AFailoverProviderInput)input.Properties.ProviderSpecificDetails);
                            
                            if (derived5.RecoveryPointId != null)
                            {
                                providerSpecificDetailsValue["recoveryPointId"] = derived5.RecoveryPointId;
                            }
                        }
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Update mobility service for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Update mobility service input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginUpdateMobilityServiceAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, UpdateMobilityServiceRequest input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginUpdateMobilityServiceAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/updateMobilityService";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject updateMobilityServiceRequestValue = new JObject();
                requestDoc = updateMobilityServiceRequestValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    updateMobilityServiceRequestValue["properties"] = propertiesValue;
                    
                    if (input.Properties.RunAsAccountId != null)
                    {
                        propertiesValue["runAsAccountId"] = input.Properties.RunAsAccountId;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Updates the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Updation input
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginUpdateProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, UpdateReplicationProtectedItemInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }
            if (input.Properties != null)
            {
                if (input.Properties.VmNics == null)
                {
                    throw new ArgumentNullException("input.Properties.VmNics");
                }
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "BeginUpdateProtectionAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/ReplicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = new HttpMethod("PATCH");
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("Agent-Authentication", customRequestHeaders.AgentAuthenticationHeader);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject updateReplicationProtectedItemInputValue = new JObject();
                requestDoc = updateReplicationProtectedItemInputValue;
                
                if (input.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    updateReplicationProtectedItemInputValue["properties"] = propertiesValue;
                    
                    if (input.Properties.RecoveryAzureVMName != null)
                    {
                        propertiesValue["recoveryAzureVMName"] = input.Properties.RecoveryAzureVMName;
                    }
                    
                    if (input.Properties.RecoveryAzureVMSize != null)
                    {
                        propertiesValue["recoveryAzureVMSize"] = input.Properties.RecoveryAzureVMSize;
                    }
                    
                    if (input.Properties.SelectedRecoveryAzureNetworkId != null)
                    {
                        propertiesValue["selectedRecoveryAzureNetworkId"] = input.Properties.SelectedRecoveryAzureNetworkId;
                    }
                    
                    if (input.Properties.EnableRDPOnTargetOption != null)
                    {
                        propertiesValue["enableRDPOnTargetOption"] = input.Properties.EnableRDPOnTargetOption;
                    }
                    
                    if (input.Properties.VmNics != null)
                    {
                        if (input.Properties.VmNics is ILazyCollection == false || ((ILazyCollection)input.Properties.VmNics).IsInitialized)
                        {
                            JArray vmNicsArray = new JArray();
                            foreach (VMNicInputDetails vmNicsItem in input.Properties.VmNics)
                            {
                                JObject vMNicInputDetailsValue = new JObject();
                                vmNicsArray.Add(vMNicInputDetailsValue);
                                
                                if (vmNicsItem.NicId != null)
                                {
                                    vMNicInputDetailsValue["nicId"] = vmNicsItem.NicId;
                                }
                                
                                if (vmNicsItem.RecoveryVMSubnetName != null)
                                {
                                    vMNicInputDetailsValue["recoveryVMSubnetName"] = vmNicsItem.RecoveryVMSubnetName;
                                }
                                
                                if (vmNicsItem.ReplicaNicStaticIPAddress != null)
                                {
                                    vMNicInputDetailsValue["replicaNicStaticIPAddress"] = vmNicsItem.ReplicaNicStaticIPAddress;
                                }
                                
                                if (vmNicsItem.SelectionType != null)
                                {
                                    vMNicInputDetailsValue["selectionType"] = vmNicsItem.SelectionType;
                                }
                            }
                            propertiesValue["vmNics"] = vmNicsArray;
                        }
                    }
                    
                    if (input.Properties.LicenseType != null)
                    {
                        propertiesValue["licenseType"] = input.Properties.LicenseType;
                    }
                    
                    if (input.Properties.RecoveryCloudServiceId != null)
                    {
                        propertiesValue["recoveryCloudServiceId"] = input.Properties.RecoveryCloudServiceId;
                    }
                    
                    if (input.Properties.RecoveryResourceGroupId != null)
                    {
                        propertiesValue["recoveryResourceGroupId"] = input.Properties.RecoveryResourceGroupId;
                    }
                    
                    if (input.Properties.RecoveryAvailabilitySetId != null)
                    {
                        propertiesValue["recoveryAvailabilitySetId"] = input.Properties.RecoveryAvailabilitySetId;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Execute commit failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> CommitFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "CommitFailoverAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginCommitFailoverAsync(fabricName, protectionContainerName, replicationProtectedItemName, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetCommitFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetCommitFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Disable Protection for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='input'>
        /// Required. Disable protection input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> DisableProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, DisableProtectionInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "DisableProtectionAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginDisableProtectionAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse result = await client.ReplicationProtectedItem.GetDisableStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetDisableStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Enable Protection for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='input'>
        /// Required. Enable protection input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> EnableProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, EnableProtectionInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "EnableProtectionAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginEnableProtectionAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetEnableStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetEnableStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Get the replication protected object by Id.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric unique name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container unique name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the fabric object
        /// </returns>
        public async Task<ReplicationProtectedItemResponse> GetAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/replicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetApplyRecoveryPointStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetApplyRecoveryPointStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetCommitFailoverStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetCommitFailoverStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> GetDisableStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetDisableStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.Accepted || statusCode == HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new LongRunningOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken locationValue = responseDoc["Location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                result.Location = locationInstance;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetEnableStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetEnableStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetPlannedFailoverStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetPlannedFailoverStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> GetPurgeStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetPurgeStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.Accepted || statusCode == HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new LongRunningOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken locationValue = responseDoc["Location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                result.Location = locationInstance;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetRepairReplicationStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetRepairReplicationStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetReprotectStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetReprotectStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetTestFailoverStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetTestFailoverStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetUnplannedFailoverStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetUnplannedFailoverStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetUpdateMobilityServiceStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetUpdateMobilityServiceStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Service response for replication protected items operation.
        /// </returns>
        public async Task<ReplicationProtectedItemOperationResponse> GetUpdateProtectionStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetUpdateProtectionStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                            result.ReplicationProtectedItem = replicationProtectedItemInstance;
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                replicationProtectedItemInstance.Properties = propertiesInstance;
                                
                                JToken friendlyNameValue = propertiesValue["friendlyName"];
                                if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                {
                                    string friendlyNameInstance = ((string)friendlyNameValue);
                                    propertiesInstance.FriendlyName = friendlyNameInstance;
                                }
                                
                                JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                {
                                    string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                    propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                }
                                
                                JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                {
                                    string protectableItemIdInstance = ((string)protectableItemIdValue);
                                    propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                }
                                
                                JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                    propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                }
                                
                                JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                    propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                    propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                }
                                
                                JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                {
                                    string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                    propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                }
                                
                                JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                    propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                }
                                
                                JToken protectionStateValue = propertiesValue["protectionState"];
                                if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                {
                                    string protectionStateInstance = ((string)protectionStateValue);
                                    propertiesInstance.ProtectionState = protectionStateInstance;
                                }
                                
                                JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                    propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                }
                                
                                JToken activeLocationValue = propertiesValue["activeLocation"];
                                if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                {
                                    string activeLocationInstance = ((string)activeLocationValue);
                                    propertiesInstance.ActiveLocation = activeLocationInstance;
                                }
                                
                                JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateInstance = ((string)testFailoverStateValue);
                                    propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                }
                                
                                JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                {
                                    string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                    propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                }
                                
                                JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                    {
                                        propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                    }
                                }
                                
                                JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                {
                                    string replicationHealthInstance = ((string)replicationHealthValue);
                                    propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                }
                                
                                JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                    {
                                        HealthError healthErrorInstance = new HealthError();
                                        propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                        
                                        JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                        if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                        {
                                            string errorLevelInstance = ((string)errorLevelValue);
                                            healthErrorInstance.ErrorLevel = errorLevelInstance;
                                        }
                                        
                                        JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                        if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                        {
                                            string errorCodeInstance = ((string)errorCodeValue);
                                            healthErrorInstance.ErrorCode = errorCodeInstance;
                                        }
                                        
                                        JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                        if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                        {
                                            string errorMessageInstance = ((string)errorMessageValue);
                                            healthErrorInstance.ErrorMessage = errorMessageInstance;
                                        }
                                        
                                        JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                        if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                        {
                                            string possibleCausesInstance = ((string)possibleCausesValue);
                                            healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                        }
                                        
                                        JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                        if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                        {
                                            string recommendedActionInstance = ((string)recommendedActionValue);
                                            healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                        }
                                        
                                        JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                        if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                        {
                                            string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                            healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                        }
                                        
                                        JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                        if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                            healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                        }
                                        
                                        JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                        if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                        {
                                            string entityIdInstance = ((string)entityIdValue);
                                            healthErrorInstance.EntityId = entityIdInstance;
                                        }
                                    }
                                }
                                
                                JToken policyIdValue = propertiesValue["policyId"];
                                if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                {
                                    string policyIdInstance = ((string)policyIdValue);
                                    propertiesInstance.PolicyID = policyIdInstance;
                                }
                                
                                JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                {
                                    string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                    propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                }
                                
                                JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                }
                                
                                JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                    propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                }
                                
                                JToken currentScenarioValue = propertiesValue["currentScenario"];
                                if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                {
                                    CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                    propertiesInstance.CurrentScenario = currentScenarioInstance;
                                    
                                    JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                    if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                    {
                                        string scenarioNameInstance = ((string)scenarioNameValue);
                                        currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                    }
                                    
                                    JToken jobIdValue = currentScenarioValue["jobId"];
                                    if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                    {
                                        string jobIdInstance = ((string)jobIdValue);
                                        currentScenarioInstance.JobId = jobIdInstance;
                                    }
                                    
                                    JToken startTimeValue = currentScenarioValue["startTime"];
                                    if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                    {
                                        DateTime startTimeInstance = ((DateTime)startTimeValue);
                                        currentScenarioInstance.StartTime = startTimeInstance;
                                    }
                                }
                                
                                JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                {
                                    string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                    propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                }
                                
                                JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                {
                                    string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                    if (typeName == "HyperVReplica2012")
                                    {
                                        HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                        }
                                        
                                        JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance = ((string)vmIdValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                        }
                                        
                                        JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                            {
                                                VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                
                                                JToken nicIdValue = vmNicsValue["nicId"];
                                                if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance = ((string)nicIdValue);
                                                    vMNicDetailsInstance.NicId = nicIdInstance;
                                                }
                                                
                                                JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                    vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                }
                                                
                                                JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                    vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                    vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                    vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                }
                                                
                                                JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                    vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                    vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                }
                                                
                                                JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance = ((string)selectionTypeValue);
                                                    vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                            hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                            {
                                                DiskDetails diskDetailsInstance = new DiskDetails();
                                                hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                
                                                JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                    diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                }
                                                
                                                JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance = ((string)vhdTypeValue);
                                                    diskDetailsInstance.VhdType = vhdTypeInstance;
                                                }
                                                
                                                JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance = ((string)vhdIdValue);
                                                    diskDetailsInstance.VhdId = vhdIdInstance;
                                                }
                                                
                                                JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance = ((string)vhdNameValue);
                                                    diskDetailsInstance.VhdName = vhdNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                            hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                        }
                                        
                                        JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance = ((string)instanceTypeValue);
                                            hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                    }
                                    if (typeName == "HyperVReplica2012R2")
                                    {
                                        HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                        
                                        JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                        }
                                        
                                        JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance2 = ((string)vmIdValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                            {
                                                VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                
                                                JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance2 = ((string)nicIdValue2);
                                                    vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                }
                                                
                                                JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                    vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                }
                                                
                                                JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                    vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                    vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                    vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                }
                                                
                                                JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                    vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                    vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                }
                                                
                                                JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                    vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                            hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                            
                                            JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                            if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                            }
                                        }
                                        
                                        JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                        if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                            {
                                                DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                
                                                JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                {
                                                    ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                    diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                }
                                                
                                                JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                    diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                }
                                                
                                                JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance2 = ((string)vhdIdValue2);
                                                    diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                }
                                                
                                                JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance2 = ((string)vhdNameValue2);
                                                    diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                        }
                                        
                                        JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                            hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                    }
                                    if (typeName == "A2A")
                                    {
                                        A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                        
                                        JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                        if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                        {
                                            string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                            a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                        }
                                        
                                        JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                            a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                        }
                                        
                                        JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                            {
                                                A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                
                                                JToken diskUriValue = protectedDisksValue["diskUri"];
                                                if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                {
                                                    string diskUriInstance = ((string)diskUriValue);
                                                    a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                    a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                }
                                                
                                                JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                    a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                            a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                    }
                                    if (typeName == "InMageAzureV2")
                                    {
                                        InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                        
                                        JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                        }
                                        
                                        JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance = ((string)protectionStageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                        }
                                        
                                        JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance3 = ((string)vmIdValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                        }
                                        
                                        JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                        if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                        {
                                            int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                        }
                                        
                                        JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                        }
                                        
                                        JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance = ((string)ipAddressValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                        }
                                        
                                        JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                        if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                        {
                                            string agentVersionInstance = ((string)agentVersionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                        }
                                        
                                        JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                        if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                        }
                                        
                                        JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                        if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                        {
                                            string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                        }
                                        
                                        JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                        }
                                        
                                        JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance = ((string)processServerIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                        }
                                        
                                        JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                        }
                                        
                                        JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                            {
                                                InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue = protectedDisksValue2["diskId"];
                                                if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance = ((string)diskIdValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                }
                                                
                                                JToken diskNameValue = protectedDisksValue2["diskName"];
                                                if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance = ((string)diskNameValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                }
                                                
                                                JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance2 = ((string)protectionStageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                }
                                                
                                                JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                }
                                                
                                                JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                }
                                                
                                                JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                }
                                                
                                                JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                }
                                                
                                                JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                }
                                                
                                                JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                }
                                                
                                                JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                }
                                                
                                                JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance = ((string)diskResizedValue);
                                                    inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance2 = ((string)diskResizedValue2);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance = ((string)masterTargetIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                        }
                                        
                                        JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                        if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                        {
                                            string osTypeInstance = ((string)osTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                        }
                                        
                                        JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                        if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                        {
                                            string vhdNameInstance3 = ((string)vhdNameValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                        }
                                        
                                        JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                        if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                        {
                                            string osDiskIdInstance = ((string)osDiskIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                        }
                                        
                                        JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                
                                                JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                    azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                }
                                                
                                                JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance3 = ((string)vhdIdValue3);
                                                    azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                }
                                                
                                                JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance4 = ((string)vhdNameValue4);
                                                    azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                }
                                                
                                                JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                    azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                }
                                                
                                                JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                    azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                }
                                                
                                                JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance = ((string)lunIdValue);
                                                    azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                }
                                                
                                                JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                    azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                        }
                                        
                                        JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                            {
                                                VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                
                                                JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance3 = ((string)nicIdValue3);
                                                    vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                }
                                                
                                                JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                    vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                }
                                                
                                                JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                    vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                    vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                    vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                }
                                                
                                                JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                    vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                    vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                }
                                                
                                                JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                    vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                        }
                                        
                                        JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance = ((string)discoveryTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                        }
                                        
                                        JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance = ((string)licenseTypeValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                        }
                                        
                                        JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                        if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                            inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                        }
                                        
                                        JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                            inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                        }
                                        
                                        JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                            inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "InMage")
                                    {
                                        InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                        
                                        JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                        if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                        {
                                            string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                            inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                        }
                                        
                                        JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                            inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                        }
                                        
                                        JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                        if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                            inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                            
                                            JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                            if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                            {
                                                string osVhdIdInstance = ((string)osVhdIdValue);
                                                osDetailsInstance.OSVhdId = osVhdIdInstance;
                                            }
                                            
                                            JToken osTypeValue2 = osDetailsValue["osType"];
                                            if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance2 = ((string)osTypeValue2);
                                                osDetailsInstance.OSType = osTypeInstance2;
                                            }
                                            
                                            JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                            if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                            {
                                                string vhdNameInstance5 = ((string)vhdNameValue5);
                                                osDetailsInstance.VhdName = vhdNameInstance5;
                                            }
                                        }
                                        
                                        JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                        if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                        {
                                            string protectionStageInstance3 = ((string)protectionStageValue3);
                                            inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                        }
                                        
                                        JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance4 = ((string)vmIdValue4);
                                            inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                        }
                                        
                                        JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                        if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                            inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                            
                                            JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                            if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                            }
                                        }
                                        
                                        JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                        if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                        }
                                        
                                        JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                        if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                        {
                                            DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                            inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                        }
                                        
                                        JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                        if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                        if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                        {
                                            double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                            inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                        }
                                        
                                        JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                        if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                        {
                                            long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                            inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                        }
                                        
                                        JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                        if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                        {
                                            foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                            {
                                                InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                
                                                JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                {
                                                    string diskIdInstance2 = ((string)diskIdValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                }
                                                
                                                JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string diskNameInstance2 = ((string)diskNameValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                }
                                                
                                                JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance4 = ((string)protectionStageValue4);
                                                    inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                }
                                                
                                                JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                {
                                                    string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                    inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                    inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                }
                                                
                                                JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                {
                                                    string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                }
                                                
                                                JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                    inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                }
                                                
                                                JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                {
                                                    long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                    inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                }
                                                
                                                JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                }
                                                
                                                JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                {
                                                    long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                    inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                }
                                                
                                                JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                }
                                                
                                                JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double psDataInMBInstance = ((double)psDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                }
                                                
                                                JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                {
                                                    double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                    inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                }
                                                
                                                JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance3 = ((string)diskResizedValue3);
                                                    inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                }
                                            }
                                        }
                                        
                                        JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                        if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                        {
                                            string ipAddressInstance2 = ((string)ipAddressValue2);
                                            inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                        }
                                        
                                        JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                        if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                        {
                                            DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                            inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                        }
                                        
                                        JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                        if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                        {
                                            string processServerIdInstance2 = ((string)processServerIdValue2);
                                            inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                        }
                                        
                                        JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                        if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                        {
                                            string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                            inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                        }
                                        
                                        JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                        if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property in consistencyPointsSequenceElement)
                                            {
                                                string consistencyPointsKey = ((string)property.Name);
                                                DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                            }
                                        }
                                        
                                        JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                        if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                        {
                                            string diskResizedInstance4 = ((string)diskResizedValue4);
                                            inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                        }
                                        
                                        JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                        if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                        {
                                            string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                            inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                        }
                                        
                                        JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                        if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                        }
                                        
                                        JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                        if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                        {
                                            string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                            inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                        }
                                        
                                        JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                        if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                        {
                                            InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                            inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                            
                                            JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                            if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                            {
                                                string agentVersionInstance2 = ((string)agentVersionValue2);
                                                agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                            }
                                            
                                            JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                            if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                            {
                                                string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                            }
                                            
                                            JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                            if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                            {
                                                string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                            }
                                        }
                                        
                                        JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                        if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                        {
                                            string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                            inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                        }
                                        
                                        JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                        if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                        {
                                            string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                            inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                        }
                                        
                                        JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                            {
                                                VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                
                                                JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance4 = ((string)nicIdValue4);
                                                    vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                }
                                                
                                                JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                    vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                }
                                                
                                                JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                    vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                    vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                    vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                }
                                                
                                                JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                    vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                    vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                }
                                                
                                                JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                    vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                }
                                            }
                                        }
                                        
                                        JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                        if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                        {
                                            string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                            inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                            inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                        }
                                        
                                        JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                        if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                        {
                                            string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                            inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                        }
                                        
                                        JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                            inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                    }
                                    if (typeName == "HyperVReplicaAzure")
                                    {
                                        HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                        
                                        JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                        if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                            {
                                                AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                
                                                JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                    azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                }
                                                
                                                JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vhdIdInstance4 = ((string)vhdIdValue4);
                                                    azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                }
                                                
                                                JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance6 = ((string)vhdNameValue6);
                                                    azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                }
                                                
                                                JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                {
                                                    string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                    azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                }
                                                
                                                JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                }
                                                
                                                JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                {
                                                    string lunIdInstance2 = ((string)lunIdValue2);
                                                    azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                }
                                                
                                                JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                {
                                                    string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                    azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                }
                                            }
                                        }
                                        
                                        JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                        if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                        }
                                        
                                        JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                        if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                        }
                                        
                                        JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                        if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                        {
                                            string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                        }
                                        
                                        JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                        if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                        {
                                            DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                        }
                                        
                                        JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                        if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                        {
                                            string vmIdInstance5 = ((string)vmIdValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                        }
                                        
                                        JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                        if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                        {
                                            InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                            
                                            JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                            if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                            }
                                            
                                            JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                            if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                            {
                                                string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                            }
                                        }
                                        
                                        JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                        if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                        {
                                            foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                            {
                                                VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                
                                                JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                {
                                                    string nicIdInstance5 = ((string)nicIdValue5);
                                                    vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                }
                                                
                                                JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                    vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                }
                                                
                                                JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                {
                                                    string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                    vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                }
                                                
                                                JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                    vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                }
                                                
                                                JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                {
                                                    string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                    vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                }
                                                
                                                JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                    vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                }
                                                
                                                JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                {
                                                    string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                    vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                }
                                                
                                                JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                    vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                }
                                            }
                                        }
                                        
                                        JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                        if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                        {
                                            string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                        }
                                        
                                        JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                        if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                        {
                                            string encryptionInstance = ((string)encryptionValue);
                                            hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                        }
                                        
                                        JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                        if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                        {
                                            OSDetails oSDetailsInstance = new OSDetails();
                                            hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                            
                                            JToken osTypeValue3 = oSDetailsValue["osType"];
                                            if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                            {
                                                string osTypeInstance3 = ((string)osTypeValue3);
                                                oSDetailsInstance.OsType = osTypeInstance3;
                                            }
                                            
                                            JToken productTypeValue = oSDetailsValue["productType"];
                                            if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                            {
                                                string productTypeInstance = ((string)productTypeValue);
                                                oSDetailsInstance.ProductType = productTypeInstance;
                                            }
                                            
                                            JToken osEditionValue = oSDetailsValue["osEdition"];
                                            if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                            {
                                                string osEditionInstance = ((string)osEditionValue);
                                                oSDetailsInstance.OSEdition = osEditionInstance;
                                            }
                                            
                                            JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                            if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSVersionInstance = ((string)oSVersionValue);
                                                oSDetailsInstance.OSVersion = oSVersionInstance;
                                            }
                                            
                                            JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                            if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                            }
                                            
                                            JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                            if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                            {
                                                string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                            }
                                        }
                                        
                                        JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                        if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                        }
                                        
                                        JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                        if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                        {
                                            int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                            hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                        }
                                        
                                        JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                        if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                        {
                                            string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                        }
                                        
                                        JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                        if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                        {
                                            string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                            hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                        }
                                        
                                        JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                        if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                        }
                                        
                                        JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                        if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                        {
                                            string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                            hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                        }
                                        
                                        JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                        if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                        {
                                            string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                            hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                        }
                                        propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                    }
                                }
                            }
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                replicationProtectedItemInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                replicationProtectedItemInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                replicationProtectedItemInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                replicationProtectedItemInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property2 in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property2.Name);
                                    string tagsValue = ((string)property2.Value);
                                    replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken locationValue2 = responseDoc["Location"];
                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                            {
                                string locationInstance2 = ((string)locationValue2);
                                result.Location = locationInstance2;
                            }
                            
                            JToken retryAfterValue = responseDoc["RetryAfter"];
                            if (retryAfterValue != null && retryAfterValue.Type != JTokenType.Null)
                            {
                                int retryAfterInstance = ((int)retryAfterValue);
                                result.RetryAfter = retryAfterInstance;
                            }
                            
                            JToken asyncOperationValue = responseDoc["AsyncOperation"];
                            if (asyncOperationValue != null && asyncOperationValue.Type != JTokenType.Null)
                            {
                                string asyncOperationInstance = ((string)asyncOperationValue);
                                result.AsyncOperation = asyncOperationInstance;
                            }
                            
                            JToken statusValue = responseDoc["Status"];
                            if (statusValue != null && statusValue.Type != JTokenType.Null)
                            {
                                OperationStatus statusInstance = ((OperationStatus)Enum.Parse(typeof(OperationStatus), ((string)statusValue), true));
                                result.Status = statusInstance;
                            }
                            
                            JToken cultureValue = responseDoc["Culture"];
                            if (cultureValue != null && cultureValue.Type != JTokenType.Null)
                            {
                                string cultureInstance = ((string)cultureValue);
                                result.Culture = cultureInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Azure-AsyncOperation"))
                    {
                        result.AsyncOperation = httpResponse.Headers.GetValues("Azure-AsyncOperation").FirstOrDefault();
                    }
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.Location = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Accepted)
                    {
                        result.Status = OperationStatus.InProgress;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Get the replication protected object by Id.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric unique name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container unique name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list replicated protected items.
        /// </returns>
        public async Task<ReplicationProtectedItemListResponse> ListAsync(string fabricName, string protectionContainerName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/replicationProtectedItems";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                                    result.ReplicationProtectedItems.Add(replicationProtectedItemInstance);
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                        replicationProtectedItemInstance.Properties = propertiesInstance;
                                        
                                        JToken friendlyNameValue = propertiesValue["friendlyName"];
                                        if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance = ((string)friendlyNameValue);
                                            propertiesInstance.FriendlyName = friendlyNameInstance;
                                        }
                                        
                                        JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                        if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                        {
                                            string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                            propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                        }
                                        
                                        JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                        if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                        {
                                            string protectableItemIdInstance = ((string)protectableItemIdValue);
                                            propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                        }
                                        
                                        JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                        if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                            propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                        }
                                        
                                        JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                        if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                            propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                        if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                            propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                        if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                            propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                        }
                                        
                                        JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                        if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                            propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                        if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                            propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                        }
                                        
                                        JToken protectionStateValue = propertiesValue["protectionState"];
                                        if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                        {
                                            string protectionStateInstance = ((string)protectionStateValue);
                                            propertiesInstance.ProtectionState = protectionStateInstance;
                                        }
                                        
                                        JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                        if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                            propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                        }
                                        
                                        JToken activeLocationValue = propertiesValue["activeLocation"];
                                        if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                        {
                                            string activeLocationInstance = ((string)activeLocationValue);
                                            propertiesInstance.ActiveLocation = activeLocationInstance;
                                        }
                                        
                                        JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                        if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStateInstance = ((string)testFailoverStateValue);
                                            propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                        }
                                        
                                        JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                        if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                            propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                        }
                                        
                                        JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                        if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                            {
                                                propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                            }
                                        }
                                        
                                        JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                        if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                        {
                                            string replicationHealthInstance = ((string)replicationHealthValue);
                                            propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                        }
                                        
                                        JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                        if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                            {
                                                HealthError healthErrorInstance = new HealthError();
                                                propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                                
                                                JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                    healthErrorInstance.ErrorLevel = errorLevelInstance;
                                                }
                                                
                                                JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                                if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string errorCodeInstance = ((string)errorCodeValue);
                                                    healthErrorInstance.ErrorCode = errorCodeInstance;
                                                }
                                                
                                                JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                                if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                {
                                                    string errorMessageInstance = ((string)errorMessageValue);
                                                    healthErrorInstance.ErrorMessage = errorMessageInstance;
                                                }
                                                
                                                JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                                if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                {
                                                    string possibleCausesInstance = ((string)possibleCausesValue);
                                                    healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                                }
                                                
                                                JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                                if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                {
                                                    string recommendedActionInstance = ((string)recommendedActionValue);
                                                    healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                                }
                                                
                                                JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                                if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                                {
                                                    string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                                    healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                                }
                                                
                                                JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                                if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                                    healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                                }
                                                
                                                JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                                if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                                {
                                                    string entityIdInstance = ((string)entityIdValue);
                                                    healthErrorInstance.EntityId = entityIdInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken policyIdValue = propertiesValue["policyId"];
                                        if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                        {
                                            string policyIdInstance = ((string)policyIdValue);
                                            propertiesInstance.PolicyID = policyIdInstance;
                                        }
                                        
                                        JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                        if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                            propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                        }
                                        
                                        JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                        if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                            propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                        }
                                        
                                        JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                        if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                            propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                        }
                                        
                                        JToken currentScenarioValue = propertiesValue["currentScenario"];
                                        if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                        {
                                            CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                            propertiesInstance.CurrentScenario = currentScenarioInstance;
                                            
                                            JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                            if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                            {
                                                string scenarioNameInstance = ((string)scenarioNameValue);
                                                currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                            }
                                            
                                            JToken jobIdValue = currentScenarioValue["jobId"];
                                            if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                            {
                                                string jobIdInstance = ((string)jobIdValue);
                                                currentScenarioInstance.JobId = jobIdInstance;
                                            }
                                            
                                            JToken startTimeValue = currentScenarioValue["startTime"];
                                            if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                            {
                                                DateTime startTimeInstance = ((DateTime)startTimeValue);
                                                currentScenarioInstance.StartTime = startTimeInstance;
                                            }
                                        }
                                        
                                        JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                        if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                        {
                                            string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                            propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                        }
                                        
                                        JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                        if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                            if (typeName == "HyperVReplica2012")
                                            {
                                                HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                                
                                                JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                                }
                                                
                                                JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance = ((string)vmIdValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                                }
                                                
                                                JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                        hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                        
                                                        JToken nicIdValue = vmNicsValue["nicId"];
                                                        if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance = ((string)nicIdValue);
                                                            vMNicDetailsInstance.NicId = nicIdInstance;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                        if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                            vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                        if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                            vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                            vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                            vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                        if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                            vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                            vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                        }
                                                        
                                                        JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                        if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance = ((string)selectionTypeValue);
                                                            vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                                    hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                                    
                                                    JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                                    if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                        initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                        initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                                    }
                                                }
                                                
                                                JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                                if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                                    {
                                                        DiskDetails diskDetailsInstance = new DiskDetails();
                                                        hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                        
                                                        JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                        if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                        {
                                                            ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                            diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                        }
                                                        
                                                        JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                        if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance = ((string)vhdTypeValue);
                                                            diskDetailsInstance.VhdType = vhdTypeInstance;
                                                        }
                                                        
                                                        JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                        if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance = ((string)vhdIdValue);
                                                            diskDetailsInstance.VhdId = vhdIdInstance;
                                                        }
                                                        
                                                        JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                        if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance = ((string)vhdNameValue);
                                                            diskDetailsInstance.VhdName = vhdNameInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                                }
                                                
                                                JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                            }
                                            if (typeName == "HyperVReplica2012R2")
                                            {
                                                HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                                
                                                JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                                }
                                                
                                                JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                                }
                                                
                                                JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                        hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                        
                                                        JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                        if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance2 = ((string)nicIdValue2);
                                                            vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                        if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                            vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                        if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                            vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                            vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                            vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                        if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                            vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                            vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                        }
                                                        
                                                        JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                        if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                            vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                                    hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                                    
                                                    JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                                    if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                        initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                        initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                                    }
                                                }
                                                
                                                JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                                if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                                    {
                                                        DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                        hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                        
                                                        JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                        if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                        {
                                                            ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                            diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                        }
                                                        
                                                        JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                        if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                            diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                        }
                                                        
                                                        JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                        if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance2 = ((string)vhdIdValue2);
                                                            diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                        }
                                                        
                                                        JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                        if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance2 = ((string)vhdNameValue2);
                                                            diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                                }
                                                
                                                JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                            }
                                            if (typeName == "A2A")
                                            {
                                                A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                                
                                                JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                                if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                                {
                                                    string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                                    a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                                    a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                                }
                                                
                                                JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                                    a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                                }
                                                
                                                JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                                    {
                                                        A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                        a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskUriValue = protectedDisksValue["diskUri"];
                                                        if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                        {
                                                            string diskUriInstance = ((string)diskUriValue);
                                                            a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                        }
                                                        
                                                        JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                        if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                        if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                        if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                        if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                            a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                    a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                            }
                                            if (typeName == "InMageAzureV2")
                                            {
                                                InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                                
                                                JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                                if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                                {
                                                    string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                                }
                                                
                                                JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                                if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                                {
                                                    string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                                }
                                                
                                                JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                                if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance = ((string)protectionStageValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                                }
                                                
                                                JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                                }
                                                
                                                JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                                if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                                }
                                                
                                                JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                                if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                                {
                                                    double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                                }
                                                
                                                JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                                if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                                {
                                                    double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                                }
                                                
                                                JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                                if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressInstance = ((string)ipAddressValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                                }
                                                
                                                JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                                if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                                {
                                                    string agentVersionInstance = ((string)agentVersionValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                                }
                                                
                                                JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                                if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                                }
                                                
                                                JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                                if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                                }
                                                
                                                JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                                if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                                {
                                                    DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                                }
                                                
                                                JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                                if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                                {
                                                    string processServerIdInstance = ((string)processServerIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                                }
                                                
                                                JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                                }
                                                
                                                JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                                    {
                                                        InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskIdValue = protectedDisksValue2["diskId"];
                                                        if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string diskIdInstance = ((string)diskIdValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                        }
                                                        
                                                        JToken diskNameValue = protectedDisksValue2["diskName"];
                                                        if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                        {
                                                            string diskNameInstance = ((string)diskNameValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                        }
                                                        
                                                        JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                        if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                        {
                                                            string protectionStageInstance2 = ((string)protectionStageValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                        }
                                                        
                                                        JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                        if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                        {
                                                            string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                        }
                                                        
                                                        JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                        if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                        {
                                                            long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                        }
                                                        
                                                        JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                        if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                        {
                                                            string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                        }
                                                        
                                                        JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                        if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                        {
                                                            int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                        }
                                                        
                                                        JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                        if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                        {
                                                            long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                        }
                                                        
                                                        JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                        if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                        {
                                                            long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                        }
                                                        
                                                        JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                        if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                        {
                                                            long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                        }
                                                        
                                                        JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                        if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                        if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                        if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                        if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                        {
                                                            string diskResizedInstance = ((string)diskResizedValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                                if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance2 = ((string)diskResizedValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                                }
                                                
                                                JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                                if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                                {
                                                    string masterTargetIdInstance = ((string)masterTargetIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                                }
                                                
                                                JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                                }
                                                
                                                JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                                if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                                {
                                                    string osTypeInstance = ((string)osTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                                }
                                                
                                                JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                                if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance3 = ((string)vhdNameValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                                }
                                                
                                                JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                                if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                                {
                                                    string osDiskIdInstance = ((string)osDiskIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                                }
                                                
                                                JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                                if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                                    {
                                                        AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                        
                                                        JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                        if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                            azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                        }
                                                        
                                                        JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                        if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance3 = ((string)vhdIdValue3);
                                                            azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                        }
                                                        
                                                        JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                        if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance4 = ((string)vhdNameValue4);
                                                            azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                        }
                                                        
                                                        JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                        if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                        {
                                                            string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                            azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                        }
                                                        
                                                        JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                        if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                            azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                        }
                                                        
                                                        JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                        if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                        {
                                                            string lunIdInstance = ((string)lunIdValue);
                                                            azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                        }
                                                        
                                                        JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                        if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                            azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                                if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                                }
                                                
                                                JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                                if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                                if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                                }
                                                
                                                JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                        
                                                        JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                        if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance3 = ((string)nicIdValue3);
                                                            vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                        if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                            vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                        if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                            vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                            vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                            vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                        if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                            vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                            vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                        }
                                                        
                                                        JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                        if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                            vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                        }
                                                    }
                                                }
                                                
                                                JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                                if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                                }
                                                
                                                JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                                if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                                {
                                                    string discoveryTypeInstance = ((string)discoveryTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                                }
                                                
                                                JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                                if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                                {
                                                    string licenseTypeInstance = ((string)licenseTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                                }
                                                
                                                JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                                if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                                {
                                                    string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                                }
                                                
                                                JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                                if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                                }
                                                
                                                JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                                }
                                                
                                                JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                            }
                                            if (typeName == "InMage")
                                            {
                                                InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                                
                                                JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                                if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                                {
                                                    string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                                    inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                                }
                                                
                                                JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                                    inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                                }
                                                
                                                JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                                if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                                {
                                                    OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                                    inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                                    
                                                    JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                                    if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                                    {
                                                        string osVhdIdInstance = ((string)osVhdIdValue);
                                                        osDetailsInstance.OSVhdId = osVhdIdInstance;
                                                    }
                                                    
                                                    JToken osTypeValue2 = osDetailsValue["osType"];
                                                    if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                                    {
                                                        string osTypeInstance2 = ((string)osTypeValue2);
                                                        osDetailsInstance.OSType = osTypeInstance2;
                                                    }
                                                    
                                                    JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                                    if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                                    {
                                                        string vhdNameInstance5 = ((string)vhdNameValue5);
                                                        osDetailsInstance.VhdName = vhdNameInstance5;
                                                    }
                                                }
                                                
                                                JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                                if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance3 = ((string)protectionStageValue3);
                                                    inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                                }
                                                
                                                JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                                if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                                    inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                                    
                                                    JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                                    if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                        resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                        resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                                    }
                                                }
                                                
                                                JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                                if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                                {
                                                    DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                                    inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                                }
                                                
                                                JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                                if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                                {
                                                    DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                                    inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                                }
                                                
                                                JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                                if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                                {
                                                    double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                                }
                                                
                                                JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                                if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                                {
                                                    double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                                if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                                    inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                                }
                                                
                                                JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                                    {
                                                        InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                        inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                        if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string diskIdInstance2 = ((string)diskIdValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                        }
                                                        
                                                        JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                        if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string diskNameInstance2 = ((string)diskNameValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                        }
                                                        
                                                        JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                        if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                        {
                                                            string protectionStageInstance4 = ((string)protectionStageValue4);
                                                            inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                        }
                                                        
                                                        JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                        if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                        {
                                                            string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                            inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                        }
                                                        
                                                        JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                        if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                        {
                                                            long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                            inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                        }
                                                        
                                                        JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                        if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                        {
                                                            string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                            inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                        }
                                                        
                                                        JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                        if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                        {
                                                            int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                            inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                        }
                                                        
                                                        JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                        if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                        {
                                                            long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                            inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                        }
                                                        
                                                        JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                        if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                        {
                                                            long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                        }
                                                        
                                                        JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                        if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                        {
                                                            long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                            inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                        }
                                                        
                                                        JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                        if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                        }
                                                        
                                                        JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                        if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double psDataInMBInstance = ((double)psDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                        }
                                                        
                                                        JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                        if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                        }
                                                        
                                                        JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                        if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                        {
                                                            string diskResizedInstance3 = ((string)diskResizedValue3);
                                                            inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                        }
                                                    }
                                                }
                                                
                                                JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                                if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressInstance2 = ((string)ipAddressValue2);
                                                    inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                                }
                                                
                                                JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                                if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                                {
                                                    DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                                    inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                                }
                                                
                                                JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                                if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                                {
                                                    string processServerIdInstance2 = ((string)processServerIdValue2);
                                                    inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                                }
                                                
                                                JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                                if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                                {
                                                    string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                                    inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                                }
                                                
                                                JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                                if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                                {
                                                    foreach (JProperty property in consistencyPointsSequenceElement)
                                                    {
                                                        string consistencyPointsKey = ((string)property.Name);
                                                        DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                        inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                                    }
                                                }
                                                
                                                JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                                if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance4 = ((string)diskResizedValue4);
                                                    inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                                }
                                                
                                                JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                                if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                                {
                                                    string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                                    inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                                    inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                                }
                                                
                                                JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                                    inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                                }
                                                
                                                JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                                if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                                    inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                                    
                                                    JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                                    if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                                    {
                                                        string agentVersionInstance2 = ((string)agentVersionValue2);
                                                        agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                                    }
                                                    
                                                    JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                                    if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                                    {
                                                        string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                        agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                                    }
                                                    
                                                    JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                                    if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                                    {
                                                        string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                        agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                                    }
                                                }
                                                
                                                JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                                if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                                {
                                                    string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                                    inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                                }
                                                
                                                JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                                if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                                    inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                                }
                                                
                                                JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                        inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                        
                                                        JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                        if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance4 = ((string)nicIdValue4);
                                                            vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                        if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                            vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                        if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                            vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                            vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                            vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                        if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                            vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                            vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                        }
                                                        
                                                        JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                        if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                            vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                        }
                                                    }
                                                }
                                                
                                                JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                                if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                                    inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                                }
                                                
                                                JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                                    inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                                    inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                                }
                                                
                                                JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                                if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                                    inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                                }
                                                
                                                JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                    inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                            }
                                            if (typeName == "HyperVReplicaAzure")
                                            {
                                                HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                                
                                                JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                                if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                                    {
                                                        AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                        hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                        
                                                        JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                        if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                            azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                        }
                                                        
                                                        JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                        if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance4 = ((string)vhdIdValue4);
                                                            azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                        }
                                                        
                                                        JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                        if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance6 = ((string)vhdNameValue6);
                                                            azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                        }
                                                        
                                                        JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                        if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                        {
                                                            string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                            azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                        }
                                                        
                                                        JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                        if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                            azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                        }
                                                        
                                                        JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                        if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string lunIdInstance2 = ((string)lunIdValue2);
                                                            azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                        }
                                                        
                                                        JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                        if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                            azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                                if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                                }
                                                
                                                JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                                if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                                if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                                }
                                                
                                                JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                                }
                                                
                                                JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance5 = ((string)vmIdValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                                }
                                                
                                                JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                                    hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                                    
                                                    JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                                    if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                        initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                        initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                                    }
                                                }
                                                
                                                JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                        hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                        
                                                        JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                        if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance5 = ((string)nicIdValue5);
                                                            vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                        if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                            vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                        if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                            vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                            vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                            vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                        if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                            vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                            vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                        }
                                                        
                                                        JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                        if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                            vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                        }
                                                    }
                                                }
                                                
                                                JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                                if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                                }
                                                
                                                JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                                if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                                {
                                                    string encryptionInstance = ((string)encryptionValue);
                                                    hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                                }
                                                
                                                JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                                if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                                {
                                                    OSDetails oSDetailsInstance = new OSDetails();
                                                    hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                                    
                                                    JToken osTypeValue3 = oSDetailsValue["osType"];
                                                    if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                                    {
                                                        string osTypeInstance3 = ((string)osTypeValue3);
                                                        oSDetailsInstance.OsType = osTypeInstance3;
                                                    }
                                                    
                                                    JToken productTypeValue = oSDetailsValue["productType"];
                                                    if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                                    {
                                                        string productTypeInstance = ((string)productTypeValue);
                                                        oSDetailsInstance.ProductType = productTypeInstance;
                                                    }
                                                    
                                                    JToken osEditionValue = oSDetailsValue["osEdition"];
                                                    if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                                    {
                                                        string osEditionInstance = ((string)osEditionValue);
                                                        oSDetailsInstance.OSEdition = osEditionInstance;
                                                    }
                                                    
                                                    JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                                    if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSVersionInstance = ((string)oSVersionValue);
                                                        oSDetailsInstance.OSVersion = oSVersionInstance;
                                                    }
                                                    
                                                    JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                                    if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                        oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                                    }
                                                    
                                                    JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                                    if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                        oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                                    }
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                                }
                                                
                                                JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                                }
                                                
                                                JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                                if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                                }
                                                
                                                JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                                if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                                {
                                                    string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                                }
                                                
                                                JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                                }
                                                
                                                JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                    hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                            }
                                        }
                                    }
                                    
                                    JToken idValue = valueValue["id"];
                                    if (idValue != null && idValue.Type != JTokenType.Null)
                                    {
                                        string idInstance = ((string)idValue);
                                        replicationProtectedItemInstance.Id = idInstance;
                                    }
                                    
                                    JToken nameValue = valueValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        replicationProtectedItemInstance.Name = nameInstance;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        replicationProtectedItemInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        replicationProtectedItemInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property2 in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property2.Name);
                                            string tagsValue = ((string)property2.Value);
                                            replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Enumerate all replication protected items under vault.
        /// </summary>
        /// <param name='skipToken'>
        /// Optional. Continuation Token.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Protected items query parameter.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list replicated protected items.
        /// </returns>
        public async Task<ReplicationProtectedItemListResponse> ListAllAsync(string skipToken, ProtectedItemsQueryParameter parameters, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("skipToken", skipToken);
                tracingParameters.Add("parameters", parameters);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ListAllAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationProtectedItems";
            List<string> queryParameters = new List<string>();
            if (skipToken != null)
            {
                queryParameters.Add("skipToken=" + Uri.EscapeDataString(skipToken));
            }
            queryParameters.Add("api-version=2015-11-10");
            List<string> odataFilter = new List<string>();
            if (parameters != null && parameters.ToQueryString() != null)
            {
                odataFilter.Add(Uri.EscapeDataString(parameters.ToQueryString().ToString()));
            }
            if (odataFilter.Count > 0)
            {
                queryParameters.Add("$filter=" + string.Join(null, odataFilter));
            }
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                                    result.ReplicationProtectedItems.Add(replicationProtectedItemInstance);
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                        replicationProtectedItemInstance.Properties = propertiesInstance;
                                        
                                        JToken friendlyNameValue = propertiesValue["friendlyName"];
                                        if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance = ((string)friendlyNameValue);
                                            propertiesInstance.FriendlyName = friendlyNameInstance;
                                        }
                                        
                                        JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                        if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                        {
                                            string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                            propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                        }
                                        
                                        JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                        if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                        {
                                            string protectableItemIdInstance = ((string)protectableItemIdValue);
                                            propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                        }
                                        
                                        JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                        if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                            propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                        }
                                        
                                        JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                        if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                            propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                        if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                            propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                        if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                            propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                        }
                                        
                                        JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                        if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                            propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                        if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                            propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                        }
                                        
                                        JToken protectionStateValue = propertiesValue["protectionState"];
                                        if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                        {
                                            string protectionStateInstance = ((string)protectionStateValue);
                                            propertiesInstance.ProtectionState = protectionStateInstance;
                                        }
                                        
                                        JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                        if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                            propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                        }
                                        
                                        JToken activeLocationValue = propertiesValue["activeLocation"];
                                        if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                        {
                                            string activeLocationInstance = ((string)activeLocationValue);
                                            propertiesInstance.ActiveLocation = activeLocationInstance;
                                        }
                                        
                                        JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                        if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStateInstance = ((string)testFailoverStateValue);
                                            propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                        }
                                        
                                        JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                        if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                            propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                        }
                                        
                                        JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                        if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                            {
                                                propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                            }
                                        }
                                        
                                        JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                        if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                        {
                                            string replicationHealthInstance = ((string)replicationHealthValue);
                                            propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                        }
                                        
                                        JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                        if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                            {
                                                HealthError healthErrorInstance = new HealthError();
                                                propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                                
                                                JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                    healthErrorInstance.ErrorLevel = errorLevelInstance;
                                                }
                                                
                                                JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                                if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string errorCodeInstance = ((string)errorCodeValue);
                                                    healthErrorInstance.ErrorCode = errorCodeInstance;
                                                }
                                                
                                                JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                                if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                {
                                                    string errorMessageInstance = ((string)errorMessageValue);
                                                    healthErrorInstance.ErrorMessage = errorMessageInstance;
                                                }
                                                
                                                JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                                if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                {
                                                    string possibleCausesInstance = ((string)possibleCausesValue);
                                                    healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                                }
                                                
                                                JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                                if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                {
                                                    string recommendedActionInstance = ((string)recommendedActionValue);
                                                    healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                                }
                                                
                                                JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                                if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                                {
                                                    string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                                    healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                                }
                                                
                                                JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                                if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                                    healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                                }
                                                
                                                JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                                if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                                {
                                                    string entityIdInstance = ((string)entityIdValue);
                                                    healthErrorInstance.EntityId = entityIdInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken policyIdValue = propertiesValue["policyId"];
                                        if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                        {
                                            string policyIdInstance = ((string)policyIdValue);
                                            propertiesInstance.PolicyID = policyIdInstance;
                                        }
                                        
                                        JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                        if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                            propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                        }
                                        
                                        JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                        if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                            propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                        }
                                        
                                        JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                        if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                            propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                        }
                                        
                                        JToken currentScenarioValue = propertiesValue["currentScenario"];
                                        if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                        {
                                            CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                            propertiesInstance.CurrentScenario = currentScenarioInstance;
                                            
                                            JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                            if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                            {
                                                string scenarioNameInstance = ((string)scenarioNameValue);
                                                currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                            }
                                            
                                            JToken jobIdValue = currentScenarioValue["jobId"];
                                            if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                            {
                                                string jobIdInstance = ((string)jobIdValue);
                                                currentScenarioInstance.JobId = jobIdInstance;
                                            }
                                            
                                            JToken startTimeValue = currentScenarioValue["startTime"];
                                            if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                            {
                                                DateTime startTimeInstance = ((DateTime)startTimeValue);
                                                currentScenarioInstance.StartTime = startTimeInstance;
                                            }
                                        }
                                        
                                        JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                        if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                        {
                                            string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                            propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                        }
                                        
                                        JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                        if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                            if (typeName == "HyperVReplica2012")
                                            {
                                                HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                                
                                                JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                                }
                                                
                                                JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance = ((string)vmIdValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                                }
                                                
                                                JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                        hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                        
                                                        JToken nicIdValue = vmNicsValue["nicId"];
                                                        if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance = ((string)nicIdValue);
                                                            vMNicDetailsInstance.NicId = nicIdInstance;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                        if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                            vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                        if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                            vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                            vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                            vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                        if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                            vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                            vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                        }
                                                        
                                                        JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                        if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance = ((string)selectionTypeValue);
                                                            vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                                    hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                                    
                                                    JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                                    if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                        initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                        initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                                    }
                                                }
                                                
                                                JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                                if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                                    {
                                                        DiskDetails diskDetailsInstance = new DiskDetails();
                                                        hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                        
                                                        JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                        if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                        {
                                                            ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                            diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                        }
                                                        
                                                        JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                        if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance = ((string)vhdTypeValue);
                                                            diskDetailsInstance.VhdType = vhdTypeInstance;
                                                        }
                                                        
                                                        JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                        if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance = ((string)vhdIdValue);
                                                            diskDetailsInstance.VhdId = vhdIdInstance;
                                                        }
                                                        
                                                        JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                        if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance = ((string)vhdNameValue);
                                                            diskDetailsInstance.VhdName = vhdNameInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                                }
                                                
                                                JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                            }
                                            if (typeName == "HyperVReplica2012R2")
                                            {
                                                HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                                
                                                JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                                }
                                                
                                                JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                                }
                                                
                                                JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                        hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                        
                                                        JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                        if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance2 = ((string)nicIdValue2);
                                                            vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                        if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                            vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                        if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                            vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                            vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                            vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                        if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                            vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                            vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                        }
                                                        
                                                        JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                        if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                            vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                                    hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                                    
                                                    JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                                    if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                        initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                        initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                                    }
                                                }
                                                
                                                JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                                if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                                    {
                                                        DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                        hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                        
                                                        JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                        if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                        {
                                                            ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                            diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                        }
                                                        
                                                        JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                        if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                            diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                        }
                                                        
                                                        JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                        if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance2 = ((string)vhdIdValue2);
                                                            diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                        }
                                                        
                                                        JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                        if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance2 = ((string)vhdNameValue2);
                                                            diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                                }
                                                
                                                JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                            }
                                            if (typeName == "A2A")
                                            {
                                                A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                                
                                                JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                                if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                                {
                                                    string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                                    a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                                    a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                                }
                                                
                                                JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                                    a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                                }
                                                
                                                JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                                    {
                                                        A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                        a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskUriValue = protectedDisksValue["diskUri"];
                                                        if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                        {
                                                            string diskUriInstance = ((string)diskUriValue);
                                                            a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                        }
                                                        
                                                        JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                        if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                        if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                        if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                        if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                            a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                    a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                            }
                                            if (typeName == "InMageAzureV2")
                                            {
                                                InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                                
                                                JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                                if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                                {
                                                    string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                                }
                                                
                                                JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                                if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                                {
                                                    string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                                }
                                                
                                                JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                                if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance = ((string)protectionStageValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                                }
                                                
                                                JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                                }
                                                
                                                JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                                if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                                }
                                                
                                                JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                                if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                                {
                                                    double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                                }
                                                
                                                JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                                if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                                {
                                                    double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                                }
                                                
                                                JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                                if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressInstance = ((string)ipAddressValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                                }
                                                
                                                JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                                if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                                {
                                                    string agentVersionInstance = ((string)agentVersionValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                                }
                                                
                                                JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                                if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                                }
                                                
                                                JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                                if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                                }
                                                
                                                JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                                if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                                {
                                                    DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                                }
                                                
                                                JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                                if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                                {
                                                    string processServerIdInstance = ((string)processServerIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                                }
                                                
                                                JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                                }
                                                
                                                JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                                    {
                                                        InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskIdValue = protectedDisksValue2["diskId"];
                                                        if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string diskIdInstance = ((string)diskIdValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                        }
                                                        
                                                        JToken diskNameValue = protectedDisksValue2["diskName"];
                                                        if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                        {
                                                            string diskNameInstance = ((string)diskNameValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                        }
                                                        
                                                        JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                        if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                        {
                                                            string protectionStageInstance2 = ((string)protectionStageValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                        }
                                                        
                                                        JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                        if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                        {
                                                            string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                        }
                                                        
                                                        JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                        if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                        {
                                                            long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                        }
                                                        
                                                        JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                        if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                        {
                                                            string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                        }
                                                        
                                                        JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                        if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                        {
                                                            int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                        }
                                                        
                                                        JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                        if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                        {
                                                            long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                        }
                                                        
                                                        JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                        if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                        {
                                                            long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                        }
                                                        
                                                        JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                        if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                        {
                                                            long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                        }
                                                        
                                                        JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                        if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                        if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                        if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                        if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                        {
                                                            string diskResizedInstance = ((string)diskResizedValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                                if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance2 = ((string)diskResizedValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                                }
                                                
                                                JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                                if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                                {
                                                    string masterTargetIdInstance = ((string)masterTargetIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                                }
                                                
                                                JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                                }
                                                
                                                JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                                if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                                {
                                                    string osTypeInstance = ((string)osTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                                }
                                                
                                                JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                                if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance3 = ((string)vhdNameValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                                }
                                                
                                                JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                                if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                                {
                                                    string osDiskIdInstance = ((string)osDiskIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                                }
                                                
                                                JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                                if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                                    {
                                                        AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                        
                                                        JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                        if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                            azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                        }
                                                        
                                                        JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                        if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance3 = ((string)vhdIdValue3);
                                                            azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                        }
                                                        
                                                        JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                        if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance4 = ((string)vhdNameValue4);
                                                            azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                        }
                                                        
                                                        JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                        if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                        {
                                                            string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                            azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                        }
                                                        
                                                        JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                        if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                            azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                        }
                                                        
                                                        JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                        if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                        {
                                                            string lunIdInstance = ((string)lunIdValue);
                                                            azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                        }
                                                        
                                                        JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                        if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                            azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                                if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                                }
                                                
                                                JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                                if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                                if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                                }
                                                
                                                JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                        
                                                        JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                        if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance3 = ((string)nicIdValue3);
                                                            vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                        if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                            vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                        if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                            vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                            vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                            vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                        if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                            vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                            vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                        }
                                                        
                                                        JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                        if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                            vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                        }
                                                    }
                                                }
                                                
                                                JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                                if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                                }
                                                
                                                JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                                if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                                {
                                                    string discoveryTypeInstance = ((string)discoveryTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                                }
                                                
                                                JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                                if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                                {
                                                    string licenseTypeInstance = ((string)licenseTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                                }
                                                
                                                JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                                if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                                {
                                                    string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                                }
                                                
                                                JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                                if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                                }
                                                
                                                JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                                }
                                                
                                                JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                            }
                                            if (typeName == "InMage")
                                            {
                                                InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                                
                                                JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                                if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                                {
                                                    string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                                    inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                                }
                                                
                                                JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                                    inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                                }
                                                
                                                JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                                if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                                {
                                                    OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                                    inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                                    
                                                    JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                                    if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                                    {
                                                        string osVhdIdInstance = ((string)osVhdIdValue);
                                                        osDetailsInstance.OSVhdId = osVhdIdInstance;
                                                    }
                                                    
                                                    JToken osTypeValue2 = osDetailsValue["osType"];
                                                    if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                                    {
                                                        string osTypeInstance2 = ((string)osTypeValue2);
                                                        osDetailsInstance.OSType = osTypeInstance2;
                                                    }
                                                    
                                                    JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                                    if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                                    {
                                                        string vhdNameInstance5 = ((string)vhdNameValue5);
                                                        osDetailsInstance.VhdName = vhdNameInstance5;
                                                    }
                                                }
                                                
                                                JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                                if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance3 = ((string)protectionStageValue3);
                                                    inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                                }
                                                
                                                JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                                if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                                    inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                                    
                                                    JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                                    if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                        resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                        resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                                    }
                                                }
                                                
                                                JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                                if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                                {
                                                    DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                                    inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                                }
                                                
                                                JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                                if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                                {
                                                    DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                                    inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                                }
                                                
                                                JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                                if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                                {
                                                    double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                                }
                                                
                                                JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                                if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                                {
                                                    double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                                if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                                    inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                                }
                                                
                                                JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                                    {
                                                        InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                        inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                        if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string diskIdInstance2 = ((string)diskIdValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                        }
                                                        
                                                        JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                        if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string diskNameInstance2 = ((string)diskNameValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                        }
                                                        
                                                        JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                        if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                        {
                                                            string protectionStageInstance4 = ((string)protectionStageValue4);
                                                            inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                        }
                                                        
                                                        JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                        if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                        {
                                                            string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                            inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                        }
                                                        
                                                        JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                        if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                        {
                                                            long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                            inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                        }
                                                        
                                                        JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                        if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                        {
                                                            string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                            inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                        }
                                                        
                                                        JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                        if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                        {
                                                            int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                            inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                        }
                                                        
                                                        JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                        if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                        {
                                                            long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                            inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                        }
                                                        
                                                        JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                        if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                        {
                                                            long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                        }
                                                        
                                                        JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                        if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                        {
                                                            long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                            inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                        }
                                                        
                                                        JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                        if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                        }
                                                        
                                                        JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                        if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double psDataInMBInstance = ((double)psDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                        }
                                                        
                                                        JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                        if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                        }
                                                        
                                                        JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                        if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                        {
                                                            string diskResizedInstance3 = ((string)diskResizedValue3);
                                                            inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                        }
                                                    }
                                                }
                                                
                                                JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                                if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressInstance2 = ((string)ipAddressValue2);
                                                    inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                                }
                                                
                                                JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                                if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                                {
                                                    DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                                    inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                                }
                                                
                                                JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                                if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                                {
                                                    string processServerIdInstance2 = ((string)processServerIdValue2);
                                                    inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                                }
                                                
                                                JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                                if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                                {
                                                    string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                                    inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                                }
                                                
                                                JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                                if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                                {
                                                    foreach (JProperty property in consistencyPointsSequenceElement)
                                                    {
                                                        string consistencyPointsKey = ((string)property.Name);
                                                        DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                        inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                                    }
                                                }
                                                
                                                JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                                if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance4 = ((string)diskResizedValue4);
                                                    inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                                }
                                                
                                                JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                                if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                                {
                                                    string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                                    inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                                    inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                                }
                                                
                                                JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                                    inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                                }
                                                
                                                JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                                if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                                    inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                                    
                                                    JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                                    if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                                    {
                                                        string agentVersionInstance2 = ((string)agentVersionValue2);
                                                        agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                                    }
                                                    
                                                    JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                                    if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                                    {
                                                        string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                        agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                                    }
                                                    
                                                    JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                                    if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                                    {
                                                        string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                        agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                                    }
                                                }
                                                
                                                JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                                if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                                {
                                                    string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                                    inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                                }
                                                
                                                JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                                if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                                    inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                                }
                                                
                                                JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                        inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                        
                                                        JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                        if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance4 = ((string)nicIdValue4);
                                                            vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                        if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                            vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                        if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                            vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                            vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                            vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                        if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                            vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                            vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                        }
                                                        
                                                        JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                        if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                            vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                        }
                                                    }
                                                }
                                                
                                                JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                                if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                                    inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                                }
                                                
                                                JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                                    inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                                    inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                                }
                                                
                                                JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                                if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                                    inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                                }
                                                
                                                JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                    inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                            }
                                            if (typeName == "HyperVReplicaAzure")
                                            {
                                                HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                                
                                                JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                                if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                                    {
                                                        AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                        hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                        
                                                        JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                        if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                            azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                        }
                                                        
                                                        JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                        if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance4 = ((string)vhdIdValue4);
                                                            azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                        }
                                                        
                                                        JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                        if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance6 = ((string)vhdNameValue6);
                                                            azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                        }
                                                        
                                                        JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                        if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                        {
                                                            string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                            azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                        }
                                                        
                                                        JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                        if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                            azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                        }
                                                        
                                                        JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                        if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string lunIdInstance2 = ((string)lunIdValue2);
                                                            azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                        }
                                                        
                                                        JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                        if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                            azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                                if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                                }
                                                
                                                JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                                if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                                if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                                }
                                                
                                                JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                                }
                                                
                                                JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance5 = ((string)vmIdValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                                }
                                                
                                                JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                                    hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                                    
                                                    JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                                    if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                        initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                        initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                                    }
                                                }
                                                
                                                JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                        hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                        
                                                        JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                        if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance5 = ((string)nicIdValue5);
                                                            vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                        if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                            vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                        if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                            vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                            vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                            vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                        if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                            vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                            vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                        }
                                                        
                                                        JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                        if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                            vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                        }
                                                    }
                                                }
                                                
                                                JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                                if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                                }
                                                
                                                JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                                if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                                {
                                                    string encryptionInstance = ((string)encryptionValue);
                                                    hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                                }
                                                
                                                JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                                if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                                {
                                                    OSDetails oSDetailsInstance = new OSDetails();
                                                    hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                                    
                                                    JToken osTypeValue3 = oSDetailsValue["osType"];
                                                    if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                                    {
                                                        string osTypeInstance3 = ((string)osTypeValue3);
                                                        oSDetailsInstance.OsType = osTypeInstance3;
                                                    }
                                                    
                                                    JToken productTypeValue = oSDetailsValue["productType"];
                                                    if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                                    {
                                                        string productTypeInstance = ((string)productTypeValue);
                                                        oSDetailsInstance.ProductType = productTypeInstance;
                                                    }
                                                    
                                                    JToken osEditionValue = oSDetailsValue["osEdition"];
                                                    if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                                    {
                                                        string osEditionInstance = ((string)osEditionValue);
                                                        oSDetailsInstance.OSEdition = osEditionInstance;
                                                    }
                                                    
                                                    JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                                    if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSVersionInstance = ((string)oSVersionValue);
                                                        oSDetailsInstance.OSVersion = oSVersionInstance;
                                                    }
                                                    
                                                    JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                                    if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                        oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                                    }
                                                    
                                                    JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                                    if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                        oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                                    }
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                                }
                                                
                                                JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                                }
                                                
                                                JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                                if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                                }
                                                
                                                JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                                if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                                {
                                                    string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                                }
                                                
                                                JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                                }
                                                
                                                JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                    hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                            }
                                        }
                                    }
                                    
                                    JToken idValue = valueValue["id"];
                                    if (idValue != null && idValue.Type != JTokenType.Null)
                                    {
                                        string idInstance = ((string)idValue);
                                        replicationProtectedItemInstance.Id = idInstance;
                                    }
                                    
                                    JToken nameValue = valueValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        replicationProtectedItemInstance.Name = nameInstance;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        replicationProtectedItemInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        replicationProtectedItemInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property2 in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property2.Name);
                                            string tagsValue = ((string)property2.Value);
                                            replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Get subsequent page data for replication protected items under
        /// vault.
        /// </summary>
        /// <param name='nextLink'>
        /// Required. The url to the next protected items page.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list replicated protected items.
        /// </returns>
        public async Task<ReplicationProtectedItemListResponse> ListAllNextAsync(string nextLink, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (nextLink == null)
            {
                throw new ArgumentNullException("nextLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("nextLink", nextLink);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ListAllNextAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + nextLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ReplicationProtectedItemListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ReplicationProtectedItemListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    ReplicationProtectedItem replicationProtectedItemInstance = new ReplicationProtectedItem();
                                    result.ReplicationProtectedItems.Add(replicationProtectedItemInstance);
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        ReplicationProtectedItemProperties propertiesInstance = new ReplicationProtectedItemProperties();
                                        replicationProtectedItemInstance.Properties = propertiesInstance;
                                        
                                        JToken friendlyNameValue = propertiesValue["friendlyName"];
                                        if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string friendlyNameInstance = ((string)friendlyNameValue);
                                            propertiesInstance.FriendlyName = friendlyNameInstance;
                                        }
                                        
                                        JToken protectedItemTypeValue = propertiesValue["protectedItemType"];
                                        if (protectedItemTypeValue != null && protectedItemTypeValue.Type != JTokenType.Null)
                                        {
                                            string protectedItemTypeInstance = ((string)protectedItemTypeValue);
                                            propertiesInstance.ProtectedItemType = protectedItemTypeInstance;
                                        }
                                        
                                        JToken protectableItemIdValue = propertiesValue["protectableItemId"];
                                        if (protectableItemIdValue != null && protectableItemIdValue.Type != JTokenType.Null)
                                        {
                                            string protectableItemIdInstance = ((string)protectableItemIdValue);
                                            propertiesInstance.ProtectableItemId = protectableItemIdInstance;
                                        }
                                        
                                        JToken recoveryServicesProviderIdValue = propertiesValue["recoveryServicesProviderId"];
                                        if (recoveryServicesProviderIdValue != null && recoveryServicesProviderIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryServicesProviderIdInstance = ((string)recoveryServicesProviderIdValue);
                                            propertiesInstance.RecoveryServicesProviderId = recoveryServicesProviderIdInstance;
                                        }
                                        
                                        JToken primaryFabricFriendlyNameValue = propertiesValue["primaryFabricFriendlyName"];
                                        if (primaryFabricFriendlyNameValue != null && primaryFabricFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string primaryFabricFriendlyNameInstance = ((string)primaryFabricFriendlyNameValue);
                                            propertiesInstance.PrimaryFabricFriendlyName = primaryFabricFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryFabricFriendlyNameValue = propertiesValue["recoveryFabricFriendlyName"];
                                        if (recoveryFabricFriendlyNameValue != null && recoveryFabricFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryFabricFriendlyNameInstance = ((string)recoveryFabricFriendlyNameValue);
                                            propertiesInstance.RecoveryFabricFriendlyName = recoveryFabricFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryFabricIdValue = propertiesValue["recoveryFabricId"];
                                        if (recoveryFabricIdValue != null && recoveryFabricIdValue.Type != JTokenType.Null)
                                        {
                                            string recoveryFabricIdInstance = ((string)recoveryFabricIdValue);
                                            propertiesInstance.RecoveryFabricId = recoveryFabricIdInstance;
                                        }
                                        
                                        JToken primaryProtectionContainerFriendlyNameValue = propertiesValue["primaryProtectionContainerFriendlyName"];
                                        if (primaryProtectionContainerFriendlyNameValue != null && primaryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string primaryProtectionContainerFriendlyNameInstance = ((string)primaryProtectionContainerFriendlyNameValue);
                                            propertiesInstance.PrimaryProtectionContainerFriendlyName = primaryProtectionContainerFriendlyNameInstance;
                                        }
                                        
                                        JToken recoveryProtectionContainerFriendlyNameValue = propertiesValue["recoveryProtectionContainerFriendlyName"];
                                        if (recoveryProtectionContainerFriendlyNameValue != null && recoveryProtectionContainerFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string recoveryProtectionContainerFriendlyNameInstance = ((string)recoveryProtectionContainerFriendlyNameValue);
                                            propertiesInstance.RecoveryProtectionContainerFriendlyName = recoveryProtectionContainerFriendlyNameInstance;
                                        }
                                        
                                        JToken protectionStateValue = propertiesValue["protectionState"];
                                        if (protectionStateValue != null && protectionStateValue.Type != JTokenType.Null)
                                        {
                                            string protectionStateInstance = ((string)protectionStateValue);
                                            propertiesInstance.ProtectionState = protectionStateInstance;
                                        }
                                        
                                        JToken protectionStateDescriptionValue = propertiesValue["protectionStateDescription"];
                                        if (protectionStateDescriptionValue != null && protectionStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string protectionStateDescriptionInstance = ((string)protectionStateDescriptionValue);
                                            propertiesInstance.ProtectionStateDescription = protectionStateDescriptionInstance;
                                        }
                                        
                                        JToken activeLocationValue = propertiesValue["activeLocation"];
                                        if (activeLocationValue != null && activeLocationValue.Type != JTokenType.Null)
                                        {
                                            string activeLocationInstance = ((string)activeLocationValue);
                                            propertiesInstance.ActiveLocation = activeLocationInstance;
                                        }
                                        
                                        JToken testFailoverStateValue = propertiesValue["testFailoverState"];
                                        if (testFailoverStateValue != null && testFailoverStateValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStateInstance = ((string)testFailoverStateValue);
                                            propertiesInstance.TestFailoverState = testFailoverStateInstance;
                                        }
                                        
                                        JToken testFailoverStateDescriptionValue = propertiesValue["testFailoverStateDescription"];
                                        if (testFailoverStateDescriptionValue != null && testFailoverStateDescriptionValue.Type != JTokenType.Null)
                                        {
                                            string testFailoverStateDescriptionInstance = ((string)testFailoverStateDescriptionValue);
                                            propertiesInstance.TestFailoverStateDescription = testFailoverStateDescriptionInstance;
                                        }
                                        
                                        JToken allowedOperationsArray = propertiesValue["allowedOperations"];
                                        if (allowedOperationsArray != null && allowedOperationsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken allowedOperationsValue in ((JArray)allowedOperationsArray))
                                            {
                                                propertiesInstance.AllowedOperations.Add(((string)allowedOperationsValue));
                                            }
                                        }
                                        
                                        JToken replicationHealthValue = propertiesValue["replicationHealth"];
                                        if (replicationHealthValue != null && replicationHealthValue.Type != JTokenType.Null)
                                        {
                                            string replicationHealthInstance = ((string)replicationHealthValue);
                                            propertiesInstance.ReplicationHealth = replicationHealthInstance;
                                        }
                                        
                                        JToken replicationHealthErrorsArray = propertiesValue["replicationHealthErrors"];
                                        if (replicationHealthErrorsArray != null && replicationHealthErrorsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken replicationHealthErrorsValue in ((JArray)replicationHealthErrorsArray))
                                            {
                                                HealthError healthErrorInstance = new HealthError();
                                                propertiesInstance.ReplicationHealthErrors.Add(healthErrorInstance);
                                                
                                                JToken errorLevelValue = replicationHealthErrorsValue["errorLevel"];
                                                if (errorLevelValue != null && errorLevelValue.Type != JTokenType.Null)
                                                {
                                                    string errorLevelInstance = ((string)errorLevelValue);
                                                    healthErrorInstance.ErrorLevel = errorLevelInstance;
                                                }
                                                
                                                JToken errorCodeValue = replicationHealthErrorsValue["errorCode"];
                                                if (errorCodeValue != null && errorCodeValue.Type != JTokenType.Null)
                                                {
                                                    string errorCodeInstance = ((string)errorCodeValue);
                                                    healthErrorInstance.ErrorCode = errorCodeInstance;
                                                }
                                                
                                                JToken errorMessageValue = replicationHealthErrorsValue["errorMessage"];
                                                if (errorMessageValue != null && errorMessageValue.Type != JTokenType.Null)
                                                {
                                                    string errorMessageInstance = ((string)errorMessageValue);
                                                    healthErrorInstance.ErrorMessage = errorMessageInstance;
                                                }
                                                
                                                JToken possibleCausesValue = replicationHealthErrorsValue["possibleCauses"];
                                                if (possibleCausesValue != null && possibleCausesValue.Type != JTokenType.Null)
                                                {
                                                    string possibleCausesInstance = ((string)possibleCausesValue);
                                                    healthErrorInstance.PossibleCauses = possibleCausesInstance;
                                                }
                                                
                                                JToken recommendedActionValue = replicationHealthErrorsValue["recommendedAction"];
                                                if (recommendedActionValue != null && recommendedActionValue.Type != JTokenType.Null)
                                                {
                                                    string recommendedActionInstance = ((string)recommendedActionValue);
                                                    healthErrorInstance.RecommendedAction = recommendedActionInstance;
                                                }
                                                
                                                JToken creationTimeUtcValue = replicationHealthErrorsValue["creationTimeUtc"];
                                                if (creationTimeUtcValue != null && creationTimeUtcValue.Type != JTokenType.Null)
                                                {
                                                    string creationTimeUtcInstance = ((string)creationTimeUtcValue);
                                                    healthErrorInstance.CreationTimeUtc = creationTimeUtcInstance;
                                                }
                                                
                                                JToken recoveryProviderErrorMessageValue = replicationHealthErrorsValue["recoveryProviderErrorMessage"];
                                                if (recoveryProviderErrorMessageValue != null && recoveryProviderErrorMessageValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryProviderErrorMessageInstance = ((string)recoveryProviderErrorMessageValue);
                                                    healthErrorInstance.RecoveryProviderErrorMessage = recoveryProviderErrorMessageInstance;
                                                }
                                                
                                                JToken entityIdValue = replicationHealthErrorsValue["entityId"];
                                                if (entityIdValue != null && entityIdValue.Type != JTokenType.Null)
                                                {
                                                    string entityIdInstance = ((string)entityIdValue);
                                                    healthErrorInstance.EntityId = entityIdInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken policyIdValue = propertiesValue["policyId"];
                                        if (policyIdValue != null && policyIdValue.Type != JTokenType.Null)
                                        {
                                            string policyIdInstance = ((string)policyIdValue);
                                            propertiesInstance.PolicyID = policyIdInstance;
                                        }
                                        
                                        JToken policyFriendlyNameValue = propertiesValue["policyFriendlyName"];
                                        if (policyFriendlyNameValue != null && policyFriendlyNameValue.Type != JTokenType.Null)
                                        {
                                            string policyFriendlyNameInstance = ((string)policyFriendlyNameValue);
                                            propertiesInstance.PolicyFriendlyName = policyFriendlyNameInstance;
                                        }
                                        
                                        JToken lastSuccessfulFailoverTimeValue = propertiesValue["lastSuccessfulFailoverTime"];
                                        if (lastSuccessfulFailoverTimeValue != null && lastSuccessfulFailoverTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastSuccessfulFailoverTimeInstance = ((DateTime)lastSuccessfulFailoverTimeValue);
                                            propertiesInstance.LastSuccessfulFailoverTime = lastSuccessfulFailoverTimeInstance;
                                        }
                                        
                                        JToken lastSuccessfulTestFailoverTimeValue = propertiesValue["lastSuccessfulTestFailoverTime"];
                                        if (lastSuccessfulTestFailoverTimeValue != null && lastSuccessfulTestFailoverTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastSuccessfulTestFailoverTimeInstance = ((DateTime)lastSuccessfulTestFailoverTimeValue);
                                            propertiesInstance.LastSuccessfulTestFailoverTime = lastSuccessfulTestFailoverTimeInstance;
                                        }
                                        
                                        JToken currentScenarioValue = propertiesValue["currentScenario"];
                                        if (currentScenarioValue != null && currentScenarioValue.Type != JTokenType.Null)
                                        {
                                            CurrentScenarioDetails currentScenarioInstance = new CurrentScenarioDetails();
                                            propertiesInstance.CurrentScenario = currentScenarioInstance;
                                            
                                            JToken scenarioNameValue = currentScenarioValue["scenarioName"];
                                            if (scenarioNameValue != null && scenarioNameValue.Type != JTokenType.Null)
                                            {
                                                string scenarioNameInstance = ((string)scenarioNameValue);
                                                currentScenarioInstance.ScenarioName = scenarioNameInstance;
                                            }
                                            
                                            JToken jobIdValue = currentScenarioValue["jobId"];
                                            if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                            {
                                                string jobIdInstance = ((string)jobIdValue);
                                                currentScenarioInstance.JobId = jobIdInstance;
                                            }
                                            
                                            JToken startTimeValue = currentScenarioValue["startTime"];
                                            if (startTimeValue != null && startTimeValue.Type != JTokenType.Null)
                                            {
                                                DateTime startTimeInstance = ((DateTime)startTimeValue);
                                                currentScenarioInstance.StartTime = startTimeInstance;
                                            }
                                        }
                                        
                                        JToken failoverRecoveryPointIdValue = propertiesValue["failoverRecoveryPointId"];
                                        if (failoverRecoveryPointIdValue != null && failoverRecoveryPointIdValue.Type != JTokenType.Null)
                                        {
                                            string failoverRecoveryPointIdInstance = ((string)failoverRecoveryPointIdValue);
                                            propertiesInstance.FailoverRecoveryPointId = failoverRecoveryPointIdInstance;
                                        }
                                        
                                        JToken providerSpecificDetailsValue = propertiesValue["providerSpecificDetails"];
                                        if (providerSpecificDetailsValue != null && providerSpecificDetailsValue.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)providerSpecificDetailsValue["instanceType"]);
                                            if (typeName == "HyperVReplica2012")
                                            {
                                                HyperVReplica2012ReplicationDetails hyperVReplica2012ReplicationDetailsInstance = new HyperVReplica2012ReplicationDetails();
                                                
                                                JToken lastReplicatedTimeValue = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue != null && lastReplicatedTimeValue.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance = ((DateTime)lastReplicatedTimeValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance;
                                                }
                                                
                                                JToken vmIdValue = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue != null && vmIdValue.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance = ((string)vmIdValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmId = vmIdInstance;
                                                }
                                                
                                                JToken vmNicsArray = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray != null && vmNicsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue in ((JArray)vmNicsArray))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance = new VMNicDetails();
                                                        hyperVReplica2012ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance);
                                                        
                                                        JToken nicIdValue = vmNicsValue["nicId"];
                                                        if (nicIdValue != null && nicIdValue.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance = ((string)nicIdValue);
                                                            vMNicDetailsInstance.NicId = nicIdInstance;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue = vmNicsValue["vMSubnetName"];
                                                        if (vMSubnetNameValue != null && vMSubnetNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance = ((string)vMSubnetNameValue);
                                                            vMNicDetailsInstance.VMSubnetName = vMSubnetNameInstance;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue = vmNicsValue["vMNetworkName"];
                                                        if (vMNetworkNameValue != null && vMNetworkNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance = ((string)vMNetworkNameValue);
                                                            vMNicDetailsInstance.VMNetworkName = vMNetworkNameInstance;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue = vmNicsValue["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue != null && recoveryVMNetworkIdValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance = ((string)recoveryVMNetworkIdValue);
                                                            vMNicDetailsInstance.RecoveryVMNetworkId = recoveryVMNetworkIdInstance;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue = vmNicsValue["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue != null && recoveryVMSubnetNameValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance = ((string)recoveryVMSubnetNameValue);
                                                            vMNicDetailsInstance.RecoveryVMSubnetName = recoveryVMSubnetNameInstance;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue = vmNicsValue["ipAddressType"];
                                                        if (ipAddressTypeValue != null && ipAddressTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance = ((string)ipAddressTypeValue);
                                                            vMNicDetailsInstance.IpAddressType = ipAddressTypeInstance;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue = vmNicsValue["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue != null && replicaNicStaticIPAddressValue.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance = ((string)replicaNicStaticIPAddressValue);
                                                            vMNicDetailsInstance.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance;
                                                        }
                                                        
                                                        JToken selectionTypeValue = vmNicsValue["selectionType"];
                                                        if (selectionTypeValue != null && selectionTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance = ((string)selectionTypeValue);
                                                            vMNicDetailsInstance.SelectionType = selectionTypeInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken initialReplicationDetailsValue = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue != null && initialReplicationDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance = new InitialReplicationDetails();
                                                    hyperVReplica2012ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance;
                                                    
                                                    JToken initialReplicationTypeValue = initialReplicationDetailsValue["initialReplicationType"];
                                                    if (initialReplicationTypeValue != null && initialReplicationTypeValue.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance = ((string)initialReplicationTypeValue);
                                                        initialReplicationDetailsInstance.InitialReplicationType = initialReplicationTypeInstance;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue = initialReplicationDetailsValue["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue != null && initialReplicationProgressPercentageValue.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance = ((string)initialReplicationProgressPercentageValue);
                                                        initialReplicationDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance;
                                                    }
                                                }
                                                
                                                JToken vMDiskDetailsArray = providerSpecificDetailsValue["vMDiskDetails"];
                                                if (vMDiskDetailsArray != null && vMDiskDetailsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vMDiskDetailsValue in ((JArray)vMDiskDetailsArray))
                                                    {
                                                        DiskDetails diskDetailsInstance = new DiskDetails();
                                                        hyperVReplica2012ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance);
                                                        
                                                        JToken maxSizeMBValue = vMDiskDetailsValue["maxSizeMB"];
                                                        if (maxSizeMBValue != null && maxSizeMBValue.Type != JTokenType.Null)
                                                        {
                                                            ulong maxSizeMBInstance = ((ulong)maxSizeMBValue);
                                                            diskDetailsInstance.MaxSizeMB = maxSizeMBInstance;
                                                        }
                                                        
                                                        JToken vhdTypeValue = vMDiskDetailsValue["vhdType"];
                                                        if (vhdTypeValue != null && vhdTypeValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance = ((string)vhdTypeValue);
                                                            diskDetailsInstance.VhdType = vhdTypeInstance;
                                                        }
                                                        
                                                        JToken vhdIdValue = vMDiskDetailsValue["vhdId"];
                                                        if (vhdIdValue != null && vhdIdValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance = ((string)vhdIdValue);
                                                            diskDetailsInstance.VhdId = vhdIdInstance;
                                                        }
                                                        
                                                        JToken vhdNameValue = vMDiskDetailsValue["vhdName"];
                                                        if (vhdNameValue != null && vhdNameValue.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance = ((string)vhdNameValue);
                                                            diskDetailsInstance.VhdName = vhdNameInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken vmProtectionStateValue = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue != null && vmProtectionStateValue.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance = ((string)vmProtectionStateValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue != null && vmProtectionStateDescriptionValue.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance = ((string)vmProtectionStateDescriptionValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance;
                                                }
                                                
                                                JToken instanceTypeValue = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue != null && instanceTypeValue.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance = ((string)instanceTypeValue);
                                                    hyperVReplica2012ReplicationDetailsInstance.InstanceType = instanceTypeInstance;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplica2012ReplicationDetailsInstance;
                                            }
                                            if (typeName == "HyperVReplica2012R2")
                                            {
                                                HyperVReplica2012R2ReplicationDetails hyperVReplica2012R2ReplicationDetailsInstance = new HyperVReplica2012R2ReplicationDetails();
                                                
                                                JToken lastReplicatedTimeValue2 = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue2 != null && lastReplicatedTimeValue2.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance2 = ((DateTime)lastReplicatedTimeValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance2;
                                                }
                                                
                                                JToken vmIdValue2 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue2 != null && vmIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance2 = ((string)vmIdValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmId = vmIdInstance2;
                                                }
                                                
                                                JToken vmNicsArray2 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray2 != null && vmNicsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue2 in ((JArray)vmNicsArray2))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance2 = new VMNicDetails();
                                                        hyperVReplica2012R2ReplicationDetailsInstance.VmNics.Add(vMNicDetailsInstance2);
                                                        
                                                        JToken nicIdValue2 = vmNicsValue2["nicId"];
                                                        if (nicIdValue2 != null && nicIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance2 = ((string)nicIdValue2);
                                                            vMNicDetailsInstance2.NicId = nicIdInstance2;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue2 = vmNicsValue2["vMSubnetName"];
                                                        if (vMSubnetNameValue2 != null && vMSubnetNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance2 = ((string)vMSubnetNameValue2);
                                                            vMNicDetailsInstance2.VMSubnetName = vMSubnetNameInstance2;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue2 = vmNicsValue2["vMNetworkName"];
                                                        if (vMNetworkNameValue2 != null && vMNetworkNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance2 = ((string)vMNetworkNameValue2);
                                                            vMNicDetailsInstance2.VMNetworkName = vMNetworkNameInstance2;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue2 = vmNicsValue2["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue2 != null && recoveryVMNetworkIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance2 = ((string)recoveryVMNetworkIdValue2);
                                                            vMNicDetailsInstance2.RecoveryVMNetworkId = recoveryVMNetworkIdInstance2;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue2 = vmNicsValue2["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue2 != null && recoveryVMSubnetNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance2 = ((string)recoveryVMSubnetNameValue2);
                                                            vMNicDetailsInstance2.RecoveryVMSubnetName = recoveryVMSubnetNameInstance2;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue2 = vmNicsValue2["ipAddressType"];
                                                        if (ipAddressTypeValue2 != null && ipAddressTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance2 = ((string)ipAddressTypeValue2);
                                                            vMNicDetailsInstance2.IpAddressType = ipAddressTypeInstance2;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue2 = vmNicsValue2["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue2 != null && replicaNicStaticIPAddressValue2.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance2 = ((string)replicaNicStaticIPAddressValue2);
                                                            vMNicDetailsInstance2.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance2;
                                                        }
                                                        
                                                        JToken selectionTypeValue2 = vmNicsValue2["selectionType"];
                                                        if (selectionTypeValue2 != null && selectionTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance2 = ((string)selectionTypeValue2);
                                                            vMNicDetailsInstance2.SelectionType = selectionTypeInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken initialReplicationDetailsValue2 = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue2 != null && initialReplicationDetailsValue2.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance2 = new InitialReplicationDetails();
                                                    hyperVReplica2012R2ReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance2;
                                                    
                                                    JToken initialReplicationTypeValue2 = initialReplicationDetailsValue2["initialReplicationType"];
                                                    if (initialReplicationTypeValue2 != null && initialReplicationTypeValue2.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance2 = ((string)initialReplicationTypeValue2);
                                                        initialReplicationDetailsInstance2.InitialReplicationType = initialReplicationTypeInstance2;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue2 = initialReplicationDetailsValue2["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue2 != null && initialReplicationProgressPercentageValue2.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance2 = ((string)initialReplicationProgressPercentageValue2);
                                                        initialReplicationDetailsInstance2.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance2;
                                                    }
                                                }
                                                
                                                JToken vMDiskDetailsArray2 = providerSpecificDetailsValue["vMDiskDetails"];
                                                if (vMDiskDetailsArray2 != null && vMDiskDetailsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vMDiskDetailsValue2 in ((JArray)vMDiskDetailsArray2))
                                                    {
                                                        DiskDetails diskDetailsInstance2 = new DiskDetails();
                                                        hyperVReplica2012R2ReplicationDetailsInstance.VMDiskDetails.Add(diskDetailsInstance2);
                                                        
                                                        JToken maxSizeMBValue2 = vMDiskDetailsValue2["maxSizeMB"];
                                                        if (maxSizeMBValue2 != null && maxSizeMBValue2.Type != JTokenType.Null)
                                                        {
                                                            ulong maxSizeMBInstance2 = ((ulong)maxSizeMBValue2);
                                                            diskDetailsInstance2.MaxSizeMB = maxSizeMBInstance2;
                                                        }
                                                        
                                                        JToken vhdTypeValue2 = vMDiskDetailsValue2["vhdType"];
                                                        if (vhdTypeValue2 != null && vhdTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance2 = ((string)vhdTypeValue2);
                                                            diskDetailsInstance2.VhdType = vhdTypeInstance2;
                                                        }
                                                        
                                                        JToken vhdIdValue2 = vMDiskDetailsValue2["vhdId"];
                                                        if (vhdIdValue2 != null && vhdIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance2 = ((string)vhdIdValue2);
                                                            diskDetailsInstance2.VhdId = vhdIdInstance2;
                                                        }
                                                        
                                                        JToken vhdNameValue2 = vMDiskDetailsValue2["vhdName"];
                                                        if (vhdNameValue2 != null && vhdNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance2 = ((string)vhdNameValue2);
                                                            diskDetailsInstance2.VhdName = vhdNameInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken vmProtectionStateValue2 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue2 != null && vmProtectionStateValue2.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance2 = ((string)vmProtectionStateValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance2;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue2 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue2 != null && vmProtectionStateDescriptionValue2.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance2 = ((string)vmProtectionStateDescriptionValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance2;
                                                }
                                                
                                                JToken instanceTypeValue2 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue2 != null && instanceTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance2 = ((string)instanceTypeValue2);
                                                    hyperVReplica2012R2ReplicationDetailsInstance.InstanceType = instanceTypeInstance2;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplica2012R2ReplicationDetailsInstance;
                                            }
                                            if (typeName == "A2A")
                                            {
                                                A2AReplicationDetails a2AReplicationDetailsInstance = new A2AReplicationDetails();
                                                
                                                JToken fabricObjectIdValue = providerSpecificDetailsValue["fabricObjectId"];
                                                if (fabricObjectIdValue != null && fabricObjectIdValue.Type != JTokenType.Null)
                                                {
                                                    string fabricObjectIdInstance = ((string)fabricObjectIdValue);
                                                    a2AReplicationDetailsInstance.FabricObjectId = fabricObjectIdInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue != null && multiVmGroupIdValue.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance = ((string)multiVmGroupIdValue);
                                                    a2AReplicationDetailsInstance.MultiVmGroupId = multiVmGroupIdInstance;
                                                }
                                                
                                                JToken multiVmGroupNameValue = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue != null && multiVmGroupNameValue.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance = ((string)multiVmGroupNameValue);
                                                    a2AReplicationDetailsInstance.MultiVmGroupName = multiVmGroupNameInstance;
                                                }
                                                
                                                JToken protectedDisksArray = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray != null && protectedDisksArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue in ((JArray)protectedDisksArray))
                                                    {
                                                        A2AProtectedDiskDetails a2AProtectedDiskDetailsInstance = new A2AProtectedDiskDetails();
                                                        a2AReplicationDetailsInstance.ProtectedDisks.Add(a2AProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskUriValue = protectedDisksValue["diskUri"];
                                                        if (diskUriValue != null && diskUriValue.Type != JTokenType.Null)
                                                        {
                                                            string diskUriInstance = ((string)diskUriValue);
                                                            a2AProtectedDiskDetailsInstance.DiskUri = diskUriInstance;
                                                        }
                                                        
                                                        JToken recoveryAzureStorageAccountIdValue = protectedDisksValue["recoveryAzureStorageAccountId"];
                                                        if (recoveryAzureStorageAccountIdValue != null && recoveryAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryAzureStorageAccountIdInstance = ((string)recoveryAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.RecoveryAzureStorageAccountId = recoveryAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken primaryStagingAzureStorageAccountIdValue = protectedDisksValue["primaryStagingAzureStorageAccountId"];
                                                        if (primaryStagingAzureStorageAccountIdValue != null && primaryStagingAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string primaryStagingAzureStorageAccountIdInstance = ((string)primaryStagingAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.PrimaryStagingAzureStorageAccountId = primaryStagingAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken primaryDiskAzureStorageAccountIdValue = protectedDisksValue["primaryDiskAzureStorageAccountId"];
                                                        if (primaryDiskAzureStorageAccountIdValue != null && primaryDiskAzureStorageAccountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string primaryDiskAzureStorageAccountIdInstance = ((string)primaryDiskAzureStorageAccountIdValue);
                                                            a2AProtectedDiskDetailsInstance.PrimaryDiskAzureStorageAccountId = primaryDiskAzureStorageAccountIdInstance;
                                                        }
                                                        
                                                        JToken recoveryDiskUriValue = protectedDisksValue["recoveryDiskUri"];
                                                        if (recoveryDiskUriValue != null && recoveryDiskUriValue.Type != JTokenType.Null)
                                                        {
                                                            string recoveryDiskUriInstance = ((string)recoveryDiskUriValue);
                                                            a2AProtectedDiskDetailsInstance.RecoveryDiskUri = recoveryDiskUriInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken instanceTypeValue3 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue3 != null && instanceTypeValue3.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance3 = ((string)instanceTypeValue3);
                                                    a2AReplicationDetailsInstance.InstanceType = instanceTypeInstance3;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = a2AReplicationDetailsInstance;
                                            }
                                            if (typeName == "InMageAzureV2")
                                            {
                                                InMageAzureV2ProviderSpecificSettings inMageAzureV2ProviderSpecificSettingsInstance = new InMageAzureV2ProviderSpecificSettings();
                                                
                                                JToken infrastructureVmIdValue = providerSpecificDetailsValue["infrastructureVmId"];
                                                if (infrastructureVmIdValue != null && infrastructureVmIdValue.Type != JTokenType.Null)
                                                {
                                                    string infrastructureVmIdInstance = ((string)infrastructureVmIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance;
                                                }
                                                
                                                JToken vCenterInfrastructureIdValue = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                                if (vCenterInfrastructureIdValue != null && vCenterInfrastructureIdValue.Type != JTokenType.Null)
                                                {
                                                    string vCenterInfrastructureIdInstance = ((string)vCenterInfrastructureIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance;
                                                }
                                                
                                                JToken protectionStageValue = providerSpecificDetailsValue["protectionStage"];
                                                if (protectionStageValue != null && protectionStageValue.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance = ((string)protectionStageValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance;
                                                }
                                                
                                                JToken vmIdValue3 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue3 != null && vmIdValue3.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance3 = ((string)vmIdValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmId = vmIdInstance3;
                                                }
                                                
                                                JToken resyncProgressPercentageValue = providerSpecificDetailsValue["resyncProgressPercentage"];
                                                if (resyncProgressPercentageValue != null && resyncProgressPercentageValue.Type != JTokenType.Null)
                                                {
                                                    int resyncProgressPercentageInstance = ((int)resyncProgressPercentageValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance;
                                                }
                                                
                                                JToken rpoInSecondsValue = providerSpecificDetailsValue["rpoInSeconds"];
                                                if (rpoInSecondsValue != null && rpoInSecondsValue.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance = ((long)rpoInSecondsValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance;
                                                }
                                                
                                                JToken compressedDataRateInMBValue = providerSpecificDetailsValue["compressedDataRateInMB"];
                                                if (compressedDataRateInMBValue != null && compressedDataRateInMBValue.Type != JTokenType.Null)
                                                {
                                                    double compressedDataRateInMBInstance = ((double)compressedDataRateInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance;
                                                }
                                                
                                                JToken uncompressedDataRateInMBValue = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                                if (uncompressedDataRateInMBValue != null && uncompressedDataRateInMBValue.Type != JTokenType.Null)
                                                {
                                                    double uncompressedDataRateInMBInstance = ((double)uncompressedDataRateInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance;
                                                }
                                                
                                                JToken ipAddressValue = providerSpecificDetailsValue["ipAddress"];
                                                if (ipAddressValue != null && ipAddressValue.Type != JTokenType.Null)
                                                {
                                                    string ipAddressInstance = ((string)ipAddressValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IpAddress = ipAddressInstance;
                                                }
                                                
                                                JToken agentVersionValue = providerSpecificDetailsValue["agentVersion"];
                                                if (agentVersionValue != null && agentVersionValue.Type != JTokenType.Null)
                                                {
                                                    string agentVersionInstance = ((string)agentVersionValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.AgentVersion = agentVersionInstance;
                                                }
                                                
                                                JToken isAgentUpdateRequiredValue = providerSpecificDetailsValue["isAgentUpdateRequired"];
                                                if (isAgentUpdateRequiredValue != null && isAgentUpdateRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string isAgentUpdateRequiredInstance = ((string)isAgentUpdateRequiredValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IsAgentUpdateRequired = isAgentUpdateRequiredInstance;
                                                }
                                                
                                                JToken isRebootAfterUpdateRequiredValue = providerSpecificDetailsValue["isRebootAfterUpdateRequired"];
                                                if (isRebootAfterUpdateRequiredValue != null && isRebootAfterUpdateRequiredValue.Type != JTokenType.Null)
                                                {
                                                    string isRebootAfterUpdateRequiredInstance = ((string)isRebootAfterUpdateRequiredValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.IsRebootAfterUpdateRequired = isRebootAfterUpdateRequiredInstance;
                                                }
                                                
                                                JToken lastHeartbeatValue = providerSpecificDetailsValue["lastHeartbeat"];
                                                if (lastHeartbeatValue != null && lastHeartbeatValue.Type != JTokenType.Null)
                                                {
                                                    DateTime lastHeartbeatInstance = ((DateTime)lastHeartbeatValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance;
                                                }
                                                
                                                JToken processServerIdValue = providerSpecificDetailsValue["processServerId"];
                                                if (processServerIdValue != null && processServerIdValue.Type != JTokenType.Null)
                                                {
                                                    string processServerIdInstance = ((string)processServerIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue2 = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue2 != null && multiVmGroupIdValue2.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance2 = ((string)multiVmGroupIdValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance2;
                                                }
                                                
                                                JToken multiVmGroupNameValue2 = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue2 != null && multiVmGroupNameValue2.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance2 = ((string)multiVmGroupNameValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance2;
                                                }
                                                
                                                JToken protectedDisksArray2 = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray2 != null && protectedDisksArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue2 in ((JArray)protectedDisksArray2))
                                                    {
                                                        InMageAzureV2ProtectedDiskDetails inMageAzureV2ProtectedDiskDetailsInstance = new InMageAzureV2ProtectedDiskDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageAzureV2ProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskIdValue = protectedDisksValue2["diskId"];
                                                        if (diskIdValue != null && diskIdValue.Type != JTokenType.Null)
                                                        {
                                                            string diskIdInstance = ((string)diskIdValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskId = diskIdInstance;
                                                        }
                                                        
                                                        JToken diskNameValue = protectedDisksValue2["diskName"];
                                                        if (diskNameValue != null && diskNameValue.Type != JTokenType.Null)
                                                        {
                                                            string diskNameInstance = ((string)diskNameValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskName = diskNameInstance;
                                                        }
                                                        
                                                        JToken protectionStageValue2 = protectedDisksValue2["protectionStage"];
                                                        if (protectionStageValue2 != null && protectionStageValue2.Type != JTokenType.Null)
                                                        {
                                                            string protectionStageInstance2 = ((string)protectionStageValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance2;
                                                        }
                                                        
                                                        JToken healthErrorCodeValue = protectedDisksValue2["healthErrorCode"];
                                                        if (healthErrorCodeValue != null && healthErrorCodeValue.Type != JTokenType.Null)
                                                        {
                                                            string healthErrorCodeInstance = ((string)healthErrorCodeValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance;
                                                        }
                                                        
                                                        JToken rpoInSecondsValue2 = protectedDisksValue2["rpoInSeconds"];
                                                        if (rpoInSecondsValue2 != null && rpoInSecondsValue2.Type != JTokenType.Null)
                                                        {
                                                            long rpoInSecondsInstance2 = ((long)rpoInSecondsValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance2;
                                                        }
                                                        
                                                        JToken resyncRequiredValue = protectedDisksValue2["resyncRequired"];
                                                        if (resyncRequiredValue != null && resyncRequiredValue.Type != JTokenType.Null)
                                                        {
                                                            string resyncRequiredInstance = ((string)resyncRequiredValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance;
                                                        }
                                                        
                                                        JToken resyncProgressPercentageValue2 = protectedDisksValue2["resyncProgressPercentage"];
                                                        if (resyncProgressPercentageValue2 != null && resyncProgressPercentageValue2.Type != JTokenType.Null)
                                                        {
                                                            int resyncProgressPercentageInstance2 = ((int)resyncProgressPercentageValue2);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance2;
                                                        }
                                                        
                                                        JToken resyncDurationInSecondsValue = protectedDisksValue2["resyncDurationInSeconds"];
                                                        if (resyncDurationInSecondsValue != null && resyncDurationInSecondsValue.Type != JTokenType.Null)
                                                        {
                                                            long resyncDurationInSecondsInstance = ((long)resyncDurationInSecondsValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance;
                                                        }
                                                        
                                                        JToken diskCapacityInBytesValue = protectedDisksValue2["diskCapacityInBytes"];
                                                        if (diskCapacityInBytesValue != null && diskCapacityInBytesValue.Type != JTokenType.Null)
                                                        {
                                                            long diskCapacityInBytesInstance = ((long)diskCapacityInBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance;
                                                        }
                                                        
                                                        JToken fileSystemCapacityInBytesValue = protectedDisksValue2["fileSystemCapacityInBytes"];
                                                        if (fileSystemCapacityInBytesValue != null && fileSystemCapacityInBytesValue.Type != JTokenType.Null)
                                                        {
                                                            long fileSystemCapacityInBytesInstance = ((long)fileSystemCapacityInBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance;
                                                        }
                                                        
                                                        JToken sourceDataInMegaBytesValue = protectedDisksValue2["sourceDataInMegaBytes"];
                                                        if (sourceDataInMegaBytesValue != null && sourceDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double sourceDataInMegaBytesInstance = ((double)sourceDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.SourceDataInMB = sourceDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken psDataInMegaBytesValue = protectedDisksValue2["psDataInMegaBytes"];
                                                        if (psDataInMegaBytesValue != null && psDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double psDataInMegaBytesInstance = ((double)psDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.PSDataInMB = psDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken targetDataInMegaBytesValue = protectedDisksValue2["targetDataInMegaBytes"];
                                                        if (targetDataInMegaBytesValue != null && targetDataInMegaBytesValue.Type != JTokenType.Null)
                                                        {
                                                            double targetDataInMegaBytesInstance = ((double)targetDataInMegaBytesValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.TargetDataInMB = targetDataInMegaBytesInstance;
                                                        }
                                                        
                                                        JToken diskResizedValue = protectedDisksValue2["diskResized"];
                                                        if (diskResizedValue != null && diskResizedValue.Type != JTokenType.Null)
                                                        {
                                                            string diskResizedInstance = ((string)diskResizedValue);
                                                            inMageAzureV2ProtectedDiskDetailsInstance.DiskResized = diskResizedInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken diskResizedValue2 = providerSpecificDetailsValue["diskResized"];
                                                if (diskResizedValue2 != null && diskResizedValue2.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance2 = ((string)diskResizedValue2);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.DiskResized = diskResizedInstance2;
                                                }
                                                
                                                JToken masterTargetIdValue = providerSpecificDetailsValue["masterTargetId"];
                                                if (masterTargetIdValue != null && masterTargetIdValue.Type != JTokenType.Null)
                                                {
                                                    string masterTargetIdInstance = ((string)masterTargetIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance;
                                                }
                                                
                                                JToken sourceVmCPUCountValue = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue != null && sourceVmCPUCountValue.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance = ((int)sourceVmCPUCountValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance;
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue != null && sourceVmRAMSizeInMBValue.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance = ((int)sourceVmRAMSizeInMBValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance;
                                                }
                                                
                                                JToken osTypeValue = providerSpecificDetailsValue["osType"];
                                                if (osTypeValue != null && osTypeValue.Type != JTokenType.Null)
                                                {
                                                    string osTypeInstance = ((string)osTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.OSType = osTypeInstance;
                                                }
                                                
                                                JToken vhdNameValue3 = providerSpecificDetailsValue["vhdName"];
                                                if (vhdNameValue3 != null && vhdNameValue3.Type != JTokenType.Null)
                                                {
                                                    string vhdNameInstance3 = ((string)vhdNameValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VHDName = vhdNameInstance3;
                                                }
                                                
                                                JToken osDiskIdValue = providerSpecificDetailsValue["osDiskId"];
                                                if (osDiskIdValue != null && osDiskIdValue.Type != JTokenType.Null)
                                                {
                                                    string osDiskIdInstance = ((string)osDiskIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.OSDiskId = osDiskIdInstance;
                                                }
                                                
                                                JToken azureVMDiskDetailsArray = providerSpecificDetailsValue["azureVMDiskDetails"];
                                                if (azureVMDiskDetailsArray != null && azureVMDiskDetailsArray.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken azureVMDiskDetailsValue in ((JArray)azureVMDiskDetailsArray))
                                                    {
                                                        AzureVmDiskDetails azureVmDiskDetailsInstance = new AzureVmDiskDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance);
                                                        
                                                        JToken vhdTypeValue3 = azureVMDiskDetailsValue["vhdType"];
                                                        if (vhdTypeValue3 != null && vhdTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance3 = ((string)vhdTypeValue3);
                                                            azureVmDiskDetailsInstance.VhdType = vhdTypeInstance3;
                                                        }
                                                        
                                                        JToken vhdIdValue3 = azureVMDiskDetailsValue["vhdId"];
                                                        if (vhdIdValue3 != null && vhdIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance3 = ((string)vhdIdValue3);
                                                            azureVmDiskDetailsInstance.VhdId = vhdIdInstance3;
                                                        }
                                                        
                                                        JToken vhdNameValue4 = azureVMDiskDetailsValue["vhdName"];
                                                        if (vhdNameValue4 != null && vhdNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance4 = ((string)vhdNameValue4);
                                                            azureVmDiskDetailsInstance.VhdName = vhdNameInstance4;
                                                        }
                                                        
                                                        JToken maxSizeMBValue3 = azureVMDiskDetailsValue["maxSizeMB"];
                                                        if (maxSizeMBValue3 != null && maxSizeMBValue3.Type != JTokenType.Null)
                                                        {
                                                            string maxSizeMBInstance3 = ((string)maxSizeMBValue3);
                                                            azureVmDiskDetailsInstance.MaxSizeMB = maxSizeMBInstance3;
                                                        }
                                                        
                                                        JToken targetDiskLocationValue = azureVMDiskDetailsValue["targetDiskLocation"];
                                                        if (targetDiskLocationValue != null && targetDiskLocationValue.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskLocationInstance = ((string)targetDiskLocationValue);
                                                            azureVmDiskDetailsInstance.TargetDiskLocation = targetDiskLocationInstance;
                                                        }
                                                        
                                                        JToken lunIdValue = azureVMDiskDetailsValue["lunId"];
                                                        if (lunIdValue != null && lunIdValue.Type != JTokenType.Null)
                                                        {
                                                            string lunIdInstance = ((string)lunIdValue);
                                                            azureVmDiskDetailsInstance.LunId = lunIdInstance;
                                                        }
                                                        
                                                        JToken targetDiskNameValue = azureVMDiskDetailsValue["targetDiskName"];
                                                        if (targetDiskNameValue != null && targetDiskNameValue.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskNameInstance = ((string)targetDiskNameValue);
                                                            azureVmDiskDetailsInstance.TargetDiskName = targetDiskNameInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken recoveryAzureVMNameValue = providerSpecificDetailsValue["recoveryAzureVMName"];
                                                if (recoveryAzureVMNameValue != null && recoveryAzureVMNameValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMNameInstance = ((string)recoveryAzureVMNameValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance;
                                                }
                                                
                                                JToken recoveryAzureVMSizeValue = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                                if (recoveryAzureVMSizeValue != null && recoveryAzureVMSizeValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMSizeInstance = ((string)recoveryAzureVMSizeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountValue = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                                if (recoveryAzureStorageAccountValue != null && recoveryAzureStorageAccountValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountInstance = ((string)recoveryAzureStorageAccountValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance;
                                                }
                                                
                                                JToken vmNicsArray3 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray3 != null && vmNicsArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue3 in ((JArray)vmNicsArray3))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance3 = new VMNicDetails();
                                                        inMageAzureV2ProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance3);
                                                        
                                                        JToken nicIdValue3 = vmNicsValue3["nicId"];
                                                        if (nicIdValue3 != null && nicIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance3 = ((string)nicIdValue3);
                                                            vMNicDetailsInstance3.NicId = nicIdInstance3;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue3 = vmNicsValue3["vMSubnetName"];
                                                        if (vMSubnetNameValue3 != null && vMSubnetNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance3 = ((string)vMSubnetNameValue3);
                                                            vMNicDetailsInstance3.VMSubnetName = vMSubnetNameInstance3;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue3 = vmNicsValue3["vMNetworkName"];
                                                        if (vMNetworkNameValue3 != null && vMNetworkNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance3 = ((string)vMNetworkNameValue3);
                                                            vMNicDetailsInstance3.VMNetworkName = vMNetworkNameInstance3;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue3 = vmNicsValue3["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue3 != null && recoveryVMNetworkIdValue3.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance3 = ((string)recoveryVMNetworkIdValue3);
                                                            vMNicDetailsInstance3.RecoveryVMNetworkId = recoveryVMNetworkIdInstance3;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue3 = vmNicsValue3["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue3 != null && recoveryVMSubnetNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance3 = ((string)recoveryVMSubnetNameValue3);
                                                            vMNicDetailsInstance3.RecoveryVMSubnetName = recoveryVMSubnetNameInstance3;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue3 = vmNicsValue3["ipAddressType"];
                                                        if (ipAddressTypeValue3 != null && ipAddressTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance3 = ((string)ipAddressTypeValue3);
                                                            vMNicDetailsInstance3.IpAddressType = ipAddressTypeInstance3;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue3 = vmNicsValue3["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue3 != null && replicaNicStaticIPAddressValue3.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance3 = ((string)replicaNicStaticIPAddressValue3);
                                                            vMNicDetailsInstance3.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance3;
                                                        }
                                                        
                                                        JToken selectionTypeValue3 = vmNicsValue3["selectionType"];
                                                        if (selectionTypeValue3 != null && selectionTypeValue3.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance3 = ((string)selectionTypeValue3);
                                                            vMNicDetailsInstance3.SelectionType = selectionTypeInstance3;
                                                        }
                                                    }
                                                }
                                                
                                                JToken selectedRecoveryAzureNetworkIdValue = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                                if (selectedRecoveryAzureNetworkIdValue != null && selectedRecoveryAzureNetworkIdValue.Type != JTokenType.Null)
                                                {
                                                    string selectedRecoveryAzureNetworkIdInstance = ((string)selectedRecoveryAzureNetworkIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance;
                                                }
                                                
                                                JToken discoveryTypeValue = providerSpecificDetailsValue["discoveryType"];
                                                if (discoveryTypeValue != null && discoveryTypeValue.Type != JTokenType.Null)
                                                {
                                                    string discoveryTypeInstance = ((string)discoveryTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance;
                                                }
                                                
                                                JToken licenseTypeValue = providerSpecificDetailsValue["licenseType"];
                                                if (licenseTypeValue != null && licenseTypeValue.Type != JTokenType.Null)
                                                {
                                                    string licenseTypeInstance = ((string)licenseTypeValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.LicenseType = licenseTypeInstance;
                                                }
                                                
                                                JToken enableRDPOnTargetOptionValue = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                                if (enableRDPOnTargetOptionValue != null && enableRDPOnTargetOptionValue.Type != JTokenType.Null)
                                                {
                                                    string enableRDPOnTargetOptionInstance = ((string)enableRDPOnTargetOptionValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance;
                                                }
                                                
                                                JToken recoveryAzureLogStorageAccountIdValue = providerSpecificDetailsValue["recoveryAzureLogStorageAccountId"];
                                                if (recoveryAzureLogStorageAccountIdValue != null && recoveryAzureLogStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureLogStorageAccountIdInstance = ((string)recoveryAzureLogStorageAccountIdValue);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.RecoveryAzureLogStorageAccountId = recoveryAzureLogStorageAccountIdInstance;
                                                }
                                                
                                                JToken vmProtectionStateValue3 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue3 != null && vmProtectionStateValue3.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance3 = ((string)vmProtectionStateValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance3;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue3 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue3 != null && vmProtectionStateDescriptionValue3.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance3 = ((string)vmProtectionStateDescriptionValue3);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance3;
                                                }
                                                
                                                JToken instanceTypeValue4 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue4 != null && instanceTypeValue4.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance4 = ((string)instanceTypeValue4);
                                                    inMageAzureV2ProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance4;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = inMageAzureV2ProviderSpecificSettingsInstance;
                                            }
                                            if (typeName == "InMage")
                                            {
                                                InMageProviderSpecificSettings inMageProviderSpecificSettingsInstance = new InMageProviderSpecificSettings();
                                                
                                                JToken activeSiteTypeValue = providerSpecificDetailsValue["activeSiteType"];
                                                if (activeSiteTypeValue != null && activeSiteTypeValue.Type != JTokenType.Null)
                                                {
                                                    string activeSiteTypeInstance = ((string)activeSiteTypeValue);
                                                    inMageProviderSpecificSettingsInstance.ActiveSiteType = activeSiteTypeInstance;
                                                }
                                                
                                                JToken sourceVmCPUCountValue2 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue2 != null && sourceVmCPUCountValue2.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance2 = ((int)sourceVmCPUCountValue2);
                                                    inMageProviderSpecificSettingsInstance.SourceVmCPUCount = sourceVmCPUCountInstance2;
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue2 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue2 != null && sourceVmRAMSizeInMBValue2.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance2 = ((int)sourceVmRAMSizeInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance2;
                                                }
                                                
                                                JToken osDetailsValue = providerSpecificDetailsValue["osDetails"];
                                                if (osDetailsValue != null && osDetailsValue.Type != JTokenType.Null)
                                                {
                                                    OSDiskDetails osDetailsInstance = new OSDiskDetails();
                                                    inMageProviderSpecificSettingsInstance.OsDetails = osDetailsInstance;
                                                    
                                                    JToken osVhdIdValue = osDetailsValue["osVhdId"];
                                                    if (osVhdIdValue != null && osVhdIdValue.Type != JTokenType.Null)
                                                    {
                                                        string osVhdIdInstance = ((string)osVhdIdValue);
                                                        osDetailsInstance.OSVhdId = osVhdIdInstance;
                                                    }
                                                    
                                                    JToken osTypeValue2 = osDetailsValue["osType"];
                                                    if (osTypeValue2 != null && osTypeValue2.Type != JTokenType.Null)
                                                    {
                                                        string osTypeInstance2 = ((string)osTypeValue2);
                                                        osDetailsInstance.OSType = osTypeInstance2;
                                                    }
                                                    
                                                    JToken vhdNameValue5 = osDetailsValue["vhdName"];
                                                    if (vhdNameValue5 != null && vhdNameValue5.Type != JTokenType.Null)
                                                    {
                                                        string vhdNameInstance5 = ((string)vhdNameValue5);
                                                        osDetailsInstance.VhdName = vhdNameInstance5;
                                                    }
                                                }
                                                
                                                JToken protectionStageValue3 = providerSpecificDetailsValue["protectionStage"];
                                                if (protectionStageValue3 != null && protectionStageValue3.Type != JTokenType.Null)
                                                {
                                                    string protectionStageInstance3 = ((string)protectionStageValue3);
                                                    inMageProviderSpecificSettingsInstance.ProtectionStage = protectionStageInstance3;
                                                }
                                                
                                                JToken vmIdValue4 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue4 != null && vmIdValue4.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance4 = ((string)vmIdValue4);
                                                    inMageProviderSpecificSettingsInstance.VmId = vmIdInstance4;
                                                }
                                                
                                                JToken resyncDetailsValue = providerSpecificDetailsValue["resyncDetails"];
                                                if (resyncDetailsValue != null && resyncDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails resyncDetailsInstance = new InitialReplicationDetails();
                                                    inMageProviderSpecificSettingsInstance.ResyncDetails = resyncDetailsInstance;
                                                    
                                                    JToken initialReplicationTypeValue3 = resyncDetailsValue["initialReplicationType"];
                                                    if (initialReplicationTypeValue3 != null && initialReplicationTypeValue3.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance3 = ((string)initialReplicationTypeValue3);
                                                        resyncDetailsInstance.InitialReplicationType = initialReplicationTypeInstance3;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue3 = resyncDetailsValue["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue3 != null && initialReplicationProgressPercentageValue3.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance3 = ((string)initialReplicationProgressPercentageValue3);
                                                        resyncDetailsInstance.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance3;
                                                    }
                                                }
                                                
                                                JToken retentionWindowStartValue = providerSpecificDetailsValue["retentionWindowStart"];
                                                if (retentionWindowStartValue != null && retentionWindowStartValue.Type != JTokenType.Null)
                                                {
                                                    DateTime retentionWindowStartInstance = ((DateTime)retentionWindowStartValue);
                                                    inMageProviderSpecificSettingsInstance.RetentionWindowStart = retentionWindowStartInstance;
                                                }
                                                
                                                JToken retentionWindowEndValue = providerSpecificDetailsValue["retentionWindowEnd"];
                                                if (retentionWindowEndValue != null && retentionWindowEndValue.Type != JTokenType.Null)
                                                {
                                                    DateTime retentionWindowEndInstance = ((DateTime)retentionWindowEndValue);
                                                    inMageProviderSpecificSettingsInstance.RetentionWindowEnd = retentionWindowEndInstance;
                                                }
                                                
                                                JToken compressedDataRateInMBValue2 = providerSpecificDetailsValue["compressedDataRateInMB"];
                                                if (compressedDataRateInMBValue2 != null && compressedDataRateInMBValue2.Type != JTokenType.Null)
                                                {
                                                    double compressedDataRateInMBInstance2 = ((double)compressedDataRateInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.CompressedDataRateInMB = compressedDataRateInMBInstance2;
                                                }
                                                
                                                JToken uncompressedDataRateInMBValue2 = providerSpecificDetailsValue["uncompressedDataRateInMB"];
                                                if (uncompressedDataRateInMBValue2 != null && uncompressedDataRateInMBValue2.Type != JTokenType.Null)
                                                {
                                                    double uncompressedDataRateInMBInstance2 = ((double)uncompressedDataRateInMBValue2);
                                                    inMageProviderSpecificSettingsInstance.UncompressedDataRateInMB = uncompressedDataRateInMBInstance2;
                                                }
                                                
                                                JToken rpoInSecondsValue3 = providerSpecificDetailsValue["rpoInSeconds"];
                                                if (rpoInSecondsValue3 != null && rpoInSecondsValue3.Type != JTokenType.Null)
                                                {
                                                    long rpoInSecondsInstance3 = ((long)rpoInSecondsValue3);
                                                    inMageProviderSpecificSettingsInstance.RpoInSeconds = rpoInSecondsInstance3;
                                                }
                                                
                                                JToken protectedDisksArray3 = providerSpecificDetailsValue["protectedDisks"];
                                                if (protectedDisksArray3 != null && protectedDisksArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken protectedDisksValue3 in ((JArray)protectedDisksArray3))
                                                    {
                                                        InMageProtectedDiskDetails inMageProtectedDiskDetailsInstance = new InMageProtectedDiskDetails();
                                                        inMageProviderSpecificSettingsInstance.ProtectedDisks.Add(inMageProtectedDiskDetailsInstance);
                                                        
                                                        JToken diskIdValue2 = protectedDisksValue3["diskId"];
                                                        if (diskIdValue2 != null && diskIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string diskIdInstance2 = ((string)diskIdValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskId = diskIdInstance2;
                                                        }
                                                        
                                                        JToken diskNameValue2 = protectedDisksValue3["diskName"];
                                                        if (diskNameValue2 != null && diskNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string diskNameInstance2 = ((string)diskNameValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskName = diskNameInstance2;
                                                        }
                                                        
                                                        JToken protectionStageValue4 = protectedDisksValue3["protectionStage"];
                                                        if (protectionStageValue4 != null && protectionStageValue4.Type != JTokenType.Null)
                                                        {
                                                            string protectionStageInstance4 = ((string)protectionStageValue4);
                                                            inMageProtectedDiskDetailsInstance.ProtectionStage = protectionStageInstance4;
                                                        }
                                                        
                                                        JToken healthErrorCodeValue2 = protectedDisksValue3["healthErrorCode"];
                                                        if (healthErrorCodeValue2 != null && healthErrorCodeValue2.Type != JTokenType.Null)
                                                        {
                                                            string healthErrorCodeInstance2 = ((string)healthErrorCodeValue2);
                                                            inMageProtectedDiskDetailsInstance.HealthErrorCode = healthErrorCodeInstance2;
                                                        }
                                                        
                                                        JToken rpoInSecondsValue4 = protectedDisksValue3["rpoInSeconds"];
                                                        if (rpoInSecondsValue4 != null && rpoInSecondsValue4.Type != JTokenType.Null)
                                                        {
                                                            long rpoInSecondsInstance4 = ((long)rpoInSecondsValue4);
                                                            inMageProtectedDiskDetailsInstance.RpoInSeconds = rpoInSecondsInstance4;
                                                        }
                                                        
                                                        JToken resyncRequiredValue2 = protectedDisksValue3["resyncRequired"];
                                                        if (resyncRequiredValue2 != null && resyncRequiredValue2.Type != JTokenType.Null)
                                                        {
                                                            string resyncRequiredInstance2 = ((string)resyncRequiredValue2);
                                                            inMageProtectedDiskDetailsInstance.ResyncRequired = resyncRequiredInstance2;
                                                        }
                                                        
                                                        JToken resyncProgressPercentageValue3 = protectedDisksValue3["resyncProgressPercentage"];
                                                        if (resyncProgressPercentageValue3 != null && resyncProgressPercentageValue3.Type != JTokenType.Null)
                                                        {
                                                            int resyncProgressPercentageInstance3 = ((int)resyncProgressPercentageValue3);
                                                            inMageProtectedDiskDetailsInstance.ResyncProgressPercentage = resyncProgressPercentageInstance3;
                                                        }
                                                        
                                                        JToken resyncDurationInSecondsValue2 = protectedDisksValue3["resyncDurationInSeconds"];
                                                        if (resyncDurationInSecondsValue2 != null && resyncDurationInSecondsValue2.Type != JTokenType.Null)
                                                        {
                                                            long resyncDurationInSecondsInstance2 = ((long)resyncDurationInSecondsValue2);
                                                            inMageProtectedDiskDetailsInstance.ResyncDurationInSeconds = resyncDurationInSecondsInstance2;
                                                        }
                                                        
                                                        JToken diskCapacityInBytesValue2 = protectedDisksValue3["diskCapacityInBytes"];
                                                        if (diskCapacityInBytesValue2 != null && diskCapacityInBytesValue2.Type != JTokenType.Null)
                                                        {
                                                            long diskCapacityInBytesInstance2 = ((long)diskCapacityInBytesValue2);
                                                            inMageProtectedDiskDetailsInstance.DiskCapacityInBytes = diskCapacityInBytesInstance2;
                                                        }
                                                        
                                                        JToken fileSystemCapacityInBytesValue2 = protectedDisksValue3["fileSystemCapacityInBytes"];
                                                        if (fileSystemCapacityInBytesValue2 != null && fileSystemCapacityInBytesValue2.Type != JTokenType.Null)
                                                        {
                                                            long fileSystemCapacityInBytesInstance2 = ((long)fileSystemCapacityInBytesValue2);
                                                            inMageProtectedDiskDetailsInstance.FileSystemCapacityInBytes = fileSystemCapacityInBytesInstance2;
                                                        }
                                                        
                                                        JToken sourceDataInMBValue = protectedDisksValue3["sourceDataInMB"];
                                                        if (sourceDataInMBValue != null && sourceDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double sourceDataInMBInstance = ((double)sourceDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.SourceDataInMegaBytes = sourceDataInMBInstance;
                                                        }
                                                        
                                                        JToken psDataInMBValue = protectedDisksValue3["psDataInMB"];
                                                        if (psDataInMBValue != null && psDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double psDataInMBInstance = ((double)psDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.PSDataInMegaBytes = psDataInMBInstance;
                                                        }
                                                        
                                                        JToken targetDataInMBValue = protectedDisksValue3["targetDataInMB"];
                                                        if (targetDataInMBValue != null && targetDataInMBValue.Type != JTokenType.Null)
                                                        {
                                                            double targetDataInMBInstance = ((double)targetDataInMBValue);
                                                            inMageProtectedDiskDetailsInstance.TargetDataInMegaBytes = targetDataInMBInstance;
                                                        }
                                                        
                                                        JToken diskResizedValue3 = protectedDisksValue3["diskResized"];
                                                        if (diskResizedValue3 != null && diskResizedValue3.Type != JTokenType.Null)
                                                        {
                                                            string diskResizedInstance3 = ((string)diskResizedValue3);
                                                            inMageProtectedDiskDetailsInstance.DiskResized = diskResizedInstance3;
                                                        }
                                                    }
                                                }
                                                
                                                JToken ipAddressValue2 = providerSpecificDetailsValue["ipAddress"];
                                                if (ipAddressValue2 != null && ipAddressValue2.Type != JTokenType.Null)
                                                {
                                                    string ipAddressInstance2 = ((string)ipAddressValue2);
                                                    inMageProviderSpecificSettingsInstance.IpAddress = ipAddressInstance2;
                                                }
                                                
                                                JToken lastHeartbeatValue2 = providerSpecificDetailsValue["lastHeartbeat"];
                                                if (lastHeartbeatValue2 != null && lastHeartbeatValue2.Type != JTokenType.Null)
                                                {
                                                    DateTime lastHeartbeatInstance2 = ((DateTime)lastHeartbeatValue2);
                                                    inMageProviderSpecificSettingsInstance.LastHeartbeat = lastHeartbeatInstance2;
                                                }
                                                
                                                JToken processServerIdValue2 = providerSpecificDetailsValue["processServerId"];
                                                if (processServerIdValue2 != null && processServerIdValue2.Type != JTokenType.Null)
                                                {
                                                    string processServerIdInstance2 = ((string)processServerIdValue2);
                                                    inMageProviderSpecificSettingsInstance.ProcessServerId = processServerIdInstance2;
                                                }
                                                
                                                JToken masterTargetIdValue2 = providerSpecificDetailsValue["masterTargetId"];
                                                if (masterTargetIdValue2 != null && masterTargetIdValue2.Type != JTokenType.Null)
                                                {
                                                    string masterTargetIdInstance2 = ((string)masterTargetIdValue2);
                                                    inMageProviderSpecificSettingsInstance.MasterTargetId = masterTargetIdInstance2;
                                                }
                                                
                                                JToken consistencyPointsSequenceElement = ((JToken)providerSpecificDetailsValue["consistencyPoints"]);
                                                if (consistencyPointsSequenceElement != null && consistencyPointsSequenceElement.Type != JTokenType.Null)
                                                {
                                                    foreach (JProperty property in consistencyPointsSequenceElement)
                                                    {
                                                        string consistencyPointsKey = ((string)property.Name);
                                                        DateTime consistencyPointsValue = ((DateTime)property.Value);
                                                        inMageProviderSpecificSettingsInstance.ConsistencyPoints.Add(consistencyPointsKey, consistencyPointsValue);
                                                    }
                                                }
                                                
                                                JToken diskResizedValue4 = providerSpecificDetailsValue["diskResized"];
                                                if (diskResizedValue4 != null && diskResizedValue4.Type != JTokenType.Null)
                                                {
                                                    string diskResizedInstance4 = ((string)diskResizedValue4);
                                                    inMageProviderSpecificSettingsInstance.DiskResized = diskResizedInstance4;
                                                }
                                                
                                                JToken rebootAfterUpdateStatusValue = providerSpecificDetailsValue["rebootAfterUpdateStatus"];
                                                if (rebootAfterUpdateStatusValue != null && rebootAfterUpdateStatusValue.Type != JTokenType.Null)
                                                {
                                                    string rebootAfterUpdateStatusInstance = ((string)rebootAfterUpdateStatusValue);
                                                    inMageProviderSpecificSettingsInstance.RebootAfterUpdateStatus = rebootAfterUpdateStatusInstance;
                                                }
                                                
                                                JToken multiVmGroupIdValue3 = providerSpecificDetailsValue["multiVmGroupId"];
                                                if (multiVmGroupIdValue3 != null && multiVmGroupIdValue3.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupIdInstance3 = ((string)multiVmGroupIdValue3);
                                                    inMageProviderSpecificSettingsInstance.MultiVmGroupId = multiVmGroupIdInstance3;
                                                }
                                                
                                                JToken multiVmGroupNameValue3 = providerSpecificDetailsValue["multiVmGroupName"];
                                                if (multiVmGroupNameValue3 != null && multiVmGroupNameValue3.Type != JTokenType.Null)
                                                {
                                                    string multiVmGroupNameInstance3 = ((string)multiVmGroupNameValue3);
                                                    inMageProviderSpecificSettingsInstance.MultiVmGroupName = multiVmGroupNameInstance3;
                                                }
                                                
                                                JToken agentDetailsValue = providerSpecificDetailsValue["agentDetails"];
                                                if (agentDetailsValue != null && agentDetailsValue.Type != JTokenType.Null)
                                                {
                                                    InMageAgentDetails agentDetailsInstance = new InMageAgentDetails();
                                                    inMageProviderSpecificSettingsInstance.AgentDetails = agentDetailsInstance;
                                                    
                                                    JToken agentVersionValue2 = agentDetailsValue["agentVersion"];
                                                    if (agentVersionValue2 != null && agentVersionValue2.Type != JTokenType.Null)
                                                    {
                                                        string agentVersionInstance2 = ((string)agentVersionValue2);
                                                        agentDetailsInstance.AgentVersion = agentVersionInstance2;
                                                    }
                                                    
                                                    JToken agentUpdateStatusValue = agentDetailsValue["agentUpdateStatus"];
                                                    if (agentUpdateStatusValue != null && agentUpdateStatusValue.Type != JTokenType.Null)
                                                    {
                                                        string agentUpdateStatusInstance = ((string)agentUpdateStatusValue);
                                                        agentDetailsInstance.AgentUpdateStatus = agentUpdateStatusInstance;
                                                    }
                                                    
                                                    JToken postUpdateRebootStatusValue = agentDetailsValue["postUpdateRebootStatus"];
                                                    if (postUpdateRebootStatusValue != null && postUpdateRebootStatusValue.Type != JTokenType.Null)
                                                    {
                                                        string postUpdateRebootStatusInstance = ((string)postUpdateRebootStatusValue);
                                                        agentDetailsInstance.PostUpdateRebootStatus = postUpdateRebootStatusInstance;
                                                    }
                                                }
                                                
                                                JToken infrastructureVmIdValue2 = providerSpecificDetailsValue["infrastructureVmId"];
                                                if (infrastructureVmIdValue2 != null && infrastructureVmIdValue2.Type != JTokenType.Null)
                                                {
                                                    string infrastructureVmIdInstance2 = ((string)infrastructureVmIdValue2);
                                                    inMageProviderSpecificSettingsInstance.InfrastructureVmId = infrastructureVmIdInstance2;
                                                }
                                                
                                                JToken vCenterInfrastructureIdValue2 = providerSpecificDetailsValue["vCenterInfrastructureId"];
                                                if (vCenterInfrastructureIdValue2 != null && vCenterInfrastructureIdValue2.Type != JTokenType.Null)
                                                {
                                                    string vCenterInfrastructureIdInstance2 = ((string)vCenterInfrastructureIdValue2);
                                                    inMageProviderSpecificSettingsInstance.VCenterInfrastructureId = vCenterInfrastructureIdInstance2;
                                                }
                                                
                                                JToken vmNicsArray4 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray4 != null && vmNicsArray4.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue4 in ((JArray)vmNicsArray4))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance4 = new VMNicDetails();
                                                        inMageProviderSpecificSettingsInstance.VMNics.Add(vMNicDetailsInstance4);
                                                        
                                                        JToken nicIdValue4 = vmNicsValue4["nicId"];
                                                        if (nicIdValue4 != null && nicIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance4 = ((string)nicIdValue4);
                                                            vMNicDetailsInstance4.NicId = nicIdInstance4;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue4 = vmNicsValue4["vMSubnetName"];
                                                        if (vMSubnetNameValue4 != null && vMSubnetNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance4 = ((string)vMSubnetNameValue4);
                                                            vMNicDetailsInstance4.VMSubnetName = vMSubnetNameInstance4;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue4 = vmNicsValue4["vMNetworkName"];
                                                        if (vMNetworkNameValue4 != null && vMNetworkNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance4 = ((string)vMNetworkNameValue4);
                                                            vMNicDetailsInstance4.VMNetworkName = vMNetworkNameInstance4;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue4 = vmNicsValue4["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue4 != null && recoveryVMNetworkIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance4 = ((string)recoveryVMNetworkIdValue4);
                                                            vMNicDetailsInstance4.RecoveryVMNetworkId = recoveryVMNetworkIdInstance4;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue4 = vmNicsValue4["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue4 != null && recoveryVMSubnetNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance4 = ((string)recoveryVMSubnetNameValue4);
                                                            vMNicDetailsInstance4.RecoveryVMSubnetName = recoveryVMSubnetNameInstance4;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue4 = vmNicsValue4["ipAddressType"];
                                                        if (ipAddressTypeValue4 != null && ipAddressTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance4 = ((string)ipAddressTypeValue4);
                                                            vMNicDetailsInstance4.IpAddressType = ipAddressTypeInstance4;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue4 = vmNicsValue4["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue4 != null && replicaNicStaticIPAddressValue4.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance4 = ((string)replicaNicStaticIPAddressValue4);
                                                            vMNicDetailsInstance4.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance4;
                                                        }
                                                        
                                                        JToken selectionTypeValue4 = vmNicsValue4["selectionType"];
                                                        if (selectionTypeValue4 != null && selectionTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance4 = ((string)selectionTypeValue4);
                                                            vMNicDetailsInstance4.SelectionType = selectionTypeInstance4;
                                                        }
                                                    }
                                                }
                                                
                                                JToken discoveryTypeValue2 = providerSpecificDetailsValue["discoveryType"];
                                                if (discoveryTypeValue2 != null && discoveryTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string discoveryTypeInstance2 = ((string)discoveryTypeValue2);
                                                    inMageProviderSpecificSettingsInstance.DiscoveryType = discoveryTypeInstance2;
                                                }
                                                
                                                JToken vmProtectionStateValue4 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue4 != null && vmProtectionStateValue4.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance4 = ((string)vmProtectionStateValue4);
                                                    inMageProviderSpecificSettingsInstance.VmProtectionState = vmProtectionStateInstance4;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue4 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue4 != null && vmProtectionStateDescriptionValue4.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance4 = ((string)vmProtectionStateDescriptionValue4);
                                                    inMageProviderSpecificSettingsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance4;
                                                }
                                                
                                                JToken azureStorageAccountIdValue = providerSpecificDetailsValue["azureStorageAccountId"];
                                                if (azureStorageAccountIdValue != null && azureStorageAccountIdValue.Type != JTokenType.Null)
                                                {
                                                    string azureStorageAccountIdInstance = ((string)azureStorageAccountIdValue);
                                                    inMageProviderSpecificSettingsInstance.AzureStorageAccountId = azureStorageAccountIdInstance;
                                                }
                                                
                                                JToken instanceTypeValue5 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue5 != null && instanceTypeValue5.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance5 = ((string)instanceTypeValue5);
                                                    inMageProviderSpecificSettingsInstance.InstanceType = instanceTypeInstance5;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = inMageProviderSpecificSettingsInstance;
                                            }
                                            if (typeName == "HyperVReplicaAzure")
                                            {
                                                HyperVReplicaAzureReplicationDetails hyperVReplicaAzureReplicationDetailsInstance = new HyperVReplicaAzureReplicationDetails();
                                                
                                                JToken azureVMDiskDetailsArray2 = providerSpecificDetailsValue["azureVMDiskDetails"];
                                                if (azureVMDiskDetailsArray2 != null && azureVMDiskDetailsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken azureVMDiskDetailsValue2 in ((JArray)azureVMDiskDetailsArray2))
                                                    {
                                                        AzureVmDiskDetails azureVmDiskDetailsInstance2 = new AzureVmDiskDetails();
                                                        hyperVReplicaAzureReplicationDetailsInstance.AzureVMDiskDetails.Add(azureVmDiskDetailsInstance2);
                                                        
                                                        JToken vhdTypeValue4 = azureVMDiskDetailsValue2["vhdType"];
                                                        if (vhdTypeValue4 != null && vhdTypeValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdTypeInstance4 = ((string)vhdTypeValue4);
                                                            azureVmDiskDetailsInstance2.VhdType = vhdTypeInstance4;
                                                        }
                                                        
                                                        JToken vhdIdValue4 = azureVMDiskDetailsValue2["vhdId"];
                                                        if (vhdIdValue4 != null && vhdIdValue4.Type != JTokenType.Null)
                                                        {
                                                            string vhdIdInstance4 = ((string)vhdIdValue4);
                                                            azureVmDiskDetailsInstance2.VhdId = vhdIdInstance4;
                                                        }
                                                        
                                                        JToken vhdNameValue6 = azureVMDiskDetailsValue2["vhdName"];
                                                        if (vhdNameValue6 != null && vhdNameValue6.Type != JTokenType.Null)
                                                        {
                                                            string vhdNameInstance6 = ((string)vhdNameValue6);
                                                            azureVmDiskDetailsInstance2.VhdName = vhdNameInstance6;
                                                        }
                                                        
                                                        JToken maxSizeMBValue4 = azureVMDiskDetailsValue2["maxSizeMB"];
                                                        if (maxSizeMBValue4 != null && maxSizeMBValue4.Type != JTokenType.Null)
                                                        {
                                                            string maxSizeMBInstance4 = ((string)maxSizeMBValue4);
                                                            azureVmDiskDetailsInstance2.MaxSizeMB = maxSizeMBInstance4;
                                                        }
                                                        
                                                        JToken targetDiskLocationValue2 = azureVMDiskDetailsValue2["targetDiskLocation"];
                                                        if (targetDiskLocationValue2 != null && targetDiskLocationValue2.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskLocationInstance2 = ((string)targetDiskLocationValue2);
                                                            azureVmDiskDetailsInstance2.TargetDiskLocation = targetDiskLocationInstance2;
                                                        }
                                                        
                                                        JToken lunIdValue2 = azureVMDiskDetailsValue2["lunId"];
                                                        if (lunIdValue2 != null && lunIdValue2.Type != JTokenType.Null)
                                                        {
                                                            string lunIdInstance2 = ((string)lunIdValue2);
                                                            azureVmDiskDetailsInstance2.LunId = lunIdInstance2;
                                                        }
                                                        
                                                        JToken targetDiskNameValue2 = azureVMDiskDetailsValue2["targetDiskName"];
                                                        if (targetDiskNameValue2 != null && targetDiskNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string targetDiskNameInstance2 = ((string)targetDiskNameValue2);
                                                            azureVmDiskDetailsInstance2.TargetDiskName = targetDiskNameInstance2;
                                                        }
                                                    }
                                                }
                                                
                                                JToken recoveryAzureVMNameValue2 = providerSpecificDetailsValue["recoveryAzureVMName"];
                                                if (recoveryAzureVMNameValue2 != null && recoveryAzureVMNameValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMNameInstance2 = ((string)recoveryAzureVMNameValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMName = recoveryAzureVMNameInstance2;
                                                }
                                                
                                                JToken recoveryAzureVMSizeValue2 = providerSpecificDetailsValue["recoveryAzureVMSize"];
                                                if (recoveryAzureVMSizeValue2 != null && recoveryAzureVMSizeValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureVMSizeInstance2 = ((string)recoveryAzureVMSizeValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureVMSize = recoveryAzureVMSizeInstance2;
                                                }
                                                
                                                JToken recoveryAzureStorageAccountValue2 = providerSpecificDetailsValue["recoveryAzureStorageAccount"];
                                                if (recoveryAzureStorageAccountValue2 != null && recoveryAzureStorageAccountValue2.Type != JTokenType.Null)
                                                {
                                                    string recoveryAzureStorageAccountInstance2 = ((string)recoveryAzureStorageAccountValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.RecoveryAzureStorageAccount = recoveryAzureStorageAccountInstance2;
                                                }
                                                
                                                JToken lastReplicatedTimeValue3 = providerSpecificDetailsValue["lastReplicatedTime"];
                                                if (lastReplicatedTimeValue3 != null && lastReplicatedTimeValue3.Type != JTokenType.Null)
                                                {
                                                    DateTime lastReplicatedTimeInstance3 = ((DateTime)lastReplicatedTimeValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.LastReplicatedTime = lastReplicatedTimeInstance3;
                                                }
                                                
                                                JToken vmIdValue5 = providerSpecificDetailsValue["vmId"];
                                                if (vmIdValue5 != null && vmIdValue5.Type != JTokenType.Null)
                                                {
                                                    string vmIdInstance5 = ((string)vmIdValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmId = vmIdInstance5;
                                                }
                                                
                                                JToken initialReplicationDetailsValue3 = providerSpecificDetailsValue["initialReplicationDetails"];
                                                if (initialReplicationDetailsValue3 != null && initialReplicationDetailsValue3.Type != JTokenType.Null)
                                                {
                                                    InitialReplicationDetails initialReplicationDetailsInstance3 = new InitialReplicationDetails();
                                                    hyperVReplicaAzureReplicationDetailsInstance.InitialReplicationDetails = initialReplicationDetailsInstance3;
                                                    
                                                    JToken initialReplicationTypeValue4 = initialReplicationDetailsValue3["initialReplicationType"];
                                                    if (initialReplicationTypeValue4 != null && initialReplicationTypeValue4.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationTypeInstance4 = ((string)initialReplicationTypeValue4);
                                                        initialReplicationDetailsInstance3.InitialReplicationType = initialReplicationTypeInstance4;
                                                    }
                                                    
                                                    JToken initialReplicationProgressPercentageValue4 = initialReplicationDetailsValue3["initialReplicationProgressPercentage"];
                                                    if (initialReplicationProgressPercentageValue4 != null && initialReplicationProgressPercentageValue4.Type != JTokenType.Null)
                                                    {
                                                        string initialReplicationProgressPercentageInstance4 = ((string)initialReplicationProgressPercentageValue4);
                                                        initialReplicationDetailsInstance3.InitialReplicationProgressPercentage = initialReplicationProgressPercentageInstance4;
                                                    }
                                                }
                                                
                                                JToken vmNicsArray5 = providerSpecificDetailsValue["vmNics"];
                                                if (vmNicsArray5 != null && vmNicsArray5.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken vmNicsValue5 in ((JArray)vmNicsArray5))
                                                    {
                                                        VMNicDetails vMNicDetailsInstance5 = new VMNicDetails();
                                                        hyperVReplicaAzureReplicationDetailsInstance.VMNics.Add(vMNicDetailsInstance5);
                                                        
                                                        JToken nicIdValue5 = vmNicsValue5["nicId"];
                                                        if (nicIdValue5 != null && nicIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string nicIdInstance5 = ((string)nicIdValue5);
                                                            vMNicDetailsInstance5.NicId = nicIdInstance5;
                                                        }
                                                        
                                                        JToken vMSubnetNameValue5 = vmNicsValue5["vMSubnetName"];
                                                        if (vMSubnetNameValue5 != null && vMSubnetNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string vMSubnetNameInstance5 = ((string)vMSubnetNameValue5);
                                                            vMNicDetailsInstance5.VMSubnetName = vMSubnetNameInstance5;
                                                        }
                                                        
                                                        JToken vMNetworkNameValue5 = vmNicsValue5["vMNetworkName"];
                                                        if (vMNetworkNameValue5 != null && vMNetworkNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string vMNetworkNameInstance5 = ((string)vMNetworkNameValue5);
                                                            vMNicDetailsInstance5.VMNetworkName = vMNetworkNameInstance5;
                                                        }
                                                        
                                                        JToken recoveryVMNetworkIdValue5 = vmNicsValue5["recoveryVMNetworkId"];
                                                        if (recoveryVMNetworkIdValue5 != null && recoveryVMNetworkIdValue5.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMNetworkIdInstance5 = ((string)recoveryVMNetworkIdValue5);
                                                            vMNicDetailsInstance5.RecoveryVMNetworkId = recoveryVMNetworkIdInstance5;
                                                        }
                                                        
                                                        JToken recoveryVMSubnetNameValue5 = vmNicsValue5["recoveryVMSubnetName"];
                                                        if (recoveryVMSubnetNameValue5 != null && recoveryVMSubnetNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string recoveryVMSubnetNameInstance5 = ((string)recoveryVMSubnetNameValue5);
                                                            vMNicDetailsInstance5.RecoveryVMSubnetName = recoveryVMSubnetNameInstance5;
                                                        }
                                                        
                                                        JToken ipAddressTypeValue5 = vmNicsValue5["ipAddressType"];
                                                        if (ipAddressTypeValue5 != null && ipAddressTypeValue5.Type != JTokenType.Null)
                                                        {
                                                            string ipAddressTypeInstance5 = ((string)ipAddressTypeValue5);
                                                            vMNicDetailsInstance5.IpAddressType = ipAddressTypeInstance5;
                                                        }
                                                        
                                                        JToken replicaNicStaticIPAddressValue5 = vmNicsValue5["replicaNicStaticIPAddress"];
                                                        if (replicaNicStaticIPAddressValue5 != null && replicaNicStaticIPAddressValue5.Type != JTokenType.Null)
                                                        {
                                                            string replicaNicStaticIPAddressInstance5 = ((string)replicaNicStaticIPAddressValue5);
                                                            vMNicDetailsInstance5.ReplicaNicStaticIPAddress = replicaNicStaticIPAddressInstance5;
                                                        }
                                                        
                                                        JToken selectionTypeValue5 = vmNicsValue5["selectionType"];
                                                        if (selectionTypeValue5 != null && selectionTypeValue5.Type != JTokenType.Null)
                                                        {
                                                            string selectionTypeInstance5 = ((string)selectionTypeValue5);
                                                            vMNicDetailsInstance5.SelectionType = selectionTypeInstance5;
                                                        }
                                                    }
                                                }
                                                
                                                JToken selectedRecoveryAzureNetworkIdValue2 = providerSpecificDetailsValue["selectedRecoveryAzureNetworkId"];
                                                if (selectedRecoveryAzureNetworkIdValue2 != null && selectedRecoveryAzureNetworkIdValue2.Type != JTokenType.Null)
                                                {
                                                    string selectedRecoveryAzureNetworkIdInstance2 = ((string)selectedRecoveryAzureNetworkIdValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SelectedRecoveryAzureNetworkId = selectedRecoveryAzureNetworkIdInstance2;
                                                }
                                                
                                                JToken encryptionValue = providerSpecificDetailsValue["encryption"];
                                                if (encryptionValue != null && encryptionValue.Type != JTokenType.Null)
                                                {
                                                    string encryptionInstance = ((string)encryptionValue);
                                                    hyperVReplicaAzureReplicationDetailsInstance.Encryption = encryptionInstance;
                                                }
                                                
                                                JToken oSDetailsValue = providerSpecificDetailsValue["oSDetails"];
                                                if (oSDetailsValue != null && oSDetailsValue.Type != JTokenType.Null)
                                                {
                                                    OSDetails oSDetailsInstance = new OSDetails();
                                                    hyperVReplicaAzureReplicationDetailsInstance.OSDetails = oSDetailsInstance;
                                                    
                                                    JToken osTypeValue3 = oSDetailsValue["osType"];
                                                    if (osTypeValue3 != null && osTypeValue3.Type != JTokenType.Null)
                                                    {
                                                        string osTypeInstance3 = ((string)osTypeValue3);
                                                        oSDetailsInstance.OsType = osTypeInstance3;
                                                    }
                                                    
                                                    JToken productTypeValue = oSDetailsValue["productType"];
                                                    if (productTypeValue != null && productTypeValue.Type != JTokenType.Null)
                                                    {
                                                        string productTypeInstance = ((string)productTypeValue);
                                                        oSDetailsInstance.ProductType = productTypeInstance;
                                                    }
                                                    
                                                    JToken osEditionValue = oSDetailsValue["osEdition"];
                                                    if (osEditionValue != null && osEditionValue.Type != JTokenType.Null)
                                                    {
                                                        string osEditionInstance = ((string)osEditionValue);
                                                        oSDetailsInstance.OSEdition = osEditionInstance;
                                                    }
                                                    
                                                    JToken oSVersionValue = oSDetailsValue["oSVersion"];
                                                    if (oSVersionValue != null && oSVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSVersionInstance = ((string)oSVersionValue);
                                                        oSDetailsInstance.OSVersion = oSVersionInstance;
                                                    }
                                                    
                                                    JToken oSMajorVersionValue = oSDetailsValue["oSMajorVersion"];
                                                    if (oSMajorVersionValue != null && oSMajorVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSMajorVersionInstance = ((string)oSMajorVersionValue);
                                                        oSDetailsInstance.OSMajorVersion = oSMajorVersionInstance;
                                                    }
                                                    
                                                    JToken oSMinorVersionValue = oSDetailsValue["oSMinorVersion"];
                                                    if (oSMinorVersionValue != null && oSMinorVersionValue.Type != JTokenType.Null)
                                                    {
                                                        string oSMinorVersionInstance = ((string)oSMinorVersionValue);
                                                        oSDetailsInstance.OSMinorVersion = oSMinorVersionInstance;
                                                    }
                                                }
                                                
                                                JToken sourceVmRAMSizeInMBValue3 = providerSpecificDetailsValue["sourceVmRAMSizeInMB"];
                                                if (sourceVmRAMSizeInMBValue3 != null && sourceVmRAMSizeInMBValue3.Type != JTokenType.Null)
                                                {
                                                    int sourceVmRAMSizeInMBInstance3 = ((int)sourceVmRAMSizeInMBValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SourceVmRAMSizeInMB = sourceVmRAMSizeInMBInstance3;
                                                }
                                                
                                                JToken sourceVmCPUCountValue3 = providerSpecificDetailsValue["sourceVmCPUCount"];
                                                if (sourceVmCPUCountValue3 != null && sourceVmCPUCountValue3.Type != JTokenType.Null)
                                                {
                                                    int sourceVmCPUCountInstance3 = ((int)sourceVmCPUCountValue3);
                                                    hyperVReplicaAzureReplicationDetailsInstance.SourceVmCPUCount = sourceVmCPUCountInstance3;
                                                }
                                                
                                                JToken licenseTypeValue2 = providerSpecificDetailsValue["licenseType"];
                                                if (licenseTypeValue2 != null && licenseTypeValue2.Type != JTokenType.Null)
                                                {
                                                    string licenseTypeInstance2 = ((string)licenseTypeValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.LicenseType = licenseTypeInstance2;
                                                }
                                                
                                                JToken enableRDPOnTargetOptionValue2 = providerSpecificDetailsValue["enableRDPOnTargetOption"];
                                                if (enableRDPOnTargetOptionValue2 != null && enableRDPOnTargetOptionValue2.Type != JTokenType.Null)
                                                {
                                                    string enableRDPOnTargetOptionInstance2 = ((string)enableRDPOnTargetOptionValue2);
                                                    hyperVReplicaAzureReplicationDetailsInstance.EnableRDPOnTargetOption = enableRDPOnTargetOptionInstance2;
                                                }
                                                
                                                JToken vmProtectionStateValue5 = providerSpecificDetailsValue["vmProtectionState"];
                                                if (vmProtectionStateValue5 != null && vmProtectionStateValue5.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateInstance5 = ((string)vmProtectionStateValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmProtectionState = vmProtectionStateInstance5;
                                                }
                                                
                                                JToken vmProtectionStateDescriptionValue5 = providerSpecificDetailsValue["vmProtectionStateDescription"];
                                                if (vmProtectionStateDescriptionValue5 != null && vmProtectionStateDescriptionValue5.Type != JTokenType.Null)
                                                {
                                                    string vmProtectionStateDescriptionInstance5 = ((string)vmProtectionStateDescriptionValue5);
                                                    hyperVReplicaAzureReplicationDetailsInstance.VmProtectionStateDescription = vmProtectionStateDescriptionInstance5;
                                                }
                                                
                                                JToken instanceTypeValue6 = providerSpecificDetailsValue["instanceType"];
                                                if (instanceTypeValue6 != null && instanceTypeValue6.Type != JTokenType.Null)
                                                {
                                                    string instanceTypeInstance6 = ((string)instanceTypeValue6);
                                                    hyperVReplicaAzureReplicationDetailsInstance.InstanceType = instanceTypeInstance6;
                                                }
                                                propertiesInstance.ProviderSpecificDetails = hyperVReplicaAzureReplicationDetailsInstance;
                                            }
                                        }
                                    }
                                    
                                    JToken idValue = valueValue["id"];
                                    if (idValue != null && idValue.Type != JTokenType.Null)
                                    {
                                        string idInstance = ((string)idValue);
                                        replicationProtectedItemInstance.Id = idInstance;
                                    }
                                    
                                    JToken nameValue = valueValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        replicationProtectedItemInstance.Name = nameInstance;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        replicationProtectedItemInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        replicationProtectedItemInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property2 in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property2.Name);
                                            string tagsValue = ((string)property2.Value);
                                            replicationProtectedItemInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets list of recovery azure vm sizes for a replication protected
        /// item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric unique name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container unique name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response model for the list of recovery azure vm sizes.
        /// </returns>
        public async Task<TargetComputeSizeResponse> ListTargetComputeSizesAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            // Validate
            if (fabricName == null)
            {
                throw new ArgumentNullException("fabricName");
            }
            if (protectionContainerName == null)
            {
                throw new ArgumentNullException("protectionContainerName");
            }
            if (replicationProtectedItemName == null)
            {
                throw new ArgumentNullException("replicationProtectedItemName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ListTargetComputeSizesAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/Subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(this.Client.ResourceGroupName);
            url = url + "/providers/";
            url = url + Uri.EscapeDataString(this.Client.ResourceNamespace);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceType);
            url = url + "/";
            url = url + Uri.EscapeDataString(this.Client.ResourceName);
            url = url + "/replicationFabrics/";
            url = url + Uri.EscapeDataString(fabricName);
            url = url + "/replicationProtectionContainers/";
            url = url + Uri.EscapeDataString(protectionContainerName);
            url = url + "/replicationProtectedItems/";
            url = url + Uri.EscapeDataString(replicationProtectedItemName);
            url = url + "/targetComputeSizes";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-11-10");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("Accept-Language", customRequestHeaders.Culture);
                httpRequest.Headers.Add("x-ms-client-request-id", customRequestHeaders.ClientRequestId);
                httpRequest.Headers.Add("x-ms-version", "2015-01-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    TargetComputeSizeResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new TargetComputeSizeResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    TargetComputeSize targetComputeSizeInstance = new TargetComputeSize();
                                    result.TargetComputeSizes.Add(targetComputeSizeInstance);
                                    
                                    JToken nameValue = valueValue["Name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        targetComputeSizeInstance.Name = nameInstance;
                                    }
                                    
                                    JToken friendlyNameValue = valueValue["FriendlyName"];
                                    if (friendlyNameValue != null && friendlyNameValue.Type != JTokenType.Null)
                                    {
                                        string friendlyNameInstance = ((string)friendlyNameValue);
                                        targetComputeSizeInstance.FriendlyName = friendlyNameInstance;
                                    }
                                    
                                    JToken cpuCoresCountValue = valueValue["cpuCoresCount"];
                                    if (cpuCoresCountValue != null && cpuCoresCountValue.Type != JTokenType.Null)
                                    {
                                        int cpuCoresCountInstance = ((int)cpuCoresCountValue);
                                        targetComputeSizeInstance.CpuCoresCount = cpuCoresCountInstance;
                                    }
                                    
                                    JToken memoryInGBValue = valueValue["memoryInGB"];
                                    if (memoryInGBValue != null && memoryInGBValue.Type != JTokenType.Null)
                                    {
                                        double memoryInGBInstance = ((double)memoryInGBValue);
                                        targetComputeSizeInstance.MemoryInGB = memoryInGBInstance;
                                    }
                                    
                                    JToken maxDataDiskCountValue = valueValue["maxDataDiskCount"];
                                    if (maxDataDiskCountValue != null && maxDataDiskCountValue.Type != JTokenType.Null)
                                    {
                                        int maxDataDiskCountInstance = ((int)maxDataDiskCountValue);
                                        targetComputeSizeInstance.MaxDataDiskCount = maxDataDiskCountInstance;
                                    }
                                    
                                    JToken maxNicsCountValue = valueValue["maxNicsCount"];
                                    if (maxNicsCountValue != null && maxNicsCountValue.Type != JTokenType.Null)
                                    {
                                        int maxNicsCountInstance = ((int)maxNicsCountValue);
                                        targetComputeSizeInstance.MaxNicsCount = maxNicsCountInstance;
                                    }
                                    
                                    JToken highIopsSupportedValue = valueValue["highIopsSupported"];
                                    if (highIopsSupportedValue != null && highIopsSupportedValue.Type != JTokenType.Null)
                                    {
                                        string highIopsSupportedInstance = ((string)highIopsSupportedValue);
                                        targetComputeSizeInstance.HighIopsSupported = highIopsSupportedInstance;
                                    }
                                    
                                }
                            }
                            
                            JToken clientRequestIdValue = responseDoc["ClientRequestId"];
                            if (clientRequestIdValue != null && clientRequestIdValue.Type != JTokenType.Null)
                            {
                                string clientRequestIdInstance = ((string)clientRequestIdValue);
                                result.ClientRequestId = clientRequestIdInstance;
                            }
                            
                            JToken correlationRequestIdValue = responseDoc["CorrelationRequestId"];
                            if (correlationRequestIdValue != null && correlationRequestIdValue.Type != JTokenType.Null)
                            {
                                string correlationRequestIdInstance = ((string)correlationRequestIdValue);
                                result.CorrelationRequestId = correlationRequestIdInstance;
                            }
                            
                            JToken dateValue = responseDoc["Date"];
                            if (dateValue != null && dateValue.Type != JTokenType.Null)
                            {
                                string dateInstance = ((string)dateValue);
                                result.Date = dateInstance;
                            }
                            
                            JToken contentTypeValue = responseDoc["ContentType"];
                            if (contentTypeValue != null && contentTypeValue.Type != JTokenType.Null)
                            {
                                string contentTypeInstance = ((string)contentTypeValue);
                                result.ContentType = contentTypeInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Content != null && httpResponse.Content.Headers.Contains("Content-Type"))
                    {
                        result.ContentType = httpResponse.Content.Headers.GetValues("Content-Type").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = httpResponse.Headers.GetValues("Date").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-client-request-id"))
                    {
                        result.ClientRequestId = httpResponse.Headers.GetValues("x-ms-client-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-correlation-request-id"))
                    {
                        result.CorrelationRequestId = httpResponse.Headers.GetValues("x-ms-correlation-request-id").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Execute planned failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Planned failover input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> PlannedFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, PlannedFailoverInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "PlannedFailoverAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginPlannedFailoverAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetPlannedFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetPlannedFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Purge Protection for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item unique name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> PurgeProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "PurgeProtectionAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginPurgeProtectionAsync(fabricName, protectionContainerName, replicationProtectedItemName, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse result = await client.ReplicationProtectedItem.GetPurgeStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetPurgeStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Executes repair replication for the given Replication protected
        /// item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> RepairReplicationAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "RepairReplicationAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginRepairReplicationAsync(fabricName, protectionContainerName, replicationProtectedItemName, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetRepairReplicationStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetRepairReplicationStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Execute reprotect for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Reprotect input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> ReprotectAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, ReverseReplicationInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "ReprotectAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginReprotectAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetReprotectStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetReprotectStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Execute Test failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Test failover input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> TestFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, TestFailoverInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "TestFailoverAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginTestFailoverAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetTestFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetTestFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Execute unplanned failover for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Unplanned failover input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> UnplannedFailoverAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, UnplannedFailoverInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "UnplannedFailoverAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginUnplannedFailoverAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetUnplannedFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetUnplannedFailoverStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Update mobility service for the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Update mobility service input.
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> UpdateMobilityServiceAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, UpdateMobilityServiceRequest input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "UpdateMobilityServiceAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginUpdateMobilityServiceAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetUpdateMobilityServiceStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetUpdateMobilityServiceStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Updates the given Replication protected item.
        /// </summary>
        /// <param name='fabricName'>
        /// Required. Fabric name.
        /// </param>
        /// <param name='protectionContainerName'>
        /// Required. Protection container name.
        /// </param>
        /// <param name='replicationProtectedItemName'>
        /// Required. Replication protected item name.
        /// </param>
        /// <param name='input'>
        /// Required. Updation input
        /// </param>
        /// <param name='customRequestHeaders'>
        /// Optional. Request header parameters.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> UpdateProtectionAsync(string fabricName, string protectionContainerName, string replicationProtectedItemName, UpdateReplicationProtectedItemInput input, CustomRequestHeaders customRequestHeaders, CancellationToken cancellationToken)
        {
            SiteRecoveryManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("fabricName", fabricName);
                tracingParameters.Add("protectionContainerName", protectionContainerName);
                tracingParameters.Add("replicationProtectedItemName", replicationProtectedItemName);
                tracingParameters.Add("input", input);
                tracingParameters.Add("customRequestHeaders", customRequestHeaders);
                TracingAdapter.Enter(invocationId, this, "UpdateProtectionAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.ReplicationProtectedItem.BeginUpdateProtectionAsync(fabricName, protectionContainerName, replicationProtectedItemName, input, customRequestHeaders, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ReplicationProtectedItemOperationResponse result = await client.ReplicationProtectedItem.GetUpdateProtectionStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = 30;
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ReplicationProtectedItem.GetUpdateProtectionStatusAsync(response.Location, cancellationToken).ConfigureAwait(false);
                delayInSeconds = 30;
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
    }
}
