// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure;
using Microsoft.Azure.Management.StreamAnalytics;
using Microsoft.Azure.Management.StreamAnalytics.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Management.StreamAnalytics
{
    /// <summary>
    /// Operations for managing the stream analytics job.
    /// </summary>
    internal partial class JobOperations : IServiceOperations<StreamAnalyticsManagementClient>, IJobOperations
    {
        /// <summary>
        /// Initializes a new instance of the JobOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal JobOperations(StreamAnalyticsManagementClient client)
        {
            this._client = client;
        }
        
        private StreamAnalyticsManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.Azure.Management.StreamAnalytics.StreamAnalyticsManagementClient.
        /// </summary>
        public StreamAnalyticsManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Delete a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginDeleteAsync(string resourceGroupName, string jobName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                TracingAdapter.Enter(invocationId, this, "BeginDeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs/";
            url = url + Uri.EscapeDataString(jobName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.PreconditionFailed)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.NotFound)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.NoContent)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Start a stream analytics job. Asynchronous call.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Parameters for a job start operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginStartAsync(string resourceGroupName, string jobName, JobStartParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginStartAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs/";
            url = url + Uri.EscapeDataString(jobName);
            url = url + "/start";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                if (parameters != null)
                {
                    JObject jobStartParametersValue = new JObject();
                    requestDoc = jobStartParametersValue;
                    
                    if (parameters.OutputStartMode != null)
                    {
                        jobStartParametersValue["outputStartMode"] = parameters.OutputStartMode;
                    }
                    
                    if (parameters.OutputStartTime != null)
                    {
                        jobStartParametersValue["outputStartTime"] = parameters.OutputStartTime.Value;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted && statusCode != HttpStatusCode.NotFound && statusCode != HttpStatusCode.Conflict && statusCode != HttpStatusCode.PreconditionFailed)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.NotFound)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.PreconditionFailed)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Stop a stream analytics job. Asynchronous call.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> BeginStopAsync(string resourceGroupName, string jobName, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                TracingAdapter.Enter(invocationId, this, "BeginStopAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs/";
            url = url + Uri.EscapeDataString(jobName);
            url = url + "/stop";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.PreconditionFailed)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.NotFound)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Create or update a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a stream
        /// analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the create stream analytics job operation.
        /// </returns>
        public async Task<JobCreateOrUpdateResponse> CreateOrUpdateAsync(string resourceGroupName, JobCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Job == null)
            {
                throw new ArgumentNullException("parameters.Job");
            }
            if (parameters.Job.Name == null)
            {
                throw new ArgumentNullException("parameters.Job.Name");
            }
            if (parameters.Job.Properties == null)
            {
                throw new ArgumentNullException("parameters.Job.Properties");
            }
            if (parameters.Job.Properties.Functions != null)
            {
                foreach (Function functionsParameterItem in parameters.Job.Properties.Functions)
                {
                    if (functionsParameterItem.Name == null)
                    {
                        throw new ArgumentNullException("parameters.Job.Properties.Functions.Name");
                    }
                }
            }
            if (parameters.Job.Properties.Inputs != null)
            {
                foreach (Input inputsParameterItem in parameters.Job.Properties.Inputs)
                {
                    if (inputsParameterItem.Name == null)
                    {
                        throw new ArgumentNullException("parameters.Job.Properties.Inputs.Name");
                    }
                }
            }
            if (parameters.Job.Properties.Outputs != null)
            {
                foreach (Output outputsParameterItem in parameters.Job.Properties.Outputs)
                {
                    if (outputsParameterItem.Name == null)
                    {
                        throw new ArgumentNullException("parameters.Job.Properties.Outputs.Name");
                    }
                }
            }
            if (parameters.Job.Properties.Transformation != null)
            {
                if (parameters.Job.Properties.Transformation.Name == null)
                {
                    throw new ArgumentNullException("parameters.Job.Properties.Transformation.Name");
                }
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs/";
            url = url + Uri.EscapeDataString(parameters.Job.Name);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("If-None-Match", parameters.IfNoneMatch);
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject jobCreateOrUpdateParametersValue = new JObject();
                requestDoc = jobCreateOrUpdateParametersValue;
                
                if (parameters.IfNoneMatch != null)
                {
                    jobCreateOrUpdateParametersValue["IfNoneMatch"] = JObject.Parse(parameters.IfNoneMatch);
                }
                
                if (parameters.Job.Id != null)
                {
                    jobCreateOrUpdateParametersValue["id"] = parameters.Job.Id;
                }
                
                jobCreateOrUpdateParametersValue["name"] = parameters.Job.Name;
                
                if (parameters.Job.Type != null)
                {
                    jobCreateOrUpdateParametersValue["type"] = parameters.Job.Type;
                }
                
                if (parameters.Job.Location != null)
                {
                    jobCreateOrUpdateParametersValue["location"] = parameters.Job.Location;
                }
                
                if (parameters.Job.Tags != null)
                {
                    JObject tagsDictionary = new JObject();
                    foreach (KeyValuePair<string, string> pair in parameters.Job.Tags)
                    {
                        string tagsKey = pair.Key;
                        string tagsValue = pair.Value;
                        tagsDictionary[tagsKey] = tagsValue;
                    }
                    jobCreateOrUpdateParametersValue["tags"] = tagsDictionary;
                }
                
                JObject propertiesValue = new JObject();
                jobCreateOrUpdateParametersValue["properties"] = propertiesValue;
                
                if (parameters.Job.Properties.Sku != null)
                {
                    JObject skuValue = new JObject();
                    propertiesValue["sku"] = skuValue;
                    
                    if (parameters.Job.Properties.Sku.Name != null)
                    {
                        skuValue["name"] = parameters.Job.Properties.Sku.Name;
                    }
                }
                
                if (parameters.Job.Properties.EventsOutOfOrderPolicy != null)
                {
                    propertiesValue["eventsOutOfOrderPolicy"] = parameters.Job.Properties.EventsOutOfOrderPolicy;
                }
                
                if (parameters.Job.Properties.OutputErrorPolicy != null)
                {
                    propertiesValue["outputErrorPolicy"] = parameters.Job.Properties.OutputErrorPolicy;
                }
                
                if (parameters.Job.Properties.EventsOutOfOrderMaxDelayInSeconds != null)
                {
                    propertiesValue["eventsOutOfOrderMaxDelayInSeconds"] = parameters.Job.Properties.EventsOutOfOrderMaxDelayInSeconds.Value;
                }
                
                if (parameters.Job.Properties.EventsLateArrivalMaxDelayInSeconds != null)
                {
                    propertiesValue["eventsLateArrivalMaxDelayInSeconds"] = parameters.Job.Properties.EventsLateArrivalMaxDelayInSeconds.Value;
                }
                
                if (parameters.Job.Properties.DataLocale != null)
                {
                    propertiesValue["dataLocale"] = parameters.Job.Properties.DataLocale;
                }
                
                if (parameters.Job.Properties.OutputStartMode != null)
                {
                    propertiesValue["outputStartMode"] = parameters.Job.Properties.OutputStartMode;
                }
                
                if (parameters.Job.Properties.OutputStartTime != null)
                {
                    propertiesValue["outputStartTime"] = parameters.Job.Properties.OutputStartTime.Value;
                }
                
                if (parameters.Job.Properties.LastOutputEventTime != null)
                {
                    propertiesValue["lastOutputEventTime"] = parameters.Job.Properties.LastOutputEventTime.Value;
                }
                
                if (parameters.Job.Properties.JobId != null)
                {
                    propertiesValue["jobId"] = parameters.Job.Properties.JobId;
                }
                
                if (parameters.Job.Properties.Etag != null)
                {
                    propertiesValue["etag"] = parameters.Job.Properties.Etag;
                }
                
                if (parameters.Job.Properties.ProvisioningState != null)
                {
                    propertiesValue["provisioningState"] = parameters.Job.Properties.ProvisioningState;
                }
                
                if (parameters.Job.Properties.JobState != null)
                {
                    propertiesValue["jobState"] = parameters.Job.Properties.JobState;
                }
                
                if (parameters.Job.Properties.CreatedDate != null)
                {
                    propertiesValue["createdDate"] = parameters.Job.Properties.CreatedDate.Value;
                }
                
                if (parameters.Job.Properties.Inputs != null)
                {
                    JArray inputsArray = new JArray();
                    foreach (Input inputsItem in parameters.Job.Properties.Inputs)
                    {
                        JObject inputValue = new JObject();
                        inputsArray.Add(inputValue);
                        
                        inputValue["name"] = inputsItem.Name;
                        
                        if (inputsItem.Properties != null)
                        {
                            JObject propertiesValue2 = new JObject();
                            inputValue["properties"] = propertiesValue2;
                            if (inputsItem.Properties is ReferenceInputProperties)
                            {
                                propertiesValue2["type"] = "Reference";
                                ReferenceInputProperties derived = ((ReferenceInputProperties)inputsItem.Properties);
                                
                                if (derived.DataSource != null)
                                {
                                    JObject datasourceValue = new JObject();
                                    propertiesValue2["datasource"] = datasourceValue;
                                    if (derived.DataSource is BlobReferenceInputDataSource)
                                    {
                                        datasourceValue["type"] = "Microsoft.Storage/Blob";
                                        BlobReferenceInputDataSource derived2 = ((BlobReferenceInputDataSource)derived.DataSource);
                                        
                                        if (derived2.Properties != null)
                                        {
                                            JObject propertiesValue3 = new JObject();
                                            datasourceValue["properties"] = propertiesValue3;
                                            
                                            if (derived2.Properties.StorageAccounts != null)
                                            {
                                                JArray storageAccountsArray = new JArray();
                                                foreach (StorageAccount storageAccountsItem in derived2.Properties.StorageAccounts)
                                                {
                                                    JObject storageAccountValue = new JObject();
                                                    storageAccountsArray.Add(storageAccountValue);
                                                    
                                                    if (storageAccountsItem.AccountName != null)
                                                    {
                                                        storageAccountValue["accountName"] = storageAccountsItem.AccountName;
                                                    }
                                                    
                                                    if (storageAccountsItem.AccountKey != null)
                                                    {
                                                        storageAccountValue["accountKey"] = storageAccountsItem.AccountKey;
                                                    }
                                                }
                                                propertiesValue3["storageAccounts"] = storageAccountsArray;
                                            }
                                            
                                            if (derived2.Properties.Container != null)
                                            {
                                                propertiesValue3["container"] = derived2.Properties.Container;
                                            }
                                            
                                            if (derived2.Properties.PathPattern != null)
                                            {
                                                propertiesValue3["pathPattern"] = derived2.Properties.PathPattern;
                                            }
                                            
                                            if (derived2.Properties.DateFormat != null)
                                            {
                                                propertiesValue3["dateFormat"] = derived2.Properties.DateFormat;
                                            }
                                            
                                            if (derived2.Properties.TimeFormat != null)
                                            {
                                                propertiesValue3["timeFormat"] = derived2.Properties.TimeFormat;
                                            }
                                        }
                                        
                                        if (derived2.Type != null)
                                        {
                                            datasourceValue["type"] = derived2.Type;
                                        }
                                    }
                                }
                                
                                if (derived.Etag != null)
                                {
                                    propertiesValue2["etag"] = derived.Etag;
                                }
                                
                                if (derived.Type != null)
                                {
                                    propertiesValue2["type"] = derived.Type;
                                }
                                
                                if (derived.Serialization != null)
                                {
                                    JObject serializationValue = new JObject();
                                    propertiesValue2["serialization"] = serializationValue;
                                    if (derived.Serialization is CsvSerialization)
                                    {
                                        serializationValue["type"] = "Csv";
                                        CsvSerialization derived3 = ((CsvSerialization)derived.Serialization);
                                        
                                        if (derived3.Properties != null)
                                        {
                                            JObject propertiesValue4 = new JObject();
                                            serializationValue["properties"] = propertiesValue4;
                                            
                                            if (derived3.Properties.FieldDelimiter != null)
                                            {
                                                propertiesValue4["fieldDelimiter"] = derived3.Properties.FieldDelimiter;
                                            }
                                            
                                            if (derived3.Properties.Encoding != null)
                                            {
                                                propertiesValue4["encoding"] = derived3.Properties.Encoding;
                                            }
                                        }
                                        
                                        if (derived3.Type != null)
                                        {
                                            serializationValue["type"] = derived3.Type;
                                        }
                                    }
                                    if (derived.Serialization is JsonSerialization)
                                    {
                                        serializationValue["type"] = "Json";
                                        JsonSerialization derived4 = ((JsonSerialization)derived.Serialization);
                                        
                                        if (derived4.Properties != null)
                                        {
                                            JObject propertiesValue5 = new JObject();
                                            serializationValue["properties"] = propertiesValue5;
                                            
                                            if (derived4.Properties.Encoding != null)
                                            {
                                                propertiesValue5["encoding"] = derived4.Properties.Encoding;
                                            }
                                            
                                            if (derived4.Properties.Format != null)
                                            {
                                                propertiesValue5["format"] = derived4.Properties.Format;
                                            }
                                        }
                                        
                                        if (derived4.Type != null)
                                        {
                                            serializationValue["type"] = derived4.Type;
                                        }
                                    }
                                    if (derived.Serialization is AvroSerialization)
                                    {
                                        serializationValue["type"] = "Avro";
                                        AvroSerialization derived5 = ((AvroSerialization)derived.Serialization);
                                        
                                        if (derived5.Properties != null)
                                        {
                                            serializationValue["properties"] = derived5.Properties.ToString();
                                        }
                                        
                                        if (derived5.Type != null)
                                        {
                                            serializationValue["type"] = derived5.Type;
                                        }
                                    }
                                }
                                
                                if (derived.Diagnostics != null)
                                {
                                    JObject diagnosticsValue = new JObject();
                                    propertiesValue2["diagnostics"] = diagnosticsValue;
                                    
                                    if (derived.Diagnostics.Conditions != null)
                                    {
                                        if (derived.Diagnostics.Conditions is ILazyCollection == false || ((ILazyCollection)derived.Diagnostics.Conditions).IsInitialized)
                                        {
                                            JArray conditionsArray = new JArray();
                                            foreach (DiagnosticCondition conditionsItem in derived.Diagnostics.Conditions)
                                            {
                                                JObject diagnosticConditionValue = new JObject();
                                                conditionsArray.Add(diagnosticConditionValue);
                                                
                                                if (conditionsItem.Since != null)
                                                {
                                                    diagnosticConditionValue["since"] = conditionsItem.Since.Value;
                                                }
                                                
                                                if (conditionsItem.Code != null)
                                                {
                                                    diagnosticConditionValue["code"] = conditionsItem.Code;
                                                }
                                                
                                                if (conditionsItem.Message != null)
                                                {
                                                    diagnosticConditionValue["message"] = conditionsItem.Message;
                                                }
                                            }
                                            diagnosticsValue["conditions"] = conditionsArray;
                                        }
                                    }
                                }
                            }
                            if (inputsItem.Properties is StreamInputProperties)
                            {
                                propertiesValue2["type"] = "Stream";
                                StreamInputProperties derived6 = ((StreamInputProperties)inputsItem.Properties);
                                
                                if (derived6.DataSource != null)
                                {
                                    JObject datasourceValue2 = new JObject();
                                    propertiesValue2["datasource"] = datasourceValue2;
                                    if (derived6.DataSource is BlobStreamInputDataSource)
                                    {
                                        datasourceValue2["type"] = "Microsoft.Storage/Blob";
                                        BlobStreamInputDataSource derived7 = ((BlobStreamInputDataSource)derived6.DataSource);
                                        
                                        if (derived7.Properties != null)
                                        {
                                            JObject propertiesValue6 = new JObject();
                                            datasourceValue2["properties"] = propertiesValue6;
                                            
                                            if (derived7.Properties.StorageAccounts != null)
                                            {
                                                JArray storageAccountsArray2 = new JArray();
                                                foreach (StorageAccount storageAccountsItem2 in derived7.Properties.StorageAccounts)
                                                {
                                                    JObject storageAccountValue2 = new JObject();
                                                    storageAccountsArray2.Add(storageAccountValue2);
                                                    
                                                    if (storageAccountsItem2.AccountName != null)
                                                    {
                                                        storageAccountValue2["accountName"] = storageAccountsItem2.AccountName;
                                                    }
                                                    
                                                    if (storageAccountsItem2.AccountKey != null)
                                                    {
                                                        storageAccountValue2["accountKey"] = storageAccountsItem2.AccountKey;
                                                    }
                                                }
                                                propertiesValue6["storageAccounts"] = storageAccountsArray2;
                                            }
                                            
                                            if (derived7.Properties.Container != null)
                                            {
                                                propertiesValue6["container"] = derived7.Properties.Container;
                                            }
                                            
                                            if (derived7.Properties.PathPattern != null)
                                            {
                                                propertiesValue6["pathPattern"] = derived7.Properties.PathPattern;
                                            }
                                            
                                            if (derived7.Properties.DateFormat != null)
                                            {
                                                propertiesValue6["dateFormat"] = derived7.Properties.DateFormat;
                                            }
                                            
                                            if (derived7.Properties.TimeFormat != null)
                                            {
                                                propertiesValue6["timeFormat"] = derived7.Properties.TimeFormat;
                                            }
                                            
                                            if (derived7.Properties.SourcePartitionCount != null)
                                            {
                                                propertiesValue6["sourcePartitionCount"] = derived7.Properties.SourcePartitionCount.Value;
                                            }
                                        }
                                        
                                        if (derived7.Type != null)
                                        {
                                            datasourceValue2["type"] = derived7.Type;
                                        }
                                    }
                                    if (derived6.DataSource is EventHubStreamInputDataSource)
                                    {
                                        datasourceValue2["type"] = "Microsoft.ServiceBus/EventHub";
                                        EventHubStreamInputDataSource derived8 = ((EventHubStreamInputDataSource)derived6.DataSource);
                                        
                                        if (derived8.Properties != null)
                                        {
                                            JObject propertiesValue7 = new JObject();
                                            datasourceValue2["properties"] = propertiesValue7;
                                            
                                            if (derived8.Properties.ServiceBusNamespace != null)
                                            {
                                                propertiesValue7["serviceBusNamespace"] = derived8.Properties.ServiceBusNamespace;
                                            }
                                            
                                            if (derived8.Properties.SharedAccessPolicyName != null)
                                            {
                                                propertiesValue7["sharedAccessPolicyName"] = derived8.Properties.SharedAccessPolicyName;
                                            }
                                            
                                            if (derived8.Properties.SharedAccessPolicyKey != null)
                                            {
                                                propertiesValue7["sharedAccessPolicyKey"] = derived8.Properties.SharedAccessPolicyKey;
                                            }
                                            
                                            if (derived8.Properties.EventHubName != null)
                                            {
                                                propertiesValue7["eventHubName"] = derived8.Properties.EventHubName;
                                            }
                                            
                                            if (derived8.Properties.ConsumerGroupName != null)
                                            {
                                                propertiesValue7["consumerGroupName"] = derived8.Properties.ConsumerGroupName;
                                            }
                                        }
                                        
                                        if (derived8.Type != null)
                                        {
                                            datasourceValue2["type"] = derived8.Type;
                                        }
                                    }
                                    if (derived6.DataSource is IoTHubStreamInputDataSource)
                                    {
                                        datasourceValue2["type"] = "Microsoft.Devices/IotHubs";
                                        IoTHubStreamInputDataSource derived9 = ((IoTHubStreamInputDataSource)derived6.DataSource);
                                        
                                        if (derived9.Properties != null)
                                        {
                                            JObject propertiesValue8 = new JObject();
                                            datasourceValue2["properties"] = propertiesValue8;
                                            
                                            if (derived9.Properties.IotHubNamespace != null)
                                            {
                                                propertiesValue8["iotHubNamespace"] = derived9.Properties.IotHubNamespace;
                                            }
                                            
                                            if (derived9.Properties.SharedAccessPolicyName != null)
                                            {
                                                propertiesValue8["sharedAccessPolicyName"] = derived9.Properties.SharedAccessPolicyName;
                                            }
                                            
                                            if (derived9.Properties.SharedAccessPolicyKey != null)
                                            {
                                                propertiesValue8["sharedAccessPolicyKey"] = derived9.Properties.SharedAccessPolicyKey;
                                            }
                                            
                                            if (derived9.Properties.ConsumerGroupName != null)
                                            {
                                                propertiesValue8["consumerGroupName"] = derived9.Properties.ConsumerGroupName;
                                            }
                                        }
                                        
                                        if (derived9.Type != null)
                                        {
                                            datasourceValue2["type"] = derived9.Type;
                                        }
                                    }
                                }
                                
                                if (derived6.Etag != null)
                                {
                                    propertiesValue2["etag"] = derived6.Etag;
                                }
                                
                                if (derived6.Type != null)
                                {
                                    propertiesValue2["type"] = derived6.Type;
                                }
                                
                                if (derived6.Serialization != null)
                                {
                                    JObject serializationValue2 = new JObject();
                                    propertiesValue2["serialization"] = serializationValue2;
                                    if (derived6.Serialization is CsvSerialization)
                                    {
                                        serializationValue2["type"] = "Csv";
                                        CsvSerialization derived10 = ((CsvSerialization)derived6.Serialization);
                                        
                                        if (derived10.Properties != null)
                                        {
                                            JObject propertiesValue9 = new JObject();
                                            serializationValue2["properties"] = propertiesValue9;
                                            
                                            if (derived10.Properties.FieldDelimiter != null)
                                            {
                                                propertiesValue9["fieldDelimiter"] = derived10.Properties.FieldDelimiter;
                                            }
                                            
                                            if (derived10.Properties.Encoding != null)
                                            {
                                                propertiesValue9["encoding"] = derived10.Properties.Encoding;
                                            }
                                        }
                                        
                                        if (derived10.Type != null)
                                        {
                                            serializationValue2["type"] = derived10.Type;
                                        }
                                    }
                                    if (derived6.Serialization is JsonSerialization)
                                    {
                                        serializationValue2["type"] = "Json";
                                        JsonSerialization derived11 = ((JsonSerialization)derived6.Serialization);
                                        
                                        if (derived11.Properties != null)
                                        {
                                            JObject propertiesValue10 = new JObject();
                                            serializationValue2["properties"] = propertiesValue10;
                                            
                                            if (derived11.Properties.Encoding != null)
                                            {
                                                propertiesValue10["encoding"] = derived11.Properties.Encoding;
                                            }
                                            
                                            if (derived11.Properties.Format != null)
                                            {
                                                propertiesValue10["format"] = derived11.Properties.Format;
                                            }
                                        }
                                        
                                        if (derived11.Type != null)
                                        {
                                            serializationValue2["type"] = derived11.Type;
                                        }
                                    }
                                    if (derived6.Serialization is AvroSerialization)
                                    {
                                        serializationValue2["type"] = "Avro";
                                        AvroSerialization derived12 = ((AvroSerialization)derived6.Serialization);
                                        
                                        if (derived12.Properties != null)
                                        {
                                            serializationValue2["properties"] = derived12.Properties.ToString();
                                        }
                                        
                                        if (derived12.Type != null)
                                        {
                                            serializationValue2["type"] = derived12.Type;
                                        }
                                    }
                                }
                                
                                if (derived6.Diagnostics != null)
                                {
                                    JObject diagnosticsValue2 = new JObject();
                                    propertiesValue2["diagnostics"] = diagnosticsValue2;
                                    
                                    if (derived6.Diagnostics.Conditions != null)
                                    {
                                        if (derived6.Diagnostics.Conditions is ILazyCollection == false || ((ILazyCollection)derived6.Diagnostics.Conditions).IsInitialized)
                                        {
                                            JArray conditionsArray2 = new JArray();
                                            foreach (DiagnosticCondition conditionsItem2 in derived6.Diagnostics.Conditions)
                                            {
                                                JObject diagnosticConditionValue2 = new JObject();
                                                conditionsArray2.Add(diagnosticConditionValue2);
                                                
                                                if (conditionsItem2.Since != null)
                                                {
                                                    diagnosticConditionValue2["since"] = conditionsItem2.Since.Value;
                                                }
                                                
                                                if (conditionsItem2.Code != null)
                                                {
                                                    diagnosticConditionValue2["code"] = conditionsItem2.Code;
                                                }
                                                
                                                if (conditionsItem2.Message != null)
                                                {
                                                    diagnosticConditionValue2["message"] = conditionsItem2.Message;
                                                }
                                            }
                                            diagnosticsValue2["conditions"] = conditionsArray2;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    propertiesValue["inputs"] = inputsArray;
                }
                
                if (parameters.Job.Properties.Transformation != null)
                {
                    JObject transformationValue = new JObject();
                    propertiesValue["transformation"] = transformationValue;
                    
                    transformationValue["name"] = parameters.Job.Properties.Transformation.Name;
                    
                    if (parameters.Job.Properties.Transformation.Properties != null)
                    {
                        JObject propertiesValue11 = new JObject();
                        transformationValue["properties"] = propertiesValue11;
                        
                        if (parameters.Job.Properties.Transformation.Properties.Etag != null)
                        {
                            propertiesValue11["etag"] = parameters.Job.Properties.Transformation.Properties.Etag;
                        }
                        
                        if (parameters.Job.Properties.Transformation.Properties.StreamingUnits != null)
                        {
                            propertiesValue11["streamingUnits"] = parameters.Job.Properties.Transformation.Properties.StreamingUnits.Value;
                        }
                        
                        if (parameters.Job.Properties.Transformation.Properties.Query != null)
                        {
                            propertiesValue11["query"] = parameters.Job.Properties.Transformation.Properties.Query;
                        }
                    }
                }
                
                if (parameters.Job.Properties.Outputs != null)
                {
                    JArray outputsArray = new JArray();
                    foreach (Output outputsItem in parameters.Job.Properties.Outputs)
                    {
                        JObject outputValue = new JObject();
                        outputsArray.Add(outputValue);
                        
                        outputValue["name"] = outputsItem.Name;
                        
                        if (outputsItem.Properties != null)
                        {
                            JObject propertiesValue12 = new JObject();
                            outputValue["properties"] = propertiesValue12;
                            
                            if (outputsItem.Properties.Etag != null)
                            {
                                propertiesValue12["etag"] = outputsItem.Properties.Etag;
                            }
                            
                            if (outputsItem.Properties.DataSource != null)
                            {
                                JObject datasourceValue3 = new JObject();
                                propertiesValue12["datasource"] = datasourceValue3;
                                if (outputsItem.Properties.DataSource is BlobOutputDataSource)
                                {
                                    datasourceValue3["type"] = "Microsoft.Storage/Blob";
                                    BlobOutputDataSource derived13 = ((BlobOutputDataSource)outputsItem.Properties.DataSource);
                                    
                                    if (derived13.Properties != null)
                                    {
                                        JObject propertiesValue13 = new JObject();
                                        datasourceValue3["properties"] = propertiesValue13;
                                        
                                        if (derived13.Properties.StorageAccounts != null)
                                        {
                                            JArray storageAccountsArray3 = new JArray();
                                            foreach (StorageAccount storageAccountsItem3 in derived13.Properties.StorageAccounts)
                                            {
                                                JObject storageAccountValue3 = new JObject();
                                                storageAccountsArray3.Add(storageAccountValue3);
                                                
                                                if (storageAccountsItem3.AccountName != null)
                                                {
                                                    storageAccountValue3["accountName"] = storageAccountsItem3.AccountName;
                                                }
                                                
                                                if (storageAccountsItem3.AccountKey != null)
                                                {
                                                    storageAccountValue3["accountKey"] = storageAccountsItem3.AccountKey;
                                                }
                                            }
                                            propertiesValue13["storageAccounts"] = storageAccountsArray3;
                                        }
                                        
                                        if (derived13.Properties.Container != null)
                                        {
                                            propertiesValue13["container"] = derived13.Properties.Container;
                                        }
                                        
                                        if (derived13.Properties.PathPattern != null)
                                        {
                                            propertiesValue13["pathPattern"] = derived13.Properties.PathPattern;
                                        }
                                        
                                        if (derived13.Properties.DateFormat != null)
                                        {
                                            propertiesValue13["dateFormat"] = derived13.Properties.DateFormat;
                                        }
                                        
                                        if (derived13.Properties.TimeFormat != null)
                                        {
                                            propertiesValue13["timeFormat"] = derived13.Properties.TimeFormat;
                                        }
                                    }
                                    
                                    if (derived13.Type != null)
                                    {
                                        datasourceValue3["type"] = derived13.Type;
                                    }
                                }
                                if (outputsItem.Properties.DataSource is AzureTableOutputDataSource)
                                {
                                    datasourceValue3["type"] = "Microsoft.Storage/Table";
                                    AzureTableOutputDataSource derived14 = ((AzureTableOutputDataSource)outputsItem.Properties.DataSource);
                                    
                                    if (derived14.Properties != null)
                                    {
                                        JObject propertiesValue14 = new JObject();
                                        datasourceValue3["properties"] = propertiesValue14;
                                        
                                        if (derived14.Properties.AccountName != null)
                                        {
                                            propertiesValue14["accountName"] = derived14.Properties.AccountName;
                                        }
                                        
                                        if (derived14.Properties.AccountKey != null)
                                        {
                                            propertiesValue14["accountKey"] = derived14.Properties.AccountKey;
                                        }
                                        
                                        if (derived14.Properties.Table != null)
                                        {
                                            propertiesValue14["table"] = derived14.Properties.Table;
                                        }
                                        
                                        if (derived14.Properties.PartitionKey != null)
                                        {
                                            propertiesValue14["partitionKey"] = derived14.Properties.PartitionKey;
                                        }
                                        
                                        if (derived14.Properties.RowKey != null)
                                        {
                                            propertiesValue14["rowKey"] = derived14.Properties.RowKey;
                                        }
                                        
                                        if (derived14.Properties.ColumnsToRemove != null)
                                        {
                                            JArray columnsToRemoveArray = new JArray();
                                            foreach (string columnsToRemoveItem in derived14.Properties.ColumnsToRemove)
                                            {
                                                columnsToRemoveArray.Add(columnsToRemoveItem);
                                            }
                                            propertiesValue14["columnsToRemove"] = columnsToRemoveArray;
                                        }
                                        
                                        if (derived14.Properties.BatchSize != null)
                                        {
                                            propertiesValue14["batchSize"] = derived14.Properties.BatchSize.Value;
                                        }
                                    }
                                    
                                    if (derived14.Type != null)
                                    {
                                        datasourceValue3["type"] = derived14.Type;
                                    }
                                }
                                if (outputsItem.Properties.DataSource is EventHubOutputDataSource)
                                {
                                    datasourceValue3["type"] = "Microsoft.ServiceBus/EventHub";
                                    EventHubOutputDataSource derived15 = ((EventHubOutputDataSource)outputsItem.Properties.DataSource);
                                    
                                    if (derived15.Properties != null)
                                    {
                                        JObject propertiesValue15 = new JObject();
                                        datasourceValue3["properties"] = propertiesValue15;
                                        
                                        if (derived15.Properties.ServiceBusNamespace != null)
                                        {
                                            propertiesValue15["serviceBusNamespace"] = derived15.Properties.ServiceBusNamespace;
                                        }
                                        
                                        if (derived15.Properties.SharedAccessPolicyName != null)
                                        {
                                            propertiesValue15["sharedAccessPolicyName"] = derived15.Properties.SharedAccessPolicyName;
                                        }
                                        
                                        if (derived15.Properties.SharedAccessPolicyKey != null)
                                        {
                                            propertiesValue15["sharedAccessPolicyKey"] = derived15.Properties.SharedAccessPolicyKey;
                                        }
                                        
                                        if (derived15.Properties.EventHubName != null)
                                        {
                                            propertiesValue15["eventHubName"] = derived15.Properties.EventHubName;
                                        }
                                        
                                        if (derived15.Properties.PartitionKey != null)
                                        {
                                            propertiesValue15["partitionKey"] = derived15.Properties.PartitionKey;
                                        }
                                    }
                                    
                                    if (derived15.Type != null)
                                    {
                                        datasourceValue3["type"] = derived15.Type;
                                    }
                                }
                                if (outputsItem.Properties.DataSource is ServiceBusQueueOutputDataSource)
                                {
                                    datasourceValue3["type"] = "Microsoft.ServiceBus/Queue";
                                    ServiceBusQueueOutputDataSource derived16 = ((ServiceBusQueueOutputDataSource)outputsItem.Properties.DataSource);
                                    
                                    if (derived16.Properties != null)
                                    {
                                        JObject propertiesValue16 = new JObject();
                                        datasourceValue3["properties"] = propertiesValue16;
                                        
                                        if (derived16.Properties.ServiceBusNamespace != null)
                                        {
                                            propertiesValue16["serviceBusNamespace"] = derived16.Properties.ServiceBusNamespace;
                                        }
                                        
                                        if (derived16.Properties.SharedAccessPolicyName != null)
                                        {
                                            propertiesValue16["sharedAccessPolicyName"] = derived16.Properties.SharedAccessPolicyName;
                                        }
                                        
                                        if (derived16.Properties.SharedAccessPolicyKey != null)
                                        {
                                            propertiesValue16["sharedAccessPolicyKey"] = derived16.Properties.SharedAccessPolicyKey;
                                        }
                                        
                                        if (derived16.Properties.QueueName != null)
                                        {
                                            propertiesValue16["queueName"] = derived16.Properties.QueueName;
                                        }
                                    }
                                    
                                    if (derived16.Type != null)
                                    {
                                        datasourceValue3["type"] = derived16.Type;
                                    }
                                }
                                if (outputsItem.Properties.DataSource is ServiceBusTopicOutputDataSource)
                                {
                                    datasourceValue3["type"] = "Microsoft.ServiceBus/Topic";
                                    ServiceBusTopicOutputDataSource derived17 = ((ServiceBusTopicOutputDataSource)outputsItem.Properties.DataSource);
                                    
                                    if (derived17.Properties != null)
                                    {
                                        JObject propertiesValue17 = new JObject();
                                        datasourceValue3["properties"] = propertiesValue17;
                                        
                                        if (derived17.Properties.ServiceBusNamespace != null)
                                        {
                                            propertiesValue17["serviceBusNamespace"] = derived17.Properties.ServiceBusNamespace;
                                        }
                                        
                                        if (derived17.Properties.SharedAccessPolicyName != null)
                                        {
                                            propertiesValue17["sharedAccessPolicyName"] = derived17.Properties.SharedAccessPolicyName;
                                        }
                                        
                                        if (derived17.Properties.SharedAccessPolicyKey != null)
                                        {
                                            propertiesValue17["sharedAccessPolicyKey"] = derived17.Properties.SharedAccessPolicyKey;
                                        }
                                        
                                        if (derived17.Properties.TopicName != null)
                                        {
                                            propertiesValue17["topicName"] = derived17.Properties.TopicName;
                                        }
                                    }
                                    
                                    if (derived17.Type != null)
                                    {
                                        datasourceValue3["type"] = derived17.Type;
                                    }
                                }
                                if (outputsItem.Properties.DataSource is SqlAzureOutputDataSource)
                                {
                                    datasourceValue3["type"] = "Microsoft.Sql/Server/Database";
                                    SqlAzureOutputDataSource derived18 = ((SqlAzureOutputDataSource)outputsItem.Properties.DataSource);
                                    
                                    if (derived18.Properties != null)
                                    {
                                        JObject propertiesValue18 = new JObject();
                                        datasourceValue3["properties"] = propertiesValue18;
                                        
                                        if (derived18.Properties.Server != null)
                                        {
                                            propertiesValue18["server"] = derived18.Properties.Server;
                                        }
                                        
                                        if (derived18.Properties.Database != null)
                                        {
                                            propertiesValue18["database"] = derived18.Properties.Database;
                                        }
                                        
                                        if (derived18.Properties.User != null)
                                        {
                                            propertiesValue18["user"] = derived18.Properties.User;
                                        }
                                        
                                        if (derived18.Properties.Password != null)
                                        {
                                            propertiesValue18["password"] = derived18.Properties.Password;
                                        }
                                        
                                        if (derived18.Properties.Table != null)
                                        {
                                            propertiesValue18["table"] = derived18.Properties.Table;
                                        }
                                    }
                                    
                                    if (derived18.Type != null)
                                    {
                                        datasourceValue3["type"] = derived18.Type;
                                    }
                                }
                                if (outputsItem.Properties.DataSource is DocumentDbOutputDataSource)
                                {
                                    datasourceValue3["type"] = "Microsoft.Storage/DocumentDB";
                                    DocumentDbOutputDataSource derived19 = ((DocumentDbOutputDataSource)outputsItem.Properties.DataSource);
                                    
                                    if (derived19.Properties != null)
                                    {
                                        JObject propertiesValue19 = new JObject();
                                        datasourceValue3["properties"] = propertiesValue19;
                                        
                                        if (derived19.Properties.AccountId != null)
                                        {
                                            propertiesValue19["accountId"] = derived19.Properties.AccountId;
                                        }
                                        
                                        if (derived19.Properties.AccountKey != null)
                                        {
                                            propertiesValue19["accountKey"] = derived19.Properties.AccountKey;
                                        }
                                        
                                        if (derived19.Properties.Database != null)
                                        {
                                            propertiesValue19["database"] = derived19.Properties.Database;
                                        }
                                        
                                        if (derived19.Properties.CollectionNamePattern != null)
                                        {
                                            propertiesValue19["collectionNamePattern"] = derived19.Properties.CollectionNamePattern;
                                        }
                                        
                                        if (derived19.Properties.PartitionKey != null)
                                        {
                                            propertiesValue19["partitionKey"] = derived19.Properties.PartitionKey;
                                        }
                                        
                                        if (derived19.Properties.DocumentId != null)
                                        {
                                            propertiesValue19["documentId"] = derived19.Properties.DocumentId;
                                        }
                                    }
                                    
                                    if (derived19.Type != null)
                                    {
                                        datasourceValue3["type"] = derived19.Type;
                                    }
                                }
                            }
                            
                            if (outputsItem.Properties.Serialization != null)
                            {
                                JObject serializationValue3 = new JObject();
                                propertiesValue12["serialization"] = serializationValue3;
                                if (outputsItem.Properties.Serialization is CsvSerialization)
                                {
                                    serializationValue3["type"] = "Csv";
                                    CsvSerialization derived20 = ((CsvSerialization)outputsItem.Properties.Serialization);
                                    
                                    if (derived20.Properties != null)
                                    {
                                        JObject propertiesValue20 = new JObject();
                                        serializationValue3["properties"] = propertiesValue20;
                                        
                                        if (derived20.Properties.FieldDelimiter != null)
                                        {
                                            propertiesValue20["fieldDelimiter"] = derived20.Properties.FieldDelimiter;
                                        }
                                        
                                        if (derived20.Properties.Encoding != null)
                                        {
                                            propertiesValue20["encoding"] = derived20.Properties.Encoding;
                                        }
                                    }
                                    
                                    if (derived20.Type != null)
                                    {
                                        serializationValue3["type"] = derived20.Type;
                                    }
                                }
                                if (outputsItem.Properties.Serialization is JsonSerialization)
                                {
                                    serializationValue3["type"] = "Json";
                                    JsonSerialization derived21 = ((JsonSerialization)outputsItem.Properties.Serialization);
                                    
                                    if (derived21.Properties != null)
                                    {
                                        JObject propertiesValue21 = new JObject();
                                        serializationValue3["properties"] = propertiesValue21;
                                        
                                        if (derived21.Properties.Encoding != null)
                                        {
                                            propertiesValue21["encoding"] = derived21.Properties.Encoding;
                                        }
                                        
                                        if (derived21.Properties.Format != null)
                                        {
                                            propertiesValue21["format"] = derived21.Properties.Format;
                                        }
                                    }
                                    
                                    if (derived21.Type != null)
                                    {
                                        serializationValue3["type"] = derived21.Type;
                                    }
                                }
                                if (outputsItem.Properties.Serialization is AvroSerialization)
                                {
                                    serializationValue3["type"] = "Avro";
                                    AvroSerialization derived22 = ((AvroSerialization)outputsItem.Properties.Serialization);
                                    
                                    if (derived22.Properties != null)
                                    {
                                        serializationValue3["properties"] = derived22.Properties.ToString();
                                    }
                                    
                                    if (derived22.Type != null)
                                    {
                                        serializationValue3["type"] = derived22.Type;
                                    }
                                }
                            }
                            
                            if (outputsItem.Properties.Diagnostics != null)
                            {
                                JObject diagnosticsValue3 = new JObject();
                                propertiesValue12["diagnostics"] = diagnosticsValue3;
                                
                                if (outputsItem.Properties.Diagnostics.Conditions != null)
                                {
                                    if (outputsItem.Properties.Diagnostics.Conditions is ILazyCollection == false || ((ILazyCollection)outputsItem.Properties.Diagnostics.Conditions).IsInitialized)
                                    {
                                        JArray conditionsArray3 = new JArray();
                                        foreach (DiagnosticCondition conditionsItem3 in outputsItem.Properties.Diagnostics.Conditions)
                                        {
                                            JObject diagnosticConditionValue3 = new JObject();
                                            conditionsArray3.Add(diagnosticConditionValue3);
                                            
                                            if (conditionsItem3.Since != null)
                                            {
                                                diagnosticConditionValue3["since"] = conditionsItem3.Since.Value;
                                            }
                                            
                                            if (conditionsItem3.Code != null)
                                            {
                                                diagnosticConditionValue3["code"] = conditionsItem3.Code;
                                            }
                                            
                                            if (conditionsItem3.Message != null)
                                            {
                                                diagnosticConditionValue3["message"] = conditionsItem3.Message;
                                            }
                                        }
                                        diagnosticsValue3["conditions"] = conditionsArray3;
                                    }
                                }
                            }
                        }
                    }
                    propertiesValue["outputs"] = outputsArray;
                }
                
                if (parameters.Job.Properties.Functions != null)
                {
                    JArray functionsArray = new JArray();
                    foreach (Function functionsItem in parameters.Job.Properties.Functions)
                    {
                        JObject functionValue = new JObject();
                        functionsArray.Add(functionValue);
                        
                        functionValue["name"] = functionsItem.Name;
                        
                        if (functionsItem.Properties != null)
                        {
                            JObject propertiesValue22 = new JObject();
                            functionValue["properties"] = propertiesValue22;
                            if (functionsItem.Properties is ScalarFunctionProperties)
                            {
                                propertiesValue22["type"] = "Scalar";
                                ScalarFunctionProperties derived23 = ((ScalarFunctionProperties)functionsItem.Properties);
                                
                                if (derived23.Properties != null)
                                {
                                    JObject propertiesValue23 = new JObject();
                                    propertiesValue22["properties"] = propertiesValue23;
                                    
                                    if (derived23.Properties.Inputs != null)
                                    {
                                        JArray inputsArray2 = new JArray();
                                        foreach (FunctionInput inputsItem2 in derived23.Properties.Inputs)
                                        {
                                            JObject functionInputValue = new JObject();
                                            inputsArray2.Add(functionInputValue);
                                            
                                            if (inputsItem2.DataType != null)
                                            {
                                                functionInputValue["dataType"] = inputsItem2.DataType;
                                            }
                                            
                                            if (inputsItem2.IsConfigurationParameter != null)
                                            {
                                                functionInputValue["isConfigurationParameter"] = inputsItem2.IsConfigurationParameter.Value;
                                            }
                                        }
                                        propertiesValue23["inputs"] = inputsArray2;
                                    }
                                    
                                    if (derived23.Properties.Output != null)
                                    {
                                        JObject outputValue2 = new JObject();
                                        propertiesValue23["output"] = outputValue2;
                                        
                                        if (derived23.Properties.Output.DataType != null)
                                        {
                                            outputValue2["dataType"] = derived23.Properties.Output.DataType;
                                        }
                                    }
                                    
                                    if (derived23.Properties.Binding != null)
                                    {
                                        JObject bindingValue = new JObject();
                                        propertiesValue23["binding"] = bindingValue;
                                        if (derived23.Properties.Binding is AzureMachineLearningWebServiceFunctionBinding)
                                        {
                                            bindingValue["type"] = "Microsoft.MachineLearning/WebService";
                                            AzureMachineLearningWebServiceFunctionBinding derived24 = ((AzureMachineLearningWebServiceFunctionBinding)derived23.Properties.Binding);
                                            
                                            if (derived24.Properties != null)
                                            {
                                                JObject propertiesValue24 = new JObject();
                                                bindingValue["properties"] = propertiesValue24;
                                                
                                                if (derived24.Properties.Endpoint != null)
                                                {
                                                    propertiesValue24["endpoint"] = derived24.Properties.Endpoint;
                                                }
                                                
                                                if (derived24.Properties.ApiKey != null)
                                                {
                                                    propertiesValue24["apiKey"] = derived24.Properties.ApiKey;
                                                }
                                                
                                                if (derived24.Properties.Inputs != null)
                                                {
                                                    JObject inputsValue = new JObject();
                                                    propertiesValue24["inputs"] = inputsValue;
                                                    
                                                    inputsValue["name"] = derived24.Properties.Inputs.Name;
                                                    
                                                    if (derived24.Properties.Inputs.ColumnNames != null)
                                                    {
                                                        JArray columnNamesArray = new JArray();
                                                        foreach (AzureMachineLearningWebServiceInputColumn columnNamesItem in derived24.Properties.Inputs.ColumnNames)
                                                        {
                                                            JObject azureMachineLearningWebServiceInputColumnValue = new JObject();
                                                            columnNamesArray.Add(azureMachineLearningWebServiceInputColumnValue);
                                                            
                                                            azureMachineLearningWebServiceInputColumnValue["name"] = columnNamesItem.Name;
                                                            
                                                            if (columnNamesItem.DataType != null)
                                                            {
                                                                azureMachineLearningWebServiceInputColumnValue["dataType"] = columnNamesItem.DataType;
                                                            }
                                                            
                                                            if (columnNamesItem.MapTo != null)
                                                            {
                                                                azureMachineLearningWebServiceInputColumnValue["mapTo"] = columnNamesItem.MapTo.Value;
                                                            }
                                                        }
                                                        inputsValue["columnNames"] = columnNamesArray;
                                                    }
                                                }
                                                
                                                if (derived24.Properties.Outputs != null)
                                                {
                                                    JArray outputsArray2 = new JArray();
                                                    foreach (AzureMachineLearningWebServiceOutputColumn outputsItem2 in derived24.Properties.Outputs)
                                                    {
                                                        JObject azureMachineLearningWebServiceOutputColumnValue = new JObject();
                                                        outputsArray2.Add(azureMachineLearningWebServiceOutputColumnValue);
                                                        
                                                        azureMachineLearningWebServiceOutputColumnValue["name"] = outputsItem2.Name;
                                                        
                                                        if (outputsItem2.DataType != null)
                                                        {
                                                            azureMachineLearningWebServiceOutputColumnValue["dataType"] = outputsItem2.DataType;
                                                        }
                                                    }
                                                    propertiesValue24["outputs"] = outputsArray2;
                                                }
                                                
                                                if (derived24.Properties.BatchSize != null)
                                                {
                                                    propertiesValue24["batchSize"] = derived24.Properties.BatchSize.Value;
                                                }
                                            }
                                            
                                            if (derived24.Type != null)
                                            {
                                                bindingValue["type"] = derived24.Type;
                                            }
                                        }
                                        if (derived23.Properties.Binding is JavaScriptFunctionBinding)
                                        {
                                            bindingValue["type"] = "Microsoft.StreamAnalytics/JavascriptUdf";
                                            JavaScriptFunctionBinding derived25 = ((JavaScriptFunctionBinding)derived23.Properties.Binding);
                                            
                                            if (derived25.Properties != null)
                                            {
                                                JObject propertiesValue25 = new JObject();
                                                bindingValue["properties"] = propertiesValue25;
                                                
                                                if (derived25.Properties.Script != null)
                                                {
                                                    propertiesValue25["script"] = derived25.Properties.Script;
                                                }
                                            }
                                            
                                            if (derived25.Type != null)
                                            {
                                                bindingValue["type"] = derived25.Type;
                                            }
                                        }
                                    }
                                }
                                
                                if (derived23.Etag != null)
                                {
                                    propertiesValue22["etag"] = derived23.Etag;
                                }
                                
                                if (derived23.Type != null)
                                {
                                    propertiesValue22["type"] = derived23.Type;
                                }
                            }
                        }
                    }
                    propertiesValue["functions"] = functionsArray;
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobCreateOrUpdateResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobCreateOrUpdateResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                jobInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                jobInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                jobInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                jobInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                jobInstance.Tags = new Dictionary<string, string>();
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey2 = ((string)property.Name);
                                    string tagsValue2 = ((string)property.Value);
                                    jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken propertiesValue26 = responseDoc["properties"];
                            if (propertiesValue26 != null && propertiesValue26.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken skuValue2 = propertiesValue26["sku"];
                                if (skuValue2 != null && skuValue2.Type != JTokenType.Null)
                                {
                                    Sku skuInstance = new Sku();
                                    propertiesInstance.Sku = skuInstance;
                                    
                                    JToken nameValue2 = skuValue2["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        string nameInstance2 = ((string)nameValue2);
                                        skuInstance.Name = nameInstance2;
                                    }
                                }
                                
                                JToken eventsOutOfOrderPolicyValue = propertiesValue26["eventsOutOfOrderPolicy"];
                                if (eventsOutOfOrderPolicyValue != null && eventsOutOfOrderPolicyValue.Type != JTokenType.Null)
                                {
                                    string eventsOutOfOrderPolicyInstance = ((string)eventsOutOfOrderPolicyValue);
                                    propertiesInstance.EventsOutOfOrderPolicy = eventsOutOfOrderPolicyInstance;
                                }
                                
                                JToken outputErrorPolicyValue = propertiesValue26["outputErrorPolicy"];
                                if (outputErrorPolicyValue != null && outputErrorPolicyValue.Type != JTokenType.Null)
                                {
                                    string outputErrorPolicyInstance = ((string)outputErrorPolicyValue);
                                    propertiesInstance.OutputErrorPolicy = outputErrorPolicyInstance;
                                }
                                
                                JToken eventsOutOfOrderMaxDelayInSecondsValue = propertiesValue26["eventsOutOfOrderMaxDelayInSeconds"];
                                if (eventsOutOfOrderMaxDelayInSecondsValue != null && eventsOutOfOrderMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsOutOfOrderMaxDelayInSecondsInstance = ((int)eventsOutOfOrderMaxDelayInSecondsValue);
                                    propertiesInstance.EventsOutOfOrderMaxDelayInSeconds = eventsOutOfOrderMaxDelayInSecondsInstance;
                                }
                                
                                JToken eventsLateArrivalMaxDelayInSecondsValue = propertiesValue26["eventsLateArrivalMaxDelayInSeconds"];
                                if (eventsLateArrivalMaxDelayInSecondsValue != null && eventsLateArrivalMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsLateArrivalMaxDelayInSecondsInstance = ((int)eventsLateArrivalMaxDelayInSecondsValue);
                                    propertiesInstance.EventsLateArrivalMaxDelayInSeconds = eventsLateArrivalMaxDelayInSecondsInstance;
                                }
                                
                                JToken dataLocaleValue = propertiesValue26["dataLocale"];
                                if (dataLocaleValue != null && dataLocaleValue.Type != JTokenType.Null)
                                {
                                    string dataLocaleInstance = ((string)dataLocaleValue);
                                    propertiesInstance.DataLocale = dataLocaleInstance;
                                }
                                
                                JToken outputStartModeValue = propertiesValue26["outputStartMode"];
                                if (outputStartModeValue != null && outputStartModeValue.Type != JTokenType.Null)
                                {
                                    string outputStartModeInstance = ((string)outputStartModeValue);
                                    propertiesInstance.OutputStartMode = outputStartModeInstance;
                                }
                                
                                JToken outputStartTimeValue = propertiesValue26["outputStartTime"];
                                if (outputStartTimeValue != null && outputStartTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime outputStartTimeInstance = ((DateTime)outputStartTimeValue);
                                    propertiesInstance.OutputStartTime = outputStartTimeInstance;
                                }
                                
                                JToken lastOutputEventTimeValue = propertiesValue26["lastOutputEventTime"];
                                if (lastOutputEventTimeValue != null && lastOutputEventTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastOutputEventTimeInstance = ((DateTime)lastOutputEventTimeValue);
                                    propertiesInstance.LastOutputEventTime = lastOutputEventTimeInstance;
                                }
                                
                                JToken jobIdValue = propertiesValue26["jobId"];
                                if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                {
                                    string jobIdInstance = ((string)jobIdValue);
                                    propertiesInstance.JobId = jobIdInstance;
                                }
                                
                                JToken etagValue = propertiesValue26["etag"];
                                if (etagValue != null && etagValue.Type != JTokenType.Null)
                                {
                                    string etagInstance = ((string)etagValue);
                                    propertiesInstance.Etag = etagInstance;
                                }
                                
                                JToken provisioningStateValue = propertiesValue26["provisioningState"];
                                if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance = ((string)provisioningStateValue);
                                    propertiesInstance.ProvisioningState = provisioningStateInstance;
                                }
                                
                                JToken jobStateValue = propertiesValue26["jobState"];
                                if (jobStateValue != null && jobStateValue.Type != JTokenType.Null)
                                {
                                    string jobStateInstance = ((string)jobStateValue);
                                    propertiesInstance.JobState = jobStateInstance;
                                }
                                
                                JToken createdDateValue = propertiesValue26["createdDate"];
                                if (createdDateValue != null && createdDateValue.Type != JTokenType.Null)
                                {
                                    DateTime createdDateInstance = ((DateTime)createdDateValue);
                                    propertiesInstance.CreatedDate = createdDateInstance;
                                }
                                
                                JToken inputsArray3 = propertiesValue26["inputs"];
                                if (inputsArray3 != null && inputsArray3.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Inputs = new List<Input>();
                                    foreach (JToken inputsValue2 in ((JArray)inputsArray3))
                                    {
                                        Input inputInstance = new Input();
                                        propertiesInstance.Inputs.Add(inputInstance);
                                        
                                        JToken nameValue3 = inputsValue2["name"];
                                        if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                        {
                                            string nameInstance3 = ((string)nameValue3);
                                            inputInstance.Name = nameInstance3;
                                        }
                                        
                                        JToken propertiesValue27 = inputsValue2["properties"];
                                        if (propertiesValue27 != null && propertiesValue27.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)propertiesValue27["type"]);
                                            if (typeName == "Reference")
                                            {
                                                ReferenceInputProperties referenceInputPropertiesInstance = new ReferenceInputProperties();
                                                
                                                JToken datasourceValue4 = propertiesValue27["datasource"];
                                                if (datasourceValue4 != null && datasourceValue4.Type != JTokenType.Null)
                                                {
                                                    string typeName2 = ((string)datasourceValue4["type"]);
                                                    if (typeName2 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobReferenceInputDataSource blobReferenceInputDataSourceInstance = new BlobReferenceInputDataSource();
                                                        
                                                        JToken propertiesValue28 = datasourceValue4["properties"];
                                                        if (propertiesValue28 != null && propertiesValue28.Type != JTokenType.Null)
                                                        {
                                                            BlobReferenceInputDataSourceProperties propertiesInstance2 = new BlobReferenceInputDataSourceProperties();
                                                            blobReferenceInputDataSourceInstance.Properties = propertiesInstance2;
                                                            
                                                            JToken storageAccountsArray4 = propertiesValue28["storageAccounts"];
                                                            if (storageAccountsArray4 != null && storageAccountsArray4.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance2.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue in ((JArray)storageAccountsArray4))
                                                                {
                                                                    StorageAccount storageAccountInstance = new StorageAccount();
                                                                    propertiesInstance2.StorageAccounts.Add(storageAccountInstance);
                                                                    
                                                                    JToken accountNameValue = storageAccountsValue["accountName"];
                                                                    if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance = ((string)accountNameValue);
                                                                        storageAccountInstance.AccountName = accountNameInstance;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue = storageAccountsValue["accountKey"];
                                                                    if (accountKeyValue != null && accountKeyValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance = ((string)accountKeyValue);
                                                                        storageAccountInstance.AccountKey = accountKeyInstance;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue = propertiesValue28["container"];
                                                            if (containerValue != null && containerValue.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance = ((string)containerValue);
                                                                propertiesInstance2.Container = containerInstance;
                                                            }
                                                            
                                                            JToken pathPatternValue = propertiesValue28["pathPattern"];
                                                            if (pathPatternValue != null && pathPatternValue.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance = ((string)pathPatternValue);
                                                                propertiesInstance2.PathPattern = pathPatternInstance;
                                                            }
                                                            
                                                            JToken dateFormatValue = propertiesValue28["dateFormat"];
                                                            if (dateFormatValue != null && dateFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance = ((string)dateFormatValue);
                                                                propertiesInstance2.DateFormat = dateFormatInstance;
                                                            }
                                                            
                                                            JToken timeFormatValue = propertiesValue28["timeFormat"];
                                                            if (timeFormatValue != null && timeFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance = ((string)timeFormatValue);
                                                                propertiesInstance2.TimeFormat = timeFormatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue2 = datasourceValue4["type"];
                                                        if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance2 = ((string)typeValue2);
                                                            blobReferenceInputDataSourceInstance.Type = typeInstance2;
                                                        }
                                                        referenceInputPropertiesInstance.DataSource = blobReferenceInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue2 = propertiesValue27["etag"];
                                                if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                {
                                                    string etagInstance2 = ((string)etagValue2);
                                                    referenceInputPropertiesInstance.Etag = etagInstance2;
                                                }
                                                
                                                JToken typeValue3 = propertiesValue27["type"];
                                                if (typeValue3 != null && typeValue3.Type != JTokenType.Null)
                                                {
                                                    string typeInstance3 = ((string)typeValue3);
                                                    referenceInputPropertiesInstance.Type = typeInstance3;
                                                }
                                                
                                                JToken serializationValue4 = propertiesValue27["serialization"];
                                                if (serializationValue4 != null && serializationValue4.Type != JTokenType.Null)
                                                {
                                                    string typeName3 = ((string)serializationValue4["type"]);
                                                    if (typeName3 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance = new CsvSerialization();
                                                        
                                                        JToken propertiesValue29 = serializationValue4["properties"];
                                                        if (propertiesValue29 != null && propertiesValue29.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance3 = new CsvSerializationProperties();
                                                            csvSerializationInstance.Properties = propertiesInstance3;
                                                            
                                                            JToken fieldDelimiterValue = propertiesValue29["fieldDelimiter"];
                                                            if (fieldDelimiterValue != null && fieldDelimiterValue.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance = ((string)fieldDelimiterValue);
                                                                propertiesInstance3.FieldDelimiter = fieldDelimiterInstance;
                                                            }
                                                            
                                                            JToken encodingValue = propertiesValue29["encoding"];
                                                            if (encodingValue != null && encodingValue.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance = ((string)encodingValue);
                                                                propertiesInstance3.Encoding = encodingInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue4 = serializationValue4["type"];
                                                        if (typeValue4 != null && typeValue4.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance4 = ((string)typeValue4);
                                                            csvSerializationInstance.Type = typeInstance4;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = csvSerializationInstance;
                                                    }
                                                    if (typeName3 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance = new JsonSerialization();
                                                        
                                                        JToken propertiesValue30 = serializationValue4["properties"];
                                                        if (propertiesValue30 != null && propertiesValue30.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance4 = new JsonSerializationProperties();
                                                            jsonSerializationInstance.Properties = propertiesInstance4;
                                                            
                                                            JToken encodingValue2 = propertiesValue30["encoding"];
                                                            if (encodingValue2 != null && encodingValue2.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance2 = ((string)encodingValue2);
                                                                propertiesInstance4.Encoding = encodingInstance2;
                                                            }
                                                            
                                                            JToken formatValue = propertiesValue30["format"];
                                                            if (formatValue != null && formatValue.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance = ((string)formatValue);
                                                                propertiesInstance4.Format = formatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue5 = serializationValue4["type"];
                                                        if (typeValue5 != null && typeValue5.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance5 = ((string)typeValue5);
                                                            jsonSerializationInstance.Type = typeInstance5;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = jsonSerializationInstance;
                                                    }
                                                    if (typeName3 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance = new AvroSerialization();
                                                        
                                                        JToken propertiesValue31 = serializationValue4["properties"];
                                                        if (propertiesValue31 != null && propertiesValue31.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance5 = new AvroSerializationProperties();
                                                            avroSerializationInstance.Properties = propertiesInstance5;
                                                        }
                                                        
                                                        JToken typeValue6 = serializationValue4["type"];
                                                        if (typeValue6 != null && typeValue6.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance6 = ((string)typeValue6);
                                                            avroSerializationInstance.Type = typeInstance6;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = avroSerializationInstance;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue4 = propertiesValue27["diagnostics"];
                                                if (diagnosticsValue4 != null && diagnosticsValue4.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance = new Diagnostics();
                                                    referenceInputPropertiesInstance.Diagnostics = diagnosticsInstance;
                                                    
                                                    JToken conditionsArray4 = diagnosticsValue4["conditions"];
                                                    if (conditionsArray4 != null && conditionsArray4.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue in ((JArray)conditionsArray4))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance = new DiagnosticCondition();
                                                            diagnosticsInstance.Conditions.Add(diagnosticConditionInstance);
                                                            
                                                            JToken sinceValue = conditionsValue["since"];
                                                            if (sinceValue != null && sinceValue.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance = ((DateTime)sinceValue);
                                                                diagnosticConditionInstance.Since = sinceInstance;
                                                            }
                                                            
                                                            JToken codeValue = conditionsValue["code"];
                                                            if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance = ((string)codeValue);
                                                                diagnosticConditionInstance.Code = codeInstance;
                                                            }
                                                            
                                                            JToken messageValue = conditionsValue["message"];
                                                            if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance = ((string)messageValue);
                                                                diagnosticConditionInstance.Message = messageInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = referenceInputPropertiesInstance;
                                            }
                                            if (typeName == "Stream")
                                            {
                                                StreamInputProperties streamInputPropertiesInstance = new StreamInputProperties();
                                                
                                                JToken datasourceValue5 = propertiesValue27["datasource"];
                                                if (datasourceValue5 != null && datasourceValue5.Type != JTokenType.Null)
                                                {
                                                    string typeName4 = ((string)datasourceValue5["type"]);
                                                    if (typeName4 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobStreamInputDataSource blobStreamInputDataSourceInstance = new BlobStreamInputDataSource();
                                                        
                                                        JToken propertiesValue32 = datasourceValue5["properties"];
                                                        if (propertiesValue32 != null && propertiesValue32.Type != JTokenType.Null)
                                                        {
                                                            BlobStreamInputDataSourceProperties propertiesInstance6 = new BlobStreamInputDataSourceProperties();
                                                            blobStreamInputDataSourceInstance.Properties = propertiesInstance6;
                                                            
                                                            JToken storageAccountsArray5 = propertiesValue32["storageAccounts"];
                                                            if (storageAccountsArray5 != null && storageAccountsArray5.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance6.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue2 in ((JArray)storageAccountsArray5))
                                                                {
                                                                    StorageAccount storageAccountInstance2 = new StorageAccount();
                                                                    propertiesInstance6.StorageAccounts.Add(storageAccountInstance2);
                                                                    
                                                                    JToken accountNameValue2 = storageAccountsValue2["accountName"];
                                                                    if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance2 = ((string)accountNameValue2);
                                                                        storageAccountInstance2.AccountName = accountNameInstance2;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue2 = storageAccountsValue2["accountKey"];
                                                                    if (accountKeyValue2 != null && accountKeyValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance2 = ((string)accountKeyValue2);
                                                                        storageAccountInstance2.AccountKey = accountKeyInstance2;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue2 = propertiesValue32["container"];
                                                            if (containerValue2 != null && containerValue2.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance2 = ((string)containerValue2);
                                                                propertiesInstance6.Container = containerInstance2;
                                                            }
                                                            
                                                            JToken pathPatternValue2 = propertiesValue32["pathPattern"];
                                                            if (pathPatternValue2 != null && pathPatternValue2.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance2 = ((string)pathPatternValue2);
                                                                propertiesInstance6.PathPattern = pathPatternInstance2;
                                                            }
                                                            
                                                            JToken dateFormatValue2 = propertiesValue32["dateFormat"];
                                                            if (dateFormatValue2 != null && dateFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance2 = ((string)dateFormatValue2);
                                                                propertiesInstance6.DateFormat = dateFormatInstance2;
                                                            }
                                                            
                                                            JToken timeFormatValue2 = propertiesValue32["timeFormat"];
                                                            if (timeFormatValue2 != null && timeFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance2 = ((string)timeFormatValue2);
                                                                propertiesInstance6.TimeFormat = timeFormatInstance2;
                                                            }
                                                            
                                                            JToken sourcePartitionCountValue = propertiesValue32["sourcePartitionCount"];
                                                            if (sourcePartitionCountValue != null && sourcePartitionCountValue.Type != JTokenType.Null)
                                                            {
                                                                int sourcePartitionCountInstance = ((int)sourcePartitionCountValue);
                                                                propertiesInstance6.SourcePartitionCount = sourcePartitionCountInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue7 = datasourceValue5["type"];
                                                        if (typeValue7 != null && typeValue7.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance7 = ((string)typeValue7);
                                                            blobStreamInputDataSourceInstance.Type = typeInstance7;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = blobStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.ServiceBus/EventHub")
                                                    {
                                                        EventHubStreamInputDataSource eventHubStreamInputDataSourceInstance = new EventHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue33 = datasourceValue5["properties"];
                                                        if (propertiesValue33 != null && propertiesValue33.Type != JTokenType.Null)
                                                        {
                                                            EventHubStreamInputDataSourceProperties propertiesInstance7 = new EventHubStreamInputDataSourceProperties();
                                                            eventHubStreamInputDataSourceInstance.Properties = propertiesInstance7;
                                                            
                                                            JToken serviceBusNamespaceValue = propertiesValue33["serviceBusNamespace"];
                                                            if (serviceBusNamespaceValue != null && serviceBusNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string serviceBusNamespaceInstance = ((string)serviceBusNamespaceValue);
                                                                propertiesInstance7.ServiceBusNamespace = serviceBusNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue = propertiesValue33["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue != null && sharedAccessPolicyNameValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance = ((string)sharedAccessPolicyNameValue);
                                                                propertiesInstance7.SharedAccessPolicyName = sharedAccessPolicyNameInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue = propertiesValue33["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue != null && sharedAccessPolicyKeyValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance = ((string)sharedAccessPolicyKeyValue);
                                                                propertiesInstance7.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance;
                                                            }
                                                            
                                                            JToken eventHubNameValue = propertiesValue33["eventHubName"];
                                                            if (eventHubNameValue != null && eventHubNameValue.Type != JTokenType.Null)
                                                            {
                                                                string eventHubNameInstance = ((string)eventHubNameValue);
                                                                propertiesInstance7.EventHubName = eventHubNameInstance;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue = propertiesValue33["consumerGroupName"];
                                                            if (consumerGroupNameValue != null && consumerGroupNameValue.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance = ((string)consumerGroupNameValue);
                                                                propertiesInstance7.ConsumerGroupName = consumerGroupNameInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue8 = datasourceValue5["type"];
                                                        if (typeValue8 != null && typeValue8.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance8 = ((string)typeValue8);
                                                            eventHubStreamInputDataSourceInstance.Type = typeInstance8;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = eventHubStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.Devices/IotHubs")
                                                    {
                                                        IoTHubStreamInputDataSource ioTHubStreamInputDataSourceInstance = new IoTHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue34 = datasourceValue5["properties"];
                                                        if (propertiesValue34 != null && propertiesValue34.Type != JTokenType.Null)
                                                        {
                                                            IoTHubStreamInputDataSourceProperties propertiesInstance8 = new IoTHubStreamInputDataSourceProperties();
                                                            ioTHubStreamInputDataSourceInstance.Properties = propertiesInstance8;
                                                            
                                                            JToken iotHubNamespaceValue = propertiesValue34["iotHubNamespace"];
                                                            if (iotHubNamespaceValue != null && iotHubNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string iotHubNamespaceInstance = ((string)iotHubNamespaceValue);
                                                                propertiesInstance8.IotHubNamespace = iotHubNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue2 = propertiesValue34["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue2 != null && sharedAccessPolicyNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance2 = ((string)sharedAccessPolicyNameValue2);
                                                                propertiesInstance8.SharedAccessPolicyName = sharedAccessPolicyNameInstance2;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue2 = propertiesValue34["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue2 != null && sharedAccessPolicyKeyValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance2 = ((string)sharedAccessPolicyKeyValue2);
                                                                propertiesInstance8.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance2;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue2 = propertiesValue34["consumerGroupName"];
                                                            if (consumerGroupNameValue2 != null && consumerGroupNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance2 = ((string)consumerGroupNameValue2);
                                                                propertiesInstance8.ConsumerGroupName = consumerGroupNameInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue9 = datasourceValue5["type"];
                                                        if (typeValue9 != null && typeValue9.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance9 = ((string)typeValue9);
                                                            ioTHubStreamInputDataSourceInstance.Type = typeInstance9;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = ioTHubStreamInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue3 = propertiesValue27["etag"];
                                                if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                                {
                                                    string etagInstance3 = ((string)etagValue3);
                                                    streamInputPropertiesInstance.Etag = etagInstance3;
                                                }
                                                
                                                JToken typeValue10 = propertiesValue27["type"];
                                                if (typeValue10 != null && typeValue10.Type != JTokenType.Null)
                                                {
                                                    string typeInstance10 = ((string)typeValue10);
                                                    streamInputPropertiesInstance.Type = typeInstance10;
                                                }
                                                
                                                JToken serializationValue5 = propertiesValue27["serialization"];
                                                if (serializationValue5 != null && serializationValue5.Type != JTokenType.Null)
                                                {
                                                    string typeName5 = ((string)serializationValue5["type"]);
                                                    if (typeName5 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance2 = new CsvSerialization();
                                                        
                                                        JToken propertiesValue35 = serializationValue5["properties"];
                                                        if (propertiesValue35 != null && propertiesValue35.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance9 = new CsvSerializationProperties();
                                                            csvSerializationInstance2.Properties = propertiesInstance9;
                                                            
                                                            JToken fieldDelimiterValue2 = propertiesValue35["fieldDelimiter"];
                                                            if (fieldDelimiterValue2 != null && fieldDelimiterValue2.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance2 = ((string)fieldDelimiterValue2);
                                                                propertiesInstance9.FieldDelimiter = fieldDelimiterInstance2;
                                                            }
                                                            
                                                            JToken encodingValue3 = propertiesValue35["encoding"];
                                                            if (encodingValue3 != null && encodingValue3.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance3 = ((string)encodingValue3);
                                                                propertiesInstance9.Encoding = encodingInstance3;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue11 = serializationValue5["type"];
                                                        if (typeValue11 != null && typeValue11.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance11 = ((string)typeValue11);
                                                            csvSerializationInstance2.Type = typeInstance11;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = csvSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance2 = new JsonSerialization();
                                                        
                                                        JToken propertiesValue36 = serializationValue5["properties"];
                                                        if (propertiesValue36 != null && propertiesValue36.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance10 = new JsonSerializationProperties();
                                                            jsonSerializationInstance2.Properties = propertiesInstance10;
                                                            
                                                            JToken encodingValue4 = propertiesValue36["encoding"];
                                                            if (encodingValue4 != null && encodingValue4.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance4 = ((string)encodingValue4);
                                                                propertiesInstance10.Encoding = encodingInstance4;
                                                            }
                                                            
                                                            JToken formatValue2 = propertiesValue36["format"];
                                                            if (formatValue2 != null && formatValue2.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance2 = ((string)formatValue2);
                                                                propertiesInstance10.Format = formatInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue12 = serializationValue5["type"];
                                                        if (typeValue12 != null && typeValue12.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance12 = ((string)typeValue12);
                                                            jsonSerializationInstance2.Type = typeInstance12;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = jsonSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance2 = new AvroSerialization();
                                                        
                                                        JToken propertiesValue37 = serializationValue5["properties"];
                                                        if (propertiesValue37 != null && propertiesValue37.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance11 = new AvroSerializationProperties();
                                                            avroSerializationInstance2.Properties = propertiesInstance11;
                                                        }
                                                        
                                                        JToken typeValue13 = serializationValue5["type"];
                                                        if (typeValue13 != null && typeValue13.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance13 = ((string)typeValue13);
                                                            avroSerializationInstance2.Type = typeInstance13;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = avroSerializationInstance2;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue5 = propertiesValue27["diagnostics"];
                                                if (diagnosticsValue5 != null && diagnosticsValue5.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance2 = new Diagnostics();
                                                    streamInputPropertiesInstance.Diagnostics = diagnosticsInstance2;
                                                    
                                                    JToken conditionsArray5 = diagnosticsValue5["conditions"];
                                                    if (conditionsArray5 != null && conditionsArray5.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue2 in ((JArray)conditionsArray5))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance2 = new DiagnosticCondition();
                                                            diagnosticsInstance2.Conditions.Add(diagnosticConditionInstance2);
                                                            
                                                            JToken sinceValue2 = conditionsValue2["since"];
                                                            if (sinceValue2 != null && sinceValue2.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance2 = ((DateTime)sinceValue2);
                                                                diagnosticConditionInstance2.Since = sinceInstance2;
                                                            }
                                                            
                                                            JToken codeValue2 = conditionsValue2["code"];
                                                            if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance2 = ((string)codeValue2);
                                                                diagnosticConditionInstance2.Code = codeInstance2;
                                                            }
                                                            
                                                            JToken messageValue2 = conditionsValue2["message"];
                                                            if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance2 = ((string)messageValue2);
                                                                diagnosticConditionInstance2.Message = messageInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = streamInputPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                                
                                JToken transformationValue2 = propertiesValue26["transformation"];
                                if (transformationValue2 != null && transformationValue2.Type != JTokenType.Null)
                                {
                                    Transformation transformationInstance = new Transformation();
                                    propertiesInstance.Transformation = transformationInstance;
                                    
                                    JToken nameValue4 = transformationValue2["name"];
                                    if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                    {
                                        string nameInstance4 = ((string)nameValue4);
                                        transformationInstance.Name = nameInstance4;
                                    }
                                    
                                    JToken propertiesValue38 = transformationValue2["properties"];
                                    if (propertiesValue38 != null && propertiesValue38.Type != JTokenType.Null)
                                    {
                                        TransformationProperties propertiesInstance12 = new TransformationProperties();
                                        transformationInstance.Properties = propertiesInstance12;
                                        
                                        JToken etagValue4 = propertiesValue38["etag"];
                                        if (etagValue4 != null && etagValue4.Type != JTokenType.Null)
                                        {
                                            string etagInstance4 = ((string)etagValue4);
                                            propertiesInstance12.Etag = etagInstance4;
                                        }
                                        
                                        JToken streamingUnitsValue = propertiesValue38["streamingUnits"];
                                        if (streamingUnitsValue != null && streamingUnitsValue.Type != JTokenType.Null)
                                        {
                                            int streamingUnitsInstance = ((int)streamingUnitsValue);
                                            propertiesInstance12.StreamingUnits = streamingUnitsInstance;
                                        }
                                        
                                        JToken queryValue = propertiesValue38["query"];
                                        if (queryValue != null && queryValue.Type != JTokenType.Null)
                                        {
                                            string queryInstance = ((string)queryValue);
                                            propertiesInstance12.Query = queryInstance;
                                        }
                                    }
                                }
                                
                                JToken outputsArray3 = propertiesValue26["outputs"];
                                if (outputsArray3 != null && outputsArray3.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Outputs = new List<Output>();
                                    foreach (JToken outputsValue in ((JArray)outputsArray3))
                                    {
                                        Output outputInstance = new Output();
                                        propertiesInstance.Outputs.Add(outputInstance);
                                        
                                        JToken nameValue5 = outputsValue["name"];
                                        if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                        {
                                            string nameInstance5 = ((string)nameValue5);
                                            outputInstance.Name = nameInstance5;
                                        }
                                        
                                        JToken propertiesValue39 = outputsValue["properties"];
                                        if (propertiesValue39 != null && propertiesValue39.Type != JTokenType.Null)
                                        {
                                            OutputProperties propertiesInstance13 = new OutputProperties();
                                            outputInstance.Properties = propertiesInstance13;
                                            
                                            JToken etagValue5 = propertiesValue39["etag"];
                                            if (etagValue5 != null && etagValue5.Type != JTokenType.Null)
                                            {
                                                string etagInstance5 = ((string)etagValue5);
                                                propertiesInstance13.Etag = etagInstance5;
                                            }
                                            
                                            JToken datasourceValue6 = propertiesValue39["datasource"];
                                            if (datasourceValue6 != null && datasourceValue6.Type != JTokenType.Null)
                                            {
                                                string typeName6 = ((string)datasourceValue6["type"]);
                                                if (typeName6 == "Microsoft.Storage/Blob")
                                                {
                                                    BlobOutputDataSource blobOutputDataSourceInstance = new BlobOutputDataSource();
                                                    
                                                    JToken propertiesValue40 = datasourceValue6["properties"];
                                                    if (propertiesValue40 != null && propertiesValue40.Type != JTokenType.Null)
                                                    {
                                                        BlobOutputDataSourceProperties propertiesInstance14 = new BlobOutputDataSourceProperties();
                                                        blobOutputDataSourceInstance.Properties = propertiesInstance14;
                                                        
                                                        JToken storageAccountsArray6 = propertiesValue40["storageAccounts"];
                                                        if (storageAccountsArray6 != null && storageAccountsArray6.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance14.StorageAccounts = new System.Collections.Generic.List<Microsoft.Azure.Management.StreamAnalytics.Models.StorageAccount>();
                                                            foreach (JToken storageAccountsValue3 in ((JArray)storageAccountsArray6))
                                                            {
                                                                StorageAccount storageAccountInstance3 = new StorageAccount();
                                                                propertiesInstance14.StorageAccounts.Add(storageAccountInstance3);
                                                                
                                                                JToken accountNameValue3 = storageAccountsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    storageAccountInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken accountKeyValue3 = storageAccountsValue3["accountKey"];
                                                                if (accountKeyValue3 != null && accountKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance3 = ((string)accountKeyValue3);
                                                                    storageAccountInstance3.AccountKey = accountKeyInstance3;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken containerValue3 = propertiesValue40["container"];
                                                        if (containerValue3 != null && containerValue3.Type != JTokenType.Null)
                                                        {
                                                            string containerInstance3 = ((string)containerValue3);
                                                            propertiesInstance14.Container = containerInstance3;
                                                        }
                                                        
                                                        JToken pathPatternValue3 = propertiesValue40["pathPattern"];
                                                        if (pathPatternValue3 != null && pathPatternValue3.Type != JTokenType.Null)
                                                        {
                                                            string pathPatternInstance3 = ((string)pathPatternValue3);
                                                            propertiesInstance14.PathPattern = pathPatternInstance3;
                                                        }
                                                        
                                                        JToken dateFormatValue3 = propertiesValue40["dateFormat"];
                                                        if (dateFormatValue3 != null && dateFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string dateFormatInstance3 = ((string)dateFormatValue3);
                                                            propertiesInstance14.DateFormat = dateFormatInstance3;
                                                        }
                                                        
                                                        JToken timeFormatValue3 = propertiesValue40["timeFormat"];
                                                        if (timeFormatValue3 != null && timeFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string timeFormatInstance3 = ((string)timeFormatValue3);
                                                            propertiesInstance14.TimeFormat = timeFormatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue14 = datasourceValue6["type"];
                                                    if (typeValue14 != null && typeValue14.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance14 = ((string)typeValue14);
                                                        blobOutputDataSourceInstance.Type = typeInstance14;
                                                    }
                                                    propertiesInstance13.DataSource = blobOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/Table")
                                                {
                                                    AzureTableOutputDataSource azureTableOutputDataSourceInstance = new AzureTableOutputDataSource();
                                                    
                                                    JToken propertiesValue41 = datasourceValue6["properties"];
                                                    if (propertiesValue41 != null && propertiesValue41.Type != JTokenType.Null)
                                                    {
                                                        AzureTableOutputDataSourceProperties propertiesInstance15 = new AzureTableOutputDataSourceProperties();
                                                        azureTableOutputDataSourceInstance.Properties = propertiesInstance15;
                                                        
                                                        JToken accountNameValue4 = propertiesValue41["accountName"];
                                                        if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountNameInstance4 = ((string)accountNameValue4);
                                                            propertiesInstance15.AccountName = accountNameInstance4;
                                                        }
                                                        
                                                        JToken accountKeyValue4 = propertiesValue41["accountKey"];
                                                        if (accountKeyValue4 != null && accountKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance4 = ((string)accountKeyValue4);
                                                            propertiesInstance15.AccountKey = accountKeyInstance4;
                                                        }
                                                        
                                                        JToken tableValue = propertiesValue41["table"];
                                                        if (tableValue != null && tableValue.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance = ((string)tableValue);
                                                            propertiesInstance15.Table = tableInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue = propertiesValue41["partitionKey"];
                                                        if (partitionKeyValue != null && partitionKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance = ((string)partitionKeyValue);
                                                            propertiesInstance15.PartitionKey = partitionKeyInstance;
                                                        }
                                                        
                                                        JToken rowKeyValue = propertiesValue41["rowKey"];
                                                        if (rowKeyValue != null && rowKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string rowKeyInstance = ((string)rowKeyValue);
                                                            propertiesInstance15.RowKey = rowKeyInstance;
                                                        }
                                                        
                                                        JToken columnsToRemoveArray2 = propertiesValue41["columnsToRemove"];
                                                        if (columnsToRemoveArray2 != null && columnsToRemoveArray2.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance15.ColumnsToRemove = new System.Collections.Generic.List<string>();
                                                            foreach (JToken columnsToRemoveValue in ((JArray)columnsToRemoveArray2))
                                                            {
                                                                propertiesInstance15.ColumnsToRemove.Add(((string)columnsToRemoveValue));
                                                            }
                                                        }
                                                        
                                                        JToken batchSizeValue = propertiesValue41["batchSize"];
                                                        if (batchSizeValue != null && batchSizeValue.Type != JTokenType.Null)
                                                        {
                                                            int batchSizeInstance = ((int)batchSizeValue);
                                                            propertiesInstance15.BatchSize = batchSizeInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue15 = datasourceValue6["type"];
                                                    if (typeValue15 != null && typeValue15.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance15 = ((string)typeValue15);
                                                        azureTableOutputDataSourceInstance.Type = typeInstance15;
                                                    }
                                                    propertiesInstance13.DataSource = azureTableOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/EventHub")
                                                {
                                                    EventHubOutputDataSource eventHubOutputDataSourceInstance = new EventHubOutputDataSource();
                                                    
                                                    JToken propertiesValue42 = datasourceValue6["properties"];
                                                    if (propertiesValue42 != null && propertiesValue42.Type != JTokenType.Null)
                                                    {
                                                        EventHubOutputDataSourceProperties propertiesInstance16 = new EventHubOutputDataSourceProperties();
                                                        eventHubOutputDataSourceInstance.Properties = propertiesInstance16;
                                                        
                                                        JToken serviceBusNamespaceValue2 = propertiesValue42["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue2 != null && serviceBusNamespaceValue2.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance2 = ((string)serviceBusNamespaceValue2);
                                                            propertiesInstance16.ServiceBusNamespace = serviceBusNamespaceInstance2;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue3 = propertiesValue42["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue3 != null && sharedAccessPolicyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance3 = ((string)sharedAccessPolicyNameValue3);
                                                            propertiesInstance16.SharedAccessPolicyName = sharedAccessPolicyNameInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue3 = propertiesValue42["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue3 != null && sharedAccessPolicyKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance3 = ((string)sharedAccessPolicyKeyValue3);
                                                            propertiesInstance16.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance3;
                                                        }
                                                        
                                                        JToken eventHubNameValue2 = propertiesValue42["eventHubName"];
                                                        if (eventHubNameValue2 != null && eventHubNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string eventHubNameInstance2 = ((string)eventHubNameValue2);
                                                            propertiesInstance16.EventHubName = eventHubNameInstance2;
                                                        }
                                                        
                                                        JToken partitionKeyValue2 = propertiesValue42["partitionKey"];
                                                        if (partitionKeyValue2 != null && partitionKeyValue2.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance2 = ((string)partitionKeyValue2);
                                                            propertiesInstance16.PartitionKey = partitionKeyInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue16 = datasourceValue6["type"];
                                                    if (typeValue16 != null && typeValue16.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance16 = ((string)typeValue16);
                                                        eventHubOutputDataSourceInstance.Type = typeInstance16;
                                                    }
                                                    propertiesInstance13.DataSource = eventHubOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Queue")
                                                {
                                                    ServiceBusQueueOutputDataSource serviceBusQueueOutputDataSourceInstance = new ServiceBusQueueOutputDataSource();
                                                    
                                                    JToken propertiesValue43 = datasourceValue6["properties"];
                                                    if (propertiesValue43 != null && propertiesValue43.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusQueueOutputDataSourceProperties propertiesInstance17 = new ServiceBusQueueOutputDataSourceProperties();
                                                        serviceBusQueueOutputDataSourceInstance.Properties = propertiesInstance17;
                                                        
                                                        JToken serviceBusNamespaceValue3 = propertiesValue43["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue3 != null && serviceBusNamespaceValue3.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance3 = ((string)serviceBusNamespaceValue3);
                                                            propertiesInstance17.ServiceBusNamespace = serviceBusNamespaceInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue4 = propertiesValue43["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue4 != null && sharedAccessPolicyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance4 = ((string)sharedAccessPolicyNameValue4);
                                                            propertiesInstance17.SharedAccessPolicyName = sharedAccessPolicyNameInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue4 = propertiesValue43["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue4 != null && sharedAccessPolicyKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance4 = ((string)sharedAccessPolicyKeyValue4);
                                                            propertiesInstance17.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance4;
                                                        }
                                                        
                                                        JToken queueNameValue = propertiesValue43["queueName"];
                                                        if (queueNameValue != null && queueNameValue.Type != JTokenType.Null)
                                                        {
                                                            string queueNameInstance = ((string)queueNameValue);
                                                            propertiesInstance17.QueueName = queueNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue17 = datasourceValue6["type"];
                                                    if (typeValue17 != null && typeValue17.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance17 = ((string)typeValue17);
                                                        serviceBusQueueOutputDataSourceInstance.Type = typeInstance17;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusQueueOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Topic")
                                                {
                                                    ServiceBusTopicOutputDataSource serviceBusTopicOutputDataSourceInstance = new ServiceBusTopicOutputDataSource();
                                                    
                                                    JToken propertiesValue44 = datasourceValue6["properties"];
                                                    if (propertiesValue44 != null && propertiesValue44.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusTopicOutputDataSourceProperties propertiesInstance18 = new ServiceBusTopicOutputDataSourceProperties();
                                                        serviceBusTopicOutputDataSourceInstance.Properties = propertiesInstance18;
                                                        
                                                        JToken serviceBusNamespaceValue4 = propertiesValue44["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue4 != null && serviceBusNamespaceValue4.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance4 = ((string)serviceBusNamespaceValue4);
                                                            propertiesInstance18.ServiceBusNamespace = serviceBusNamespaceInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue5 = propertiesValue44["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue5 != null && sharedAccessPolicyNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance5 = ((string)sharedAccessPolicyNameValue5);
                                                            propertiesInstance18.SharedAccessPolicyName = sharedAccessPolicyNameInstance5;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue5 = propertiesValue44["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue5 != null && sharedAccessPolicyKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance5 = ((string)sharedAccessPolicyKeyValue5);
                                                            propertiesInstance18.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance5;
                                                        }
                                                        
                                                        JToken topicNameValue = propertiesValue44["topicName"];
                                                        if (topicNameValue != null && topicNameValue.Type != JTokenType.Null)
                                                        {
                                                            string topicNameInstance = ((string)topicNameValue);
                                                            propertiesInstance18.TopicName = topicNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue18 = datasourceValue6["type"];
                                                    if (typeValue18 != null && typeValue18.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance18 = ((string)typeValue18);
                                                        serviceBusTopicOutputDataSourceInstance.Type = typeInstance18;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusTopicOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Sql/Server/Database")
                                                {
                                                    SqlAzureOutputDataSource sqlAzureOutputDataSourceInstance = new SqlAzureOutputDataSource();
                                                    
                                                    JToken propertiesValue45 = datasourceValue6["properties"];
                                                    if (propertiesValue45 != null && propertiesValue45.Type != JTokenType.Null)
                                                    {
                                                        SqlAzureOutputDataSourceProperties propertiesInstance19 = new SqlAzureOutputDataSourceProperties();
                                                        sqlAzureOutputDataSourceInstance.Properties = propertiesInstance19;
                                                        
                                                        JToken serverValue = propertiesValue45["server"];
                                                        if (serverValue != null && serverValue.Type != JTokenType.Null)
                                                        {
                                                            string serverInstance = ((string)serverValue);
                                                            propertiesInstance19.Server = serverInstance;
                                                        }
                                                        
                                                        JToken databaseValue = propertiesValue45["database"];
                                                        if (databaseValue != null && databaseValue.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance = ((string)databaseValue);
                                                            propertiesInstance19.Database = databaseInstance;
                                                        }
                                                        
                                                        JToken userValue = propertiesValue45["user"];
                                                        if (userValue != null && userValue.Type != JTokenType.Null)
                                                        {
                                                            string userInstance = ((string)userValue);
                                                            propertiesInstance19.User = userInstance;
                                                        }
                                                        
                                                        JToken passwordValue = propertiesValue45["password"];
                                                        if (passwordValue != null && passwordValue.Type != JTokenType.Null)
                                                        {
                                                            string passwordInstance = ((string)passwordValue);
                                                            propertiesInstance19.Password = passwordInstance;
                                                        }
                                                        
                                                        JToken tableValue2 = propertiesValue45["table"];
                                                        if (tableValue2 != null && tableValue2.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance2 = ((string)tableValue2);
                                                            propertiesInstance19.Table = tableInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue19 = datasourceValue6["type"];
                                                    if (typeValue19 != null && typeValue19.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance19 = ((string)typeValue19);
                                                        sqlAzureOutputDataSourceInstance.Type = typeInstance19;
                                                    }
                                                    propertiesInstance13.DataSource = sqlAzureOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/DocumentDB")
                                                {
                                                    DocumentDbOutputDataSource documentDbOutputDataSourceInstance = new DocumentDbOutputDataSource();
                                                    
                                                    JToken propertiesValue46 = datasourceValue6["properties"];
                                                    if (propertiesValue46 != null && propertiesValue46.Type != JTokenType.Null)
                                                    {
                                                        DocumentDbOutputDataSourceProperties propertiesInstance20 = new DocumentDbOutputDataSourceProperties();
                                                        documentDbOutputDataSourceInstance.Properties = propertiesInstance20;
                                                        
                                                        JToken accountIdValue = propertiesValue46["accountId"];
                                                        if (accountIdValue != null && accountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string accountIdInstance = ((string)accountIdValue);
                                                            propertiesInstance20.AccountId = accountIdInstance;
                                                        }
                                                        
                                                        JToken accountKeyValue5 = propertiesValue46["accountKey"];
                                                        if (accountKeyValue5 != null && accountKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance5 = ((string)accountKeyValue5);
                                                            propertiesInstance20.AccountKey = accountKeyInstance5;
                                                        }
                                                        
                                                        JToken databaseValue2 = propertiesValue46["database"];
                                                        if (databaseValue2 != null && databaseValue2.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance2 = ((string)databaseValue2);
                                                            propertiesInstance20.Database = databaseInstance2;
                                                        }
                                                        
                                                        JToken collectionNamePatternValue = propertiesValue46["collectionNamePattern"];
                                                        if (collectionNamePatternValue != null && collectionNamePatternValue.Type != JTokenType.Null)
                                                        {
                                                            string collectionNamePatternInstance = ((string)collectionNamePatternValue);
                                                            propertiesInstance20.CollectionNamePattern = collectionNamePatternInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue3 = propertiesValue46["partitionKey"];
                                                        if (partitionKeyValue3 != null && partitionKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance3 = ((string)partitionKeyValue3);
                                                            propertiesInstance20.PartitionKey = partitionKeyInstance3;
                                                        }
                                                        
                                                        JToken documentIdValue = propertiesValue46["documentId"];
                                                        if (documentIdValue != null && documentIdValue.Type != JTokenType.Null)
                                                        {
                                                            string documentIdInstance = ((string)documentIdValue);
                                                            propertiesInstance20.DocumentId = documentIdInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue20 = datasourceValue6["type"];
                                                    if (typeValue20 != null && typeValue20.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance20 = ((string)typeValue20);
                                                        documentDbOutputDataSourceInstance.Type = typeInstance20;
                                                    }
                                                    propertiesInstance13.DataSource = documentDbOutputDataSourceInstance;
                                                }
                                            }
                                            
                                            JToken serializationValue6 = propertiesValue39["serialization"];
                                            if (serializationValue6 != null && serializationValue6.Type != JTokenType.Null)
                                            {
                                                string typeName7 = ((string)serializationValue6["type"]);
                                                if (typeName7 == "Csv")
                                                {
                                                    CsvSerialization csvSerializationInstance3 = new CsvSerialization();
                                                    
                                                    JToken propertiesValue47 = serializationValue6["properties"];
                                                    if (propertiesValue47 != null && propertiesValue47.Type != JTokenType.Null)
                                                    {
                                                        CsvSerializationProperties propertiesInstance21 = new CsvSerializationProperties();
                                                        csvSerializationInstance3.Properties = propertiesInstance21;
                                                        
                                                        JToken fieldDelimiterValue3 = propertiesValue47["fieldDelimiter"];
                                                        if (fieldDelimiterValue3 != null && fieldDelimiterValue3.Type != JTokenType.Null)
                                                        {
                                                            string fieldDelimiterInstance3 = ((string)fieldDelimiterValue3);
                                                            propertiesInstance21.FieldDelimiter = fieldDelimiterInstance3;
                                                        }
                                                        
                                                        JToken encodingValue5 = propertiesValue47["encoding"];
                                                        if (encodingValue5 != null && encodingValue5.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance5 = ((string)encodingValue5);
                                                            propertiesInstance21.Encoding = encodingInstance5;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue21 = serializationValue6["type"];
                                                    if (typeValue21 != null && typeValue21.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance21 = ((string)typeValue21);
                                                        csvSerializationInstance3.Type = typeInstance21;
                                                    }
                                                    propertiesInstance13.Serialization = csvSerializationInstance3;
                                                }
                                                if (typeName7 == "Json")
                                                {
                                                    JsonSerialization jsonSerializationInstance3 = new JsonSerialization();
                                                    
                                                    JToken propertiesValue48 = serializationValue6["properties"];
                                                    if (propertiesValue48 != null && propertiesValue48.Type != JTokenType.Null)
                                                    {
                                                        JsonSerializationProperties propertiesInstance22 = new JsonSerializationProperties();
                                                        jsonSerializationInstance3.Properties = propertiesInstance22;
                                                        
                                                        JToken encodingValue6 = propertiesValue48["encoding"];
                                                        if (encodingValue6 != null && encodingValue6.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance6 = ((string)encodingValue6);
                                                            propertiesInstance22.Encoding = encodingInstance6;
                                                        }
                                                        
                                                        JToken formatValue3 = propertiesValue48["format"];
                                                        if (formatValue3 != null && formatValue3.Type != JTokenType.Null)
                                                        {
                                                            string formatInstance3 = ((string)formatValue3);
                                                            propertiesInstance22.Format = formatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue22 = serializationValue6["type"];
                                                    if (typeValue22 != null && typeValue22.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance22 = ((string)typeValue22);
                                                        jsonSerializationInstance3.Type = typeInstance22;
                                                    }
                                                    propertiesInstance13.Serialization = jsonSerializationInstance3;
                                                }
                                                if (typeName7 == "Avro")
                                                {
                                                    AvroSerialization avroSerializationInstance3 = new AvroSerialization();
                                                    
                                                    JToken propertiesValue49 = serializationValue6["properties"];
                                                    if (propertiesValue49 != null && propertiesValue49.Type != JTokenType.Null)
                                                    {
                                                        AvroSerializationProperties propertiesInstance23 = new AvroSerializationProperties();
                                                        avroSerializationInstance3.Properties = propertiesInstance23;
                                                    }
                                                    
                                                    JToken typeValue23 = serializationValue6["type"];
                                                    if (typeValue23 != null && typeValue23.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance23 = ((string)typeValue23);
                                                        avroSerializationInstance3.Type = typeInstance23;
                                                    }
                                                    propertiesInstance13.Serialization = avroSerializationInstance3;
                                                }
                                            }
                                            
                                            JToken diagnosticsValue6 = propertiesValue39["diagnostics"];
                                            if (diagnosticsValue6 != null && diagnosticsValue6.Type != JTokenType.Null)
                                            {
                                                Diagnostics diagnosticsInstance3 = new Diagnostics();
                                                propertiesInstance13.Diagnostics = diagnosticsInstance3;
                                                
                                                JToken conditionsArray6 = diagnosticsValue6["conditions"];
                                                if (conditionsArray6 != null && conditionsArray6.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken conditionsValue3 in ((JArray)conditionsArray6))
                                                    {
                                                        DiagnosticCondition diagnosticConditionInstance3 = new DiagnosticCondition();
                                                        diagnosticsInstance3.Conditions.Add(diagnosticConditionInstance3);
                                                        
                                                        JToken sinceValue3 = conditionsValue3["since"];
                                                        if (sinceValue3 != null && sinceValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime sinceInstance3 = ((DateTime)sinceValue3);
                                                            diagnosticConditionInstance3.Since = sinceInstance3;
                                                        }
                                                        
                                                        JToken codeValue3 = conditionsValue3["code"];
                                                        if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                        {
                                                            string codeInstance3 = ((string)codeValue3);
                                                            diagnosticConditionInstance3.Code = codeInstance3;
                                                        }
                                                        
                                                        JToken messageValue3 = conditionsValue3["message"];
                                                        if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                        {
                                                            string messageInstance3 = ((string)messageValue3);
                                                            diagnosticConditionInstance3.Message = messageInstance3;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                
                                JToken functionsArray2 = propertiesValue26["functions"];
                                if (functionsArray2 != null && functionsArray2.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Functions = new List<Function>();
                                    foreach (JToken functionsValue in ((JArray)functionsArray2))
                                    {
                                        Function functionInstance = new Function();
                                        propertiesInstance.Functions.Add(functionInstance);
                                        
                                        JToken nameValue6 = functionsValue["name"];
                                        if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                        {
                                            string nameInstance6 = ((string)nameValue6);
                                            functionInstance.Name = nameInstance6;
                                        }
                                        
                                        JToken propertiesValue50 = functionsValue["properties"];
                                        if (propertiesValue50 != null && propertiesValue50.Type != JTokenType.Null)
                                        {
                                            string typeName8 = ((string)propertiesValue50["type"]);
                                            if (typeName8 == "Scalar")
                                            {
                                                ScalarFunctionProperties scalarFunctionPropertiesInstance = new ScalarFunctionProperties();
                                                
                                                JToken propertiesValue51 = propertiesValue50["properties"];
                                                if (propertiesValue51 != null && propertiesValue51.Type != JTokenType.Null)
                                                {
                                                    ScalarFunctionConfiguration propertiesInstance24 = new ScalarFunctionConfiguration();
                                                    scalarFunctionPropertiesInstance.Properties = propertiesInstance24;
                                                    
                                                    JToken inputsArray4 = propertiesValue51["inputs"];
                                                    if (inputsArray4 != null && inputsArray4.Type != JTokenType.Null)
                                                    {
                                                        propertiesInstance24.Inputs = new List<FunctionInput>();
                                                        foreach (JToken inputsValue3 in ((JArray)inputsArray4))
                                                        {
                                                            FunctionInput functionInputInstance = new FunctionInput();
                                                            propertiesInstance24.Inputs.Add(functionInputInstance);
                                                            
                                                            JToken dataTypeValue = inputsValue3["dataType"];
                                                            if (dataTypeValue != null && dataTypeValue.Type != JTokenType.Null)
                                                            {
                                                                string dataTypeInstance = ((string)dataTypeValue);
                                                                functionInputInstance.DataType = dataTypeInstance;
                                                            }
                                                            
                                                            JToken isConfigurationParameterValue = inputsValue3["isConfigurationParameter"];
                                                            if (isConfigurationParameterValue != null && isConfigurationParameterValue.Type != JTokenType.Null)
                                                            {
                                                                bool isConfigurationParameterInstance = ((bool)isConfigurationParameterValue);
                                                                functionInputInstance.IsConfigurationParameter = isConfigurationParameterInstance;
                                                            }
                                                        }
                                                    }
                                                    
                                                    JToken outputValue3 = propertiesValue51["output"];
                                                    if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                    {
                                                        FunctionOutput outputInstance2 = new FunctionOutput();
                                                        propertiesInstance24.Output = outputInstance2;
                                                        
                                                        JToken dataTypeValue2 = outputValue3["dataType"];
                                                        if (dataTypeValue2 != null && dataTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string dataTypeInstance2 = ((string)dataTypeValue2);
                                                            outputInstance2.DataType = dataTypeInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken bindingValue2 = propertiesValue51["binding"];
                                                    if (bindingValue2 != null && bindingValue2.Type != JTokenType.Null)
                                                    {
                                                        string typeName9 = ((string)bindingValue2["type"]);
                                                        if (typeName9 == "Microsoft.MachineLearning/WebService")
                                                        {
                                                            AzureMachineLearningWebServiceFunctionBinding azureMachineLearningWebServiceFunctionBindingInstance = new AzureMachineLearningWebServiceFunctionBinding();
                                                            
                                                            JToken propertiesValue52 = bindingValue2["properties"];
                                                            if (propertiesValue52 != null && propertiesValue52.Type != JTokenType.Null)
                                                            {
                                                                AzureMachineLearningWebServiceFunctionBindingProperties propertiesInstance25 = new AzureMachineLearningWebServiceFunctionBindingProperties();
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Properties = propertiesInstance25;
                                                                
                                                                JToken endpointValue = propertiesValue52["endpoint"];
                                                                if (endpointValue != null && endpointValue.Type != JTokenType.Null)
                                                                {
                                                                    string endpointInstance = ((string)endpointValue);
                                                                    propertiesInstance25.Endpoint = endpointInstance;
                                                                }
                                                                
                                                                JToken apiKeyValue = propertiesValue52["apiKey"];
                                                                if (apiKeyValue != null && apiKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string apiKeyInstance = ((string)apiKeyValue);
                                                                    propertiesInstance25.ApiKey = apiKeyInstance;
                                                                }
                                                                
                                                                JToken inputsValue4 = propertiesValue52["inputs"];
                                                                if (inputsValue4 != null && inputsValue4.Type != JTokenType.Null)
                                                                {
                                                                    AzureMachineLearningWebServiceInputs inputsInstance = new AzureMachineLearningWebServiceInputs();
                                                                    propertiesInstance25.Inputs = inputsInstance;
                                                                    
                                                                    JToken nameValue7 = inputsValue4["name"];
                                                                    if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string nameInstance7 = ((string)nameValue7);
                                                                        inputsInstance.Name = nameInstance7;
                                                                    }
                                                                    
                                                                    JToken columnNamesArray2 = inputsValue4["columnNames"];
                                                                    if (columnNamesArray2 != null && columnNamesArray2.Type != JTokenType.Null)
                                                                    {
                                                                        inputsInstance.ColumnNames = new List<AzureMachineLearningWebServiceInputColumn>();
                                                                        foreach (JToken columnNamesValue in ((JArray)columnNamesArray2))
                                                                        {
                                                                            AzureMachineLearningWebServiceInputColumn azureMachineLearningWebServiceInputColumnInstance = new AzureMachineLearningWebServiceInputColumn();
                                                                            inputsInstance.ColumnNames.Add(azureMachineLearningWebServiceInputColumnInstance);
                                                                            
                                                                            JToken nameValue8 = columnNamesValue["name"];
                                                                            if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string nameInstance8 = ((string)nameValue8);
                                                                                azureMachineLearningWebServiceInputColumnInstance.Name = nameInstance8;
                                                                            }
                                                                            
                                                                            JToken dataTypeValue3 = columnNamesValue["dataType"];
                                                                            if (dataTypeValue3 != null && dataTypeValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string dataTypeInstance3 = ((string)dataTypeValue3);
                                                                                azureMachineLearningWebServiceInputColumnInstance.DataType = dataTypeInstance3;
                                                                            }
                                                                            
                                                                            JToken mapToValue = columnNamesValue["mapTo"];
                                                                            if (mapToValue != null && mapToValue.Type != JTokenType.Null)
                                                                            {
                                                                                int mapToInstance = ((int)mapToValue);
                                                                                azureMachineLearningWebServiceInputColumnInstance.MapTo = mapToInstance;
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken outputsArray4 = propertiesValue52["outputs"];
                                                                if (outputsArray4 != null && outputsArray4.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance25.Outputs = new List<AzureMachineLearningWebServiceOutputColumn>();
                                                                    foreach (JToken outputsValue2 in ((JArray)outputsArray4))
                                                                    {
                                                                        AzureMachineLearningWebServiceOutputColumn azureMachineLearningWebServiceOutputColumnInstance = new AzureMachineLearningWebServiceOutputColumn();
                                                                        propertiesInstance25.Outputs.Add(azureMachineLearningWebServiceOutputColumnInstance);
                                                                        
                                                                        JToken nameValue9 = outputsValue2["name"];
                                                                        if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance9 = ((string)nameValue9);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.Name = nameInstance9;
                                                                        }
                                                                        
                                                                        JToken dataTypeValue4 = outputsValue2["dataType"];
                                                                        if (dataTypeValue4 != null && dataTypeValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string dataTypeInstance4 = ((string)dataTypeValue4);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.DataType = dataTypeInstance4;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken batchSizeValue2 = propertiesValue52["batchSize"];
                                                                if (batchSizeValue2 != null && batchSizeValue2.Type != JTokenType.Null)
                                                                {
                                                                    int batchSizeInstance2 = ((int)batchSizeValue2);
                                                                    propertiesInstance25.BatchSize = batchSizeInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue24 = bindingValue2["type"];
                                                            if (typeValue24 != null && typeValue24.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance24 = ((string)typeValue24);
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Type = typeInstance24;
                                                            }
                                                            propertiesInstance24.Binding = azureMachineLearningWebServiceFunctionBindingInstance;
                                                        }
                                                        if (typeName9 == "Microsoft.StreamAnalytics/JavascriptUdf")
                                                        {
                                                            JavaScriptFunctionBinding javaScriptFunctionBindingInstance = new JavaScriptFunctionBinding();
                                                            
                                                            JToken propertiesValue53 = bindingValue2["properties"];
                                                            if (propertiesValue53 != null && propertiesValue53.Type != JTokenType.Null)
                                                            {
                                                                JavaScriptFunctionBindingProperties propertiesInstance26 = new JavaScriptFunctionBindingProperties();
                                                                javaScriptFunctionBindingInstance.Properties = propertiesInstance26;
                                                                
                                                                JToken scriptValue = propertiesValue53["script"];
                                                                if (scriptValue != null && scriptValue.Type != JTokenType.Null)
                                                                {
                                                                    string scriptInstance = ((string)scriptValue);
                                                                    propertiesInstance26.Script = scriptInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue25 = bindingValue2["type"];
                                                            if (typeValue25 != null && typeValue25.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance25 = ((string)typeValue25);
                                                                javaScriptFunctionBindingInstance.Type = typeInstance25;
                                                            }
                                                            propertiesInstance24.Binding = javaScriptFunctionBindingInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken etagValue6 = propertiesValue50["etag"];
                                                if (etagValue6 != null && etagValue6.Type != JTokenType.Null)
                                                {
                                                    string etagInstance6 = ((string)etagValue6);
                                                    scalarFunctionPropertiesInstance.Etag = etagInstance6;
                                                }
                                                
                                                JToken typeValue26 = propertiesValue50["type"];
                                                if (typeValue26 != null && typeValue26.Type != JTokenType.Null)
                                                {
                                                    string typeInstance26 = ((string)typeValue26);
                                                    scalarFunctionPropertiesInstance.Type = typeInstance26;
                                                }
                                                functionInstance.Properties = scalarFunctionPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = DateTime.Parse(httpResponse.Headers.GetValues("Date").FirstOrDefault(), CultureInfo.InvariantCulture, DateTimeStyles.AdjustToUniversal).ToLocalTime();
                    }
                    if (httpResponse.Headers.Contains("ETag"))
                    {
                        result.Job.Properties.Etag = httpResponse.Headers.GetValues("ETag").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Create or update a stream analytics job. The raw json content will
        /// be used.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to create or update a stream
        /// analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the create stream analytics job operation.
        /// </returns>
        public async Task<JobCreateOrUpdateResponse> CreateOrUpdateWithRawJsonContentAsync(string resourceGroupName, string jobName, JobCreateOrUpdateWithRawJsonContentParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Content == null)
            {
                throw new ArgumentNullException("parameters.Content");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "CreateOrUpdateWithRawJsonContentAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs/";
            url = url + Uri.EscapeDataString(jobName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("If-None-Match", parameters.IfNoneMatch);
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = parameters.Content;
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobCreateOrUpdateResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobCreateOrUpdateResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                jobInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                jobInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                jobInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                jobInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                jobInstance.Tags = new Dictionary<string, string>();
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property.Name);
                                    string tagsValue = ((string)property.Value);
                                    jobInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken skuValue = propertiesValue["sku"];
                                if (skuValue != null && skuValue.Type != JTokenType.Null)
                                {
                                    Sku skuInstance = new Sku();
                                    propertiesInstance.Sku = skuInstance;
                                    
                                    JToken nameValue2 = skuValue["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        string nameInstance2 = ((string)nameValue2);
                                        skuInstance.Name = nameInstance2;
                                    }
                                }
                                
                                JToken eventsOutOfOrderPolicyValue = propertiesValue["eventsOutOfOrderPolicy"];
                                if (eventsOutOfOrderPolicyValue != null && eventsOutOfOrderPolicyValue.Type != JTokenType.Null)
                                {
                                    string eventsOutOfOrderPolicyInstance = ((string)eventsOutOfOrderPolicyValue);
                                    propertiesInstance.EventsOutOfOrderPolicy = eventsOutOfOrderPolicyInstance;
                                }
                                
                                JToken outputErrorPolicyValue = propertiesValue["outputErrorPolicy"];
                                if (outputErrorPolicyValue != null && outputErrorPolicyValue.Type != JTokenType.Null)
                                {
                                    string outputErrorPolicyInstance = ((string)outputErrorPolicyValue);
                                    propertiesInstance.OutputErrorPolicy = outputErrorPolicyInstance;
                                }
                                
                                JToken eventsOutOfOrderMaxDelayInSecondsValue = propertiesValue["eventsOutOfOrderMaxDelayInSeconds"];
                                if (eventsOutOfOrderMaxDelayInSecondsValue != null && eventsOutOfOrderMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsOutOfOrderMaxDelayInSecondsInstance = ((int)eventsOutOfOrderMaxDelayInSecondsValue);
                                    propertiesInstance.EventsOutOfOrderMaxDelayInSeconds = eventsOutOfOrderMaxDelayInSecondsInstance;
                                }
                                
                                JToken eventsLateArrivalMaxDelayInSecondsValue = propertiesValue["eventsLateArrivalMaxDelayInSeconds"];
                                if (eventsLateArrivalMaxDelayInSecondsValue != null && eventsLateArrivalMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsLateArrivalMaxDelayInSecondsInstance = ((int)eventsLateArrivalMaxDelayInSecondsValue);
                                    propertiesInstance.EventsLateArrivalMaxDelayInSeconds = eventsLateArrivalMaxDelayInSecondsInstance;
                                }
                                
                                JToken dataLocaleValue = propertiesValue["dataLocale"];
                                if (dataLocaleValue != null && dataLocaleValue.Type != JTokenType.Null)
                                {
                                    string dataLocaleInstance = ((string)dataLocaleValue);
                                    propertiesInstance.DataLocale = dataLocaleInstance;
                                }
                                
                                JToken outputStartModeValue = propertiesValue["outputStartMode"];
                                if (outputStartModeValue != null && outputStartModeValue.Type != JTokenType.Null)
                                {
                                    string outputStartModeInstance = ((string)outputStartModeValue);
                                    propertiesInstance.OutputStartMode = outputStartModeInstance;
                                }
                                
                                JToken outputStartTimeValue = propertiesValue["outputStartTime"];
                                if (outputStartTimeValue != null && outputStartTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime outputStartTimeInstance = ((DateTime)outputStartTimeValue);
                                    propertiesInstance.OutputStartTime = outputStartTimeInstance;
                                }
                                
                                JToken lastOutputEventTimeValue = propertiesValue["lastOutputEventTime"];
                                if (lastOutputEventTimeValue != null && lastOutputEventTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastOutputEventTimeInstance = ((DateTime)lastOutputEventTimeValue);
                                    propertiesInstance.LastOutputEventTime = lastOutputEventTimeInstance;
                                }
                                
                                JToken jobIdValue = propertiesValue["jobId"];
                                if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                {
                                    string jobIdInstance = ((string)jobIdValue);
                                    propertiesInstance.JobId = jobIdInstance;
                                }
                                
                                JToken etagValue = propertiesValue["etag"];
                                if (etagValue != null && etagValue.Type != JTokenType.Null)
                                {
                                    string etagInstance = ((string)etagValue);
                                    propertiesInstance.Etag = etagInstance;
                                }
                                
                                JToken provisioningStateValue = propertiesValue["provisioningState"];
                                if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance = ((string)provisioningStateValue);
                                    propertiesInstance.ProvisioningState = provisioningStateInstance;
                                }
                                
                                JToken jobStateValue = propertiesValue["jobState"];
                                if (jobStateValue != null && jobStateValue.Type != JTokenType.Null)
                                {
                                    string jobStateInstance = ((string)jobStateValue);
                                    propertiesInstance.JobState = jobStateInstance;
                                }
                                
                                JToken createdDateValue = propertiesValue["createdDate"];
                                if (createdDateValue != null && createdDateValue.Type != JTokenType.Null)
                                {
                                    DateTime createdDateInstance = ((DateTime)createdDateValue);
                                    propertiesInstance.CreatedDate = createdDateInstance;
                                }
                                
                                JToken inputsArray = propertiesValue["inputs"];
                                if (inputsArray != null && inputsArray.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Inputs = new List<Input>();
                                    foreach (JToken inputsValue in ((JArray)inputsArray))
                                    {
                                        Input inputInstance = new Input();
                                        propertiesInstance.Inputs.Add(inputInstance);
                                        
                                        JToken nameValue3 = inputsValue["name"];
                                        if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                        {
                                            string nameInstance3 = ((string)nameValue3);
                                            inputInstance.Name = nameInstance3;
                                        }
                                        
                                        JToken propertiesValue2 = inputsValue["properties"];
                                        if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)propertiesValue2["type"]);
                                            if (typeName == "Reference")
                                            {
                                                ReferenceInputProperties referenceInputPropertiesInstance = new ReferenceInputProperties();
                                                
                                                JToken datasourceValue = propertiesValue2["datasource"];
                                                if (datasourceValue != null && datasourceValue.Type != JTokenType.Null)
                                                {
                                                    string typeName2 = ((string)datasourceValue["type"]);
                                                    if (typeName2 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobReferenceInputDataSource blobReferenceInputDataSourceInstance = new BlobReferenceInputDataSource();
                                                        
                                                        JToken propertiesValue3 = datasourceValue["properties"];
                                                        if (propertiesValue3 != null && propertiesValue3.Type != JTokenType.Null)
                                                        {
                                                            BlobReferenceInputDataSourceProperties propertiesInstance2 = new BlobReferenceInputDataSourceProperties();
                                                            blobReferenceInputDataSourceInstance.Properties = propertiesInstance2;
                                                            
                                                            JToken storageAccountsArray = propertiesValue3["storageAccounts"];
                                                            if (storageAccountsArray != null && storageAccountsArray.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance2.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue in ((JArray)storageAccountsArray))
                                                                {
                                                                    StorageAccount storageAccountInstance = new StorageAccount();
                                                                    propertiesInstance2.StorageAccounts.Add(storageAccountInstance);
                                                                    
                                                                    JToken accountNameValue = storageAccountsValue["accountName"];
                                                                    if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance = ((string)accountNameValue);
                                                                        storageAccountInstance.AccountName = accountNameInstance;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue = storageAccountsValue["accountKey"];
                                                                    if (accountKeyValue != null && accountKeyValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance = ((string)accountKeyValue);
                                                                        storageAccountInstance.AccountKey = accountKeyInstance;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue = propertiesValue3["container"];
                                                            if (containerValue != null && containerValue.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance = ((string)containerValue);
                                                                propertiesInstance2.Container = containerInstance;
                                                            }
                                                            
                                                            JToken pathPatternValue = propertiesValue3["pathPattern"];
                                                            if (pathPatternValue != null && pathPatternValue.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance = ((string)pathPatternValue);
                                                                propertiesInstance2.PathPattern = pathPatternInstance;
                                                            }
                                                            
                                                            JToken dateFormatValue = propertiesValue3["dateFormat"];
                                                            if (dateFormatValue != null && dateFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance = ((string)dateFormatValue);
                                                                propertiesInstance2.DateFormat = dateFormatInstance;
                                                            }
                                                            
                                                            JToken timeFormatValue = propertiesValue3["timeFormat"];
                                                            if (timeFormatValue != null && timeFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance = ((string)timeFormatValue);
                                                                propertiesInstance2.TimeFormat = timeFormatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue2 = datasourceValue["type"];
                                                        if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance2 = ((string)typeValue2);
                                                            blobReferenceInputDataSourceInstance.Type = typeInstance2;
                                                        }
                                                        referenceInputPropertiesInstance.DataSource = blobReferenceInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue2 = propertiesValue2["etag"];
                                                if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                {
                                                    string etagInstance2 = ((string)etagValue2);
                                                    referenceInputPropertiesInstance.Etag = etagInstance2;
                                                }
                                                
                                                JToken typeValue3 = propertiesValue2["type"];
                                                if (typeValue3 != null && typeValue3.Type != JTokenType.Null)
                                                {
                                                    string typeInstance3 = ((string)typeValue3);
                                                    referenceInputPropertiesInstance.Type = typeInstance3;
                                                }
                                                
                                                JToken serializationValue = propertiesValue2["serialization"];
                                                if (serializationValue != null && serializationValue.Type != JTokenType.Null)
                                                {
                                                    string typeName3 = ((string)serializationValue["type"]);
                                                    if (typeName3 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance = new CsvSerialization();
                                                        
                                                        JToken propertiesValue4 = serializationValue["properties"];
                                                        if (propertiesValue4 != null && propertiesValue4.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance3 = new CsvSerializationProperties();
                                                            csvSerializationInstance.Properties = propertiesInstance3;
                                                            
                                                            JToken fieldDelimiterValue = propertiesValue4["fieldDelimiter"];
                                                            if (fieldDelimiterValue != null && fieldDelimiterValue.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance = ((string)fieldDelimiterValue);
                                                                propertiesInstance3.FieldDelimiter = fieldDelimiterInstance;
                                                            }
                                                            
                                                            JToken encodingValue = propertiesValue4["encoding"];
                                                            if (encodingValue != null && encodingValue.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance = ((string)encodingValue);
                                                                propertiesInstance3.Encoding = encodingInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue4 = serializationValue["type"];
                                                        if (typeValue4 != null && typeValue4.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance4 = ((string)typeValue4);
                                                            csvSerializationInstance.Type = typeInstance4;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = csvSerializationInstance;
                                                    }
                                                    if (typeName3 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance = new JsonSerialization();
                                                        
                                                        JToken propertiesValue5 = serializationValue["properties"];
                                                        if (propertiesValue5 != null && propertiesValue5.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance4 = new JsonSerializationProperties();
                                                            jsonSerializationInstance.Properties = propertiesInstance4;
                                                            
                                                            JToken encodingValue2 = propertiesValue5["encoding"];
                                                            if (encodingValue2 != null && encodingValue2.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance2 = ((string)encodingValue2);
                                                                propertiesInstance4.Encoding = encodingInstance2;
                                                            }
                                                            
                                                            JToken formatValue = propertiesValue5["format"];
                                                            if (formatValue != null && formatValue.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance = ((string)formatValue);
                                                                propertiesInstance4.Format = formatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue5 = serializationValue["type"];
                                                        if (typeValue5 != null && typeValue5.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance5 = ((string)typeValue5);
                                                            jsonSerializationInstance.Type = typeInstance5;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = jsonSerializationInstance;
                                                    }
                                                    if (typeName3 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance = new AvroSerialization();
                                                        
                                                        JToken propertiesValue6 = serializationValue["properties"];
                                                        if (propertiesValue6 != null && propertiesValue6.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance5 = new AvroSerializationProperties();
                                                            avroSerializationInstance.Properties = propertiesInstance5;
                                                        }
                                                        
                                                        JToken typeValue6 = serializationValue["type"];
                                                        if (typeValue6 != null && typeValue6.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance6 = ((string)typeValue6);
                                                            avroSerializationInstance.Type = typeInstance6;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = avroSerializationInstance;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue = propertiesValue2["diagnostics"];
                                                if (diagnosticsValue != null && diagnosticsValue.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance = new Diagnostics();
                                                    referenceInputPropertiesInstance.Diagnostics = diagnosticsInstance;
                                                    
                                                    JToken conditionsArray = diagnosticsValue["conditions"];
                                                    if (conditionsArray != null && conditionsArray.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue in ((JArray)conditionsArray))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance = new DiagnosticCondition();
                                                            diagnosticsInstance.Conditions.Add(diagnosticConditionInstance);
                                                            
                                                            JToken sinceValue = conditionsValue["since"];
                                                            if (sinceValue != null && sinceValue.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance = ((DateTime)sinceValue);
                                                                diagnosticConditionInstance.Since = sinceInstance;
                                                            }
                                                            
                                                            JToken codeValue = conditionsValue["code"];
                                                            if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance = ((string)codeValue);
                                                                diagnosticConditionInstance.Code = codeInstance;
                                                            }
                                                            
                                                            JToken messageValue = conditionsValue["message"];
                                                            if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance = ((string)messageValue);
                                                                diagnosticConditionInstance.Message = messageInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = referenceInputPropertiesInstance;
                                            }
                                            if (typeName == "Stream")
                                            {
                                                StreamInputProperties streamInputPropertiesInstance = new StreamInputProperties();
                                                
                                                JToken datasourceValue2 = propertiesValue2["datasource"];
                                                if (datasourceValue2 != null && datasourceValue2.Type != JTokenType.Null)
                                                {
                                                    string typeName4 = ((string)datasourceValue2["type"]);
                                                    if (typeName4 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobStreamInputDataSource blobStreamInputDataSourceInstance = new BlobStreamInputDataSource();
                                                        
                                                        JToken propertiesValue7 = datasourceValue2["properties"];
                                                        if (propertiesValue7 != null && propertiesValue7.Type != JTokenType.Null)
                                                        {
                                                            BlobStreamInputDataSourceProperties propertiesInstance6 = new BlobStreamInputDataSourceProperties();
                                                            blobStreamInputDataSourceInstance.Properties = propertiesInstance6;
                                                            
                                                            JToken storageAccountsArray2 = propertiesValue7["storageAccounts"];
                                                            if (storageAccountsArray2 != null && storageAccountsArray2.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance6.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue2 in ((JArray)storageAccountsArray2))
                                                                {
                                                                    StorageAccount storageAccountInstance2 = new StorageAccount();
                                                                    propertiesInstance6.StorageAccounts.Add(storageAccountInstance2);
                                                                    
                                                                    JToken accountNameValue2 = storageAccountsValue2["accountName"];
                                                                    if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance2 = ((string)accountNameValue2);
                                                                        storageAccountInstance2.AccountName = accountNameInstance2;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue2 = storageAccountsValue2["accountKey"];
                                                                    if (accountKeyValue2 != null && accountKeyValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance2 = ((string)accountKeyValue2);
                                                                        storageAccountInstance2.AccountKey = accountKeyInstance2;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue2 = propertiesValue7["container"];
                                                            if (containerValue2 != null && containerValue2.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance2 = ((string)containerValue2);
                                                                propertiesInstance6.Container = containerInstance2;
                                                            }
                                                            
                                                            JToken pathPatternValue2 = propertiesValue7["pathPattern"];
                                                            if (pathPatternValue2 != null && pathPatternValue2.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance2 = ((string)pathPatternValue2);
                                                                propertiesInstance6.PathPattern = pathPatternInstance2;
                                                            }
                                                            
                                                            JToken dateFormatValue2 = propertiesValue7["dateFormat"];
                                                            if (dateFormatValue2 != null && dateFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance2 = ((string)dateFormatValue2);
                                                                propertiesInstance6.DateFormat = dateFormatInstance2;
                                                            }
                                                            
                                                            JToken timeFormatValue2 = propertiesValue7["timeFormat"];
                                                            if (timeFormatValue2 != null && timeFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance2 = ((string)timeFormatValue2);
                                                                propertiesInstance6.TimeFormat = timeFormatInstance2;
                                                            }
                                                            
                                                            JToken sourcePartitionCountValue = propertiesValue7["sourcePartitionCount"];
                                                            if (sourcePartitionCountValue != null && sourcePartitionCountValue.Type != JTokenType.Null)
                                                            {
                                                                int sourcePartitionCountInstance = ((int)sourcePartitionCountValue);
                                                                propertiesInstance6.SourcePartitionCount = sourcePartitionCountInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue7 = datasourceValue2["type"];
                                                        if (typeValue7 != null && typeValue7.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance7 = ((string)typeValue7);
                                                            blobStreamInputDataSourceInstance.Type = typeInstance7;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = blobStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.ServiceBus/EventHub")
                                                    {
                                                        EventHubStreamInputDataSource eventHubStreamInputDataSourceInstance = new EventHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue8 = datasourceValue2["properties"];
                                                        if (propertiesValue8 != null && propertiesValue8.Type != JTokenType.Null)
                                                        {
                                                            EventHubStreamInputDataSourceProperties propertiesInstance7 = new EventHubStreamInputDataSourceProperties();
                                                            eventHubStreamInputDataSourceInstance.Properties = propertiesInstance7;
                                                            
                                                            JToken serviceBusNamespaceValue = propertiesValue8["serviceBusNamespace"];
                                                            if (serviceBusNamespaceValue != null && serviceBusNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string serviceBusNamespaceInstance = ((string)serviceBusNamespaceValue);
                                                                propertiesInstance7.ServiceBusNamespace = serviceBusNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue = propertiesValue8["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue != null && sharedAccessPolicyNameValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance = ((string)sharedAccessPolicyNameValue);
                                                                propertiesInstance7.SharedAccessPolicyName = sharedAccessPolicyNameInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue = propertiesValue8["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue != null && sharedAccessPolicyKeyValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance = ((string)sharedAccessPolicyKeyValue);
                                                                propertiesInstance7.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance;
                                                            }
                                                            
                                                            JToken eventHubNameValue = propertiesValue8["eventHubName"];
                                                            if (eventHubNameValue != null && eventHubNameValue.Type != JTokenType.Null)
                                                            {
                                                                string eventHubNameInstance = ((string)eventHubNameValue);
                                                                propertiesInstance7.EventHubName = eventHubNameInstance;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue = propertiesValue8["consumerGroupName"];
                                                            if (consumerGroupNameValue != null && consumerGroupNameValue.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance = ((string)consumerGroupNameValue);
                                                                propertiesInstance7.ConsumerGroupName = consumerGroupNameInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue8 = datasourceValue2["type"];
                                                        if (typeValue8 != null && typeValue8.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance8 = ((string)typeValue8);
                                                            eventHubStreamInputDataSourceInstance.Type = typeInstance8;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = eventHubStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.Devices/IotHubs")
                                                    {
                                                        IoTHubStreamInputDataSource ioTHubStreamInputDataSourceInstance = new IoTHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue9 = datasourceValue2["properties"];
                                                        if (propertiesValue9 != null && propertiesValue9.Type != JTokenType.Null)
                                                        {
                                                            IoTHubStreamInputDataSourceProperties propertiesInstance8 = new IoTHubStreamInputDataSourceProperties();
                                                            ioTHubStreamInputDataSourceInstance.Properties = propertiesInstance8;
                                                            
                                                            JToken iotHubNamespaceValue = propertiesValue9["iotHubNamespace"];
                                                            if (iotHubNamespaceValue != null && iotHubNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string iotHubNamespaceInstance = ((string)iotHubNamespaceValue);
                                                                propertiesInstance8.IotHubNamespace = iotHubNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue2 = propertiesValue9["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue2 != null && sharedAccessPolicyNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance2 = ((string)sharedAccessPolicyNameValue2);
                                                                propertiesInstance8.SharedAccessPolicyName = sharedAccessPolicyNameInstance2;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue2 = propertiesValue9["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue2 != null && sharedAccessPolicyKeyValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance2 = ((string)sharedAccessPolicyKeyValue2);
                                                                propertiesInstance8.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance2;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue2 = propertiesValue9["consumerGroupName"];
                                                            if (consumerGroupNameValue2 != null && consumerGroupNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance2 = ((string)consumerGroupNameValue2);
                                                                propertiesInstance8.ConsumerGroupName = consumerGroupNameInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue9 = datasourceValue2["type"];
                                                        if (typeValue9 != null && typeValue9.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance9 = ((string)typeValue9);
                                                            ioTHubStreamInputDataSourceInstance.Type = typeInstance9;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = ioTHubStreamInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue3 = propertiesValue2["etag"];
                                                if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                                {
                                                    string etagInstance3 = ((string)etagValue3);
                                                    streamInputPropertiesInstance.Etag = etagInstance3;
                                                }
                                                
                                                JToken typeValue10 = propertiesValue2["type"];
                                                if (typeValue10 != null && typeValue10.Type != JTokenType.Null)
                                                {
                                                    string typeInstance10 = ((string)typeValue10);
                                                    streamInputPropertiesInstance.Type = typeInstance10;
                                                }
                                                
                                                JToken serializationValue2 = propertiesValue2["serialization"];
                                                if (serializationValue2 != null && serializationValue2.Type != JTokenType.Null)
                                                {
                                                    string typeName5 = ((string)serializationValue2["type"]);
                                                    if (typeName5 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance2 = new CsvSerialization();
                                                        
                                                        JToken propertiesValue10 = serializationValue2["properties"];
                                                        if (propertiesValue10 != null && propertiesValue10.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance9 = new CsvSerializationProperties();
                                                            csvSerializationInstance2.Properties = propertiesInstance9;
                                                            
                                                            JToken fieldDelimiterValue2 = propertiesValue10["fieldDelimiter"];
                                                            if (fieldDelimiterValue2 != null && fieldDelimiterValue2.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance2 = ((string)fieldDelimiterValue2);
                                                                propertiesInstance9.FieldDelimiter = fieldDelimiterInstance2;
                                                            }
                                                            
                                                            JToken encodingValue3 = propertiesValue10["encoding"];
                                                            if (encodingValue3 != null && encodingValue3.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance3 = ((string)encodingValue3);
                                                                propertiesInstance9.Encoding = encodingInstance3;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue11 = serializationValue2["type"];
                                                        if (typeValue11 != null && typeValue11.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance11 = ((string)typeValue11);
                                                            csvSerializationInstance2.Type = typeInstance11;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = csvSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance2 = new JsonSerialization();
                                                        
                                                        JToken propertiesValue11 = serializationValue2["properties"];
                                                        if (propertiesValue11 != null && propertiesValue11.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance10 = new JsonSerializationProperties();
                                                            jsonSerializationInstance2.Properties = propertiesInstance10;
                                                            
                                                            JToken encodingValue4 = propertiesValue11["encoding"];
                                                            if (encodingValue4 != null && encodingValue4.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance4 = ((string)encodingValue4);
                                                                propertiesInstance10.Encoding = encodingInstance4;
                                                            }
                                                            
                                                            JToken formatValue2 = propertiesValue11["format"];
                                                            if (formatValue2 != null && formatValue2.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance2 = ((string)formatValue2);
                                                                propertiesInstance10.Format = formatInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue12 = serializationValue2["type"];
                                                        if (typeValue12 != null && typeValue12.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance12 = ((string)typeValue12);
                                                            jsonSerializationInstance2.Type = typeInstance12;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = jsonSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance2 = new AvroSerialization();
                                                        
                                                        JToken propertiesValue12 = serializationValue2["properties"];
                                                        if (propertiesValue12 != null && propertiesValue12.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance11 = new AvroSerializationProperties();
                                                            avroSerializationInstance2.Properties = propertiesInstance11;
                                                        }
                                                        
                                                        JToken typeValue13 = serializationValue2["type"];
                                                        if (typeValue13 != null && typeValue13.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance13 = ((string)typeValue13);
                                                            avroSerializationInstance2.Type = typeInstance13;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = avroSerializationInstance2;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue2 = propertiesValue2["diagnostics"];
                                                if (diagnosticsValue2 != null && diagnosticsValue2.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance2 = new Diagnostics();
                                                    streamInputPropertiesInstance.Diagnostics = diagnosticsInstance2;
                                                    
                                                    JToken conditionsArray2 = diagnosticsValue2["conditions"];
                                                    if (conditionsArray2 != null && conditionsArray2.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue2 in ((JArray)conditionsArray2))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance2 = new DiagnosticCondition();
                                                            diagnosticsInstance2.Conditions.Add(diagnosticConditionInstance2);
                                                            
                                                            JToken sinceValue2 = conditionsValue2["since"];
                                                            if (sinceValue2 != null && sinceValue2.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance2 = ((DateTime)sinceValue2);
                                                                diagnosticConditionInstance2.Since = sinceInstance2;
                                                            }
                                                            
                                                            JToken codeValue2 = conditionsValue2["code"];
                                                            if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance2 = ((string)codeValue2);
                                                                diagnosticConditionInstance2.Code = codeInstance2;
                                                            }
                                                            
                                                            JToken messageValue2 = conditionsValue2["message"];
                                                            if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance2 = ((string)messageValue2);
                                                                diagnosticConditionInstance2.Message = messageInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = streamInputPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                                
                                JToken transformationValue = propertiesValue["transformation"];
                                if (transformationValue != null && transformationValue.Type != JTokenType.Null)
                                {
                                    Transformation transformationInstance = new Transformation();
                                    propertiesInstance.Transformation = transformationInstance;
                                    
                                    JToken nameValue4 = transformationValue["name"];
                                    if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                    {
                                        string nameInstance4 = ((string)nameValue4);
                                        transformationInstance.Name = nameInstance4;
                                    }
                                    
                                    JToken propertiesValue13 = transformationValue["properties"];
                                    if (propertiesValue13 != null && propertiesValue13.Type != JTokenType.Null)
                                    {
                                        TransformationProperties propertiesInstance12 = new TransformationProperties();
                                        transformationInstance.Properties = propertiesInstance12;
                                        
                                        JToken etagValue4 = propertiesValue13["etag"];
                                        if (etagValue4 != null && etagValue4.Type != JTokenType.Null)
                                        {
                                            string etagInstance4 = ((string)etagValue4);
                                            propertiesInstance12.Etag = etagInstance4;
                                        }
                                        
                                        JToken streamingUnitsValue = propertiesValue13["streamingUnits"];
                                        if (streamingUnitsValue != null && streamingUnitsValue.Type != JTokenType.Null)
                                        {
                                            int streamingUnitsInstance = ((int)streamingUnitsValue);
                                            propertiesInstance12.StreamingUnits = streamingUnitsInstance;
                                        }
                                        
                                        JToken queryValue = propertiesValue13["query"];
                                        if (queryValue != null && queryValue.Type != JTokenType.Null)
                                        {
                                            string queryInstance = ((string)queryValue);
                                            propertiesInstance12.Query = queryInstance;
                                        }
                                    }
                                }
                                
                                JToken outputsArray = propertiesValue["outputs"];
                                if (outputsArray != null && outputsArray.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Outputs = new List<Output>();
                                    foreach (JToken outputsValue in ((JArray)outputsArray))
                                    {
                                        Output outputInstance = new Output();
                                        propertiesInstance.Outputs.Add(outputInstance);
                                        
                                        JToken nameValue5 = outputsValue["name"];
                                        if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                        {
                                            string nameInstance5 = ((string)nameValue5);
                                            outputInstance.Name = nameInstance5;
                                        }
                                        
                                        JToken propertiesValue14 = outputsValue["properties"];
                                        if (propertiesValue14 != null && propertiesValue14.Type != JTokenType.Null)
                                        {
                                            OutputProperties propertiesInstance13 = new OutputProperties();
                                            outputInstance.Properties = propertiesInstance13;
                                            
                                            JToken etagValue5 = propertiesValue14["etag"];
                                            if (etagValue5 != null && etagValue5.Type != JTokenType.Null)
                                            {
                                                string etagInstance5 = ((string)etagValue5);
                                                propertiesInstance13.Etag = etagInstance5;
                                            }
                                            
                                            JToken datasourceValue3 = propertiesValue14["datasource"];
                                            if (datasourceValue3 != null && datasourceValue3.Type != JTokenType.Null)
                                            {
                                                string typeName6 = ((string)datasourceValue3["type"]);
                                                if (typeName6 == "Microsoft.Storage/Blob")
                                                {
                                                    BlobOutputDataSource blobOutputDataSourceInstance = new BlobOutputDataSource();
                                                    
                                                    JToken propertiesValue15 = datasourceValue3["properties"];
                                                    if (propertiesValue15 != null && propertiesValue15.Type != JTokenType.Null)
                                                    {
                                                        BlobOutputDataSourceProperties propertiesInstance14 = new BlobOutputDataSourceProperties();
                                                        blobOutputDataSourceInstance.Properties = propertiesInstance14;
                                                        
                                                        JToken storageAccountsArray3 = propertiesValue15["storageAccounts"];
                                                        if (storageAccountsArray3 != null && storageAccountsArray3.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance14.StorageAccounts = new System.Collections.Generic.List<Microsoft.Azure.Management.StreamAnalytics.Models.StorageAccount>();
                                                            foreach (JToken storageAccountsValue3 in ((JArray)storageAccountsArray3))
                                                            {
                                                                StorageAccount storageAccountInstance3 = new StorageAccount();
                                                                propertiesInstance14.StorageAccounts.Add(storageAccountInstance3);
                                                                
                                                                JToken accountNameValue3 = storageAccountsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    storageAccountInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken accountKeyValue3 = storageAccountsValue3["accountKey"];
                                                                if (accountKeyValue3 != null && accountKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance3 = ((string)accountKeyValue3);
                                                                    storageAccountInstance3.AccountKey = accountKeyInstance3;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken containerValue3 = propertiesValue15["container"];
                                                        if (containerValue3 != null && containerValue3.Type != JTokenType.Null)
                                                        {
                                                            string containerInstance3 = ((string)containerValue3);
                                                            propertiesInstance14.Container = containerInstance3;
                                                        }
                                                        
                                                        JToken pathPatternValue3 = propertiesValue15["pathPattern"];
                                                        if (pathPatternValue3 != null && pathPatternValue3.Type != JTokenType.Null)
                                                        {
                                                            string pathPatternInstance3 = ((string)pathPatternValue3);
                                                            propertiesInstance14.PathPattern = pathPatternInstance3;
                                                        }
                                                        
                                                        JToken dateFormatValue3 = propertiesValue15["dateFormat"];
                                                        if (dateFormatValue3 != null && dateFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string dateFormatInstance3 = ((string)dateFormatValue3);
                                                            propertiesInstance14.DateFormat = dateFormatInstance3;
                                                        }
                                                        
                                                        JToken timeFormatValue3 = propertiesValue15["timeFormat"];
                                                        if (timeFormatValue3 != null && timeFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string timeFormatInstance3 = ((string)timeFormatValue3);
                                                            propertiesInstance14.TimeFormat = timeFormatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue14 = datasourceValue3["type"];
                                                    if (typeValue14 != null && typeValue14.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance14 = ((string)typeValue14);
                                                        blobOutputDataSourceInstance.Type = typeInstance14;
                                                    }
                                                    propertiesInstance13.DataSource = blobOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/Table")
                                                {
                                                    AzureTableOutputDataSource azureTableOutputDataSourceInstance = new AzureTableOutputDataSource();
                                                    
                                                    JToken propertiesValue16 = datasourceValue3["properties"];
                                                    if (propertiesValue16 != null && propertiesValue16.Type != JTokenType.Null)
                                                    {
                                                        AzureTableOutputDataSourceProperties propertiesInstance15 = new AzureTableOutputDataSourceProperties();
                                                        azureTableOutputDataSourceInstance.Properties = propertiesInstance15;
                                                        
                                                        JToken accountNameValue4 = propertiesValue16["accountName"];
                                                        if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountNameInstance4 = ((string)accountNameValue4);
                                                            propertiesInstance15.AccountName = accountNameInstance4;
                                                        }
                                                        
                                                        JToken accountKeyValue4 = propertiesValue16["accountKey"];
                                                        if (accountKeyValue4 != null && accountKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance4 = ((string)accountKeyValue4);
                                                            propertiesInstance15.AccountKey = accountKeyInstance4;
                                                        }
                                                        
                                                        JToken tableValue = propertiesValue16["table"];
                                                        if (tableValue != null && tableValue.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance = ((string)tableValue);
                                                            propertiesInstance15.Table = tableInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue = propertiesValue16["partitionKey"];
                                                        if (partitionKeyValue != null && partitionKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance = ((string)partitionKeyValue);
                                                            propertiesInstance15.PartitionKey = partitionKeyInstance;
                                                        }
                                                        
                                                        JToken rowKeyValue = propertiesValue16["rowKey"];
                                                        if (rowKeyValue != null && rowKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string rowKeyInstance = ((string)rowKeyValue);
                                                            propertiesInstance15.RowKey = rowKeyInstance;
                                                        }
                                                        
                                                        JToken columnsToRemoveArray = propertiesValue16["columnsToRemove"];
                                                        if (columnsToRemoveArray != null && columnsToRemoveArray.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance15.ColumnsToRemove = new System.Collections.Generic.List<string>();
                                                            foreach (JToken columnsToRemoveValue in ((JArray)columnsToRemoveArray))
                                                            {
                                                                propertiesInstance15.ColumnsToRemove.Add(((string)columnsToRemoveValue));
                                                            }
                                                        }
                                                        
                                                        JToken batchSizeValue = propertiesValue16["batchSize"];
                                                        if (batchSizeValue != null && batchSizeValue.Type != JTokenType.Null)
                                                        {
                                                            int batchSizeInstance = ((int)batchSizeValue);
                                                            propertiesInstance15.BatchSize = batchSizeInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue15 = datasourceValue3["type"];
                                                    if (typeValue15 != null && typeValue15.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance15 = ((string)typeValue15);
                                                        azureTableOutputDataSourceInstance.Type = typeInstance15;
                                                    }
                                                    propertiesInstance13.DataSource = azureTableOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/EventHub")
                                                {
                                                    EventHubOutputDataSource eventHubOutputDataSourceInstance = new EventHubOutputDataSource();
                                                    
                                                    JToken propertiesValue17 = datasourceValue3["properties"];
                                                    if (propertiesValue17 != null && propertiesValue17.Type != JTokenType.Null)
                                                    {
                                                        EventHubOutputDataSourceProperties propertiesInstance16 = new EventHubOutputDataSourceProperties();
                                                        eventHubOutputDataSourceInstance.Properties = propertiesInstance16;
                                                        
                                                        JToken serviceBusNamespaceValue2 = propertiesValue17["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue2 != null && serviceBusNamespaceValue2.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance2 = ((string)serviceBusNamespaceValue2);
                                                            propertiesInstance16.ServiceBusNamespace = serviceBusNamespaceInstance2;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue3 = propertiesValue17["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue3 != null && sharedAccessPolicyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance3 = ((string)sharedAccessPolicyNameValue3);
                                                            propertiesInstance16.SharedAccessPolicyName = sharedAccessPolicyNameInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue3 = propertiesValue17["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue3 != null && sharedAccessPolicyKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance3 = ((string)sharedAccessPolicyKeyValue3);
                                                            propertiesInstance16.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance3;
                                                        }
                                                        
                                                        JToken eventHubNameValue2 = propertiesValue17["eventHubName"];
                                                        if (eventHubNameValue2 != null && eventHubNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string eventHubNameInstance2 = ((string)eventHubNameValue2);
                                                            propertiesInstance16.EventHubName = eventHubNameInstance2;
                                                        }
                                                        
                                                        JToken partitionKeyValue2 = propertiesValue17["partitionKey"];
                                                        if (partitionKeyValue2 != null && partitionKeyValue2.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance2 = ((string)partitionKeyValue2);
                                                            propertiesInstance16.PartitionKey = partitionKeyInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue16 = datasourceValue3["type"];
                                                    if (typeValue16 != null && typeValue16.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance16 = ((string)typeValue16);
                                                        eventHubOutputDataSourceInstance.Type = typeInstance16;
                                                    }
                                                    propertiesInstance13.DataSource = eventHubOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Queue")
                                                {
                                                    ServiceBusQueueOutputDataSource serviceBusQueueOutputDataSourceInstance = new ServiceBusQueueOutputDataSource();
                                                    
                                                    JToken propertiesValue18 = datasourceValue3["properties"];
                                                    if (propertiesValue18 != null && propertiesValue18.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusQueueOutputDataSourceProperties propertiesInstance17 = new ServiceBusQueueOutputDataSourceProperties();
                                                        serviceBusQueueOutputDataSourceInstance.Properties = propertiesInstance17;
                                                        
                                                        JToken serviceBusNamespaceValue3 = propertiesValue18["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue3 != null && serviceBusNamespaceValue3.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance3 = ((string)serviceBusNamespaceValue3);
                                                            propertiesInstance17.ServiceBusNamespace = serviceBusNamespaceInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue4 = propertiesValue18["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue4 != null && sharedAccessPolicyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance4 = ((string)sharedAccessPolicyNameValue4);
                                                            propertiesInstance17.SharedAccessPolicyName = sharedAccessPolicyNameInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue4 = propertiesValue18["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue4 != null && sharedAccessPolicyKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance4 = ((string)sharedAccessPolicyKeyValue4);
                                                            propertiesInstance17.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance4;
                                                        }
                                                        
                                                        JToken queueNameValue = propertiesValue18["queueName"];
                                                        if (queueNameValue != null && queueNameValue.Type != JTokenType.Null)
                                                        {
                                                            string queueNameInstance = ((string)queueNameValue);
                                                            propertiesInstance17.QueueName = queueNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue17 = datasourceValue3["type"];
                                                    if (typeValue17 != null && typeValue17.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance17 = ((string)typeValue17);
                                                        serviceBusQueueOutputDataSourceInstance.Type = typeInstance17;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusQueueOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Topic")
                                                {
                                                    ServiceBusTopicOutputDataSource serviceBusTopicOutputDataSourceInstance = new ServiceBusTopicOutputDataSource();
                                                    
                                                    JToken propertiesValue19 = datasourceValue3["properties"];
                                                    if (propertiesValue19 != null && propertiesValue19.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusTopicOutputDataSourceProperties propertiesInstance18 = new ServiceBusTopicOutputDataSourceProperties();
                                                        serviceBusTopicOutputDataSourceInstance.Properties = propertiesInstance18;
                                                        
                                                        JToken serviceBusNamespaceValue4 = propertiesValue19["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue4 != null && serviceBusNamespaceValue4.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance4 = ((string)serviceBusNamespaceValue4);
                                                            propertiesInstance18.ServiceBusNamespace = serviceBusNamespaceInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue5 = propertiesValue19["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue5 != null && sharedAccessPolicyNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance5 = ((string)sharedAccessPolicyNameValue5);
                                                            propertiesInstance18.SharedAccessPolicyName = sharedAccessPolicyNameInstance5;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue5 = propertiesValue19["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue5 != null && sharedAccessPolicyKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance5 = ((string)sharedAccessPolicyKeyValue5);
                                                            propertiesInstance18.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance5;
                                                        }
                                                        
                                                        JToken topicNameValue = propertiesValue19["topicName"];
                                                        if (topicNameValue != null && topicNameValue.Type != JTokenType.Null)
                                                        {
                                                            string topicNameInstance = ((string)topicNameValue);
                                                            propertiesInstance18.TopicName = topicNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue18 = datasourceValue3["type"];
                                                    if (typeValue18 != null && typeValue18.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance18 = ((string)typeValue18);
                                                        serviceBusTopicOutputDataSourceInstance.Type = typeInstance18;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusTopicOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Sql/Server/Database")
                                                {
                                                    SqlAzureOutputDataSource sqlAzureOutputDataSourceInstance = new SqlAzureOutputDataSource();
                                                    
                                                    JToken propertiesValue20 = datasourceValue3["properties"];
                                                    if (propertiesValue20 != null && propertiesValue20.Type != JTokenType.Null)
                                                    {
                                                        SqlAzureOutputDataSourceProperties propertiesInstance19 = new SqlAzureOutputDataSourceProperties();
                                                        sqlAzureOutputDataSourceInstance.Properties = propertiesInstance19;
                                                        
                                                        JToken serverValue = propertiesValue20["server"];
                                                        if (serverValue != null && serverValue.Type != JTokenType.Null)
                                                        {
                                                            string serverInstance = ((string)serverValue);
                                                            propertiesInstance19.Server = serverInstance;
                                                        }
                                                        
                                                        JToken databaseValue = propertiesValue20["database"];
                                                        if (databaseValue != null && databaseValue.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance = ((string)databaseValue);
                                                            propertiesInstance19.Database = databaseInstance;
                                                        }
                                                        
                                                        JToken userValue = propertiesValue20["user"];
                                                        if (userValue != null && userValue.Type != JTokenType.Null)
                                                        {
                                                            string userInstance = ((string)userValue);
                                                            propertiesInstance19.User = userInstance;
                                                        }
                                                        
                                                        JToken passwordValue = propertiesValue20["password"];
                                                        if (passwordValue != null && passwordValue.Type != JTokenType.Null)
                                                        {
                                                            string passwordInstance = ((string)passwordValue);
                                                            propertiesInstance19.Password = passwordInstance;
                                                        }
                                                        
                                                        JToken tableValue2 = propertiesValue20["table"];
                                                        if (tableValue2 != null && tableValue2.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance2 = ((string)tableValue2);
                                                            propertiesInstance19.Table = tableInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue19 = datasourceValue3["type"];
                                                    if (typeValue19 != null && typeValue19.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance19 = ((string)typeValue19);
                                                        sqlAzureOutputDataSourceInstance.Type = typeInstance19;
                                                    }
                                                    propertiesInstance13.DataSource = sqlAzureOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/DocumentDB")
                                                {
                                                    DocumentDbOutputDataSource documentDbOutputDataSourceInstance = new DocumentDbOutputDataSource();
                                                    
                                                    JToken propertiesValue21 = datasourceValue3["properties"];
                                                    if (propertiesValue21 != null && propertiesValue21.Type != JTokenType.Null)
                                                    {
                                                        DocumentDbOutputDataSourceProperties propertiesInstance20 = new DocumentDbOutputDataSourceProperties();
                                                        documentDbOutputDataSourceInstance.Properties = propertiesInstance20;
                                                        
                                                        JToken accountIdValue = propertiesValue21["accountId"];
                                                        if (accountIdValue != null && accountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string accountIdInstance = ((string)accountIdValue);
                                                            propertiesInstance20.AccountId = accountIdInstance;
                                                        }
                                                        
                                                        JToken accountKeyValue5 = propertiesValue21["accountKey"];
                                                        if (accountKeyValue5 != null && accountKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance5 = ((string)accountKeyValue5);
                                                            propertiesInstance20.AccountKey = accountKeyInstance5;
                                                        }
                                                        
                                                        JToken databaseValue2 = propertiesValue21["database"];
                                                        if (databaseValue2 != null && databaseValue2.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance2 = ((string)databaseValue2);
                                                            propertiesInstance20.Database = databaseInstance2;
                                                        }
                                                        
                                                        JToken collectionNamePatternValue = propertiesValue21["collectionNamePattern"];
                                                        if (collectionNamePatternValue != null && collectionNamePatternValue.Type != JTokenType.Null)
                                                        {
                                                            string collectionNamePatternInstance = ((string)collectionNamePatternValue);
                                                            propertiesInstance20.CollectionNamePattern = collectionNamePatternInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue3 = propertiesValue21["partitionKey"];
                                                        if (partitionKeyValue3 != null && partitionKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance3 = ((string)partitionKeyValue3);
                                                            propertiesInstance20.PartitionKey = partitionKeyInstance3;
                                                        }
                                                        
                                                        JToken documentIdValue = propertiesValue21["documentId"];
                                                        if (documentIdValue != null && documentIdValue.Type != JTokenType.Null)
                                                        {
                                                            string documentIdInstance = ((string)documentIdValue);
                                                            propertiesInstance20.DocumentId = documentIdInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue20 = datasourceValue3["type"];
                                                    if (typeValue20 != null && typeValue20.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance20 = ((string)typeValue20);
                                                        documentDbOutputDataSourceInstance.Type = typeInstance20;
                                                    }
                                                    propertiesInstance13.DataSource = documentDbOutputDataSourceInstance;
                                                }
                                            }
                                            
                                            JToken serializationValue3 = propertiesValue14["serialization"];
                                            if (serializationValue3 != null && serializationValue3.Type != JTokenType.Null)
                                            {
                                                string typeName7 = ((string)serializationValue3["type"]);
                                                if (typeName7 == "Csv")
                                                {
                                                    CsvSerialization csvSerializationInstance3 = new CsvSerialization();
                                                    
                                                    JToken propertiesValue22 = serializationValue3["properties"];
                                                    if (propertiesValue22 != null && propertiesValue22.Type != JTokenType.Null)
                                                    {
                                                        CsvSerializationProperties propertiesInstance21 = new CsvSerializationProperties();
                                                        csvSerializationInstance3.Properties = propertiesInstance21;
                                                        
                                                        JToken fieldDelimiterValue3 = propertiesValue22["fieldDelimiter"];
                                                        if (fieldDelimiterValue3 != null && fieldDelimiterValue3.Type != JTokenType.Null)
                                                        {
                                                            string fieldDelimiterInstance3 = ((string)fieldDelimiterValue3);
                                                            propertiesInstance21.FieldDelimiter = fieldDelimiterInstance3;
                                                        }
                                                        
                                                        JToken encodingValue5 = propertiesValue22["encoding"];
                                                        if (encodingValue5 != null && encodingValue5.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance5 = ((string)encodingValue5);
                                                            propertiesInstance21.Encoding = encodingInstance5;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue21 = serializationValue3["type"];
                                                    if (typeValue21 != null && typeValue21.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance21 = ((string)typeValue21);
                                                        csvSerializationInstance3.Type = typeInstance21;
                                                    }
                                                    propertiesInstance13.Serialization = csvSerializationInstance3;
                                                }
                                                if (typeName7 == "Json")
                                                {
                                                    JsonSerialization jsonSerializationInstance3 = new JsonSerialization();
                                                    
                                                    JToken propertiesValue23 = serializationValue3["properties"];
                                                    if (propertiesValue23 != null && propertiesValue23.Type != JTokenType.Null)
                                                    {
                                                        JsonSerializationProperties propertiesInstance22 = new JsonSerializationProperties();
                                                        jsonSerializationInstance3.Properties = propertiesInstance22;
                                                        
                                                        JToken encodingValue6 = propertiesValue23["encoding"];
                                                        if (encodingValue6 != null && encodingValue6.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance6 = ((string)encodingValue6);
                                                            propertiesInstance22.Encoding = encodingInstance6;
                                                        }
                                                        
                                                        JToken formatValue3 = propertiesValue23["format"];
                                                        if (formatValue3 != null && formatValue3.Type != JTokenType.Null)
                                                        {
                                                            string formatInstance3 = ((string)formatValue3);
                                                            propertiesInstance22.Format = formatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue22 = serializationValue3["type"];
                                                    if (typeValue22 != null && typeValue22.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance22 = ((string)typeValue22);
                                                        jsonSerializationInstance3.Type = typeInstance22;
                                                    }
                                                    propertiesInstance13.Serialization = jsonSerializationInstance3;
                                                }
                                                if (typeName7 == "Avro")
                                                {
                                                    AvroSerialization avroSerializationInstance3 = new AvroSerialization();
                                                    
                                                    JToken propertiesValue24 = serializationValue3["properties"];
                                                    if (propertiesValue24 != null && propertiesValue24.Type != JTokenType.Null)
                                                    {
                                                        AvroSerializationProperties propertiesInstance23 = new AvroSerializationProperties();
                                                        avroSerializationInstance3.Properties = propertiesInstance23;
                                                    }
                                                    
                                                    JToken typeValue23 = serializationValue3["type"];
                                                    if (typeValue23 != null && typeValue23.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance23 = ((string)typeValue23);
                                                        avroSerializationInstance3.Type = typeInstance23;
                                                    }
                                                    propertiesInstance13.Serialization = avroSerializationInstance3;
                                                }
                                            }
                                            
                                            JToken diagnosticsValue3 = propertiesValue14["diagnostics"];
                                            if (diagnosticsValue3 != null && diagnosticsValue3.Type != JTokenType.Null)
                                            {
                                                Diagnostics diagnosticsInstance3 = new Diagnostics();
                                                propertiesInstance13.Diagnostics = diagnosticsInstance3;
                                                
                                                JToken conditionsArray3 = diagnosticsValue3["conditions"];
                                                if (conditionsArray3 != null && conditionsArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken conditionsValue3 in ((JArray)conditionsArray3))
                                                    {
                                                        DiagnosticCondition diagnosticConditionInstance3 = new DiagnosticCondition();
                                                        diagnosticsInstance3.Conditions.Add(diagnosticConditionInstance3);
                                                        
                                                        JToken sinceValue3 = conditionsValue3["since"];
                                                        if (sinceValue3 != null && sinceValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime sinceInstance3 = ((DateTime)sinceValue3);
                                                            diagnosticConditionInstance3.Since = sinceInstance3;
                                                        }
                                                        
                                                        JToken codeValue3 = conditionsValue3["code"];
                                                        if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                        {
                                                            string codeInstance3 = ((string)codeValue3);
                                                            diagnosticConditionInstance3.Code = codeInstance3;
                                                        }
                                                        
                                                        JToken messageValue3 = conditionsValue3["message"];
                                                        if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                        {
                                                            string messageInstance3 = ((string)messageValue3);
                                                            diagnosticConditionInstance3.Message = messageInstance3;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                
                                JToken functionsArray = propertiesValue["functions"];
                                if (functionsArray != null && functionsArray.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Functions = new List<Function>();
                                    foreach (JToken functionsValue in ((JArray)functionsArray))
                                    {
                                        Function functionInstance = new Function();
                                        propertiesInstance.Functions.Add(functionInstance);
                                        
                                        JToken nameValue6 = functionsValue["name"];
                                        if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                        {
                                            string nameInstance6 = ((string)nameValue6);
                                            functionInstance.Name = nameInstance6;
                                        }
                                        
                                        JToken propertiesValue25 = functionsValue["properties"];
                                        if (propertiesValue25 != null && propertiesValue25.Type != JTokenType.Null)
                                        {
                                            string typeName8 = ((string)propertiesValue25["type"]);
                                            if (typeName8 == "Scalar")
                                            {
                                                ScalarFunctionProperties scalarFunctionPropertiesInstance = new ScalarFunctionProperties();
                                                
                                                JToken propertiesValue26 = propertiesValue25["properties"];
                                                if (propertiesValue26 != null && propertiesValue26.Type != JTokenType.Null)
                                                {
                                                    ScalarFunctionConfiguration propertiesInstance24 = new ScalarFunctionConfiguration();
                                                    scalarFunctionPropertiesInstance.Properties = propertiesInstance24;
                                                    
                                                    JToken inputsArray2 = propertiesValue26["inputs"];
                                                    if (inputsArray2 != null && inputsArray2.Type != JTokenType.Null)
                                                    {
                                                        propertiesInstance24.Inputs = new List<FunctionInput>();
                                                        foreach (JToken inputsValue2 in ((JArray)inputsArray2))
                                                        {
                                                            FunctionInput functionInputInstance = new FunctionInput();
                                                            propertiesInstance24.Inputs.Add(functionInputInstance);
                                                            
                                                            JToken dataTypeValue = inputsValue2["dataType"];
                                                            if (dataTypeValue != null && dataTypeValue.Type != JTokenType.Null)
                                                            {
                                                                string dataTypeInstance = ((string)dataTypeValue);
                                                                functionInputInstance.DataType = dataTypeInstance;
                                                            }
                                                            
                                                            JToken isConfigurationParameterValue = inputsValue2["isConfigurationParameter"];
                                                            if (isConfigurationParameterValue != null && isConfigurationParameterValue.Type != JTokenType.Null)
                                                            {
                                                                bool isConfigurationParameterInstance = ((bool)isConfigurationParameterValue);
                                                                functionInputInstance.IsConfigurationParameter = isConfigurationParameterInstance;
                                                            }
                                                        }
                                                    }
                                                    
                                                    JToken outputValue = propertiesValue26["output"];
                                                    if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                    {
                                                        FunctionOutput outputInstance2 = new FunctionOutput();
                                                        propertiesInstance24.Output = outputInstance2;
                                                        
                                                        JToken dataTypeValue2 = outputValue["dataType"];
                                                        if (dataTypeValue2 != null && dataTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string dataTypeInstance2 = ((string)dataTypeValue2);
                                                            outputInstance2.DataType = dataTypeInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken bindingValue = propertiesValue26["binding"];
                                                    if (bindingValue != null && bindingValue.Type != JTokenType.Null)
                                                    {
                                                        string typeName9 = ((string)bindingValue["type"]);
                                                        if (typeName9 == "Microsoft.MachineLearning/WebService")
                                                        {
                                                            AzureMachineLearningWebServiceFunctionBinding azureMachineLearningWebServiceFunctionBindingInstance = new AzureMachineLearningWebServiceFunctionBinding();
                                                            
                                                            JToken propertiesValue27 = bindingValue["properties"];
                                                            if (propertiesValue27 != null && propertiesValue27.Type != JTokenType.Null)
                                                            {
                                                                AzureMachineLearningWebServiceFunctionBindingProperties propertiesInstance25 = new AzureMachineLearningWebServiceFunctionBindingProperties();
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Properties = propertiesInstance25;
                                                                
                                                                JToken endpointValue = propertiesValue27["endpoint"];
                                                                if (endpointValue != null && endpointValue.Type != JTokenType.Null)
                                                                {
                                                                    string endpointInstance = ((string)endpointValue);
                                                                    propertiesInstance25.Endpoint = endpointInstance;
                                                                }
                                                                
                                                                JToken apiKeyValue = propertiesValue27["apiKey"];
                                                                if (apiKeyValue != null && apiKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string apiKeyInstance = ((string)apiKeyValue);
                                                                    propertiesInstance25.ApiKey = apiKeyInstance;
                                                                }
                                                                
                                                                JToken inputsValue3 = propertiesValue27["inputs"];
                                                                if (inputsValue3 != null && inputsValue3.Type != JTokenType.Null)
                                                                {
                                                                    AzureMachineLearningWebServiceInputs inputsInstance = new AzureMachineLearningWebServiceInputs();
                                                                    propertiesInstance25.Inputs = inputsInstance;
                                                                    
                                                                    JToken nameValue7 = inputsValue3["name"];
                                                                    if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string nameInstance7 = ((string)nameValue7);
                                                                        inputsInstance.Name = nameInstance7;
                                                                    }
                                                                    
                                                                    JToken columnNamesArray = inputsValue3["columnNames"];
                                                                    if (columnNamesArray != null && columnNamesArray.Type != JTokenType.Null)
                                                                    {
                                                                        inputsInstance.ColumnNames = new List<AzureMachineLearningWebServiceInputColumn>();
                                                                        foreach (JToken columnNamesValue in ((JArray)columnNamesArray))
                                                                        {
                                                                            AzureMachineLearningWebServiceInputColumn azureMachineLearningWebServiceInputColumnInstance = new AzureMachineLearningWebServiceInputColumn();
                                                                            inputsInstance.ColumnNames.Add(azureMachineLearningWebServiceInputColumnInstance);
                                                                            
                                                                            JToken nameValue8 = columnNamesValue["name"];
                                                                            if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string nameInstance8 = ((string)nameValue8);
                                                                                azureMachineLearningWebServiceInputColumnInstance.Name = nameInstance8;
                                                                            }
                                                                            
                                                                            JToken dataTypeValue3 = columnNamesValue["dataType"];
                                                                            if (dataTypeValue3 != null && dataTypeValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string dataTypeInstance3 = ((string)dataTypeValue3);
                                                                                azureMachineLearningWebServiceInputColumnInstance.DataType = dataTypeInstance3;
                                                                            }
                                                                            
                                                                            JToken mapToValue = columnNamesValue["mapTo"];
                                                                            if (mapToValue != null && mapToValue.Type != JTokenType.Null)
                                                                            {
                                                                                int mapToInstance = ((int)mapToValue);
                                                                                azureMachineLearningWebServiceInputColumnInstance.MapTo = mapToInstance;
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken outputsArray2 = propertiesValue27["outputs"];
                                                                if (outputsArray2 != null && outputsArray2.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance25.Outputs = new List<AzureMachineLearningWebServiceOutputColumn>();
                                                                    foreach (JToken outputsValue2 in ((JArray)outputsArray2))
                                                                    {
                                                                        AzureMachineLearningWebServiceOutputColumn azureMachineLearningWebServiceOutputColumnInstance = new AzureMachineLearningWebServiceOutputColumn();
                                                                        propertiesInstance25.Outputs.Add(azureMachineLearningWebServiceOutputColumnInstance);
                                                                        
                                                                        JToken nameValue9 = outputsValue2["name"];
                                                                        if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance9 = ((string)nameValue9);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.Name = nameInstance9;
                                                                        }
                                                                        
                                                                        JToken dataTypeValue4 = outputsValue2["dataType"];
                                                                        if (dataTypeValue4 != null && dataTypeValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string dataTypeInstance4 = ((string)dataTypeValue4);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.DataType = dataTypeInstance4;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken batchSizeValue2 = propertiesValue27["batchSize"];
                                                                if (batchSizeValue2 != null && batchSizeValue2.Type != JTokenType.Null)
                                                                {
                                                                    int batchSizeInstance2 = ((int)batchSizeValue2);
                                                                    propertiesInstance25.BatchSize = batchSizeInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue24 = bindingValue["type"];
                                                            if (typeValue24 != null && typeValue24.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance24 = ((string)typeValue24);
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Type = typeInstance24;
                                                            }
                                                            propertiesInstance24.Binding = azureMachineLearningWebServiceFunctionBindingInstance;
                                                        }
                                                        if (typeName9 == "Microsoft.StreamAnalytics/JavascriptUdf")
                                                        {
                                                            JavaScriptFunctionBinding javaScriptFunctionBindingInstance = new JavaScriptFunctionBinding();
                                                            
                                                            JToken propertiesValue28 = bindingValue["properties"];
                                                            if (propertiesValue28 != null && propertiesValue28.Type != JTokenType.Null)
                                                            {
                                                                JavaScriptFunctionBindingProperties propertiesInstance26 = new JavaScriptFunctionBindingProperties();
                                                                javaScriptFunctionBindingInstance.Properties = propertiesInstance26;
                                                                
                                                                JToken scriptValue = propertiesValue28["script"];
                                                                if (scriptValue != null && scriptValue.Type != JTokenType.Null)
                                                                {
                                                                    string scriptInstance = ((string)scriptValue);
                                                                    propertiesInstance26.Script = scriptInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue25 = bindingValue["type"];
                                                            if (typeValue25 != null && typeValue25.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance25 = ((string)typeValue25);
                                                                javaScriptFunctionBindingInstance.Type = typeInstance25;
                                                            }
                                                            propertiesInstance24.Binding = javaScriptFunctionBindingInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken etagValue6 = propertiesValue25["etag"];
                                                if (etagValue6 != null && etagValue6.Type != JTokenType.Null)
                                                {
                                                    string etagInstance6 = ((string)etagValue6);
                                                    scalarFunctionPropertiesInstance.Etag = etagInstance6;
                                                }
                                                
                                                JToken typeValue26 = propertiesValue25["type"];
                                                if (typeValue26 != null && typeValue26.Type != JTokenType.Null)
                                                {
                                                    string typeInstance26 = ((string)typeValue26);
                                                    scalarFunctionPropertiesInstance.Type = typeInstance26;
                                                }
                                                functionInstance.Properties = scalarFunctionPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = DateTime.Parse(httpResponse.Headers.GetValues("Date").FirstOrDefault(), CultureInfo.InvariantCulture, DateTimeStyles.AdjustToUniversal).ToLocalTime();
                    }
                    if (httpResponse.Headers.Contains("ETag"))
                    {
                        result.Job.Properties.Etag = httpResponse.Headers.GetValues("ETag").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Delete a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. A unique stream analytics job name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> DeleteAsync(string resourceGroupName, string jobName, CancellationToken cancellationToken)
        {
            StreamAnalyticsManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                TracingAdapter.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.StreamingJobs.BeginDeleteAsync(resourceGroupName, jobName, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse result = await client.GetLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 10;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.GetLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 10;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Get a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to get a stream analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the get stream analytics job operation.
        /// </returns>
        public async Task<JobGetResponse> GetAsync(string resourceGroupName, string jobName, JobGetParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.PropertiesToExpand == null)
            {
                throw new ArgumentNullException("parameters.PropertiesToExpand");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs/";
            url = url + Uri.EscapeDataString(jobName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("$expand=" + Uri.EscapeDataString(parameters.PropertiesToExpand));
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobGetResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobGetResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                jobInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                jobInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                jobInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                jobInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                jobInstance.Tags = new Dictionary<string, string>();
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property.Name);
                                    string tagsValue = ((string)property.Value);
                                    jobInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken skuValue = propertiesValue["sku"];
                                if (skuValue != null && skuValue.Type != JTokenType.Null)
                                {
                                    Sku skuInstance = new Sku();
                                    propertiesInstance.Sku = skuInstance;
                                    
                                    JToken nameValue2 = skuValue["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        string nameInstance2 = ((string)nameValue2);
                                        skuInstance.Name = nameInstance2;
                                    }
                                }
                                
                                JToken eventsOutOfOrderPolicyValue = propertiesValue["eventsOutOfOrderPolicy"];
                                if (eventsOutOfOrderPolicyValue != null && eventsOutOfOrderPolicyValue.Type != JTokenType.Null)
                                {
                                    string eventsOutOfOrderPolicyInstance = ((string)eventsOutOfOrderPolicyValue);
                                    propertiesInstance.EventsOutOfOrderPolicy = eventsOutOfOrderPolicyInstance;
                                }
                                
                                JToken outputErrorPolicyValue = propertiesValue["outputErrorPolicy"];
                                if (outputErrorPolicyValue != null && outputErrorPolicyValue.Type != JTokenType.Null)
                                {
                                    string outputErrorPolicyInstance = ((string)outputErrorPolicyValue);
                                    propertiesInstance.OutputErrorPolicy = outputErrorPolicyInstance;
                                }
                                
                                JToken eventsOutOfOrderMaxDelayInSecondsValue = propertiesValue["eventsOutOfOrderMaxDelayInSeconds"];
                                if (eventsOutOfOrderMaxDelayInSecondsValue != null && eventsOutOfOrderMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsOutOfOrderMaxDelayInSecondsInstance = ((int)eventsOutOfOrderMaxDelayInSecondsValue);
                                    propertiesInstance.EventsOutOfOrderMaxDelayInSeconds = eventsOutOfOrderMaxDelayInSecondsInstance;
                                }
                                
                                JToken eventsLateArrivalMaxDelayInSecondsValue = propertiesValue["eventsLateArrivalMaxDelayInSeconds"];
                                if (eventsLateArrivalMaxDelayInSecondsValue != null && eventsLateArrivalMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsLateArrivalMaxDelayInSecondsInstance = ((int)eventsLateArrivalMaxDelayInSecondsValue);
                                    propertiesInstance.EventsLateArrivalMaxDelayInSeconds = eventsLateArrivalMaxDelayInSecondsInstance;
                                }
                                
                                JToken dataLocaleValue = propertiesValue["dataLocale"];
                                if (dataLocaleValue != null && dataLocaleValue.Type != JTokenType.Null)
                                {
                                    string dataLocaleInstance = ((string)dataLocaleValue);
                                    propertiesInstance.DataLocale = dataLocaleInstance;
                                }
                                
                                JToken outputStartModeValue = propertiesValue["outputStartMode"];
                                if (outputStartModeValue != null && outputStartModeValue.Type != JTokenType.Null)
                                {
                                    string outputStartModeInstance = ((string)outputStartModeValue);
                                    propertiesInstance.OutputStartMode = outputStartModeInstance;
                                }
                                
                                JToken outputStartTimeValue = propertiesValue["outputStartTime"];
                                if (outputStartTimeValue != null && outputStartTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime outputStartTimeInstance = ((DateTime)outputStartTimeValue);
                                    propertiesInstance.OutputStartTime = outputStartTimeInstance;
                                }
                                
                                JToken lastOutputEventTimeValue = propertiesValue["lastOutputEventTime"];
                                if (lastOutputEventTimeValue != null && lastOutputEventTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastOutputEventTimeInstance = ((DateTime)lastOutputEventTimeValue);
                                    propertiesInstance.LastOutputEventTime = lastOutputEventTimeInstance;
                                }
                                
                                JToken jobIdValue = propertiesValue["jobId"];
                                if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                {
                                    string jobIdInstance = ((string)jobIdValue);
                                    propertiesInstance.JobId = jobIdInstance;
                                }
                                
                                JToken etagValue = propertiesValue["etag"];
                                if (etagValue != null && etagValue.Type != JTokenType.Null)
                                {
                                    string etagInstance = ((string)etagValue);
                                    propertiesInstance.Etag = etagInstance;
                                }
                                
                                JToken provisioningStateValue = propertiesValue["provisioningState"];
                                if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance = ((string)provisioningStateValue);
                                    propertiesInstance.ProvisioningState = provisioningStateInstance;
                                }
                                
                                JToken jobStateValue = propertiesValue["jobState"];
                                if (jobStateValue != null && jobStateValue.Type != JTokenType.Null)
                                {
                                    string jobStateInstance = ((string)jobStateValue);
                                    propertiesInstance.JobState = jobStateInstance;
                                }
                                
                                JToken createdDateValue = propertiesValue["createdDate"];
                                if (createdDateValue != null && createdDateValue.Type != JTokenType.Null)
                                {
                                    DateTime createdDateInstance = ((DateTime)createdDateValue);
                                    propertiesInstance.CreatedDate = createdDateInstance;
                                }
                                
                                JToken inputsArray = propertiesValue["inputs"];
                                if (inputsArray != null && inputsArray.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Inputs = new List<Input>();
                                    foreach (JToken inputsValue in ((JArray)inputsArray))
                                    {
                                        Input inputInstance = new Input();
                                        propertiesInstance.Inputs.Add(inputInstance);
                                        
                                        JToken nameValue3 = inputsValue["name"];
                                        if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                        {
                                            string nameInstance3 = ((string)nameValue3);
                                            inputInstance.Name = nameInstance3;
                                        }
                                        
                                        JToken propertiesValue2 = inputsValue["properties"];
                                        if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)propertiesValue2["type"]);
                                            if (typeName == "Reference")
                                            {
                                                ReferenceInputProperties referenceInputPropertiesInstance = new ReferenceInputProperties();
                                                
                                                JToken datasourceValue = propertiesValue2["datasource"];
                                                if (datasourceValue != null && datasourceValue.Type != JTokenType.Null)
                                                {
                                                    string typeName2 = ((string)datasourceValue["type"]);
                                                    if (typeName2 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobReferenceInputDataSource blobReferenceInputDataSourceInstance = new BlobReferenceInputDataSource();
                                                        
                                                        JToken propertiesValue3 = datasourceValue["properties"];
                                                        if (propertiesValue3 != null && propertiesValue3.Type != JTokenType.Null)
                                                        {
                                                            BlobReferenceInputDataSourceProperties propertiesInstance2 = new BlobReferenceInputDataSourceProperties();
                                                            blobReferenceInputDataSourceInstance.Properties = propertiesInstance2;
                                                            
                                                            JToken storageAccountsArray = propertiesValue3["storageAccounts"];
                                                            if (storageAccountsArray != null && storageAccountsArray.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance2.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue in ((JArray)storageAccountsArray))
                                                                {
                                                                    StorageAccount storageAccountInstance = new StorageAccount();
                                                                    propertiesInstance2.StorageAccounts.Add(storageAccountInstance);
                                                                    
                                                                    JToken accountNameValue = storageAccountsValue["accountName"];
                                                                    if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance = ((string)accountNameValue);
                                                                        storageAccountInstance.AccountName = accountNameInstance;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue = storageAccountsValue["accountKey"];
                                                                    if (accountKeyValue != null && accountKeyValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance = ((string)accountKeyValue);
                                                                        storageAccountInstance.AccountKey = accountKeyInstance;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue = propertiesValue3["container"];
                                                            if (containerValue != null && containerValue.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance = ((string)containerValue);
                                                                propertiesInstance2.Container = containerInstance;
                                                            }
                                                            
                                                            JToken pathPatternValue = propertiesValue3["pathPattern"];
                                                            if (pathPatternValue != null && pathPatternValue.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance = ((string)pathPatternValue);
                                                                propertiesInstance2.PathPattern = pathPatternInstance;
                                                            }
                                                            
                                                            JToken dateFormatValue = propertiesValue3["dateFormat"];
                                                            if (dateFormatValue != null && dateFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance = ((string)dateFormatValue);
                                                                propertiesInstance2.DateFormat = dateFormatInstance;
                                                            }
                                                            
                                                            JToken timeFormatValue = propertiesValue3["timeFormat"];
                                                            if (timeFormatValue != null && timeFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance = ((string)timeFormatValue);
                                                                propertiesInstance2.TimeFormat = timeFormatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue2 = datasourceValue["type"];
                                                        if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance2 = ((string)typeValue2);
                                                            blobReferenceInputDataSourceInstance.Type = typeInstance2;
                                                        }
                                                        referenceInputPropertiesInstance.DataSource = blobReferenceInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue2 = propertiesValue2["etag"];
                                                if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                {
                                                    string etagInstance2 = ((string)etagValue2);
                                                    referenceInputPropertiesInstance.Etag = etagInstance2;
                                                }
                                                
                                                JToken typeValue3 = propertiesValue2["type"];
                                                if (typeValue3 != null && typeValue3.Type != JTokenType.Null)
                                                {
                                                    string typeInstance3 = ((string)typeValue3);
                                                    referenceInputPropertiesInstance.Type = typeInstance3;
                                                }
                                                
                                                JToken serializationValue = propertiesValue2["serialization"];
                                                if (serializationValue != null && serializationValue.Type != JTokenType.Null)
                                                {
                                                    string typeName3 = ((string)serializationValue["type"]);
                                                    if (typeName3 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance = new CsvSerialization();
                                                        
                                                        JToken propertiesValue4 = serializationValue["properties"];
                                                        if (propertiesValue4 != null && propertiesValue4.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance3 = new CsvSerializationProperties();
                                                            csvSerializationInstance.Properties = propertiesInstance3;
                                                            
                                                            JToken fieldDelimiterValue = propertiesValue4["fieldDelimiter"];
                                                            if (fieldDelimiterValue != null && fieldDelimiterValue.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance = ((string)fieldDelimiterValue);
                                                                propertiesInstance3.FieldDelimiter = fieldDelimiterInstance;
                                                            }
                                                            
                                                            JToken encodingValue = propertiesValue4["encoding"];
                                                            if (encodingValue != null && encodingValue.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance = ((string)encodingValue);
                                                                propertiesInstance3.Encoding = encodingInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue4 = serializationValue["type"];
                                                        if (typeValue4 != null && typeValue4.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance4 = ((string)typeValue4);
                                                            csvSerializationInstance.Type = typeInstance4;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = csvSerializationInstance;
                                                    }
                                                    if (typeName3 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance = new JsonSerialization();
                                                        
                                                        JToken propertiesValue5 = serializationValue["properties"];
                                                        if (propertiesValue5 != null && propertiesValue5.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance4 = new JsonSerializationProperties();
                                                            jsonSerializationInstance.Properties = propertiesInstance4;
                                                            
                                                            JToken encodingValue2 = propertiesValue5["encoding"];
                                                            if (encodingValue2 != null && encodingValue2.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance2 = ((string)encodingValue2);
                                                                propertiesInstance4.Encoding = encodingInstance2;
                                                            }
                                                            
                                                            JToken formatValue = propertiesValue5["format"];
                                                            if (formatValue != null && formatValue.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance = ((string)formatValue);
                                                                propertiesInstance4.Format = formatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue5 = serializationValue["type"];
                                                        if (typeValue5 != null && typeValue5.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance5 = ((string)typeValue5);
                                                            jsonSerializationInstance.Type = typeInstance5;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = jsonSerializationInstance;
                                                    }
                                                    if (typeName3 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance = new AvroSerialization();
                                                        
                                                        JToken propertiesValue6 = serializationValue["properties"];
                                                        if (propertiesValue6 != null && propertiesValue6.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance5 = new AvroSerializationProperties();
                                                            avroSerializationInstance.Properties = propertiesInstance5;
                                                        }
                                                        
                                                        JToken typeValue6 = serializationValue["type"];
                                                        if (typeValue6 != null && typeValue6.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance6 = ((string)typeValue6);
                                                            avroSerializationInstance.Type = typeInstance6;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = avroSerializationInstance;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue = propertiesValue2["diagnostics"];
                                                if (diagnosticsValue != null && diagnosticsValue.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance = new Diagnostics();
                                                    referenceInputPropertiesInstance.Diagnostics = diagnosticsInstance;
                                                    
                                                    JToken conditionsArray = diagnosticsValue["conditions"];
                                                    if (conditionsArray != null && conditionsArray.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue in ((JArray)conditionsArray))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance = new DiagnosticCondition();
                                                            diagnosticsInstance.Conditions.Add(diagnosticConditionInstance);
                                                            
                                                            JToken sinceValue = conditionsValue["since"];
                                                            if (sinceValue != null && sinceValue.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance = ((DateTime)sinceValue);
                                                                diagnosticConditionInstance.Since = sinceInstance;
                                                            }
                                                            
                                                            JToken codeValue = conditionsValue["code"];
                                                            if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance = ((string)codeValue);
                                                                diagnosticConditionInstance.Code = codeInstance;
                                                            }
                                                            
                                                            JToken messageValue = conditionsValue["message"];
                                                            if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance = ((string)messageValue);
                                                                diagnosticConditionInstance.Message = messageInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = referenceInputPropertiesInstance;
                                            }
                                            if (typeName == "Stream")
                                            {
                                                StreamInputProperties streamInputPropertiesInstance = new StreamInputProperties();
                                                
                                                JToken datasourceValue2 = propertiesValue2["datasource"];
                                                if (datasourceValue2 != null && datasourceValue2.Type != JTokenType.Null)
                                                {
                                                    string typeName4 = ((string)datasourceValue2["type"]);
                                                    if (typeName4 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobStreamInputDataSource blobStreamInputDataSourceInstance = new BlobStreamInputDataSource();
                                                        
                                                        JToken propertiesValue7 = datasourceValue2["properties"];
                                                        if (propertiesValue7 != null && propertiesValue7.Type != JTokenType.Null)
                                                        {
                                                            BlobStreamInputDataSourceProperties propertiesInstance6 = new BlobStreamInputDataSourceProperties();
                                                            blobStreamInputDataSourceInstance.Properties = propertiesInstance6;
                                                            
                                                            JToken storageAccountsArray2 = propertiesValue7["storageAccounts"];
                                                            if (storageAccountsArray2 != null && storageAccountsArray2.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance6.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue2 in ((JArray)storageAccountsArray2))
                                                                {
                                                                    StorageAccount storageAccountInstance2 = new StorageAccount();
                                                                    propertiesInstance6.StorageAccounts.Add(storageAccountInstance2);
                                                                    
                                                                    JToken accountNameValue2 = storageAccountsValue2["accountName"];
                                                                    if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance2 = ((string)accountNameValue2);
                                                                        storageAccountInstance2.AccountName = accountNameInstance2;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue2 = storageAccountsValue2["accountKey"];
                                                                    if (accountKeyValue2 != null && accountKeyValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance2 = ((string)accountKeyValue2);
                                                                        storageAccountInstance2.AccountKey = accountKeyInstance2;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue2 = propertiesValue7["container"];
                                                            if (containerValue2 != null && containerValue2.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance2 = ((string)containerValue2);
                                                                propertiesInstance6.Container = containerInstance2;
                                                            }
                                                            
                                                            JToken pathPatternValue2 = propertiesValue7["pathPattern"];
                                                            if (pathPatternValue2 != null && pathPatternValue2.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance2 = ((string)pathPatternValue2);
                                                                propertiesInstance6.PathPattern = pathPatternInstance2;
                                                            }
                                                            
                                                            JToken dateFormatValue2 = propertiesValue7["dateFormat"];
                                                            if (dateFormatValue2 != null && dateFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance2 = ((string)dateFormatValue2);
                                                                propertiesInstance6.DateFormat = dateFormatInstance2;
                                                            }
                                                            
                                                            JToken timeFormatValue2 = propertiesValue7["timeFormat"];
                                                            if (timeFormatValue2 != null && timeFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance2 = ((string)timeFormatValue2);
                                                                propertiesInstance6.TimeFormat = timeFormatInstance2;
                                                            }
                                                            
                                                            JToken sourcePartitionCountValue = propertiesValue7["sourcePartitionCount"];
                                                            if (sourcePartitionCountValue != null && sourcePartitionCountValue.Type != JTokenType.Null)
                                                            {
                                                                int sourcePartitionCountInstance = ((int)sourcePartitionCountValue);
                                                                propertiesInstance6.SourcePartitionCount = sourcePartitionCountInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue7 = datasourceValue2["type"];
                                                        if (typeValue7 != null && typeValue7.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance7 = ((string)typeValue7);
                                                            blobStreamInputDataSourceInstance.Type = typeInstance7;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = blobStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.ServiceBus/EventHub")
                                                    {
                                                        EventHubStreamInputDataSource eventHubStreamInputDataSourceInstance = new EventHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue8 = datasourceValue2["properties"];
                                                        if (propertiesValue8 != null && propertiesValue8.Type != JTokenType.Null)
                                                        {
                                                            EventHubStreamInputDataSourceProperties propertiesInstance7 = new EventHubStreamInputDataSourceProperties();
                                                            eventHubStreamInputDataSourceInstance.Properties = propertiesInstance7;
                                                            
                                                            JToken serviceBusNamespaceValue = propertiesValue8["serviceBusNamespace"];
                                                            if (serviceBusNamespaceValue != null && serviceBusNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string serviceBusNamespaceInstance = ((string)serviceBusNamespaceValue);
                                                                propertiesInstance7.ServiceBusNamespace = serviceBusNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue = propertiesValue8["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue != null && sharedAccessPolicyNameValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance = ((string)sharedAccessPolicyNameValue);
                                                                propertiesInstance7.SharedAccessPolicyName = sharedAccessPolicyNameInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue = propertiesValue8["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue != null && sharedAccessPolicyKeyValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance = ((string)sharedAccessPolicyKeyValue);
                                                                propertiesInstance7.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance;
                                                            }
                                                            
                                                            JToken eventHubNameValue = propertiesValue8["eventHubName"];
                                                            if (eventHubNameValue != null && eventHubNameValue.Type != JTokenType.Null)
                                                            {
                                                                string eventHubNameInstance = ((string)eventHubNameValue);
                                                                propertiesInstance7.EventHubName = eventHubNameInstance;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue = propertiesValue8["consumerGroupName"];
                                                            if (consumerGroupNameValue != null && consumerGroupNameValue.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance = ((string)consumerGroupNameValue);
                                                                propertiesInstance7.ConsumerGroupName = consumerGroupNameInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue8 = datasourceValue2["type"];
                                                        if (typeValue8 != null && typeValue8.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance8 = ((string)typeValue8);
                                                            eventHubStreamInputDataSourceInstance.Type = typeInstance8;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = eventHubStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.Devices/IotHubs")
                                                    {
                                                        IoTHubStreamInputDataSource ioTHubStreamInputDataSourceInstance = new IoTHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue9 = datasourceValue2["properties"];
                                                        if (propertiesValue9 != null && propertiesValue9.Type != JTokenType.Null)
                                                        {
                                                            IoTHubStreamInputDataSourceProperties propertiesInstance8 = new IoTHubStreamInputDataSourceProperties();
                                                            ioTHubStreamInputDataSourceInstance.Properties = propertiesInstance8;
                                                            
                                                            JToken iotHubNamespaceValue = propertiesValue9["iotHubNamespace"];
                                                            if (iotHubNamespaceValue != null && iotHubNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string iotHubNamespaceInstance = ((string)iotHubNamespaceValue);
                                                                propertiesInstance8.IotHubNamespace = iotHubNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue2 = propertiesValue9["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue2 != null && sharedAccessPolicyNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance2 = ((string)sharedAccessPolicyNameValue2);
                                                                propertiesInstance8.SharedAccessPolicyName = sharedAccessPolicyNameInstance2;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue2 = propertiesValue9["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue2 != null && sharedAccessPolicyKeyValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance2 = ((string)sharedAccessPolicyKeyValue2);
                                                                propertiesInstance8.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance2;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue2 = propertiesValue9["consumerGroupName"];
                                                            if (consumerGroupNameValue2 != null && consumerGroupNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance2 = ((string)consumerGroupNameValue2);
                                                                propertiesInstance8.ConsumerGroupName = consumerGroupNameInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue9 = datasourceValue2["type"];
                                                        if (typeValue9 != null && typeValue9.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance9 = ((string)typeValue9);
                                                            ioTHubStreamInputDataSourceInstance.Type = typeInstance9;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = ioTHubStreamInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue3 = propertiesValue2["etag"];
                                                if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                                {
                                                    string etagInstance3 = ((string)etagValue3);
                                                    streamInputPropertiesInstance.Etag = etagInstance3;
                                                }
                                                
                                                JToken typeValue10 = propertiesValue2["type"];
                                                if (typeValue10 != null && typeValue10.Type != JTokenType.Null)
                                                {
                                                    string typeInstance10 = ((string)typeValue10);
                                                    streamInputPropertiesInstance.Type = typeInstance10;
                                                }
                                                
                                                JToken serializationValue2 = propertiesValue2["serialization"];
                                                if (serializationValue2 != null && serializationValue2.Type != JTokenType.Null)
                                                {
                                                    string typeName5 = ((string)serializationValue2["type"]);
                                                    if (typeName5 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance2 = new CsvSerialization();
                                                        
                                                        JToken propertiesValue10 = serializationValue2["properties"];
                                                        if (propertiesValue10 != null && propertiesValue10.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance9 = new CsvSerializationProperties();
                                                            csvSerializationInstance2.Properties = propertiesInstance9;
                                                            
                                                            JToken fieldDelimiterValue2 = propertiesValue10["fieldDelimiter"];
                                                            if (fieldDelimiterValue2 != null && fieldDelimiterValue2.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance2 = ((string)fieldDelimiterValue2);
                                                                propertiesInstance9.FieldDelimiter = fieldDelimiterInstance2;
                                                            }
                                                            
                                                            JToken encodingValue3 = propertiesValue10["encoding"];
                                                            if (encodingValue3 != null && encodingValue3.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance3 = ((string)encodingValue3);
                                                                propertiesInstance9.Encoding = encodingInstance3;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue11 = serializationValue2["type"];
                                                        if (typeValue11 != null && typeValue11.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance11 = ((string)typeValue11);
                                                            csvSerializationInstance2.Type = typeInstance11;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = csvSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance2 = new JsonSerialization();
                                                        
                                                        JToken propertiesValue11 = serializationValue2["properties"];
                                                        if (propertiesValue11 != null && propertiesValue11.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance10 = new JsonSerializationProperties();
                                                            jsonSerializationInstance2.Properties = propertiesInstance10;
                                                            
                                                            JToken encodingValue4 = propertiesValue11["encoding"];
                                                            if (encodingValue4 != null && encodingValue4.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance4 = ((string)encodingValue4);
                                                                propertiesInstance10.Encoding = encodingInstance4;
                                                            }
                                                            
                                                            JToken formatValue2 = propertiesValue11["format"];
                                                            if (formatValue2 != null && formatValue2.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance2 = ((string)formatValue2);
                                                                propertiesInstance10.Format = formatInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue12 = serializationValue2["type"];
                                                        if (typeValue12 != null && typeValue12.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance12 = ((string)typeValue12);
                                                            jsonSerializationInstance2.Type = typeInstance12;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = jsonSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance2 = new AvroSerialization();
                                                        
                                                        JToken propertiesValue12 = serializationValue2["properties"];
                                                        if (propertiesValue12 != null && propertiesValue12.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance11 = new AvroSerializationProperties();
                                                            avroSerializationInstance2.Properties = propertiesInstance11;
                                                        }
                                                        
                                                        JToken typeValue13 = serializationValue2["type"];
                                                        if (typeValue13 != null && typeValue13.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance13 = ((string)typeValue13);
                                                            avroSerializationInstance2.Type = typeInstance13;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = avroSerializationInstance2;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue2 = propertiesValue2["diagnostics"];
                                                if (diagnosticsValue2 != null && diagnosticsValue2.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance2 = new Diagnostics();
                                                    streamInputPropertiesInstance.Diagnostics = diagnosticsInstance2;
                                                    
                                                    JToken conditionsArray2 = diagnosticsValue2["conditions"];
                                                    if (conditionsArray2 != null && conditionsArray2.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue2 in ((JArray)conditionsArray2))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance2 = new DiagnosticCondition();
                                                            diagnosticsInstance2.Conditions.Add(diagnosticConditionInstance2);
                                                            
                                                            JToken sinceValue2 = conditionsValue2["since"];
                                                            if (sinceValue2 != null && sinceValue2.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance2 = ((DateTime)sinceValue2);
                                                                diagnosticConditionInstance2.Since = sinceInstance2;
                                                            }
                                                            
                                                            JToken codeValue2 = conditionsValue2["code"];
                                                            if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance2 = ((string)codeValue2);
                                                                diagnosticConditionInstance2.Code = codeInstance2;
                                                            }
                                                            
                                                            JToken messageValue2 = conditionsValue2["message"];
                                                            if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance2 = ((string)messageValue2);
                                                                diagnosticConditionInstance2.Message = messageInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = streamInputPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                                
                                JToken transformationValue = propertiesValue["transformation"];
                                if (transformationValue != null && transformationValue.Type != JTokenType.Null)
                                {
                                    Transformation transformationInstance = new Transformation();
                                    propertiesInstance.Transformation = transformationInstance;
                                    
                                    JToken nameValue4 = transformationValue["name"];
                                    if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                    {
                                        string nameInstance4 = ((string)nameValue4);
                                        transformationInstance.Name = nameInstance4;
                                    }
                                    
                                    JToken propertiesValue13 = transformationValue["properties"];
                                    if (propertiesValue13 != null && propertiesValue13.Type != JTokenType.Null)
                                    {
                                        TransformationProperties propertiesInstance12 = new TransformationProperties();
                                        transformationInstance.Properties = propertiesInstance12;
                                        
                                        JToken etagValue4 = propertiesValue13["etag"];
                                        if (etagValue4 != null && etagValue4.Type != JTokenType.Null)
                                        {
                                            string etagInstance4 = ((string)etagValue4);
                                            propertiesInstance12.Etag = etagInstance4;
                                        }
                                        
                                        JToken streamingUnitsValue = propertiesValue13["streamingUnits"];
                                        if (streamingUnitsValue != null && streamingUnitsValue.Type != JTokenType.Null)
                                        {
                                            int streamingUnitsInstance = ((int)streamingUnitsValue);
                                            propertiesInstance12.StreamingUnits = streamingUnitsInstance;
                                        }
                                        
                                        JToken queryValue = propertiesValue13["query"];
                                        if (queryValue != null && queryValue.Type != JTokenType.Null)
                                        {
                                            string queryInstance = ((string)queryValue);
                                            propertiesInstance12.Query = queryInstance;
                                        }
                                    }
                                }
                                
                                JToken outputsArray = propertiesValue["outputs"];
                                if (outputsArray != null && outputsArray.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Outputs = new List<Output>();
                                    foreach (JToken outputsValue in ((JArray)outputsArray))
                                    {
                                        Output outputInstance = new Output();
                                        propertiesInstance.Outputs.Add(outputInstance);
                                        
                                        JToken nameValue5 = outputsValue["name"];
                                        if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                        {
                                            string nameInstance5 = ((string)nameValue5);
                                            outputInstance.Name = nameInstance5;
                                        }
                                        
                                        JToken propertiesValue14 = outputsValue["properties"];
                                        if (propertiesValue14 != null && propertiesValue14.Type != JTokenType.Null)
                                        {
                                            OutputProperties propertiesInstance13 = new OutputProperties();
                                            outputInstance.Properties = propertiesInstance13;
                                            
                                            JToken etagValue5 = propertiesValue14["etag"];
                                            if (etagValue5 != null && etagValue5.Type != JTokenType.Null)
                                            {
                                                string etagInstance5 = ((string)etagValue5);
                                                propertiesInstance13.Etag = etagInstance5;
                                            }
                                            
                                            JToken datasourceValue3 = propertiesValue14["datasource"];
                                            if (datasourceValue3 != null && datasourceValue3.Type != JTokenType.Null)
                                            {
                                                string typeName6 = ((string)datasourceValue3["type"]);
                                                if (typeName6 == "Microsoft.Storage/Blob")
                                                {
                                                    BlobOutputDataSource blobOutputDataSourceInstance = new BlobOutputDataSource();
                                                    
                                                    JToken propertiesValue15 = datasourceValue3["properties"];
                                                    if (propertiesValue15 != null && propertiesValue15.Type != JTokenType.Null)
                                                    {
                                                        BlobOutputDataSourceProperties propertiesInstance14 = new BlobOutputDataSourceProperties();
                                                        blobOutputDataSourceInstance.Properties = propertiesInstance14;
                                                        
                                                        JToken storageAccountsArray3 = propertiesValue15["storageAccounts"];
                                                        if (storageAccountsArray3 != null && storageAccountsArray3.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance14.StorageAccounts = new System.Collections.Generic.List<Microsoft.Azure.Management.StreamAnalytics.Models.StorageAccount>();
                                                            foreach (JToken storageAccountsValue3 in ((JArray)storageAccountsArray3))
                                                            {
                                                                StorageAccount storageAccountInstance3 = new StorageAccount();
                                                                propertiesInstance14.StorageAccounts.Add(storageAccountInstance3);
                                                                
                                                                JToken accountNameValue3 = storageAccountsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    storageAccountInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken accountKeyValue3 = storageAccountsValue3["accountKey"];
                                                                if (accountKeyValue3 != null && accountKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance3 = ((string)accountKeyValue3);
                                                                    storageAccountInstance3.AccountKey = accountKeyInstance3;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken containerValue3 = propertiesValue15["container"];
                                                        if (containerValue3 != null && containerValue3.Type != JTokenType.Null)
                                                        {
                                                            string containerInstance3 = ((string)containerValue3);
                                                            propertiesInstance14.Container = containerInstance3;
                                                        }
                                                        
                                                        JToken pathPatternValue3 = propertiesValue15["pathPattern"];
                                                        if (pathPatternValue3 != null && pathPatternValue3.Type != JTokenType.Null)
                                                        {
                                                            string pathPatternInstance3 = ((string)pathPatternValue3);
                                                            propertiesInstance14.PathPattern = pathPatternInstance3;
                                                        }
                                                        
                                                        JToken dateFormatValue3 = propertiesValue15["dateFormat"];
                                                        if (dateFormatValue3 != null && dateFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string dateFormatInstance3 = ((string)dateFormatValue3);
                                                            propertiesInstance14.DateFormat = dateFormatInstance3;
                                                        }
                                                        
                                                        JToken timeFormatValue3 = propertiesValue15["timeFormat"];
                                                        if (timeFormatValue3 != null && timeFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string timeFormatInstance3 = ((string)timeFormatValue3);
                                                            propertiesInstance14.TimeFormat = timeFormatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue14 = datasourceValue3["type"];
                                                    if (typeValue14 != null && typeValue14.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance14 = ((string)typeValue14);
                                                        blobOutputDataSourceInstance.Type = typeInstance14;
                                                    }
                                                    propertiesInstance13.DataSource = blobOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/Table")
                                                {
                                                    AzureTableOutputDataSource azureTableOutputDataSourceInstance = new AzureTableOutputDataSource();
                                                    
                                                    JToken propertiesValue16 = datasourceValue3["properties"];
                                                    if (propertiesValue16 != null && propertiesValue16.Type != JTokenType.Null)
                                                    {
                                                        AzureTableOutputDataSourceProperties propertiesInstance15 = new AzureTableOutputDataSourceProperties();
                                                        azureTableOutputDataSourceInstance.Properties = propertiesInstance15;
                                                        
                                                        JToken accountNameValue4 = propertiesValue16["accountName"];
                                                        if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountNameInstance4 = ((string)accountNameValue4);
                                                            propertiesInstance15.AccountName = accountNameInstance4;
                                                        }
                                                        
                                                        JToken accountKeyValue4 = propertiesValue16["accountKey"];
                                                        if (accountKeyValue4 != null && accountKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance4 = ((string)accountKeyValue4);
                                                            propertiesInstance15.AccountKey = accountKeyInstance4;
                                                        }
                                                        
                                                        JToken tableValue = propertiesValue16["table"];
                                                        if (tableValue != null && tableValue.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance = ((string)tableValue);
                                                            propertiesInstance15.Table = tableInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue = propertiesValue16["partitionKey"];
                                                        if (partitionKeyValue != null && partitionKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance = ((string)partitionKeyValue);
                                                            propertiesInstance15.PartitionKey = partitionKeyInstance;
                                                        }
                                                        
                                                        JToken rowKeyValue = propertiesValue16["rowKey"];
                                                        if (rowKeyValue != null && rowKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string rowKeyInstance = ((string)rowKeyValue);
                                                            propertiesInstance15.RowKey = rowKeyInstance;
                                                        }
                                                        
                                                        JToken columnsToRemoveArray = propertiesValue16["columnsToRemove"];
                                                        if (columnsToRemoveArray != null && columnsToRemoveArray.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance15.ColumnsToRemove = new System.Collections.Generic.List<string>();
                                                            foreach (JToken columnsToRemoveValue in ((JArray)columnsToRemoveArray))
                                                            {
                                                                propertiesInstance15.ColumnsToRemove.Add(((string)columnsToRemoveValue));
                                                            }
                                                        }
                                                        
                                                        JToken batchSizeValue = propertiesValue16["batchSize"];
                                                        if (batchSizeValue != null && batchSizeValue.Type != JTokenType.Null)
                                                        {
                                                            int batchSizeInstance = ((int)batchSizeValue);
                                                            propertiesInstance15.BatchSize = batchSizeInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue15 = datasourceValue3["type"];
                                                    if (typeValue15 != null && typeValue15.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance15 = ((string)typeValue15);
                                                        azureTableOutputDataSourceInstance.Type = typeInstance15;
                                                    }
                                                    propertiesInstance13.DataSource = azureTableOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/EventHub")
                                                {
                                                    EventHubOutputDataSource eventHubOutputDataSourceInstance = new EventHubOutputDataSource();
                                                    
                                                    JToken propertiesValue17 = datasourceValue3["properties"];
                                                    if (propertiesValue17 != null && propertiesValue17.Type != JTokenType.Null)
                                                    {
                                                        EventHubOutputDataSourceProperties propertiesInstance16 = new EventHubOutputDataSourceProperties();
                                                        eventHubOutputDataSourceInstance.Properties = propertiesInstance16;
                                                        
                                                        JToken serviceBusNamespaceValue2 = propertiesValue17["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue2 != null && serviceBusNamespaceValue2.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance2 = ((string)serviceBusNamespaceValue2);
                                                            propertiesInstance16.ServiceBusNamespace = serviceBusNamespaceInstance2;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue3 = propertiesValue17["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue3 != null && sharedAccessPolicyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance3 = ((string)sharedAccessPolicyNameValue3);
                                                            propertiesInstance16.SharedAccessPolicyName = sharedAccessPolicyNameInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue3 = propertiesValue17["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue3 != null && sharedAccessPolicyKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance3 = ((string)sharedAccessPolicyKeyValue3);
                                                            propertiesInstance16.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance3;
                                                        }
                                                        
                                                        JToken eventHubNameValue2 = propertiesValue17["eventHubName"];
                                                        if (eventHubNameValue2 != null && eventHubNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string eventHubNameInstance2 = ((string)eventHubNameValue2);
                                                            propertiesInstance16.EventHubName = eventHubNameInstance2;
                                                        }
                                                        
                                                        JToken partitionKeyValue2 = propertiesValue17["partitionKey"];
                                                        if (partitionKeyValue2 != null && partitionKeyValue2.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance2 = ((string)partitionKeyValue2);
                                                            propertiesInstance16.PartitionKey = partitionKeyInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue16 = datasourceValue3["type"];
                                                    if (typeValue16 != null && typeValue16.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance16 = ((string)typeValue16);
                                                        eventHubOutputDataSourceInstance.Type = typeInstance16;
                                                    }
                                                    propertiesInstance13.DataSource = eventHubOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Queue")
                                                {
                                                    ServiceBusQueueOutputDataSource serviceBusQueueOutputDataSourceInstance = new ServiceBusQueueOutputDataSource();
                                                    
                                                    JToken propertiesValue18 = datasourceValue3["properties"];
                                                    if (propertiesValue18 != null && propertiesValue18.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusQueueOutputDataSourceProperties propertiesInstance17 = new ServiceBusQueueOutputDataSourceProperties();
                                                        serviceBusQueueOutputDataSourceInstance.Properties = propertiesInstance17;
                                                        
                                                        JToken serviceBusNamespaceValue3 = propertiesValue18["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue3 != null && serviceBusNamespaceValue3.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance3 = ((string)serviceBusNamespaceValue3);
                                                            propertiesInstance17.ServiceBusNamespace = serviceBusNamespaceInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue4 = propertiesValue18["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue4 != null && sharedAccessPolicyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance4 = ((string)sharedAccessPolicyNameValue4);
                                                            propertiesInstance17.SharedAccessPolicyName = sharedAccessPolicyNameInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue4 = propertiesValue18["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue4 != null && sharedAccessPolicyKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance4 = ((string)sharedAccessPolicyKeyValue4);
                                                            propertiesInstance17.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance4;
                                                        }
                                                        
                                                        JToken queueNameValue = propertiesValue18["queueName"];
                                                        if (queueNameValue != null && queueNameValue.Type != JTokenType.Null)
                                                        {
                                                            string queueNameInstance = ((string)queueNameValue);
                                                            propertiesInstance17.QueueName = queueNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue17 = datasourceValue3["type"];
                                                    if (typeValue17 != null && typeValue17.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance17 = ((string)typeValue17);
                                                        serviceBusQueueOutputDataSourceInstance.Type = typeInstance17;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusQueueOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Topic")
                                                {
                                                    ServiceBusTopicOutputDataSource serviceBusTopicOutputDataSourceInstance = new ServiceBusTopicOutputDataSource();
                                                    
                                                    JToken propertiesValue19 = datasourceValue3["properties"];
                                                    if (propertiesValue19 != null && propertiesValue19.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusTopicOutputDataSourceProperties propertiesInstance18 = new ServiceBusTopicOutputDataSourceProperties();
                                                        serviceBusTopicOutputDataSourceInstance.Properties = propertiesInstance18;
                                                        
                                                        JToken serviceBusNamespaceValue4 = propertiesValue19["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue4 != null && serviceBusNamespaceValue4.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance4 = ((string)serviceBusNamespaceValue4);
                                                            propertiesInstance18.ServiceBusNamespace = serviceBusNamespaceInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue5 = propertiesValue19["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue5 != null && sharedAccessPolicyNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance5 = ((string)sharedAccessPolicyNameValue5);
                                                            propertiesInstance18.SharedAccessPolicyName = sharedAccessPolicyNameInstance5;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue5 = propertiesValue19["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue5 != null && sharedAccessPolicyKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance5 = ((string)sharedAccessPolicyKeyValue5);
                                                            propertiesInstance18.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance5;
                                                        }
                                                        
                                                        JToken topicNameValue = propertiesValue19["topicName"];
                                                        if (topicNameValue != null && topicNameValue.Type != JTokenType.Null)
                                                        {
                                                            string topicNameInstance = ((string)topicNameValue);
                                                            propertiesInstance18.TopicName = topicNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue18 = datasourceValue3["type"];
                                                    if (typeValue18 != null && typeValue18.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance18 = ((string)typeValue18);
                                                        serviceBusTopicOutputDataSourceInstance.Type = typeInstance18;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusTopicOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Sql/Server/Database")
                                                {
                                                    SqlAzureOutputDataSource sqlAzureOutputDataSourceInstance = new SqlAzureOutputDataSource();
                                                    
                                                    JToken propertiesValue20 = datasourceValue3["properties"];
                                                    if (propertiesValue20 != null && propertiesValue20.Type != JTokenType.Null)
                                                    {
                                                        SqlAzureOutputDataSourceProperties propertiesInstance19 = new SqlAzureOutputDataSourceProperties();
                                                        sqlAzureOutputDataSourceInstance.Properties = propertiesInstance19;
                                                        
                                                        JToken serverValue = propertiesValue20["server"];
                                                        if (serverValue != null && serverValue.Type != JTokenType.Null)
                                                        {
                                                            string serverInstance = ((string)serverValue);
                                                            propertiesInstance19.Server = serverInstance;
                                                        }
                                                        
                                                        JToken databaseValue = propertiesValue20["database"];
                                                        if (databaseValue != null && databaseValue.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance = ((string)databaseValue);
                                                            propertiesInstance19.Database = databaseInstance;
                                                        }
                                                        
                                                        JToken userValue = propertiesValue20["user"];
                                                        if (userValue != null && userValue.Type != JTokenType.Null)
                                                        {
                                                            string userInstance = ((string)userValue);
                                                            propertiesInstance19.User = userInstance;
                                                        }
                                                        
                                                        JToken passwordValue = propertiesValue20["password"];
                                                        if (passwordValue != null && passwordValue.Type != JTokenType.Null)
                                                        {
                                                            string passwordInstance = ((string)passwordValue);
                                                            propertiesInstance19.Password = passwordInstance;
                                                        }
                                                        
                                                        JToken tableValue2 = propertiesValue20["table"];
                                                        if (tableValue2 != null && tableValue2.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance2 = ((string)tableValue2);
                                                            propertiesInstance19.Table = tableInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue19 = datasourceValue3["type"];
                                                    if (typeValue19 != null && typeValue19.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance19 = ((string)typeValue19);
                                                        sqlAzureOutputDataSourceInstance.Type = typeInstance19;
                                                    }
                                                    propertiesInstance13.DataSource = sqlAzureOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/DocumentDB")
                                                {
                                                    DocumentDbOutputDataSource documentDbOutputDataSourceInstance = new DocumentDbOutputDataSource();
                                                    
                                                    JToken propertiesValue21 = datasourceValue3["properties"];
                                                    if (propertiesValue21 != null && propertiesValue21.Type != JTokenType.Null)
                                                    {
                                                        DocumentDbOutputDataSourceProperties propertiesInstance20 = new DocumentDbOutputDataSourceProperties();
                                                        documentDbOutputDataSourceInstance.Properties = propertiesInstance20;
                                                        
                                                        JToken accountIdValue = propertiesValue21["accountId"];
                                                        if (accountIdValue != null && accountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string accountIdInstance = ((string)accountIdValue);
                                                            propertiesInstance20.AccountId = accountIdInstance;
                                                        }
                                                        
                                                        JToken accountKeyValue5 = propertiesValue21["accountKey"];
                                                        if (accountKeyValue5 != null && accountKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance5 = ((string)accountKeyValue5);
                                                            propertiesInstance20.AccountKey = accountKeyInstance5;
                                                        }
                                                        
                                                        JToken databaseValue2 = propertiesValue21["database"];
                                                        if (databaseValue2 != null && databaseValue2.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance2 = ((string)databaseValue2);
                                                            propertiesInstance20.Database = databaseInstance2;
                                                        }
                                                        
                                                        JToken collectionNamePatternValue = propertiesValue21["collectionNamePattern"];
                                                        if (collectionNamePatternValue != null && collectionNamePatternValue.Type != JTokenType.Null)
                                                        {
                                                            string collectionNamePatternInstance = ((string)collectionNamePatternValue);
                                                            propertiesInstance20.CollectionNamePattern = collectionNamePatternInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue3 = propertiesValue21["partitionKey"];
                                                        if (partitionKeyValue3 != null && partitionKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance3 = ((string)partitionKeyValue3);
                                                            propertiesInstance20.PartitionKey = partitionKeyInstance3;
                                                        }
                                                        
                                                        JToken documentIdValue = propertiesValue21["documentId"];
                                                        if (documentIdValue != null && documentIdValue.Type != JTokenType.Null)
                                                        {
                                                            string documentIdInstance = ((string)documentIdValue);
                                                            propertiesInstance20.DocumentId = documentIdInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue20 = datasourceValue3["type"];
                                                    if (typeValue20 != null && typeValue20.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance20 = ((string)typeValue20);
                                                        documentDbOutputDataSourceInstance.Type = typeInstance20;
                                                    }
                                                    propertiesInstance13.DataSource = documentDbOutputDataSourceInstance;
                                                }
                                            }
                                            
                                            JToken serializationValue3 = propertiesValue14["serialization"];
                                            if (serializationValue3 != null && serializationValue3.Type != JTokenType.Null)
                                            {
                                                string typeName7 = ((string)serializationValue3["type"]);
                                                if (typeName7 == "Csv")
                                                {
                                                    CsvSerialization csvSerializationInstance3 = new CsvSerialization();
                                                    
                                                    JToken propertiesValue22 = serializationValue3["properties"];
                                                    if (propertiesValue22 != null && propertiesValue22.Type != JTokenType.Null)
                                                    {
                                                        CsvSerializationProperties propertiesInstance21 = new CsvSerializationProperties();
                                                        csvSerializationInstance3.Properties = propertiesInstance21;
                                                        
                                                        JToken fieldDelimiterValue3 = propertiesValue22["fieldDelimiter"];
                                                        if (fieldDelimiterValue3 != null && fieldDelimiterValue3.Type != JTokenType.Null)
                                                        {
                                                            string fieldDelimiterInstance3 = ((string)fieldDelimiterValue3);
                                                            propertiesInstance21.FieldDelimiter = fieldDelimiterInstance3;
                                                        }
                                                        
                                                        JToken encodingValue5 = propertiesValue22["encoding"];
                                                        if (encodingValue5 != null && encodingValue5.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance5 = ((string)encodingValue5);
                                                            propertiesInstance21.Encoding = encodingInstance5;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue21 = serializationValue3["type"];
                                                    if (typeValue21 != null && typeValue21.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance21 = ((string)typeValue21);
                                                        csvSerializationInstance3.Type = typeInstance21;
                                                    }
                                                    propertiesInstance13.Serialization = csvSerializationInstance3;
                                                }
                                                if (typeName7 == "Json")
                                                {
                                                    JsonSerialization jsonSerializationInstance3 = new JsonSerialization();
                                                    
                                                    JToken propertiesValue23 = serializationValue3["properties"];
                                                    if (propertiesValue23 != null && propertiesValue23.Type != JTokenType.Null)
                                                    {
                                                        JsonSerializationProperties propertiesInstance22 = new JsonSerializationProperties();
                                                        jsonSerializationInstance3.Properties = propertiesInstance22;
                                                        
                                                        JToken encodingValue6 = propertiesValue23["encoding"];
                                                        if (encodingValue6 != null && encodingValue6.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance6 = ((string)encodingValue6);
                                                            propertiesInstance22.Encoding = encodingInstance6;
                                                        }
                                                        
                                                        JToken formatValue3 = propertiesValue23["format"];
                                                        if (formatValue3 != null && formatValue3.Type != JTokenType.Null)
                                                        {
                                                            string formatInstance3 = ((string)formatValue3);
                                                            propertiesInstance22.Format = formatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue22 = serializationValue3["type"];
                                                    if (typeValue22 != null && typeValue22.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance22 = ((string)typeValue22);
                                                        jsonSerializationInstance3.Type = typeInstance22;
                                                    }
                                                    propertiesInstance13.Serialization = jsonSerializationInstance3;
                                                }
                                                if (typeName7 == "Avro")
                                                {
                                                    AvroSerialization avroSerializationInstance3 = new AvroSerialization();
                                                    
                                                    JToken propertiesValue24 = serializationValue3["properties"];
                                                    if (propertiesValue24 != null && propertiesValue24.Type != JTokenType.Null)
                                                    {
                                                        AvroSerializationProperties propertiesInstance23 = new AvroSerializationProperties();
                                                        avroSerializationInstance3.Properties = propertiesInstance23;
                                                    }
                                                    
                                                    JToken typeValue23 = serializationValue3["type"];
                                                    if (typeValue23 != null && typeValue23.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance23 = ((string)typeValue23);
                                                        avroSerializationInstance3.Type = typeInstance23;
                                                    }
                                                    propertiesInstance13.Serialization = avroSerializationInstance3;
                                                }
                                            }
                                            
                                            JToken diagnosticsValue3 = propertiesValue14["diagnostics"];
                                            if (diagnosticsValue3 != null && diagnosticsValue3.Type != JTokenType.Null)
                                            {
                                                Diagnostics diagnosticsInstance3 = new Diagnostics();
                                                propertiesInstance13.Diagnostics = diagnosticsInstance3;
                                                
                                                JToken conditionsArray3 = diagnosticsValue3["conditions"];
                                                if (conditionsArray3 != null && conditionsArray3.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken conditionsValue3 in ((JArray)conditionsArray3))
                                                    {
                                                        DiagnosticCondition diagnosticConditionInstance3 = new DiagnosticCondition();
                                                        diagnosticsInstance3.Conditions.Add(diagnosticConditionInstance3);
                                                        
                                                        JToken sinceValue3 = conditionsValue3["since"];
                                                        if (sinceValue3 != null && sinceValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime sinceInstance3 = ((DateTime)sinceValue3);
                                                            diagnosticConditionInstance3.Since = sinceInstance3;
                                                        }
                                                        
                                                        JToken codeValue3 = conditionsValue3["code"];
                                                        if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                        {
                                                            string codeInstance3 = ((string)codeValue3);
                                                            diagnosticConditionInstance3.Code = codeInstance3;
                                                        }
                                                        
                                                        JToken messageValue3 = conditionsValue3["message"];
                                                        if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                        {
                                                            string messageInstance3 = ((string)messageValue3);
                                                            diagnosticConditionInstance3.Message = messageInstance3;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                
                                JToken functionsArray = propertiesValue["functions"];
                                if (functionsArray != null && functionsArray.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Functions = new List<Function>();
                                    foreach (JToken functionsValue in ((JArray)functionsArray))
                                    {
                                        Function functionInstance = new Function();
                                        propertiesInstance.Functions.Add(functionInstance);
                                        
                                        JToken nameValue6 = functionsValue["name"];
                                        if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                        {
                                            string nameInstance6 = ((string)nameValue6);
                                            functionInstance.Name = nameInstance6;
                                        }
                                        
                                        JToken propertiesValue25 = functionsValue["properties"];
                                        if (propertiesValue25 != null && propertiesValue25.Type != JTokenType.Null)
                                        {
                                            string typeName8 = ((string)propertiesValue25["type"]);
                                            if (typeName8 == "Scalar")
                                            {
                                                ScalarFunctionProperties scalarFunctionPropertiesInstance = new ScalarFunctionProperties();
                                                
                                                JToken propertiesValue26 = propertiesValue25["properties"];
                                                if (propertiesValue26 != null && propertiesValue26.Type != JTokenType.Null)
                                                {
                                                    ScalarFunctionConfiguration propertiesInstance24 = new ScalarFunctionConfiguration();
                                                    scalarFunctionPropertiesInstance.Properties = propertiesInstance24;
                                                    
                                                    JToken inputsArray2 = propertiesValue26["inputs"];
                                                    if (inputsArray2 != null && inputsArray2.Type != JTokenType.Null)
                                                    {
                                                        propertiesInstance24.Inputs = new List<FunctionInput>();
                                                        foreach (JToken inputsValue2 in ((JArray)inputsArray2))
                                                        {
                                                            FunctionInput functionInputInstance = new FunctionInput();
                                                            propertiesInstance24.Inputs.Add(functionInputInstance);
                                                            
                                                            JToken dataTypeValue = inputsValue2["dataType"];
                                                            if (dataTypeValue != null && dataTypeValue.Type != JTokenType.Null)
                                                            {
                                                                string dataTypeInstance = ((string)dataTypeValue);
                                                                functionInputInstance.DataType = dataTypeInstance;
                                                            }
                                                            
                                                            JToken isConfigurationParameterValue = inputsValue2["isConfigurationParameter"];
                                                            if (isConfigurationParameterValue != null && isConfigurationParameterValue.Type != JTokenType.Null)
                                                            {
                                                                bool isConfigurationParameterInstance = ((bool)isConfigurationParameterValue);
                                                                functionInputInstance.IsConfigurationParameter = isConfigurationParameterInstance;
                                                            }
                                                        }
                                                    }
                                                    
                                                    JToken outputValue = propertiesValue26["output"];
                                                    if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                    {
                                                        FunctionOutput outputInstance2 = new FunctionOutput();
                                                        propertiesInstance24.Output = outputInstance2;
                                                        
                                                        JToken dataTypeValue2 = outputValue["dataType"];
                                                        if (dataTypeValue2 != null && dataTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string dataTypeInstance2 = ((string)dataTypeValue2);
                                                            outputInstance2.DataType = dataTypeInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken bindingValue = propertiesValue26["binding"];
                                                    if (bindingValue != null && bindingValue.Type != JTokenType.Null)
                                                    {
                                                        string typeName9 = ((string)bindingValue["type"]);
                                                        if (typeName9 == "Microsoft.MachineLearning/WebService")
                                                        {
                                                            AzureMachineLearningWebServiceFunctionBinding azureMachineLearningWebServiceFunctionBindingInstance = new AzureMachineLearningWebServiceFunctionBinding();
                                                            
                                                            JToken propertiesValue27 = bindingValue["properties"];
                                                            if (propertiesValue27 != null && propertiesValue27.Type != JTokenType.Null)
                                                            {
                                                                AzureMachineLearningWebServiceFunctionBindingProperties propertiesInstance25 = new AzureMachineLearningWebServiceFunctionBindingProperties();
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Properties = propertiesInstance25;
                                                                
                                                                JToken endpointValue = propertiesValue27["endpoint"];
                                                                if (endpointValue != null && endpointValue.Type != JTokenType.Null)
                                                                {
                                                                    string endpointInstance = ((string)endpointValue);
                                                                    propertiesInstance25.Endpoint = endpointInstance;
                                                                }
                                                                
                                                                JToken apiKeyValue = propertiesValue27["apiKey"];
                                                                if (apiKeyValue != null && apiKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string apiKeyInstance = ((string)apiKeyValue);
                                                                    propertiesInstance25.ApiKey = apiKeyInstance;
                                                                }
                                                                
                                                                JToken inputsValue3 = propertiesValue27["inputs"];
                                                                if (inputsValue3 != null && inputsValue3.Type != JTokenType.Null)
                                                                {
                                                                    AzureMachineLearningWebServiceInputs inputsInstance = new AzureMachineLearningWebServiceInputs();
                                                                    propertiesInstance25.Inputs = inputsInstance;
                                                                    
                                                                    JToken nameValue7 = inputsValue3["name"];
                                                                    if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string nameInstance7 = ((string)nameValue7);
                                                                        inputsInstance.Name = nameInstance7;
                                                                    }
                                                                    
                                                                    JToken columnNamesArray = inputsValue3["columnNames"];
                                                                    if (columnNamesArray != null && columnNamesArray.Type != JTokenType.Null)
                                                                    {
                                                                        inputsInstance.ColumnNames = new List<AzureMachineLearningWebServiceInputColumn>();
                                                                        foreach (JToken columnNamesValue in ((JArray)columnNamesArray))
                                                                        {
                                                                            AzureMachineLearningWebServiceInputColumn azureMachineLearningWebServiceInputColumnInstance = new AzureMachineLearningWebServiceInputColumn();
                                                                            inputsInstance.ColumnNames.Add(azureMachineLearningWebServiceInputColumnInstance);
                                                                            
                                                                            JToken nameValue8 = columnNamesValue["name"];
                                                                            if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string nameInstance8 = ((string)nameValue8);
                                                                                azureMachineLearningWebServiceInputColumnInstance.Name = nameInstance8;
                                                                            }
                                                                            
                                                                            JToken dataTypeValue3 = columnNamesValue["dataType"];
                                                                            if (dataTypeValue3 != null && dataTypeValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string dataTypeInstance3 = ((string)dataTypeValue3);
                                                                                azureMachineLearningWebServiceInputColumnInstance.DataType = dataTypeInstance3;
                                                                            }
                                                                            
                                                                            JToken mapToValue = columnNamesValue["mapTo"];
                                                                            if (mapToValue != null && mapToValue.Type != JTokenType.Null)
                                                                            {
                                                                                int mapToInstance = ((int)mapToValue);
                                                                                azureMachineLearningWebServiceInputColumnInstance.MapTo = mapToInstance;
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken outputsArray2 = propertiesValue27["outputs"];
                                                                if (outputsArray2 != null && outputsArray2.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance25.Outputs = new List<AzureMachineLearningWebServiceOutputColumn>();
                                                                    foreach (JToken outputsValue2 in ((JArray)outputsArray2))
                                                                    {
                                                                        AzureMachineLearningWebServiceOutputColumn azureMachineLearningWebServiceOutputColumnInstance = new AzureMachineLearningWebServiceOutputColumn();
                                                                        propertiesInstance25.Outputs.Add(azureMachineLearningWebServiceOutputColumnInstance);
                                                                        
                                                                        JToken nameValue9 = outputsValue2["name"];
                                                                        if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance9 = ((string)nameValue9);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.Name = nameInstance9;
                                                                        }
                                                                        
                                                                        JToken dataTypeValue4 = outputsValue2["dataType"];
                                                                        if (dataTypeValue4 != null && dataTypeValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string dataTypeInstance4 = ((string)dataTypeValue4);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.DataType = dataTypeInstance4;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken batchSizeValue2 = propertiesValue27["batchSize"];
                                                                if (batchSizeValue2 != null && batchSizeValue2.Type != JTokenType.Null)
                                                                {
                                                                    int batchSizeInstance2 = ((int)batchSizeValue2);
                                                                    propertiesInstance25.BatchSize = batchSizeInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue24 = bindingValue["type"];
                                                            if (typeValue24 != null && typeValue24.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance24 = ((string)typeValue24);
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Type = typeInstance24;
                                                            }
                                                            propertiesInstance24.Binding = azureMachineLearningWebServiceFunctionBindingInstance;
                                                        }
                                                        if (typeName9 == "Microsoft.StreamAnalytics/JavascriptUdf")
                                                        {
                                                            JavaScriptFunctionBinding javaScriptFunctionBindingInstance = new JavaScriptFunctionBinding();
                                                            
                                                            JToken propertiesValue28 = bindingValue["properties"];
                                                            if (propertiesValue28 != null && propertiesValue28.Type != JTokenType.Null)
                                                            {
                                                                JavaScriptFunctionBindingProperties propertiesInstance26 = new JavaScriptFunctionBindingProperties();
                                                                javaScriptFunctionBindingInstance.Properties = propertiesInstance26;
                                                                
                                                                JToken scriptValue = propertiesValue28["script"];
                                                                if (scriptValue != null && scriptValue.Type != JTokenType.Null)
                                                                {
                                                                    string scriptInstance = ((string)scriptValue);
                                                                    propertiesInstance26.Script = scriptInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue25 = bindingValue["type"];
                                                            if (typeValue25 != null && typeValue25.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance25 = ((string)typeValue25);
                                                                javaScriptFunctionBindingInstance.Type = typeInstance25;
                                                            }
                                                            propertiesInstance24.Binding = javaScriptFunctionBindingInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken etagValue6 = propertiesValue25["etag"];
                                                if (etagValue6 != null && etagValue6.Type != JTokenType.Null)
                                                {
                                                    string etagInstance6 = ((string)etagValue6);
                                                    scalarFunctionPropertiesInstance.Etag = etagInstance6;
                                                }
                                                
                                                JToken typeValue26 = propertiesValue25["type"];
                                                if (typeValue26 != null && typeValue26.Type != JTokenType.Null)
                                                {
                                                    string typeInstance26 = ((string)typeValue26);
                                                    scalarFunctionPropertiesInstance.Type = typeInstance26;
                                                }
                                                functionInstance.Properties = scalarFunctionPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = DateTime.Parse(httpResponse.Headers.GetValues("Date").FirstOrDefault(), CultureInfo.InvariantCulture, DateTimeStyles.AdjustToUniversal).ToLocalTime();
                    }
                    if (httpResponse.Headers.Contains("ETag"))
                    {
                        result.Job.Properties.Etag = httpResponse.Headers.GetValues("ETag").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// List all the stream analytics jobs in the given resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to list all the stream analytics
        /// job in the same resource group.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the list stream analytics job operation.
        /// </returns>
        public async Task<JobListResponse> ListJobsInResourceGroupAsync(string resourceGroupName, JobListParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.PropertiesToExpand == null)
            {
                throw new ArgumentNullException("parameters.PropertiesToExpand");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "ListJobsInResourceGroupAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("$expand=" + Uri.EscapeDataString(parameters.PropertiesToExpand));
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    Job jobInstance = new Job();
                                    result.Value.Add(jobInstance);
                                    
                                    JToken idValue = valueValue["id"];
                                    if (idValue != null && idValue.Type != JTokenType.Null)
                                    {
                                        string idInstance = ((string)idValue);
                                        jobInstance.Id = idInstance;
                                    }
                                    
                                    JToken nameValue = valueValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        jobInstance.Name = nameInstance;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        jobInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        jobInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        jobInstance.Tags = new Dictionary<string, string>();
                                        foreach (JProperty property in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property.Name);
                                            string tagsValue = ((string)property.Value);
                                            jobInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        JobProperties propertiesInstance = new JobProperties();
                                        jobInstance.Properties = propertiesInstance;
                                        
                                        JToken skuValue = propertiesValue["sku"];
                                        if (skuValue != null && skuValue.Type != JTokenType.Null)
                                        {
                                            Sku skuInstance = new Sku();
                                            propertiesInstance.Sku = skuInstance;
                                            
                                            JToken nameValue2 = skuValue["name"];
                                            if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                            {
                                                string nameInstance2 = ((string)nameValue2);
                                                skuInstance.Name = nameInstance2;
                                            }
                                        }
                                        
                                        JToken eventsOutOfOrderPolicyValue = propertiesValue["eventsOutOfOrderPolicy"];
                                        if (eventsOutOfOrderPolicyValue != null && eventsOutOfOrderPolicyValue.Type != JTokenType.Null)
                                        {
                                            string eventsOutOfOrderPolicyInstance = ((string)eventsOutOfOrderPolicyValue);
                                            propertiesInstance.EventsOutOfOrderPolicy = eventsOutOfOrderPolicyInstance;
                                        }
                                        
                                        JToken outputErrorPolicyValue = propertiesValue["outputErrorPolicy"];
                                        if (outputErrorPolicyValue != null && outputErrorPolicyValue.Type != JTokenType.Null)
                                        {
                                            string outputErrorPolicyInstance = ((string)outputErrorPolicyValue);
                                            propertiesInstance.OutputErrorPolicy = outputErrorPolicyInstance;
                                        }
                                        
                                        JToken eventsOutOfOrderMaxDelayInSecondsValue = propertiesValue["eventsOutOfOrderMaxDelayInSeconds"];
                                        if (eventsOutOfOrderMaxDelayInSecondsValue != null && eventsOutOfOrderMaxDelayInSecondsValue.Type != JTokenType.Null)
                                        {
                                            int eventsOutOfOrderMaxDelayInSecondsInstance = ((int)eventsOutOfOrderMaxDelayInSecondsValue);
                                            propertiesInstance.EventsOutOfOrderMaxDelayInSeconds = eventsOutOfOrderMaxDelayInSecondsInstance;
                                        }
                                        
                                        JToken eventsLateArrivalMaxDelayInSecondsValue = propertiesValue["eventsLateArrivalMaxDelayInSeconds"];
                                        if (eventsLateArrivalMaxDelayInSecondsValue != null && eventsLateArrivalMaxDelayInSecondsValue.Type != JTokenType.Null)
                                        {
                                            int eventsLateArrivalMaxDelayInSecondsInstance = ((int)eventsLateArrivalMaxDelayInSecondsValue);
                                            propertiesInstance.EventsLateArrivalMaxDelayInSeconds = eventsLateArrivalMaxDelayInSecondsInstance;
                                        }
                                        
                                        JToken dataLocaleValue = propertiesValue["dataLocale"];
                                        if (dataLocaleValue != null && dataLocaleValue.Type != JTokenType.Null)
                                        {
                                            string dataLocaleInstance = ((string)dataLocaleValue);
                                            propertiesInstance.DataLocale = dataLocaleInstance;
                                        }
                                        
                                        JToken outputStartModeValue = propertiesValue["outputStartMode"];
                                        if (outputStartModeValue != null && outputStartModeValue.Type != JTokenType.Null)
                                        {
                                            string outputStartModeInstance = ((string)outputStartModeValue);
                                            propertiesInstance.OutputStartMode = outputStartModeInstance;
                                        }
                                        
                                        JToken outputStartTimeValue = propertiesValue["outputStartTime"];
                                        if (outputStartTimeValue != null && outputStartTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime outputStartTimeInstance = ((DateTime)outputStartTimeValue);
                                            propertiesInstance.OutputStartTime = outputStartTimeInstance;
                                        }
                                        
                                        JToken lastOutputEventTimeValue = propertiesValue["lastOutputEventTime"];
                                        if (lastOutputEventTimeValue != null && lastOutputEventTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastOutputEventTimeInstance = ((DateTime)lastOutputEventTimeValue);
                                            propertiesInstance.LastOutputEventTime = lastOutputEventTimeInstance;
                                        }
                                        
                                        JToken jobIdValue = propertiesValue["jobId"];
                                        if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                        {
                                            string jobIdInstance = ((string)jobIdValue);
                                            propertiesInstance.JobId = jobIdInstance;
                                        }
                                        
                                        JToken etagValue = propertiesValue["etag"];
                                        if (etagValue != null && etagValue.Type != JTokenType.Null)
                                        {
                                            string etagInstance = ((string)etagValue);
                                            propertiesInstance.Etag = etagInstance;
                                        }
                                        
                                        JToken provisioningStateValue = propertiesValue["provisioningState"];
                                        if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                        {
                                            string provisioningStateInstance = ((string)provisioningStateValue);
                                            propertiesInstance.ProvisioningState = provisioningStateInstance;
                                        }
                                        
                                        JToken jobStateValue = propertiesValue["jobState"];
                                        if (jobStateValue != null && jobStateValue.Type != JTokenType.Null)
                                        {
                                            string jobStateInstance = ((string)jobStateValue);
                                            propertiesInstance.JobState = jobStateInstance;
                                        }
                                        
                                        JToken createdDateValue = propertiesValue["createdDate"];
                                        if (createdDateValue != null && createdDateValue.Type != JTokenType.Null)
                                        {
                                            DateTime createdDateInstance = ((DateTime)createdDateValue);
                                            propertiesInstance.CreatedDate = createdDateInstance;
                                        }
                                        
                                        JToken inputsArray = propertiesValue["inputs"];
                                        if (inputsArray != null && inputsArray.Type != JTokenType.Null)
                                        {
                                            propertiesInstance.Inputs = new List<Input>();
                                            foreach (JToken inputsValue in ((JArray)inputsArray))
                                            {
                                                Input inputInstance = new Input();
                                                propertiesInstance.Inputs.Add(inputInstance);
                                                
                                                JToken nameValue3 = inputsValue["name"];
                                                if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                {
                                                    string nameInstance3 = ((string)nameValue3);
                                                    inputInstance.Name = nameInstance3;
                                                }
                                                
                                                JToken propertiesValue2 = inputsValue["properties"];
                                                if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                                                {
                                                    string typeName = ((string)propertiesValue2["type"]);
                                                    if (typeName == "Reference")
                                                    {
                                                        ReferenceInputProperties referenceInputPropertiesInstance = new ReferenceInputProperties();
                                                        
                                                        JToken datasourceValue = propertiesValue2["datasource"];
                                                        if (datasourceValue != null && datasourceValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName2 = ((string)datasourceValue["type"]);
                                                            if (typeName2 == "Microsoft.Storage/Blob")
                                                            {
                                                                BlobReferenceInputDataSource blobReferenceInputDataSourceInstance = new BlobReferenceInputDataSource();
                                                                
                                                                JToken propertiesValue3 = datasourceValue["properties"];
                                                                if (propertiesValue3 != null && propertiesValue3.Type != JTokenType.Null)
                                                                {
                                                                    BlobReferenceInputDataSourceProperties propertiesInstance2 = new BlobReferenceInputDataSourceProperties();
                                                                    blobReferenceInputDataSourceInstance.Properties = propertiesInstance2;
                                                                    
                                                                    JToken storageAccountsArray = propertiesValue3["storageAccounts"];
                                                                    if (storageAccountsArray != null && storageAccountsArray.Type != JTokenType.Null)
                                                                    {
                                                                        propertiesInstance2.StorageAccounts = new List<StorageAccount>();
                                                                        foreach (JToken storageAccountsValue in ((JArray)storageAccountsArray))
                                                                        {
                                                                            StorageAccount storageAccountInstance = new StorageAccount();
                                                                            propertiesInstance2.StorageAccounts.Add(storageAccountInstance);
                                                                            
                                                                            JToken accountNameValue = storageAccountsValue["accountName"];
                                                                            if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                            {
                                                                                string accountNameInstance = ((string)accountNameValue);
                                                                                storageAccountInstance.AccountName = accountNameInstance;
                                                                            }
                                                                            
                                                                            JToken accountKeyValue = storageAccountsValue["accountKey"];
                                                                            if (accountKeyValue != null && accountKeyValue.Type != JTokenType.Null)
                                                                            {
                                                                                string accountKeyInstance = ((string)accountKeyValue);
                                                                                storageAccountInstance.AccountKey = accountKeyInstance;
                                                                            }
                                                                        }
                                                                    }
                                                                    
                                                                    JToken containerValue = propertiesValue3["container"];
                                                                    if (containerValue != null && containerValue.Type != JTokenType.Null)
                                                                    {
                                                                        string containerInstance = ((string)containerValue);
                                                                        propertiesInstance2.Container = containerInstance;
                                                                    }
                                                                    
                                                                    JToken pathPatternValue = propertiesValue3["pathPattern"];
                                                                    if (pathPatternValue != null && pathPatternValue.Type != JTokenType.Null)
                                                                    {
                                                                        string pathPatternInstance = ((string)pathPatternValue);
                                                                        propertiesInstance2.PathPattern = pathPatternInstance;
                                                                    }
                                                                    
                                                                    JToken dateFormatValue = propertiesValue3["dateFormat"];
                                                                    if (dateFormatValue != null && dateFormatValue.Type != JTokenType.Null)
                                                                    {
                                                                        string dateFormatInstance = ((string)dateFormatValue);
                                                                        propertiesInstance2.DateFormat = dateFormatInstance;
                                                                    }
                                                                    
                                                                    JToken timeFormatValue = propertiesValue3["timeFormat"];
                                                                    if (timeFormatValue != null && timeFormatValue.Type != JTokenType.Null)
                                                                    {
                                                                        string timeFormatInstance = ((string)timeFormatValue);
                                                                        propertiesInstance2.TimeFormat = timeFormatInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue2 = datasourceValue["type"];
                                                                if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance2 = ((string)typeValue2);
                                                                    blobReferenceInputDataSourceInstance.Type = typeInstance2;
                                                                }
                                                                referenceInputPropertiesInstance.DataSource = blobReferenceInputDataSourceInstance;
                                                            }
                                                        }
                                                        
                                                        JToken etagValue2 = propertiesValue2["etag"];
                                                        if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                        {
                                                            string etagInstance2 = ((string)etagValue2);
                                                            referenceInputPropertiesInstance.Etag = etagInstance2;
                                                        }
                                                        
                                                        JToken typeValue3 = propertiesValue2["type"];
                                                        if (typeValue3 != null && typeValue3.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance3 = ((string)typeValue3);
                                                            referenceInputPropertiesInstance.Type = typeInstance3;
                                                        }
                                                        
                                                        JToken serializationValue = propertiesValue2["serialization"];
                                                        if (serializationValue != null && serializationValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName3 = ((string)serializationValue["type"]);
                                                            if (typeName3 == "Csv")
                                                            {
                                                                CsvSerialization csvSerializationInstance = new CsvSerialization();
                                                                
                                                                JToken propertiesValue4 = serializationValue["properties"];
                                                                if (propertiesValue4 != null && propertiesValue4.Type != JTokenType.Null)
                                                                {
                                                                    CsvSerializationProperties propertiesInstance3 = new CsvSerializationProperties();
                                                                    csvSerializationInstance.Properties = propertiesInstance3;
                                                                    
                                                                    JToken fieldDelimiterValue = propertiesValue4["fieldDelimiter"];
                                                                    if (fieldDelimiterValue != null && fieldDelimiterValue.Type != JTokenType.Null)
                                                                    {
                                                                        string fieldDelimiterInstance = ((string)fieldDelimiterValue);
                                                                        propertiesInstance3.FieldDelimiter = fieldDelimiterInstance;
                                                                    }
                                                                    
                                                                    JToken encodingValue = propertiesValue4["encoding"];
                                                                    if (encodingValue != null && encodingValue.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance = ((string)encodingValue);
                                                                        propertiesInstance3.Encoding = encodingInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue4 = serializationValue["type"];
                                                                if (typeValue4 != null && typeValue4.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance4 = ((string)typeValue4);
                                                                    csvSerializationInstance.Type = typeInstance4;
                                                                }
                                                                referenceInputPropertiesInstance.Serialization = csvSerializationInstance;
                                                            }
                                                            if (typeName3 == "Json")
                                                            {
                                                                JsonSerialization jsonSerializationInstance = new JsonSerialization();
                                                                
                                                                JToken propertiesValue5 = serializationValue["properties"];
                                                                if (propertiesValue5 != null && propertiesValue5.Type != JTokenType.Null)
                                                                {
                                                                    JsonSerializationProperties propertiesInstance4 = new JsonSerializationProperties();
                                                                    jsonSerializationInstance.Properties = propertiesInstance4;
                                                                    
                                                                    JToken encodingValue2 = propertiesValue5["encoding"];
                                                                    if (encodingValue2 != null && encodingValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance2 = ((string)encodingValue2);
                                                                        propertiesInstance4.Encoding = encodingInstance2;
                                                                    }
                                                                    
                                                                    JToken formatValue = propertiesValue5["format"];
                                                                    if (formatValue != null && formatValue.Type != JTokenType.Null)
                                                                    {
                                                                        string formatInstance = ((string)formatValue);
                                                                        propertiesInstance4.Format = formatInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue5 = serializationValue["type"];
                                                                if (typeValue5 != null && typeValue5.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance5 = ((string)typeValue5);
                                                                    jsonSerializationInstance.Type = typeInstance5;
                                                                }
                                                                referenceInputPropertiesInstance.Serialization = jsonSerializationInstance;
                                                            }
                                                            if (typeName3 == "Avro")
                                                            {
                                                                AvroSerialization avroSerializationInstance = new AvroSerialization();
                                                                
                                                                JToken propertiesValue6 = serializationValue["properties"];
                                                                if (propertiesValue6 != null && propertiesValue6.Type != JTokenType.Null)
                                                                {
                                                                    AvroSerializationProperties propertiesInstance5 = new AvroSerializationProperties();
                                                                    avroSerializationInstance.Properties = propertiesInstance5;
                                                                }
                                                                
                                                                JToken typeValue6 = serializationValue["type"];
                                                                if (typeValue6 != null && typeValue6.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance6 = ((string)typeValue6);
                                                                    avroSerializationInstance.Type = typeInstance6;
                                                                }
                                                                referenceInputPropertiesInstance.Serialization = avroSerializationInstance;
                                                            }
                                                        }
                                                        
                                                        JToken diagnosticsValue = propertiesValue2["diagnostics"];
                                                        if (diagnosticsValue != null && diagnosticsValue.Type != JTokenType.Null)
                                                        {
                                                            Diagnostics diagnosticsInstance = new Diagnostics();
                                                            referenceInputPropertiesInstance.Diagnostics = diagnosticsInstance;
                                                            
                                                            JToken conditionsArray = diagnosticsValue["conditions"];
                                                            if (conditionsArray != null && conditionsArray.Type != JTokenType.Null)
                                                            {
                                                                foreach (JToken conditionsValue in ((JArray)conditionsArray))
                                                                {
                                                                    DiagnosticCondition diagnosticConditionInstance = new DiagnosticCondition();
                                                                    diagnosticsInstance.Conditions.Add(diagnosticConditionInstance);
                                                                    
                                                                    JToken sinceValue = conditionsValue["since"];
                                                                    if (sinceValue != null && sinceValue.Type != JTokenType.Null)
                                                                    {
                                                                        DateTime sinceInstance = ((DateTime)sinceValue);
                                                                        diagnosticConditionInstance.Since = sinceInstance;
                                                                    }
                                                                    
                                                                    JToken codeValue = conditionsValue["code"];
                                                                    if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance = ((string)codeValue);
                                                                        diagnosticConditionInstance.Code = codeInstance;
                                                                    }
                                                                    
                                                                    JToken messageValue = conditionsValue["message"];
                                                                    if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance = ((string)messageValue);
                                                                        diagnosticConditionInstance.Message = messageInstance;
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        inputInstance.Properties = referenceInputPropertiesInstance;
                                                    }
                                                    if (typeName == "Stream")
                                                    {
                                                        StreamInputProperties streamInputPropertiesInstance = new StreamInputProperties();
                                                        
                                                        JToken datasourceValue2 = propertiesValue2["datasource"];
                                                        if (datasourceValue2 != null && datasourceValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName4 = ((string)datasourceValue2["type"]);
                                                            if (typeName4 == "Microsoft.Storage/Blob")
                                                            {
                                                                BlobStreamInputDataSource blobStreamInputDataSourceInstance = new BlobStreamInputDataSource();
                                                                
                                                                JToken propertiesValue7 = datasourceValue2["properties"];
                                                                if (propertiesValue7 != null && propertiesValue7.Type != JTokenType.Null)
                                                                {
                                                                    BlobStreamInputDataSourceProperties propertiesInstance6 = new BlobStreamInputDataSourceProperties();
                                                                    blobStreamInputDataSourceInstance.Properties = propertiesInstance6;
                                                                    
                                                                    JToken storageAccountsArray2 = propertiesValue7["storageAccounts"];
                                                                    if (storageAccountsArray2 != null && storageAccountsArray2.Type != JTokenType.Null)
                                                                    {
                                                                        propertiesInstance6.StorageAccounts = new List<StorageAccount>();
                                                                        foreach (JToken storageAccountsValue2 in ((JArray)storageAccountsArray2))
                                                                        {
                                                                            StorageAccount storageAccountInstance2 = new StorageAccount();
                                                                            propertiesInstance6.StorageAccounts.Add(storageAccountInstance2);
                                                                            
                                                                            JToken accountNameValue2 = storageAccountsValue2["accountName"];
                                                                            if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string accountNameInstance2 = ((string)accountNameValue2);
                                                                                storageAccountInstance2.AccountName = accountNameInstance2;
                                                                            }
                                                                            
                                                                            JToken accountKeyValue2 = storageAccountsValue2["accountKey"];
                                                                            if (accountKeyValue2 != null && accountKeyValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string accountKeyInstance2 = ((string)accountKeyValue2);
                                                                                storageAccountInstance2.AccountKey = accountKeyInstance2;
                                                                            }
                                                                        }
                                                                    }
                                                                    
                                                                    JToken containerValue2 = propertiesValue7["container"];
                                                                    if (containerValue2 != null && containerValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string containerInstance2 = ((string)containerValue2);
                                                                        propertiesInstance6.Container = containerInstance2;
                                                                    }
                                                                    
                                                                    JToken pathPatternValue2 = propertiesValue7["pathPattern"];
                                                                    if (pathPatternValue2 != null && pathPatternValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string pathPatternInstance2 = ((string)pathPatternValue2);
                                                                        propertiesInstance6.PathPattern = pathPatternInstance2;
                                                                    }
                                                                    
                                                                    JToken dateFormatValue2 = propertiesValue7["dateFormat"];
                                                                    if (dateFormatValue2 != null && dateFormatValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string dateFormatInstance2 = ((string)dateFormatValue2);
                                                                        propertiesInstance6.DateFormat = dateFormatInstance2;
                                                                    }
                                                                    
                                                                    JToken timeFormatValue2 = propertiesValue7["timeFormat"];
                                                                    if (timeFormatValue2 != null && timeFormatValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string timeFormatInstance2 = ((string)timeFormatValue2);
                                                                        propertiesInstance6.TimeFormat = timeFormatInstance2;
                                                                    }
                                                                    
                                                                    JToken sourcePartitionCountValue = propertiesValue7["sourcePartitionCount"];
                                                                    if (sourcePartitionCountValue != null && sourcePartitionCountValue.Type != JTokenType.Null)
                                                                    {
                                                                        int sourcePartitionCountInstance = ((int)sourcePartitionCountValue);
                                                                        propertiesInstance6.SourcePartitionCount = sourcePartitionCountInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue7 = datasourceValue2["type"];
                                                                if (typeValue7 != null && typeValue7.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance7 = ((string)typeValue7);
                                                                    blobStreamInputDataSourceInstance.Type = typeInstance7;
                                                                }
                                                                streamInputPropertiesInstance.DataSource = blobStreamInputDataSourceInstance;
                                                            }
                                                            if (typeName4 == "Microsoft.ServiceBus/EventHub")
                                                            {
                                                                EventHubStreamInputDataSource eventHubStreamInputDataSourceInstance = new EventHubStreamInputDataSource();
                                                                
                                                                JToken propertiesValue8 = datasourceValue2["properties"];
                                                                if (propertiesValue8 != null && propertiesValue8.Type != JTokenType.Null)
                                                                {
                                                                    EventHubStreamInputDataSourceProperties propertiesInstance7 = new EventHubStreamInputDataSourceProperties();
                                                                    eventHubStreamInputDataSourceInstance.Properties = propertiesInstance7;
                                                                    
                                                                    JToken serviceBusNamespaceValue = propertiesValue8["serviceBusNamespace"];
                                                                    if (serviceBusNamespaceValue != null && serviceBusNamespaceValue.Type != JTokenType.Null)
                                                                    {
                                                                        string serviceBusNamespaceInstance = ((string)serviceBusNamespaceValue);
                                                                        propertiesInstance7.ServiceBusNamespace = serviceBusNamespaceInstance;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyNameValue = propertiesValue8["sharedAccessPolicyName"];
                                                                    if (sharedAccessPolicyNameValue != null && sharedAccessPolicyNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyNameInstance = ((string)sharedAccessPolicyNameValue);
                                                                        propertiesInstance7.SharedAccessPolicyName = sharedAccessPolicyNameInstance;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyKeyValue = propertiesValue8["sharedAccessPolicyKey"];
                                                                    if (sharedAccessPolicyKeyValue != null && sharedAccessPolicyKeyValue.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyKeyInstance = ((string)sharedAccessPolicyKeyValue);
                                                                        propertiesInstance7.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance;
                                                                    }
                                                                    
                                                                    JToken eventHubNameValue = propertiesValue8["eventHubName"];
                                                                    if (eventHubNameValue != null && eventHubNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string eventHubNameInstance = ((string)eventHubNameValue);
                                                                        propertiesInstance7.EventHubName = eventHubNameInstance;
                                                                    }
                                                                    
                                                                    JToken consumerGroupNameValue = propertiesValue8["consumerGroupName"];
                                                                    if (consumerGroupNameValue != null && consumerGroupNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string consumerGroupNameInstance = ((string)consumerGroupNameValue);
                                                                        propertiesInstance7.ConsumerGroupName = consumerGroupNameInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue8 = datasourceValue2["type"];
                                                                if (typeValue8 != null && typeValue8.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance8 = ((string)typeValue8);
                                                                    eventHubStreamInputDataSourceInstance.Type = typeInstance8;
                                                                }
                                                                streamInputPropertiesInstance.DataSource = eventHubStreamInputDataSourceInstance;
                                                            }
                                                            if (typeName4 == "Microsoft.Devices/IotHubs")
                                                            {
                                                                IoTHubStreamInputDataSource ioTHubStreamInputDataSourceInstance = new IoTHubStreamInputDataSource();
                                                                
                                                                JToken propertiesValue9 = datasourceValue2["properties"];
                                                                if (propertiesValue9 != null && propertiesValue9.Type != JTokenType.Null)
                                                                {
                                                                    IoTHubStreamInputDataSourceProperties propertiesInstance8 = new IoTHubStreamInputDataSourceProperties();
                                                                    ioTHubStreamInputDataSourceInstance.Properties = propertiesInstance8;
                                                                    
                                                                    JToken iotHubNamespaceValue = propertiesValue9["iotHubNamespace"];
                                                                    if (iotHubNamespaceValue != null && iotHubNamespaceValue.Type != JTokenType.Null)
                                                                    {
                                                                        string iotHubNamespaceInstance = ((string)iotHubNamespaceValue);
                                                                        propertiesInstance8.IotHubNamespace = iotHubNamespaceInstance;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyNameValue2 = propertiesValue9["sharedAccessPolicyName"];
                                                                    if (sharedAccessPolicyNameValue2 != null && sharedAccessPolicyNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyNameInstance2 = ((string)sharedAccessPolicyNameValue2);
                                                                        propertiesInstance8.SharedAccessPolicyName = sharedAccessPolicyNameInstance2;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyKeyValue2 = propertiesValue9["sharedAccessPolicyKey"];
                                                                    if (sharedAccessPolicyKeyValue2 != null && sharedAccessPolicyKeyValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyKeyInstance2 = ((string)sharedAccessPolicyKeyValue2);
                                                                        propertiesInstance8.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance2;
                                                                    }
                                                                    
                                                                    JToken consumerGroupNameValue2 = propertiesValue9["consumerGroupName"];
                                                                    if (consumerGroupNameValue2 != null && consumerGroupNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string consumerGroupNameInstance2 = ((string)consumerGroupNameValue2);
                                                                        propertiesInstance8.ConsumerGroupName = consumerGroupNameInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue9 = datasourceValue2["type"];
                                                                if (typeValue9 != null && typeValue9.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance9 = ((string)typeValue9);
                                                                    ioTHubStreamInputDataSourceInstance.Type = typeInstance9;
                                                                }
                                                                streamInputPropertiesInstance.DataSource = ioTHubStreamInputDataSourceInstance;
                                                            }
                                                        }
                                                        
                                                        JToken etagValue3 = propertiesValue2["etag"];
                                                        if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                                        {
                                                            string etagInstance3 = ((string)etagValue3);
                                                            streamInputPropertiesInstance.Etag = etagInstance3;
                                                        }
                                                        
                                                        JToken typeValue10 = propertiesValue2["type"];
                                                        if (typeValue10 != null && typeValue10.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance10 = ((string)typeValue10);
                                                            streamInputPropertiesInstance.Type = typeInstance10;
                                                        }
                                                        
                                                        JToken serializationValue2 = propertiesValue2["serialization"];
                                                        if (serializationValue2 != null && serializationValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName5 = ((string)serializationValue2["type"]);
                                                            if (typeName5 == "Csv")
                                                            {
                                                                CsvSerialization csvSerializationInstance2 = new CsvSerialization();
                                                                
                                                                JToken propertiesValue10 = serializationValue2["properties"];
                                                                if (propertiesValue10 != null && propertiesValue10.Type != JTokenType.Null)
                                                                {
                                                                    CsvSerializationProperties propertiesInstance9 = new CsvSerializationProperties();
                                                                    csvSerializationInstance2.Properties = propertiesInstance9;
                                                                    
                                                                    JToken fieldDelimiterValue2 = propertiesValue10["fieldDelimiter"];
                                                                    if (fieldDelimiterValue2 != null && fieldDelimiterValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string fieldDelimiterInstance2 = ((string)fieldDelimiterValue2);
                                                                        propertiesInstance9.FieldDelimiter = fieldDelimiterInstance2;
                                                                    }
                                                                    
                                                                    JToken encodingValue3 = propertiesValue10["encoding"];
                                                                    if (encodingValue3 != null && encodingValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance3 = ((string)encodingValue3);
                                                                        propertiesInstance9.Encoding = encodingInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue11 = serializationValue2["type"];
                                                                if (typeValue11 != null && typeValue11.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance11 = ((string)typeValue11);
                                                                    csvSerializationInstance2.Type = typeInstance11;
                                                                }
                                                                streamInputPropertiesInstance.Serialization = csvSerializationInstance2;
                                                            }
                                                            if (typeName5 == "Json")
                                                            {
                                                                JsonSerialization jsonSerializationInstance2 = new JsonSerialization();
                                                                
                                                                JToken propertiesValue11 = serializationValue2["properties"];
                                                                if (propertiesValue11 != null && propertiesValue11.Type != JTokenType.Null)
                                                                {
                                                                    JsonSerializationProperties propertiesInstance10 = new JsonSerializationProperties();
                                                                    jsonSerializationInstance2.Properties = propertiesInstance10;
                                                                    
                                                                    JToken encodingValue4 = propertiesValue11["encoding"];
                                                                    if (encodingValue4 != null && encodingValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance4 = ((string)encodingValue4);
                                                                        propertiesInstance10.Encoding = encodingInstance4;
                                                                    }
                                                                    
                                                                    JToken formatValue2 = propertiesValue11["format"];
                                                                    if (formatValue2 != null && formatValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string formatInstance2 = ((string)formatValue2);
                                                                        propertiesInstance10.Format = formatInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue12 = serializationValue2["type"];
                                                                if (typeValue12 != null && typeValue12.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance12 = ((string)typeValue12);
                                                                    jsonSerializationInstance2.Type = typeInstance12;
                                                                }
                                                                streamInputPropertiesInstance.Serialization = jsonSerializationInstance2;
                                                            }
                                                            if (typeName5 == "Avro")
                                                            {
                                                                AvroSerialization avroSerializationInstance2 = new AvroSerialization();
                                                                
                                                                JToken propertiesValue12 = serializationValue2["properties"];
                                                                if (propertiesValue12 != null && propertiesValue12.Type != JTokenType.Null)
                                                                {
                                                                    AvroSerializationProperties propertiesInstance11 = new AvroSerializationProperties();
                                                                    avroSerializationInstance2.Properties = propertiesInstance11;
                                                                }
                                                                
                                                                JToken typeValue13 = serializationValue2["type"];
                                                                if (typeValue13 != null && typeValue13.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance13 = ((string)typeValue13);
                                                                    avroSerializationInstance2.Type = typeInstance13;
                                                                }
                                                                streamInputPropertiesInstance.Serialization = avroSerializationInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken diagnosticsValue2 = propertiesValue2["diagnostics"];
                                                        if (diagnosticsValue2 != null && diagnosticsValue2.Type != JTokenType.Null)
                                                        {
                                                            Diagnostics diagnosticsInstance2 = new Diagnostics();
                                                            streamInputPropertiesInstance.Diagnostics = diagnosticsInstance2;
                                                            
                                                            JToken conditionsArray2 = diagnosticsValue2["conditions"];
                                                            if (conditionsArray2 != null && conditionsArray2.Type != JTokenType.Null)
                                                            {
                                                                foreach (JToken conditionsValue2 in ((JArray)conditionsArray2))
                                                                {
                                                                    DiagnosticCondition diagnosticConditionInstance2 = new DiagnosticCondition();
                                                                    diagnosticsInstance2.Conditions.Add(diagnosticConditionInstance2);
                                                                    
                                                                    JToken sinceValue2 = conditionsValue2["since"];
                                                                    if (sinceValue2 != null && sinceValue2.Type != JTokenType.Null)
                                                                    {
                                                                        DateTime sinceInstance2 = ((DateTime)sinceValue2);
                                                                        diagnosticConditionInstance2.Since = sinceInstance2;
                                                                    }
                                                                    
                                                                    JToken codeValue2 = conditionsValue2["code"];
                                                                    if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance2 = ((string)codeValue2);
                                                                        diagnosticConditionInstance2.Code = codeInstance2;
                                                                    }
                                                                    
                                                                    JToken messageValue2 = conditionsValue2["message"];
                                                                    if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance2 = ((string)messageValue2);
                                                                        diagnosticConditionInstance2.Message = messageInstance2;
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        inputInstance.Properties = streamInputPropertiesInstance;
                                                    }
                                                }
                                            }
                                        }
                                        
                                        JToken transformationValue = propertiesValue["transformation"];
                                        if (transformationValue != null && transformationValue.Type != JTokenType.Null)
                                        {
                                            Transformation transformationInstance = new Transformation();
                                            propertiesInstance.Transformation = transformationInstance;
                                            
                                            JToken nameValue4 = transformationValue["name"];
                                            if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                            {
                                                string nameInstance4 = ((string)nameValue4);
                                                transformationInstance.Name = nameInstance4;
                                            }
                                            
                                            JToken propertiesValue13 = transformationValue["properties"];
                                            if (propertiesValue13 != null && propertiesValue13.Type != JTokenType.Null)
                                            {
                                                TransformationProperties propertiesInstance12 = new TransformationProperties();
                                                transformationInstance.Properties = propertiesInstance12;
                                                
                                                JToken etagValue4 = propertiesValue13["etag"];
                                                if (etagValue4 != null && etagValue4.Type != JTokenType.Null)
                                                {
                                                    string etagInstance4 = ((string)etagValue4);
                                                    propertiesInstance12.Etag = etagInstance4;
                                                }
                                                
                                                JToken streamingUnitsValue = propertiesValue13["streamingUnits"];
                                                if (streamingUnitsValue != null && streamingUnitsValue.Type != JTokenType.Null)
                                                {
                                                    int streamingUnitsInstance = ((int)streamingUnitsValue);
                                                    propertiesInstance12.StreamingUnits = streamingUnitsInstance;
                                                }
                                                
                                                JToken queryValue = propertiesValue13["query"];
                                                if (queryValue != null && queryValue.Type != JTokenType.Null)
                                                {
                                                    string queryInstance = ((string)queryValue);
                                                    propertiesInstance12.Query = queryInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken outputsArray = propertiesValue["outputs"];
                                        if (outputsArray != null && outputsArray.Type != JTokenType.Null)
                                        {
                                            propertiesInstance.Outputs = new List<Output>();
                                            foreach (JToken outputsValue in ((JArray)outputsArray))
                                            {
                                                Output outputInstance = new Output();
                                                propertiesInstance.Outputs.Add(outputInstance);
                                                
                                                JToken nameValue5 = outputsValue["name"];
                                                if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                {
                                                    string nameInstance5 = ((string)nameValue5);
                                                    outputInstance.Name = nameInstance5;
                                                }
                                                
                                                JToken propertiesValue14 = outputsValue["properties"];
                                                if (propertiesValue14 != null && propertiesValue14.Type != JTokenType.Null)
                                                {
                                                    OutputProperties propertiesInstance13 = new OutputProperties();
                                                    outputInstance.Properties = propertiesInstance13;
                                                    
                                                    JToken etagValue5 = propertiesValue14["etag"];
                                                    if (etagValue5 != null && etagValue5.Type != JTokenType.Null)
                                                    {
                                                        string etagInstance5 = ((string)etagValue5);
                                                        propertiesInstance13.Etag = etagInstance5;
                                                    }
                                                    
                                                    JToken datasourceValue3 = propertiesValue14["datasource"];
                                                    if (datasourceValue3 != null && datasourceValue3.Type != JTokenType.Null)
                                                    {
                                                        string typeName6 = ((string)datasourceValue3["type"]);
                                                        if (typeName6 == "Microsoft.Storage/Blob")
                                                        {
                                                            BlobOutputDataSource blobOutputDataSourceInstance = new BlobOutputDataSource();
                                                            
                                                            JToken propertiesValue15 = datasourceValue3["properties"];
                                                            if (propertiesValue15 != null && propertiesValue15.Type != JTokenType.Null)
                                                            {
                                                                BlobOutputDataSourceProperties propertiesInstance14 = new BlobOutputDataSourceProperties();
                                                                blobOutputDataSourceInstance.Properties = propertiesInstance14;
                                                                
                                                                JToken storageAccountsArray3 = propertiesValue15["storageAccounts"];
                                                                if (storageAccountsArray3 != null && storageAccountsArray3.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance14.StorageAccounts = new System.Collections.Generic.List<Microsoft.Azure.Management.StreamAnalytics.Models.StorageAccount>();
                                                                    foreach (JToken storageAccountsValue3 in ((JArray)storageAccountsArray3))
                                                                    {
                                                                        StorageAccount storageAccountInstance3 = new StorageAccount();
                                                                        propertiesInstance14.StorageAccounts.Add(storageAccountInstance3);
                                                                        
                                                                        JToken accountNameValue3 = storageAccountsValue3["accountName"];
                                                                        if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string accountNameInstance3 = ((string)accountNameValue3);
                                                                            storageAccountInstance3.AccountName = accountNameInstance3;
                                                                        }
                                                                        
                                                                        JToken accountKeyValue3 = storageAccountsValue3["accountKey"];
                                                                        if (accountKeyValue3 != null && accountKeyValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string accountKeyInstance3 = ((string)accountKeyValue3);
                                                                            storageAccountInstance3.AccountKey = accountKeyInstance3;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken containerValue3 = propertiesValue15["container"];
                                                                if (containerValue3 != null && containerValue3.Type != JTokenType.Null)
                                                                {
                                                                    string containerInstance3 = ((string)containerValue3);
                                                                    propertiesInstance14.Container = containerInstance3;
                                                                }
                                                                
                                                                JToken pathPatternValue3 = propertiesValue15["pathPattern"];
                                                                if (pathPatternValue3 != null && pathPatternValue3.Type != JTokenType.Null)
                                                                {
                                                                    string pathPatternInstance3 = ((string)pathPatternValue3);
                                                                    propertiesInstance14.PathPattern = pathPatternInstance3;
                                                                }
                                                                
                                                                JToken dateFormatValue3 = propertiesValue15["dateFormat"];
                                                                if (dateFormatValue3 != null && dateFormatValue3.Type != JTokenType.Null)
                                                                {
                                                                    string dateFormatInstance3 = ((string)dateFormatValue3);
                                                                    propertiesInstance14.DateFormat = dateFormatInstance3;
                                                                }
                                                                
                                                                JToken timeFormatValue3 = propertiesValue15["timeFormat"];
                                                                if (timeFormatValue3 != null && timeFormatValue3.Type != JTokenType.Null)
                                                                {
                                                                    string timeFormatInstance3 = ((string)timeFormatValue3);
                                                                    propertiesInstance14.TimeFormat = timeFormatInstance3;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue14 = datasourceValue3["type"];
                                                            if (typeValue14 != null && typeValue14.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance14 = ((string)typeValue14);
                                                                blobOutputDataSourceInstance.Type = typeInstance14;
                                                            }
                                                            propertiesInstance13.DataSource = blobOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.Storage/Table")
                                                        {
                                                            AzureTableOutputDataSource azureTableOutputDataSourceInstance = new AzureTableOutputDataSource();
                                                            
                                                            JToken propertiesValue16 = datasourceValue3["properties"];
                                                            if (propertiesValue16 != null && propertiesValue16.Type != JTokenType.Null)
                                                            {
                                                                AzureTableOutputDataSourceProperties propertiesInstance15 = new AzureTableOutputDataSourceProperties();
                                                                azureTableOutputDataSourceInstance.Properties = propertiesInstance15;
                                                                
                                                                JToken accountNameValue4 = propertiesValue16["accountName"];
                                                                if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance4 = ((string)accountNameValue4);
                                                                    propertiesInstance15.AccountName = accountNameInstance4;
                                                                }
                                                                
                                                                JToken accountKeyValue4 = propertiesValue16["accountKey"];
                                                                if (accountKeyValue4 != null && accountKeyValue4.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance4 = ((string)accountKeyValue4);
                                                                    propertiesInstance15.AccountKey = accountKeyInstance4;
                                                                }
                                                                
                                                                JToken tableValue = propertiesValue16["table"];
                                                                if (tableValue != null && tableValue.Type != JTokenType.Null)
                                                                {
                                                                    string tableInstance = ((string)tableValue);
                                                                    propertiesInstance15.Table = tableInstance;
                                                                }
                                                                
                                                                JToken partitionKeyValue = propertiesValue16["partitionKey"];
                                                                if (partitionKeyValue != null && partitionKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string partitionKeyInstance = ((string)partitionKeyValue);
                                                                    propertiesInstance15.PartitionKey = partitionKeyInstance;
                                                                }
                                                                
                                                                JToken rowKeyValue = propertiesValue16["rowKey"];
                                                                if (rowKeyValue != null && rowKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string rowKeyInstance = ((string)rowKeyValue);
                                                                    propertiesInstance15.RowKey = rowKeyInstance;
                                                                }
                                                                
                                                                JToken columnsToRemoveArray = propertiesValue16["columnsToRemove"];
                                                                if (columnsToRemoveArray != null && columnsToRemoveArray.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance15.ColumnsToRemove = new System.Collections.Generic.List<string>();
                                                                    foreach (JToken columnsToRemoveValue in ((JArray)columnsToRemoveArray))
                                                                    {
                                                                        propertiesInstance15.ColumnsToRemove.Add(((string)columnsToRemoveValue));
                                                                    }
                                                                }
                                                                
                                                                JToken batchSizeValue = propertiesValue16["batchSize"];
                                                                if (batchSizeValue != null && batchSizeValue.Type != JTokenType.Null)
                                                                {
                                                                    int batchSizeInstance = ((int)batchSizeValue);
                                                                    propertiesInstance15.BatchSize = batchSizeInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue15 = datasourceValue3["type"];
                                                            if (typeValue15 != null && typeValue15.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance15 = ((string)typeValue15);
                                                                azureTableOutputDataSourceInstance.Type = typeInstance15;
                                                            }
                                                            propertiesInstance13.DataSource = azureTableOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.ServiceBus/EventHub")
                                                        {
                                                            EventHubOutputDataSource eventHubOutputDataSourceInstance = new EventHubOutputDataSource();
                                                            
                                                            JToken propertiesValue17 = datasourceValue3["properties"];
                                                            if (propertiesValue17 != null && propertiesValue17.Type != JTokenType.Null)
                                                            {
                                                                EventHubOutputDataSourceProperties propertiesInstance16 = new EventHubOutputDataSourceProperties();
                                                                eventHubOutputDataSourceInstance.Properties = propertiesInstance16;
                                                                
                                                                JToken serviceBusNamespaceValue2 = propertiesValue17["serviceBusNamespace"];
                                                                if (serviceBusNamespaceValue2 != null && serviceBusNamespaceValue2.Type != JTokenType.Null)
                                                                {
                                                                    string serviceBusNamespaceInstance2 = ((string)serviceBusNamespaceValue2);
                                                                    propertiesInstance16.ServiceBusNamespace = serviceBusNamespaceInstance2;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyNameValue3 = propertiesValue17["sharedAccessPolicyName"];
                                                                if (sharedAccessPolicyNameValue3 != null && sharedAccessPolicyNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyNameInstance3 = ((string)sharedAccessPolicyNameValue3);
                                                                    propertiesInstance16.SharedAccessPolicyName = sharedAccessPolicyNameInstance3;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyKeyValue3 = propertiesValue17["sharedAccessPolicyKey"];
                                                                if (sharedAccessPolicyKeyValue3 != null && sharedAccessPolicyKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyKeyInstance3 = ((string)sharedAccessPolicyKeyValue3);
                                                                    propertiesInstance16.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance3;
                                                                }
                                                                
                                                                JToken eventHubNameValue2 = propertiesValue17["eventHubName"];
                                                                if (eventHubNameValue2 != null && eventHubNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string eventHubNameInstance2 = ((string)eventHubNameValue2);
                                                                    propertiesInstance16.EventHubName = eventHubNameInstance2;
                                                                }
                                                                
                                                                JToken partitionKeyValue2 = propertiesValue17["partitionKey"];
                                                                if (partitionKeyValue2 != null && partitionKeyValue2.Type != JTokenType.Null)
                                                                {
                                                                    string partitionKeyInstance2 = ((string)partitionKeyValue2);
                                                                    propertiesInstance16.PartitionKey = partitionKeyInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue16 = datasourceValue3["type"];
                                                            if (typeValue16 != null && typeValue16.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance16 = ((string)typeValue16);
                                                                eventHubOutputDataSourceInstance.Type = typeInstance16;
                                                            }
                                                            propertiesInstance13.DataSource = eventHubOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.ServiceBus/Queue")
                                                        {
                                                            ServiceBusQueueOutputDataSource serviceBusQueueOutputDataSourceInstance = new ServiceBusQueueOutputDataSource();
                                                            
                                                            JToken propertiesValue18 = datasourceValue3["properties"];
                                                            if (propertiesValue18 != null && propertiesValue18.Type != JTokenType.Null)
                                                            {
                                                                ServiceBusQueueOutputDataSourceProperties propertiesInstance17 = new ServiceBusQueueOutputDataSourceProperties();
                                                                serviceBusQueueOutputDataSourceInstance.Properties = propertiesInstance17;
                                                                
                                                                JToken serviceBusNamespaceValue3 = propertiesValue18["serviceBusNamespace"];
                                                                if (serviceBusNamespaceValue3 != null && serviceBusNamespaceValue3.Type != JTokenType.Null)
                                                                {
                                                                    string serviceBusNamespaceInstance3 = ((string)serviceBusNamespaceValue3);
                                                                    propertiesInstance17.ServiceBusNamespace = serviceBusNamespaceInstance3;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyNameValue4 = propertiesValue18["sharedAccessPolicyName"];
                                                                if (sharedAccessPolicyNameValue4 != null && sharedAccessPolicyNameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyNameInstance4 = ((string)sharedAccessPolicyNameValue4);
                                                                    propertiesInstance17.SharedAccessPolicyName = sharedAccessPolicyNameInstance4;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyKeyValue4 = propertiesValue18["sharedAccessPolicyKey"];
                                                                if (sharedAccessPolicyKeyValue4 != null && sharedAccessPolicyKeyValue4.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyKeyInstance4 = ((string)sharedAccessPolicyKeyValue4);
                                                                    propertiesInstance17.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance4;
                                                                }
                                                                
                                                                JToken queueNameValue = propertiesValue18["queueName"];
                                                                if (queueNameValue != null && queueNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string queueNameInstance = ((string)queueNameValue);
                                                                    propertiesInstance17.QueueName = queueNameInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue17 = datasourceValue3["type"];
                                                            if (typeValue17 != null && typeValue17.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance17 = ((string)typeValue17);
                                                                serviceBusQueueOutputDataSourceInstance.Type = typeInstance17;
                                                            }
                                                            propertiesInstance13.DataSource = serviceBusQueueOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.ServiceBus/Topic")
                                                        {
                                                            ServiceBusTopicOutputDataSource serviceBusTopicOutputDataSourceInstance = new ServiceBusTopicOutputDataSource();
                                                            
                                                            JToken propertiesValue19 = datasourceValue3["properties"];
                                                            if (propertiesValue19 != null && propertiesValue19.Type != JTokenType.Null)
                                                            {
                                                                ServiceBusTopicOutputDataSourceProperties propertiesInstance18 = new ServiceBusTopicOutputDataSourceProperties();
                                                                serviceBusTopicOutputDataSourceInstance.Properties = propertiesInstance18;
                                                                
                                                                JToken serviceBusNamespaceValue4 = propertiesValue19["serviceBusNamespace"];
                                                                if (serviceBusNamespaceValue4 != null && serviceBusNamespaceValue4.Type != JTokenType.Null)
                                                                {
                                                                    string serviceBusNamespaceInstance4 = ((string)serviceBusNamespaceValue4);
                                                                    propertiesInstance18.ServiceBusNamespace = serviceBusNamespaceInstance4;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyNameValue5 = propertiesValue19["sharedAccessPolicyName"];
                                                                if (sharedAccessPolicyNameValue5 != null && sharedAccessPolicyNameValue5.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyNameInstance5 = ((string)sharedAccessPolicyNameValue5);
                                                                    propertiesInstance18.SharedAccessPolicyName = sharedAccessPolicyNameInstance5;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyKeyValue5 = propertiesValue19["sharedAccessPolicyKey"];
                                                                if (sharedAccessPolicyKeyValue5 != null && sharedAccessPolicyKeyValue5.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyKeyInstance5 = ((string)sharedAccessPolicyKeyValue5);
                                                                    propertiesInstance18.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance5;
                                                                }
                                                                
                                                                JToken topicNameValue = propertiesValue19["topicName"];
                                                                if (topicNameValue != null && topicNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string topicNameInstance = ((string)topicNameValue);
                                                                    propertiesInstance18.TopicName = topicNameInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue18 = datasourceValue3["type"];
                                                            if (typeValue18 != null && typeValue18.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance18 = ((string)typeValue18);
                                                                serviceBusTopicOutputDataSourceInstance.Type = typeInstance18;
                                                            }
                                                            propertiesInstance13.DataSource = serviceBusTopicOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.Sql/Server/Database")
                                                        {
                                                            SqlAzureOutputDataSource sqlAzureOutputDataSourceInstance = new SqlAzureOutputDataSource();
                                                            
                                                            JToken propertiesValue20 = datasourceValue3["properties"];
                                                            if (propertiesValue20 != null && propertiesValue20.Type != JTokenType.Null)
                                                            {
                                                                SqlAzureOutputDataSourceProperties propertiesInstance19 = new SqlAzureOutputDataSourceProperties();
                                                                sqlAzureOutputDataSourceInstance.Properties = propertiesInstance19;
                                                                
                                                                JToken serverValue = propertiesValue20["server"];
                                                                if (serverValue != null && serverValue.Type != JTokenType.Null)
                                                                {
                                                                    string serverInstance = ((string)serverValue);
                                                                    propertiesInstance19.Server = serverInstance;
                                                                }
                                                                
                                                                JToken databaseValue = propertiesValue20["database"];
                                                                if (databaseValue != null && databaseValue.Type != JTokenType.Null)
                                                                {
                                                                    string databaseInstance = ((string)databaseValue);
                                                                    propertiesInstance19.Database = databaseInstance;
                                                                }
                                                                
                                                                JToken userValue = propertiesValue20["user"];
                                                                if (userValue != null && userValue.Type != JTokenType.Null)
                                                                {
                                                                    string userInstance = ((string)userValue);
                                                                    propertiesInstance19.User = userInstance;
                                                                }
                                                                
                                                                JToken passwordValue = propertiesValue20["password"];
                                                                if (passwordValue != null && passwordValue.Type != JTokenType.Null)
                                                                {
                                                                    string passwordInstance = ((string)passwordValue);
                                                                    propertiesInstance19.Password = passwordInstance;
                                                                }
                                                                
                                                                JToken tableValue2 = propertiesValue20["table"];
                                                                if (tableValue2 != null && tableValue2.Type != JTokenType.Null)
                                                                {
                                                                    string tableInstance2 = ((string)tableValue2);
                                                                    propertiesInstance19.Table = tableInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue19 = datasourceValue3["type"];
                                                            if (typeValue19 != null && typeValue19.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance19 = ((string)typeValue19);
                                                                sqlAzureOutputDataSourceInstance.Type = typeInstance19;
                                                            }
                                                            propertiesInstance13.DataSource = sqlAzureOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.Storage/DocumentDB")
                                                        {
                                                            DocumentDbOutputDataSource documentDbOutputDataSourceInstance = new DocumentDbOutputDataSource();
                                                            
                                                            JToken propertiesValue21 = datasourceValue3["properties"];
                                                            if (propertiesValue21 != null && propertiesValue21.Type != JTokenType.Null)
                                                            {
                                                                DocumentDbOutputDataSourceProperties propertiesInstance20 = new DocumentDbOutputDataSourceProperties();
                                                                documentDbOutputDataSourceInstance.Properties = propertiesInstance20;
                                                                
                                                                JToken accountIdValue = propertiesValue21["accountId"];
                                                                if (accountIdValue != null && accountIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string accountIdInstance = ((string)accountIdValue);
                                                                    propertiesInstance20.AccountId = accountIdInstance;
                                                                }
                                                                
                                                                JToken accountKeyValue5 = propertiesValue21["accountKey"];
                                                                if (accountKeyValue5 != null && accountKeyValue5.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance5 = ((string)accountKeyValue5);
                                                                    propertiesInstance20.AccountKey = accountKeyInstance5;
                                                                }
                                                                
                                                                JToken databaseValue2 = propertiesValue21["database"];
                                                                if (databaseValue2 != null && databaseValue2.Type != JTokenType.Null)
                                                                {
                                                                    string databaseInstance2 = ((string)databaseValue2);
                                                                    propertiesInstance20.Database = databaseInstance2;
                                                                }
                                                                
                                                                JToken collectionNamePatternValue = propertiesValue21["collectionNamePattern"];
                                                                if (collectionNamePatternValue != null && collectionNamePatternValue.Type != JTokenType.Null)
                                                                {
                                                                    string collectionNamePatternInstance = ((string)collectionNamePatternValue);
                                                                    propertiesInstance20.CollectionNamePattern = collectionNamePatternInstance;
                                                                }
                                                                
                                                                JToken partitionKeyValue3 = propertiesValue21["partitionKey"];
                                                                if (partitionKeyValue3 != null && partitionKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string partitionKeyInstance3 = ((string)partitionKeyValue3);
                                                                    propertiesInstance20.PartitionKey = partitionKeyInstance3;
                                                                }
                                                                
                                                                JToken documentIdValue = propertiesValue21["documentId"];
                                                                if (documentIdValue != null && documentIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string documentIdInstance = ((string)documentIdValue);
                                                                    propertiesInstance20.DocumentId = documentIdInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue20 = datasourceValue3["type"];
                                                            if (typeValue20 != null && typeValue20.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance20 = ((string)typeValue20);
                                                                documentDbOutputDataSourceInstance.Type = typeInstance20;
                                                            }
                                                            propertiesInstance13.DataSource = documentDbOutputDataSourceInstance;
                                                        }
                                                    }
                                                    
                                                    JToken serializationValue3 = propertiesValue14["serialization"];
                                                    if (serializationValue3 != null && serializationValue3.Type != JTokenType.Null)
                                                    {
                                                        string typeName7 = ((string)serializationValue3["type"]);
                                                        if (typeName7 == "Csv")
                                                        {
                                                            CsvSerialization csvSerializationInstance3 = new CsvSerialization();
                                                            
                                                            JToken propertiesValue22 = serializationValue3["properties"];
                                                            if (propertiesValue22 != null && propertiesValue22.Type != JTokenType.Null)
                                                            {
                                                                CsvSerializationProperties propertiesInstance21 = new CsvSerializationProperties();
                                                                csvSerializationInstance3.Properties = propertiesInstance21;
                                                                
                                                                JToken fieldDelimiterValue3 = propertiesValue22["fieldDelimiter"];
                                                                if (fieldDelimiterValue3 != null && fieldDelimiterValue3.Type != JTokenType.Null)
                                                                {
                                                                    string fieldDelimiterInstance3 = ((string)fieldDelimiterValue3);
                                                                    propertiesInstance21.FieldDelimiter = fieldDelimiterInstance3;
                                                                }
                                                                
                                                                JToken encodingValue5 = propertiesValue22["encoding"];
                                                                if (encodingValue5 != null && encodingValue5.Type != JTokenType.Null)
                                                                {
                                                                    string encodingInstance5 = ((string)encodingValue5);
                                                                    propertiesInstance21.Encoding = encodingInstance5;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue21 = serializationValue3["type"];
                                                            if (typeValue21 != null && typeValue21.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance21 = ((string)typeValue21);
                                                                csvSerializationInstance3.Type = typeInstance21;
                                                            }
                                                            propertiesInstance13.Serialization = csvSerializationInstance3;
                                                        }
                                                        if (typeName7 == "Json")
                                                        {
                                                            JsonSerialization jsonSerializationInstance3 = new JsonSerialization();
                                                            
                                                            JToken propertiesValue23 = serializationValue3["properties"];
                                                            if (propertiesValue23 != null && propertiesValue23.Type != JTokenType.Null)
                                                            {
                                                                JsonSerializationProperties propertiesInstance22 = new JsonSerializationProperties();
                                                                jsonSerializationInstance3.Properties = propertiesInstance22;
                                                                
                                                                JToken encodingValue6 = propertiesValue23["encoding"];
                                                                if (encodingValue6 != null && encodingValue6.Type != JTokenType.Null)
                                                                {
                                                                    string encodingInstance6 = ((string)encodingValue6);
                                                                    propertiesInstance22.Encoding = encodingInstance6;
                                                                }
                                                                
                                                                JToken formatValue3 = propertiesValue23["format"];
                                                                if (formatValue3 != null && formatValue3.Type != JTokenType.Null)
                                                                {
                                                                    string formatInstance3 = ((string)formatValue3);
                                                                    propertiesInstance22.Format = formatInstance3;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue22 = serializationValue3["type"];
                                                            if (typeValue22 != null && typeValue22.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance22 = ((string)typeValue22);
                                                                jsonSerializationInstance3.Type = typeInstance22;
                                                            }
                                                            propertiesInstance13.Serialization = jsonSerializationInstance3;
                                                        }
                                                        if (typeName7 == "Avro")
                                                        {
                                                            AvroSerialization avroSerializationInstance3 = new AvroSerialization();
                                                            
                                                            JToken propertiesValue24 = serializationValue3["properties"];
                                                            if (propertiesValue24 != null && propertiesValue24.Type != JTokenType.Null)
                                                            {
                                                                AvroSerializationProperties propertiesInstance23 = new AvroSerializationProperties();
                                                                avroSerializationInstance3.Properties = propertiesInstance23;
                                                            }
                                                            
                                                            JToken typeValue23 = serializationValue3["type"];
                                                            if (typeValue23 != null && typeValue23.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance23 = ((string)typeValue23);
                                                                avroSerializationInstance3.Type = typeInstance23;
                                                            }
                                                            propertiesInstance13.Serialization = avroSerializationInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken diagnosticsValue3 = propertiesValue14["diagnostics"];
                                                    if (diagnosticsValue3 != null && diagnosticsValue3.Type != JTokenType.Null)
                                                    {
                                                        Diagnostics diagnosticsInstance3 = new Diagnostics();
                                                        propertiesInstance13.Diagnostics = diagnosticsInstance3;
                                                        
                                                        JToken conditionsArray3 = diagnosticsValue3["conditions"];
                                                        if (conditionsArray3 != null && conditionsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken conditionsValue3 in ((JArray)conditionsArray3))
                                                            {
                                                                DiagnosticCondition diagnosticConditionInstance3 = new DiagnosticCondition();
                                                                diagnosticsInstance3.Conditions.Add(diagnosticConditionInstance3);
                                                                
                                                                JToken sinceValue3 = conditionsValue3["since"];
                                                                if (sinceValue3 != null && sinceValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime sinceInstance3 = ((DateTime)sinceValue3);
                                                                    diagnosticConditionInstance3.Since = sinceInstance3;
                                                                }
                                                                
                                                                JToken codeValue3 = conditionsValue3["code"];
                                                                if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string codeInstance3 = ((string)codeValue3);
                                                                    diagnosticConditionInstance3.Code = codeInstance3;
                                                                }
                                                                
                                                                JToken messageValue3 = conditionsValue3["message"];
                                                                if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                {
                                                                    string messageInstance3 = ((string)messageValue3);
                                                                    diagnosticConditionInstance3.Message = messageInstance3;
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        
                                        JToken functionsArray = propertiesValue["functions"];
                                        if (functionsArray != null && functionsArray.Type != JTokenType.Null)
                                        {
                                            propertiesInstance.Functions = new List<Function>();
                                            foreach (JToken functionsValue in ((JArray)functionsArray))
                                            {
                                                Function functionInstance = new Function();
                                                propertiesInstance.Functions.Add(functionInstance);
                                                
                                                JToken nameValue6 = functionsValue["name"];
                                                if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                {
                                                    string nameInstance6 = ((string)nameValue6);
                                                    functionInstance.Name = nameInstance6;
                                                }
                                                
                                                JToken propertiesValue25 = functionsValue["properties"];
                                                if (propertiesValue25 != null && propertiesValue25.Type != JTokenType.Null)
                                                {
                                                    string typeName8 = ((string)propertiesValue25["type"]);
                                                    if (typeName8 == "Scalar")
                                                    {
                                                        ScalarFunctionProperties scalarFunctionPropertiesInstance = new ScalarFunctionProperties();
                                                        
                                                        JToken propertiesValue26 = propertiesValue25["properties"];
                                                        if (propertiesValue26 != null && propertiesValue26.Type != JTokenType.Null)
                                                        {
                                                            ScalarFunctionConfiguration propertiesInstance24 = new ScalarFunctionConfiguration();
                                                            scalarFunctionPropertiesInstance.Properties = propertiesInstance24;
                                                            
                                                            JToken inputsArray2 = propertiesValue26["inputs"];
                                                            if (inputsArray2 != null && inputsArray2.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance24.Inputs = new List<FunctionInput>();
                                                                foreach (JToken inputsValue2 in ((JArray)inputsArray2))
                                                                {
                                                                    FunctionInput functionInputInstance = new FunctionInput();
                                                                    propertiesInstance24.Inputs.Add(functionInputInstance);
                                                                    
                                                                    JToken dataTypeValue = inputsValue2["dataType"];
                                                                    if (dataTypeValue != null && dataTypeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string dataTypeInstance = ((string)dataTypeValue);
                                                                        functionInputInstance.DataType = dataTypeInstance;
                                                                    }
                                                                    
                                                                    JToken isConfigurationParameterValue = inputsValue2["isConfigurationParameter"];
                                                                    if (isConfigurationParameterValue != null && isConfigurationParameterValue.Type != JTokenType.Null)
                                                                    {
                                                                        bool isConfigurationParameterInstance = ((bool)isConfigurationParameterValue);
                                                                        functionInputInstance.IsConfigurationParameter = isConfigurationParameterInstance;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken outputValue = propertiesValue26["output"];
                                                            if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                            {
                                                                FunctionOutput outputInstance2 = new FunctionOutput();
                                                                propertiesInstance24.Output = outputInstance2;
                                                                
                                                                JToken dataTypeValue2 = outputValue["dataType"];
                                                                if (dataTypeValue2 != null && dataTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string dataTypeInstance2 = ((string)dataTypeValue2);
                                                                    outputInstance2.DataType = dataTypeInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken bindingValue = propertiesValue26["binding"];
                                                            if (bindingValue != null && bindingValue.Type != JTokenType.Null)
                                                            {
                                                                string typeName9 = ((string)bindingValue["type"]);
                                                                if (typeName9 == "Microsoft.MachineLearning/WebService")
                                                                {
                                                                    AzureMachineLearningWebServiceFunctionBinding azureMachineLearningWebServiceFunctionBindingInstance = new AzureMachineLearningWebServiceFunctionBinding();
                                                                    
                                                                    JToken propertiesValue27 = bindingValue["properties"];
                                                                    if (propertiesValue27 != null && propertiesValue27.Type != JTokenType.Null)
                                                                    {
                                                                        AzureMachineLearningWebServiceFunctionBindingProperties propertiesInstance25 = new AzureMachineLearningWebServiceFunctionBindingProperties();
                                                                        azureMachineLearningWebServiceFunctionBindingInstance.Properties = propertiesInstance25;
                                                                        
                                                                        JToken endpointValue = propertiesValue27["endpoint"];
                                                                        if (endpointValue != null && endpointValue.Type != JTokenType.Null)
                                                                        {
                                                                            string endpointInstance = ((string)endpointValue);
                                                                            propertiesInstance25.Endpoint = endpointInstance;
                                                                        }
                                                                        
                                                                        JToken apiKeyValue = propertiesValue27["apiKey"];
                                                                        if (apiKeyValue != null && apiKeyValue.Type != JTokenType.Null)
                                                                        {
                                                                            string apiKeyInstance = ((string)apiKeyValue);
                                                                            propertiesInstance25.ApiKey = apiKeyInstance;
                                                                        }
                                                                        
                                                                        JToken inputsValue3 = propertiesValue27["inputs"];
                                                                        if (inputsValue3 != null && inputsValue3.Type != JTokenType.Null)
                                                                        {
                                                                            AzureMachineLearningWebServiceInputs inputsInstance = new AzureMachineLearningWebServiceInputs();
                                                                            propertiesInstance25.Inputs = inputsInstance;
                                                                            
                                                                            JToken nameValue7 = inputsValue3["name"];
                                                                            if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string nameInstance7 = ((string)nameValue7);
                                                                                inputsInstance.Name = nameInstance7;
                                                                            }
                                                                            
                                                                            JToken columnNamesArray = inputsValue3["columnNames"];
                                                                            if (columnNamesArray != null && columnNamesArray.Type != JTokenType.Null)
                                                                            {
                                                                                inputsInstance.ColumnNames = new List<AzureMachineLearningWebServiceInputColumn>();
                                                                                foreach (JToken columnNamesValue in ((JArray)columnNamesArray))
                                                                                {
                                                                                    AzureMachineLearningWebServiceInputColumn azureMachineLearningWebServiceInputColumnInstance = new AzureMachineLearningWebServiceInputColumn();
                                                                                    inputsInstance.ColumnNames.Add(azureMachineLearningWebServiceInputColumnInstance);
                                                                                    
                                                                                    JToken nameValue8 = columnNamesValue["name"];
                                                                                    if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                                    {
                                                                                        string nameInstance8 = ((string)nameValue8);
                                                                                        azureMachineLearningWebServiceInputColumnInstance.Name = nameInstance8;
                                                                                    }
                                                                                    
                                                                                    JToken dataTypeValue3 = columnNamesValue["dataType"];
                                                                                    if (dataTypeValue3 != null && dataTypeValue3.Type != JTokenType.Null)
                                                                                    {
                                                                                        string dataTypeInstance3 = ((string)dataTypeValue3);
                                                                                        azureMachineLearningWebServiceInputColumnInstance.DataType = dataTypeInstance3;
                                                                                    }
                                                                                    
                                                                                    JToken mapToValue = columnNamesValue["mapTo"];
                                                                                    if (mapToValue != null && mapToValue.Type != JTokenType.Null)
                                                                                    {
                                                                                        int mapToInstance = ((int)mapToValue);
                                                                                        azureMachineLearningWebServiceInputColumnInstance.MapTo = mapToInstance;
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                        
                                                                        JToken outputsArray2 = propertiesValue27["outputs"];
                                                                        if (outputsArray2 != null && outputsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            propertiesInstance25.Outputs = new List<AzureMachineLearningWebServiceOutputColumn>();
                                                                            foreach (JToken outputsValue2 in ((JArray)outputsArray2))
                                                                            {
                                                                                AzureMachineLearningWebServiceOutputColumn azureMachineLearningWebServiceOutputColumnInstance = new AzureMachineLearningWebServiceOutputColumn();
                                                                                propertiesInstance25.Outputs.Add(azureMachineLearningWebServiceOutputColumnInstance);
                                                                                
                                                                                JToken nameValue9 = outputsValue2["name"];
                                                                                if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                                {
                                                                                    string nameInstance9 = ((string)nameValue9);
                                                                                    azureMachineLearningWebServiceOutputColumnInstance.Name = nameInstance9;
                                                                                }
                                                                                
                                                                                JToken dataTypeValue4 = outputsValue2["dataType"];
                                                                                if (dataTypeValue4 != null && dataTypeValue4.Type != JTokenType.Null)
                                                                                {
                                                                                    string dataTypeInstance4 = ((string)dataTypeValue4);
                                                                                    azureMachineLearningWebServiceOutputColumnInstance.DataType = dataTypeInstance4;
                                                                                }
                                                                            }
                                                                        }
                                                                        
                                                                        JToken batchSizeValue2 = propertiesValue27["batchSize"];
                                                                        if (batchSizeValue2 != null && batchSizeValue2.Type != JTokenType.Null)
                                                                        {
                                                                            int batchSizeInstance2 = ((int)batchSizeValue2);
                                                                            propertiesInstance25.BatchSize = batchSizeInstance2;
                                                                        }
                                                                    }
                                                                    
                                                                    JToken typeValue24 = bindingValue["type"];
                                                                    if (typeValue24 != null && typeValue24.Type != JTokenType.Null)
                                                                    {
                                                                        string typeInstance24 = ((string)typeValue24);
                                                                        azureMachineLearningWebServiceFunctionBindingInstance.Type = typeInstance24;
                                                                    }
                                                                    propertiesInstance24.Binding = azureMachineLearningWebServiceFunctionBindingInstance;
                                                                }
                                                                if (typeName9 == "Microsoft.StreamAnalytics/JavascriptUdf")
                                                                {
                                                                    JavaScriptFunctionBinding javaScriptFunctionBindingInstance = new JavaScriptFunctionBinding();
                                                                    
                                                                    JToken propertiesValue28 = bindingValue["properties"];
                                                                    if (propertiesValue28 != null && propertiesValue28.Type != JTokenType.Null)
                                                                    {
                                                                        JavaScriptFunctionBindingProperties propertiesInstance26 = new JavaScriptFunctionBindingProperties();
                                                                        javaScriptFunctionBindingInstance.Properties = propertiesInstance26;
                                                                        
                                                                        JToken scriptValue = propertiesValue28["script"];
                                                                        if (scriptValue != null && scriptValue.Type != JTokenType.Null)
                                                                        {
                                                                            string scriptInstance = ((string)scriptValue);
                                                                            propertiesInstance26.Script = scriptInstance;
                                                                        }
                                                                    }
                                                                    
                                                                    JToken typeValue25 = bindingValue["type"];
                                                                    if (typeValue25 != null && typeValue25.Type != JTokenType.Null)
                                                                    {
                                                                        string typeInstance25 = ((string)typeValue25);
                                                                        javaScriptFunctionBindingInstance.Type = typeInstance25;
                                                                    }
                                                                    propertiesInstance24.Binding = javaScriptFunctionBindingInstance;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken etagValue6 = propertiesValue25["etag"];
                                                        if (etagValue6 != null && etagValue6.Type != JTokenType.Null)
                                                        {
                                                            string etagInstance6 = ((string)etagValue6);
                                                            scalarFunctionPropertiesInstance.Etag = etagInstance6;
                                                        }
                                                        
                                                        JToken typeValue26 = propertiesValue25["type"];
                                                        if (typeValue26 != null && typeValue26.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance26 = ((string)typeValue26);
                                                            scalarFunctionPropertiesInstance.Type = typeInstance26;
                                                        }
                                                        functionInstance.Properties = scalarFunctionPropertiesInstance;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = DateTime.Parse(httpResponse.Headers.GetValues("Date").FirstOrDefault(), CultureInfo.InvariantCulture, DateTimeStyles.AdjustToUniversal).ToLocalTime();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// List all the stream analytics jobs in the given subscription.
        /// </summary>
        /// <param name='parameters'>
        /// Required. The parameters required to list all the stream analytics
        /// job in the same subscription.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the list stream analytics job operation.
        /// </returns>
        public async Task<JobListResponse> ListJobsInSubscriptionAsync(JobListParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.PropertiesToExpand == null)
            {
                throw new ArgumentNullException("parameters.PropertiesToExpand");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "ListJobsInSubscriptionAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("$expand=" + Uri.EscapeDataString(parameters.PropertiesToExpand));
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    Job jobInstance = new Job();
                                    result.Value.Add(jobInstance);
                                    
                                    JToken idValue = valueValue["id"];
                                    if (idValue != null && idValue.Type != JTokenType.Null)
                                    {
                                        string idInstance = ((string)idValue);
                                        jobInstance.Id = idInstance;
                                    }
                                    
                                    JToken nameValue = valueValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        jobInstance.Name = nameInstance;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        jobInstance.Type = typeInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        jobInstance.Location = locationInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        jobInstance.Tags = new Dictionary<string, string>();
                                        foreach (JProperty property in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property.Name);
                                            string tagsValue = ((string)property.Value);
                                            jobInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        JobProperties propertiesInstance = new JobProperties();
                                        jobInstance.Properties = propertiesInstance;
                                        
                                        JToken skuValue = propertiesValue["sku"];
                                        if (skuValue != null && skuValue.Type != JTokenType.Null)
                                        {
                                            Sku skuInstance = new Sku();
                                            propertiesInstance.Sku = skuInstance;
                                            
                                            JToken nameValue2 = skuValue["name"];
                                            if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                            {
                                                string nameInstance2 = ((string)nameValue2);
                                                skuInstance.Name = nameInstance2;
                                            }
                                        }
                                        
                                        JToken eventsOutOfOrderPolicyValue = propertiesValue["eventsOutOfOrderPolicy"];
                                        if (eventsOutOfOrderPolicyValue != null && eventsOutOfOrderPolicyValue.Type != JTokenType.Null)
                                        {
                                            string eventsOutOfOrderPolicyInstance = ((string)eventsOutOfOrderPolicyValue);
                                            propertiesInstance.EventsOutOfOrderPolicy = eventsOutOfOrderPolicyInstance;
                                        }
                                        
                                        JToken outputErrorPolicyValue = propertiesValue["outputErrorPolicy"];
                                        if (outputErrorPolicyValue != null && outputErrorPolicyValue.Type != JTokenType.Null)
                                        {
                                            string outputErrorPolicyInstance = ((string)outputErrorPolicyValue);
                                            propertiesInstance.OutputErrorPolicy = outputErrorPolicyInstance;
                                        }
                                        
                                        JToken eventsOutOfOrderMaxDelayInSecondsValue = propertiesValue["eventsOutOfOrderMaxDelayInSeconds"];
                                        if (eventsOutOfOrderMaxDelayInSecondsValue != null && eventsOutOfOrderMaxDelayInSecondsValue.Type != JTokenType.Null)
                                        {
                                            int eventsOutOfOrderMaxDelayInSecondsInstance = ((int)eventsOutOfOrderMaxDelayInSecondsValue);
                                            propertiesInstance.EventsOutOfOrderMaxDelayInSeconds = eventsOutOfOrderMaxDelayInSecondsInstance;
                                        }
                                        
                                        JToken eventsLateArrivalMaxDelayInSecondsValue = propertiesValue["eventsLateArrivalMaxDelayInSeconds"];
                                        if (eventsLateArrivalMaxDelayInSecondsValue != null && eventsLateArrivalMaxDelayInSecondsValue.Type != JTokenType.Null)
                                        {
                                            int eventsLateArrivalMaxDelayInSecondsInstance = ((int)eventsLateArrivalMaxDelayInSecondsValue);
                                            propertiesInstance.EventsLateArrivalMaxDelayInSeconds = eventsLateArrivalMaxDelayInSecondsInstance;
                                        }
                                        
                                        JToken dataLocaleValue = propertiesValue["dataLocale"];
                                        if (dataLocaleValue != null && dataLocaleValue.Type != JTokenType.Null)
                                        {
                                            string dataLocaleInstance = ((string)dataLocaleValue);
                                            propertiesInstance.DataLocale = dataLocaleInstance;
                                        }
                                        
                                        JToken outputStartModeValue = propertiesValue["outputStartMode"];
                                        if (outputStartModeValue != null && outputStartModeValue.Type != JTokenType.Null)
                                        {
                                            string outputStartModeInstance = ((string)outputStartModeValue);
                                            propertiesInstance.OutputStartMode = outputStartModeInstance;
                                        }
                                        
                                        JToken outputStartTimeValue = propertiesValue["outputStartTime"];
                                        if (outputStartTimeValue != null && outputStartTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime outputStartTimeInstance = ((DateTime)outputStartTimeValue);
                                            propertiesInstance.OutputStartTime = outputStartTimeInstance;
                                        }
                                        
                                        JToken lastOutputEventTimeValue = propertiesValue["lastOutputEventTime"];
                                        if (lastOutputEventTimeValue != null && lastOutputEventTimeValue.Type != JTokenType.Null)
                                        {
                                            DateTime lastOutputEventTimeInstance = ((DateTime)lastOutputEventTimeValue);
                                            propertiesInstance.LastOutputEventTime = lastOutputEventTimeInstance;
                                        }
                                        
                                        JToken jobIdValue = propertiesValue["jobId"];
                                        if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                        {
                                            string jobIdInstance = ((string)jobIdValue);
                                            propertiesInstance.JobId = jobIdInstance;
                                        }
                                        
                                        JToken etagValue = propertiesValue["etag"];
                                        if (etagValue != null && etagValue.Type != JTokenType.Null)
                                        {
                                            string etagInstance = ((string)etagValue);
                                            propertiesInstance.Etag = etagInstance;
                                        }
                                        
                                        JToken provisioningStateValue = propertiesValue["provisioningState"];
                                        if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                        {
                                            string provisioningStateInstance = ((string)provisioningStateValue);
                                            propertiesInstance.ProvisioningState = provisioningStateInstance;
                                        }
                                        
                                        JToken jobStateValue = propertiesValue["jobState"];
                                        if (jobStateValue != null && jobStateValue.Type != JTokenType.Null)
                                        {
                                            string jobStateInstance = ((string)jobStateValue);
                                            propertiesInstance.JobState = jobStateInstance;
                                        }
                                        
                                        JToken createdDateValue = propertiesValue["createdDate"];
                                        if (createdDateValue != null && createdDateValue.Type != JTokenType.Null)
                                        {
                                            DateTime createdDateInstance = ((DateTime)createdDateValue);
                                            propertiesInstance.CreatedDate = createdDateInstance;
                                        }
                                        
                                        JToken inputsArray = propertiesValue["inputs"];
                                        if (inputsArray != null && inputsArray.Type != JTokenType.Null)
                                        {
                                            propertiesInstance.Inputs = new List<Input>();
                                            foreach (JToken inputsValue in ((JArray)inputsArray))
                                            {
                                                Input inputInstance = new Input();
                                                propertiesInstance.Inputs.Add(inputInstance);
                                                
                                                JToken nameValue3 = inputsValue["name"];
                                                if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                                {
                                                    string nameInstance3 = ((string)nameValue3);
                                                    inputInstance.Name = nameInstance3;
                                                }
                                                
                                                JToken propertiesValue2 = inputsValue["properties"];
                                                if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                                                {
                                                    string typeName = ((string)propertiesValue2["type"]);
                                                    if (typeName == "Reference")
                                                    {
                                                        ReferenceInputProperties referenceInputPropertiesInstance = new ReferenceInputProperties();
                                                        
                                                        JToken datasourceValue = propertiesValue2["datasource"];
                                                        if (datasourceValue != null && datasourceValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName2 = ((string)datasourceValue["type"]);
                                                            if (typeName2 == "Microsoft.Storage/Blob")
                                                            {
                                                                BlobReferenceInputDataSource blobReferenceInputDataSourceInstance = new BlobReferenceInputDataSource();
                                                                
                                                                JToken propertiesValue3 = datasourceValue["properties"];
                                                                if (propertiesValue3 != null && propertiesValue3.Type != JTokenType.Null)
                                                                {
                                                                    BlobReferenceInputDataSourceProperties propertiesInstance2 = new BlobReferenceInputDataSourceProperties();
                                                                    blobReferenceInputDataSourceInstance.Properties = propertiesInstance2;
                                                                    
                                                                    JToken storageAccountsArray = propertiesValue3["storageAccounts"];
                                                                    if (storageAccountsArray != null && storageAccountsArray.Type != JTokenType.Null)
                                                                    {
                                                                        propertiesInstance2.StorageAccounts = new List<StorageAccount>();
                                                                        foreach (JToken storageAccountsValue in ((JArray)storageAccountsArray))
                                                                        {
                                                                            StorageAccount storageAccountInstance = new StorageAccount();
                                                                            propertiesInstance2.StorageAccounts.Add(storageAccountInstance);
                                                                            
                                                                            JToken accountNameValue = storageAccountsValue["accountName"];
                                                                            if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                            {
                                                                                string accountNameInstance = ((string)accountNameValue);
                                                                                storageAccountInstance.AccountName = accountNameInstance;
                                                                            }
                                                                            
                                                                            JToken accountKeyValue = storageAccountsValue["accountKey"];
                                                                            if (accountKeyValue != null && accountKeyValue.Type != JTokenType.Null)
                                                                            {
                                                                                string accountKeyInstance = ((string)accountKeyValue);
                                                                                storageAccountInstance.AccountKey = accountKeyInstance;
                                                                            }
                                                                        }
                                                                    }
                                                                    
                                                                    JToken containerValue = propertiesValue3["container"];
                                                                    if (containerValue != null && containerValue.Type != JTokenType.Null)
                                                                    {
                                                                        string containerInstance = ((string)containerValue);
                                                                        propertiesInstance2.Container = containerInstance;
                                                                    }
                                                                    
                                                                    JToken pathPatternValue = propertiesValue3["pathPattern"];
                                                                    if (pathPatternValue != null && pathPatternValue.Type != JTokenType.Null)
                                                                    {
                                                                        string pathPatternInstance = ((string)pathPatternValue);
                                                                        propertiesInstance2.PathPattern = pathPatternInstance;
                                                                    }
                                                                    
                                                                    JToken dateFormatValue = propertiesValue3["dateFormat"];
                                                                    if (dateFormatValue != null && dateFormatValue.Type != JTokenType.Null)
                                                                    {
                                                                        string dateFormatInstance = ((string)dateFormatValue);
                                                                        propertiesInstance2.DateFormat = dateFormatInstance;
                                                                    }
                                                                    
                                                                    JToken timeFormatValue = propertiesValue3["timeFormat"];
                                                                    if (timeFormatValue != null && timeFormatValue.Type != JTokenType.Null)
                                                                    {
                                                                        string timeFormatInstance = ((string)timeFormatValue);
                                                                        propertiesInstance2.TimeFormat = timeFormatInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue2 = datasourceValue["type"];
                                                                if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance2 = ((string)typeValue2);
                                                                    blobReferenceInputDataSourceInstance.Type = typeInstance2;
                                                                }
                                                                referenceInputPropertiesInstance.DataSource = blobReferenceInputDataSourceInstance;
                                                            }
                                                        }
                                                        
                                                        JToken etagValue2 = propertiesValue2["etag"];
                                                        if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                        {
                                                            string etagInstance2 = ((string)etagValue2);
                                                            referenceInputPropertiesInstance.Etag = etagInstance2;
                                                        }
                                                        
                                                        JToken typeValue3 = propertiesValue2["type"];
                                                        if (typeValue3 != null && typeValue3.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance3 = ((string)typeValue3);
                                                            referenceInputPropertiesInstance.Type = typeInstance3;
                                                        }
                                                        
                                                        JToken serializationValue = propertiesValue2["serialization"];
                                                        if (serializationValue != null && serializationValue.Type != JTokenType.Null)
                                                        {
                                                            string typeName3 = ((string)serializationValue["type"]);
                                                            if (typeName3 == "Csv")
                                                            {
                                                                CsvSerialization csvSerializationInstance = new CsvSerialization();
                                                                
                                                                JToken propertiesValue4 = serializationValue["properties"];
                                                                if (propertiesValue4 != null && propertiesValue4.Type != JTokenType.Null)
                                                                {
                                                                    CsvSerializationProperties propertiesInstance3 = new CsvSerializationProperties();
                                                                    csvSerializationInstance.Properties = propertiesInstance3;
                                                                    
                                                                    JToken fieldDelimiterValue = propertiesValue4["fieldDelimiter"];
                                                                    if (fieldDelimiterValue != null && fieldDelimiterValue.Type != JTokenType.Null)
                                                                    {
                                                                        string fieldDelimiterInstance = ((string)fieldDelimiterValue);
                                                                        propertiesInstance3.FieldDelimiter = fieldDelimiterInstance;
                                                                    }
                                                                    
                                                                    JToken encodingValue = propertiesValue4["encoding"];
                                                                    if (encodingValue != null && encodingValue.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance = ((string)encodingValue);
                                                                        propertiesInstance3.Encoding = encodingInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue4 = serializationValue["type"];
                                                                if (typeValue4 != null && typeValue4.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance4 = ((string)typeValue4);
                                                                    csvSerializationInstance.Type = typeInstance4;
                                                                }
                                                                referenceInputPropertiesInstance.Serialization = csvSerializationInstance;
                                                            }
                                                            if (typeName3 == "Json")
                                                            {
                                                                JsonSerialization jsonSerializationInstance = new JsonSerialization();
                                                                
                                                                JToken propertiesValue5 = serializationValue["properties"];
                                                                if (propertiesValue5 != null && propertiesValue5.Type != JTokenType.Null)
                                                                {
                                                                    JsonSerializationProperties propertiesInstance4 = new JsonSerializationProperties();
                                                                    jsonSerializationInstance.Properties = propertiesInstance4;
                                                                    
                                                                    JToken encodingValue2 = propertiesValue5["encoding"];
                                                                    if (encodingValue2 != null && encodingValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance2 = ((string)encodingValue2);
                                                                        propertiesInstance4.Encoding = encodingInstance2;
                                                                    }
                                                                    
                                                                    JToken formatValue = propertiesValue5["format"];
                                                                    if (formatValue != null && formatValue.Type != JTokenType.Null)
                                                                    {
                                                                        string formatInstance = ((string)formatValue);
                                                                        propertiesInstance4.Format = formatInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue5 = serializationValue["type"];
                                                                if (typeValue5 != null && typeValue5.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance5 = ((string)typeValue5);
                                                                    jsonSerializationInstance.Type = typeInstance5;
                                                                }
                                                                referenceInputPropertiesInstance.Serialization = jsonSerializationInstance;
                                                            }
                                                            if (typeName3 == "Avro")
                                                            {
                                                                AvroSerialization avroSerializationInstance = new AvroSerialization();
                                                                
                                                                JToken propertiesValue6 = serializationValue["properties"];
                                                                if (propertiesValue6 != null && propertiesValue6.Type != JTokenType.Null)
                                                                {
                                                                    AvroSerializationProperties propertiesInstance5 = new AvroSerializationProperties();
                                                                    avroSerializationInstance.Properties = propertiesInstance5;
                                                                }
                                                                
                                                                JToken typeValue6 = serializationValue["type"];
                                                                if (typeValue6 != null && typeValue6.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance6 = ((string)typeValue6);
                                                                    avroSerializationInstance.Type = typeInstance6;
                                                                }
                                                                referenceInputPropertiesInstance.Serialization = avroSerializationInstance;
                                                            }
                                                        }
                                                        
                                                        JToken diagnosticsValue = propertiesValue2["diagnostics"];
                                                        if (diagnosticsValue != null && diagnosticsValue.Type != JTokenType.Null)
                                                        {
                                                            Diagnostics diagnosticsInstance = new Diagnostics();
                                                            referenceInputPropertiesInstance.Diagnostics = diagnosticsInstance;
                                                            
                                                            JToken conditionsArray = diagnosticsValue["conditions"];
                                                            if (conditionsArray != null && conditionsArray.Type != JTokenType.Null)
                                                            {
                                                                foreach (JToken conditionsValue in ((JArray)conditionsArray))
                                                                {
                                                                    DiagnosticCondition diagnosticConditionInstance = new DiagnosticCondition();
                                                                    diagnosticsInstance.Conditions.Add(diagnosticConditionInstance);
                                                                    
                                                                    JToken sinceValue = conditionsValue["since"];
                                                                    if (sinceValue != null && sinceValue.Type != JTokenType.Null)
                                                                    {
                                                                        DateTime sinceInstance = ((DateTime)sinceValue);
                                                                        diagnosticConditionInstance.Since = sinceInstance;
                                                                    }
                                                                    
                                                                    JToken codeValue = conditionsValue["code"];
                                                                    if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance = ((string)codeValue);
                                                                        diagnosticConditionInstance.Code = codeInstance;
                                                                    }
                                                                    
                                                                    JToken messageValue = conditionsValue["message"];
                                                                    if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance = ((string)messageValue);
                                                                        diagnosticConditionInstance.Message = messageInstance;
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        inputInstance.Properties = referenceInputPropertiesInstance;
                                                    }
                                                    if (typeName == "Stream")
                                                    {
                                                        StreamInputProperties streamInputPropertiesInstance = new StreamInputProperties();
                                                        
                                                        JToken datasourceValue2 = propertiesValue2["datasource"];
                                                        if (datasourceValue2 != null && datasourceValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName4 = ((string)datasourceValue2["type"]);
                                                            if (typeName4 == "Microsoft.Storage/Blob")
                                                            {
                                                                BlobStreamInputDataSource blobStreamInputDataSourceInstance = new BlobStreamInputDataSource();
                                                                
                                                                JToken propertiesValue7 = datasourceValue2["properties"];
                                                                if (propertiesValue7 != null && propertiesValue7.Type != JTokenType.Null)
                                                                {
                                                                    BlobStreamInputDataSourceProperties propertiesInstance6 = new BlobStreamInputDataSourceProperties();
                                                                    blobStreamInputDataSourceInstance.Properties = propertiesInstance6;
                                                                    
                                                                    JToken storageAccountsArray2 = propertiesValue7["storageAccounts"];
                                                                    if (storageAccountsArray2 != null && storageAccountsArray2.Type != JTokenType.Null)
                                                                    {
                                                                        propertiesInstance6.StorageAccounts = new List<StorageAccount>();
                                                                        foreach (JToken storageAccountsValue2 in ((JArray)storageAccountsArray2))
                                                                        {
                                                                            StorageAccount storageAccountInstance2 = new StorageAccount();
                                                                            propertiesInstance6.StorageAccounts.Add(storageAccountInstance2);
                                                                            
                                                                            JToken accountNameValue2 = storageAccountsValue2["accountName"];
                                                                            if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string accountNameInstance2 = ((string)accountNameValue2);
                                                                                storageAccountInstance2.AccountName = accountNameInstance2;
                                                                            }
                                                                            
                                                                            JToken accountKeyValue2 = storageAccountsValue2["accountKey"];
                                                                            if (accountKeyValue2 != null && accountKeyValue2.Type != JTokenType.Null)
                                                                            {
                                                                                string accountKeyInstance2 = ((string)accountKeyValue2);
                                                                                storageAccountInstance2.AccountKey = accountKeyInstance2;
                                                                            }
                                                                        }
                                                                    }
                                                                    
                                                                    JToken containerValue2 = propertiesValue7["container"];
                                                                    if (containerValue2 != null && containerValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string containerInstance2 = ((string)containerValue2);
                                                                        propertiesInstance6.Container = containerInstance2;
                                                                    }
                                                                    
                                                                    JToken pathPatternValue2 = propertiesValue7["pathPattern"];
                                                                    if (pathPatternValue2 != null && pathPatternValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string pathPatternInstance2 = ((string)pathPatternValue2);
                                                                        propertiesInstance6.PathPattern = pathPatternInstance2;
                                                                    }
                                                                    
                                                                    JToken dateFormatValue2 = propertiesValue7["dateFormat"];
                                                                    if (dateFormatValue2 != null && dateFormatValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string dateFormatInstance2 = ((string)dateFormatValue2);
                                                                        propertiesInstance6.DateFormat = dateFormatInstance2;
                                                                    }
                                                                    
                                                                    JToken timeFormatValue2 = propertiesValue7["timeFormat"];
                                                                    if (timeFormatValue2 != null && timeFormatValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string timeFormatInstance2 = ((string)timeFormatValue2);
                                                                        propertiesInstance6.TimeFormat = timeFormatInstance2;
                                                                    }
                                                                    
                                                                    JToken sourcePartitionCountValue = propertiesValue7["sourcePartitionCount"];
                                                                    if (sourcePartitionCountValue != null && sourcePartitionCountValue.Type != JTokenType.Null)
                                                                    {
                                                                        int sourcePartitionCountInstance = ((int)sourcePartitionCountValue);
                                                                        propertiesInstance6.SourcePartitionCount = sourcePartitionCountInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue7 = datasourceValue2["type"];
                                                                if (typeValue7 != null && typeValue7.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance7 = ((string)typeValue7);
                                                                    blobStreamInputDataSourceInstance.Type = typeInstance7;
                                                                }
                                                                streamInputPropertiesInstance.DataSource = blobStreamInputDataSourceInstance;
                                                            }
                                                            if (typeName4 == "Microsoft.ServiceBus/EventHub")
                                                            {
                                                                EventHubStreamInputDataSource eventHubStreamInputDataSourceInstance = new EventHubStreamInputDataSource();
                                                                
                                                                JToken propertiesValue8 = datasourceValue2["properties"];
                                                                if (propertiesValue8 != null && propertiesValue8.Type != JTokenType.Null)
                                                                {
                                                                    EventHubStreamInputDataSourceProperties propertiesInstance7 = new EventHubStreamInputDataSourceProperties();
                                                                    eventHubStreamInputDataSourceInstance.Properties = propertiesInstance7;
                                                                    
                                                                    JToken serviceBusNamespaceValue = propertiesValue8["serviceBusNamespace"];
                                                                    if (serviceBusNamespaceValue != null && serviceBusNamespaceValue.Type != JTokenType.Null)
                                                                    {
                                                                        string serviceBusNamespaceInstance = ((string)serviceBusNamespaceValue);
                                                                        propertiesInstance7.ServiceBusNamespace = serviceBusNamespaceInstance;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyNameValue = propertiesValue8["sharedAccessPolicyName"];
                                                                    if (sharedAccessPolicyNameValue != null && sharedAccessPolicyNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyNameInstance = ((string)sharedAccessPolicyNameValue);
                                                                        propertiesInstance7.SharedAccessPolicyName = sharedAccessPolicyNameInstance;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyKeyValue = propertiesValue8["sharedAccessPolicyKey"];
                                                                    if (sharedAccessPolicyKeyValue != null && sharedAccessPolicyKeyValue.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyKeyInstance = ((string)sharedAccessPolicyKeyValue);
                                                                        propertiesInstance7.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance;
                                                                    }
                                                                    
                                                                    JToken eventHubNameValue = propertiesValue8["eventHubName"];
                                                                    if (eventHubNameValue != null && eventHubNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string eventHubNameInstance = ((string)eventHubNameValue);
                                                                        propertiesInstance7.EventHubName = eventHubNameInstance;
                                                                    }
                                                                    
                                                                    JToken consumerGroupNameValue = propertiesValue8["consumerGroupName"];
                                                                    if (consumerGroupNameValue != null && consumerGroupNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string consumerGroupNameInstance = ((string)consumerGroupNameValue);
                                                                        propertiesInstance7.ConsumerGroupName = consumerGroupNameInstance;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue8 = datasourceValue2["type"];
                                                                if (typeValue8 != null && typeValue8.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance8 = ((string)typeValue8);
                                                                    eventHubStreamInputDataSourceInstance.Type = typeInstance8;
                                                                }
                                                                streamInputPropertiesInstance.DataSource = eventHubStreamInputDataSourceInstance;
                                                            }
                                                            if (typeName4 == "Microsoft.Devices/IotHubs")
                                                            {
                                                                IoTHubStreamInputDataSource ioTHubStreamInputDataSourceInstance = new IoTHubStreamInputDataSource();
                                                                
                                                                JToken propertiesValue9 = datasourceValue2["properties"];
                                                                if (propertiesValue9 != null && propertiesValue9.Type != JTokenType.Null)
                                                                {
                                                                    IoTHubStreamInputDataSourceProperties propertiesInstance8 = new IoTHubStreamInputDataSourceProperties();
                                                                    ioTHubStreamInputDataSourceInstance.Properties = propertiesInstance8;
                                                                    
                                                                    JToken iotHubNamespaceValue = propertiesValue9["iotHubNamespace"];
                                                                    if (iotHubNamespaceValue != null && iotHubNamespaceValue.Type != JTokenType.Null)
                                                                    {
                                                                        string iotHubNamespaceInstance = ((string)iotHubNamespaceValue);
                                                                        propertiesInstance8.IotHubNamespace = iotHubNamespaceInstance;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyNameValue2 = propertiesValue9["sharedAccessPolicyName"];
                                                                    if (sharedAccessPolicyNameValue2 != null && sharedAccessPolicyNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyNameInstance2 = ((string)sharedAccessPolicyNameValue2);
                                                                        propertiesInstance8.SharedAccessPolicyName = sharedAccessPolicyNameInstance2;
                                                                    }
                                                                    
                                                                    JToken sharedAccessPolicyKeyValue2 = propertiesValue9["sharedAccessPolicyKey"];
                                                                    if (sharedAccessPolicyKeyValue2 != null && sharedAccessPolicyKeyValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string sharedAccessPolicyKeyInstance2 = ((string)sharedAccessPolicyKeyValue2);
                                                                        propertiesInstance8.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance2;
                                                                    }
                                                                    
                                                                    JToken consumerGroupNameValue2 = propertiesValue9["consumerGroupName"];
                                                                    if (consumerGroupNameValue2 != null && consumerGroupNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string consumerGroupNameInstance2 = ((string)consumerGroupNameValue2);
                                                                        propertiesInstance8.ConsumerGroupName = consumerGroupNameInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue9 = datasourceValue2["type"];
                                                                if (typeValue9 != null && typeValue9.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance9 = ((string)typeValue9);
                                                                    ioTHubStreamInputDataSourceInstance.Type = typeInstance9;
                                                                }
                                                                streamInputPropertiesInstance.DataSource = ioTHubStreamInputDataSourceInstance;
                                                            }
                                                        }
                                                        
                                                        JToken etagValue3 = propertiesValue2["etag"];
                                                        if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                                        {
                                                            string etagInstance3 = ((string)etagValue3);
                                                            streamInputPropertiesInstance.Etag = etagInstance3;
                                                        }
                                                        
                                                        JToken typeValue10 = propertiesValue2["type"];
                                                        if (typeValue10 != null && typeValue10.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance10 = ((string)typeValue10);
                                                            streamInputPropertiesInstance.Type = typeInstance10;
                                                        }
                                                        
                                                        JToken serializationValue2 = propertiesValue2["serialization"];
                                                        if (serializationValue2 != null && serializationValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeName5 = ((string)serializationValue2["type"]);
                                                            if (typeName5 == "Csv")
                                                            {
                                                                CsvSerialization csvSerializationInstance2 = new CsvSerialization();
                                                                
                                                                JToken propertiesValue10 = serializationValue2["properties"];
                                                                if (propertiesValue10 != null && propertiesValue10.Type != JTokenType.Null)
                                                                {
                                                                    CsvSerializationProperties propertiesInstance9 = new CsvSerializationProperties();
                                                                    csvSerializationInstance2.Properties = propertiesInstance9;
                                                                    
                                                                    JToken fieldDelimiterValue2 = propertiesValue10["fieldDelimiter"];
                                                                    if (fieldDelimiterValue2 != null && fieldDelimiterValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string fieldDelimiterInstance2 = ((string)fieldDelimiterValue2);
                                                                        propertiesInstance9.FieldDelimiter = fieldDelimiterInstance2;
                                                                    }
                                                                    
                                                                    JToken encodingValue3 = propertiesValue10["encoding"];
                                                                    if (encodingValue3 != null && encodingValue3.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance3 = ((string)encodingValue3);
                                                                        propertiesInstance9.Encoding = encodingInstance3;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue11 = serializationValue2["type"];
                                                                if (typeValue11 != null && typeValue11.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance11 = ((string)typeValue11);
                                                                    csvSerializationInstance2.Type = typeInstance11;
                                                                }
                                                                streamInputPropertiesInstance.Serialization = csvSerializationInstance2;
                                                            }
                                                            if (typeName5 == "Json")
                                                            {
                                                                JsonSerialization jsonSerializationInstance2 = new JsonSerialization();
                                                                
                                                                JToken propertiesValue11 = serializationValue2["properties"];
                                                                if (propertiesValue11 != null && propertiesValue11.Type != JTokenType.Null)
                                                                {
                                                                    JsonSerializationProperties propertiesInstance10 = new JsonSerializationProperties();
                                                                    jsonSerializationInstance2.Properties = propertiesInstance10;
                                                                    
                                                                    JToken encodingValue4 = propertiesValue11["encoding"];
                                                                    if (encodingValue4 != null && encodingValue4.Type != JTokenType.Null)
                                                                    {
                                                                        string encodingInstance4 = ((string)encodingValue4);
                                                                        propertiesInstance10.Encoding = encodingInstance4;
                                                                    }
                                                                    
                                                                    JToken formatValue2 = propertiesValue11["format"];
                                                                    if (formatValue2 != null && formatValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string formatInstance2 = ((string)formatValue2);
                                                                        propertiesInstance10.Format = formatInstance2;
                                                                    }
                                                                }
                                                                
                                                                JToken typeValue12 = serializationValue2["type"];
                                                                if (typeValue12 != null && typeValue12.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance12 = ((string)typeValue12);
                                                                    jsonSerializationInstance2.Type = typeInstance12;
                                                                }
                                                                streamInputPropertiesInstance.Serialization = jsonSerializationInstance2;
                                                            }
                                                            if (typeName5 == "Avro")
                                                            {
                                                                AvroSerialization avroSerializationInstance2 = new AvroSerialization();
                                                                
                                                                JToken propertiesValue12 = serializationValue2["properties"];
                                                                if (propertiesValue12 != null && propertiesValue12.Type != JTokenType.Null)
                                                                {
                                                                    AvroSerializationProperties propertiesInstance11 = new AvroSerializationProperties();
                                                                    avroSerializationInstance2.Properties = propertiesInstance11;
                                                                }
                                                                
                                                                JToken typeValue13 = serializationValue2["type"];
                                                                if (typeValue13 != null && typeValue13.Type != JTokenType.Null)
                                                                {
                                                                    string typeInstance13 = ((string)typeValue13);
                                                                    avroSerializationInstance2.Type = typeInstance13;
                                                                }
                                                                streamInputPropertiesInstance.Serialization = avroSerializationInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken diagnosticsValue2 = propertiesValue2["diagnostics"];
                                                        if (diagnosticsValue2 != null && diagnosticsValue2.Type != JTokenType.Null)
                                                        {
                                                            Diagnostics diagnosticsInstance2 = new Diagnostics();
                                                            streamInputPropertiesInstance.Diagnostics = diagnosticsInstance2;
                                                            
                                                            JToken conditionsArray2 = diagnosticsValue2["conditions"];
                                                            if (conditionsArray2 != null && conditionsArray2.Type != JTokenType.Null)
                                                            {
                                                                foreach (JToken conditionsValue2 in ((JArray)conditionsArray2))
                                                                {
                                                                    DiagnosticCondition diagnosticConditionInstance2 = new DiagnosticCondition();
                                                                    diagnosticsInstance2.Conditions.Add(diagnosticConditionInstance2);
                                                                    
                                                                    JToken sinceValue2 = conditionsValue2["since"];
                                                                    if (sinceValue2 != null && sinceValue2.Type != JTokenType.Null)
                                                                    {
                                                                        DateTime sinceInstance2 = ((DateTime)sinceValue2);
                                                                        diagnosticConditionInstance2.Since = sinceInstance2;
                                                                    }
                                                                    
                                                                    JToken codeValue2 = conditionsValue2["code"];
                                                                    if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string codeInstance2 = ((string)codeValue2);
                                                                        diagnosticConditionInstance2.Code = codeInstance2;
                                                                    }
                                                                    
                                                                    JToken messageValue2 = conditionsValue2["message"];
                                                                    if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string messageInstance2 = ((string)messageValue2);
                                                                        diagnosticConditionInstance2.Message = messageInstance2;
                                                                    }
                                                                }
                                                            }
                                                        }
                                                        inputInstance.Properties = streamInputPropertiesInstance;
                                                    }
                                                }
                                            }
                                        }
                                        
                                        JToken transformationValue = propertiesValue["transformation"];
                                        if (transformationValue != null && transformationValue.Type != JTokenType.Null)
                                        {
                                            Transformation transformationInstance = new Transformation();
                                            propertiesInstance.Transformation = transformationInstance;
                                            
                                            JToken nameValue4 = transformationValue["name"];
                                            if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                            {
                                                string nameInstance4 = ((string)nameValue4);
                                                transformationInstance.Name = nameInstance4;
                                            }
                                            
                                            JToken propertiesValue13 = transformationValue["properties"];
                                            if (propertiesValue13 != null && propertiesValue13.Type != JTokenType.Null)
                                            {
                                                TransformationProperties propertiesInstance12 = new TransformationProperties();
                                                transformationInstance.Properties = propertiesInstance12;
                                                
                                                JToken etagValue4 = propertiesValue13["etag"];
                                                if (etagValue4 != null && etagValue4.Type != JTokenType.Null)
                                                {
                                                    string etagInstance4 = ((string)etagValue4);
                                                    propertiesInstance12.Etag = etagInstance4;
                                                }
                                                
                                                JToken streamingUnitsValue = propertiesValue13["streamingUnits"];
                                                if (streamingUnitsValue != null && streamingUnitsValue.Type != JTokenType.Null)
                                                {
                                                    int streamingUnitsInstance = ((int)streamingUnitsValue);
                                                    propertiesInstance12.StreamingUnits = streamingUnitsInstance;
                                                }
                                                
                                                JToken queryValue = propertiesValue13["query"];
                                                if (queryValue != null && queryValue.Type != JTokenType.Null)
                                                {
                                                    string queryInstance = ((string)queryValue);
                                                    propertiesInstance12.Query = queryInstance;
                                                }
                                            }
                                        }
                                        
                                        JToken outputsArray = propertiesValue["outputs"];
                                        if (outputsArray != null && outputsArray.Type != JTokenType.Null)
                                        {
                                            propertiesInstance.Outputs = new List<Output>();
                                            foreach (JToken outputsValue in ((JArray)outputsArray))
                                            {
                                                Output outputInstance = new Output();
                                                propertiesInstance.Outputs.Add(outputInstance);
                                                
                                                JToken nameValue5 = outputsValue["name"];
                                                if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                                {
                                                    string nameInstance5 = ((string)nameValue5);
                                                    outputInstance.Name = nameInstance5;
                                                }
                                                
                                                JToken propertiesValue14 = outputsValue["properties"];
                                                if (propertiesValue14 != null && propertiesValue14.Type != JTokenType.Null)
                                                {
                                                    OutputProperties propertiesInstance13 = new OutputProperties();
                                                    outputInstance.Properties = propertiesInstance13;
                                                    
                                                    JToken etagValue5 = propertiesValue14["etag"];
                                                    if (etagValue5 != null && etagValue5.Type != JTokenType.Null)
                                                    {
                                                        string etagInstance5 = ((string)etagValue5);
                                                        propertiesInstance13.Etag = etagInstance5;
                                                    }
                                                    
                                                    JToken datasourceValue3 = propertiesValue14["datasource"];
                                                    if (datasourceValue3 != null && datasourceValue3.Type != JTokenType.Null)
                                                    {
                                                        string typeName6 = ((string)datasourceValue3["type"]);
                                                        if (typeName6 == "Microsoft.Storage/Blob")
                                                        {
                                                            BlobOutputDataSource blobOutputDataSourceInstance = new BlobOutputDataSource();
                                                            
                                                            JToken propertiesValue15 = datasourceValue3["properties"];
                                                            if (propertiesValue15 != null && propertiesValue15.Type != JTokenType.Null)
                                                            {
                                                                BlobOutputDataSourceProperties propertiesInstance14 = new BlobOutputDataSourceProperties();
                                                                blobOutputDataSourceInstance.Properties = propertiesInstance14;
                                                                
                                                                JToken storageAccountsArray3 = propertiesValue15["storageAccounts"];
                                                                if (storageAccountsArray3 != null && storageAccountsArray3.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance14.StorageAccounts = new System.Collections.Generic.List<Microsoft.Azure.Management.StreamAnalytics.Models.StorageAccount>();
                                                                    foreach (JToken storageAccountsValue3 in ((JArray)storageAccountsArray3))
                                                                    {
                                                                        StorageAccount storageAccountInstance3 = new StorageAccount();
                                                                        propertiesInstance14.StorageAccounts.Add(storageAccountInstance3);
                                                                        
                                                                        JToken accountNameValue3 = storageAccountsValue3["accountName"];
                                                                        if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string accountNameInstance3 = ((string)accountNameValue3);
                                                                            storageAccountInstance3.AccountName = accountNameInstance3;
                                                                        }
                                                                        
                                                                        JToken accountKeyValue3 = storageAccountsValue3["accountKey"];
                                                                        if (accountKeyValue3 != null && accountKeyValue3.Type != JTokenType.Null)
                                                                        {
                                                                            string accountKeyInstance3 = ((string)accountKeyValue3);
                                                                            storageAccountInstance3.AccountKey = accountKeyInstance3;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken containerValue3 = propertiesValue15["container"];
                                                                if (containerValue3 != null && containerValue3.Type != JTokenType.Null)
                                                                {
                                                                    string containerInstance3 = ((string)containerValue3);
                                                                    propertiesInstance14.Container = containerInstance3;
                                                                }
                                                                
                                                                JToken pathPatternValue3 = propertiesValue15["pathPattern"];
                                                                if (pathPatternValue3 != null && pathPatternValue3.Type != JTokenType.Null)
                                                                {
                                                                    string pathPatternInstance3 = ((string)pathPatternValue3);
                                                                    propertiesInstance14.PathPattern = pathPatternInstance3;
                                                                }
                                                                
                                                                JToken dateFormatValue3 = propertiesValue15["dateFormat"];
                                                                if (dateFormatValue3 != null && dateFormatValue3.Type != JTokenType.Null)
                                                                {
                                                                    string dateFormatInstance3 = ((string)dateFormatValue3);
                                                                    propertiesInstance14.DateFormat = dateFormatInstance3;
                                                                }
                                                                
                                                                JToken timeFormatValue3 = propertiesValue15["timeFormat"];
                                                                if (timeFormatValue3 != null && timeFormatValue3.Type != JTokenType.Null)
                                                                {
                                                                    string timeFormatInstance3 = ((string)timeFormatValue3);
                                                                    propertiesInstance14.TimeFormat = timeFormatInstance3;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue14 = datasourceValue3["type"];
                                                            if (typeValue14 != null && typeValue14.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance14 = ((string)typeValue14);
                                                                blobOutputDataSourceInstance.Type = typeInstance14;
                                                            }
                                                            propertiesInstance13.DataSource = blobOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.Storage/Table")
                                                        {
                                                            AzureTableOutputDataSource azureTableOutputDataSourceInstance = new AzureTableOutputDataSource();
                                                            
                                                            JToken propertiesValue16 = datasourceValue3["properties"];
                                                            if (propertiesValue16 != null && propertiesValue16.Type != JTokenType.Null)
                                                            {
                                                                AzureTableOutputDataSourceProperties propertiesInstance15 = new AzureTableOutputDataSourceProperties();
                                                                azureTableOutputDataSourceInstance.Properties = propertiesInstance15;
                                                                
                                                                JToken accountNameValue4 = propertiesValue16["accountName"];
                                                                if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance4 = ((string)accountNameValue4);
                                                                    propertiesInstance15.AccountName = accountNameInstance4;
                                                                }
                                                                
                                                                JToken accountKeyValue4 = propertiesValue16["accountKey"];
                                                                if (accountKeyValue4 != null && accountKeyValue4.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance4 = ((string)accountKeyValue4);
                                                                    propertiesInstance15.AccountKey = accountKeyInstance4;
                                                                }
                                                                
                                                                JToken tableValue = propertiesValue16["table"];
                                                                if (tableValue != null && tableValue.Type != JTokenType.Null)
                                                                {
                                                                    string tableInstance = ((string)tableValue);
                                                                    propertiesInstance15.Table = tableInstance;
                                                                }
                                                                
                                                                JToken partitionKeyValue = propertiesValue16["partitionKey"];
                                                                if (partitionKeyValue != null && partitionKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string partitionKeyInstance = ((string)partitionKeyValue);
                                                                    propertiesInstance15.PartitionKey = partitionKeyInstance;
                                                                }
                                                                
                                                                JToken rowKeyValue = propertiesValue16["rowKey"];
                                                                if (rowKeyValue != null && rowKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string rowKeyInstance = ((string)rowKeyValue);
                                                                    propertiesInstance15.RowKey = rowKeyInstance;
                                                                }
                                                                
                                                                JToken columnsToRemoveArray = propertiesValue16["columnsToRemove"];
                                                                if (columnsToRemoveArray != null && columnsToRemoveArray.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance15.ColumnsToRemove = new System.Collections.Generic.List<string>();
                                                                    foreach (JToken columnsToRemoveValue in ((JArray)columnsToRemoveArray))
                                                                    {
                                                                        propertiesInstance15.ColumnsToRemove.Add(((string)columnsToRemoveValue));
                                                                    }
                                                                }
                                                                
                                                                JToken batchSizeValue = propertiesValue16["batchSize"];
                                                                if (batchSizeValue != null && batchSizeValue.Type != JTokenType.Null)
                                                                {
                                                                    int batchSizeInstance = ((int)batchSizeValue);
                                                                    propertiesInstance15.BatchSize = batchSizeInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue15 = datasourceValue3["type"];
                                                            if (typeValue15 != null && typeValue15.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance15 = ((string)typeValue15);
                                                                azureTableOutputDataSourceInstance.Type = typeInstance15;
                                                            }
                                                            propertiesInstance13.DataSource = azureTableOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.ServiceBus/EventHub")
                                                        {
                                                            EventHubOutputDataSource eventHubOutputDataSourceInstance = new EventHubOutputDataSource();
                                                            
                                                            JToken propertiesValue17 = datasourceValue3["properties"];
                                                            if (propertiesValue17 != null && propertiesValue17.Type != JTokenType.Null)
                                                            {
                                                                EventHubOutputDataSourceProperties propertiesInstance16 = new EventHubOutputDataSourceProperties();
                                                                eventHubOutputDataSourceInstance.Properties = propertiesInstance16;
                                                                
                                                                JToken serviceBusNamespaceValue2 = propertiesValue17["serviceBusNamespace"];
                                                                if (serviceBusNamespaceValue2 != null && serviceBusNamespaceValue2.Type != JTokenType.Null)
                                                                {
                                                                    string serviceBusNamespaceInstance2 = ((string)serviceBusNamespaceValue2);
                                                                    propertiesInstance16.ServiceBusNamespace = serviceBusNamespaceInstance2;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyNameValue3 = propertiesValue17["sharedAccessPolicyName"];
                                                                if (sharedAccessPolicyNameValue3 != null && sharedAccessPolicyNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyNameInstance3 = ((string)sharedAccessPolicyNameValue3);
                                                                    propertiesInstance16.SharedAccessPolicyName = sharedAccessPolicyNameInstance3;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyKeyValue3 = propertiesValue17["sharedAccessPolicyKey"];
                                                                if (sharedAccessPolicyKeyValue3 != null && sharedAccessPolicyKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyKeyInstance3 = ((string)sharedAccessPolicyKeyValue3);
                                                                    propertiesInstance16.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance3;
                                                                }
                                                                
                                                                JToken eventHubNameValue2 = propertiesValue17["eventHubName"];
                                                                if (eventHubNameValue2 != null && eventHubNameValue2.Type != JTokenType.Null)
                                                                {
                                                                    string eventHubNameInstance2 = ((string)eventHubNameValue2);
                                                                    propertiesInstance16.EventHubName = eventHubNameInstance2;
                                                                }
                                                                
                                                                JToken partitionKeyValue2 = propertiesValue17["partitionKey"];
                                                                if (partitionKeyValue2 != null && partitionKeyValue2.Type != JTokenType.Null)
                                                                {
                                                                    string partitionKeyInstance2 = ((string)partitionKeyValue2);
                                                                    propertiesInstance16.PartitionKey = partitionKeyInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue16 = datasourceValue3["type"];
                                                            if (typeValue16 != null && typeValue16.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance16 = ((string)typeValue16);
                                                                eventHubOutputDataSourceInstance.Type = typeInstance16;
                                                            }
                                                            propertiesInstance13.DataSource = eventHubOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.ServiceBus/Queue")
                                                        {
                                                            ServiceBusQueueOutputDataSource serviceBusQueueOutputDataSourceInstance = new ServiceBusQueueOutputDataSource();
                                                            
                                                            JToken propertiesValue18 = datasourceValue3["properties"];
                                                            if (propertiesValue18 != null && propertiesValue18.Type != JTokenType.Null)
                                                            {
                                                                ServiceBusQueueOutputDataSourceProperties propertiesInstance17 = new ServiceBusQueueOutputDataSourceProperties();
                                                                serviceBusQueueOutputDataSourceInstance.Properties = propertiesInstance17;
                                                                
                                                                JToken serviceBusNamespaceValue3 = propertiesValue18["serviceBusNamespace"];
                                                                if (serviceBusNamespaceValue3 != null && serviceBusNamespaceValue3.Type != JTokenType.Null)
                                                                {
                                                                    string serviceBusNamespaceInstance3 = ((string)serviceBusNamespaceValue3);
                                                                    propertiesInstance17.ServiceBusNamespace = serviceBusNamespaceInstance3;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyNameValue4 = propertiesValue18["sharedAccessPolicyName"];
                                                                if (sharedAccessPolicyNameValue4 != null && sharedAccessPolicyNameValue4.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyNameInstance4 = ((string)sharedAccessPolicyNameValue4);
                                                                    propertiesInstance17.SharedAccessPolicyName = sharedAccessPolicyNameInstance4;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyKeyValue4 = propertiesValue18["sharedAccessPolicyKey"];
                                                                if (sharedAccessPolicyKeyValue4 != null && sharedAccessPolicyKeyValue4.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyKeyInstance4 = ((string)sharedAccessPolicyKeyValue4);
                                                                    propertiesInstance17.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance4;
                                                                }
                                                                
                                                                JToken queueNameValue = propertiesValue18["queueName"];
                                                                if (queueNameValue != null && queueNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string queueNameInstance = ((string)queueNameValue);
                                                                    propertiesInstance17.QueueName = queueNameInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue17 = datasourceValue3["type"];
                                                            if (typeValue17 != null && typeValue17.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance17 = ((string)typeValue17);
                                                                serviceBusQueueOutputDataSourceInstance.Type = typeInstance17;
                                                            }
                                                            propertiesInstance13.DataSource = serviceBusQueueOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.ServiceBus/Topic")
                                                        {
                                                            ServiceBusTopicOutputDataSource serviceBusTopicOutputDataSourceInstance = new ServiceBusTopicOutputDataSource();
                                                            
                                                            JToken propertiesValue19 = datasourceValue3["properties"];
                                                            if (propertiesValue19 != null && propertiesValue19.Type != JTokenType.Null)
                                                            {
                                                                ServiceBusTopicOutputDataSourceProperties propertiesInstance18 = new ServiceBusTopicOutputDataSourceProperties();
                                                                serviceBusTopicOutputDataSourceInstance.Properties = propertiesInstance18;
                                                                
                                                                JToken serviceBusNamespaceValue4 = propertiesValue19["serviceBusNamespace"];
                                                                if (serviceBusNamespaceValue4 != null && serviceBusNamespaceValue4.Type != JTokenType.Null)
                                                                {
                                                                    string serviceBusNamespaceInstance4 = ((string)serviceBusNamespaceValue4);
                                                                    propertiesInstance18.ServiceBusNamespace = serviceBusNamespaceInstance4;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyNameValue5 = propertiesValue19["sharedAccessPolicyName"];
                                                                if (sharedAccessPolicyNameValue5 != null && sharedAccessPolicyNameValue5.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyNameInstance5 = ((string)sharedAccessPolicyNameValue5);
                                                                    propertiesInstance18.SharedAccessPolicyName = sharedAccessPolicyNameInstance5;
                                                                }
                                                                
                                                                JToken sharedAccessPolicyKeyValue5 = propertiesValue19["sharedAccessPolicyKey"];
                                                                if (sharedAccessPolicyKeyValue5 != null && sharedAccessPolicyKeyValue5.Type != JTokenType.Null)
                                                                {
                                                                    string sharedAccessPolicyKeyInstance5 = ((string)sharedAccessPolicyKeyValue5);
                                                                    propertiesInstance18.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance5;
                                                                }
                                                                
                                                                JToken topicNameValue = propertiesValue19["topicName"];
                                                                if (topicNameValue != null && topicNameValue.Type != JTokenType.Null)
                                                                {
                                                                    string topicNameInstance = ((string)topicNameValue);
                                                                    propertiesInstance18.TopicName = topicNameInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue18 = datasourceValue3["type"];
                                                            if (typeValue18 != null && typeValue18.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance18 = ((string)typeValue18);
                                                                serviceBusTopicOutputDataSourceInstance.Type = typeInstance18;
                                                            }
                                                            propertiesInstance13.DataSource = serviceBusTopicOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.Sql/Server/Database")
                                                        {
                                                            SqlAzureOutputDataSource sqlAzureOutputDataSourceInstance = new SqlAzureOutputDataSource();
                                                            
                                                            JToken propertiesValue20 = datasourceValue3["properties"];
                                                            if (propertiesValue20 != null && propertiesValue20.Type != JTokenType.Null)
                                                            {
                                                                SqlAzureOutputDataSourceProperties propertiesInstance19 = new SqlAzureOutputDataSourceProperties();
                                                                sqlAzureOutputDataSourceInstance.Properties = propertiesInstance19;
                                                                
                                                                JToken serverValue = propertiesValue20["server"];
                                                                if (serverValue != null && serverValue.Type != JTokenType.Null)
                                                                {
                                                                    string serverInstance = ((string)serverValue);
                                                                    propertiesInstance19.Server = serverInstance;
                                                                }
                                                                
                                                                JToken databaseValue = propertiesValue20["database"];
                                                                if (databaseValue != null && databaseValue.Type != JTokenType.Null)
                                                                {
                                                                    string databaseInstance = ((string)databaseValue);
                                                                    propertiesInstance19.Database = databaseInstance;
                                                                }
                                                                
                                                                JToken userValue = propertiesValue20["user"];
                                                                if (userValue != null && userValue.Type != JTokenType.Null)
                                                                {
                                                                    string userInstance = ((string)userValue);
                                                                    propertiesInstance19.User = userInstance;
                                                                }
                                                                
                                                                JToken passwordValue = propertiesValue20["password"];
                                                                if (passwordValue != null && passwordValue.Type != JTokenType.Null)
                                                                {
                                                                    string passwordInstance = ((string)passwordValue);
                                                                    propertiesInstance19.Password = passwordInstance;
                                                                }
                                                                
                                                                JToken tableValue2 = propertiesValue20["table"];
                                                                if (tableValue2 != null && tableValue2.Type != JTokenType.Null)
                                                                {
                                                                    string tableInstance2 = ((string)tableValue2);
                                                                    propertiesInstance19.Table = tableInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue19 = datasourceValue3["type"];
                                                            if (typeValue19 != null && typeValue19.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance19 = ((string)typeValue19);
                                                                sqlAzureOutputDataSourceInstance.Type = typeInstance19;
                                                            }
                                                            propertiesInstance13.DataSource = sqlAzureOutputDataSourceInstance;
                                                        }
                                                        if (typeName6 == "Microsoft.Storage/DocumentDB")
                                                        {
                                                            DocumentDbOutputDataSource documentDbOutputDataSourceInstance = new DocumentDbOutputDataSource();
                                                            
                                                            JToken propertiesValue21 = datasourceValue3["properties"];
                                                            if (propertiesValue21 != null && propertiesValue21.Type != JTokenType.Null)
                                                            {
                                                                DocumentDbOutputDataSourceProperties propertiesInstance20 = new DocumentDbOutputDataSourceProperties();
                                                                documentDbOutputDataSourceInstance.Properties = propertiesInstance20;
                                                                
                                                                JToken accountIdValue = propertiesValue21["accountId"];
                                                                if (accountIdValue != null && accountIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string accountIdInstance = ((string)accountIdValue);
                                                                    propertiesInstance20.AccountId = accountIdInstance;
                                                                }
                                                                
                                                                JToken accountKeyValue5 = propertiesValue21["accountKey"];
                                                                if (accountKeyValue5 != null && accountKeyValue5.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance5 = ((string)accountKeyValue5);
                                                                    propertiesInstance20.AccountKey = accountKeyInstance5;
                                                                }
                                                                
                                                                JToken databaseValue2 = propertiesValue21["database"];
                                                                if (databaseValue2 != null && databaseValue2.Type != JTokenType.Null)
                                                                {
                                                                    string databaseInstance2 = ((string)databaseValue2);
                                                                    propertiesInstance20.Database = databaseInstance2;
                                                                }
                                                                
                                                                JToken collectionNamePatternValue = propertiesValue21["collectionNamePattern"];
                                                                if (collectionNamePatternValue != null && collectionNamePatternValue.Type != JTokenType.Null)
                                                                {
                                                                    string collectionNamePatternInstance = ((string)collectionNamePatternValue);
                                                                    propertiesInstance20.CollectionNamePattern = collectionNamePatternInstance;
                                                                }
                                                                
                                                                JToken partitionKeyValue3 = propertiesValue21["partitionKey"];
                                                                if (partitionKeyValue3 != null && partitionKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string partitionKeyInstance3 = ((string)partitionKeyValue3);
                                                                    propertiesInstance20.PartitionKey = partitionKeyInstance3;
                                                                }
                                                                
                                                                JToken documentIdValue = propertiesValue21["documentId"];
                                                                if (documentIdValue != null && documentIdValue.Type != JTokenType.Null)
                                                                {
                                                                    string documentIdInstance = ((string)documentIdValue);
                                                                    propertiesInstance20.DocumentId = documentIdInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue20 = datasourceValue3["type"];
                                                            if (typeValue20 != null && typeValue20.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance20 = ((string)typeValue20);
                                                                documentDbOutputDataSourceInstance.Type = typeInstance20;
                                                            }
                                                            propertiesInstance13.DataSource = documentDbOutputDataSourceInstance;
                                                        }
                                                    }
                                                    
                                                    JToken serializationValue3 = propertiesValue14["serialization"];
                                                    if (serializationValue3 != null && serializationValue3.Type != JTokenType.Null)
                                                    {
                                                        string typeName7 = ((string)serializationValue3["type"]);
                                                        if (typeName7 == "Csv")
                                                        {
                                                            CsvSerialization csvSerializationInstance3 = new CsvSerialization();
                                                            
                                                            JToken propertiesValue22 = serializationValue3["properties"];
                                                            if (propertiesValue22 != null && propertiesValue22.Type != JTokenType.Null)
                                                            {
                                                                CsvSerializationProperties propertiesInstance21 = new CsvSerializationProperties();
                                                                csvSerializationInstance3.Properties = propertiesInstance21;
                                                                
                                                                JToken fieldDelimiterValue3 = propertiesValue22["fieldDelimiter"];
                                                                if (fieldDelimiterValue3 != null && fieldDelimiterValue3.Type != JTokenType.Null)
                                                                {
                                                                    string fieldDelimiterInstance3 = ((string)fieldDelimiterValue3);
                                                                    propertiesInstance21.FieldDelimiter = fieldDelimiterInstance3;
                                                                }
                                                                
                                                                JToken encodingValue5 = propertiesValue22["encoding"];
                                                                if (encodingValue5 != null && encodingValue5.Type != JTokenType.Null)
                                                                {
                                                                    string encodingInstance5 = ((string)encodingValue5);
                                                                    propertiesInstance21.Encoding = encodingInstance5;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue21 = serializationValue3["type"];
                                                            if (typeValue21 != null && typeValue21.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance21 = ((string)typeValue21);
                                                                csvSerializationInstance3.Type = typeInstance21;
                                                            }
                                                            propertiesInstance13.Serialization = csvSerializationInstance3;
                                                        }
                                                        if (typeName7 == "Json")
                                                        {
                                                            JsonSerialization jsonSerializationInstance3 = new JsonSerialization();
                                                            
                                                            JToken propertiesValue23 = serializationValue3["properties"];
                                                            if (propertiesValue23 != null && propertiesValue23.Type != JTokenType.Null)
                                                            {
                                                                JsonSerializationProperties propertiesInstance22 = new JsonSerializationProperties();
                                                                jsonSerializationInstance3.Properties = propertiesInstance22;
                                                                
                                                                JToken encodingValue6 = propertiesValue23["encoding"];
                                                                if (encodingValue6 != null && encodingValue6.Type != JTokenType.Null)
                                                                {
                                                                    string encodingInstance6 = ((string)encodingValue6);
                                                                    propertiesInstance22.Encoding = encodingInstance6;
                                                                }
                                                                
                                                                JToken formatValue3 = propertiesValue23["format"];
                                                                if (formatValue3 != null && formatValue3.Type != JTokenType.Null)
                                                                {
                                                                    string formatInstance3 = ((string)formatValue3);
                                                                    propertiesInstance22.Format = formatInstance3;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue22 = serializationValue3["type"];
                                                            if (typeValue22 != null && typeValue22.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance22 = ((string)typeValue22);
                                                                jsonSerializationInstance3.Type = typeInstance22;
                                                            }
                                                            propertiesInstance13.Serialization = jsonSerializationInstance3;
                                                        }
                                                        if (typeName7 == "Avro")
                                                        {
                                                            AvroSerialization avroSerializationInstance3 = new AvroSerialization();
                                                            
                                                            JToken propertiesValue24 = serializationValue3["properties"];
                                                            if (propertiesValue24 != null && propertiesValue24.Type != JTokenType.Null)
                                                            {
                                                                AvroSerializationProperties propertiesInstance23 = new AvroSerializationProperties();
                                                                avroSerializationInstance3.Properties = propertiesInstance23;
                                                            }
                                                            
                                                            JToken typeValue23 = serializationValue3["type"];
                                                            if (typeValue23 != null && typeValue23.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance23 = ((string)typeValue23);
                                                                avroSerializationInstance3.Type = typeInstance23;
                                                            }
                                                            propertiesInstance13.Serialization = avroSerializationInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken diagnosticsValue3 = propertiesValue14["diagnostics"];
                                                    if (diagnosticsValue3 != null && diagnosticsValue3.Type != JTokenType.Null)
                                                    {
                                                        Diagnostics diagnosticsInstance3 = new Diagnostics();
                                                        propertiesInstance13.Diagnostics = diagnosticsInstance3;
                                                        
                                                        JToken conditionsArray3 = diagnosticsValue3["conditions"];
                                                        if (conditionsArray3 != null && conditionsArray3.Type != JTokenType.Null)
                                                        {
                                                            foreach (JToken conditionsValue3 in ((JArray)conditionsArray3))
                                                            {
                                                                DiagnosticCondition diagnosticConditionInstance3 = new DiagnosticCondition();
                                                                diagnosticsInstance3.Conditions.Add(diagnosticConditionInstance3);
                                                                
                                                                JToken sinceValue3 = conditionsValue3["since"];
                                                                if (sinceValue3 != null && sinceValue3.Type != JTokenType.Null)
                                                                {
                                                                    DateTime sinceInstance3 = ((DateTime)sinceValue3);
                                                                    diagnosticConditionInstance3.Since = sinceInstance3;
                                                                }
                                                                
                                                                JToken codeValue3 = conditionsValue3["code"];
                                                                if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                                {
                                                                    string codeInstance3 = ((string)codeValue3);
                                                                    diagnosticConditionInstance3.Code = codeInstance3;
                                                                }
                                                                
                                                                JToken messageValue3 = conditionsValue3["message"];
                                                                if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                                {
                                                                    string messageInstance3 = ((string)messageValue3);
                                                                    diagnosticConditionInstance3.Message = messageInstance3;
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        
                                        JToken functionsArray = propertiesValue["functions"];
                                        if (functionsArray != null && functionsArray.Type != JTokenType.Null)
                                        {
                                            propertiesInstance.Functions = new List<Function>();
                                            foreach (JToken functionsValue in ((JArray)functionsArray))
                                            {
                                                Function functionInstance = new Function();
                                                propertiesInstance.Functions.Add(functionInstance);
                                                
                                                JToken nameValue6 = functionsValue["name"];
                                                if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                                {
                                                    string nameInstance6 = ((string)nameValue6);
                                                    functionInstance.Name = nameInstance6;
                                                }
                                                
                                                JToken propertiesValue25 = functionsValue["properties"];
                                                if (propertiesValue25 != null && propertiesValue25.Type != JTokenType.Null)
                                                {
                                                    string typeName8 = ((string)propertiesValue25["type"]);
                                                    if (typeName8 == "Scalar")
                                                    {
                                                        ScalarFunctionProperties scalarFunctionPropertiesInstance = new ScalarFunctionProperties();
                                                        
                                                        JToken propertiesValue26 = propertiesValue25["properties"];
                                                        if (propertiesValue26 != null && propertiesValue26.Type != JTokenType.Null)
                                                        {
                                                            ScalarFunctionConfiguration propertiesInstance24 = new ScalarFunctionConfiguration();
                                                            scalarFunctionPropertiesInstance.Properties = propertiesInstance24;
                                                            
                                                            JToken inputsArray2 = propertiesValue26["inputs"];
                                                            if (inputsArray2 != null && inputsArray2.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance24.Inputs = new List<FunctionInput>();
                                                                foreach (JToken inputsValue2 in ((JArray)inputsArray2))
                                                                {
                                                                    FunctionInput functionInputInstance = new FunctionInput();
                                                                    propertiesInstance24.Inputs.Add(functionInputInstance);
                                                                    
                                                                    JToken dataTypeValue = inputsValue2["dataType"];
                                                                    if (dataTypeValue != null && dataTypeValue.Type != JTokenType.Null)
                                                                    {
                                                                        string dataTypeInstance = ((string)dataTypeValue);
                                                                        functionInputInstance.DataType = dataTypeInstance;
                                                                    }
                                                                    
                                                                    JToken isConfigurationParameterValue = inputsValue2["isConfigurationParameter"];
                                                                    if (isConfigurationParameterValue != null && isConfigurationParameterValue.Type != JTokenType.Null)
                                                                    {
                                                                        bool isConfigurationParameterInstance = ((bool)isConfigurationParameterValue);
                                                                        functionInputInstance.IsConfigurationParameter = isConfigurationParameterInstance;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken outputValue = propertiesValue26["output"];
                                                            if (outputValue != null && outputValue.Type != JTokenType.Null)
                                                            {
                                                                FunctionOutput outputInstance2 = new FunctionOutput();
                                                                propertiesInstance24.Output = outputInstance2;
                                                                
                                                                JToken dataTypeValue2 = outputValue["dataType"];
                                                                if (dataTypeValue2 != null && dataTypeValue2.Type != JTokenType.Null)
                                                                {
                                                                    string dataTypeInstance2 = ((string)dataTypeValue2);
                                                                    outputInstance2.DataType = dataTypeInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken bindingValue = propertiesValue26["binding"];
                                                            if (bindingValue != null && bindingValue.Type != JTokenType.Null)
                                                            {
                                                                string typeName9 = ((string)bindingValue["type"]);
                                                                if (typeName9 == "Microsoft.MachineLearning/WebService")
                                                                {
                                                                    AzureMachineLearningWebServiceFunctionBinding azureMachineLearningWebServiceFunctionBindingInstance = new AzureMachineLearningWebServiceFunctionBinding();
                                                                    
                                                                    JToken propertiesValue27 = bindingValue["properties"];
                                                                    if (propertiesValue27 != null && propertiesValue27.Type != JTokenType.Null)
                                                                    {
                                                                        AzureMachineLearningWebServiceFunctionBindingProperties propertiesInstance25 = new AzureMachineLearningWebServiceFunctionBindingProperties();
                                                                        azureMachineLearningWebServiceFunctionBindingInstance.Properties = propertiesInstance25;
                                                                        
                                                                        JToken endpointValue = propertiesValue27["endpoint"];
                                                                        if (endpointValue != null && endpointValue.Type != JTokenType.Null)
                                                                        {
                                                                            string endpointInstance = ((string)endpointValue);
                                                                            propertiesInstance25.Endpoint = endpointInstance;
                                                                        }
                                                                        
                                                                        JToken apiKeyValue = propertiesValue27["apiKey"];
                                                                        if (apiKeyValue != null && apiKeyValue.Type != JTokenType.Null)
                                                                        {
                                                                            string apiKeyInstance = ((string)apiKeyValue);
                                                                            propertiesInstance25.ApiKey = apiKeyInstance;
                                                                        }
                                                                        
                                                                        JToken inputsValue3 = propertiesValue27["inputs"];
                                                                        if (inputsValue3 != null && inputsValue3.Type != JTokenType.Null)
                                                                        {
                                                                            AzureMachineLearningWebServiceInputs inputsInstance = new AzureMachineLearningWebServiceInputs();
                                                                            propertiesInstance25.Inputs = inputsInstance;
                                                                            
                                                                            JToken nameValue7 = inputsValue3["name"];
                                                                            if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                                            {
                                                                                string nameInstance7 = ((string)nameValue7);
                                                                                inputsInstance.Name = nameInstance7;
                                                                            }
                                                                            
                                                                            JToken columnNamesArray = inputsValue3["columnNames"];
                                                                            if (columnNamesArray != null && columnNamesArray.Type != JTokenType.Null)
                                                                            {
                                                                                inputsInstance.ColumnNames = new List<AzureMachineLearningWebServiceInputColumn>();
                                                                                foreach (JToken columnNamesValue in ((JArray)columnNamesArray))
                                                                                {
                                                                                    AzureMachineLearningWebServiceInputColumn azureMachineLearningWebServiceInputColumnInstance = new AzureMachineLearningWebServiceInputColumn();
                                                                                    inputsInstance.ColumnNames.Add(azureMachineLearningWebServiceInputColumnInstance);
                                                                                    
                                                                                    JToken nameValue8 = columnNamesValue["name"];
                                                                                    if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                                    {
                                                                                        string nameInstance8 = ((string)nameValue8);
                                                                                        azureMachineLearningWebServiceInputColumnInstance.Name = nameInstance8;
                                                                                    }
                                                                                    
                                                                                    JToken dataTypeValue3 = columnNamesValue["dataType"];
                                                                                    if (dataTypeValue3 != null && dataTypeValue3.Type != JTokenType.Null)
                                                                                    {
                                                                                        string dataTypeInstance3 = ((string)dataTypeValue3);
                                                                                        azureMachineLearningWebServiceInputColumnInstance.DataType = dataTypeInstance3;
                                                                                    }
                                                                                    
                                                                                    JToken mapToValue = columnNamesValue["mapTo"];
                                                                                    if (mapToValue != null && mapToValue.Type != JTokenType.Null)
                                                                                    {
                                                                                        int mapToInstance = ((int)mapToValue);
                                                                                        azureMachineLearningWebServiceInputColumnInstance.MapTo = mapToInstance;
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                        
                                                                        JToken outputsArray2 = propertiesValue27["outputs"];
                                                                        if (outputsArray2 != null && outputsArray2.Type != JTokenType.Null)
                                                                        {
                                                                            propertiesInstance25.Outputs = new List<AzureMachineLearningWebServiceOutputColumn>();
                                                                            foreach (JToken outputsValue2 in ((JArray)outputsArray2))
                                                                            {
                                                                                AzureMachineLearningWebServiceOutputColumn azureMachineLearningWebServiceOutputColumnInstance = new AzureMachineLearningWebServiceOutputColumn();
                                                                                propertiesInstance25.Outputs.Add(azureMachineLearningWebServiceOutputColumnInstance);
                                                                                
                                                                                JToken nameValue9 = outputsValue2["name"];
                                                                                if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                                {
                                                                                    string nameInstance9 = ((string)nameValue9);
                                                                                    azureMachineLearningWebServiceOutputColumnInstance.Name = nameInstance9;
                                                                                }
                                                                                
                                                                                JToken dataTypeValue4 = outputsValue2["dataType"];
                                                                                if (dataTypeValue4 != null && dataTypeValue4.Type != JTokenType.Null)
                                                                                {
                                                                                    string dataTypeInstance4 = ((string)dataTypeValue4);
                                                                                    azureMachineLearningWebServiceOutputColumnInstance.DataType = dataTypeInstance4;
                                                                                }
                                                                            }
                                                                        }
                                                                        
                                                                        JToken batchSizeValue2 = propertiesValue27["batchSize"];
                                                                        if (batchSizeValue2 != null && batchSizeValue2.Type != JTokenType.Null)
                                                                        {
                                                                            int batchSizeInstance2 = ((int)batchSizeValue2);
                                                                            propertiesInstance25.BatchSize = batchSizeInstance2;
                                                                        }
                                                                    }
                                                                    
                                                                    JToken typeValue24 = bindingValue["type"];
                                                                    if (typeValue24 != null && typeValue24.Type != JTokenType.Null)
                                                                    {
                                                                        string typeInstance24 = ((string)typeValue24);
                                                                        azureMachineLearningWebServiceFunctionBindingInstance.Type = typeInstance24;
                                                                    }
                                                                    propertiesInstance24.Binding = azureMachineLearningWebServiceFunctionBindingInstance;
                                                                }
                                                                if (typeName9 == "Microsoft.StreamAnalytics/JavascriptUdf")
                                                                {
                                                                    JavaScriptFunctionBinding javaScriptFunctionBindingInstance = new JavaScriptFunctionBinding();
                                                                    
                                                                    JToken propertiesValue28 = bindingValue["properties"];
                                                                    if (propertiesValue28 != null && propertiesValue28.Type != JTokenType.Null)
                                                                    {
                                                                        JavaScriptFunctionBindingProperties propertiesInstance26 = new JavaScriptFunctionBindingProperties();
                                                                        javaScriptFunctionBindingInstance.Properties = propertiesInstance26;
                                                                        
                                                                        JToken scriptValue = propertiesValue28["script"];
                                                                        if (scriptValue != null && scriptValue.Type != JTokenType.Null)
                                                                        {
                                                                            string scriptInstance = ((string)scriptValue);
                                                                            propertiesInstance26.Script = scriptInstance;
                                                                        }
                                                                    }
                                                                    
                                                                    JToken typeValue25 = bindingValue["type"];
                                                                    if (typeValue25 != null && typeValue25.Type != JTokenType.Null)
                                                                    {
                                                                        string typeInstance25 = ((string)typeValue25);
                                                                        javaScriptFunctionBindingInstance.Type = typeInstance25;
                                                                    }
                                                                    propertiesInstance24.Binding = javaScriptFunctionBindingInstance;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken etagValue6 = propertiesValue25["etag"];
                                                        if (etagValue6 != null && etagValue6.Type != JTokenType.Null)
                                                        {
                                                            string etagInstance6 = ((string)etagValue6);
                                                            scalarFunctionPropertiesInstance.Etag = etagInstance6;
                                                        }
                                                        
                                                        JToken typeValue26 = propertiesValue25["type"];
                                                        if (typeValue26 != null && typeValue26.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance26 = ((string)typeValue26);
                                                            scalarFunctionPropertiesInstance.Type = typeInstance26;
                                                        }
                                                        functionInstance.Properties = scalarFunctionPropertiesInstance;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = DateTime.Parse(httpResponse.Headers.GetValues("Date").FirstOrDefault(), CultureInfo.InvariantCulture, DateTimeStyles.AdjustToUniversal).ToLocalTime();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Update a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters required to update a stream analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the patch job operation.
        /// </returns>
        public async Task<JobPatchResponse> PatchAsync(string resourceGroupName, string jobName, JobPatchParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (jobName == null)
            {
                throw new ArgumentNullException("jobName");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.JobPatchRequest == null)
            {
                throw new ArgumentNullException("parameters.JobPatchRequest");
            }
            if (parameters.JobPatchRequest.Properties != null)
            {
                if (parameters.JobPatchRequest.Properties.Functions != null)
                {
                    foreach (Function functionsParameterItem in parameters.JobPatchRequest.Properties.Functions)
                    {
                        if (functionsParameterItem.Name == null)
                        {
                            throw new ArgumentNullException("parameters.JobPatchRequest.Properties.Functions.Name");
                        }
                    }
                }
                if (parameters.JobPatchRequest.Properties.Inputs != null)
                {
                    foreach (Input inputsParameterItem in parameters.JobPatchRequest.Properties.Inputs)
                    {
                        if (inputsParameterItem.Name == null)
                        {
                            throw new ArgumentNullException("parameters.JobPatchRequest.Properties.Inputs.Name");
                        }
                    }
                }
                if (parameters.JobPatchRequest.Properties.Outputs != null)
                {
                    foreach (Output outputsParameterItem in parameters.JobPatchRequest.Properties.Outputs)
                    {
                        if (outputsParameterItem.Name == null)
                        {
                            throw new ArgumentNullException("parameters.JobPatchRequest.Properties.Outputs.Name");
                        }
                    }
                }
                if (parameters.JobPatchRequest.Properties.Transformation != null)
                {
                    if (parameters.JobPatchRequest.Properties.Transformation.Name == null)
                    {
                        throw new ArgumentNullException("parameters.JobPatchRequest.Properties.Transformation.Name");
                    }
                }
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "PatchAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourcegroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/Microsoft.StreamAnalytics/streamingjobs/";
            url = url + Uri.EscapeDataString(jobName);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2015-10-01");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = new HttpMethod("PATCH");
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-client-request-id", Guid.NewGuid().ToString());
                httpRequest.Headers.Add("x-ms-jobState-if-match", parameters.JobStateIfMatch);
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject jobPatchParametersValue = new JObject();
                requestDoc = jobPatchParametersValue;
                
                if (parameters.JobStateIfMatch != null)
                {
                    jobPatchParametersValue["JobStateIfMatch"] = JObject.Parse(parameters.JobStateIfMatch);
                }
                
                if (parameters.JobPatchRequest.Tags != null)
                {
                    JObject tagsDictionary = new JObject();
                    foreach (KeyValuePair<string, string> pair in parameters.JobPatchRequest.Tags)
                    {
                        string tagsKey = pair.Key;
                        string tagsValue = pair.Value;
                        tagsDictionary[tagsKey] = tagsValue;
                    }
                    jobPatchParametersValue["tags"] = tagsDictionary;
                }
                
                if (parameters.JobPatchRequest.Properties != null)
                {
                    JObject propertiesValue = new JObject();
                    jobPatchParametersValue["properties"] = propertiesValue;
                    
                    if (parameters.JobPatchRequest.Properties.Sku != null)
                    {
                        JObject skuValue = new JObject();
                        propertiesValue["sku"] = skuValue;
                        
                        if (parameters.JobPatchRequest.Properties.Sku.Name != null)
                        {
                            skuValue["name"] = parameters.JobPatchRequest.Properties.Sku.Name;
                        }
                    }
                    
                    if (parameters.JobPatchRequest.Properties.EventsOutOfOrderPolicy != null)
                    {
                        propertiesValue["eventsOutOfOrderPolicy"] = parameters.JobPatchRequest.Properties.EventsOutOfOrderPolicy;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.OutputErrorPolicy != null)
                    {
                        propertiesValue["outputErrorPolicy"] = parameters.JobPatchRequest.Properties.OutputErrorPolicy;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.EventsOutOfOrderMaxDelayInSeconds != null)
                    {
                        propertiesValue["eventsOutOfOrderMaxDelayInSeconds"] = parameters.JobPatchRequest.Properties.EventsOutOfOrderMaxDelayInSeconds.Value;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.EventsLateArrivalMaxDelayInSeconds != null)
                    {
                        propertiesValue["eventsLateArrivalMaxDelayInSeconds"] = parameters.JobPatchRequest.Properties.EventsLateArrivalMaxDelayInSeconds.Value;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.DataLocale != null)
                    {
                        propertiesValue["dataLocale"] = parameters.JobPatchRequest.Properties.DataLocale;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.OutputStartMode != null)
                    {
                        propertiesValue["outputStartMode"] = parameters.JobPatchRequest.Properties.OutputStartMode;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.OutputStartTime != null)
                    {
                        propertiesValue["outputStartTime"] = parameters.JobPatchRequest.Properties.OutputStartTime.Value;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.LastOutputEventTime != null)
                    {
                        propertiesValue["lastOutputEventTime"] = parameters.JobPatchRequest.Properties.LastOutputEventTime.Value;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.JobId != null)
                    {
                        propertiesValue["jobId"] = parameters.JobPatchRequest.Properties.JobId;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.Etag != null)
                    {
                        propertiesValue["etag"] = parameters.JobPatchRequest.Properties.Etag;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.ProvisioningState != null)
                    {
                        propertiesValue["provisioningState"] = parameters.JobPatchRequest.Properties.ProvisioningState;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.JobState != null)
                    {
                        propertiesValue["jobState"] = parameters.JobPatchRequest.Properties.JobState;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.CreatedDate != null)
                    {
                        propertiesValue["createdDate"] = parameters.JobPatchRequest.Properties.CreatedDate.Value;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.Inputs != null)
                    {
                        JArray inputsArray = new JArray();
                        foreach (Input inputsItem in parameters.JobPatchRequest.Properties.Inputs)
                        {
                            JObject inputValue = new JObject();
                            inputsArray.Add(inputValue);
                            
                            inputValue["name"] = inputsItem.Name;
                            
                            if (inputsItem.Properties != null)
                            {
                                JObject propertiesValue2 = new JObject();
                                inputValue["properties"] = propertiesValue2;
                                if (inputsItem.Properties is ReferenceInputProperties)
                                {
                                    propertiesValue2["type"] = "Reference";
                                    ReferenceInputProperties derived = ((ReferenceInputProperties)inputsItem.Properties);
                                    
                                    if (derived.DataSource != null)
                                    {
                                        JObject datasourceValue = new JObject();
                                        propertiesValue2["datasource"] = datasourceValue;
                                        if (derived.DataSource is BlobReferenceInputDataSource)
                                        {
                                            datasourceValue["type"] = "Microsoft.Storage/Blob";
                                            BlobReferenceInputDataSource derived2 = ((BlobReferenceInputDataSource)derived.DataSource);
                                            
                                            if (derived2.Properties != null)
                                            {
                                                JObject propertiesValue3 = new JObject();
                                                datasourceValue["properties"] = propertiesValue3;
                                                
                                                if (derived2.Properties.StorageAccounts != null)
                                                {
                                                    JArray storageAccountsArray = new JArray();
                                                    foreach (StorageAccount storageAccountsItem in derived2.Properties.StorageAccounts)
                                                    {
                                                        JObject storageAccountValue = new JObject();
                                                        storageAccountsArray.Add(storageAccountValue);
                                                        
                                                        if (storageAccountsItem.AccountName != null)
                                                        {
                                                            storageAccountValue["accountName"] = storageAccountsItem.AccountName;
                                                        }
                                                        
                                                        if (storageAccountsItem.AccountKey != null)
                                                        {
                                                            storageAccountValue["accountKey"] = storageAccountsItem.AccountKey;
                                                        }
                                                    }
                                                    propertiesValue3["storageAccounts"] = storageAccountsArray;
                                                }
                                                
                                                if (derived2.Properties.Container != null)
                                                {
                                                    propertiesValue3["container"] = derived2.Properties.Container;
                                                }
                                                
                                                if (derived2.Properties.PathPattern != null)
                                                {
                                                    propertiesValue3["pathPattern"] = derived2.Properties.PathPattern;
                                                }
                                                
                                                if (derived2.Properties.DateFormat != null)
                                                {
                                                    propertiesValue3["dateFormat"] = derived2.Properties.DateFormat;
                                                }
                                                
                                                if (derived2.Properties.TimeFormat != null)
                                                {
                                                    propertiesValue3["timeFormat"] = derived2.Properties.TimeFormat;
                                                }
                                            }
                                            
                                            if (derived2.Type != null)
                                            {
                                                datasourceValue["type"] = derived2.Type;
                                            }
                                        }
                                    }
                                    
                                    if (derived.Etag != null)
                                    {
                                        propertiesValue2["etag"] = derived.Etag;
                                    }
                                    
                                    if (derived.Type != null)
                                    {
                                        propertiesValue2["type"] = derived.Type;
                                    }
                                    
                                    if (derived.Serialization != null)
                                    {
                                        JObject serializationValue = new JObject();
                                        propertiesValue2["serialization"] = serializationValue;
                                        if (derived.Serialization is CsvSerialization)
                                        {
                                            serializationValue["type"] = "Csv";
                                            CsvSerialization derived3 = ((CsvSerialization)derived.Serialization);
                                            
                                            if (derived3.Properties != null)
                                            {
                                                JObject propertiesValue4 = new JObject();
                                                serializationValue["properties"] = propertiesValue4;
                                                
                                                if (derived3.Properties.FieldDelimiter != null)
                                                {
                                                    propertiesValue4["fieldDelimiter"] = derived3.Properties.FieldDelimiter;
                                                }
                                                
                                                if (derived3.Properties.Encoding != null)
                                                {
                                                    propertiesValue4["encoding"] = derived3.Properties.Encoding;
                                                }
                                            }
                                            
                                            if (derived3.Type != null)
                                            {
                                                serializationValue["type"] = derived3.Type;
                                            }
                                        }
                                        if (derived.Serialization is JsonSerialization)
                                        {
                                            serializationValue["type"] = "Json";
                                            JsonSerialization derived4 = ((JsonSerialization)derived.Serialization);
                                            
                                            if (derived4.Properties != null)
                                            {
                                                JObject propertiesValue5 = new JObject();
                                                serializationValue["properties"] = propertiesValue5;
                                                
                                                if (derived4.Properties.Encoding != null)
                                                {
                                                    propertiesValue5["encoding"] = derived4.Properties.Encoding;
                                                }
                                                
                                                if (derived4.Properties.Format != null)
                                                {
                                                    propertiesValue5["format"] = derived4.Properties.Format;
                                                }
                                            }
                                            
                                            if (derived4.Type != null)
                                            {
                                                serializationValue["type"] = derived4.Type;
                                            }
                                        }
                                        if (derived.Serialization is AvroSerialization)
                                        {
                                            serializationValue["type"] = "Avro";
                                            AvroSerialization derived5 = ((AvroSerialization)derived.Serialization);
                                            
                                            if (derived5.Properties != null)
                                            {
                                                serializationValue["properties"] = derived5.Properties.ToString();
                                            }
                                            
                                            if (derived5.Type != null)
                                            {
                                                serializationValue["type"] = derived5.Type;
                                            }
                                        }
                                    }
                                    
                                    if (derived.Diagnostics != null)
                                    {
                                        JObject diagnosticsValue = new JObject();
                                        propertiesValue2["diagnostics"] = diagnosticsValue;
                                        
                                        if (derived.Diagnostics.Conditions != null)
                                        {
                                            if (derived.Diagnostics.Conditions is ILazyCollection == false || ((ILazyCollection)derived.Diagnostics.Conditions).IsInitialized)
                                            {
                                                JArray conditionsArray = new JArray();
                                                foreach (DiagnosticCondition conditionsItem in derived.Diagnostics.Conditions)
                                                {
                                                    JObject diagnosticConditionValue = new JObject();
                                                    conditionsArray.Add(diagnosticConditionValue);
                                                    
                                                    if (conditionsItem.Since != null)
                                                    {
                                                        diagnosticConditionValue["since"] = conditionsItem.Since.Value;
                                                    }
                                                    
                                                    if (conditionsItem.Code != null)
                                                    {
                                                        diagnosticConditionValue["code"] = conditionsItem.Code;
                                                    }
                                                    
                                                    if (conditionsItem.Message != null)
                                                    {
                                                        diagnosticConditionValue["message"] = conditionsItem.Message;
                                                    }
                                                }
                                                diagnosticsValue["conditions"] = conditionsArray;
                                            }
                                        }
                                    }
                                }
                                if (inputsItem.Properties is StreamInputProperties)
                                {
                                    propertiesValue2["type"] = "Stream";
                                    StreamInputProperties derived6 = ((StreamInputProperties)inputsItem.Properties);
                                    
                                    if (derived6.DataSource != null)
                                    {
                                        JObject datasourceValue2 = new JObject();
                                        propertiesValue2["datasource"] = datasourceValue2;
                                        if (derived6.DataSource is BlobStreamInputDataSource)
                                        {
                                            datasourceValue2["type"] = "Microsoft.Storage/Blob";
                                            BlobStreamInputDataSource derived7 = ((BlobStreamInputDataSource)derived6.DataSource);
                                            
                                            if (derived7.Properties != null)
                                            {
                                                JObject propertiesValue6 = new JObject();
                                                datasourceValue2["properties"] = propertiesValue6;
                                                
                                                if (derived7.Properties.StorageAccounts != null)
                                                {
                                                    JArray storageAccountsArray2 = new JArray();
                                                    foreach (StorageAccount storageAccountsItem2 in derived7.Properties.StorageAccounts)
                                                    {
                                                        JObject storageAccountValue2 = new JObject();
                                                        storageAccountsArray2.Add(storageAccountValue2);
                                                        
                                                        if (storageAccountsItem2.AccountName != null)
                                                        {
                                                            storageAccountValue2["accountName"] = storageAccountsItem2.AccountName;
                                                        }
                                                        
                                                        if (storageAccountsItem2.AccountKey != null)
                                                        {
                                                            storageAccountValue2["accountKey"] = storageAccountsItem2.AccountKey;
                                                        }
                                                    }
                                                    propertiesValue6["storageAccounts"] = storageAccountsArray2;
                                                }
                                                
                                                if (derived7.Properties.Container != null)
                                                {
                                                    propertiesValue6["container"] = derived7.Properties.Container;
                                                }
                                                
                                                if (derived7.Properties.PathPattern != null)
                                                {
                                                    propertiesValue6["pathPattern"] = derived7.Properties.PathPattern;
                                                }
                                                
                                                if (derived7.Properties.DateFormat != null)
                                                {
                                                    propertiesValue6["dateFormat"] = derived7.Properties.DateFormat;
                                                }
                                                
                                                if (derived7.Properties.TimeFormat != null)
                                                {
                                                    propertiesValue6["timeFormat"] = derived7.Properties.TimeFormat;
                                                }
                                                
                                                if (derived7.Properties.SourcePartitionCount != null)
                                                {
                                                    propertiesValue6["sourcePartitionCount"] = derived7.Properties.SourcePartitionCount.Value;
                                                }
                                            }
                                            
                                            if (derived7.Type != null)
                                            {
                                                datasourceValue2["type"] = derived7.Type;
                                            }
                                        }
                                        if (derived6.DataSource is EventHubStreamInputDataSource)
                                        {
                                            datasourceValue2["type"] = "Microsoft.ServiceBus/EventHub";
                                            EventHubStreamInputDataSource derived8 = ((EventHubStreamInputDataSource)derived6.DataSource);
                                            
                                            if (derived8.Properties != null)
                                            {
                                                JObject propertiesValue7 = new JObject();
                                                datasourceValue2["properties"] = propertiesValue7;
                                                
                                                if (derived8.Properties.ServiceBusNamespace != null)
                                                {
                                                    propertiesValue7["serviceBusNamespace"] = derived8.Properties.ServiceBusNamespace;
                                                }
                                                
                                                if (derived8.Properties.SharedAccessPolicyName != null)
                                                {
                                                    propertiesValue7["sharedAccessPolicyName"] = derived8.Properties.SharedAccessPolicyName;
                                                }
                                                
                                                if (derived8.Properties.SharedAccessPolicyKey != null)
                                                {
                                                    propertiesValue7["sharedAccessPolicyKey"] = derived8.Properties.SharedAccessPolicyKey;
                                                }
                                                
                                                if (derived8.Properties.EventHubName != null)
                                                {
                                                    propertiesValue7["eventHubName"] = derived8.Properties.EventHubName;
                                                }
                                                
                                                if (derived8.Properties.ConsumerGroupName != null)
                                                {
                                                    propertiesValue7["consumerGroupName"] = derived8.Properties.ConsumerGroupName;
                                                }
                                            }
                                            
                                            if (derived8.Type != null)
                                            {
                                                datasourceValue2["type"] = derived8.Type;
                                            }
                                        }
                                        if (derived6.DataSource is IoTHubStreamInputDataSource)
                                        {
                                            datasourceValue2["type"] = "Microsoft.Devices/IotHubs";
                                            IoTHubStreamInputDataSource derived9 = ((IoTHubStreamInputDataSource)derived6.DataSource);
                                            
                                            if (derived9.Properties != null)
                                            {
                                                JObject propertiesValue8 = new JObject();
                                                datasourceValue2["properties"] = propertiesValue8;
                                                
                                                if (derived9.Properties.IotHubNamespace != null)
                                                {
                                                    propertiesValue8["iotHubNamespace"] = derived9.Properties.IotHubNamespace;
                                                }
                                                
                                                if (derived9.Properties.SharedAccessPolicyName != null)
                                                {
                                                    propertiesValue8["sharedAccessPolicyName"] = derived9.Properties.SharedAccessPolicyName;
                                                }
                                                
                                                if (derived9.Properties.SharedAccessPolicyKey != null)
                                                {
                                                    propertiesValue8["sharedAccessPolicyKey"] = derived9.Properties.SharedAccessPolicyKey;
                                                }
                                                
                                                if (derived9.Properties.ConsumerGroupName != null)
                                                {
                                                    propertiesValue8["consumerGroupName"] = derived9.Properties.ConsumerGroupName;
                                                }
                                            }
                                            
                                            if (derived9.Type != null)
                                            {
                                                datasourceValue2["type"] = derived9.Type;
                                            }
                                        }
                                    }
                                    
                                    if (derived6.Etag != null)
                                    {
                                        propertiesValue2["etag"] = derived6.Etag;
                                    }
                                    
                                    if (derived6.Type != null)
                                    {
                                        propertiesValue2["type"] = derived6.Type;
                                    }
                                    
                                    if (derived6.Serialization != null)
                                    {
                                        JObject serializationValue2 = new JObject();
                                        propertiesValue2["serialization"] = serializationValue2;
                                        if (derived6.Serialization is CsvSerialization)
                                        {
                                            serializationValue2["type"] = "Csv";
                                            CsvSerialization derived10 = ((CsvSerialization)derived6.Serialization);
                                            
                                            if (derived10.Properties != null)
                                            {
                                                JObject propertiesValue9 = new JObject();
                                                serializationValue2["properties"] = propertiesValue9;
                                                
                                                if (derived10.Properties.FieldDelimiter != null)
                                                {
                                                    propertiesValue9["fieldDelimiter"] = derived10.Properties.FieldDelimiter;
                                                }
                                                
                                                if (derived10.Properties.Encoding != null)
                                                {
                                                    propertiesValue9["encoding"] = derived10.Properties.Encoding;
                                                }
                                            }
                                            
                                            if (derived10.Type != null)
                                            {
                                                serializationValue2["type"] = derived10.Type;
                                            }
                                        }
                                        if (derived6.Serialization is JsonSerialization)
                                        {
                                            serializationValue2["type"] = "Json";
                                            JsonSerialization derived11 = ((JsonSerialization)derived6.Serialization);
                                            
                                            if (derived11.Properties != null)
                                            {
                                                JObject propertiesValue10 = new JObject();
                                                serializationValue2["properties"] = propertiesValue10;
                                                
                                                if (derived11.Properties.Encoding != null)
                                                {
                                                    propertiesValue10["encoding"] = derived11.Properties.Encoding;
                                                }
                                                
                                                if (derived11.Properties.Format != null)
                                                {
                                                    propertiesValue10["format"] = derived11.Properties.Format;
                                                }
                                            }
                                            
                                            if (derived11.Type != null)
                                            {
                                                serializationValue2["type"] = derived11.Type;
                                            }
                                        }
                                        if (derived6.Serialization is AvroSerialization)
                                        {
                                            serializationValue2["type"] = "Avro";
                                            AvroSerialization derived12 = ((AvroSerialization)derived6.Serialization);
                                            
                                            if (derived12.Properties != null)
                                            {
                                                serializationValue2["properties"] = derived12.Properties.ToString();
                                            }
                                            
                                            if (derived12.Type != null)
                                            {
                                                serializationValue2["type"] = derived12.Type;
                                            }
                                        }
                                    }
                                    
                                    if (derived6.Diagnostics != null)
                                    {
                                        JObject diagnosticsValue2 = new JObject();
                                        propertiesValue2["diagnostics"] = diagnosticsValue2;
                                        
                                        if (derived6.Diagnostics.Conditions != null)
                                        {
                                            if (derived6.Diagnostics.Conditions is ILazyCollection == false || ((ILazyCollection)derived6.Diagnostics.Conditions).IsInitialized)
                                            {
                                                JArray conditionsArray2 = new JArray();
                                                foreach (DiagnosticCondition conditionsItem2 in derived6.Diagnostics.Conditions)
                                                {
                                                    JObject diagnosticConditionValue2 = new JObject();
                                                    conditionsArray2.Add(diagnosticConditionValue2);
                                                    
                                                    if (conditionsItem2.Since != null)
                                                    {
                                                        diagnosticConditionValue2["since"] = conditionsItem2.Since.Value;
                                                    }
                                                    
                                                    if (conditionsItem2.Code != null)
                                                    {
                                                        diagnosticConditionValue2["code"] = conditionsItem2.Code;
                                                    }
                                                    
                                                    if (conditionsItem2.Message != null)
                                                    {
                                                        diagnosticConditionValue2["message"] = conditionsItem2.Message;
                                                    }
                                                }
                                                diagnosticsValue2["conditions"] = conditionsArray2;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        propertiesValue["inputs"] = inputsArray;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.Transformation != null)
                    {
                        JObject transformationValue = new JObject();
                        propertiesValue["transformation"] = transformationValue;
                        
                        transformationValue["name"] = parameters.JobPatchRequest.Properties.Transformation.Name;
                        
                        if (parameters.JobPatchRequest.Properties.Transformation.Properties != null)
                        {
                            JObject propertiesValue11 = new JObject();
                            transformationValue["properties"] = propertiesValue11;
                            
                            if (parameters.JobPatchRequest.Properties.Transformation.Properties.Etag != null)
                            {
                                propertiesValue11["etag"] = parameters.JobPatchRequest.Properties.Transformation.Properties.Etag;
                            }
                            
                            if (parameters.JobPatchRequest.Properties.Transformation.Properties.StreamingUnits != null)
                            {
                                propertiesValue11["streamingUnits"] = parameters.JobPatchRequest.Properties.Transformation.Properties.StreamingUnits.Value;
                            }
                            
                            if (parameters.JobPatchRequest.Properties.Transformation.Properties.Query != null)
                            {
                                propertiesValue11["query"] = parameters.JobPatchRequest.Properties.Transformation.Properties.Query;
                            }
                        }
                    }
                    
                    if (parameters.JobPatchRequest.Properties.Outputs != null)
                    {
                        JArray outputsArray = new JArray();
                        foreach (Output outputsItem in parameters.JobPatchRequest.Properties.Outputs)
                        {
                            JObject outputValue = new JObject();
                            outputsArray.Add(outputValue);
                            
                            outputValue["name"] = outputsItem.Name;
                            
                            if (outputsItem.Properties != null)
                            {
                                JObject propertiesValue12 = new JObject();
                                outputValue["properties"] = propertiesValue12;
                                
                                if (outputsItem.Properties.Etag != null)
                                {
                                    propertiesValue12["etag"] = outputsItem.Properties.Etag;
                                }
                                
                                if (outputsItem.Properties.DataSource != null)
                                {
                                    JObject datasourceValue3 = new JObject();
                                    propertiesValue12["datasource"] = datasourceValue3;
                                    if (outputsItem.Properties.DataSource is BlobOutputDataSource)
                                    {
                                        datasourceValue3["type"] = "Microsoft.Storage/Blob";
                                        BlobOutputDataSource derived13 = ((BlobOutputDataSource)outputsItem.Properties.DataSource);
                                        
                                        if (derived13.Properties != null)
                                        {
                                            JObject propertiesValue13 = new JObject();
                                            datasourceValue3["properties"] = propertiesValue13;
                                            
                                            if (derived13.Properties.StorageAccounts != null)
                                            {
                                                JArray storageAccountsArray3 = new JArray();
                                                foreach (StorageAccount storageAccountsItem3 in derived13.Properties.StorageAccounts)
                                                {
                                                    JObject storageAccountValue3 = new JObject();
                                                    storageAccountsArray3.Add(storageAccountValue3);
                                                    
                                                    if (storageAccountsItem3.AccountName != null)
                                                    {
                                                        storageAccountValue3["accountName"] = storageAccountsItem3.AccountName;
                                                    }
                                                    
                                                    if (storageAccountsItem3.AccountKey != null)
                                                    {
                                                        storageAccountValue3["accountKey"] = storageAccountsItem3.AccountKey;
                                                    }
                                                }
                                                propertiesValue13["storageAccounts"] = storageAccountsArray3;
                                            }
                                            
                                            if (derived13.Properties.Container != null)
                                            {
                                                propertiesValue13["container"] = derived13.Properties.Container;
                                            }
                                            
                                            if (derived13.Properties.PathPattern != null)
                                            {
                                                propertiesValue13["pathPattern"] = derived13.Properties.PathPattern;
                                            }
                                            
                                            if (derived13.Properties.DateFormat != null)
                                            {
                                                propertiesValue13["dateFormat"] = derived13.Properties.DateFormat;
                                            }
                                            
                                            if (derived13.Properties.TimeFormat != null)
                                            {
                                                propertiesValue13["timeFormat"] = derived13.Properties.TimeFormat;
                                            }
                                        }
                                        
                                        if (derived13.Type != null)
                                        {
                                            datasourceValue3["type"] = derived13.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.DataSource is AzureTableOutputDataSource)
                                    {
                                        datasourceValue3["type"] = "Microsoft.Storage/Table";
                                        AzureTableOutputDataSource derived14 = ((AzureTableOutputDataSource)outputsItem.Properties.DataSource);
                                        
                                        if (derived14.Properties != null)
                                        {
                                            JObject propertiesValue14 = new JObject();
                                            datasourceValue3["properties"] = propertiesValue14;
                                            
                                            if (derived14.Properties.AccountName != null)
                                            {
                                                propertiesValue14["accountName"] = derived14.Properties.AccountName;
                                            }
                                            
                                            if (derived14.Properties.AccountKey != null)
                                            {
                                                propertiesValue14["accountKey"] = derived14.Properties.AccountKey;
                                            }
                                            
                                            if (derived14.Properties.Table != null)
                                            {
                                                propertiesValue14["table"] = derived14.Properties.Table;
                                            }
                                            
                                            if (derived14.Properties.PartitionKey != null)
                                            {
                                                propertiesValue14["partitionKey"] = derived14.Properties.PartitionKey;
                                            }
                                            
                                            if (derived14.Properties.RowKey != null)
                                            {
                                                propertiesValue14["rowKey"] = derived14.Properties.RowKey;
                                            }
                                            
                                            if (derived14.Properties.ColumnsToRemove != null)
                                            {
                                                JArray columnsToRemoveArray = new JArray();
                                                foreach (string columnsToRemoveItem in derived14.Properties.ColumnsToRemove)
                                                {
                                                    columnsToRemoveArray.Add(columnsToRemoveItem);
                                                }
                                                propertiesValue14["columnsToRemove"] = columnsToRemoveArray;
                                            }
                                            
                                            if (derived14.Properties.BatchSize != null)
                                            {
                                                propertiesValue14["batchSize"] = derived14.Properties.BatchSize.Value;
                                            }
                                        }
                                        
                                        if (derived14.Type != null)
                                        {
                                            datasourceValue3["type"] = derived14.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.DataSource is EventHubOutputDataSource)
                                    {
                                        datasourceValue3["type"] = "Microsoft.ServiceBus/EventHub";
                                        EventHubOutputDataSource derived15 = ((EventHubOutputDataSource)outputsItem.Properties.DataSource);
                                        
                                        if (derived15.Properties != null)
                                        {
                                            JObject propertiesValue15 = new JObject();
                                            datasourceValue3["properties"] = propertiesValue15;
                                            
                                            if (derived15.Properties.ServiceBusNamespace != null)
                                            {
                                                propertiesValue15["serviceBusNamespace"] = derived15.Properties.ServiceBusNamespace;
                                            }
                                            
                                            if (derived15.Properties.SharedAccessPolicyName != null)
                                            {
                                                propertiesValue15["sharedAccessPolicyName"] = derived15.Properties.SharedAccessPolicyName;
                                            }
                                            
                                            if (derived15.Properties.SharedAccessPolicyKey != null)
                                            {
                                                propertiesValue15["sharedAccessPolicyKey"] = derived15.Properties.SharedAccessPolicyKey;
                                            }
                                            
                                            if (derived15.Properties.EventHubName != null)
                                            {
                                                propertiesValue15["eventHubName"] = derived15.Properties.EventHubName;
                                            }
                                            
                                            if (derived15.Properties.PartitionKey != null)
                                            {
                                                propertiesValue15["partitionKey"] = derived15.Properties.PartitionKey;
                                            }
                                        }
                                        
                                        if (derived15.Type != null)
                                        {
                                            datasourceValue3["type"] = derived15.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.DataSource is ServiceBusQueueOutputDataSource)
                                    {
                                        datasourceValue3["type"] = "Microsoft.ServiceBus/Queue";
                                        ServiceBusQueueOutputDataSource derived16 = ((ServiceBusQueueOutputDataSource)outputsItem.Properties.DataSource);
                                        
                                        if (derived16.Properties != null)
                                        {
                                            JObject propertiesValue16 = new JObject();
                                            datasourceValue3["properties"] = propertiesValue16;
                                            
                                            if (derived16.Properties.ServiceBusNamespace != null)
                                            {
                                                propertiesValue16["serviceBusNamespace"] = derived16.Properties.ServiceBusNamespace;
                                            }
                                            
                                            if (derived16.Properties.SharedAccessPolicyName != null)
                                            {
                                                propertiesValue16["sharedAccessPolicyName"] = derived16.Properties.SharedAccessPolicyName;
                                            }
                                            
                                            if (derived16.Properties.SharedAccessPolicyKey != null)
                                            {
                                                propertiesValue16["sharedAccessPolicyKey"] = derived16.Properties.SharedAccessPolicyKey;
                                            }
                                            
                                            if (derived16.Properties.QueueName != null)
                                            {
                                                propertiesValue16["queueName"] = derived16.Properties.QueueName;
                                            }
                                        }
                                        
                                        if (derived16.Type != null)
                                        {
                                            datasourceValue3["type"] = derived16.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.DataSource is ServiceBusTopicOutputDataSource)
                                    {
                                        datasourceValue3["type"] = "Microsoft.ServiceBus/Topic";
                                        ServiceBusTopicOutputDataSource derived17 = ((ServiceBusTopicOutputDataSource)outputsItem.Properties.DataSource);
                                        
                                        if (derived17.Properties != null)
                                        {
                                            JObject propertiesValue17 = new JObject();
                                            datasourceValue3["properties"] = propertiesValue17;
                                            
                                            if (derived17.Properties.ServiceBusNamespace != null)
                                            {
                                                propertiesValue17["serviceBusNamespace"] = derived17.Properties.ServiceBusNamespace;
                                            }
                                            
                                            if (derived17.Properties.SharedAccessPolicyName != null)
                                            {
                                                propertiesValue17["sharedAccessPolicyName"] = derived17.Properties.SharedAccessPolicyName;
                                            }
                                            
                                            if (derived17.Properties.SharedAccessPolicyKey != null)
                                            {
                                                propertiesValue17["sharedAccessPolicyKey"] = derived17.Properties.SharedAccessPolicyKey;
                                            }
                                            
                                            if (derived17.Properties.TopicName != null)
                                            {
                                                propertiesValue17["topicName"] = derived17.Properties.TopicName;
                                            }
                                        }
                                        
                                        if (derived17.Type != null)
                                        {
                                            datasourceValue3["type"] = derived17.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.DataSource is SqlAzureOutputDataSource)
                                    {
                                        datasourceValue3["type"] = "Microsoft.Sql/Server/Database";
                                        SqlAzureOutputDataSource derived18 = ((SqlAzureOutputDataSource)outputsItem.Properties.DataSource);
                                        
                                        if (derived18.Properties != null)
                                        {
                                            JObject propertiesValue18 = new JObject();
                                            datasourceValue3["properties"] = propertiesValue18;
                                            
                                            if (derived18.Properties.Server != null)
                                            {
                                                propertiesValue18["server"] = derived18.Properties.Server;
                                            }
                                            
                                            if (derived18.Properties.Database != null)
                                            {
                                                propertiesValue18["database"] = derived18.Properties.Database;
                                            }
                                            
                                            if (derived18.Properties.User != null)
                                            {
                                                propertiesValue18["user"] = derived18.Properties.User;
                                            }
                                            
                                            if (derived18.Properties.Password != null)
                                            {
                                                propertiesValue18["password"] = derived18.Properties.Password;
                                            }
                                            
                                            if (derived18.Properties.Table != null)
                                            {
                                                propertiesValue18["table"] = derived18.Properties.Table;
                                            }
                                        }
                                        
                                        if (derived18.Type != null)
                                        {
                                            datasourceValue3["type"] = derived18.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.DataSource is DocumentDbOutputDataSource)
                                    {
                                        datasourceValue3["type"] = "Microsoft.Storage/DocumentDB";
                                        DocumentDbOutputDataSource derived19 = ((DocumentDbOutputDataSource)outputsItem.Properties.DataSource);
                                        
                                        if (derived19.Properties != null)
                                        {
                                            JObject propertiesValue19 = new JObject();
                                            datasourceValue3["properties"] = propertiesValue19;
                                            
                                            if (derived19.Properties.AccountId != null)
                                            {
                                                propertiesValue19["accountId"] = derived19.Properties.AccountId;
                                            }
                                            
                                            if (derived19.Properties.AccountKey != null)
                                            {
                                                propertiesValue19["accountKey"] = derived19.Properties.AccountKey;
                                            }
                                            
                                            if (derived19.Properties.Database != null)
                                            {
                                                propertiesValue19["database"] = derived19.Properties.Database;
                                            }
                                            
                                            if (derived19.Properties.CollectionNamePattern != null)
                                            {
                                                propertiesValue19["collectionNamePattern"] = derived19.Properties.CollectionNamePattern;
                                            }
                                            
                                            if (derived19.Properties.PartitionKey != null)
                                            {
                                                propertiesValue19["partitionKey"] = derived19.Properties.PartitionKey;
                                            }
                                            
                                            if (derived19.Properties.DocumentId != null)
                                            {
                                                propertiesValue19["documentId"] = derived19.Properties.DocumentId;
                                            }
                                        }
                                        
                                        if (derived19.Type != null)
                                        {
                                            datasourceValue3["type"] = derived19.Type;
                                        }
                                    }
                                }
                                
                                if (outputsItem.Properties.Serialization != null)
                                {
                                    JObject serializationValue3 = new JObject();
                                    propertiesValue12["serialization"] = serializationValue3;
                                    if (outputsItem.Properties.Serialization is CsvSerialization)
                                    {
                                        serializationValue3["type"] = "Csv";
                                        CsvSerialization derived20 = ((CsvSerialization)outputsItem.Properties.Serialization);
                                        
                                        if (derived20.Properties != null)
                                        {
                                            JObject propertiesValue20 = new JObject();
                                            serializationValue3["properties"] = propertiesValue20;
                                            
                                            if (derived20.Properties.FieldDelimiter != null)
                                            {
                                                propertiesValue20["fieldDelimiter"] = derived20.Properties.FieldDelimiter;
                                            }
                                            
                                            if (derived20.Properties.Encoding != null)
                                            {
                                                propertiesValue20["encoding"] = derived20.Properties.Encoding;
                                            }
                                        }
                                        
                                        if (derived20.Type != null)
                                        {
                                            serializationValue3["type"] = derived20.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.Serialization is JsonSerialization)
                                    {
                                        serializationValue3["type"] = "Json";
                                        JsonSerialization derived21 = ((JsonSerialization)outputsItem.Properties.Serialization);
                                        
                                        if (derived21.Properties != null)
                                        {
                                            JObject propertiesValue21 = new JObject();
                                            serializationValue3["properties"] = propertiesValue21;
                                            
                                            if (derived21.Properties.Encoding != null)
                                            {
                                                propertiesValue21["encoding"] = derived21.Properties.Encoding;
                                            }
                                            
                                            if (derived21.Properties.Format != null)
                                            {
                                                propertiesValue21["format"] = derived21.Properties.Format;
                                            }
                                        }
                                        
                                        if (derived21.Type != null)
                                        {
                                            serializationValue3["type"] = derived21.Type;
                                        }
                                    }
                                    if (outputsItem.Properties.Serialization is AvroSerialization)
                                    {
                                        serializationValue3["type"] = "Avro";
                                        AvroSerialization derived22 = ((AvroSerialization)outputsItem.Properties.Serialization);
                                        
                                        if (derived22.Properties != null)
                                        {
                                            serializationValue3["properties"] = derived22.Properties.ToString();
                                        }
                                        
                                        if (derived22.Type != null)
                                        {
                                            serializationValue3["type"] = derived22.Type;
                                        }
                                    }
                                }
                                
                                if (outputsItem.Properties.Diagnostics != null)
                                {
                                    JObject diagnosticsValue3 = new JObject();
                                    propertiesValue12["diagnostics"] = diagnosticsValue3;
                                    
                                    if (outputsItem.Properties.Diagnostics.Conditions != null)
                                    {
                                        if (outputsItem.Properties.Diagnostics.Conditions is ILazyCollection == false || ((ILazyCollection)outputsItem.Properties.Diagnostics.Conditions).IsInitialized)
                                        {
                                            JArray conditionsArray3 = new JArray();
                                            foreach (DiagnosticCondition conditionsItem3 in outputsItem.Properties.Diagnostics.Conditions)
                                            {
                                                JObject diagnosticConditionValue3 = new JObject();
                                                conditionsArray3.Add(diagnosticConditionValue3);
                                                
                                                if (conditionsItem3.Since != null)
                                                {
                                                    diagnosticConditionValue3["since"] = conditionsItem3.Since.Value;
                                                }
                                                
                                                if (conditionsItem3.Code != null)
                                                {
                                                    diagnosticConditionValue3["code"] = conditionsItem3.Code;
                                                }
                                                
                                                if (conditionsItem3.Message != null)
                                                {
                                                    diagnosticConditionValue3["message"] = conditionsItem3.Message;
                                                }
                                            }
                                            diagnosticsValue3["conditions"] = conditionsArray3;
                                        }
                                    }
                                }
                            }
                        }
                        propertiesValue["outputs"] = outputsArray;
                    }
                    
                    if (parameters.JobPatchRequest.Properties.Functions != null)
                    {
                        JArray functionsArray = new JArray();
                        foreach (Function functionsItem in parameters.JobPatchRequest.Properties.Functions)
                        {
                            JObject functionValue = new JObject();
                            functionsArray.Add(functionValue);
                            
                            functionValue["name"] = functionsItem.Name;
                            
                            if (functionsItem.Properties != null)
                            {
                                JObject propertiesValue22 = new JObject();
                                functionValue["properties"] = propertiesValue22;
                                if (functionsItem.Properties is ScalarFunctionProperties)
                                {
                                    propertiesValue22["type"] = "Scalar";
                                    ScalarFunctionProperties derived23 = ((ScalarFunctionProperties)functionsItem.Properties);
                                    
                                    if (derived23.Properties != null)
                                    {
                                        JObject propertiesValue23 = new JObject();
                                        propertiesValue22["properties"] = propertiesValue23;
                                        
                                        if (derived23.Properties.Inputs != null)
                                        {
                                            JArray inputsArray2 = new JArray();
                                            foreach (FunctionInput inputsItem2 in derived23.Properties.Inputs)
                                            {
                                                JObject functionInputValue = new JObject();
                                                inputsArray2.Add(functionInputValue);
                                                
                                                if (inputsItem2.DataType != null)
                                                {
                                                    functionInputValue["dataType"] = inputsItem2.DataType;
                                                }
                                                
                                                if (inputsItem2.IsConfigurationParameter != null)
                                                {
                                                    functionInputValue["isConfigurationParameter"] = inputsItem2.IsConfigurationParameter.Value;
                                                }
                                            }
                                            propertiesValue23["inputs"] = inputsArray2;
                                        }
                                        
                                        if (derived23.Properties.Output != null)
                                        {
                                            JObject outputValue2 = new JObject();
                                            propertiesValue23["output"] = outputValue2;
                                            
                                            if (derived23.Properties.Output.DataType != null)
                                            {
                                                outputValue2["dataType"] = derived23.Properties.Output.DataType;
                                            }
                                        }
                                        
                                        if (derived23.Properties.Binding != null)
                                        {
                                            JObject bindingValue = new JObject();
                                            propertiesValue23["binding"] = bindingValue;
                                            if (derived23.Properties.Binding is AzureMachineLearningWebServiceFunctionBinding)
                                            {
                                                bindingValue["type"] = "Microsoft.MachineLearning/WebService";
                                                AzureMachineLearningWebServiceFunctionBinding derived24 = ((AzureMachineLearningWebServiceFunctionBinding)derived23.Properties.Binding);
                                                
                                                if (derived24.Properties != null)
                                                {
                                                    JObject propertiesValue24 = new JObject();
                                                    bindingValue["properties"] = propertiesValue24;
                                                    
                                                    if (derived24.Properties.Endpoint != null)
                                                    {
                                                        propertiesValue24["endpoint"] = derived24.Properties.Endpoint;
                                                    }
                                                    
                                                    if (derived24.Properties.ApiKey != null)
                                                    {
                                                        propertiesValue24["apiKey"] = derived24.Properties.ApiKey;
                                                    }
                                                    
                                                    if (derived24.Properties.Inputs != null)
                                                    {
                                                        JObject inputsValue = new JObject();
                                                        propertiesValue24["inputs"] = inputsValue;
                                                        
                                                        inputsValue["name"] = derived24.Properties.Inputs.Name;
                                                        
                                                        if (derived24.Properties.Inputs.ColumnNames != null)
                                                        {
                                                            JArray columnNamesArray = new JArray();
                                                            foreach (AzureMachineLearningWebServiceInputColumn columnNamesItem in derived24.Properties.Inputs.ColumnNames)
                                                            {
                                                                JObject azureMachineLearningWebServiceInputColumnValue = new JObject();
                                                                columnNamesArray.Add(azureMachineLearningWebServiceInputColumnValue);
                                                                
                                                                azureMachineLearningWebServiceInputColumnValue["name"] = columnNamesItem.Name;
                                                                
                                                                if (columnNamesItem.DataType != null)
                                                                {
                                                                    azureMachineLearningWebServiceInputColumnValue["dataType"] = columnNamesItem.DataType;
                                                                }
                                                                
                                                                if (columnNamesItem.MapTo != null)
                                                                {
                                                                    azureMachineLearningWebServiceInputColumnValue["mapTo"] = columnNamesItem.MapTo.Value;
                                                                }
                                                            }
                                                            inputsValue["columnNames"] = columnNamesArray;
                                                        }
                                                    }
                                                    
                                                    if (derived24.Properties.Outputs != null)
                                                    {
                                                        JArray outputsArray2 = new JArray();
                                                        foreach (AzureMachineLearningWebServiceOutputColumn outputsItem2 in derived24.Properties.Outputs)
                                                        {
                                                            JObject azureMachineLearningWebServiceOutputColumnValue = new JObject();
                                                            outputsArray2.Add(azureMachineLearningWebServiceOutputColumnValue);
                                                            
                                                            azureMachineLearningWebServiceOutputColumnValue["name"] = outputsItem2.Name;
                                                            
                                                            if (outputsItem2.DataType != null)
                                                            {
                                                                azureMachineLearningWebServiceOutputColumnValue["dataType"] = outputsItem2.DataType;
                                                            }
                                                        }
                                                        propertiesValue24["outputs"] = outputsArray2;
                                                    }
                                                    
                                                    if (derived24.Properties.BatchSize != null)
                                                    {
                                                        propertiesValue24["batchSize"] = derived24.Properties.BatchSize.Value;
                                                    }
                                                }
                                                
                                                if (derived24.Type != null)
                                                {
                                                    bindingValue["type"] = derived24.Type;
                                                }
                                            }
                                            if (derived23.Properties.Binding is JavaScriptFunctionBinding)
                                            {
                                                bindingValue["type"] = "Microsoft.StreamAnalytics/JavascriptUdf";
                                                JavaScriptFunctionBinding derived25 = ((JavaScriptFunctionBinding)derived23.Properties.Binding);
                                                
                                                if (derived25.Properties != null)
                                                {
                                                    JObject propertiesValue25 = new JObject();
                                                    bindingValue["properties"] = propertiesValue25;
                                                    
                                                    if (derived25.Properties.Script != null)
                                                    {
                                                        propertiesValue25["script"] = derived25.Properties.Script;
                                                    }
                                                }
                                                
                                                if (derived25.Type != null)
                                                {
                                                    bindingValue["type"] = derived25.Type;
                                                }
                                            }
                                        }
                                    }
                                    
                                    if (derived23.Etag != null)
                                    {
                                        propertiesValue22["etag"] = derived23.Etag;
                                    }
                                    
                                    if (derived23.Type != null)
                                    {
                                        propertiesValue22["type"] = derived23.Type;
                                    }
                                }
                            }
                        }
                        propertiesValue["functions"] = functionsArray;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    JobPatchResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new JobPatchResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            Job jobInstance = new Job();
                            result.Job = jobInstance;
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                jobInstance.Id = idInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                jobInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                jobInstance.Type = typeInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                jobInstance.Location = locationInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                jobInstance.Tags = new Dictionary<string, string>();
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey2 = ((string)property.Name);
                                    string tagsValue2 = ((string)property.Value);
                                    jobInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken propertiesValue26 = responseDoc["properties"];
                            if (propertiesValue26 != null && propertiesValue26.Type != JTokenType.Null)
                            {
                                JobProperties propertiesInstance = new JobProperties();
                                jobInstance.Properties = propertiesInstance;
                                
                                JToken skuValue2 = propertiesValue26["sku"];
                                if (skuValue2 != null && skuValue2.Type != JTokenType.Null)
                                {
                                    Sku skuInstance = new Sku();
                                    propertiesInstance.Sku = skuInstance;
                                    
                                    JToken nameValue2 = skuValue2["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        string nameInstance2 = ((string)nameValue2);
                                        skuInstance.Name = nameInstance2;
                                    }
                                }
                                
                                JToken eventsOutOfOrderPolicyValue = propertiesValue26["eventsOutOfOrderPolicy"];
                                if (eventsOutOfOrderPolicyValue != null && eventsOutOfOrderPolicyValue.Type != JTokenType.Null)
                                {
                                    string eventsOutOfOrderPolicyInstance = ((string)eventsOutOfOrderPolicyValue);
                                    propertiesInstance.EventsOutOfOrderPolicy = eventsOutOfOrderPolicyInstance;
                                }
                                
                                JToken outputErrorPolicyValue = propertiesValue26["outputErrorPolicy"];
                                if (outputErrorPolicyValue != null && outputErrorPolicyValue.Type != JTokenType.Null)
                                {
                                    string outputErrorPolicyInstance = ((string)outputErrorPolicyValue);
                                    propertiesInstance.OutputErrorPolicy = outputErrorPolicyInstance;
                                }
                                
                                JToken eventsOutOfOrderMaxDelayInSecondsValue = propertiesValue26["eventsOutOfOrderMaxDelayInSeconds"];
                                if (eventsOutOfOrderMaxDelayInSecondsValue != null && eventsOutOfOrderMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsOutOfOrderMaxDelayInSecondsInstance = ((int)eventsOutOfOrderMaxDelayInSecondsValue);
                                    propertiesInstance.EventsOutOfOrderMaxDelayInSeconds = eventsOutOfOrderMaxDelayInSecondsInstance;
                                }
                                
                                JToken eventsLateArrivalMaxDelayInSecondsValue = propertiesValue26["eventsLateArrivalMaxDelayInSeconds"];
                                if (eventsLateArrivalMaxDelayInSecondsValue != null && eventsLateArrivalMaxDelayInSecondsValue.Type != JTokenType.Null)
                                {
                                    int eventsLateArrivalMaxDelayInSecondsInstance = ((int)eventsLateArrivalMaxDelayInSecondsValue);
                                    propertiesInstance.EventsLateArrivalMaxDelayInSeconds = eventsLateArrivalMaxDelayInSecondsInstance;
                                }
                                
                                JToken dataLocaleValue = propertiesValue26["dataLocale"];
                                if (dataLocaleValue != null && dataLocaleValue.Type != JTokenType.Null)
                                {
                                    string dataLocaleInstance = ((string)dataLocaleValue);
                                    propertiesInstance.DataLocale = dataLocaleInstance;
                                }
                                
                                JToken outputStartModeValue = propertiesValue26["outputStartMode"];
                                if (outputStartModeValue != null && outputStartModeValue.Type != JTokenType.Null)
                                {
                                    string outputStartModeInstance = ((string)outputStartModeValue);
                                    propertiesInstance.OutputStartMode = outputStartModeInstance;
                                }
                                
                                JToken outputStartTimeValue = propertiesValue26["outputStartTime"];
                                if (outputStartTimeValue != null && outputStartTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime outputStartTimeInstance = ((DateTime)outputStartTimeValue);
                                    propertiesInstance.OutputStartTime = outputStartTimeInstance;
                                }
                                
                                JToken lastOutputEventTimeValue = propertiesValue26["lastOutputEventTime"];
                                if (lastOutputEventTimeValue != null && lastOutputEventTimeValue.Type != JTokenType.Null)
                                {
                                    DateTime lastOutputEventTimeInstance = ((DateTime)lastOutputEventTimeValue);
                                    propertiesInstance.LastOutputEventTime = lastOutputEventTimeInstance;
                                }
                                
                                JToken jobIdValue = propertiesValue26["jobId"];
                                if (jobIdValue != null && jobIdValue.Type != JTokenType.Null)
                                {
                                    string jobIdInstance = ((string)jobIdValue);
                                    propertiesInstance.JobId = jobIdInstance;
                                }
                                
                                JToken etagValue = propertiesValue26["etag"];
                                if (etagValue != null && etagValue.Type != JTokenType.Null)
                                {
                                    string etagInstance = ((string)etagValue);
                                    propertiesInstance.Etag = etagInstance;
                                }
                                
                                JToken provisioningStateValue = propertiesValue26["provisioningState"];
                                if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance = ((string)provisioningStateValue);
                                    propertiesInstance.ProvisioningState = provisioningStateInstance;
                                }
                                
                                JToken jobStateValue = propertiesValue26["jobState"];
                                if (jobStateValue != null && jobStateValue.Type != JTokenType.Null)
                                {
                                    string jobStateInstance = ((string)jobStateValue);
                                    propertiesInstance.JobState = jobStateInstance;
                                }
                                
                                JToken createdDateValue = propertiesValue26["createdDate"];
                                if (createdDateValue != null && createdDateValue.Type != JTokenType.Null)
                                {
                                    DateTime createdDateInstance = ((DateTime)createdDateValue);
                                    propertiesInstance.CreatedDate = createdDateInstance;
                                }
                                
                                JToken inputsArray3 = propertiesValue26["inputs"];
                                if (inputsArray3 != null && inputsArray3.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Inputs = new List<Input>();
                                    foreach (JToken inputsValue2 in ((JArray)inputsArray3))
                                    {
                                        Input inputInstance = new Input();
                                        propertiesInstance.Inputs.Add(inputInstance);
                                        
                                        JToken nameValue3 = inputsValue2["name"];
                                        if (nameValue3 != null && nameValue3.Type != JTokenType.Null)
                                        {
                                            string nameInstance3 = ((string)nameValue3);
                                            inputInstance.Name = nameInstance3;
                                        }
                                        
                                        JToken propertiesValue27 = inputsValue2["properties"];
                                        if (propertiesValue27 != null && propertiesValue27.Type != JTokenType.Null)
                                        {
                                            string typeName = ((string)propertiesValue27["type"]);
                                            if (typeName == "Reference")
                                            {
                                                ReferenceInputProperties referenceInputPropertiesInstance = new ReferenceInputProperties();
                                                
                                                JToken datasourceValue4 = propertiesValue27["datasource"];
                                                if (datasourceValue4 != null && datasourceValue4.Type != JTokenType.Null)
                                                {
                                                    string typeName2 = ((string)datasourceValue4["type"]);
                                                    if (typeName2 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobReferenceInputDataSource blobReferenceInputDataSourceInstance = new BlobReferenceInputDataSource();
                                                        
                                                        JToken propertiesValue28 = datasourceValue4["properties"];
                                                        if (propertiesValue28 != null && propertiesValue28.Type != JTokenType.Null)
                                                        {
                                                            BlobReferenceInputDataSourceProperties propertiesInstance2 = new BlobReferenceInputDataSourceProperties();
                                                            blobReferenceInputDataSourceInstance.Properties = propertiesInstance2;
                                                            
                                                            JToken storageAccountsArray4 = propertiesValue28["storageAccounts"];
                                                            if (storageAccountsArray4 != null && storageAccountsArray4.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance2.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue in ((JArray)storageAccountsArray4))
                                                                {
                                                                    StorageAccount storageAccountInstance = new StorageAccount();
                                                                    propertiesInstance2.StorageAccounts.Add(storageAccountInstance);
                                                                    
                                                                    JToken accountNameValue = storageAccountsValue["accountName"];
                                                                    if (accountNameValue != null && accountNameValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance = ((string)accountNameValue);
                                                                        storageAccountInstance.AccountName = accountNameInstance;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue = storageAccountsValue["accountKey"];
                                                                    if (accountKeyValue != null && accountKeyValue.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance = ((string)accountKeyValue);
                                                                        storageAccountInstance.AccountKey = accountKeyInstance;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue = propertiesValue28["container"];
                                                            if (containerValue != null && containerValue.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance = ((string)containerValue);
                                                                propertiesInstance2.Container = containerInstance;
                                                            }
                                                            
                                                            JToken pathPatternValue = propertiesValue28["pathPattern"];
                                                            if (pathPatternValue != null && pathPatternValue.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance = ((string)pathPatternValue);
                                                                propertiesInstance2.PathPattern = pathPatternInstance;
                                                            }
                                                            
                                                            JToken dateFormatValue = propertiesValue28["dateFormat"];
                                                            if (dateFormatValue != null && dateFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance = ((string)dateFormatValue);
                                                                propertiesInstance2.DateFormat = dateFormatInstance;
                                                            }
                                                            
                                                            JToken timeFormatValue = propertiesValue28["timeFormat"];
                                                            if (timeFormatValue != null && timeFormatValue.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance = ((string)timeFormatValue);
                                                                propertiesInstance2.TimeFormat = timeFormatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue2 = datasourceValue4["type"];
                                                        if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance2 = ((string)typeValue2);
                                                            blobReferenceInputDataSourceInstance.Type = typeInstance2;
                                                        }
                                                        referenceInputPropertiesInstance.DataSource = blobReferenceInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue2 = propertiesValue27["etag"];
                                                if (etagValue2 != null && etagValue2.Type != JTokenType.Null)
                                                {
                                                    string etagInstance2 = ((string)etagValue2);
                                                    referenceInputPropertiesInstance.Etag = etagInstance2;
                                                }
                                                
                                                JToken typeValue3 = propertiesValue27["type"];
                                                if (typeValue3 != null && typeValue3.Type != JTokenType.Null)
                                                {
                                                    string typeInstance3 = ((string)typeValue3);
                                                    referenceInputPropertiesInstance.Type = typeInstance3;
                                                }
                                                
                                                JToken serializationValue4 = propertiesValue27["serialization"];
                                                if (serializationValue4 != null && serializationValue4.Type != JTokenType.Null)
                                                {
                                                    string typeName3 = ((string)serializationValue4["type"]);
                                                    if (typeName3 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance = new CsvSerialization();
                                                        
                                                        JToken propertiesValue29 = serializationValue4["properties"];
                                                        if (propertiesValue29 != null && propertiesValue29.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance3 = new CsvSerializationProperties();
                                                            csvSerializationInstance.Properties = propertiesInstance3;
                                                            
                                                            JToken fieldDelimiterValue = propertiesValue29["fieldDelimiter"];
                                                            if (fieldDelimiterValue != null && fieldDelimiterValue.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance = ((string)fieldDelimiterValue);
                                                                propertiesInstance3.FieldDelimiter = fieldDelimiterInstance;
                                                            }
                                                            
                                                            JToken encodingValue = propertiesValue29["encoding"];
                                                            if (encodingValue != null && encodingValue.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance = ((string)encodingValue);
                                                                propertiesInstance3.Encoding = encodingInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue4 = serializationValue4["type"];
                                                        if (typeValue4 != null && typeValue4.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance4 = ((string)typeValue4);
                                                            csvSerializationInstance.Type = typeInstance4;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = csvSerializationInstance;
                                                    }
                                                    if (typeName3 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance = new JsonSerialization();
                                                        
                                                        JToken propertiesValue30 = serializationValue4["properties"];
                                                        if (propertiesValue30 != null && propertiesValue30.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance4 = new JsonSerializationProperties();
                                                            jsonSerializationInstance.Properties = propertiesInstance4;
                                                            
                                                            JToken encodingValue2 = propertiesValue30["encoding"];
                                                            if (encodingValue2 != null && encodingValue2.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance2 = ((string)encodingValue2);
                                                                propertiesInstance4.Encoding = encodingInstance2;
                                                            }
                                                            
                                                            JToken formatValue = propertiesValue30["format"];
                                                            if (formatValue != null && formatValue.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance = ((string)formatValue);
                                                                propertiesInstance4.Format = formatInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue5 = serializationValue4["type"];
                                                        if (typeValue5 != null && typeValue5.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance5 = ((string)typeValue5);
                                                            jsonSerializationInstance.Type = typeInstance5;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = jsonSerializationInstance;
                                                    }
                                                    if (typeName3 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance = new AvroSerialization();
                                                        
                                                        JToken propertiesValue31 = serializationValue4["properties"];
                                                        if (propertiesValue31 != null && propertiesValue31.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance5 = new AvroSerializationProperties();
                                                            avroSerializationInstance.Properties = propertiesInstance5;
                                                        }
                                                        
                                                        JToken typeValue6 = serializationValue4["type"];
                                                        if (typeValue6 != null && typeValue6.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance6 = ((string)typeValue6);
                                                            avroSerializationInstance.Type = typeInstance6;
                                                        }
                                                        referenceInputPropertiesInstance.Serialization = avroSerializationInstance;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue4 = propertiesValue27["diagnostics"];
                                                if (diagnosticsValue4 != null && diagnosticsValue4.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance = new Diagnostics();
                                                    referenceInputPropertiesInstance.Diagnostics = diagnosticsInstance;
                                                    
                                                    JToken conditionsArray4 = diagnosticsValue4["conditions"];
                                                    if (conditionsArray4 != null && conditionsArray4.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue in ((JArray)conditionsArray4))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance = new DiagnosticCondition();
                                                            diagnosticsInstance.Conditions.Add(diagnosticConditionInstance);
                                                            
                                                            JToken sinceValue = conditionsValue["since"];
                                                            if (sinceValue != null && sinceValue.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance = ((DateTime)sinceValue);
                                                                diagnosticConditionInstance.Since = sinceInstance;
                                                            }
                                                            
                                                            JToken codeValue = conditionsValue["code"];
                                                            if (codeValue != null && codeValue.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance = ((string)codeValue);
                                                                diagnosticConditionInstance.Code = codeInstance;
                                                            }
                                                            
                                                            JToken messageValue = conditionsValue["message"];
                                                            if (messageValue != null && messageValue.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance = ((string)messageValue);
                                                                diagnosticConditionInstance.Message = messageInstance;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = referenceInputPropertiesInstance;
                                            }
                                            if (typeName == "Stream")
                                            {
                                                StreamInputProperties streamInputPropertiesInstance = new StreamInputProperties();
                                                
                                                JToken datasourceValue5 = propertiesValue27["datasource"];
                                                if (datasourceValue5 != null && datasourceValue5.Type != JTokenType.Null)
                                                {
                                                    string typeName4 = ((string)datasourceValue5["type"]);
                                                    if (typeName4 == "Microsoft.Storage/Blob")
                                                    {
                                                        BlobStreamInputDataSource blobStreamInputDataSourceInstance = new BlobStreamInputDataSource();
                                                        
                                                        JToken propertiesValue32 = datasourceValue5["properties"];
                                                        if (propertiesValue32 != null && propertiesValue32.Type != JTokenType.Null)
                                                        {
                                                            BlobStreamInputDataSourceProperties propertiesInstance6 = new BlobStreamInputDataSourceProperties();
                                                            blobStreamInputDataSourceInstance.Properties = propertiesInstance6;
                                                            
                                                            JToken storageAccountsArray5 = propertiesValue32["storageAccounts"];
                                                            if (storageAccountsArray5 != null && storageAccountsArray5.Type != JTokenType.Null)
                                                            {
                                                                propertiesInstance6.StorageAccounts = new List<StorageAccount>();
                                                                foreach (JToken storageAccountsValue2 in ((JArray)storageAccountsArray5))
                                                                {
                                                                    StorageAccount storageAccountInstance2 = new StorageAccount();
                                                                    propertiesInstance6.StorageAccounts.Add(storageAccountInstance2);
                                                                    
                                                                    JToken accountNameValue2 = storageAccountsValue2["accountName"];
                                                                    if (accountNameValue2 != null && accountNameValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountNameInstance2 = ((string)accountNameValue2);
                                                                        storageAccountInstance2.AccountName = accountNameInstance2;
                                                                    }
                                                                    
                                                                    JToken accountKeyValue2 = storageAccountsValue2["accountKey"];
                                                                    if (accountKeyValue2 != null && accountKeyValue2.Type != JTokenType.Null)
                                                                    {
                                                                        string accountKeyInstance2 = ((string)accountKeyValue2);
                                                                        storageAccountInstance2.AccountKey = accountKeyInstance2;
                                                                    }
                                                                }
                                                            }
                                                            
                                                            JToken containerValue2 = propertiesValue32["container"];
                                                            if (containerValue2 != null && containerValue2.Type != JTokenType.Null)
                                                            {
                                                                string containerInstance2 = ((string)containerValue2);
                                                                propertiesInstance6.Container = containerInstance2;
                                                            }
                                                            
                                                            JToken pathPatternValue2 = propertiesValue32["pathPattern"];
                                                            if (pathPatternValue2 != null && pathPatternValue2.Type != JTokenType.Null)
                                                            {
                                                                string pathPatternInstance2 = ((string)pathPatternValue2);
                                                                propertiesInstance6.PathPattern = pathPatternInstance2;
                                                            }
                                                            
                                                            JToken dateFormatValue2 = propertiesValue32["dateFormat"];
                                                            if (dateFormatValue2 != null && dateFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string dateFormatInstance2 = ((string)dateFormatValue2);
                                                                propertiesInstance6.DateFormat = dateFormatInstance2;
                                                            }
                                                            
                                                            JToken timeFormatValue2 = propertiesValue32["timeFormat"];
                                                            if (timeFormatValue2 != null && timeFormatValue2.Type != JTokenType.Null)
                                                            {
                                                                string timeFormatInstance2 = ((string)timeFormatValue2);
                                                                propertiesInstance6.TimeFormat = timeFormatInstance2;
                                                            }
                                                            
                                                            JToken sourcePartitionCountValue = propertiesValue32["sourcePartitionCount"];
                                                            if (sourcePartitionCountValue != null && sourcePartitionCountValue.Type != JTokenType.Null)
                                                            {
                                                                int sourcePartitionCountInstance = ((int)sourcePartitionCountValue);
                                                                propertiesInstance6.SourcePartitionCount = sourcePartitionCountInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue7 = datasourceValue5["type"];
                                                        if (typeValue7 != null && typeValue7.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance7 = ((string)typeValue7);
                                                            blobStreamInputDataSourceInstance.Type = typeInstance7;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = blobStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.ServiceBus/EventHub")
                                                    {
                                                        EventHubStreamInputDataSource eventHubStreamInputDataSourceInstance = new EventHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue33 = datasourceValue5["properties"];
                                                        if (propertiesValue33 != null && propertiesValue33.Type != JTokenType.Null)
                                                        {
                                                            EventHubStreamInputDataSourceProperties propertiesInstance7 = new EventHubStreamInputDataSourceProperties();
                                                            eventHubStreamInputDataSourceInstance.Properties = propertiesInstance7;
                                                            
                                                            JToken serviceBusNamespaceValue = propertiesValue33["serviceBusNamespace"];
                                                            if (serviceBusNamespaceValue != null && serviceBusNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string serviceBusNamespaceInstance = ((string)serviceBusNamespaceValue);
                                                                propertiesInstance7.ServiceBusNamespace = serviceBusNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue = propertiesValue33["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue != null && sharedAccessPolicyNameValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance = ((string)sharedAccessPolicyNameValue);
                                                                propertiesInstance7.SharedAccessPolicyName = sharedAccessPolicyNameInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue = propertiesValue33["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue != null && sharedAccessPolicyKeyValue.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance = ((string)sharedAccessPolicyKeyValue);
                                                                propertiesInstance7.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance;
                                                            }
                                                            
                                                            JToken eventHubNameValue = propertiesValue33["eventHubName"];
                                                            if (eventHubNameValue != null && eventHubNameValue.Type != JTokenType.Null)
                                                            {
                                                                string eventHubNameInstance = ((string)eventHubNameValue);
                                                                propertiesInstance7.EventHubName = eventHubNameInstance;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue = propertiesValue33["consumerGroupName"];
                                                            if (consumerGroupNameValue != null && consumerGroupNameValue.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance = ((string)consumerGroupNameValue);
                                                                propertiesInstance7.ConsumerGroupName = consumerGroupNameInstance;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue8 = datasourceValue5["type"];
                                                        if (typeValue8 != null && typeValue8.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance8 = ((string)typeValue8);
                                                            eventHubStreamInputDataSourceInstance.Type = typeInstance8;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = eventHubStreamInputDataSourceInstance;
                                                    }
                                                    if (typeName4 == "Microsoft.Devices/IotHubs")
                                                    {
                                                        IoTHubStreamInputDataSource ioTHubStreamInputDataSourceInstance = new IoTHubStreamInputDataSource();
                                                        
                                                        JToken propertiesValue34 = datasourceValue5["properties"];
                                                        if (propertiesValue34 != null && propertiesValue34.Type != JTokenType.Null)
                                                        {
                                                            IoTHubStreamInputDataSourceProperties propertiesInstance8 = new IoTHubStreamInputDataSourceProperties();
                                                            ioTHubStreamInputDataSourceInstance.Properties = propertiesInstance8;
                                                            
                                                            JToken iotHubNamespaceValue = propertiesValue34["iotHubNamespace"];
                                                            if (iotHubNamespaceValue != null && iotHubNamespaceValue.Type != JTokenType.Null)
                                                            {
                                                                string iotHubNamespaceInstance = ((string)iotHubNamespaceValue);
                                                                propertiesInstance8.IotHubNamespace = iotHubNamespaceInstance;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyNameValue2 = propertiesValue34["sharedAccessPolicyName"];
                                                            if (sharedAccessPolicyNameValue2 != null && sharedAccessPolicyNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyNameInstance2 = ((string)sharedAccessPolicyNameValue2);
                                                                propertiesInstance8.SharedAccessPolicyName = sharedAccessPolicyNameInstance2;
                                                            }
                                                            
                                                            JToken sharedAccessPolicyKeyValue2 = propertiesValue34["sharedAccessPolicyKey"];
                                                            if (sharedAccessPolicyKeyValue2 != null && sharedAccessPolicyKeyValue2.Type != JTokenType.Null)
                                                            {
                                                                string sharedAccessPolicyKeyInstance2 = ((string)sharedAccessPolicyKeyValue2);
                                                                propertiesInstance8.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance2;
                                                            }
                                                            
                                                            JToken consumerGroupNameValue2 = propertiesValue34["consumerGroupName"];
                                                            if (consumerGroupNameValue2 != null && consumerGroupNameValue2.Type != JTokenType.Null)
                                                            {
                                                                string consumerGroupNameInstance2 = ((string)consumerGroupNameValue2);
                                                                propertiesInstance8.ConsumerGroupName = consumerGroupNameInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue9 = datasourceValue5["type"];
                                                        if (typeValue9 != null && typeValue9.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance9 = ((string)typeValue9);
                                                            ioTHubStreamInputDataSourceInstance.Type = typeInstance9;
                                                        }
                                                        streamInputPropertiesInstance.DataSource = ioTHubStreamInputDataSourceInstance;
                                                    }
                                                }
                                                
                                                JToken etagValue3 = propertiesValue27["etag"];
                                                if (etagValue3 != null && etagValue3.Type != JTokenType.Null)
                                                {
                                                    string etagInstance3 = ((string)etagValue3);
                                                    streamInputPropertiesInstance.Etag = etagInstance3;
                                                }
                                                
                                                JToken typeValue10 = propertiesValue27["type"];
                                                if (typeValue10 != null && typeValue10.Type != JTokenType.Null)
                                                {
                                                    string typeInstance10 = ((string)typeValue10);
                                                    streamInputPropertiesInstance.Type = typeInstance10;
                                                }
                                                
                                                JToken serializationValue5 = propertiesValue27["serialization"];
                                                if (serializationValue5 != null && serializationValue5.Type != JTokenType.Null)
                                                {
                                                    string typeName5 = ((string)serializationValue5["type"]);
                                                    if (typeName5 == "Csv")
                                                    {
                                                        CsvSerialization csvSerializationInstance2 = new CsvSerialization();
                                                        
                                                        JToken propertiesValue35 = serializationValue5["properties"];
                                                        if (propertiesValue35 != null && propertiesValue35.Type != JTokenType.Null)
                                                        {
                                                            CsvSerializationProperties propertiesInstance9 = new CsvSerializationProperties();
                                                            csvSerializationInstance2.Properties = propertiesInstance9;
                                                            
                                                            JToken fieldDelimiterValue2 = propertiesValue35["fieldDelimiter"];
                                                            if (fieldDelimiterValue2 != null && fieldDelimiterValue2.Type != JTokenType.Null)
                                                            {
                                                                string fieldDelimiterInstance2 = ((string)fieldDelimiterValue2);
                                                                propertiesInstance9.FieldDelimiter = fieldDelimiterInstance2;
                                                            }
                                                            
                                                            JToken encodingValue3 = propertiesValue35["encoding"];
                                                            if (encodingValue3 != null && encodingValue3.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance3 = ((string)encodingValue3);
                                                                propertiesInstance9.Encoding = encodingInstance3;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue11 = serializationValue5["type"];
                                                        if (typeValue11 != null && typeValue11.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance11 = ((string)typeValue11);
                                                            csvSerializationInstance2.Type = typeInstance11;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = csvSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Json")
                                                    {
                                                        JsonSerialization jsonSerializationInstance2 = new JsonSerialization();
                                                        
                                                        JToken propertiesValue36 = serializationValue5["properties"];
                                                        if (propertiesValue36 != null && propertiesValue36.Type != JTokenType.Null)
                                                        {
                                                            JsonSerializationProperties propertiesInstance10 = new JsonSerializationProperties();
                                                            jsonSerializationInstance2.Properties = propertiesInstance10;
                                                            
                                                            JToken encodingValue4 = propertiesValue36["encoding"];
                                                            if (encodingValue4 != null && encodingValue4.Type != JTokenType.Null)
                                                            {
                                                                string encodingInstance4 = ((string)encodingValue4);
                                                                propertiesInstance10.Encoding = encodingInstance4;
                                                            }
                                                            
                                                            JToken formatValue2 = propertiesValue36["format"];
                                                            if (formatValue2 != null && formatValue2.Type != JTokenType.Null)
                                                            {
                                                                string formatInstance2 = ((string)formatValue2);
                                                                propertiesInstance10.Format = formatInstance2;
                                                            }
                                                        }
                                                        
                                                        JToken typeValue12 = serializationValue5["type"];
                                                        if (typeValue12 != null && typeValue12.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance12 = ((string)typeValue12);
                                                            jsonSerializationInstance2.Type = typeInstance12;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = jsonSerializationInstance2;
                                                    }
                                                    if (typeName5 == "Avro")
                                                    {
                                                        AvroSerialization avroSerializationInstance2 = new AvroSerialization();
                                                        
                                                        JToken propertiesValue37 = serializationValue5["properties"];
                                                        if (propertiesValue37 != null && propertiesValue37.Type != JTokenType.Null)
                                                        {
                                                            AvroSerializationProperties propertiesInstance11 = new AvroSerializationProperties();
                                                            avroSerializationInstance2.Properties = propertiesInstance11;
                                                        }
                                                        
                                                        JToken typeValue13 = serializationValue5["type"];
                                                        if (typeValue13 != null && typeValue13.Type != JTokenType.Null)
                                                        {
                                                            string typeInstance13 = ((string)typeValue13);
                                                            avroSerializationInstance2.Type = typeInstance13;
                                                        }
                                                        streamInputPropertiesInstance.Serialization = avroSerializationInstance2;
                                                    }
                                                }
                                                
                                                JToken diagnosticsValue5 = propertiesValue27["diagnostics"];
                                                if (diagnosticsValue5 != null && diagnosticsValue5.Type != JTokenType.Null)
                                                {
                                                    Diagnostics diagnosticsInstance2 = new Diagnostics();
                                                    streamInputPropertiesInstance.Diagnostics = diagnosticsInstance2;
                                                    
                                                    JToken conditionsArray5 = diagnosticsValue5["conditions"];
                                                    if (conditionsArray5 != null && conditionsArray5.Type != JTokenType.Null)
                                                    {
                                                        foreach (JToken conditionsValue2 in ((JArray)conditionsArray5))
                                                        {
                                                            DiagnosticCondition diagnosticConditionInstance2 = new DiagnosticCondition();
                                                            diagnosticsInstance2.Conditions.Add(diagnosticConditionInstance2);
                                                            
                                                            JToken sinceValue2 = conditionsValue2["since"];
                                                            if (sinceValue2 != null && sinceValue2.Type != JTokenType.Null)
                                                            {
                                                                DateTime sinceInstance2 = ((DateTime)sinceValue2);
                                                                diagnosticConditionInstance2.Since = sinceInstance2;
                                                            }
                                                            
                                                            JToken codeValue2 = conditionsValue2["code"];
                                                            if (codeValue2 != null && codeValue2.Type != JTokenType.Null)
                                                            {
                                                                string codeInstance2 = ((string)codeValue2);
                                                                diagnosticConditionInstance2.Code = codeInstance2;
                                                            }
                                                            
                                                            JToken messageValue2 = conditionsValue2["message"];
                                                            if (messageValue2 != null && messageValue2.Type != JTokenType.Null)
                                                            {
                                                                string messageInstance2 = ((string)messageValue2);
                                                                diagnosticConditionInstance2.Message = messageInstance2;
                                                            }
                                                        }
                                                    }
                                                }
                                                inputInstance.Properties = streamInputPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                                
                                JToken transformationValue2 = propertiesValue26["transformation"];
                                if (transformationValue2 != null && transformationValue2.Type != JTokenType.Null)
                                {
                                    Transformation transformationInstance = new Transformation();
                                    propertiesInstance.Transformation = transformationInstance;
                                    
                                    JToken nameValue4 = transformationValue2["name"];
                                    if (nameValue4 != null && nameValue4.Type != JTokenType.Null)
                                    {
                                        string nameInstance4 = ((string)nameValue4);
                                        transformationInstance.Name = nameInstance4;
                                    }
                                    
                                    JToken propertiesValue38 = transformationValue2["properties"];
                                    if (propertiesValue38 != null && propertiesValue38.Type != JTokenType.Null)
                                    {
                                        TransformationProperties propertiesInstance12 = new TransformationProperties();
                                        transformationInstance.Properties = propertiesInstance12;
                                        
                                        JToken etagValue4 = propertiesValue38["etag"];
                                        if (etagValue4 != null && etagValue4.Type != JTokenType.Null)
                                        {
                                            string etagInstance4 = ((string)etagValue4);
                                            propertiesInstance12.Etag = etagInstance4;
                                        }
                                        
                                        JToken streamingUnitsValue = propertiesValue38["streamingUnits"];
                                        if (streamingUnitsValue != null && streamingUnitsValue.Type != JTokenType.Null)
                                        {
                                            int streamingUnitsInstance = ((int)streamingUnitsValue);
                                            propertiesInstance12.StreamingUnits = streamingUnitsInstance;
                                        }
                                        
                                        JToken queryValue = propertiesValue38["query"];
                                        if (queryValue != null && queryValue.Type != JTokenType.Null)
                                        {
                                            string queryInstance = ((string)queryValue);
                                            propertiesInstance12.Query = queryInstance;
                                        }
                                    }
                                }
                                
                                JToken outputsArray3 = propertiesValue26["outputs"];
                                if (outputsArray3 != null && outputsArray3.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Outputs = new List<Output>();
                                    foreach (JToken outputsValue in ((JArray)outputsArray3))
                                    {
                                        Output outputInstance = new Output();
                                        propertiesInstance.Outputs.Add(outputInstance);
                                        
                                        JToken nameValue5 = outputsValue["name"];
                                        if (nameValue5 != null && nameValue5.Type != JTokenType.Null)
                                        {
                                            string nameInstance5 = ((string)nameValue5);
                                            outputInstance.Name = nameInstance5;
                                        }
                                        
                                        JToken propertiesValue39 = outputsValue["properties"];
                                        if (propertiesValue39 != null && propertiesValue39.Type != JTokenType.Null)
                                        {
                                            OutputProperties propertiesInstance13 = new OutputProperties();
                                            outputInstance.Properties = propertiesInstance13;
                                            
                                            JToken etagValue5 = propertiesValue39["etag"];
                                            if (etagValue5 != null && etagValue5.Type != JTokenType.Null)
                                            {
                                                string etagInstance5 = ((string)etagValue5);
                                                propertiesInstance13.Etag = etagInstance5;
                                            }
                                            
                                            JToken datasourceValue6 = propertiesValue39["datasource"];
                                            if (datasourceValue6 != null && datasourceValue6.Type != JTokenType.Null)
                                            {
                                                string typeName6 = ((string)datasourceValue6["type"]);
                                                if (typeName6 == "Microsoft.Storage/Blob")
                                                {
                                                    BlobOutputDataSource blobOutputDataSourceInstance = new BlobOutputDataSource();
                                                    
                                                    JToken propertiesValue40 = datasourceValue6["properties"];
                                                    if (propertiesValue40 != null && propertiesValue40.Type != JTokenType.Null)
                                                    {
                                                        BlobOutputDataSourceProperties propertiesInstance14 = new BlobOutputDataSourceProperties();
                                                        blobOutputDataSourceInstance.Properties = propertiesInstance14;
                                                        
                                                        JToken storageAccountsArray6 = propertiesValue40["storageAccounts"];
                                                        if (storageAccountsArray6 != null && storageAccountsArray6.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance14.StorageAccounts = new System.Collections.Generic.List<Microsoft.Azure.Management.StreamAnalytics.Models.StorageAccount>();
                                                            foreach (JToken storageAccountsValue3 in ((JArray)storageAccountsArray6))
                                                            {
                                                                StorageAccount storageAccountInstance3 = new StorageAccount();
                                                                propertiesInstance14.StorageAccounts.Add(storageAccountInstance3);
                                                                
                                                                JToken accountNameValue3 = storageAccountsValue3["accountName"];
                                                                if (accountNameValue3 != null && accountNameValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountNameInstance3 = ((string)accountNameValue3);
                                                                    storageAccountInstance3.AccountName = accountNameInstance3;
                                                                }
                                                                
                                                                JToken accountKeyValue3 = storageAccountsValue3["accountKey"];
                                                                if (accountKeyValue3 != null && accountKeyValue3.Type != JTokenType.Null)
                                                                {
                                                                    string accountKeyInstance3 = ((string)accountKeyValue3);
                                                                    storageAccountInstance3.AccountKey = accountKeyInstance3;
                                                                }
                                                            }
                                                        }
                                                        
                                                        JToken containerValue3 = propertiesValue40["container"];
                                                        if (containerValue3 != null && containerValue3.Type != JTokenType.Null)
                                                        {
                                                            string containerInstance3 = ((string)containerValue3);
                                                            propertiesInstance14.Container = containerInstance3;
                                                        }
                                                        
                                                        JToken pathPatternValue3 = propertiesValue40["pathPattern"];
                                                        if (pathPatternValue3 != null && pathPatternValue3.Type != JTokenType.Null)
                                                        {
                                                            string pathPatternInstance3 = ((string)pathPatternValue3);
                                                            propertiesInstance14.PathPattern = pathPatternInstance3;
                                                        }
                                                        
                                                        JToken dateFormatValue3 = propertiesValue40["dateFormat"];
                                                        if (dateFormatValue3 != null && dateFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string dateFormatInstance3 = ((string)dateFormatValue3);
                                                            propertiesInstance14.DateFormat = dateFormatInstance3;
                                                        }
                                                        
                                                        JToken timeFormatValue3 = propertiesValue40["timeFormat"];
                                                        if (timeFormatValue3 != null && timeFormatValue3.Type != JTokenType.Null)
                                                        {
                                                            string timeFormatInstance3 = ((string)timeFormatValue3);
                                                            propertiesInstance14.TimeFormat = timeFormatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue14 = datasourceValue6["type"];
                                                    if (typeValue14 != null && typeValue14.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance14 = ((string)typeValue14);
                                                        blobOutputDataSourceInstance.Type = typeInstance14;
                                                    }
                                                    propertiesInstance13.DataSource = blobOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/Table")
                                                {
                                                    AzureTableOutputDataSource azureTableOutputDataSourceInstance = new AzureTableOutputDataSource();
                                                    
                                                    JToken propertiesValue41 = datasourceValue6["properties"];
                                                    if (propertiesValue41 != null && propertiesValue41.Type != JTokenType.Null)
                                                    {
                                                        AzureTableOutputDataSourceProperties propertiesInstance15 = new AzureTableOutputDataSourceProperties();
                                                        azureTableOutputDataSourceInstance.Properties = propertiesInstance15;
                                                        
                                                        JToken accountNameValue4 = propertiesValue41["accountName"];
                                                        if (accountNameValue4 != null && accountNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountNameInstance4 = ((string)accountNameValue4);
                                                            propertiesInstance15.AccountName = accountNameInstance4;
                                                        }
                                                        
                                                        JToken accountKeyValue4 = propertiesValue41["accountKey"];
                                                        if (accountKeyValue4 != null && accountKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance4 = ((string)accountKeyValue4);
                                                            propertiesInstance15.AccountKey = accountKeyInstance4;
                                                        }
                                                        
                                                        JToken tableValue = propertiesValue41["table"];
                                                        if (tableValue != null && tableValue.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance = ((string)tableValue);
                                                            propertiesInstance15.Table = tableInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue = propertiesValue41["partitionKey"];
                                                        if (partitionKeyValue != null && partitionKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance = ((string)partitionKeyValue);
                                                            propertiesInstance15.PartitionKey = partitionKeyInstance;
                                                        }
                                                        
                                                        JToken rowKeyValue = propertiesValue41["rowKey"];
                                                        if (rowKeyValue != null && rowKeyValue.Type != JTokenType.Null)
                                                        {
                                                            string rowKeyInstance = ((string)rowKeyValue);
                                                            propertiesInstance15.RowKey = rowKeyInstance;
                                                        }
                                                        
                                                        JToken columnsToRemoveArray2 = propertiesValue41["columnsToRemove"];
                                                        if (columnsToRemoveArray2 != null && columnsToRemoveArray2.Type != JTokenType.Null)
                                                        {
                                                            propertiesInstance15.ColumnsToRemove = new System.Collections.Generic.List<string>();
                                                            foreach (JToken columnsToRemoveValue in ((JArray)columnsToRemoveArray2))
                                                            {
                                                                propertiesInstance15.ColumnsToRemove.Add(((string)columnsToRemoveValue));
                                                            }
                                                        }
                                                        
                                                        JToken batchSizeValue = propertiesValue41["batchSize"];
                                                        if (batchSizeValue != null && batchSizeValue.Type != JTokenType.Null)
                                                        {
                                                            int batchSizeInstance = ((int)batchSizeValue);
                                                            propertiesInstance15.BatchSize = batchSizeInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue15 = datasourceValue6["type"];
                                                    if (typeValue15 != null && typeValue15.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance15 = ((string)typeValue15);
                                                        azureTableOutputDataSourceInstance.Type = typeInstance15;
                                                    }
                                                    propertiesInstance13.DataSource = azureTableOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/EventHub")
                                                {
                                                    EventHubOutputDataSource eventHubOutputDataSourceInstance = new EventHubOutputDataSource();
                                                    
                                                    JToken propertiesValue42 = datasourceValue6["properties"];
                                                    if (propertiesValue42 != null && propertiesValue42.Type != JTokenType.Null)
                                                    {
                                                        EventHubOutputDataSourceProperties propertiesInstance16 = new EventHubOutputDataSourceProperties();
                                                        eventHubOutputDataSourceInstance.Properties = propertiesInstance16;
                                                        
                                                        JToken serviceBusNamespaceValue2 = propertiesValue42["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue2 != null && serviceBusNamespaceValue2.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance2 = ((string)serviceBusNamespaceValue2);
                                                            propertiesInstance16.ServiceBusNamespace = serviceBusNamespaceInstance2;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue3 = propertiesValue42["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue3 != null && sharedAccessPolicyNameValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance3 = ((string)sharedAccessPolicyNameValue3);
                                                            propertiesInstance16.SharedAccessPolicyName = sharedAccessPolicyNameInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue3 = propertiesValue42["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue3 != null && sharedAccessPolicyKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance3 = ((string)sharedAccessPolicyKeyValue3);
                                                            propertiesInstance16.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance3;
                                                        }
                                                        
                                                        JToken eventHubNameValue2 = propertiesValue42["eventHubName"];
                                                        if (eventHubNameValue2 != null && eventHubNameValue2.Type != JTokenType.Null)
                                                        {
                                                            string eventHubNameInstance2 = ((string)eventHubNameValue2);
                                                            propertiesInstance16.EventHubName = eventHubNameInstance2;
                                                        }
                                                        
                                                        JToken partitionKeyValue2 = propertiesValue42["partitionKey"];
                                                        if (partitionKeyValue2 != null && partitionKeyValue2.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance2 = ((string)partitionKeyValue2);
                                                            propertiesInstance16.PartitionKey = partitionKeyInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue16 = datasourceValue6["type"];
                                                    if (typeValue16 != null && typeValue16.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance16 = ((string)typeValue16);
                                                        eventHubOutputDataSourceInstance.Type = typeInstance16;
                                                    }
                                                    propertiesInstance13.DataSource = eventHubOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Queue")
                                                {
                                                    ServiceBusQueueOutputDataSource serviceBusQueueOutputDataSourceInstance = new ServiceBusQueueOutputDataSource();
                                                    
                                                    JToken propertiesValue43 = datasourceValue6["properties"];
                                                    if (propertiesValue43 != null && propertiesValue43.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusQueueOutputDataSourceProperties propertiesInstance17 = new ServiceBusQueueOutputDataSourceProperties();
                                                        serviceBusQueueOutputDataSourceInstance.Properties = propertiesInstance17;
                                                        
                                                        JToken serviceBusNamespaceValue3 = propertiesValue43["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue3 != null && serviceBusNamespaceValue3.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance3 = ((string)serviceBusNamespaceValue3);
                                                            propertiesInstance17.ServiceBusNamespace = serviceBusNamespaceInstance3;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue4 = propertiesValue43["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue4 != null && sharedAccessPolicyNameValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance4 = ((string)sharedAccessPolicyNameValue4);
                                                            propertiesInstance17.SharedAccessPolicyName = sharedAccessPolicyNameInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue4 = propertiesValue43["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue4 != null && sharedAccessPolicyKeyValue4.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance4 = ((string)sharedAccessPolicyKeyValue4);
                                                            propertiesInstance17.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance4;
                                                        }
                                                        
                                                        JToken queueNameValue = propertiesValue43["queueName"];
                                                        if (queueNameValue != null && queueNameValue.Type != JTokenType.Null)
                                                        {
                                                            string queueNameInstance = ((string)queueNameValue);
                                                            propertiesInstance17.QueueName = queueNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue17 = datasourceValue6["type"];
                                                    if (typeValue17 != null && typeValue17.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance17 = ((string)typeValue17);
                                                        serviceBusQueueOutputDataSourceInstance.Type = typeInstance17;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusQueueOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.ServiceBus/Topic")
                                                {
                                                    ServiceBusTopicOutputDataSource serviceBusTopicOutputDataSourceInstance = new ServiceBusTopicOutputDataSource();
                                                    
                                                    JToken propertiesValue44 = datasourceValue6["properties"];
                                                    if (propertiesValue44 != null && propertiesValue44.Type != JTokenType.Null)
                                                    {
                                                        ServiceBusTopicOutputDataSourceProperties propertiesInstance18 = new ServiceBusTopicOutputDataSourceProperties();
                                                        serviceBusTopicOutputDataSourceInstance.Properties = propertiesInstance18;
                                                        
                                                        JToken serviceBusNamespaceValue4 = propertiesValue44["serviceBusNamespace"];
                                                        if (serviceBusNamespaceValue4 != null && serviceBusNamespaceValue4.Type != JTokenType.Null)
                                                        {
                                                            string serviceBusNamespaceInstance4 = ((string)serviceBusNamespaceValue4);
                                                            propertiesInstance18.ServiceBusNamespace = serviceBusNamespaceInstance4;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyNameValue5 = propertiesValue44["sharedAccessPolicyName"];
                                                        if (sharedAccessPolicyNameValue5 != null && sharedAccessPolicyNameValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyNameInstance5 = ((string)sharedAccessPolicyNameValue5);
                                                            propertiesInstance18.SharedAccessPolicyName = sharedAccessPolicyNameInstance5;
                                                        }
                                                        
                                                        JToken sharedAccessPolicyKeyValue5 = propertiesValue44["sharedAccessPolicyKey"];
                                                        if (sharedAccessPolicyKeyValue5 != null && sharedAccessPolicyKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string sharedAccessPolicyKeyInstance5 = ((string)sharedAccessPolicyKeyValue5);
                                                            propertiesInstance18.SharedAccessPolicyKey = sharedAccessPolicyKeyInstance5;
                                                        }
                                                        
                                                        JToken topicNameValue = propertiesValue44["topicName"];
                                                        if (topicNameValue != null && topicNameValue.Type != JTokenType.Null)
                                                        {
                                                            string topicNameInstance = ((string)topicNameValue);
                                                            propertiesInstance18.TopicName = topicNameInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue18 = datasourceValue6["type"];
                                                    if (typeValue18 != null && typeValue18.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance18 = ((string)typeValue18);
                                                        serviceBusTopicOutputDataSourceInstance.Type = typeInstance18;
                                                    }
                                                    propertiesInstance13.DataSource = serviceBusTopicOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Sql/Server/Database")
                                                {
                                                    SqlAzureOutputDataSource sqlAzureOutputDataSourceInstance = new SqlAzureOutputDataSource();
                                                    
                                                    JToken propertiesValue45 = datasourceValue6["properties"];
                                                    if (propertiesValue45 != null && propertiesValue45.Type != JTokenType.Null)
                                                    {
                                                        SqlAzureOutputDataSourceProperties propertiesInstance19 = new SqlAzureOutputDataSourceProperties();
                                                        sqlAzureOutputDataSourceInstance.Properties = propertiesInstance19;
                                                        
                                                        JToken serverValue = propertiesValue45["server"];
                                                        if (serverValue != null && serverValue.Type != JTokenType.Null)
                                                        {
                                                            string serverInstance = ((string)serverValue);
                                                            propertiesInstance19.Server = serverInstance;
                                                        }
                                                        
                                                        JToken databaseValue = propertiesValue45["database"];
                                                        if (databaseValue != null && databaseValue.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance = ((string)databaseValue);
                                                            propertiesInstance19.Database = databaseInstance;
                                                        }
                                                        
                                                        JToken userValue = propertiesValue45["user"];
                                                        if (userValue != null && userValue.Type != JTokenType.Null)
                                                        {
                                                            string userInstance = ((string)userValue);
                                                            propertiesInstance19.User = userInstance;
                                                        }
                                                        
                                                        JToken passwordValue = propertiesValue45["password"];
                                                        if (passwordValue != null && passwordValue.Type != JTokenType.Null)
                                                        {
                                                            string passwordInstance = ((string)passwordValue);
                                                            propertiesInstance19.Password = passwordInstance;
                                                        }
                                                        
                                                        JToken tableValue2 = propertiesValue45["table"];
                                                        if (tableValue2 != null && tableValue2.Type != JTokenType.Null)
                                                        {
                                                            string tableInstance2 = ((string)tableValue2);
                                                            propertiesInstance19.Table = tableInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue19 = datasourceValue6["type"];
                                                    if (typeValue19 != null && typeValue19.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance19 = ((string)typeValue19);
                                                        sqlAzureOutputDataSourceInstance.Type = typeInstance19;
                                                    }
                                                    propertiesInstance13.DataSource = sqlAzureOutputDataSourceInstance;
                                                }
                                                if (typeName6 == "Microsoft.Storage/DocumentDB")
                                                {
                                                    DocumentDbOutputDataSource documentDbOutputDataSourceInstance = new DocumentDbOutputDataSource();
                                                    
                                                    JToken propertiesValue46 = datasourceValue6["properties"];
                                                    if (propertiesValue46 != null && propertiesValue46.Type != JTokenType.Null)
                                                    {
                                                        DocumentDbOutputDataSourceProperties propertiesInstance20 = new DocumentDbOutputDataSourceProperties();
                                                        documentDbOutputDataSourceInstance.Properties = propertiesInstance20;
                                                        
                                                        JToken accountIdValue = propertiesValue46["accountId"];
                                                        if (accountIdValue != null && accountIdValue.Type != JTokenType.Null)
                                                        {
                                                            string accountIdInstance = ((string)accountIdValue);
                                                            propertiesInstance20.AccountId = accountIdInstance;
                                                        }
                                                        
                                                        JToken accountKeyValue5 = propertiesValue46["accountKey"];
                                                        if (accountKeyValue5 != null && accountKeyValue5.Type != JTokenType.Null)
                                                        {
                                                            string accountKeyInstance5 = ((string)accountKeyValue5);
                                                            propertiesInstance20.AccountKey = accountKeyInstance5;
                                                        }
                                                        
                                                        JToken databaseValue2 = propertiesValue46["database"];
                                                        if (databaseValue2 != null && databaseValue2.Type != JTokenType.Null)
                                                        {
                                                            string databaseInstance2 = ((string)databaseValue2);
                                                            propertiesInstance20.Database = databaseInstance2;
                                                        }
                                                        
                                                        JToken collectionNamePatternValue = propertiesValue46["collectionNamePattern"];
                                                        if (collectionNamePatternValue != null && collectionNamePatternValue.Type != JTokenType.Null)
                                                        {
                                                            string collectionNamePatternInstance = ((string)collectionNamePatternValue);
                                                            propertiesInstance20.CollectionNamePattern = collectionNamePatternInstance;
                                                        }
                                                        
                                                        JToken partitionKeyValue3 = propertiesValue46["partitionKey"];
                                                        if (partitionKeyValue3 != null && partitionKeyValue3.Type != JTokenType.Null)
                                                        {
                                                            string partitionKeyInstance3 = ((string)partitionKeyValue3);
                                                            propertiesInstance20.PartitionKey = partitionKeyInstance3;
                                                        }
                                                        
                                                        JToken documentIdValue = propertiesValue46["documentId"];
                                                        if (documentIdValue != null && documentIdValue.Type != JTokenType.Null)
                                                        {
                                                            string documentIdInstance = ((string)documentIdValue);
                                                            propertiesInstance20.DocumentId = documentIdInstance;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue20 = datasourceValue6["type"];
                                                    if (typeValue20 != null && typeValue20.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance20 = ((string)typeValue20);
                                                        documentDbOutputDataSourceInstance.Type = typeInstance20;
                                                    }
                                                    propertiesInstance13.DataSource = documentDbOutputDataSourceInstance;
                                                }
                                            }
                                            
                                            JToken serializationValue6 = propertiesValue39["serialization"];
                                            if (serializationValue6 != null && serializationValue6.Type != JTokenType.Null)
                                            {
                                                string typeName7 = ((string)serializationValue6["type"]);
                                                if (typeName7 == "Csv")
                                                {
                                                    CsvSerialization csvSerializationInstance3 = new CsvSerialization();
                                                    
                                                    JToken propertiesValue47 = serializationValue6["properties"];
                                                    if (propertiesValue47 != null && propertiesValue47.Type != JTokenType.Null)
                                                    {
                                                        CsvSerializationProperties propertiesInstance21 = new CsvSerializationProperties();
                                                        csvSerializationInstance3.Properties = propertiesInstance21;
                                                        
                                                        JToken fieldDelimiterValue3 = propertiesValue47["fieldDelimiter"];
                                                        if (fieldDelimiterValue3 != null && fieldDelimiterValue3.Type != JTokenType.Null)
                                                        {
                                                            string fieldDelimiterInstance3 = ((string)fieldDelimiterValue3);
                                                            propertiesInstance21.FieldDelimiter = fieldDelimiterInstance3;
                                                        }
                                                        
                                                        JToken encodingValue5 = propertiesValue47["encoding"];
                                                        if (encodingValue5 != null && encodingValue5.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance5 = ((string)encodingValue5);
                                                            propertiesInstance21.Encoding = encodingInstance5;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue21 = serializationValue6["type"];
                                                    if (typeValue21 != null && typeValue21.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance21 = ((string)typeValue21);
                                                        csvSerializationInstance3.Type = typeInstance21;
                                                    }
                                                    propertiesInstance13.Serialization = csvSerializationInstance3;
                                                }
                                                if (typeName7 == "Json")
                                                {
                                                    JsonSerialization jsonSerializationInstance3 = new JsonSerialization();
                                                    
                                                    JToken propertiesValue48 = serializationValue6["properties"];
                                                    if (propertiesValue48 != null && propertiesValue48.Type != JTokenType.Null)
                                                    {
                                                        JsonSerializationProperties propertiesInstance22 = new JsonSerializationProperties();
                                                        jsonSerializationInstance3.Properties = propertiesInstance22;
                                                        
                                                        JToken encodingValue6 = propertiesValue48["encoding"];
                                                        if (encodingValue6 != null && encodingValue6.Type != JTokenType.Null)
                                                        {
                                                            string encodingInstance6 = ((string)encodingValue6);
                                                            propertiesInstance22.Encoding = encodingInstance6;
                                                        }
                                                        
                                                        JToken formatValue3 = propertiesValue48["format"];
                                                        if (formatValue3 != null && formatValue3.Type != JTokenType.Null)
                                                        {
                                                            string formatInstance3 = ((string)formatValue3);
                                                            propertiesInstance22.Format = formatInstance3;
                                                        }
                                                    }
                                                    
                                                    JToken typeValue22 = serializationValue6["type"];
                                                    if (typeValue22 != null && typeValue22.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance22 = ((string)typeValue22);
                                                        jsonSerializationInstance3.Type = typeInstance22;
                                                    }
                                                    propertiesInstance13.Serialization = jsonSerializationInstance3;
                                                }
                                                if (typeName7 == "Avro")
                                                {
                                                    AvroSerialization avroSerializationInstance3 = new AvroSerialization();
                                                    
                                                    JToken propertiesValue49 = serializationValue6["properties"];
                                                    if (propertiesValue49 != null && propertiesValue49.Type != JTokenType.Null)
                                                    {
                                                        AvroSerializationProperties propertiesInstance23 = new AvroSerializationProperties();
                                                        avroSerializationInstance3.Properties = propertiesInstance23;
                                                    }
                                                    
                                                    JToken typeValue23 = serializationValue6["type"];
                                                    if (typeValue23 != null && typeValue23.Type != JTokenType.Null)
                                                    {
                                                        string typeInstance23 = ((string)typeValue23);
                                                        avroSerializationInstance3.Type = typeInstance23;
                                                    }
                                                    propertiesInstance13.Serialization = avroSerializationInstance3;
                                                }
                                            }
                                            
                                            JToken diagnosticsValue6 = propertiesValue39["diagnostics"];
                                            if (diagnosticsValue6 != null && diagnosticsValue6.Type != JTokenType.Null)
                                            {
                                                Diagnostics diagnosticsInstance3 = new Diagnostics();
                                                propertiesInstance13.Diagnostics = diagnosticsInstance3;
                                                
                                                JToken conditionsArray6 = diagnosticsValue6["conditions"];
                                                if (conditionsArray6 != null && conditionsArray6.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken conditionsValue3 in ((JArray)conditionsArray6))
                                                    {
                                                        DiagnosticCondition diagnosticConditionInstance3 = new DiagnosticCondition();
                                                        diagnosticsInstance3.Conditions.Add(diagnosticConditionInstance3);
                                                        
                                                        JToken sinceValue3 = conditionsValue3["since"];
                                                        if (sinceValue3 != null && sinceValue3.Type != JTokenType.Null)
                                                        {
                                                            DateTime sinceInstance3 = ((DateTime)sinceValue3);
                                                            diagnosticConditionInstance3.Since = sinceInstance3;
                                                        }
                                                        
                                                        JToken codeValue3 = conditionsValue3["code"];
                                                        if (codeValue3 != null && codeValue3.Type != JTokenType.Null)
                                                        {
                                                            string codeInstance3 = ((string)codeValue3);
                                                            diagnosticConditionInstance3.Code = codeInstance3;
                                                        }
                                                        
                                                        JToken messageValue3 = conditionsValue3["message"];
                                                        if (messageValue3 != null && messageValue3.Type != JTokenType.Null)
                                                        {
                                                            string messageInstance3 = ((string)messageValue3);
                                                            diagnosticConditionInstance3.Message = messageInstance3;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                                
                                JToken functionsArray2 = propertiesValue26["functions"];
                                if (functionsArray2 != null && functionsArray2.Type != JTokenType.Null)
                                {
                                    propertiesInstance.Functions = new List<Function>();
                                    foreach (JToken functionsValue in ((JArray)functionsArray2))
                                    {
                                        Function functionInstance = new Function();
                                        propertiesInstance.Functions.Add(functionInstance);
                                        
                                        JToken nameValue6 = functionsValue["name"];
                                        if (nameValue6 != null && nameValue6.Type != JTokenType.Null)
                                        {
                                            string nameInstance6 = ((string)nameValue6);
                                            functionInstance.Name = nameInstance6;
                                        }
                                        
                                        JToken propertiesValue50 = functionsValue["properties"];
                                        if (propertiesValue50 != null && propertiesValue50.Type != JTokenType.Null)
                                        {
                                            string typeName8 = ((string)propertiesValue50["type"]);
                                            if (typeName8 == "Scalar")
                                            {
                                                ScalarFunctionProperties scalarFunctionPropertiesInstance = new ScalarFunctionProperties();
                                                
                                                JToken propertiesValue51 = propertiesValue50["properties"];
                                                if (propertiesValue51 != null && propertiesValue51.Type != JTokenType.Null)
                                                {
                                                    ScalarFunctionConfiguration propertiesInstance24 = new ScalarFunctionConfiguration();
                                                    scalarFunctionPropertiesInstance.Properties = propertiesInstance24;
                                                    
                                                    JToken inputsArray4 = propertiesValue51["inputs"];
                                                    if (inputsArray4 != null && inputsArray4.Type != JTokenType.Null)
                                                    {
                                                        propertiesInstance24.Inputs = new List<FunctionInput>();
                                                        foreach (JToken inputsValue3 in ((JArray)inputsArray4))
                                                        {
                                                            FunctionInput functionInputInstance = new FunctionInput();
                                                            propertiesInstance24.Inputs.Add(functionInputInstance);
                                                            
                                                            JToken dataTypeValue = inputsValue3["dataType"];
                                                            if (dataTypeValue != null && dataTypeValue.Type != JTokenType.Null)
                                                            {
                                                                string dataTypeInstance = ((string)dataTypeValue);
                                                                functionInputInstance.DataType = dataTypeInstance;
                                                            }
                                                            
                                                            JToken isConfigurationParameterValue = inputsValue3["isConfigurationParameter"];
                                                            if (isConfigurationParameterValue != null && isConfigurationParameterValue.Type != JTokenType.Null)
                                                            {
                                                                bool isConfigurationParameterInstance = ((bool)isConfigurationParameterValue);
                                                                functionInputInstance.IsConfigurationParameter = isConfigurationParameterInstance;
                                                            }
                                                        }
                                                    }
                                                    
                                                    JToken outputValue3 = propertiesValue51["output"];
                                                    if (outputValue3 != null && outputValue3.Type != JTokenType.Null)
                                                    {
                                                        FunctionOutput outputInstance2 = new FunctionOutput();
                                                        propertiesInstance24.Output = outputInstance2;
                                                        
                                                        JToken dataTypeValue2 = outputValue3["dataType"];
                                                        if (dataTypeValue2 != null && dataTypeValue2.Type != JTokenType.Null)
                                                        {
                                                            string dataTypeInstance2 = ((string)dataTypeValue2);
                                                            outputInstance2.DataType = dataTypeInstance2;
                                                        }
                                                    }
                                                    
                                                    JToken bindingValue2 = propertiesValue51["binding"];
                                                    if (bindingValue2 != null && bindingValue2.Type != JTokenType.Null)
                                                    {
                                                        string typeName9 = ((string)bindingValue2["type"]);
                                                        if (typeName9 == "Microsoft.MachineLearning/WebService")
                                                        {
                                                            AzureMachineLearningWebServiceFunctionBinding azureMachineLearningWebServiceFunctionBindingInstance = new AzureMachineLearningWebServiceFunctionBinding();
                                                            
                                                            JToken propertiesValue52 = bindingValue2["properties"];
                                                            if (propertiesValue52 != null && propertiesValue52.Type != JTokenType.Null)
                                                            {
                                                                AzureMachineLearningWebServiceFunctionBindingProperties propertiesInstance25 = new AzureMachineLearningWebServiceFunctionBindingProperties();
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Properties = propertiesInstance25;
                                                                
                                                                JToken endpointValue = propertiesValue52["endpoint"];
                                                                if (endpointValue != null && endpointValue.Type != JTokenType.Null)
                                                                {
                                                                    string endpointInstance = ((string)endpointValue);
                                                                    propertiesInstance25.Endpoint = endpointInstance;
                                                                }
                                                                
                                                                JToken apiKeyValue = propertiesValue52["apiKey"];
                                                                if (apiKeyValue != null && apiKeyValue.Type != JTokenType.Null)
                                                                {
                                                                    string apiKeyInstance = ((string)apiKeyValue);
                                                                    propertiesInstance25.ApiKey = apiKeyInstance;
                                                                }
                                                                
                                                                JToken inputsValue4 = propertiesValue52["inputs"];
                                                                if (inputsValue4 != null && inputsValue4.Type != JTokenType.Null)
                                                                {
                                                                    AzureMachineLearningWebServiceInputs inputsInstance = new AzureMachineLearningWebServiceInputs();
                                                                    propertiesInstance25.Inputs = inputsInstance;
                                                                    
                                                                    JToken nameValue7 = inputsValue4["name"];
                                                                    if (nameValue7 != null && nameValue7.Type != JTokenType.Null)
                                                                    {
                                                                        string nameInstance7 = ((string)nameValue7);
                                                                        inputsInstance.Name = nameInstance7;
                                                                    }
                                                                    
                                                                    JToken columnNamesArray2 = inputsValue4["columnNames"];
                                                                    if (columnNamesArray2 != null && columnNamesArray2.Type != JTokenType.Null)
                                                                    {
                                                                        inputsInstance.ColumnNames = new List<AzureMachineLearningWebServiceInputColumn>();
                                                                        foreach (JToken columnNamesValue in ((JArray)columnNamesArray2))
                                                                        {
                                                                            AzureMachineLearningWebServiceInputColumn azureMachineLearningWebServiceInputColumnInstance = new AzureMachineLearningWebServiceInputColumn();
                                                                            inputsInstance.ColumnNames.Add(azureMachineLearningWebServiceInputColumnInstance);
                                                                            
                                                                            JToken nameValue8 = columnNamesValue["name"];
                                                                            if (nameValue8 != null && nameValue8.Type != JTokenType.Null)
                                                                            {
                                                                                string nameInstance8 = ((string)nameValue8);
                                                                                azureMachineLearningWebServiceInputColumnInstance.Name = nameInstance8;
                                                                            }
                                                                            
                                                                            JToken dataTypeValue3 = columnNamesValue["dataType"];
                                                                            if (dataTypeValue3 != null && dataTypeValue3.Type != JTokenType.Null)
                                                                            {
                                                                                string dataTypeInstance3 = ((string)dataTypeValue3);
                                                                                azureMachineLearningWebServiceInputColumnInstance.DataType = dataTypeInstance3;
                                                                            }
                                                                            
                                                                            JToken mapToValue = columnNamesValue["mapTo"];
                                                                            if (mapToValue != null && mapToValue.Type != JTokenType.Null)
                                                                            {
                                                                                int mapToInstance = ((int)mapToValue);
                                                                                azureMachineLearningWebServiceInputColumnInstance.MapTo = mapToInstance;
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken outputsArray4 = propertiesValue52["outputs"];
                                                                if (outputsArray4 != null && outputsArray4.Type != JTokenType.Null)
                                                                {
                                                                    propertiesInstance25.Outputs = new List<AzureMachineLearningWebServiceOutputColumn>();
                                                                    foreach (JToken outputsValue2 in ((JArray)outputsArray4))
                                                                    {
                                                                        AzureMachineLearningWebServiceOutputColumn azureMachineLearningWebServiceOutputColumnInstance = new AzureMachineLearningWebServiceOutputColumn();
                                                                        propertiesInstance25.Outputs.Add(azureMachineLearningWebServiceOutputColumnInstance);
                                                                        
                                                                        JToken nameValue9 = outputsValue2["name"];
                                                                        if (nameValue9 != null && nameValue9.Type != JTokenType.Null)
                                                                        {
                                                                            string nameInstance9 = ((string)nameValue9);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.Name = nameInstance9;
                                                                        }
                                                                        
                                                                        JToken dataTypeValue4 = outputsValue2["dataType"];
                                                                        if (dataTypeValue4 != null && dataTypeValue4.Type != JTokenType.Null)
                                                                        {
                                                                            string dataTypeInstance4 = ((string)dataTypeValue4);
                                                                            azureMachineLearningWebServiceOutputColumnInstance.DataType = dataTypeInstance4;
                                                                        }
                                                                    }
                                                                }
                                                                
                                                                JToken batchSizeValue2 = propertiesValue52["batchSize"];
                                                                if (batchSizeValue2 != null && batchSizeValue2.Type != JTokenType.Null)
                                                                {
                                                                    int batchSizeInstance2 = ((int)batchSizeValue2);
                                                                    propertiesInstance25.BatchSize = batchSizeInstance2;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue24 = bindingValue2["type"];
                                                            if (typeValue24 != null && typeValue24.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance24 = ((string)typeValue24);
                                                                azureMachineLearningWebServiceFunctionBindingInstance.Type = typeInstance24;
                                                            }
                                                            propertiesInstance24.Binding = azureMachineLearningWebServiceFunctionBindingInstance;
                                                        }
                                                        if (typeName9 == "Microsoft.StreamAnalytics/JavascriptUdf")
                                                        {
                                                            JavaScriptFunctionBinding javaScriptFunctionBindingInstance = new JavaScriptFunctionBinding();
                                                            
                                                            JToken propertiesValue53 = bindingValue2["properties"];
                                                            if (propertiesValue53 != null && propertiesValue53.Type != JTokenType.Null)
                                                            {
                                                                JavaScriptFunctionBindingProperties propertiesInstance26 = new JavaScriptFunctionBindingProperties();
                                                                javaScriptFunctionBindingInstance.Properties = propertiesInstance26;
                                                                
                                                                JToken scriptValue = propertiesValue53["script"];
                                                                if (scriptValue != null && scriptValue.Type != JTokenType.Null)
                                                                {
                                                                    string scriptInstance = ((string)scriptValue);
                                                                    propertiesInstance26.Script = scriptInstance;
                                                                }
                                                            }
                                                            
                                                            JToken typeValue25 = bindingValue2["type"];
                                                            if (typeValue25 != null && typeValue25.Type != JTokenType.Null)
                                                            {
                                                                string typeInstance25 = ((string)typeValue25);
                                                                javaScriptFunctionBindingInstance.Type = typeInstance25;
                                                            }
                                                            propertiesInstance24.Binding = javaScriptFunctionBindingInstance;
                                                        }
                                                    }
                                                }
                                                
                                                JToken etagValue6 = propertiesValue50["etag"];
                                                if (etagValue6 != null && etagValue6.Type != JTokenType.Null)
                                                {
                                                    string etagInstance6 = ((string)etagValue6);
                                                    scalarFunctionPropertiesInstance.Etag = etagInstance6;
                                                }
                                                
                                                JToken typeValue26 = propertiesValue50["type"];
                                                if (typeValue26 != null && typeValue26.Type != JTokenType.Null)
                                                {
                                                    string typeInstance26 = ((string)typeValue26);
                                                    scalarFunctionPropertiesInstance.Type = typeInstance26;
                                                }
                                                functionInstance.Properties = scalarFunctionPropertiesInstance;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Date"))
                    {
                        result.Date = DateTime.Parse(httpResponse.Headers.GetValues("Date").FirstOrDefault(), CultureInfo.InvariantCulture, DateTimeStyles.AdjustToUniversal).ToLocalTime();
                    }
                    if (httpResponse.Headers.Contains("ETag"))
                    {
                        result.Job.Properties.Etag = httpResponse.Headers.GetValues("ETag").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Start a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Parameters for a job start operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> StartAsync(string resourceGroupName, string jobName, JobStartParameters parameters, CancellationToken cancellationToken)
        {
            StreamAnalyticsManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "StartAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.StreamingJobs.BeginStartAsync(resourceGroupName, jobName, parameters, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse result = await client.GetLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 10;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.GetLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 10;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Stop a stream analytics job.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The resource group name of the stream analytics job.
        /// </param>
        /// <param name='jobName'>
        /// Required. The name of the stream analytics job.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> StopAsync(string resourceGroupName, string jobName, CancellationToken cancellationToken)
        {
            StreamAnalyticsManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("jobName", jobName);
                TracingAdapter.Enter(invocationId, this, "StopAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse response = await client.StreamingJobs.BeginStopAsync(resourceGroupName, jobName, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            LongRunningOperationResponse result = await client.GetLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 10;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while (result.Status == OperationStatus.InProgress)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.GetLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 10;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
    }
}
